----------------------------------------------------------------------

local AB_TEST_NAME   = "authentication_reward";
local listener       = {};

----------------------------------------------------------------------

function listener:activate()

   EventMgr:addListener( self, self._onIAPVerified,      "iap-verification-accepted"   );
   EventMgr:addListener( self, self._onLevelUp,          "player-level-up"             );
   EventMgr:addListener( self, self._onLogin,            "login-success"               );
   EventMgr:addListener( self, self._onAuthentication,   "will-authenticate"           );
   EventMgr:addListener( self, self._onMessageSent,      "tango.message.success"       );

end

----------------------------------------------------------------------

function listener:deactivate()

   EventMgr:rmvListener( self );

end

----------------------------------------------------------------------

function listener:_onIAPVerified( event )

   if event  then

      local loop = 1;

      if event.id and AbTestMgr.IAP_LOOP_ID then
         loop = AbTestMgr.IAP_LOOP_ID[event.id] or 1;
      end

      for i=1,loop do
         AbTestMgr:onSuccess( AB_TEST_NAME, "buy-iap-verified_loop", false, " - Tango : " .. tostring(LoginMgr:isTango()) );
      end

   end

end

----------------------------------------------------------------------

function listener:_onLevelUp( event )

   if event and event.level and ( not App.data:isThief() ) then
      if math.abs(event.level % 5) < 1 then
         AbTestMgr:onSuccess(  AB_TEST_NAME, "Level-Up", false, " - " .. tostring(event.level)  );
      end
   end

end


----------------------------------------------------------------------

function listener:_onAuthentication( event )

   --Ok to let thief send data
   if event then
      AbTestMgr:onSuccess( AB_TEST_NAME, "authenticate-attempt", true );
   end

end

----------------------------------------------------------------------

function listener:_onLogin( event )

   if event and ( not App.data:isThief() ) then

      if MOAITangoSdk:tangoHasSdkSupport() then
         AbTestMgr:onSuccess(  AB_TEST_NAME, "Login-Tango-Install", true  );
      end

      if event.tango and event.tangoLoginCount == 1 then
         AbTestMgr:onSuccess(  AB_TEST_NAME, "Login-Tango-Authenticate-once"  );
      end

   end

end

----------------------------------------------------------------------

function listener:_onMessageSent( event )

   if event and ( not App.data:isThief() ) then

      local realCount = 0;

      --Make sure we are not dealing with Ghost
      if type(event.recipientList) == "table" then

         for k,v in pairs(event.recipientList) do

            if type(v) == "userdata" then

               if type(v.isGhost) == "function" and ( not v:isGhost()) then

                  realCount = realCount + 1;

               end
            end
         end

      end

      if realCount > 0 then
         AbTestMgr:onSuccess(  AB_TEST_NAME, "Message-Sent",    true, "-" .. tostring(event.type) );
      end

   end

end

----------------------------------------------------------------------

return listener;

