----------------------------------------------------------------------

local AB_TEST_NAME   = "consumable_cost_v2";
local listener       = {};

local FIRST_LAUNCH_ID   = "ab-test-"..AB_TEST_NAME.."-firstLaunch-day";
local LAST_DAY_ID       = "ab-test-"..AB_TEST_NAME.."-last-day";

----------------------------------------------------------------------

function listener:activate()

   EventMgr:addListener( self, self._onIAPVerified,      "iap-verification-accepted"   );
   EventMgr:addListener( self, self._onGamePlay,         "game-start"                  );
   EventMgr:addListener( self, self._onConsumableUsed,   "consumable-used"             );
   EventMgr:addListener( self, self._onRevive,           "revive-used"                 );
   EventMgr:addListener( self, self._onVideoGold,        "video-gold"                  );
   EventMgr:addListener( self, self._onVideoKey,         "video-key"                   );

   App.data:setValue(FIRST_LAUNCH_ID, App.data:getValue(FIRST_LAUNCH_ID, os.time()));

end

----------------------------------------------------------------------

function listener:deactivate()

   EventMgr:rmvListener( self );

end

----------------------------------------------------------------------

function listener:_onIAPVerified( event )

   Debug.assert( event );

   if event and event.id and ( not App.data:isThief() ) then
      AbTestMgr:onSuccess( AB_TEST_NAME, "buy-iap-verified", true, " : $" .. tostring(AbTestMgr.IAP_COST[event.id]) );
   end

end

----------------------------------------------------------------------

function listener:_onGamePlay( event )

   if not App.data:isThief() then
      AbTestMgr:onSuccess(  AB_TEST_NAME, "game-play" );
		self:_reportRetention( App.data:getValue(LAST_DAY_ID, -1), App.data:getValue(FIRST_LAUNCH_ID) );
   end

end

----------------------------------------------------------------------

function listener:_reportRetention( lastDay, firstLaunch )

   if firstLaunch and lastDay then

      local days = math.floor((os.time()-firstLaunch)/(24*60*60));

      if days > lastDay then

         --Set before we transform into a string
         App.data:setValue(LAST_DAY_ID, days);

         if days < 10 then
            days = "00"..days;
         elseif days < 100 then
            days = "0"..days;
         end

         AbTestMgr:onSuccess(  AB_TEST_NAME, "retention", false, ": D-".. days );
      end
   else
      Debug.assert( firstLaunch, "listener:_onGamePlay -> firstLaunch"  );
      Debug.assert( lastDay,     "listener:_onGamePlay -> lastDay"      );
   end

end

----------------------------------------------------------------------

function listener:_onVideoGold( event )

   AbTestMgr:onSuccess( AB_TEST_NAME, "video-gold"             );

end

----------------------------------------------------------------------

function listener:_onVideoKey( event )

   AbTestMgr:onSuccess(  AB_TEST_NAME, "video-key"            );

end

----------------------------------------------------------------------

function listener:_onConsumableUsed( event )

   if not App.data:isThief() then
      AbTestMgr:onSuccess(  AB_TEST_NAME, "consumable-used" );
      AbTestMgr:onSuccess(  AB_TEST_NAME, "consumable-used:" .. tostring(event.type) );
   end

end

----------------------------------------------------------------------

function listener:_onRevive( event )

   if not App.data:isThief() then
      AbTestMgr:onSuccess(  AB_TEST_NAME, "revive" );
   end

end

----------------------------------------------------------------------

return listener;

