----------------------------------------------------------------------

local AB_TEST_NAME   = "game_center";
local listener       = {};

----------------------------------------------------------------------

function listener:activate()

   EventMgr:addListener( self, self._onIAPVerified,                        "iap-verification-accepted"            );
   EventMgr:addListener( self, self._onLevelUp,                            "player-level-up"                      );
   EventMgr:addListener( self, self._onGameCenterAuthenticationAttempt,    "game-center-authentication-attempt"   );
   EventMgr:addListener( self, self._onGameCenterAuthenticationSuccess,    "game-center-authentication-success"   );
   EventMgr:addListener( self, self._onGameCenterUploadScoreSuccess,       "game-center-upload-score-success"     );
   EventMgr:addListener( self, self._onGameCenterShowLeaderboardSuccess,   "game-center-show-leaderboard-success" );
   EventMgr:addListener( self, self._onVideo,                              "video-gold"                           );
   EventMgr:addListener( self, self._onVideo,                              "video-key"                            );

end

----------------------------------------------------------------------

function listener:deactivate()
   EventMgr:rmvListener( self );
end

----------------------------------------------------------------------

function listener:_onIAPVerified( event )

   if event  then

      local loop = 1;

      if event.id and AbTestMgr.IAP_LOOP_ID then
         loop = AbTestMgr.IAP_LOOP_ID[event.id] or 1;
      end

      for i=1,loop do
         AbTestMgr:onSuccess( AB_TEST_NAME, "buy-iap-verified_loop", true );
      end

   end

end

----------------------------------------------------------------------

function listener:_onLevelUp( event )

   if event and event.level and ( not App.data:isThief() ) then
      if math.abs(event.level % 5) < 1 then
         AbTestMgr:onSuccess(  AB_TEST_NAME, "Level-Up", false, " - " .. tostring(event.level)  );
      end
   end

end

----------------------------------------------------------------------

function listener:_onGameCenterAuthenticationAttempt( event )

   if ( not App.data:isThief() ) then
      if event.attempt_count and event.attempt_count == 1 then
         AbTestMgr:onSuccess(  AB_TEST_NAME, "authenticate-attempt-once", false);
      end
   end

end

----------------------------------------------------------------------

function listener:_onGameCenterAuthenticationSuccess( event )

   if ( not App.data:isThief() ) then
      if event.success_count and event.success_count == 1 then
         AbTestMgr:onSuccess(  AB_TEST_NAME, "authenticate-success-once", false);
      end
   end

end

----------------------------------------------------------------------

function listener:_onGameCenterUploadScoreSuccess( event )

   if ( not App.data:isThief() ) then
      AbTestMgr:onSuccess(  AB_TEST_NAME, "upload-score-success");
   end

end

----------------------------------------------------------------------

function listener:_onGameCenterShowLeaderboardSuccess( event )

   if ( not App.data:isThief() ) then
      AbTestMgr:onSuccess(  AB_TEST_NAME, "upload-show-leaderboard", true);
   end

end

----------------------------------------------------------------------

function listener:_onVideo( event )

   if ( not App.data:isThief() ) then
      AbTestMgr:onSuccess(  AB_TEST_NAME, "video" );
   end

end


----------------------------------------------------------------------

return listener;

