----------------------------------------------------------------------

local AB_TEST_NAME      = "interstitial_schedule_v2";
local listener          = {};

local FIRST_LAUNCH_ID   = "ab-test-"..AB_TEST_NAME.."guest-firstLaunch";
local LAST_DAY_ID       = "ab-test-"..AB_TEST_NAME.."-last-day";

----------------------------------------------------------------------

function listener:activate()

   EventMgr:addListener( self, self._onIAPVerified,                        "iap-verification-accepted"   );
   EventMgr:addListener( self, self._onGamePlay,                           "game-start"                  );
   EventMgr:addListener( self, self._onVideo,                              "video-gold"                  );
   EventMgr:addListener( self, self._onVideo,                              "video-key"                   );
   EventMgr:addListener( self, self._onInterstitialSuccess,                "interstitial-success"        );
   EventMgr:addListener( self, self._onSpendGem,                           "Spend-Gem"                   );

   App.data:setValue(FIRST_LAUNCH_ID, App.data:getValue(FIRST_LAUNCH_ID, os.time()))

end

----------------------------------------------------------------------

function listener:deactivate()
   EventMgr:rmvListener( self );
end

----------------------------------------------------------------------

function listener:_onIAPVerified( event )

   if event then

      local loop = 1;

      if event.id and AbTestMgr.IAP_LOOP_ID then
         loop = AbTestMgr.IAP_LOOP_ID[event.id] or 1;
      end

      for i=1,loop do
         AbTestMgr:onSuccess( AB_TEST_NAME, "buy-iap-verified_loop" );
      end

      AbTestMgr:onSuccess( AB_TEST_NAME, "buy-iap-verified", true, " : $" .. tostring(AbTestMgr.IAP_COST[event.id]) );

   end

end

----------------------------------------------------------------------

function listener:_onGamePlay( event )

   if ( not App.data:isThief() ) then

      AbTestMgr:onSuccess(  AB_TEST_NAME, "game-play" );

      self:_reportRetention( App.data:getValue(LAST_DAY_ID, -1), App.data:getValue(FIRST_LAUNCH_ID) );

   end

end

----------------------------------------------------------------------

function listener:_reportRetention( lastDay, firstLaunch )

   if firstLaunch and lastDay then

      local days = math.floor((os.time()-firstLaunch)/(24*60*60));

      if days > lastDay then

         --Set before we transform into a string
         App.data:setValue(LAST_DAY_ID, days);

         if days < 10 then
            days = "00"..days;
         elseif days < 100 then
            days = "0"..days;
         end

         AbTestMgr:onSuccess(  AB_TEST_NAME, "retention", false, ": D-".. days );

      end

   else

      Debug.assert( firstLaunch, "listener:_onGamePlay -> firstLaunch"  );
      Debug.assert( lastDay,     "listener:_onGamePlay -> lastDay"      );

   end

end

----------------------------------------------------------------------

function listener:_onVideo( event )

   if ( not App.data:isThief() ) then
      AbTestMgr:onSuccess(  AB_TEST_NAME, "video" );
   end

end

----------------------------------------------------------------------

function listener:_onInterstitialSuccess( event )

   if ( not App.data:isThief() ) then
      AbTestMgr:onSuccess(  AB_TEST_NAME, "interstitial-success", true );
   end

end

----------------------------------------------------------------------

function listener:_onSpendGem( event )

   if ( not App.data:isThief() ) then
      AbTestMgr:onSuccess(  AB_TEST_NAME, "spend-gem", false, "   amount : " .. tostring(event.cost) );
   end

end

----------------------------------------------------------------------

return listener;