----------------------------------------------------------------------

local AB_TEST_NAME   = "native_ads_v2";
local listener       = {};

local FIRST_LAUNCH_ID   = "ab-test-"..AB_TEST_NAME.."-first-launch";
local LAST_DAY_ID       = "ab-test-"..AB_TEST_NAME.."-last-day";

----------------------------------------------------------------------

function listener:activate()

   EventMgr:addListener( self, self._onIAPVerified,  "iap-verification-accepted" );
   EventMgr:addListener( self, self._onGamePlay,     "game-start"                );
   EventMgr:addListener( self, self._onLevelUp,      "player-level-up"           );

   App.data:setValue(FIRST_LAUNCH_ID, App.data:getValue(FIRST_LAUNCH_ID, os.time()));

end

----------------------------------------------------------------------

function listener:deactivate()

   EventMgr:rmvListener( self );

end

----------------------------------------------------------------------

function listener:_onIAPVerified( event )

   Debug.assertType( event, "table", "listener:_onIAPVerified -> event" );

   if App.data:isThief() then
      return;
   end

   if event and event.id then
      AbTestMgr:onSuccess( AB_TEST_NAME, "buy-iap-verified", true, " : $" .. tostring(AbTestMgr.IAP_COST[event.id]) );
   end

end

----------------------------------------------------------------------

function listener:_onGamePlay( event )

   Debug.assertType( event, "table", "listener:_onGamePlay -> event" );

   if App.data:isThief() then
      return;
   end

   AbTestMgr:onSuccess(  AB_TEST_NAME, "game-play" );
   self:_reportRetention( App.data:getValue(LAST_DAY_ID, -1), App.data:getValue(FIRST_LAUNCH_ID) );

end

----------------------------------------------------------------------

function listener:_reportRetention( lastDay, firstLaunch )

   if firstLaunch and lastDay then

      local days = math.floor((os.time()-firstLaunch)/(24*60*60));

      if days > lastDay then

         --Set before we transform into a string
         App.data:setValue(LAST_DAY_ID, days);

         if days < 10 then
            days = "00"..days;
         elseif days < 100 then
            days = "0"..days;
         end

         AbTestMgr:onSuccess(  AB_TEST_NAME, "retention", false, ": D-".. days );

      end

   else

      Debug.assert( firstLaunch, "listener:_onGamePlay -> firstLaunch"  );
      Debug.assert( lastDay,     "listener:_onGamePlay -> lastDay"      );

   end

end

----------------------------------------------------------------------

function listener:_onLevelUp( event )

   if App.data:isThief() then
      return;
   end

   if event and event.level then
      if math.abs(event.level % 5) < 1 then
         AbTestMgr:onSuccess(  AB_TEST_NAME, "Level-Up", false, " - " .. tostring(event.level)  );
      end
   end

end

----------------------------------------------------------------------

return listener;
