----------------------------------------------------------------------

local AB_TEST_NAME   = "notification_info";
local listener       = {};

----------------------------------------------------------------------

function listener:activate()

	--[[

   EventMgr:addListener( self, self._onIAPVerified,                        "iap-verification-accepted"            );
   EventMgr:addListener( self, self._onLevelUp,                            "player-level-up"                      );
   EventMgr:addListener( self, self._onVideo,                              "video-gold"                           );
   EventMgr:addListener( self, self._onVideo,                              "video-key"                            );
   EventMgr:addListener( self, self._onBuyKey,                             "keys-bought"                          );
   EventMgr:addListener( self, self._onMysterySpin,                        "mystery-spin"                         );
   EventMgr:addListener( self, self._onDailyCollected,                     "daily-reward-collected"               );
   EventMgr:addListener( self, self._onMissionCollected,                   "mission-reward-collected"             );
   EventMgr:addListener( self, self._onGameCenterShowLeaderboardSuccess,   "game-center-show-leaderboard-success" );
   EventMgr:addListener( self, self._onUpgdadeSuccess,                     "upgrade-success"                      );
   EventMgr:addListener( self, self._onUnlockSuccess,                      "unlock-success"                       );
   EventMgr:addListener( self, self._onBountyCollect,                      "bounty-collected"                     );
   ]]
end

----------------------------------------------------------------------

function listener:deactivate()
   EventMgr:rmvListener( self );
end

----------------------------------------------------------------------

function listener:_onIAPVerified( event )

   if event  then

      local loop = 1;

      if event.id and AbTestMgr.IAP_LOOP_ID then
         loop = AbTestMgr.IAP_LOOP_ID[event.id] or 1;
      end

      for i=1,loop do
         AbTestMgr:onSuccess( AB_TEST_NAME, "buy-iap-verified_loop", true );
      end

   end

end

----------------------------------------------------------------------

function listener:_onLevelUp( event )

   if event and event.level and ( not App.data:isThief() ) then

      if ( event.level <= 10 ) or ( math.abs(event.level % 5) < 1 ) then
         AbTestMgr:onSuccess(  AB_TEST_NAME, "Level-Up", false, " - " .. tostring(event.level)  );
      end
   end

end

----------------------------------------------------------------------

function listener:_onVideo( event )

   if ( not App.data:isThief() ) then

      AbTestMgr:onSuccess( AB_TEST_NAME, "video" );

   end

end


----------------------------------------------------------------------

function listener:_onBuyKey( event )

   Debug.assert( event );

   if ( not App.data:isThief() ) then

      AbTestMgr:onSuccess( AB_TEST_NAME, "buy-key", false, " - KEYS-INDEX=" .. tostring(event.index) );

   end

end

----------------------------------------------------------------------

function listener:_onMysterySpin( event )

   if event and ( not App.data:isThief() ) then

      AbTestMgr:onSuccess(  AB_TEST_NAME, "action", false, " => spin" );

   end

end

----------------------------------------------------------------------

function listener:_onDailyCollected( event )

   if event and ( not App.data:isThief() ) then

      AbTestMgr:onSuccess(  AB_TEST_NAME, "action", false, " => daily-collected" );

   end

end

----------------------------------------------------------------------

function listener:_onMissionCollected( event )

   if event and ( not App.data:isThief() ) then

      AbTestMgr:onSuccess(  AB_TEST_NAME, "action", false, " => mission-collected" );

   end

end

----------------------------------------------------------------------

function listener:_onGameCenterShowLeaderboardSuccess( event )

   if ( not App.data:isThief() ) then

      AbTestMgr:onSuccess(  AB_TEST_NAME, "action", false, " => open-leaderboard" );

   end

end

----------------------------------------------------------------------

function listener:_onBountyCollect( event )

   if ( not App.data:isThief() ) then

      AbTestMgr:onSuccess(  AB_TEST_NAME, "action", false, " => bounty-collected" );

   end

end

----------------------------------------------------------------------

function listener:_onUpgdadeSuccess( event )

   if ( not App.data:isThief() ) then

      AbTestMgr:onSuccess(  AB_TEST_NAME, "upgrade"   );

   end

end

----------------------------------------------------------------------

function listener:_onUnlockSuccess( event )

   if event and ( not App.data:isThief() ) then

      if event.item == "vehicle" then
         AbTestMgr:onSuccess(  AB_TEST_NAME, "unlock-vehicle-details",  true,   " - " .. tostring(AbTestMgr.VEHICLE_NAME[event.vehicle]) );
      end

   end

end


----------------------------------------------------------------------

return listener;

