----------------------------------------------------------------------

local AB_TEST_NAME   = "promo_settings_v2";
local listener       = {};

local FIRST_LAUNCH_ID   = "ab-test-"..AB_TEST_NAME.."drone-firstLaunch"; -- TODO Drone?'
local LAST_DAY_ID       = "ab-test-"..AB_TEST_NAME.."-last-day";

----------------------------------------------------------------------

function listener:activate()

   EventMgr:addListener( self, self._onIAPVerified,      "iap-verification-accepted"   );
   EventMgr:addListener( self, self._onGamePlay,         "game-start"                  );
	EventMgr:addListener( self, self._onClickThrough,	   "promo-click-through"         );
	EventMgr:addListener( self, self._onDroneBayUnlock,	"drone-bay-unlock"            );
	EventMgr:addListener( self, self._onDroneBuild,			"drone-build"                 );
	EventMgr:addListener( self, self._onDroneUpgrade,		"drone-upgrade"               );
	EventMgr:addListener( self, self._onDroneEvolve,		"drone-evolve"                );

	App.data:setValue(FIRST_LAUNCH_ID, App.data:getValue(FIRST_LAUNCH_ID, os.time()));

end

----------------------------------------------------------------------

function listener:deactivate()

   EventMgr:rmvListener( self );

end

----------------------------------------------------------------------

function listener:_onIAPVerified( event )

   Debug.assert( event );

   if event and event.id and ( not App.data:isThief() ) then
      AbTestMgr:onSuccess( AB_TEST_NAME, "buy-iap-verified", true, " : $" .. tostring(AbTestMgr.IAP_COST[event.id]) );
   end

end

----------------------------------------------------------------------

function listener:_onGamePlay( event )

   if ( not App.data:isThief() ) then

      AbTestMgr:onSuccess(  AB_TEST_NAME, "game-play" );
		self:_reportRetention( App.data:getValue(LAST_DAY_ID, -1), App.data:getValue(FIRST_LAUNCH_ID) );

   end

end

----------------------------------------------------------------------

function listener:_reportRetention( lastDay, firstLaunch )

   if firstLaunch and lastDay then

      local days = math.floor((os.time()-firstLaunch)/(24*60*60));

      if days > lastDay then

         --Set before we transform into a string
         App.data:setValue(LAST_DAY_ID, days);

         if days < 10 then
            days = "00"..days;
         elseif days < 100 then
            days = "0"..days;
         end

         AbTestMgr:onSuccess(  AB_TEST_NAME, "retention", false, ": D-".. days );

      end

   else

      Debug.assert( firstLaunch, "listener:_onGamePlay -> firstLaunch"  );
      Debug.assert( lastDay,     "listener:_onGamePlay -> lastDay"      );

   end

end

----------------------------------------------------------------------

function listener:_onClickThrough( event )

   Debug.assert( event );

   if event and event.source and event.success ~= nil and ( not App.data:isThief() ) then
      AbTestMgr:onSuccess( AB_TEST_NAME, "click-through", false, "-From:"..event.source.."-Success:"..tostring(event.success) );
   end

end

----------------------------------------------------------------------

function listener:_onDroneBayUnlock( event )

   if ( not App.data:isThief() ) then
      AbTestMgr:onSuccess(  AB_TEST_NAME, "drone-bay-unlock" );
   end

end

----------------------------------------------------------------------

function listener:_onDroneBuild( event )

   if ( not App.data:isThief() ) then
      AbTestMgr:onSuccess(  AB_TEST_NAME, "drone-build" );
   end

end

----------------------------------------------------------------------

function listener:_onDroneUpgrade( event )

   if ( not App.data:isThief() ) then
      AbTestMgr:onSuccess(  AB_TEST_NAME, "drone-upgrade" );
   end

end

----------------------------------------------------------------------

function listener:_onDroneEvolve( event )

   if ( not App.data:isThief() ) then
      AbTestMgr:onSuccess(  AB_TEST_NAME, "drone-evolve" );
   end

end
----------------------------------------------------------------------

return listener;
