----------------------------------------------------------------------

require('app/achievement/achievement')
require('app/template/template-manager')

----------------------------------------------------------------------

AchievementMgr  =
{
   active = false,
};

----------------------------------------------------------------------

function AchievementMgr:init()

   print("AchievementMgr:init");

   self.active    = false;
   self.template  = TemplateMgr:getAchievement();
   self.achvList  = {};
   self.achvMap   = {};

   Debug.assert(self.template);

   for i,v in ipairs(self.template) do

      if v and v.id then
         local achv = Achievement:new(self, v.id, v);

         if achv then
            self.achvMap[v.id] = achv;
            table.insert(self.achvList , achv);
         else
            Debug.assert( achv, "AchievementMgr:activate -> achv : " .. tostring(i)  );
         end
      else
         Debug.assert(v and v.id, "AchievementMgr:activate -> v and v.id : " .. tostring(i)  );
      end

   end

EventMgr:addListener(self, self.onDataServerMerged, "data-server-merged");

end

----------------------------------------------------------------------

function AchievementMgr:isActive()

   return self.active;

end

----------------------------------------------------------------------

function AchievementMgr:activate()

   print("AchievementMgr:activate",self.active);

   if not self.active then

      self.active = true;

      for i,achv in ipairs(self.achvList) do
         achv:activate()
      end

   end

end

----------------------------------------------------------------------

function AchievementMgr:onDataServerMerged()

   print("AchievementMgr:onDataServerMerged");

   if App.data:isAchievementUnlocked() then

      if not self:isActive() then
         AchievementMgr:activate();
      end
   end

   if self:isActive() then

      for i,achv in pairs(self.achvList) do
         achv:onDataServerMerged();
      end

   end

end

----------------------------------------------------------------------

function AchievementMgr:onAchievementCompleted( achievement, reward  )

end

----------------------------------------------------------------------

function AchievementMgr:getAchievementCount()

   return #self.achvList;

end

----------------------------------------------------------------------

function AchievementMgr:getAchievementByIndex(index)

   if type(index) == "number" and index >= 1 and index <= #self.achvList then
      return self.achvList[index];
   else
      Debug.assert(type(index) == "number" and index >= 1 and index <= #self.achvList, "AchievementMgr:getAchievementByIndex -> type(index) == number and index >= 1 and index <= #self.achvList" )
   end

end

----------------------------------------------------------------------

function AchievementMgr:getAchievementById(id)

   return self.achvMap[id];

end

----------------------------------------------------------------------

function AchievementMgr:isPendingRewards()

   if self:isActive() then

      for i,achv in ipairs(self.achvList) do

         if achv and achv:isPendingReward() then
            return true;
         end

      end
   end
end


----------------------------------------------------------------------
