require('app/achievement/goal/achievement-goal')
require('engine/object/object')

local HACK_FORCE_RANK = nil;

----------------------------------------------------------------------

Achievement =
{
};

----------------------------------------------------------------------

function Achievement:new( manager, id, template  )

   Debug.assert( manager   );
   Debug.assert( id        );
   Debug.assert( template  );

   local achv     = Object.new( self );

   achv.manager   = manager;
   achv.id        = id;
   achv.template  = template;
   achv.active    = false;
   achv.completed = false;
   achv.goal      = nil;

   return achv;

end

----------------------------------------------------------------------

function Achievement:delete()

end

----------------------------------------------------------------------

function Achievement:activate()

print("Achievement:activate", self.active);

   if not self.active then

      self.active = true;

      self.rank   = HACK_FORCE_RANK or math.max(App.data:getAchievementRank(self.template.id) or 1, 1);

print("Achievement:activate-2", self.rank);

      self:_CreateGoal(false);

   end

end

----------------------------------------------------------------------

function Achievement:deactivate()

   if self.active then

      self.active = false;

      if self.goal then
         self.goal:deactivate();
      end
   end

end

----------------------------------------------------------------------

function Achievement:isActive()

   return self.active;

end

----------------------------------------------------------------------

function Achievement:onDataServerMerged()

   local newRank = App.data:getAchievementRank(self.template.id) or 1;

print("Achievement:onDataServerMerged", newRank, self.rank );

   if newRank then
      if newRank > self.rank then

         self.rank = newRank;
         self:_CreateGoal( false );

      elseif newRank == self.rank then

         if self.goal then
            self.goal:onDataServerMerged();
         else
            Debug.assert( self.goal, "Achievement:onDataServerMerged -> self.goal");
         end

      else

         Debug.assert( newRank >=self.rank, "Achievement:onDataServerMerged -> newRank >=self.rank");

      end
   end
end

----------------------------------------------------------------------

function Achievement:_CreateGoal( reset )

   if self.goal then
      self.goal:deactivate();
      self.goal:delete();
      self.goal = nil;
   end

   if self.rank and self.template and self.template.rank[self.rank] and self.template.rank[self.rank].goal then

      local tplGoal = self.template.rank[self.rank].goal;

      if tplGoal.factory then

         self.goal = tplGoal.factory:new( self, tplGoal );

         if self.goal then
            self.goal:activate( reset );
            self.completed = self:_IsCompleted();

            --It is possible to be already completed for achievement like level up
            --Debug.assert( not self.goal:isCompleted(), "Achievement:nextRank -> not self.goal:isCompleted()" );
         end
      else

         Debug.assert(tplGoal.factory, "Achievement:_CreateGoal->tplGoal.factory");
         self.goal = nil;

      end
   end



end
----------------------------------------------------------------------

function Achievement:onGoalProgress( goal, progress )

   Debug.assert( not self:isCompleted(), "Achievement:onGoalProgress -> self:isCompleted()");

print("Achievement:onGoalProgress", self.template.id, progress );

   App.data:updateAchievementProgress( self.template.id, progress );

end

----------------------------------------------------------------------

function Achievement:onGoalCompleted( goal )

print("Achievement:onGoalCompleted", goal, self:_IsCompleted());

   if (not self:isFinished()) and (not self:isCompleted()) then

      EventMgr:postEvent( {  achievement=self , goal=goal }, "achievement-goal-completed" );
      
      if self:_IsCompleted() then

   print("Achievement:onGoalCompleted-2", self.manager, self.rank );

         App.data:updateAchievementProgress( self.template.id, -1 );

         if self.manager and self.rank then

            local tplRank = self.template.rank[self.rank];

   print("Achievement:onGoalCompleted-3", tplRank, tplRank.reward  );

            if tplRank and tplRank.reward then
               self.completed = true;
            else
               Debug.assert( tplRank and tplRank.reward, "Achievement:onGoalCompleted -> tplRank and tplRank.reward"  );
            end
         else

            Debug.assert( self.manager and self.rank, "Achievement:onGoalCompleted -> self.manager and self.rank" );

         end
         
         if not self:isFinished() then
            EventMgr:postEvent( { achievement=self }, "achievement-completed" );
         end

      end

   else

      Debug.assert( not self:isFinished(),  "Achievement:onGoalCompleted -> self:isFinished()");
      Debug.assert( not self:isCompleted(), "Achievement:onGoalCompleted -> self:isCompleted()");

   end
   
end

----------------------------------------------------------------------

function Achievement:isCompleted()

   return self.completed;

end

----------------------------------------------------------------------

function Achievement:isFinished()

   --this means they're done with the entire achivement
   
   if self.rank and self.template.rank then
      return self.rank >= #self.template.rank;
   end
   
   return false;

end

----------------------------------------------------------------------

function Achievement:getProgress()

   if self.goal then
      return self.goal:getProgress();
   end

   return -1;

end

----------------------------------------------------------------------

function Achievement:getObjective()

   if self.goal then
      return self.goal:getObjective();
   end

   return -1;

end

----------------------------------------------------------------------

function Achievement:getDesc()

   if self.goal then
      return self.goal:getDesc();
   end

   return;

end

----------------------------------------------------------------------

function Achievement:getTitle()

   if self.goal then
      return self.goal:getTitle();
   end

   return;

end

----------------------------------------------------------------------

function Achievement:getIcon()

   if self.goal then
      return self.goal:getIcon();
   end

   return;

end

----------------------------------------------------------------------

function Achievement:isPendingReward()

   return ( not self:isFinished() ) and (self:getProgress() < 0);

end

----------------------------------------------------------------------

function Achievement:getReward()

   if self.rank then

      local settings   =      AbTestMgr:getRewardInfoTable("achievement-reward")
                        or    App.data:getValue("App.settings.achievement-reward") or {};

      local mySettings = settings[self.template.id];

      if mySettings then

         local reward = mySettings[self.rank];

         if reward then
            return reward;
         end

      end

      local tplRank = self.template.rank[self.rank];

      if tplRank then

         return tplRank.reward;

      end

   end

end

----------------------------------------------------------------------

function Achievement:getRewardTxt()

   if self.goal then

      local reward = self:getReward();

      if type(reward) == "table" and type(reward.gem) == "number" and reward.gem > 0 then
         return tostring(reward.gem)
      end

      return "-";
   end

   return;

end


----------------------------------------------------------------------

function Achievement:getRank()

   return self.rank;

end

----------------------------------------------------------------------

function Achievement:getMaxRank()

   if self.template and self.template.rank then
      return #self.template.rank -1;
   end

end

----------------------------------------------------------------------

function Achievement:getId()

   return self.id

end

----------------------------------------------------------------------

function Achievement:nextRank()

print( "Achievement:nextRank", self:isCompleted(), self.rank );

   if self:isCompleted() then

      if self.goal then

         self.goal:deactivate();
         self.goal:delete();
         self.goal = nil;
         
      end

      App.data:nextAchievementRank( self.template.id, self:getReward() );

      self.rank = App.data:getAchievementRank(self.template.id) or 1;

      print( "Achievement:nextRank-2", self.rank, self.template.rank[self.rank] );

      self:_CreateGoal( true );

   end
end

----------------------------------------------------------------------

function Achievement:_IsCompleted()

   if self.goal then
      if self.goal:isCompleted() then
         return true;
      end
   end

   return false;

end

----------------------------------------------------------------------