require('engine/event/event-manager')
require('engine/object/object')

----------------------------------------------------------------------

local Base        = Object;
AchievementGoal   = Base:new();

----------------------------------------------------------------------

function AchievementGoal:new( achievement, template  )

   -- print("AchievementGoal:new", achievement, template);

   local goal           = Base.new( self );

   goal.template        = template;
   goal.achievement     = achievement;
   goal.active          = false;
   goal.progress        = 0;
   goal.progressSaved   = 0;

   return goal;

end

----------------------------------------------------------------------

function AchievementGoal:delete()

end

----------------------------------------------------------------------

function AchievementGoal:activate( reset )

   Debug.assert( not self:isActive(), "AchievementGoal:activate -> not self:isActive()" );

   self.active    = true;

   --Make sure we are not done with the achievement
   if self.achievement and self.achievement:isFinished() then
      self.progress = -1;
   else
      if reset then
         self.progress = 0;
      else
         self.progress  = App.data:getAchievementProgress(self.achievement.template.id) or 0;
      end
   end

   self.progressSaved = self.progress;

   if not self:isCompleted() then
      EventMgr:addListener( self, self.onGameStart ,  "game-start" );
      EventMgr:addListener( self, self.onGameOver ,   "game-over"  );
      EventMgr:addListener( self, self.onLogin,       "login-success" );
   end
end

----------------------------------------------------------------------

function AchievementGoal:deactivate()

   self.active = false;

   EventMgr:rmvListener( self );

end

----------------------------------------------------------------------

function AchievementGoal:onDataServerMerged()

   -- print("Achievement:onDataServerMerged", App.data:getAchievementProgress(self.achievement.template.id), self.progress);

   if not self:isCompleted() then

      local newProg  = App.data:getAchievementProgress(self.achievement.template.id) or 0;

      if newProg < 0 then
         self:_OnCompleted();
      else
         self.progress = newProg;
      end
   end

end

----------------------------------------------------------------------

function AchievementGoal:achievementScreenFocus( )

   -- print("AchievementGoal:achievementScreenFocus");

   --Reset if not cumulative
   if not self:isCompleted() then
      if not self.template.cumulative then
         self.progress = 0;
      end
   end

end

----------------------------------------------------------------------

function AchievementGoal:onLogin( event )


end

----------------------------------------------------------------------

function AchievementGoal:onGameStart(event)

   --Reset if not cumulative
   if not self:isCompleted() then
      if not self.template.cumulative then
         self.progress = 0;
      end
   end

end

----------------------------------------------------------------------

function AchievementGoal:onGameOver(event)

   -- print("AchievementGoal:onGameOver", self.achievement.template.id, not self:isCompleted() );

   if not self:isCompleted() then

      -- print("AchievementGoal:onGameOver-2", self.template.gameOverSave);

      -- print("AchievementGoal:onGameOver-3", self.achievement.template.id, self.progress, self._progressGameSaved );

      local gp = App.data:deltaValue( App.data:createAchievemetGpId( self.achievement.template.id  ), 1 );

      -- print("AchievementGoal:onGameOver-4", App.data:createAchievemetGpId( self.achievement.template.id  ), gp );

      self:_SaveProgress();

   end

end

----------------------------------------------------------------------

function AchievementGoal:isActive()

   return self.active;

end

----------------------------------------------------------------------

function AchievementGoal:isCompleted()

   return self.progress < 0;

end

----------------------------------------------------------------------

function AchievementGoal:getProgress()

   return self.progress;

end

----------------------------------------------------------------------

function AchievementGoal:getObjective()

   if self.template then
      return self.template.objective or -1;
   end

   return -1;


end

----------------------------------------------------------------------

function AchievementGoal:getDesc()

   if self.template then
      return self.template.description;
   end

   return;

end

----------------------------------------------------------------------

function AchievementGoal:getTitle()

   if self.template then
      return self.template.title;
   end

   return;

end

----------------------------------------------------------------------

function AchievementGoal:getIcon()

   if self.template then
      return self.template.icon;
   end

   return;

end

----------------------------------------------------------------------

function AchievementGoal:getRewardTxt()

   if self.template then
      return self.template.rewardTxt;
   end

   return;

end

----------------------------------------------------------------------

function AchievementGoal:getAppDataProgressId()

   if self.achievement and self.achievement.id  then
      return "achv-goal-"..self.achievement.id.."-progress";
   else
      Debug.assert( self.id, "AchievementGoal:getAppDataProgressId -> self.achievement and self.achievement.id" );
   end

end

----------------------------------------------------------------------

function AchievementGoal:_SaveProgress()

   -- print("AchievementGoal:_SaveProgress", self.achievement, self.progressSaved, self.progress);

   Debug.assert( not self:isCompleted(), "AchievementGoal:_SaveProgress -> not self:isCompleted()");

   if self.achievement then

      if self.progressSaved ~= self.progress then

         self.progressSaved = self.progress;
         self.achievement:onGoalProgress(self, self.progress );
      end

   else

      Debug.assert( self.achievement, "AchievementGoal:_SaveProgress -> self.achievement");

   end

end


----------------------------------------------------------------------

function AchievementGoal:_OnCompleted()

   -- print("AchievementGoal:_OnCompleted", self.achievement);

   Debug.assert( not self:isCompleted(), "AchievementGoal:_OnCompleted -> not self:isCompleted()");

   --If there is no achievement, dont mark as completed
   -- to avoid getting in a bad state where the goal think it is done but not the achievement
   if self.achievement then

      --Make as completed
      self.progress        = -1;
      self.progressSaved   = self.progress;

      EventMgr:rmvListener( self );

      self.achievement:onGoalCompleted(self);
   else
      Debug.assert(self.achievement, "AchievementGoal:_OnCompleted -> self.achievement");
   end

end

----------------------------------------------------------------------
