
----------------------------------------------------------------------
require('app/ad/server/ad-retriever')
require('app/ad/vungle/vungle-controller-android')
require('app/ad/vungle/vungle-controller-ios')
require('app/ad/chartboost/chartboost-video-controller')
require('app/ad/chartboost/chartboost-interstitial-controller')
require('app/ad/adcolony/adcolony-video-controller')
require('app/ad/applovin/applovin-video-controller')
require('app/ad/tapjoy/tapjoy-video-controller')
require('app/ad/fyber/fyber-video-controller')
require('app/ad/fyber/fyber-interstitial-controller')
require('app/ad/ironsource/ironsource-video-controller')
require('app/ad/ironsource/ironsource-interstitial-controller')
require('app/ad/flurry/flurry-video-controller')
require('app/ad/flurry/flurry-interstitial-controller')
require('engine/utility/tableUtil')
require('app/ad/native-ad-retriever')

----------------------------------------------------------------------

AdMgr  =
{
   _active = false;
   _downloadTimer = 0;
   IsAdPlaying = false;
};

----------------------------------------------------------------------

local VIDEO_VIEW_RESET_TIME = 60*60*24;

----------------------------------------------------------------------

function AdMgr:init()

   self._active                         = false;
   self._enable                         = false;

   self._video                          = {};
   self._video.enable                   = false;
   self._video.networkWeight            = {};
   self._video.networkCtrl              = {};

   self._interstitial                   = {};
   self._interstitial.enable            = false;
   self._interstitial.networkWeight     = {};
   self._interstitial.networkCtrl       = {};

   self._banner                         = {};
   self._banner.enable                  = false;
   self._banner.networkWeight           = {};
   self._banner.networkCtrl             = {};
   self._banner.settings                = {};
   self.IsAdPlaying                     = false;

   EventMgr:addListener(self, self._onCloudAdsInfo, "cloud:master-data-ads-info" );

   self:_ensureVideoViewDataExists();

   self._adRetriever = NativeAdRetriever:new();
end

----------------------------------------------------------------------

function AdMgr:activate( force )

   if ( not self._active ) or force then

      self._active         = true;

      self._showAdsTimer   = nil;
      --should be nil, we will set later
      self._downloadTimer  = self._onActivateDownloadDelay;

      self:_createAndActivateAllVideoNetworkCtrl();
      self:_createAndActivateAllInterstitialNetworkCtrl();

   end
end

----------------------------------------------------------------------

function AdMgr:deactivate()
   self._active = false;
end

----------------------------------------------------------------------

function AdMgr:isActive(  )
   return self._active;
end

----------------------------------------------------------------------

function AdMgr:isEnable(  )
   return self._enable;
end

----------------------------------------------------------------------

function AdMgr:update( dt )
   Debug.assert(dt);

end

----------------------------------------------------------------------

-- Pick a random value from the first table using a weighted random determined by the second table.
-- Additionally, the user can pass in a function to filter out values.
-- The function should return false if the object should be filtered, true otherwise.
-- The user can also forward additional arguments to the filter function if needed.
function AdMgr:_pickWeightedRandom(objTable, weightTable, filterFunc, ...)

   Debug.assert(objTable and type(objTable) == "table");
   Debug.assert(weightTable and type(objTable) == "table");

   local pick = nil;
   local sum = 0;

   for k, obj in pairs(objTable) do
      local weight = weightTable[k];

      if weight and weight > 0 then
         if obj then

            local shouldFilter = false;

            if filterFunc then
               shouldFilter = not filterFunc(obj, unpack(arg));
            end

            if not shouldFilter then
               local rand = math.random();

               sum = sum + weight;

               if (sum * rand) <= weight then
                  pick = obj
               end
            end
         end
      end
   end

   return pick;
end

----------------------------------------------------------------------

function AdMgr:isInterstitialEnable(  )
   return self:isActive() and self._interstitial and self._interstitial.enable and self._enable;
end

----------------------------------------------------------------------

function AdMgr:isInterstitialAvailable( location )

   if self:isInterstitialEnable() then
      if self._interstitial and self._interstitial.networkCtrl and self._interstitial.networkWeight then
         for k,ctrl in pairs(self._interstitial.networkCtrl) do
            if ctrl and self._interstitial.networkWeight[k] and self._interstitial.networkWeight[k] > 0 and ctrl:isInterstitialAvailable(location) then
               return true;
            end
         end
      end
   end

end

----------------------------------------------------------------------

function AdMgr:cacheInterstitial(location)

   if self._interstitial and self._interstitial.networkCtrl and self._interstitial.networkWeight then

      for k,ctrl in pairs(self._interstitial.networkCtrl) do

         local w = self._interstitial.networkWeight[k];

         if w and w > 0 then
            ctrl:cacheInterstitial(location);
         end
      end

   end

end

----------------------------------------------------------------------

function AdMgr:showInterstitial( location, callObj, callSuccess, callFailure )

   if self:isInterstitialEnable() then

      local ctrl = self:_pickCtrlToShowInterstitial(location);

      if ctrl then

		 print("Showing interstitial at location: " .. location);

         self._interstitialShowing = location or "yes";

         if ctrl:showInterstitial( location
                                 ,
                                    nil
                                 ,
                                    function()
                                       --to avoid having this portion of the code called before we return from ctrl:showInterstitial
                                       delay(0.01,
                                             function()
                                                self._interstitialShowing = nil;
                                                if callSuccess then
                                                   callSuccess(callObj);
                                                end
                                       end );
                                    end
                                 ,
                                    function()
                                       --to avoid having this portion of the code called before we return from ctrl:showInterstitial
                                       delay(0.01,
                                             function()
                                                self._interstitialShowing = nil;
                                                if callFailure then
                                                   callFailure(callObj);
                                                end
                                             end );
                                    end
                                 ) then

            return true;

         else

            self._interstitialShowing = nil;

         end
      end
   end

end

----------------------------------------------------------------------

function AdMgr:isInterstitialShowing( location )

   if location then
      return self._interstitialShowing == location;
   else
      return self._interstitialShowing ~= nil;
   end

end

----------------------------------------------------------------------

function AdMgr:_pickCtrlToShowInterstitial(location)

   Debug.assertType(self._interstitial, "table", "self._interstitial")
   Debug.assertType(self._interstitial.networkCtrl, "table", "self._interstitial.networkCtrl")
   Debug.assertType(self._interstitial.networkWeight, "table", "self._interstitial.networkWeight")

   local filterFunc = function(obj, location)
      return obj:isInterstitialAvailable(location);
   end

   return self:_pickWeightedRandom(self._interstitial.networkCtrl,
                                   self._interstitial.networkWeight,
                                   filterFunc, location);

end

----------------------------------------------------------------------

function AdMgr:isVideoEnable(  )

   return self:isActive() and self._video.enable and self._enable;

end

----------------------------------------------------------------------

function AdMgr:isVideoAvailable()

   if self:isVideoEnable() then
      if self._video.networkCtrl and self._video.networkWeight then
         for k, ctrl in pairs(self._video.networkCtrl) do

			-- Ensure ctrl:isVideoAvailable returns a boolean; we had issues with AdColony
			Debug.assert(type(ctrl:isVideoAvailable()) == "boolean", "isVideoAvailable didn't return a boolean: " .. tostring(k));

            if ctrl and self._video.networkWeight[k] and self._video.networkWeight[k] > 0 and ctrl:isVideoAvailable() then
               return true;
            end
         end
      end
   end

   return false;

end

----------------------------------------------------------------------

function AdMgr:playVideo( callObj, callSuccess, callFailure )

   self.IsAdPlaying = false;


   if self:isVideoEnable() then

	  self:_ensureVideoViewDataExists();
	  self:_removeExpiredVideoViewsFromData();

      local ctrl = self:_pickCtrlToPlayVideo();

      if ctrl then

         AudioMgr:mute();

		 AdMgr:_logVideoViewTimestamp(self:_getNameFromVideoController(ctrl));

		 local playSuccess = ctrl:playVideo( nil,
											 function()
												AudioMgr:unMute();
												if callSuccess then
												   callSuccess(callObj);
												end
											 end
											 ,
											 function()
												AudioMgr:unMute();
												if callFailure then
												   callFailure(callObj);
												end
											 end
		 )

         if playSuccess then
            self.IsAdPlaying = true;
            return true;
         else
            AudioMgr:unMute();
         end

      end

   end

   return false;

end

----------------------------------------------------------------------

function AdMgr:_pickCtrlToPlayVideo()

   if self._video and self._video.networkCtrl and self._video.networkWeight then

      local filterFunc = function(obj)
         return obj:isVideoAvailable() and AdMgr:_hasDailyVideosRemaining(AdMgr:_getNameFromVideoController(obj));
      end

      return self:_pickWeightedRandom(self._video.networkCtrl,
                                      self._video.networkWeight,
                                      filterFunc);
   end

end

----------------------------------------------------------------------

local function timeIsExpired(timestamp)
   local time = TimeMgr:getTimeGMT() or TimeMgr:getTimeLocalGMT();

   local timeSinceTimestamp = time - timestamp;

   return timeSinceTimestamp >= VIDEO_VIEW_RESET_TIME;
end

function AdMgr:_ensureVideoViewDataExists()
   local views = App.data:getValue("daily-video-views");

   views = views or {}

   for name, _ in pairs(self._video.networkCtrl) do
	  views[name] = views[name] or {};
   end

   App.data:setValue("daily-video-views", views);
end

function AdMgr:_removeExpiredVideoViewsFromData()
   local views = App.data:getValue("daily-video-views");

   for name, viewList in pairs(views) do
	  removeIf(viewList, timeIsExpired)
   end

   App.data:setValue("daily-video-views", views);
end

function AdMgr:_getDailyVideoViewCount(controllerName)
   local views = App.data:getValue("daily-video-views");

   if not views then
	  return 0;
   end

   if not views[controllerName] then
	  return 0;
   end

   return #views[controllerName];
end

function AdMgr:_getNameFromVideoController(videoController)
   Debug.assert(videoController.name);
   return videoController.name;
end

function AdMgr:_hasDailyVideosRemaining(controllerName)

   if not self._video.dailyViewCaps then
	  -- Client has a cloud file that doesn't have view cap data
	  return true;
   end

   if not self._video.dailyViewCaps[controllerName] then
	  -- Video provider has no cap
	  return true;
   end

   return self._video.dailyViewCaps[controllerName] - self:_getDailyVideoViewCount(controllerName) > 0;
end

function AdMgr:_logVideoViewTimestamp(controllerName)
   local views = App.data:getValue("daily-video-views");

   Debug.assert(type(views[controllerName]) == "table", "table expected, found: " .. type(views[controllerName]));

   table.insert(views[controllerName], TimeMgr:getTimeGMT() or TimeMgr:getTimeLocalGMT());
end

----------------------------------------------------------------------

function AdMgr:_onCloudAdsInfo( data )

   local header                  = (data or {}).header;
   local content                 = (data or {}).content;

   self._enable                  = content.enable;

   local video                   = content.video or {};
   self._video.enable            = video.enable;
   self._video.forceGiveReward   = video.forceGiveReward;
   self._video.networkWeight     = video.networkWeight;
   self._video.dailyViewCaps     = video.dailyViewCaps or {};
   if self._video.networkWeight and self:isActive() then
      self:_createAndActivateAllVideoNetworkCtrl();
   end

   local interstitial                  = content.interstitial or {};
   self._interstitial.enable           = interstitial.enable;
   self._interstitial.networkWeight    = interstitial.networkWeight;
   if self._interstitial.networkWeight and self:isActive() then
      self:_createAndActivateAllInterstitialNetworkCtrl();
   end

   self._adRetriever:applyCloudData(content.native or {});

   self._adRetriever:attemptAdFetch();

end

----------------------------------------------------------------------

function AdMgr:_createVideoCtrl( name )

   print("AdMgr: createVideoCtrl name:"..name)
   if name == "AdColony" then
      return AdColonyController:new();

   elseif name == "ChartBoost" then
      return AdChartBoostController:new();

   elseif name == "Vungle" then
	  -- TODO Use constants instead of hardcoding IDs and package paths
      if MOAIEnvironment.osBrand == "iOS" then
         return AdVungleControllerIOS:new("646803444");
      elseif MOAIEnvironment.osBrand == "Android" then
         return AdVungleControllerAndroid:new("me.tango.roadriot");
      end

   elseif name == "FlurryAds" then
      return FlurryAdsController:new();

   elseif name == "AppLovin" then
      return AppLovinController:new();

   elseif name == "Tapjoy" then
      return TapjoyController:new();
   elseif name == "Fyber" then
      return FyberController:new();
   elseif name == "IronSource" then
      return IronSourceController:new();
   end

   Debug.error("Unhandled video controller of name: \"" .. name .. "\"");

end

----------------------------------------------------------------------

function AdMgr:_createAndActivateAllVideoNetworkCtrl()

   Debug.assert(self:isActive(), "AdMgr:_createAndActivateAllVideoNetworkCtrl -> self:isActive()");

   if not self._video then
	  Debug.error("Failed to create video controllers; no cloud data!");
	  return;
   end

   if self._video.networkWeight then
      for k,v in pairs(self._video.networkWeight) do

         if v > 0 then

            --If this controller didnt exist yet, create one
            if self._video.networkCtrl[k] == nil then
               self._video.networkCtrl[k] = self:_createVideoCtrl(k);
               if self._video.networkCtrl[k] then
				  self._video.networkCtrl[k].name = k;
                  self._video.networkCtrl[k]:start();
               end
            end

         else
            self._video.networkCtrl[k] = nil;
         end

      end
   end

   if self._video.dailyViewCaps then
	  for k, v in pairs(self._video.dailyViewCaps) do
		 local controller = self._video.networkCtrl[k]
		 if controller then
			controller.dailyViewCap = v;
		 end
	  end
   end

end

----------------------------------------------------------------------

function AdMgr:_createInterstitialCtrl( name )

   if name == "ChartBoost" then
      return AdChartBoostInterstitialController:new();

   elseif name == "FlurryAds" then
     return AdFlurryInterstitialController:new();

   elseif name == "Fyber" then
     return AdFyberInterstitialController:new();
   elseif name == "Fyber" then
     return AdFyberInterstitialController:new();
   elseif name == "IronSource" then
     return AdIronSourceInterstitialController:new();
   end

   Debug.assert( false, "AdMgr:_createInterstitialCtrl -> name: ".. tostring(name) );

end

----------------------------------------------------------------------

function AdMgr:_createAndActivateAllInterstitialNetworkCtrl()

   Debug.assert(self:isActive(), "AdMgr:_createAndActivateAllInterstitialNetworkCtrl -> self:isActive()");

   if self._interstitial and self._interstitial.networkWeight then
      for k,v in pairs(self._interstitial.networkWeight) do

         if v > 0 then

            --If this controller didnt exist yet, create one
            if self._interstitial.networkCtrl[k] == nil then
               self._interstitial.networkCtrl[k] = self:_createInterstitialCtrl(k);
               if self._interstitial.networkCtrl[k] then
                  self._interstitial.networkCtrl[k]:start();
               end
            end

         else

            self._interstitial.networkCtrl[k] = nil;

         end
      end
   end

end

----------------------------------------------------------------------

function AdMgr:shouldForceVideoReward()
   Debug.assert(type(self._video.forceGiveReward) == "boolean");
   return self._video.forceGiveReward;
end

----------------------------------------------------------------------

----------------
-- NATIVE ADS --
----------------

----------------------------------------------------------------------

-- Returns true if this call results in an ad fetch.
function AdMgr:attemptNativeAdFetch(numAds)

   if not self._adRetriever then
      -- adRetriever is either disabled or failed initialization.
      return false;
   end

   return self._adRetriever:attemptAdFetch(numAds);

end

----------------------------------------------------------------------

function AdMgr:findNativeAdWithUID(uid)

   if not self._adRetriever then
      -- adRetriever is either disabled or failed initialization.
      return nil;
   end

   return self._adRetriever:findAdWithUID(uid);

end

----------------------------------------------------------------------

function AdMgr:handleNativeAdImpression(ad)

   if not self._adRetriever then
      -- adRetriever is either disabled or failed initialization.
      return;
   end

   self._adRetriever:handleAdImpression(ad)

end

----------------------------------------------------------------------

function AdMgr:handleNativeAdClick(ad)

   if not self._adRetriever then
      -- adRetriever is either disabled or failed initialization.
      return;
   end

   self._adRetriever:handleAdClick(ad);

end

----------------------------------------------------------------------

function AdMgr:getDesiredNativeAdSpacing()

   if not self._adRetriever then
      -- adRetriever is either disabled or failed initialization.
      return nil;
   end

   return self._adRetriever:getDesiredAdSpacing();

end

----------------------------------------------------------------------

function AdMgr:sendUserToURL(url)

   if not self._adRetriever then
      -- adRetriever is either disabled or failed initialization.
      return;
   end

   self._adRetriever:sendUserToURL(url);

end

----------------------------------------------------------------------

function AdMgr:getNativeAds()

   if not self._adRetriever then
      -- adRetriever is either disabled or failed initialization.
      return nil;
   end

   return self._adRetriever:getNativeAds();

end

----------------------------------------------------------------------

function AdMgr:getNativeAdImageFromUID(uid)

   Debug.assertType(uid, {"number", "string"}, "AdMgr:getNativeAdImageFromUID -> uid");

   if not self._adRetriever then
      -- adRetriever is either disabled or failed initialization.
      return nil;
   end

   return self._adRetriever:getLocalAdImageFromAdUID(uid);

end

----------------------------------------------------------------------

function AdMgr:clearAdImpressions()

   if not self._adRetriever then
      -- adRetriever is either disabled or failed initialization.
      return;
   end

   self._adRetriever:clearAdImpressions();

end
