
----------------------------------------------------------------------
require('engine/object/object')
----------------------------------------------------------------------

local Base         = Object;

AdColonyController = Base:new();

----------------------------------------------------------------------

function AdColonyController:new()
   local obj         = Base.new( self );
   obj._adcolony   = MOAIAdColony;
   Debug.assert(obj._adcolony, "_adcolony was nil in AdColonyController:new");
   return obj;
end

----------------------------------------------------------------------

function AdColonyController:start(  )
   self._started = true;
   self:_setupListeners();
end

----------------------------------------------------------------------

function AdColonyController:isStarted(  )
   return self._started;
end

----------------------------------------------------------------------

function AdColonyController:isVideoAvailable()
   Debug.assert(self._adcolony, "self.adcolony was nil!");
   return self._adcolony.isRewardedVideoAvailable();
end

----------------------------------------------------------------------

function AdColonyController:isPlayingVideo(  )
   return self._playing;
end

----------------------------------------------------------------------

function AdColonyController:playVideo( callObj, callSuccess, callFailure )

   EventMgr:postEvent( { network="AdColony" }, "reward-video-attempt");

   if (self._started) and (self._adcolony) and (not self._playing) then

      --it doesnt hurt to do it a few times
      self:_setupListeners();

      if self:isVideoAvailable(  ) then
         --doesnt return a value

         self._deserveReward = nil;
         self._playing       = true;
         self._showing       = nil;
         self._callObj       = callObj;
         self._callSuccess   = callSuccess;
         self._callFailure   = callFailure;

         --Call after we have set our variable because it will call some of the callback right away
         self._adcolony.showRewardedVideo();

         --Because we cannot be 100% certain the video will show (vungle doest return a value when calling play video)
         -- make sure we check the video is started after a little while, otherwise the user could get in a weird situation

         --increase the time limit to see if it will help reducing the number of errors we are getting
         delay( 5, function()
                   if (self._playing) and (not self._showing) then
                      self:_onVideoNotShowing();
                   end
         end );

         return true;
      else

         EventMgr:postEvent( {result="failure", network="AdColony", reason="AdColony:self:isVideoAvailable()"}, "reward-video-failure" );

      end

   else

      EventMgr:postEvent( {result="failure", network="AdColony", reason="AdColony:(self._started) and (self._adcolony) and (not self._playing)"}, "reward-video-failure" );

   end

   return false;
end

----------------------------------------------------------------------

function AdColonyController:_startSession( )

end

----------------------------------------------------------------------

function AdColonyController:_setupListeners()

   if (self._adcolony) then
      self._adcolony.setListener(  MOAIAdColony.DID_V4VC_REWARD_REWARDED_VIDEO
                                   ,
                                   function( amount )
                                      self:_onVideoRewarded( amount )
                                   end
      );
      self._adcolony.setListener(  MOAIAdColony.DID_V4VC_STARTED_REWARDED_VIDEO
                                   ,
                                   function()
                                      self:_onVideoStart();
                                   end
      );
      self._adcolony.setListener(  MOAIAdColony.DID_V4VC_FINISHED_REWARDED_VIDEO
                                   ,
                                   function( shown )
                                      if  self._playing  then
                                         self:_onVideoEnd();
                                      end
                                   end
      );
   else

      Debug.error("AdColonyController:_setupListeners -> self._adcolony was nil" );

   end

end

----------------------------------------------------------------------

function AdColonyController:_onVideoPlayed( played )

   self._deserveReward = self._deserveReward or played;

end

----------------------------------------------------------------------

function AdColonyController:_onVideoStart()

   self._showing = true;

end

----------------------------------------------------------------------

function AdColonyController:_onVideoEnd()

   delay( 0.1, function()
                  self:_onVideoDone( );
               end                        );
end

----------------------------------------------------------------------

function AdColonyController:_onVideoRewarded( reward )

   self._deserveReward = reward;

end

----------------------------------------------------------------------

function AdColonyController:_onVideoNotShowing()

   self:_onVideoDone( );

end

----------------------------------------------------------------------

function AdColonyController:_onVideoDone( )

   if  self._deserveReward then

      EventMgr:postEvent( {result="success", network="AdColony"}, "reward-video-success" );

      if self._callSuccess then
         self._callSuccess(self._callObj);
      end

   else

      EventMgr:postEvent(
         {
            result="failure",
            network="AdColony",
            reason="AdColony:_deserveReward:  playing=" .. tostring(self._playing) .. ":  showing=" .. tostring(self._showing)
         },
         "reward-video-failure" );

      if self._callFailure then
         self._callFailure(self._callObj);
      end

   end

   self._playing        = nil;
   self._deserveReward  = nil;
   self._showing        = nil;
   self._callObj        = nil;
   self._callSuccess    = nil;
   self._callFailure    = nil;

end


----------------------------------------------------------------------
