
----------------------------------------------------------------------
require('engine/object/object')
----------------------------------------------------------------------

local Base         = Object;

AppLovinController = Base:new();

----------------------------------------------------------------------

function AppLovinController:new()
   local obj         = Base.new( self );
   obj._applovin   = MOAIAppLovin;
   Debug.assert(obj._applovin, "_applovin was nil in AppLovinController:new");
   return obj;
end

----------------------------------------------------------------------

function AppLovinController:start(  )
   self._started = true;
   self:_setupListeners();
end

----------------------------------------------------------------------

function AppLovinController:isStarted(  )
   return self._started;
end

----------------------------------------------------------------------

function AppLovinController:isVideoAvailable()
   Debug.assert(self._applovin, "self.adcolony was nil!");
   return self._applovin.isRewardedVideoAvailable();
end

----------------------------------------------------------------------

function AppLovinController:isPlayingVideo(  )
   return self._playing;
end

----------------------------------------------------------------------

function AppLovinController:playVideo( callObj, callSuccess, callFailure )

   EventMgr:postEvent( { network="AppLovin" }, "reward-video-attempt");

   if (self._started) and (self._applovin) and (not self._playing) then

      --it doesnt hurt to do it a few times
      self:_setupListeners();

      if self:isVideoAvailable(  ) then
         --doesnt return a value

         self._deserveReward = nil;
         self._playing       = true;
         self._showing       = nil;
         self._callObj       = callObj;
         self._callSuccess   = callSuccess;
         self._callFailure   = callFailure;

         --Call after we have set our variable because it will call some of the callback right away
         self._applovin.showRewardedVideo();

         --Because we cannot be 100% certain the video will show (vungle doest return a value when calling play video)
         -- make sure we check the video is started after a little while, otherwise the user could get in a weird situation

         --increase the time limit to see if it will help reducing the number of errors we are getting
         delay( 5, function()
                   if (self._playing) and (not self._showing) then
                      self:_onVideoNotShowing();
                   end
         end );

         return true;
      else

         EventMgr:postEvent( {result="failure", network="AppLovin", reason="AppLovin:self:isVideoAvailable()"}, "reward-video-failure" );

      end

   else

      EventMgr:postEvent( {result="failure", network="AppLovin", reason="AppLovin:(self._started) and (self._applovin) and (not self._playing)"}, "reward-video-failure" );

   end

   return false;
end

----------------------------------------------------------------------

function AppLovinController:_startSession( )

end

----------------------------------------------------------------------

function AppLovinController:_setupListeners()

   if (self._applovin) then

      self._applovin.setListener( MOAIAppLovin.DID_V4VC_STARTED_REWARDED_VIDEO,
                                  function()
                                     self:_onVideoStart();
                                  end
      );

      self._applovin.setListener( MOAIAppLovin.DID_V4VC_FINISHED_REWARDED_VIDEO,
                                  function( shown )
                                     print("DID_V4VC_FINISHED_REWARDED_VIDEO : shown -> ", shown);
                                     self:_onVideoRewarded(shown);
                                  end
      );

      self._applovin.setListener( MOAIAppLovin.DID_DISMISS_AD,
                                  function( shown )
                                     if self._playing then
                                        self:_onVideoEnd();
                                     end
                                  end
      );

   else

      Debug.error("AppLovinController:_setupListeners -> self._applovin was nil" );

   end

end

----------------------------------------------------------------------

function AppLovinController:_onVideoPlayed( played )

   self._deserveReward = self._deserveReward or played;

end

----------------------------------------------------------------------

function AppLovinController:_onVideoStart()

   print("AppLovinController:_onVideoStart");

   self._showing = true;

end

----------------------------------------------------------------------

function AppLovinController:_onVideoEnd()

   print("AppLovinController:_onVideoEnd");

   self:_onVideoDone( );
end

----------------------------------------------------------------------

function AppLovinController:_onVideoRewarded( reward )

   self._deserveReward = reward;

end

----------------------------------------------------------------------

function AppLovinController:_onVideoNotShowing()

   self:_onVideoDone( );

end

----------------------------------------------------------------------

function AppLovinController:_onVideoDone( )

   if  self._deserveReward then

      EventMgr:postEvent( {result="success", network="AppLovin"}, "reward-video-success" );

      if self._callSuccess then
         self._callSuccess(self._callObj);
      end

   else

      EventMgr:postEvent(
         {
            result="failure",
            network="AppLovin",
            reason="AppLovin:_deserveReward:  playing=" .. tostring(self._playing) .. ":  showing=" .. tostring(self._showing)
         },
         "reward-video-failure" );

      if self._callFailure then
         self._callFailure(self._callObj);
      end

   end

   self._playing        = nil;
   self._deserveReward  = nil;
   self._showing        = nil;
   self._callObj        = nil;
   self._callSuccess    = nil;
   self._callFailure    = nil;

end

----------------------------------------------------------------------
