
----------------------------------------------------------------------
require('engine/object/object')
----------------------------------------------------------------------

local Base                           = Object;

AdChartBoostInterstitialController   = Base:new();

----------------------------------------------------------------------

function AdChartBoostInterstitialController:new()

   local obj         = Base.new( self );
   obj._chartBoost   = MOAIChartBoost;

   return obj;
end

----------------------------------------------------------------------

function AdChartBoostInterstitialController:start(  )

   self:_setupListeners();
	self._started = true;

end

----------------------------------------------------------------------

function AdChartBoostInterstitialController:isStarted(  )
	return self._started;
end

----------------------------------------------------------------------

function AdChartBoostInterstitialController:isInterstitialAvailable( location )

   Debug.assertType(self._chartBoost, {"table", "userdata"}, "AdChartBoostInterstitialController:isInterstitialAvailable -> self._chartBoost")

   return self._chartBoost.hasInterstitial(location);
end

----------------------------------------------------------------------

function AdChartBoostInterstitialController:isIntertstitialShowing(  )
   return self._showing;
end

----------------------------------------------------------------------

function AdChartBoostInterstitialController:cacheInterstitial( location )

   if self._chartBoost then
      self._chartBoost.cacheInterstitial(location);
   else
      Debug.assert(self._chartBoost, "AdChartBoostInterstitialController:cacheInterstitial->self._chartBoost");
   end

end

----------------------------------------------------------------------

function AdChartBoostInterstitialController:showInterstitial( location, callObj, callSuccess, callFailure )

   if (not self._trying) and (not self._showing) then

      if self:isInterstitialAvailable( location ) then

         EventMgr:postEvent( { network="Chartboost", location=location }, "interstitial-attempt");

         local attemptID      = (self._attemptID or 0 ) + 1;

         self._trying         = true;
         self._showing        = nil;
         self._callObj        = callObj;
         self._callSuccess    = callSuccess;
         self._callFailure    = callFailure;
         self._attemptID      = attemptID;

         self._chartBoost.showInterstitial(location);


         delay( 2.0,
               function()
               --because we have some delay, it is conceivable that we would try to open another interstitial by now
               -- and we want to make sure this check is for the appropriate attempt

               if attemptID == self._attemptID then
                  if (self._trying) and (not self._showing ) then
					self:_onInterstitialFail(location, "time elapsed without showing");
                     end
                  end

               end );

         return self._trying or self._showing;
      end
   end
end

----------------------------------------------------------------------

function AdChartBoostInterstitialController:_setupListeners()

   if (self._chartBoost) then

      self._chartBoost.setListener(  MOAIChartBoost.DID_CACHE_INTERSTITIAL
                                 ,
                                    function( location )
                                    end
                                 );

       self._chartBoost.setListener(  MOAIChartBoost.DID_DISMISS_INTERSTITIAL
                                 ,
                                    function( location )
                                       self:_onInterstitialSuccess( location );
                                    end
                                 );

       self._chartBoost.setListener(  MOAIChartBoost.DID_CLOSE_INTERSTITIAL
                                 ,
                                    function( location )
                                    end
                                 );

       self._chartBoost.setListener(  MOAIChartBoost.DID_CLICK_INTERSTITIAL
                                 ,
                                    function( location )
                                    end
                                 );

       self._chartBoost.setListener(  MOAIChartBoost.DID_DISPLAY_INTERSTITIAL
                                 ,
                                    function( location )
                                       self:_onInterstitialShow( location );
                                    end
                                 );

       self._chartBoost.setListener(  MOAIChartBoost.DID_FAIL_TO_LOAD_INTERSTITIAL
                                 ,
                                    function( location, _error )
                                      --Do nothing, this is a caching error only
                                    end
                                 );

   else

      Debug.assert(self._chartBoost, "AdChartBoostInterstitialController:_setupListeners -> self._chartBoost"      );

   end

end

----------------------------------------------------------------------

function AdChartBoostInterstitialController:_onInterstitialShow( location )

   if (self._trying)  and ( not self._showing ) then

      EventMgr:postEvent( {result="success", network="Chartboost", location=location}, "interstitial-success" );

      self._trying  = nil;
      self._showing = true;

   else

      Debug.assert( self._trying,      "AdChartBoostInterstitialController:_onInterstitialShow -> self._trying" .. tostring(location) );
      Debug.assert( not self._showing, "AdChartBoostInterstitialController:_onInterstitialShow -> not self._showing" .. tostring(location) );

   end
end

----------------------------------------------------------------------

function AdChartBoostInterstitialController:_onInterstitialFail( location, reason )

   --Make sure we were trying to show an interstitial
   if (self._trying) and (not self._showing) then

      EventMgr:postEvent( {result="failure", network="Chartboost", location=location, reason=reason}, "interstitial-failure" );

      self:_onDone( false );

   end

end

----------------------------------------------------------------------

function AdChartBoostInterstitialController:_onInterstitialSuccess( location )

   --Make sure we were trying to show an interstitial
   if (not self._trying) and (self._showing) then
      self:_onDone( true );
   else
      Debug.assert(not self._trying,   "AdChartBoostInterstitialController:_onInterstitialSuccess -> not self._trying" .. tostring(location) );
      Debug.assert(self._showing,      "AdChartBoostInterstitialController:_onInterstitialSuccess -> self._showing"    .. tostring(location) );
   end

end

----------------------------------------------------------------------

function AdChartBoostInterstitialController:_onDone( success )

   if success then
      if self._callSuccess then
         self._callSuccess(self._callObj);
      end
   else
      if self._callFailure then
         self._callFailure(self._callObj);
      end
   end

   self._trying         = nil;
   self._showing        = nil;
   self._callObj        = nil;
   self._callSuccess    = nil;
   self._callFailure    = nil;

end


----------------------------------------------------------------------
