
----------------------------------------------------------------------
require('engine/object/object')
----------------------------------------------------------------------

local Base           = Object;

AdChartBoostController   = Base:new();

----------------------------------------------------------------------

function AdChartBoostController:new()
   local obj         = Base.new( self );
   obj._chartboost   = MOAIChartBoost;
   obj._location     = "roadriot_video_reward";
   return obj;
end

----------------------------------------------------------------------

function AdChartBoostController:start(  )

	self._chartboost.cacheRewardedVideo( self._location );
	self._started = true;

   self:_setupListeners();

end

----------------------------------------------------------------------

function AdChartBoostController:isStarted(  )
	return self._started;
end

----------------------------------------------------------------------

function AdChartBoostController:isVideoAvailable()
	return self._chartboost.hasRewardedVideo( self._location );
end

----------------------------------------------------------------------

function AdChartBoostController:isPlayingVideo(  )
   return self._playing;
end

----------------------------------------------------------------------

function AdChartBoostController:playVideo( callObj, callSuccess, callFailure )

   EventMgr:postEvent( { network="Chartboost" }, "reward-video-attempt");

   if (self._started) and (self._chartboost) and (not self._playing) then

      --it doesnt hurt to do it a few times
      self:_setupListeners();

      if self:isVideoAvailable(  ) then
         --doesnt return a value

         self._deserveReward  = nil;
         self._playing        = true;
         self._showing        = nil;
         self._callObj        = callObj;
         self._callSuccess    = callSuccess;
         self._callFailure    = callFailure;

         --Call after we have set our variable because it will call some of the callback right away
         self._chartboost.showRewardedVideo( self._location );

         --Because we cannot be 100% certain the video will show (vungle doest return a value when calling play video)
         -- make sure we check the video is started after a little while, otherwise the user could get in a weird situation

         delay( 5, function()
                     if (self._playing) and (not self._showing) then
                        self:_onVideoNotShowing();
                     end
                   end        );


         return true;
      else

         EventMgr:postEvent( {result="failure", network="Chartboost", reason="Chartboost:self:isVideoAvailable()"}, "reward-video-failure" );

      end

   else

      EventMgr:postEvent( {result="failure", network="Chartboost", reason="Chartboost:(self._started) and (self._chartboost) and (not self._playing)"}, "reward-video-failure" );

   end

end

----------------------------------------------------------------------

function AdChartBoostController:_startSession( )

end

----------------------------------------------------------------------

function AdChartBoostController:_setupListeners()

   if (self._chartboost) then
    self._chartboost.setListener(  MOAIChartBoost.DID_CACHE_REWARDED_VIDEO
                              ,
                                 function( location )
                                 end
                              );
    self._chartboost.setListener(  MOAIChartBoost.DID_CLICK_REWARDED_VIDEO
                              ,
                                 function( location )
                                 end
                              );
    self._chartboost.setListener(  MOAIChartBoost.DID_CLOSE_REWARDED_VIDEO
                              ,
                                 function( location )
                                    if  self._playing  then
	                                    self:_onVideoEnd();
	                                 end
                                 end
                              );
    self._chartboost.setListener(  MOAIChartBoost.DID_COMPLETE_REWARDED_VIDEO
                              ,
                                 function( location, reward )
                                    self:_onVideoPlayed( true );
                                 end
                              );
    self._chartboost.setListener(  MOAIChartBoost.DID_DISMISS_REWARDED_VIDEO
                              ,
                                 function( location )
                                    if  self._playing  then
                                       self:_onVideoEnd();
                                    end

                                 end
                              );
    self._chartboost.setListener(  MOAIChartBoost.DID_DISPLAY_REWARDED_VIDEO
                              ,
                                 function( location )
                                    self:_onVideoStart();
                                 end
                              );
    self._chartboost.setListener(  MOAIChartBoost.DID_FAIL_TO_LOAD_REWARDED_VIDEO
                              ,
                                 function( location, _error )
                                    --Do nothing, this is a caching error only
                                 end
                              );

   else

      Debug.assert(self._chartboost, "AdChartBoostController:_setupListeners -> self._chartboost"      );

   end

end

----------------------------------------------------------------------

function AdChartBoostController:_onVideoPlayed( played )

   self._deserveReward = self._deserveReward or played;

end

----------------------------------------------------------------------

function AdChartBoostController:_onVideoStart()

   self._showing = true;

end

----------------------------------------------------------------------

function AdChartBoostController:_onVideoEnd()

  -- Debug.assert( self._playing, " AdChartBoostController:_onVungleViewDisappeared -> self._playing ");

   if self._chartboost then
      self._chartboost.cacheRewardedVideo( self._location );
   else
      Debug.assert(self._chartboost, "AdChartBoostController:_onVideoPlayed -> self._chartboost");
   end

   delay( 0.1, function()
                  self:_onVideoDone( );
               end                        );

end

----------------------------------------------------------------------

function AdChartBoostController:_onVungleVideoView( reward )

   self._deserveReward = reward;

end

----------------------------------------------------------------------

function AdChartBoostController:_onVideoNotShowing()

   self:_onVideoDone( );

end

----------------------------------------------------------------------

function AdChartBoostController:_onVideoDone( )

   if  self._deserveReward then

      EventMgr:postEvent( {result="success", network="Chartboost"}, "reward-video-success" );

      if self._callSuccess then
         self._callSuccess(self._callObj);
      end

   else

      EventMgr:postEvent( {
                              result="failure",
                              network="Chartboost",
                              reason="Chartboost:_deserveReward:  playing=" .. tostring(self._playing) .. ":  showing=" .. tostring(self._showing)
                           },
                           "reward-video-failure" );

   if self._callFailure then
         self._callFailure(self._callObj);
      end

   end

   self._playing	    = nil;
   self._deserveReward  = nil;
   self._showing        = nil;
   self._callObj        = nil;
   self._callSuccess    = nil;
   self._callFailure    = nil;

end


----------------------------------------------------------------------