require('engine/object/object')
----------------------------------------------------------------------

local Base                           = Object;

AdFlurryInterstitialController   = Base:new();

----------------------------------------------------------------------

function AdFlurryInterstitialController:new()

   local obj    = Base.new( self );
   obj._flurry  = MOAIFlurry;

   obj._started = false

   return obj;
end

----------------------------------------------------------------------

function AdFlurryInterstitialController:start()
   self:_setupListeners();
   self._started = true;
end

----------------------------------------------------------------------

function AdFlurryInterstitialController:isStarted()
   return self._started;
end

----------------------------------------------------------------------

function AdFlurryInterstitialController:isInterstitialAvailable()
   Debug.assert(self._flurry, "Flurry was nil!");

   return self._flurry.isInterstitialAvailable();
end

----------------------------------------------------------------------

function AdFlurryInterstitialController:isInterstitialShowing()
   return self._showing;
end

----------------------------------------------------------------------

function AdFlurryInterstitialController:cacheInterstitial()
   -- Flurry ads are downloaded in the background auotmatically.
end

----------------------------------------------------------------------

function AdFlurryInterstitialController:showInterstitial(location, callObj, callSuccess, callFailure)

   if (not self._trying) and (not self._showing) then

      if self:isInterstitialAvailable() then

         EventMgr:postEvent( { network="FlurryAds", location=location }, "interstitial-attempt");

         local attemptID      = (self._attemptID or 0 ) + 1;

         self._trying         = true;
         self._showing        = nil;
         self._callObj        = callObj;
         self._callSuccess    = callSuccess;
         self._callFailure    = callFailure;
         self._attemptID      = attemptID;

         self._flurry.showInterstitial(location);

         delay( 2.0,
                function()
                   --because we have some delay, it is conceivable that we would try to open another interstitial by now
                   -- and we want to make sure this check is for the appropriate attempt

                   if attemptID == self._attemptID then
                      if (self._trying) and (not self._showing ) then
                         self:_onInterstitialFail(location, "time elapsed without showing");
                      end
                   end

         end );

         return self._trying or self._showing;
      end
   end

end

----------------------------------------------------------------------

function AdFlurryInterstitialController:_setupListeners()
   Debug.assert(self._flurry, "Flurry did not exist!");

   self._flurry.setListener(  MOAIFlurry.FLURRY_INTERSTITIAL_DISPLAY
                              ,
							  function()
								 self:_onInterstitialShow();
							  end
   );

   self._flurry.setListener(  MOAIFlurry.FLURRY_INTERSTITIAL_CLOSE
                              ,
							  function()
								 self:_onInterstitialSuccess();
							  end
   );


end

----------------------------------------------------------------------

function AdFlurryInterstitialController:_onInterstitialShow(location)

   Debug.assert( self._trying );
   Debug.assert( not self._showing );

   EventMgr:postEvent( {result="success", network="FlurryAds", location=location}, "interstitial-success" );

   self._trying  = nil;
   self._showing = true;

end

----------------------------------------------------------------------

function AdFlurryInterstitialController:_onInterstitialFail(location, reason)
   if self._trying and not self._showing then
      EventMgr:postEvent( {result="failure", network="FlurryAds", location=location, reason=reason}, "interstitial-failure" );
      self:_onDone( false );
   end
end

----------------------------------------------------------------------

function AdFlurryInterstitialController:_onInterstitialSuccess(location)

   Debug.assert(not self._trying);
   Debug.assert(self._showing);

   self:_onDone( true );
end

----------------------------------------------------------------------

function AdFlurryInterstitialController:_onDone(success)

   if success then
      if self._callSuccess then
         self._callSuccess(self._callObj);
      end
   else
      if self._callFailure then
         self._callFailure(self._callObj);
      end
   end

   self._trying         = nil;
   self._showing        = nil;
   self._callObj        = nil;
   self._callSuccess    = nil;
   self._callFailure    = nil;
end
