
----------------------------------------------------------------------
require('engine/object/object')
----------------------------------------------------------------------

local Base           = Object;

FlurryAdsController   = Base:new();

----------------------------------------------------------------------

function FlurryAdsController:new()
   local obj         = Base.new( self );
   obj._flurry		= MOAIFlurry;
   return obj;
end

----------------------------------------------------------------------

function FlurryAdsController:start(  )
	self._started = true;
	self:_setupListeners();
end

----------------------------------------------------------------------

function FlurryAdsController:isStarted(  )
	return self._started;
end

----------------------------------------------------------------------

function FlurryAdsController:isVideoAvailable()
	return self._flurry.isVideoAvailable();
end

----------------------------------------------------------------------

function FlurryAdsController:isPlayingVideo(  )
   return self._playing;
end

----------------------------------------------------------------------

function FlurryAdsController:playVideo( callObj, callSuccess, callFailure )

   EventMgr:postEvent( { network="Flurry" }, "reward-video-attempt");

   if (self._started) and (self._flurry) and (not self._playing) then

      --it doesnt hurt to do it a few times
      self:_setupListeners();

      if self:isVideoAvailable(  ) then
         --doesnt return a value

         self._deserveReward  = nil;
         self._playing        = true;
         self._showing        = nil;
         self._callObj        = callObj;
         self._callSuccess    = callSuccess;
         self._callFailure    = callFailure;

         --Call after we have set our variable because it will call some of the callback right away
         self._flurry.showVideo();

         --Because we cannot be 100% certain the video will show (vungle doest return a value when calling play video)
         -- make sure we check the video is started after a little while, otherwise the user could get in a weird situation

         delay( 5, function()
                     if (self._playing) and (not self._showing) then
                        self:_onVideoNotShowing();
                     end
                   end        );


         return true;
      else

         EventMgr:postEvent( {result="failure", network="Flurry", reason="Flurry:self:isVideoAvailable()"}, "reward-video-failure" );

      end

   else

      EventMgr:postEvent( {result="failure", network="Flurry", reason="Flurry:(self._started) and (self._flurry) and (not self._playing)"}, "reward-video-failure" );

   end

end

----------------------------------------------------------------------

function FlurryAdsController:_startSession( )

end

----------------------------------------------------------------------

function FlurryAdsController:_setupListeners()

   if (self._flurry) then
    self._flurry.setListener(  MOAIFlurry.FLURRY_REWARDED_VIDEO_DISPLAY
                              ,
                                 function()
									self:_onVideoStart();
									print("=!= start");
                                 end
                              );
    self._flurry.setListener(  MOAIFlurry.FLURRY_REWARDED_VIDEO_COMPLETED
                              ,
                                 function()
									print("=!= complete");
                                 	self:_onVideoRewarded()
                                 end
                              );
    self._flurry.setListener(  MOAIFlurry.FLURRY_REWARDED_VIDEO_CLOSE
                              ,
                                 function()
									print("=!= close 1");
                                    if  self._playing  then
									print("=!= close 2");
	                                    self:_onVideoEnd();
	                                 end
                                 end
                              );
   else

      Debug.assert(self._flurry, "FlurryAdsController:_setupListeners -> self._flurry"      );

   end

end

----------------------------------------------------------------------

function FlurryAdsController:_onVideoPlayed( played )

   self._deserveReward = self._deserveReward or played;

end

----------------------------------------------------------------------

function FlurryAdsController:_onVideoStart()

   self._showing = true;

end

----------------------------------------------------------------------

function FlurryAdsController:_onVideoEnd()

   Debug.assert(self._deserveReward, "FlurryAdsController:_onVideoEnd -> self._deserveReward");

   self._deserveReward = self._deserveReward or AdMgr:shouldForceVideoReward();

   delay( 0.1, function()
                  self:_onVideoDone( );
               end                        );
end

----------------------------------------------------------------------

function FlurryAdsController:_onVideoRewarded( )

   self._deserveReward = true;

end

----------------------------------------------------------------------

function FlurryAdsController:_onVideoNotShowing()

   self:_onVideoDone( );

end

----------------------------------------------------------------------

function FlurryAdsController:_onVideoDone( )

   if self._deserveReward then

      EventMgr:postEvent( {result="success", network="Flurry"}, "reward-video-success" );

      if self._callSuccess then
         self._callSuccess(self._callObj);
      end

   else

      EventMgr:postEvent( {
                              result="failure",
                              network="Flurry",
                              reason="Flurry:_deserveReward:  playing=" .. tostring(self._playing) .. ":  showing=" .. tostring(self._showing)
                           },
                           "reward-video-failure" );

      if self._callFailure then
         self._callFailure(self._callObj);
      end


   end

   self._playing	    = nil;
   self._deserveReward  = nil;
   self._showing        = nil;
   self._callObj        = nil;
   self._callSuccess    = nil;
   self._callFailure    = nil;

end


----------------------------------------------------------------------
