
----------------------------------------------------------------------
require('engine/object/object')
----------------------------------------------------------------------

local Base         = Object;

FyberController = Base:new();

----------------------------------------------------------------------

function FyberController:new()
   local obj         = Base.new( self );
   obj._fyber   = MOAIFyber;
   Debug.assert(obj._fyber, "_fyber was nil in FyberController:new");
   return obj;
end

----------------------------------------------------------------------

function FyberController:start(  )
   self._started = true;
   self:_setupListeners();
end

----------------------------------------------------------------------

function FyberController:isStarted(  )
   return self._started;
end

----------------------------------------------------------------------

function FyberController:isVideoAvailable()
   Debug.assert(self._fyber, "self._fyber was nil!");
   return self._fyber.isRewardedVideoAvailable();
end

----------------------------------------------------------------------

function FyberController:isPlayingVideo(  )
   return self._playing;
end

----------------------------------------------------------------------

function FyberController:playVideo( callObj, callSuccess, callFailure )

   EventMgr:postEvent( { network="Fyber" }, "reward-video-attempt");

   if (self._started) and (self._fyber) and (not self._playing) then

      --it doesnt hurt to do it a few times
      self:_setupListeners();

      if self:isVideoAvailable(  ) then
         --doesnt return a value

         self._deserveReward = nil;
         self._playing       = true;
         self._showing       = nil;
         self._callObj       = callObj;
         self._callSuccess   = callSuccess;
         self._callFailure   = callFailure;

         --Call after we have set our variable because it will call some of the callback right away
         self._fyber.showRewardedVideo();

         --Because we cannot be 100% certain the video will show (vungle doest return a value when calling play video)
         -- make sure we check the video is started after a little while, otherwise the user could get in a weird situation

         --increase the time limit to see if it will help reducing the number of errors we are getting
         delay( 5, function()
                   if (self._playing) and (not self._showing) then
                      self:_onVideoNotShowing();
                   end
         end );

         return true;
      else

         EventMgr:postEvent( {result="failure", network="Fyber", reason="Fyber:self:isVideoAvailable()"}, "reward-video-failure" );

      end

   else

      EventMgr:postEvent( {result="failure", network="Fyber", reason="Fyber:(self._started) and (self._fyber) and (not self._playing)"}, "reward-video-failure" );

   end

   return false;
end

----------------------------------------------------------------------

function FyberController:_startSession( )

end

----------------------------------------------------------------------

function FyberController:_setupListeners()

   if (self._fyber) then

      self._fyber.setListener( MOAIFyber.DID_V4VC_STARTED_REWARDED_VIDEO,
                                  function()
                                     self:_onVideoStart();
                                  end
      );

      self._fyber.setListener( MOAIFyber.DID_V4VC_FINISHED_REWARDED_VIDEO,
                                  function( shown )
                                     print("DID_V4VC_FINISHED_REWARDED_VIDEO : shown -> ", shown);
                                     self:_onVideoRewarded(shown);
                                     self:_onVideoEnd();
                                  end
      );

      self._fyber.setListener( MOAIFyber.DID_DISMISS_AD,
                                  function( shown )
                                    print("DID_DISMISS_AD : shown -> ", shown," playing -> ",self._playing);
                                     if self._playing then
                                        self:_onVideoEnd();
                                     end
                                  end
      );

   else

      Debug.error("FyberController:_setupListeners -> self._fyber was nil" );

   end

end

----------------------------------------------------------------------

function FyberController:_onVideoPlayed( played )

   self._deserveReward = self._deserveReward or played;

end

----------------------------------------------------------------------

function FyberController:_onVideoStart()

   print("FyberController:_onVideoStart");

   self._showing = true;

end

----------------------------------------------------------------------

function FyberController:_onVideoEnd()

   print("FyberController:_onVideoEnd");

   self:_onVideoDone( );
end

----------------------------------------------------------------------

function FyberController:_onVideoRewarded( reward )

   self._deserveReward = reward;

end

----------------------------------------------------------------------

function FyberController:_onVideoNotShowing()

   self:_onVideoDone( );

end

----------------------------------------------------------------------

function FyberController:_onVideoDone( )

   print("FyberController:_onVideoDone");

   if  self._deserveReward then

      EventMgr:postEvent( {result="success", network="Fyber"}, "reward-video-success" );

      if self._callSuccess then
         self._callSuccess(self._callObj);
      end

   else

      EventMgr:postEvent(
         {
            result="failure",
            network="Fyber",
            reason="Fyber:_deserveReward:  playing=" .. tostring(self._playing) .. ":  showing=" .. tostring(self._showing)
         },
         "reward-video-failure" );

      if self._callFailure then
         self._callFailure(self._callObj);
      end

   end

   self._playing        = nil;
   self._deserveReward  = nil;
   self._showing        = nil;
   self._callObj        = nil;
   self._callSuccess    = nil;
   self._callFailure    = nil;

end

----------------------------------------------------------------------
