require('engine/object/object')
----------------------------------------------------------------------

local Base                           = Object;

AdIronSourceInterstitialController   = Base:new();

----------------------------------------------------------------------

function AdIronSourceInterstitialController:new()

   local obj    = Base.new( self );
   obj._ironSource  = MOAIIronSource;

   obj._started = false

   return obj;
end

----------------------------------------------------------------------

function AdIronSourceInterstitialController:start()
print("AdIronSourceInterstitialController:start");

   self:_setupListeners();
   self._started = true;
end

----------------------------------------------------------------------

function AdIronSourceInterstitialController:isStarted()
   return self._started;
end

----------------------------------------------------------------------

function AdIronSourceInterstitialController:isInterstitialAvailable()
   Debug.assert(self._ironSource, "Iron Source was nil!");

   return self._ironSource.isInterstitialAvailable();
end

----------------------------------------------------------------------

function AdIronSourceInterstitialController:isInterstitialShowing()
   return self._showing;
end

----------------------------------------------------------------------

function AdIronSourceInterstitialController:cacheInterstitial()
end

----------------------------------------------------------------------

function AdIronSourceInterstitialController:showInterstitial(location, callObj, callSuccess, callFailure)
print("AdIronSourceInterstitialController:showInterstitial");

   if (not self._trying) and (not self._showing) then

      if self:isInterstitialAvailable() then

         EventMgr:postEvent( { network="IronSourceAds", location=location }, "interstitial-attempt");

         local attemptID      = (self._attemptID or 0 ) + 1;

         self._trying         = true;
         self._showing        = nil;
         self._callObj        = callObj;
         self._callSuccess    = callSuccess;
         self._callFailure    = callFailure;
         self._attemptID      = attemptID;

         self._ironSource.showInterstitial(location);

         delay( 2.0,
                function()
                   --because we have some delay, it is conceivable that we would try to open another interstitial by now
                   -- and we want to make sure this check is for the appropriate attempt

                   if attemptID == self._attemptID then
                      if (self._trying) and (not self._showing ) then
                         self:_onInterstitialFail(location, "time elapsed without showing");
                      end
                   end

         end );

         return self._trying or self._showing;
      end
   end

end

----------------------------------------------------------------------

function AdIronSourceInterstitialController:_setupListeners()
   Debug.assert(self._ironSource, "IronSource did not exist!");
print("AdIronSourceInterstitialController:_setupListeners");

   self._ironSource.setListener(  MOAIIronSource.INTERSTITIAL_DISPLAY
                              ,
							  function()
								 self:_onInterstitialShow();
							  end
   );

   self._ironSource.setListener(  MOAIIronSource.INTERSTITIAL_CLOSE
                              ,
							  function()
                    print("AdIronSourceInterstitialController:_setupListeners");

								 self:_onInterstitialSuccess();
							  end
   );


end

----------------------------------------------------------------------

function AdIronSourceInterstitialController:_onInterstitialShow(location)

   Debug.assert( self._trying );
   Debug.assert( not self._showing );
print("AdIronSourceInterstitialController:_onInterstitialShow");

   EventMgr:postEvent( {result="success", network="IronSourceAds", location=location}, "interstitial-success" );

   self._trying  = nil;
   self._showing = true;

end

----------------------------------------------------------------------

function AdIronSourceInterstitialController:_onInterstitialFail(location, reason)
print("AdIronSourceInterstitialController:_onInterstitialFail");

   if self._trying and not self._showing then
      EventMgr:postEvent( {result="failure", network="IronSourceAds", location=location, reason=reason}, "interstitial-failure" );
      self:_onDone( false );
   end
end

----------------------------------------------------------------------

function AdIronSourceInterstitialController:_onInterstitialSuccess(location)
print("AdIronSourceInterstitialController:_onInterstitialSuccess");

   Debug.assert(not self._trying);
   Debug.assert(self._showing);
   self:_onDone( true );
end

----------------------------------------------------------------------

function AdIronSourceInterstitialController:_onDone(success)

  print("AdIronSourceInterstitialController:_onDone");

   if success then
      if self._callSuccess then
         self._callSuccess(self._callObj);
      end
   else
      if self._callFailure then
         self._callFailure(self._callObj);
      end
   end

   self._trying         = nil;
   self._showing        = nil;
   self._callObj        = nil;
   self._callSuccess    = nil;
   self._callFailure    = nil;
end
