
----------------------------------------------------------------------
require('engine/object/object')
----------------------------------------------------------------------

local Base         = Object;

IronSourceController = Base:new();

----------------------------------------------------------------------

function IronSourceController:new()
   local obj         = Base.new( self );
   obj._ironsource   = MOAIIronSource;
   Debug.assert(obj._ironsource, "_ironsource was nil in IronSourceController:new");
   return obj;
end

----------------------------------------------------------------------

function IronSourceController:start(  )
   self._started = true;
   self:_setupListeners();
end

----------------------------------------------------------------------

function IronSourceController:isStarted(  )
   return self._started;
end

----------------------------------------------------------------------

function IronSourceController:isVideoAvailable()
   Debug.assert(self._ironsource, "self._ironsource was nil!");
   return self._ironsource.isRewardedVideoAvailable();
end

----------------------------------------------------------------------

function IronSourceController:isPlayingVideo(  )
   return self._playing;
end

----------------------------------------------------------------------

function IronSourceController:playVideo( callObj, callSuccess, callFailure )

   EventMgr:postEvent( { network="IronSource" }, "reward-video-attempt");

   if (self._started) and (self._ironsource) and (not self._playing) then

      --it doesnt hurt to do it a few times
      self:_setupListeners();

      if self:isVideoAvailable(  ) then
         --doesnt return a value

         self._deserveReward = nil;
         self._playing       = true;
         self._showing       = nil;
         self._callObj       = callObj;
         self._callSuccess   = callSuccess;
         self._callFailure   = callFailure;

         --Call after we have set our variable because it will call some of the callback right away
         self._ironsource.showRewardedVideo();

         --Because we cannot be 100% certain the video will show (vungle doest return a value when calling play video)
         -- make sure we check the video is started after a little while, otherwise the user could get in a weird situation

         --increase the time limit to see if it will help reducing the number of errors we are getting
         delay( 5, function()
                   if (self._playing) and (not self._showing) then
                      self:_onVideoNotShowing();
                   end
         end );

         return true;
      else

         EventMgr:postEvent( {result="failure", network="IronSource", reason="IronSource:self:isVideoAvailable()"}, "reward-video-failure" );

      end

   else

      EventMgr:postEvent( {result="failure", network="IronSource", reason="IronSource:(self._started) and (self._ironsource) and (not self._playing)"}, "reward-video-failure" );

   end

   return false;
end

----------------------------------------------------------------------

function IronSourceController:_startSession( )

end

----------------------------------------------------------------------

function IronSourceController:_setupListeners()

   if (self._ironsource) then

      self._ironsource.setListener( MOAIIronSource.DID_V4VC_STARTED_REWARDED_VIDEO,
                                  function()
                                     self:_onVideoStart();
                                  end
      );

      self._ironsource.setListener( MOAIIronSource.DID_V4VC_FINISHED_REWARDED_VIDEO,
                                  function( shown )
                                     print("DID_V4VC_FINISHED_REWARDED_VIDEO : shown -> ", shown);
                                     self:_onVideoRewarded(shown);
                                     self:_onVideoEnd();
                                  end
      );

      self._ironsource.setListener( MOAIIronSource.DID_DISMISS_AD,
                                  function( shown )
                                    print("DID_DISMISS_AD : shown -> ", shown," playing -> ",self._playing);
                                     if self._playing then
                                        self:_onVideoEnd();
                                     end
                                  end
      );

   else

      Debug.error("IronSourceController:_setupListeners -> self._ironsource was nil" );

   end

end

----------------------------------------------------------------------

function IronSourceController:_onVideoPlayed( played )

   self._deserveReward = self._deserveReward or played;

end

----------------------------------------------------------------------

function IronSourceController:_onVideoStart()

   print("IronSourceController:_onVideoStart");

   self._showing = true;

end

----------------------------------------------------------------------

function IronSourceController:_onVideoEnd()

   print("IronSourceController:_onVideoEnd");

   self:_onVideoDone( );
end

----------------------------------------------------------------------

function IronSourceController:_onVideoRewarded( reward )

   self._deserveReward = reward;

end

----------------------------------------------------------------------

function IronSourceController:_onVideoNotShowing()
   print("IronSourceController:_onVideoNotShowing");

   self:_onVideoDone( );

end

----------------------------------------------------------------------

function IronSourceController:_onVideoDone( )

   print("IronSourceController:_onVideoDone");

   if  self._deserveReward then

      EventMgr:postEvent( {result="success", network="IronSource"}, "reward-video-success" );

      if self._callSuccess then
         self._callSuccess(self._callObj);
      end

   else

      EventMgr:postEvent(
         {
            result="failure",
            network="IronSource",
            reason="IronSource:_deserveReward:  playing=" .. tostring(self._playing) .. ":  showing=" .. tostring(self._showing)
         },
         "reward-video-failure" );

      if self._callFailure then
         self._callFailure(self._callObj);
      end

   end

   self._playing        = nil;
   self._deserveReward  = nil;
   self._showing        = nil;
   self._callObj        = nil;
   self._callSuccess    = nil;
   self._callFailure    = nil;

end

----------------------------------------------------------------------
