
require('app/server/tango')

----------------------------------------------------------------------

local IMAGE_SIZE_MAX    = { normal = { 512, 128 }, mini = { 256, 64 }  };

----------------------------------------------------------------------

local Base     = TangoServer;
AdRetriever    = Base:new();

----------------------------------------------------------------------

function AdRetriever:new()

   local loader = Base.new(self);

   return loader;

end

----------------------------------------------------------------------

function AdRetriever:exec( callObj, callSuccess, callFailure )

   if OnlineMgr:isOnline() then

      local successFunc = function( result )

         if result["banner"] and result["link"] then

            local result2 = {};
            result2["banner"]    = self:_createFilename( result["banner"] );
            result2["link"]      = result["link"];

            downloadToFile( result["banner"],
                            result2["banner"],
                            self,
                            function()
                               local w,h = self:_fixupImage( result2["banner"] );
                               if w and h then
                                  result2["width"]     = w;
                                  result2["height"]    = h;
                                  self:callCallback( callObj, callSuccess, result2 );
                               else
                                  self:callCallback( callObj,  callFailure, -1, "Image Fixed up failed"  );
                               end
                            end,
                            function()
                               self:callCallback( callObj,  callFailure, -1, "Download Banner Failed"  );
                            end
            );

         else
            self:callCallback( callObj, callFailure, -1, "Missing banner or link" );
         end
      end

      local failureFunc = function(code, message)
         self:callCallback( callObj, callFailure, code, message );
      end

      local requestOptions = {
         requestClass = MOAITangoGetAdvertisementRequest,
         requestData = {},
         successFunc = successFunc,
         failureFunc = failureFunc,
      }

      NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

   else

      -- Use a timer to create be more consistent with the authenticate mode
      delay(
               0.1
            ,
               function ()
                  self:callCallback( callObj,  callFailure, -1, "You are offline"  );
               end

            );
   end

end

----------------------------------------------------------------------

function AdRetriever:_createFilename( url )

   local path, file, ext = splitFileName( url );

   --Remove extenstion
   file = file:gsub("."..ext, ".png");

   return "_ad_" .. file;

end

----------------------------------------------------------------------

function AdRetriever:_isFormatValid( format )

   if    format == MOAIImage.COLOR_FMT_A_8
      or format == MOAIImage.COLOR_FMT_RGB_888
      or format == MOAIImage.COLOR_FMT_RGBA_4444
      or format == MOAIImage.COLOR_FMT_RGBA_8888 then
      return true;
end


end

----------------------------------------------------------------------

function AdRetriever:_fixupImage( filename )

   local maxSize  = IMAGE_SIZE_MAX.normal;
   if App.useMiniTexture then
      maxSize  = IMAGE_SIZE_MAX.mini;
   end

   local workingDir = MOAIFileSystem.getWorkingDirectory ()

   MOAIFileSystem.setWorkingDirectory ( MOAIEnvironment.documentDirectory )

   local image    = MOAIImage.new ();

   image:load(filename);
   local x,y       = image:getSize();
   local image2    = image:resize( maxSize[1], maxSize[2] );

   image2:writePNG( filename  );

   MOAIFileSystem.setWorkingDirectory ( workingDir )

   return x,y;

end

----------------------------------------------------------------------
