
----------------------------------------------------------------------
require('engine/object/object')
----------------------------------------------------------------------

local Base         = Object;

TapjoyController = Base:new();

----------------------------------------------------------------------

function TapjoyController:new()
   local obj         = Base.new( self );
   obj._tapjoy   = MOAITapjoy;
   Debug.assert(obj._tapjoy, "_tapjoy was nil in TapjoyController:new");
   return obj;
end

----------------------------------------------------------------------

function TapjoyController:start(  )
   self._started = true;
   self:_setupListeners();
end

----------------------------------------------------------------------

function TapjoyController:isStarted(  )
   return self._started;
end

----------------------------------------------------------------------

function TapjoyController:isVideoAvailable()
   Debug.assert(self._tapjoy, "self._tapjoy was nil!");
   return self._tapjoy.isRewardedVideoAvailable();
end

----------------------------------------------------------------------

function TapjoyController:isPlayingVideo(  )
   return self._playing;
end

----------------------------------------------------------------------

function TapjoyController:playVideo( callObj, callSuccess, callFailure )

   EventMgr:postEvent( { network="Tapjoy" }, "reward-video-attempt");

   if (self._started) and (self._tapjoy) and (not self._playing) then

      --it doesnt hurt to do it a few times
      self:_setupListeners();

      if self:isVideoAvailable(  ) then
         --doesnt return a value

         self._deserveReward = nil;
         self._playing       = true;
         self._showing       = nil;
         self._callObj       = callObj;
         self._callSuccess   = callSuccess;
         self._callFailure   = callFailure;

         --Call after we have set our variable because it will call some of the callback right away
         self._tapjoy.showRewardedVideo();

         --Because we cannot be 100% certain the video will show (vungle doest return a value when calling play video)
         -- make sure we check the video is started after a little while, otherwise the user could get in a weird situation

         --increase the time limit to see if it will help reducing the number of errors we are getting
         delay( 5, function()
                   if (self._playing) and (not self._showing) then
                      self:_onVideoNotShowing();
                   end
         end );

         return true;
      else

         EventMgr:postEvent( {result="failure", network="Tapjoy", reason="Tapjoy:self:isVideoAvailable()"}, "reward-video-failure" );

      end

   else

      EventMgr:postEvent( {result="failure", network="Tapjoy", reason="Tapjoy:(self._started) and (self._tapjoy) and (not self._playing)"}, "reward-video-failure" );

   end

   return false;
end

----------------------------------------------------------------------

function TapjoyController:_startSession( )

end

----------------------------------------------------------------------

function TapjoyController:_setupListeners()

   if (self._tapjoy) then

      self._tapjoy.setListener( MOAITapjoy.DID_V4VC_STARTED_REWARDED_VIDEO,
                                  function()
                                     self:_onVideoStart();
                                  end
      );

      self._tapjoy.setListener( MOAITapjoy.DID_V4VC_FINISHED_REWARDED_VIDEO,
                                  function( shown )
                                     print("DID_V4VC_FINISHED_REWARDED_VIDEO : shown -> ", shown);
                                     self:_onVideoRewarded(shown);
                                     self:_onVideoEnd();
                                  end
      );

      self._tapjoy.setListener( MOAITapjoy.DID_DISMISS_AD,
                                  function( shown )
                                     if self._playing then
                                        self:_onVideoEnd();
                                     end
                                  end
      );

   else

      Debug.error("TapjoyController:_setupListeners -> self._tapjoy was nil" );

   end

end

----------------------------------------------------------------------

function TapjoyController:_onVideoPlayed( played )

   self._deserveReward = self._deserveReward or played;

end

----------------------------------------------------------------------

function TapjoyController:_onVideoStart()

   print("TapjoyController:_onVideoStart");

   self._showing = true;

end

----------------------------------------------------------------------

function TapjoyController:_onVideoEnd()

   print("TapjoyController:_onVideoEnd");

   self:_onVideoDone( );
end

----------------------------------------------------------------------

function TapjoyController:_onVideoRewarded( reward )

   self._deserveReward = reward;

end

----------------------------------------------------------------------

function TapjoyController:_onVideoNotShowing()

   self:_onVideoDone( );

end

----------------------------------------------------------------------

function TapjoyController:_onVideoDone( )

   print("TapjoyController:_onVideoDone");

   if  self._deserveReward then

      EventMgr:postEvent( {result="success", network="Tapjoy"}, "reward-video-success" );

      if self._callSuccess then
         self._callSuccess(self._callObj);
      end

   else

      EventMgr:postEvent(
         {
            result="failure",
            network="Tapjoy",
            reason="Tapjoy:_deserveReward:  playing=" .. tostring(self._playing) .. ":  showing=" .. tostring(self._showing)
         },
         "reward-video-failure" );

      if self._callFailure then
         self._callFailure(self._callObj);
      end

   end

   self._playing        = nil;
   self._deserveReward  = nil;
   self._showing        = nil;
   self._callObj        = nil;
   self._callSuccess    = nil;
   self._callFailure    = nil;

end

----------------------------------------------------------------------
