
----------------------------------------------------------------------
require('engine/object/object')
----------------------------------------------------------------------

local Base           = Object;

AdVungleControllerAndroid   = Base:new();

----------------------------------------------------------------------

function AdVungleControllerAndroid:new( appId )

   Debug.assert( appId );

   local obj   = Base.new( self );

   obj._appId  = appId;
   obj._vungle = MOAIVungleAndroid;

   return obj;

end

----------------------------------------------------------------------

function AdVungleControllerAndroid:start(  )

   if not self._started then

      if self._appId then
         if self:_startSession(self._appId) then
            self:_setupListeners();
         end
      else
         Debug.assert(self._appId);
      end

   end
end

----------------------------------------------------------------------

function AdVungleControllerAndroid:isStarted(  )

   return self._started;

end

----------------------------------------------------------------------

function AdVungleControllerAndroid:isVideoAvailable(  )

   if (self._started) and (self._vungle ~= nil) then

      return self._vungle.isVideoAvailable();

   end

end

----------------------------------------------------------------------

function AdVungleControllerAndroid:isPlayingVideo(  )

   return self._playing;

end

----------------------------------------------------------------------

function AdVungleControllerAndroid:playVideo( callObj, callSuccess, callFailure )

   EventMgr:postEvent( { network="Vungle" }, "reward-video-attempt");

   if (self._started) and (self._vungle) and (not self._playing) then

      if self:isVideoAvailable(  ) then

         self:_setupListeners();

         if self._vungle.playVideo() then

            self._deserveReward  = nil;
            self._playing        = true;
            self._showing        = nil;
            self._callObj        = callObj;
            self._callSuccess    = callSuccess;
            self._callFailure    = callFailure;

            --Because we cannot be 100% certain the video will show (vungle doest return a value when calling play video)
            -- make sure we check the video is started after a little while, otherwise the user could get in a weird situation
            delay( 5, function()
                        if (self._playing) and (not self._showing) then
                           self:_onVideoNotShowing();
                        end
                      end        );

            return true;

         else

            EventMgr:postEvent( { result="failure", network="Vungle", reason="self._vungle.playVideo()"}, "reward-video-failure" );

         end
      else
         EventMgr:postEvent( {result="failure", network="Vungle", reason="self:isVideoAvailable()"}, "reward-video-failure" );
      end
   else

      EventMgr:postEvent( {result="failure", network="Vungle", reason="(self._started) and (self._vungle) and (not self._playing)"}, "reward-video-failure" );

   end

end

----------------------------------------------------------------------

function AdVungleControllerAndroid:_startSession( appId )

   if (self._vungle ~= nil) and (not self._started) and (appId ~= nil) then

      self._vungle.startSession(appId);

      self._started = true;

      return true;

   else

      Debug.assert(not self._started,  "AdVungleControllerAndroid:_startSession -> not self._started" );

   end

end

----------------------------------------------------------------------

function AdVungleControllerAndroid:_setupListeners()

   if (self._vungle) then

      self._vungle.setListener(  self._vungle.VUNGLE_VIDEO_START
                              ,
                                 function(  )
                                    self:_onVungleVideoStart();
                                 end
                              );

     self._vungle.setListener(   self._vungle.VUNGLE_VIDEO_END
                              ,
                                 function(  )
                                    self:_onVungleVideoEnd();
                                 end
                              );

      self._vungle.setListener(  self._vungle.VUNGLE_VIDEO_VIEW
                              ,
                              function( reward )
                                 self:_onVungleVideoView( reward );
                              end
                              );
   else

      Debug.assert(self._vungle, "AdVungleControllerAndroid:_setupListeners -> self._vungle"      );

   end

end

----------------------------------------------------------------------

function AdVungleControllerAndroid:_onVungleMoviePlayed( played )

   self._deserveReward = self._deserveReward or played;

end

----------------------------------------------------------------------

function AdVungleControllerAndroid:_onVungleVideoStart()

   self._showing = true;

end

----------------------------------------------------------------------

function AdVungleControllerAndroid:_onVungleVideoEnd()

   Debug.assert( self._playing, " AdVungleControllerAndroid:_onVungleViewDisappeared -> self._playing ");

   delay( 0.1, function()
                  self:_onVideoDone( );
               end                        );

end

----------------------------------------------------------------------

function AdVungleControllerAndroid:_onVungleVideoView( reward )

   self._deserveReward = reward;

end

----------------------------------------------------------------------

function AdVungleControllerAndroid:_onVideoNotShowing()

   self:_onVideoDone( );

end

----------------------------------------------------------------------

function AdVungleControllerAndroid:_onVideoDone( )

   if  self._deserveReward then

      App.data:deltaValue("Video-Vungle-Succeeded-Count",1);

      EventMgr:postEvent( {result="success", network="Vungle"}, "reward-video-success" );

      if self._callSuccess then
         self._callSuccess(self._callObj);
      end

   else

      App.data:deltaValue("Video-Vungle-Failure-Count",1);

      EventMgr:postEvent( {
                              result="failure",
                              network="Vungle",
                              reason="_deserveReward:  playing=" .. tostring(self._playing) .. ":  showing=" .. tostring(self._showing)
                           },
                           "reward-video-failure" );

      if self._callFailure then
         self._callFailure(self._callObj);
      end

   end

   self._deserveReward  = nil;
   self._playing        = nil;
   self._showing        = nil;
   self._callObj        = nil;
   self._callSuccess    = nil;
   self._callFailure    = nil;

end


----------------------------------------------------------------------