
----------------------------------------------------------------------
require('engine/object/object')
----------------------------------------------------------------------

local Base              = Object;

AdVungleControllerIOS   = Base:new();

----------------------------------------------------------------------

function AdVungleControllerIOS:new( appId )

   Debug.assert( appId );

   local obj   = Base.new( self );

   obj._vungle = MOAIVungleIOS;
   obj._appId  = appId;

   return obj;

end

----------------------------------------------------------------------

function AdVungleControllerIOS:start( )

   if not self._started then

      if self._appId then
         if self:_startSession(self._appId) then
            self:_setupListeners();
         end
      else
         Debug.assert(self._appId);
      end

   end

end


----------------------------------------------------------------------

function AdVungleControllerIOS:isStarted(  )

   return self._started;

end

----------------------------------------------------------------------

function AdVungleControllerIOS:isVideoAvailable(  )

   if (self._started) and (self._vungle ~= nil) then

      return self._vungle.isVideoAvailable();

   end

end

----------------------------------------------------------------------

function AdVungleControllerIOS:isPlayingVideo(  )

   return self._playing;

end

----------------------------------------------------------------------

function AdVungleControllerIOS:playVideo( callObj, callSuccess, callFailure )

   Debug.assert( (not self._playing) );

   EventMgr:postEvent( {network="Vungle"}, "reward-video-attempt");

   if (self._started) and (self._vungle) and (not self._playing) then

      if self:isVideoAvailable(  ) then
         self:_setupListeners();

         --_onVungleViewWillAppear calls before _vungle.playVideo returns
         self._showing        = nil;
         self._deserveReward  = nil;

         if self._vungle.playVideo() then

            self._playing        = true;
            self._callObj        = callObj;
            self._callSuccess    = callSuccess;
            self._callFailure    = callFailure;

            --Because we cannot be 100% certain the video will show (vungle doest return a value when calling play video)
            -- make sure we check the video is started after a little while, otherwise the user could get in a weird situation
			-- Some error handling was added in MOAIVungleIOS::_playVideo which could now make this irrelevant.
            delay( 5, function()
                        if (self._playing) and (not self._showing) then
                           self:_onVideoNotShowing();
                        end
                      end        );

            return true;

         else

            EventMgr:postEvent( { result="failure", network="Vungle", reason="self._vungle.playVideo()"}, "reward-video-failure" );

         end

      else

         EventMgr:postEvent( {result="failure", network="Vungle", reason="self:isVideoAvailable()"}, "reward-video-failure" );

      end

   else

      EventMgr:postEvent( {result="failure", network="Vungle", reason="(self._started) and (self._vungle) and (not self._playing)"}, "reward-video-failure" );

   end

end

----------------------------------------------------------------------

function AdVungleControllerIOS:_startSession( appId )

   if (self._vungle ~= nil) and (not self._started) and (appId ~= nil) then

      self._vungle.startSession(appId);

      self._started = true;

      return true;

   else

      Debug.assert( (self._vungle ~= nil) and (not self._started) and (appId ~= nil),  "AdVungleControllerIOS:_startSession -> not self._started" );

   end

end

----------------------------------------------------------------------

function AdVungleControllerIOS:_setupListeners()

   if (self._vungle) then

      self._vungle.setListener(  self._vungle.VUNGLE_STATUS_CHANGED
                              ,
                                 function( status )
                                    self:_onVungleStatusChange();
                                 end
                              );

     self._vungle.setListener(   self._vungle.VUNGLE_MOVIE_PLAYED
                              ,
                                 function( played )
                                    self:_onVungleMoviePlayed(played);
                                 end
                              );

      self._vungle.setListener(  self._vungle.VUNGLE_VIEW_WILL_APPEAR
                              ,
                                 function(  )
                                    self:_onVungleViewWillAppear();
                                 end
                              );

      self._vungle.setListener(  self._vungle.VUNGLE_VIEW_DISAPPEARED
                              ,
                                 function(  )
                                    self:_onVungleViewDisappeared();
                                 end
                              );

   else

      Debug.assert(self._vungle, "AdVungleControllerIOS:_setupListeners -> self._vungle"      );

   end

end


----------------------------------------------------------------------

function AdVungleControllerIOS:_onVungleStatusChange()
end

----------------------------------------------------------------------

function AdVungleControllerIOS:_onVungleMoviePlayed( played )

   self._deserveReward = self._deserveReward or played;

end

----------------------------------------------------------------------

function AdVungleControllerIOS:_onVungleViewWillAppear()

   self._showing = true;

end

----------------------------------------------------------------------

function AdVungleControllerIOS:_onVungleViewDisappeared()

   Debug.assert( self._playing, " AdVungleControllerIOS:_onVungleViewDisappeared -> self._playing ");

   self:_onVideoDone( );

end

----------------------------------------------------------------------

function AdVungleControllerIOS:_onVideoNotShowing()

   self:_onVideoDone( );

end

----------------------------------------------------------------------

function AdVungleControllerIOS:_onVideoDone( )

   if self._deserveReward then

      App.data:deltaValue("Video-Vungle-Succeeded-Count",1);

      EventMgr:postEvent( {result="success", network="Vungle"}, "reward-video-success" );

      if self._callSuccess then
         self._callSuccess(self._callObj);
      end

   else

      App.data:deltaValue("Video-Vungle-Failure-Count",1);

      EventMgr:postEvent( {
                              result="failure",
                              network="Vungle",
                              reason="_deserveReward:  playing=" .. tostring(self._playing) .. ":  showing=" .. tostring(self._showing)
                           },
                           "reward-video-failure" );

      if self._callFailure then
         self._callFailure(self._callObj);
      end

   end

   self._deserveReward  = nil;
   self._playing        = nil;
   self._showing        = nil;
   self._callObj        = nil;
   self._callSuccess    = nil;
   self._callFailure    = nil;

end


----------------------------------------------------------------------
