require('app/ab-test/ab-test-manager')
require('app/achievement/achievement-manager')
require('app/ad/ad-manager')
require('app/analytics/analytics-manager')
require('app/audio/robo-dj')
require('app/data/appData')
require('app/cloud/cloud-manager')
require('app/contact/contact-manager')
require('app/feed/feed-manager')
require('app/fraud/fraud-manager')
require('app/ghost/ghost-manager')
require('app/guest/guest-manager')
require('app/league/league-manager')
require('app/localization/localization-manager')
require('app/login/login-manager')
require('app/message/message-manager')
require('app/notification/notification-manager')
require('app/online/online-manager')
require('app/possession/possession-manager')
require('app/promo/promo-manager')
require('app/ranking/ranking-manager')
require('app/store/store-manager')
require('app/template/template-manager')
require('app/template/tango-reminder');
require('app/time/time-manager');
require('app/update/update-manager')
require('engine/audio/audio-manager')
require('engine/event/event-manager')
require('engine/file/fileMgr')
require('engine/input/input-manager')
require('engine/mission/mission')
require('state/stateMgr')
require('state/guide/guide-manager')
require('app/bounty/bounty-manager')
require('app/profiler/profiler-manager')
require('app/drone/drone-manager')
require('app/consumable/consumable-manager')
require('app/monkeytester')
require('app/cloud/june-api-manager')

----------------------------------------------------------------------
----------------------------------------------------------------------

local APP_ID                    = 'road-riot'
local APP_NAME                  = "Road Riot";
local GAME_FILENAME             = "game/game.lua";

local SCORE_TABLE_NAME          = "scoresV";

--The game was designed on an iPhone 5 and shorter device has some disadvantage
local MAX_TIME_SCALE_HEIGHT     = 1136;

local MINI_TEXTURE_SIZE_MAX     = 1024;
local TARGET_VIEWPORT_WIDTH     = 640;
local TARGET_VIEWPORT_HEIGHT    = 960;
local APP_STORE_LINK            = "https://itunes.apple.com/us/app/road-riot-for-tango/id646803444?ls=1&mt=8";
local APP_STORE_REVIEW_LINK     = "https://itunes.apple.com/us/app/road-riot-for-tango/id646803444?ls=1&mt=8";
local GOOGLE_PLAY_LINK          = "http://play.google.com/store/apps/details?id=me.tango.roadriot&hl=en";
local GOOGLE_PLAY_REVIEW_LINK   = "http://play.google.com/store/apps/details?id=me.tango.roadriot";
local AMAZON_STORE_LINK         = "https://www.amazon.com/gp/mas/dl/android?p=me.tango.roadriot";
local CHANNEL_LINK              = "tango://deeplink/q?target=channel_subscribe&channel_id=IA3fgSDihiNpfVSTl5zYGA&tracker=sometracker";
local TEMPLATE_SAVE_FILENAME    = "ROAD_RIOT_SAVE_TEMPLATE.lua";

local TIME_BETWEEN_GC_CYCLE     = 0;

local HACK_DATA_RESET           = false;
local HACK_FORCE_MINI_TEXTURE   = false;
local HACK_FORCE_IPHONE4        = false;

local MONKEY_TESTER_ENABLED     = false;

-- Modify above, not below.
if not BUILD_DEBUG then
   HACK_DATA_RESET              = false;
   HACK_FORCE_MINI_TEXTURE      = false;
   HACK_FORCE_IPHONE4           = false;
   MONKEY_TESTER_ENABLED        = false;
end

HACK_QUICK_WEEKLY_RESET          = false;
HACK_QUICK_WEEKLY_RESET_TIME     = 60*60;

GIFT_KEY                         = "key"
GUEST_MODE_VERSION_1             = 1;

ERROR_IMAGE_NOT_SUPPORTED        = 1;
ERROR_DOWNLOAD_FAILED            = 2;

--Status isnt great but I couldt find a better name to represent some data
--that can be local, online, etc
STATUS_UNKNOWN             = 0;
STATUS_GUEST               = 1;
STATUS_TANGO_NIL           = 2;
STATUS_TANGO_LOCAL         = 3;
STATUS_TANGO_DOWNLOADED    = 4;

INVITE_TIME_LIMIT                = 7*24*60*60;
GIFT_TIME_LIMIT                  = 3*60*60;
GIFT_WEEKLY_COLLECTION_LIMIT     = 24;
BRAG_REWARD_TIME_LIMIT           = 1*24*60*60;

-- Auto and Store Purchase
AUTO_PURCHASE_DATA					= {};
AUTO_PURCHASE_FUNC					= nil;
STORE_PURCHASE_DATA					= {};

----------------------------------------------------------------------

local Base           = Object;
App                  = Base:new();

----------------------------------------------------------------------

function App:init()

   self.updateThread = MOAIThread.new ();
   self.isSameOfferSession = false;
   self.shouldResetWebTimestamp = true;
   self.giftsFromServer = nil;
   self.gameReviveIndex = 0;
   App.VideoReviveCount = 0;

   self:_createAdjustMap();

   MOAISim.setTraceback( _onAppTraceback );

   math.randomseed( os.time() );

   self.info                               =
   {
      id          	= APP_ID,
      appStoreUrl 	= APP_STORE_LINK,
      googlePlayUrl  = GOOGLE_PLAY_LINK,
      amazonStoreUrl = AMAZON_STORE_LINK,
      name        	= APP_NAME,
      version     	= MOAIRoadRiot:getVersion(),
      versionNumber  = self:versionStringToNumber(MOAIRoadRiot:getVersion()),
   };

   self.gameFile                           = GAME_FILENAME;
   self.viewWidth,     self.viewHeight     = self:_calcViewSize(     TARGET_VIEWPORT_WIDTH, TARGET_VIEWPORT_HEIGHT);
   self.viewportWidth, self.viewportHeight = self:_calcViewportScale(TARGET_VIEWPORT_WIDTH, TARGET_VIEWPORT_HEIGHT, self.viewWidth,self.viewHeight);

   self.maxTextureSize =      MOAIGfxDevice.getMaxTextureSize();

   self.useMiniTexture =      HACK_FORCE_MINI_TEXTURE
                        or   (        self.viewWidth/TARGET_VIEWPORT_WIDTH   < 0.75
                                and   self.viewHeight/TARGET_VIEWPORT_HEIGHT < 0.75 );


   if (self.maxTextureSize <= MINI_TEXTURE_SIZE_MAX) and (not self.useMiniTexture ) then
      MOAIFlurry:event( "Error : Mini Texture",
                        "info",                    "calculation was wrong",
                        "Texture Max Size",        tostring(self.maxTextureSize),
                        "View Size",               tostring(self.viewWidth).."x"..tostring(self.viewHeight));
      self.useMiniTexture = true;
   end

   --If the device has a smaller screen size, it must be a lower end device... use mini Audio as well
   self.useMiniAudio    = self.useMiniTexture;

   self.headerAvailable = self.viewportHeight >= MIN_VIEWPORT_HEIGHT_HEADER;

   self.scoresTableName = SCORE_TABLE_NAME;

   self.viewport        = self:_initView (APP_NAME, self.viewWidth, self.viewHeight, self.viewportWidth, self.viewportHeight);
   self.layer           = self:_initLayer(self.viewport) ;

   self.timeScale       = math.clamp( math.lerp( self.viewportHeight/MAX_TIME_SCALE_HEIGHT, 1, 0.5), 0.9, 1.0);

   -- Garbage collector
   self.timeSinceGC     = 0;
   self.runningGC       = true;
   self.cloudId         = nil;
   self:_FixeUpMOAI();

   --The templateMgr must be initiate before we load our data
   TemplateMgr:init( TEMPLATE_SAVE_FILENAME );

   self.data = AppData:new( self:calcEncryptionKey() );

   TransactionMgr:init();

   AnalyticsMgr:init();

   FraudMgr:init();

   --Must be before _initLaunchData
   self:_detectCrash();

   self:_initLaunchData();

   self:_initListeners();

   self:_initAudio();

   TimeMgr:init();

   ProfilerMgr:init();

   NetworkMgr:init();

   OnlineMgr:init();

   LocalizationMgr:init();

   LoginMgr:init();

   ContactMgr:init();

   RankingMgr:init( SCORE_TABLE_NAME );

   MessageMgr:init();

   GhostMgr:init();

   AdMgr:init()

   PossessionMgr:init();
   JuneApi:init();

   self.roboDJ = RoboDJ:new();
   self.roboDJ:onStarted();

   StateMgr:init( "state/state-splash.lua", self.headerAvailable );
--   StateMgr:init( "state/state-test-layout.lua", self.headerAvailable );
--   StateMgr:init( "state/support.lua", self.headerAvailable );

   StoreMgr:init(  );

   AbTestMgr:init();

   LeagueMgr:init();

   GuestMgr:init();

   GuideMgr:init(  );

   PromoMgr:init(  );

   FeedMgr:init();

   NotificationMgr:init();

   AchievementMgr:init();

   BountyMgr:init();

   DroneMgr:init();

   MonkeyTester:init();
   MonkeyTester:setEnabled(MONKEY_TESTER_ENABLED);

   self:_pickAllAbTests();

   self:_initGameCenter();

   self:_initThread();

   self._ready = true;

   --Check to see if we are coming off an update (or first launch)
   if self.data:getValue("last-played-version") ~= self.info.version then
      EventMgr:postEvent({old=self.data:getValue("last-played-version"), new=self.info.version}, "new-version" );
      self.data:setValue("last-played-version",self.info.version);
   end

   -- Used for gathering stats on garbage collector stepping.
   if BUILD_DEBUG then
      self.verboseGCStats = false;
      self.stepsSinceLastFullcycle = 0;
      self.timeOfLastFullCycle = 0;
      self.stepTimes = {};
   end


    if self.info.version == "1.29.21" then
        AbTestMgr:resetValue("revive_cost_v5");
    end

    App:exitOnBackground();
end

----------------------------------------------------------------

function App:_initThread()

   MOAISim.setStep(1/30);
   MOAISim.clearLoopFlags(MOAISim.SIM_LOOP_ALLOW_SPIN);

   -- MOAIActionMgr.setProfilingEnabled( true );
   -- MOAIActionMgr.setThreadInfoEnabled( true );

end

----------------------------------------------------------------

function App:_initListeners()



   EventMgr:addListener( self, self.onAppSettings,             "cloud:master-data-app-settings"    );

   EventMgr:addListener( self, self.onMissionCompleted,			"mission-completed"                 );
   EventMgr:addListener( self, self.onDailyCompleted,				"daily-success"                     );
   EventMgr:addListener( self, self.onAchievementCompleted,		"achievement-completed"             );
   EventMgr:addListener( self, self.onMissionReward,				"mission-reward"                    );
   EventMgr:addListener( self, self.onDailyReward,					"daily-reward"                      );
   EventMgr:addListener( self, self.onAchievementReward,       "achievement-reward"                );
   EventMgr:addListener( self, self.onMissionGoalCompleted,		"mission-goal-completed"            );
   EventMgr:addListener( self, self.onMissionGoalProgress,		"mission-goal-progress"             );
   EventMgr:addListener( self, self.onBountyCollect,           "bounty-collected"                  );
   EventMgr:addListener( self, self.onBountyPickup,            "bounty-pickup"                     );

   EventMgr:addListener( self, self.onAvatarPickup,            "avatar-pickup"                     );
   EventMgr:addListener( self, self.onGameStart,					"game-start"                        );
   EventMgr:addListener( self, self.onGameEnd,                 "game-end"                          );

   EventMgr:addListener( self, self.onIapVerificationAccepted, "iap-verification-accepted"         );
   EventMgr:addListener( self, self.onIapVerificationDeclined, "iap-verification-declined"         );
   EventMgr:addListener( self, self.onIapVerificationFailure,  "iap-verification-failure"          );

   EventMgr:addListener( self, self._onStateChanged,           "StateMgr.state.new"                );
   EventMgr:addListener(self, self.onGameStatusCheckFailed, "game-status-check-failed" );
   EventMgr:addListener(self, self.onGameStatusCheckPassed, "game-status-check-passed" );

   EventMgr:addListener(self, self.onPlayerStatusCheckFailed, "player-status-check-failed" );
   EventMgr:addListener(self, self.onPlayerStatusCheckPassed, "player-status-check-passed" );

   MOAIApp.setListener ( MOAIApp.BACK_BUTTON_PRESSED, self.onBackButtonPressed );

end

----------------------------------------------------------------

function App:_initLaunchData()

   local launchCount = (App.data:getValue("launch-count") or 0)+1;
   local firstLaunch = App.data:getValue("firstLaunch") or os.time();
   local since       = math.floor((os.time()-firstLaunch)/(24*60*60));
   local firstVersion= self.data:getValue("first-version-played");

   if  firstVersion == nil then

      --Because we introduce this well into development, we need to find a way
      --to not create fake "new user"... since we will be using this to detect new user

      if self.data:getGamePlayed() > 0 then
         firstVersion = self:versionStringToNumber("1.00.00")
      else
         firstVersion = self.info.versionNumber;
      end

      self.data:setValue("first-version-played", firstVersion );

   end

   App.data:setValue("firstLaunch",  firstLaunch );
   App.data:setValue("launch-count", launchCount );

   local dailyReset = nil;
   if self.data:getDailyChallengeTime() then
      dailyReset = os.date("%H", self.data:getDailyChallengeTime()) .. ":" ..   os.date("%M", self.data:getDailyChallengeTime());
   end
   MOAIFlurry:event( "App-Launch-Info" ,     "gamePlayed",        tostring(App.data:getGamePlayed())
                                       ,     "firstLaunch",       tostring(since)
                                       ,     "firstVersion",      tostring(firstVersion)
                                       ,     "launch-count",      tostring(launchCount)
                                       ,     "key",               tostring(App.data:getKeyCount())
                                       ,     "hour",              tostring(os.date("%H", os.time()))
                                       ,     "Daily Reset Time",  tostring(dailyReset)
                                       ,     "level",             tostring(App.data:getCurrentPlayerLevel())
                                       ,     "level by 10",       tostring(math.max(math.floor(App.data:getCurrentPlayerLevel()/10)*10,1)));


   MOAIFlurry:event( "App-Launch-Info-2",     "view-width",        tostring(self.viewWidth)
                                          ,   "view-height",       tostring(self.viewHeight)
                                          ,   "texture-max-size",  tostring(self.maxTextureSize)
                                          ,   "mini-texture",      tostring(self.useMiniTexture)
                                          ,   "header",            tostring(self.headerAvailable)
                                          ,   "big-endian",        tostring(BIG_ENDIAN)                  );

   if MOAITangoSdk:tangoIsInstalled() then
      MOAIFlurry:event( "Tango-Installed");
   end

   if MOAITangoSdk:tangoHasSdkSupport() then
      MOAIFlurry:event( "Tango-SDK-Supported");
   end

   if not App.data:getValue("firstLaunchDone") then

      local tangoState = "";
      if MOAITangoSdk:tangoHasSdkSupport() then
         tangoState = "yes-sdk-supported";
      elseif MOAITangoSdk:tangoIsInstalled() then
         tangoState = "yes-sdk-unsupported";
      else
         tangoState = "no";
      end

      --For funnel reason, we need a separate event
      MOAIFlurry:event( "FirstLaunch-Tango-"..tangoState);

      MOAIFlurry:event(    "FirstLaunch",
                           "tango-installed",           tangoState,
                           "hour",                    ""..os.date("%H", os.time()));

      App.data:setValue("firstLaunchDone", 1 );

   end

   App.data:setValue("session.games", 0);

   --Because we introduce the notion of GamePlayedGroup later on
   -- we cant rely on "Start Session"... we need to have our own event to start the funnel
   MOAIFlurry:event("GamePlayedGroup");


end

----------------------------------------------------------------

function App:_initAudio()

   if  App.data:getValue("Audio.GlobalSoundVolume", DEFAULT_GLOBAL_SOUND_VOLUME) > 0.0 then
      self.data:setValue("Audio.GlobalSoundVolume", DEFAULT_GLOBAL_SOUND_VOLUME);
   end

   if  App.data:getValue("Audio.GlobalMusicVolume", DEFAULT_GLOBAL_MUSIC_VOLUME) > 0.0 then
      self.data:setValue("Audio.GlobalMusicVolume", DEFAULT_GLOBAL_MUSIC_VOLUME);
   end

   AudioMgr:init( self.data:getGlobalVolume(),
                  self.data:getValue("Audio.GlobalSoundVolume"),
                  self.data:getValue("Audio.GlobalMusicVolume") );

end

----------------------------------------------------------------

function App:_detectCrash()

   local state = App.data:getValue("App.data.crash.state");

   if  state ~= nil then

      EventMgr:postEvent({ count=App.data:deltaValue("App.data.crash.count",1),
                           state=state,
                           sessionGames=App.data:getValue("session.games") }, "crash-detected");

   end

   App.data:setValue("App.data.crash.state","running");

end

----------------------------------------------------------------

function App:isVideoRewardEnabled()

   --Disabled for amazon
   if AmazonBillingProvider == true then
      return false;
   elseif MOAIEnvironment.osBrand == "Android" then
      return true;
   else -- iOS
      return true;
   end

end

----------------------------------------------------------------

function App:isSavingForCriticalOnly()

   return self.data:getValue("save-critical-only", false);

end

----------------------------------------------------------------

function _onAppTraceback(msg, stack)

   MOAIFlurry:event("Lua Traceback", "msg", msg, "stack", stack);

   if string.find(msg,"Debug.assert") then

      MOAIFlurry:event("Lua Traceback-Debug.assert", "msg", msg, "stack", stack);

   elseif string.find(msg,"Debug.error") then

      MOAIFlurry:event("Lua Traceback-Debug.error", "msg", msg, "stack", stack);

   else

      MOAIFlurry:event("Lua Traceback-Crash", "msg", msg, "stack", stack);
      MOAIFlurry:error("Lua Traceback-".. (msg or {}), stack);

   end
end

----------------------------------------------------------------

function App:gotoPage(url)

   if MOAIEnvironment.osBrand == "iOS" then
      MOAISafariIOS.openURL(url);
   else
      MOAIAppAndroid.openURL(url);
   end

end

----------------------------------------------------------------

function App:gotoUpdatePage()

   if MOAIEnvironment.osBrand == "iOS" then
      MOAISafariIOS.openURL(APP_STORE_LINK);
   else
      MOAIAppAndroid.openURL(GOOGLE_PLAY_LINK);
   end

end

----------------------------------------------------------------

function App:gotoReviewPage()

   if MOAIEnvironment.osBrand == "iOS" then
      MOAISafariIOS.openURL(APP_STORE_REVIEW_LINK);
   else
      MOAIAppAndroid.openURL(GOOGLE_PLAY_REVIEW_LINK);
   end

end

----------------------------------------------------------------

function App:gotoChannelPage()

   if MOAIEnvironment.osBrand == "iOS" then
      MOAISafariIOS.openURL(CHANNEL_LINK);
   else
      MOAIAppAndroid.openURL(CHANNEL_LINK);
   end

end

----------------------------------------------------------------

function App:createAuthenticateName()

   if LoginMgr:isTango() then
      return "Tango-User";
   else
      return "Guest-User";
   end

end

----------------------------------------------------------------

function App:createGarageBadgingName()

   local index  = self.data:getValue("garage-badging") or 1;

   return "Garage-Badging-"..index;

end

----------------------------------------------------------------

function App:_pickAllAbTests()

   self:_pickTangoBadgeIndex();
	self:_pickClaireNewCarIndex();
   self:_fixupGarageUseCount();

end

----------------------------------------------------------------

function App:_pickTangoBadgeIndex()

   --Only pick it once
   local index = App.data:getValue("tango-badge-index") or math.random(1,100);

   App.data:setValue( "tango-badge-index", index );


   -- We are moving forward and use Popup 3 100%
   local version = "Popup";
	local wait = 3;
--[[
   if index <= 20 then
      version = "Original";
   elseif index <= 40 then
      version = "Icons";
   elseif index <= 60 then
      version = "Popup";
		wait = 0;
   elseif index <= 80 then
      version = "Popup";
		wait = 3;
   else
      version = "Popup";
		wait = 6;
   end ]]


   MOAIFlurry:event("Test-Badge-Version-"..version.."-wait-"..wait);

	App.data:setValue( "tango-badge-version",  version  );
	App.data:setValue( "tango-badge-popup-wait",  wait  );
	local mWaitCount = App.data:getValue("tango-badge-popup-waitCount-mission") or -1;
	local dWaitCount = App.data:getValue("tango-badge-popup-waitCount-daily") or -1;
	local mFirstCollect = App.data:getValue("tango-badge-popup-first-collect-mission") or -1;
	local dFirstCollect = App.data:getValue("tango-badge-popup-first-collect-daily") or -1;
	local mGamesSince = App.data:getValue("mission-games-since-complete") or -1;
	local dGamesSince = App.data:getValue("daily-games-since-complete") or -1
   local aGamesSince = App.data:getValue("achv-games-since-complete") or -1;

	App.data:setValue( "tango-badge-popup-waitCount-mission",      mWaitCount           );
	App.data:setValue( "tango-badge-popup-waitCount-daily",        dWaitCount           );
	App.data:setValue( "tango-badge-popup-first-collect-mission",  mFirstCollect        );
	App.data:setValue( "tango-badge-popup-first-collect-daily",    dFirstCollect        );
	App.data:setValue( "daily-games-since-complete",               dGamesSince          );
	App.data:setValue( "mission-games-since-complete",             mGamesSince          );
   App.data:setValue( "achv-games-since-complete",                aGamesSince          );



   print( "App.data:getValue(tango-badge-index)" , App.data:getValue("tango-badge-index")
                                                   , App.data:getValue("tango-badge-version")
																	, App.data:getValue("tango-badge-popup-wait")
																	, App.data:getValue("tango-badge-popup-waitCount-mission")
																	, App.data:getValue("mission-games-since-complete")
                                                   , App.data:getValue("achv-games-since-complete")
																	, App.data:getValue("tango-badge-popup-waitCount-daily")
																	, App.data:getValue("daily-games-since-complete") );

end

----------------------------------------------------------------

function App:calcEncryptionKey()

   return "oidqwu!@#@$#@poiduwqdoiqwudsdaslkdas'l;kd's;alqk&79209UUjjdlslljdlkdjal;kjd__8798723198732109-=0!#!@KKLS::S:dks;s;jdlkasjddsakdjsalkds&*^&^*^*&^5556ddshaskjgdaskjhdskajhsahkVVKJHAKJHsakdjhaslkhdj&*(^*&^%$^%$^@##8329823083298djksahldkjashdk(*()*_)hhkdshklahslkdh";

end

----------------------------------------------------------------

function App:_fixupGarageUseCount()

   local count = App.data:getValue("garage-use-count");

   if count == nil then

         --This is to not polluated the test of how many games before using the garage
         -- If this is not a new player... just mark it as he use the garage once
      if App.data:getGamePlayed() > 0 then
         App.data:setValue("garage-use-count", 1);
      else
         App.data:setValue("garage-use-count", 0);
      end

   end


end
----------------------------------------------------------------

function App:_pickClaireNewCarIndex()

   --Only pick it once
   local index = App.data:getValue("ab-claire-newcar-index");

   if index == nil then

		index = math.random(1,100);

		App.data:setValue( "ab-claire-newcar-index", index );

		if index < 50 then
			App.data:setValue( "ab-claire-newcar-flow", "force-car-view"    );
		else
			App.data:setValue( "ab-claire-newcar-flow", "car-reminder"      );
		end

   end

   local flowType = App.data:getValue( "ab-claire-newcar-flow" );
	print("App:_pickClaireNewCarFlow:", flowType);

   if flowType then
      --MOAIFlurry:event("ab-claire-newcar-flow-"..flowType);
      --joe add flurry here?
   end

end


----------------------------------------------------------------

function App:onFromUrl( url )

   if #url > 15 then
      url = "authentication";
   end

   MOAIFlurry:event( "launchFromUrl", "Url", url or "" );

   if url and  string.find(url,'gift')  then
      MessageMgr:checkMailNow();
   end

end


----------------------------------------------------------------

function App:PauseGameplay()

   MOAIFlurry:eventStart("App-PauseGameplay" );

   if StateMgr then
   		StateMgr:PauseGameplay();
   end

   --Save when going in background in case app data was planning to do it on the next update
   if self.data then
      self.data:save();
   end

end

----------------------------------------------------------------

function App:exitOnBackground()

   self._exitOnBackground = true;

end

----------------------------------------------------------------

function App:onEnterBackground()

   if self._ready then

      AdMgr:clearAdImpressions();

      self._backgroundTime = os.time();

      MOAIFlurry:eventStart("App-Backgrounded", "ending", "killed", "time", "-1" );

      StateMgr:onEnterBackground();

      StoreMgr:onEnterBackground();

      NotificationMgr:onEnterBackground();

      --Make it last because we are playing with the time
      TimeMgr:onEnterBackground();

      OnlineMgr:onEnterBackground();

      --Flush our events
      -- This is usefull for Analytics Mgr and other system
      EventMgr:update(0.01);

      App.data:setValue("App.data.crash.state", nil);

      --Save at the end when going in background in case app data was planning to do it on the next update
      if self.data then
         self.data:save();
      end

        print("App:EnterBg");

      if (self._exitOnBackground) and (not self.gamePlaying) and (not AdMgr.IsAdPlaying) then

         MOAIFlurry:event("App-Exit", "type", "_exitOnBackground" );
         MOAIRoadRiot:exit();

      end
   else

      --Since we are not in a good state, just exit out
      MOAIFlurry:event("App-Exit", "type", "_ready"  );
      MOAIRoadRiot:exit();

   end

end

----------------------------------------------------------------

function App:onEnterForeground()

   if self._ready then

      MOAIFlurry:eventStop("App-Backgrounded", "ending", "foreground", "time", ""..(os.time()-(self._backgroundTime or os.time()) ) );

      self._backgroundTime = nil;
      self.isSameOfferSession = false;
      self.shouldResetWebTimestamp = true;
      App.data:setValue("App.data.crash.state","running");

      OnlineMgr:onEnterForeground();
      TimeMgr:onEnterForeground();
      StateMgr:onEnterForeground();
      ContactMgr:onEnterForeground();
      RankingMgr:onEnterForeground();
      StoreMgr:onEnterForeground();
      CloudMgr:onEnterForeground();

      NotificationMgr:onEnterForeground();

   end

end

----------------------------------------------------------------

function App:CheckStatuses()
    print ("App:CheckStatuses : "..debug.traceback());
    JuneApi:CheckGameStatus();
    JuneApi:CheckPlayerStatus();
    delay( 5, function()
                JuneApi:CheckGifts();
             end );

end
----------------------------------------------------------------

function App:onReceiveMemoryWarning()

   EventMgr:postEvent( { }, "low-memory-warning" );

end

----------------------------------------------------------------

function App:onBackButtonPressed()

   MOAIFlurry:event("Back-Button-Pressed");

	StateMgr:onBackButton();

   return true;

end

----------------------------------------------------------------------

function App:hackReset( gem, gold, key )

   if LoginMgr:isTango() and OnlineMgr:isOnline() and (not self._hackResetDone) then

      if self.data:hackReset( gem, gold, key ) then

         self._hackResetDone = true;
         self:_pickAllAbTests();

      end

   end

end

----------------------------------------------------------------

local function _HackTestStates()

   local tmp =
   {
      { 4.00, function() App.data:setValue("currentLevel", 10);                  end; },
      { 4.25, function() StateMgr:push("state/state-garage.lua");                end; },
      { 4.50, function() StateMgr:swap("state/state-pickup.lua");                end; },
      { 5.00, function() StateMgr:swap("state/state-coin.lua");                  end; },
      { 5.50, function() StateMgr:swap("state/state-key.lua");                   end; },
      { 6.00, function() StateMgr:swap("state/state-gem.lua");                   end; },
      { 6.50, function() StateMgr:swap("state/state-options.lua");               end; },
      { 7.00, function() StateMgr:swap("state/state-credits.lua");               end; },
      { 7.50, function() StateMgr:swap("state/state-daily.lua");                 end; },
      { 8.00, function() StateMgr:swap("state/state-invite.lua");                end; },
      { 8.50, function() StateMgr:swap("state/state-mission-popup.lua");         end; },
      { 9.00, function() StateMgr:push("state/state-missionskip-confirm.lua");   end; },
      { 9.50, function() StateMgr:pop();                                         end; },
      {10.00, function() StateMgr:pop();                                         end; },
      {10.50, function() App.data:setValue(App.data:createPickupId(PICKUP_COIN_INDEX), 1);
                         PromoMgr:createPopup( "doubleCoin" );               end; },
      {11.00, function() StateMgr:pop();                                         end; },
      {11.50, function() App.data:addKey(1); StateMgr.curState:goGame();         end; },
      {13.50, function() StateMgr:pop();                                         end; },
      {14.00, function() StateMgr:push( "state/state-pause.lua" );               end; },
      {14.50, function() HACK_ALWAYS_LEVEL = true;  StateMgr.curState:goQuit();  end; },
      {19.00, function() StateMgr:pop();                                         end; },
      {19.50, function() StateMgr.curState:goBrag();                             end; },
      {20.00, function() StateMgr:pop();                                         end; },
      {20.50, function() StateMgr.curState:goBack();                             end; },
      {21.00, function() StateMgr:push("state/state-achievement.lua");           end; },
      {26.00, function() StateMgr:swap("state/state-mystery-box.lua");           end; },
      {31.00, function() StateMgr:pop();                                         end; },

      --{32.00, function() PromoMgr:createPopup("holidaySpecial");                 end; },
      --{33.00, function() StateMgr:pop();                                         end; },

--[[
      {34.00, function() PromoMgr:createPopup("updateGame");                     end; },
      {35.00, function() StateMgr:pop();                                         end; },

      {36.00, function() PromoMgr:createPopup("rateGame");                       end; },
      {37.00, function() StateMgr:pop();                                         end; },

      {38.00, function() PromoMgr:createPopup("channelPage");                    end; },
      {39.00, function() StateMgr:pop();                                         end; },

      {40.00, function() PromoMgr:createPopup("tangoLogin");                     end; },
      {41.00, function() StateMgr:pop();                                         end; },

      {42.00, function() PromoMgr:createPopup("oldTangoLogin");                  end; },
      {43.00, function() StateMgr:pop();                                         end; },

      {44.00, function() PromoMgr:createPopup("giftFriends");                    end; },
      {45.00, function() StateMgr:pop();                                         end; },
]]
   }

   for i,v in ipairs(tmp) do
      delay(v[1], function ()
                     v[2]();
                  end                                    );
   end

--[[
   -- Go in and out of the game continously for a number of passes.
   local gameStateTestLoop = {
      {0, function() App.data:addKey(1); StateMgr.curState:goGame();         end; },
      {2, function() StateMgr:pop();                                         end; },
      {3.5, function() StateMgr:push( "state/state-pause.lua" );               end; },
      {4, function() HACK_ALWAYS_LEVEL = true;  StateMgr.curState:goQuit();  end; },
      {8.5, function() StateMgr:pop();                                         end; },
      {9, function() StateMgr.curState:goBrag();                             end; },
      {9.5, function() StateMgr:pop();                                         end; },
      {10, function() StateMgr.curState:goBack();                             end; }
   }

   local START_TIME = 32;
   local TOTAL_PASS_TIME = gameStateTestLoop[#gameStateTestLoop][1] + 1;
   local NUM_PASSES = 10;

   for passes = 0, NUM_PASSES - 1 do

      for i, v in ipairs(gameStateTestLoop) do

         delay(START_TIME + passes * TOTAL_PASS_TIME + v[1],
                 function() v[2]() end);

      end
   end
]]

end

----------------------------------------------------------------
-- run loop
----------------------------------------------------------------

local function updateFunction ()

   local last     = MOAISim.getElapsedTime ();

   -- _HackTestStates();

   while true do

      coroutine.yield ()

      local time     = MOAISim.getElapsedTime ();
      local dt       = math.clamp((time - last)*App.timeScale, 1/100, 1/20);
      local dt2      = math.clamp((time - last), 1/100, 1/20);

      last           = time;

      TimeMgr:update(dt);

      ProfilerMgr:update(dt);

      NetworkMgr:update(dt);

      OnlineMgr:update(dt);

      App.data:update(dt);

      InputMgr:update(dt2);

      MonkeyTester:update(dt);

      EventMgr:update(dt);

      AudioMgr:update(dt);

      StateMgr:update(dt);

      GuideMgr:update(dt);

      RoboDJ:update(dt);

      ContactMgr:update(dt);

      RankingMgr:update(dt);

      MessageMgr:update(dt);

      PossessionMgr:update(dt);

      CloudMgr:update(dt);

      AdMgr:update(dt);

      MOAIRoadRiot:update(dt);

      StoreMgr:update(dt);

      App:updateGC(dt);

      if HACK_DATA_RESET then
         App:hackReset( 0, 0, 0 );
	     FraudMgr:hackReset();
      end

   end
end

----------------------------------------------------------------------

function App:updateGC(dt)

   if self.runningGC then
      local cycleFinished = App:_stepGC();

      if cycleFinished then
         self.runningGC = false;
         self.timeSinceGC = 0;
         --print("End GC");
      end
   else
      self.timeSinceGC = self.timeSinceGC + dt;
      if self.timeSinceGC >= TIME_BETWEEN_GC_CYCLE then
         self.runningGC = true;
         --print("Start GC");
      end
   end

end

----------------------------------------------------------------

function App:collectGarbageNow()
   print("Forcing garbage collection!");
   MOAISim.forceGarbageCollection ();

   self.runningGC   = false;
   self.timeSinceGC = 0;
end

----------------------------------------------------------------------

function App:start()


	self.updateThread:run ( updateFunction )

end

----------------------------------------------------------------

function App:stop ( )

	self.updateThread:stop ()

end

-------------------------------------------------------------------------------------

function App:getDeviceSize()

    if HACK_FORCE_IPHONE4 then
        return  640,960;
    else
        return  MOAIGfxDevice.getViewSize();
    end

end

-------------------------------------------------------------------------------------

function App:_FixeUpMOAI()

   X                             = 1;
   Y                             = 2;

   MOAITextBox.BOTTOM_JUSTIFY    = MOAITextBox.RIGHT_JUSTIFY;
   MOAITextBox.TOP_JUSTIFY       = MOAITextBox.LEFT_JUSTIFY;

end


-------------------------------------------------------------------------------------

function App:_initLayer(viewport)

   local layer = MOAILayer2D.new();

   layer:setViewport ( viewport )
   MOAISim.pushRenderPass ( layer )

   return layer;

end

-------------------------------------------------------------------------------------

function  App:_initView( name, viewWidth,  viewHeight, scaleX, scaleY )

   --Will be ignore on a smartphone
   MOAISim.openWindow ( name, scaleX, scaleY  )

   local viewWidth2, viewHeight2   = self:getDeviceSize();

   local left     = (viewWidth2-viewWidth)/2;
   local  top     = (viewHeight2-viewHeight)/2;
   local right    = viewWidth2-left;
   local bottom   = viewHeight2-top;

   local viewport = MOAIViewport.new ()
   viewport:setSize ( left, top, right, bottom)
   viewport:setScale( scaleX, scaleY )

   return viewport;

end

-------------------------------------------------------------------------------------

function App:_calcViewSize( camWidth, camHeight )

   local viewWidth, viewHeight   = self:getDeviceSize();

   local viewRatio               = viewHeight/viewWidth;
   local camRatio                = camHeight/camWidth;

   if viewRatio >= camRatio then

      return viewWidth, viewHeight;

   else

      return viewHeight/camRatio, viewHeight;

   end
end

-------------------------------------------------------------------------------------

function App:_calcViewportScale( camWidth, camHeight, viewWidth, viewHeight )

   local viewRatio = viewHeight/viewWidth;
   local camRatio  = camHeight/camWidth;

   local viewportWidth;
   local viewportHeight;

   if viewRatio >= camRatio then

      viewportWidth  = camWidth;
      viewportHeight = camWidth*viewRatio;

   else

      --Dont go smaller than our camera width
      viewportWidth  = camHeight/camRatio;
      viewportHeight = camHeight;

   end

   return  viewportWidth,viewportHeight;

end


----------------------------------------------------------------

function App:_initGameCenter()


   if MOAIGameCenterIOS then
      MOAIGameCenterIOS:setListener(   MOAIGameCenterIOS.AUTHENTICATION_ATTEMPT,
                                       function()
                                          local count = App.data:deltaValue("game-center-authentication-success-attempt",1);
                                          EventMgr:postEvent({attempt_count=count}, "game-center-authentication-attempt");
                                       end
                                    );

      MOAIGameCenterIOS:setListener(   MOAIGameCenterIOS.AUTHENTICATION_SUCCESS,
                                       function()
                                          local count = App.data:deltaValue("game-center-authentication-success-count",1);
                                          EventMgr:postEvent({result="success", success_count=count},                   "game-center-authentication-success");
                                       end
                                    );

      MOAIGameCenterIOS:setListener(   MOAIGameCenterIOS.AUTHENTICATION_FAILURE,
                                       function( errorMsg )
                                          EventMgr:postEvent({result="failure", error=errorMsg}, "game-center-authentication-failure");
                                       end
                                    );
   end

end

----------------------------------------------------------------

function App:updateAchievementActivate()

print("App:updateAchievementActivate",  self.data:isAchievementUnlocked(), AchievementMgr:isActive() );
   if self.data:isAchievementUnlocked() then

      if not AchievementMgr:isActive() then
            AchievementMgr:activate();
      end

   end

end

----------------------------------------------------------------

function App:onAchievementCompleted( )

   local games = App.data:getValue( "achv-games-since-complete" );
   if games and games < 0 then
      App.data:setValue( "achv-games-since-complete", 0 );
   end

end

----------------------------------------------------------------

function App:onAchievementReward( )

   App.data:setValue( "achv-games-since-complete", -1 );

end

----------------------------------------------------------------

function App:updateMissionActivate()

   if self.data:isMissionUnlocked() then

      local tplMission = self.data:getCurrentMissionTemplate();

      if tplMission then
         self.mission = self.mission or Mission:new( tplMission );
         self.mission:activate();
      end

   else

      if self.mission then
         self.mission:deactivate();
      end

   end
end

----------------------------------------------------------------

function App:onAppSettings( data )

   local content = (data or {}).content;

   for k,v in pairs(content) do

      App.data:setValue("App.settings."..tostring(k), v);

   end

end

----------------------------------------------------------------

function App:onMissionCompleted( )

	--funnel
	local version = App.data:getValue("tango-badge-version");
	if version == "Popup" then
		App.data:setValue( "tango-badge-popup-waitCount-mission", 0 );
	end
	App.data:setValue( "mission-games-since-complete", 0 );

end

----------------------------------------------------------------

function App:onMissionReward( )

	--send flurry about games since mission visit
	print("Funnel");
	local mGames = App.data:getValue("mission-games-since-complete");
	local version = App.data:getValue("tango-badge-version");
	local wait = App.data:getValue("tango-badge-popup-wait");
   local first = App.data:getValue( "tango-badge-popup-first-collect-mission" ) or -1;
	local flurryName = "Test-Collect-Mission-".. version;
	if version == "Popup" then
		flurryName = flurryName.. "-".. tostring(wait);
	end

   if first < 0 then
      first  = mGames;
      App.data:setValue( "tango-badge-popup-first-collect-mission", mGames );

      MOAIFlurry:event( "Test-Collect-Mission"
                      , "type",  flurryName
                      , "games", ""..mGames
                      , "first", ""..first    );


   end

	MOAIFlurry:event( flurryName, "games", ""..mGames
                               , "first", ""..first      );
	print(flurryName, mGames);





	App.data:setValue( "mission-games-since-complete", -1 );

   MOAIFlurry:event("Mission-Badge-Collect", "games", ""..mGames);

end

----------------------------------------------------------------

function App:onDailyCompleted( )

	--funnel
	local version = App.data:getValue("tango-badge-version");
	if version == "Popup" then
		App.data:setValue( "tango-badge-popup-waitCount-daily", 0 );
	end
	App.data:setValue( "daily-games-since-complete", 0 );

end

----------------------------------------------------------------

function App:onDailyReward( )

	print("Funnel");
	local dGames = App.data:getValue("daily-games-since-complete");
	local version = App.data:getValue("tango-badge-version");
	local wait = App.data:getValue("tango-badge-popup-wait");
	local flurryName = "Test-Collect-Daily-".. version;
	if version == "Popup" then
		flurryName = flurryName.. "-".. tostring(wait);
	end
   local first = App.data:getValue( "tango-badge-popup-first-collect-daily" ) or -1;
   if first < 0 then
      first  = dGames;
      App.data:setValue( "tango-badge-popup-first-collect-mission", dGames );

      MOAIFlurry:event( "Test-Collect-Daily"
                      , "type",  flurryName
                      , "games", ""..dGames
                      , "first", ""..first    );

   end

	MOAIFlurry:event( flurryName, "games", ""..dGames
                               , "first", ""..first );
	print(flurryName, dGames);

	App.data:setValue( "daily-games-since-complete", -1 );

   MOAIFlurry:event("Daily-Badge-Collect", "games", ""..dGames);

end

----------------------------------------------------------------

function App:onGameStart( )

   self.gamePlaying = true;

   local mGames         = App.data:getValue("mission-games-since-complete");
   local dGames         = App.data:getValue("daily-games-since-complete");
   local aGames         = App.data:getValue("achv-games-since-complete");
   local bGames         = App.data:getValue("boss-games-since-encounter") or 0;
   local bountyGames    = App.data:getValue( "guide-games-since-bounty-collect");
   local mFirst         = App.data:getValue( "tango-badge-popup-first-mission-daily" ) or -1;
   local dFirst         = App.data:getValue( "tango-badge-popup-first-collect-daily" ) or -1;
   local version        = App.data:getValue("tango-badge-version");
   local wait           = App.data:getValue("tango-badge-popup-wait");
   local mysteryGames   = App.data:getValue("mystery-games-since-screen");
   local flurryName;

   --joehack: once the rollout changes via AB, this list needs to be generated based on the current rollout
   local rolloutList     = { "daily", "mission", "achv", "mystery", "video" };

   if mGames then
      flurryName = "Test-Ignore-";
      if mGames >= 0 then
         mGames = mGames + 1;
         flurryName = flurryName.."Mission-"..version;
         if version == "Popup" then
            flurryName = flurryName.."-"..tostring(wait);
         end
         MOAIFlurry:event( flurryName, "games", ""..mGames, "first", ""..mFirst );
         print(flurryName, mGames);
         App.data:setValue( "mission-games-since-complete", mGames );
      end
   end

   if dGames then
      flurryName = "Test-Ignore-";
      if dGames >= 0 then
         dGames = dGames + 1;
         flurryName = flurryName.."Daily-"..version;
         if version == "Popup" then
            flurryName = flurryName.."-"..tostring(wait);
         end
         MOAIFlurry:event( flurryName, "games", ""..dGames, "first", ""..dFirst );
         print(flurryName, dGames);
         App.data:setValue( "daily-games-since-complete", dGames );
      end
   end

   if aGames then
      if aGames >= 0 then
         aGames = aGames + 1;
         App.data:setValue( "achv-games-since-complete", aGames );
      end
   end

   if bGames then
      bGames = bGames + 1;
      App.data:setValue("boss-games-since-encounter", bGames);
   end

   if bountyGames and bountyGames >= 0 then
	  bountyGames = bountyGames + 1;
	  App.data:setValue( "guide-games-since-bounty-collect", bountyGames );

	  --MOAIFlurry:event( "Test-Ignore-Bounty", "games", ""..bountyGames );
   end

   local mTokens = App.data:getFreebieCount("mystery");
   if mTokens and mTokens > 0 then
      if not mysteryGames then
         App.data:setValue("mystery-games-since-screen", 1);
      else
         if mysteryGames >= 0 then
            App.data:deltaValue("mystery-games-since-screen", 1);
         end
      end
   else
      App.data:setValue("mystery-games-since-screen", 0);
   end

   if rolloutList and #rolloutList > 0 then
      for i,v in ipairs( rolloutList ) do
         local id = "rollout_games_"..v;
         local cGames = App.data:getValue(id);
         if cGames and cGames >= 0 then
            App.data:deltaValue(id, 1);
         end
      end
   end

   App.data:deltaValue("session.games", 1);

end

----------------------------------------------------------------

function App:onGameEnd( )

   self.gamePlaying = false;

end

----------------------------------------------------------------

function App:onBountyCollect( event )

   App.data:setValue( "guide-games-since-bounty-collect", -1 );

end
----------------------------------------------------------------

function App:onBountyPickup( event )

   App.data:setValue( "guide-games-since-bounty-collect", 0 );

end

----------------------------------------------------------------

function App:onMissionGoalCompleted( event )

   self.data:onMissionGoalCompleted( event.goal.index );

end

----------------------------------------------------------------

function App:onMissionGoalProgress()

end

----------------------------------------------------------------

function App:onAvatarPickup( p )

   if p.pickup.pickupType == "daily" then
      self.data:onDailyPickup();

   elseif p.pickup.pickupType == "mystery" then
      self.data:giveFreebie("mystery", 1, "drop");

   end

end

----------------------------------------------------------------

function App:onIapVerificationAccepted( event )

   print("App:onIapVerificationAccepted");
   self.data:onIapVerificationAccepted();

end

----------------------------------------------------------------

function App:onIapVerificationDeclined( event )

   print("App:onIapVerificationDeclined");
   self.data:onIapVerificationDeclined();

end

----------------------------------------------------------------

function App:onIapVerificationFailure( event )

   print("App:onIapVerificationFailure");
   self.data:onIapVerificationFailure();

end

----------------------------------------------------------------

function App:versionStringToNumber( str )

   local dot   = string.byte(".", 1);
   local n1    = "0";
   local n2    = "0";
   local n3    = "0";
   local start = 1;

   for i=start,#str do
      local b = string.byte(str, i);
      if b == dot then
         n1 = string.sub(str, start, i-1);
         start = i + 1;
         break;
      end
   end

   for i=start,#str do
      local b = string.byte(str, i);
      if b == dot then
         n2 = string.sub(str, start, i-1);
         start = i + 1;
         break;
      end
   end

   n3 = string.sub(str, start, #str);

   local num = tonumber(n1) * 1000000  +
               tonumber(n2) * 1000     +
               tonumber(n3);


   return num;

end

----------------------------------------------------------------

--To be move in analyrics manager
function App:_createAdjustMap()

   self.adjustMap =  {};

   if MOAIEnvironment.osBrand == "Android" then

      self.adjustMap["Tango-Login"]          = "j4050p";
      self.adjustMap["Gameplay"]             = "h9cvrc";
      self.adjustMap["Video"]                = "y51qcv";
      self.adjustMap["Thief"]                = "g1qu1a";
      self.adjustMap["IAP"]                  = "5om7ri";
      self.adjustMap["Tutorial-Start"]       = "sjzrj4";
      self.adjustMap["Tutorial-Progress"]    = "2l8n7v";
      self.adjustMap["Tutorial-Finish"]      = "3icfny";
      self.adjustMap["HC-Spend"]             = "1x36br";
      self.adjustMap["Invite-Sent"]          = "xuts2y";
      self.adjustMap["View-Store"]           = "i1cean";
      self.adjustMap["View-IAP-Store"]       = "zeiwni";
      self.adjustMap["IAP-Clicked"]          = "aclmt5";
      self.adjustMap["Video-Attempt"]        = "2p28x1";
      self.adjustMap["Video-Success"]        = "m81ov7";
      self.adjustMap["Video-Failure"]        = "87ejfn";
      self.adjustMap["Interstitial-Attempt"] = "s168e1";
      self.adjustMap["Interstitial-Success"] = "mekw6u";
      self.adjustMap["Interstitial-Failure"] = "3dbwig";
      self.adjustMap["Level-5"]              = "55v8eg";

   else

      self.adjustMap["Tango-Login"]          = "ag1d5b";
      self.adjustMap["Gameplay"]             = "vh2pxj";
      self.adjustMap["Video"]                = "sxpsz8";
      self.adjustMap["Thief"]                = "eolfgr";
      self.adjustMap["IAP"]                  = "7n0z3g";
      self.adjustMap["IAP-Clicked"]          = "jjkbpp";
      self.adjustMap["Tutorial-Start"]       = "hxn8ye";
      self.adjustMap["Tutorial-Progress"]    = "39mq56";
      self.adjustMap["Tutorial-Finish"]      = "nugjlp";
      self.adjustMap["HC-Spend"]             = "5gnrks";
      self.adjustMap["Invite-Sent"]          = "pzawls";
      self.adjustMap["View-Store"]           = "197laj";
      self.adjustMap["View-IAP-Store"]       = "9n4f55";
      self.adjustMap["Video-Attempt"]        = "rpjvbm";
      self.adjustMap["Video-Success"]        = "r3yhgf";
      self.adjustMap["Video-Failure"]        = "nj6hj5";
      self.adjustMap["Interstitial-Attempt"] = "8b98gw";
      self.adjustMap["Interstitial-Success"] = "7zhw9b";
      self.adjustMap["Interstitial-Failure"] = "rpuwca";
      self.adjustMap["Level-5"]              = "rihs5z";

   end

end

----------------------------------------------------------------

function App:_onStateChanged( event )

 	App.data:setValue("App.data.crash.state", event.state);

end

----------------------------------------------------------------

-- Returns true if a cycle finished, false otherwise
function App:_stepGC()

   -- Invariant: self.verboseGCStats is not changed throughout this function

   -- Measure GC step time
   if BUILD_DEBUG then
      if self.verboseGCStats then
         ProfilerMgr:beginTimestamp("GCStep");
      end
   end

   -- After testing values of 1000 and 100, a value of 50 was chosen because
   -- it causes a full GC cycle to typical finish in about 7 seconds.
   -- Eventually, we can try something more dynamic.
   local cycleFinished = collectgarbage("step", 50);

   -- "step" restarts the GC in Lua 5.1, but not in 5.2
   collectgarbage("stop")

   -- Record stats on GC stepping
   if BUILD_DEBUG then

      if self.verboseGCStats then
         local timeTaken = ProfilerMgr:endTimestamp("GCStep", true);
         table.insert(self.stepTimes, timeTaken);
      end

      if cycleFinished then
         if self.verboseGCStats then

            self.gcCyclesCount = ( self.gcCyclesCount or 0 ) + 1;
            print("GC cycle finished.");
            print("Cycle steps: " .. self.stepsSinceLastFullcycle);

            local timeDiff = MOAISim.getElapsedTime() - self.timeOfLastFullCycle;
            print("Cycle time: " .. timeDiff);

            local sum  = 0;
            local max  = 0;
            local bad1 = 0;
            local bad2 = 0;

            for _, step in ipairs(self.stepTimes) do
               sum = sum + step;
               max = math.max(max, step);
               if step > 0.01 then
                  bad1 = bad1 + 1;
               end
               if step > 0.02 then
                     bad2 = bad2 + 1;
               end
            end

            print("-----------------");
            print("GC Cycle Info");
            print("---");
            print("Cycles          ",     self.gcCyclesCount);
            print("Steps           ",     #self.stepTimes);
            print("Bad Steps(>0.01)",     bad1 );
            print("Bad Steps(>0.02)",     bad2 );
            print("Avg Time        ",     sum/#self.stepTimes );
            print("Max Time        ",     max );

         end

         self.stepsSinceLastFullcycle = 0;
         self.timeOfLastFullCycle = MOAISim.getElapsedTime();
      else
         self.stepsSinceLastFullcycle = self.stepsSinceLastFullcycle + 1;
      end
   end

   return cycleFinished

end

function App:onGameStatusCheckFailed(event)

    local message = ((event or {}).message) or "Sorry, the game is under maintenance. Please try again in about 30 mins.";
    MOAIDialogIOS.showDialog(     "Maintenance Break"
                               ,     message
                               ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                               ,  nil
                               ,  nil
                               ,  false
                               ,  function()
                                     App.data:setValue("big-game-red-button-pressed",true);
                                     App.data:setValue("big-game-red-button-message",message);
                                     MOAIRoadRiot:exit();
                                  end
                            );

end

function App:onGameStatusCheckPassed(event)
     App.data:setValue("big-game-red-button-pressed",false);
     App.data:setValue("big-game-red-button-message",nil);
end

function App:onPlayerStatusCheckFailed(event)

    local message = ((event or {}).message) or "Sorry, the game is under maintenance. Please try again in about 30 mins.";
    MOAIDialogIOS.showDialog(     "Game Loading Failed"
                               ,     message
                               ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                               ,  nil
                               ,  nil
                               ,  false
                               ,  function()
                                     --App:gotoNewAppPage();
                                     App.data:setValue("big-player-red-button-pressed",true);
                                     App.data:setValue("big-player-red-button-message",message);
                                     MOAIRoadRiot:exit();
                                  end
                            );

end

function App:onPlayerStatusCheckPassed(event)
     App.data:setValue("big-player-red-button-pressed",false);
     App.data:setValue("big-player-red-button-message",nil);
end

function App:IsMigratedToJune()
    return App.data:getValue("june-api-migration-status" , false);
end

function App:OnMigratedToJune()
    App.data:setValue("june-api-migration-status", true);
end


function App:OnDownloadedFromTango()
    return App.data:setValue("june-api-download-from-tango" , true);
end

function App:HasDownloadedFromTango()
    App.data:getValue("june-api-download-from-tango", false);
end



function App:IsTangoIdLinkedToJuneProfile(acId)
    local accountId = acId;
    print("App:IsTangoIdLinkedToJuneProfile: AccountId: ", accountId or "nil");
    if accountId == nil then
        local profile = (ContactMgr:getProfile() or {});

        print("App:IsTangoIdLinkedToJuneProfile: profile: ", profile:getAccountId() or "nil");

        if profile:getAccountId() ~= nil and profile:getAccountId() ~= "__GUEST__MODE__" then
            accountId = profile:getAccountId();
            print("App:IsTangoIdLinkedToJuneProfile: AccountId 2: ", accountId or "nil");
        end
    end

    print("App:IsTangoIdLinkedToJuneProfile: AccountId 3: ", accountId or "nil");

    if accountId == nil then return false; end

    local value = App.data:getValue("june-tango-link-status-"..accountId,false);

    print("App:IsTangoIdLinkedToJuneProfile: value: ", tostring(value));

    return value;
end

function App:LinkTangoIdToJuneProfile(accountId)
    App.data:setValue("june-tango-link-status-"..accountId,true);
end

function App:getJunePlayerId()
    return App.data:getValue("june-api-player-id",nil);
end

function App:setJunePlayerId(pid)
    App.data:setValue("june-api-player-id",pid);
end

function App:DisableInApps(message)
    print("App:DisableInApps");
    App.data:setValue("june-in-apps-disabled",true);
    App.data:setValue("june-in-apps-disabled-message",message);
end

function App:InAppsDisabledMessage()
    return App.data:getValue("june-in-apps-disabled-message","In App purchases have been disabled in the game for the time being. Please try again later.");
end

function App:EnableInApps()
    print("App:EnableInApps");
    App.data:setValue("june-in-apps-disabled",false);
end

function App:InAppsAvailable()
    local value = not App.data:getValue("june-in-apps-disabled",false);
    print("App:InAppsAvailable: ", value);
    return value;
end

function App:SetCloudId(cloudId)
    print("App:SetCloudId: cloudPlayerId: ", tostring(cloudId), " localPlayerId: ", tostring(App:getJunePlayerId() or "nil"));
    App.data:setValue("june-cloud-player-id",cloudId);
    print("App:SetCloudId: cloudPlayerId After setting: ", tostring(App.data:getValue("june-cloud-player-id","ERROR_ID")));
end

function App:ReplaceLocalPlayerIdWithCloud()
    local cloudId = App.data:getValue("june-cloud-player-id","ERROR_ID");
    print("App:ReplaceLocalPlayerIdWithCloud: cloudPlayerId: ", tostring(cloudId), " localPlayerId: ", tostring(App:getJunePlayerId() or "nil"));
    if cloudId ~= "ERROR_ID" then
        self:setJunePlayerId(cloudId);
        App:SetCloudId(nil);
    end
end

