----------------------------------------------------------------------
require('engine/object/object')
require('engine/audio/audio-manager');
require('engine/event/event-manager');
require('app/template/audio/master_volume_table');


----------------------------------------------------------------------
--local CONSTANTS
local MAX_FOREGROUND = 4;

local HACK_DISABLE_SOUND_EFFECTS = false;

-- Modify above, not below.
if not BUILD_DEBUG then
   HACK_DISABLE_SOUND_EFFECTS = false;
end

----------------------------------------------------------------------

RoboDJ =
{
};

----------------------------------------------------------------------

local function roboPrint( ... )

   --Joe uncomment if you want to have your prints back (please check in commented)
   --print( ... );

end

----------------------------------------------------------------------

function RoboDJ:new( )

   local dj = Object.new( self );
   return dj;

end

----------------------------------------------------------------------

function RoboDJ:onStarted()

   self.audioList = {};
   self.loopList = {};
   self.musicList = {};

   if not HACK_DISABLE_SOUND_EFFECTS then
      EventMgr:addListener( self, self.onSFXAdded,          "add-sfx"     );
   end

   EventMgr:addListener( self, self.onMusicAdded,        "add-music"   );
   EventMgr:addListener( self, self.sfxStop,					"sfx-stop"   );
   EventMgr:addListener( self, self.loopStop,            "loop-stop"   );
   EventMgr:addListener( self, self.onGameStart,         "game-load");
   EventMgr:addListener( self, self.onPlayerDeath,       "avatar-death");
   EventMgr:addListener( self, self.onPlayerDeath,       "game-over");
   EventMgr:addListener( self, self.onPause,             "game-paused"  );
   EventMgr:addListener( self, self.onUnpause,           "game-unpaused");


end

----------------------------------------------------------------------

function RoboDJ:update( dt )

   --Update

end

----------------------------------------------------------------------

function RoboDJ:duckCheck( name )

   --Checks for ducking before adding sound.
   --returns the name of the AudioLayer that the sound will be added to (volume)

   return "vNormal";

end

----------------------------------------------------------------------

function RoboDJ:getFilename( id )

   for i,v in pairs(MASTER_VOLUME_TABLE) do
      if v.id == id then
         return v.filename;
      end
   end

end

----------------------------------------------------------------------

function RoboDJ:onSFXAdded( event, type )

   local sounds = event.sounds;

   -- garbage collection on the audio list
   if #self.audioList > 0 then
      for i,v in ipairs(self.audioList) do
         if v:isDone() then
            table.remove(self.audioList, i);
         end
      end
   end

   -- if the event doesn't return a valid table, exit the func
   if sounds == nil then
      return;
   end
   -- if the table has "start" sound(s) then pick one and play it
   if sounds.start then

      local n = math.random(1,#sounds.start);
      local id = sounds.start[n];
      local filename = self:getFilename(id);

      if event.useAlt then
         n = math.random(1,#sounds.alt);
         id = sounds.alt[n];
         filename = self:getFilename(id);
      end

      if filename then
         local vLayer = self:duckCheck( id );
         if vLayer then
            local layer = AudioMgr:getLayer( vLayer );
            local volume, mem = self:volumeCheck( id );
            local s = layer:addSound(filename, mem, false );
            s.id = id;
            table.insert(self.audioList, s);
            s:play(0, volume);
         end
      else
         Debug.assert( filename, "RoboDJ:onSFXAdded -> RoboDJ:getFilename -> oneshot filename" );
      end

   end
   --if the table has "loop" sound(s), pick one and play it
   if sounds.loop then

      local exLoop = self:loopCheck( sounds.loop );

      if event.useAlt then
         exLoop = self:loopCheck( sounds.alt );
      end

      if exLoop then

         local vLayer = self:duckCheck( exLoop.id )
         local layer = AudioMgr:getLayer( vLayer );
         local tarVolume, mem = self:volumeCheck( exLoop.id );
         local curVolume = exLoop:getMyVolume();
         if curVolume ~= tarVolume then
            exLoop:setMyVolume( tarVolume );
         end

      else

         local n = math.random(1,#sounds.loop);
         local id = sounds.loop[n];
         local filename = self:getFilename(id);

         if event.useAlt then
            n = math.random(1,#sounds.alt);
            id = sounds.alt[n];
            filename = self:getFilename(id);
         end

         if filename then
            local vLayer = self:duckCheck( id );
            if vLayer then

               local layer = AudioMgr:getLayer( vLayer );
               local volume, mem = self:volumeCheck( id );
               local l = layer:addSound( filename, mem, true );
               l.id = id;
               table.insert(self.audioList, l );
               table.insert(self.loopList, l );
               l:play(0, volume);
            end
         else
            Debug.assert( filename, "RoboDJ:onSFXAdded -> RoboDJ:getFilename -> loop filename" );
         end
      end

   end

end

----------------------------------------------------------------------

function RoboDJ:onMusicAdded( event, type )

   --check self.audiolist.music
   --cleanup music table
   --if it's the same music, do nothing
   --else fade out old music, fade in new


end

----------------------------------------------------------------------

function RoboDJ:sfxStop( event, type )

   local sounds = event.sounds;
   if sounds == nil then
      return;
   end

   local checkList = sounds.start

   if event.useAlt then
      checkList = sounds.alt
      if checkList == nil then
         return;
      end
   end


   if self.audioList and checkList then

      for i,v in ipairs(self.audioList) do

         for j,k in ipairs ( checkList ) do

            if v.id and v.id == k then
               v:stop();
               table.remove(self.audioList, i);
            end
         end
      end
   end
end

----------------------------------------------------------------------

function RoboDJ:loopStop( event, type )

   local sounds = event.sounds;
   if sounds == nil then
      return;
   end

   local checkList = sounds.loop

   if event.useAlt then
      checkList = sounds.alt
      if checkList == nil then
         return;
      end
   end


   if self.loopList and checkList then

      for i,v in ipairs(self.loopList) do

         for j,k in ipairs ( checkList ) do

            if v.id and v.id == k then
               v:stop();
               table.remove(self.loopList, i);

               for q,r in ipairs(self.audioList) do

                  if r.id and r.id == k then

                     table.remove(self.audioList, q );

                  end
               end
            end
         end
      end
   end
end

----------------------------------------------------------------------

function RoboDJ:loopCheck( sTable )

   if self.loopList then

      for _, v in ipairs(self.loopList) do

         for _, w in ipairs ( sTable ) do

            if v.id and v.id == w then
               return v;
            end

         end
      end
   end

end

----------------------------------------------------------------------

function RoboDJ:volumeCheck( id )

	local rVol, rMem = 1, true;

   for i,v in ipairs(MASTER_VOLUME_TABLE) do
      if v.id == id then
			if v.mem ~= nil then
				rMem = v.mem
			end
			if v.vol ~= nil then
				rVol = v.vol
			end
			break;
      end
   end

   return rVol, rMem;

end


----------------------------------------------------------------------

function RoboDJ:onGameStart()

   for i,v in pairs(MASTER_VOLUME_TABLE) do
      if v.preload == "game" then
         AudioMgr:cacheSound( v.filename, v.mem, false );
      end
   end

end

----------------------------------------------------------------------

function RoboDJ:onPlayerDeath( event, type )

   if self.loopList then
      for i,v in ipairs(self.loopList) do
         v:stop();
      end
   end

   self.loopList = {};
   self.audioList = {};

end

----------------------------------------------------------------------

function RoboDJ:onPause( event, type )

   if self.loopList then
      for i,v in ipairs(self.loopList) do
         if v:isPlaying() then
            v:pause();
         end
      end
   end

end

----------------------------------------------------------------------

function RoboDJ:onUnpause( event, type )

   if self.loopList then
      for i,v in ipairs(self.loopList) do
         if v:isPaused() then
            --get master table volume
			local volume = 0;
            if v.filename then
               volume = self:volumeCheck( v.id );
            end
            v:play(0, volume);
         end
      end
   end

end
