require('app/template/template')

----------------------------------------------------------------------

local BOUNTY_LIST_FILENAME_LUA  = "ROAD_RIOT_SAVE_BOUNTY_LIST_v2.lua";
local BOUNTY_LIST_FILENAME_JSON = "b81o32u033n22t55y2.jp5";

local STATE_ACTIVE   = 1;
local STATE_READY    = 2;
local STATE_INACTIVE = 3;

local BOUNTY_KILL_TIME = 2.0;

local PLAYER_VEHC_MIN_ARMOR = 3;

local WEEK = 60*60*24*7;

local HACK_BOUNTY_STATE = nil;

----------------------------------------------------------------------

BountyMgr  =
{
   _bountyList     = {};
   _bountyListSize = 0;
   _bountyReset    = false;
};

----------------------------------------------------------------------

function BountyMgr:init()

   --MQL: load happens before player list is properly updated, this now is explicitly called in sign-in
   --EventMgr:addListener( self, self._loadBountyList, "new-ranking"   );
   EventMgr:addListener( self, self._onPickup,       "avatar-pickup" );

end

----------------------------------------------------------------

function BountyMgr:loadBountyList()
   self:_loadBountyList();
end

function BountyMgr:_loadBountyList()

   if App.data:isBountyUnlocked()  then

      local saveFile = loadFileMultiFormat( BOUNTY_LIST_FILENAME_LUA, BOUNTY_LIST_FILENAME_JSON );

      if not saveFile then
         self._bountyReset = false;

         self:_pickBountySelection();
      else
         self._bountyList     = saveFile.list;
         self._bountyListSize = saveFile.size;
         self._bountyReset    = saveFile.reset;
      end

      if not self:isBountyListValid() then
         self:_pickBountySelection();
      end

   end

end

----------------------------------------------------------------

function BountyMgr:_saveBountyList()

   if not self._bountyList or not self._bountyListSize then
      return;
   end

   local data = { list=self._bountyList, size=self._bountyListSize, reset=self._bountyReset };
   saveFileMultiFormat( BOUNTY_LIST_FILENAME_LUA, BOUNTY_LIST_FILENAME_JSON, data, App:calcEncryptionKey() );

end

----------------------------------------------------------------------

function BountyMgr:_insertBountySelection( list, entry, score, isGhost, fName )

   for i=1,#list do
      if score < list[i].score then
         table.insert(list, i, {tangoId=entry.tangoId, score=score, isGhost=isGhost, name=fName});
         return;
      end
   end

   table.insert(list, {tangoId=entry.tangoId, score=score, isGhost=isGhost, name=fName});

end

----------------------------------------------------------------------

function BountyMgr:_forceSortGhost( list, ghostIndex )

   local gCount = 0;

   if LeagueMgr:isLeagueEnabled() then

      for i=#list, 1, -1 do
         if App.data:isLeagueUnlocked() and list[i].isGhost then
            table.remove(list, i);
         elseif not App.data:isLeagueUnlocked() and not list[i].isGhost then
            table.remove(list, i);
         end
      end
   else
      for i=#list, 1, -1 do
         if list[i].isGhost then
            gCount = gCount + 1;

            if gCount == ghostIndex then
               local t = list[i];

               table.remove(list, i);
               table.insert(list, 1, t);

               break;
            end
         end
      end
   end

   return list;

end

----------------------------------------------------------------------

function BountyMgr:_getBountyScoreRating( playerScore, targetScore, behindPenalty )

   local score = math.abs(targetScore - playerScore);

   if targetScore <= playerScore then
      score = score * behindPenalty;
   end

   return score;

end

----------------------------------------------------------------------

function BountyMgr:_pickBountySelection()

   local playerID       = ContactMgr:getProfile():getAccountId();
   local zeroScorePen   = AbTestMgr:getBountyZeroScorePenalty();
   local ghostPenK      = AbTestMgr:getBountyGhostPenaltyScale();
   local pursuitPenK    = AbTestMgr:getBountyPursuitPenaltyScale();
   local prevScorePenK  = AbTestMgr:getBountyPrevScorePenaltyScale();
   local cap            = AbTestMgr:getBountyCap();
   local rankList       = RankingMgr:getRankingList();

   if not rankList or not rankList[RankingMgr:getPlayerRank()] then
      return;
   end

   local pPScore        = rankList[RankingMgr:getPlayerRank()].prevScore;
   local pScore         = rankList[RankingMgr:getPlayerRank()].score;
   local hiScore        = math.max(App.data:getValue("alltime-high-score",0), App.data:getValue("level-high-score",0) );
   local pMaxScore      = math.max( math.max(pPScore or 0, pScore or 0), hiScore or 0);
   local count          = cap - App.data:getBountyRewardCount();
   local t              = 0;
   local size           = 0;

   if LeagueMgr:isLeagueEnabled() and App.data:isLeagueUnlocked() then
      local list, rank = RankingMgr:getRankingLeagueList();

      rankList = list;

      if not rankList or not rankList[rank] then
         return;
      end

      pPScore  = rankList[rank].prevScore;
      pScore   = rankList[rank].score;
   end

   self._bountyList     = {};
   self._bountyListSize = nil;

   local tbl = {};

   for i=#rankList,1,-1 do

      if rankList[i].tangoId ~= playerID then

         local scorePrev = self:_getBountyScoreRating(pMaxScore, rankList[i].prevScore, pursuitPenK)*prevScorePenK;
         local scoreCur  = self:_getBountyScoreRating(pMaxScore, rankList[i].score,     pursuitPenK);
         local score     = scorePrev + scoreCur;

         if rankList[i].contact:isGhost() then
            score = score*ghostPenK;
         else

            --Ghost should never have a zero score

            if rankList[i].prevScore == 0 then
               score = score + zeroScorePen/10; --We should have a different penalty variable here but we need ship
            end

            if rankList[i].score == 0 then
               score = score + zeroScorePen;
            end
         end

         -- print("BountyMgr:_pickBountySelection - 1", i, rankList[i].contact:getFirstName(), rankList[i].tangoId, score);
         -- print("   ", rankList[i].prevScore, scorePrev, rankList[i].score, scoreCur, rankList[i].contact:isGhost(), ghostPenK, rankList[i].score == 0, zeroScorePen);

         self:_insertBountySelection( tbl, rankList[i], score, rankList[i].contact:isGhost(), rankList[i].contact:getFirstName() );
      end
   end

   self:_forceSortGhost(tbl, math.random(1,2));

   cap = math.min(cap, #tbl);

   for i=1,cap do
      size = size + 1;

      -- print("BountyMgr:_pickBountySelection - 2", i, tbl[i].name, tbl[i].tangoId, tbl[i].score);

      if count < 1 then
         self._bountyList[tbl[i].tangoId] = STATE_INACTIVE;
      else
         count = count - 1;
         self._bountyList[tbl[i].tangoId] = STATE_ACTIVE;
      end
   end

   self._bountyListSize = size;

   if HACK_BOUNTY_STATE then
      for k,v in pairs(self._bountyList) do
         self._bountyList[k] = HACK_BOUNTY_STATE;
      end
   end

   self:_saveBountyList();

end

----------------------------------------------------------------------

function BountyMgr:refreshBountyList()

   if App.data:isBountyUnlocked() then
      self:_pickBountySelection();
   end

end

----------------------------------------------------------------------

function BountyMgr:_resetBountyList()

   App.data:_resetBountyRewardCount();

   self:_pickBountySelection();

end

----------------------------------------------------------------------

function BountyMgr:resetBounty()

   self:_onWeeklyReset();

end

----------------------------------------------------------------------

function BountyMgr:_onWeeklyReset()

   if TemplateMgr:isBountyUnlocked( App.data:getCurrentPlayerLevel() ) then

      Debug.assert( RankingMgr:isReady() );

      App.data:setValue("bounty-weekly-intro-seen", nil);

      self._bountyReset = true;

      self:autoCollectBounty();
      self:_resetBountyList();
   end

end

----------------------------------------------------------------------

function BountyMgr:_onPickup( event )

   if event.pickup.pickupType == "bounty" then

      if event.pickup.tangoId then
         self:setBountyEntryStateReady(event.pickup.tangoId);
         EventMgr:postEvent({id=event.pickup.tangoId}, "bounty-pickup");

         if LeagueMgr:isLeagueEnabled() then
            App.data:addBountyReward(event.pickup.tangoId);

            if event.pickup.tangoId == "__GHOST_01" then
               App.data:setValue("guide-bounty-collected", event.pickup.tangoId);
            elseif event.pickup.tangoId == "__GHOST_02" and App.data:getValue("guide-bounty-collected") ~= "__GHOST_01" then
               App.data:setValue("guide-bounty-collected", event.pickup.tangoId);
            elseif         event.pickup.tangoId == "__GHOST_03"
                     and   (App.data:getValue("guide-bounty-collected") ~= "__GHOST_01"
                     and    App.data:getValue("guide-bounty-collected") ~= "__GHOST_02") then

               App.data:setValue("guide-bounty-collected", event.pickup.tangoId);
            else
               App.data:setValue("guide-bounty-collected", event.pickup.tangoId);
            end
         end
      end
   end

end

----------------------------------------------------------------------

function BountyMgr:_setBountyEntryCollectedState( id, state )

   if self:isBountyEntryValid(id) then
      self._bountyList[id] = state;

      self:_saveBountyList();
   end

end

----------------------------------------------------------------------

function BountyMgr:autoCollectBounty( )

   if App.data:isBountyUnlocked() then

      for k,v in pairs(self._bountyList) do

         if self:isBountyEntryCollectReady(k) then
            App.data:addBountyReward(k);
         end
      end
   end

end

----------------------------------------------------------------------

function BountyMgr:getBountyNotCollectedCount( )

   if App.data:isBountyUnlocked() then

      local count = 0;

      for k,v in pairs(self._bountyList) do
         if self:isBountyEntryNotCollected(k) then
            count = count + 1;
         end
      end

      return count;
   end

end

----------------------------------------------------------------------

function BountyMgr:getBountyReadyToCollectCount( )

   if App.data:isBountyUnlocked() then

      local count = 0;

      for k,v in pairs(self._bountyList) do
         if self:isBountyEntryCollectReady(k) then
            count = count + 1;
         end
      end

      return count;
   end

end

----------------------------------------------------------------------

function BountyMgr:getBountySelection()

   if App.data:isBountyUnlocked() then

      if not self:isBountyListValid() then
         self:_loadBountyList();
      end

      return self._bountyList;

   end

end

----------------------------------------------------------------------

function BountyMgr:getBountySelectionSize()

   return self._bountyListSize;

end

----------------------------------------------------------------------

function BountyMgr:getBountyEntryCollectedState( id )

   if self:isBountyEntryValid(id) then
      return self._bountyList[id];
   end

end

----------------------------------------------------------------------

function BountyMgr:getBountyVehicleHeath()

   local dps = App.data:calcCurrentPlayerDPS(1.00, 0.30, 1.00);

   return math.floor(BOUNTY_KILL_TIME * dps);

end

----------------------------------------------------------------------

function BountyMgr:getBountyVehicleTemplate( entry )

   if entry and entry.contact then

      if entry.contact:isGhost() then
         local data = GhostMgr:getGhostVehicleInfo(entry.tangoId);
         local vi = math.clamp(data.vehicleIndex, 1, App.data:getPlayerVehicleCount());

         return
            {
               template = App.data:getBountyVehicleTemplate(vi),
               paint    = App.data:getBountyPaintTemplate(vi, data.vehiclePaint),
               armor    = App.data:getBountyArmorTemplate(vi, data.vehicleArmor),
            };
      elseif entry.contact:isGuestFriend() then
         local data = GuestMgr:getGuestVehicleInfo(entry.tangoId);
         local vi   = math.clamp(data.vehicleIndex, 1, App.data:getPlayerVehicleCount());

         return
            {
               template = App.data:getBountyVehicleTemplate(vi),
               paint    = App.data:getBountyPaintTemplate(vi, data.vehiclePaint),
               armor    = App.data:getBountyArmorTemplate(vi, data.vehicleArmor),
            };
      else
         local prof = ContactMgr:findContact(entry.tangoId);

         if prof then
            local vi = math.clamp(prof:getCarIndex() or 1, 1, App.data:getPlayerVehicleCount());
            local va = math.clamp(prof:getArmorUpgrade() or PLAYER_VEHC_MIN_ARMOR, PLAYER_VEHC_MIN_ARMOR, App.data:getPlayerArmorMaxLevel(vi));

            return
               {
                  template = App.data:getBountyVehicleTemplate(vi),
                  paint    = App.data:getBountyPaintTemplate(vi, prof:getPaintIndex() or 1),
                  armor    = App.data:getBountyArmorTemplate(vi, va),
               };
         else
            return
               {
                  template = App.data:getBountyVehicleTemplate(1),
                  paint    = App.data:getBountyPaintTemplate(1, 1),
                  armor    = App.data:getBountyArmorTemplate(1, PLAYER_VEHC_MIN_ARMOR),
               };
         end
      end
   end

end

----------------------------------------------------------------------

function BountyMgr:isBountyEntryValid( id )

   if self._bountyList and self._bountyList[id] ~= nil then
      return true;
   end

end

----------------------------------------------------------------------

function BountyMgr:isBountyEntryActive( id )

   if self:isBountyEntryValid(id) and self._bountyList[id] == STATE_ACTIVE then
      return true;
   end

end

----------------------------------------------------------------------

function BountyMgr:isBountyEntryNotCollected( id )

   if self:isBountyEntryValid(id) and self._bountyList[id] < STATE_INACTIVE then
      return true;
   end

end

----------------------------------------------------------------------

function BountyMgr:isBountyEntryCollected( id )

   if self:isBountyEntryValid(id) and self._bountyList[id] == STATE_INACTIVE then
      return true;
   end

end

----------------------------------------------------------------------

function BountyMgr:isBountyEntryCollectReady( id )

   if self:isBountyEntryValid(id) and self._bountyList[id] == STATE_READY then
      return true;
   end

end

----------------------------------------------------------------------

function BountyMgr:isBountyListValid()

   if not App.data:isBountyUnlocked() then
      return false;
   end

   local count       = math.min(RankingMgr:getRankEntryCount()-1, AbTestMgr:getBountyCap());
   local maxCollect  = AbTestMgr:getBountyCap() - App.data:getBountyRewardCount();
   local tbl         = RankingMgr:getRankingList();
   local n           = 0;

   if LeagueMgr:isLeagueEnabled() and App.data:isLeagueUnlocked() then
      count    = math.min(#RankingMgr:getRankingLeagueList()-1, AbTestMgr:getBountyCap());
   end

   for k,v in ipairs(tbl) do
      if self:isBountyEntryValid(v.tangoId) then
         n = n + 1;
      end
   end

   if       n == self:getBountySelectionSize()
      and   self:getBountySelectionSize() <= count
      and   n > 0  then

      if       LeagueMgr:isLeagueEnabled()
         and   App.data:isLeagueUnlocked()
         and   self:getBountySelectionSize() ~= count then

         return false;
      end

      return true;
   end

   return false;

end

----------------------------------------------------------------------

function BountyMgr:isBountyResetTime()

   local ts = RankingMgr:getWeeklyResetTime();
   local time = TimeMgr:getTimeGMT() or TimeMgr:getTimeLocalGMT();
   local delta = App.data:getValue("App.settings.bounty-reset-offset") or 0;

   if delta == 0 then
      return false;
   end

   if ts then
      if HACK_QUICK_WEEKLY_RESET and HACK_QUICK_WEEKLY_RESET_TIME then
         delta = delta * (HACK_QUICK_WEEKLY_RESET_TIME/WEEK);
         ts = ts - HACK_QUICK_WEEKLY_RESET_TIME + delta;
      else
         ts = ts - WEEK + delta;
      end
   end

   if time and ts and not self._bountyReset then

      if time >= ts then
         return true;
      end
   end

end

----------------------------------------------------------------------

function BountyMgr:resetBountyTimeStamp()

   self._bountyReset = false;

end

----------------------------------------------------------------------

function BountyMgr:anyBountyCollected()

   if self:getBountyNotCollectedCount() ~= self:getBountySelectionSize() then
      return true;
   end

   return false;

end


----------------------------------------------------------------------

function BountyMgr:setBountyEntryStateCollected( id )

   self:_setBountyEntryCollectedState(id, STATE_INACTIVE);

end

----------------------------------------------------------------------

function BountyMgr:setBountyEntryStateReady( id )

   self:_setBountyEntryCollectedState(id, STATE_READY);

end

----------------------------------------------------------------------

function BountyMgr:setBountyPickup( tpl, tangoId, pic )

   if tpl.type == "bounty" and tangoId and pic then

      local t = Template.clone(tpl);

      t.element1.texture = pic;
      t.tangoId = tangoId;

      return t;
   end

   return tpl;

end

----------------------------------------------------------------------
