----------------------------------------------------------------

require('engine/utility/delay');

----------------------------------------------------------------

--URL is to download, Filename is the save version

local INFO_SAVE_FILENAME  = "CLOUD_MGR_INFO_FILENAME_2.lua";
local MASTER_LOCAL_FILENAME  = "cloud/_master.content";
--We need to split android and ios because they are becoming "live" at different time
local MASTER_URL_IOS          = "http://roadriot-cdn.tango.me/production/_master/master.iOS";
local MASTER_URL_ANDROID      = "http://roadriot-cdn.tango.me/production/_master/master.android";
local MASTER_URL_AMAZON       = "http://roadriot-cdn.tango.me/production/_master/master.amazon";
local MASTER_SAVE_FILENAME  = "CLOUD_MGR_MASTER_FILENAME.lua";

----------------------------------------------------------------

local FORCE_BUILTIN           = false;
local USE_LOCAL_MASTER        = true;
local FORCE_OUT_OF_DATE       = false;

--Do not touch these value, it is to make sure we are forgeting debug stuff for release
if not BUILD_DEBUG then
   USE_LOCAL_MASTER           = nil;
   FORCE_BUILTIN              = nil;
   FORCE_OUT_OF_DATE          = nil;
end

--Wait at least 1 hour before redownloading everything (when the app is foregrounded)
local MIN_TIME_MASTER_UPDATE  = 60*60*1;

----------------------------------------------------------------

CloudMgr =
{
   _active                       = false,
   _activeCount                  = 0,
   _ready                        = false,
   _state                        = nil,
   _info                         = nil,
   _masterTbl                    = nil,
   _masterUpToDate               = false,
   _fileListToUpdate             = {},
   _fileUpdating                 = nil,
};

----------------------------------------------------------------

function CloudMgr:activate()

   if not self._active then
      self._activeCount = self._activeCount + 1;
      self._active      = true;
      self:_reset();

      --Mark all file to be out of date until we recheck them
	  --Set before calling the cloud:data- message
      for fileId,file in pairs(self._info.file) do
         file.upToDate = false;
      end

      --Some system depend on us to give them their data at the begining
      --Dont wait the update... we will send them the latest later
      self:_postMasterDataEvents( false );

      for fileId,file in pairs(self._info.file) do
         if file.filename then
            local content = loadLocalFile(file.filename);
            if content then
               self:_postFileDataEvent( fileId, content, false, false, file.version );
            else
               Debug.assert(content, "CloudMgr:activate -> content");
            end
         else
            Debug.assert(file.filename, "CloudMgr:activate -> file.filename: Id = "..tostring(fileId));
         end
      end

      --Go on the internet to download the latest
      self._downloadMasterTimer = 0;

   end

   self._ready = true;

end

----------------------------------------------------------------

function CloudMgr:deactivate()

   if  self._active then
      self._active = false;
      self._ready  = false;
   end

end

----------------------------------------------------------------

function CloudMgr:isActive()
   return self._active == true;
end

----------------------------------------------------------------

function CloudMgr:isReady()
   return  self._ready == true;
end

----------------------------------------------------------------

function CloudMgr:isUpdating()

   if self._ready then
      return (self._fileUpdating ~= nil) or (#self._fileListToUpdate > 0) or (self._masterUpdating == true);
   end

   return false;

end

----------------------------------------------------------------

function CloudMgr:isMasterUpToDate(  )
   return self._masterUpToDate;
end

----------------------------------------------------------------

function CloudMgr:getFileVersion( id )

   if id and self._info.file[id] then
      return self._info.file[id].version;
   end

end

----------------------------------------------------------------

function CloudMgr:isFileUpToDate( id )

   if self._info.file[id] then
      return self._info.file[id].upToDate;
   end

end

----------------------------------------------------------------

function CloudMgr:getFileFilename( id )

   if self._info.file[id] then
      return self._info.file[id].filename;
   end

end


----------------------------------------------------------------

function CloudMgr:onEnterForeground()

   if not self:isUpdating() then

      self._downloadMasterTimer = 0;

   end
end

----------------------------------------------------------------

function CloudMgr:update( dt )

   --Not a great fix but there is a bug that make self._fileUpdating nil
   self._fileListToUpdate = self._fileListToUpdate or {};

   if       self._active
      and   self._ready    then

      if OnlineMgr:isOnline() then

         if not self:isUpdating() then
            if self._downloadMasterTimer then

               self._downloadMasterTimer = self._downloadMasterTimer - dt;

               if self._downloadMasterTimer < 0 then
                  self._downloadMasterTimer = nil;
                  self:_updateMaster();
               end

            end
         end

         --Check again in case we started updating master
         if       (not self._masterUpdating)
            and   (self._fileUpdating == nil)
            and   (#self._fileListToUpdate > 0)          then

            local entry = self._fileListToUpdate[1];
            table.remove(self._fileListToUpdate,1);

            self._fileUpdatingAttempt  = nil;
            self:_updateFile( entry.key,  entry.value );
         end
      end
   end

   if self._saveInfoFlag then
      self:_saveInfo();
   end

end

----------------------------------------------------------------

function CloudMgr:_reset()

   self._ready                   = false;
   self._masterUpToDate          = false;

   self._downloadMasterTimer     = 0;

   local curVersion              = App:versionStringToNumber( App.info.version );

   self._masterTbl               = loadLocalFile( MASTER_SAVE_FILENAME );

   self._info                    = loadLocalFile( INFO_SAVE_FILENAME ) or {};
   self._info.file               = self._info.file                     or {};
   self._info.appVersion         = tonumber(self._info.appVersion)     or 0;


   --This is the first time we run the app since updating from the store
   --Update local files (to avoid unecessary download)
   if (curVersion > self._info.appVersion) or ( FORCE_BUILTIN ) then
      self:_initBuiltin();
   end


end

----------------------------------------------------------------

function CloudMgr:_initBuiltin()

   self._masterTbl = dofile(MASTER_LOCAL_FILENAME);

   Debug.assert( self._masterTbl and self._masterTbl.file and type(self._masterTbl.file) == "table" );

   if self._masterTbl and self._masterTbl.file and type(self._masterTbl.file) == "table" then

      saveLocalFile( MASTER_SAVE_FILENAME, self._masterTbl );

      for key,value in pairs(self._masterTbl.file) do

         Debug.assert( key and value and type(value) == "table" );

         if key and value and type(value) == "table" then

            local id       = key;
            local url      = value.url;
            local version  = value.version or -1;
            local filename = self:_createFilename( key );

            Debug.assert( url and self:_isUrlLocal( url ) );

            --Dont do anything if not local
            if url and self:_isUrlLocal( url ) then

               local content = dofile( url );

               Debug.assert( content, "_initBuiltin no content" .. url );

               if content then

                  local fileName = self:_createFilename(key);
                  Debug.assert(fileName);

                  saveLocalFile( fileName, content );

                  self._info.file[id]           = self._info.file[id] or {};
                  self._info.file[id].version   = version or -1;
                  self._info.file[id].upToDate  = false;
                  self._info.file[id].filename  = filename;

               end
            end
         end
      end
   end

   self._info.appVersion         =  App:versionStringToNumber( App.info.version );

   --Force re-downloading master file
   self._info.lastMasterDownload = 0;

   self._saveInfoFlag            = true;

end

----------------------------------------------------------------

function CloudMgr:_createFilename(key)
   return "CLOUD_MGR_FILE_"..key;
end

----------------------------------------------------------------

function CloudMgr:_createMasterUrl( forceUrl )

   --forceUrl has priority over USE_LOCAL_MASTER, in case the USE_LOCAL_MASTER user redirect
   if forceUrl  then
      return forceUrl;
   end

   if USE_LOCAL_MASTER then
      return MASTER_LOCAL_FILENAME;
   end

   if AmazonBillingProvider == true then
      return ( MASTER_URL_AMAZON .."."..App.info.version );
   elseif MOAIEnvironment.osBrand == "Android" then
      return ( MASTER_URL_ANDROID .."."..App.info.version );
   else
      return ( MASTER_URL_IOS .."."..App.info.version );
   end

end

----------------------------------------------------------------

function CloudMgr:_isUrlLocal( url )

   if url then
      return string.find( url, "http" ) == nil;
   end

end

----------------------------------------------------------------

function CloudMgr:_saveInfo()

   if saveLocalFile( INFO_SAVE_FILENAME, self._info ) then
      self._saveInfoFlag = false;
   end

end

----------------------------------------------------------------

function CloudMgr:_onFileUpdated( success )

   Debug.assert( self._fileUpdating );

   if self._fileUpdating then

      local id = self._fileUpdating.id;

      Debug.assert( id  );

      if id then

         self._info.file[id]  = self._info.file[id] or {};

         local filename       = self._fileUpdating.filename;
         local content        = loadLocalFile(filename);
         local version        = self._fileUpdating.version or 0;
         local upToDate       = success and self._masterLastUpdateSucceeded;
         local new            = upToDate and (version > (self:getFileVersion(id) or 0));

         self._info.file[id].filename = filename;
         self._info.file[id].upToDate = upToDate;

         --Only save the version if we are a success... otherwise we will not try to download again
         if success then
            self._info.file[id].version = version or -1;
         else
            --we failed = we are using the old version, not the one we try to update
            version  =  self:getFileVersion(id);
         end

         self._saveInfoFlag            = true;

         self:_postFileDataEvent( id, content, upToDate, new, version );

      end
   end

   self._fileUpdating    = nil;

   if  (#self._fileListToUpdate == 0) then
      EventMgr:postEvent( {},       "cloud:download-done"  );
   end
end

----------------------------------------------------------------

function CloudMgr:_onFileUpdateSuccess()

   self:_onFileUpdated(true);

end

----------------------------------------------------------------

function CloudMgr:_onFileUpdateFailure()

   if self._fileUpdating then

      self._fileUpdatingAttempt = (self._fileUpdatingAttempt or 0 ) + 1;

      if self._fileUpdatingAttempt <= 1 then
         self:_updateFile( self._fileUpdating.key,  self._fileUpdating.value );
      else
         self:_onFileUpdated(false);
      end

   end
end

----------------------------------------------------------------

function CloudMgr:_updateFile( key, value )

   Debug.assert( key, value );

   self._fileUpdating = nil;

   if key and value then

      local id          = key;
      local url         = value.url;
      local version     = value.version;
      --activeCount is use to detect when we were deactivate while waiting for these
      local activeCount = self._activeCount;

      Debug.assert( url and version );

      if url and version then

         self._fileUpdating= {   id       = id,
                                 url      = url,
                                 version  = version,
                                 filename = self:_createFilename(key),
                                 key      = key,
                                 value    = value                          };

         delay( 0.01,
                function()

                     if self:isActive() and activeCount == self._activeCount then
                        --Make sure the version we want to download is newer
                        if (FORCE_OUT_OF_DATE) or (version < 0) or (version >  (self:getFileVersion(id) or -1 )) then
                           if not self._masterLastUpdateSucceeded then
                              if self:isActive() and activeCount == self._activeCount then
                                 self:_onFileUpdateFailure();
                              end
                           else
                              if self:_isUrlLocal( url ) then
                                 dofileToFile ( url,
                                                self._fileUpdating.filename,
                                                self,
                                                function()
                                                   if self:isActive() and activeCount == self._activeCount then
                                                      self:_onFileUpdateSuccess();
                                                   end
                                                end,
                                                function()
                                                   if self:isActive() and activeCount == self._activeCount then
                                                      self:_onFileUpdateFailure();
                                                   end
                                                end
                                                );
                              else
                                 downloadToFile(   url,
                                                   self._fileUpdating.filename,
                                                   self,
                                                   function()
                                                      if self:isActive() and activeCount == self._activeCount then
                                                         self:_onFileUpdateSuccess();
                                                      end
                                                   end,
                                                   function()
                                                      if self:isActive() and activeCount == self._activeCount then
                                                         self:_onFileUpdateFailure();
                                                      end
                                                   end
                                                );
                              end
                           end
                        else
                           if self:isActive() and activeCount == self._activeCount then
                              self:_onFileUpdateSuccess();
                           end
                        end
                     end
                  end
               );

         return true;

      end
   end
end


----------------------------------------------------------------

function CloudMgr:_postFileDataEvent( id, content, upToDate, new, version )

   --Send at the end
   EventMgr:postEvent( { header = { new            = new,
                                    upToDate       = upToDate,
                                    version        = version,     },
                        content  =  content },
                        "cloud:file-data-"..id  );

end

----------------------------------------------------------------

function CloudMgr:_postMasterDataEvents(upToDate)

   if self._masterTbl and self._masterTbl.data and type(self._masterTbl.data) == "table" then
      for k,content in pairs(self._masterTbl.data) do
         if type(content) == "table" then
            EventMgr:postEvent( { header  = { upToDate = upToDate },
                                  content = content },
                                 "cloud:master-data-"..k                                                 );
         end
      end
   end

end

----------------------------------------------------------------

function CloudMgr:_onMasterUpdated( downloadSuccess )

   print("CloudMgr:_onMasterUpdated", downloadSuccess );

   self._masterTbl = loadLocalFile( MASTER_SAVE_FILENAME );

   --It is possible that we failed and dont have a master tbl
   if self._masterTbl and type(self._masterTbl) == "table" then

      local redirectUrl = self._masterTbl.redirectUrl;

      if redirectUrl then

         if downloadSuccess then

            self:_updateMaster( redirectUrl );

         else
            --If we failed with a redirect = we are stucked
            --Invalidate the master (to revert to builtin)
            self._masterTbl= nil;
         end

      else

         self:_postMasterDataEvents( downloadSuccess );

         if self._masterTbl.file and type(self._masterTbl.file) == "table" then
            for k,v in pairs(self._masterTbl.file) do
               if type(v) == "table" then
                  table.insert( self._fileListToUpdate, { key=k, value=v } );
               end
            end
         end

         if downloadSuccess then
            self._info.lastMasterDownload    = os.time();
         end

         self._masterLastUpdateSucceeded  = downloadSuccess;
         self._masterLastUpdateSuccessTime= os.time();
         self._masterUpToDate             = downloadSuccess;

      end

   else

      self._masterLastUpdateSucceeded   = false;

   end

   self._saveInfoFlag = true;

   MOAIFlurry:event(  "Cloud:Update-Master-Done",
                      "success",                    tostring(downloadSuccess),
                      "type(self._masterTbl)",      type(self._masterTbl)      );

   if (self._masterTbl == nil) or (type(self._masterTbl) ~= "table") then

      --If we have an empty masterTbl == it is bad news
      --revert to the builtin master
      dofileToFile ( MASTER_LOCAL_FILENAME,
                     MASTER_SAVE_FILENAME,
                     self,
                     function()
                        self:_onMasterUpdateSuccess();
                     end,
                     function()
                        MOAIDialogIOS.showDialog(  LocalizationMgr:getString( "NATIVE_FILE_ERROR_TITLE" )
                                                ,  LocalizationMgr:getString( "NATIVE_FILE_ERROR_MSG" )
                                                ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                                                ,  nil
                                                ,  nil
                                                ,  false
                                                ,  function()
                                                      self:_onMasterUpdateFailure();
                                                   end
                                                );
                     end
                     );
   else

      EventMgr:postEvent( { success = self._masterLastUpdateSucceeded }, "cloud:master-updated"  );

   end
end

----------------------------------------------------------------

function CloudMgr:_onMasterUpdateSuccess()

   self:_onMasterUpdated( true );

end

----------------------------------------------------------------

function CloudMgr:_onMasterUpdateFailure( forceUrl )

   self:_onMasterUpdated( false );

end

----------------------------------------------------------------

function CloudMgr:_updateMaster( forceUrl )

   Debug.assert( not self._masterUpdating );

   local url = self:_createMasterUrl(forceUrl);

   print("CloudMgr:_updateMaster", url, forceUrl );

   --activeCount is use to detect when we were deactivate while waiting for these
   local activeCount    = self._activeCount;
   self._masterUpdating = true;

   --Delay to be more consistent and avoid potential bug (i.e. we setup data right after this function call)
   delay( 0.01,
          function()

             local timeStamp = MOAISim.getDeviceTime();
             MOAIFlurry:eventStart("Cloud:Update-Master", "Result", "Crash");

             if self:_isUrlLocal( url ) then
                dofileToFile ( url,
                               MASTER_SAVE_FILENAME,
                               self,
                               function()
                                  self._masterUpdating = false;
                                  if self:isActive() and activeCount == self._activeCount then
                                     self:_onMasterUpdateSuccess();
                                     MOAIFlurry:eventStop("Cloud:Update-Master", "Result", "Success-Local", "Time", tostring(math.floor(MOAISim.getDeviceTime()-timeStamp)) );
                                  end
                               end,
                               function()
                                  self._masterUpdating = false;
                                  if self:isActive() and activeCount == self._activeCount then
                                     self:_onMasterUpdateFailure(forceUrl);
                                     MOAIFlurry:eventStop("Cloud:Update-Master", "Result", "Failure-Local", "Time", tostring(math.floor(MOAISim.getDeviceTime()-timeStamp)) );
                                  end
                               end
                );
             else
                downloadToFile(   url,
                                  MASTER_SAVE_FILENAME,
                                  nil,
                                  function()
                                     self._masterUpdating = false;
                                     if self:isActive() and activeCount == self._activeCount then
                                        self:_onMasterUpdateSuccess();
                                        MOAIFlurry:eventStop("Cloud:Update-Master", "Result", "Success", "Time", tostring(math.floor(MOAISim.getDeviceTime()-timeStamp)) );
                                     else
                                        MOAIFlurry:eventStop("Cloud:Update-Master", "Result", "Success-Bad-Count", "Time", tostring(math.floor(MOAISim.getDeviceTime()-timeStamp)));
                                     end
                                  end,
                                  function()
                                     self._masterUpdating = false;
                                     if self:isActive() and activeCount == self._activeCount then
                                        self:_onMasterUpdateFailure(forceUrl);
                                        MOAIFlurry:eventStop("Cloud:Update-Master", "Result", "Failure", "Time", tostring(math.floor(MOAISim.getDeviceTime()-timeStamp)) );
                                     else
                                        MOAIFlurry:eventStop("Cloud:Update-Master", "Result", "Failure-Bad-Count", "Time", tostring(math.floor(MOAISim.getDeviceTime()-timeStamp)));
                                     end
                                  end,
                                  Request.PRIORITY_HIGH
                );
             end
          end
   );

end

----------------------------------------------------------------
