----------------------------------------------------------------

require('engine/utility/delay');
require('engine/json/json');
require("engine/utility/stringUtils");

local json = require('json');

----------------------------------------------------------------

JuneApi =
{
   _active                       = false,
   _activeCount                  = 0,
   _ready                        = false,
   _state                        = nil,
   _fileListToUpdate             = {},
   _fileUpdating                 = nil,
   _playerId                     = nil,
   hasTriedTangoId               = false,
};

----------------------------------------------------------------

local JuneSocialLinkTypes =
{
    tango                       = "t",
    facebook                    = "f",
    gameCenter                  = "c",
    googlePlay                  = "g",
}

local JuneApiURLs =
{
    prod                        = "http://prod.api.roadriot.mgaming.net/api",
    staging                     = "http://roadriot-api-staging.us-west-2.elasticbeanstalk.com/api",
    playerRegister              = "/player/register",
    playerGet                   = "/player/get/via/nickname",
    playerGetSocial             = "/player/get/via/profile",
    playerUpdate                = "/player/update",
    playerUpdatePossessions     = "/player/update/possessions",
    playerUpdateDevice          = "/player/update/device",
    playerStatusCheck           = "/player/check",
    playerLinkSocial            = "/player/link/profile",
    playerUnlinkSocial          = "/player/unlink/profile",

    giftCheck                   = "/gift/check",
    giftClaim                   = "/gift/claim",

    purchaseVerify              = "/purchase/verify",

    statusPing                  = "/status/ping",
    gameStatusCheck             = "/status/check",

};

local ApiURL = JuneApiURLs.prod;


----------------------------------------------------------------

local JuneApiKeys =
{
    deviceIdKey                 = "di",
    deviceTypeKey               = "dt",
    statusCodeKey               = "sc",
    playerIdKey                 = "nk",
    valueKey                    = "vl",
    itemIdKey                   = "it",
    purchaseProviderKey         = "pr",
    purchaseReceiptKey          = "rc",
    socialProfileId             = "tk",
    socialType                  = "tp",
    forced                      = "f",
    cloudPlayerId               = "_id",
    messageKey                  = "mg",
    localeKey                   = "lc",
    currentVersion              = "cv",
    lastUpdatedKey              = "ut",
    giftIsClaimableKey          = "ic",
    giftAutoClaimKey            = "ac",
    giftTypeKey                 =  "t",
    giftValueKey                =  "v",
};

---------------------------------------------------------------

function JuneApi:init()
end

---------------------------------------------------------------

function JuneApi:activate()

   if not self._active then
      self._activeCount = self._activeCount + 1;
      self._active      = true;
   end

   local playerId = App:getJunePlayerId();
   if playerId == nil then
      self:RegisterPlayer();
   else
    if App.data:getValue("june-should-update-device",false) then
        self:UpdateDevice();
    end
   end

   self._ready = true;

end

----------------------------------------------------------------

function JuneApi:deactivate()

   if  self._active then
      self._active = false;
      self._ready  = false;
   end

end

----------------------------------------------------------------

function JuneApi:isActive()
   return self._active == true;
end

----------------------------------------------------------------

function JuneApi:isReady()
   return  self._ready == true;
end

----------------------------------------------------------------

function JuneApi:playerId()
   return  App:getJunePlayerId();
end

----------------------------------------------------------------

function JuneApi:RegisterPlayer()

    local wrapperCallObj = {}
    print("JuneApi:RegisterPlayer");
    function wrapperCallObj:callSuccess(response)
        print("web request RegisterPlayer success  data: " .. tostring(response:getString() or "nil"));
        local resData = json.decode(response:getString());
        local statusCode = resData[JuneApiKeys.statusCodeKey];
        if statusCode == 201 then
            local pid = resData[JuneApiKeys.playerIdKey];
            App:setJunePlayerId(pid);
            JuneApi:CheckPlayerStatus();
            TransactionMgr:uploadEverythingToJune( );
        end
   end

    function wrapperCallObj:callFailure(response)
    
        print("web request failure data: " .. tostring(response:getString() or "nil"));
        local resData = json.decode(response:getString());
        if resData then
        local statusCode = resData[JuneApiKeys.statusCodeKey];
            if statusCode and statusCode == 201 then
                local pid = resData[JuneApiKeys.playerIdKey];
                if pid then
                    App:setJunePlayerId(pid);
                    TransactionMgr:uploadEverythingToJune( );
                end
            end
        end

   end


    local devType = MOAIEnvironment.osBrand:lower();
    local udid = MOAIEnvironment.udid;
    if devType == "ios" then
        udid = MOAIEnvironment.openUdid;
    end

    local locale = MOAIEnvironment.languageCode or "en";

    local requestOptions = {
        priority = Request.PRIORITY_NORMAL,
        callHandler = wrapperCallObj,
        successFunc = wrapperCallObj.callSuccess,
        failureFunc = wrapperCallObj.callFailure,
        data = string.format("{\"%s\":\"%s\",\"%s\":\"%s\",\"%s\":\"%s\",\"%s\":\"%s\"}",JuneApiKeys.deviceIdKey,udid,JuneApiKeys.deviceTypeKey,devType,JuneApiKeys.localeKey,locale,JuneApiKeys.currentVersion,tostring(App.info.version)  );
        url = ApiURL..JuneApiURLs.playerRegister,
        retryCount = 0,
    }

--    requestOptions.data [ JuneApiKeys.deviceIdKey ] = MOAIEnvironment.openUdid;
--    requestOptions.data [ JuneApiKeys.deviceTypeKey ] = MOAIEnvironment.osBrand:lower();

   local request = PushWebRequest.newFromBase(requestOptions);
   NetworkMgr:sendPushWebRequest(request);

end

----------------------------------------------------------------
function JuneApi:CheckGameStatus()
    print("JuneApi:CheckGameStatus");

    if not OnlineMgr:isOnline() then
        local redButtonPressed = App.data:getValue("big-game-red-button-pressed",false);
        if redButtonPressed then
            local message = App.data:getValue("big-game-red-button-message");
            EventMgr:postEvent( {message=message }, "game-status-check-failed" );
        end
        return;
    end

    local wrapperCallObj = {}
    function wrapperCallObj:callSuccess(response)
            print("web request GameStatus success data  : " .. response:getString());
            local resData = json.decode(response:getString());
            local statusCode = resData[JuneApiKeys.statusCodeKey];
            local versionString = resData[JuneApiKeys.currentVersion] ;

            if versionString then
                local version = App:versionStringToNumber( versionString);
                UpdateMgr:SetMandatoryVersionNumber(version);
            end
            if statusCode == 200 then
                EventMgr:postEvent( { }, "game-status-check-passed" );
                App:EnableInApps();
            end
            if statusCode == 500 or statusCode == 502  then
                    App:EnableInApps();
                    local message = resData[JuneApiKeys.messageKey];
                    EventMgr:postEvent( {message=message }, "game-status-check-failed" );
             end
             if statusCode == 501 then
                local message = resData[JuneApiKeys.messageKey];
                App:DisableInApps(message);
             end
       end

        function wrapperCallObj:callFailure(response)
            print("web request failure data: " .. tostring(response or "nil"));
       end

    local devType = MOAIEnvironment.osBrand:lower();
    local locale = MOAIEnvironment.languageCode or "en";

    local requestOptions = {
                  priority = Request.PRIORITY_LOW,
                  callHandler = wrapperCallObj,
                  successFunc = wrapperCallObj.callSuccess,
                  failureFunc = wrapperCallObj.callFailure,
                  data = string.format("{\"%s\":\"%s\",\"%s\":\"%s\"}",JuneApiKeys.deviceTypeKey,devType,JuneApiKeys.localeKey,locale),
                  url = ApiURL..JuneApiURLs.gameStatusCheck,
                  retryCount = 3,
               }

    local request = PushWebRequest.newFromBase(requestOptions);
     NetworkMgr:sendPushWebRequest(request);
end

----------------------------------------------------------------

function JuneApi:CheckPlayerStatus()
    print("JuneApi:CheckPlayerStatus");

   local playerId = App:getJunePlayerId();
    print("JuneApi:CheckPlayerStatus. PlayerId: ".. (playerId or "nil"));

    if playerId == nil then
        return;
    end

    if not OnlineMgr:isOnline()  then
        local redButtonPressed = App.data:getValue("big-player-red-button-pressed",false);
        if redButtonPressed then
            local message = App.data:getValue("big-player-red-button-message");
            EventMgr:postEvent( {message=message }, "player-status-check-failed" );
        end
        return;
    end

    local wrapperCallObj = {}

    function wrapperCallObj:callSuccess(response)
            print("web request PlayerStatus success  data  : " .. response:getString());
            local resData = json.decode(response:getString());
            local statusCode = resData[JuneApiKeys.statusCodeKey];
               if statusCode == 200 then
                EventMgr:postEvent( { }, "player-status-check-passed" );
            end

            if statusCode == 403 then
                    local message = "You have been banned from the game. If you think this is a mistake, please contact us at roadriot@junegaming.freshdesk.com. Please mention your player id: ".. App:getJunePlayerId() .. " in the email. Sorry for any inconvenience caused.";
                    --local message = resData[JuneApiKeys.messageKey] or msg;

                    EventMgr:postEvent( {message=message }, "player-status-check-failed" );
             end
             if statusCode == 501 then
                local message = resData[JuneApiKeys.messageKey];
                App:DisableInApps(message);
             end

       end

        function wrapperCallObj:callFailure(response)
            print("web request failure data: " .. tostring(response or "nil"));
       end

    local devType = MOAIEnvironment.osBrand:lower();
    local locale = MOAIEnvironment.languageCode or "en";
    local udid = MOAIEnvironment.udid;
    if devType == "ios" then
        udid = MOAIEnvironment.openUdid;
    end

    local requestOptions = {
                  priority = Request.PRIORITY_LOW,
                  callHandler = wrapperCallObj,
                  successFunc = wrapperCallObj.callSuccess,
                  failureFunc = wrapperCallObj.callFailure,
                  data = string.format("{\"%s\":\"%s\",\"%s\":\"%s\",\"%s\":\"%s\",\"%s\":\"%s\",\"%s\":\"%s\"}",JuneApiKeys.deviceIdKey,udid,JuneApiKeys.playerIdKey,playerId or "__GUEST__MODE__",JuneApiKeys.deviceTypeKey,devType,JuneApiKeys.localeKey,locale,JuneApiKeys.currentVersion,tostring(App.info.version) ),
                  url = ApiURL..JuneApiURLs.playerStatusCheck,
                  retryCount = 3,
               }

    local request = PushWebRequest.newFromBase(requestOptions);
     NetworkMgr:sendPushWebRequest(request);

end

----------------------------------------------------------------

function JuneApi:CheckGifts()
    print("JuneApi:CheckGifts");

    local playerId = App:getJunePlayerId();
    print("JuneApi:CheckGifts. PlayerId: ".. (playerId or "nil"));

    if playerId == nil then
        return;
    end


    if not OnlineMgr:isOnline() then
        return;
    end

    local wrapperCallObj = {}
    function wrapperCallObj:callSuccess(response)
        print("web request gift check success data  : " .. response:getString());
        local resData = json.decode(response:getString());
        local statusCode = resData[JuneApiKeys.statusCodeKey];

        if statusCode == 200 then
            local isClaimable = resData[JuneApiKeys.giftIsClaimableKey];
            local autoClaim = resData[JuneApiKeys.giftAutoClaimKey];
            local giftValuesTable = resData[JuneApiKeys.valueKey];
            if giftValuesTable ~= nil then
                App.giftsFromServer = giftValuesTable;
                JuneApi:ClaimGifts();
            end
        end
    end

    function wrapperCallObj:callFailure(response)
        print("web request failure data: " .. tostring(response or "nil"));
    end

    local requestOptions = {
                  priority = Request.PRIORITY_LOW,
                  callHandler = wrapperCallObj,
                  successFunc = wrapperCallObj.callSuccess,
                  failureFunc = wrapperCallObj.callFailure,
                  data = string.format("{\"%s\":\"%s\"}",JuneApiKeys.playerIdKey,playerId),
                  url = ApiURL..JuneApiURLs.giftCheck,
                  retryCount = 3,
               }

    local request = PushWebRequest.newFromBase(requestOptions);
     NetworkMgr:sendPushWebRequest(request);
end


----------------------------------------------------------------

function JuneApi:ClaimGifts()
    print("JuneApi:ClaimGifts");

    local playerId = App:getJunePlayerId();
    print("JuneApi:ClaimGifts. PlayerId: ".. (playerId or "nil"));

    if playerId == nil then
        return;
    end


    if not OnlineMgr:isOnline() then
        return;
    end

    local wrapperCallObj = {}
    function wrapperCallObj:callSuccess(response)
        print("web request gift claim success data  : " .. response:getString());
        local resData = json.decode(response:getString());
        local statusCode = resData[JuneApiKeys.statusCodeKey];

        if statusCode == 200 then
            local shouldPostEvent = false;
            if App.giftsFromServer ~= nil and #App.giftsFromServer > 0 then
                local eventDict = {};
                for idx = 1, #App.giftsFromServer do
                    local gift = App.giftsFromServer[idx];
                    local giftType = gift[JuneApiKeys.giftTypeKey];
                    local giftValue = gift[JuneApiKeys.giftValueKey];
                    local reason = "june gift";
                    if giftType == "gold" then
                        App.data:addGold(giftValue, reason);
                        shouldPostEvent = true;
                        eventDict.gold = giftValue;
                    end
                    if giftType == "gem" then
                        App.data:addGem(giftValue, reason);
                        shouldPostEvent = true;
                        eventDict.gems = giftValue;
                    end
                    if giftType == "key" then
                        App.data:addKey(giftValue, reason);
                        shouldPostEvent = true;
                        eventDict.keys = giftValue;
                    end
                    local type = giftType;
                    if (type == "revive") or (type == "head-start") or (type == "power-time") or (type == "double-score") or (type == "mystery") then
                        App.data:giveFreebie(type, giftValue, reason);
                        shouldPostEvent = true;
                        eventDict.consumable = type;
                        eventDict.count = giftValue;
                    end
                end
                if shouldPostEvent then
                    EventMgr:postEvent( eventDict, "june-gift-received" );
                end
            end

        end

        if statusCode == 402 then
            App.giftsFromServer = nil;
        end

    end

    function wrapperCallObj:callFailure(response)
        print("web request failure data: " .. tostring(response or "nil"));
    end

    local requestOptions = {
                  priority = Request.PRIORITY_LOW,
                  callHandler = wrapperCallObj,
                  successFunc = wrapperCallObj.callSuccess,
                  failureFunc = wrapperCallObj.callFailure,
                  data = string.format("{\"%s\":\"%s\"}",JuneApiKeys.playerIdKey,playerId),
                  url = ApiURL..JuneApiURLs.giftClaim,
                  retryCount = 3,
               }

    local request = PushWebRequest.newFromBase(requestOptions);
     NetworkMgr:sendPushWebRequest(request);
end

----------------------------------------------------------------

function JuneApi:processPossessionsTable(posTable,requestSuccessFunc, requestFailureFunc)

    if not posTable or not posTable.Possessions or #getTableKeys(posTable.Possessions) < 1 then
            TransactionMgr:uploadEverythingToJune( );
            return;
    end

    if posTable then
            local newArray = {};
            for k,v in pairs(posTable.Possessions) do
                local dict = {};
                dict["ItemId"] = k;
                for k1,v1 in pairs(v) do
                    if k1 ~= "LastModified" then
                        dict[k1] = tostring(v1);
                    else
                        dict[k1] = v1;
                    end
                end
                table.insert(newArray, dict);
            end
            posTable.Possessions = newArray;
            local arrPos = posTable.Possessions;
            for i = 1,#arrPos do
                local pos = arrPos[i];
                if pos.LastModified == nil then
                    pos.LastModified = TimeMgr:getTimeGMT();
                end
            end

            local posJson = json.encode(posTable);
            local dataTable = { posJson };
            local createPossessionsRequestClass = MOAIJuneGetPossessionsHandler;
            local createPossessionsRequest = createPossessionsRequestClass:new();
            createPossessionsRequest:setListener(
                                                MOAIJuneResponseHandler.RESPONSE_PROCESSED,
                                                function(...)
                                                    local data = {...};
                                                    local success, possessions = unpack(data);
                                                    if possessions then
                                                        requestSuccessFunc(true, possessions);
                                                    else
                                                        print("JuneApi:_onResponseProcessed possessions = NIL");
                                                        requestFailureFunc(false, -2, "Possessions download failed");
                                                    end
                                                end
                                                );
            createPossessionsRequest:processResponse(unpack(dataTable));
    else
        requestSuccessFunc(true, nil);
    end

end

function JuneApi:FetchPossessions(requestSuccessFunc, requestFailureFunc)

   self._playerId = App:getJunePlayerId();
    print("JuneApi:FetchPossessions. PlayerId: ", self._playerId);

    if self._playerId == nil then
        requestFailureFunc(false, -2, "Possessions download failed");
        return;
    end

    local wrapperCallObj = {}

    function wrapperCallObj:callSuccess(response)
            print("JuneApi:FetchPossessions. Success response: ", response:getString());

        local resData = json.decode(response:getString());
        local statusCode = resData[JuneApiKeys.statusCodeKey];
        if statusCode == 200 then
            local posTable = resData[JuneApiKeys.valueKey];
            local lastUpdateTime = resData[JuneApiKeys.lastUpdatedKey];
            App.data:setValue("june-api-last-update-value", lastUpdateTime);

            JuneApi:processPossessionsTable(posTable, requestSuccessFunc, requestFailureFunc);
        else
            print("JuneApi:FetchPossessions. Failure 1");
            requestFailureFunc(false, statusCode, "Possessions download failed");
        end
   end

    function wrapperCallObj:callFailure(response)
        print("JuneApi:FetchPossessions. Failure 2 response: ", response:getString());
        requestFailureFunc(false, -1, "Possessions download failed");
   end

    local requestOptions = {
        priority = Request.PRIORITY_NORMAL,
        callHandler = wrapperCallObj,
        successFunc = wrapperCallObj.callSuccess,
        failureFunc = wrapperCallObj.callFailure,
        data = string.format("{\"%s\":\"%s\"}",JuneApiKeys.playerIdKey,self._playerId);
        url = ApiURL..JuneApiURLs.playerGet,
        retryCount = 3,
    }

   local request = PushWebRequest.newFromBase(requestOptions);
   NetworkMgr:sendPushWebRequest(request);

end


function JuneApi:FetchPossessionsWithTangoId(providerAccountId, requestSuccessFunc, requestFailureFunc)
    print("JuneApi:FetchPossessionsFromProvider. PlayerId: ", self._playerId);
    JuneApi.hasTriedTangoId = false;

    local wrapperCallObj = {};
    function wrapperCallObj:callSuccess(response)
                print("JuneApi:FetchPossessionsWithTangoId. Success response: ", response:getString());

        local resData = json.decode(response:getString());
        local statusCode = resData[JuneApiKeys.statusCodeKey];

        if statusCode == 200 then
            local posTable = resData[JuneApiKeys.valueKey];
            local cloudId = resData[JuneApiKeys.playerIdKey];
            local lastUpdateTime = resData[JuneApiKeys.lastUpdatedKey];
            App.data:setValue("june-api-last-update-value", lastUpdateTime);

            App:SetCloudId(cloudId);
            App:OnMigratedToJune();
            JuneApi:processPossessionsTable(posTable, requestSuccessFunc, requestFailureFunc);
        elseif statusCode == 404 then
            JuneApi:LinkTango(false, function (...) end, function (...) end);
            TransactionMgr:uploadEverythingToJune( );
            JuneApi.hasTriedTangoId = true;
            PossessionMgr:flagShouldDownloadImmediately();
            requestFailureFunc(false, statusCode, "Download from Tango Pending");
        else
            print("JuneApi:FetchPossessions. Failure 1");
            requestFailureFunc(false, statusCode, "Possessions download failed");
        end
    end

    function wrapperCallObj:callFailure(response)
        print("JuneApi:FetchPossessions. Failure 2 response: ", response:getString());
        requestFailureFunc(false, -1, "Possessions download failed");
    end

    local tp = JuneSocialLinkTypes.tango;

    local requestOptions = {
        priority = Request.PRIORITY_NORMAL,
        callHandler = wrapperCallObj,
        successFunc = wrapperCallObj.callSuccess,
        failureFunc = wrapperCallObj.callFailure,
        data = string.format("{\"%s\":\"%s\",\"%s\":\"%s\"}",JuneApiKeys.socialProfileId,providerAccountId,JuneApiKeys.socialType,tp);
        url = ApiURL..JuneApiURLs.playerGetSocial,
        retryCount = 3,
    }

   local request = PushWebRequest.newFromBase(requestOptions);
   NetworkMgr:sendPushWebRequest(request);

end


function JuneApi:_onResponseProcessed(request,requestSuccessFunc, requestFailureFunc, processedResponse)
print("\nJuneApi:_onResponseProcessed ");
    local success, possessions = unpack(processedResponse);
    if possessions then
        requestSuccessFunc(true, possessions);
    else
        print("JuneApi:_onResponseProcessed possessions = NIL");
        requestFailureFunc(false, -2, "Possessions download failed");
    end
end

----------------------------------------------------------------

function JuneApi:UploadPossessions(possessionsJson,requestSuccessFunc, requestFailureFunc)

   self._playerId = App:getJunePlayerId();
    print("JuneApi:UploadPossessions. PlayerId: ", self._playerId);
   if self._playerId == nil then return; end

    local wrapperCallObj = {}

    function wrapperCallObj:callSuccess(response)
        print("JuneApi:UploadPossessions. Success response: ", response:getString());

        local resData = json.decode(response:getString());
        local statusCode = resData[JuneApiKeys.statusCodeKey];
        local outOfDate = false;
        if statusCode == 410 then
--            outOfDate = true;
            local lastUpdateTime = resData[JuneApiKeys.lastUpdatedKey];
            if lastUpdateTime ~= nil then
                App.data:setValue("june-api-last-update-value", lastUpdateTime);
            end
        end
        if statusCode == 200 then
            local lastUpdateTime = resData[JuneApiKeys.lastUpdatedKey];
            App.data:setValue("june-api-last-update-value", lastUpdateTime);
        end
        if statusCode == 200 or statusCode == 410 then
            print("Data uploaded to june");
            requestSuccessFunc(outOfDate);
        else
            requestFailureFunc();
        end
   end

    function wrapperCallObj:callFailure(response)
        requestFailureFunc();
   end


    local possessionsTable  = (json.decode(possessionsJson)).Possessions;
    local newArray = {};
    for idx = 1, #possessionsTable do
        local value = possessionsTable[idx];
        local key = value["ItemId"];
        local dict = {};
        for k1,v1 in pairs(value) do
            if k1 ~= "ItemId" then
                if k1 ~= "LastModified" then
                    dict[k1] = tostring(v1);
                else
                    dict[k1] = v1;
                end
            end
        end

        newArray[key] = dict;
    end

    possessionsTable = {};
    possessionsTable["Possessions"] = newArray;
    local encString = json.encode(possessionsTable);
    possessionsJson = encString;
    print("JuneAPi:Upload: posJson: ".. possessionsJson);

    local lastUpdateTime = App.data:getValue("june-api-last-update-value", 0);

    local requestOptions = {
        priority = Request.PRIORITY_NORMAL,
        callHandler = wrapperCallObj,
        successFunc = wrapperCallObj.callSuccess,
        failureFunc = wrapperCallObj.callFailure,
        data = string.format("{\"%s\":\"%s\",\"%s\":%s, \"%s\":%s}",JuneApiKeys.playerIdKey,self._playerId, JuneApiKeys.lastUpdatedKey, tostring(lastUpdateTime) ,JuneApiKeys.valueKey, possessionsJson);
        url = ApiURL..JuneApiURLs.playerUpdatePossessions,
        retryCount = 3,
    }

--    requestOptions.data [ JuneApiKeys.playerIdKey ] = self._playerId;
--    requestOptions.data [ JuneApiKeys.valueKey ] = possessionsJson;

   local request = PushWebRequest.newFromBase(requestOptions);
   NetworkMgr:sendPushWebRequest(request);

end

----------------------------------------------------------------

function JuneApi:ValidatePurchase(origRequestOptions)
    print("JuneApi:ValidatePurchase");
    self._playerId = App:getJunePlayerId();
    print("JuneApi:UploadPossessions. PlayerId: ", self._playerId);
    if self._playerId == nil then
        origRequestOptions.failureFunc(-1,"Player not found");
        return;
    end

    local wrapperCallObj = {}

    function wrapperCallObj:callSuccess(response)
        print("JuneApi:ValidatePurchase response: ", response:getString());
        local resData = json.decode(response:getString());
        local statusCode = resData[JuneApiKeys.statusCodeKey];
        if statusCode == 200 then
            origRequestOptions.successFunc({ Valid = true });
        else
            origRequestOptions.failureFunc(statusCode,"Purchase failed");
        end
   end

    function wrapperCallObj:callFailure(response)
        origRequestOptions.failureFunc(-1,"Purchase failed");
   end

    local signedData = origRequestOptions.requestData.receipt;

    local devType = MOAIEnvironment.osBrand:lower();
    local purchaseProvider = "google";
    if devType == "ios" then
        purchaseProvider = "apple";
        signedData = StringUtils.base64Encode(signedData);
    end

    local requestOptions = {
        priority = Request.PRIORITY_NORMAL,
        callHandler = wrapperCallObj,
        successFunc = wrapperCallObj.callSuccess,
        failureFunc = wrapperCallObj.callFailure,
        data = "",
        url = ApiURL..JuneApiURLs.purchaseVerify,
        retryCount = 3,
    }

    local dataTable = { };
    dataTable[JuneApiKeys.playerIdKey] = self._playerId;
    dataTable[JuneApiKeys.deviceTypeKey] =  devType;
    dataTable[JuneApiKeys.itemIdKey] = origRequestOptions.requestData.itemId;
    dataTable[JuneApiKeys.purchaseProviderKey] =  purchaseProvider;
    dataTable[JuneApiKeys.purchaseReceiptKey] = signedData;


    requestOptions.data = json.encode(dataTable);

print("purchaseVerification. requestOptions.data = ", requestOptions.data);
   local request = PushWebRequest.newFromBase(requestOptions);
   NetworkMgr:sendPushWebRequest(request);

end

----------------------------------------------------------------

function JuneApi:_onTangoProfileUpdated(event)

    if LoginMgr:isTango() then
        local profile = ContactMgr:getProfile();
        if profile ~= nil then
            print("JuneApi:_onTangoProfileUpdated profile:getAccountId(): ", tostring(profile:getAccountId() or "nil"));

            if profile:getAccountId() ~= nil and profile:getAccountId() ~= "__GUEST__MODE__" then
                local accountId = profile:getAccountId();
                local tangoLinked = App:IsTangoIdLinkedToJuneProfile(accountId);
                if not tangoLinked then
                    local isForced = false;
                    local successCallback = function() App:LinkTangoIdToJuneProfile(accountId); end;
                    local conflictCallback = function ( cloudId, tangoAccountId ) JuneApi:TangoLinkConflict(cloudId); end;
                    JuneApi:LinkTango(isForced, successCallback, conflictCallback);
                end
            end
        end
    end
end

function JuneApi:TangoLinkConflict(cloudId)
    self._juneDownloadSuccess = true;
    PossessionMgr:ForceDownload({name="tango", cloudId=cloudId});
end

function JuneApi:LinkTango (forced, successCallback, conflictCallback)

    self._playerId = App:getJunePlayerId();
    print("JuneApi:LinkTango. PlayerId: ", self._playerId);
    if self._playerId == nil then
        return;
    end

    if not LoginMgr:isTango() then
        return;
    end

    local profile = (ContactMgr:getProfile() or {});
    if profile:getAccountId() ~= nil and profile:getAccountId() ~= "__GUEST__MODE__" then
        local accountId = profile:getAccountId();
        local wrapperCallObj = {}

        function wrapperCallObj:callSuccess(response)
            local resData = json.decode(response:getString());
            local statusCode = resData[JuneApiKeys.statusCodeKey];
            local cloudId = resData[JuneApiKeys.playerIdKey];

            if statusCode == 200 and successCallback ~= nil then
                App:LinkTangoIdToJuneProfile(accountId);
                successCallback();
            end

            if statusCode == 409 then
                if conflictCallback ~= nil then
                    conflictCallback(cloudId, accountId);
                end
            end

       end

        function wrapperCallObj:callFailure(response)
            print("web request failure data: " .. tostring(response or "nil"));
       end


        local requestOptions = {
            priority = Request.PRIORITY_NORMAL,
            callHandler = wrapperCallObj,
            successFunc = wrapperCallObj.callSuccess,
            failureFunc = wrapperCallObj.callFailure,

            data = string.format("{\"%s\":\"%s\",\"%s\":\"%s\",\"%s\":\"%s\",\"%s\":%s}",
                                    JuneApiKeys.playerIdKey, self._playerId,
                                    JuneApiKeys.socialProfileId, accountId,
                                    JuneApiKeys.socialType, JuneSocialLinkTypes.tango,
                                    JuneApiKeys.forced, tostring(forced)
                                    );
            url = ApiURL..JuneApiURLs.playerLinkSocial,
            retryCount = 3,
        }

    --    requestOptions.data [ JuneApiKeys.deviceIdKey ] = MOAIEnvironment.openUdid;
    --    requestOptions.data [ JuneApiKeys.deviceTypeKey ] = MOAIEnvironment.osBrand:lower();

       local request = PushWebRequest.newFromBase(requestOptions);
       NetworkMgr:sendPushWebRequest(request);

    end
end

function JuneApi:UpdateDevice()

    self._playerId = App:getJunePlayerId();
    print("JuneApi:UpdateDevice. PlayerId: ", self._playerId);
    if self._playerId == nil then
        return;
    end

    local wrapperCallObj = {}
    print("JuneApi:UpdateDevice");

    function wrapperCallObj:callSuccess(response)
        print("web request UpdateDevice success data: " .. tostring(response:getString() or "nil"));
        local resData = json.decode(response:getString());
        local statusCode = resData[JuneApiKeys.statusCodeKey];
        if statusCode == 200 then
            App.data:setValue("june-should-update-device",false);
        else
            App.data:setValue("june-should-update-device",true);
        end
    end

    function wrapperCallObj:callFailure(response)
        print("web request failure data: " .. tostring(response or "nil"));
        App.data:setValue("june-should-update-device",true);
    end


    local devType = MOAIEnvironment.osBrand:lower();
    local udid = MOAIEnvironment.udid;
    if devType == "ios" then
        udid = MOAIEnvironment.openUdid;
    end


    local requestOptions = {
        priority = Request.PRIORITY_NORMAL,
        callHandler = wrapperCallObj,
        successFunc = wrapperCallObj.callSuccess,
        failureFunc = wrapperCallObj.callFailure,
        data = string.format("{\"%s\":\"%s\",\"%s\":\"%s\",\"%s\":\"%s\",\"%s\":\"%s\"}",JuneApiKeys.playerIdKey, self._playerId,JuneApiKeys.deviceIdKey,udid,JuneApiKeys.deviceTypeKey,devType ,JuneApiKeys.currentVersion,tostring(App.info.version));
        url = ApiURL..JuneApiURLs.playerUpdateDevice,
        retryCount = 3,
    }

    local request = PushWebRequest.newFromBase(requestOptions);
    NetworkMgr:sendPushWebRequest(request);
end
