require('app/server/tango')

----------------------------------------------------------------------

local Base						= TangoServer;
CarInfoRetriever				= Base:new();

--TODO: Centralize (for now, there is duplicate in the uploaded)
local CAR                  = "carInfo2";

local TIME_FIELD_ORDERED   =
{
   "MAX_THIS_HOUR",
   "MAX_LAST_HOUR",
   "MAX_THIS_DAY",
   "MAX_LAST_DAY",
   "MAX_THIS_WEEK",
   "MAX_LAST_WEEK",
};

----------------------------------------------------------------------

function CarInfoRetriever:new( contactMgr  )

   local loader        = Base.new(self);

   loader.contactMgr   = contactMgr;
   loader.rank         = {};
   loader.contactList  = {};

   for i=1,contactMgr:getContactCount() do
      local contact = contactMgr:getContact(i);
      if contact and contact:isAppInstalled() and (not contact:isGhost()) and (not contact:isGuestFriend()) then
         table.insert( loader.contactList,  contact:getAccountId() );
      end
   end

   return loader;

end

----------------------------------------------------------------------

function CarInfoRetriever:exec( callObj, callSuccess, callFailure )

   assert( LoginMgr:isTango() );

   local returnTable = {};

   if OnlineMgr:isOnline() then

      local successFunc = function( result )

         if result then
            for k,res2 in pairs(result) do
               returnTable[k] = self:_createEntry( k, res2 );
            end
         end

         self:callCallback( callObj, callSuccess, returnTable );

      end

      local failureFunc = function(code, message)

         self:callCallback( callObj, callFailure, code, message );

      end

      local metricData = {
         {
            metric  = CAR,
            rollups = {
               MOAITangoMetrics.MAX_THIS_WEEK, MOAITangoMetrics.MAX_LAST_WEEK,
               MOAITangoMetrics.MAX_THIS_DAY,  MOAITangoMetrics.MAX_LAST_DAY,
               MOAITangoMetrics.MAX_THIS_HOUR, MOAITangoMetrics.MAX_LAST_HOUR,
            }
         }
      }

      local requestOptions = {
         requestClass = MOAITangoGetComputedMetricsRequest,
         requestData = {self.contactList, metricData},
         successFunc = successFunc,
         failureFunc = failureFunc,
      }

      NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

   else

      -- Use a timer to create be more consistent with the authenticate mode
      delay(
               0.1
            ,
               function ()
                  self:callCallback( callObj,  callFailure, -1, "You are offline"  );
               end

            );
   end

end

----------------------------------------------------------------------

function CarInfoRetriever:_pickMostRecentValue(  tbl )

   if tbl then

      for i,v in ipairs(TIME_FIELD_ORDERED) do
         if type(tbl[v]) == "number" and tbl[v] > 0 then
            return tbl[v];
         end
      end

   end

end

----------------------------------------------------------------------

function CarInfoRetriever:_createEntry(  account, info )

   if account and info then

      local carTbl            = info[CAR]   or {};

      local carCode           = self:_pickMostRecentValue(carTbl);

      return { code = carCode};

   end

end


----------------------------------------------------------------------
