--
require('engine/object/object')

----------------------------------------------------------------------

local Base        		= Object;
PictureRetriever  		= Base:new();

local IMAGE_SIZE_MAX    = { normal = 128, mini = 64 };

----------------------------------------------------------------------

function  PictureRetriever:new( accountId, url, destination )

   local o           = Base.new(self);

   o._accountId      = accountId;
   o._url            = url;
   o._destination    = destination;

   return o;

end


----------------------------------------------------------------------

function PictureRetriever:exec( callObj,  successFunc, failureFunc )

   downloadToFile(   self._url
                  ,  self._destination .. ".part"
                  ,  self
                  ,  function ()
                        if successFunc then

                           if self:_fixupImage() then

                              local workingDir = MOAIFileSystem.getWorkingDirectory ()
                              MOAIFileSystem.setWorkingDirectory ( MOAIEnvironment.documentDirectory )
                              Debug.assert(MOAIFileSystem.checkFileExists(self._destination), "PictureRetriever - File \"" .. tostring(self._destination) .. "\" did not exist!");
                              MOAIFileSystem.setWorkingDirectory ( workingDir )

                              successFunc(callObj, self._accountId, self._url, self._destination );
                           else
                              if failureFunc then
                                 failureFunc(callObj, ERROR_IMAGE_NOT_SUPPORTED );
                              end
                           end
                        end
                     end
                  ,  function ()
                        if failureFunc then
                           failureFunc(callObj, ERROR_DOWNLOAD_FAILED );
                        end
                     end);
end
----------------------------------------------------------------------

function PictureRetriever:_isFormatValid( format )

   if    format == MOAIImage.COLOR_FMT_A_8
      or format == MOAIImage.COLOR_FMT_RGB_888
      or format == MOAIImage.COLOR_FMT_RGBA_4444
      or format == MOAIImage.COLOR_FMT_RGBA_8888 then
      return true;
   end


end

----------------------------------------------------------------------

function PictureRetriever:_fixupImage()

   print("PictureRetriever:_fixupImage() begin")

   local workingDir = MOAIFileSystem.getWorkingDirectory ()

   MOAIFileSystem.setWorkingDirectory ( MOAIEnvironment.documentDirectory )

   local image    = MOAIImage.new ()
   local maxSize  = IMAGE_SIZE_MAX.normal;

   if App.useMiniTexture then
      maxSize  = IMAGE_SIZE_MAX.mini;
   end

   print("PictureRetriever:_fixupImage() loading PNG:", self._destination)

   image:load(self._destination .. ".part");

   print("PictureRetriever:_fixupImage() loaded PNG:", self._destination)

   MOAIFileSystem.setWorkingDirectory ( workingDir )

   local x1, y1 = image:getSize();

   if image and x1 > 0 and  y1 > 0 and self:_isFormatValid( image:getFormat() )then

      local x1, y1 = image:getSize();

      local image2 = MOAIImage.new();

      if x1 < y1 then
         local diff = y1 - x1;
         local size = math.min(x1,maxSize);
         image2:init ( size, size );
         image2:copyRect(image, 0, diff/2, x1, y1-diff/2,  0, 0, size, size );
         y1 = x1;
      else
         local diff = x1 - y1;
         local size = math.min(y1,maxSize);
         image2:init ( size, size );
         image2:copyRect(image, diff/2, 0, x1-diff/2, y1,  0, 0, size, size );
         x1 = y1;
      end

      MOAIFileSystem.setWorkingDirectory ( MOAIEnvironment.documentDirectory )

      print("PictureRetriever:_fixupImage() writing PNG to:", self._destination)

      image2:writePNG( self._destination  );

      MOAIFileSystem.deleteFile(self._destination .. ".part");

      MOAIFileSystem.setWorkingDirectory ( workingDir )

      print("PictureRetriever:_fixupImage() succeeded")

      return true;

   end

   print("PictureRetriever:_fixupImage() failed")

   MOAIFileSystem.setWorkingDirectory ( MOAIEnvironment.documentDirectory )
   MOAIFileSystem.deleteFile(self._destination .. ".part");
   MOAIFileSystem.setWorkingDirectory ( workingDir )

   return false;
end

----------------------------------------------------------------------
