require('app/contact/contact-manager')
require('app/ranking/ranking-manager')
require('app/bounty/bounty-manager')
require('app/data/transaction-manager')
require('app/data/transaction/data-transaction-add-consumable')
require('app/data/transaction/data-transaction-add-drone-mount')
require('app/data/transaction/data-transaction-add-drone')
require('app/data/transaction/data-transaction-add-freebie')
require('app/data/transaction/data-transaction-add-gem')
require('app/data/transaction/data-transaction-add-gold')
require('app/data/transaction/data-transaction-add-key')
require('app/data/transaction/data-transaction-add-reward-level')
require('app/data/transaction/data-transaction-add-weekly-reward')
require('app/data/transaction/data-transaction-add-xp')
require('app/data/transaction/data-transaction-buy-gold')
require('app/data/transaction/data-transaction-buy-key')
require('app/data/transaction/data-transaction-consume-consumable')
require('app/data/transaction/data-transaction-consume-key')
require('app/data/transaction/data-transaction-daily-pickup')
require('app/data/transaction/data-transaction-daily-collect-reward')
require('app/data/transaction/data-transaction-iap-verification')
require('app/data/transaction/data-transaction-invite-reward')
require('app/data/transaction/data-transaction-spend-freebie')
require('app/data/transaction/data-transaction-spend-gem')
require('app/data/transaction/data-transaction-tango-bonus')
require('app/data/transaction/data-transaction-unlock-paint')
require('app/data/transaction/data-transaction-unlock-vehicle')
require('app/data/transaction/data-transaction-update-achievement-progress')
require('app/data/transaction/data-transaction-update-achievement-rank')
require('app/data/transaction/data-transaction-update-bounty-reward')
require('app/data/transaction/data-transaction-update-drone-stats')
require('app/data/transaction/data-transaction-update-game-stats')
require('app/data/transaction/data-transaction-update-league')
require('app/data/transaction/data-transaction-update-level')
require('app/data/transaction/data-transaction-update-mission')
require('app/data/transaction/data-transaction-update-mission-goal')
require('app/data/transaction/data-transaction-upgrade-armor')
require('app/data/transaction/data-transaction-upgrade-drone')
require('app/data/transaction/data-transaction-upgrade-main-weapon')
require('app/data/transaction/data-transaction-upgrade-pickup')
require('app/data/transaction/data-transaction-upgrade-side-weapon')
require('app/data/transaction/data-transaction-upgrade-turret-weapon')
require('app/possession/possession-manager')
require('app/template/template-manager')
require('engine/object/object')
require('engine/file/fileMgr')
require('engine/utility/tableUtil')

----------------------------------------------------------------------

HACK_CAN_CHANGE_MISSION_INDEX = true;
HACK_BOUNTY_ALWAYS_UNLOCKED   = nil;

MISSION_GOAL_COUNT            = 3;

MAIN_WEAPON_LEVEL_DEFAULT     = 1;
MAIN_WEAPON_LEVEL_MIN         = 1;
MAIN_WEAPON_LEVEL_MAX         = 15;

SIDE_WEAPON_LEVEL_DEFAULT     = 0;
SIDE_WEAPON_LEVEL_MIN         = 1;
SIDE_WEAPON_LEVEL_MAX         = 15;

ARMOR_LEVEL_DEFAULT           = 1;
ARMOR_LEVEL_MIN               = 1;
ARMOR_LEVEL_MAX               = 15;

TURRET_LEVEL_DEFAULT          = 0;
TURRET_LEVEL_MIN              = 1;
TURRET_LEVEL_MAX              = 15;

DRONE_LEVEL_DEFAULT           = 1;
DRONE_LEVEL_MIN               = 1;
DRONE_LEVEL_MAX					= 15;

DRONE_RARITY_MAX					= 5;

--Dont make this local... it is being use externally
TUTORIAL_DONE_NUMBER          = 10;

local FIRST_MK2_VEHICLE_INDEX    = 6;
local DAILY_TARGET_RESET         = 19.50;       -- 7:30pm
--  ********************************
local DAILY_ONE_DAY              = 60*60*24;
--local DAILY_ONE_DAY              = 60*15;
--  ********************************
local KEY_AMOUNT_START           = 6;
local KEY_REGEN_TIME_DEFAULT     = 60*15;
local KEY_MAX_REGEN              = 6;
local KEY_MAX_REWARD             = 3;
local KEY_REFERENCE_TIME_OFFSET  = 60*60*24*10;--This is a cheap way to make sure our users will refresh there key after a new update if needed
local MAX_PLAYER_LEVEL           = 1000; --There is a copy in Player Levels
local INVITE_REWARD_GOLD         = 20;

local FILE_SAVE_MAX_ATTEMPT		= 5;
local FILE_LOAD_MAX_ATTEMPT		= 5;

local SAVE_FILENAME                 = "ROAD_RIOT_SAVE_DATA.lua";
local SAVE_FILENAME_JSON_PROTECTED  = "d87a01t7666a.jp5";         --Weird looking file name to lower suspicious from hackers
local VEHICLE_SALE_ID_FILENAME      = "ROAD_RIOT_725343.lua";
local VEHICLE_PAINT_ID_FILENAME     = "a918k3190483.k98";
local CRITICAL_SAVE_FILENAME        = "c95r84i56t27i89c0a43l.409";

local SAVE_DATA_COOLDOWN               = 1.0;

local DATA_PROTECTION_ENABLED          = false;

--The game has been tweaked for 5
local LEVEL_BAND                       = 5;

local DRONE_MIN_COUNT_DEFAULT		= 1;
local DRONE_MAX_COUNT_DEFAULT		= 21;
local DRONE_LEFT						= "left";  -- mirror in drone manager
local DRONE_RIGHT						= "right"; -- mirror in drone manager

local CONSUMABLES_MAX            = 6; --max types of consumables

local HACK_DAILY_DISABLED              = false;
local HACK_QUICK_KEY                   = false;
local HACK_QUICK_DAILY                 = false;
local HACK_QUICK_RESET                 = false;
local HACK_BOSS_ENCOUNTER_RATE         = nil;
local HACK_VEHICLE_SALE_ID             = nil;

--Do not change here but above
if not BUILD_DEBUG then
   HACK_DAILY_DISABLED        = false;
   HACK_QUICK_KEY             = false;
   HACK_QUICK_DAILY           = false;
   HACK_QUICK_RESET           = false;
   HACK_BOSS_ENCOUNTER_RATE   = nil;
   HACK_VEHICLE_SALE_ID       = nil;
end

if HACK_QUICK_KEY then
   KEY_REGEN_TIME_DEFAULT = 60*2;
end

if HACK_QUICK_DAILY then
   DAILY_ONE_DAY = 60*15;
end


----------------------------------------------------------------------

local Base = Object;

AppData =
{
};

----------------------------------------------------------------------

function AppData:new( encryptionKey  )

   local obj = Base.new( self );

   obj.fileName                  = SAVE_FILENAME;
   obj._builtinInfo              = obj:_createBuiltinInfoTable();
   obj._data                     = {};
   obj._encryptionKey            = encryptionKey;
   obj._regenKeyEnable           = false;
   obj._saveDataCooldown         = 0;

   obj:_load();
   obj:ensureValidState();

   EventMgr:addListener( obj, obj._onDataSaveError, "data-save-error" );

   return obj;

end

-------------------------------------------------------------------------------------

function AppData:_onDataSaveError()

   self:setValue( "save-critical-only", true );

end

-------------------------------------------------------------------------------------

function AppData:save()

   self:_saveData();

   TransactionMgr:save(self);

end

-------------------------------------------------------------------------------------

function AppData:_saveData()

   --The proof file is a few bytes long which will NEVER grow
   --We should always be able to save unless something is wrong with the device
   --It is use also to detect if the user previously had a version with "critical" implemented
   local success = saveLocalFile( self:_createProofFilename(),     self:_createProofTable(),       FILE_SAVE_MAX_ATTEMPT);

   if success then

      success = saveLocalFile( self:_createCriticalFilename2(), self:_createCriticalTable(),       FILE_SAVE_MAX_ATTEMPT );

      --Only try saving the full data table if we succeeded.
      --Otherwise, we will most likely correupt both files and the user will loose everything
      if success then

         success = saveLocalFile( self.fileName, self._data,       FILE_SAVE_MAX_ATTEMPT );

         if success then
            self:_saveDataProtected();
         else
            --Now we know that data is corrupted... try to overwrite with critical only for now
            --We will try to make sore room and hopefully next time will work
            --If we dont do it, and if saving critical failed next time = the user loose everything
            saveLocalFile( self.fileName, self:_createCriticalTable(),       FILE_SAVE_MAX_ATTEMPT );

            EventMgr:postEvent( { critical = true }, "data-save-error" );

         end
      else
         EventMgr:postEvent( {critical = true }, "data-save-error"  );
      end
   else
      EventMgr:postEvent( {critical = true }, "data-save-error"  );
   end

   self._saveDataCooldown  = 0;
   self._saveDataFlag      = not success;

   return success;


end

-------------------------------------------------------------------------------------

function AppData:_saveDataProtected()

   if DATA_PROTECTION_ENABLED then
      saveJsonFileProtected( SAVE_FILENAME_JSON_PROTECTED, self._data, self._encryptionKey );
   end
end

-------------------------------------------------------------------------------------
-- Load all the data related to the player and games
-- Some of these data are save on the cloud and will be overwritten once we load from server
-- We still save everything here so it is more robust
function AppData:_load()

   self:_restoreData();

   TransactionMgr:load(self);

   self:_setDefaultValueIfNeeded();

end

-------------------------------------------------------------------------------------

function AppData:ensureValidState()

   --This will ensure to not have undesigned nil value which could crash some part of the game
   self:_setDefaultValueIfNeeded();

   --Make sure to handle and adjust values related to timeStamp
   self:_updateDaily();
   self:_regenKeys();

   --Make sure the current car is usable... you never know (we could reset)
   if not self:isPlayerVehicleUnlocked(self:getValue("currentPlayerVehicle")) then
      self:setValue("currentPlayerVehicle", 1);
   end

   --Make sure the current paint is usable...you never know (we could reset)
   self:setValue("currentPlayerVehiclePaintVariant", self:getValue("currentPlayerVehiclePaintVariant") or 1);
   if not self:isPlayerVehiclePaintVariantUnlocked( self:getValue("currentPlayerVehicle"), self:getValue("currentPlayerVehiclePaintVariant") ) then
      self:setValue("currentPlayerVehiclePaintVariant", 1);
   end

end

-------------------------------------------------------------------------------------

function AppData:update( dt )

   self:_updateDaily(dt);

   self:_regenKeys(dt);

   TransactionMgr:update(dt);

   self:_updateSaveData( dt );

end

-------------------------------------------------------------------------------------

function AppData:_updateSaveData( dt )

   self._saveDataCooldown = self._saveDataCooldown + dt;
   if self._saveDataFlag and self._saveDataCooldown >= SAVE_DATA_COOLDOWN then
      self:_saveData();
   end

end

-------------------------------------------------------------------------------------

--This will make sure each data has expected default value
function AppData:_createBuiltinInfoTable()

   -- We have a limit of 99 server side variable... please talk to Michel before adding any new one
   -- Thank you

   local info = {
      { name = "save-critical-only",               default = false,            critical = true,  server = false                                          },
      { name = "_guestMode",                       default = 0,                critical = true,  server = false                                          },
      { name = "controlSensitivity",               default = 0.33,             critical = false, server = false                                          },
      { name = "globalVolume2",                    default = 1.00,             critical = false, server = false                                          },
      { name = "gold",                             default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "gem",                              default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "freebie-revive",                   default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "freebie-head-start",               default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "freebie-power-time",               default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "freebie-double-score",             default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "freebie-mystery",                  default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "gamePlayed",                       default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "signingGold",                      default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "signingGem",                       default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "key",                              default = KEY_AMOUNT_START, critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "keyTimeStamp",                     default = -1,               critical = false, server = false                                          },
      { name = "currentPlayerVehicle",             default = 1,                critical = false, server = false                                          },
      { name = "currentPlayerVehiclePaintVariant", default = 1,                critical = false, server = false                                          },
      { name = "dailyStreak",                      default = 0,                critical = true,  server = true,    merge = self._possessionMergeDaily    },
      { name = "dailyProgress",                    default = 0,                critical = false, server = true,    merge = nil                           },
      --use dailyTimeStamp4 instead of dailyTimeStamp because we change the source timer and didnt want to break older user (it will be reset)
      { name = "dailyTimeStamp4",                  default = nil,              critical = false, server = false                                          },
      { name = "dailyGamePlayed",                  default = 0,                critical = false, server = true,    merge = nil                           },
      { name = "dailyRewardGem",                   default = 0,                critical = false, server = true,    merge = self._possessionMergeMaxValue },
      { name = "dailyRewardGold",                  default = 0,                critical = false, server = true,    merge = self._possessionMergeMaxValue },
      { name = "currentLevel",                     default = 1,                critical = true,  server = true,    merge = self._mergePossessionsLevel   },
      { name = "xp",                               default = 0,                critical = true,  server = true,    merge = nil                           },
      { name = "levelGamePlayed",                  default = 0,                critical = false, server = true,    merge = nil                           },
      { name = "rewardLevel",                      default = 1,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "iap-verified",                     default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "iap-fake",                         default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "cheater-tango-account-id",         default = nil,              critical = true,  server = false                                          },
      { name = "guest-mgr-target-score",           default = 0,                critical = true,  server = false                                          },
      { name = "weekly-reset-timestamp",           default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
      { name = "weekly-gifts-collected",           default = 0,                critical = true,  server = true,    merge = self._possessionMergeMaxValue },
   };

   for i=1,TemplateMgr:getPlayerVehicleCount() do
      -- MQL: set main weapon and armor to 0 default. If 1, it means the car is unlocked
      table.insert(  info,  {  name  = self:createMainWeaponId(i),             default  = 0, critical = true, server = true,   merge = self._possessionMergeMaxValue  }  );
      table.insert(  info,  {  name  = self:createArmorId(i),                  default  = 0, critical = true, server = true,   merge = self._possessionMergeMaxValue  }  );
      table.insert(  info,  {  name  = self:createSideWeaponId(i),             default  = 0, critical = true, server = true,   merge = self._possessionMergeMaxValue  }  );
      table.insert(  info,  {  name  = self:createTurretWeaponId(i),           default  = 0, critical = true, server = true,   merge = self._possessionMergeMaxValue  }  );

      for j=1, TemplateMgr:getPlayerVehiclePaintCount(i) do
         table.insert(info, { name = self:createPlayerVehiclePaintVariantId(i, j), default  = 0,    critical = true, server  = true, merge = self._possessionMergeMaxValue  }  );
      end
   end

   for i=1,TemplateMgr:getPickupCount() do
      table.insert(info, { name  = self:createPickupId(i),                          default  = 1,        critical = true,     server = true,        merge = self._possessionMergeMaxValue  }  );
   end

   table.insert(info, {  name = "missionIndex",                                     default = 1,         critical = true,     server = true,        merge = self._mergePossessionsMission   });

   for i=1,MISSION_GOAL_COUNT do
      table.insert(info, { name = self:createMissionGoalProgressId(i),              default  = 0,        critical = true,     server = true,        merge = nil }   );
      table.insert(info, { name = self:createMissionGoalGamePlayedId(i),            default  = 0,        critical = false,    server = true,        merge = nil }   );
   end

   local achvList = TemplateMgr:getAchievement();

   if achvList then

      for i,achv in ipairs(achvList) do
         if achv and achv.id then
            table.insert(info, { name = self:createAchievemetProgressId(achv.id),      default  = 0,        critical = true,     server = true,        merge = nil }   );
            table.insert(info, { name = self:createAchievemetRankId(achv.id),          default  = 1,        critical = true,     server = true,        merge = nil }   );
         end
      end
   end

   for i=1, TemplateMgr:getBossIndexMax() do
      table.insert(info, { name = self:createBossKillCountID(i), default	= 0, critical = true, server = true, merge = self._possessionMergeMaxValue }	);
      table.insert(info, { name = self:createBossLevelID(i),     default    = 1, critical = true, server = true, merge = self._possessionMergeMaxValue }    );
   end

   -- Bounties
   table.insert(info, {  name = "bountyPlayerCount",    default = 0, critical = true, server = true, merge = self._possessionMergeMaxValue   });
   table.insert(info, {  name = "activeWeekCount",		default = 0, critical = true, server = true, merge = self._possessionMergeMaxValue   });

   -- Leagues
   table.insert(info, {  name = "league-LeagueIndex",	default = 0, critical = true, server = true, merge = self._possessionMergeMinValue   });
   table.insert(info, {  name = "league-DivisionIndex",	default = 0, critical = true, server = true, merge = self._possessionMergeMinValue   });

   -- Drones
   for i = 1, DRONE_MAX_COUNT_DEFAULT do
      table.insert(info, self:_createDroneDefaultData(i, DRONE_LEFT));
      table.insert(info, self:_createDroneDefaultData(i, DRONE_RIGHT));
   end

   table.insert(info, {  name = "drone_current_mount_left",                         default = 1,         critical = true,     server = true,        merge = self._possessionMergeMaxValue  });
   table.insert(info, {  name = "drone_current_mount_right",                        default = -1,        critical = true,     server = true,        merge = self._possessionMergeMaxValue  });
   table.insert(info, {  name = "drone_spawn_total_count_left",                     default = 0,         critical = true,     server = true,        merge = self._possessionMergeMaxValue  });
   table.insert(info, {  name = "drone_spawn_total_count_right",                    default = 0,         critical = true,     server = true,        merge = self._possessionMergeMaxValue  });

   -- Consumables
   for i=1, CONSUMABLES_MAX do
      table.insert(info, self:_createConsumableDefaultData(i));
   end

   return info;
end

-------------------------------------------------------------------------------------

--This will make sure each data has expected default value
function AppData:_setDefaultValueIfNeeded()

   if self._builtinInfo then
      for i,tbl in ipairs(self._builtinInfo) do
         self._data[tbl.name] = self._data[tbl.name] or tbl.default;
      end
   end

   --make sure the first car is unlocked
   self:_onPlayerVehicleUnlocked(1);

end

-------------------------------------------------------------------------------------

function AppData:_restoreData()

   local proof    =  loadLocalFile( self:_createProofFilename(),         FILE_LOAD_MAX_ATTEMPT );
   --We are replacing critical filename, keep using the old one as a backup for now
   local critical =  loadLocalFile( self:_createCriticalFilename2(),     FILE_LOAD_MAX_ATTEMPT )
                  or loadLocalFile( self:_createCriticalFilename(),      FILE_LOAD_MAX_ATTEMPT ) ;
   local data     =  loadLocalFile( self.fileName,                       FILE_LOAD_MAX_ATTEMPT );

   --This is what we want, both a critical file and a full data file
   if critical and data then

      self._data     = data;
      self._critical = critical;

   else

      if data then

         --Make sure the reason we are missing the critical file is NOT because the previous version didnt support it
         if proof then
            EventMgr:postEvent( { dataOk = true, criticalOk = false }, "data-load-error");
            MOAIFlurry:event( "Error-Data-Load", "Type", "Critical-Nil", "Big Endian", tostring(BIG_ENDIAN), "Tango-User", tostring(LoginMgr:isTango()) );
         end

         self._data = data;

      elseif critical then

         MOAIFlurry:event( "Error-Data-Load", "Type", "Data-Nil", "Big Endian", tostring(BIG_ENDIAN), "Tango-User", tostring(LoginMgr:isTango()) );

         --Notified the user
         EventMgr:postEvent( {dataOk = false, criticalOk = true }, "data-load-error" );

         self._data = critical;
         TransactionMgr:flagForcedMigration();

      elseif proof then

         MOAIFlurry:event( "Error-Data-Load", "Type", "Both-Nil", "Big Endian", tostring(BIG_ENDIAN), "Tango-User", tostring(LoginMgr:isTango()) );
         EventMgr:postEvent( {dataOk = false, criticalOk = false }, "data-load-error" );

         self._data = {};
         TransactionMgr:flagForcedMigration();

      else

         --New user
         self._data = {};

         TransactionMgr:flagForcedMigration();

      end
   end

   self:_restoreDataProtected();

end

-------------------------------------------------------------------------------------

function AppData:_restoreDataProtected()

   print("_restoreDataProtected", DATA_PROTECTION_ENABLED);

   if DATA_PROTECTION_ENABLED then
      local data, error  = loadJsonFileProtected( SAVE_FILENAME_JSON_PROTECTED, self._encryptionKey );

      local state        = "ok";
      local diff         = nil;

      if type(self._data) == "table" and type(data) == "table" then

         diff = countTableDifferences(self._data, data);

         if diff > 0 then

            state = "diff-error";

            MOAIFlurry:event( "Data-JSON-Encryption-Error",
                              "diff",                    tostring(diff),
                              "gamePlayed",              tostring(self:getGamePlayed()or 0),
                              "diff gamePlayed",         tostring((self:getGamePlayed()or 0) - (data.gamePlayed or 0)),
                              "diff gem",                tostring((self:getGemCount() or 0) - (data.gem or 0)),
                              "gem",                     tostring(self:getGemCount()or 0),
                              "log10(gem)",              tostring(math.floor(math.log10(math.max(self:getGemCount()or 0,1)))) );

            Debug.assert( diff==0, "AppData:_restoreDataProtected -> diff==0" );

         end

      else

         state = "missing";

         MOAIFlurry:event( "Data-JSON-Encryption-Missing",
                           "gamePlayed",                 tostring(self:getGamePlayed()or 0),
                           "gem",                        tostring(self:getGemCount()or 0),
                           "log10(gem)",                 tostring(math.floor(math.log10(math.max(self:getGemCount()or 0,1)))) );

      end

      MOAIFlurry:event( "Data-JSON-Encryption-Report",
                        "state",                   state,
                        "error",                   tostring(error),
                        "diff",                    tostring(diff),
                        "gamePlayed",              tostring(self:getGamePlayed()or 0),
                        "gem",                     tostring(self:getGemCount()or 0),
                        "log10(gem)",              tostring(math.floor(math.log10(math.max(self:getGemCount() or 0,1)))) );
   end

end

-------------------------------------------------------------------------------------

function AppData:_saveDataSoon()

   self._saveDataFlag = true;

end

-------------------------------------------------------------------------------------

function AppData:setGuestMode( version )

   self:setValue( "_guestMode", version );
   self:_saveDataSoon();

end

-------------------------------------------------------------------------------------

function AppData:isGuestMode()

   local val = self:getValue( "_guestMode", 0 );

   if type(val) == "number" then
      return val > 0;
   else
      return val == true;
   end
end

-------------------------------------------------------------------------------------

function AppData:isPayer()

   return App.data:getValue("iap-verified", 0 ) > 0;

end

-------------------------------------------------------------------------------------

function AppData:getBuiltinInfo()

   return self._builtinInfo;

end

-------------------------------------------------------------------------------------

function AppData:_createCriticalFilename()
   return self.fileName.."._critical";
end

-------------------------------------------------------------------------------------

function AppData:_createCriticalFilename2()
   return CRITICAL_SAVE_FILENAME;
end



-------------------------------------------------------------------------------------

function AppData:_createCriticalTable()

   local criticalTbl = {};

   for _, tbl in ipairs(self._builtinInfo) do
      if tbl.critical then
         criticalTbl[tbl.name] = self:getValue(tbl.name);
      end
   end

   return criticalTbl;

end

-------------------------------------------------------------------------------------

function AppData:_createProofFilename()
   return self.fileName.."._proof";
end

-------------------------------------------------------------------------------------

function AppData:_createProofTable()
   return  { true };
end

-------------------------------------------------------------------------------------

function AppData:_createTransactionSyncingFilename()
   return self.fileName .. "._trans_sync";
end

-------------------------------------------------------------------------------------

function AppData:_createTransactionToSyncFilename()
   return self.fileName .. "._trans_to_sync";
end

-------------------------------------------------------------------------------------

function AppData:_possessionMergeMaxValue( name, possessions )

   if possessions:getValue(name) and self:getValue(name) then
      self._data[name] = math.max( possessions:getValue(name), self:getValue(name) );
   else
      self._data[name] = possessions:getValue(name) or self:getValue(name);
   end

end

-------------------------------------------------------------------------------------

function AppData:_possessionMergeMinValue( name, possessions )

   if possessions:getValue(name) and self:getValue(name) then
      self._data[name] = math.min( possessions:getValue(name), self:getValue(name) );
   else
      self._data[name] = possessions:getValue(name) or self:getValue(name);
   end

end

-------------------------------------------------------------------------------------

function AppData:_possessionMergeDaily( name, possessions )

   assert( name == "dailyStreak" );

   --Daily time Stamp is not merge anymore
   --self._data.dailyTimeStamp4;

   --server is further... us that instead
   if ( self:getValue('dailyStreak') or 0 ) < (possessions:getValue('dailyStreak') or 0) then
      self._data.dailyStreak      =  possessions:getValue('dailyStreak');
      self._data.dailyProgress    =  possessions:getValue('dailyProgress');
      self._data.dailyGamePlayed  =  possessions:getValue('dailyGamePlayed');
   else
      --For simplicity reason, if we are at least on the same day streak, keep local
      --We understand we might loose small progress but it is okay
   end

end

-------------------------------------------------------------------------------------

function AppData:_mergePossessionsLevel( name, possessions )

   assert( name == "currentLevel" );

   if (self:getValue('currentLevel') or 0) < (possessions:getValue('currentLevel') or 0) then
      self._data.currentLevel    = possessions:getValue('currentLevel');
      self._data.xp              = possessions:getValue('xp');
      self._data.levelGamePlayed = possessions:getValue('levelGamePlayed');
   else
      --For simplicity reason, if we are at least on the same day streak, keep local
      --We understand we might loose small progress but it is okay
   end

end

-------------------------------------------------------------------------------------

function AppData:_mergePossessionsAchievement( possessions )

   local achvList = TemplateMgr:getAchievement();
   if achvList then

      for i,achv in ipairs(achvList) do

         local progId   = self:createAchievemetProgressId(achv.id);
         local rankId   = self:createAchievemetRankId(achv.id);

         local rankP    = possessions:getValue(rankId) or 0;
         local progP    = possessions:getValue(progId) or 0;
         local rankD    = self:getValue(rankId) or 0;
         local progD    = self:getValue(progId) or 0;

         --If the server is higher rank, use that
         if rankP > rankD then

            self._data[rankId] = rankP;
            self._data[progId] = progP;

         elseif rankP == rankD  then

            if progP < 0 or progD < 0 then
               self._data[progId] = -1;
            else
               self._data[progId] = math.max(progP,progD);
            end
         end
      end
   end

end

-------------------------------------------------------------------------------------

function AppData:_mergePossessionsMission( name, possessions )

   assert( name == "missionIndex" );

   if (self:getValue("missionIndex") or 1) < (possessions:getValue("missionIndex") or 1) then

      self._data.missionIndex    = (possessions:getValue("missionIndex") or 1);

      for i=1,MISSION_GOAL_COUNT do
         local id = self:createMissionGoalProgressId(i);
         self._data[id]       = possessions:getValue(id) or  0;
         id = self:createMissionGoalGamePlayedId(i);
         self._data[id]       = possessions:getValue(id) or  0;
      end

   else
      --For simplicity reason, if we are at least on the same day streak, keep local
      --We understand we might loose small progress but it is okay
   end

end

-------------------------------------------------------------------------------------


-- If the types are different, take the server's data
-- Otherwise, if the rarities are different, take the higher rarity and its level
-- Otherwise, take the maximum level.
function AppData:_mergePossessionsDrone( name, possessions )

   local serverData = possessions:getValue(name);
   local localData = self:getValue(name);

   if serverData and localData then

      local serverType, serverRarity, serverLevel = DroneMgr:decodeDroneData(serverData);
      local localType, localRarity, localLevel = DroneMgr:decodeDroneData(localData);

      if serverType ~= localType then

         self._data[name] = serverData;

      elseif serverRarity ~= localRarity then

         if serverRarity > localRarity then
            self._data[name] = serverData;
         else
            self._data[name] = localData;
         end

      else

         if serverLevel > localLevel then
            self._data[name] = serverData;
         else
            self._data[name] = localData;
         end

      end

   else
      self._data[name] = serverData or localData;
   end

end

-------------------------------------------------------------------------------------

function AppData:_mergeAllPossessions( possessions )

   if possessions and self._builtinInfo and self._data then

      for i,tbl in ipairs(self._builtinInfo) do
         if tbl.server and tbl.merge then
            tbl.merge( self, tbl.name, possessions );
         end
      end

      self:_mergePossessionsAchievement(possessions);

      --This will ensure to not have undesigned nil value which could crash some part of the game
      self:_setDefaultValueIfNeeded();

      --Make sure to handle and adjust values related to timeStamp
      self:_updateDaily();
      self:_regenKeys();

      --Make sure the current car is usable... you never know (we could reset)
      if not self:isPlayerVehicleUnlocked(self._data.currentPlayerVehicle) then
         self._data.currentPlayerVehicle = 1;
      end

      --Make sure the current paint is usable...you never know (we could reset)
      self._data.currentPlayerVehiclePaintVariant = self._data.currentPlayerVehiclePaintVariant or 1;
      if not self:isPlayerVehiclePaintVariantUnlocked( self._data.currentPlayerVehicle, self._data.currentPlayerVehiclePaintVariant ) then
         self._data.currentPlayerVehiclePaintVariant = 1;
      end

      EventMgr:postEvent({}, "data-server-merged");

      return true;
   end

end

----------------------------------------------------------------------

function AppData:_reset()

   self._data = {};

   -- TransactionMgr shouldn't normally be reset.
   TransactionMgr:_reset();

   self:_setDefaultValueIfNeeded();

end

----------------------------------------------------------------------

function AppData:hackReset( gem, gold, key )

   if LoginMgr:isTango() and OnlineMgr:isOnline() then

      if PossessionMgr:getLastDownloadTime() and PossessionMgr:getPossessions() then

         --Not sure the purpose of this code... it was making reset failing very frequently
        -- if PossessionMgr:getLastDownloadTime() ~= self._possessionTime then

            if not TransactionMgr:isSyncing() then

               self:_reset();

               -- Quick way to give us resources
               self._data.gold = gold or self._data.gold;
               self._data.gem  = gem  or self._data.gem;
               self._data.key  = key  or self._data.key;

               self:_uploadEverythingToServer();

               return true;

            end
         --end
      end
   end
end

----------------------------------------------------------------------

function AppData:createBackup()

   -- Put specialized backup code here

   return self:getValuesClone();

end

----------------------------------------------------------------------

function AppData:restoreFromBackup(backup)

   -- Put specialized restore code here

   self:setDataValues(backup);

end

----------------------------------------------------------------------

function AppData:getValuesClone()
   return copyTable(self._data);
end

----------------------------------------------------------------------

function AppData:setDataValues(newData)
   self._data = newData;
end

----------------------------------------------------------------------

function AppData:_onPlayerVehicleUnlocked( vehicle_index )

   --Make sure the first vehicle is unlock... could be done in the group logic as well
   self._data[self:createMainWeaponId(vehicle_index)]                  = math.max(self._data[self:createMainWeaponId(vehicle_index)]    or 1, 1);
   self._data[self:createSideWeaponId(vehicle_index)]                  = math.max(self._data[self:createSideWeaponId(vehicle_index)]    or 0, 0);
   self._data[self:createTurretWeaponId(vehicle_index)]                = math.max(self._data[self:createTurretWeaponId(vehicle_index)]  or 0, 0);
   self._data[self:createArmorId(vehicle_index)]                       = math.max(self._data[self:createArmorId(vehicle_index)]         or 1, 1);
   self._data[self:createPlayerVehiclePaintVariantId(vehicle_index,1)] = 1;

end

----------------------------------------------------------------------

function AppData:getControlSensitivity()

   return self._data.controlSensitivity;

end

----------------------------------------------------------------------

function AppData:getGlobalVolume()

   return self._data.globalVolume2;

end

----------------------------------------------------------------------

function AppData:setControlSensitivity( val )

   self._data.controlSensitivity = math.clamp( val, 0, 1 );

   self:_saveDataSoon();

end

----------------------------------------------------------------------

function AppData:setGlobalVolume( val, save )

   self._data.globalVolume2 = math.clamp( val, 0, 1 );

   self:_saveDataSoon();

end

----------------------------------------------------------------------

function AppData:spendGem( amount, type )

   local t = DataTransactionSpendGem:new( self, amount, type );

   return TransactionMgr:execTransaction( t );

end


----------------------------------------------------------------------

function AppData:getDailyChallengeTemplate( day )

   local tpl =  TemplateMgr:getDailyChallenge();

   if tpl then

		day = day or (self._data.dailyStreak+1);
      return tpl.day[math.clamp(day,1,#tpl.day)];

   end
end

----------------------------------------------------------------------

function AppData:isDailyActive()

   if HACK_DAILY_DISABLED then
      return false;
   end

   if not self:isDailyUnlocked() then
      return false;
   end

   return self._data.dailyProgress>= 0;

end

----------------------------------------------------------------------

function AppData:getDailyChallengeReward(day)

   local settings =     AbTestMgr:getRewardInfoTable("daily-challenge-reward")
                     or self:getValue("App.settings.daily-challenge-reward");

   if settings then

      local day2   = day or (self._data.dailyStreak+1);
      local day3   = math.clamp(day2, 1, #settings);
      local reward = settings[day3];

      if reward then
         return reward;
      end

   end

   local tpl = self:getDailyChallengeTemplate(day);

   return tpl.reward;

end

----------------------------------------------------------------------

function AppData:getDailyChallengeRewardToCollect()

   local gem  = self._data.dailyRewardGem  or 0;
   local gold = self._data.dailyRewardGold or 0;
   if gem > 0 or gold > 0 then

      return { gem  = self._data.dailyRewardGem
             , gold = self._data.dailyRewardGold  };

   end

   --if there is no daily, return nil or dont return anything, some logic depend on that (i.e. isDailyUnlocked)

end

----------------------------------------------------------------------

function AppData:getDailyChallengeGoal(day)

   local tpl = self:getDailyChallengeTemplate(day);

   return tpl.goal;

end

----------------------------------------------------------------------

function AppData:getDailyChallengeProgress()

   return self._data.dailyProgress;

end

----------------------------------------------------------------------

function AppData:getDailyChallengeStreak()

   return self._data.dailyStreak;

end

----------------------------------------------------------------------

function AppData:onDailyPickup()

   local t = DataTransactionDailyPickup:new( self );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:collectDailyReward()

   local t = DataTransactionDailyCollectReward:new( self );

   return TransactionMgr:execTransaction( t );

end


-------------------------------------------------------------------------------------

function AppData:getDailyReferenceTime()

   return TimeMgr:getTimeGMT();

end

-------------------------------------------------------------------------------------

function AppData:_initDailyTimeStamp()

   local time = self:getDailyReferenceTime();
   if time then

      local delta = 0;
      local hour  = tonumber(os.date("%H", time )) + tonumber(os.date("%M", time ))/60;
		local delta	= 0;

      if hour < DAILY_TARGET_RESET then
         delta =  (( 24 - (DAILY_TARGET_RESET - hour) )*60*60);
      elseif hour > DAILY_TARGET_RESET then
         delta =  (( (hour - DAILY_TARGET_RESET) )*60*60);
      end

      local time2 = time - delta;

      MOAIFlurry:event( "Daily-Time-Init-2",
                        "current-time",      tostring(os.date("%H", time ))      .. ":" .. tostring(os.date("%M", time   )),
                        "reset-hour",        tostring(os.date("%H", time2 ))     .. ":" .. tostring(os.date("%M", time2  )),
                        "delta",             tostring(os.date("%H", delta))      .. ":" .. tostring(os.date("%M", delta  )),
                        "dailyTimeStamp4",   tostring(time),
                        "streak",            tostring(self._data.dailyStreak )   );


      self._data.dailyTimeStamp4  = time2;
      self:_saveDataSoon();

   end
end

-------------------------------------------------------------------------------------

function AppData:_updateDaily()

   local time = self:getDailyReferenceTime();
   --[[
    print ("[AppData:_updateDaily() :"
    .."\ntime: "..tostring(time)
    .."\nself._data.dailyTimeStamp4: "..tostring(self._data.dailyTimeStamp4 or "nil")
    );
    ]]--

   if time then

  	 if self._data.dailyTimeStamp4 == nil or  time < self._data.dailyTimeStamp4 then
  	    --print("self:_initDailyTimeStamp()");
      	self:_initDailyTimeStamp();
  	 end

      local delta = (time-self._data.dailyTimeStamp4);
      --print("[AppData:_updateDaily() delta: "..tostring(delta));

      --The day is over
      if delta > DAILY_ONE_DAY  then
         --print("[AppData:_updateDaily() delta (" ..tostring(delta).. ") > DAILY_ONE_DAY ("..tostring(DAILY_ONE_DAY)..")");

         local streakBackup = self._data.dailyStreak or 0;

         --Make sure the player collected his reward
         --Michel: disable until we get a cool animation of getting reward in Daily
     --    if self:getDailyChallengeRewardToCollect() == nil then

            -- if we didnt succeeded last time or more than 1 day passed by
            if self._data.dailyProgress and self._data.dailyProgress >=0 or delta >= 2*DAILY_ONE_DAY then

               --Because we are changing the way our timer work, which could have people loosing a streak
               --Give them a free save
               if self:getValue("Daily-Free-Save-1") == nil then
                  self:setValue("Daily-Free-Save-1", true);
               else
                  self._data.dailyStreak = 0;
               end
            end

            --Skip to today
            while (time-self._data.dailyTimeStamp4) >= DAILY_ONE_DAY do
               self._data.dailyTimeStamp4 = self._data.dailyTimeStamp4 + DAILY_ONE_DAY;
            end

            self._data.dailyProgress = 0;

            self:_saveDataSoon();

            --missed daily event
            if streakBackup > 0 and self._data.dailyStreak==0 and self._data.dailyProgress == 0 then
               EventMgr:postEvent( {progress=self._data.dailyProgress}, "daily-failed" );
            end


         end
   --   end

   end
end

----------------------------------------------------------------------

function AppData:getMissionIndex()

   return self._data.missionIndex;

end

----------------------------------------------------------------------

function AppData:getMissionGoalCount()

   return MISSION_GOAL_COUNT;

end

----------------------------------------------------------------------

function AppData:onMissionGoalCompleted( index )

   if index >= 1 and index <=  MISSION_GOAL_COUNT then

      self:updateMissionGoal(  self._data.missionIndex, index, -1 );

   end

   return true;

end

----------------------------------------------------------------------

function AppData:isCurrentMissionCompleted()

   for i=1,MISSION_GOAL_COUNT do
      local progress = self._data[self:createMissionGoalProgressId(i)];
      if (not progress) or (progress >= 0) then
         return false;
      end
   end

   return true;

end

----------------------------------------------------------------------

function AppData:getCurrentMissionGoalProgress( index )

   return self._data[self:createMissionGoalProgressId(index)];

end

-------------------------------------------------------------------------------------

function AppData:createMissionGoalGamePlayedId( index )
   return "missionGoal-GP" .. index;
end

----------------------------------------------------------------------

function AppData:isCurrentMissionGoalCompleted( index )

   local progress = self._data[self:createMissionGoalProgressId(index)];
   if (not progress) or (progress >= 0) then
      return false;
   end
   return true;

end

----------------------------------------------------------------------

function AppData:getMissionTemplate( index )

   return TemplateMgr:getMission( index);

end

----------------------------------------------------------------------

function AppData:getCurrentMissionTemplate( )

   return TemplateMgr:getMission( self._data.missionIndex );

end

----------------------------------------------------------------------

function AppData:getDailyChallengeTime()

   return self._data.dailyTimeStamp4;

end

----------------------------------------------------------------------

function AppData:getDailyChallengeTimer()

   local time = self:getDailyReferenceTime();

   if time and self._data.dailyTimeStamp4 then
      return DAILY_ONE_DAY-(time-self._data.dailyTimeStamp4);
   end
end

----------------------------------------------------------------------

function AppData:getGoldCount()

     return 99999999;

end

----------------------------------------------------------------------

function AppData:getGemCount()

     return 99999999;

end
----------------------------------------------------------------------

function AppData:getFreebieCount( type )

   if (type == "revive") or (type == "head-start") or (type == "power-time") or (type == "double-score") or (type == "mystery")  then

      return self._data["freebie-"..type];

   else

      Debug.assert( (type == "revive") or (type == "head-start") or (type == "power-time") or (type == "double-score")  or (type == "mystery") );

   end

end

----------------------------------------------------------------------

function AppData:giveFreebie( type, amount, reason )

   if (type == "revive") or (type == "head-start") or (type == "power-time") or (type == "double-score") or (type == "mystery")  then

      local t = DataTransactionAddFreebie:new( self, "freebie-"..type, amount or 1, reason or "unknown");

      return TransactionMgr:execTransaction( t );

   else

      Debug.assert( (type == "revive") or (type == "head-start") or (type == "power-time") or (type == "double-score")  or (type == "mystery") );

   end


end


----------------------------------------------------------------------

function AppData:spendFreebie( type, amount, reason )

   if (type == "revive") or (type == "head-start") or (type == "power-time") or (type == "double-score")  or (type == "mystery")  then

      local t = DataTransactionSpendFreebie:new( self, "freebie-"..type, amount or 1, reason or "unknown");

      return TransactionMgr:execTransaction( t );

   else

      Debug.assert( (type == "revive") or (type == "head-start") or (type == "power-time") or (type == "double-score")  or (type == "mystery") );

   end

end

----------------------------------------------------------------------

function AppData:canUpgradeItem(  itemId, costGold, costGem  )

   --TODO: Check if the item is maxed out

   return      self._data.gold >= costGold
          and  self._data.gem  >= costGem;

end

----------------------------------------------------------------------

function AppData:getKeyCount()
   return self._data.key or 0;
end

----------------------------------------------------------------------

function AppData:getKeyTimeStamp()
   return self._data.keyTimeStamp;
end

----------------------------------------------------------------------

function AppData:_getKeyRegenTime()

   return KEY_REGEN_TIME_DEFAULT;

end

----------------------------------------------------------------------

function AppData:getKeyMaxRegen()
   return KEY_MAX_REGEN;
end

----------------------------------------------------------------------

function AppData:getKeyMaxReward()
   return KEY_MAX_REWARD;
end

----------------------------------------------------------------------

function AppData:getInviteRewardGold()
   return INVITE_REWARD_GOLD;
end

----------------------------------------------------------------------

function AppData:calcFullKeyRegenTime()

   local numKeys   = self:getKeyCount();
   local toRegen = KEY_MAX_REGEN-numKeys;

   if  toRegen > 0  then

      local regenTime = self:_getKeyRegenTime();
      local curentTimer = self:calcKeyTimeToRegen();

      if regenTime and curentTimer then
         local othersTime  = math.max(toRegen-1, 0 )*regenTime;

         return curentTimer + othersTime;
      end
   end

end

----------------------------------------------------------------------

function AppData:calcKeyTimeToRegen()

   if self._data.key < KEY_MAX_REGEN then

      local time = self:getKeyReferenceTime();

      if time and self._data.keyTimeStamp then
         return math.max(self:_getKeyRegenTime()-(time-self._data.keyTimeStamp),1);
      end
   end
end

-------------------------------------------------------------------------------------

function AppData:getKeyReferenceTime()

   local time = TimeMgr:getTimeGMT();
   if time then
      return time + KEY_REFERENCE_TIME_OFFSET;
   end

end

-------------------------------------------------------------------------------------

function AppData:setRegenKeyEnable( val )

   self._regenKeyEnable  = val;

end

-------------------------------------------------------------------------------------

function AppData:_regenKeys()

   local time = self:getKeyReferenceTime();

   if self._regenKeyEnable and time then

      if self._data.key < KEY_MAX_REGEN then

         --If someone mess with their clock... or something weird happen
         -- fix the time
         if (time-self._data.keyTimeStamp)< 0 then
            self._data.keyTimeStamp    = time;
            self:_saveDataSoon();
         end

         local regenTime = self:_getKeyRegenTime();

         while       ((time-self._data.keyTimeStamp) > regenTime)
               and   (self._data.key < KEY_MAX_REGEN)                             do

            self._data.key             = self._data.key + 1;
            self._data.keyTimeStamp    = math.floor(self._data.keyTimeStamp + regenTime);
            self:_saveDataSoon();

            EventMgr:postEvent({}, "key-regen");

         end

      else

         -- if we are fully regenerated, invalidate our stamp
         if ( self._data.keyTimeStamp or -1 ) > 0 then
            self._data.keyTimeStamp    = -1;
            self:_saveDataSoon();
         end

      end
   end

end

----------------------------------------------------------------------

function AppData:getGamePlayed()

   return self._data.gamePlayed or 0;

end

----------------------------------------------------------------------

function AppData:isThief()

   return ((self._data["iap-fake"] or 0) > 0) or (self._data["thief-3"]);

end

----------------------------------------------------------------------

function AppData:getPlayerVehicleCount()

   return math.min(TemplateMgr:getPlayerVehicleCount(), self:getValue("App.settings.player-vehicle-count", TemplateMgr:getPlayerVehicleCount()));

end

----------------------------------------------------------------------

function AppData:getPlayerVehicleShowableCount()

   local tbl   =  AbTestMgr:getVehicleInfoTable("player-vehicle-showableLevels")
               or self:getValue("App.settings.player-vehicle-showableLevels");
   local lvl   = self:getCurrentPlayerLevel();

   if tbl then

      local count = 1;
      for i,v in pairs(tbl) do

         --Follow the table unless the user unlocked a car further along the line
         -- it should be impossible but in case we change some of the value
         if (lvl >= v) or (self:isPlayerVehicleUnlocked(i) )then
            count = i;
         end
      end

      return math.min(count, self:getPlayerVehicleCount());

   else

      Debug.error("AppData:getPlayerVehicleShowableCount -> tbl");

   end

   return self:getPlayerVehicleCount();

end

----------------------------------------------------------------------

function AppData:getCurrentPlayerVehicle()

   return self._data.currentPlayerVehicle;

end

----------------------------------------------------------------------

function AppData:getHighestPlayerVehicleUnlocked()

   for i=self:getPlayerVehicleCount(),1,-1 do
		if self:isPlayerVehicleUnlocked(i) then
			return i;
		end
	end

	return 1;

end

----------------------------------------------------------------------

function AppData:setCurrentPlayerVehicle( vehicle_index )

   if vehicle_index and vehicle_index >= 1 and vehicle_index <= self:getPlayerVehicleCount() then

      if self:isPlayerVehicleUnlocked( vehicle_index ) then

         self._data.currentPlayerVehicle = vehicle_index;

         if not self:isPlayerVehiclePaintVariantUnlocked(vehicle_index, self:getCurrentPlayerVehiclePaintVariant() ) then

            self:setCurrentPlayerVehiclePaintVariant( 1 )

         end
      end
   end

end

----------------------------------------------------------------------

function AppData:isPlayerVehicleUnlocked( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      local mainWeaponLevel = self:getPlayerMainWeaponLevel(vehicle_index);

      return mainWeaponLevel and mainWeaponLevel > 0;

   else

      return false;

   end
end

----------------------------------------------------------------------

function AppData:getPlayerVehicleTemplate( vehicle_index )

   return TemplateMgr:getPlayerVehicle( vehicle_index );

end

----------------------------------------------------------------------

function AppData:getCurrentPlayerVehicleTemplate( vehicle_index )

   return TemplateMgr:getPlayerVehicle( self:getCurrentPlayerVehicle() );

end

----------------------------------------------------------------------

function AppData:getPlayerVehicleUnlockCost( vehicle_index )

   if not self:isPlayerVehicleUnlocked(vehicle_index) then

      local playerLevel    = self:getCurrentPlayerLevel()                        or 1;
      local unlockLevel    = self:getVehicleUnlockableLevel(vehicle_index)       or 100000;
      local earlyLevel     = self:getVehicleEarlyUnlockableLevel(vehicle_index)  or 100000;

      if playerLevel >= unlockLevel then

         local onSale, saleIndex, saleID, salePrice = self:isPlayerVehicleOnSale( vehicle_index );

         if onSale and salePrice then

            --TODO: rename to .car
            return { gem=salePrice.unlock };

         else

            local costTable = self:getVehicleUnlockCostTable(vehicle_index);

            if costTable then

               return { gem=costTable.car };
            end

         end

      elseif playerLevel >= earlyLevel then

         local costTable = self:getVehicleEarlyUnlockCostTable(vehicle_index);

         if costTable then

            return { gem=costTable.car };

         end
      end
   end

   local tpl = self:getPlayerVehicleTemplate( vehicle_index );

   if tpl then
      return tpl.cost;
   end

   return { gold=0, gem=0 };

end

----------------------------------------------------------------------

function AppData:canUnlockPlayerVehicle( vehicle_index )

   if not self:isPlayerVehicleUnlocked(vehicle_index) then

      local cost     = self:getPlayerVehicleUnlockCost( vehicle_index );

      local costGold = cost.gold or 0;
      local costGem  = cost.gem  or 0;

      if       costGold <= self:getGoldCount()
         and   costGem  <= self:getGemCount()  then
         return true;
      end

   end

   return false;

end

----------------------------------------------------------------------

function AppData:getCurrentPlayerMainWeaponMaxDPS()

	local main	= TemplateMgr:getPlayerVehicleMainWeapon( self:getCurrentPlayerVehicle() );
	local lvl	= self:getPlayerMainWeaponMaxLevel( self:getCurrentPlayerVehicle() );

	if main and lvl then
		local tpl	= main.level[lvl];

		if tpl then
			Debug.assert( tpl.delay and tpl.delay > 0  );

			return tpl.projectile.damage/( math.max( tpl.delay or 0, 0.1 ) );
		end
	end

	return 0;

end

----------------------------------------------------------------------

function AppData:calcCurrentPlayerDPS( mainK, sideK, turretK, droneK )

   local tplMain = self:getCurrentPlayerMainWeaponTemplate();

   local mainDPS = 0;
   local sideDPS = 0;
   local turretDPS = 0;
	local droneLDPS	= 0;
	local droneRDPS	= 0;

   if tplMain then

      local tplProj = tplMain.projectile;

      Debug.assert( tplMain.delay and tplMain.delay > 0  );

      mainDPS = tplProj.damage/(math.max(tplMain.delay or 0, 0.10));

   end

   local tplSide = self:getCurrentPlayerSideWeaponTemplate();
   if tplSide then

      tplProj = tplSide.projectile;

      Debug.assert( tplSide.delay and tplSide.delay > 0 );

      sideDPS = tplProj.damage/(math.max(tplSide.delay or 0, 0.10));

   end

   local tplTurret = self:getCurrentPlayerTurretWeaponTemplate();

   if tplTurret then

      tplProj   = tplTurret.projectile;

      if tplProj then

         Debug.assert(tplTurret.bulletCount and tplTurret.bulletCount > 0);

         --Turret are a little more complicated
         local dmg  = tplTurret.bulletCount * tplProj.damage;
         local time = tplTurret.bulletCount * tplTurret.bulletDelay  + (tplTurret.shootDelay or 0);

         Debug.assert( time > 0 );

         turretDPS =  dmg / time;

      end
   end

	local droneL	= DroneMgr:isDroneMountLeftAssigned();
	local droneR	= DroneMgr:isDroneMountRightAssigned();

	if droneL then
		local tpl, visData, stats	= DroneMgr:getDroneTemplateLeft(droneL);

		if tpl and tpl.projectile and tpl.projectile.damage then
			local rarity	= DroneMgr:getDroneRarityLeft(droneL);
			local level		= DroneMgr:getDroneLevelLeft(droneL);
			local base		= stats.rarity[rarity];
			local mod		= stats.level[level];

			if type(base) ~= "number" then
				base	= 1;
			end

			if type(mod) ~= "number" then
				mod	= 0;
			end

			droneLDPS	= tpl.projectile.damage * App.data:getCurrentPlayerMainWeaponMaxDPS() / ( base - mod );
		end
	end

	if droneR then
		local tpl, visData, stats	= DroneMgr:getDroneTemplateRight(droneR);

		if tpl and tpl.projectile and tpl.projectile.damage then
			local rarity	= DroneMgr:getDroneRarityRight(droneL);
			local level		= DroneMgr:getDroneLevelRight(droneR);
			local base		= stats.rarity[rarity];
			local mod		= stats.level[level];

			if type(base) ~= "number" then
				base	= 1;
			end

			if type(mod) ~= "number" then
				mod	= 0;
			end

			droneRDPS	= tpl.projectile.damage * App.data:getCurrentPlayerMainWeaponMaxDPS() / ( base - mod );
		end
	end

   return   mainDPS  * ( mainK   or 1 )
         + sideDPS   * ( sideK   or 1 )
         + turretDPS * ( turretK or 1 )
			+ (droneLDPS + droneRDPS) * ( droneK or 1 );

end

----------------------------------------------------------------------

function AppData:consumeKey()

   local t = DataTransactionConsumeKey:new( self );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:addKey( amount, type )

   local t = DataTransactionAddKey:new( self, amount, type );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:unlockPlayerVehicle( vehicle_index )

   local t = DataTransactionUnlockVehicle:new( self, vehicle_index );

   local onSale, saleIndex, saleID = self:isPlayerVehicleOnSale( vehicle_index );

   if onSale and saleID then

      local saleIDList = self:getValue("vehicle-sale-id-list") or {};

      saleIDList[saleID] = true;

      saveLocalFile(VEHICLE_SALE_ID_FILENAME, saleIDList);
      self:setValue("vehicle-sale-id-list", saleIDList);

   end

   if TransactionMgr:execTransaction( t ) then

      local turretCost = self:getPlayerTurretWeaponUpgradeCost( vehicle_index ) or {};

      if ((turretCost.gold or 0) == 0) and ( (turretCost.gem or 0)==0) then

         local t2 = DataTransactionUpgradeTurretWeapon:new( self, vehicle_index );

         TransactionMgr:execTransaction( t2 );

      end

      local sideCost = self:getPlayerSideWeaponUpgradeCost( vehicle_index ) or {};

      if ((sideCost.gold or 0) ==0) and ((sideCost.gem or 0)==0) then

         local t3 = DataTransactionUpgradeSideWeapon:new( self, vehicle_index );

         TransactionMgr:execTransaction( t3 );

      end

      return true;

   end

end

----------------------------------------------------------------------

function AppData:_getCurrentMainWeaponId()
   return self:createMainWeaponId(self:getCurrentPlayerVehicle());
end

----------------------------------------------------------------------

function AppData:isPlayerMainWeaponUnlocked( vehicle_index )

   local mainWeaponLevel = self:getPlayerMainWeaponLevel(vehicle_index);
   return mainWeaponLevel and mainWeaponLevel > 0;

end

----------------------------------------------------------------------

function AppData:getPlayerMainWeaponLevel( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      return self._data[self:createMainWeaponId(vehicle_index)];

   end

   return 0;

end

----------------------------------------------------------------------

function AppData:getCurrentMainWeaponLevel()

   return self:getPlayerMainWeaponLevel( self:getCurrentPlayerVehicle() );

end

----------------------------------------------------------------------

function AppData:getPlayerMainWeaponMaxLevel( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      local mainWeaponTpl   = TemplateMgr:getPlayerVehicleMainWeapon(vehicle_index);

      if mainWeaponTpl then

         return #mainWeaponTpl.level;

      end

   end

   return 0;

end

----------------------------------------------------------------------

function AppData:getPlayerMainWeaponTemplate( vehicle_index )

   local mainWeaponTpl   = TemplateMgr:getPlayerVehicleMainWeapon(vehicle_index);

   local level           = math.clamp(self:getPlayerMainWeaponLevel(vehicle_index),1,#mainWeaponTpl.level);

    return mainWeaponTpl.level[level];

end

----------------------------------------------------------------------

function AppData:getCurrentPlayerMainWeaponTemplate()

   return self:getPlayerMainWeaponTemplate( self:getCurrentPlayerVehicle() );

end

----------------------------------------------------------------------

function AppData:getPlayerMainWeaponUpgradeCost( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      if self:isPlayerVehicleUnlocked(vehicle_index) then

         local level           = self:getPlayerMainWeaponLevel(vehicle_index);

         local mainWeaponTpl   = TemplateMgr:getPlayerVehicleMainWeapon(vehicle_index);

         if level ~= nil and level < #mainWeaponTpl.level then

            local tpl   = mainWeaponTpl.level[level+1];

            if tpl then

               return tpl.cost;

            end
         end
      end
   end

   --Maxed out  or invalid parameters
   return nil;

end

----------------------------------------------------------------------

function AppData:canUpgradePlayerMainWeapon( vehicle_index )

   local cost = self:getPlayerMainWeaponUpgradeCost( vehicle_index );

   if cost then

      local costGold = cost.gold or 0;
      local costGem  = cost.gem  or 0;

      if       costGold <= self:getGoldCount()
         and   costGem  <= self:getGemCount()  then

         return true;

      end
   end

   return false;

end

----------------------------------------------------------------------

function AppData:upgradePlayerMainWeapon( vehicle_index )

   local t = DataTransactionUpgradeMainWeapon:new( self, vehicle_index );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:_getCurrentArmorId()
   return self:createArmorId(self:getCurrentPlayerVehicle());
end

----------------------------------------------------------------------

function AppData:isPlayerArmorUnlocked( vehicle_index )

   local armorLevel = self:getPlayerArmorLevel(vehicle_index);
   return armorLevel and armorLevel > 0;

end

----------------------------------------------------------------------

function AppData:getPlayerArmorLevel( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then
      return self._data[self:createArmorId(vehicle_index)];
   end

   return 0;

end

----------------------------------------------------------------------

function AppData:getCurrentArmorLevel()

    return self:getPlayerArmorLevel(self:getCurrentPlayerVehicle());

end

----------------------------------------------------------------------

function AppData:getPlayerArmorMaxLevel( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      local armorTpl    = TemplateMgr:getPlayerVehicleArmor( vehicle_index );

      return #armorTpl.level;

   end

   return 0;

end

----------------------------------------------------------------------

function AppData:getPlayerArmorTemplate( vehicle_index )

   local armorTpl = TemplateMgr:getPlayerVehicleArmor(vehicle_index);

   if armorTpl then

      local level = math.clamp(self:getPlayerArmorLevel(vehicle_index),1,#armorTpl.level);

      return armorTpl.level[level];

   end

end

----------------------------------------------------------------------

function AppData:getCurrentPlayerVehicleHPMin()

	local main	= TemplateMgr:getPlayerVehicleArmor( self:getCurrentPlayerVehicle() );

	if main then
		local tpl	= main.level[1];

		if tpl and tpl.hitPoints then
			return tpl.hitPoints;
		end
	end

	return 0;

end

----------------------------------------------------------------------

function AppData:getCurrentPlayerVehicleHPMax()

	local main	= TemplateMgr:getPlayerVehicleArmor( self:getCurrentPlayerVehicle() );
	local lvl	= self:getPlayerArmorMaxLevel( self:getCurrentPlayerVehicle() );

	if main and lvl then
		local tpl	= main.level[lvl];

		if tpl and tpl.hitPoints then
			return tpl.hitPoints;
		end
	end

	return 0;

end

----------------------------------------------------------------------

function AppData:getCurrentArmorTemplate()

    return self:getPlayerArmorTemplate(self:getCurrentPlayerVehicle());

end

----------------------------------------------------------------------

function AppData:getPlayerArmorUpgradeCost( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      if self:isPlayerVehicleUnlocked(vehicle_index) then

         local level      = self:getPlayerArmorLevel(vehicle_index);
         local armorTpl   = TemplateMgr:getPlayerVehicleArmor(vehicle_index);

         if level and level < #armorTpl.level then

            local tpl = armorTpl.level[level+1];

            if tpl then

               return tpl.cost;

            end
         end
      end
   end

   --Maxed out  or invalid parameters
   return nil;

end

----------------------------------------------------------------------

function AppData:canUpgradePlayerArmor( vehicle_index )

   local cost = self:getPlayerArmorUpgradeCost( vehicle_index );

   if cost then

      local costGold = cost.gold or 0;
      local costGem  = cost.gem  or 0;

      if       costGold <= self:getGoldCount()
         and   costGem  <= self:getGemCount()  then


         return true;

      end
   end

   return false;

end

----------------------------------------------------------------------

function AppData:upgradePlayerArmor( vehicle_index )

   local t = DataTransactionUpgradeArmor:new( self, vehicle_index );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:_getCurrentSideWeaponId()
   return self:createSideWeaponId(self:getCurrentPlayerVehicle());
end

----------------------------------------------------------------------

function AppData:isPlayerSideWeaponUnlocked( vehicle_index )

   local sideWeaponLevel = self:getPlayerSideWeaponLevel(vehicle_index);
   return sideWeaponLevel and sideWeaponLevel > 0;

end

----------------------------------------------------------------------

function AppData:getPlayerSideWeaponLevel( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      return self._data[self:createSideWeaponId(vehicle_index)];

   end

   return 0;

end

----------------------------------------------------------------------

function AppData:getCurrentSideWeaponLevel()

   return self:getPlayerSideWeaponLevel( self:getCurrentPlayerVehicle() );

end

----------------------------------------------------------------------

function AppData:getPlayerSideWeaponMaxLevel( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      local sideWeaponTpl = TemplateMgr:getPlayerVehicleSideWeapon(vehicle_index);

      if sideWeaponTpl then
         return #sideWeaponTpl.level;
      end

   end

   return 0;

end

----------------------------------------------------------------------

function AppData:getPlayerSideWeaponTemplate( vehicle_index )

   local sideWeaponTpl   = TemplateMgr:getPlayerVehicleSideWeapon(vehicle_index);

   local level           = math.clamp(self:getPlayerSideWeaponLevel(vehicle_index),0,#sideWeaponTpl.level);

   if level > 0 then
      return sideWeaponTpl.level[level];
   end
end

----------------------------------------------------------------------

function AppData:getCurrentPlayerSideWeaponTemplate( )

    return self:getPlayerSideWeaponTemplate(self:getCurrentPlayerVehicle());

end

----------------------------------------------------------------------

function AppData:getPlayerSideWeaponUnlockCost( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      local level           = self:getPlayerSideWeaponLevel(vehicle_index);

      if level == 0  then

         local playerLevel    = self:getCurrentPlayerLevel()                        or 1;
         local unlockLevel    = self:getVehicleUnlockableLevel(vehicle_index)       or 100000;
         local earlyLevel     = self:getVehicleEarlyUnlockableLevel(vehicle_index)  or 100000;

         if playerLevel >= unlockLevel then

            local onSale, saleIndex, saleID, salePrice  = self:isVehicleSideWeaponOnSale( vehicle_index );

            if onSale and salePrice and type(salePrice) == "number" then

               return { gem=salePrice };

            else

               local costTable = self:getVehicleUnlockCostTable(vehicle_index);
               if costTable then
                  return { gem=costTable.side };
               end

            end

         elseif playerLevel >= earlyLevel then

            local costTable = self:getVehicleEarlyUnlockCostTable(vehicle_index);

            if costTable then
               return { gem=costTable.side };

            end
         end

         local sideWeaponTpl   = TemplateMgr:getPlayerVehicleSideWeapon(vehicle_index);

         if sideWeaponTpl and sideWeaponTpl.level then

            local sideWeaponTpl2   = sideWeaponTpl.level[level+1];

            if sideWeaponTpl2 then

               return sideWeaponTpl2.cost;

            end
         end
      end
   end

   return { gold=0, gem=0 };

end

----------------------------------------------------------------------

function AppData:getPlayerSideWeaponUpgradeCost( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      local level           = self:getPlayerSideWeaponLevel(vehicle_index);

      local sideWeaponTpl   = TemplateMgr:getPlayerVehicleSideWeapon(vehicle_index);

      if level ~= nil and level < #sideWeaponTpl.level then

         --Unlock cost
         if level == 0 then

            return self:getPlayerSideWeaponUnlockCost( vehicle_index );

         end

         local tpl   = sideWeaponTpl.level[level+1];

         if tpl then

            return tpl.cost;

         end
      end
   end

   --Maxed out  or invalid parameters
   return nil;

end

----------------------------------------------------------------------

function AppData:canUpgradePlayerSideWeapon( vehicle_index )

   local cost = self:getPlayerSideWeaponUpgradeCost( vehicle_index );

   if cost then

      local costGold = cost.gold or 0;
      local costGem  = cost.gem  or 0;

      if       costGold <= self:getGoldCount()
         and   costGem  <= self:getGemCount()  then


         return true;

      end
   end

   return false;

end

----------------------------------------------------------------------

function AppData:upgradePlayerSideWeapon( vehicle_index )

   local t = DataTransactionUpgradeSideWeapon:new( self, vehicle_index );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:isPlayerTurretWeaponUnlocked( vehicle_index )

   local turretLevel = self:getPlayerTurretWeaponLevel(vehicle_index);
   return turretLevel and turretLevel > 0;

end

----------------------------------------------------------------------

function AppData:getPlayerTurretWeaponLevel( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      return self._data[self:createTurretWeaponId(vehicle_index)];

   end

   return 0;

end

----------------------------------------------------------------------

function AppData:getCurrentPlayerTurretWeaponLevel()

   return self:getPlayerTurretWeaponLevel( self:getCurrentPlayerVehicle() );

end

----------------------------------------------------------------------

function AppData:getPlayerTurretWeaponMaxLevel( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      local turretWeaponTpl   = TemplateMgr:getPlayerVehicleTurretWeapon(vehicle_index);

      if turretWeaponTpl then

         return #turretWeaponTpl.level;

      end

   end

   return 0;end

----------------------------------------------------------------------

function AppData:getPlayerTurretWeaponTemplate( vehicle_index )

   local turretWeaponTpl   = TemplateMgr:getPlayerVehicleTurretWeapon(vehicle_index);

   local level           = math.clamp(self:getPlayerTurretWeaponLevel(vehicle_index),0,#turretWeaponTpl.level);

   if level > 0 then
      return turretWeaponTpl.level[level];
   end

end


----------------------------------------------------------------------

function AppData:getCurrentPlayerTurretWeaponTemplate()

    return self:getPlayerTurretWeaponTemplate(self:getCurrentPlayerVehicle());

end

----------------------------------------------------------------------

function AppData:getCurrentPlayerTurretWeaponLevel()

   return self:getPlayerTurretWeaponLevel( self:getCurrentPlayerVehicle() );

end


----------------------------------------------------------------------

function AppData:getPlayerTurretWeaponUnlockCost( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      local level           = self:getPlayerTurretWeaponLevel(vehicle_index);

      if level == 0 then

         local playerLevel    = self:getCurrentPlayerLevel()                        or 1;
         local unlockLevel    = self:getVehicleUnlockableLevel(vehicle_index)       or 100000;
         local earlyLevel     = self:getVehicleEarlyUnlockableLevel(vehicle_index)  or 100000;

         if playerLevel >= unlockLevel then

            local onSale, saleIndex, saleID, salePrice  = self:isVehicleTurretOnSale( vehicle_index );
            if onSale and salePrice and type(salePrice) == "number" then

               return { gem=salePrice };

            else

               local costTable = self:getVehicleUnlockCostTable(vehicle_index);

               if costTable then
                  return { gem=costTable.turret };
               end

            end

         elseif playerLevel >= earlyLevel then

            local costTable = self:getVehicleEarlyUnlockCostTable(vehicle_index);

            if costTable then
               return { gem=costTable.turret };

            end
         end

         local turretWeaponTpl   = TemplateMgr:getPlayerVehicleTurretWeapon(vehicle_index);

         if turretWeaponTpl and turretWeaponTpl.level then

            local turretWeaponTpl2   = turretWeaponTpl.level[level+1];

            if turretWeaponTpl2 then

               return turretWeaponTpl2.cost;

            end
         end
      end

   end

   return { gold=0, gem=0 };

end


----------------------------------------------------------------------

function AppData:getPlayerTurretWeaponUpgradeCost( vehicle_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      if self:isPlayerVehicleUnlocked(vehicle_index) then

         local level			= self:getPlayerTurretWeaponLevel(vehicle_index);
         local turretTpl	= TemplateMgr:getPlayerVehicleTurretWeapon(vehicle_index);

         if level ~= nil and level < #turretTpl.level then

            if level == 0 then

               return self:getPlayerTurretWeaponUnlockCost(vehicle_index);

            end

            local tpl   = turretTpl.level[level+1];

            if tpl then

               return tpl.cost;

            end
         end
      end
   end

   --Maxed out  or invalid parameters
   return nil;

end

----------------------------------------------------------------------

function AppData:canUpgradePlayerTurretWeapon( vehicle_index )

   local cost = self:getPlayerTurretWeaponUpgradeCost( vehicle_index );

   if cost then

      local costGold = cost.gold or 0;
      local costGem  = cost.gem  or 0;

      if       costGold <= self:getGoldCount()
         and   costGem  <= self:getGemCount()  then


         return true;

      end
   end

   return false;

end

----------------------------------------------------------------------

function AppData:upgradePlayerTurretWeapon( vehicle_index )

   local t = DataTransactionUpgradeTurretWeapon:new( self, vehicle_index );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:_getPlayerVehicleLocalPaintIndex()

	if not self._localPlayerVehcPaint then
		self._localPlayerVehcPaint = self:_loadPlayerVehicleLocalPaintIndex();
	end

	return self._localPlayerVehcPaint;

end

----------------------------------------------------------------------

function AppData:_loadPlayerVehicleLocalPaintIndex()

	local tbl = loadLocalFile( VEHICLE_PAINT_ID_FILENAME );

	if not tbl then
		tbl = {};

		for i=1,self:getPlayerVehicleCount() do
			table.insert(tbl, 1);
		end

		self:_savePlayerVehicleLocalPaintIndex(tbl);
	end

	return tbl;

end

----------------------------------------------------------------------

function AppData:_savePlayerVehicleLocalPaintIndex( tbl )

	saveLocalFile( VEHICLE_PAINT_ID_FILENAME, tbl );

end

----------------------------------------------------------------------

function AppData:getPlayerVehiclePaintVariant( vID )

	local tbl				= self:_getPlayerVehicleLocalPaintIndex();
	local vehc				= math.clamp(vID or self:getCurrentPlayerVehicle(), 1, self:getPlayerVehicleCount());
	local paint_index		= tbl[vehc];
	local isCurVehc		= vehc == self:getCurrentPlayerVehicle();

	if not paint_index then
		tbl[vehc] = 1;

		self:_savePlayerVehicleLocalPaintIndex(tbl);

		return 1;
	elseif paint_index >= 1 and paint_index <= self:getPlayerVehiclePaintVariantCount(vehc) then

		if isCurVehc and tbl[vehc] ~= self._data.currentPlayerVehiclePaintVariant then
			if self:isPlayerVehiclePaintVariantUnlocked(vehc, tbl[vehc]) then
				self._data.currentPlayerVehiclePaintVariant = tbl[vehc];
			else
				tbl[vehc] = self._data.currentPlayerVehiclePaintVariant;
			end

			self:_savePlayerVehicleLocalPaintIndex(tbl);
		end

		return tbl[vehc];
	end

	return self._data.currentPlayerVehiclePaintVariant;

end

----------------------------------------------------------------------

function AppData:getCurrentPlayerVehiclePaintVariant()

	local paint = self:getPlayerVehiclePaintVariant();

   return paint;

end

----------------------------------------------------------------------

function AppData:setCurrentPlayerVehiclePaintVariant( paint_index )

   local curVeh = self:getCurrentPlayerVehicle();

   if     paint_index ~= nil
      and paint_index >= 1
      and paint_index <= self:getPlayerVehiclePaintVariantCount( curVeh ) then

      if self:isPlayerVehiclePaintVariantUnlocked( curVeh, paint_index ) then
			local tbl	= self:_getPlayerVehicleLocalPaintIndex();

			tbl[curVeh]	= paint_index;
         self._data.currentPlayerVehiclePaintVariant = paint_index;

			self:_savePlayerVehicleLocalPaintIndex(tbl);
      end
   end

end

----------------------------------------------------------------------

function AppData:_getCurrentPlayerVehiclePaintVariantId()

   return self:createPlayerVehiclePaintVariantId( self:getCurrentPlayerVehicle()
                                                ,  self:getCurrentPlayerVehiclePaintVariant() );

end

----------------------------------------------------------------------

function AppData:getPlayerVehiclePaintVariantCount( vehicle_index )

   local tplPaint = TemplateMgr:getPlayerVehiclePaint( vehicle_index );

   return #tplPaint.variant;

end

----------------------------------------------------------------------

function AppData:isPlayerVehiclePaintVariantUnlocked( vehicle_index, paint_index )

   if self:isPlayerVehicleUnlocked( vehicle_index ) then

      --Paint index 1 is always unlocked... if the vehicle is unlocked
      if paint_index == 1 then
         return true;
      else

         local id = self:createPlayerVehiclePaintVariantId( vehicle_index, paint_index );

         return self._data[id] and self._data[id] > 0 ;

      end

   end

   return false;

end

----------------------------------------------------------------------

function AppData:getPlayerVehiclePaintVariantTemplate( vehicle_index, paint_index  )

   local paintTpl   = TemplateMgr:getPlayerVehiclePaint(vehicle_index);

   return paintTpl.variant[paint_index];

end

----------------------------------------------------------------------

function AppData:getCurrentPlayerVehiclePaintVariantTemplate()

   return self:getPlayerVehiclePaintVariantTemplate(   self:getCurrentPlayerVehicle(),
                                                       self:getCurrentPlayerVehiclePaintVariant() );

end

----------------------------------------------------------------------

function AppData:getPlayerVehiclePaintVariantUnlockCost( vehicle_index, paint_index )

   if vehicle_index and vehicle_index > 0 and vehicle_index <= self:getPlayerVehicleCount() then

      if self:isPlayerVehicleUnlocked( vehicle_index ) then

         local paintTpl = TemplateMgr:getPlayerVehiclePaint( vehicle_index );

         if paintTpl ~= nil and paint_index and paint_index <= #paintTpl.variant  then

            local tpl = paintTpl.variant[paint_index];

            if tpl then

               return tpl.cost;

            end
         end
      end
   end

   --Maxed out  or invalid parameters
   return nil;

end

----------------------------------------------------------------------

function AppData:canUnlockPlayerVehiclePaintVariant( vehicle_index, paint_index )

   local cost = self:getPlayerVehiclePaintVariantUnlockCost( vehicle_index, paint_index );

   if cost then

      local costGold = cost.gold or 0;
      local costGem  = cost.gem  or 0;

      if       costGold <= self:getGoldCount()
         and   costGem  <= self:getGemCount()  then

         return true;

      end
   end

   return false;

end

----------------------------------------------------------------------

function AppData:unlockPlayerVehiclePaintVariant( vehicle_index, paint_index )

   local t = DataTransactionUnlockPaint:new( self, vehicle_index, paint_index );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:addConsumable( index, fCountID, price, autoActivate, freeUse )

   local t = DataTransactionAddConsumable:new( self, index, fCountID, price, autoActivate, freeUse );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:_createConsumableDefaultData( index )

   return { name     = self:createConsumableId(index),
            default  = 0,
            critical = true,
            server   = true,
            merge    = self._possessionMergeMaxValue, };

end

----------------------------------------------------------------------

function AppData:createConsumableId( index )

   return "consumable_"..index.."_inventory";

end

----------------------------------------------------------------------

function AppData:consumeConsumable( index, freeUse )

   local t = DataTransactionConsumeConsumable:new( self, index, freeUse );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:getPickupCount()
   return TemplateMgr:getPickupCount();
end

----------------------------------------------------------------------

function AppData:isPickupUnlocked( pickup_index )

   return true;

end

----------------------------------------------------------------------

function AppData:getPickupLevel( pickup_index )

   if pickup_index == PICKUP_COIN_10_INDEX then
      pickup_index = PICKUP_COIN_INDEX;
   end

   if pickup_index and pickup_index > 0 and pickup_index <= self:getPickupCount() then

      return self._data[self:createPickupId(pickup_index)];

   end

   return 0;

end

----------------------------------------------------------------------

function AppData:getPickupMaxLevel( pickup_index )

   if pickup_index == PICKUP_COIN_10_INDEX then
      pickup_index = PICKUP_COIN_INDEX;
   end

   if pickup_index and pickup_index > 0 and pickup_index <= self:getPickupCount() then

      local pickupTpl   = TemplateMgr:getPickup(pickup_index);

      if pickupTpl and pickupTpl.level then
         return #pickupTpl.level;
      end

   end

   return 0;

end

----------------------------------------------------------------------

function AppData:getPickupTemplate( pickup_index )

   local pickupTpl   = TemplateMgr:getPickup(pickup_index);

   if pickupTpl and pickupTpl.level then

      local level = math.clamp(self:getPickupLevel(pickup_index),0,#pickupTpl.level);

      if level > 0 then

         return pickupTpl.level[level];

      end
   end
end

----------------------------------------------------------------------

function AppData:getPickupUpgradeCost( pickup_index )

   if pickup_index and pickup_index > 0 and pickup_index <= self:getPickupCount() then

      local level       = self:getPickupLevel(pickup_index);
      local pickupTpl   = TemplateMgr:getPickup(pickup_index);

      if pickupTpl and pickupTpl.level then

         if level ~= nil and level < #pickupTpl.level  then

            local tpl = pickupTpl.level[level+1];

            if tpl then
               return tpl.cost;
            end
         end
      end
   end

   --Maxed out  or invalid parameters
   return nil;

end

----------------------------------------------------------------------

function AppData:canPickupBeUpgraded( pickup_index )
   return self:getPickupLevel(pickup_index) < self:getPickupMaxLevel(pickup_index);
end

----------------------------------------------------------------------

function AppData:canAffordUpgradePickup( pickup_index )

   local cost = self:getPickupUpgradeCost( pickup_index );

   if cost then

      local costGold = cost.gold or 0;
      local costGem  = cost.gem  or 0;

      if       costGold <= self:getGoldCount()
         and   costGem  <= self:getGemCount()  then

         return true;

      end
   end

   return false;

end

----------------------------------------------------------------------

function AppData:upgradePickup( pickup_index )

   local t = DataTransactionUpgradePickup:new( self, pickup_index );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:updateGameStats( goldCollected, gameDuration, distance )

   local t = DataTransactionUpdateGameStats:new( self, goldCollected, gameDuration, distance );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:updateAchievementProgress( achvIdx, progress )

   local t = DataTransactionUpdateAchievementProgress:new( self, achvIdx, progress );

   return TransactionMgr:execTransaction( t );

end
----------------------------------------------------------------------

function AppData:getAchievementProgress( achvIdx )

   return self._data[self:createAchievemetProgressId(achvIdx)];

end

----------------------------------------------------------------------

function AppData:nextAchievementRank( achvIdx, reward  )

   local rank = (self:getAchievementRank(achvIdx) or 0 ) + 1;

   local t = DataTransactionUpdateAchievementRank:new( self, achvIdx, rank, reward );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:getAchievementRank( achvIdx )

   return self._data[self:createAchievemetRankId(achvIdx)];

end

----------------------------------------------------------------------

function AppData:isAchievementUnlocked()

   local level = self:getCurrentPlayerLevel();

   if level < 1 then
      return false;
   end


   return TemplateMgr:isAchievementUnlocked(level);

end

----------------------------------------------------------------------

function AppData:getAchievementUnlockLevel()

   return TemplateMgr:getAchievementUnlockLevel();

end

----------------------------------------------------------------------

function AppData:getMissionSkipCost( missionIndex )

   local tpl  = self:getMissionTemplate(missionIndex);
   if tpl then
      return tpl.skipCost;
   end

end

----------------------------------------------------------------------

function AppData:getMissionReward( missionIndex )

   local settings =     AbTestMgr:getRewardInfoTable("mission-reward")
                     or self:getValue("App.settings.mission-reward");

   if settings then

      local index  = math.min(missionIndex, #settings);
      local reward = settings[index];

      if reward then
         return reward;
      end

   end


   local tpl  = self:getMissionTemplate(missionIndex);
   if tpl then
      return tpl.reward;
   end

end

----------------------------------------------------------------------

function AppData:getCurrentMissionReward(  )

   return self:getMissionReward(self._data.missionIndex);

end

----------------------------------------------------------------------

function AppData:canSkipMission( missionIndex )

   local tpl  = self:getMissionTemplate(missionIndex);
   local cost = (tpl or {}).skipCost;

   if tpl.skipCost then

      local costGold = cost.gold or 0;
      local costGem  = cost.gem  or 0;

      if      costGold > self:getGoldCount()
         or   costGem  > self:getGemCount()  then

         return false;
      end

   end

   return true;

end

----------------------------------------------------------------------

function AppData:canSkipCurrentMission( )

   return self:canSkipMission(self._data.missionIndex);

end

----------------------------------------------------------------------

function AppData:nextMission( skipped )

   return self:updateMission(  self._data.missionIndex+1, skipped );

end

----------------------------------------------------------------------

function AppData:skipCurrentMission()

   return self:updateMission(  self._data.missionIndex+1, true )

end

----------------------------------------------------------------------

function AppData:updateMission( missionIndex, skipped )

   local t = DataTransactionUpdateMission:new( self, missionIndex, skipped );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:updateMissionGoal( missionIndex, goalIndex, progress )

   local t = DataTransactionUpdateMissionGoal:new( self, missionIndex, goalIndex, progress );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:updateCurrentMissionGoal( goalIndex, progress )

   return self:updateMissionGoal( self._data.missionIndex, goalIndex, progress );

end

----------------------------------------------------------------------

function AppData:addGem( amount, reason )

   local t = DataTransactionAddGem:new( self, amount, reason );

   if reason == "store" then

      t.alwaysExecute = true;
   end

   return TransactionMgr:execTransaction( t );
end

----------------------------------------------------------------------

function AppData:addGold( amount, reason )

   local t = DataTransactionAddGold:new( self, amount, reason );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:inviteReward( destination, gold, gem )

   local t = DataTransactionInvite:new( self, destination, gold, gem );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:getTangoBonus()

   local bonus;

   if MOAITangoSdk:tangoHasSdkSupport() then
      bonus = AbTestMgr:getAuthenticationRewardTangoInstalled();
   else
      bonus	= AbTestMgr:getAuthenticationRewardTangoNotInstalled();
   end

   if bonus == nil then

      local settings =     AbTestMgr:getRewardInfoTable("tango-reward")
                        or self:getValue("App.settings.tango-reward")
                        or {};

      if MOAITangoSdk:tangoHasSdkSupport() then
         bonus  = (settings["tango-installed"] or {}).gem;
      else
         bonus  = (settings["tango-not-installed"] or {}).gem;
      end

   end

   return bonus;

end

----------------------------------------------------------------------

function AppData:hasReceiveTangoBonus()

   return      (self._data.signingGold or 0) > 0
            or (self._data.signingGem  or 0) > 0;

end

----------------------------------------------------------------------

function AppData:receiveTangoBonus(  )

   local t = DataTransactionTangoBonus:new( self, 0, self:getTangoBonus() );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------

function AppData:onIapVerificationAccepted( event )

   local t = DataTransactionIapVerify:new( self, "verified" );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------

function AppData:onIapVerificationDeclined( event )

   local t = DataTransactionIapVerify:new( self, "fake" );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------

function AppData:onIapVerificationFailure( event )

   local t = DataTransactionIapVerify:new( self, "failure" );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:getStoreGoldTemplate( index )

   return TemplateMgr:getStoreGold( index);

end

----------------------------------------------------------------------

function AppData:getStoreGoldCost( index )

   local cost = AbTestMgr:getStoreGoldCost( index );

   if type(cost) == "number" then
      return { gem = cost };
   else

      local tpl  = self:getStoreGoldTemplate(index);
      if tpl then
         return tpl.cost;
      end
   end
end

----------------------------------------------------------------------

function AppData:getStoreGoldAmount( index )

   local amount = AbTestMgr:getStoreGoldAmount( index );

   if type(amount) == "number" then
      return amount;
   else
      local tpl  = self:getStoreGoldTemplate(index);
      if tpl then
         return tpl.amount;
      end
   end

end

----------------------------------------------------------------------

function AppData:canBuyStoreGold( index )

   local cost = self:getStoreGoldCost( index );

   if  cost then

      local costGold    = cost.gold or 0;
      local costGem     = cost.gem  or 0;

      if       costGold <= self:getGoldCount()
         and   costGem  <= self:getGemCount()  then

         return true;

      end
   end

   return false;

end

----------------------------------------------------------------------

function AppData:buyStoreGold( index )

   local t = DataTransactionBuyGold:new( self, index );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:getStoreKeyTemplate( index )

   return TemplateMgr:getStoreKey( index);

end

----------------------------------------------------------------------

function AppData:getStoreKeyCost( index )

   local abVal = AbTestMgr:getStoreKeyCost( index );

   if type(abVal) == "number" then
      return { gem = abVal };
   else

      local tpl  = self:getStoreKeyTemplate(index);
      if tpl then
         return tpl.cost;
      end
   end

end

----------------------------------------------------------------------

function AppData:getStoreKeyAmount( index )

   local abVal = AbTestMgr:getStoreKeyAmount( index );

   if type(abVal) == "number" then
      return abVal;
   else

      local tpl  = self:getStoreKeyTemplate(index);
      if tpl then
         return tpl.amount;
      end
   end

end

----------------------------------------------------------------------

function AppData:canBuyStoreKey( index )

   local cost = self:getStoreKeyCost( index );

   if  cost then

      local costGold    = cost.gold or 0;
      local costGem     = cost.gem  or 0;

      if       costGold <= self:getGoldCount()
         and   costGem  <= self:getGemCount()  then
         return true;
      end
   end

   return false;

end

----------------------------------------------------------------------

function AppData:buyStoreKey( index )

   local t = DataTransactionBuyKey:new( self, index );

   return TransactionMgr:execTransaction( t );

end

-------------------------------------------------------------------------------------

function AppData:createAchievemetProgressId( achvId )

   Debug.assert( achvId, "AppData:createAchievemetProgressId -> achvId" );

   return "achv-" .. tostring(achvId) .. "-prog";

end

-------------------------------------------------------------------------------------

function AppData:createAchievemetRankId( achvId )

   Debug.assert( achvId, "AppData:createAchievemetRankId -> achvId" );

   return "achv-" .. tostring(achvId) .. "-rank";

end

-------------------------------------------------------------------------------------

function AppData:createAchievemetGpId( achvId )

   Debug.assert( achvId, "AppData:createAchievemetGpId -> achvId" );

   return "achv-" .. tostring(achvId) .. "-gp";

end

-------------------------------------------------------------------------------------

function AppData:createMissionGoalProgressId( index )
   return "missionGoal" .. index;
end

-------------------------------------------------------------------------------------

function AppData:createMainWeaponId( vehicle_index )
   return "veh" .. vehicle_index.. "mw";
end

-------------------------------------------------------------------------------------

function AppData:createArmorId( vehicle_index )
   return "veh" .. vehicle_index .. "armor";
end

-------------------------------------------------------------------------------------

function AppData:createSideWeaponId( vehicle_index )
   return "veh" .. vehicle_index.. "sw";
end

-------------------------------------------------------------------------------------

function AppData:createTurretWeaponId( vehicle_index )
   return "veh" .. vehicle_index.. "turret";
end

-------------------------------------------------------------------------------------

function AppData:createPlayerVehiclePaintVariantId( vehicle_index, paint_index )
   return "veh" .. vehicle_index .. "paint" .. paint_index;
end

-------------------------------------------------------------------------------------

function AppData:createPickupId( pickup_index )
   return "pickup" .. pickup_index;
end

----------------------------------------------------------------------

function AppData:getValue( name, default )

   local val = self._data[name];

   if val == nil then
      val = default;
   end

   return val;

end

----------------------------------------------------------------------

function AppData:isCriticalExist( )

   return self._critical;

end

----------------------------------------------------------------------

function AppData:isValidKeyName( name )

   if not Debug.assertType(name, "string", "AppData:isValidKeyName -> name") then
      return false;
   end

   local invalidChars = { "." };

   for _, badChar in ipairs(invalidChars) do
      if string.find(name, badChar, 1, true) then
         print("WARNING: AppData: Bad character \"" .. tostring(badChar) .. "\" in name: " .. tostring(name));
         return false;
      end
   end

   return true;
end

----------------------------------------------------------------------

function AppData:getValueFromCritical( name, default )

   -- Check for valid keyname for prints; discard result
   self:isValidKeyName(name);

   if self._critical then

      local val = self._critical[name];

      if val == nil then
         val = default;
      end

      return val;

   end

end

----------------------------------------------------------------------

function AppData:setValue( name, value )

   -- Check for valid keyname for prints; discard result
   self:isValidKeyName(name);

   self._data[name] = value;

   self:_saveDataSoon();

   return value;

end

----------------------------------------------------------------------

function AppData:setValues( values )

   if not Debug.assertType(values, "table", "AppData:setValues -> values") then
      return 0;
   end

   local count = 0;

   for name, v in pairs(values) do

      self:setValue(name, v);

      count = count + 1;
   end

   self:_saveDataSoon();

   return count;
end

----------------------------------------------------------------------

function AppData:deltaValue( name, diff )

   -- Check for valid keyname for prints; discard result
   self:isValidKeyName(name);

   if self._data[name] then
      self._data[name] = self._data[name] + diff;
   else
      self._data[name] = diff;
   end

   self:_saveDataSoon();

   return self._data[name];
end

----------------------------------------------------------------------

function AppData:getAutoKeyBuyIndex( reqAmount )

	local delta	 = reqAmount - self:getKeyCount();

	for i=1,6 do
		if self:getStoreKeyAmount(i) >= delta then

			return i;
		end
	end

end

----------------------------------------------------------------------

function AppData:getAutoCoinBuyIndex( reqAmount )

	local delta	 = reqAmount - self:getGoldCount();

	for i=1,6 do
		if self:getStoreGoldAmount(i) >= delta then

			return i;
		end
	end

   return 6;

end

----------------------------------------------------------------------

function AppData:getSpawnMatrixDimensions( )

	local vehicle = self:getCurrentPlayerVehicle();

	return TemplateMgr:getSpawnMatrixDimensions(vehicle);

end

----------------------------------------------------------------------

function AppData:getPlayerWeaponLevel( )

   local mainLvl        = self:getCurrentMainWeaponLevel()         or 0;
   local sideLvl        = self:getCurrentSideWeaponLevel()         or 0;
   local turretLvl      = self:getCurrentPlayerTurretWeaponLevel() or 0;

   return math.max( math.max(mainLvl, sideLvl), turretLvl);

end

----------------------------------------------------------------------

function AppData:getSpawnList( level, dps )

	local vehicle				= self:getCurrentPlayerVehicle();
	local h, w					= self:getSpawnMatrixDimensions();
	local levelBand			= self:getLevelBand();
	local levelIndex			= math.ceil(math.clamp(level/levelBand, 1, h));

	return TemplateMgr:getSpawnList(vehicle, levelIndex, math.min(dps,w));

end

----------------------------------------------------------------------

function AppData:getGameDirectorSettings( level )

	local vehicle				= self:getCurrentPlayerVehicle();
	local h, w					= self:getSpawnMatrixDimensions();
	local levelBand			= self:getLevelBand();
	local levelIndex			= math.ceil(math.clamp(level/levelBand, 1, h));

	return TemplateMgr:getGameDirectorSettings(vehicle, levelIndex);

end

----------------------------------------------------------------------

function AppData:getGameDistanceScore( level )

	local vehicle				= self:getCurrentPlayerVehicle();
	local h, w					= self:getSpawnMatrixDimensions();
	local levelBand			= self:getLevelBand();
	local levelIndex			= math.ceil(math.clamp(level/levelBand, 1, h));

	return TemplateMgr:getGameDistanceScore(vehicle, levelIndex);

end

----------------------------------------------------------------------

function AppData:getMaxPlayerLevel( )

--	return TemplateMgr:getMaxPlayerLevel();

	return MAX_PLAYER_LEVEL;

end

----------------------------------------------------------------------

function AppData:getLevelBand()

	return LEVEL_BAND;

end

----------------------------------------------------------------------

function AppData:getCurrentPlayerLevel( )

	local level = self._data.currentLevel or 1;

	return level;

end

----------------------------------------------------------------------

function AppData:setPlayerLevel( level, xp )

	local lvl = level or self:getCurrentPlayerLevel();
	local exp = xp or 0;
	local max = self:getMaxPlayerLevel();

	if lvl > max then

		lvl = max;
	end

   local t = DataTransactionUpdateLevel:new( self, lvl, exp );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:canIncreasePlayerLevel( xp )

	local level		= self:getCurrentPlayerLevel();
	local exp 		= self:getCurrentPlayerXP();
	local totalXP	= exp + (xp or 0);

	if (level + 1) > self:getMaxPlayerLevel() then

		return false;
	end

	if totalXP >= self:getXPLevelReq(level + 1) then

		return true;
	else

		return false;
	end

end

----------------------------------------------------------------------

function AppData:getCurrentPlayerXP()

	local xp		= self._data.xp or 0;

	return xp;

end

----------------------------------------------------------------------

function AppData:getXPLevelReq( level )

	local lvl = level or (self:getCurrentPlayerLevel() + 1);
   local  k  = AbTestMgr:getValue("xp_level_req_k") or 1;

	return k*TemplateMgr:getXPLevelReq(lvl);

end

----------------------------------------------------------------------

function AppData:addXP( xp )

	local exp = xp or 0;
   local t = DataTransactionAddXP:new( self, exp );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:getCurrentRewardLevel( )

	local level = self._data.rewardLevel or 1;

	return level;

end

----------------------------------------------------------------------

function AppData:addRewardLevel( )

   local t = DataTransactionAddRewardLevel:new( self );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:getLevelReward( level )

   level = level or self:getCurrentPlayerLevel();

	if (level) and (level > 0) and (level <= self:getMaxPlayerLevel()) then

		local t	= TemplateMgr:getLevelReward(level);
		local u	= self:getVehicleUnlockableTable();

      if u then
         local c	= 1;

         for i=1,#u do
            if u[i] <= level then
               c = i;
            end
         end

         t.unlockVehc = c;

         local settings    =     AbTestMgr:getRewardInfoTable("level-reward")
                              or self:getValue("App.settings.level-reward")
                              or {};
         local mySettings  = settings[level] or {};

         if #settings > 0 and level > #settings then
            local mod   = level % 10;
            local index = #settings - 10 + mod;
            if mod == 0 then
               index = #settings;
            end

            mySettings  = settings[index] or {};
         end

         t.key    = mySettings.key  or t.key;
         t.gem    = mySettings.gem  or t.gem;
         t.gold   = mySettings.gold or t.gold;

      end

		return t;
	end

end

----------------------------------------------------------------------

function AppData:updateRewardLevel()

	local delta	 = self:getCurrentPlayerLevel() - self:getCurrentRewardLevel();

	if delta > 0 then
		for i=1,delta do
			self:addRewardLevel( );
		end
	end

end


----------------------------------------------------------------------

function AppData:isMissionUnlocked()

   --If we are pass mission 1, we have played mission before
   if self:getMissionIndex() > 1 then
      return true;
   end

   --If we had any progress with a mission, it means it was unlocked at some point
   for i=1,MISSION_GOAL_COUNT do
      local progress = self._data[self:createMissionGoalProgressId(i)];
      if (progress) and (progress  ~= 0) then
         return true;
      end
   end

	local level = self:getCurrentPlayerLevel();

	if level < 1 then
      return false;
	end

   return TemplateMgr:isMissionUnlocked(level);

end

----------------------------------------------------------------------

function AppData:getMissionUnlockLevel()

	return TemplateMgr:getMissionUnlockLevel();

end

----------------------------------------------------------------------

function AppData:isConsumableUnlocked()

	return self:getCurrentPlayerLevel() >= TemplateMgr:getConsumableUnlockLevel();

end

----------------------------------------------------------------------

function AppData:getConsumableUnlockLevel()

	return TemplateMgr:getConsumableUnlockLevel();

end


----------------------------------------------------------------------

function AppData:getMysteryUnlockLevel()

   return TemplateMgr:getMysteryUnlockLevel();

end


----------------------------------------------------------------------

function AppData:isMysteryUnlocked()

   return self:getCurrentPlayerLevel() >= self:getMysteryUnlockLevel();

end

----------------------------------------------------------------------

function AppData:isDailyUnlocked()


   if    ( self:getDailyChallengeStreak()   > 0 )
      or ( self:getDailyChallengeProgress() > 0 )
      or ( self:getDailyChallengeRewardToCollect() ~= nil )  then
      return true;
   end

	local level = self:getCurrentPlayerLevel();

	if level < 1 then
		return false;
	end

	return TemplateMgr:isDailyUnlocked(level);

end

----------------------------------------------------------------------

function AppData:getDailyUnlockLevel()

	return TemplateMgr:getDailyUnlockLevel();

end

----------------------------------------------------------------------

function AppData:isFriendsUnlocked()

	local level = self:getCurrentPlayerLevel();

	if level < 1 then
		return false;
	end

	return TemplateMgr:isFriendsUnlocked(level)

end


----------------------------------------------------------------------

function AppData:getFriendsUnlockLevel()

	return TemplateMgr:getFriendsUnlockLevel();

end

----------------------------------------------------------------------

function AppData:isLeagueUnlocked()

	local level = self:getCurrentPlayerLevel();

	if level < 1 then
		return false;
	end

	return TemplateMgr:isLeagueUnlocked(level)

end


----------------------------------------------------------------------

function AppData:getLeagueUnlockLevel()

	return TemplateMgr:getLeagueUnlockLevel();

end

----------------------------------------------------------------------

function AppData:isBountyUnlocked()

   if HACK_BOUNTY_ALWAYS_UNLOCKED then
      return true;
   end

	local level = self:getCurrentPlayerLevel();

	if level < 1 then
		return false;
	end

	return TemplateMgr:isBountyUnlocked(level)

end


----------------------------------------------------------------------

function AppData:getBountyUnlockLevel()

	return TemplateMgr:getBountyUnlockLevel();

end

----------------------------------------------------------------------

function AppData:isReviveUnlocked()


	local level = self:getCurrentPlayerLevel();

	if level < 1 then
		return false;
	end

	return TemplateMgr:isReviveUnlocked(level);

end

----------------------------------------------------------------------

function AppData:getReviveUnlockLevel()

	return TemplateMgr:getReviveUnlockLevel();

end

----------------------------------------------------------------

function AppData:getMaxRevive()

   return AbTestMgr:getReviveMax() or #TEMPLATE_REVIVE_COST;

end

----------------------------------------------------------------

function AppData:getReviveCost( index )

   local cost = AbTestMgr:getReviveCost(  index  );

   if cost then

      return cost;

   else

      Debug.assert( index and index >=1 and index <= #TEMPLATE_REVIVE_COST  );

      return TEMPLATE_REVIVE_COST[math.clamp(index or 1, 1, #TEMPLATE_REVIVE_COST)];

   end

end

----------------------------------------------------------------------

function AppData:isBossUnlocked()

	local level = self:getCurrentPlayerLevel();

	if level < 1 then
		return false;
	end

	return TemplateMgr:isBossUnlocked(level);

end

----------------------------------------------------------------------

function AppData:getBossUnlockLevel()

	return TemplateMgr:getBossUnlockLevel();

end

----------------------------------------------------------------------

function AppData:isPlayerVehicleLevelLocked( index )

	if not index then
		return false;
	end

	local level	= self:getCurrentPlayerLevel();

	local t		= self:getVehicleUnlockableTable();

	if (not t) or (not t[index]) then
		return false;
    end

    if t[index] > level then
       return true;
    end

end


----------------------------------------------------------------------

function AppData:showPlayerVehicleLevelGuide()


	local level	= self:getCurrentPlayerLevel();
	local t		= self:getVehicleUnlockGuideTable();

	if t then

      local highest = nil;
      local maxCars = self:getPlayerVehicleCount();
      if maxCars then
      local coundDown = math.min(#t, maxCars);
         for i = coundDown, 1, -1 do

            if not self:isPlayerVehicleLevelLocked(i) and level >= t[i] then

               local id = "newCarIndex"..i.."shown"
               local shown = self:getValue(id);
               if not shown then

                  local unlocked = self:isPlayerVehicleUnlocked(i);
                  if unlocked then
                     self:setValue(id, true);
                  end
                  if not highest then
                     highest = i;
                  else
                     self:setValue(id, true);
                  end
               end

            end
         end
      end
      return highest;
	end

end

----------------------------------------------------------------------

function AppData:getVehicleUnlockableTable()

    local abTstMgrTable = AbTestMgr:getVehicleInfoTable("player-vehicle-unlockableLevels");
    local selfTable = self:getValue("App.settings.player-vehicle-unlockableLevels");

   return      abTstMgrTable
               or selfTable
               or { 1, 6,11,16,21,26,31,36,41,46, 51 };

end

----------------------------------------------------------------------

function AppData:getVehicleUnlockableLevel( index )

   if not index then
      return 1;
   end

   local t = self:getVehicleUnlockableTable();

   if t then
      if index > #t then
         return 1;
      else
         return t[index];
      end
   end

   return 1;


end


----------------------------------------------------------------------

function AppData:getVehicleEarlyUnlockableTable()

   return   AbTestMgr:getVehicleInfoTable("player-vehicle-earlyUnlockableLevels")
         or self:getValue("App.settings.player-vehicle-earlyUnlockableLevels");

end

----------------------------------------------------------------------

function AppData:getVehicleEarlyUnlockableLevel( vehIdx )

   local tbl =  self:getVehicleEarlyUnlockableTable();
   if tbl then

      return tbl[vehIdx];

   end

end

----------------------------------------------------------------------

function AppData:getVehicleUnlockCostTable( vehIdx )

   local tbl =    AbTestMgr:getVehicleInfoTable("player-vehicle-cost")
               or self:getValue("App.settings.player-vehicle-cost");
   if tbl then

      return tbl[vehIdx];

   end

end

----------------------------------------------------------------------

function AppData:getVehicleEarlyUnlockCostTable( vehIdx )

   local tbl =    AbTestMgr:getVehicleInfoTable("player-vehicle-early-cost")
               or self:getValue("App.settings.player-vehicle-early-cost");
   if tbl then

      return tbl[vehIdx];

   end


end

----------------------------------------------------------------------

function AppData:getVehicleUnlockGuideTable()

   return      AbTestMgr:getVehicleInfoTable("player-vehicle-guideLevels")
            or self:getValue("App.settings.player-vehicle-guideLevels");

end
----------------------------------------------------------------------

function AppData:isPlayerVehicleMk2( index )

	if index and index >= FIRST_MK2_VEHICLE_INDEX then
		return true;
	end

end

----------------------------------------------------------------------

function AppData:isPlayerVehicleOnSale( index )

	local count = self:getPlayerVehicleCount();

	if	index and index > 0 and	index <= count then

		local saleList	= self:_getPlayerVehicleSaleList();

		if saleList and #saleList > 0 then
			for i=1,#saleList do
				if			saleList[i].vehcID <= count
					and	not self:isPlayerVehicleUnlocked(saleList[i].vehcID)
					and	index == saleList[i].vehcID then

					return true, i, saleList[i].saleID, saleList[i].price;
				end
			end
		end
	end

end

----------------------------------------------------------------------

function AppData:isVehicleTurretOnSale( index )

	local count = self:getPlayerVehicleCount();

	if	index and index > 0 and	index <= count then

		local saleList	= self:_getPlayerVehicleSaleList();

		if saleList and #saleList > 0 then
			for i=1,#saleList do
				if			saleList[i].vehcID <= count
					and	index == saleList[i].vehcID then
               if saleList[i].price and saleList[i].price.turret and saleList[i].price.turret > 0 then
                  return true, i, saleList[i].saleID, saleList[i].price.turret;
               end
				end
			end
		end
	end

end

----------------------------------------------------------------------

function AppData:isVehicleSideWeaponOnSale( index )

	local count = self:getPlayerVehicleCount();

	if	index and index > 0 and	index <= count then

		local saleList	= self:_getPlayerVehicleSaleList();

		if saleList and #saleList > 0 then
			for i=1,#saleList do
				if			saleList[i].vehcID <= count
					and	index == saleList[i].vehcID then

                  if saleList[i].price and saleList[i].price.side and saleList[i].price.side > 0 then
                     return true, i, saleList[i].saleID, saleList[i].price.side;
                  end
				end
			end
		end
	end

end

----------------------------------------------------------------------

function AppData:_isTableEntryDuplicate( tbl, data )

	if tbl and type(tbl) == "table" then
		for k,v in pairs(tbl) do
			if v == data then
				return true;
			end
		end
	end

end

----------------------------------------------------------------------

function AppData:_getPlayerVehicleSaleList()

	local updateList	= false;
	local count			= self:getPlayerVehicleCount();
	local saleIDList	= loadLocalFile(VEHICLE_SALE_ID_FILENAME);

	if not saleIDList then
		saleIDList = {};
		saveLocalFile(VEHICLE_SALE_ID_FILENAME, saleIDList);
	end

	self:setValue("vehicle-sale-id-list", saleIDList);

	if not self._playerVehicleSaleList then
		updateList = true;
	elseif self._playerVehicleSaleList and #self._playerVehicleSaleList < 1 then
		updateList = true;
	elseif			self._playerVehicleSaleList and #self._playerVehicleSaleList > 0
				and	self._playerVehicleSaleList[1].to < TimeMgr:getTimeGMT() then
		updateList = true;
	end

	if updateList then
		local sale = PromoMgr:getCurrentSales("vehicle");
      local abSales = AbTestMgr:getVehicleCostSales();
      if abSales then
         sale = abSales;
      end

		self._playerVehicleSaleList = {};

		if sale then
			for k,v in ipairs(sale) do
				for i=1,#v.vehicles do
					if v.vehicles[i] > 0 and v.vehicles[i] <= count then
						local t	= {};

						t.from	= v.from;
						t.to		= v.to;
						t.vehcID = v.vehicles[i];
						t.saleID	= t.from.."-";
                  t.price  = v.prices[i];

						for j=1,#v.vehicles do
							t.saleID = t.saleID..v.vehicles[j];
						end

						if HACK_VEHICLE_SALE_ID then
							t.vehcID = HACK_VEHICLE_SALE_ID;
						end

						table.insert(self._playerVehicleSaleList, t);
						break;
					end
				end
			end
		end
	end

	if self._playerVehicleSaleList and #self._playerVehicleSaleList > 0 then
		return self._playerVehicleSaleList;
	end

end

----------------------------------------------------------------------

function AppData:_calcSaleTime( time )

	if not time then
		return "7 days";
	elseif time < 0 then
		return "1 min";
	else
		local min	= time/60;
		local hour	= min/60;
		local day	= hour/24;
		local t		= ""..math.ceil(min);
		local text	= " min";

		if day > 1 then
			t		= ''..math.ceil(day);
			text	= ' days'
		elseif day == 1 then
			t		= ''..math.ceil(day);
			text	= ' day'
		elseif hour > 1 then
			t		= ''..math.ceil(hour);
			text	= ' hours'
		elseif hour == 1 then
			t		= ''..math.ceil(hour);
			text	= ' hour'
		end

		return t..text;
	end

	return "1 min";

end

----------------------------------------------------------------------

function AppData:getPlayerVehicleSaleTime( index )

	local count			= self:getPlayerVehicleCount();
	local onSale,id	= self:isPlayerVehicleOnSale(index);

	if index and index > 0 and index <= count and onSale then

		local saleList	= self:_getPlayerVehicleSaleList();
		local dt;

		if saleList and #saleList > 0 and saleList[id] then
			dt = saleList[id].to - TimeMgr:getTimeGMT();

			return dt;

		end
	end

end

----------------------------------------------------------------------

function AppData:getVehicleTurretSaleTime( index )

	local count          = self:getPlayerVehicleCount();

	local onSale,i, id	= self:isVehicleTurretOnSale(index);

	if index and index > 0 and index <= count and onSale then

		local saleList	= self:_getPlayerVehicleSaleList();
		local dt;

		if saleList and #saleList > 0 and saleList[i] then
			dt = saleList[i].to - TimeMgr:getTimeGMT();

			return dt;

		end
	end

end

----------------------------------------------------------------------

function AppData:getVehicleSideWeaponSaleTime( index )

	local count          = self:getPlayerVehicleCount();
	local onSale,i, id	= self:isVehicleSideWeaponOnSale(index);

	if index and index > 0 and index <= count and onSale then

		local saleList	= self:_getPlayerVehicleSaleList();
		local dt;

		if saleList and #saleList > 0 and saleList[i] then
			dt = saleList[i].to - TimeMgr:getTimeGMT();

			return dt;

		end
	end

end

----------------------------------------------------------------------

function AppData:getPlayerVehicleSaleText( index )

   if index and index > 0  then
      local dt = self:getPlayerVehicleSaleTime( index );
      if dt then
         return self:_calcSaleTime(dt);
      end
	end

	return "1 min";

end

----------------------------------------------------------------------

function AppData:getVehicleTurretSaleText( index )

   if index and index > 0  then
      local dt = self:getVehicleTurretSaleTime( index );
      if dt then
         return self:_calcSaleTime(dt);
      end
	end

	return "1 min";

end

----------------------------------------------------------------------

function AppData:getVehicleSideWeaponSaleText( index )

   if index and index > 0  then
      local dt = self:getVehicleSideWeaponSaleTime( index );
      if dt then
         return self:_calcSaleTime(dt);
      end
	end

	return "1 min";

end

----------------------------------------------------------------------

function AppData:getPlayerVehicleIndexMap()

	local count			= self:getPlayerVehicleCount();
	local tbl			= {};

	for i=1,count do
		table.insert(tbl, i);
	end

	return tbl;

end

----------------------------------------------------------------------

function AppData:getScoreBonus()

	local level = self:getCurrentPlayerLevel();

	if level < 1 then

		return 0;
	end

	return TemplateMgr:getScoreBonus(level);

end

----------------------------------------------------------------------

function AppData:createBossKillCountID( index )

	return "boss"..index.."KillCount";

end

----------------------------------------------------------------------

function AppData:createBossLevelID( index )

	return "boss"..index.."Level";

end

----------------------------------------------------------------------

function AppData:isBossSpawnRateValid()

	local count	= self:getValue("boss-games-since-encounter") or 0;
	local max	= AbTestMgr:getBossEncounterRate() or 5;

   if HACK_BOSS_ENCOUNTER_RATE then
      max = HACK_BOSS_ENCOUNTER_RATE;
   end

	if count >= max then
		return true;
	end

	return false;

end

----------------------------------------------------------------------

function AppData:isBossIndexValid( index )

	if			index
		and	index > 0
		and	index <= TemplateMgr:getBossIndexMax() then

		return true;
	end

end

----------------------------------------------------------------------

function AppData:getBossKillCount( index )

	return self._data[self:createBossKillCountID(index)];

end

----------------------------------------------------------------------

function AppData:getBossLevel( index, tgtHitPoints )

   if tgtHitPoints then

      local maxLevel = TemplateMgr:getBossMaxLevel(index);
      local level    = 1;

      for i = 1, maxLevel do
         local tpl = TemplateMgr:getBossTemplate( index, i )

         if tpl.hitPoints >= tgtHitPoints then
            return i;
         end
      end

      return maxLevel;
   else
      return self._data[self:createBossLevelID(index)];
   end

end

----------------------------------------------------------------------

function AppData:getBossMaxLevel( index )

	return TemplateMgr:getBossMaxLevel(index);

end

----------------------------------------------------------------------

function AppData:getBossTemplate( index, level )

	return TemplateMgr:getBossTemplate(index, level or (self:getBossLevel(index) or 1) );

end

----------------------------------------------------------------------

function AppData:_resetBountyRewardCount()

	local t = DataTransactionUpdateBountyReward:new(self, "reset");

	return TransactionMgr:execTransaction(t);

end

----------------------------------------------------------------------

function AppData:getBountyReward()

   return      AbTestMgr:getRewardInfoTable("bounty-reward")
         or    AbTestMgr:getBountyReward()
         or    self:getValue("App.settings.bounty-reward");

end

----------------------------------------------------------------------

function AppData:getBountyRewardCount()

	return self._data.bountyPlayerCount or 0;

end

----------------------------------------------------------------------

function AppData:addBountyReward( id )

	if id then

		local t = DataTransactionUpdateBountyReward:new(self, id);

		return TransactionMgr:execTransaction(t);
	end

end

----------------------------------------------------------------------

function AppData:getBountyPaintTemplate( vehcIndex, index )

	return TemplateMgr:getBountyPaintTemplate(vehcIndex, index);

end

----------------------------------------------------------------------

function AppData:getBountyArmorTemplate( vehcIndex, index )

	return TemplateMgr:getBountyArmorTemplate(vehcIndex, index);

end

----------------------------------------------------------------------

function AppData:getBountyVehicleTemplate( index )

	return TemplateMgr:getBountyVehicleTemplate(index);

end

-------------------------------------------------------------------------------------

function AppData:createDroneId( drone_index, side )

   return "drone_garage_spot_" .. drone_index .. "_side_" .. side;
end

----------------------------------------------------------------------

function AppData:_getDroneType( drone_index, side )

   local code = self:getValue(self:createDroneId(drone_index, side));

   local droneType, _, _ = DroneMgr:decodeDroneData(code);

   return droneType;
end

----------------------------------------------------------------------

function AppData:_getDroneUpgradeLevel( drone_index, side )

   local code = self:getValue(self:createDroneId(drone_index, side));

   local _, _, level = DroneMgr:decodeDroneData(code);

   return level
end

----------------------------------------------------------------------

function AppData:_getDroneRarity( drone_index, side )

   local code = self:getValue(self:createDroneId(drone_index, side));

   local _, rarity, _ = DroneMgr:decodeDroneData(code);

   return rarity
end

----------------------------------------------------------------------

function AppData:getDroneTypeLeft( drone_index )

   return self:_getDroneType(drone_index, DRONE_LEFT);
end

----------------------------------------------------------------------

function AppData:getDroneTypeRight( drone_index )
   return self:_getDroneType(drone_index, DRONE_RIGHT);
end

----------------------------------------------------------------------

function AppData:getDroneRarityLeft( drone_index )
   return self:_getDroneRarity(drone_index, DRONE_LEFT);
end

----------------------------------------------------------------------

function AppData:getDroneRarityRight( drone_index )
   return self:_getDroneRarity(drone_index, DRONE_RIGHT);
end

----------------------------------------------------------------------

function AppData:getDroneUpgradeLevelLeft( drone_index )
   return self:_getDroneUpgradeLevel(drone_index, DRONE_LEFT)
end

----------------------------------------------------------------------

function AppData:getDroneUpgradeLevelRight( drone_index )
   return self:_getDroneUpgradeLevel(drone_index, DRONE_RIGHT)
end

----------------------------------------------------------------------

function AppData:unlockDroneMount( position, price )

   local t = DataTransactionAddDroneMount:new( self, position, price );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:addDroneLeft( index, price )

   local t = DataTransactionAddDrone:new( self, index, price, DRONE_LEFT );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:addDroneRight( index, price )

   local t = DataTransactionAddDrone:new( self, index, price, DRONE_RIGHT );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:updateDroneStatsLeft( index, dType, rarity, level, cost )

   local t = DataTransactionUpdateDrone:new( self, index, DRONE_LEFT, dType, rarity, level, cost );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:updateDroneStatsRight( index, dType, rarity, level, cost )

   local t = DataTransactionUpdateDrone:new( self, index, DRONE_RIGHT, dType, rarity, level, cost );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:upgradeDroneLeft( index, price )

   local t = DataTransactionUpgradeDrone:new( self, index, price, DRONE_LEFT );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:upgradeDroneRight( index, price )

   local t = DataTransactionUpgradeDrone:new( self, index, price, DRONE_RIGHT );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:getDroneCount()
	return DRONE_MAX_COUNT_DEFAULT;
end

----------------------------------------------------------------------

function AppData:getDroneMinCount()
	return DRONE_MIN_COUNT_DEFAULT;
end

----------------------------------------------------------------------

function AppData:getTotalDroneBuiltCount()
	return self:getValue("drone_spawn_total_count_left", 0) + self:getValue("drone_spawn_total_count_right", 0);
end

----------------------------------------------------------------------

function AppData:getDroneBuiltCountLeft()
	return self:getValue("drone_spawn_total_count_left", 0);
end

----------------------------------------------------------------------

function AppData:getDroneBuiltCountRight()
	return self:getValue("drone_spawn_total_count_right", 0);
end

----------------------------------------------------------------------

function AppData:getDroneMountLeft()
	return self:getValue("drone_current_mount_left", -1);
end

----------------------------------------------------------------------

function AppData:getDroneMountRight()
	return self:getValue("drone_current_mount_right", -1);
end

----------------------------------------------------------------------

function AppData:setDroneMountLeft( value )
	return self:setValue("drone_current_mount_left", value);
end

----------------------------------------------------------------------

function AppData:setDroneMountRight( value )
	return self:setValue("drone_current_mount_right", value);
end

----------------------------------------------------------------------

function AppData:getDroneDefaultLevel()
	return DRONE_LEVEL_DEFAULT;
end

----------------------------------------------------------------------

function AppData:getDroneMinLevel()
	return DRONE_LEVEL_MIN;
end

----------------------------------------------------------------------

function AppData:getDroneMaxLevel()
	return DRONE_LEVEL_MAX;
end

----------------------------------------------------------------------

function AppData:getDroneMaxRarity()
	return DRONE_RARITY_MAX;
end

----------------------------------------------------------------------

function AppData:_createDroneDefaultData( drone_index, side )

   return { name     = self:createDroneId(drone_index, side),
            default  = 0,
            critical = true,
            server   = true,
            merge    = self._mergePossessionsDrone, };
end

----------------------------------------------------------------------

function AppData:getNextWeeklyResetTime()

   local time      = self:getValue("weekly-reset-timestamp", 0);
   local date      = os.date("*t", time);
   local deltadate = os.date("*t", time+((8 - date.wday) * 24 * 60 * 60));
   local delta     = os.time(deltadate) - ( (deltadate.wday - 1) * 24 * 60 * 60);
   local timezone  = 60*60*8 - TimeMgr:calcLocalDeltaGMT();

   deltadate      = os.date("*t", delta);
   deltadate.sec  = 0;
   deltadate.min  = 0;
   deltadate.hour = 24;

   if HACK_QUICK_WEEKLY_RESET  and HACK_QUICK_WEEKLY_RESET_TIME then
      deltadate      = os.date("*t", time+HACK_QUICK_WEEKLY_RESET_TIME);
      deltadate.sec  = 0;

      if HACK_QUICK_WEEKLY_RESET_TIME >= 60*60 then
         deltadate.min  = 0;
      end

      return os.time(deltadate) or 0;
   end

   return math.max( (os.time(deltadate) or 0) - timezone, 0 );

end

----------------------------------------------------------------------

function AppData:giveWeeklyReward( rank )

   local reward		= TemplateMgr:getWeeklyRewardAmount(rank) or 0;
   local settings    =  AbTestMgr:getRewardInfoTable("weekly-reward")
                     or self:getValue("App.settings.weekly-reward")
                     or {};

   if settings and settings[rank] and settings[rank].gem then
      reward = settings[rank].gem;
   end

   local t = DataTransactionAddWeeklyReward:new( self, reward, rank );

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------

function AppData:isWeeklyGiftCollectFull( key )

   local addKey = key or 0;
print("AppData:isWeeklyGiftCollectFull", self:getValue("weekly-gifts-collected", 0), addKey, self:getValue("weekly-gifts-collected", 0) + addKey, self:getValue("App.settings.gift-collected-limit", GIFT_WEEKLY_COLLECTION_LIMIT))
   return self:getValue("weekly-gifts-collected", 0) + addKey >= self:getValue("App.settings.gift-collected-limit", GIFT_WEEKLY_COLLECTION_LIMIT);

end


-------------------------------------------------------------------------------------

function AppData:getLeagueIndex()
	return self:getValue("league-LeagueIndex", 0);
end

----------------------------------------------------------------------

function AppData:getDivisionIndex()
	return self:getValue("league-DivisionIndex", 0);
end

----------------------------------------------------------------------

function AppData:updateLeagueData( newLeagueIndex, newDivisionIndex, reward, forceSet )

   reward = reward or { gem = 0, gold = 0, rank = 1 };

   local t = DataTransactionUpdateLeague:new(self, newLeagueIndex, newDivisionIndex, reward, forceSet);

   return TransactionMgr:execTransaction( t );

end

----------------------------------------------------------------------
