require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                       = DataTransaction;
DataTransactionAddConsumable     = Base:_new("DataTransactionAddConsumable");

----------------------------------------------------------------------

function DataTransactionAddConsumable:new( data, index, fCountID, price, autoActivate, freeUse )

   Debug.assertType(index, "number", "DataTransactionAddConsumable:new -> index");

	if	data and index and fCountID and price then

		local task				= Base.new( self );

		task.index           = index;
      task.consumableId    = data:createConsumableId(index);
      task.template        = ConsumableMgr:getTemplate(index);
		task.fCountID        = fCountID;
      task.time            = TimeMgr:getTimeGMT();
      task.currency        = price.currency;
		task.cost				= price.cost;
      task.freeUse         = freeUse;
      task.autoActivate    = autoActivate;

      if task.freeUse == nil then
         task.freeUse = false;
      end
      
      if task.autoActivate == nil then
         task.autoActivate = false;
      end

		return task;
	end

end

----------------------------------------------------------------------

function DataTransactionAddConsumable:buildValues( data )

   Debug.assert(self.template, "DataTransactionAddConsumable:buildValues - > template nil");
   Debug.assert(self.template.duration, "DataTransactionAddConsumable:buildValues - > duration nil");

   if self.template and self.template.quantity then

      local currency    = data:getValue(self.currency, 0);
      local inventory   = data:getValue(self.consumableId, 0);

      self._values      = {};

      if currency - self.cost >= 0 then

         self._values[self.currency]   = currency - self.cost;

         if math.max(self.template.duration or 0, 0) > 0 then
            self._values[self.consumableId]  = self.time;
            self.freeUse               = true;
            self.autoActivate          = true;
         else
            self._values[self.consumableId]  = inventory + math.max(self.template.quantity or 1, 1);
         end
      end

      return self._values;
   end

end

----------------------------------------------------------------------

function DataTransactionAddConsumable:onSuccess( data )

	if self.cost and self.template and self.fCountID then

      local cost     = "free";
      local cType    = tostring(self.template.type);
      local cCount   = tostring(data:deltaValue("consumable-count", 1));
      local fCountID = tostring(data:deltaValue("consumable-"..self.fCountID, 1));

      if self.cost > 0 then
         cost  = tostring(self.cost);

         if self.currency == "gem" then
            EventMgr:postEvent(  {
                                    reason   = "consumable "..cType or "unknown",
                                    cost     = cost
                                 },
                                 "Spend-Gem" );
         end
      end

      MOAIFlurry:event(    "Consumable"
                           , "type",            cType
                           , "count",           cCount
                           , self.fCountID,     fCountID
                           , "cost",            cost   );

      if self.autoActivate then
         ConsumableMgr:consume( self.index, self.freeUse );
      end
	end

end

----------------------------------------------------------------------
