require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

local MOUNT_POINT_LEFT			= "left";
local MOUNT_POINT_RIGHT			= "right";
local MOUNT_POINT_EMPTY			= 0;

----------------------------------------------------------------------

local Base                     	= DataTransaction;
DataTransactionAddDroneMount		= Base:_new("DataTransactionAddDroneMount");

----------------------------------------------------------------------

function DataTransactionAddDroneMount:new( data, position, price )

	if			position
		and	price
		and	type(price) == "table"
		and	price.currency
		and	price.cost then

		local task				= Base.new( self );

		task.position			= position;
		task.currency			= price.currency;
		task.cost				= price.cost;

		return task;
	end

end

----------------------------------------------------------------------

function DataTransactionAddDroneMount:buildValues( data )

	local bank			= data:getValue(self.currency, 0);

	self._values		= {};
	self.unlockMount	= false;

	if			self.position == MOUNT_POINT_LEFT
		and	DroneMgr:canUnlockDroneMountLeft() then

		self.unlockMount	= true;
		bank					= bank-self.cost;

		self._values["drone_current_mount_left"] = MOUNT_POINT_EMPTY;
	elseif			self.position == MOUNT_POINT_RIGHT
				and	DroneMgr:canUnlockDroneMountRight() then

		self.unlockMount	= true;
		bank					= bank-self.cost;

		self._values["drone_current_mount_right"] = MOUNT_POINT_EMPTY;
	end

	self._values[self.currency] 	= bank;

	return self._values;

end


----------------------------------------------------------------------

function DataTransactionAddDroneMount:onSuccess( data )

   if self.unlockMount then

		if self.cost > 0 then

			EventMgr:postEvent(	{	reason    = "drone-bay-unlock",
											cost      = self.cost  },
										"Spend-Gem"	);
		end

		local lvl	= math.floor((data:getCurrentPlayerLevel() or 1) / 10);
		local gem	= self.cost;

		EventMgr:postEvent(	{	level	= lvl,
										cost	= gem	},
									"drone-bay-unlock"	);
   end

end

----------------------------------------------------------------------
