require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

local DRONE_LEFT						= "left";  -- mirror in app data
local DRONE_RIGHT						= "right"; -- mirror in app data

----------------------------------------------------------------------

local Base					= DataTransaction;
DataTransactionAddDrone	= Base:_new("DataTransactionAddDrone");

----------------------------------------------------------------------

function DataTransactionAddDrone:new( data, index, price, side )

	if	index and side then

		local task				= Base.new( self );

		task.index				= index;
		task.side				= side;
		task.currency			= price.currency;
		task.cost				= price.cost;

		return task;
	end

end

----------------------------------------------------------------------

function DataTransactionAddDrone:buildValues( data )

   self._values     = {};
   self.buildDrone  = false;

   if    ( self.side == DRONE_LEFT and DroneMgr:canBuildDroneLeft(self.index) )
      or ( self.side == DRONE_RIGHT and DroneMgr:canBuildDroneRight(self.index) ) then

      self.buildDrone = true;

      if self.side == DRONE_LEFT then
         self.droneCount = data:getDroneBuiltCountLeft() + 1;
      else
         self.droneCount = data:getDroneBuiltCountRight() + 1;
      end

      self.rarity = DroneMgr:pickDroneRarity(self.side) or 1;
      self.dType  = DroneMgr:pickDroneType(self.side);

      self._values[self.currency] = data:getValue(self.currency, 0) - self.cost;

      self._values[data:createDroneId(self.index, self.side)] = DroneMgr:encodeDroneData(self.dType, self.rarity, DroneMgr:getDroneDefaultLevel());

      self._values["drone_spawn_total_count_" .. self.side] = self.droneCount;

   end

   return self._values;

end

----------------------------------------------------------------------

function DataTransactionAddDrone:onSuccess( data )

	if self.buildDrone and self.dType and self.rarity then

		if self.cost > 0 then

			EventMgr:postEvent(	{	reason    = "drone-build",
											cost      = self.cost  },
										"Spend-Gem"	);
		end

		local lvl	= math.floor((data:getCurrentPlayerLevel() or 1) / 10);
		local gem	= self.cost;

		EventMgr:postEvent(	{	level	= lvl,
										cost	= gem,
										side	= self.side,
                              dType = DroneMgr:getDroneName( self.dType, self.rarity ), },
									"drone-build"	);
	end

end

----------------------------------------------------------------------
