require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                       = DataTransaction;
DataTransactionAddWeeklyReward   = Base:_new("DataTransactionAddWeeklyReward");

----------------------------------------------------------------------

function DataTransactionAddWeeklyReward:new( data, amount, rank )

   local task  = Base.new(  self );
   task.amount = amount;
   task.rank   = rank;  --Rank <= 0 is used to setup initial time stamp
   task.time   = TimeMgr:getTimeGMT();
   task.award  = false;

   return task;

end

----------------------------------------------------------------------

function DataTransactionAddWeeklyReward:testMe()

   print("DataTransactionAddWeeklyReward:testMe()");
   
end

----------------------------------------------------------------------

function DataTransactionAddWeeklyReward:buildValues(data)

   if self.time >= data:getNextWeeklyResetTime() then

      local gem         = data:getValue("gem") + self.amount;
      local activeWeek  = data:getValue("activeWeekCount", 0)

      if self.rank > 0 then
         activeWeek = activeWeek + 1;
      end

      self._values                              =  {};
      self._values["gem"]                       = gem;
      self._values["weekly-reset-timestamp"]    = self.time;
      self._values["weekly-gifts-collected"]    = 0;
      self._values["activeWeekCount"]           = activeWeek;

      self.award                                = true;

      return self._values;
   end   

end

----------------------------------------------------------------------

function DataTransactionAddWeeklyReward:onSuccess(data)

   if self.award and self.rank > 0 then
      local firstLaunch = App.data:getValue("firstLaunch") or self.time;
      local since       = math.floor( (self.time-firstLaunch)/(24*60*60) );

      MOAIFlurry:event( "Weekly-Reward",
                        "Rank", tostring(self.rank),
                        "Reward", tostring(self.amount) );

      EventMgr:postEvent( { gem=self.amount, rank=self.rank, weeksActive=data:getValue("activeWeekCount", 1), daysInstalled=since }, "weekly-reward" );

      if self.amount > 0 then
      	EventMgr:postEvent( { type="Weekly Reward", gem=self.amount }, "More-Gem" );
   	end
   end

end

----------------------------------------------------------------------
