require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                     	= DataTransaction;
DataTransactionAddXP					= Base:_new("DataTransactionAddXP");

----------------------------------------------------------------------

function DataTransactionAddXP:new( data, xp )

   local task	= Base.new(  self );

	task.curLevel        = data:getCurrentPlayerLevel();
   task.newXP           = xp or 0;
	task.reqXP           = data:getXPLevelReq();
   task.levelGamePlayed = data:getValue("levelGamePlayed");
   task.highScore       = data:getValue("level-high-score",0);



   return task;

end

----------------------------------------------------------------------

function DataTransactionAddXP:buildValues( data )

   local newXP = self.newXP;

   local curLevel  = self.curLevel;
   local currentXP = data:getCurrentPlayerXP();
   local reqXP     = self.reqXP;

   if curLevel ~= data:getCurrentPlayerLevel() then

      curLevel = data:getCurrentPlayerLevel();
      reqXP = data:getXPLevelReq();
   end

   local totalXP  = newXP + currentXP;
   local maxLevel = data:getMaxPlayerLevel();

   self._values	=	{};

   self._values.levelGamePlayed  = data.levelGamePlayed;

   if totalXP >= reqXP then

      if (curLevel + 1) <= maxLevel then

         self._values.currentLevel        = curLevel + 1;
         self._values.xp                  = totalXP - self.reqXP;
         self._values.levelGamePlayed     = 0;
         self._values["level-high-score"] = 0;

         self.levelUp    = true;

         if self._values.currentLevel >= maxLevel then

            if self._values.xp >= data:getXPLevelReq(self._values.currentLevel) then

               self._values.xp = data:getXPLevelReq(self._values.currentLevel);
            end
         else

         end

      else

         self._values.xp = reqXP;
      end

   else

      self._values.xp = totalXP;
   end

   return self._values;

end


----------------------------------------------------------------------

function DataTransactionAddXP:onSuccess( data )

   RankingMgr:submitLevel( self._values.currentLevel );

   if self.levelUp then

      EventMgr:postEvent( {   level             = self._values.currentLevel,
                              levelGamePlayed   = self.levelGamePlayed,
                              levelHiScore      = self.highScore                }, "player-level-up" );

   end

end

----------------------------------------------------------------------
