require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                    = DataTransaction;
DataTransactionDailyPickup    = Base:_new("DataTransactionDailyPickup");

----------------------------------------------------------------------

function DataTransactionDailyPickup:new( data )

print("DataTransactionDailyPickup:new");

   local task              = Base.new(  self );
   task.type               = type;
   task.dailyGamePlayed    = data:getValue("dailyGamePlayed")  or 0;
   task.objective          = data:getDailyChallengeGoal();

   return task;

end

----------------------------------------------------------------------

function DataTransactionDailyPickup:testMe()

   print("DataTransactionDailyPickup:testMe()");

end
----------------------------------------------------------------------

function DataTransactionDailyPickup:buildValues(data)

   local dailyProgress = data:getValue("dailyProgress") or 0;

   --Make sure the daily is active
   if  dailyProgress >= 0 then

      local dailyRewardGem       = data:getValue("dailyRewardGem")   or 0;
      local dailyRewardGold      = data:getValue("dailyRewardGold")  or 0;
      local dailyStreak          = data:getValue("dailyStreak")      or 0;
      local dailyGamePlayed      = data:getValue("dailyGamePlayed")  or 0;
      local dailyGamePlayedBackup= dailyGamePlayed;
      local hasSeenDaily         = data:getValue("hasSeenDaily")     or false;

      dailyProgress              = dailyProgress + 1;

      -- did we completed
      if dailyProgress  >= self.objective then
         local reward =    data:getDailyChallengeReward();
         if reward then
            dailyRewardGem  = dailyRewardGem   + (reward.gem  or 0);
            dailyRewardGold = dailyRewardGold  + (reward.gold or 0);
         end

         dailyStreak     = dailyStreak + 1;
         dailyProgress   = -1;
         dailyGamePlayed =  0;
         hasSeenDaily    = false;
      end;

      self._values                     = {};
      self._values["dailyProgress"]    = dailyProgress;
      self._values["dailyStreak"]      = dailyStreak;
      self._values["dailyGamePlayed"]  = dailyGamePlayed;
      self._values["dailyRewardGold"]  = dailyRewardGold;
      self._values["dailyRewardGem"]   = dailyRewardGem;
      self._values["hasSeenDaily"]     = hasSeenDaily;

      return self._values;
   end

end

----------------------------------------------------------------------

function DataTransactionDailyPickup:onSuccess(data)

   EventMgr:postEvent(  {
                           curProg=self._values["dailyProgress"],
                           curObj=self.objective
                        },
                        "daily-progress");

   MOAIFlurry:event(    "Daily-Progress"
                        , "progress",     ""..self._values["dailyProgress"]
                        , "GamePlayed",   ""..self.dailyGamePlayed
                        , "Streak",       ""..self._values["dailyStreak"]);

   if self._values["dailyProgress"] < 0 then

      EventMgr:postEvent(  {
                              currentStreak  = self._values["dailyStreak"],
                              curObj         = self.objective
                           },
                           "daily-success"                                    );

      MOAIFlurry:event(   "Daily-Success-".. math.min(self._values["dailyStreak"],5)
                     ,    "GamePlayed",   ""..self.dailyGamePlayed
                     ,    "Streak",       ""..self._values["dailyStreak"]   );
   end


end

----------------------------------------------------------------------
