require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                    = DataTransaction;
DataTransactionSpendFreebie   = Base:_new("DataTransactionSpendFreebie");

----------------------------------------------------------------------

function DataTransactionSpendFreebie:new( data, type, amount, reason )

   Debug.assert(data and type and amount);

   local task     = Base.new(  self );
   task.type      = type or "unknown";
   task.amount    = amount;
   task.reason    = reason;

   return task;

end

----------------------------------------------------------------------

function DataTransactionSpendFreebie:testMe()

   print("DataTransactionSpendFreebie:testMe()");
   
end

----------------------------------------------------------------------

function DataTransactionSpendFreebie:buildValues(data)

   local available = data:getValue(self.type, 0);

   if self.amount and self.amount > 0 and self.amount <= available then

      self.newAmount          = available - self.amount;
      self._values            =  {};
      self._values[self.type] =  self.newAmount;

      return self._values;

   else

      Debug.assert( self.amount and self.amount > 0 and self.amount < 100000 );

   end

end

----------------------------------------------------------------------

function DataTransactionSpendFreebie:onSuccess(data)

   if App and App.game then

      if self.type == "freebie-head-start" then

         EventMgr:postEvent( {type="powerup_speed_burst"}, "consumable-used" );
         MOAIFlurry:event(   "Consumable"
                           , "type",               "powerup_speed_burst"
                           , "count",              tostring(App.data:deltaValue("consumable-count",1))
                           , "headstart-count",    tostring(App.data:deltaValue("consumable-headstart-count",1))
                           , "cost",               "free"                                                            );

      elseif self.type == "freebie-power-time" then

         EventMgr:postEvent( {type="powerup_powertime"}, "consumable-used" );
         MOAIFlurry:event(    "Consumable"
                           ,  "type",                 "powerup_powertime"
                           , "count",                 tostring(App.data:deltaValue("consumable-count",1))
                           , "powertime-count",     tostring(App.data:deltaValue("consumable-powertime-count",1))
                           , "cost",                  "free"                                                            );

      elseif self.type == "freebie-double-score" then

         EventMgr:postEvent( {type="powerup_double_score"}, "consumable-used" );
         MOAIFlurry:event(    "Consumable"
                           ,  "type",                 "powerup_double_score"
                           , "count",                 tostring(App.data:deltaValue("consumable-count",1))
                           , "doublescore-count",     tostring(App.data:deltaValue("consumable-doublescore-count",1))
                           , "cost",                  "free"                                                            );

      end
   end

   MOAIFlurry:event(    "Spend-Freebie",
                        "type",              self.type or "unknown",
                        "spent",             tostring(self.amount),
                        "total",             tostring(self.newAmount),
                        "reason",            self.reason or "unknown"      );

end


----------------------------------------------------------------------
