require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                    = DataTransaction;
DataTransactionTangoBonus     = Base:_new("DataTransactionTangoBonus");

----------------------------------------------------------------------

function DataTransactionTangoBonus:new( data, gold, gem )

   local task  = Base.new(  self );
   task.rewardGem    = gem    or 0;
   task.rewardGold   = gold   or 0;
   task.gamePlayed   = data:getGamePlayed()
   
   return task;

end

----------------------------------------------------------------------

function DataTransactionTangoBonus:testMe()

   print("DataTransactionTangoBonus:testMe()");
   
end

----------------------------------------------------------------------

function DataTransactionTangoBonus:buildValues(data)

   if self.rewardGold > 0 or self.rewardGem > 0 then

      local signingGold       = data:getValue("signingGold") or 0;
      local signingGem        = data:getValue("signingGem")  or 0;

      --You should only receive your reward once
      if signingGold == 0 and signingGem == 0 then

         local gold                 = data:getGoldCount() + self.rewardGold;
         local gem                  = data:getGemCount()  + self.rewardGem;
      
         --Save
         self._values               = {};
         self._values["gold"]       = math.max( data:getGoldCount()  + self.rewardGold,   0   );
         self._values["gem"]        = math.max( data:getGemCount()   + self.rewardGem,    0   );
         self._values["signingGold"]= self.rewardGold;
         self._values["signingGem"] = self.rewardGem;

         return self._values;

      end
   end
 
end


----------------------------------------------------------------------

function DataTransactionTangoBonus:onSuccess(data)

   if self.rewardGem > 0 then

      EventMgr:postEvent( { type="tango-carrot", gem=self.rewardGem }, "More-Gem" );
                           
   end


   if self.rewardGold > 0 then
      MOAIFlurry:event(    "More-Gold",
                           "type",          "tango-carrot",
                           "amount-gold",   ""..self.rewardGold         );
                           
   end
   
   MOAIFlurry:event(    "Tango-Convert",
                        "amount-gem",     ""..self.rewardGem,
                        "amount-gold",    ""..self.rewardGold,
                        "game-played",    ""..self.gamePlayed     );
                           
end

----------------------------------------------------------------------
