require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                 = DataTransaction;
DataTransactionUnlockPaint = Base:_new("DataTransactionUnlockPaint");

----------------------------------------------------------------------

function DataTransactionUnlockPaint:new( data, vehicle_index, paint_index )

   assert(data);
   
   local task  = Base.new( self, data );
   task.vehicle_index  = vehicle_index;
   task.paint_index    = paint_index;
    
   local cost     = data:getPlayerVehiclePaintVariantUnlockCost( vehicle_index, paint_index ) or {};
   task.costGold  = cost.gold or 0;
   task.costGem   = cost.gem  or 0;
   
   task.first     = (data:getValue("garage-use-count") or 0) <= 0;

   return task;

end

----------------------------------------------------------------------

function DataTransactionUnlockPaint:testMe()

   print("DataTransactionUnlockPaint:testMe()");
   
end

----------------------------------------------------------------------

function DataTransactionUnlockPaint:buildValues(data)

   if  data:canUnlockPlayerVehiclePaintVariant( self.vehicle_index, self.paint_index ) then

      local gold         = data:getGoldCount();
      local gem          = data:getGemCount();

      if       self.costGold <= gold
         and   self.costGem  <= gem  then
         
         local id             = data:createPlayerVehiclePaintVariantId( self.vehicle_index, self.paint_index );
            
         --Save
         self._values                     = {};
         self._values["gold"]             = math.max( gold-self.costGold,    0        );
         self._values["gem"]              = math.max( gem-self.costGem,      0        );
         self._values[id]                 = 1;   
         self._values["garage-use-count"] = (data:getValue("garage-use-count") or  0 ) + 1;
           
         return self._values;
            
      end
   end

end


----------------------------------------------------------------------

function DataTransactionUnlockPaint:onSuccess(data)

   EventMgr:postEvent({ item     = "paint",
                        vehicle  = self.vehicle_index,
                        paint    = self.paint_index,   },
                        "unlock-success"                       );

   if self.first then
      MOAIFlurry:event(    "Use-Garage-First",
                           "type",              "unlock-paint",
                           "game Played",       "" .. data:getGamePlayed() );
   end
                   
   MOAIFlurry:event(    "Use-Garage",
                        "type",              "unlock-paint",
                        "count",             "".. self._values["garage-use-count"]    );

   MOAIFlurry:event(    "Unlock-Vehicle-Paint",
                        "vehicle",        ""..self.vehicle_index,
                        "paint",          ""..self.paint_index,
                        "cost-gold",      ""..self.costGold,
                        "cost-gem",       ""..self.costGem,
                        "gold",           ""..self._values.gold,
                        "gem",            ""..self._values.gem    );

   if self.costGem > 0 then

      EventMgr:postEvent({ reason    = "unlock-paint",
                           cost      = self.costGem  },
                           "Spend-Gem"                            );
   end

   if self.costGold > 0 then
      MOAIFlurry:event( "Spend-Gold",
                        "type",          "unlock-paint",
                         "amount-gold",  ""..self.costGold          );
   end


end

----------------------------------------------------------------------
