require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                    = DataTransaction;
DataTransactionUnlockVehicle  = Base:_new("DataTransactionUnlockVehicle");

----------------------------------------------------------------------

function DataTransactionUnlockVehicle:new( data, index )

   assert(data);
   
   local task     = Base.new(  self, data );
   task.index     = index;
   
   local cost     = data:getPlayerVehicleUnlockCost( index ) or {};
   task.costGold  = cost.gold or 0;
   task.costGem   = cost.gem  or 0;

   task.first     = (data:getValue("garage-use-count") or 0) <= 0;
   
   return task;

end

----------------------------------------------------------------------

function DataTransactionUnlockVehicle:testMe()

   print("DataTransactionUnlockVehicle:testMe()");
   
end


----------------------------------------------------------------------

function DataTransactionUnlockVehicle:buildValues( data )

print( "DataTransactionUnlockVehicle:exec", self.index, data:canUnlockPlayerVehicle(self.index) );

   if data:canUnlockPlayerVehicle( self.index ) then

      local gold  = data:getGoldCount();
      local gem   = data:getGemCount();

      if       self.costGold <= gold
         and   self.costGem  <= gem  then

         self._values                                                         =  {};
         self._values["gold"]                                                 = math.max( gold-self.costGold,     0        );
         self._values["gem"]                                                  = math.max( gem-self.costGem,       0        );
         self._values[data:createMainWeaponId(self.index)]                    = math.max(data:getValue(data:createMainWeaponId(self.index))or 1, 1);
         self._values[data:createArmorId(self.index)]                         = math.max(data:getValue(data:createArmorId(self.index)) or 1, 1);
         self._values[data:createPlayerVehiclePaintVariantId(self.index,1)]   = math.max(data:getValue(data:createPlayerVehiclePaintVariantId(self.index,1)) or 1, 1);
         self._values["garage-use-count"]                                     = (data:getValue("garage-use-count") or  0 ) + 1;

	    return self._values;

      end
   end
end

----------------------------------------------------------------------

function DataTransactionUnlockVehicle:onSuccess( data)

   EventMgr:postEvent({ item                 = "vehicle",
                        vehicle              =  self.index,
                        costGold             =  self.costGold,
                        costGem              =  self.costGem      },
                        "unlock-success"                             );
                       
   if self.costGem > 0 then

      EventMgr:postEvent({ reason    = "unlock-vehicle",
                           cost      = self.costGem  },
                           "Spend-Gem"                            );

   end

   if self.costGold > 0 then
      MOAIFlurry:event(    "Spend-Gold",
                           "type",          "unlock-vehicle",
                           "amount-gold",    ""..self.costGold              );
   end     
end

----------------------------------------------------------------------
