require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                             = DataTransaction;
DataTransactionUpdateAchievementRank   = Base:_new("DataTransactionUpdateAchievementRank");

----------------------------------------------------------------------

function DataTransactionUpdateAchievementRank:new( data,  achvId, rank, reward  )

   local task  = Base.new(  self );
   --save the cost and reward here in case we tweak and the user retry the transaction
   task.achvId       = achvId;
   task.rank         = rank;

   task.reward     = reward or  {};
   task.rewardGold = task.reward.gold or 0;
   task.rewardGem  = task.reward.gem  or 0;
   task.gamePlayed = data:getValue(data:createAchievemetGpId( achvId ), 0) + 1;
   task.level      = data:getCurrentPlayerLevel();


   return task;

end

----------------------------------------------------------------------

function DataTransactionUpdateAchievementRank:testMe()

   print("DataTransactionUpdateAchievementRank:testMe()");
   
end

----------------------------------------------------------------------

function DataTransactionUpdateAchievementRank:buildValues( data )

   local achvProgId  = data:createAchievemetProgressId( self.achvId );
   local achvRankId  = data:createAchievemetRankId( self.achvId );
   local achvGPId    = data:createAchievemetGpId( self.achvId );

   local gold        = data:getGoldCount() or 0;
   local gem         = data:getGemCount()  or 0;

   --Save
   self._values                     = {};
   self._values[achvProgId]         = 0;
   self._values[achvRankId]         = self.rank;
   self._values[achvGPId]           = 0;

   self._values["gold"]             = math.max(gold+self.rewardGold,0);
   self._values["gem"]              = math.max(gem+self.rewardGem,  0);

print("DataTransactionUpdateAchievementProgress:exec-1", self.achvId, self.progress, achvProgId,
                                                         self._values[achvProgId], self._values[achvRankId], self._values["gold"], self._values["gem"] );

   return self._values;

end

----------------------------------------------------------------------

function DataTransactionUpdateAchievementRank:onSuccess( data )


--print("first-version", self.data:getValue("first-version-played") );

   local oldPlayer = tonumber((App.data:getValue("first-version-played") or 0) ) <  App:versionStringToNumber("1.15.00");

   local paramGpId      = "Rank-" .. (self.rank-1) .. "-GP";
   local valueGp        = tostring(math.max(math.floor(self.gamePlayed/10)*10,1));
   local paramLevelId   = "Rank-" .. (self.rank-1) .. "-Level";
   local valueLevel     = tostring(self.level);

   --Dont mix old player with new player
   --Also, we dont have the place to track what level the player was when achieving
   -- (and it would be bad data since there level could be anything)
   if oldPlayer then
      paramGpId    = paramGpId .. "-OLD";
      paramLevelId = nil;
      valueLevel   = nil;
   end


   MOAIFlurry:event(    "Achievement-"   .. self.achvId,
                        paramGpId,          tostring(valueGp),
                        paramLevelId,       tostring(valueLevel)  );



   if self.rewardGold  > 0 then
      MOAIFlurry:event(    "More-Gold",
                           "type",          "achievement-reward",
                           "amount-gold",   ""..self.rewardGold         );
   end

   if self.rewardGem > 0 then

      EventMgr:postEvent( { type="achievement-reward", gem=self.rewardGem }, "More-Gem" );

   end

end

----------------------------------------------------------------------
