require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

local DRONE_LEFT					= "left";  -- mirror in app data
local DRONE_RIGHT					= "right"; -- mirror in app data

----------------------------------------------------------------------

local Base							= DataTransaction;
DataTransactionUpdateDrone		= Base:_new("DataTransactionUpdateDrone");

----------------------------------------------------------------------

function DataTransactionUpdateDrone:new( data, index, side, dType, rarity, level, cost )

	if			index
		and	side
		and	dType
		and	rarity
		and	level then

		local task	= Base.new( self );
		local price	= cost or {currency="gem", cost=0};

		task.index		= index;
		task.side		= side;
		task.dType		= dType;
		task.rarity		= rarity;
		task.level		= level;
		task.currency	= price.currency;
		task.cost		= price.cost;

		return task;
	end

end

----------------------------------------------------------------------

function DataTransactionUpdateDrone:buildValues( data )

   local bank = data:getValue(self.currency, 0);

   self._values	= {};

   if bank >= self.cost then

      self._values[self.currency] = bank - self.cost;

      self._values[data:createDroneId(self.index, self.side)] = DroneMgr:encodeDroneData(self.dType, self.rarity, self.level);
   end

   return self._values;

end


----------------------------------------------------------------------

function DataTransactionUpdateDrone:onSuccess( data )

	if self.currency == "gem" and self.cost > 0 then

		EventMgr:postEvent(	{	reason    = "drone-evolve",
										cost      = self.cost  },
									"Spend-Gem"	);

		local lvl	= math.floor((data:getCurrentPlayerLevel() or 1) / 10);
		local gem	= self.cost;

		if self.side == DRONE_LEFT then
			id	= DroneMgr:getDroneName(	self.dType,	self.rarity);
		else
			id	= DroneMgr:getDroneName(	self.dType,	self.rarity);
		end

		EventMgr:postEvent(	{	level	= lvl,
										cost	= gem,
										dType	= id	},
									"drone-evolve"	);
	end

end

----------------------------------------------------------------------
