require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                       = DataTransaction;
DataTransactionUpdateGameStats   = Base:_new("DataTransactionUpdateGameStats");

----------------------------------------------------------------------

function DataTransactionUpdateGameStats:new( data, goldCollected, gameDuration, distance )

print("DataTransactionUpdateGameStats:new");

   local task  = Base.new(  self );
   
   task.goldCollected   = goldCollected         or 0;
   task.gameDuration    = gameDuration          or 0;
   task.distance        = distance              or 0;
   task.dailyActive     = data:isDailyActive();
   
   return task;

end

----------------------------------------------------------------------

function DataTransactionUpdateGameStats:testMe()

   print("DataTransactionUpdateGameStats:testMe()");
   
end

----------------------------------------------------------------------

function DataTransactionUpdateGameStats:buildValues(data)

   local xtraDailyPlay = 0;
   if self.dailyActive then
      xtraDailyPlay  = 1;
   end
   
   local xtraMission = { 0, 0, 0 };

   if data:isMissionUnlocked() then
      for i=1,data:getMissionGoalCount() do
         local p = data:getValue(data:createMissionGoalProgressId(i)) or 0;
         if p >= 0 then
            xtraMission[i] = 1;
         end
      end
   end
  
   self._values                        =  {};
   self._values["gamePlayed"]          = (data:getValue("gamePlayed")         or 0) + 1;
   self._values["gold"]                = (data:getValue("gold")               or 0) + self.goldCollected;
   self._values["dailyGamePlayed"]     = (data:getValue("dailyGamePlayed")    or 0) + xtraDailyPlay;
   self._values["levelGamePlayed"]     = (data:getValue("levelGamePlayed")    or 0) + 1;

   for i=1,data:getMissionGoalCount() do
      local       id       = data:createMissionGoalGamePlayedId(i);
      self._values[id]     = (data:getValue(id) or 0) + xtraMission[i];
      local       id2      = data:createMissionGoalProgressId(i);
      self._values[id2]    = (data:getValue(id2) or 0);
   end
   
   return self._values;
   
end

----------------------------------------------------------------------

function DataTransactionUpdateGameStats:onSuccess( data)

   MOAIFlurry:event(    "Mission-Progress"
                     ,   "Index",               ""..data:getMissionIndex()
                     ,   "Goal-1-Progress",     ""..math.floor(self._values[data:createMissionGoalProgressId(1)])
                     ,   "Goal-2-Progress",     ""..math.floor(self._values[data:createMissionGoalProgressId(2)])
                     ,   "Goal-3-Progress",     ""..math.floor(self._values[data:createMissionGoalProgressId(3)])
                     ,   "Goal-1-GamePlayed",   ""..self._values[data:createMissionGoalGamePlayedId(1)]
                     ,   "Goal-2-GamePlayed",   ""..self._values[data:createMissionGoalGamePlayedId(2)]
                     ,   "Goal-3-GamePlayed",   ""..self._values[data:createMissionGoalGamePlayedId(3)]   );



end

----------------------------------------------------------------------
