require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base = DataTransaction;
DataTransactionUpdateLeague = Base:_new("DataTransactionUpdateLeague");

----------------------------------------------------------------------

function DataTransactionUpdateLeague:new( data, leagueIndex, divisionIndex, reward, forceSet )

   Debug.assert( data );
   Debug.assert( leagueIndex, "DataTransactionUpdateLeague:new -> nil league index" );
   Debug.assert( divisionIndex, "DataTransactionUpdateLeague:new -> nil division index" );

   local prize = reward or { gem=0,  gold=0, rank=1 };

   Debug.assert( type(reward)=="table", "DataTransactionUpdateLeague:new -> invalid reward data" );
   Debug.assert( reward.gem, "DataTransactionUpdateLeague:new -> nil gem reward" );
   Debug.assert( reward.gold, "DataTransactionUpdateLeague:new -> nil gold reward" );

   local task = Base.new( self, data );

   task.leagueIndex   = leagueIndex;
   task.divisionIndex = divisionIndex;
   task.forceSet      = forceSet or false;

   if reward then
      task.gem   = prize.gem or 0;
      task.gold  = prize.gold or 0;
      task.rank  = prize.rank or 1;
      task.time  = TimeMgr:getTimeGMT();
      task.reset = false;
   else
      task.gem   = 0;
      task.gold  = 0;
      task.rank  = LeagueMgr:getPlayerCohortSize();
      task.reset = true;
   end

   return task;
end

----------------------------------------------------------------------

function DataTransactionUpdateLeague:buildValues(data)

   Debug.assertType(self.leagueIndex, "number", "DataTransactionUpdateLeague:buildValues -> self.leagueIndex")
   Debug.assertType(self.divisionIndex, "number", "DataTransactionUpdateLeague:buildValues -> self.divisionIndex")

   local gemBank  = data:getValue("gem", 0);
   local goldBank = data:getValue("gold", 0);

   gemBank  = gemBank + self.gem;
   goldBank = goldBank + self.gold;

   self._values = {};

   self._values["league-LeagueIndex"]   = self.leagueIndex;
   self._values["league-DivisionIndex"] = self.divisionIndex;

   if self.reset then

      return self._values;

   elseif not self.reset and ( (self.time and self.time >= data:getNextWeeklyResetTime()) or self.forceSet ) then

      self._values["gem"]                    = gemBank;
      self._values["gold"]                   = goldBank;
      self._values["weekly-reset-timestamp"] = self.time;

      return self._values;
   end

end

----------------------------------------------------------------------

function DataTransactionUpdateLeague:onSuccess( data )

   Debug.assertType(self.rank, "number", "DataTransactionUpdateLeague:onSuccess -> self.rank");
   Debug.assertType(self.gem, "number", "DataTransactionUpdateLeague:onSuccess -> self.gem");
   Debug.assertType(self.gold, "number", "DataTransactionUpdateLeague:onSuccess -> self.gold");

   if self.gem > 0 or self.gold > 0 and not self.reset then

      MOAIFlurry:event( "Weekly-Reward",
                        "Rank",        tostring(self.rank),
                        "Reward-Gem",  tostring(self.gem),
                        "Reward-Gold", tostring(self.gold) );

      local firstLaunch = App.data:getValue("firstLaunch") or os.time();
      local since       = math.floor((os.time()-firstLaunch)/(24*60*60));

      EventMgr:postEvent( { leagueIndex = self.leagueIndex,
                            divisionIndex = self.divisionIndex,
                            rank = self.rank,
                            rewardGem = self.gem,
                            rewardGold = self.gold,
                            weeksActive = App.data:getValue("activeWeekCount", 1),
                            daysInstalled = since },
         "weekly-reward" );

   end

end

----------------------------------------------------------------------
