require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

local HACK_CAN_CHANGE_MISSION_INDEX = true;
----------------------------------------------------------------------

local Base                    = DataTransaction;
DataTransactionUpdateMission  = Base:_new("DataTransactionUpdateMission");

----------------------------------------------------------------------

function DataTransactionUpdateMission:new( data,  missionIndex, skipped  )

   local task  = Base.new(  self );
   --save the cost and reward here in case we tweak and the user retry the transaction
   task.missionIndex    = missionIndex;
   task.skipped         = skipped;
   
   task.goalGp1       = data:getValue(data:createMissionGoalGamePlayedId(1)) or 0;
   task.goalGp2       = data:getValue(data:createMissionGoalGamePlayedId(2)) or 0;
   task.goalGp3       = data:getValue(data:createMissionGoalGamePlayedId(3)) or 0;
   
   task.goalPro1      = data:getValue(data:createMissionGoalProgressId(1)) or 0;
   task.goalPro2      = data:getValue(data:createMissionGoalProgressId(2)) or 0;
   task.goalPro3      = data:getValue(data:createMissionGoalProgressId(3)) or 0;

   task.goalGpMax     = math.max( math.max(  task.goalGp1
                                          ,  task.goalGp2)
                                          ,  task.goalGp3);         
                                 
   if skipped then

      local cost     = data:getMissionSkipCost( data:getMissionIndex() or 1 ) or  {};
      task.costGold  = (cost.gold or 0);
      task.costGem   = (cost.gem  or 0);

   else

      local reward    = data:getMissionReward( data:getMissionIndex() or 1 ) or  {};
      task.rewardGold = (reward.gold or 0);
      task.rewardGem  = (reward.gem  or 0);
      
   end

   return task;

end

----------------------------------------------------------------------

function DataTransactionUpdateMission:testMe()

   print("DataTransactionUpdateMission:testMe()");
   
end

----------------------------------------------------------------------

function DataTransactionUpdateMission:buildValues( data )


   if (not self.skipped) or (data:canSkipMission( data:getMissionIndex() )) then
  
      --In case more than 1 person is changing this, we want to make sure the player wont "regress"
      if HACK_CAN_CHANGE_MISSION_INDEX or self.missionIndex == ( data:getMissionIndex() + 1) then
          
         local goalId1     = data:createMissionGoalProgressId(1);
         local goalId2     = data:createMissionGoalProgressId(2);
         local goalId3     = data:createMissionGoalProgressId(3);

         local goalGpId1   = data:createMissionGoalGamePlayedId(1);
         local goalGpId2   = data:createMissionGoalGamePlayedId(2);
         local goalGpId3   = data:createMissionGoalGamePlayedId(3);
         
         local gold        = data:getGoldCount() or 0;
         local gem         = data:getGemCount()  or 0;
         
         if self.skipped then
           gem   = math.max(gem-self.costGem,  0);
           gold  = math.max(gold-self.costGold,0);
         else
           gem   = math.max(gem+self.rewardGem,  0);
           gold  = math.max(gold+self.rewardGold,0);
         end
                                                      
         --Save
         self._values                     = {};
         self._values["gold"]             = gold;
         self._values["gem"]              = gem;
         self._values["missionIndex"]     = self.missionIndex;
         self._values[goalId1]            = 0;
         self._values[goalId2]            = 0;
         self._values[goalId3]            = 0;
         self._values[goalGpId1]          = 0;
         self._values[goalGpId2]          = 0;
         self._values[goalGpId3]          = 0;

         return self._values;
         
      end
   end
end

----------------------------------------------------------------------

function DataTransactionUpdateMission:formatGamePlayed( gp )

   local val;

   --Rounding a little bit as well
   if gp >= 100 then
      val =  math.max(math.floor((gp+2)/10)*10,1);
   else
      val = math.max(math.floor((gp+2)/5)*5,1);
   end

   return val;

end

----------------------------------------------------------------------

function DataTransactionUpdateMission:onSuccess( data )

   local prevIndex = data:getMissionIndex()-1;
   local event     = "Mission-Group-V3-"   .. math.floor( prevIndex / 10 );
   local param     = "Mission-"            .. prevIndex          .. "-GP";
   --Add 1 becasue the game the mission was finished is not precalculated
   -- i.e. if a useer complete in his 1st game, it will report 0
   local value     =             tostring(self:formatGamePlayed(self.goalGpMax))
                     ..  "=" ..  tostring(self:formatGamePlayed(self.goalGp1))
                     ..  "-" ..  tostring(self:formatGamePlayed(self.goalGp2))
                     ..  "-" ..  tostring(self:formatGamePlayed(self.goalGp3));

   if self.skipped then

      --if the user skipped, just says so
      value = "skipped";

      if self.costGem > 0 then

         EventMgr:postEvent({ reason    = "skip-mission",
                              cost      = self.costGem  },
                              "Spend-Gem"                            );

      end

      if self.costGold > 0 then
         MOAIFlurry:event(    "Spend-Gold",
                              "type",           "skip-mission",
                              "amount-gold",    ""..self.costGold       );
      end
      
      EventMgr:postEvent( { item="skip"}, "buy-success" );
            
   else

      if self.rewardGold  > 0 then      
         MOAIFlurry:event(    "More-Gold",
                              "type",          "mission-reward",
                              "amount-gold",   ""..self.rewardGold         );
      end

      if self.rewardGem > 0 then

         EventMgr:postEvent( { type="mission-reward", gem=self.rewardGem }, "More-Gem" );

      end

      EventMgr:postEvent( { }, "mission-reward-collected" );

   end

	MOAIFlurry:event(    event, param, value     );

end

----------------------------------------------------------------------
