require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                       = DataTransaction;
DataTransactionUpgradeArmor      = Base:_new("DataTransactionUpgradeArmor");

----------------------------------------------------------------------

function DataTransactionUpgradeArmor:new( data, index )

   assert(data);
   
   local task     = Base.new(  self, data );
   local id       = data:createArmorId(index);
   task.index     = index;
   task.level     = data:getValue(id);
   
   local cost     = data:getPlayerArmorUpgradeCost( index ) or {};

   task.costGold  = cost.gold    or 0;
   task.costGem   = cost.gem     or 0;
   
   task.level     = data:getValue(id);
   task.levelMax  = data:getPlayerArmorMaxLevel( index );
   
   task.first     = (data:getValue("garage-use-count") or 0) <= 0;
   return task;

end

----------------------------------------------------------------------

function DataTransactionUpgradeArmor:testMe()

   print("DataTransactionUpgradeArmor:testMe()");
   
end

----------------------------------------------------------------------

function DataTransactionUpgradeArmor:buildValues(data)

   if  data:canUpgradePlayerArmor( self.index ) then

      local gold        = data:getGoldCount()   or 0;
      local gem         = data:getGemCount()    or 0;

      if       self.costGold <= gold
         and   self.costGem  <= gem  then

         self._values    =  {};
         local id        = data:createArmorId(self.index);
         local level     = data:getValue(id);
            
         if self.level == level then
            
            local levelNew  = self.level+1;

            if levelNew <= self.levelMax then
                                 
               self._values["gold"]       = math.max( gold-self.costGold,    0              );
               self._values["gem"]        = math.max( gem-self.costGem,      0              );
               self._values[id]           = math.min(levelNew,             self.levelMax  ) ;
               self._values["garage-use-count"] = (data:getValue("garage-use-count") or  0 ) + 1;
            
            end
         end
            
         --As long as this transaction should not be be redone, mark it as done
         return self._values;

      end
   end
end

----------------------------------------------------------------------

function DataTransactionUpgradeArmor:onSuccess( data )

   EventMgr:postEvent({ item     = "armor",
                        vehicle  = self.index,
                        level    = self.level,
                        levelMax = self.levelMax     },
                        "upgrade-success"    );
            
   if self.first then
      MOAIFlurry:event(    "Use-Garage-First",
                           "type",              "upgrade-armor",
                           "game Played",       "" .. data:getGamePlayed());
   end

   MOAIFlurry:event(    "Use-Garage",
                        "type",              "upgrade-armor",
                        "count",             "".. self._values["garage-use-count"]     );
                

                                                      
   MOAIFlurry:event(    "Uprgade-Vehicle-Armor",
                        "vehicle",     ""..self.index,
                        "cost-gold",   ""..self.costGold,
                        "cost-gem",    ""..self.costGem,
                        "level",       ""..self.level+1,
                        "levelMax",    ""..self.levelMax               );
                                    
   if self.costGem > 0 then

      EventMgr:postEvent({ reason    = "upgrade-armor",
                           cost      = self.costGem  },
                           "Spend-Gem"                            );

   end

   if self.costGold > 0 then
      MOAIFlurry:event(    "Spend-Gold",
                           "type",          "upgrade-armor",
                           "amount-gold",   ""..self.costGold             );
   end

end

----------------------------------------------------------------------
