require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

local DRONE_LEFT					= "left";  -- mirror in app data
local DRONE_RIGHT					= "right"; -- mirror in app data

----------------------------------------------------------------------

local Base							= DataTransaction;
DataTransactionUpgradeDrone	= Base:_new("DataTransactionUpgradeDrone");

----------------------------------------------------------------------

function DataTransactionUpgradeDrone:new( data, index, price, side )

	if			index
		and	side
		and	price
		and	type(price) == "table"
		and	price.currency
		and	price.cost then

		local task				= Base.new( self );

		task.index				= index;
		task.side				= side;
		task.currency			= price.currency or "gold";
		task.cost				= price.cost or 100000;

		return task;
	end

end

----------------------------------------------------------------------

function DataTransactionUpgradeDrone:buildValues( data )

   Debug.assert(self.index, "DataTransactionUpgradeDrone:buildValues -> self.index nil");
   Debug.assert(self.side, "DataTransactionUpgradeDrone:buildValues -> self.side nil");
   Debug.assert(self.currency, "DataTransactionUpgradeDrone:buildValues -> self.currency nil");
   Debug.assert(self.cost, "DataTransactionUpgradeDrone:buildValues -> self.cost nil");

	self._values		= {};
	self.upgradeDrone	= false;

	if		( self.side == DRONE_LEFT and DroneMgr:canUpgradeDroneLeft(self.index) )
		or	( self.side == DRONE_RIGHT and DroneMgr:canUpgradeDroneRight(self.index) ) then

		self.upgradeDrone	= true;

        local droneType, rarity, level = DroneMgr:decodeDroneData(data:getValue(data:createDroneId(self.index, self.side)));
        self._values[data:createDroneId(self.index, self.side)] = DroneMgr:encodeDroneData(droneType, rarity, level + 1);

        self._values[self.currency] = data:getValue(self.currency, 0) - self.cost;
	end

	return self._values;
end


----------------------------------------------------------------------

function DataTransactionUpgradeDrone:onSuccess( data )

   if self.upgradeDrone then

      Debug.assert(self.index, "DataTransactionUpgradeDrone:onSuccess -> self.index nil");
      Debug.assert(self.side, "DataTransactionUpgradeDrone:onSuccess -> self.side nil");
      Debug.assert(self.currency, "DataTransactionUpgradeDrone:onSuccess -> self.currency nil");
      Debug.assert(self.cost, "DataTransactionUpgradeDrone:onSuccess -> self.cost nil");

		if self.currency == "gold" and self.cost > 0 then
			MOAIFlurry:event(	"Spend-Gold",
									"type",				"upgrade-drone",
									"amount-gold",		""..self.cost	);
		end

		local lvl	= math.floor((data:getCurrentPlayerLevel() or 1) / 10);
		local gold	= self.cost;
		local id;

		if self.side == DRONE_LEFT then
			id	= DroneMgr:getDroneName(	DroneMgr:getDroneTypeLeft(self.index), DroneMgr:getDroneRarityLeft(self.index)	);
		else
			id	= DroneMgr:getDroneName(	DroneMgr:getDroneTypeRight(self.index), DroneMgr:getDroneRarityRight(self.index)	);
		end

		EventMgr:postEvent(	{	level	= lvl,
										cost	= gold,
										dType	= id	},
									"drone-upgrade"	);
   end

end

----------------------------------------------------------------------
