require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                       = DataTransaction;
DataTransactionUpgradeMainWeapon = Base:_new("DataTransactionUpgradeMainWeapon");

print("_G[DataTransactionUpgradeMainWeapon]",DataTransactionUpgradeMainWeapon, _G, _G.DataTransactionUpgradeMainWeapon, _G["DataTransactionUpgradeMainWeapon"] );
print("more", _G[DataTransactionUpgradeMainWeapon]);
----------------------------------------------------------------------

function DataTransactionUpgradeMainWeapon:new( data, index )

   assert(data);
   assert(index);
   
   local task     = Base.new(  self, data );
   task.index     = index;
   task.level     = data:getValue(data:createMainWeaponId(index));
   task.levelMax  = data:getPlayerMainWeaponMaxLevel( index );
   
   local cost     = data:getPlayerMainWeaponUpgradeCost( index ) or {};
   task.costGold  = cost.gold or 0;
   task.costGem   = cost.gem  or 0;
   
   task.first     = (data:getValue("garage-use-count") or 0) <= 0;

   return task;

end


function DataTransactionUpgradeMainWeapon:testMe()

   return "DataTransactionUpgradeMainWeapon:testMe";
end

----------------------------------------------------------------------

function DataTransactionUpgradeMainWeapon:buildValues(data)

   if  data:canUpgradePlayerMainWeapon( self.index ) then

      local gold        = data:getGoldCount()   or 0;
      local gem         = data:getGemCount()    or 0;

      if       self.costGold <= gold
         and   self.costGem  <= gem  then

         self._values    =  {};
         local id        = data:createMainWeaponId(self.index);
         local level     = data:getValue(id);
         
         if self.level == level then
         
            local levelNew  = level+1;

            if levelNew <= self.levelMax then
                           
               self._values["gold"]             = math.max( gold-self.costGold,    0        );
               self._values["gem"]              = math.max( gem-self.costGem,      0        );
               self._values[id]                 = math.min(levelNew,          self.levelMax ) ;
               self._values["garage-use-count"] = (data:getValue("garage-use-count") or  0 ) + 1;
            
            end
         end
         
         --As long as this transaction should not be be redone, mark it as done
         return self._values;

      end
   end
end


----------------------------------------------------------------------

function DataTransactionUpgradeMainWeapon:onSuccess( data )

   EventMgr:postEvent({ item     = "main-weapon",
                        vehicle  = self.index,
                        level    = self.level,
                        levelMax = self.levelMax     },
                        "upgrade-success"    );
          
   if self.first then
      MOAIFlurry:event(    "Use-Garage-First",
                           "type",              "upgrade-main",
                           "game Played",       "" .. data:getGamePlayed() );
   end
                   
   MOAIFlurry:event(    "Use-Garage",
                        "type",              "upgrade-main",
                        "count",             "".. self._values["garage-use-count"]    );
                
                                  
   MOAIFlurry:event(    "Uprgade-Vehicle-Main-Weapon",
                        "vehicle",     ""..self.index,
                        "cost-gold",   ""..self.costGold,
                        "cost-gem",    ""..self.costGem,
                        "level",       ""..self.level+1,
                        "levelMax",    ""..self.levelMax               );
                     
   if self.costGem > 0 then

      EventMgr:postEvent({ reason    = "upgrade-main-weapon",
                           cost      = self.costGem  },
                           "Spend-Gem"                            );

   end

   if self.costGold > 0 then
      MOAIFlurry:event(    "Spend-Gold",
                           "type",          "upgrade-main-weapon",
                           "amount-gold",    ""..self.costGold              );
   end
end

----------------------------------------------------------------------
