require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                   = DataTransaction;
DataTransactionUpgradePickup = Base:_new("DataTransactionUpgradePickup");

----------------------------------------------------------------------

function DataTransactionUpgradePickup:new( data, index )

   assert(data);
   
print("DataTransactionUpgradePickup:new", index);

   local task     = Base.new(  self, data );
   task.index     = index;
   task.level     = data:getValue(data:createPickupId(index));
   task.levelMax  = data:getPickupMaxLevel( index );
   local cost     = data:getPickupUpgradeCost( index ) or {};
   task.costGold  = cost.gold    or 0;
   task.costGem   = cost.gem     or 0;
         
   task.first     = (data:getValue("garage-use-count") or 0) <= 0;
   
   return task;

end

----------------------------------------------------------------------

function DataTransaction:testMe()

   print("DataTransactionUpgradePickup:testMe()");
   
end

----------------------------------------------------------------------

function DataTransactionUpgradePickup:buildValues(data)

   if  data:canAffordUpgradePickup( self.index ) then

      --It is Okay to be free (i.e. double coin use money but we still have to use a "free gem and gold" transcation to level up)
      local gold        = data:getGoldCount()   or 0;
      local gem         = data:getGemCount()    or 0;

      if       self.costGold <= gold
         and   self.costGem  <= gem  then

         self._values    =  {};
         local id        = data:createPickupId(self.index);
         local level     = data:getValue(id);
         
         if self.level == level then
         
            local levelNew  = level+1;

            if levelNew <= self.levelMax then
            
               self._values["gold"]             = math.max( gold-self.costGold,    0              );
               self._values["gem"]              = math.max( gem-self.costGem,      0              );
               self._values[id]                 = math.min(levelNew,              self.levelMax  ) ;
               self._values["garage-use-count"] = (data:getValue("garage-use-count") or  0 ) + 1;
           
            end
         end
         
         --As long as this transaction should not be be redone, mark it as done
         return self._values;

      end
   end
end

----------------------------------------------------------------------

function DataTransactionUpgradePickup:onSuccess( data )


   EventMgr:postEvent({ item     = "pickup",
                        pickup   = self.index,
                        level    = self.level+1,
                        levelMax = self.levelMax    },
                        "upgrade-success"                         );


   if self.index == PICKUP_COIN_INDEX then
      EventMgr:postEvent({},"double-coin-bought");
   end

   if self.first then
      MOAIFlurry:event(    "Use-Garage-First",
                           "type",              "upgrade-pickup",
                           "game Played",       "" .. data:getGamePlayed() );
   end
    
   MOAIFlurry:event(    "Use-Garage",
                        "type",              "upgrade-pickup",
                        "count",             "".. self._values["garage-use-count"]    );
       
                  
   MOAIFlurry:event(    "Upgrade-Pickup",
                        "pickup",         ""..self.index,
                        "cost-gold",      ""..self.costGold,
                        "cost-gem",       ""..self.costGem,
                        "level",          ""..(self.level+1),
                        "levelMax",       ""..self.levelMax            );

   MOAIFlurry:event(    "Upgrade-Pickup-"..self.index,
                        "cost-gold",      ""..self.costGold,
                        "cost-gem",       ""..self.costGem,
                        "level",          ""..(self.level+1),
                        "levelMax",       ""..self.levelMax            );

   if self.costGem > 0 then

      EventMgr:postEvent({ reason    = "upgrade-pickup",
                           cost      = self.costGem  },
                           "Spend-Gem"                            );

   end

   if self.costGold > 0 then
      MOAIFlurry:event(    "Spend-Gold",
                           "type",          "upgrade-pickup",
                           "amount-gold",   ""..self.costGold           );
   end

end

----------------------------------------------------------------------
