require('app/data/transaction/data-transaction')
require('app/possession/possession-manager')

----------------------------------------------------------------------

local Base                          = DataTransaction;
DataTransactionUpgradeTurretWeapon  = Base:_new("DataTransactionUpgradeTurretWeapon");

----------------------------------------------------------------------

function DataTransactionUpgradeTurretWeapon:new( data, index )

   assert(data);

print("DataTransactionUpgradeTurretWeapon:new", index);

   local task     = Base.new(  self, data );
   task.index     = index;
   task.level     = data:getValue(data:createTurretWeaponId(index));
   task.levelMax  = data:getPlayerTurretWeaponMaxLevel( index );

   local cost     = data:getPlayerTurretWeaponUpgradeCost( index ) or {};
   task.costGold  = cost.gold or 0;
   task.costGem   = cost.gem  or 0;

   task.first     = (data:getValue("garage-use-count") or 0) <= 0;

   return task;

end

----------------------------------------------------------------------

function DataTransactionUpgradeTurretWeapon:buildValues(data)

   if data:canUpgradePlayerTurretWeapon( self.index ) then

      Debug.assertType(self.level,    "number", "DataTransactionUpgradeTurretWeapon:buildValues -> self.level");
      Debug.assertType(self.levelMax, "number", "DataTransactionUpgradeTurretWeapon:buildValues -> self.levelMax");

      local gold = data:getGoldCount() or 0;
      local gem  = data:getGemCount()  or 0;

      if    not Debug.assertType(self.level, "number", "DataTransactionUpgradeTurretWeapon:buildValues -> self.level")
         or not Debug.assertType(self.levelMax, "number", "DataTransactionUpgradeTurretWeapon:buildValues -> self.levelMax") then

         return;
      end

      if     self.costGold  <= gold
         and self.costGem   <= gem
         and self.level     <  self.levelMax   then

         local id                = data:createTurretWeaponId(self.index);
         self._values            =  {};
         self._values["gold"]    = math.max( gold-self.costGold,    0        );
         self._values["gem"]     = math.max( gem-self.costGem,      0        );
         self._values[id]        = self.level+1;
         self._values["garage-use-count"] = (data:getValue("garage-use-count") or  0 ) + 1;

         return self._values;

      end
   end
end

----------------------------------------------------------------------

function DataTransactionUpgradeTurretWeapon:onSuccess(data)

   EventMgr:postEvent({ item     = "turret-weapon",
                        vehicle  = self.index,
                        level    = self.level,
                        levelMax = self.levelMax     },
                        "upgrade-success"    );

   if self.first then
      MOAIFlurry:event(    "Use-Garage-First",
                           "type",              "upgrade-turret",
                           "game Played",       "" .. data:getGamePlayed());
   end

   MOAIFlurry:event(    "Use-Garage",
                        "type",              "upgrade-turret",
                        "count",             "".. self._values["garage-use-count"]   );

   MOAIFlurry:event(    "Uprgade-Vehicle-Turret-Weapon",
                        "vehicle",     ""..self.index,
                        "cost-gold",   ""..self.costGold,
                        "cost-gem",    ""..self.costGem,
                        "level",       ""..self.level+1,
                        "levelMax",    ""..self.levelMax                  );

   if self.level == 0 then

      MOAIFlurry:event(    "Unlock-Vehicle-Turret-Weapon",
                           "vehicle",                 ""..self.index,
                           "cost-gold",               ""..self.costGold,
                           "cost-gem",                ""..self.costGem,
                           "player-level",            "" .. data:getCurrentPlayerLevel(),
                           "vehicle-player-level",    "v-" ..self.index .. "-l-" .. data:getCurrentPlayerLevel()      );
   end

   if self.costGem > 0 then

      EventMgr:postEvent({ reason    = "upgrade-turret-weapon",
                           cost      = self.costGem  },
                           "Spend-Gem"                            );

   end

   if self.costGold> 0 then
      MOAIFlurry:event(    "Spend-Gold",
                           "type",          "upgrade-turret-weapon",
                           "amount-gold",   ""..self.costGold              );
   end

end

----------------------------------------------------------------------

function DataTransactionUpgradeTurretWeapon:testMe()

   print("DataTransactionUpgradeTurretWeapon:testMe()");

end
----------------------------------------------------------------------
