
Debug = Debug or {};

if BUILD_DEBUG then

   -------------------------------------------------------------------------------------

   local function  _countGlobalMarkerRecursive( marker, toBeIterated, visitedTables )

      local count = 0;

      if type(toBeIterated) == "userdata" then
         local mt = getmetatable(toBeIterated)

         if not mt then
            return count;
         end

         toBeIterated = mt
      end

      for k,v in pairs(toBeIterated) do

         if k == "__marker" and  v == marker then
            count = count + 1;
         end

         if type(v) == "table" or type(v) == "userdata" then
            if not visitedTables[v] then
               visitedTables[v] = true;
               count = count + _countGlobalMarkerRecursive(marker, v, visitedTables);
            end
         end

      end

      return count;

   end

   Debug.countGlobalMarker = function ( marker )

      local count = 0;
      return _countGlobalMarkerRecursive( marker, _G, {} );

   end

   -------------------------------------------------------------------------------------

   local function  _findGlobalMarkerObjectRecursive( marker, ownerTbl, toBeIterated, visitedTables )

      if type(toBeIterated) == "userdata" then
         local mt = getmetatable(toBeIterated)

         if not mt then
            return count;
         end

         toBeIterated = mt
      end

      for k,v in pairs(toBeIterated) do

         if k == "__marker" and  v == marker then
            table.insert(ownerTbl, toBeIterated);
         end

         if type(v) == "table" or type(v) == "userdata" then
            if not visitedTables[v] then
               visitedTables[v] = true;
               _findGlobalMarkerObjectRecursive(marker, ownerTbl, v, visitedTables);
            end
         end

      end

      return ownerTbl;

   end

   Debug.findGlobalMarkerObject = function ( marker )

      local ownerTbl = {};
      return _findGlobalMarkerObjectRecursive( marker, ownerTbl, _G, {} );

   end

   -------------------------------------------------------------------------------------

   local function  _findTableOwnerRecursive( tgt, ownerTbl, toBeIterated, visitedTables )

      if type(toBeIterated) == "userdata" then
         local mt = getmetatable(toBeIterated)

         if not mt then
            return count;
         end

         toBeIterated = mt
      end

      for k,v in pairs(toBeIterated) do

         if v == tgt then
            table.insert(ownerTbl, toBeIterated);
         end

         if type(v) == "table" or type(v) == "userdata" then
            if not visitedTables[v] then
               visitedTables[v] = true;
               _findTableOwnerRecursive(tgt, ownerTbl, v, visitedTables);
            end
         end
      end

      return ownerTbl;

   end

   Debug.findGlobalTableOwner = function ( tgt )

      local ownerTbl = {};
      return _findTableOwnerRecursive( tgt, ownerTbl, _G, {} );

   end

   -------------------------------------------------------------------------------------

   local function  _printTableNameRecursive( tgt, toBeIterated, visitedTables, name )

      if type(toBeIterated) == "userdata" then
         local mt = getmetatable(toBeIterated)

         if not mt then
            return count;
         end

         toBeIterated = mt
      end

      for k,v in pairs(toBeIterated) do

         if v == tgt then
            print(name .. "."  .. tostring(k) );
         end

         if type(v) == "table" or type(v) == "userdata" then
            if not visitedTables[v] then
               visitedTables[v] = true;
               _printTableNameRecursive( tgt, v, visitedTables, name .. "." .. tostring(k) );
            end
         end
      end

   end

   Debug.printGlobalTableName = function ( tgt )
      _printTableNameRecursive( tgt, _G, {}, ""  );
   end

   -------------------------------------------------------------------------------------

   local PRINT_TABLE_INDENTION = "  "

   -- Call like so: printTable(myTable, true)
   function Debug.printTable(t, recursive, visited, depth)

	  Debug.assert(visited == nil or type(visited) == "table")
	  Debug.assert(depth == nil or type(depth) == "number")

	  visited = visited or {}
	  depth = depth or 0

	  visited[t] = true;

	  local rep = string.rep

	  for k, v in pairs(t) do
		 print(rep(PRINT_TABLE_INDENTION, depth) .. tostring(k) .. " = " .. tostring(v))

		 if recursive and type(v) == "table" and not visited[v] then
			printTable(v, recursive, visited, depth + 1)
		 end
	  end
   end

   -------------------------------------------------------------------------------------

   Debug.error = function( message )
      assert(nil, message);
   end

   Debug.assert = function( test, message )
      assert(test, message);
   end

   -------------------------------------------------------------------------------------

   Debug.testTableEquality = function( tbl1, tbl2 )
      return Debug._testTableEquality(tbl1, tbl2) and Debug._testTableEquality(tbl2, tbl1);
   end

   Debug._testTableEquality = function( tbl1, tbl2 )

      Debug.assert(tbl1 and tbl2, "One of the tables was nil!");

      for k, v in pairs(tbl1) do
         if type(v) == "table" then

            -- Ignore keys starting with __
            if type(k) ~= "string" or not string.find(k, "__", 1, true) then
               Debug.assert(tbl2[k], "Table \"" .. k .. "\" didn't exist!")

               -- DEBUG:
               --print("Debug: recursing into: " .. k .. " for equality check.");

               if not Debug._testTableEquality(v, tbl2[k]) then
                  return false
               end
            end
         else
            if v ~= tbl2[k] then
               if v == nil or tbl2[k] == nil then
                  print("Key \"" .. k .. "\": Only one of the values was nil!")
               else
                  print("Key \"" .. k .. "\" was not equal! Values: "
                           .. v .. " ~= " .. tbl2[k])
               end

               return false
            end
         end
      end

      return true;
   end

   -------------------------------------------------------------------------------------

   Debug.printStack = function()
      print( debug.traceback() );
   end

   -------------------------------------------------------------------------------------

   -------------------------------------------------------------------------------------
   -- RUN TESTS ON DEBUG FUNCTIONS:
   -------------------------------------------------------------------------------------

   -- Test: testTableEquality
   local testTbl1 = { a=1, b=2, c={ d="bob", e={x=1, y=2, z=5} } };
   local testTbl2 = { a=1, b=2, c={ d="bob", e={x=1, y=2, z=5} } };
   local testTbl3 = { a=1, b=2, c={ d="bobby", e={x=1, y=-3.4, z=5} } };
   Debug.assert(Debug.testTableEquality(testTbl1, testTbl2), "The two tables were not equal!");
   Debug.assert(not Debug.testTableEquality(testTbl1, testTbl3), "The two tables were not equal!");

else

   Debug.countGlobalMarker = function ( marker )

   end

   Debug.assert = function( test, message )
      if not test then
         moai.traceback( "Debug.assert:" .. (message or "") );
      end
   end

   Debug.error = function( message )
      moai.traceback( "Debug.error:" .. tostring(message or "") );
   end

   Debug.printStack = function()

   end

   function Debug.printTable(t, recursive, visited, depth)

   end

end

Debug._validTypes = {};
Debug._validTypes["nil"]      = true;
Debug._validTypes["number"]   = true;
Debug._validTypes["string"]   = true;
Debug._validTypes["boolean"]  = true;
Debug._validTypes["table"]    = true;
Debug._validTypes["function"] = true;
Debug._validTypes["thread"]   = true;
Debug._validTypes["userdata"] = true;

function Debug.isValidTypeString( typeStr )

   -- Written this way to avoid returning a nil
   if Debug._validTypes[typeStr] then
      return true;
   end

   return false;
end

function Debug._createBadTypeMessage(value, typeStr, optVariableName)
   optVariableName = optVariableName or "variable";

   local expected = "";
   if type(typeStr) == "string" then
      expected = typeStr;
   elseif type(typeStr) == "table" then
      expected = table.concat(typeStr, " or ");
   end

   return "Invalid value given for \"" .. tostring(optVariableName) .. "\". (Expected " .. expected .. ", got " .. type(value) .. "!)"
end

-- Asserts that the given value is of specified type.
-- Optionally accepts a variable name for better error messages.
-- Exists in both debug and non-debug builds because Debug.assert handles itself.
function Debug.assertType(value, typeStr, optVariableName)

   -- Handle optional variables
   optVariableName = optVariableName or "variable";

   if type(typeStr) == "string" then

      -- Is typeStr a valid type string?
      Debug.assert(Debug.isValidTypeString(typeStr), "Bad typestring: " .. tostring(typeStr));

      -- Compare type to type string
      Debug.assert(type(value) == typeStr, Debug._createBadTypeMessage(value, typeStr, optVariableName));

      return type(value) == typeStr;

   elseif type(typeStr) == "table" then

      -- Iterate though types - we just need to match one
      for _, v in pairs(typeStr) do

         Debug.assert(Debug.isValidTypeString(v), "Bad typestring: " .. tostring(v));

         if type(value) == v then
            return true;
         end
      end

      -- No match
      Debug.error(Debug._createBadTypeMessage(value, typeStr, optVariableName));
      return false;
   end

   -- No match
   Debug.error("Debug.assertType -> typeStr is a bad type.");
   return false;

end

-------------------------------------------------------------------------------------

local blacklistedGlobalTables = {
   "string",
   "table",
   "math",
   "os",
   "debug",
   "io",
}

local blacklistedFunctionNames = {
   "(for generator)",
   "type",
   "tostring",
   "pairs",
   "ipairs",
   "tonumber",
   "assert",
   "setmetatable",
   "print",
}

local blacklistedFiles = {
   "engine/json/json",
}

-- Example: [string "app/ranking/ranking-manager"]
local function getFileNameFromInfo(funcInfo)

   return string.match(funcInfo.short_src, "%[.*\"(.*)\"%]");
end

local function shouldIgnore(funcInfo)

   for _, v in ipairs(blacklistedGlobalTables) do
      if type(_G[v]) == "table" and type(_G[v][funcInfo.name]) == "function" then
         return true;
      end
   end

   for _, v in ipairs(blacklistedFunctionNames) do
      if funcInfo.name == v then
         return true;
      end
   end

   local fileName = getFileNameFromInfo(funcInfo)
   for _, v in ipairs(blacklistedFiles) do
      if fileName == v then
         return true;
      end
   end

   return false;
end

local function debugHook(event)

   local info = debug.getinfo(2);

   if not shouldIgnore(info) then
      print("Calling function: ", tostring(info.name), tostring(info.short_src));
   end

end

function Debug.setDebugEnabled(enabled)
   if enabled then
      debug.sethook(debugHook, "c");
   else
      debug.sethook()
   end
end
