require('engine/utility/random');

----------------------------------------------------------------------

local MIN							= 60;
local HOUR							= 60*60;
local DAY							= 60*60*24;
local UPDATE_TIME					= 1;

-- duplicated in data transaction and avatar
local CURRENCY_GEM				= "gem";
local CURRENCY_GOLD				= "gold";
local GARAGE_SPACE_LOCKED		= -1;
local GARAGE_SPACE_EMPTY		= 0;
local MOUNT_POINT_LOCKED		= -1;
local MOUNT_POINT_EMPTY			= 0;
local MOUNT_POINT_LEFT			= "left";
local MOUNT_POINT_RIGHT			= "right";

local DRONE_LEFT					= "left";  -- mirror in app data
local DRONE_RIGHT					= "right"; -- mirror in app data

local DUPE_RESOLVE_MERGE		= "auto-merge";
local DUPE_RESOLVE_EVOLVE		= "evolve";
local DUPE_RESOLVE_HYBRID		= "hybrid";

local DEFAULT_DATA_TABLE		= dofile("app/template/drone/drone-default-tables.lua");
local DRONE_INFO_CODE_SHIFT   = 128; -- Indicates the number of possible values per variable

if not BUILD_DEBUG then
end

----------------------------------------------------------------------

DroneMgr  =
{
	_enabled						= true,
	_deviceTime					= MOAISim.getDeviceTime(),
	_droneMinLevel				= DEFAULT_DATA_TABLE.droneMinLevel,
	_droneDupeResolve			= DEFAULT_DATA_TABLE.droneDupeResolve,
	_garageMaxSpace			= DEFAULT_DATA_TABLE.droneMaxGarageSpace,
	_droneMountCost			= DEFAULT_DATA_TABLE.droneMountUnlockCost,
	_droneBuildCost			= DEFAULT_DATA_TABLE.droneBuildCost,
	_droneUpgradeCost			= DEFAULT_DATA_TABLE.droneUpgradeCost,
	_droneEvolveCost			= DEFAULT_DATA_TABLE.droneEvolveCost,
	_abilityOdds				= DEFAULT_DATA_TABLE.droneAbilityOdds,
	_abilityOddsK				= DEFAULT_DATA_TABLE.droneAbilityOddsK,
	_buildOrder					= DEFAULT_DATA_TABLE.droneBuildOrder,
	_rarityOdds					= DEFAULT_DATA_TABLE.droneRarityOddsNorm,
	_droneStats					= DEFAULT_DATA_TABLE.droneStats,
   _cloudStatsMap          =
	{
		"gun",
		"laser",
		"boost",
		"missile",
		"rocket",
		"repair",
		"shield",
      "barricade",
	},
	_droneBaseTmpltTbl		=
	{
		"app/template/drone/drone-gun-base.lua",
		"app/template/drone/drone-laser-base.lua",
		"app/template/drone/drone-boost-base.lua",
		"app/template/drone/drone-missile-base.lua",
		"app/template/drone/drone-rocket-base.lua",
		"app/template/drone/drone-repair-base.lua",
		"app/template/drone/drone-shield-base.lua",
      "app/template/drone/drone-barricade-base.lua",
	},
	_droneBodyVisTbl			=
	{
		"app/template/drone/drone-gun-body-texture.lua",
		"app/template/drone/drone-laser-body-texture.lua",
		"app/template/drone/drone-boost-body-texture.lua",
		"app/template/drone/drone-missile-body-texture.lua",
		"app/template/drone/drone-rocket-body-texture.lua",
		"app/template/drone/drone-repair-body-texture.lua",
		"app/template/drone/drone-shield-body-texture.lua",
      "app/template/drone/drone-barricade-body-texture.lua",
	},
	_droneAbltyVisTbl			=
	{
		"app/template/drone/drone-gun-proj-texture.lua",
		"app/template/drone/drone-laser-proj-texture.lua",
		"app/template/drone/drone-boost-proj-texture.lua",
		"app/template/drone/drone-missile-proj-texture.lua",
		"app/template/drone/drone-rocket-proj-texture.lua",
		"app/template/drone/drone-repair-proj-texture.lua",
		"app/template/drone/drone-shield-proj-texture.lua",
      "app/template/drone/drone-barricade-proj-texture.lua",
	},
};

----------------------------------------------------------------------

function DroneMgr:init()

   EventMgr:addListener( self, self._onUpdateMasterDroneInfo,     "cloud:master-data-drone-info"			);
   EventMgr:addListener( self, self._onUpdateFileDroneInfo,       "cloud:file-data-drone-info"				);

end


----------------------------------------------------------------------
-- Updates and Init Callbacks
----------------------------------------------------------------------


function DroneMgr:_onUpdateMasterDroneInfo( data )

   local content = (data or {}).content;

   if content then

      if content.enable ~= nil then
         self._enabled					= content.enable;
      end

      self._droneMinLevel			= content.droneMinLevel or self._droneMinLevel;
      self._droneDupeResolve		= content.droneDupeResolve or self._droneDupeResolve;
      self._garageMaxSpace			= content.droneMaxGarageSpace or self._garageMaxSpace;
      self._droneMountCost			= content.droneMountUnlockCost or self._droneMountCost;
      self._droneBuildCost			= content.droneBuildCost or self._droneBuildCost;
      self._droneFinishCost		= content.droneBuildFinishCost or self._droneFinishCost;
      self._droneUpgradeCost		= content.droneUpgradeCost or self._droneUpgradeCost;
      self._droneEvolveCost		= content.droneEvolveCost or self._droneEvolveCost;
      self._abilityOdds				= content.droneAbilityOdds or self._abilityOdds;
      self._abilityOddsK			= content.droneAbilityOddsK or self._abilityOddsK;
      self._buildOrder				= content.droneBuildOrder or self._buildOrder;
      self._rarityOdds				= content.droneRarityOdds or self._rarityOdds;
   end

end

----------------------------------------------------------------------

function DroneMgr:_onUpdateFileDroneInfo( data )

   local content	= (data or {}).content;

   if content then
		self._droneStats = content or self._droneStats;
   end

end


----------------------------------------------------------------------
-- Debug Test Functions
----------------------------------------------------------------------


----------------------------------------------------------------------
-- Data Tables
----------------------------------------------------------------------


function DroneMgr:isEnabled()

	if AbTestMgr:getDroneEnabled() ~= nil then
		return AbTestMgr:getDroneEnabled();
	end

	return self._enabled;

end

----------------------------------------------------------------------

function DroneMgr:isDroneLevelLocked()

	if App.data:getCurrentPlayerLevel() < ( AbTestMgr:getDroneMinLevel() or self._droneMinLevel ) then
		return true;
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:isDroneDupeResolveModeEvolve()

	if AbTestMgr:getDroneDupeResolve() then
		return AbTestMgr:getDroneDupeResolve() == DUPE_RESOLVE_EVOLVE;
	end

	return self._droneDupeResolve == DUPE_RESOLVE_EVOLVE;

end

----------------------------------------------------------------------

function DroneMgr:isDroneDupeResolveModeAutoMerge()

	if AbTestMgr:getDroneDupeResolve() then
		return AbTestMgr:getDroneDupeResolve() == DUPE_RESOLVE_MERGE;
	end

	return self._droneDupeResolve == DUPE_RESOLVE_MERGE;

end

----------------------------------------------------------------------

function DroneMgr:isDroneDupeResolveModeHybrid()

	if AbTestMgr:getDroneDupeResolve() then
		return AbTestMgr:getDroneDupeResolve() == DUPE_RESOLVE_HYBRID;
	end

	return self._droneDupeResolve == DUPE_RESOLVE_HYBRID;

end

----------------------------------------------------------------------

function DroneMgr:getDroneLevelUnlock()
	return AbTestMgr:getDroneMinLevel() or self._droneMinLevel;
end

----------------------------------------------------------------------

function DroneMgr:getDroneMountCostTable()
	return AbTestMgr:getDroneMountCost() or self._droneMountCost;
end

----------------------------------------------------------------------

function DroneMgr:getDroneBuildCostTable()
	return AbTestMgr:getDroneBuildCost() or self._droneBuildCost;
end

----------------------------------------------------------------------

function DroneMgr:getDroneUpgradeCostTable()
	return AbTestMgr:getDroneUpgradeCost() or self._droneUpgradeCost;
end

----------------------------------------------------------------------

function DroneMgr:getDroneEvolveCostTable()
	return AbTestMgr:getDroneEvolveCost() or self._droneEvolveCost;
end

----------------------------------------------------------------------

function DroneMgr:getDroneTypeOddsTable()
	return AbTestMgr:getDroneAbilityOdds() or self._abilityOdds;
end

----------------------------------------------------------------------

function DroneMgr:getDroneTypeOddsScaleTable()
	return AbTestMgr:getDroneAbilityOddsK() or self._abilityOddsK;
end

----------------------------------------------------------------------

function DroneMgr:getDroneBuildOrderTable()
	return AbTestMgr:getDroneBuildOrder() or self._buildOrder;
end

----------------------------------------------------------------------

function DroneMgr:getDroneRarityOddsTable()
	return AbTestMgr:getDroneRarityOdds() or self._rarityOdds;
end

----------------------------------------------------------------------

function DroneMgr:getDroneStatsTable()
	return self._droneStats;
end


----------------------------------------------------------------------
-- Saved Data
----------------------------------------------------------------------

function DroneMgr:getDroneGarageSpaceMax()
	return math.max(self._garageMaxSpace, App.data:getDroneCount());
end

----------------------------------------------------------------------

--NOTE: only use this if you want to start with a min spaces unlocked otherwise defaults to 0
function DroneMgr:getDroneGarageSpaceMin()
	return App.data:getDroneMinCount();
end

----------------------------------------------------------------------

function DroneMgr:getDroneTypeLeft( garage_index )
	return App.data:getDroneTypeLeft(garage_index);
end

----------------------------------------------------------------------

function DroneMgr:getDroneTypeRight( garage_index )
	return App.data:getDroneTypeRight(garage_index);
end

----------------------------------------------------------------------

function DroneMgr:getDroneRarityLeft( garage_index )
	return App.data:getDroneRarityLeft(garage_index);
end

----------------------------------------------------------------------

function DroneMgr:getDroneRarityRight( garage_index )
	return App.data:getDroneRarityRight(garage_index);
end

----------------------------------------------------------------------

function DroneMgr:getDroneLevelLeft( garage_index )
	return App.data:getDroneUpgradeLevelLeft(garage_index);
end

----------------------------------------------------------------------

function DroneMgr:getDroneLevelRight( garage_index )
	return App.data:getDroneUpgradeLevelRight(garage_index);
end

----------------------------------------------------------------------

function DroneMgr:getDroneBuiltCountLeft()
	return App.data:getDroneBuiltCountLeft() or 0;
end

----------------------------------------------------------------------

function DroneMgr:getDroneBuiltCountRight()
	return App.data:getDroneBuiltCountRight() or 0;
end

----------------------------------------------------------------------

function DroneMgr:getDroneDefaultLevel()
	return App.data:getDroneDefaultLevel();
end

----------------------------------------------------------------------

function DroneMgr:getDroneMinLevel()
	return App.data:getDroneMinLevel();
end

----------------------------------------------------------------------

function DroneMgr:getDroneMaxLevel()
	return App.data:getDroneMaxLevel();
end

----------------------------------------------------------------------

function DroneMgr:getDroneMaxRarity()
	return App.data:getDroneMaxRarity();
end

----------------------------------------------------------------------

function DroneMgr:resetDroneStatsLeft( garage_index )
	self:updateDroneStatsLeft( garage_index, GARAGE_SPACE_EMPTY, 1, 1 );
end

----------------------------------------------------------------------

function DroneMgr:resetDroneStatsRight( garage_index )
	self:updateDroneStatsRight( garage_index, GARAGE_SPACE_EMPTY, 1, 1 );
end

----------------------------------------------------------------------

function DroneMgr:updateDroneStatsLeft( garage_index, dType, rarity, level, cost )
	App.data:updateDroneStatsLeft( garage_index, dType, rarity, level, cost );
end

----------------------------------------------------------------------

function DroneMgr:updateDroneStatsRight( garage_index, dType, rarity, level, cost )
	App.data:updateDroneStatsRight( garage_index, dType, rarity, level, cost );
end


----------------------------------------------------------------------
-- Utility
----------------------------------------------------------------------


function DroneMgr:anyDroneCollected()

	local maxDrones	= math.min( #self._droneBaseTmpltTbl, #self:getDroneTypeOddsTable() );

	for i=1, maxDrones do
		if self:isDroneGarageSpaceFilledLeft(i) or self:isDroneGarageSpaceFilledRight(i) then
			return true;
		end
	end

end

----------------------------------------------------------------------

function DroneMgr:getDroneDifficultyMod( scale )

	local indexL	= self:isDroneMountLeftAssigned();
	local indexR	= self:isDroneMountRightAssigned();
	local score		= 0;
	local k			= scale or 1;

	if indexL then
		score = score + ( (self:getDroneRarityLeft(indexL) - 1) * 15 + self:getDroneLevelLeft(indexL) ) / 75;
	end

	if indexR then
		score = score + ( (self:getDroneRarityRight(indexR) - 1) * 15 + self:getDroneLevelRight(indexR) ) / 75;
	end

	return score * k;

end

----------------------------------------------------------------------

function DroneMgr:getDroneTypeCount()

	local odds	= self:getDroneTypeOddsTable();

	if self._droneBaseTmpltTbl and odds then
		local count = 0;

		for k,v in ipairs(odds) do
			if v and v > 0 then
				count = count + 1;
			end
		end

		return math.min(#self._droneBaseTmpltTbl, count);
	end

	return 1;

end

----------------------------------------------------------------------

function DroneMgr:_getDroneCount(side)

	local maxDrones	= math.min(#self._droneBaseTmpltTbl, #self:getDroneTypeOddsTable());
	local count			= 0;

	if side == DRONE_LEFT then
		for i=1,maxDrones do
			if self:isDroneGarageSpaceFilledLeft(i) then
				count = count + 1;
			end
		end
	else
		for i=1,maxDrones do
			if self:isDroneGarageSpaceFilledRight(i) then
				count = count + 1;
			end
		end
	end

	return count;

end

----------------------------------------------------------------------

function DroneMgr:getDroneCountLeft()
	return self:_getDroneCount(DRONE_LEFT);
end

----------------------------------------------------------------------

function DroneMgr:getDroneCountRight()
	return self:_getDroneCount(DRONE_RIGHT);
end

----------------------------------------------------------------------

function DroneMgr:isDroneCollectionComplete( side )

	local maxDrones	= math.min(#self._droneBaseTmpltTbl, #self:getDroneTypeOddsTable());
	local odds			= self:getDroneTypeOddsTable();

	if side == DRONE_LEFT then
		for i=1,maxDrones do

			if			( self:isDroneDupeResolveModeAutoMerge()
				or		self:isDroneDupeResolveModeHybrid() )
				and	not self:isDroneTypeMaxedLeft(i)
				and	odds[i] > 0 then

				return false;
			elseif			self:isDroneDupeResolveModeEvolve()
						and	not self:isDroneTypeCollectedLeft(i)
						and	odds[i] > 0 then

				return false;
			end
		end
	else
		for i=1,maxDrones do

			if			( self:isDroneDupeResolveModeAutoMerge()
				or		self:isDroneDupeResolveModeHybrid() )
				and	not self:isDroneTypeMaxedRight(i)
				and	odds[i] > 0 then

				return false;
			elseif			self:isDroneDupeResolveModeEvolve()
						and	not self:isDroneTypeCollectedRight(i)
						and	odds[i] > 0 then

				return false;
			end
		end
	end

	return true;

end

----------------------------------------------------------------------

function DroneMgr:isDroneCollectionCompleteLeft()
	return self:isDroneCollectionComplete( DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:isDroneCollectionCompleteRight()
	return self:isDroneCollectionComplete( DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:isDroneTypeCollected( droneType, side )

	if droneType and side then

		local maxDrones	= self:getDroneTypeCount();
		local	maxRarity	= self:getDroneMaxRarity();
		local maxLevel		= self:getDroneMaxLevel();

		if side == DRONE_LEFT then
			for i=1,maxDrones do
				if			self:isDroneGarageSpaceFilledLeft(i)
					and	self:getDroneTypeLeft(i) == droneType then

					return math.min( self:getDroneRarityLeft(i), maxRarity ), i, math.min( self:getDroneLevelLeft(i), maxLevel );
				end
			end
		else
			for i=1,maxDrones do
				if			self:isDroneGarageSpaceFilledRight(i)
					and	self:getDroneTypeRight(i) == droneType then

					return math.min( self:getDroneRarityRight(i), maxRarity ), i, math.min( self:getDroneLevelRight(i), maxLevel );
				end
			end
		end
	end

end

----------------------------------------------------------------------

function DroneMgr:isDroneTypeCollectedLeft( droneType )

	local rarity, index, level	= self:isDroneTypeCollected( droneType, DRONE_LEFT );

	return rarity, index, level;

end

----------------------------------------------------------------------

function DroneMgr:isDroneTypeCollectedRight( droneType )

	local rarity, index, level	= self:isDroneTypeCollected( droneType, DRONE_RIGHT );

	return rarity, index, level;

end

----------------------------------------------------------------------

function DroneMgr:isDroneTypeMaxed( droneType, side )

	local	maxRarity = self:getDroneMaxRarity();
	local rarity;

	if side == DRONE_LEFT then
		rarity = self:isDroneTypeCollectedLeft( droneType );

		if rarity and rarity >= maxRarity then
			return true;
		end
	else
		rarity = self:isDroneTypeCollectedRight( droneType );

		if rarity and rarity >= maxRarity then
			return true;
		end
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:isDroneTypeMaxedLeft( droneType )
	return self:isDroneTypeMaxed( droneType, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:isDroneTypeMaxedRight( droneType )
	return self:isDroneTypeMaxed( droneType, DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:isMountUnlockAvailable()
	return self:canUnlockDroneMountLeft() or self:canUnlockDroneMountRight();
end

----------------------------------------------------------------------

function DroneMgr:isBuildAvailable()

	local maxDrones	= self:getDroneTypeCount();

	for i=1,maxDrones do
		if not self:isDroneMountLeftLocked() and self:isDroneGarageSpaceEmptyLeft(i) and self:canBuildDroneLeft(i) then
			return true;
		end

		if not self:isDroneMountRightLocked() and self:isDroneGarageSpaceEmptyRight(i) and self:canBuildDroneRight(i) then
			return true;
		end
	end

end

----------------------------------------------------------------------

function DroneMgr:isUpgradeAvailable()

	local maxDrones	= self:getDroneTypeCount();

	for i=1,maxDrones do
		if not self:isDroneMountLeftLocked() and self:isDroneGarageSpaceFilledLeft(i) and self:canUpgradeDroneLeft(i) then
			return true;
		end

		if not self:isDroneMountRightLocked() and self:isDroneGarageSpaceFilledRight(i) and self:canUpgradeDroneRight(i) then
			return true;
		end
	end

end

----------------------------------------------------------------------

function DroneMgr:isEvolveAvailable()

	if self:isDroneDupeResolveModeAutoMerge() then
		return false;
	end

	local maxDrones	= self:getDroneTypeCount();

	for i=1,maxDrones do
		if not self:isDroneMountLeftLocked() and self:isDroneGarageSpaceFilledLeft(i) and self:canEvolveDroneLeft(i) then
			return true;
		end

		if not self:isDroneMountRightLocked() and self:isDroneGarageSpaceFilledRight(i) and self:canEvolveDroneRight(i) then
			return true;
		end
	end

end

----------------------------------------------------------------------

function DroneMgr:isPurchaseAvailable()

	if			not DroneMgr:isDroneLevelLocked()
		and	( self:isMountUnlockAvailable()
		or	self:isBuildAvailable()
		or	self:isUpgradeAvailable()
		or	self:isEvolveAvailable() ) then

		return true;
	end

end

----------------------------------------------------------------------

function DroneMgr:getDroneBodyVisTable( droneType )

	local odds	= self:getDroneTypeOddsTable();

	if			droneType
		and	droneType <= #self:getDroneTypeOddsTable()
		and	odds
		and	odds[droneType]
		and	odds[droneType] > 0
		and	self._droneAbltyVisTbl
		and	self._droneBodyVisTbl[droneType] then

		return self:_getDroneFile(self._droneBodyVisTbl[droneType]);
	end

end

----------------------------------------------------------------------

function DroneMgr:getDroneName( droneType, rarity )

	local odds	= self:getDroneTypeOddsTable();
	local stats	= self:getDroneStatsTable();

	if			droneType
		and	odds
      and	droneType <= #odds
		and	odds[droneType]
		and	odds[droneType] > 0
		and	self._cloudStatsMap
		and	self._cloudStatsMap[droneType]
		and	stats
		and	stats[self._cloudStatsMap[droneType]] then

		return stats[self._cloudStatsMap[droneType]].name[rarity];
	end

	return "";

end

----------------------------------------------------------------------

function DroneMgr:getDroneDesc( droneType )

	local odds	= self:getDroneTypeOddsTable();
	local stats	= self:getDroneStatsTable();

	if			droneType
		and	droneType <= #self:getDroneTypeOddsTable()
		and	odds
		and	odds[droneType]
		and	odds[droneType] > 0
		and	self._cloudStatsMap
		and	self._cloudStatsMap[droneType]
		and	stats
		and	stats[self._cloudStatsMap[droneType]] then

		return stats[self._cloudStatsMap[droneType]].desc or {""};
	end

	return {""};

end


----------------------------------------------------------------------
-- Garage Space
----------------------------------------------------------------------


function DroneMgr:isAllDroneGarageSpaceFilledLeft()

	for i=1, self:getDroneGarageSpaceMax() do
		if not self:isDroneGarageSpaceFilledLeft(i) then
			return false;
		end
	end

	return true;

end

----------------------------------------------------------------------

function DroneMgr:isAllDroneGarageSpaceFilledRight()

	for i=1, self:getDroneGarageSpaceMax() do
		if not self:isDroneGarageSpaceFilledRight(i) then
			return false;
		end
	end

	return true;

end

----------------------------------------------------------------------

function DroneMgr:isDroneGarageSpaceInRange( garage_index )

	if			garage_index
		and	garage_index >= 1
		and	garage_index <= self:getDroneGarageSpaceMax() then

		return true;
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:isDroneGarageSpaceEmptyLeft( garage_index )

	if self:getDroneTypeLeft(garage_index) == GARAGE_SPACE_EMPTY then
		return true;
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:isDroneGarageSpaceEmptyRight( garage_index )

	if self:getDroneTypeRight(garage_index) == GARAGE_SPACE_EMPTY then
		return true;
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:isDroneGarageSpaceFilledLeft( garage_index )

	if not self:isDroneGarageSpaceEmptyLeft(garage_index) then
		return true;
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:isDroneGarageSpaceFilledRight( garage_index )

	if not self:isDroneGarageSpaceEmptyRight(garage_index) then
		return true;
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:moveDroneToGarageSpaceLeft( object_index, target_index, merge )

	local rarity	= self:getDroneRarityLeft(object_index);

	if merge then
		rarity = math.min( self:getDroneRarityLeft(target_index) + rarity, self:getDroneMaxRarity() );
	end

	self:updateDroneStatsLeft(	target_index,
										self:getDroneTypeLeft(object_index),
										rarity,
										self:getDroneLevelLeft(object_index)
										);

	self:resetDroneStatsLeft( object_index );

end

----------------------------------------------------------------------

function DroneMgr:moveDroneToGarageSpaceRight( object_index, target_index, merge )

	local rarity	= self:getDroneRarityRight(object_index);

	if merge then
		rarity = math.min( self:getDroneRarityRight(target_index) + rarity, self:getDroneMaxRarity() );
	end

	self:updateDroneStatsRight(	target_index,
											self:getDroneTypeRight(object_index),
											rarity,
											self:getDroneLevelRight(object_index)
											);

	self:resetDroneStatsRight( object_index );

end

----------------------------------------------------------------------

function DroneMgr:mergeDroneToGarageSpaceLeft( object_index, target_index )
	self:moveDroneToGarageSpaceLeft( object_index, target_index, true );
end

----------------------------------------------------------------------

function DroneMgr:mergeDroneToGarageSpaceRight( object_index, target_index )
	self:moveDroneToGarageSpaceRight( object_index, target_index, true );
end


----------------------------------------------------------------------
-- Mount Points
----------------------------------------------------------------------


function DroneMgr:anyDroneMountUnlocked()

	if not self:isDroneMountLeftLocked() or not self:isDroneMountRightLocked() then
		return true;
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:canUnlockDroneMountLeft()

	if			not self:isDroneLevelLocked()
		and	self:isDroneMountLeftLocked() then

		local cost = self:getDroneMountUnlockCost();

		if cost then
			local pMoney;

			if cost.currency == CURRENCY_GEM then
				pMoney = App.data:getGemCount();
			elseif cost.currency == CURRENCY_GOLD then
				pMoney = App.data:getGoldCount();
			end

			if pMoney and pMoney >= cost.cost then
				return true;
			end
		end

	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:canUnlockDroneMountRight()

	if			not self:isDroneLevelLocked()
		and	self:isDroneMountRightLocked() then

		local cost = self:getDroneMountUnlockCost();

		if cost then
			local pMoney;

			if cost.currency == CURRENCY_GEM then
				pMoney = App.data:getGemCount();
			elseif cost.currency == CURRENCY_GOLD then
				pMoney = App.data:getGoldCount();
			end

			if pMoney and pMoney >= cost.cost then
				return true;
			end
		end

	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:clearDroneMountLeft()

	if			not self:isDroneLevelLocked()
		and	not self:isDroneMountLeftLocked()
		and	not self:isDroneMountLeftEmpty() then

		self:setDroneMountLeft(MOUNT_POINT_EMPTY);
	end

end

----------------------------------------------------------------------

function DroneMgr:clearDroneMountRight()

	if			not self:isDroneLevelLocked()
		and	not self:isDroneMountRightLocked()
		and	not self:isDroneMountRightEmpty() then

		self:setDroneMountRight(MOUNT_POINT_EMPTY);
	end

end

----------------------------------------------------------------------

function DroneMgr:isDroneMountLeftEmpty()

	if App.data:getDroneMountLeft() == MOUNT_POINT_EMPTY then
		return true;
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:isDroneMountRightEmpty()

	if App.data:getDroneMountRight() == MOUNT_POINT_EMPTY then
		return true;
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:isDroneMountLeftAssigned()

	if			not self:isDroneLevelLocked()
		and	not self:isDroneMountLeftLocked()
		and	not self:isDroneMountLeftEmpty() then

		return App.data:getDroneMountLeft();
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:isDroneMountRightAssigned()

	if			not self:isDroneLevelLocked()
		and	not self:isDroneMountRightLocked()
		and	not self:isDroneMountRightEmpty() then

		return App.data:getDroneMountRight();
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:isDroneMountLeftLocked()

	if App.data:getDroneMountLeft() == MOUNT_POINT_LOCKED then
		return true;
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:isDroneMountRightLocked()

	if App.data:getDroneMountRight() == MOUNT_POINT_LOCKED then
		return true;
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:getDroneMountLeft()
	return App.data:getDroneMountLeft();
end

----------------------------------------------------------------------

function DroneMgr:getDroneMountRight()
	return App.data:getDroneMountRight();
end

----------------------------------------------------------------------

function DroneMgr:getDroneMountUnlockCost()

	local	tbl	= self:getDroneMountCostTable();
	local count	= math.clamp(1, 1, #tbl);

	if self:anyDroneMountUnlocked() then
		count = math.clamp(2, 1, #tbl);
	end

	if		tbl == nil
		or	count == nil
		or	not tbl[count] then

		return {currency=CURRENCY_GEM, cost=5};
	end

	local cGold	= tbl[count].gold;
	local cGem	= tbl[count].gem;

	if cGem then
		return {currency=CURRENCY_GEM, cost=cGem};
	end

	if cGold then
		return {currency=CURRENCY_GOLD, cost=cGold};
	end

end

----------------------------------------------------------------------

function DroneMgr:setDroneMountLeft( garage_index, force )

	if	not self:isDroneMountLeftLocked() or force then
		return App.data:setDroneMountLeft(garage_index);
	end

end

----------------------------------------------------------------------

function DroneMgr:setDroneMountRight( garage_index, force )

	if	not self:isDroneMountRightLocked() or force then
		return App.data:setDroneMountRight(garage_index);
	end

end

----------------------------------------------------------------------

function DroneMgr:unlockDroneMountLeft()

	if DroneMgr:isDroneMountLeftLocked() then
		App.data:unlockDroneMount(MOUNT_POINT_LEFT, self:getDroneMountUnlockCost());
	end

end

----------------------------------------------------------------------

function DroneMgr:unlockDroneMountRight()

	if DroneMgr:isDroneMountRightLocked() then
		App.data:unlockDroneMount(MOUNT_POINT_RIGHT, self:getDroneMountUnlockCost());
	end

end


----------------------------------------------------------------------
-- Build/Finish/Upgrade info
----------------------------------------------------------------------


function DroneMgr:_canBuildDrone( garage_index, side )

	if			not self:isDroneLevelLocked()
		and	self:isDroneGarageSpaceInRange(garage_index)
		and	( ( side == DRONE_LEFT and self:isDroneGarageSpaceEmptyLeft(garage_index) )
		or		( side == DRONE_RIGHT and self:isDroneGarageSpaceEmptyRight(garage_index) ) ) then

		local cost;

		if side == DRONE_LEFT then
			cost = self:getBuildCostLeft();
		else
			cost = self:getBuildCostRight();
		end

		if cost then
			local pMoney;

			if cost.currency == CURRENCY_GEM then
				pMoney = App.data:getGemCount();
			elseif cost.currency == CURRENCY_GOLD then
				pMoney = App.data:getGoldCount();
			end

			if pMoney and pMoney >= cost.cost then
				return true;
			end
		end
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:canBuildDroneLeft( garage_index )
	return self:_canBuildDrone( garage_index, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:canBuildDroneRight( garage_index )
	return self:_canBuildDrone( garage_index, DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:_getBuildCost( side )

	local price = self:getDroneBuildCostTable();

	local count;

	if side == DRONE_LEFT then
		count	= math.clamp(self:getDroneBuiltCountLeft()+1, 1, #price);
	else
		count	= math.clamp(self:getDroneBuiltCountRight()+1, 1, #price);
	end

	local cGold	= price[count].gold;
	local cGem	= price[count].gem;

	if cGem then
		return {currency=CURRENCY_GEM, cost=cGem};
	end

	if cGold then
		return {currency=CURRENCY_GOLD, cost=cGold};
	end

end

----------------------------------------------------------------------

function DroneMgr:getBuildCostLeft( )
	return self:_getBuildCost( DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:getBuildCostRight( )
	return self:_getBuildCost( DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:_canUpgradeDrone( garage_index, side )

	if			not self:isDroneLevelLocked()
		and	self:isDroneGarageSpaceInRange(garage_index)
		and	( ( side == DRONE_LEFT and self:isDroneGarageSpaceFilledLeft(garage_index) )
		or		( side == DRONE_RIGHT and self:isDroneGarageSpaceFilledRight(garage_index) ) )
		and	( ( side == DRONE_LEFT and not self:isDroneUpgradeMaxedLeft(garage_index) )
		or		( side == DRONE_RIGHT and not self:isDroneUpgradeMaxedRight(garage_index) ) ) then

		local cost = self:getUpgradeCostLeft(garage_index);

		if side == DRONE_RIGHT then
			cost = self:getUpgradeCostRight(garage_index);
		end

		if cost then
			local pMoney;

			if cost.currency == CURRENCY_GOLD then
				pMoney = App.data:getGoldCount();
			elseif cost.currency == CURRENCY_GEM then
				pMoney = App.data:getGemCount();
			end

			if pMoney and pMoney >= cost.cost then
				return true;
			end
		end
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:canUpgradeDroneLeft( garage_index )
	return self:_canUpgradeDrone( garage_index, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:canUpgradeDroneRight( garage_index )
	return self:_canUpgradeDrone( garage_index, DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:_isDroneUpgradeMaxed( garage_index, side )

	if side == DRONE_LEFT then
		if self:getDroneLevelLeft(garage_index) >= self:getDroneMaxLevel() then
			return true;
		end
	else
		if self:getDroneLevelRight(garage_index) >= self:getDroneMaxLevel() then
			return true;
		end
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:isDroneUpgradeMaxedLeft( garage_index )
	return self:_isDroneUpgradeMaxed( garage_index, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:isDroneUpgradeMaxedRight( garage_index )
	return self:_isDroneUpgradeMaxed( garage_index, DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:_getUpgradeCost( garage_index, side )

	if self:isDroneGarageSpaceInRange(garage_index) then

		local tbl	= self:getDroneUpgradeCostTable();
		local rarity;
		local level;
		local cGold;
		local cGem;

		if side == DRONE_LEFT then
			rarity	= math.clamp(self:getDroneRarityLeft(garage_index), 1, #tbl);
			level		= math.clamp(self:getDroneLevelLeft(garage_index), self:getDroneMinLevel(), self:getDroneMaxLevel());
		else
			rarity	= math.clamp(self:getDroneRarityRight(garage_index), 1, #tbl);
			level		= math.clamp(self:getDroneLevelRight(garage_index), self:getDroneMinLevel(), self:getDroneMaxLevel());
		end

		cGold		= tbl[rarity][level].gold;
		cGem		= tbl[rarity][level].gem;

		if cGem then
			return {currency=CURRENCY_GEM, cost=cGem};
		end

		if cGold then
			return {currency=CURRENCY_GOLD, cost=cGold};
		end
	end

end

----------------------------------------------------------------------

function DroneMgr:getUpgradeCostLeft( garage_index )
	return self:_getUpgradeCost( garage_index, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:getUpgradeCostRight( garage_index )
	return self:_getUpgradeCost( garage_index, DRONE_RIGHT );
end


----------------------------------------------------------------------
-- Build/Upgrade Drone
----------------------------------------------------------------------


function DroneMgr:_addDrone( garage_index, cost, side )

	if side == DRONE_LEFT then
		App.data:addDroneLeft(garage_index, cost);
	else
		App.data:addDroneRight(garage_index, cost);
	end

end

----------------------------------------------------------------------

function DroneMgr:addDroneLeft( garage_index, cost )
	self:_addDrone( garage_index, cost, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:addDroneRight( garage_index, cost )
	self:_addDrone( garage_index, cost, DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:_getAbilityOdds( side )

	local odds	= self:getDroneTypeOddsTable();
	local t		= {};

	for i=1, #odds do

		local r,a,b;

		if side == DRONE_LEFT then
			a	= self:_getCollectedDroneOddsKLeft(i);
			b	= self:_getCollectedDroneOddsKRight(i);
		else
			a	= self:_getCollectedDroneOddsKRight(i);
			b	= self:_getCollectedDroneOddsKLeft(i);
		end

		if self:isDroneDupeResolveModeEvolve() then

			if		( side == DRONE_LEFT and self:isDroneTypeCollectedLeft(i) )
				or	( side == DRONE_RIGHT and self:isDroneTypeCollectedRight(i) ) then

				r = 0;
			else
				r = 1;
			end
		elseif			not self:isDroneDupeResolveModeEvolve()
					and	( ( side == DRONE_LEFT and self:isDroneTypeMaxedLeft(i) )
					or		( side == DRONE_RIGHT and self:isDroneTypeMaxedRight(i) ) ) then

			r = 0;
		elseif a <= 0 and b <= 0 then
			r = 0;
		elseif a > 0 and b > 0 then
			r = math.min(a, b);
		else
			r = math.max(a,b);
		end

		if r then
			t[i]	= odds[i]*r;
		else
			t[i]	= odds[i];
		end
	end

	return t;

end

----------------------------------------------------------------------

function DroneMgr:_getAbilityOddsLeft()
	return self:_getAbilityOdds(DRONE_LEFT);
end

----------------------------------------------------------------------

function DroneMgr:_getAbilityOddsRight()
	return self:_getAbilityOdds(DRONE_RIGHT);
end

----------------------------------------------------------------------

function DroneMgr:_getCollectedDroneOddsKLeft( droneType )

	local oddsK			= self:getDroneTypeOddsScaleTable();
	local maxDrones	= math.min(#self._droneBaseTmpltTbl, #self:getDroneTypeOddsTable());

	for i=1, maxDrones do
		if			self:isDroneGarageSpaceFilledLeft(i)
			and	self:getDroneTypeLeft(i) == droneType then

			if self:isDroneDupeResolveModeEvolve() then
				return 0;
			end

			return oddsK[ self:getDroneRarityLeft(i) ] or 1;
		end
	end

	return 1;

end

----------------------------------------------------------------------

function DroneMgr:_getCollectedDroneOddsKRight( droneType )

	local oddsK			= self:getDroneTypeOddsScaleTable();
	local maxDrones	= math.min(#self._droneBaseTmpltTbl, #self:getDroneTypeOddsTable());

	for i=1, maxDrones do
		if			self:isDroneGarageSpaceFilledRight(i)
			and	self:getDroneTypeRight(i) == droneType then

			if self:isDroneDupeResolveModeEvolve() then
				return 0;
			end

			return oddsK[ self:getDroneRarityRight(i) ] or 1;
		end
	end

	return 1;

end

----------------------------------------------------------------------

function DroneMgr:pickDroneType( side )

	local odds;
	local buildCount;
	local buildTbl	= self:getDroneBuildOrderTable();

	if side == DRONE_LEFT then
		odds			= self:_getAbilityOddsLeft();
		buildCount	= math.max(1, self:getDroneBuiltCountLeft()+1);
	else
		odds			= self:_getAbilityOddsRight();
		buildCount	= math.max(1, self:getDroneBuiltCountRight()+1);
	end

	-- check build order
	if buildCount <= #buildTbl[side] then
		return buildTbl[side][buildCount];
	end

	local pick = random.pickWeighted(odds);

	if pick then
		return math.clamp( pick, 1, #self._cloudStatsMap);
	end

	return -1;

end

----------------------------------------------------------------------

function DroneMgr:pickDroneRarity( side )

   	--MQL: Force rarity build to be 1
	return 1;



	-- local odds = self:getDroneRarityOddsTable();
	-- local count;

	-- if side == DRONE_LEFT then
	-- 	count = math.clamp(self:getDroneBuiltCountLeft(), 1, #odds);
	-- else
	-- 	count = math.clamp(self:getDroneBuiltCountRight(), 1, #odds);
	-- end

	-- -- force first drone built to be a gun drone or rarity 1
	-- if App.data:getTotalDroneBuiltCount() <= 1 then
	-- 	return 1;
	-- end

	-- local pick = random.pickWeighted(odds[count]);
	--return pick or 1;

end

----------------------------------------------------------------------

function DroneMgr:_upgradeDrone( garage_index, side )

	if side == DRONE_LEFT then
		App.data:upgradeDroneLeft(garage_index, self:getUpgradeCostLeft(garage_index));
	else
		App.data:upgradeDroneRight(garage_index, self:getUpgradeCostRight(garage_index));
	end

end

----------------------------------------------------------------------

function DroneMgr:upgradeDroneLeft( garage_index )
	self:_upgradeDrone( garage_index, DRONE_LEFT );
end
----------------------------------------------------------------------

function DroneMgr:upgradeDroneRight( garage_index )
	self:_upgradeDrone( garage_index, DRONE_RIGHT );
end


----------------------------------------------------------------------
-- Evolve
----------------------------------------------------------------------


function DroneMgr:_canEvolveDrone( garage_index, side )

	if			not self:isDroneLevelLocked()
		and	not self:isDroneDupeResolveModeAutoMerge()
		and	self:isDroneGarageSpaceInRange(garage_index)
		and	( ( side == DRONE_LEFT and self:isDroneGarageSpaceFilledLeft(garage_index) )
		or		( side == DRONE_RIGHT and self:isDroneGarageSpaceFilledRight(garage_index) ) )
		and	( ( side == DRONE_LEFT and self:isDroneUpgradeMaxedLeft(garage_index) )
		or		( side == DRONE_RIGHT and self:isDroneUpgradeMaxedRight(garage_index) ) )
		and	( ( side == DRONE_LEFT and self:getDroneRarityLeft(garage_index) < self:getDroneMaxRarity() )
		or		( side == DRONE_RIGHT and self:getDroneRarityRight(garage_index) < self:getDroneMaxRarity() ) ) then

		local cost = self:getEvolveCostLeft(garage_index);

		if side == DRONE_RIGHT then
			cost = self:getEvolveCostRight(garage_index);
		end

		if cost then
			local pMoney;

			if cost.currency == CURRENCY_GOLD then
				pMoney = App.data:getGoldCount();
			elseif cost.currency == CURRENCY_GEM then
				pMoney = App.data:getGemCount();
			end

			if pMoney and pMoney >= cost.cost then
				return true;
			end
		end
	end

	return false;

end

----------------------------------------------------------------------

function DroneMgr:canEvolveDroneLeft( garage_index )
	return self:_canEvolveDrone( garage_index, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:canEvolveDroneRight( garage_index )
	return self:_canEvolveDrone( garage_index, DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:_getEvolveCost( garage_index, side )

	if self:isDroneGarageSpaceInRange(garage_index) then

		local rarity;
		local cGold;
		local cGem;
		local tbl	= self:getDroneEvolveCostTable();

		if side == DRONE_LEFT then
			rarity	= math.clamp(self:getDroneRarityLeft(garage_index), 1, #tbl);
		else
			rarity	= math.clamp(self:getDroneRarityRight(garage_index), 1, #tbl);
		end

		cGold		= tbl[rarity].gold;
		cGem		= tbl[rarity].gem;

		if cGem then
			return {currency=CURRENCY_GEM, cost=cGem};
		end

		if cGold then
			return {currency=CURRENCY_GOLD, cost=cGold};
		end
	end

end

----------------------------------------------------------------------

function DroneMgr:getEvolveCostLeft( garage_index )
	return self:_getEvolveCost( garage_index, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:getEvolveCostRight( garage_index )
	return self:_getEvolveCost( garage_index, DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:evolveDroneLeft( garage_index )

	if garage_index and self:isDroneGarageSpaceFilledLeft( garage_index ) then

		local rarity	= self:getDroneRarityLeft(garage_index) + 1;
		local cost		= self:getEvolveCostLeft(garage_index);

		if rarity <= self:getDroneMaxRarity() then

			self:updateDroneStatsLeft(	garage_index,
												self:getDroneTypeLeft(garage_index),
												rarity,
												1,
												cost
												);
		end
	end

end

----------------------------------------------------------------------

function DroneMgr:evolveDroneRight( garage_index )

	if garage_index and self:isDroneGarageSpaceFilledRight( garage_index ) then

		local rarity	= self:getDroneRarityRight(garage_index) + 1;
		local cost		= self:getEvolveCostRight(garage_index);

		if rarity <= self:getDroneMaxRarity() then

			self:updateDroneStatsRight(	garage_index,
													self:getDroneTypeRight(garage_index),
													rarity,
													1,
													cost
												);
		end
	end

end


----------------------------------------------------------------------
-- drone templates
----------------------------------------------------------------------


function DroneMgr:_getDroneFile( file )

	if file and type(file) == "string" then

		return dofile(file);
	end

end

----------------------------------------------------------------------

function DroneMgr:_getDroneBaseTemplate( garage_index, side )

	local index;

	if side == DRONE_LEFT then
		index	= math.min(self:getDroneTypeLeft(garage_index), #self._droneBaseTmpltTbl);
	else
		index	= math.min(self:getDroneTypeRight(garage_index), #self._droneBaseTmpltTbl);
	end

	return self:_getDroneFile(self._droneBaseTmpltTbl[index]);

end

----------------------------------------------------------------------

function DroneMgr:getDroneBaseTemplateLeft( garage_index )
	return self:_getDroneBaseTemplate( garage_index, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:getDroneBaseTemplateRight( garage_index )
	return self:_getDroneBaseTemplate( garage_index, DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:_getDroneAbilityVisTable( garage_index, side )

	if			not self:isDroneLevelLocked()
		and	self:isDroneGarageSpaceInRange(garage_index)
		and	( ( side == DRONE_LEFT and self:isDroneGarageSpaceFilledLeft(garage_index) )
		or		( side == DRONE_RIGHT and self:isDroneGarageSpaceFilledRight(garage_index) ) )
		and	self._droneAbltyVisTbl
		and	type(self._droneAbltyVisTbl) == "table"
		and	next(self._droneAbltyVisTbl) ~= nil then

		local index = math.min(self:getDroneTypeLeft(garage_index), #self._droneAbltyVisTbl);

		if side == DRONE_RIGHT then
			index = math.min(self:getDroneTypeRight(garage_index), #self._droneAbltyVisTbl);
		end

		return self:_getDroneFile(self._droneAbltyVisTbl[index]);
	end

end

----------------------------------------------------------------------

function DroneMgr:getDroneAbilityVisTableLeft( garage_index )
	return self:_getDroneAbilityVisTable( garage_index, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:getDroneAbilityVisTableRight( garage_index )
	return self:_getDroneAbilityVisTable( garage_index, DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:_getDroneBodyVis( garage_index, side )

	if			not self:isDroneLevelLocked()
		and	self:isDroneGarageSpaceInRange(garage_index)
		and	( ( side == DRONE_LEFT and self:isDroneGarageSpaceFilledLeft(garage_index) )
		or		( side == DRONE_RIGHT and self:isDroneGarageSpaceFilledRight(garage_index) ) )
		and	self._droneBodyVisTbl
		and	type(self._droneBodyVisTbl) == "table"
		and	next(self._droneBodyVisTbl) ~= nil then

		local index;
		local tbl;
		local rarity;

		if side == DRONE_LEFT then
			index		= math.min(self:getDroneTypeLeft(garage_index), #self._droneBodyVisTbl);
			tbl		= self:_getDroneFile(self._droneBodyVisTbl[index]);
			rarity	= math.min(self:getDroneRarityLeft(garage_index), self:getDroneMaxRarity());
		else
			index		= math.min(self:getDroneTypeRight(garage_index), #self._droneBodyVisTbl);
			tbl		= self:_getDroneFile(self._droneBodyVisTbl[index]);
			rarity	= math.min(self:getDroneRarityRight(garage_index), self:getDroneMaxRarity());
		end

		return tbl[rarity];
	end

end

----------------------------------------------------------------------

function DroneMgr:getDroneBodyVisLeft( garage_index )
	return self:_getDroneBodyVis( garage_index, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:getDroneBodyVisRight( garage_index )
	return self:_getDroneBodyVis( garage_index, DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:_getDroneName( garage_index, side )

	if			not self:isDroneLevelLocked()
		and	self:isDroneGarageSpaceInRange(garage_index)
		and	( ( side == DRONE_LEFT and self:isDroneGarageSpaceFilledLeft(garage_index) )
		or		( side == DRONE_RIGHT and self:isDroneGarageSpaceFilledRight(garage_index) ) ) then

		local stats = self:getDroneStatTblLeft(garage_index);

		if side == DRONE_RIGHT then
			stats = self:getDroneStatTblRight(garage_index);
		end

		local rarity = math.min(self:getDroneRarityLeft(garage_index), self:getDroneMaxRarity());

		if side == DRONE_RIGHT then
			rarity = math.min(self:getDroneRarityRight(garage_index), self:getDroneMaxRarity());
		end

		return stats.name[rarity];
	end

	return "";

end

----------------------------------------------------------------------

function DroneMgr:getDroneNameLeft( garage_index )
	return self:_getDroneName( garage_index, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:getDroneNameRight( garage_index )
	return self:_getDroneName( garage_index, DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:_getDroneDesc( garage_index, side )

	if			not self:isDroneLevelLocked()
		and	self:isDroneGarageSpaceInRange(garage_index)
		and	( ( side == DRONE_LEFT and self:isDroneGarageSpaceFilledLeft(garage_index) )
		or		( side == DRONE_RIGHT and self:isDroneGarageSpaceFilledRight(garage_index) ) ) then

		local stats;

		if side == DRONE_RIGHT then
			stats = self:getDroneStatTblRight(garage_index);
		else
			stats = self:getDroneStatTblLeft(garage_index);
		end

		return stats.desc or {""};
	end

	return {""};

end

----------------------------------------------------------------------

function DroneMgr:getDroneDescLeft( garage_index )
	return self:_getDroneDesc( garage_index, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:getDroneDescRight( garage_index )
	return self:_getDroneDesc( garage_index, DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:_getDroneStatTbl( garage_index, side )

	local stats	= self:getDroneStatsTable();

	if side == DRONE_LEFT then
		local index	= math.min(self:getDroneTypeLeft(garage_index), #self._cloudStatsMap);

		return stats[self._cloudStatsMap[index]];
	else
		local index	= math.min(self:getDroneTypeRight(garage_index), #self._cloudStatsMap);

		return stats[self._cloudStatsMap[index]];
	end

end

----------------------------------------------------------------------

function DroneMgr:getDroneStatTblLeft( garage_index )
	return self:_getDroneStatTbl( garage_index, DRONE_LEFT );
end

----------------------------------------------------------------------

function DroneMgr:getDroneStatTblRight( garage_index )
	return self:_getDroneStatTbl( garage_index, DRONE_RIGHT );
end

----------------------------------------------------------------------

function DroneMgr:_getDroneTemplate( garage_index, side )

	if			not self:isDroneLevelLocked()
		and	self:isDroneGarageSpaceInRange(garage_index)
		and	( ( side == DRONE_LEFT and self:isDroneGarageSpaceFilledLeft(garage_index) )
		or		( side == DRONE_RIGHT and self:isDroneGarageSpaceFilledRight(garage_index) ) )
		and	self._droneBaseTmpltTbl
		and	type(self._droneBaseTmpltTbl) == "table"
		and	next(self._droneBaseTmpltTbl) ~= nil then

		local base;
		local body;
		local ablVis;
		local stats;

		if side == DRONE_LEFT then
			base		= self:getDroneBaseTemplateLeft(garage_index);
			body		= self:getDroneBodyVisLeft(garage_index);
			ablVis	= self:getDroneAbilityVisTableLeft(garage_index);
			stats		= self:getDroneStatTblLeft(garage_index);
		else
			base		= self:getDroneBaseTemplateRight(garage_index);
			body		= self:getDroneBodyVisRight(garage_index);
			ablVis	= self:getDroneAbilityVisTableRight(garage_index);
			stats		= self:getDroneStatTblRight(garage_index);
		end

		--Drone entity needs to compose special ablVis into the template
		if base and body and ablVis and stats then
			base.texture = body;

			return base, ablVis, stats;
		end
	end

end

----------------------------------------------------------------------

function DroneMgr:getDroneTemplateLeft( garage_index )

	local base, ablVis, stats = self:_getDroneTemplate( garage_index, DRONE_LEFT );

	return base, ablVis, stats;

end

----------------------------------------------------------------------

function DroneMgr:getDroneTemplateRight( garage_index )

	local base, ablVis, stats = self:_getDroneTemplate( garage_index, DRONE_RIGHT );

	return base, ablVis, stats;

end

----------------------------------------------------------------------

function DroneMgr:encodeDroneData( droneType, rarity, level )

   Debug.assert( droneType and droneType >= 0 and droneType < DRONE_INFO_CODE_SHIFT, "DroneMgr:encodeDroneData -> droneType and droneType >= 1 and droneType < DRONE_INFO_CODE_SHIFT");
   Debug.assert( rarity and rarity >= 1 and rarity < DRONE_INFO_CODE_SHIFT, "DroneMgr:encodeDroneData -> rarity and rarity >= 1 and rarity < DRONE_INFO_CODE_SHIFT");
   Debug.assert( level and level >= 1 and level < DRONE_INFO_CODE_SHIFT, "DroneMgr:encodeDroneData -> level and level >= 1 and level < DRONE_INFO_CODE_SHIFT");

   local code = (droneType or 0);
   code = (code*DRONE_INFO_CODE_SHIFT + (rarity or 1));
   code = (code*DRONE_INFO_CODE_SHIFT + (level or 1));

   return code;
end

function DroneMgr:decodeDroneData( code )

   Debug.assertType(code, "number", "DroneMgr:decodeDroneData -> code");
   Debug.assert(code >= 0, "DroneMgr:decodeDroneData -> code >= 0");

   local code2 = math.floor( code );
   local code3 = math.floor( code / DRONE_INFO_CODE_SHIFT );
   local code4 = math.floor( code / (DRONE_INFO_CODE_SHIFT * DRONE_INFO_CODE_SHIFT) );

   local level = code2 % DRONE_INFO_CODE_SHIFT;
   local rarity = code3 % DRONE_INFO_CODE_SHIFT;
   local droneType = code4 % DRONE_INFO_CODE_SHIFT;

   return droneType, rarity, level;
end

----------------------------------------------------------------------
