require('app/feed/server/sender-feed')

----------------------------------------------------------------------

local HACK_FORCE_POST         = false;
local HACK_NO_TIME_CHECK      = false;
local REFERENCE_TIME_OFFSET   =  60*60*24*1;--This is a cheap way to make sure our users will refresh there key after a new update if needed


----------------------------------------------------------------------

FeedMgr  =
{
};

----------------------------------------------------------------------

function FeedMgr:init()

   EventMgr:addListener( self, self._onUpdateMasterFeedInfo,     "cloud:master-data-feed-info"   );

   if self:isEnable() then
      MOAIFlurry:event("Tango-Feed-Share-Report", "active", tostring(App.data:getValue( "feed_notification_active" )) );
   end

end

----------------------------------------------------------------------

function FeedMgr:isEnable()

   --Until we localize the feed, we only post to english speaker
   return LoginMgr:isTango();

end

----------------------------------------------------------------------

function FeedMgr:_GetFeedTimeReference( )

   return (TimeMgr:getTimeGMT() or 0) + REFERENCE_TIME_OFFSET;

end

----------------------------------------------------------------------

function FeedMgr:shareScreenshot( feedId, delayTime, callDone )

   local tpl = ( self._feedInfo or {} )[feedId];

   if tpl then

      local should, reason = self:_shouldPost( feedId , tpl);

      MOAIFlurry:event("Tango-Feed-Share-Attempt", "feedId", feedId, "Posted", tostring(should), "Failed-Reason", tostring(reason) );

      if should then

         Debug.assert(tpl.caption);

         --Stamp here even to avoid posting again while waiting for the delay
         App.data:setValue("feedTimeStamp", self:_GetFeedTimeReference() );

         local filename       = "screenshot.png";

         --TODO: use a more robust function to make sure the last "/"  is not double
         local url            = "file://"..MOAIEnvironment.documentDirectory.."/"..filename;
         local display_target = "my_feed";
         local type_i         = "image\/png";

         local image          = {url = url, mime = type_i };
         local thumbnail      = {url = url, mime = type_i};

         local data           = {};
         data.caption         = tpl.caption  or "FEED_GENERIC_1";
         data.linkText        = tpl.linkText or "";

         if type(tpl.caption) == "table" then
            data.caption = tpl.caption[math.random(1,#tpl.caption)];
         end

         data.caption         = getLocalStr( data.caption );

         if data.caption then

            delay( delayTime or -1,
                  function()

                     local shot_image = MOAIImage.new();

                     MOAIRenderMgr.grabNextFrame( shot_image,

                        function()

                           local workingDir = MOAIFileSystem.getWorkingDirectory ()
                           MOAIFileSystem.setWorkingDirectory ( MOAIEnvironment.documentDirectory )

                           shot_image:writePNG( filename );

                           MOAIFileSystem.setWorkingDirectory ( workingDir )

                           local image2 = {url = url, mime = type_i};
                           local share = SharingFeed:new(data, display_target, nil, image2, nil );

                           if callDone then
                              callDone();
                           end

                           share:exec( self,
                                       function(self,response)

                                          local count = App.data:deltaValue("feed-share-count",1);
                                          EventMgr:postEvent({feedId=feedId, count= count}, "feed-post-success");
                                          MOAIFlurry:event("Tango-Feed-Share", "feedId", feedId, "count", tostring(count));

                                       end,
                                       function(self, code, message)
                                       end
                                       );

                        end
                  );
               end
            );

            return true;

         else

            Debug.assert(data.caption);

         end
      end
   end

end

----------------------------------------------------------------

function FeedMgr:_shouldPost( feedId, tpl )

   if HACK_FORCE_POST then

      return true;

   else

      if not LoginMgr:isTango() then

         return false, "Not Tango";

      elseif not self:isEnable() then

         return false, "Disabled";

      else

         if App.data:getValue( "feed_notification_active" )  then

            local time     = self:_GetFeedTimeReference();
            local stamp    = App.data:getValue("feedTimeStamp") or 0;
            local interval = tpl.intervalMin                    or 1000000;
            local delta    = time-stamp;

            if (delta > interval) or HACK_NO_TIME_CHECK then

               local odds = tpl.odds;
               local rand = math.random();

               if rand < odds then
                  return true;
               else
                  return false, "Random";
               end

            else
               return false, "Time Interval";
            end
         else

            return false, "Notification Flag Off";

         end
      end
   end

end

----------------------------------------------------------------

function FeedMgr:_onUpdateMasterFeedInfo(data)

   self._feedInfo = (data or {}).content;

end


----------------------------------------------------------------------
