----------------------------------------------------------------------
local SAVE_FILENAME_PROTECTED       = "fr00a34125u99d.jp3";
local SAVE_FILENAME_PROTECTED_KEY   = "3dSSadklasjhdkjashdkjsah)(*)(*)_*)(*_jbaskjhdlkashdkasjhdl%^$^%$^&%$kjhdkjashdksajhdlksajhHKLJHLKdasblkjdsahkjdashkjld*_)(*_KJHKJ982";
local SECRET_DATA_ID                = "FraudMgr.secret3";
local DEVICE_DEV_MODEL_ID           = "FraudMgr.dev_model_id.v2";
local DEVICE_DEV_PRODUCT_ID         = "FraudMgr.dev_product_id.v2";
----------------------------------------------------------------------

FraudMgr  =
{
   _internal = {};
};

----------------------------------------------------------------------

function FraudMgr:init()

   self:_load();

   self._dataAtLaunch =
   {
      gem      =  App.data:getValue("gem");
      gemC     =  App.data:getValueFromCritical("gem");
      gold     =  App.data:getValue("gold");
      goldC    =  App.data:getValueFromCritical("gold");
   };

   self._crash = false;
   self._video = false;

   self:_initListeners();

end

----------------------------------------------------------------------

function FraudMgr:_initListeners()

   EventMgr:addListener( self, self._onCloudSettings, "cloud:master-data-fraud-settings" );

end

----------------------------------------------------------------------

function FraudMgr:_onCloudSettings( data )

   local content = ( data or {} ).content;

   if content then

      for k,v in pairs(content) do
         self:_setSetting(k,v);
      end

   end


end

----------------------------------------------------------------------

function FraudMgr:_save()

   saveJsonFileProtected(SAVE_FILENAME_PROTECTED, self._internal, SAVE_FILENAME_PROTECTED_KEY );

end

----------------------------------------------------------------------

function FraudMgr:_load()

   local data, error = loadJsonFileProtected(SAVE_FILENAME_PROTECTED, SAVE_FILENAME_PROTECTED_KEY ) or {};

   if not error  then

      self._internal  = data or {};

   end

end

----------------------------------------------------------------------

function FraudMgr:hackReset()

   self._internal  = {};

   self._crash 		= false;
   self._video 		= false;
end

----------------------------------------------------------------------

function FraudMgr:detectFraud()

   self:_checkSecret();

   self:_checkGemCount();

   self:_checkDevice();

   self:_save();

end

----------------------------------------------------------------------

function FraudMgr:shouldForceCrash()

   return self._crash;

end

----------------------------------------------------------------------

function FraudMgr:shouldForceVideo()

   return self._video;

end

----------------------------------------------------------------

function FraudMgr:_checkSecret()

   local mySecret    = self:_getInternalValue(SECRET_DATA_ID);
   local dataSecret  = App.data:getValue(SECRET_DATA_ID);

print( "FraudMgr:_checkSecret", mySecret, dataSecret );

   if mySecret ~= dataSecret then

      self:_onTestedPositive("secret");

   else

      --First Time
      if mySecret == nil then

         mySecret = MOAIEnvironment.generateGUID ();

         self:_setInternalValue(SECRET_DATA_ID, mySecret);
         App.data:setValue(SECRET_DATA_ID,      mySecret);

         --since App.data can take a few frame before saving... dont wait and do it now
         self:_save();
         App.data:save();

      end

   end

end


----------------------------------------------------------------

function FraudMgr:_checkGemCount()

   local gem       = App.data:getGemCount();
   local iapReal   = App.data:getValue("iap-verified", 0 );
   local threshold = math.max(iapReal,1) * 10000;

   --If this user have way too many gems
   if gem > threshold then

      self:_onTestedPositive("gem-count");

   end

end


----------------------------------------------------------------

function FraudMgr:_checkDevice()

   local myDevModel     = self:_getInternalValue(DEVICE_DEV_MODEL_ID);
   local myDevProduct   = self:_getInternalValue(DEVICE_DEV_PRODUCT_ID);
   local devModel       = MOAIEnvironment.devModel;
   local devProduct     = MOAIEnvironment.devProduct;

   if myDevModel == nil and myDevProduct == nil then
      self:_setInternalValue( DEVICE_DEV_MODEL_ID,   devModel  );
      self:_setInternalValue( DEVICE_DEV_PRODUCT_ID, devProduct);
   else

      if myDevModel~=devModel or myDevProduct ~= devProduct then
         self:_onTestedPositive("device");
      end

   end

end

----------------------------------------------------------------

function FraudMgr:_getSettingsForTypeVersion( type, version )

   local tbl = self:_getSetting("Test");

   if type and version and tbl then

      local tblType  = tbl[type] or tbl["default"];

      if tblType and tblType.versions then

         for k,v in pairs(tblType.versions) do
            if v  then
               if v.version == version then
                  return v;
               elseif v.version == nil then
                  default = v;
               end
            end
         end
      end
   end

   return default;

end


----------------------------------------------------------------

function FraudMgr:_getCurrentVersionForTestType( type )

   local tbl = self:_getSetting("Test");

   if tbl then

      local tblType  = tbl[type] or tbl["default"];

      if tblType then

         return tblType.current_version;
      end

   end

end

----------------------------------------------------------------

function FraudMgr:_getVideoRollId( type )

   return "FraudMgr.".. type .. ".video_roll";

end

----------------------------------------------------------------

function FraudMgr:_getVideoRoll( type )

   local rollId = self:_getVideoRollId(type);
   local roll   = self:_getInternalValue();

   if roll == nil then

      roll = math.random();
      self:_setInternalValue( rollId , roll );

   end

   return roll;

end


----------------------------------------------------------------

function FraudMgr:_getCrashRollId( type )

   return "FraudMgr.".. type .. ".crash_roll";

end

----------------------------------------------------------------

function FraudMgr:_getCrashRoll( type )

   local rollId = self:_getCrashRollId(type);
   local roll   = self:_getInternalValue();

   if roll == nil then

      roll = math.random();

      self:_setInternalValue( rollId , roll );

   end

   return roll;

end

----------------------------------------------------------------

function FraudMgr:_getVersionId( type )
   return "FraudMgr.".. type .. ".version";
end

----------------------------------------------------------------

function FraudMgr:_pickPenaltyVersion( type )

   local versionId   = self:_getVersionId( type );
   local version     = self:_getInternalValue( versionId  );

   --This is the first time it happens, initialize our version
   if version == nil then

      version = self:_getCurrentVersionForTestType( type );

      if version then

         self:_setInternalValue( versionId, version  );

      end

   end

   return version;

end

----------------------------------------------------------------

--We want to be able to have a % of our users base crashing or being force to watch video
-- for being caught cheating as other dont
-- This will help us, slowly ramp up our effort to slow down fraud
-- In general, we want the behavior to be consistent:
--    On the first time I got caught, we will decide if I will crash
--    then on every time I get caught again, the same behavior will happen
-- However, we also want to be able to readjust the behavior

function FraudMgr:_onTestedPositive( type )

   local version = self:_pickPenaltyVersion(type);

   if version then

      local settings = self:_getSettingsForTypeVersion( type, version );

      if settings then

         local count       = self:_deltaInternalValue("FraudMgr._check.failed-count",                         1);
         local typeCount   = self:_deltaInternalValue("FraudMgr." .. tostring(type) .. ".failed-count",       1);
         local crash       = settings.crash_threshold and settings.crash_threshold > 0 and self:_getCrashRoll(type) <= settings.crash_threshold;
         local video       = settings.video_threshold and settings.video_threshold > 0 and self:_getVideoRoll(type) <= settings.video_threshold;

         local params       =
         {
            type        = type,
            count       = count,
            typeCount   = typeCount,
            report      = settings.report,
            video       = video,
            crash       = crash,
            mySecret    = self:_getInternalValue(SECRET_DATA_ID);
            dataSecret  = App.data:getValue(SECRET_DATA_ID);
            myDevModel  = self:_getInternalValue(DEVICE_DEV_MODEL_ID);
            myDevProduct= self:_getInternalValue(DEVICE_DEV_PRODUCT_ID);
            devModel    = MOAIEnvironment.devModel;
            devProduct  = MOAIEnvironment.devProduct;
         };

         self._crash = self._crash or crash;
         self._video = self._video or video;

         EventMgr:postEvent( params, "FraudMgr.detectCheater" );

      end
   end

end

----------------------------------------------------------------

function FraudMgr:_getSetting( id, default)

   local settings = self:_getInternalValue("FraudMgr.settings", default);

   if type(settings) == "table" then
      local val = settings[id];
      if val ~= nil then
         return val;
      end
   end

   return default;
end

----------------------------------------------------------------

function FraudMgr:_setSetting( id, val )

   local settings = self:_getInternalValue("FraudMgr.settings", {});

   settings[id] = val;

   self:_setInternalValue("FraudMgr.settings", settings );

end

----------------------------------------------------------------

function FraudMgr:_setInternalValue( key, val )

   self._internal[key] = val;

   return val;

end

----------------------------------------------------------------

function FraudMgr:_deltaInternalValue( key, delta )

   local newVal = (self._internal[key] or 0) + delta;

   self._internal[key] = newVal;

   return newVal;

end

----------------------------------------------------------------

function FraudMgr:_getInternalValue( key, default )

   local ret = self._internal[key];

   if ret ~= nil then
      return ret;
   end

   return default;

end


----------------------------------------------------------------------

