----------------------------------------------------------------------

GhostMgr  =
{
   _ghostList     = {};
   _ghostScoreK   = {};
};

----------------------------------------------------------------------

function GhostMgr:init()

   EventMgr:addListener( self, self._onUpdateMasterGhostInfo,     "cloud:master-data-ghost-info"   );

   EventMgr:addListener( self, self._onUpdateFileGhostInfo,       "cloud:file-data-ghost-info"     );

   EventMgr:addListener( self, self._onGameStart,                 "game-start"                     );

	EventMgr:addListener( self, self._onWeeklyReset,               "weekly-reset"                   );

end

----------------------------------------------------------------------

function GhostMgr:submitScore()

   if RankingMgr:isReady() then
      local level =  App.data:getCurrentPlayerLevel();
      if self._ghostList and level then

         for k,ghost in pairs(self._ghostList) do

            local scores = ghost.scores or {};
            local levels = ghost.levels or {};
            if       (scores and type(scores) == "table")
               and   (levels and type(levels) == "table")   then

               local abK    = AbTestMgr:getGhostDifficultyScoreK() or 1;
               local scoreK = self._ghostScoreK[ghost.accountId]*abK;

               if self._ghostScoreK[ghost.accountId] then
                  RankingMgr:submitGhostScore( ghost.accountId,  scoreK*scores[math.min(level,#scores)]
                                                              ,  levels[math.min(level,#levels)] );
               else

                  Debug.assert(  self._ghostScoreK[ghost.accountId],
                                 "GhostMgr:submitScore->self._ghostScoreK[ghost.accountId] : ".. tostring(ghost.accountId)  );

               end
            end
         end
      end
   end
end

----------------------------------------------------------------------

function GhostMgr:_onUpdateMasterGhostInfo( data )

   local content = (data or {}).content;

   self._ghostScoreK = content.scoreMultiplier;

end

----------------------------------------------------------------------

function GhostMgr:_onUpdateFileGhostInfo( data )

   local content = (data or {}).content;

   if content then

      self._ghostList = {};

      for i,ghost in ipairs(content) do
         self._ghostList[i] = ghost;
      end

      EventMgr:postEvent( {list = self._ghostList }, "ghost-info-updated");

   end

end

----------------------------------------------------------------------

function GhostMgr:_onGameStart()

   if RankingMgr:isReady() then

      local submit = AbTestMgr:getGhostDifficultySubmit();
      Debug.assert( submit == "always" or submit == "weekly" or submit == "never" );

      if submit == "always" then
         self:submitScore();
      elseif submit == "weekly" then
         if (not RankingMgr:isGhostMapValid()) then
            self:submitScore();
         end
      end
   end

end

----------------------------------------------------------------------

function GhostMgr:_onWeeklyReset()

   Debug.assert( RankingMgr:isReady()  );

   local submit = AbTestMgr:getGhostDifficultySubmit();

   Debug.assert( submit == "always" or submit == "weekly" or submit == "never" );

   if submit ~= "never" then
      self:submitScore();
   end

	EventMgr:postEvent( {}, "ghost-weekly-reset");

end

----------------------------------------------------------------------

function GhostMgr:getGhostVehicleInfo( id )

	for k,ghost in pairs(self._ghostList) do
		if ghost.accountId == id then
			return { vehicleIndex = ghost.vehicle, vehiclePaint = ghost.vehcPaint, vehicleArmor = ghost.vehcArmor };
		end
	end

end

----------------------------------------------------------------------

function GhostMgr:getGhostIndex( id )

	for i,ghost in ipairs(self._ghostList) do

		if ghost.accountId == id then
			return i;
		end
	end

end

----------------------------------------------------------------------


