----------------------------------------------------------------------

local GUEST_LIST_FILENAME_LUA  = "98q74htv72.w37";
local GUEST_LIST_FILENAME_JSON = "98q74htv72gue57.jp5";
local MIN_TARGET_SCORE         = 8719;
local MIN                      = 60;
local HOUR                     = 60*60;
local DAY                      = 60*60*24;

local DEFAULT_COOLDOWN_TBL =
{
   {365*DAY, 365*DAY},
   {365*DAY, 365*DAY},
   {365*DAY, 365*DAY},
   {365*DAY, 365*DAY},
};

local DEFAULT_ZERO_SCALE_TBL =
{
   {0, 0},
   {0, 0},
   {0, 0},
   {0, 0},
};

local DEFAULT_100_SCALE_TBL =
{
   {100, 100},
   {100, 100},
   {100, 100},
   {100, 100},
};

-- 0.016667 to min per sec: 1/60
-- 0.000278 to hour per sec: 1/(60*60)
-- 0.005952 to week per hour: (60*60)/(60*60*24*7)
local HACK_COOLDOWN_K = nil;

if not BUILD_DEBUG then
   HACK_COOLDOWN_K = nil;
end

----------------------------------------------------------------------

GuestMgr  =
{
   _guestList           = {},
   _guestStats          = {},
   _guestListIndex      = nil,
   _guestPick           = {},
   _loggedIn            = false,
   _enable              = true,
   _unlockLevel         = 2,
   _tangoOn             = true,
   _pickCount           = 16,
   _cycleCount          = 16,
   _initUpdateK         = 0.5,
   _initScoreK          = 1.0,
   _scoreUpdateK        = 1,
   _scoreRangeK         = 1.25,
   _scoreIncK           = 1.25,
   _giftUpdateK         = 2,
   _giftRepayK          = 0.75,
   _levelUpdateK        = 1,
   _levelRangeK         = 1,
   _scoreOldK           = 0.8,
   _scoreNewK           = 0.2,
   _weeklyScoreCapK     = {2, 5},
   _guestInitScoreTbl   =
        {
            {0.0,       0.00},
            {0.25,  0.75},
            {0.50,  1.00},
            {0.75,  1.25},
        },
   _guestUpdateCDTbl    =
        {
            {365*DAY,   365*DAY},
            {1.5*DAY,   3.0*DAY},
            {1.0*HOUR,  2.0*DAY},
            {0.5*DAY,   1.0*DAY},
        },
   _guestScoreRngTbl    =
        {
            {0.0,       0.00},
            {0.0,       0.75},
            {0.0,       1.00},
            {0.0,       1.25},
        },
   _guestScoreProgTbl   =
        {
            {0.0,       0.00},
            {0.0,       0.10},
            {0.02,  0.12},
            {0.05,  0.15},
        },
   _guestGiftCDTbl      =
        {
            {365*DAY,   365*DAY},
            {4*DAY,     10*DAY},
            {2*DAY,     5*DAY},
            {1*DAY,     3*DAY},
        },
   _guestGiftRepayTbl   =
        {
            100.0,
            1.0,
            0.9,
            0.8,
        },
   _guestLevelUpTbl     =
        {
            {365*DAY,   365*DAY},
            {6*DAY,     8*DAY},
            {3*DAY,     5*DAY},
            {2*DAY,     4*DAY},
        },
   _guestLevelRngTbl    =
        {
            {0, 0},
            {-5,    -1},
            {-2,    2},
            {-1,    4},
        },
};

----------------------------------------------------------------------

function GuestMgr:init()

   EventMgr:addListener( self, self._onUpdateMasterGuestInfo, "cloud:master-data-guest-friend-info" );
   EventMgr:addListener( self, self._onUpdateFileGuestInfo,   "cloud:file-data-guest-friend-info" );

   if BUILD_DEBUG then
      if HACK_QUICK_WEEKLY_RESET and HACK_QUICK_WEEKLY_RESET_TIME then
         HACK_COOLDOWN_K = HACK_QUICK_WEEKLY_RESET_TIME/(60*60*24*7);
      end

      EventMgr:addListener( self, self._onTestResetPick, "test-guest-friend-reset" );
   end

end

----------------------------------------------------------------------

function GuestMgr:onLoginSuccess()

   self._loggedIn = true;
   self:_initGuestStats();

end

----------------------------------------------------------------------

function GuestMgr:_onUpdateMasterGuestInfo( data )

   local content           = (data or {}).content;

   if content.enable ~= nil then
      self._enable = content.enable;
   end

   if content.tangoOn ~= nil then
      self._tangoOn = content.tangoOn;
   end

   self._unlockLevel       = content.unlockLevel or self._unlockLevel;
   self._pickCount         = content.pickCount or self._pickCount;
   self._cycleCount        = content.cycleCount or self._cycleCount;
   self._initUpdateK       = content.initUpdateK or self._initUpdateK;
   self._initScoreK        = content.initScoreK or self._initScoreK;
   self._scoreUpdateK      = content.scoreUpdateK or self._scoreUpdateK;
   self._scoreRangeK       = content.scoreRangeK or self._scoreRangeK;
   self._scoreIncK         = content.scoreIncK or self._scoreIncK;
   self._giftUpdateK       = content.giftUpdateK or self._giftUpdateK;
   self._giftRepayK        = content.giftRepayK or self._giftRepayK;
   self._levelUpdateK      = content.levelUpdateK or self._levelUpdateK;
   self._levelRangeK       = content.levelRangeK or self._levelRangeK;
   self._scoreOldK         = content.scoreOldK or self._scoreOldK;
   self._scoreNewK         = content.scoreNewK or self._scoreNewK;
   self._weeklyScoreCapK   = content.weeklyScoreCapK or self._weeklyScoreCapK;

   self._guestInitScoreTbl = content.initRange or self._guestInitScoreTbl;
   self._guestUpdateCDTbl  = content.updateCooldown or self._guestUpdateCDTbl;
   self._guestScoreRngTbl  = content.scoreRange or self._guestScoreRngTbl;
   self._guestScoreProgTbl = content.scoreProgress or self._guestScoreProgTbl;
   self._guestGiftCDTbl    = content.giftCooldown or self._guestGiftCDTbl;
   self._guestGiftRepayTbl = content.giftRepay or self._guestGiftRepayTbl;
   self._guestLevelUpTbl   = content.levelUpCooldown or self._guestLevelUpTbl;
   self._guestLevelRngTbl  = content.levelRange or self._guestLevelRngTbl;

end

----------------------------------------------------------------------

function GuestMgr:_onUpdateFileGuestInfo( data )

   local content = (data or {}).content;

   self:_loadGuestList();

   if content then
      self._guestList = content;

      EventMgr:postEvent( {list = self._guestList }, "guest-info-updated");

      if self._loggedIn then
         self:_initGuestStats();
      end
   end

end

----------------------------------------------------------------------

function GuestMgr:_loadGuestList()

   local saveFile = loadFileMultiFormat( GUEST_LIST_FILENAME_LUA, GUEST_LIST_FILENAME_JSON );

   if saveFile then
      self._guestStats     = saveFile.stats or {};
      self._guestListIndex = saveFile.index or 1;
      self._guestPick      = saveFile.pick or {};
   end

end

----------------------------------------------------------------------

function GuestMgr:_saveGuestList()

   if self._guestStats and next(self._guestStats) ~= nil then

      local data = { stats=self._guestStats, index=self._guestListIndex, pick=self._guestPick };

      saveFileMultiFormat(GUEST_LIST_FILENAME_LUA, GUEST_LIST_FILENAME_JSON, data);

   end

end

----------------------------------------------------------------------

function GuestMgr:_initGuestStats()

   if not LeagueMgr:isLeagueLoggedIn() then
      LeagueMgr:onLoginSuccess();
   end

   local timeGMT  = TimeMgr:getTimeGMT() or TimeMgr:getTimeLocalGMT();
   local rTime    = RankingMgr:getWeeklyResetTime();
   local maxVhc   = App.data:getHighestPlayerVehicleUnlocked();
   local vhcCount = App.data:getPlayerVehicleCount();
   local pHiScore = math.max(App.data:getValue("guest-mgr-target-score", MIN_TARGET_SCORE), MIN_TARGET_SCORE);
   local pLevel   = App.data:getCurrentPlayerLevel();
   local mLevel   = App.data:getMaxPlayerLevel();
   local initScK  = AbTestMgr:getGuestFriendInitScoreK() or self._initScoreK;
   local scrCapK  = AbTestMgr:getGuestFriendWeeklyScoreCapK() or self._weeklyScoreCapK;
   local l,d      = LeagueMgr:getPlayerPlacement();

   if rTime then
      if HACK_QUICK_WEEKLY_RESET and HACK_QUICK_WEEKLY_RESET_TIME then
         rTime = rTime - HACK_QUICK_WEEKLY_RESET_TIME;
      else
         rTime = rTime - (60*60*24*7);
      end
   end

   for _, v in ipairs(self._guestList) do
      if not self._guestStats[v.accountId] then

         local initRng     = self._guestInitScoreTbl[v.initScore];
         local minScore    = math.floor(pHiScore*initRng[1]*initScK);
         local maxScore    = math.ceil(pHiScore*initRng[2]*initScK);
         local t           = {};

         t.vehicle         = math.clamp( maxVhc+math.random(v.vehcRange[1], v.vehcRange[2]), 1, vhcCount);
         t.vehcPaint       = v.vehcPaintId;
         t.vehcArmor       = math.random(v.vehcArmrRng[1], v.vehcArmrRng[2]);
         t.updateTStamp    = rTime or timeGMT;
         t.updateTDelta    = math.random();
         t.score           = math.floor(math.mapClamped( 0, minScore, 1, maxScore, math.random() ));
         t.weeklyCap       = math.floor(maxScore * math.mapClamped(0, scrCapK[1], 1, scrCapK[2], math.random()));
         t.prevScoreTStamp = rTime;
         t.prevScore       = math.floor(math.mapClamped( 0, minScore, 1, maxScore, math.random() ));
         t.giftTStamp      = timeGMT;
         t.giftTDelta      = math.random();
         t.giftReceived    = false;
         t.levelTStamp     = rTime or timeGMT;
         t.levelTDelta     = math.random();
         t.level           = math.clamp( pLevel+math.random(self._guestLevelRngTbl[v.levelRange][1], self._guestLevelRngTbl[v.levelRange][2]), 2, mLevel);

         local lvl = t.level + 1;

         for i=lvl,App.data:getMaxPlayerLevel() do
            if i >= App.data:getMaxPlayerLevel() then
               break;
            end

            if t.score >= App.data:getXPLevelReq(i) then
               t.level = t.level + 1;
            else
               break;
            end
         end

         self._guestStats[v.accountId] = t;
      end

      self._guestStats[v.accountId].leagueIndex = l;
      self._guestStats[v.accountId].divIndex    = d;
   end

   self:_cleanGuestPick();
   self:_pickGuestOpponents(nil, LeagueMgr:isLeagueEnabled());
   self:_saveGuestList();

end

----------------------------------------------------------------------

function GuestMgr:updateTargetScore( score )

   if score and score > 0 and score > App.data:getValue("guest-mgr-target-score", 0) then

      local avg  = App.data:getValue("guest-mgr-target-score", MIN_TARGET_SCORE);
      local oldK = AbTestMgr:getGuestFriendOldScoreK() or self._scoreOldK;
      local newK = AbTestMgr:getGuestFriendNewScoreK() or self._scoreNewK;

      avg = math.floor(oldK*avg + newK*score);
      App.data:setValue("guest-mgr-target-score", math.max(avg, MIN_TARGET_SCORE) );
   end

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function GuestMgr:_onTestResetPick()

   Debug.assert( RankingMgr:isReady() );

   if self:isGuestFriendEnabled() then

      local timeRef = TimeMgr:getTimeGMT() or TimeMgr:getTimeLocalGMT();
      local cycle   = AbTestMgr:getGuestFriendCycle() or self._cycleCount;

      self:_pickGuestOpponents(cycle, LeagueMgr:isLeagueEnabled());
      self:submitPrevGuestFriendScore();
      self:_resetGuestStats(true);
      self:_updateGuestLevel(timeRef, true);
      self:_saveGuestList();
      RankingMgr:_downloadRank();
   end

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function GuestMgr:_onWeeklyReset()

   Debug.assert( RankingMgr:isReady() );

   if self:isGuestFriendEnabled() then

      local timeRef = TimeMgr:getTimeGMT() or TimeMgr:getTimeLocalGMT();
      local cycle   = AbTestMgr:getGuestFriendCycle() or self._cycleCount;

      self:_pickGuestOpponents(cycle, LeagueMgr:isLeagueEnabled());
      self:_resetGuestStats();
      self:_updateGuestScore(timeRef);
      self:_updateGuestLevel(timeRef, true);
      self:_saveGuestList();
   end

end

----------------------------------------------------------------------

function GuestMgr:_resetGuestStats( forceCurTime )

   local timeStamp = RankingMgr:getWeeklyResetTime() or TimeMgr:getTimeGMT() or TimeMgr:getTimeLocalGMT();

   timeStamp = timeStamp - 60*60*24*7;

   if forceCurTime then
      timeStamp = TimeMgr:getTimeGMT();
   elseif HACK_QUICK_WEEKLY_RESET and HACK_QUICK_WEEKLY_RESET_TIME then
      timeStamp = RankingMgr:getWeeklyResetTime() - HACK_QUICK_WEEKLY_RESET_TIME;
   end

   if self._guestStats then
      for k,v in pairs(self._guestStats) do
         v.score        = 0;
         v.weeklyCap    = nil;
         v.updateTStamp = timeStamp;
         v.updateTDelta = math.random();
         v.giftTStamp   = timeStamp;
         v.giftTDelta   = math.random();
         v.levelTStamp  = timeStamp;
         v.levelTDelta  = math.random();
      end
   end

end

----------------------------------------------------------------------

 function GuestMgr:resetGuestFriend()

   Debug.assert( RankingMgr:isReady() );

   if not self:isGuestFriendEnabled() then
      return;
   end

   local timeRef = TimeMgr:getTimeGMT() or TimeMgr:getTimeLocalGMT();
   local cycle   = AbTestMgr:getGuestFriendMax() or self._pickCount;
   local player  = RankingMgr:getPlayer();

   if LeagueMgr:isLeagueEnabled() then
      cycle = math.max( 0, (LeagueMgr:getPlayerCohortSize() or cycle) - RankingMgr:getRankEntryNonGuestCount(true) );
   end

   App.data:setValue("guest-mgr-target-score", math.max(player.score, player.prevScore or 0, MIN_TARGET_SCORE, App.data:getValue("alltime-high-score", MIN_TARGET_SCORE)));

   self:_pickGuestOpponents(cycle, LeagueMgr:isLeagueEnabled());
   self:_resetGuestStats();
   self:_updateGuestScore(timeRef);
   self:_updateGuestLevel(timeRef, true);
   self:_saveGuestList();
   RankingMgr:removeAllGuestFriends();

end

----------------------------------------------------------------------

function GuestMgr:cycleGuestFriend()

   self:_onWeeklyReset();

end

----------------------------------------------------------------------

function GuestMgr:submitPrevGuestFriendScore( force )

   local timeStamp = RankingMgr:getWeeklyResetTime();
   local time      = TimeMgr:getTimeGMT() or TimeMgr:getTimeLocalGMT();

   timeStamp = timeStamp - 60*60*24*7;

   if HACK_QUICK_WEEKLY_RESET and HACK_QUICK_WEEKLY_RESET_TIME then
      timeStamp = RankingMgr:getWeeklyResetTime() - HACK_QUICK_WEEKLY_RESET_TIME;
      time      = time - HACK_QUICK_WEEKLY_RESET_TIME;
   else
      time      = time - 60*60*24*7;
   end

   if self._guestStats then
      for k,v in pairs(self._guestStats) do
         if not v.prevScoreTStamp or time >= (v.prevScoreTStamp or time) or force then

            if v.score > 0 then
               v.prevScore       = v.score
            end

            v.prevScoreTStamp = timeStamp;
         end
      end
   end

   self:_saveGuestList();

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function GuestMgr:_addGuestPick( initIndex, countMax )

   if initIndex and countMax then
      local index = initIndex;

      for i=1,#self._guestList do

         if #self._guestPick >= countMax then
            return index;
         end

         if index > #self._guestList then
            index = 1;
         end

         if not self:_isGuestPickDupe(index) then
            table.insert(self._guestPick, index);
         end

         index = index + 1;
      end

      return index;
   end

   return self._guestListIndex;

end

----------------------------------------------------------------------

function GuestMgr:_isGuestPickDupe( pick )

   for k,v in ipairs(self._guestPick) do
      if v == pick then
         return true;
      end
   end

   return false;

end

----------------------------------------------------------------------

function GuestMgr:_cleanGuestPick()

   local maxPick = AbTestMgr:getGuestFriendMax() or self._pickCount;

   if LeagueMgr:isLeagueEnabled() then
      maxPick = math.max( 0, (LeagueMgr:getPlayerCohortSize() or maxPick) - RankingMgr:getRankEntryNonGuestCount(true) );
   end

   if not self:isGuestFriendEnabled() then
      maxPick = 0;
   end

   if self._guestPick and self._guestStats and self._guestList then
      for i=#self._guestPick,1,-1 do
         if    not self._guestPick[i]
            or not self._guestList[self._guestPick[i]]
            or not self._guestList[self._guestPick[i]].accountId
            or not self._guestStats[self._guestList[self._guestPick[i]].accountId] then

            table.remove(self._guestPick, i);
         end
      end

      if #self._guestPick > maxPick then
         for i=#self._guestPick, maxPick, -1 do
            if self._guestPick[i] then
               table.remove(self._guestPick, i);
            end
         end
      end
   end

end

----------------------------------------------------------------------

function GuestMgr:_pickGuestOpponents( remCount, enableLeague )

   Debug.assertType(enableLeague, "boolean", "GuestMgr:_pickGuestOpponents -> enableLeague");

   if self._guestList and #self._guestList > 0 then

      local delCount = remCount or 0;
      local pick     = AbTestMgr:getGuestFriendMax() or self._pickCount;

      if LeagueMgr:isLeagueEnabled() then
         pick = math.max( 0, (LeagueMgr:getPlayerCohortSize() or pick) - RankingMgr:getRankEntryNonGuestCount(true) );
      end

      if not self:isGuestFriendEnabled() then
         pick = 0;
      end

      if not self._guestListIndex then
         self._guestListIndex = math.random(1, #self._guestList);
      elseif self._guestListIndex > #self._guestList then
         self._guestListIndex = 1;
      end

      self._guestListIndex = math.clamp(self._guestListIndex, 1, #self._guestList);

      self:_cycleGuestPick(delCount, enableLeague);
      self._guestListIndex = self:_addGuestPick(self._guestListIndex, pick);
   end

end

----------------------------------------------------------------------

function GuestMgr:_removeGuestPick( guestIndex )

   if self._guestPick and #self._guestPick > 0 then
      for k,v in ipairs(self._guestPick) do
         if v == guestIndex then
            table.remove(self._guestPick, k);

            return true;
         end
      end
   end

end

----------------------------------------------------------------------

function GuestMgr:_cycleGuestPick( remCount, enableLeague )

   if enableLeague then
      self:_cycleGuestPick_League(remCount);
   else
      self:_cycleGuestPick_NonLeague(remCount);
   end

end

function GuestMgr:_cycleGuestPick_League( remCount )

   if remCount < 1 then
      return; -- remCount < 1 means don't cycle out the guests.
   end

   local rankCount;
   local playerRnk;

   if RankingMgr:isLastWeekRankingValid() then
      rankCount = RankingMgr:getLastWeekRankEntryCount();
      playerRnk = RankingMgr:getPlayerLastWeekRank();
   else
      rankCount = RankingMgr:getRankEntryCount();
      playerRnk = RankingMgr:getPlayerRank();
   end

   print("GuestMgr rankCount", rankCount, "playerRnk", playerRnk) -- TEMP

   local leaguePlace, divisionPlace = LeagueMgr:getPlayerPlacement();
   local promotionCount             = LeagueMgr:getPromoCount(leaguePlace, divisionPlace);
   local demotionCount              = LeagueMgr:getDemoCount(leaguePlace, divisionPlace);

   local ceil  = promotionCount or 0;
   local floor = rankCount - demotionCount;
   local mid   = math.max(1, floor - 1);

   if playerRnk <= ceil then
      print("GuestMgr Calling _removeLowestPick") -- TEMP
      self:_removeLowestPick(math.max(LeagueMgr:getPlayerCohortSize()-1, 0));
   elseif playerRnk <= mid then
      print("GuestMgr Calling _removeOutlierPick") -- TEMP
      self:_removeOutlierPick(math.max(LeagueMgr:getPlayerCohortSize()-1, 0));
   else
      print("GuestMgr Calling _removeHighestPick") -- TEMP
      self:_removeHighestPick(math.max(LeagueMgr:getPlayerCohortSize()-1, 0));
   end

end

function GuestMgr:_cycleGuestPick_NonLeague( remCount )

   if remCount < 1 then
      return; -- remCount < 1 means don't cycle out the guests.
   end

   local rankCount;
   local playerRnk;

   if RankingMgr:isLastWeekRankingValid() then
      rankCount = RankingMgr:getLastWeekRankEntryCount();
      playerRnk = RankingMgr:getPlayerLastWeekRank();
   else
      rankCount = RankingMgr:getRankEntryCount();
      playerRnk = RankingMgr:getPlayerRank();
   end

   local ceil  = math.ceil(rankCount/3);
   local floor = math.floor(rankCount/3);
   local mid   = ceil + floor;

   if playerRnk <= ceil then
      self:_removeLowestPick(remCount);
   elseif playerRnk <= mid then
      self:_removeOutlierPick(remCount);
   else
      self:_removeHighestPick(remCount);
   end

end

----------------------------------------------------------------------

function GuestMgr:_removeHighestPick( remCount )

   local rankCount = RankingMgr:getLastWeekRankEntryCount();
   local flag      = RankingMgr:isLastWeekRankingValid();

   if not flag then
      rankCount = RankingMgr:getRankEntryCount();
   end

   if     remCount and remCount > 0
      and self._guestPick and #self._guestPick > 0
      and rankCount and rankCount > 0 then

      self:_bumpTargetScoreDown();

      local count = 0;

      for i=1,rankCount do

         if count >= remCount then
            break;
         else

            local entry = RankingMgr:getLastWeekRankEntry(i);

            if not flag then
               entry = RankingMgr:getRankEntry(i);
            end

            if entry and ( entry.contact == nil or entry.contact:isGuestFriend() ) then
               local index = self:getGuestIndex(entry.tangoId);

               if index and self:_removeGuestPick(index) then
                  count = count + 1;
               end
            end
         end
      end
   end

end

----------------------------------------------------------------------

function GuestMgr:_removeLowestPick( remCount )

   local rankCount = RankingMgr:getLastWeekRankEntryCount();
   local flag      = RankingMgr:isLastWeekRankingValid();

   if not flag then
      rankCount = RankingMgr:getRankEntryCount();
   end

   if     remCount and remCount > 0
      and self._guestPick and #self._guestPick > 0
      and rankCount and rankCount > 0 then

      self:_bumpTargetScoreUp();

      local count = 0;

      for i=rankCount,1,-1 do

         if count >= remCount then
            break;
         else

            local entry = RankingMgr:getLastWeekRankEntry(i);

            if not flag then
               entry = RankingMgr:getRankEntry(i);
            end

            if entry and ( entry.contact == nil or entry.contact:isGuestFriend() ) then
               local index = self:getGuestIndex(entry.tangoId);

               if index and self:_removeGuestPick(index) then
                  count = count + 1;
               end
            end
         end
      end
   end

end

----------------------------------------------------------------------

function GuestMgr:_removeOutlierPick( remCount, remCount2 )

   local rankCount = RankingMgr:getLastWeekRankEntryCount();
   local flag      = RankingMgr:isLastWeekRankingValid();
   local lowCount  = math.ceil(remCount/2);
   local top       = remCount or 0;
   local bottom    = remCount2 or top;

   if not flag then
      rankCount = RankingMgr:getRankEntryCount();
   end

   if remCount2 then
      lowCount = bottom;
   end

   if bottom and bottom > 0
      and self._guestPick and #self._guestPick > 0
      and rankCount and rankCount > 0 then

      local count = 0;

      for i=rankCount,1,-1 do

         if count >= lowCount then
            break;
         else

            local entry = RankingMgr:getLastWeekRankEntry(i);

            if not flag then
               entry = RankingMgr:getRankEntry(i);
            end

            if entry and ( entry.contact == nil or entry.contact:isGuestFriend() ) then
               local index = self:getGuestIndex(entry.tangoId);

               if index and self:_removeGuestPick(index) then
                  count = count + 1;
               end
            end
         end
      end
   end

   if      top and top > 0
      and self._guestPick and #self._guestPick > 0
      and rankCount and rankCount > 0 then

      local count = 0;

      for i=1,rankCount do

         if count >= top then
            break;
         else

            local entry = RankingMgr:getLastWeekRankEntry(i);

            if not flag then
               entry = RankingMgr:getRankEntry(i);
            end

            if entry and ( entry.contact == nil or entry.contact:isGuestFriend() ) then
               local index = self:getGuestIndex(entry.tangoId);

               if index and self:_removeGuestPick(index) then
                  count = count + 1;
               end
            end
         end
      end
   end

end

----------------------------------------------------------------------

function GuestMgr:_bumpTargetScoreDown()

   local avgScore = App.data:getValue("guest-mgr-target-score", MIN_TARGET_SCORE);
   local oldK     = AbTestMgr:getGuestFriendOldScoreK() or self._scoreOldK;

   App.data:setValue( "guest-mgr-target-score", math.floor( oldK*avgScore ) );

end

----------------------------------------------------------------------

function GuestMgr:_bumpTargetScoreUp()

   local hiScore  = App.data:getValue("alltime-high-score", MIN_TARGET_SCORE);
   local avgScore = App.data:getValue("guest-mgr-target-score", MIN_TARGET_SCORE);
   local oldK     = AbTestMgr:getGuestFriendOldScoreK() or self._scoreOldK;

   App.data:setValue( "guest-mgr-target-score", math.floor( math.max(oldK*hiScore, avgScore) ) );

end


----------------------------------------------------------------------
----------------------------------------------------------------------

function GuestMgr:isGuestFriendEnabled()

   local enable  = AbTestMgr:getGuestFriendEnable() or LeagueMgr:isLeagueEnabled();
   local lvlLock = self:isGuestFriendLevelLocked();
   local tango   = self:isGuestFriendTangoEnabled();
   local online  = LoginMgr:isTango();

   if enable == nil then
      enable = self._enable;
   end

   if online then
      return enable and not lvlLock and tango;
   end

   return enable and not lvlLock;

end

----------------------------------------------------------------------

function GuestMgr:isGuestFriendLevelLocked()

   local lvl  = AbTestMgr:getGuestFriendUnlockLevel() or self._unlockLevel;
   local plvl = App.data:getCurrentPlayerLevel() or 1;

   if lvl and plvl and ( plvl >= lvl ) then
      return false;
   end

   return true;

end

----------------------------------------------------------------------

function GuestMgr:isGuestFriendLoggedIn()
   return self._loggedIn;
end

----------------------------------------------------------------------

function GuestMgr:isGuestFriendTangoEnabled()

   local enable = AbTestMgr:getGuestFriendTangoOn();

   if enable == nil then
      enable = self._tangoOn;
   end

   return enable;

end

----------------------------------------------------------------------

function GuestMgr:getGuestVehicleInfo( id )

   if id and self._guestStats and self._guestStats[id] then
      return { vehicleIndex = self._guestStats[id].vehicle, vehiclePaint = self._guestStats[id].vehcPaint, vehicleArmor = self._guestStats[id].vehcArmor };
   end

   return { vehicleIndex = 1, vehiclePaint = 1, vehicleArmor = 1 };
end

----------------------------------------------------------------------

function GuestMgr:getGuestIndex( id )

   for i,guest in ipairs(self._guestList) do
      if guest.accountId == id then
         return i;
      end
   end

end

----------------------------------------------------------------------

function GuestMgr:sendGuestGift( id )

   local index = self:getGuestIndex(id);

   if id and self._guestStats and self._guestStats[id] then
      self._guestStats[id].giftReceived = true;
   end

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function GuestMgr:_addPlayer( list )

   local player = RankingMgr:getPlayer();

   if player then
      table.insert(list, {tangoId=player.tangoId, score=player.score or 0, level=player.level or 1, prevScore=player.prevScore or 0, prevScoreStamp=player.prevScoreStamp });
   end

end

----------------------------------------------------------------------

function GuestMgr:_rankInsert( tbl, tangoId, score, level, prevScore, prevScoreStamp )

   local entry = {tangoId=tangoId,
                  score=score or 0,
                  level=level or 1,
                  prevScore=prevScore or 0,
                  prevScoreStamp=prevScoreStamp};

   for i=1,#tbl do
      -- Use >= instead of > to have player on top when having same score
      if score >= tbl[i].score then
         table.insert( tbl, i, entry );
         return;
      end
   end

   table.insert(tbl, entry);

end

----------------------------------------------------------------------

function GuestMgr:_updateGuestGift( timeRef )

   local curState = StateMgr:getCurState();

   if not curState
      or (curState and curState.stateFilename ~= "state/state-ranking.lua") then

      return false;
   end

   local player = RankingMgr:getPlayer();

   if player and player.score and player.score < 1 then
      return false;
   end

   local totalKey    = 0;
   local abCooldownK = AbTestMgr:getGuestFriendGiftUpdateK() or self._giftUpdateK;
   local abRepayK    = AbTestMgr:getGuestFriendGiftRepayK() or self._giftRepayK;

   if HACK_COOLDOWN_K then
      abCooldownK = abCooldownK * HACK_COOLDOWN_K;
   end

   for k,v in pairs(self._guestPick) do
      if self:_addGuestGift(v, timeRef, abCooldownK, abRepayK) then
         totalKey = totalKey + 1;
      end
   end

   if totalKey and totalKey > 0 then
      App.data:addKey( totalKey, "Gift" );
   end

end

function GuestMgr:_addGuestGift( index, timeRef, abCooldownK, abRepayK, totalKey )

   if type(self._guestList) ~= "table" then
      Debug.error("GuestMgr:_addGuestGift -> self._guestList is not table!")
      return;
   end

   if not self._guestList[index] then
      Debug.error("GuestMgr:_addGuestGift -> self._guestList[index] is nil!")
      return;
   end

   if self._guestList[index] then

      local timeRng = self._guestGiftCDTbl[self._guestList[index].giftCooldown or 1];
      local id      = self._guestList[index].accountId;

      if not self._guestStats[id].giftTDelta then
         self._guestStats[id].giftTDelta = math.random();
      end

      local timeK = 1;
      local delta;

      if self._guestStats[id] and self._guestStats[id].giftReceived then
         timeK = abRepayK * self._guestGiftRepayTbl[self._guestList[index].giftRepayK or 1];
      end

      delta = abCooldownK * math.mapClamped(0, timeRng[1], 1, timeRng[2], self._guestStats[id].giftTDelta) * timeK;

      if     self._guestStats[id]
         and self._guestStats[id].giftTStamp
         and ( timeRef >= (self._guestStats[id].giftTStamp+delta) ) then

         self._guestStats[id].giftTStamp   = timeRef;
         self._guestStats[id].giftReceived = false;
         self._guestStats[id].levelTDelta  = math.random();

         setCoolDownCount("Gift2Tracker", id, 1);

         if App.data:isWeeklyGiftCollectFull(totalKey) or (LeagueMgr:isLeagueEnabled() and not App.data:isLeagueUnlocked()) then
            return false;
         else
            EventMgr:postEvent({ giver=id, keys=1}, "keys-gifted");
            return true;
         end
      end
   end

end

----------------------------------------------------------------------

function GuestMgr:_updateGuestLevel( timeRef, forceUpdate )

   local baseLevel   = App.data:getCurrentPlayerLevel();
   local maxLevel    = App.data:getMaxPlayerLevel();
   local vehcBase    = App.data:getHighestPlayerVehicleUnlocked();
   local vehcMax     = App.data:getPlayerVehicleCount();
   local abCooldownK = AbTestMgr:getGuestFriendLevelUpdateK() or self._levelUpdateK;
   local abRangeK    = AbTestMgr:getGuestFriendLevelRangeK() or self._levelRangeK;

   if HACK_COOLDOWN_K then
      abCooldownK = abCooldownK * HACK_COOLDOWN_K;
   end

   for k,v in pairs(self._guestPick) do
      self:_advanceGuestLevel(v, timeRef, baseLevel, maxLevel, vehcBase, vehcMax, abCooldownK, abRangeK, forceUpdate);
   end

end

function GuestMgr:_advanceGuestLevel( index, timeRef, baseLevel, maxLevel, vehcBase, vehcMax, abCooldownK, abRangeK,  forceUpdate )

   if type(self._guestList) ~= "table" then
      Debug.error("GuestMgr:_advanceGuestLevel -> self._guestList is not table!")
      return;
   end

   if not self._guestList[index] then
      Debug.error("GuestMgr:_advanceGuestLevel -> self._guestList[index] is nil!")
      return;
   end

   if self._guestList[index] then

      local timeRng = self._guestLevelUpTbl[self._guestList[index].levelCooldown or 1];
      local id      = self._guestList[index].accountId;

      if not self._guestStats[id].levelTDelta then
         self._guestStats[id].levelTDelta = math.random();
      end

      local delta = abCooldownK * math.mapClamped(0, timeRng[1], 1, timeRng[2], self._guestStats[id].levelTDelta);

      if self._guestStats[id] and self._guestStats[id].levelTStamp
      and ( (timeRef >= self._guestStats[id].levelTStamp+delta) or forceUpdate) then

         local levelRng = self._guestLevelRngTbl[self._guestList[index].levelRange or 1];
         local minLevel = 2;
         local maxLevel = baseLevel+math.ceil(levelRng[2]*abRangeK);

         if forceUpdate then
            minLevel = math.max(baseLevel+math.floor(levelRng[1]*abRangeK), 2);

            self._guestStats[id].level = math.max(baseLevel+math.random(levelRng[1], levelRng[2]), self._guestStats[id].level+1);
         else
            self._guestStats[id].level = self._guestStats[id].level+1;
         end

         self._guestStats[id].levelTStamp = timeRef;
         self._guestStats[id].level       = math.clamp(self._guestStats[id].level, minLevel, maxLevel) or baseLevel;
         self._guestStats[id].levelTDelta = math.random();

         local minVehc = math.clamp(vehcBase + self._guestList[index].vehcRange[1], 1, vehcMax);
         local maxVehc = math.clamp(vehcBase + self._guestList[index].vehcRange[2], 1, vehcMax);
         local prvVehc = self._guestStats[id].vehicle;

         self._guestStats[id].vehcPaint = self._guestList[index].vehcPaintId;
         self._guestStats[id].vehicle   = math.max(math.random(minVehc, maxVehc), self._guestStats[id].vehicle);

         if self._guestStats[id].vehicle == prvVehc then
            self._guestStats[id].vehcArmor = math.clamp( self._guestStats[id].vehcArmor+1, self._guestList[index].vehcArmrRng[1], self._guestList[index].vehcArmrRng[2]);
         else
            self._guestStats[id].vehcArmor = math.random(self._guestList[index].vehcArmrRng[1], self._guestList[index].vehcArmrRng[2]);
         end
      end

   end

end

----------------------------------------------------------------------

function GuestMgr:_updateGuestScore( timeRef )

   local hiScore     = math.max(App.data:getValue("guest-mgr-target-score", MIN_TARGET_SCORE), MIN_TARGET_SCORE);
   local abInitK     = AbTestMgr:getGuestFriendInitScoreK( ) or self._initScoreK;
   local abCooldownK = AbTestMgr:getGuestFriendScoreUpdateK() or self._scoreUpdateK;
   local abRangeK    = AbTestMgr:getGuestFriendScoreRangeK() or self._scoreRangeK;
   local abIncK      = AbTestMgr:getGuestFriendScoreIncrementK() or self._scoreIncK;
   local abIntUptK   = AbTestMgr:getGuestFriendInitUpdateK() or self._initUpdateK;

   if HACK_COOLDOWN_K then
      abCooldownK = abCooldownK * HACK_COOLDOWN_K;
      abIntUptK   = abIntUptK   * HACK_COOLDOWN_K;
   end

   for _, v in pairs(self._guestPick) do
      self:_advanceGuestScore(v, timeRef, hiScore, abCooldownK, abRangeK, abIncK, abIntUptK, abInitK);
   end

end

function GuestMgr:_advanceGuestScore( index, timeRef, hiScore, abCooldownK, abRangeK, abIncK, abIntUptK, abInitK )

   if type(self._guestList) ~= "table" then
      Debug.error("GuestMgr:_advanceGuestScore -> self._guestList is not table!")
      return;
   end

   if not self._guestList[index] then
      Debug.error("GuestMgr:_advanceGuestScore -> self._guestList[index] is nil!")
      return;
   end

   local timeRng    = self._guestUpdateCDTbl[self._guestList[index].updateCooldown or 1];
   local initRng    = self._guestInitScoreTbl[self._guestList[index].initScore or 1];
   local scoreRng   = self._guestScoreRngTbl[self._guestList[index].scoreRangeK or 1];
   local deltaRng   = self._guestScoreProgTbl[self._guestList[index].scoreRateK or 1];
   local id         = self._guestList[index].accountId;

   if (not timeRng) or (not initRng) or (not scoreRng) or (not deltaRng) or (not id) then

      -- This state seems to occur when GuestMgr expects downloaded data but did not receive it.
      Debug.assertType(timeRng,  "table", "GuestMgr:_advanceGuestScore -> timeRng");
      Debug.assertType(initRng,  "table", "GuestMgr:_advanceGuestScore -> initRng");
      Debug.assertType(scoreRng, "table", "GuestMgr:_advanceGuestScore -> scoreRng");
      Debug.assertType(deltaRng, "table", "GuestMgr:_advanceGuestScore -> deltaRng");
      Debug.assertType(id,       "string", "GuestMgr:_advanceGuestScore -> id");

      return;
   end

   local minScore = math.floor(hiScore * scoreRng[1] * abRangeK);
   local maxScore = math.ceil(hiScore * scoreRng[2] * abRangeK);
   local minDelta = abIncK * deltaRng[1] * math.max(0, maxScore - minScore);
   local maxDelta = abIncK * deltaRng[2] * math.max(0, maxScore - minScore);
   local minInit  = math.floor(hiScore * initRng[1] * abInitK);
   local maxInit  = math.floor(hiScore * initRng[2] * abInitK);

   if self._guestStats[id] and self._guestStats[id].updateTStamp and self._guestStats[id].updateTStamp <= App.data:getNextWeeklyResetTime() then

      if self._guestStats[id].weeklyCap == nil then
         local scrCapK = AbTestMgr:getGuestFriendWeeklyScoreCapK() or self._weeklyScoreCapK;

         self._guestStats[id].weeklyCap = math.floor(maxInit * math.mapClamped(0, scrCapK[1], 1, scrCapK[2], math.random()));
      end

      if self._guestStats[id].score < 1 then
         self:_calcGuestScore( id, timeRef, {abIntUptK * timeRng[1], abIntUptK * timeRng[2]}, {minScore, maxScore}, {minInit, maxInit}, false);
      end

      if self._guestStats[id].score > 1 then
         self:_calcGuestScore( id, timeRef, {abCooldownK * timeRng[1], abCooldownK * timeRng[2]}, {minScore, maxScore}, {minDelta, maxDelta}, true);
      end

      local lvl = self._guestStats[id].level + 1;

      for i=lvl,App.data:getMaxPlayerLevel() do
         if i >= App.data:getMaxPlayerLevel() then
            break;
         end

         if self._guestStats[id].score >= App.data:getXPLevelReq(i) then
            self._guestStats[id].level = self._guestStats[id].level + 1;
         else
            break;
         end
      end
   end

end

function GuestMgr:_calcGuestScore( id, timeRef, timeRng, scoreRng, deltaRng, loop )

   if not self._guestStats[id].updateTDelta then
      self._guestStats[id].updateTDelta = math.random();
   end

   local td = math.mapClamped(0, timeRng[1], 1, timeRng[2], self._guestStats[id].updateTDelta);

   while timeRef >= (self._guestStats[id].updateTStamp+td) do

      local sd = math.mapClamped( 0, deltaRng[1], 1, deltaRng[2], math.random() );

      --create a little spread to avoid having all the AI with the same score
      local minRandK = math.lerp(0.90,1.1,math.random());
      local maxRandK = math.lerp(0.90,1.1,math.random());
      local newScore = math.floor( math.clamp(self._guestStats[id].score+sd, scoreRng[1]*minRandK, math.min(scoreRng[2]*maxRandK, self._guestStats[id].weeklyCap)));

      self._guestStats[id].score        = math.max(self._guestStats[id].score,newScore);
      self._guestStats[id].updateTStamp = self._guestStats[id].updateTStamp + td;
      self._guestStats[id].updateTDelta = math.random();

      td = math.mapClamped(0, timeRng[1], 1, timeRng[2], self._guestStats[id].updateTDelta);

      if not loop then
         break;
      end

   end

end

----------------------------------------------------------------------

function GuestMgr:updateGuestStats()

   local timeRef = TimeMgr:getTimeGMT() or TimeMgr:getTimeLocalGMT();
   local list    = {};

   self:_addPlayer(list);


--[[
   print("GuestMgr:updateGuestStats")
   for k,v in pairs(os.date("*t", timeRef)) do
      print(k, v);
   end
   if RankingMgr:getWeeklyResetTime() then
      print("GuestMgr:updateGuestStats - Reset Time")
      for k,v in pairs(os.date("*t", RankingMgr:getWeeklyResetTime())) do
         print(k, v);
      end
   end
   ]]

   if not self._loggedIn then

   elseif GuestMgr:isGuestFriendEnabled() then
      self:_initGuestStats();
      self:_updateGuestScore(timeRef);
      self:_updateGuestLevel(timeRef);
      self:_updateGuestGift(timeRef);

      for k,v in ipairs(self._guestPick) do
         if self._guestList[v] then
            local id = self._guestList[v].accountId;

            self:_rankInsert(list, id, self._guestStats[id].score, self._guestStats[id].level, self._guestStats[id].prevScore, self._guestStats[id].prevScoreTStamp);
         end
      end
   else
      self._guestPick = {};
   end

   self:_saveGuestList();

   return list;

end

----------------------------------------------------------------------
