require('app/server/tango')

local Base = TangoServer;
InboxManager = Base:new();


function InboxManager:new()
   return Base.new(self);
end


function InboxManager:makeSuccessCallback(callObj, callSuccess, result, successMessage)
   return function(result)
      local response;
      if result then
         response = MOAIJsonParser.decode ( result )
      end
      print(successMessage);
      tprint(response);
      self:callCallback(callObj, callSuccess, response);
   end
end


function InboxManager:makeFailureCallback(callObj, callFailure, code, text, failureMessage)
   return function(code, text)
      print('inboxManager failure callback: code:' .. code ..',text:' .. ',msg:'..failureMessage);
      self:callCallback(callObj, callFailure, code, text);
   end
end


function InboxManager:sendGift(callObj, recipients, resourceId, callSuccess, callFailure)

   local requestOptions = {
      requestClass = MOAITangoSendGiftRequest,
      requestData = {resourceId, recipients},
      successFunc = self:makeSuccessCallback(callObj, callSuccess, result, "Send gift succeeded, response contains blocked recipients account id:\n"),
      failureFunc = self:makeFailureCallback(callObj, callFailure, code, text, "Send gift failed:\n"),
   }

   NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

end


function InboxManager:getGifts(callObj, callSuccess, callFailure)

   local requestOptions = {
      requestClass = MOAITangoGetGiftsRequest,
      requestData = {},
      successFunc = self:makeSuccessCallback(callObj, callSuccess, result, "get gifts succeeded, response contains messages:\n"),
      failureFunc = self:makeFailureCallback(callObj, callFailure, code, text, "Get gifts failed:\n"),
   }

   NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

end

function InboxManager:consumeGifts(callObj, giftIds, callSuccess, callFailure)

   local requestOptions = {
      requestClass = MOAITangoConsumeGiftsRequest,
      requestData = {giftIds},
      successFunc = self:makeSuccessCallback(callObj, callSuccess, result, "consume gifts succeeded, response is:\n"),
      failureFunc = self:makeFailureCallback(callObj, callFailure, code, text, "Consume gifts failed:\n"),
   }

   NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

end

function InboxManager:sendInvite(callObj, recipients, resourceId, callSuccess, callFailure)

   local requestOptions = {
      requestClass = MOAITangoSendInviteRequest,
      requestData = {resourceId, recipients},
      successFunc = self:makeSuccessCallback(callObj, callSuccess, result, "send invite succeeded, response is:\n"),
      failureFunc = self:makeFailureCallback(callObj, callFailure, code, text, "send invite failed:\n"),
   }

   NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

end

function InboxManager:getGiftingRecipients(callObj, resourceId, callSuccess, callFailure)

   local requestOptions = {
      requestClass = MOAITangoGetGiftingRecipientsRequest,
      requestData = {resourceId},
      successFunc = self:makeSuccessCallback(callObj, callSuccess, result,  "get gifitng recipients succeeded, response contains friends list:\n"),
      failureFunc = self:makeFailureCallback(callObj, callFailure, code, text, "get gifting recipients failed:\n"),
   }

   NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

end

function InboxManager:getInvitingRecipients(callObj, resourceId, callSuccess, callFailure)

   local requestOptions = {
      requestClass = MOAITangoGetInvitingRecipientsRequest,
      requestData = {resourceId},
      successFunc = self:makeSuccessCallback(callObj, callSuccess, result, "get inviting recipients succeeded, response contains friends list:\n"),
      failureFunc = self:makeFailureCallback(callObj, callFailure, code, text, "get inviting recipients failed:\n"),
   }

   NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

end

function tprint (tbl, indent)
   if not indent then indent = 0 end
   for k, v in pairs(tbl) do
      formatting = string.rep("  ", indent) .. k .. ": "
      if type(v) == "table" then
         print(formatting)
         tprint(v, indent+1)
      elseif type(v) == 'boolean' then
         print(formatting .. tostring(v))
      else
         print(formatting .. v)
      end
   end
end
