----------------------------------------------------------------------

----------------------------------------------------------------------

local HACK_LANGUAGE    = nil;
local DEFAULT_LANGUAGE = "en";

----------------------------------------------------------------------

--                            IOS       Android           Final
----------------------------------------------------------------------
--Arabic                   :  ar         | ar       | ar
--Deutch                   :  de         | de       | de
--Spanish                  :  es         | es       | es
--English                  :  en         | en       | en
--French                   :  fr         | fr       | fr
--Hebrew                   :  he         |          |
--Italian                  :  it         | it       | it
--Japonese                 :  ja         |          |
--Korean                   :  ko         | ko       | ko
--Russian                  :  ru         | ru       | ru
--Thai                     :  th         |          |
--Turkish                  :  tr         | tr       | tr
--Chinese (?)              :             | zh       |
--Chinese (Simplified)     :  zh-Hans    |          | zh_hans
--Chinese (Traditional)    :  zh-Hant    |          | zh_hant
----------------------------------------------------------------------

--Create an easier/shorter function to call rather than LocalizationMgr:getString

function getLocalStr( stringId )

   Debug.assert(LocalizationMgr);

   if LocalizationMgr then
      return LocalizationMgr:getString( stringId );
   end

end

----------------------------------------------------------------------

LocalizationMgr  =
{
   _stringData  = dofile("resources/strings/local-strings.lua");
};

Debug.assert(LocalizationMgr._stringData);

----------------------------------------------------------------------

function LocalizationMgr:init()

   if HACK_LANGUAGE then
      self._language = HACK_LANGUAGE;
   else

       self._language    = MOAIEnvironment.languageCode or DEFAULT_LANGUAGE;
   end

   self._language    = self._language:gsub(  "-", "_" );
   self._language    = self._language:lower();
   if self._language == "zh" then
      self._language = zh_hans;
   end

   Debug.assert( self._language, "LocalizationMgr:init -> self._language" );

   print("Language", self:getLanguage() );

end

----------------------------------------------------------------------

function LocalizationMgr:getString( stringId )

   Debug.assert( self._stringData );

   if self._stringData then

      local lang   = self:getLanguage();

      if lang then
         local first = self._stringData[lang];
         if first then
            local str = first[stringId];
            if str then
               return str;
            end
         end
      else
         Debug.assert( lang, "LocalizationMgr:getString -> lang" );
      end

      local default = self._stringData[DEFAULT_LANGUAGE];
      if default then
         return default[stringId];
      end

      Debug.assert(false, "LocalizationMgr:getString -> Cant find string :"..tostring(stringId));

   end
end

----------------------------------------------------------------------

function LocalizationMgr:getLanguage()

   return self._language;

end


----------------------------------------------------------------------

function LocalizationMgr:getDefaultLanguage()

   return DEFAULT_LANGUAGE;

end


----------------------------------------------------------------------
