require('app/message/server/sender-brag')
require('app/message/server/sender-consumed-gift')
require('app/message/server/sender-gift')
require('app/message/server/sender-invite')
require('app/message/server/retriever-gift')

----------------------------------------------------------------------

local AUTO_MESSAGE_INTERVAL = 60*15;

----------------------------------------------------------------------

MessageMailbox  =
{
   _outbox           = {};
   _outboxSize       = 0,
   _inbox            = {};
   _inboxSize        = 0,
   _retriever        = nil,
   _giftList         = {},
   _consumedList     = nil,
   _lastRetrieveTime = 0;
};

----------------------------------------------------------------------

function MessageMailbox:init()

end

----------------------------------------------------------------------

function MessageMailbox:update()

   if self._autoCheckEnabled then

      local time = os.time()-self._lastRetrieveTime;

      if time > AUTO_MESSAGE_INTERVAL and OnlineMgr:isOnline() then

         self:_retrieveFromServer();

      end
   end
end

----------------------------------------------------------------------

function MessageMailbox:checkNow()

   self:_retrieveFromServer();

end

----------------------------------------------------------------

function MessageMailbox:anyGiftWaiting()

   return self._giftList and #self._giftList > 0;

end

----------------------------------------------------------------

function MessageMailbox:consumeGift()

   if self._giftList and #self._giftList > 0 then

      local totalKey = 0;
      local giftList = self._giftList;

      self._giftList = {};

      local sender = MessageSenderConsumedGift:new(   GIFT_KEY,
                                                      giftList,
                                                      self,
         function()

            for i=1,#giftList do

               local gift     = giftList[i];
               local contact  = ContactMgr:findContact(gift.tangoId);

               --Because there is a bug at the moment creating negative gift,
               -- we will treat it as a special case and give 1 gift in that case
               if gift and gift.amount and gift.amount ~= 0 then
print("MessageMailbox:consumeGift", gift.tangoId, totalKey)
                  --Check for gift collection cap
                  if    not App.data:isWeeklyGiftCollectFull(totalKey)
                     or ( contact and contact:isFriend() )then

                       --In case an error was detected (gift.amount < 0 )
                       -- give the user an extra key... just in case
                     local giftAmount = math.max(gift.amount,1);

                     totalKey		= totalKey + giftAmount;

                     EventMgr:postEvent({ giver=gift.tangoId, keys=giftAmount}, "keys-gifted");
                  end

						setCoolDownCount("Gift2Tracker", gift.tangoId, 1);
               end
            end

            if totalKey and totalKey > 0 then
               App.data:addKey( totalKey, "Gift" );
            end

         end,

         function()
         end
      );

      sender:exec();

   end
end

----------------------------------------------------------------

function MessageMailbox:setEnableAutoCheck( enable )

   self._autoCheckEnabled = enable;

end

----------------------------------------------------------------------

function MessageMailbox:sendBrag( msgTpl, dest, callObj,  successFunc, failureFunc )

   self:_addMessage( MessageSenderBrag:new( msgTpl, dest, callObj,  successFunc, failureFunc ), true );

end

----------------------------------------------------------------------

function MessageMailbox:sendGift(  msgTpl, dest, callObj,  successFunc, failureFunc )

   self:_addMessage( MessageSenderGift:new( msgTpl, dest, callObj, successFunc, failureFunc ), true);

end

----------------------------------------------------------------------

function MessageMailbox:sendInvite( msgTpl, dest, totalReward, callObj, successFunc, failureFunc )

   self:_addMessage( MessageSenderInvite:new( msgTpl, dest, totalReward, callObj,  successFunc, failureFunc ), true);

end

----------------------------------------------------------------------

function MessageMailbox:onMessageDone( message )

   if self._outbox[message] then
      self._outboxSize      = self._outboxSize-1;
      self._outbox[message] = nil;
   else
      Debug.assert(self._outbox[message], "MessageMailbox:onMessageDone -> self._outbox[message]");
   end
end

----------------------------------------------------------------------

function MessageMailbox:isBusy()
   return self._outboxSize > 0 or self.receiver;
end

----------------------------------------------------------------------

function MessageMailbox:_addMessage( msg, execNow )

   if msg then

      self._outbox[msg]    = msg;
      self._outboxSize     = self._outboxSize+1;

      if execNow then
         msg:exec();
      end

   else

      Debug.assert( msg );

   end

end

----------------------------------------------------------------

function MessageMailbox:_retrieveFromServer( )

   if self._retriever == nil then

      self._lastRetrieveTime  = os.time();

      self._retriever = RetrieverGift:new(   self,
                                             function ( self2, newGift )
                                                self2._retriever  = nil;
                                                self2._giftList   = newGift;
                                             end   ,
                                             function(self2)
                                                self2._retriever  = nil;
                                             end   );

      if self._retriever then

         self._retriever:exec();
         return true;

      end

   end

   return false;
end


----------------------------------------------------------------------