
require('app/contact/contact-manager')
require('app/server/tango')

----------------------------------------------------------------------

local Base        = TangoServer;
RetrieverGift     = Base:new();

local SUCCESS     = 1;
local FAILURE     = 2;

----------------------------------------------------------------------

function RetrieverGift:new( callObj, callSuccess, callFailure )

   if ContactMgr:getProfile() then

      local loader            = Base.new(self);

      loader.callObj          = callObj;
      loader.callSuccess      = callSuccess;
      loader.callFailure      = callFailure;
      loader.myAccountId      = ContactMgr:getProfile():getAccountId();
      loader.received         = {};
      loader.receivedStatus   = nil;
      loader.consumed         = {};
      loader.consumedStatus   = nil;
      loader.errorCode        = nil;
      loader.errorMessage     = nil;

      loader.contactList      = {};

      for i=1,ContactMgr:getContactCount() do
         local contact = ContactMgr:getContact(i);
         if contact and contact:isAppInstalled() and (not contact:isGhost()) and (not contact:isGuestFriend()) then
            table.insert( loader.contactList, contact:getAccountId() );
         end
      end

      return loader;

   end
end

----------------------------------------------------------------------

function RetrieverGift:exec()

print("RetrieverGift:exec");

   if LoginMgr:isTango() then

      self:_execReceived( );
      self:_execConsumed();

   else

      -- Use a timer to create be more consistent with the authenticate mode
      self.timer = MOAITimer.new();
      self.timer:setSpan( 0.1 );
      self.timer:setListener( MOAITimer.EVENT_TIMER_END_SPAN,
         function ()
            self.timer:stop( );
            self.timer = nil;

            print("RetrieverGift:exec", "GUEST" );

            self.consumedStatus  = SUCCESS;
            self.receivedStatus  = SUCCESS;

            self:_callCallbackIfDone();

         end
      );
      self.timer:start();

   end

end

----------------------------------------------------------------------

function RetrieverGift:_execReceived()

   local metricId = MessageMgr:createGiftIdTo( GIFT_KEY, self.myAccountId );

   local requestSuccessFunc = function( result )

      for i=1, #self.contactList do

         local r2 = result[self.contactList[i]];

         if r2 then

            if r2[metricId] then
               table.insert( self.received, { tangoId=self.contactList[i], amount=r2[metricId].SUM } );
            end
         end
      end

      self.receivedStatus = SUCCESS;
      self:_callCallbackIfDone();

   end

   local requestFailureFunc = function(code, message)

      self.receivedStatus = FAILURE;
      --Will override the other error code but that's ok, one is enough'
      self.errorCode    = code;
      self.errorMessage = message;

      self:_callCallbackIfDone();

   end

   local metricData = {
      {
         metric = metricId,
         rollups = {
            MOAITangoMetrics.SUM,
            MOAITangoMetrics.SUM_last_modified
         }
      }
   }

   local requestOptions = {
      requestClass = MOAITangoGetComputedMetricsRequest,
      requestData = { self.contactList,
                     metricData},
      successFunc = requestSuccessFunc,
      failureFunc = requestFailureFunc,
   }

   NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

end

----------------------------------------------------------------------

function RetrieverGift:_execConsumed()

   local requestSuccessFunc = function( result )

      local result2 = result[self.myAccountId];

      if result2 then

         for i=1, #self.contactList do

            local contactId  = self.contactList[i];
            local metricId   = MessageMgr:createGiftConsumedId( GIFT_KEY,  contactId );
            if result2[metricId] then
               self.consumed[contactId] = result2[metricId].SUM;
            end
         end
      end

      self.consumedStatus = SUCCESS;
      self:_callCallbackIfDone();

   end

   local requestFailureFunc = function( code, message )

      self.consumedStatus  = FAILURE;
      --Will override the other error code but that's ok, one is enough'
      self.errorCode       = code;
      self.errorMessage    = message;

      self:_callCallbackIfDone();

   end

   local metricList = {};

   for i = 1, #self.contactList do

      local contactId = self.contactList[i];
      table.insert(  metricList,
                     {
                        metric = MessageMgr:createGiftConsumedId(GIFT_KEY, contactId),
                        rollups = {
                           MOAITangoMetrics.SUM
                        }
                     }
      );

   end

   local requestOptions = {
      requestClass = MOAITangoGetComputedMetricsRequest,
      requestData = { { self.myAccountId }, metricList },
      successFunc = requestSuccessFunc,
      failureFunc = requestFailureFunc,
   }

   NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

end

----------------------------------------------------------------------

function RetrieverGift:_callCallbackIfDone()

   if self.receivedStatus  and self.consumedStatus then

      if self.receivedStatus == SUCCESS  and self.consumedStatus == SUCCESS then

         local giftList = {};

         for i=1,#self.received do
            local entry = self.received[i];
            if entry.amount > 0 then
               local amount2     = self.consumed[entry.tangoId] or 0;
               local newKey      = entry.amount-amount2;
               --Negative are okay to "adjust" in case of a bug
               --If we dont adjust a user might not receive key for a long time until the balance get positive again
               if newKey ~= 0 then
                  table.insert(giftList,{tangoId=entry.tangoId, amount=newKey});

                  if newKey < 0 then

                     MOAIFlurry:event( "Error-Negative-Gift",
                                       "total",                   tostring(newKey),
                                       "received",                tostring(entry.amount),
                                       "comsumed",                tostring(amount2)             );

                  end
               end
            end
         end

         self:callCallback( self.callObj, self.callSuccess, giftList, self.received, self.consumed );

      else

         self:callCallback( self.callObj,  self.callFailure, self.errorCode, self.errorMessage  );

      end
   end
end

----------------------------------------------------------------------
