--
require('app/message/server/sender')

----------------------------------------------------------------------

local FAILURE = 0;
local SUCCESS = 1;

----------------------------------------------------------------------

local Base        = MessageSender;
MessageSenderGift = Base:new();

----------------------------------------------------------------------

function  MessageSenderGift:new( msgTpl, dest, callObj, successFunc, failureFunc  )

   local o = Base.new( self, MOAITangoMessage.MESSAGE_BRAG, msgTpl, dest, callObj,  successFunc, failureFunc );

   if msgTpl and dest then

      Debug.assert(msgTpl.gift,                    "MessageSenderGift:new->msgTpl.gift");
      Debug.assert((msgTpl.gift or {}).type,       "MessageSenderGift:(msgTpl.gift or {}).type");
      Debug.assert((msgTpl.gift or {}).amount,     "MessageSenderGift:new->(msgTpl.gift or {}).amount");

      o._giftType    = (msgTpl.gift or {}).type;
      o._giftAmount  = (msgTpl.gift or {}).amount;

   end

   return o;

end

----------------------------------------------------------------------

function MessageSenderGift:exec( )

   EventMgr:postEvent({type = "gift" }, "tango.message.attempt");

   if  self._dest and self._message and self._giftType and self._giftAmount and type(self._dest) == "table" then

      local metricTbl, ghostCount  = self:_buildMetric();

      --If we have any "normal" destination, we will ignore ghost
      if type(metricTbl) == "table" and #metricTbl > 0 then

         --Send the message after the key is sent to avoid case
         -- where the user receive a message but the key was donation failed = bad UX

         self:_sendMetric( metricTbl
                        ,
                           function( result )
                              App.data:deltaValue("gift-count", #metricTbl);
                              Base.exec( self );
                           end
                        ,
                           function()
                              self:_onDone(false)
                           end
                        );
      else

         Debug.assert( type(metricTbl) == "table", 'MessageSenderGift:exec -> type(metricTbl) == table' );
         self:_delayDone( ghostCount > 0, 0.2);

      end

   else

      Debug.assert(   self._dest );
      Debug.assert(   self._message );
      Debug.assert(   self._giftType );
      Debug.assert(   self._giftAmount );
      Debug.assert(  type(self._dest) == "table" );

      self:_delayDone( false, 0.2);

   end

end

----------------------------------------------------------------------

function MessageSenderGift:_onDone( success )

   if success then

      EventMgr:postEvent({
                           type           = "gift",
                           count          = App.data:getValue("gift-count", 0),
                           batch          = self._batchSize or 0,
                           recipientList  = self._dest
                         },
                         "tango.message.success"                              );

   else

      EventMgr:postEvent({
                           type  = "gift"
                         },
                          "tango.message.failure"                              );

   end

   Base._onDone( self, success );

end

----------------------------------------------------------------------

function MessageSenderGift:_sendMetric( metricTbl, onSuccess, onFailure )

   Debug.assert( onSuccess );
   Debug.assert( onFailure );

   local requestOptions = {
      requestClass = MOAITangoSetRawMetricsRequest,
      requestData = { metricTbl },
      successFunc = onSuccess,
      failureFunc = onFailure,
   }

   NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

end

----------------------------------------------------------------------

function MessageSenderGift:_buildMetric()

   local metricTbl   = {};
   local fakeCount	= 0;
   local giftType    = self._giftType;
   local giftAmount  = self._giftAmount;

   for i,dest in ipairs(self._dest) do
      if dest:isGhost() then
         fakeCount = fakeCount + 1;
		elseif dest:isGuestFriend() then
			fakeCount = fakeCount + 1;
			GuestMgr:sendGuestGift(dest.tangoId);
      else
         self:_addMetric( metricTbl, dest, giftType, giftAmount );
      end
   end

   return metricTbl,fakeCount;

end

----------------------------------------------------------------------

function MessageSenderGift:_addMetric( list, dest, giftType, giftAmount )

   if list and dest and giftType and giftAmount then

      table.insert(  list
                  ,
                     {
                        metric   = MessageMgr:createGiftIdTo( giftType, dest:getAccountId() ),
                        value    = giftAmount,
                        rollups  =
                        {
                           MOAITangoMetrics.SUM,
                        }

                     }
                  );

   else

      Debug.assert( list );
      Debug.assert( dest );
      Debug.assert( giftType );
      Debug.assert( giftAmount );

   end
end

----------------------------------------------------------------------
