--
require('app/server/tango')

----------------------------------------------------------------------

local MAX_DEST       = 75;

local Base           = TangoServer;
MessageSender        = Base:new();

----------------------------------------------------------------------

function  MessageSender:new( msgType, msgTpl, dest, callObj, successFunc, failureFunc  )

   local o  = Base.new(self);

   --Class construction will not have any parameters
   if dest and msgType and msgTpl then

      Debug.assert(  msgType == MOAITangoMessage.MESSAGE_INVITE
                  or msgType == MOAITangoMessage.MESSAGE_BRAG
                  or msgType == MOAITangoMessage.MESSAGE_GIFT,       "MessageSender:new -> msgType");

      local title             = (msgTpl or {}).title;
      local body              = (msgTpl or {}).body;
      local action            = (msgTpl or {}).action;
      local url               = (msgTpl or {}).url;
      local thumbnail         = (msgTpl or {}).thumbnail;

      o._done                 = false;
      o._attempt              = 0;
      o._callObj              = callObj;
      o._successFunc          = successFunc;
      o._failureFunc          = failureFunc;

      o._dest                 = self:_ensureTable(dest);
      o._destSplit            = self:_splitTable( o._dest, MAX_DEST );
      o._batchSize            = #o._dest;

      if title and body and action and url  then

         o._message        = MOAITangoMessage.new();
         o._message:setDescriptionText(title);
         o._message:setMessageText(body);
         o._message:setMessageType(msgType);
         o._message:addAction(action, url, MOAITangoMessage.PLATFORM_IOS);
         o._message:addAction(action, url, MOAITangoMessage.PLATFORM_ANDROID);
         o._message:addAction(action, url, MOAITangoMessage.PLATFORM_ANY);

         if thumbnail then
            o._message:setThumbnailUrl(thumbnail);
         end

      else

         Debug.assert(title,  "MessageSender:new -> title");
         Debug.assert(body,   "MessageSender:new -> body");
         Debug.assert(action, "MessageSender:new -> action");
         Debug.assert(url,    "MessageSender:new -> url");
         --Debug.assert(thumbnail,    "MessageSender:new -> thumbnail");   its okay, the server will use the default image

      end
   end

   return o;

end

----------------------------------------------------------------------

function MessageSender:isDone()
   return self._done;
end

----------------------------------------------------------------------

function MessageSender:exec(  )

   self:_exec( 1 );

end

----------------------------------------------------------------------

function MessageSender:_exec( index )

   if self._message and index and self._destSplit and self._destSplit[index] then

      local requestSuccessFunc = function(  )
         self:_onTangoSuccess( index );
      end

      local requestFailureFunc = function( code, message )
         self:_onTangoFailure(index, code, message);
      end

      local requestOptions = {
         requestClass = MOAITangoSendMsgRequest,
         requestData = { self._destSplit[index], self._message },
         successFunc = requestSuccessFunc,
         failureFunc = requestFailureFunc,
      }

      NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

   else

      self:_delayDone(false, 0.2);

   end

end

----------------------------------------------------------------------

function MessageSender:_onTangoSuccess( index )

   if index < #self._destSplit then

      --reset to let us fail more in the next message
      self._attempt  = 0;

      self:_exec( index + 1 );

   else

      self:_onDone(true);

   end

end

----------------------------------------------------------------------

function MessageSender:_onTangoFailure( index, code, message )

   Debug.assert( self._attempt );

   self._attempt = ( self._attempt or 0 ) + 1;

   if self._attempt <= 3 then

      self:_exec( index );

   else

      self:_onDone(false);

   end

end


----------------------------------------------------------------------

function MessageSender:_delayDone( success, time )

   delay( time or 0.2,
         function()
            self:_onDone( success );
         end
         );

end

----------------------------------------------------------------------

function MessageSender:_onDone( success  )

   self._done = true;

   if success then
      self:callCallback(self._callObj, self._successFunc );
   else
      self:callCallback(self._callObj, self._failureFunc );
   end

   MessageMgr:onMessageDone( self );

end

----------------------------------------------------------------------

function MessageSender:_calcSplitItemSize( split  )

   local count = 0;

   if split then

      for i,tbl2 in ipairs(split) do
         count = count + #tbl2;
      end

      return count;

   end

end

----------------------------------------------------------------------

function MessageSender:_ensureTable( src )

   if type(src) == "table"  then
      return src;
   else
      return  { src };
   end

   return


end
----------------------------------------------------------------------

function MessageSender:_splitTable( src, sliceSize )

   if type(src) == "table"  then

      local tbl         = {};
      local sliceCount  = math.ceil(#src/MAX_DEST);
      local start       = 1;

      for i=1,sliceCount do
         tbl[i] = self:_getTableSlice( src, start, sliceSize );
         start  = start + sliceSize;
      end

     -- self:_verifyTableSplit(src, tbl);

      return tbl;

   elseif src ~= nil then

      return  { {src} };

   end

end

----------------------------------------------------------------------

function MessageSender:_getTableSlice( dest, start, size )

   if type(dest) == "table" and start >= 1 and start <= #dest and size then

      local tbl  = {};
      for i = 1, size do
         tbl[i]  = dest[start+i-1];
      end

      return tbl;

   end

end

----------------------------------------------------------------------

function MessageSender:_verifyTableSplit( src, split )

   local map   = {};
   local count = 0;

   for i,tbl2 in ipairs(split) do
      for i2,c in ipairs(tbl2) do
         Debug.assert( map[c] == nil );
         map[c] = c;
         count = count + 1;
      end
   end

   Debug.assert(count, #src);

   for k,v in pairs(src) do
      Debug.assert(map[v] == v);
   end

end

----------------------------------------------------------------------
