
----------------------------------------------------------------------

local CHECK_URL                               = "www.google.com";
local CHECK_ONLINE_TIME_MIN_SUCCESS           = 2*60+0;
local CHECK_ONLINE_TIME_MIN_FAILED_DOWNLOAD   = 2*60+0;
local CHECK_ONLINE_TIME_MIN_FAILED_CONNECTION = 1*60+0;
local CHECK_ONLINE_TIME_MIN_FOREGROUND        = 0*60+0;
local CHECK_ONLINE_QUICK_TIMEOUT              =  7;
local CHECK_ONLINE_TIMEOUT                    = 10;

local STATUS_STRING_ONLINE                    = "Online";
local STATUS_STRING_OFFLINE                   = "Offline";
local STATUS_STRING_UNKNOWN                   = "Unknown";

----------------------------------------------------------------------

OnlineMgr  =
{
};

----------------------------------------------------------------------

function OnlineMgr:init()

   self._online            = nil;
   self._downloadTimeStamp = 0;
   self._checkStatusTime   = 0;

end

----------------------------------------------------------------------

function OnlineMgr:activate( force )

   if force or self._online == nil then
      self._online            = nil;
      self._downloadTimeStamp = MOAISim.getDeviceTime();
      self._checkStatusTime   = 0.15;

   end

end

----------------------------------------------------------------------

function OnlineMgr:deactivate()

   self._online = nil;

end

----------------------------------------------------------------------

function OnlineMgr:onEnterForeground()

   if not self._downloading then

      --If the flag exit on background was set it means
      -- we want to avoid the iOS crash but we couldnt exit (i.e. playing a game)
      -- if we download now, we will crash
      if self._exitOnBackground then
      	self._checkStatusTime    = 10000000;
      else
         self._checkStatusTime   = math.min( self._checkStatusTime, CHECK_ONLINE_TIME_MIN_FOREGROUND );
      end

   end

end


----------------------------------------------------------------------

function OnlineMgr:onEnterBackground()

   if self._exitOnBackground then

      App:exitOnBackground();

   end


end
----------------------------------------------------------------------

function OnlineMgr:update(dt)

   self:_updateOnlineStatus();

end

----------------------------------------------------------------------

function OnlineMgr:isOnline()

   return  self._online == true;

end

----------------------------------------------------------------------

function OnlineMgr:isOffline()

   return self._online == false;

end

----------------------------------------------------------------------

function OnlineMgr:isUnknown()

   return self._online == nil;

end

----------------------------------------------------------------------

function OnlineMgr:isKnown()

   return self._online ~= nil;

end

----------------------------------------------------------------------

function OnlineMgr:isReady()

   return self._online ~= nil;

end

----------------------------------------------------------------------

function OnlineMgr:getStatusString()

   if self._online == true then
      return STATUS_STRING_ONLINE;
   elseif self._online == false then
      return STATUS_STRING_OFFLINE;
   else
      return STATUS_STRING_UNKONWN;
   end
end

----------------------------------------------------------------------

function OnlineMgr:_setOnlineStatus( online )

   if online ~= self._online then

      self._online = online;

      local tbl =
      {
         online = online
      };

      if online then
         tbl.headerDate = self._lastHeaderDate
      end

      EventMgr:postEvent(  tbl , "online-mgr:status-changed");

      if online == true then
         EventMgr:postEvent(  tbl , "online-mgr:online");
      elseif online == false then
         EventMgr:postEvent(  tbl , "online-mgr:offline");
      end
   end

end

----------------------------------------------------------------------

function OnlineMgr:_updateOnlineStatus()

   if self._downloading then

      Debug.assert( self._downloadTimeStamp, " OnlineMgr:_updateOnlineStatus -> self._downloadTimeStamp " );

      --If we are updating for the first time,
      -- fail quicker to avoid user to wait too long
      if ( self._online == nil ) then

         --We waited long enough and have not got our callback
         -- Mark us as failed for now (we might still receive a positive answer in a little bit)
         if (MOAISim.getDeviceTime()-self._downloadTimeStamp) > CHECK_ONLINE_QUICK_TIMEOUT then
            MOAIFlurry:event( "OnlineMgr-Download-Timeout");
            self:_setOnlineStatus(false);
         end

      end
   else

      if (MOAISim.getDeviceTime()-self._downloadTimeStamp) > self._checkStatusTime  then

         self._checkStatusTime   = CHECK_ONLINE_TIME_MIN;
         self._downloadTimeStamp = MOAISim.getDeviceTime();

         MOAIFlurry:eventStart( "OnlineMgr-Download", "Result", "crash" );

         if  ( MOAIEnvironment.connectionType or 0 ) > 0 then

            self._downloading = true;

            local task = MOAIHttpTaskCurl.new ();

            task:setCallback (
               function ( task, code )

                  self._lastHeaderDate = task:getResponseHeader("Date" );
                  if self._lastHeaderDate then
                     MOAIFlurry:eventStop( "OnlineMgr-Download", "Result", "success", "Date", tostring(self._lastHeaderDate), "Time", tostring(math.floor(MOAISim.getDeviceTime()-self._downloadTimeStamp+0.25)) );
                     self:_setOnlineStatus(true);
                     self._checkStatusTime   = CHECK_ONLINE_TIME_MIN_SUCCESS;
                  else
                     MOAIFlurry:eventStop( "OnlineMgr-Download", "Result", "failure-download", "Time", tostring(math.floor(MOAISim.getDeviceTime()-self._downloadTimeStamp))     );
                     self:_setOnlineStatus(false);
                     self._checkStatusTime   = CHECK_ONLINE_TIME_MIN_FAILED_DOWNLOAD;

                     --There is an iOS crash bug that happens if you are trying to use curl twice and it failed
                     --
                     if MOAIEnvironment.osBrand == "iOS" then
                        self._exitOnBackground  = true;
                        self._checkStatusTime   = 60*1000000;
                     end
                  end
                  self._downloading = false;
               end            );

               task:setVerb ( MOAIHttpTask.HTTP_GET )
               task:setUrl ( CHECK_URL )
               task:setUserAgent ( MOAIEnvironment.userAgent or "" )
               task:setTimeout ( CHECK_ONLINE_TIMEOUT )

               task:performAsync ();

         else

            MOAIFlurry:eventStop(   "OnlineMgr-Download",
                                    "Result",               "failure-connection"  );
            self:_setOnlineStatus(false);
            self._checkStatusTime   = CHECK_ONLINE_TIME_MIN_FAILED_CONNECTION;

         end
      end
   end
end


----------------------------------------------------------------------
