require('app/possession/task/possession-task')

----------------------------------------------------------------------

local Base           = PossessionTask;
PossessionTaskUpload = Base:new();

----------------------------------------------------------------------

function PossessionTaskUpload:new( callObj, callSuccess, callFailure, params )

   local task        = Base.new( self, callObj, callSuccess, callFailure, params );

   task.type         = "upload";
    task.uploadToJuneOnly = false;

   --copy the array,that way it can change and it wont affect us
   task._values = {};
   for k,v in pairs(params) do
      --It is possible that in the case like a reset we sent more than we really want
      if type(v) == "number" then
         task._values[k]=v;
      end
   end
   return task;

end

----------------------------------------------------------------------

function PossessionTaskUpload:canMerge( task )

   print("PossessionTaskUpload:canMerge", task and task.type == self.type, task.type);

   return task and task.type == self.type;

end


----------------------------------------------------------------------

function PossessionTaskUpload:merge( task )

   print("PossessionTaskUpload:merge", task and task.type == self.type, task.type);

   if self:canMerge(task) then

      for k,v in pairs(task._values) do

         print("merge-1", k,v, self._values[k]);

         self._values[k] = v;

         print("merge-2", k,v, self._values[k]);

      end

      return true;
   end

   return false;
end

----------------------------------------------------------------------

function PossessionTaskUpload:exec()

   assert( LoginMgr:isTango() or LoginMgr:isJune());

   MOAIFlurry:eventStart("PossessionTaskUpload", "result", "unkown");

   if OnlineMgr:isOnline() then

      local requestSuccessFunc = function(success, possessions)
         if success then
            MOAIFlurry:eventStop("PossessionTaskUpload", "result", "success");
            PossessionMgr:onUploadSuccess( self, possessions );
         else
            MOAIFlurry:eventStop("PossessionTaskUpload", "result", "out-of-date");
            PossessionMgr:onUploadOutOfDate( self );
         end
      end

      local requestFailureFunc = function(success, code, message)
         MOAIFlurry:eventStop("PossessionTaskUpload", "result", "failure");
         PossessionMgr:onUploadFailure( self, code, message );
      end

      local possessions = MOAITangoPossessions.new();

      for k,v in pairs(self._values) do

         local p = PossessionMgr:clone( k );

         if p then
            p:setValue( v );
            possessions:setPossession( k, p );
         else
            possessions:setValue( k, v );
         end
      end

    if LoginMgr:isTango() and not self.uploadToJuneOnly then

      local requestOptions = {
         requestClass = MOAITangoUpdatePossessionsRequest,
         requestData = { possessions },
         successFunc = requestSuccessFunc,
         failureFunc = requestFailureFunc,
      }

      NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));
    end

   self:uploadPossessionsToJune(possessions);

   else

      -- Use a timer to create be more consistent with the authenticate mode
      self.timer = MOAITimer.new();
      self.timer:setSpan( 0.1 );
      self.timer:setListener( MOAITimer.EVENT_TIMER_END_SPAN,
            function ()
               self.timer:stop( );
               self.timer = nil;

               MOAIFlurry:eventStop("PossessionTaskUpload", "result", "offline");
               PossessionMgr:onUploadFailure( self, -1, "You are offline"  );
            end   );
      self.timer:start();

   end
end

----------------------------------------------------------------------

function PossessionTaskUpload:uploadPossessionsToJune(possessions)

    local requestSuccessFunc = function(outOfDate, possessions)
            print("PossessionTaskUpload:uploadPossessionsToJune UploadSUccess. outofdate: "..tostring(outOfDate));
         if outOfDate then
            MOAIFlurry:eventStop("PossessionTaskUpload", "result", "out-of-date");
            PossessionMgr:onUploadOutOfDate( self );
         else
            MOAIFlurry:eventStop("PossessionTaskUpload", "result", "success");
            if App:HasDownloadedFromTango() and not App:IsMigratedToJune()  then
                App:OnMigratedToJune();
            end
            PossessionMgr:onUploadSuccess( self, possessions );
         end
      end

      local requestFailureFunc = function(success, code, message)
         MOAIFlurry:eventStop("PossessionTaskUpload", "result", "failure");
         PossessionMgr:onUploadFailure( self, code, message );
      end

    print("PossessionTaskUpload:createJunePossessionsUploadJson");
    local posTable = { possessions };

    local createJsonRequestClass = MOAIJuneUploadPossessionsHandler;
    local jsonRequest = createJsonRequestClass:new();
    jsonRequest:setListener(MOAIJuneResponseHandler.POSSESSIONS_JSON_CREATED, function(...) self:_onUploadJsonReceived(jsonRequest,function (outOfDate) requestSuccessFunc(outOfDate,possessions); end, function() requestFailureFunc(false,402,"Upload failed"); end, {...}) end);
    jsonRequest:processResponse(unpack(posTable));
end

----------------------------------------------------------------------

function PossessionTaskUpload:_onUploadJsonReceived(request,requestSuccessFunc, requestFailureFunc, jsonStr)
    print("PossessionTaskUpload:_onUploadJsonReceived - ", jsonStr[2]);
    JuneApi:UploadPossessions(jsonStr[2],requestSuccessFunc, requestFailureFunc);
end

----------------------------------------------------------------------

