require('state/guide/guide-manager')
require('engine/utility/functional')
require('engine/utility/tableUtil')

----------------------------------------------------------------------
local HACK_NO_SPACING         = false;
local HACK_FORCE_INTERSTITIAL = false;

if not BUILD_DEBUG then
   HACK_NO_SPACING            = false;
   HACK_FORCE_INTERSTITIAL    = false;
end

----------------------------------------------------------------------

promoPrint = function( ... )
--   print ("PromoMgr", ...);
end

----------------------------------------------------------------------

PromoMgr  = {};

----------------------------------------------------------------------

function PromoMgr:init()

   self.active                   = false;

   self.promoTable               = {};

   self.gamesBetweenPromo        = 6;
   self.timeBetweenPromo         = 60*60;
   self.timeBetweenSaleGuides    = 60*60;

   EventMgr:addListener( self, self.onCloudMasterData,   "cloud:master-data-promo" );
   EventMgr:addListener( self, self.onCloudInfoData,     "cloud:file-data-promo-info" );
   EventMgr:addListener( self, self.onVersionUpdated,    "new-version" );
   EventMgr:addListener( self, self.onGameStart,         "game-start" );

   self.popupCreatedCount = {};

   self.validFunctions = {};
   self.validFunctions["updateGame"]      = self.updateCheck;
   self.validFunctions["tangoLogin"]      = self.tangoLoginCheck;
   self.validFunctions["oldTangoLogin"]   = self.tangoLoginCheck;
   self.validFunctions["doubleCoin"]      = self.doubleCoinCheck;
   self.validFunctions["rateGame"]        = self.rateGameCheck;
   self.validFunctions["giftFriends"]     = self.giftFriendCheck;
   self.validFunctions["holidaySpecial"]  = self.holidaySaleCheck;

   if not App.data:getValue( "promo-games-since-promo") then
      App.data:setValue( "promo-games-since-promo", 1000);
   end

   self.gamesSincePromo = App.data:getValue( "promo-games-since-promo") ;

end

----------------------------------------------------------------------

function PromoMgr:activate()

   if not self.active then
      self.active = true;
   end

end

----------------------------------------------------------------------

function PromoMgr:deactivate()

   self.active = false;

end

----------------------------------------------------------------

function PromoMgr:onCloudMasterData( data )

   local content = (data or {}).content;

   if content then

      self.promoTable = content;
      if content.gamesBetweenPromo then
         self.gamesBetweenPromo = content.gamesBetweenPromo;
      end
      if content.timeBetweenPromo then
         self.timeBetweenPromo = content.timeBetweenPromo;
      end
      if content.timeBetweenSaleGuides then
         self.timeBetweenSaleGuides = content.timeBetweenSaleGuides;
      end

   else

      self.promoTable = self.promoTable or {};

      Debug.error("PromoMgr:onCloudMasterData -> content");

   end
end

----------------------------------------------------------------

function PromoMgr:onCloudInfoData( data )

   local content = (data or {}).content;

   if content then
      self.promoInfoTable = content;
   end

end

----------------------------------------------------------------

function PromoMgr:update( dt )
   Debug.error("NOT IMPLEMENTED");
end

----------------------------------------------------------------

function PromoMgr:attemptPopUpDisplay()

   if self.promoTable and self.promoInfoTable then
	  if self:shouldDisplayPromo() then

		 for _, v in pairs(self.promoTable) do

			if type(v) == "table" then
            local enable = AbTestMgr:isPromoEnabled(v.name);

            if enable == nil then
               enable = v.enable;
            end

			   if enable then
				  if self.promoInfoTable[v.name] and self.promoInfoTable[v.name].info then
					 if self:isValid(self.promoTable[v.name]) then
						self:createPopup(v.name);
						return true;
					 end
				  end
			   end
			end
		 end

	  end
   end

   return false;

end

----------------------------------------------------------------

local function _isViableAdspace(adspace)

   if adspace then

      if not adspace.weight or adspace.weight <= 0 then
         return false;
      end

      if  PromoMgr:_checkConfigPasses(adspace.config, adspace.name) then

         --Make sure there is an ad to be viable
         return AdMgr:isInterstitialAvailable( adspace.name );

      end
   end

   return false;

end

----------------------------------------------------------------

function PromoMgr:_checkConfigPasses(config, dataUID)

   if HACK_FORCE_INTERSTITIAL then
      return true;
   end

   --No config means enabled
   if config == nil then
      return true;
   end

   Debug.assert( type(config) == "table", "PromoMgr:_checkConfigPasse->  type(config) == table" );

   -- Is it enabled?
   if config.enabled == false then
	  print("Config failed enabled check.");
	  return false;
   end

   -- Determine if only shown for Tango users or non-Tango users and check the player
   local tango = config.tango;

   if tango ~= nil and tango ~= LoginMgr:isTango() then
	  print("Config failed Tango check. (Required: " .. tostring(tango) .. ", isTango: " .. tostring(LoginMgr:isTango()) .. ")");
	  return false;
   end

   -- If we're a payer, should we see this interstitial?
   if App.data:isPayer() then
	  local enabledForPayer = config.payer;

	  print("Interstitial - enabledForPayer: " .. tostring(enabledForPayer));

	  -- nil is an acceptable value
	  if enabledForPayer == false then
		 print("Config failed payer check.");
		 return false;
	  end
   end

   -- Ensure games in between interstitials
   local gameInterval = config.games;
   local sessionGames = config.sessionGames;
   gameInterval = gameInterval or 0;
   sessionGames = sessionGames or 0;

   local gID               = "interstitial_games_"..dataUID;
   local gamesThisSession  = App.data:getValue("session.games") or 0;
   local gamesTotal        = App.data:getValue(gID) or 0;

   if not gameInterval or not sessionGames or gamesTotal < gameInterval or gamesThisSession < sessionGames then
	  print("Config failed game interval criteria. (Only " .. tostring(gamesTotal) .. " out of " .. tostring(gameInterval) .. " games played)");
	  return false;
   end

   -- Ensure the player is high enough level
   local playerLevel   = App.data:getCurrentPlayerLevel();
   local requiredLevel = config.level;
   requiredLevel = requiredLevel or 0;

   if not requiredLevel or not playerLevel or playerLevel < requiredLevel then
	  print("Config failed player level criteria. (Required level " .. tostring(requiredLevel) .. ")");
	  return false;
   end

   -- Ensure enough time between interstitials
   local iTime			= config.time;
   iTime             = iTime or 0;

   local time			= TimeMgr:getTimeGMT() or 0;
   local tID			= "interstitial_timestamp_"..dataUID;
   local timestamp		= App.data:getValue(tID);
   local timeSinceSeen  = -1;

   if timestamp then
	  timeSinceSeen = time - timestamp;
   end

   if timeSinceSeen ~= -1 and (not iTime or timeSinceSeen < iTime) then
	  print("Config failed time interval criteria. (Required time interval: " .. tostring(timeSinceSeen) .. "/" .. tostring(iTime) .. ")");
	  return false;
   end

   return true;

end

----------------------------------------------------------------

function PromoMgr:markInterstitialAsSeen(dataUID)

   App.data:setValue("interstitial_games_" .. dataUID, 0);
   App.data:setValue("interstitial_timestamp_" .. dataUID, TimeMgr:getTimeGMT() or 0);

end

----------------------------------------------------------------

function PromoMgr:attemptInterstitialDisplay(location)

   Debug.assert(location, "PromoMgr:attemptInterstitialDisplay -> location");

   if GuideMgr:isBusy() or not self:canDisplayAnyPromo() then
	  -- Interstitial cannot be displayed at this moment
      print("Can't display interstitial because guide is busy or promos are otherwise unavailable.",  not GuideMgr:isBusy(), self:canDisplayAnyPromo() );
	  return;
   end

   local data = self:getInterstitialData();

   -- Ensure all the data is here
   if not data then
	  Debug.error("Missing promo table or interstitial table!");
	  return;
   end

   local globalConfig = data.config;

   -- Does the global configuration pass?
   if not self:_checkConfigPasses(globalConfig, "global") then
      print("Interstitial attempt failed due to global config.");
      return;
   end
   print("Interstitial - global config PASSED");

   if  not data.locations then
      Debug.error("data.locations for location:" .. tostring(location) );
      return;
   end

   local interstitial = data.locations[location];
   if not interstitial then
	  return;
   end

   local locationConfig = interstitial.config;

   -- Does the location configuration pass?
   if not self:_checkConfigPasses(locationConfig, location) then
	  print("Interstitial attempt failed due to location config.");
	  return;
   end
   print("Interstitial - interstitial config PASSED");

   -- Collect adspace candidates (i.e. those with passing configs)
   local viableAdspaces = filter(_isViableAdspace, interstitial.adspaces);

   -- Pick an adspace from candidiates
   local weights = map(function(x) return x.weight; end, viableAdspaces);
   local adspace = pickWeightedRandom(viableAdspaces, weights)

   if adspace then
	  print("Attempt to show interstitial of adspace: " .. tostring(adspace.name));

	  -- Display ad
	  if AdMgr:showInterstitial(adspace.name) then

       print("Success to show interstitial of adspace: " .. tostring(adspace.name) );

		 self:markInterstitialAsSeen("global");
		 self:markInterstitialAsSeen(location);
		 self:markInterstitialAsSeen(adspace.name);
		 self:markInterstitialViewing();
	  end
   end

end

----------------------------------------------------------------

function PromoMgr:tangoLoginCheck()

   local gpTgt = AbTestMgr:getValue("auto_authentication") or -100;

   return     ( not App.data:hasReceiveTangoBonus()   )
          and ( not LoginMgr:isTango()                )
          and (     MOAITangoSdk:tangoHasSdkSupport())
          and (     gpTgt <= -100 );

end

----------------------------------------------------------------

function PromoMgr:doubleCoinCheck()
   return App.data:canPickupBeUpgraded( PICKUP_COIN_INDEX );
end

----------------------------------------------------------------

function PromoMgr:rateGameCheck()

   local savedVersion = App.data:getValue( "promo-rated-version" );
   local curVersion = App:versionStringToNumber( App.info.version );

   return savedVersion and curVersion and savedVersion ~= curVersion;

end

----------------------------------------------------------------

function PromoMgr:giftFriendCheck()

   --Make sure I have some friends
   return ContactMgr:getFriendsCount() > 0;

end

----------------------------------------------------------------

function PromoMgr:holidaySaleCheck()

   --Make sure the holiday sale is running
   local sale = PromoMgr:getCurrentSales("holiday");
   if sale then
      return true;
   end
   return false;

end


----------------------------------------------------------------

function PromoMgr:onVersionUpdated()

   local scheduleTable = App.data:getValue( "promo-schedule-table" );

   local scheduleChange = false;

   if scheduleTable then

      for _, v in ipairs( scheduleTable ) do
         if v.name == "rateGame" then
            v.tIndex = 1;
            v.gIndex = 1;
            v.time  = self:_GetTime();
            v.games = 0;
            scheduleChange = true;
         end
         if v.name == "updateGame" then
            v.tIndex = 1;
            v.gIndex = 1;
            v.time = self:_GetTime();
            v.games = 0;
            scheduleChange = true;
         end
      end

   end

   if scheduleChange then
      App.data:setValue( "promo-schedule-table", scheduleTable );
      promoPrint("promo version schedule changed and saved!");
   end

end

----------------------------------------------------------------

function PromoMgr:sdkCheck()

   return LoginMgr:isTango();


end

----------------------------------------------------------------

function PromoMgr:upToDateCheck()

   local promoInfo = CloudMgr:isFileUpToDate( "promo-info" );
   local promoMaster = CloudMgr:isMasterUpToDate();

   if promoMaster and promoInfo then
      return true;
   end

   return false;

end

----------------------------------------------------------------

function PromoMgr:onlineCheck()

   if OnlineMgr:isOnline() then
      return true;
   end

   return false;

end

----------------------------------------------------------------

function PromoMgr:mandatoryUpdateCheck()

   if UpdateMgr:shouldUpdate() then
      if UpdateMgr:isUpdateMandatory() then

         MOAIDialogIOS.showDialog(	LocalizationMgr:getString( "NATIVE_UPDATE_TITLE" )
                           ,	LocalizationMgr:getString( "NATIVE_UPDATE_MSG" )
                           ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                           ,  nil
                           ,  nil
                           ,  false
                           ,  function()
                                 App:gotoUpdatePage();
                              end
                        );

         return true;
      end
   end

   return false;

end

----------------------------------------------------------------

function PromoMgr:updateCheck()

   return UpdateMgr:shouldUpdate();
end

----------------------------------------------------------------

function PromoMgr:isValid( promoInfo )

   if not promoInfo then
	  return false;
   end

   -- Check custom logic
   if self.validFunctions[promoInfo.name] then
	  if not self.validFunctions[promoInfo.name]() then
		 promoPrint("promo failed custom logic:", promoInfo.name);
		 return false;
	  end
   end

   -- Check schedule logic
   if not self:scheduleCheck( promoInfo ) then
	  promoPrint("promo failed schedule:", promoInfo.name);
	  return false;
   end

   -- Check for SDK
   local tangoReq = AbTestMgr:getPromoTango(promoInfo.name);

   if tangoReq == nil then
      tangoReq = promoInfo.tango;
   end

   if tangoReq ~= nil then
	  if tangoReq ~= self:sdkCheck() then
		 promoPrint("promo failed SDK check:", promoInfo.name);
		 return false;
	  end
   end

   -- Check up to date
   local upToDate = AbTestMgr:getPromoUpToDate(promoInfo.name);

   if upToDate == nil then
      upToDate = promoInfo.upToDate;
   end

   if upToDate ~= nil then
	  if upToDate ~= self:upToDateCheck() then
		 promoPrint("promo failed up to date check:", promoInfo.name);
		 return false;
	  end
   end

   -- Check online
   local onlineReq = AbTestMgr:getPromoOnline(promoInfo.name);

   if onlineReq == nil then
      onlineReq = promoInfo.online;
   end

   if onlineReq ~= nil then
	  if onlineReq ~= self:onlineCheck() then
		 promoPrint("promo failed online check:", promoInfo.name);
		 return false;
	  end
   end

   -- Check levels
   local minLevel = AbTestMgr:getPromoMinLevel(promoInfo.name) or promoInfo.minLevel;
   local maxLevel = AbTestMgr:getPromoMaxLevel(promoInfo.name) or promoInfo.maxLevel;

   if minLevel then
	  if minLevel > App.data:getCurrentPlayerLevel() then
		 promoPrint("promo failed min level check:", promoInfo.name);
		 return false;
	  end
   end
   if maxLevel then
	  if maxLevel <= App.data:getCurrentPlayerLevel() then
		 promoPrint("promo failed max level check:", promoInfo.name);
		 return false;
	  end
   end

   -- Check repeat
   if promoInfo.promoSuccessLimit then

	  local name = "promo-succeed-".. promoInfo.name;
	  local numS = App.data:getValue( name );
	  if numS then
		 if numS >= promoInfo.promoSuccessLimit then
			promoPrint("promo failed repeat check:", promoInfo.name );
			return false;
		 end
	  end
   end

   if promoInfo.sessionLimit and self.popupCreatedCount and self.popupCreatedCount[promoInfo.name] then

	  Debug.assert( self.popupCreatedCount, "PromoMgr:isValid -> self.popupCreatedCount");

	  if self.popupCreatedCount[promoInfo.name] >= promoInfo.sessionLimit then
		 promoPrint("promo failed session limit:", self.popupCreatedCount[promo],promoInfo.sessionLimit  );
		 return false;
	  end

   end

   -- Check odds
   local odds = AbTestMgr:getPromoOdds(promoInfo.name) or promoInfo.odds;

   if odds then
	  local rand = math.random();
	  if odds <= 0 or rand > odds then
		 promoPrint("promo failed random check:", promoInfo.name,rand, odds );
		 return false;
	  end
   end

   return true;

end

----------------------------------------------------------------

function PromoMgr:shouldDisplayPromo()
   return not self:mandatoryUpdateCheck()
      and not GuideMgr:isBusy()
      and self:canDisplayAnyPromo();
end

----------------------------------------------------------------

function PromoMgr:canDisplayAnyPromo()

   if HACK_NO_SPACING then
      return true;
   end

   local time = TimeMgr:getTimeGMT();
   local timestamp = App.data:getValue("video_wait_timestamp");

   if (not self.promoTable) or (not self.promoTable.videoWaitTime) or (not self.promoTable.videoWaitGames) then
      promoPrint("Missing fields");
      return false;
   end

   if time and timestamp then
      if (time - timestamp) <= self.promoTable.videoWaitTime then
         promoPrint("time since a video reward", (time - timestamp), self.promoTable.videoWaitTime);
         return false;
      end
   end


   local games = App.data:getValue("video_wait_games");

   if games and games < self.promoTable.videoWaitGames then
      promoPrint("games since a video reward", games, self.promoTable.videoWaitGames);
      return false;
   end


   local gSpacing = false;
   local tSpacing = false;

   if self.gamesSincePromo then
      promoPrint("games since, games between:", self.gamesSincePromo, self.gamesBetweenPromo);
      if self.gamesSincePromo >= self.gamesBetweenPromo then
         gSpacing = true;
      end
   end

   if self.lastPromoTime then
      if (self:_GetTime() - self.lastPromoTime) >= self.timeBetweenPromo then
         tSpacing = true;
      end
   else
      local lpt = App.data:getValue( "promo-last-promo-time-2" );
      if lpt then
         self.lastPromoTime = lpt;
         if (self:_GetTime() - self.lastPromoTime) >= self.timeBetweenPromo then
            tSpacing = true;
         end
      else
         --joe: return true, so this doesn't fail them out before seeing a promo
         tSpacing = true;
      end
   end

   if gSpacing and tSpacing then
      return true;
   end

   if not gSpacing and not tSpacing  then
      promoPrint("promo failed spacing check, games since, last seen time:", self.gamesSincePromo, self.lastPromoTime, gSpacing, tSpacing);
   elseif not gSpacing then
      promoPrint("promo failed spacing check, GAMES since", self.gamesSincePromo);
   else --not tSpacing
      promoPrint("promo failed spacing check, last seen TIME:", self.timeBetweenPromo, (self:_GetTime() - self.lastPromoTime), self:_GetTime(),  self.lastPromoTime);
   end

   return false;

end

----------------------------------------------------------------

function PromoMgr:scheduleCheck( promoInfo )

   local masterSched = AbTestMgr:getPromoSchedule(promoInfo.name) or promoInfo.schedule;

   if masterSched then

      local scheduleTable = App.data:getValue( "promo-schedule-table" );

      if not scheduleTable then
         promoPrint("creating schedule table");
         scheduleTable = {};
         App.data:setValue( "promo-schedule-table", scheduleTable );
      end

      --Michel, why not using a map ?
      local schedule = nil;
      for i,v in ipairs( scheduleTable ) do
         if v.name == promoInfo.name then
            promoPrint("matching name inside schedule table:", v.name);
            schedule = v;
            break;
         end
      end

      if not schedule then
         promoPrint("creating schedule info");

         schedule =  {
                        name  = promoInfo.name,
                        tIndex = 1,
                        gIndex = 1,
                        time  = self:_GetTime(),
                        games = 0,
                     };
         table.insert( scheduleTable, schedule );
         App.data:setValue( "promo-schedule-table", scheduleTable );
      end

      local timerTrue = true;
      local gamesTrue = true;

      if masterSched.time then

      	 --There was a bug where schedule.time was set in the future
         if schedule.time > self:_GetTime() then
            schedule.time = self:_GetTime();
            App.data:setValue( "promo-schedule-table", scheduleTable );
         end

         schedule.tIndex = math.min( schedule.tIndex, #masterSched.time);
         timerTrue = (self:_GetTime() - schedule.time) >= masterSched.time[schedule.tIndex];

         if not timerTrue then
            promoPrint("not timerTrue ", (self:_GetTime() - schedule.time), masterSched.time[schedule.tIndex]);
         end

      end

      if masterSched.games then

         schedule.gIndex = math.min(schedule.gIndex, #masterSched.games);
         gamesTrue = schedule.games >= masterSched.games[schedule.gIndex];

         if not gamesTrue then
            promoPrint("not gamesTrue ", schedule.games, masterSched.games[schedule.gIndex]);
         end

      end

      return timerTrue and gamesTrue;

   end

end

----------------------------------------------------------------

function PromoMgr:gamePlayed()

   --if we dont save every game it will reset every time we kill the app
   self.gamesSincePromo = App.data:deltaValue( "promo-games-since-promo", 1 );

   local scheduleTable = App.data:getValue( "promo-schedule-table" );
   if scheduleTable then
      for i,v in ipairs( scheduleTable ) do
         v.games = v.games + 1;
      end
      App.data:setValue( "promo-schedule-table", scheduleTable );
   end
end

----------------------------------------------------------------

function PromoMgr:promoSeen( pName, promo )

   if promo then

      self.popupCreatedCount[pName] = (self.popupCreatedCount[pName] or 0) + 1;

	  self:markInterstitialViewing()

      local scheduleTable = App.data:getValue( "promo-schedule-table" );
      if scheduleTable then
         for _, v in ipairs(scheduleTable) do
            if v.name == promo.name then
               v.time   = self:_GetTime();
               v.games  = 0;
               v.tIndex = v.tIndex + 1;
               v.gIndex = v.gIndex + 1;
            end
         end
         App.data:setValue( "promo-schedule-table", scheduleTable );
      end

   end

end

----------------------------------------------------------------

function PromoMgr:markInterstitialViewing()

   self.gamesSincePromo = 0;
   App.data:setValue( "promo-games-since-promo", 0 );

   self.lastPromoTime = self:_GetTime();
   App.data:setValue( "promo-last-promo-time-2", self.lastPromoTime );

end

----------------------------------------------------------------

function PromoMgr:createPopup( pName )

   Debug.assert(pName, "Missing promo name!");

   Debug.assert( self.promoInfoTable and self.promoInfoTable[pName],
				 "PromoMgr:createPopup -> self.promoInfoTable and self.promoInfoTable[pName] : " .. pName );

   if self.promoInfoTable and self.promoInfoTable[pName] then

	  local masterTable = self:copyTable( self.promoTable[pName] );
	  local infoTable   = self:copyTable( self.promoInfoTable[pName].info );

      Debug.assert( masterTable,
                    "PromoMgr:createPopup -> masterTable : " .. tostring(pName) );
      Debug.assert( infoTable and infoTable.popupTemplate,
                    "PromoMgr:createPopup -> infoTable and infoTable.popupTemplate : " .. tostring(pName) );

	  if masterTable and infoTable and infoTable.popupTemplate then

		 MOAIFlurry:event("PromoMgr:createPopup", "promo", pName);

		 self:promoSeen( pName, infoTable );
		 StateMgr:push( infoTable.popupTemplate, masterTable, infoTable );

	  end

   end

end

----------------------------------------------------------------

function PromoMgr:getInterstitialData()

   return AbTestMgr:choose((self.promoTable or {})["interstitial"],AbTestMgr:getInterstitialExtra());

end

----------------------------------------------------------------

function PromoMgr:getInterstitialLocations()

   local result = {};

   local data   = self:getInterstitialData();

   if data and data.locations then

      for k, _ in pairs(data.locations) do
         table.insert(result, k);
      end
   end

   return result;

end

----------------------------------------------------------------

function PromoMgr:getAdspaceNames()

   local result = {};
   local data   = self:getInterstitialData();

   if data then

      local locations = data.locations;
      if locations then
         for _, location in pairs(locations) do

            if location.adspaces then
               for _, adspace in pairs(location.adspaces) do
                  table.insert(result, adspace.name);
               end
            end
         end
      end
   end

   return result;

end

----------------------------------------------------------------

function PromoMgr:onGameStart()

   local globalInterGames = App.data:deltaValue("interstitial_games_global", 1);

   local iLocations = self:getInterstitialLocations();
   for _, v in ipairs(iLocations) do
	  local id   = "interstitial_games_" .. v;
	  App.data:deltaValue(id, 1);
   end

   local adspaceNames = self:getAdspaceNames();
   for _, v in ipairs(adspaceNames) do
	  local id = "interstitial_games_" .. v;
	  App.data:deltaValue(id, 1);
   end

   local wGames = App.data:getValue("guide_wait_games");
   if wGames and wGames >= 0 then
      App.data:deltaValue("guide_wait_games", 1);
   end

   local vGames = App.data:getValue("video_wait_games");
   if vGames and vGames >= 0 then
      App.data:deltaValue("video_wait_games", 1);
   end

end

----------------------------------------------------------------

function PromoMgr:getCurrentSales( saleType )

   if saleType and self.promoTable and self.promoTable["sales"] then

      local salesData = self:copyTable( self.promoTable["sales"]  );

      if salesData[saleType] then

         local curTime = self:_GetTime();
         if curTime and curTime > 0 then

            local r = {};
            for i,v in ipairs( salesData[saleType] ) do

               local sTime = TimeMgr:convertWebDateToTimeGMT( v.from );
               local eTime = TimeMgr:convertWebDateToTimeGMT( v.to );

               if sTime and eTime and sTime <= eTime then

                  if curTime >= sTime and curTime <= eTime then
							local tbl	= self:copyTable(v);

                     tbl.from		= sTime;
                     tbl.to		= eTime;
                     table.insert(r, tbl);
                  end
               else
                  Debug.assert("PromoMgr:getCurrentSales -> start date is later than end date");
               end
            end
            if #r > 0 then
               return r
            end
         end
      end

   end

end

----------------------------------------------------------------

function PromoMgr:getFutureSales( saleType, days )

   if saleType and self.promoTable and self.promoTable["sales"] then

      local salesData = self:copyTable( self.promoTable["sales"]  );

      if salesData[saleType] then

         local curTime = self:_GetTime();
         if curTime and curTime > 0 and days then

            local addedTime = curTime + (days*24*60*60);

            local r = {};
            for _, v in ipairs( salesData[saleType] ) do

               local valid = true;
               if saleType == "vehicle" then
                  valid = false;
                  for _, carIndex in ipairs(v.vehicles) do
                     if ( not App.data:isPlayerVehicleUnlocked(carIndex)) and (not App.data:isPlayerVehicleLevelLocked(carIndex)) then
                        valid = true;
                     end
                  end
               end

               if valid then

                  local sTime = TimeMgr:convertWebDateToTimeGMT( v.from );
                  local eTime = TimeMgr:convertWebDateToTimeGMT( v.to );

                  if sTime and eTime and sTime <= eTime then

                     if (curTime >= sTime or addedTime >= sTime) and (curTime <= eTime or addedTime <= eTime)  then

                        local tbl	= self:copyTable(v);

                        tbl.from		= sTime;
                        tbl.to		= eTime;
                        table.insert(r, tbl);
                     end
                  else
                     Debug.assert("PromoMgr:getFutureSales -> start date is later than end date");
                  end
               end
            end

            if #r > 0 then
               return r
            end
         end

      end

   end

end

----------------------------------------------------------------

function PromoMgr:copyTable( t )

   if t then
     local u = { }
     for k, v in pairs(t) do u[k] = v end
     return setmetatable(u, getmetatable(t))
   end

end

----------------------------------------------------------------

function PromoMgr:_GetTime()

   -- Debug.assert( TimeMgr:getTimeGMT(), "PromoMgr:_GetTime ->  TimeMgr:getTimeGMT() " );

   return TimeMgr:getTimeGMT() or 0;

end

----------------------------------------------------------------
