require('app/ranking/server/level-uploader')
require('app/ranking/server/ranking-retriever')
require('app/ranking/server/score-uploader')
require('app/ranking/server/game-center-score-controller')

----------------------------------------------------------------------

local RANK_SAVE_FILENAME_LUA = "ROAD_RIOT_SAVE_RANK_3.lua";
local RANK_SAVE_FILENAME_JSON = "58r8a12n55k13s990a7v334e.jp5";

local RANK_TO_UPLOAD_SAVE_FILENAME_LUA = "ROAD_RIOT_SAVE_RANK_TO_UPLOAD_2.lua";
local RANK_TO_UPLOAD_SAVE_FILENAME_JSON = "8u224p08l013o23480a461d82r23a28n23k4.jp5";

local RANK_GHOST_SCORE_MAP_SAVE_FILENAME_LUA = "ROAD_RIOT_GHOST_RANK.lua";
local RANK_GHOST_SCORE_MAP_SAVE_FILENAME_JSON = "842s801p820o422o23k238y042r2102a23n84204k13s5.jp5";

local RANK_LAST_WEEK_FILENAME_LUA = "ROAD_RIOT_SAVE_RANK_LAST_WEEK.lua";
local RANK_LAST_WEEK_FILENAME_JSON = "134la1s5t23w1e23e41k23ra8481n845k.jp5";

local RANK_WEEKLY_RESET_TIME_FILENAME_LUA = "ROAD_RIOT_WEEKLY_RESET_TIME.lua";
local RANK_WEEKLY_RESET_TIME_FILENAME_JSON = "42re222s7e0t12t82i23m56e.jp5";

local RANK_WEEKLY_RESET_ID_FILENAME_LUA = "ROAD_RIOT_WEEKLY_RESET_ID.lua";
local RANK_WEEKLY_RESET_ID_FILENAME_JSON = "r18e55s57e5t82i23d.jp5";

local AUTO_RANK_INTERVAL_SUCCESS = 60*2;
local AUTO_RANK_INTERVAL_RETRY   = 60*1;
local HACK_AUTO_RANK_INTERVAL_SUCCESS_SHORT_TIME = 60*1;

local ACTIVE_FALSE = 0;
local ACTIVE_WIP   = 1;
local ACTIVE_TRUE  = 2;

local HACK_FORCE_RESET = false;

--Do not change here but above
if not BUILD_DEBUG then
   HACK_QUICK_WEEKLY_RESET      = false;
   HACK_FORCE_RESET             = false;
   HACK_QUICK_WEEKLY_RESET_TIME = nil;
end

----------------------------------------------------------------------

RankingMgr  =
{
   _active        = ACTIVE_FALSE,
};

----------------------------------------------------------------------

function RankingMgr:init()

   self:_reset();
end

----------------------------------------------------------------------

function RankingMgr:activate()

   if not self:isActive() then
      self:_reset();
      self:_setupGameCenter();
      self._active = ACTIVE_WIP;
   end

end

----------------------------------------------------------------------

function RankingMgr:deactivate()

   EventMgr:rmvListener( self );

   self._active = ACTIVE_FALSE;
end

----------------------------------------------------------------------

function RankingMgr:isActive()
   return self._active == ACTIVE_TRUE;
end

----------------------------------------------------------------------

function RankingMgr:isActivating()
   return self._active == ACTIVE_WIP;
end

----------------------------------------------------------------

function RankingMgr:update( dt )

   if self:isActivating() then

      self:_updateActivating();
   end

   if self:isActive() then

      self:_updateDownload(dt);
      self:_updateUpload(dt);

      self:_updateGameCenter(dt);
      self:_updateWeeklyResetTime(dt);
   end

end

----------------------------------------------------------------

function RankingMgr:areScoreUpToDate()

   --Guest are up to date
   return   self._rankingStatus == STATUS_GUEST
         or self._rankingStatus == STATUS_TANGO_DOWNLOADED ;
end

----------------------------------------------------------------

function RankingMgr:areScoreUpdating()

   if       self._rankingStatus == STATUS_TANGO_LOCAL
         or self._rankingStatus == STATUS_TANGO_NIL      then

         return self:isDownloading();

   end

end

----------------------------------------------------------------

function RankingMgr:getTimeSinceLastSuccessDownload()

   return MOAISim.getDeviceTime() - (self._rankingLastDownload or 0 );
end

----------------------------------------------------------------

function RankingMgr:getTimeToReset()

   local time = self:_getReferenceTime();

   if self._weeklyResetTime2 and time then
      return self._weeklyResetTime2 - time;
   else
      return 60*60*24*7;
   end

end

----------------------------------------------------------------

function RankingMgr:getWeeklyResetTime()
   return self._weeklyResetTime2;
end

----------------------------------------------------------------

function RankingMgr:getPlayerHiScore()

   if       self._ranking
      and   self._playerRank
      and   self._playerRank > 0
      and   self._playerRank <= #self._ranking then

      return self._ranking[self._playerRank].score;
   end

   return 0;
end

----------------------------------------------------------------

function RankingMgr:getPlayerPrevScore()

   if       self._ranking
      and   self._playerRank
      and   self._playerRank > 0
      and   self._playerRank <= #self._ranking then

      return self._ranking[self._playerRank].prevScore or 0;
   end

   return 0;
end

----------------------------------------------------------------

function RankingMgr:getPlayer()

   if not Debug.assertType(self._ranking, "table", "RankingMgr:getPlayer -> self._ranking") then
      return nil;
   end

   if not Debug.assertType(self._playerRank, "number", "RankingMgr:getPlayer -> self._playerRank") then
      return nil;
   end

   return self:getRankEntry(self:getPlayerRank());
end

----------------------------------------------------------------

function RankingMgr:getPlayerRank()

   if self._playerRank then
      return self._playerRank;
   elseif self._ranking then
      return #self._ranking + 1;
   else
      return 1;
   end

end

----------------------------------------------------------------

function RankingMgr:isDownloading()
   return self._rankingDownloader ~= nil;
end

----------------------------------------------------------------

function RankingMgr:isReady()
   return self._ranking ~= nil;
end

----------------------------------------------------------------

function RankingMgr:isLoaded()
   return self._ranking ~= nil;
end

----------------------------------------------------------------

function RankingMgr:isUploading()
   return      self._scoreUploader ~= nil
         or    self._levelUploader ~= nil;
end

----------------------------------------------------------------

function RankingMgr:hasScoreToUpload()
   return self._scoreUploader ~= nil;
end

----------------------------------------------------------------

function RankingMgr:hasLevelToUpload()
   return self._levelUploader ~= nil;
end

----------------------------------------------------------------

function RankingMgr:getRankEntry(index)

   if not Debug.assertType(self._ranking, "table", "RankingMgr:getRankEntry -> self._ranking") then
      return nil;
   end

   if not Debug.assertType(index, "number", "RankingMgr:getRankEntry -> index") then
      return nil;
   end

   return self._ranking[index];
end

----------------------------------------------------------------

function RankingMgr:getRankIndex( tangoId )

   if not Debug.assertType(self._ranking, "table", "RankingMgr:getRankIndex -> self._ranking") then
      return nil;
   end

   for i = 1, #self._ranking do

      if self._ranking[i].tangoId and self._ranking[i].tangoId == tangoId then
         return i;
      end
   end

end

----------------------------------------------------------------

function RankingMgr:getRankingList()

   if not Debug.assertType(self._ranking, "table", "RankingMgr:getRankingList -> self._ranking") then
      return nil;
   end

   return self._ranking;
end

----------------------------------------------------------------

function RankingMgr:getLastWeekRankEntry(index)

   if not Debug.assertType(self._lastWeekRanking, "table", "RankingMgr:getLastWeekRankEntry -> self._lastWeekRanking") then
      return nil;
   end

   return self._lastWeekRanking[index];
end

----------------------------------------------------------------

function RankingMgr:getRankEntryCount()

   if not Debug.assertType(self._ranking, "table", "RankingMgr:getRankEntryCount -> self._ranking") then
      return 0;
   end

   return #self._ranking;
end

----------------------------------------------------------------

function RankingMgr:getLastWeekRankEntryCount()

   if not Debug.assertType(self._lastWeekRanking, "table", "RankingMgr:getLastWeekRankEntryCount -> self._lastWeekRanking") then
      return 0;
   end

   return #self._lastWeekRanking;
end

----------------------------------------------------------------

function RankingMgr:isBusy()

   return       self:isDownloading()
            or  self:isUploading();
end

----------------------------------------------------------------

function RankingMgr:submitScore( score )

   score = math.floor(score or 0);

   if score > 0 then

      if score > App.data:getValue("alltime-high-score",0) then
         App.data:setValue("alltime-high-score", score);
      end

      if score > App.data:getValue("level-high-score",0) then
         App.data:setValue("level-high-score", score);
      end

      --no need to upload any score if we are a guest
      if LoginMgr:isTango() then
         --Check to make sure we are not waiting to upload an even higher score
         if score > (self._scoreToUpload or 0) then
            self._scoreToUpload = score;
            self:_saveToUpload();
         end
      else
         self._guestScore = score;
      end

      if MOAIGameCenterIOS then
         self:_submitGameCenterScore(score);
      end

      -- This will make sure our hiscore is updated right away
      self:_addPlayerScoreToRank( score );
      self:_saveRank();

   end
end

----------------------------------------------------------------

function RankingMgr:submitLevel( level )

   level = math.floor(level or 0);

   if level > 0 then

      --no need to upload any score if we are a guest
      if LoginMgr:isTango() then
         --Check to make sure we are not waiting to upload an even higher score
         if level > (self._levelToUpload or 0) then
            self._levelToUpload = level;
            self:_saveToUpload();
         end
      end

   end
end

----------------------------------------------------------------

function RankingMgr:submitLeagueRanking( leagueIndex, divisionIndex )

   Debug.assertType(leagueIndex, "number", "RankingMgr:submitLeagueRanking -> leagueIndex");
   Debug.assertType(divisionIndex, "number", "RankingMgr:submitLeagueRanking -> divisionIndex");

   if not LeagueMgr:isValidLeagueDiv(leagueIndex, divisionIndex) then
      Debug.error("RankingMgr:submitLeagueRanking failed: league/div pair invalid! " .. tostring(leagueIndex) .. ", " .. tostring(divisionIndex));
      return;
   end

   -- TODO

end

----------------------------------------------------------------

function RankingMgr:submitGhostScore( tangoId, score, level )

   score = math.floor( score or 0 );

   self._ghostEntryMap = self._ghostEntryMap or {};

   if      score  > ((self._ghostEntryMap[tangoId] or {}).score or 0)
      or ( level and  level ~= ((self._ghostEntryMap[tangoId] or {}).level or 1) )    then

      self:_submitGhostScore( tangoId, score, level );

   end

end

----------------------------------------------------------------

function RankingMgr:getReferenceTime()

   return self:_getReferenceTime();
end

----------------------------------------------------------------

function RankingMgr:getGhostEntries()

   if not Debug.assertType(self._ranking, "table", "RankingMgr:getGhostEntries -> self._ranking") then
      return nil;
   end

   local list  = {};
   local count = 1;

   for i = 1, #self._ranking do

      if self._ranking[i].contact:isGhost() then

         list[count] = self._ranking[i];
         count       = count + 1;
      end
   end

   return list;
end

----------------------------------------------------------------

function RankingMgr:_saveWeeklyResetTime( time )

   if not Debug.assertType(time, "number", "RankingMgr:_saveWeeklyResetTime -> time") then
      return;
   end

   local tbl = {};

   self._weeklyResetTime2 = self:_getResetTime(time);
   tbl.time2 = self._weeklyResetTime2;

   saveFileMultiFormat(RANK_WEEKLY_RESET_TIME_FILENAME_LUA, RANK_WEEKLY_RESET_TIME_FILENAME_JSON, tbl);
end

----------------------------------------------------------------

function RankingMgr:_loadWeeklyResetTime()

   local tbl = loadFileMultiFormat(RANK_WEEKLY_RESET_TIME_FILENAME_LUA, RANK_WEEKLY_RESET_TIME_FILENAME_JSON) or {};

   if tbl and tbl.time2 then
      self._weeklyResetTime2 = tbl.time2;
   else
      local time = self:_getReferenceTime();

      if time then
         self:_saveWeeklyResetTime(time);
      end
   end

end

----------------------------------------------------------------

function RankingMgr:_updateWeeklyResetTime()

   local time = self:_getReferenceTime();

   if self._weeklyResetTime2 == nil and time then
      self:_loadWeeklyResetTime();
   end

end

----------------------------------------------------------------

function RankingMgr:_getResetTime( time )

   local resetTimeDelta = self:_getResetTimeDelta(time);

   return time + resetTimeDelta;
end

----------------------------------------------------------------

function RankingMgr:_getReferenceTime()

   local time = TimeMgr:getTimeGMT();

   return time;
end

----------------------------------------------------------------

function RankingMgr:_getResetTimeDelta( time )

   local date      = os.date("*t", time);
   local deltadate = os.date("*t", time+((8 - date.wday) * 24 * 60 * 60));
   local delta     = os.time(deltadate) - ( (deltadate.wday - 1) * 24 * 60 * 60);
   local timeZone  = self:_getTimeZoneOffset();

   deltadate = os.date("*t", delta);
   deltadate.sec = 0;
   deltadate.min = 0;
   deltadate.hour = 24;

   if HACK_QUICK_WEEKLY_RESET  and HACK_QUICK_WEEKLY_RESET_TIME then

      deltadate = os.date("*t", time+HACK_QUICK_WEEKLY_RESET_TIME);
      deltadate.sec = 0;
      if HACK_QUICK_WEEKLY_RESET_TIME >= 60*60 then
         deltadate.min = 0;
      end

      return os.time(deltadate) - time;
   end

   return os.time(deltadate) - timeZone - time;
end

----------------------------------------------------------------

function RankingMgr:_getTimeZoneOffset()

   -- CA is utc-08 (-8 hours)
   local utc_08 = -60*60*8;

   return utc_08 - TimeMgr:calcLocalDeltaGMT();
end

----------------------------------------------------------------

function RankingMgr:_submitGameCenterScore( score )

   if not Debug.assertType(score, "number", "RankingMgr:_submitGameCenterScore -> score") then
      return;
   end

   if not Debug.assertType(self._gameCenterScoreController, "table", "RankingMgr:_submitGameCenterScore -> self._gameCenterScoreController") then
      return;
   end

   self._gameCenterScoreController:submitScore( score );
end

----------------------------------------------------------------

function RankingMgr:_saveGhost()

   if not Debug.assertType(self._ghostEntryMap, "table", "RankingMgr:_saveGhost -> self._ghostEntryMap") then
      return;
   end

   saveFileMultiFormat(RANK_GHOST_SCORE_MAP_SAVE_FILENAME_LUA, RANK_GHOST_SCORE_MAP_SAVE_FILENAME_JSON, self._ghostEntryMap);
end

----------------------------------------------------------------

function RankingMgr:_loadGhost()

   self._ghostEntryMap = loadFileMultiFormat(RANK_GHOST_SCORE_MAP_SAVE_FILENAME_LUA, RANK_GHOST_SCORE_MAP_SAVE_FILENAME_JSON) or {};
end

----------------------------------------------------------------

function RankingMgr:_resubmitAllGhostScores()

   if not Debug.assertType(self._ghostEntryMap, "table", "RankingMgr:_resubmitAllGhostScores -> self._ghostEntryMap") then
      return;
   end

   for _, v in pairs(self._ghostEntryMap) do

      self:_submitGhostScore(v.tangoId, v.score, v.level);
   end

end

----------------------------------------------------------------

function RankingMgr:_submitAllGhostPrevScores()

   if not Debug.assertType(self._ghostEntryMap, "table", "RankingMgr:_submitAllGhostPrevScores -> self._ghostEntryMap") then
      return;
   end

   for _, v in pairs(self._ghostEntryMap) do

      self:_submitGhostScore(v.tangoId, v.score, v.level, true);
   end

end

----------------------------------------------------------------

function RankingMgr:_submitGhostScore( tangoId, score, level, updatePrevScore )

   if not Debug.assertType(tangoId, {"number", "string"}, "RankingMgr:_submitGhostScore -> tangoId") then
      return;
   end

   if not Debug.assertType(score, "number", "RankingMgr:_submitGhostScore -> score") then
      return;
   end

   local rank      = self:_findUserRank( self._ranking, tangoId );
   local prevScore = 0;

   if rank and rank > 0 and rank <= #self._ranking then
      if updatePrevScore then
         prevScore = self._ranking[rank].score;
      else
         prevScore = self._ranking[rank].prevScore or prevScore;
      end
      table.remove(self._ranking, rank);
   end

   local newRank = self:_findScoreRank(self._ranking, score);

   if newRank then

      local e2 = self:_createRankEntry(tangoId, score, level, prevScore);

      if e2 then
         table.insert(self._ranking, newRank, e2 );
      end

      self._ghostEntryMap[tangoId] = e2;
      self:_saveGhost();

   end

   if ContactMgr and ContactMgr:getProfile() then
      self._playerRank = self:_findUserRank( self._ranking, self:_getPlayerId() );
   end

end

----------------------------------------------------------------

function RankingMgr:isGhostMapValid()

   if not Debug.assertType(self._ghostEntryMap, "table", "RankingMgr:isGhostMapValid -> self._ghostEntryMap") then
      return false;
   end

   local count = 0;

   for _, v in pairs(self._ghostEntryMap) do
      if v.tangoId or v.score or v.level then
         count = count + 1;
      end
   end

   return count >= 3;
end

----------------------------------------------------------------

function RankingMgr:onEnterForeground()

   if ContactMgr:areContactLoaded() and self._rankingDownloader == nil then

      --Dont retrieve right away in case we detect we are offline
      self._rankingDownloadTimer = 2;
   end

end

----------------------------------------------------------------

function RankingMgr:forceUpdate()

   self:onEnterForeground();
end

----------------------------------------------------------------

function RankingMgr:loadGuests()

   if     ContactMgr:isProfileLoaded()
      and ContactMgr:areContactLoaded() then

      self:_downloadRank();
   end

end

----------------------------------------------------------------

function RankingMgr:updateWeeklyReset()

   if LoginMgr:isTango() then
      self:_updateTangoWeeklyReset();
   else
      self:_updateGuestWeeklyReset();
   end

end

----------------------------------------------------------------

function RankingMgr:canCycleTangoLinkedScore( time )

   local weeklyPossessionResetTime = App.data:getNextWeeklyResetTime();

   if time < weeklyPossessionResetTime then
      return true;
   end

   return false;

end

----------------------------------------------------------------

function RankingMgr:clearLastWeekRanking()

   self._lastWeekRanking = {};

   self:_saveLastWeekRanking();
   self:_downloadRank();

end

----------------------------------------------------------------

function RankingMgr:isLastWeekRankingValid()

   if     self._lastWeekRanking
      and App.data:getCurrentPlayerLevel() > 1
      and #self._lastWeekRanking > 1
      and self._playerlastWeekRank
      and self._lastWeekRanking[self._playerlastWeekRank]
      and self._lastWeekRanking[self._playerlastWeekRank].score then

      if     #self._lastWeekRanking < 4
         and self._lastWeekRanking[self._playerlastWeekRank].score <= 0 then
         return false;
      end

      return true;
   end

   return false;

end

----------------------------------------------------------------

function RankingMgr:getGhostLastWeekRanks()

   if not Debug.assertType(self._lastWeekRanking, "table", "RankingMgr:getGhostLastWeekRanks -> self._lastWeekRanking") then
      return;
   end

   local list = {};

   for i = 1, #self._lastWeekRanking do

      if self._lastWeekRanking[i].contact:isGhost() then
         list[self._lastWeekRanking[i].tangoId] = i;
      end
   end

end

----------------------------------------------------------------

function RankingMgr:getLastWeekRanking()

   return self._lastWeekRanking;
end

----------------------------------------------------------------

function RankingMgr:getPlayerLastWeekRank()

   if self._playerlastWeekRank then
      return self._playerlastWeekRank;
   elseif self._lastWeekRanking then
      return #self._lastWeekRanking + 1;
   else
      return 1;
   end

end

----------------------------------------------------------------

function RankingMgr:getPlayerLastWeekScore()

   if     self._lastWeekRanking
      and self._playerlastWeekRank
   and self._lastWeekRanking[self._playerlastWeekRank].score then

      return self._lastWeekRanking[self._playerlastWeekRank].score;
   end

   return 0;

end

----------------------------------------------------------------

function RankingMgr:resetScore()

   --this is only for non-tango because tango metric auto resets the player score

   if not LoginMgr:isTango() then
      self._ranking[self._playerRank].prevScore = self._ranking[self._playerRank].score;
      self._ranking[self._playerRank].score     = 0;

      self:submitScore(self._ranking[self._playerRank].score);
   end

end

----------------------------------------------------------------

function RankingMgr:cycleScore()

   if LoginMgr:isTango() then
      self:_cycleTangoScore();
   else
      self:_cycleGuestScore();
   end

end

----------------------------------------------------------------

function RankingMgr:getLocalPlayerId()

   if not Debug.assertType(ContactMgr, {"table", "userdata"}, "RankingMgr:getLocalPlayerId -> ContactMgr") then
      return;
   end

   if not Debug.assertType(ContactMgr:getProfile(), {"table", "userdata"}, "RankingMgr:getLocalPlayerId -> ContactMgr:getProfile()") then
      return;
   end

   if not Debug.assertType(ContactMgr:getProfile():getAccountId(), {"string", "number"}, "RankingMgr:getLocalPlayerId -> ContactMgr:getProfile():getAccountId()") then
      return;
   end

   return ContactMgr:getProfile():getAccountId();
end

----------------------------------------------------------------

function RankingMgr:removeAllGuestFriends( )

   if not Debug.assertType(self._ranking, "table", "RankingMgr:removeAllGuestFriends -> self._ranking") then
      return;
   end

   for i = #self._ranking, 1, -1 do
      if self._ranking[i].contact:isGuestFriend() then
         table.remove(self._ranking, i);
      end
   end

end

----------------------------------------------------------------

-- Counts number of "real" (i.e. non-guest) entries
function RankingMgr:getRankEntryNonGuestCount( leagueOn )

   local DEFAULT_NONGUEST_COUNT = 1;

   if not Debug.assertType(self._ranking, "table", "RankingMgr:getRankEntryNonGuestCount -> self._ranking") then
      return DEFAULT_NONGUEST_COUNT;
   end

   local count = 0;

   if leagueOn then

      local rank = self:getPlayerRank();

      for k, v in ipairs(self._ranking) do

         if not v.contact:isGuestFriend() then

            if v.contact:isGhost() then
               --ignore ghosts, they are only on friends lists
            elseif k == rank then
               count = count + 1;
--TODO MQL: for the initial league test we only want all the league to be guests
-- for full release, we need to add tango picked players
--[[
            else
               -- TODO: need to filter out for a match list
               local leagueIndex, divIndex = LeagueMgr:calcPlacement(v.prevScore);

               if LeagueMgr:doesMatchPlayer( leagueIndex, divIndex ) then
                  count = count + 1;
               end
]]
            end
         end
      end

   else
      count = #self._ranking;
   end

   return math.max(count, DEFAULT_NONGUEST_COUNT);

end

----------------------------------------------------------------

function RankingMgr:getRankingLeagueList( includeBounty )

   if not Debug.assertType(self._ranking, "table", "RankingMgr:getRankingLeagueList -> self._ranking") then
      return;
   end

   if not LeagueMgr:isLeagueEnabled() then
      Debug.error("RankingMgr:getRankingLeagueList is only valid when leagues are enabled.");
      return;
   end

   local tbl   = {};
   local pRank = self:getPlayerRank();
   local lRank = pRank;
   local count = 0;

   for i, v in ipairs(self._ranking) do

      if i == pRank then
         table.insert(tbl, v);
         count = count + 1;
         lRank = count;
      elseif v.contact == nil then
         Debug.assert(v.contact, "RankingMgr:getRankingLeagueList -> nil contact self._ranking["..tostring(i).."]")
      elseif v.contact:isFriend()  then

         -- MQL: Add in after tango online support
         --[[
            local l = v.contact:getLeagueIndex();
            local d = v.contact:getDivisionIndex();

            if LeagueMgr:doesMatchPlayer(l, d) then
               table.insert(tbl, v);
               count = count + 1;
            end
         ]]

      elseif v.contact:isLeague() then
         table.insert(tbl, v);
         count = count + 1;
      elseif includeBounty and BountyMgr:isBountyEntryValid(v.tangoId) then
         table.insert(tbl, v);
         count = count + 1;
      end
   end

   return tbl, lRank;

end

----------------------------------------------------------------

function RankingMgr:getRankingFriendsList()

   if not Debug.assertType(self._ranking, "table", "RankingMgr:getRankingFriendsList -> self._ranking") then
      return;
   end

   if not LeagueMgr:isLeagueEnabled() then
      Debug.error("RankingMgr:getRankingFriendsList is only valid when leagues are enabled.");
      return;
   end

   local tbl   = {};
   local pRank = self:getPlayerRank();
   local fRank = pRank;
   local count = 0;

   for k, v in ipairs(self._ranking) do

      if k == pRank then
         table.insert(tbl, v);
         count = count + 1;
         fRank = count;
      elseif v.contact == nil then
         Debug.assert(v.contact, "RankingMgr:getRankingFriendsList -> nil contact self._ranking["..tostring(k).."]")
      elseif         ( v.contact:isFriend() or v.contact:isGhost() )
               and   App.data:isFriendsUnlocked() then

         table.insert(tbl, v);
         count = count + 1;
      end
   end

   return tbl, fRank;

end

----------------------------------------------------------------

function RankingMgr:getRankingLeagueListLastWeek()

   if not Debug.assertType(self._lastWeekRanking, "table", "RankingMgr:getRankingLeagueListLastWeek -> self._lastWeekRanking") then
      return;
   end

   if not LeagueMgr:isLeagueEnabled() then
      Debug.error("RankingMgr:getRankingLeagueListLastWeek is only valid when leagues are enabled.");
      return;
   end

   local tbl   = {};
   local pRank = self:getPlayerLastWeekRank();
   local lRank = pRank;
   local count = 0;

   for k, v in ipairs(self._lastWeekRanking) do

      if k == pRank then
         table.insert(tbl, v);
         count = count + 1;
         lRank = count;
      elseif v.contact == nil then
         Debug.assert(v.contact, "RankingMgr:getRankingLeagueListLastWeek -> nil contact self._lastWeekRanking["..tostring(k).."]")
      elseif v.contact:isFriend() then

         -- MQL: Add in after tango online support
         --[[
            local l = v.contact:getLeagueIndex();
            local d = v.contact:getDivisionIndex();

            if LeagueMgr:doesMatchPlayer(l, d) then
               table.insert(tbl, v);
               count = count + 1;
            end
         ]]

      elseif v.contact:isLeague() then
         table.insert(tbl, v);
         count = count + 1;
      end
   end

   return tbl, lRank;

end

----------------------------------------------------------------

function RankingMgr:getRankingFriendsListLastWeek()

   if not Debug.assertType(self._lastWeekRanking, "table", "RankingMgr:getRankingFriendsListLastWeek -> self._lastWeekRanking") then
      return;
   end

   if not LeagueMgr:isLeagueEnabled() then
      Debug.error("RankingMgr:getRankingFriendsListLastWeek is only valid when leagues are enabled.");
      return;
   end

   local tbl   = {};
   local pRank = self:getPlayerLastWeekRank();
   local fRank = pRank;
   local count = 0;

   for k, v in ipairs(self._lastWeekRanking) do

      if k == pRank then
         table.insert(tbl, v);
         count = count + 1;
         fRank = count;
      elseif v.contact == nil then
         Debug.assert(v.contact, "RankingMgr:getRankingFriendsListLastWeek -> nil contact self._lastWeekRanking["..tostring(k).."]")
      elseif v.contact:isFriend() or v.contact:isGhost() then
         table.insert(tbl, v);
         count = count + 1;
      end
   end

   return tbl, fRank;

end

----------------------------------------------------------------------

function RankingMgr:_reset()

   if HACK_QUICK_WEEKLY_RESET and HACK_QUICK_WEEKLY_RESET_TIME <= 60*60 then

      AUTO_RANK_INTERVAL_SUCCESS = HACK_AUTO_RANK_INTERVAL_SUCCESS_SHORT_TIME;
   end

   self._active               = ACTIVE_FALSE;

   self._ranking              = {};
   self._rankingStatus        = STATUS_UNKNOWN;
   self._rankingDownloadTimer = nil;
   self._rankingDownloader    = nil;
   self._rankingLastDownload  = 0;

   self._playerRank           = nil;

   self._scoreToUpload        = nil;
   self._scoreUploadTimer     = nil;
   self._scoreUploader        = nil;

   self._ghostEntryMap        = {};

   self._levelToUpload        = nil;
   self._levelUploadTimer     = nil;
   self._levelUploader        = nil;

   self._leagueToUpload       = nil;
   self._divisionToUpload     = nil;

   self._weeklyResetTime      = nil; -- unused?
   self._weeklyResetTime2     = nil;
   self._weeklyResetID        = nil;

   self._lastWeekRanking      = {};
   self._playerlastWeekRank   = nil;

end

----------------------------------------------------------------

function RankingMgr:_setupGameCenter()

   if MOAIGameCenterIOS then
      self._gameCenterScoreController = GameCenterScoreController:new();
   end

end

----------------------------------------------------------------

function RankingMgr:_updateActivating()

   --We are depending on the contact MGR
   if ContactMgr:isReady() then

      EventMgr:rmvListener( self );
      EventMgr:addListener( self, self._onContactUpdated,   "tango-friends-updated" );
      EventMgr:addListener( self, self._onContactUpdated,   "guest-friends-updated" );

      --"tango-friends-updated" should have been already trigger and we just register= we probably missed the event
      self:_onContactUpdated();

      self:_loadWeeklyResetTime();

      self:_loadGhost();

      self:_loadLastWeekRanking();

      self:_activateRank();

      self:_activateScoreToUpload();

      self:_loadWeeklyResetID();

      self._active = ACTIVE_TRUE;

      --this is to handle the case where we transit from guest to tango user
      if LoginMgr:isTango() and self._guestScore then
         self:submitScore( self._guestScore );
         self._guestScore = nil;
      end

   end

end

----------------------------------------------------------------

function RankingMgr:_activateRank(  )

   if LoginMgr:isTango() then

      local rank = self:_loadRank();

      if rank then
         self:_onRankUpdated( rank, STATUS_TANGO_LOCAL );
      else
         self:_onRankUpdated( {}, STATUS_TANGO_NIL );
      end

   else

      local rank = self:_loadRank() or {};

      self:_onRankUpdated( rank , STATUS_GUEST );

   end

end

----------------------------------------------------------------

function RankingMgr:_updateDownload(dt)

   -- Make sure we are not updating already
   if self._friendsDownloader == nil then

      if self._rankingDownloadTimer then

         self._rankingDownloadTimer = self._rankingDownloadTimer - dt;

         if       self._rankingDownloadTimer < 0
            and   ContactMgr:isProfileLoaded()
            and   ContactMgr:areContactLoaded()  then

            self:_downloadRank();

         end
      end
   end

end

----------------------------------------------------------------

function RankingMgr:_onContactUpdated()

   if ContactMgr:areContactLoaded() and self._rankingDownloader == nil then

      self._rankingDownloadTimer = 0;
   end

end

----------------------------------------------------------------

function RankingMgr:_createRankEntry( tangoId, score, level, prevScore, prevScoreStamp )

   local c = ContactMgr:findContact(tangoId);

   if c or string.find(tangoId, "debugTangoId") then

      local e2          = {};
      e2.tangoId        = tangoId;
      e2.contact        = c;
      e2.score          = score or 0;
      e2.level          = level or 1;
      e2.prevScore      = prevScore or 0;
      e2.prevScoreStamp = prevScoreStamp;

      if not c and string.find(tangoId, "debugTangoId") then
         e2.contact              = {};
         e2.contact.isGhost      = function() return false end;
         e2.contact.fake         = true;
         e2.contact.getFirstName = function() return "Fake" end;
         e2.contact.getLastName  = function() return "Name" end;
      end

      return e2;
   end

end

----------------------------------------------------------------

function RankingMgr:_onRankUpdated( rank, status )

   Debug.assert( ContactMgr:isReady(), "RankingMgr:_onRankUpdated -> ContactMgr:isReady()" );

   self._rankingStatus     = status;
   self._ranking           = nil;
   self._playerRank        = nil;

   if rank and ContactMgr:isReady() then

      local contactMap = {};

      self._ranking = {};

      for i,e in ipairs(rank) do

         if contactMap[e.tangoId] == nil then

            local e2 = self:_createRankEntry(e.tangoId, e.score, e.level, e.prevScore, e.prevScoreStamp);

            if e2 then
               table.insert(self._ranking, e2);
            end

            contactMap[e.tangoId] = e;

         end

      end

      -- If we have a score to upload, it will not be part of
      -- the ranking.. otherwise, add 0 to make sure the player have a score
      self:_addPlayerScoreToRank( self._scoreToUpload or 0 );

      self:_resubmitAllGhostScores();

      self:_saveRank();

   end

   if self._playerRank then
      EventMgr:postEvent( {playerRank=self._playerRank}, "player-rank");
   end

end

----------------------------------------------------------------

function RankingMgr:_saveRank()

   if not Debug.assertType(self._ranking, "table", "RankingMgr:_saveRank -> self._ranking") then
      return;
   end

   local tbl = {};

   for _, e in ipairs(self._ranking) do

      table.insert(tbl,
                   {
                      tangoId   = e.tangoId,
                      score     = e.score,
                      level     = e.level,
                      prevScore = e.prevScore,
                   }
      );
   end

   saveFileMultiFormat(RANK_SAVE_FILENAME_LUA, RANK_SAVE_FILENAME_JSON, tbl);

end

----------------------------------------------------------------

function RankingMgr:_loadRank()

   local tbl = loadFileMultiFormat(RANK_SAVE_FILENAME_LUA, RANK_SAVE_FILENAME_JSON);

   if not tbl then
      return nil;
   end

   local rank = {};

   for _, e in ipairs(tbl) do

      table.insert(rank,
                   {
                      tangoId   = e.tangoId,
                      score     = e.score,
                      level     = e.level,
                      prevScore = e.prevScore,
                   }
      );

   end

   return rank;
end

----------------------------------------------------------------

function RankingMgr:_downloadRank(  )

   if       self._rankingDownloader == nil
      and   LoginMgr:isTango()
      and   OnlineMgr:isOnline()
      and   ContactMgr:isReady()    then

      self._rankingDownloader = RankingRetriever:new();

      self._rankingDownloader:exec(
         self
         ,
         function( self, rank )

            Debug.assert( ContactMgr:isReady(), "RankingMgr:_downloadRank -> ContactMgr:isReady()" );

            --We had a bug where contact were not ready by the time we did this
            -- it was cause by authenticated on the first time which we hope we fixed but
            -- it is better being safe than sorry
            if ContactMgr:isReady() then

               self:_onRankUpdated( rank, STATUS_TANGO_DOWNLOADED);

               self._rankingDownloadTimer    = AUTO_RANK_INTERVAL_SUCCESS;

               self._rankingLastDownload     = MOAISim.getDeviceTime();

               self._rankingDownloader       = nil;

            else

               self._rankingDownloader       = nil;
               self._rankingDownloadTimer    = AUTO_RANK_INTERVAL_RETRY;

            end

         end
         ,
         function ()

            self._rankingDownloader       = nil;
            self._rankingDownloadTimer    = AUTO_RANK_INTERVAL_RETRY;

         end
      );

   elseif self._rankingDownloader == nil
      and OnlineMgr:isOnline()
      and not LoginMgr:isTango()
      and ContactMgr:isReady() then

      self._rankingDownloader       = nil;
      self._rankingDownloadTimer    = AUTO_RANK_INTERVAL_SUCCESS;
      self._rankingLastDownload     = MOAISim.getDeviceTime();

      self:_onRankUpdated( GuestMgr:updateGuestStats(), STATUS_GUEST);

   end

end

----------------------------------------------------------------

function RankingMgr:_findUserRank( ranking, tangoId )

   if not Debug.assertType(ranking, "table", "RankingMgr:_findUserRank -> ranking") then
      return;
   end

   if not Debug.assertType(tangoId, {"string", "number"}, "RankingMgr:_findUserRank -> tangoId") then
      return;
   end

   for i, e in ipairs(ranking) do

      if e.tangoId == tangoId then
         return i;
      end
   end

end

----------------------------------------------------------------

function RankingMgr:_findScoreRank( ranking, score )

   if not Debug.assertType(ranking, "table", "RankingMgr:_findScoreRank -> ranking") then
      return 1;
   end

   for i, e in ipairs(ranking) do

      if score >= e.score  then
         return i;
      end
   end

   return #ranking + 1;

end

----------------------------------------------------------------

function RankingMgr:_findPrevScoreRank( ranking, prevScore )

   if not Debug.assertType(ranking, "table", "RankingMgr:_findPrevScoreRank -> ranking") then
      return 1;
   end

   for i, e in ipairs(ranking) do

      if prevScore >= e.prevScore then
         return i;
      end
   end

   return #ranking + 1;
end

----------------------------------------------------------------

function RankingMgr:_activateScoreToUpload()

   if LoginMgr:isTango() then

      local tbl = self:_loadToUpload();

      self._scoreToUpload = tbl.score;

      if self._scoreToUpload then
         self._scoreUploadTimer = 10;
      end

      self._levelToUpload = tbl.level;

      if self._levelToUpload then
         self._levelUploadTimer = 10;
      end
   end

end

----------------------------------------------------------------

function RankingMgr:_updateUpload( dt )

   -- Make sure we are not updating already
   if self._scoreUploader == nil  then

      if self._scoreToUpload then
         self._scoreUploadTimer = self._scoreUploadTimer or 0;
      end

      if self._scoreUploadTimer then

         self._scoreUploadTimer = self._scoreUploadTimer - dt;

         if       self._scoreUploadTimer <= 0
            and   self._scoreToUpload   ~= nil
            and   ContactMgr:isProfileLoaded()
            and   ContactMgr:areContactLoaded()  then

            self:_uploadScore();

         end
      end
   end

   -- Make sure we are not updating already
   if self._levelUploader == nil then

      if self._levelToUpload then
         self._levelUploadTimer = self._levelUploadTimer or 0;
      end

      if self._levelUploadTimer then
         self._levelUploadTimer = self._levelUploadTimer - dt;

         if       self._levelUploadTimer <= 0
            and   self._levelToUpload   ~= nil
            and   ContactMgr:isProfileLoaded()
            and   ContactMgr:areContactLoaded()  then

            self:_uploadLevel();

         end
      end
   end

   if self._leagueUploader == nil then

      if self._leagueToUpload and self._divisionToUpload then
         self._leagueUploadTimer = self._leagueUploadTimer or 0;
      end

      if self._leagueUploadTimer then

         self._leagueUploadTimer = self._leagueUploadTimer - dt;

         if     self._leagueUploadTimer <= 0
            and self._leagueToUpload
            and self._divisionToUpload
            and ContactMgr:isProfileLoaded()
            and ContactMgr:areContactLoaded() then

               self:_uploadLeagueDiv();
         end
      end

   end

end

----------------------------------------------------------------

function RankingMgr:_updateGameCenter( dt )

   if self._gameCenterScoreController then
      self._gameCenterScoreController:update(dt);
   end

end

----------------------------------------------------------------

function RankingMgr:_saveToUpload()

   local tbl = {};

   tbl.score = self._scoreToUpload;
   tbl.level = self._levelToUpload;

   saveFileMultiFormat(RANK_TO_UPLOAD_SAVE_FILENAME_LUA, RANK_TO_UPLOAD_SAVE_FILENAME_JSON, tbl);
end

----------------------------------------------------------------

function RankingMgr:_loadToUpload()

   return loadFileMultiFormat( RANK_TO_UPLOAD_SAVE_FILENAME_LUA, RANK_TO_UPLOAD_SAVE_FILENAME_JSON ) or {};
end

----------------------------------------------------------------

function RankingMgr:_uploadScore()

   if       self._scoreUploader == nil
      and   self._scoreToUpload   ~= nil
      and   LoginMgr:isTango()
      and   OnlineMgr:isOnline()
      and   ContactMgr:isReady()    then

      local backup         = self._scoreToUpload;
      self._scoreUploader  = ScoreUploader:new( self._scoreToUpload );

       self._scoreUploader:exec(
            self
         ,
            function ( self )
               --If the score we just uploaded successfully was the same
               -- or at even better
               if math.floor(backup) >= math.floor(self._scoreToUpload or 0) then
                  self._scoreToUpload      = nil;
                  self._scoreUploadTimer   = nil;
               else
                  self._scoreUploadTimer   = 0
               end

               self:_saveToUpload();
               self._scoreUploader         = nil;

            end
         ,
            function ()
               self._scoreUploadTimer   = 15;
               self._scoreUploader      = nil;
            end
      );

   end
end

----------------------------------------------------------------

function RankingMgr:_uploadLevel()

   if       self._levelUploader == nil
      and   self._levelToUpload   ~= nil
      and   LoginMgr:isTango()
      and   OnlineMgr:isOnline()
      and   ContactMgr:isReady()    then

      local backup         = self._levelToUpload;
      self._levelUploader  = LevelUploader:new( self._levelToUpload );

      self._levelUploader:exec(
                                 self
                              ,
                                 function ( self )
                                    --If the level we just uploaded successfully was the same
                                    -- or at even better
                                    if math.floor(backup) >= math.floor(self._levelToUpload or 0) then
                                       self._levelToUpload      = nil;
                                       self._levelUploadTimer   = nil;
                                    else
                                       self._levelUploadTimer   = 0
                                    end

                                    self:_saveToUpload();
                                    self._levelUploader         = nil;

                                 end
                              ,
                                 function ()
                                    self._levelUploadTimer   = 15;
                                    self._levelUploader      = nil;
                                 end
                              );

   end
end

----------------------------------------------------------------

function RankingMgr:_addPlayerScoreToRank( score )

   self._playerRank = nil;

   if self._ranking then

      local e           = nil;
      self._playerRank  =  self:_findUserRank( self._ranking, self:_getPlayerId() );

      --If I already have a score in the ranking
      if self._playerRank and self._playerRank > 0 and self._playerRank <= #self._ranking then

         e     = self._ranking[self._playerRank];
         score = math.max(e.score or 0,score);

         table.remove(self._ranking, self._playerRank);

      else

         e = self:_createRankEntry(self:_getPlayerId(), score);

      end

      --Should always be true but just in case
      if e and e.tangoId == self:_getPlayerId() then

         e.score          = score;

         self._playerRank = self:_findScoreRank(self._ranking, score);

         if self._playerRank then
            table.insert(self._ranking, self._playerRank, e );
         else
            Debug.assert(self._playerRank);
         end

      else
         Debug.error("Bad entry in RankingMgr:_addPlayerScoreToRank");
      end

      -- Update the weekly reset ID if updated
      self:_loadWeeklyResetID();
   end

   EventMgr:postEvent( {playerRank = self._playerRank }, "player-rank" );

end

----------------------------------------------------------------

function RankingMgr:_updateTangoWeeklyReset()

   -- print("RankingMgr:_updateTangoWeeklyReset");

   local time = self:_getReferenceTime();

   if HACK_FORCE_RESET then

      -- print("RankingMgr:_updateTangoWeeklyReset - HACK_FORCE_RESET");

      self:_resetTangoScore(time);
      return;
   end

   if not OnlineMgr:isOnline() then

      -- print("RankingMgr:_updateTangoWeeklyReset - failed because we're offline");

      return;
   end

   if not self._playerRank or self._playerRank <= 0 or self._playerRank > #self._ranking then

      -- print("RankingMgr:_updateTangoWeeklyReset - failed because player rank is invalid");

      return;
   end

   if not self._ranking or not self._ranking[self._playerRank] or not self._ranking[self._playerRank].prevScoreStamp then

      -- print("RankingMgr:_updateTangoWeeklyReset - failed because ranking list is bad or prevScoreStamp missing");

      return;
   end

   if self._weeklyResetID == self._ranking[self._playerRank].prevScoreStamp then

      -- print("RankingMgr:_updateTangoWeeklyReset - _weeklyResetID equals prevScoreStamp");

      return;
   end

   if self:canCycleTangoLinkedScore(time) then
      self:_cycleTangoScore(time);
   else
      self:_resetTangoScore(time);
   end

end

----------------------------------------------------------------

function RankingMgr:_updateGuestWeeklyReset()

   local time = self:_getReferenceTime();

   if HACK_FORCE_RESET then

      -- print("RankingMgr:_updateGuestWeeklyReset - HACK_FORCE_RESET");

      self:_resetGuestScore(time);
      return;
   end

   if not self._weeklyResetTime2 then

      -- print("RankingMgr:_updateGuestWeeklyReset - failed because we're offline");

      return;
   end

   if not self._playerRank or self._playerRank <= 0 or self._playerRank > #self._ranking then

      -- print("RankingMgr:_updateGuestWeeklyReset - failed because player rank is invalid");

      return;
   end

   if not self._ranking or not self._ranking[self._playerRank] then

      -- print("RankingMgr:_updateGuestWeeklyReset - failed because ranking list is bad");

      return;
   end

   if time < self._weeklyResetTime2 then

      -- print("RankingMgr:_updateGuestWeeklyReset -  time less than _weeklyResetTime2");

      return;
   end

   self:_resetGuestScore(time);

end

----------------------------------------------------------------

function RankingMgr:_saveLastWeekRanking()

   if self._lastWeekRanking then

      saveFileMultiFormat(RANK_LAST_WEEK_FILENAME_LUA, RANK_LAST_WEEK_FILENAME_JSON, self._lastWeekRanking);

      self._playerlastWeekRank = self:_getPlayerLastWeekRank();
   end

end

----------------------------------------------------------------

function RankingMgr:_loadLastWeekRanking()

   self._lastWeekRanking = loadFileMultiFormat(RANK_LAST_WEEK_FILENAME_LUA, RANK_LAST_WEEK_FILENAME_JSON) or {};

   for _, v in ipairs(self._lastWeekRanking) do
      local c = ContactMgr:findContact(v.tangoId);

      v.contact = c;
   end

   self._playerlastWeekRank = self:_getPlayerLastWeekRank();

end

----------------------------------------------------------------

function RankingMgr:_getPlayerLastWeekRank()

   self._playerlastWeekRank = nil;

   if self._lastWeekRanking and #self._lastWeekRanking > 1 then

      return self:_findUserRank(self._lastWeekRanking, self:_getPlayerId());
   end

   return nil;

end

----------------------------------------------------------------

function RankingMgr:_postWeeklyReset( silent )

   local tbl = self:_buildLastWeekRanking();

   self:_saveLastWeekRanking();

   if self:isLastWeekRankingValid() then

      local t = {lastWeekRanking = tbl, playerRank = self._playerlastWeekRank};

      if not silent then
         EventMgr:postEvent( t, "weekly-reset" );
      end
   end

end

----------------------------------------------------------------

function RankingMgr:_buildLastWeekRanking()

   if not Debug.assertType(self._ranking, "table", "RankingMgr:_buildLastWeekRanking -> self._ranking") then
      return nil;
   end

   self:clearLastWeekRanking();

   for i=1, #self._ranking do

      local contact = ContactMgr:findContact(self._ranking[i].tangoId);

      if contact and contact:isGhost() then
         self:_lastWeekInsertEntry(self._lastWeekRanking, contact, self._ranking[i].tangoId, self._ranking[i].score, self._ranking[i].level, self._ranking[i].score);
      elseif contact and contact:isGuestFriend() then
         self:_lastWeekInsertEntry(self._lastWeekRanking, contact, self._ranking[i].tangoId, self._ranking[i].score, self._ranking[i].level, self._ranking[i].score);
      else
         self:_lastWeekInsertEntry(self._lastWeekRanking, contact, self._ranking[i].tangoId, self._ranking[i].score, self._ranking[i].level, self._ranking[i].prevScore);
      end
   end

   return self._lastWeekRanking;
end

----------------------------------------------------------------------

function RankingMgr:_lastWeekInsertEntry( tbl, c, tangoId, score, level, prevScore )

   local entry = { tangoId=tangoId, contact=c, score=score, level=level, prevScore=prevScore };

   for i=1,#tbl do
      -- Use >= instead of > to have player on top when having same score
      if prevScore >= tbl[i].prevScore then
         table.insert( tbl, i, entry );

         return;
      end
   end

   table.insert( tbl, entry );

end

----------------------------------------------------------------

function RankingMgr:_resetTangoScore(time)

   Debug.assert( time, "RankingMgr:_resetTangoScore() -> time");

   HACK_FORCE_RESET = false;

   local prevScoreStamp = self._ranking[self._playerRank].prevScoreStamp;
   if prevScoreStamp then
      self:_saveWeeklyResetID(prevScoreStamp);
   end

   self:_saveWeeklyResetTime(time);
   self:_submitAllGhostPrevScores();
   GuestMgr:submitPrevGuestFriendScore(true);
   self:_downloadRank();
   self:_postWeeklyReset();

end

----------------------------------------------------------------

function RankingMgr:_resetGuestScore(time)

   Debug.assert( time, "RankingMgr:_resetGuestScore() -> time");

   HACK_FORCE_RESET = false;

   self._ranking[self._playerRank].prevScore = self._ranking[self._playerRank].score;

   local prevScoreStamp = self._ranking[self._playerRank].prevScoreStamp;
   if prevScoreStamp then
      self:_saveWeeklyResetID(prevScoreStamp);
   end

   self:_saveWeeklyResetTime(time);
   self:_submitAllGhostPrevScores();
   GuestMgr:submitPrevGuestFriendScore();
   self:submitScore(self._ranking[self._playerRank].score);
   self:_downloadRank();
   self:_postWeeklyReset();

end

----------------------------------------------------------------

function RankingMgr:_cycleTangoScore()

   local time = self:_getReferenceTime();

   Debug.assert( time, "RankingMgr:_resetTangoScore() -> time");

   local prevScoreStamp = self._ranking[self._playerRank].prevScoreStamp;
   if prevScoreStamp then
      self:_saveWeeklyResetID(prevScoreStamp);
   end

   self:_saveWeeklyResetTime(time);
   self:_submitAllGhostPrevScores();
   GuestMgr:submitPrevGuestFriendScore();
   self:_downloadRank();
   self:_postWeeklyReset(true);
   App.data:giveWeeklyReward(0); --Pass in 0 to update the time, but dont give rewards and dont post analytics
   GuestMgr:cycleGuestFriend();
   self:clearLastWeekRanking();

   if App.data:getValue("App.settings.bounty-reset-offset", 0) == 0 then
      BountyMgr:resetBounty();
   end

   BountyMgr:resetBountyTimeStamp();

end

----------------------------------------------------------------

function RankingMgr:_cycleGuestScore()

   local time = self:_getReferenceTime();

   Debug.assert( time, "RankingMgr:_resetGuestScore() -> time");

   self._ranking[self._playerRank].prevScore = self._ranking[self._playerRank].score;
   self._ranking[self._playerRank].score     = 0;

   local prevScoreStamp = self._ranking[self._playerRank].prevScoreStamp;
   if prevScoreStamp then
      self:_saveWeeklyResetID(prevScoreStamp);
   end

   self:_saveWeeklyResetTime(time);
   self:_submitAllGhostPrevScores();
   GuestMgr:submitPrevGuestFriendScore();
   self:submitScore(self._ranking[self._playerRank].score);
   self:_downloadRank();
   self:_postWeeklyReset(true);
   App.data:giveWeeklyReward(0); --Pass in 0 to update the time, but dont give rewards and dont post analytics
   GuestMgr:cycleGuestFriend();
   self:clearLastWeekRanking();

   if App.data:getValue("App.settings.bounty-reset-offset", 0) == 0 then
      BountyMgr:resetBounty();
   end

   BountyMgr:resetBountyTimeStamp();

end

----------------------------------------------------------------

function RankingMgr:_getPlayerId()

   if ContactMgr and ContactMgr:getProfile() and ContactMgr:getProfile():getAccountId() then
      return ContactMgr:getProfile():getAccountId();
   else
      Debug.assert( ContactMgr  );
      if ContactMgr then
         Debug.assert( ContactMgr:getProfile()  );
         if ContactMgr:getProfile() then
            Debug.assert( ContactMgr:getProfile():getAccountId() );
         end
      end
   end

end

----------------------------------------------------------------

function RankingMgr:_saveWeeklyResetID( id )

   if not Debug.assertType(id, {"number", "string"}, "RankingMgr:_saveWeeklyResetID -> id") then
      return;
   end

   local tbl = {};

   self._weeklyResetID = id;
   tbl.id = self._weeklyResetID;

   saveFileMultiFormat(RANK_WEEKLY_RESET_ID_FILENAME_LUA, RANK_WEEKLY_RESET_ID_FILENAME_JSON, tbl);
end

----------------------------------------------------------------

function RankingMgr:_loadWeeklyResetID()

   local tbl = loadFileMultiFormat(RANK_WEEKLY_RESET_ID_FILENAME_LUA, RANK_WEEKLY_RESET_ID_FILENAME_JSON) or {};

   if tbl and tbl.id then
      self._weeklyResetID = tbl.id;
   else
      local rank = self:getPlayerRank();

      if rank then

         local e = self:getRankEntry(rank);

         if e then
            if e.prevScoreStamp then
               self:_saveWeeklyResetID(e.prevScoreStamp);
            end
         else
            Debug.error( "RankingMgr:_loadWeeklyResetID -> e");
         end

      else
         Debug.error( "RankingMgr:_loadWeeklyResetID -> rank" );
      end
   end

end

----------------------------------------------------------------

function RankingMgr:removeAllGuestFriends( )

   if not Debug.assertType(self._ranking, "table", "RankingMgr:removeAllGuestFriends -> self._ranking") then
      return;
   end

   for i = #self._ranking, 1, -1 do
      if self._ranking[i].contact:isGuestFriend() then
         table.remove(self._ranking, i);
      end
   end

end

----------------------------------------------------------------

-- Counts number of "real" (i.e. non-guest) entries
function RankingMgr:getRankEntryNonGuestCount( leagueOn )

   local DEFAULT_NONGUEST_COUNT = 1;

   if not Debug.assertType(self._ranking, "table", "RankingMgr:getRankEntryNonGuestCount -> self._ranking") then
      return DEFAULT_NONGUEST_COUNT;
   end

   local count = 0;

   if leagueOn then

      local rank = self:getPlayerRank();

      for k, v in ipairs(self._ranking) do

         if not v.contact:isGuestFriend() then

            if v.contact:isGhost() then
               --ignore ghosts, they are only on friends lists
            elseif k == rank then
               count = count + 1;
--TODO MQL: for the initial league test we only want all the league to be guests
-- for full release, we need to add tango picked players
--[[
            else
               -- TODO: need to filter out for a match list
               local leagueIndex, divIndex = LeagueMgr:calcPlacement(v.prevScore);

               if LeagueMgr:doesMatchPlayer( leagueIndex, divIndex ) then
                  count = count + 1;
               end
]]
            end
         end
      end

   else
      count = #self._ranking;
   end

   return math.max(count, DEFAULT_NONGUEST_COUNT);

end

----------------------------------------------------------------

function RankingMgr:getRankingLeagueList( includeBounty )

   if not Debug.assertType(self._ranking, "table", "RankingMgr:getRankingLeagueList -> self._ranking") then
      return;
   end

   if not LeagueMgr:isLeagueEnabled() then
      Debug.error("RankingMgr:getRankingLeagueList is only valid when leagues are enabled.");
      return;
   end

   local tbl   = {};
   local pRank = self:getPlayerRank();
   local lRank = pRank;
   local count = 0;

   for i, v in ipairs(self._ranking) do

      if i == pRank then
         table.insert(tbl, v);
         count = count + 1;
         lRank = count;
      elseif v.contact == nil then
         Debug.assert(v.contact, "RankingMgr:getRankingLeagueList -> nil contact self._ranking["..tostring(i).."]")
      elseif v.contact:isFriend()  then

         -- MQL: Add in after tango online support
         --[[
            local l = v.contact:getLeagueIndex();
            local d = v.contact:getDivisionIndex();

            if LeagueMgr:doesMatchPlayer(l, d) then
               table.insert(tbl, v);
               count = count + 1;
            end
         ]]

      elseif v.contact:isLeague() then
         table.insert(tbl, v);
         count	= count + 1;
      elseif includeBounty and BountyMgr:isBountyEntryValid(v.tangoId) then
         table.insert(tbl, v);
         count	= count + 1;
      end
   end

   return tbl, lRank;

end

----------------------------------------------------------------

function RankingMgr:getRankingFriendsList()

   if not Debug.assertType(self._ranking, "table", "RankingMgr:getRankingFriendsList -> self._ranking") then
      return;
   end

   if not LeagueMgr:isLeagueEnabled() then
      Debug.error("RankingMgr:getRankingFriendsList is only valid when leagues are enabled.");
      return;
   end

   local tbl	= {};
   local pRank	= self:getPlayerRank();
   local fRank	= pRank;
   local count = 0;

   for k, v in ipairs(self._ranking) do

      if k == pRank then
         table.insert(tbl, v);
         count	= count + 1;
         fRank	= count;
      elseif v.contact == nil then
         Debug.assert(v.contact, "RankingMgr:getRankingFriendsList -> nil contact self._ranking["..tostring(k).."]")
      elseif         ( v.contact:isFriend() or v.contact:isGhost() )
               and   App.data:isFriendsUnlocked() then

         table.insert(tbl, v);
         count	= count + 1;
      end
   end

   return tbl, fRank;

end

----------------------------------------------------------------

function RankingMgr:getRankingLeagueListLastWeek()

   if not Debug.assertType(self._lastWeekRanking, "table", "RankingMgr:getRankingLeagueListLastWeek -> self._lastWeekRanking") then
      return;
   end

   if not LeagueMgr:isLeagueEnabled() then
      Debug.error("RankingMgr:getRankingLeagueListLastWeek is only valid when leagues are enabled.");
      return;
   end

   local tbl   = {};
   local pRank = self:getPlayerLastWeekRank();
   local lRank = pRank;
   local count = 0;

   for k, v in ipairs(self._lastWeekRanking) do

      if k == pRank then
         table.insert(tbl, v);
         count	= count + 1;
         lRank	= count;
      elseif v.contact == nil then
         Debug.assert(v.contact, "RankingMgr:getRankingLeagueListLastWeek -> nil contact self._lastWeekRanking["..tostring(k).."]")
      elseif v.contact:isFriend() then

         -- MQL: Add in after tango online support
         --[[
            local l	= v.contact:getLeagueIndex();
            local d	= v.contact:getDivisionIndex();

            if LeagueMgr:doesMatchPlayer(l, d) then
               table.insert(tbl, v);
               count = count + 1;
            end
         ]]

      elseif v.contact:isLeague() then
         table.insert(tbl, v);
         count	= count + 1;
      end
   end

   return tbl, lRank;

end

----------------------------------------------------------------

function RankingMgr:getRankingFriendsListLastWeek()

   if not Debug.assertType(self._lastWeekRanking, "table", "RankingMgr:getRankingFriendsListLastWeek -> self._lastWeekRanking") then
      return;
   end

   if not LeagueMgr:isLeagueEnabled() then
      Debug.error("RankingMgr:getRankingFriendsListLastWeek is only valid when leagues are enabled.");
      return;
   end

   local tbl   = {};
   local pRank = self:getPlayerLastWeekRank();
   local fRank = pRank;
   local count = 0;

   for k, v in ipairs(self._lastWeekRanking) do

      if k == pRank then
         table.insert(tbl, v);
         count = count + 1;
         fRank = count;
      elseif v.contact == nil then
         Debug.assert(v.contact, "RankingMgr:getRankingFriendsListLastWeek -> nil contact self._lastWeekRanking["..tostring(k).."]")
      elseif v.contact:isFriend() or v.contact:isGhost() then
         table.insert(tbl, v);
         count = count + 1;
      end
   end

   return tbl, fRank;

end

----------------------------------------------------------------------
