
require('engine/object/object')

----------------------------------------------------------------------

local GAME_CENTER_SUBMIT_SCORE_TIMER_SUCCESS      = 60*0.50;
local GAME_CENTER_SUBMIT_SCORE_TIMER_FAILURE      = 60*1.50;

----------------------------------------------------------------------

local Base                                = Object;
GameCenterScoreController                 = Base:new();

----------------------------------------------------------------------

function GameCenterScoreController:new(  )

   local controller = Base.new(self);

   --combine to submit and submitting because if we got interrupted while submitting, we want to re-submit
   controller.scoreToSubmit = math.max(   App.data:getValue("GameCenterScoreController.scoreToSubmit")   or 0
                                       ,  App.data:getValue("GameCenterScoreController.scoreSubmitting") or 0  );
   controller.scoreSubmitting     = nil;
   controller.leaderboardId       = AbTestMgr:getGameCenterSubmitLeaderboardId() or "RR_Leaderboard_Score";

   return controller;

end

----------------------------------------------------------------------

function GameCenterScoreController:update( dt )

   self.scoreSubmitTimer = (self.scoreSubmitTimer or 0) - dt;

   if  (self.scoreToSubmit) and (self.scoreToSubmit > 0) and (not self.scoreSubmitting ) then

      if self.scoreSubmitTimer <= 0 then

         if MOAIGameCenterIOS and MOAIGameCenterIOS:isAuthenticated() then

            self:_submitNextScore();

         end

      end

   end

end

----------------------------------------------------------------------

function GameCenterScoreController:submitScore( score )

   -- if the new score is higher than the one we are attempting to submit
   if (score or 0) > (self.scoreToSubmit or 0) then

      self.scoreToSubmit = App.data:setValue("GameCenterScoreController.scoreToSubmit", score );

      self:_save();

   end

end

----------------------------------------------------------------------

function GameCenterScoreController:_submitNextScore(  )

   Debug.assert( MOAIGameCenterIOS and MOAIGameCenterIOS:isAuthenticated(), "GameCenterScoreController:_submitNextScore -> MOAIGameCenterIOS and MOAIGameCenterIOS:isAuthenticated()" );

   if (not self.scoreSubmitting) and ((self.scoreToSubmit or 0) > 0) then

      self.scoreSubmitting = self.scoreToSubmit;
      self.scoreToSubmit   = nil;

      EventMgr:postEvent({score = self.scoreSubmitting }, "game-center-upload-score-attempt");

      MOAIGameCenterIOS:setListener(   MOAIGameCenterIOS.SUBMIT_SCORE_SUCCESS,
                                       function()
                                          self.scoreSubmitting  = nil;
                                          self.scoreSubmitTimer = GAME_CENTER_SUBMIT_SCORE_TIMER_SUCCESS;

                                          self:_save();
                                          EventMgr:postEvent({result="success"}, "game-center-upload-score-success");
                                       end );

      MOAIGameCenterIOS:setListener(   MOAIGameCenterIOS.SUBMIT_SCORE_FAILURE,
                                       function(errorMsg)
                                          self.scoreSubmitTimer = GAME_CENTER_SUBMIT_SCORE_TIMER_FAILURE;
                                          --Resubmit only if this is the highest score
                                          self:submitScore( self.scoreSubmitting );
                                          self.scoreSubmitting = nil;
                                          self:_save();
                                          EventMgr:postEvent({result="failure", error=errorMsg}, "game-center-upload-score-failure");
                                       end );

      local submitted, errorMsg = MOAIGameCenterIOS:submitScore( self.leaderboardId, self.scoreSubmitting );

      if (not submitted) then

         self.scoreSubmitTimer = GAME_CENTER_SUBMIT_SCORE_TIMER_FAILURE;
         self:submitScore( self.scoreSubmitting );
         self.scoreSubmitting = nil;

         EventMgr:postEvent({result="failure", error=errorMsg}, "game-center-upload-score-failure");

      end

      self:_save();

   else

      Debug.assert(score, "GameCenterScoreController:_submitGameCenterScore -> (not self.scoreSubmitting) and ((self.scoreToSubmit or 0) > 0)" );

   end

end

----------------------------------------------------------------------

function GameCenterScoreController:_save(  )

   App.data:setValue("GameCenterScoreController.scoreToSubmit",   self.scoreToSubmit   );
   App.data:setValue("GameCenterScoreController.scoreSubmitting", self.scoreSubmitting );

end

----------------------------------------------------------------------
