
require('app/contact/contact-manager')
require('app/server/tango')

----------------------------------------------------------------------

local Base           = TangoServer;
LevelUploader        = Base:new();

local LEVEL          = "level";

----------------------------------------------------------------------

function LevelUploader:new( level )

   assert( ContactMgr:getProfile() );
   assert( LoginMgr:isTango() );

   local uploader          = Base.new(self);

   uploader.level          = level;
   uploader.myAccountId    = ContactMgr:getProfile():getAccountId();

   return uploader;

end

----------------------------------------------------------------------

function LevelUploader:exec( callObj, callSuccess, callFailure )

   if OnlineMgr:isOnline() and LoginMgr:isTango() then

      local requestSuccessFunc = function( result )
         self:callCallback( callObj, callSuccess, self.level );
      end

      local requestFailureFunc = function(code, message)
         self:callCallback( callObj, callFailure, code, message );
      end

      local metricData = {
         {
            metric = LEVEL,
            value = self.level,
            rollups = {
               MOAITangoMetrics.MAX
            }
         }
      }

      local requestOptions = {
         requestClass = MOAITangoSetRawMetricsRequest,
         requestData = {metricData},
         successFunc = requestSuccessFunc,
         failureFunc = requestFailureFunc,
      }

      NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

   else

      delay(   0.1,
               function ()
                  self:callCallback( callObj,  callFailure, -1, "You are offline"  );
               end   );

   end

end
----------------------------------------------------------------------
