
require('app/contact/contact-manager')
require('app/server/tango')

----------------------------------------------------------------------

----------------------------------------------------------------------

local Base						= TangoServer;
RankingRetriever				= Base:new();

local SCORE						= "score";
local LEVEL						= "level";

local HACK_SCORE_1000      = false;
local HACK_FAKE_ENTRY		= nil;

local HACK_RAND_SCORE		= nil;
local HACK_RAND_PREVSCORE	= nil;
local HACK_RAND_LEVEL		= nil;

--Do not change here but above
if not BUILD_DEBUG then
   HACK_QUICK_WEEKLY_RESET = false;
   HACK_SCORE_1000			= false;
   HACK_FAKE_ENTRY			= nil;
   HACK_RAND_SCORE			= nil;
   HACK_RAND_PREVSCORE		= nil;
   HACK_RAND_LEVEL			= nil;
end

----------------------------------------------------------------------

function RankingRetriever:new( )

   local loader        = Base.new(self);

   loader.rank         = {};
   loader.contactList  = {};
   loader.hackScore1000= 0;


   for i=1,ContactMgr:getContactCount() do
      local contact = ContactMgr:getContact(i);
      if contact and contact:isAppInstalled() and (not contact:isGhost()) and (not contact:isGuestFriend()) then
         table.insert( loader.contactList,  contact:getAccountId() );
      end
   end

   local playing  = #loader.contactList-1;
   local previous = App.data:getValue("friends-playing-RR-count");
   App.data:setValue("friends-playing-RR-count",playing);


   MOAIFlurry:event( "Tango-Friends",  "total",      "" ..(ContactMgr:getContactCount()-1),
                                       "play-RR",    "" ..(playing)                             );

   EventMgr:postEvent({ previous = previous, playing=playing}, "friend-playing-count");


   return loader;

end

----------------------------------------------------------------------

function RankingRetriever:exec( callObj, callSuccess, callFailure )

   assert( LoginMgr:isTango() );

   if OnlineMgr:isOnline() then

      local requestSuccessFunc = function(result)
         self:_onSuccess(result, callObj, callSuccess)
      end

      local requestFailureFunc = function(code, message)
         self:_onFailure(code, mesage, callObj, callFailure);
      end

      local metricData = {
         {
            metric  = SCORE,
            rollups = {
               MOAITangoMetrics.MAX_THIS_WEEK, MOAITangoMetrics.MAX_LAST_WEEK,
               MOAITangoMetrics.MAX_THIS_DAY,  MOAITangoMetrics.MAX_LAST_DAY,
               MOAITangoMetrics.MAX_THIS_HOUR, MOAITangoMetrics.MAX_LAST_HOUR,
            }
         },
         {
            metric  = LEVEL,
            rollups = {
               MOAITangoMetrics.MAX,
            }
         }
      }

      local requestOptions = {
         requestClass = MOAITangoGetComputedMetricsRequest,
         requestData = {self.contactList, metricData},
         successFunc = requestSuccessFunc,
         failureFunc = requestFailureFunc,
      }

      NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

   else

      -- Use a timer to create be more consistent with the authenticate mode
      delay(
         0.1
         ,
         function ()
            self:callCallback( callObj,  callFailure, -1, "You are offline"  );
         end

      );
   end

end

----------------------------------------------------------------------

function RankingRetriever:rankInsert( tangoId, score, level, prevScore, prevScoreStamp )

	-- print('RankingRetriever:rankInsert', tangoId, score, level, prevScore, prevScoreStamp);

	local entry =	{tangoId=tangoId, score=score, level=level, prevScore=prevScore, prevScoreStamp=prevScoreStamp};

   for i=1,#self.rank do
      -- Use >= instead of > to have player on top when having same score
      if score >= self.rank[i].score then
         table.insert( self.rank, i, entry );
         return;

      end
   end

   table.insert( self.rank, entry );

end

----------------------------------------------------------------------

function RankingRetriever:_onSuccess( result, callObj, callSuccess )

   for i=1, #self.contactList do

      local r2    		= result[self.contactList[i]] or {};
      local score 		= (r2[SCORE] or {}).MAX_THIS_WEEK or 0;
      local prevScore	= (r2[SCORE] or {}).MAX_LAST_WEEK or 0;
      local level 		= (r2[LEVEL] or {}).MAX or 0;
      local prevScoreStamp = (r2[SCORE] or {}).MAX_LAST_WEEK_last_modified or 0;

      if HACK_QUICK_WEEKLY_RESET then

         if HACK_QUICK_WEEKY_RESET_TIME == 60*60*24*7 then

            -- Don't do anything - we should be set up

         elseif HACK_QUICK_WEEKLY_RESET_TIME >= 60*60*24 then

            score = (r2[SCORE] or {}).MAX_THIS_DAY or 0;
            prevScore = (r2[SCORE] or {}).MAX_LAST_DAY or 0;
            prevScoreStamp = (r2[SCORE] or {}).MAX_LAST_DAY_last_modified or 0;

         elseif HACK_QUICK_WEEKLY_RESET_TIME <= 60*60 then

            score = (r2[SCORE] or {}).MAX_THIS_HOUR or 0;
            prevScore = (r2[SCORE] or {}).MAX_LAST_HOUR or 0;
            prevScoreStamp = (r2[SCORE] or {}).MAX_LAST_HOUR_last_modified or 0;
         end

      end

      if HACK_SCORE_1000 then
         if self.contactList[i] ~= ContactMgr:getProfile():getAccountId() then
            self.hackScore1000   = (self.hackScore1000 or 0 ) + 1000;
            score                = self.hackScore1000;
         else
            score = 0;
         end
      end

      if HACK_RAND_SCORE and self.contactList[i] ~= ContactMgr:getProfile():getAccountId() then
         score = math.random(0, HACK_RAND_SCORE);
      end

      if HACK_RAND_PREVSCORE then
         prevScore = math.random(0, HACK_RAND_PREVSCORE);
      end

      self:rankInsert( self.contactList[i], score, level, prevScore, prevScoreStamp );
   end

   if GuestMgr:isGuestFriendEnabled() then

      if not GuestMgr:isGuestFriendLoggedIn() then
         GuestMgr:onLoginSuccess();
      end

      local guestFriendList	= GuestMgr:updateGuestStats();

      for k,v in pairs(guestFriendList) do
         if v.tangoId ~= ContactMgr:getProfile():getAccountId() then
            self:rankInsert( v.tangoId, v.score, v.level, v.prevScore, v.prevScoreStamp );
         end
      end
   end

   if HACK_FAKE_ENTRY then
      for i=1,HACK_FAKE_ENTRY do
         self:rankInsert( "debugTangoId-"..i, math.random(0, HACK_RAND_SCORE or 10000), math.random(1, HACK_RAND_LEVEL or 100), math.random(0, HACK_RAND_PREVSCORE or 10000), TimeMgr:getTimeGMT() );
      end
   end

   self:callCallback( callObj, callSuccess, self.rank );

end

----------------------------------------------------------------------

function RankingRetriever:_onFailure(code, message, callObj, callFailure)
   self:callCallback( callObj, callFailure, code, message );
end

----------------------------------------------------------------------
