
require('app/contact/contact-manager')
require('app/server/tango')

----------------------------------------------------------------------

local Base           = TangoServer;
ScoreUploader        = Base:new();

local SCORE          = "score";

----------------------------------------------------------------------

function ScoreUploader:new( score )

   assert( ContactMgr:getProfile() );
   assert( LoginMgr:isTango() );

print( "ScoreUploader:new", tableName, score );

   local uploader          = Base.new(self);

   uploader.score          = score;
   uploader.myAccountId    = ContactMgr:getProfile():getAccountId();

   return uploader;

end

----------------------------------------------------------------------

function ScoreUploader:exec( callObj, callSuccess, callFailure )

   print("ScoreUploader:exec");

   assert(LoginMgr:isTango());

   if OnlineMgr:isOnline() then

      local requestSuccessFunc = function( result )

         self:callCallback( callObj, callSuccess, self.score);

      end

      local requestFailureFunc = function(code, message)

         self:callCallback( callObj,  callFailure, code, message  );

      end

      local metricData = {
         {
            metric = SCORE,
            value = self.score,
            rollups = {
               MOAITangoMetrics.MAX,
               MOAITangoMetrics.MAX_THIS_WEEK,
               MOAITangoMetrics.MAX_LAST_WEEK,
               MOAITangoMetrics.MAX_THIS_DAY,
               MOAITangoMetrics.MAX_LAST_DAY,
               MOAITangoMetrics.MAX_THIS_HOUR,
               MOAITangoMetrics.MAX_LAST_HOUR
            }
         }
      }

      local requestOptions = {
         requestClass = MOAITangoSetRawMetricsRequest,
         requestData = { metricData },
         successFunc = requestSuccessFunc,
         failureFunc = requestFailureFunc,
      }

      NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

   else

      -- Use a timer to create be more consistent with the authenticate mode
      self.timer = MOAITimer.new();
      self.timer:setSpan( 0.1 );
      self.timer:setListener( MOAITimer.EVENT_TIMER_END_SPAN,
         function ()
            self.timer:stop( );
            self.timer = nil;
            self:callCallback( callObj,  callFailure, -1, "You are offline"  );
         end   );

      self.timer:start();

   end

end
----------------------------------------------------------------------
