----------------------------------------------------------------------
require('app/store/controller/store-controller')
----------------------------------------------------------------------

local Base              = StoreController;

AppStoreController      = Base:new();

----------------------------------------------------------------------

function AppStoreController:new( storeMgr )

   local ctrl  = Base.new(self, storeMgr);
   return ctrl;

end

----------------------------------------------------------------------

function AppStoreController:init()

   Base.init( self );

   self:_setupListeners();

end

----------------------------------------------------------------

function AppStoreController:canMakePayments()
   return MOAIBillingIOS.canMakePayments();
end

----------------------------------------------------------------

function AppStoreController:update( dt )
end

----------------------------------------------------------------------

function AppStoreController:_setupListeners()

   MOAIBillingIOS.setListener(   MOAIBillingIOS.PAYMENT_QUEUE_TRANSACTION,
                                 function( result )
                                    self:_onPaymentQueueTransactions( result );
                                 end
                              );

   MOAIBillingIOS.setListener(
                                 MOAIBillingIOS.PAYMENT_QUEUE_ERROR,
                                 function( error )
                                    self:_requestFailed(self._currRequest, StoreController.REQUEST_FAILURE_FAILED, "queue error" );
                                 end
                              );
end

----------------------------------------------------------------

function AppStoreController:_pickRequestToValidate( productId )

   -- Prioritize the current request
   if self._currRequest and self._currRequest.productId == productId then
      return self._currRequest;
   end

   if self._requests then

      for rId,request in pairs(self._requests) do

         if       (request.productId == productId)
            and   (request.status    == "purchasing")  then
            return request;
         end
      end

   end
end

----------------------------------------------------------------------

function AppStoreController:_onPaymentQueueTransactions( result )

   if result then

      if result.payment and result.payment.productIdentifier then

         --Not necessarily the current request since this is a callback and we might
         -- get call to complete a previous transaction
         local request = self:_pickRequestToValidate( result.payment.productIdentifier );

         if request then

            if result.transactionState == MOAIBillingIOS.TRANSACTION_STATE_PURCHASED then

               self:_validatePurchase( request, result.transactionReceipt, "" );

            elseif result.transactionState == MOAIBillingIOS.TRANSACTION_STATE_FAILED then

               self:_requestFailed(request, StoreController.REQUEST_FAILURE_FAILED, "failed" );

            elseif result.transactionState == MOAIBillingIOS.TRANSACTION_STATE_CANCELLED then

               self:_requestFailed(request, StoreController.REQUEST_FAILURE_CANCELED, "cancelled" );

            end

            if result.failureReason ~= "" then
               print("iOS store transaction failed for with enum: ", result.failureReason);
            else
               print("No iOS store failure.");
            end

         else
            Debug.assert(request, "AppStoreController:_onPaymentQueueTransactions -> request");
         end
      else

         Debug.assert(  result.payment,                           "AppStoreController:_onPaymentQueueTransactions -> (result or {}).payment"                           );
         Debug.assert( (result.payment or {}).productIdentifier,  "AppStoreController:_onPaymentQueueTransactions -> ((result or {}).payment or {}).productIdentifier" );

      end
   else

      Debug.assert(result, "AppStoreController:_onPaymentQueueTransactions -> result" );

   end

end

----------------------------------------------------------------

function AppStoreController:_makePurchase( request )

   if  request  then

      MOAIBillingIOS.requestPaymentForProduct( request.productId );

   else

      Debug.assert( request, "AppStoreController:_makePurchase -> request"  );

   end

end

----------------------------------------------------------------
