require('app/template/template-manager');
require('app/store/controller/amazon-v2-store-controller')
require('app/store/controller/app-store-controller')
require('app/store/controller/google-play-v3-store-controller')

----------------------------------------------------------------------

----------------------------------------------------------------------

StoreMgr  =
{
   _needReceipt = true;

   -- _productInfo stores categories of product key-value pairs,
   -- where the key is the product name and the value is the product id associated with the IAP.
   -- Product IDs should be unique; categories are provided only for convenience.
   _productInfo  = {
      gem = {
         gem1 = "gem-01",
         gem2 = "gem-02",
         gem3 = "gem-03",
         gem4 = "gem-04",
         gem5 = "gem-05",
         gem6 = "gem-06",
      },
      gemHoliday = {
         gemHoliday1 = "gem-11",
         gemHoliday2 = "gem-12",
         gemHoliday3 = "gem-13",
         gemHoliday4 = "gem-14",
         gemHoliday5 = "gem-15",
         gemHoliday6 = "gem-16",
      },
      special = {
         starter = "special-starter-01",
         offer1 = "special-offer-01",
         offer2 = "special-offer-02",
         offer3 = "special-offer-03",
      },
      consumable = {

      },
   },
};

----------------------------------------------------------------

function StoreMgr:init ()

   self._controller = self:_createController();

   if self._controller then
      self._controller:init();
   else
      Debug.assert( self._controller, "StoreMgr:init -> self._controller" );
   end

   EventMgr:addListener( self, self._onCloudData, "cloud:master-data-store-info" );

end

----------------------------------------------------------------

function StoreMgr:update( dt )

   if not Debug.assertType(self._controller, {"table", "userdata"}, "StoreMgr:update -> self._controller") then
      return;
   end

   self._controller:update( dt );

end

----------------------------------------------------------------------

function StoreMgr:getGemProducts()

   if not Debug.assertType(self._productInfo, "table", "StoreMgr:getGemProducts -> self._productInfo") then
      return {};
   end

   if PromoMgr:getCurrentSales("holiday") then
      return self._productInfo.gemHoliday;
   else
      return self._productInfo.gem;
   end

end

----------------------------------------------------------------------

function StoreMgr:getGemProductsCount()

   local tbl = self:getGemProducts();

   -- We cannot simply use the length operator (#) on the table because it has strings for keys.

   local count = 0;

   for _, _ in pairs(tbl) do
      count = count + 1;
   end

   return count;

end

----------------------------------------------------------------------

function StoreMgr:getGemProductIdFromIndex( index )

   local tbl = self:getGemProducts();

   if tbl then
      return tbl[index];
   end

end

----------------------------------------------------------------

function StoreMgr:isBusy()

   if not Debug.assertType(self._controller, {"table", "userdata"}, "StoreMgr:isBusy -> self._controller") then
      return;
   end

   return self._controller:isBusy();

end

----------------------------------------------------------------

function StoreMgr:canMakePayments()

   if not Debug.assertType(self._controller, {"table", "userdata"}, "StoreMgr:canMakePayments -> self._controller") then
      return;
   end

   return self._controller:canMakePayments() and App:InAppsAvailable();

end

----------------------------------------------------------------

function StoreMgr:onEnterBackground()

   if not Debug.assertType(self._controller, {"table", "userdata"}, "StoreMgr:canMakePayments -> self._controller") then
      return;
   end

   if self._controller then
      return self._controller:onEnterBackground();
   end

end

----------------------------------------------------------------

function StoreMgr:onEnterForeground()

   if self._controller then
      return self._controller:onEnterForeground();
   end

end

----------------------------------------------------------------

function StoreMgr:checkPurchasedItems()

   if self._controller then
      return self._controller:checkPurchasedItems();
   end

end

----------------------------------------------------------------------

function StoreMgr:getGemCost( id )

   if id then

      local tpl  = TemplateMgr:getStoreGemById( id );

      if tpl then
         return tpl.cost;
      end

   else

      Debug.assert(id, "StoreMgr:getGemCost -> id");

   end

end

----------------------------------------------------------------------

function StoreMgr:getGemAutoBuyId( reqAmount )

   if not Debug.assertType(reqAmount, "number", "StoreMgr:getGemAutoBuyId -> reqAmount") then
      return;
   end

   local needed = reqAmount - App.data:getGemCount();

   -- print("StoreMgr:getGemAutoBuyId - needed gems:", needed);

   -- Consumable screen sometime ask for <= 0 gems
   needed = math.max( needed, 1 );

   if needed <= 0 then
      -- We don't need any gems...
      return;
   end

   local tbl = self:getGemProducts();

   if not Debug.assertType(tbl, "table", "tbl") then
      return;
   end

   -- Iterate through all of our gem products and find the one that will let us spend gems with the last amount left over.

   local bestId  = nil;
   local bestVal = 1000000000000;

   for _, id in pairs(tbl) do

      local amount   = self:getProductAmount(id).gem or 0;
      local leftover = amount - needed;

      -- print("StoreMgr:getGemAutoBuyId - testing amount:", amount, "leftover:", leftover);

      if leftover >= 0 and leftover < bestVal then

         bestVal = leftover;
         bestId  = id;

      end

   end

   return bestId;

end

----------------------------------------------------------------------

function StoreMgr:getStoreProductId( id )

    print("StoreMgr:Trying to get product id for id: " .. id);

   if not Debug.assertType(id, {"number", "string"}, "StoreMgr:getGemProductId -> id") then
      return nil;
   end

   local tpl = TemplateMgr:getProductById(id);

   if tpl then
      if MOAIEnvironment.osBrand == "iOS" then
         return tpl.iOsId;
      elseif AmazonBillingProvider == true then
         return tpl.amazonId;
      else
         return tpl.googlePlayId;
      end
   end

end

----------------------------------------------------------------------

function StoreMgr:getGemTitle( id )

   local result = "";

   if not Debug.assertType(id, {"number", "string"}, "StoreMgr:getGemTitle -> id") then
      return result;
   end

   -- We want to see the number and do some formatting to it
   local reward = self:getProductAmount(id);
   if reward.gem then
      result = getFormattedNumber(reward.gem) .. ' Gems';
   end

   return result;

end

----------------------------------------------------------------------

function StoreMgr:getGemCostDisplay( id )

   local result = "";

   if not Debug.assertType(id, {"number", "string"}, "StoreMgr:getGemCostDisplay -> id") then
      return result;
   end

   local cost = self:getGemCost(id);

   if cost then
      result = '$' .. tostring(cost.dollar);
   end

   return result;

end

----------------------------------------------------------------------

function StoreMgr:buyProduct( id, callObj, callSuccess, callFailure )

   local success = false;

   if not Debug.assertType(id, "string", "StoreMgr:buyProduct -> id") then
      return success;
   end

   if not Debug.assertType(self._controller, {"table", "userdata"}, "StoreMgr:buyProduct -> self._controller") then
      return success;
   end

   success = self._controller:buy( id,
                                   self._needReceipt,
                                   callObj,
                                   callSuccess,
                                   callFailure );

   return success;

end

----------------------------------------------------------------------

-- Returns cost in US dollars.
function StoreMgr:getProductCost( id )

   local cost = { dollar = 0.00 };

   -- Type checking
   if not Debug.assertType(id, {"number", "string"}, "StoreMgr:getProductCost -> id") then
      return cost;
   end

   -- Check gem products
   local tpl  = TemplateMgr:getStoreGemById( id );
   if tpl then
      cost = tpl.cost;
      return cost;
   end

   -- Check special products
   -- TODO Fix this so that it works for more than one special product...
   local abTbl = AbTestMgr:getStarterPackCost();
   if abTbl and type(abTbl) == "table" then

      cost = abTbl;
      return cost;
   else

      local tpl = TemplateMgr:getStoreSpecialById( id );

      if tpl then
         cost = tpl.cost;
         return cost;
      end

   end

   return cost;

end

----------------------------------------------------------------------

function StoreMgr:getProductCostDisplay( id )

   local result = "";

   if not Debug.assertType(id, {"number", "string"}, "StoreMgr:getProductCostDisplay -> id") then
      return result;
   end

   local cost = self:getProductCost(id);

   if cost then
      result = "$" .. tostring(cost.dollar);
   end

   return result;

end

----------------------------------------------------------------------

function StoreMgr:getProductAmount( id )

   local DEFAULT_RESULT = { gem = 0, gold = 0 };

   local result = nil;

   if not Debug.assertType(id, {"number", "string"}, "StoreMgr:getProductAmount -> id") then
      return DEFAULT_RESULT;
   end

   -- Search for gem products
   local abTbl = AbTestMgr:getGemAmountTable();

   if type(abTbl) == "table" and type(abTbl[id]) == "number" then

      result = { gem = abTbl[id], gold = 0 };

   else

      local tpl = TemplateMgr:getStoreGemById(id);

      if type(tpl) == "table" then
         result = tpl.amount;
      end
   end

   -- It was a gem product
   if result then
      return result;
   end

   -- Search for special products
   local abTbl = AbTestMgr:getStarterPackAmount();

   if abTbl and type(abTbl) == "table" then

      result = abTbl;

   else

      local tpl = TemplateMgr:getStoreSpecialById(id);

      --if Debug.assertType(tpl, "table", "StoreMgr:getProductAmount -> tpl : " .. tostring(id)) then
      if type(tpl) == "table" then
         result = tpl.amount;
      end

   end

   -- It was a special product
   if result then
      return result;
   end

   return DEFAULT_RESULT;

end

----------------------------------------------------------------------

function StoreMgr:getProductIdFromName( name )

   Debug.assertType(name, "string", "StoreMgr:getProductIdFromName -> name");

   print("StoreMgr:getProductIdFromName, name = ", name)

   for categoryName, category in pairs(self._productInfo) do

      for productName, productID in pairs(category) do

         if productName == name then

            return productID;
         end

      end

   end

   -- Returns nil if no product is found.

end

----------------------------------------------------------------

function StoreMgr:_onCloudData( data )

   local content = (data or {}).content;


   if not Debug.assertType(content, "table", "StoreMgr:_onCloudData -> content") then
      return;
   end

   self._needReceipt = content.needReceipt;
   self._productInfo = content.products or self._productInfo;

end

----------------------------------------------------------------

function StoreMgr:_createController()

   if MOAIEnvironment.osBrand == "iOS" then

      return AppStoreController:new(self);

   elseif AmazonBillingProvider then

      return AmazonV2StoreController:new(self);

   else

      return GooglePlayV3StoreController:new(self);

   end

end
