require('app/template/template')
require('app/template/explosions/vehicle_explosion')
require("app/template/boss-vehicles/stats_template_boss")
require('app/template/barricade/barricade_01')
require('app/template/barricade/barricade_02')
require('app/template/barricade/barricade_03')
require('app/template/hazard/hazard-missile-boss')

local SIZE_WIDTH_K	= 0.70;
local SIZE_HEIGHT_K	= 0.70;
local BARR_SIZE_K		= 1.3;

local TEMPLATE_DEATH_SFX_A =
{
	start	=	{
		"Explosion1",
		"Explosion2",
		"Explosion3",
	},
};

local TEMPLATE_DEATH_SFX_B =
{
	start	=	{
		"LargeExplosion1",
	},
};

TEMPLATE_BOSS_MINELAYER_01 =
{
   type              = "boss_minelayer_1",
   layer             = 3,
   useLevelSpeed     = true,
   speedMax          = {	10000 ,   0	},
   speedMaxRnd       = {	0 ,   0	},
   avatarSpeedK      =  0.0,
   avatarSpeedKRnd   =  0.0,
   avatarImpulseMinY =  200,
   avatarImpulseMaxY =  300,
   texture           = "resources/Vehicle/Minelayer_Boss/Boss_Minelayer_Red.png",
	splitProp			= false,
	width             = 256*SIZE_WIDTH_K,
   height            = 512*SIZE_HEIGHT_K,
   collisionWidth    = 220*SIZE_WIDTH_K,
   collisionHeight   = 490*SIZE_HEIGHT_K,
	screenClearance	= 225*SIZE_HEIGHT_K,
   mass              = 500*1000,
   score             = TEMPLATE_BASE_BOSS_SCORE["Minelayer"][1],
   hitPoints         = TEMPLATE_BASE_BOSS_HP["Minelayer"][1],
   avatarHitDamage   = { 1/12, 2/12, 3/12, 4/12, 5/12, 6/12, 7/12, 8/12, 9./12, 10/12, 11/12, 12/12 },
   damageToAvatar    = TEMPLATE_BASE_BOSS_RAM_DMG["Minelayer"][1],
	dropIndex         = TEMPLATE_BASE_BOSS_DROPINDEX[1],
   drop              = TEMPLATE_BASE_BOSS_DROP,
   dropSpeedK        = TEMPLATE_BASE_BOSS_DROPSPEEDK,
   damageTypeSound   = {
                     {start={
                              "AutoSkid1",},
                              },
                     {start={
                              "HTTCollision1",},
                              },
                     {},},
   explosion      = TEMPLATE_VEHICLE_EXPLOSION.fuelTruck,
   shadow         =
   {
      layer          = 1,
      width          = 256*SIZE_WIDTH_K,
      height         = 512*SIZE_HEIGHT_K,
      avatarSpeedK   = 0,
      texture        = "resources/Vehicle/Minelayer_Boss/Boss_Minelayer_Shadow.png",
      offset         = {15*SIZE_WIDTH_K,-15*SIZE_HEIGHT_K},
      color          = {1,1,1,0.5},
   },
   healthBar      =
   {
      template    =
      {
         layer          = 2,
         texture        = "resources/square.png",
         width          = 72,
         height         = 12,
         colorFull      = { 0, 1, 0, 0.35 },
         colorHalf      = { 1, 1, 0, 0.35 },
         colorEmpty     = { 1, 0, 0, 0.35 },
      },
      offset      = { 0, -280*SIZE_HEIGHT_K },
   },
   turret  =
   {
		factory			= TurretMine,
      template    =
      {
         layer          = 4,
         texture        = "resources/square.png",
         width          = 1*SIZE_WIDTH_K,
         height         = 1*SIZE_HEIGHT_K,
         collisionWidth = 0,
         collisionHeight= 0,
         shootDelay     = 3.5,
         bulletCount    = 3,
         bulletDelay    = 1.25,
			
			glowVFX			=
			{
				texture		= 'resources/Vehicle/Minelayer_Boss/Minelayer_WarningLight.png',
				width       = 64,
				height      = 64,
				blend			= "add",
			},
			glowTime			= 1.25,
			glowOffset		= {x=0,	y=240*SIZE_HEIGHT_K},
			glowScale		= {w=3,	h=3},
			
			barrOffset		= {x=0,	y=-160*SIZE_HEIGHT_K},
         barricade		=
         {
				decals          =
				{
				},
				
				obstacles          =
				{
					{
						delta          =  { 0 , 0 },
						avatarSpeedK   =  1.0,
						avatarSpeedKRnd=  0.0,
						texture        = "resources/Vehicle/Minelayer_Boss/Mine_Cluster_S.png",
						width          = 55*BARR_SIZE_K,
						height         = 195*BARR_SIZE_K,
						collisionWidth = 55*BARR_SIZE_K,
						collisionHeight= 195*BARR_SIZE_K,
						damageToAvatar = { 1000000, 1000000 },
						explosion      = TEMPLATE_BARRICADE_EXPLOSION.freeway,
						barricade		= true,
					},
					{
						delta          =  { 0 , 70*BARR_SIZE_K},
						blend          = "add",
						spin           = 360,
						avatarSpeedK   =  1.0,
						avatarSpeedKRnd=  0.0,
						texture        = "resources/Vehicle/Minelayer_Boss/Mine_Cluster_RotatingLight.png",
						width          = 64*BARR_SIZE_K,
						height         = 64*BARR_SIZE_K,
						collisionWidth = 0,
						collisionHeight= 0,
						barricade		= true,
					},
					{
						delta          =  { 0 , 0*BARR_SIZE_K},
						blend          = "add",
						spin           = 360,
						avatarSpeedK   =  1.0,
						avatarSpeedKRnd=  0.0,
						texture        = "resources/Vehicle/Minelayer_Boss/Mine_Cluster_RotatingLight.png",
						width          = 64*BARR_SIZE_K,
						height         = 64*BARR_SIZE_K,
						collisionWidth = 0,
						collisionHeight= 0,
						barricade		= true,
					},
					{
						delta          =  { 0 , -70*BARR_SIZE_K},
						blend          = "add",
						spin           = 360,
						avatarSpeedK   =  1.0,
						avatarSpeedKRnd=  0.0,
						texture        = "resources/Vehicle/Minelayer_Boss/Mine_Cluster_RotatingLight.png",
						width          = 64*BARR_SIZE_K,
						height         = 64*BARR_SIZE_K,
						collisionWidth = 0,
						collisionHeight= 0,
						barricade		= true,
					},
				},
         },
			
      },
      offset      = { 0*SIZE_WIDTH_K, 0*SIZE_HEIGHT_K },
   },
	
-- Boss Spawn
	spawnLaneLimit		= {3,		3},
	targetGameSpeedK	= 1.25,
	warningTitle		= 'Minelayer Level 1',
	
-- Boss Strafe
	strafeLaneLimit	= {1,		5},
	strafeDelayLimit	= {0.0,	0.0},
	strafeSpeedMax		= {-1024, 1024},
	strafeAccel			= 1024,
	strafeDeccel		= 1536,
	strafeMinToMax		= true,

-- Boss Death Splosions
	playDeathAnim		= true,
	deathVFX				=
	{
		{
			texture			= 'resources/Explosion/Effect_ExplosionGray.png',
			width				= 64,
			height			= 64,
			avatarSpeedK   = 0.00,
			angleRnd       = 360,
			growth         = { 10, 10 },
			duration       = 1.0,
			spin           = 180,
			spinRnd        = 90,
			color          = { 1.00, 1.00, 1.00, 1.0 },
			colorRnd       = { 1.00, 1.00, 1.00, 0.5 },
			x					= 0*SIZE_WIDTH_K,
			y					= -130*SIZE_HEIGHT_K,
			trigTime			= 0.00,
			endTime			= 0.00,
			sounds			= TEMPLATE_DEATH_SFX_A,
		},
		{
			texture			= 'resources/Explosion/Explosion_04.png',
			width				= 64,
			height			= 64,
			avatarSpeedK   = 0.00,
			angleRnd       = 360,
			growth         = { 8, 8 },
			duration       = 0.5,
			spin           = 180,
			spinRnd        = 90,
			color          = { 0.90, 0.75, 0.00, 1.0 },
			colorRnd       = { 0.10, 0.25, 0.00, 0.5 },
			x					= 0*SIZE_WIDTH_K,
			y					= -130*SIZE_HEIGHT_K,
			trigTime			= 0.00,
			endTime			= 0.00,
		},
		{
			texture			= 'resources/Explosion/Effect_ExplosionGray.png',
			width				= 64,
			height			= 64,
			avatarSpeedK   = 0.00,
			angleRnd       = 360,
			growth         = { 10, 10 },
			duration       = 1.0,
			spin           = 180,
			spinRnd        = 90,
			color          = { 1.00, 1.00, 1.00, 1.0 },
			colorRnd       = { 1.00, 1.00, 1.00, 0.5 },
			x					= 0*SIZE_WIDTH_K,
			y					= 130*SIZE_HEIGHT_K,
			trigTime			= 0.25,
			endTime			= 0.00,
			sounds			= TEMPLATE_DEATH_SFX_A,
		},
		{
			texture			= 'resources/Explosion/Explosion_04.png',
			width				= 64,
			height			= 64,
			avatarSpeedK   = 0.00,
			angleRnd       = 360,
			growth         = { 8, 8 },
			duration       = 0.5,
			spin           = 180,
			spinRnd        = 90,
			color          = { 0.90, 0.75, 0.00, 1.0 },
			colorRnd       = { 0.10, 0.25, 0.00, 0.5 },
			x					= 0*SIZE_WIDTH_K,
			y					= 130*SIZE_HEIGHT_K,
			trigTime			= 0.25,
			endTime			= 0.00,
		},
		{
			texture			= 'resources/Explosion/Explosion_04.png',
			width				= 64,
			height			= 64,
			avatarSpeedK   = 0.00,
			angleRnd       = 360,
			growth         = { 8, 8 },
			duration       = 0.5,
			spin           = 180,
			spinRnd        = 90,
			color          = { 0.90, 0.75, 0.00, 1.0 },
			colorRnd       = { 0.10, 0.25, 0.00, 0.5 },
			x					= -30*SIZE_WIDTH_K,
			y					= -120*SIZE_HEIGHT_K,
			trigTime			= 0.45,
			endTime			= 0.00,
			sounds			= TEMPLATE_DEATH_SFX_A,
		},
		{
			texture			= 'resources/Explosion/Explosion_04.png',
			width				= 64,
			height			= 64,
			avatarSpeedK   = 0.00,
			angleRnd       = 360,
			growth         = { 8, 8 },
			duration       = 0.5,
			spin           = 180,
			spinRnd        = 90,
			color          = { 0.90, 0.75, 0.00, 1.0 },
			colorRnd       = { 0.10, 0.25, 0.00, 0.5 },
			x					= 30*SIZE_WIDTH_K,
			y					= 120*SIZE_HEIGHT_K,
			trigTime			= 0.75,
			endTime			= 0.00,
			sounds			= TEMPLATE_DEATH_SFX_A,
		},
		{
			texture			= 'resources/Explosion/Explosion_04.png',
			width				= 48,
			height			= 48,
			avatarSpeedK   = 0.00,
			angleRnd       = 360,
			growth         = { 8, 8 },
			duration       = 1.35,
			spin           = 180,
			spinRnd        = 90,
			color          = { 0.90, 0.75, 0.00, 1.0 },
			colorRnd       = { 0.10, 0.25, 0.00, 0.5 },
			x					= 0*SIZE_WIDTH_K,
			y					= 0*SIZE_HEIGHT_K,
			trigTime			= 1.10,
			endTime			= 0.00,
			sounds			= TEMPLATE_DEATH_SFX_B,
		},
		{
			texture			= 'resources/Vehicle/Minelayer_Boss/Boss_Minelayer_Red_Chunk03.png',
			width				= 128,
			height			= 128,
			avatarSpeedK   = -0.10,
			angleRnd       = 360,
			growth         = { 1, 1 },
			duration       = 1.00,
			spin           = 180,
			spinRnd        = 90,
			x					= 64*SIZE_WIDTH_K,
			y					= -64*SIZE_HEIGHT_K,
			impulseVector	= { x=-1500,	y=0 },
			trigTime			= 1.10,
			endTime			= 0.00,
			disableParent	= true,
		},
		{
			texture			= 'resources/Vehicle/Minelayer_Boss/Boss_Minelayer_Red_Chunk02.png',
			width				= 128,
			height			= 128,
			avatarSpeedK   = 0.10,
			angleRnd       = 360,
			growth         = { 1, 1 },
			duration       = 1.00,
			spin           = 180,
			spinRnd        = 90,
			x					= -64*SIZE_WIDTH_K,
			y					= 64*SIZE_HEIGHT_K,
			impulseVector	= { x=1500,		y=0 },
			trigTime			= 1.10,
			endTime			= 0.00,
			disableParent	= true,
		},
		{
			texture			= 'resources/Vehicle/Minelayer_Boss/Boss_Minelayer_Red_Chunk03.png',
			width				= 128,
			height			= 128,
			avatarSpeedK   = -1.00,
			angleRnd       = 360,
			growth         = { 1, 1 },
			duration       = 1.00,
			spin           = 180,
			spinRnd        = 90,
			x					= 64*SIZE_WIDTH_K,
			y					= 64*SIZE_HEIGHT_K,
			impulseVector	= { x=-400,	y=0 },
			trigTime			= 1.10,
			endTime			= 0.00,
			disableParent	= true,
		},
		{
			texture			= 'resources/Vehicle/Minelayer_Boss/Boss_Minelayer_Red_Chunk01.png',
			width				= 128,
			height			= 128,
			avatarSpeedK   = 1.00,
			angleRnd       = 360,
			growth         = { 1, 1 },
			duration       = 1.00,
			spin           = 180,
			spinRnd        = 90,
			x					= -64*SIZE_WIDTH_K,
			y					= 64*SIZE_HEIGHT_K,
			impulseVector	= { x=400,		y=0 },
			trigTime			= 1.10,
			endTime			= 0.00,
			disableParent	= true,
		},
		{
			texture			= 'resources/Explosion/Explosion_04.png',
			width				= 96,
			height			= 96,
			avatarSpeedK   = 0.00,
			angleRnd       = 360,
			growth         = { 16, 16 },
			duration       = 1.35,
			spin           = 180,
			spinRnd        = 90,
			x					= 0*SIZE_WIDTH_K,
			y					= 0*SIZE_HEIGHT_K,
			trigTime			= 1.10,
			endTime			= 0.00,
			postEvent		= "vehicle-turret-hide",
		},
		{
			texture			= 'resources/Explosion/Explosion_04.png',
			width				= 64,
			height			= 64,
			avatarSpeedK   = 0.00,
			angleRnd       = 360,
			growth         = { 8, 8 },
			duration       = 0.5,
			spin           = 180,
			spinRnd        = 90,
			color          = { 0.90, 0.75, 0.00, 1.0 },
			colorRnd       = { 0.10, 0.25, 0.00, 0.5 },
			x					= -64*SIZE_WIDTH_K,
			y					= -160*SIZE_HEIGHT_K,
			trigTime			= 1.45,
			endTime			= 0.00,
			sounds			= TEMPLATE_DEATH_SFX_A,
		},
		{
			texture			= 'resources/Explosion/Explosion_04.png',
			width				= 64,
			height			= 64,
			avatarSpeedK   = 0.00,
			angleRnd       = 360,
			growth         = { 8, 8 },
			duration       = 0.5,
			spin           = 180,
			spinRnd        = 90,
			color          = { 0.90, 0.75, 0.00, 1.0 },
			colorRnd       = { 0.10, 0.25, 0.00, 0.5 },
			x					= 64*SIZE_WIDTH_K,
			y					= 160*SIZE_HEIGHT_K,
			trigTime			= 1.65,
			endTime			= 0.00,
			sounds			= TEMPLATE_DEATH_SFX_A,
		},
		{
			texture			= 'resources/Explosion/Explosion_04.png',
			width				= 64,
			height			= 64,
			avatarSpeedK   = 0.00,
			angleRnd       = 360,
			growth         = { 8, 8 },
			duration       = 0.5,
			spin           = 180,
			spinRnd        = 90,
			color          = { 0.90, 0.75, 0.00, 1.0 },
			colorRnd       = { 0.10, 0.25, 0.00, 0.5 },
			x					= 64*SIZE_WIDTH_K,
			y					= -160*SIZE_HEIGHT_K,
			trigTime			= 1.90,
			endTime			= 0.00,
			sounds			= TEMPLATE_DEATH_SFX_A,
		},
		{
			texture			= 'resources/Explosion/Explosion_04.png',
			width				= 64,
			height			= 64,
			avatarSpeedK   = 0.00,
			angleRnd       = 360,
			growth         = { 8, 8 },
			duration       = 0.5,
			spin           = 180,
			spinRnd        = 90,
			color          = { 0.90, 0.75, 0.00, 1.0 },
			colorRnd       = { 0.10, 0.25, 0.00, 0.5 },
			x					= -64*SIZE_WIDTH_K,
			y					= 160*SIZE_HEIGHT_K,
			trigTime			= 2.10,
			endTime			= 2.50,
			sounds			= TEMPLATE_DEATH_SFX_A,
		},
	},
	
-- Boss Barricade
	barrStartTime		= {10, 10},
	barrRepeatDelay	= {10, 10},
	barrSpawnDelay		= {0, 0},
	barrCountLimit		= {0, 0},
	barrLaneLimit		= {true, true, true, true, true},
	barrTemplate		= TEMPLATE_BARRICADE_01,
	barrBossLaneBlock	= true,

-- Boss Missiles
	missStartTime		= {10, 10},
	missRepeatDelay	= {15, 15},
	missSpawnDelay		= {0.25, 0.75},
	missCountLimit		= {1, 1},
	missLaneLimit		= {true, true, true, true, true},
	missProgressTime	= 30,
	missTemplate		= TEMPLATE_BOSS_MISSILE_TEMPLATES[1],
	
};