require('app/template/boss-vehicles/gunship/boss_gunship_common')
require('app/template/boss-vehicles/stats_template_boss')
 
-- Base templates are stored separately from the extended templates
local baseTemplates = {}
local projectileTemplates = {}
local rocketTemplates = {}

local PROJ_SIZE_K    = 0.75;
local PROJ_DMG_K     = 0.5;
local ROCKET_SIZE_K  = 0.95;
local ROCKET_DMG_K   = 1.0;

local rocketBaseTemplate = {
   factory           = Rocket,
   layer             = 6,
   accel             = { 0, 900 },
   speed             = { 0, 300 },
   homing            = true,
   overrideTarget    = false,
   width             = ROCKET_SIZE_K*34,
   height            = ROCKET_SIZE_K*107,
   collisionWidth    = ROCKET_SIZE_K*28,
   collisionHeight   = ROCKET_SIZE_K*109,

   maxTurnRate       = 45,
   turnDelay         = 0.25,
   accelDelay        = 0.0,
   fire = {
      layer       = 5,
      texture     = "resources/Hazard/Missile-Flame-2.png",
      offset      = { 0*ROCKET_SIZE_K, -80*ROCKET_SIZE_K },
      width       = 20*ROCKET_SIZE_K,
      height      = 80*ROCKET_SIZE_K,
      animate			= true,
      launchTime		= 0.5,
      launchScale		= {1.35, -10},
      launchFade		= 0.2,
   },
   hitMarker = {
      texture         = "resources/Explosion/Explosion_04.png",
      width           = 50,
      height          = 50,
      avatarSpeedK    = 0.00,
      growth          = { 15.0, 15.0 },
      growthRnd       = { 0 ,0 },
      maxDamageRadius = 2000,
      duration        = 0.30,
      fadeStart       = 0.20,
      spin            = -180,
      spinRnd         =  360,
      color           = { 1, 0.8, 0.8, 0.85 },
      colorRnd        = { 0, 0.2, 0.2, 0.0 },
      noParent        = true,
   },
}

-- Utility functions
local function getDimensionsFromGunshipSizeEnum(size)
   Debug.assert(size == BOSS_SIZE_MEDIUM, "Invalid size.");

   if size == BOSS_SIZE_MEDIUM then
      return PROJ_SIZE_K*97, PROJ_SIZE_K*185
   end
end

local function getCollisionDimsFromGunshipSizeEnum(size)
   Debug.assert(size == BOSS_SIZE_MEDIUM, "Invalid size.");

   if size == BOSS_SIZE_MEDIUM then
      return PROJ_SIZE_K*28, PROJ_SIZE_K*116
   end
end

-- MAIN factory function
function getGunshipProjectileTemplate(size, color, level)
   local baseKey = size .. "-" .. color

   if not baseTemplates[baseKey] then
      local w, h = getDimensionsFromGunshipSizeEnum(size)
      local cw, ch = getCollisionDimsFromGunshipSizeEnum(size)

      baseTemplates[baseKey] = {
	 speed            = { 0, 900 },
	 width            = w,
	 height           = h,
	 collisionWidth   = cw,
	 collisionHeight  = ch,

	 projDetachTime   = 2/30,
	 hitMarker        = TEMPLATE_BOSS_GUNSHIP_HITMARKER,
      }
   end

   if not projectileTemplates[level] then

      local textureNum = TEMPLATE_BOSS_GUNSHIP_PROJECTILE_TEXTURE[level]
      if textureNum < 10 then
	 textureNum = "0" .. textureNum
      end

      -- print("resources/Vehicle/Gunship_Boss/Boss_Gunship_Bullet" .. textureNum .. ".png")

      projectileTemplates[level] = Template.extent(baseTemplates[baseKey]);
      projectileTemplates[level].damageAvatar = PROJ_DMG_K * TEMPLATE_BASE_BOSS_SHOT_DMG["Gunship"][level]
      projectileTemplates[level].texture = "resources/Vehicle/Gunship_Boss/Boss_Gunship_Bullet" .. textureNum .. ".png"
   end

   return projectileTemplates[level]
end

function getGunshipRocketTemplate(level)
   if not rocketTemplates[level] then

      local textureNum = TEMPLATE_BOSS_GUNSHIP_PROJECTILE_TEXTURE[level]
      if textureNum < 10 then
	 textureNum = "0" .. textureNum
      end

      rocketTemplates[level] = Template.extent(rocketBaseTemplate)
      rocketTemplates[level].damageAvatar = ROCKET_DMG_K * TEMPLATE_BASE_BOSS_SHOT_DMG["Gunship"][level]
      rocketTemplates[level].texture = "resources/Vehicle/Gunship_Boss/Gunship_Rocket_" .. textureNum .. ".png"
   end

   return rocketTemplates[level]
end
