require('app/template/boss-vehicles/gunship/boss_gunship_common')
require('app/template/boss-vehicles/gunship/glow_effect')
require('app/template/boss-vehicles/gunship/turret_shadow')
require('app/template/boss-vehicles/gunship/barrel')
require('app/template/boss-vehicles/gunship/barrel_shadow')
require('app/template/boss-vehicles/gunship/projectile')

require('app/template/barricade/barricade_01')
require('app/template/barricade/barricade_02')
require('app/template/barricade/barricade_03')
require('app/template/hazard/hazard-missile-boss')
require('game/entity/turret/turret-gunship')

local baseTemplates = {}
local turretTemplates = {}

local commonTurretTemplate = {
   factory = TurretBossGunship,
   offset = { 0*BOSS_GUNSHIP_SIZE_WIDTH_K, -185*BOSS_GUNSHIP_SIZE_HEIGHT_K }
}

local actualTemplate = {
   layer = 2,
   texture = "resources/Vehicle/Gunship_Boss/Boss_Gunship_Nosegun_Main.png",
   width = 55*BOSS_GUNSHIP_SIZE_WIDTH_K,
   height = 64*BOSS_GUNSHIP_SIZE_HEIGHT_K,
   pivot = { 0, 0*BOSS_GUNSHIP_SIZE_HEIGHT_K },
   collisionWidth = 0,
   collisionHeight = 0,
   shootAngleMax = 5,
   shootPosMax = 350,
   angleLimit = { min=120, max=240 },
   faceAndShoot = false,

   bodyFaceShoot = TEMPLATE_BOSS_GUNSHIP_BODY_FACE_SHOOT,
   bodyTurnRate = TEMPLATE_BOSS_GUNSHIP_TURN_RATE,
   bodyTurnRateK = TEMPLATE_BOSS_GUNSHIP_TURN_RATE_SCALE,
   bodyAngleLimit = TEMPLATE_BOSS_GUNSHIP_TURN_LIMIT,
   bodyAngleDelta = TEMPLATE_BOSS_GUNSHIP_TURN_ANGLE_DELTA,
   bodyTargetCone = TEMPLATE_BOSS_GUNSHIP_TARGET_FACE_CONE,

   -- shootDelay     = 2.9,
   -- bulletCount    = 2,
   bulletDelay = 4/30,

   launchDelay = 0.5,
   rocketDelay = 24/30,
   -- rocketCount = 2,

   idleAnim = TEMPLATE_BOSS_GUNSHIP_IDLE_ANIM,
   strafeLAnim = TEMPLATE_BOSS_GUNSHIP_STRAFE_LEFT_ANIM,
   strafeLEaseAnim = TEMPLATE_BOSS_GUNSHIP_STRAFE_LEFT_EASE_ANIM,
   strafeRAnim = TEMPLATE_BOSS_GUNSHIP_STRAFE_RIGHT_ANIM,
   strafeREaseAnim = TEMPLATE_BOSS_GUNSHIP_STRAFE_RIGHT_EASE_ANIM,
   idleAnimSpeedK = 4/3;
   strafeAnimSpeedK = 1;
   strafeEaseAnimSpeedK = 2;

   glowGTime = 1.5,
   glowGScale = {w=3.0, h=3.0,},
   glowRTime = 1.5,
   glowRScale = {w=3.0, h=3.0},
   glowOffset = {x=0*BOSS_GUNSHIP_SIZE_WIDTH_K, y=77*BOSS_GUNSHIP_SIZE_HEIGHT_K},

   parentProj = true,
   projOffset = { {x=0*BOSS_GUNSHIP_SIZE_WIDTH_K, y=80*BOSS_GUNSHIP_SIZE_HEIGHT_K, angle=0},	},
   bulletDelta = { {x=0*BOSS_GUNSHIP_SIZE_WIDTH_K, y=-20*BOSS_GUNSHIP_SIZE_HEIGHT_K} },

   recoilGunOffset = 41*BOSS_GUNSHIP_SIZE_HEIGHT_K,
   recoilGunTime = 2/30;
   recoilGunReturnTime = 6/30;

   recoilRocketOffset = 61*BOSS_GUNSHIP_SIZE_HEIGHT_K,
   recoilRocketTime = 3/30;
   recoilRocketReturnTime = 12/30;
   rocketOffset =
      {
	 {x=-63*BOSS_GUNSHIP_SIZE_WIDTH_K, y=80*BOSS_GUNSHIP_SIZE_HEIGHT_K}, {x=63*BOSS_GUNSHIP_SIZE_WIDTH_K, y=80*BOSS_GUNSHIP_SIZE_HEIGHT_K},
	 {x=-63*BOSS_GUNSHIP_SIZE_WIDTH_K, y=80*BOSS_GUNSHIP_SIZE_HEIGHT_K}, {x=63*BOSS_GUNSHIP_SIZE_WIDTH_K, y=80*BOSS_GUNSHIP_SIZE_HEIGHT_K},
	 {x=-63*BOSS_GUNSHIP_SIZE_WIDTH_K, y=80*BOSS_GUNSHIP_SIZE_HEIGHT_K}, {x=63*BOSS_GUNSHIP_SIZE_WIDTH_K, y=80*BOSS_GUNSHIP_SIZE_HEIGHT_K},
      },
   rocketGlowOffset = {{x=-64*BOSS_GUNSHIP_SIZE_WIDTH_K, y=-96*BOSS_GUNSHIP_SIZE_HEIGHT_K}, {x=64*BOSS_GUNSHIP_SIZE_WIDTH_K, y=-96*BOSS_GUNSHIP_SIZE_HEIGHT_K}},

   frontRotor = {
      {
         layer          = 3;
         priority       = 15;
         width          = 98*BOSS_GUNSHIP_SIZE_WIDTH_K,
         height         = 98*BOSS_GUNSHIP_SIZE_HEIGHT_K,
         texture        = "resources/Vehicle/Gunship_Boss/Boss_Gunship_Rotor_Front.png",
         offset         = {-130*BOSS_GUNSHIP_SIZE_WIDTH_K, -17*BOSS_GUNSHIP_SIZE_HEIGHT_K},
      },
      {
         layer          = 3;
         priority       = 15;
         width          = 98*BOSS_GUNSHIP_SIZE_WIDTH_K,
         height         = 98*BOSS_GUNSHIP_SIZE_HEIGHT_K,
         texture        = "resources/Vehicle/Gunship_Boss/Boss_Gunship_Rotor_Front.png",
         offset         = {130*BOSS_GUNSHIP_SIZE_WIDTH_K, -17*BOSS_GUNSHIP_SIZE_HEIGHT_K},
      },
   },
   rearRotor = {
      {
         layer          = 3;
         priority       = 15;
         width          = 71*BOSS_GUNSHIP_SIZE_WIDTH_K,
         height         = 72*BOSS_GUNSHIP_SIZE_HEIGHT_K,
         texture        = "resources/Vehicle/Gunship_Boss/Boss_Gunship_Rotor_Rear.png",
         offset         = {-102*BOSS_GUNSHIP_SIZE_WIDTH_K, 114*BOSS_GUNSHIP_SIZE_HEIGHT_K},
      },
      {
         layer          = 3;
         priority       = 15;
         width          = 71*BOSS_GUNSHIP_SIZE_WIDTH_K,
         height         = 72*BOSS_GUNSHIP_SIZE_HEIGHT_K,
         texture        = "resources/Vehicle/Gunship_Boss/Boss_Gunship_Rotor_Rear.png",
         offset         = {102*BOSS_GUNSHIP_SIZE_WIDTH_K, 114*BOSS_GUNSHIP_SIZE_HEIGHT_K},
      },
   },

   frontRotorWash = {
      {
	 layer          = 2;
	 priority       = 15;
	 width          = 213*BOSS_GUNSHIP_SIZE_WIDTH_K,
	 height         = 211*BOSS_GUNSHIP_SIZE_HEIGHT_K,
	 texture        = "resources/Vehicle/Gunship_Boss/Jetwash.png",
	 offset         = {-130*BOSS_GUNSHIP_SIZE_WIDTH_K, -17*BOSS_GUNSHIP_SIZE_HEIGHT_K},
	 blend          = "add",
      },
      {
	 layer          = 2;
	 priority       = 15;
	 width          = 213*BOSS_GUNSHIP_SIZE_WIDTH_K,
	 height         = 211*BOSS_GUNSHIP_SIZE_HEIGHT_K,
	 texture        = "resources/Vehicle/Gunship_Boss/Jetwash.png",
	 offset         = {130*BOSS_GUNSHIP_SIZE_WIDTH_K, -17*BOSS_GUNSHIP_SIZE_HEIGHT_K},
	 blend          = "add",
      },
   },

   rearRotorWash = {
      {
	 layer          = 2;
	 priority       = 15;
	 width          = 154*BOSS_GUNSHIP_SIZE_WIDTH_K,
	 height         = 153*BOSS_GUNSHIP_SIZE_HEIGHT_K,
	 texture        = "resources/Vehicle/Gunship_Boss/Jetwash.png",
	 offset         = {-102*BOSS_GUNSHIP_SIZE_WIDTH_K, 114*BOSS_GUNSHIP_SIZE_HEIGHT_K},
	 blend          = "add",
      },
      {
	 layer          = 2;
	 priority       = 15;
	 width          = 154*BOSS_GUNSHIP_SIZE_WIDTH_K,
	 height         = 153*BOSS_GUNSHIP_SIZE_HEIGHT_K,
	 texture        = "resources/Vehicle/Gunship_Boss/Jetwash.png",
	 offset         = {102*BOSS_GUNSHIP_SIZE_WIDTH_K, 114*BOSS_GUNSHIP_SIZE_HEIGHT_K},
	 blend          = "add",
      },
   },

   rocketPod = {
      {
         layer          = 3,
         priority       = 11,
         width          = 41*BOSS_GUNSHIP_SIZE_WIDTH_K,
         height         = 81*BOSS_GUNSHIP_SIZE_HEIGHT_K,
         texture        = "resources/Vehicle/Gunship_Boss/Boss_Gunship_Rocketpod.png",
         offset         = {-64*BOSS_GUNSHIP_SIZE_WIDTH_K, -64*BOSS_GUNSHIP_SIZE_HEIGHT_K},
      },
      {
         layer          = 3,
         priority       = 11,
         width          = 41*BOSS_GUNSHIP_SIZE_WIDTH_K,
         height         = 81*BOSS_GUNSHIP_SIZE_HEIGHT_K,
         texture        = "resources/Vehicle/Gunship_Boss/Boss_Gunship_Rocketpod.png",
         offset         = {64*BOSS_GUNSHIP_SIZE_WIDTH_K, -64*BOSS_GUNSHIP_SIZE_HEIGHT_K},
      },
   },
}

-- MAIN factory function
function getGunshipTurretTemplate(size, bodyColor, glowColor, level)
   local baseKey = size .. "-" .. bodyColor .. "-" .. glowColor

   if not baseTemplates[baseKey] then
      local t = Template.extent(commonTurretTemplate)
      local mainTempl = Template.extent(actualTemplate)

      mainTempl.glowVFX = getGunshipGlowEffectTemplate(size, glowColor)
      mainTempl.turretShadow = getGunshipTurretShadowTemplate(size)
      mainTempl.barrel = getGunshipBarrelTemplate(size, bodyColor)
      mainTempl.barrelShadow = getGunshipBarrelShadowTemplate(size)

      mainTempl.maxTurnRate = TEMPLATE_BASE_BOSS_TURRET_MAX_TURN_RATE["Gunship"]
      mainTempl.bulletDelay = TEMPLATE_BASE_BOSS_TURRET_BULLET_DELAY["Gunship"]

      t.template = mainTempl
      baseTemplates[baseKey] = t
   end

   if not turretTemplates[level] then

      turretTemplates[level] = Template.extent(baseTemplates[baseKey])

      local t = Template.extent(turretTemplates[level].template)

      t.projectile = getGunshipProjectileTemplate(size, glowColor, level)
      t.rocket = getGunshipRocketTemplate(level)
      t.muzzleVFX = getGunshipMuzzleVFXTemplate(size, glowColor)

		t.shootDelay = TEMPLATE_BASE_BOSS_TURRET_SHOOT_DELAY["Gunship"][math.clamp(level, 1, #TEMPLATE_BASE_BOSS_TURRET_SHOOT_DELAY["Gunship"])]
      t.bulletCount = TEMPLATE_BASE_BOSS_TURRET_BULLET_COUNT["Gunship"][math.clamp(level, 1, #TEMPLATE_BASE_BOSS_TURRET_BULLET_COUNT["Gunship"])]
      t.rocketCount = TEMPLATE_BASE_BOSS_TURRET_ROCKET_COUNT["Gunship"][math.clamp(level, 1, #TEMPLATE_BASE_BOSS_TURRET_ROCKET_COUNT["Gunship"])]

      turretTemplates[level].template = t
   end

   return turretTemplates[level]
end
