require('app/template/template')

require('app/template/boss-vehicles/tank/boss_tank_common')
require("app/template/boss-vehicles/stats_template_boss")
require('app/template/hazard/hazard-missile-boss')

require('game/entity/turret/turret-tank')

require('app/template/boss-vehicles/tank/turret')
require('app/template/boss-vehicles/tank/tank_shadow')
require('app/template/boss-vehicles/tank/tank_healthbar')
require('app/template/boss-vehicles/tank/tank_death_vfx')
require('app/template/boss-vehicles/tank/tank_armor')

function createTankBossTemplate(level)
   local t = Template.extent(TEMPLATE_BOSS_TANK_BASE_TEMPLATE)

   local BOSS_SIZE  = BOSS_SIZE_MEDIUM
   local BOSS_COLOR = TEMPLATE_BASE_BOSS_COLOR["Tank"][math.clamp(level, 1, #TEMPLATE_BASE_BOSS_COLOR["Tank"])]
   local BOSS_PROJECTILE_COLOR = TEMPLATE_BASE_BOSS_PROJECTILE_COLOR["Tank"][math.clamp(level, 1, #TEMPLATE_BASE_BOSS_PROJECTILE_COLOR["Tank"])]

   local newData = { 
      type = "boss_tank_" .. level,
      score             = TEMPLATE_BASE_BOSS_SCORE["Tank"][level],
      hitPoints         = TEMPLATE_BASE_BOSS_HP["Tank"][level],
      damageToAvatar    = TEMPLATE_BASE_BOSS_RAM_DMG["Tank"],
      dropIndex         = TEMPLATE_BASE_BOSS_DROPINDEX[math.clamp(level, 1, #TEMPLATE_BASE_BOSS_DROPINDEX)],
      missTemplate      = TEMPLATE_BOSS_MISSILE_TEMPLATES[1], -- The 1 is intentional
      strafeDelayLimit  = TEMPLATE_BASE_BOSS_STRAFE_DELAY_LIMIT["Tank"][math.clamp(level, 1, #TEMPLATE_BASE_BOSS_STRAFE_DELAY_LIMIT["Tank"])],
      strafeSpeedMax    = TEMPLATE_BASE_BOSS_STRAFE_SPEED_MAX["Tank"][math.clamp(level, 1, #TEMPLATE_BASE_BOSS_STRAFE_SPEED_MAX["Tank"])],
      strafeAccel       = TEMPLATE_BASE_BOSS_STRAFE_ACCEL["Tank"][math.clamp(level, 1, #TEMPLATE_BASE_BOSS_STRAFE_ACCEL["Tank"])],
      strafeDeccel      = TEMPLATE_BASE_BOSS_STRAFE_DECCEL["Tank"][math.clamp(level, 1, #TEMPLATE_BASE_BOSS_STRAFE_DECCEL["Tank"])],

      armor = getTankArmorTemplate(level),
      texture = "resources/Vehicle/Tank_Boss/Boss_Tank_" .. getStringFromBossColor(BOSS_COLOR) .. ".png",
      deathVFX = getTankVFXTemplate(BOSS_SIZE, BOSS_COLOR),
      shadow = getTankShadowTemplate(BOSS_SIZE),
      healthBar = getTankHealthbarTemplate(BOSS_SIZE),
      turret = getTankTurretTemplate(BOSS_SIZE, BOSS_COLOR, BOSS_PROJECTILE_COLOR, level),
   }

   Template.addDataFromMap(t, newData)

   return t
end

local function createAllTankBosses()
   for i = 1, #TEMPLATE_BASE_BOSS_HP["Tank"] do
      
      -- Prepend a 0 when less than 10
      local digits = tostring(i)
      if i < 10 then
	 digits = "0" .. digits
      end

      local templateName = "TEMPLATE_BOSS_TANK_" .. digits
      _G[templateName] = createTankBossTemplate(i)

      -- DEBUG:
      -- print("Template name: " .. templateName)
   end
end

createAllTankBosses()
