require('app/template/boss-vehicles/tank/boss_tank_common')
require('app/template/boss-vehicles/stats_template_boss')
 
-- Base templates are stored separately from the extended templates
local baseTemplates = {}
local projectileTemplates = {}

local PROJ_SIZE_K = 1.0

-- Utility functions
local function getDimensionsFromTankSizeEnum(size)
   Debug.assert(size == BOSS_SIZE_MEDIUM, "Invalid size.");

   if size == BOSS_SIZE_MEDIUM then
      return PROJ_SIZE_K*64, PROJ_SIZE_K*128
   end
end

local function getCollisionDimsFromTankSizeEnum(size)
   Debug.assert(size == BOSS_SIZE_MEDIUM, "Invalid size.");

   if size == BOSS_SIZE_MEDIUM then
      return PROJ_SIZE_K*15, PROJ_SIZE_K*70
   end
end

-- MAIN factory function
function getProjectileTemplate(size, color, level)
   local baseKey = size .. "-" .. color

   if not baseTemplates[baseKey] then
      local w, h = getDimensionsFromTankSizeEnum(size)
      local cw, ch = getCollisionDimsFromTankSizeEnum(size)
      baseTemplates[baseKey] = {
         speed            = { 0, 900 },
         width            = w,
         height           = h,
         collisionWidth   = cw,
         collisionHeight  = ch,
         texture          = 'resources/Vehicle/Tank_Boss/TankBullets/Tank_Projectile_' .. getStringFromBossColor(color) .. '.png',
         projDetachTime   = 2/30,
         hitMarker        = TEMPLATE_BOSS_TANK_HITMARKER,
         fire					=
         {
            layer				= 5,
            texture			= "resources/Hazard/smoke_trail.png",
            offset			= { 0*PROJ_SIZE_K, -50*PROJ_SIZE_K },
            width				= 32*PROJ_SIZE_K,
            height			= 128*PROJ_SIZE_K,
            animate			= true,
            launchTime		= 0.5,
            launchScale		= {1.35, 10},
            launchFade		= 0.2,
         },
      }
   end

   if not projectileTemplates[level] then
      projectileTemplates[level] = Template.extent(baseTemplates[baseKey]);
      projectileTemplates[level].damageAvatar = TEMPLATE_BASE_BOSS_SHOT_DMG["Tank"][level]
   end

   return projectileTemplates[level]
end
