require('app/template/boss-vehicles/tank/boss_tank_common')
require('app/template/boss-vehicles/tank/tank_armor')
require('app/template/boss-vehicles/tank/glow_effect')
require('app/template/boss-vehicles/tank/barrel')
require('app/template/boss-vehicles/tank/turret_shadow')
require('app/template/boss-vehicles/tank/barrel_shadow')
require('app/template/boss-vehicles/tank/projectile')

local turretTemplates = {}
local baseTemplates = {}

-- All tank bosses share these properties.
-- Modifying these two tables will modify the property for every boss

local actualTemplate = {
   layer          = 4,
   width          = 256*BOSS_TANK_SIZE_WIDTH_K,
   height         = 512*BOSS_TANK_SIZE_HEIGHT_K,
   pivot          = { 0, -69*BOSS_TANK_SIZE_HEIGHT_K },
   collisionWidth = 0,
   collisionHeight= 0,
   shootAngleMax  = 5,
   angleLimit     = { min=120, max=240 },
   faceAndShoot   = false,
   shootPosMax    = 350,

   bodyTurnStrafe	= TEMPLATE_BOSS_TANK_TURN_STRAFE,
   bodyTurnRate	= TEMPLATE_BOSS_TANK_TURN_RATE,
   bodyAngleLimit	= TEMPLATE_BOSS_TANK_TURN_LIMIT,

   glowTime = 1.5,
   glowOffset = {x=0, y=45*BOSS_TANK_SIZE_HEIGHT_K},
   glowScale = {w=3, h=3},

   parentProj		= true,
   projOffset		= {{x=0,	y=144*BOSS_TANK_SIZE_HEIGHT_K,	angle=0},},

   recoilOffset		= 70*BOSS_TANK_SIZE_HEIGHT_K,
   recoilTime			= 3/30,
   recoilReturnTime	= 12/30,
   
   muzzleBlastVFX    =
   {
      texture        = "resources/Explosion/Explosion_04.png",
      width          = 100,
      height         = 100,
      growth         = { 0, 0 };
      duration       = 15/30,
      spin           = 0,
      avatarSpeedK   =  0.00,
      sounds       = {start={"PlayerDeath",},},
      offset         = {0, 135},
      flashAnim      =
      {
         priority    = 10,
         flashId     = "resources/flash-resource-bundle/Drone_Effects",
         animId      = "Tank_Fire",
         offset      = {0, 0},
         rotOffset   = 0,
         scale       = {1, -1}, --must invert vertical due to animation being inverted
         fps         = 30,
         speed       = 1,
      },
   },
}

local commonTurretTemplate = {
   factory = TurretTank,
   offset = { 0*BOSS_TANK_SIZE_WIDTH_K, 0*BOSS_TANK_SIZE_HEIGHT_K },
}

-- MAIN factory function
-- Tables retrieved from this function are guaranteed to be unique if they have different levels
-- i.e. level is the unique id
-- Multiple requests using the same level but updated size/color will not create a new template.
function getTankTurretTemplate(size, bodyColor, glowColor, level)
   local baseKey = size .. "-" .. bodyColor .. "-" .. glowColor

   if not baseTemplates[baseKey] then
      local t = Template.extent(commonTurretTemplate)
      local mainTempl = Template.extent(actualTemplate)

      -- Create a template with the appropriate size and color
      mainTempl.texture = "resources/Vehicle/Tank_Boss/Boss_Tank_" .. getStringFromBossColor(bodyColor) .. "_Turret.png"
      mainTempl.glowVFX = getGlowEffectTemplate(size, 5) --set to 5 ("Orange") for all tanks
      mainTempl.turretShadow = getTurretShadowTemplate(size)
      mainTempl.barrel = getBarrelTemplate(size, bodyColor)
      mainTempl.barrelShadow = getBarrelShadowTemplate(size)

      t.template = mainTempl

      baseTemplates[baseKey] = t
   end

   -- Create a template with the appropriate per-level data
   if not turretTemplates[level] then
      turretTemplates[level] = Template.extent(baseTemplates[baseKey])

      local t = Template.extent(turretTemplates[level].template)

      t.projectile = getProjectileTemplate(size, glowColor, level)
		t.maxTurnRate = TEMPLATE_BASE_BOSS_TURRET_MAX_TURN_RATE["Tank"][math.clamp(level, 1, #TEMPLATE_BASE_BOSS_TURRET_MAX_TURN_RATE["Tank"])]
      t.shootDelay = TEMPLATE_BASE_BOSS_TURRET_SHOOT_DELAY["Tank"][math.clamp(level, 1, #TEMPLATE_BASE_BOSS_TURRET_SHOOT_DELAY["Tank"])]
      t.bulletCount = TEMPLATE_BASE_BOSS_TURRET_BULLET_COUNT["Tank"][math.clamp(level, 1, #TEMPLATE_BASE_BOSS_TURRET_BULLET_COUNT["Tank"])]
      t.bulletDelay = TEMPLATE_BASE_BOSS_TURRET_BULLET_DELAY["Tank"][math.clamp(level, 1, #TEMPLATE_BASE_BOSS_TURRET_BULLET_DELAY["Tank"])]

      t.armor = getTankTurretArmorTemplate(level)

      turretTemplates[level].template = t
   end

   return turretTemplates[level]
end
