require("app/template/template");
require("app/template/explosions/vehicle_explosion");
require("game/entity/drone/drone-barricade");

local SIZE_WIDTH_K	= 1.0;
local SIZE_HEIGHT_K	= 1.0;
local SHDW_WIDTH_K	= 0.75;
local SHDW_HEIGHT_K	= 0.75;
local PROJ_WIDTH_K	= 0.75;
local PROJ_HEIGHT_K	= 0.75;
local PROJ_DMG_K     = 1.75;

local drone_barricade_base =
{
	factory				= DroneBarricade,
	layer					= 5,
	priority				= 5,
	texture				= "resources/square.png", --data overwritten by DroneMgr on creation
	width					= 90*SIZE_WIDTH_K,
	height				= 90*SIZE_HEIGHT_K,
	splitProp			= true;
--	pivot					= { 0, 0 },
	collisionWidth		= 0,
	collisionHeight	= 0,
	offset				= { -128, 32 }, --default x offset to left mount point
   modOffset			= { 0, 128 },
	hitPoints         = 100, --data overwritten by DroneMgr on creation
	speedMin				= { 256,	142 }, --x,y pixels per sec
	speedMax				= { 960,	1138 }, --x,y pixels per sec
	shootDelay			= 0,
	bulletCount			= 15,
	bulletDelay			= 5/30,
	bulletDelta			= { { x=0*SIZE_WIDTH_K, y=0*SIZE_HEIGHT_K} },
	explosion			= TEMPLATE_VEHICLE_EXPLOSION.small,
	chargeThreshold	= 40, --data overwritten by DroneMgr on creation
   maxAbilityTime    = 2.2,
   maxGraceTime      = 0.8,
   maxActivationX    = 192,

	shadow				=
   {
       layer			= 3,
       priority		= 100,
       width			= 80*SHDW_WIDTH_K,
       height			= 80*SHDW_HEIGHT_K,
       avatarSpeedK	= 0,
       texture			= "resources/Vehicle/Drones/DroneBarricade/DroneBarricade_Shadow.png",
       offset			= {32,-32},
       color			= {1,1,1,0.40},
   },

	pip_on				=
	{
		layer				= 5,
		priority			= 10,
		texture        = "resources/UI/Drone_UI/dronepip_on.png",
		width          = 24*SIZE_WIDTH_K,
		height         = 24*SIZE_HEIGHT_K,
		scale				= { 1, 1 }, --data overwritten by DroneMgr on creation
		color          = { 1, 1, 1, 1 }, --data overwritten by DroneMgr on creation
		offset			=
		{
			{ -30*SIZE_WIDTH_K,	-60*SIZE_HEIGHT_K },
			{ 0*SIZE_WIDTH_K,		-60*SIZE_HEIGHT_K },
			{ 30*SIZE_WIDTH_K,	-60*SIZE_HEIGHT_K },
		},
	},

	pip_off				=
	{
		layer				= 5,
		priority			= 10,
		texture        = "resources/UI/Drone_UI/dronepip_off.png",
		width          = 24*SIZE_WIDTH_K,
		height         = 24*SIZE_HEIGHT_K,
		scale				= { 1, 1 }, --data overwritten by DroneMgr on creation
		color          = { 1, 1, 1, 1 }, --data overwritten by DroneMgr on creation
		offset			=
		{
			{ -30*SIZE_WIDTH_K,	-60*SIZE_HEIGHT_K },
			{ 0*SIZE_WIDTH_K,		-60*SIZE_HEIGHT_K },
			{ 30*SIZE_WIDTH_K,	-60*SIZE_HEIGHT_K },
		},
	},

	beam              =
	{
		layer				= 5,
		priority			= 1,
		texture			= "resources/square.png", --data overwritten by DroneMgr on creation
		width				= PROJ_WIDTH_K, --data overwritten by DroneMgr on creation
		height			= 3456,
		offset			= { SIZE_WIDTH_K, SIZE_HEIGHT_K },
		color				= {1,1,1, 0.8},
      --blend        = "add",
      maxLengthK     = 1,
	},

   muzzleGlow     =
   {
      layer				= 5,
      priority       = 10,
      flashId        = "resources/flash-resource-bundle/Drone_Effects",
      animId         = "charging_mvc_yellow",
      offset         = { SIZE_WIDTH_K, SIZE_HEIGHT_K },
      rotOffset      = 0,
      scale          = {0.75, 0.75},
      fps            = 30,
      speed          = 1,
      attrLink       = "location-only",
   },

	projectile			=
	{
      layer             = 5,
      priority          = 6,
      speed             = { 0, 2000 },
      width             = 192,
      height            = 128,
      collisionWidth    = 192,
      collisionHeight   = 128,
		blend             = "add",
      damage            = PROJ_DMG_K,
      damageBarricade   = 100,
      invicible         = true,
		noHitRepeat       = true,
		texture				= "resources/square.png", --data overwritten by DroneMgr on creation
      maxLifeTime       = 1, --data overwritten by DroneMgr on creation
      hitMarker			=
      {
         texture        = "resources/square.png", --data overwritten by DroneMgr on creation
         width          = 32,
         height         = 32,
         growth         = { 10, 10 };
         duration       = 0.5,
         fadeStart		= 0.2,
         spin           = 360,
         avatarSpeedK   = 0.00,
         --blend          = "add",
      },
	},
};

return drone_barricade_base;
