require("app/template/template");
require("app/template/explosions/vehicle_explosion");
require("game/entity/drone/drone-boost");
require("game/entity/powerup/powerupDash");

local SIZE_WIDTH_K	= 1.0;
local SIZE_HEIGHT_K	= 1.0;
local SHDW_WIDTH_K	= 0.75;
local SHDW_HEIGHT_K	= 0.75;

local drone_boost_base =
{
	factory				= DroneBoost,
	layer					= 5,
	priority				= 0,
	texture				= "resources/square.png", --data overwritten by DroneMgr on creation
	width					= 100*SIZE_WIDTH_K,
	height				= 100*SIZE_HEIGHT_K,
	splitProp			= true;
--	pivot					= { 0, 0 },
	collisionWidth		= 0,
	collisionHeight	= 0,
	offset				= { -128, 32 }, --default x offset to left mount point
	hitPoints         = 100, --data overwritten by DroneMgr on creation
	speedMin				= { 256,	142 }, --x,y pixels per sec
	speedMax				= { 960,	1138 }, --x,y pixels per sec
	explosion			= TEMPLATE_VEHICLE_EXPLOSION.small,
	respawnCDRange		= { 0, 0.25 },
	chargeThreshold	= 40, --data overwritten by DroneMgr on creation
	abilityCooldown	= 5,
	initTime				= 1.65,

	shadow				=
	 {
		 layer			= 3,
		 priority		= 100,
		 width			= 80*SHDW_WIDTH_K,
		 height			= 80*SHDW_HEIGHT_K,
		 avatarSpeedK	= 0,
		 texture			= "resources/Vehicle/Drones/DroneBoost/DroneBoost_Shadow.png",
		 offset			= {22,-42},
		 color			= {1,1,1,0.40},
	 },

	vfx					=
	{
		layer				= 5,
		priority			= 10,
		texture        =
		{
			"resources/Vehicle/Drones/DroneBoost/chev-1.png",
			"resources/Vehicle/Drones/DroneBoost/chev-2.png",
			"resources/Vehicle/Drones/DroneBoost/chev-3.png",
			"resources/Vehicle/Drones/DroneBoost/chev-4.png",
		},
		width          = 54*SIZE_WIDTH_K,
		height         = 26*SIZE_HEIGHT_K,
		scale				= { 1, 1 }, --data overwritten by DroneMgr on creation
		color          = { 1, 1, 1, 1 }, --data overwritten by DroneMgr on creation
		offset			=
		{
			{ 0*SIZE_WIDTH_K, -14*SIZE_HEIGHT_K },
			{ 0*SIZE_WIDTH_K, 0*SIZE_HEIGHT_K },
			{ 0*SIZE_WIDTH_K, 12*SIZE_HEIGHT_K },
			{ 0*SIZE_WIDTH_K, 25*SIZE_HEIGHT_K },
		},
	},

	readyvfx				=
	{
		layer				= 4,
		priority			= 10,
		texture        = "resources/Vehicle/Drones/DroneBoost/aura_field.png",
		width          = 512*SIZE_WIDTH_K,
		height         = 512*SIZE_HEIGHT_K,
		scale				= { 1, 1 }, --data overwritten by DroneMgr on creation
		color          = { 1, 1, 1, 1 }, --data overwritten by DroneMgr on creation
		offset			= { 0*SIZE_WIDTH_K, 0*SIZE_HEIGHT_K },
		blend          = "add",
	},

	pip_on				=
	{
		layer				= 5,
		priority			= 10,
		texture        = "resources/UI/Drone_UI/dronepip_on.png",
		width          = 24*SIZE_WIDTH_K,
		height         = 24*SIZE_HEIGHT_K,
		scale				= { 1, 1 }, --data overwritten by DroneMgr on creation
		color          = { 1, 1, 1, 1 }, --data overwritten by DroneMgr on creation
		offset			=
		{
			{ -30*SIZE_WIDTH_K,	-60*SIZE_HEIGHT_K },
			{ 0*SIZE_WIDTH_K,		-60*SIZE_HEIGHT_K },
			{ 30*SIZE_WIDTH_K,	-60*SIZE_HEIGHT_K },
		},
	},

	pip_off				=
	{
		layer				= 5,
		priority			= 10,
		texture        = "resources/UI/Drone_UI/dronepip_off.png",
		width          = 24*SIZE_WIDTH_K,
		height         = 24*SIZE_HEIGHT_K,
		scale				= { 1, 1 }, --data overwritten by DroneMgr on creation
		color          = { 1, 1, 1, 1 }, --data overwritten by DroneMgr on creation
		offset			=
		{
			{ -30*SIZE_WIDTH_K,	-60*SIZE_HEIGHT_K },
			{ 0*SIZE_WIDTH_K,		-60*SIZE_HEIGHT_K },
			{ 30*SIZE_WIDTH_K,	-60*SIZE_HEIGHT_K },
		},
	},

	tplPowerup			=
   {
		factory				= PowerupDash;
		type					= "powerup_speed_burst",
		duration				= 3.00, --data overwritten by DroneMgr on creation
		moreTimeScalar		= 1.33,
		warningDuration	= 1.50,
		warningCount		= 2,
		warningColor		= { 1, 1, 0, 1.00 },
		warningScale		= 2.00,
		texture				= "resources/Powerup/Effect_Boost.png",
		offset				= { 0, -10 },
		width					= 382/1.25,
		height				= 613/1.25,
		massBurst			= 5000,
		speedBurst			= 2000,
		layer					= 1,
		invicible			= true,
		disableBoostPickup	= false,
		sounds				= {start={
											"BoostPickup1",
											},},
		tplProjectile  =
		{
			speedBurst        = true,
			damage            = 10000000,
			invicible         = true,
			speed             =  { 0, 2000 },
			width             = 382/1.25*2,
			height            = 613/1.25*2,
			color             = { 1, 1, 0, 1.00 },
			texture           = "resources/Powerup/Effect_Boost.png",
		}
   }
};

return drone_boost_base;
