require("app/template/template");
require("app/template/explosions/vehicle_explosion");
require("game/entity/drone/drone-laser");

local SIZE_WIDTH_K	= 1.0;
local SIZE_HEIGHT_K	= 1.0;
local SHDW_WIDTH_K	= 0.75;
local SHDW_HEIGHT_K	= 0.75;
local PROJ_WIDTH_K	= 1.0;
local PROJ_HEIGHT_K	= 1.0;
local PROJ_DMG_K		= 1.0;

local drone_laser_base =
{
	factory				= DroneLaser,
	layer					= 5,
	priority				= 0,
	texture				= "resources/square.png", --data overwritten by DroneMgr on creation
	width					= 100*SIZE_WIDTH_K,
	height				= 100*SIZE_HEIGHT_K,
	splitProp			= true;
--	pivot					= { 0, 0 },
	collisionWidth		= 0,
	collisionHeight	= 0,
	offset				= { -128, 32 }, --default x offset to left mount point
	hitPoints         = 100, --data overwritten by DroneMgr on creation
	speedMin				= { 256,	142 }, --x,y pixels per sec
	speedMax				= { 960,	1138 }, --x,y pixels per sec
	speedRot				= { 90,	720 }, --min, max degrees per sec
	shootDelay			= 1.5,
	bulletCount			= 2,	--data overwritten by DroneMgr on creation
	bulletDelay			= 5/30,
	bulletDelta			= { { x=0*SIZE_WIDTH_K, y=-50*SIZE_HEIGHT_K} },
	mainBeamDuration	= 4/3, --data overwritten by DroneMgr on creation
	parentProj			= true,
	explosion			= TEMPLATE_VEHICLE_EXPLOSION.small,
	respawnCDRange		= { 0, 0.25 },

	shadow				=
	 {
		 layer			= 3,
		 priority		= 100,
		 width			= 80*SHDW_WIDTH_K,
		 height			= 80*SHDW_HEIGHT_K,
		 avatarSpeedK	= 0,
		 texture			= "resources/Vehicle/Drones/DroneLaser/DroneLaser_Shadow.png",
		 offset			= {22,-42},
		 color			= {1,1,1,0.40},
	 },

	beamMinor			=
	{
		layer				= 5,
		priority			= 5,
		texture			= "resources/square.png", --data overwritten by DroneMgr on creation
		width				= 16*PROJ_WIDTH_K,
		height			= 2304,
		offset			=
		{
			{ 0*SIZE_WIDTH_K, 0*SIZE_HEIGHT_K, 2.5 },
			{ 0*SIZE_WIDTH_K, 0*SIZE_HEIGHT_K, -2.5 },
		},	--data overwritten by DroneMgr on creation
		color				= {1,1,1,1},
		cooldown			= 1.0;
		pulseFreq		= 1.0;
		pulseWidthK		= 0.4;
		pulseAlpha		= 0.6;
	},

	beamMajor			=
	{
		layer				= 5,
		priority			= 5,
		texture			= "resources/square.png", --data overwritten by DroneMgr on creation
		width				= PROJ_WIDTH_K,
		height			= 3456,
		offset			= { 0*SIZE_WIDTH_K, 40*SIZE_HEIGHT_K },
		color				= {1,1,1,1},
		cooldown			= 1.0;
		pulseFreq		= 1.0;
		pulseWidthK		= 0.4;
		pulseAlpha		= 0.6;
	},

	muzzleGlow			=
	{
		priority			= 10,
		width          = 32*SIZE_WIDTH_K,
		height         = 32*SIZE_HEIGHT_K,
		texture        = "resources/square.png", --data overwritten by DroneMgr on creation
		offset			= {0*SIZE_WIDTH_K,	40*SIZE_HEIGHT_K},
		scale				= {3,	3},
	},

	muzzleSpark			=
	{
		priority			= 15,
		width          = 64*SIZE_WIDTH_K,
		height         = 64*SIZE_HEIGHT_K,
		texture        = "resources/Vehicle/Drones/DroneLaser/DroneLaser_Spark.png",
		offset			= {0*SIZE_WIDTH_K,	40*SIZE_HEIGHT_K},
		scale				= {3,	3},
	},

	muzzleFlash			=
	{
		priority			= 15,
		width          = 32*SIZE_WIDTH_K,
		height         = 8*SIZE_HEIGHT_K,
		texture        = "resources/square.png", --data overwritten by DroneMgr on creation
		offset			=
		{
			{0*SIZE_WIDTH_K,	45*SIZE_HEIGHT_K},
			{0*SIZE_WIDTH_K,	60*SIZE_HEIGHT_K},
		},
		scale				= {2/3,	4}, --data overwritten by DroneMgr on creation
	},

	projectile				=
	{
		speed					= { 0, 3000 },
      width             = 27*PROJ_WIDTH_K,
      height            = 32*PROJ_HEIGHT_K,
      collisionWidth    = 27*PROJ_WIDTH_K,
      collisionHeight   = 32*PROJ_HEIGHT_K,
		damage				= PROJ_DMG_K, --data overwritten by DroneMgr on creation
		texture				= "resources/square.png", --data overwritten by DroneMgr on creation
		blend					= "add",
		invicible			= true,
		noHitRepeat			= true,
		hitMarker			=
		{
			texture        = "resources/square.png", --data overwritten by DroneMgr on creation,
			width          = 32,
			height         = 32,
			growth         = { 10, 10 };
			duration       = 0.5,
			fadeStart		= 0.2,
			spin           = 360,
			avatarSpeedK   = 0.00,
			--blend          = "add",
		},
	},
};

return drone_laser_base;
