require("app/template/template");
require("app/template/explosions/vehicle_explosion");
require("game/entity/drone/drone-missile");

local SIZE_WIDTH_K	= 1.0;
local SIZE_HEIGHT_K	= 1.0;
local SHDW_WIDTH_K	= 0.75;
local SHDW_HEIGHT_K	= 0.75;
local PROJ_WIDTH_K	= 0.75;
local PROJ_HEIGHT_K	= 0.75;
local PROJ_DMG_K		= 1.0;

local drone_missile_base =
{
	factory				= DroneMissile,
	layer					= 5,
	priority				= 0,
	texture				= "resources/square.png", --data overwritten by DroneMgr on creation
	width					= 1*SIZE_WIDTH_K,
	height				= 1*SIZE_HEIGHT_K,
	splitProp			= true;
--	pivot					= { 0, 0 },
	collisionWidth		= 0,
	collisionHeight	= 0,
	offset				= { -96, 32 }, --default x offset to left mount point
	hitPoints         = 100, --data overwritten by DroneMgr on creation
	speedMin				= { 256,	142 }, --x,y pixels per sec
	speedMax				= { 960,	1138 }, --x,y pixels per sec
	shootDelay			= 1,
	bulletCount			= 3, --data overwritten by DroneMgr on creation
	bulletDelay			= 5/30,
	bulletDelta			=	--data overwritten by DroneMgr on creation
	{
		{	x= -40*SIZE_WIDTH_K,	y= 0*SIZE_HEIGHT_K	},
		{	x= -80*SIZE_WIDTH_K,	y= -60*SIZE_HEIGHT_K	},
		{	x= 0*SIZE_WIDTH_K,	y= -60*SIZE_HEIGHT_K	},
	},
	explosion			= TEMPLATE_VEHICLE_EXPLOSION.small,
	respawnCDRange		= { 1.0,	1.3 },

	proxyShadow				=
	{
		 layer			= 3,
		 priority		= 100,
		 width			= 100*SHDW_WIDTH_K,
		 height			= 100*SHDW_HEIGHT_K,
		 avatarSpeedK	= 0,
		 texture			= "resources/Vehicle/Drones/DroneMissile/DroneMissile_Shadow.png",
		 offset			= {22,-42},
		 color			= {1,1,1,0.40},
	},

	proxy					=
	{
		layer				= 5,
		priority			= 10,
		width          = 100*SIZE_WIDTH_K,
		height         = 100*SIZE_HEIGHT_K,
		texture        = "resources/square.png", --data overwritten by DroneMgr on creation
		offset			=	--data overwritten by DroneMgr on creation
		{
			{	-40*SIZE_WIDTH_K,		0*SIZE_HEIGHT_K	},
			{	-80*SIZE_WIDTH_K,		-60*SIZE_HEIGHT_K	},
			{	0*SIZE_WIDTH_K,		-60*SIZE_HEIGHT_K	},
		},
	},

	vfx					=
	{
		layer				= 5,
		priority			= 0,
		width          = 32*SIZE_WIDTH_K,
		height         = 64*SIZE_HEIGHT_K,
		texture        = "resources/square.png", --data overwritten by DroneMgr on creation
		offset			= {0*SIZE_WIDTH_K,	-32*SIZE_HEIGHT_K},
		scale				= {3.5,	3.5},
	},

	projectile			=
	{
		factory           = Rocket,
		layer             = 6,
		accel             = { 0, 4500 },
		speed             = { 0, 450  },
		homing            = true,
		overrideTarget    = true,
		texture				= "resources/square.png", --data overwritten by DroneMgr on creation
		width             = 100*SIZE_WIDTH_K,
		height            = 100*SIZE_HEIGHT_K,
		splitProp			= true;
		collisionWidth    = 20*SIZE_WIDTH_K,
		collisionHeight   = 20*SIZE_HEIGHT_K,
		damage            = PROJ_DMG_K, --data overwritten by DroneMgr on creation
		maxTurnRate       = 270,
		turnDelay         = 0.00,
		accelDelay        = 0.10,

		fire					=
		{
			layer          = 5,
			texture        = "resources/Vehicle/Drones/DroneMissile/DroneMissile_RocketPropulsion.png",
			offset         = { 0, -32*SIZE_HEIGHT_K },
			width          = 64*SIZE_WIDTH_K,
			height         = 128*SIZE_HEIGHT_K,
			color          = { 1, 0.576, 0.086, 1 },
			blend          = "add",
			animate			= true,
			launchTime		= 2/3,
			launchScale		= {1.5, 15},
			launchFade		= 0.3,
		},

		explosion			=
		{
			texture        = "resources/Explosion/Explosion_HunterDrone.png",
			width          = 128*PROJ_WIDTH_K,
			height         = 128*PROJ_HEIGHT_K,
			avatarSpeedK   = 0.00,
			growth         = { 15, 15 },
			duration       = 9/30,
			fadeStart      = 1/30,
			spin           = -270,
			color          = { 1, 1, 1, 0.85 },
			damage         =
			{
				start       = 6/30,
				stop        = 7/30,
				vehicleVal  = 0, --data overwritten by DroneMgr on creation
				avatarVal   = 0,
			}
		},
	},
};

return drone_missile_base;
