require("app/template/template");
require("app/template/explosions/vehicle_explosion");
require("game/entity/drone/drone-repair");

local SIZE_WIDTH_K	= 1.0;
local SIZE_HEIGHT_K	= 1.0;
local SHDW_WIDTH_K	= 0.75;
local SHDW_HEIGHT_K	= 0.75;
local PROJ_WIDTH_K	= 1.0;
local PROJ_HEIGHT_K	= 1.0;
local PROJ_DMG_K		= -1.0;

local drone_repair_base =
{
	factory				= DroneRepair,
	layer					= 5,
	priority				= 0,
	texture				= "resources/square.png", --data overwritten by DroneMgr on creation
	width					= 100*SIZE_WIDTH_K,
	height				= 100*SIZE_HEIGHT_K,
	splitProp			= true;
--	pivot					= { 0, 0 },
	collisionWidth		= 60*SIZE_WIDTH_K,
	collisionHeight	= 60*SIZE_HEIGHT_K,
	offset				= { -128, 32 }, --default x offset to left mount point
	modOffset			= { 0, 128 },
	hitPoints         = 100, --data overwritten by DroneMgr on creation
	speedMin				= { 256,	142 }, --x,y pixels per sec
	speedMax				= { 960,	1138 }, --x,y pixels per sec
	speedRot				= { 30,	360 }, --min, max degrees per sec
	shootDelay			= 0.0,
	bulletCount			= 8,
	bulletDelay			= 2/3, --used for heal tick rate, data overwritten by DroneMgr on creation
	bulletDelta			= { { 0*SIZE_WIDTH_K, y=0*SIZE_HEIGHT_K} },
	parentProj			= true,
	explosion			= TEMPLATE_VEHICLE_EXPLOSION.small,
	respawnCDRange		= { 1.25, 1.25 },
	chargeThreshold	= 40, --data overwritten by DroneMgr on creation
	abilityCooldown	= 3,
	initTime				= 1,
	heal					= 10, --percentage of avatar health healed per tick, data overwritten by DroneMgr on creation
	
	shadow				=
	{
		layer				= 3,
		priority			= 100,
		width				= 100*SHDW_WIDTH_K,
		height			= 100*SHDW_HEIGHT_K,
		avatarSpeedK	= 0,
		texture			= "resources/Vehicle/Drones/DroneRepair/DroneRepair_Shadow.png",
		offset			= {22,-42},
		color			= {1,1,1,0.40},
	},

	healvfx					=
	{
		layer				= 2,
		texture        = "resources/Vehicle/Drones/DroneRepair/DroneRepair_Aura.png",
		width          = 100,
		height         = 100,
		offset			= { 0, 0 },
		growth         = { 7, 7 };
		duration       = 2/3, --data overwritten by DroneMgr on creation
		fadeStart		= 0.5, --data overwritten by DroneMgr on creation
		spin           = 0,
		avatarSpeedK   = 0.00,
		animate			= false,
	},

	hudvfx				=
	{
		layer				= 6,
		texture        = "resources/Vehicle/Drones/DroneRepair/DroneRepair_Aura.png",
		width          = 200,
		height         = 70,
		offset			= { 0, 0 },
		growth         = { 3, 3 };
		duration       = 2/3, --data overwritten by DroneMgr on creation
		fadeStart		= 0.5, --data overwritten by DroneMgr on creation
		spin           = 0,
		avatarSpeedK   = 0.00,
		animate			= false,
	},

	readyvfx				=
	{
		layer				= 4,
		priority			= 10,
		texture        = "resources/Vehicle/Drones/DroneRepair/DroneRepair_Aura.png",
		width          = 512*SIZE_WIDTH_K,
		height         = 512*SIZE_HEIGHT_K,
		scale				= { 1, 1 }, --data overwritten by DroneMgr on creation
		color          = { 1, 1, 1, 1 },
		offset			= { 0*SIZE_WIDTH_K, 0*SIZE_HEIGHT_K },
		blend          = "add",
	},

	pip_on				=
	{
		layer				= 5,
		priority			= 10,
		texture        = "resources/UI/Drone_UI/dronepip_on.png",
		width          = 24*SIZE_WIDTH_K,
		height         = 24*SIZE_HEIGHT_K,
		scale				= { 1, 1 }, --data overwritten by DroneMgr on creation
		color          = { 1, 1, 1, 1 }, --data overwritten by DroneMgr on creation
		offset			=
		{
			{ -30*SIZE_WIDTH_K,	-60*SIZE_HEIGHT_K },
			{ 0*SIZE_WIDTH_K,		-60*SIZE_HEIGHT_K },
			{ 30*SIZE_WIDTH_K,	-60*SIZE_HEIGHT_K },
		},
	},

	pip_off				=
	{
		layer				= 5,
		priority			= 10,
		texture        = "resources/UI/Drone_UI/dronepip_off.png",
		width          = 24*SIZE_WIDTH_K,
		height         = 24*SIZE_HEIGHT_K,
		scale				= { 1, 1 }, --data overwritten by DroneMgr on creation
		color          = { 1, 1, 1, 1 }, --data overwritten by DroneMgr on creation
		offset			=
		{
			{ -30*SIZE_WIDTH_K,	-60*SIZE_HEIGHT_K },
			{ 0*SIZE_WIDTH_K,		-60*SIZE_HEIGHT_K },
			{ 30*SIZE_WIDTH_K,	-60*SIZE_HEIGHT_K },
		},
	},

	beam					=
	{
		layer				= 4,
		priority			= 5,
		texture			= "resources/Vehicle/Drones/DroneRepair/DroneRepair_Beam.png",
		width				= 128*PROJ_WIDTH_K,
		height			= 128*3,
		offset			= { 0*SIZE_WIDTH_K, 30*SIZE_HEIGHT_K },
		color				= {1,1,1,1},
		extendTime		= 1/3,
		pulseFreq		= 1.0,
		pulseMinWidthK	= 0.5,
		pulseMaxWidthK	= 2.0,
	},
};

return drone_repair_base;
