require("app/template/template");
require("app/template/explosions/vehicle_explosion");
require("game/entity/drone/drone-rocket");

local SIZE_WIDTH_K	= 1.0;
local SIZE_HEIGHT_K	= 1.0;
local SHDW_WIDTH_K	= 0.75;
local SHDW_HEIGHT_K	= 0.75;
local PROJ_WIDTH_K	= 0.75;
local PROJ_HEIGHT_K	= 0.75;
local PROJ_DMG_K		= 0.5;

local drone_rocket_base =
{
	factory				= DroneRocket,
	layer					= 5,
	priority				= 0,
	texture				= "resources/square.png", --data overwritten by DroneMgr on creation
	width					= 100*SIZE_WIDTH_K,
	height				= 100*SIZE_HEIGHT_K,
	splitProp			= true;
--	pivot					= { 0, 0 },
	collisionWidth		= 0,
	collisionHeight	= 0,
	offset				= { -128, 32 }, --default x offset to left mount point
	hitPoints         = 100, --data overwritten by DroneMgr on creation
	speedMin				= { 256,	142 }, --x,y pixels per sec
	speedMax				= { 960,	1138 }, --x,y pixels per sec
	shootDelay			= 20/30,
	bulletCount			= 6,	--data overwritten by DroneMgr on creation
	bulletDelay			= 2/30,
	bulletDelta			= { { x=0, y=0} },
	explosion			= TEMPLATE_VEHICLE_EXPLOSION.small,
	respawnCDRange		= { 0, 0.25 },

	shadow				=
	 {
		 layer			= 3,
		 priority		= 100,
		 width			= 100*SHDW_WIDTH_K,
		 height			= 100*SHDW_HEIGHT_K,
		 avatarSpeedK	= 0,
		 texture			= "resources/Vehicle/Drones/DroneRocket/DroneRocket_Shadow.png",
		 offset			= {22,-42},
		 color			= {1,1,1,0.40},
	 },

	muzzleblastvfx		=
	{
		layer				= 4,
		texture        = "resources/Vehicle/Drones/DroneRocket/DroneRocket_MuzzleBlast.png",
		width          = 16*SIZE_WIDTH_K,
		height         = 16*SIZE_HEIGHT_K,
		offset			= { -35*SIZE_WIDTH_K, 22*SIZE_HEIGHT_K },
		growth         = { 7, 35 };
		duration       = 0.25,
		fadeStart		= 0.20,
		spin           = 0,
		avatarSpeedK   = 0.00,
		animate			= false,
	},

	muzzleflashvfx		=
	{
		layer				= 4,
		texture        = "resources/Vehicle/Drones/DroneRocket/DroneRocket_MuzzleFlash.png",
		width          = 40*SIZE_WIDTH_K,
		height         = 10*SIZE_HEIGHT_K,
		offset			= { -35*SIZE_WIDTH_K, 28*SIZE_HEIGHT_K },
		growth         = { 2, 0.5 };
		duration       = 0.25,
		fadeStart		= 0.15,
		spin           = 0,
		avatarSpeedK   = 0.00,
		animate			= false,
	},

	backblastvfx		=
	{
		layer				= 4,
		texture        = "resources/Vehicle/Drones/DroneRocket/DroneRocket_BackBlast.png",
		width          = 32*SIZE_WIDTH_K,
		height         = 32*SIZE_HEIGHT_K,
		offset			= { -35*SIZE_WIDTH_K, 0*SIZE_HEIGHT_K },
		growth         = { 5, 100 };
		duration       = 0.25,
		fadeStart		= 0.15,
		spin           = 0,
		avatarSpeedK   = 0.00,
		animate			= false,
	},

	projectile			=
	{
		speed					= { 0, 1000 },
		width					= 60*PROJ_WIDTH_K,
		height				= 120*PROJ_HEIGHT_K,
		collisionWidth		= 64*PROJ_WIDTH_K,
		collisionHeight	= 128*PROJ_HEIGHT_K,
		damage				= PROJ_DMG_K, --data overwritten by DroneMgr on creation
		texture				= "resources/square.png", --data overwritten by DroneMgr on creation
		fire					=
		{
			layer				= 5,
			texture			= "resources/Hazard/Missile-Flame-2.png",
			offset			= { 0*PROJ_WIDTH_K, -50*PROJ_HEIGHT_K },
			width				= 20*PROJ_WIDTH_K,
			height			= 64*PROJ_HEIGHT_K,
			animate			= true,
			launchTime		= 0.5,
			launchScale		= {1.35, 10},
			launchFade		= 0.2,
		},
		hitMarker			=
		{
			texture        = "resources/Explosion/Explosion_04.png",
			width          = 32,
			height         = 32,
			growth         = { 10, 10 };
			duration       = 0.5,
			spin           = 360,
			avatarSpeedK   = 0.00,
		},
	},
};

return drone_rocket_base;
