
local TIME_MGR_SAVE_FILENAME_LUA     = "TIME_MGR_SAVE_FILENAME.lua";
local TIME_MGR_SAVE_FILENAME_JSON     = "52t83i1520m41e.jp5";
local MONTHS                     = {jan=1,feb=2,mar=3,apr=4,may=5,jun=6,jul=7,aug=8,sep=9,oct=10,nov=11,dec=12};

--Anything that is not between these 2 times means something went wrong
local TIME_MIN                   = 1389837512;
local SERVER_TIME_MIN            = 1410836528; --sometimes we get some old time from 1970 etc... make sure it doesnt happen


local TIME_INTERVAL_CHANGE_MIN   = - 24*60*60;    -- 24  hours : If the new time is in the pass, dont reset unless it is WAY off
                                                  -- Make it 24 hours because we fixed a bug in the time calculation and it could
                                                  -- create some issue, so be more lenient for now
local TIME_INTERVAL_CHANGE_MAX   =    6*60*60;    -- 6   hours

----------------------------------------------------------------------

--For now we only care for 2 timeZone
local TIMEZONE_DELTA             =
{
   GMT       =  0*60*60;
   PST       = -8*60*60;   --8 hours
};

----------------------------------------------------------------------

TimeMgr  =
{
};


----------------------------------------------------------------------

--1.If you reset your phone and the clock is now 0 (see above)
--  and if this happen again before you can received a new web stamp and reset appropriately

--2.The clock reset but we "missed" it and not reset the clock...
--  it reset but the device is still above the clockStamp

function TimeMgr:handleClockReset()

   local resetType   = nil;
   local suggestClock= nil;

   self._clockWasReset = false;

   if self._clockStamp  then

      local deviceTime = MOAISim.getDeviceTime() or 0;

      if deviceTime == 0 then

         resetType    = "deviceTime == 0";
         suggestClock = 0;

      elseif self._savedDeviceTime and self._savedClockStamp and deviceTime <= self._savedDeviceTime then

         if self._savedClockStamp > self._savedDeviceTime then

            resetType    = "self._savedDeviceTime >= deviceTime : self._savedClockStamp > self._savedDeviceTime";
            suggestClock = 0;

         else

            resetType    = "self._savedDeviceTime >= deviceTime";
            suggestClock = -(self._savedDeviceTime-self._savedClockStamp);

            Debug.assert( suggestClock <= 0, "TimeMgr:handleClockReset -> suggestClock <= 0" );

         end

      elseif self._clockStamp >= deviceTime then

         --This case should be impossible, unless we are using an old version that was not using _savedDeviceTime nor _savedClockStamp
         Debug.assert( not (self._savedDeviceTime and self._savedClockStamp), "");

         resetType    = "self._clockStamp >= deviceTime";
         suggestClock = 0;

      end

      --Other check, make sure we do it at the end and not as a elseif since it might be needed on top of what we already did
      --If the new time is still behind the one we calculated before
      --At least make the progression to our time back to what it was
      if  self:_calcTimeGMT(suggestClock) < (self._savedTimeGMT or 0) then

         resetType    = "self:_calcTimeGMT(suggestClock) < (self._savedTimeGMT or 0)";
         suggestClock = (self._savedTimeGMT or 0) - self:_calcTimeGMT(suggestClock);

         Debug.assert( (self:_calcTimeGMT(suggestClock) or 0) >= (self._savedTimeGMT or 0), "TimeMgr:handleClockReset -> self:_calcTimeGMT(suggestClock) >= (self._savedTimeGMT or 0)" );

      end

      if resetType then

         MOAIFlurry:event( "TimeMgr-Detect-ClockReset",
                           "resetType",                  tostring(resetType),
                           "Would fix",                  tostring(self:_calcTimeGMT(suggestClock) >= (self._savedTimeGMT or 0)),
                           "Delta",                      tostring(self:_calcTimeGMT(suggestClock) -  (self._savedTimeGMT or 0)),
                           "Delta (hours)",              tostring(math.floor((self:_calcTimeGMT(suggestClock) - (self._savedTimeGMT or 0)) / (60*60))) );

      end

      --Lets reset our clock
      if suggestClock and suggestClock ~= self._clockStamp then

         self._clockStamp     = suggestClock;
         self._clockWasReset  = true;

         self:_save();

      end

      Debug.assert( self:_calcTimeGMT() >= (self._savedTimeGMT or 0),                    "TimeMgr:handleClockReset -> 1 : resetType : " .. tostring(resetType) );
      Debug.assert( self._clockStamp and (self._clockStamp < MOAISim.getDeviceTime()) ,  "TimeMgr:handleClockReset -> 2 : resetType : " .. tostring(resetType) );

      --When our reset is better handled, we will be able to remove the following event and depend on assert instead (uncomment above)
      if self:_calcTimeGMT() < (self._savedTimeGMT or 0) then

         MOAIFlurry:event( "TimeMgr-Detect-ClockReset-Failed",
                           "resetType",                  tostring(resetType),
                           "failureType",                "self:_calcTimeGMT() < (self._savedTimeGMT or 0)",
                           "Would fix",                  tostring(self:_calcTimeGMT(suggestClock) >= (self._savedTimeGMT or 0)),
                           "Delta",                      tostring(self:_calcTimeGMT(suggestClock) - (self._savedTimeGMT or 0)),
                           "Delta (hours)",              tostring(math.floor((self:_calcTimeGMT(suggestClock) - (self._savedTimeGMT or 0)) / (60*60))) );

      elseif self._clockStamp and (self._clockStamp >= MOAISim.getDeviceTime())  then

         MOAIFlurry:event( "TimeMgr-Detect-ClockReset-Failed",
                           "resetType",                  tostring(resetType),
                           "failureType",                "self._clockStamp and (self._clockStamp >= MOAISim.getDeviceTime())",
                           "Would fix",                  tostring(self:_calcTimeGMT(suggestClock) >= (self._savedTimeGMT or 0)),
                           "Delta",                      tostring(self:_calcTimeGMT(suggestClock) - (self._savedTimeGMT or 0)),
                           "Delta (hours)",              tostring(math.floor((self:_calcTimeGMT(suggestClock) - (self._savedTimeGMT or 0)) / (60*60))) );

      end
   end
end


----------------------------------------------------------------------

function TimeMgr:init()

   EventMgr:addListener(  self, self._onOnlineMgrStatus , "online-mgr:status-changed");

   self:_load();

   self:handleClockReset();

   self._timeGMT              = self:_calcTimeGMT();

   ----------------------------------------------------------------
   --Throwing a check on something we assume to be always true
   ----------------------------------------------------------------
   if self._timeGMT and self._savedTimeGMT and self._timeGMT < self._savedTimeGMT then
      if self._webStampGMT then
         Debug.assert( self._timeGMT >= self._savedTimeGMT, "TimeMgr:init -> self._timeGMT >= self._savedTimeGMT (server)" );
      elseif self._localStampGMT then
         Debug.assert( self._timeGMT >= self._savedTimeGMT, "TimeMgr:init -> self._timeGMT >= self._savedTimeGMT (local)" );
      else
         Debug.assert( self._timeGMT >= self._savedTimeGMT, "TimeMgr:init -> self._timeGMT >= self._savedTimeGMT (unknown)" );
      end
   end
   ----------------------------------------------------------------


end

----------------------------------------------------------------------

function TimeMgr:activate( force )


end

----------------------------------------------------------------------

function TimeMgr:deactivate()


end

----------------------------------------------------------------------

function TimeMgr:onEnterForeground()

end


----------------------------------------------------------------------

function TimeMgr:onEnterBackground()

   --Make it more resilient for reboot

   local time = self:_calcTimeGMT();
   if time then
      if self._webStampGMT then
         self:_resetWebStamp( time );
         self:_save();
      elseif self._localStampGMT then
         self:_resetLocalStamp( time );
         self:_save();
      else
         Debug.assert( self._webStampGMT or self._localStampGMT );
      end

      Debug.assert(   time and self:_calcTimeGMT() and (math.abs(time - self:_calcTimeGMT()) <= 1),
                     "TimeMgr:onEnterBackground -> time and self:_calcTimeGMT() and (math.abs(time - self:_calcTimeGMT()) <= 1)" );
   end

end

----------------------------------------------------------------------

function TimeMgr:isReady()

   --Make sure we have updated our time before we are considered really ready
   return  self._timeGMT;

end

----------------------------------------------------------------------

function TimeMgr:update(dt)

   --we cache the time for 2 reason
   --1. Speed     : calcTime can be call multiple time in a frame
   --2. Stability : you are sure that the time will be consistent from frame to frame
   self._timeGMT = self:_calcTimeGMT();

end

----------------------------------------------------------------------

function TimeMgr:getTimeGMT()

   return self._timeGMT;

end

----------------------------------------------------------------------

function TimeMgr:getTimeLocalGMT()

   return os.time();

end

----------------------------------------------------------------------

function TimeMgr:_calcTimeGMT( clockStamp )

   local stamp = MOAISim.getDeviceTime();

   clockStamp = clockStamp or self._clockStamp;

   if clockStamp and stamp then

      local delta = math.floor(stamp-clockStamp);

      --If the timer went back... it probably means a reboot
      -- dont return time until we adust
      if delta >= 0 then

         --We always want to use time from the web because it is harder to cheat
         if self._webStampGMT then
            return self._webStampGMT   + delta;
         elseif self._localStampGMT then
            return self._localStampGMT + delta;
         end
      end

   end


end

----------------------------------------------------------------------

function TimeMgr:_save()

   local tbl            = {};

   --Update our saved Time GMT
   self._savedTimeGMT   =  self:_calcTimeGMT();

   tbl.timeGMT          =  self._savedTimeGMT;
   tbl.clockStamp       =  self._clockStamp;
   tbl.deviceTime       =  MOAISim.getDeviceTime();
   tbl.webStamp         =  self._webStampGMT;
   tbl.localStamp       =  self._localStampGMT;
   tbl.webSuccessCount  =  self._webSuccessCount;
   tbl.webFailureCount  =  self._webFailureCount;
   tbl.webDate          =  self._webDate;

   saveFileMultiFormat(TIME_MGR_SAVE_FILENAME_LUA, TIME_MGR_SAVE_FILENAME_JSON, tbl);

end

----------------------------------------------------------------------

function TimeMgr:_load()

   local tbl                  = loadFileMultiFormat(TIME_MGR_SAVE_FILENAME_LUA, TIME_MGR_SAVE_FILENAME_JSON) or {};

   self._clockStamp           =  tbl.clockStamp;
   self._webStampGMT          =  tbl.webStamp;
   self._localStampGMT        =  tbl.localStamp;
   self._webFailureCount      =  tbl.webFailureCount  or 0;
   self._webSuccessCount      =  tbl.webSuccessCount  or 0;
   self._webDate              =  tbl.webDate;
   self._savedTimeGMT         =  tbl.timeGMT;
   self._savedClockStamp      =  tbl.clockStamp;
   self._savedWebStampGMT     =  tbl.webStamp;
   self._savedLocalStampGMT   =  tbl.localStamp;
   self._savedDeviceTime      =  tbl.deviceTime;

   local state                = nil;

   if self._clockStamp then
      if self._clockStamp <= MOAISim.getDeviceTime() then
         state = "self._clockStamp is smaller : Great";
      else
         state = "self._clockStamp is greater : Reboot?";
      end
   else
      state = "self._clockStamp is nil";
   end

   MOAIFlurry:event( "TimeMgr-Load",
                     "State",                   tostring(state),
                     "self._clockStamp",        tostring(self._clockStamp),
                     "MOAISim.getDeviceTime()", tostring(MOAISim.getDeviceTime()),
                     "self._webStampGMT",       tostring(self._webStampGMT),
                     "self._localStampGMT",     tostring(self._localStampGMT),
                     "webFailureCount",         tostring(self._webFailureCount),
                     "webSuccessCount",         tostring(self._webSuccessCount),
                     "webDate",                 tostring(self._webDate),
                     "self._savedTimeGMT",      tostring(self._savedTimeGMT)
                     );


end


----------------------------------------------------------------------

function TimeMgr:_onOnlineMgrStatus( event )

   if event then

      local webDate = event.headerDate;

      if webDate then

         local webTimeGMT = self:convertWebDateToTimeGMT(webDate);

         --Make sure we can translate our header Date to a number
         if event.headerDate then
            --TIME_MIN was the development time...
            --  so unless time travel exist we should not be able to ho there
            Debug.assert(webTimeGMT and webTimeGMT >= TIME_MIN, "TimeMgr:_onOnlineMgrStatus -> webTimeGMT : " .. tostring(webTimeGMT) .. " >> " .. tostring(event.headerDate) );
         end

         if webTimeGMT and webTimeGMT >= TIME_MIN then
            self:_onOnlineMgrStatusOnline( webDate, webTimeGMT );
         else
            self:_onOnlineMgrStatusOffline(  );
         end

      else

         Debug.assert( event, "TimeMgr:_onOnlineMgrStatus -> webDate" );

         self:_onOnlineMgrStatusOffline();

      end

   else

      Debug.assert( event, "TimeMgr:_onOnlineMgrStatus -> event" );

      self:_onOnlineMgrStatusOffline();

   end
end

----------------------------------------------------------------------

function TimeMgr:_calcOnlineMgrStatusOnlineFlurryType( webStampOld, localStampGMT, clockStamp, deltaTime )

   local flurryType = "unknown";

   --This is our first time getting web
   if self._clockWasReset then
      flurryType = "clock was reset";
   elseif webStampOld == nil then

      --we never get local as well
      if localStampGMT == nil then
         flurryType = "online-web-success:reset-first-time";
      else
         flurryType = "online-web-success:reset-from-local";
      end

   else

      --The clockStamp should never be nil, but just in case
      if clockStamp == nil then
         flurryType = "online-web-success: self._clockStamp == nil";
      else

         --This will be hapening more when a reboot happens
         if clockStamp > MOAISim.getDeviceTime() then
            flurryType = "online-web-success: self._clockStamp > MOAISim.getDeviceTime() : Reboot?";
         elseif clockStamp == 0 then
            flurryType = "online-web-success: self._clockStamp == 0 : Reboot";
         else

            if deltaTime > TIME_INTERVAL_CHANGE_MAX then
               flurryType = "online-web-success: delta > TIME_INTERVAL_CHANGE_MAX";
            elseif deltaTime < TIME_INTERVAL_CHANGE_MIN then
               flurryType = "online-web-success: delta < TIME_INTERVAL_CHANGE_MIN";
            else
               flurryType = "online-web-success: eveything is fine";
            end

         end
      end
   end

   return flurryType;

end

----------------------------------------------------------------------

function TimeMgr:_checkWebStampMoveForward( webStampOld, webStampNew,  webDateOld , webDateNew)

   if webStampNew and webStampNew < SERVER_TIME_MIN then

      Debug.assert( webStampNew >= SERVER_TIME_MIN, "TimeMgr:_checkWebStampMoveForward -> webStampNew >= SERVER_TIME_MIN");

      MOAIFlurry:event( "TimeMgr-bad-webStampNew",
                        "Type",                                      "webStampNew and webStampNew < SERVER_TIME_MIN",
                        "Web-Stamp-New",                             tostring(webStampNew),
                        "Web-Stamp-Old",                             tostring(webStampOld),
                        "Web-Date-New",                              tostring(webDateNew),
                        "Web-Date-Old",                              tostring(webDateOld),
                        "OS.Date-Local",                             tostring(os.date("%c",  os.time())),
                        "OS.Date-GMT",                               tostring(os.date("!%c", os.time())),
                        "webStampNew - webStampOld",                 tostring(webStampNew - webStampOld ),
                        "webStampNew - webStampOld (MINUTES)",       tostring(math.floor(((webStampNew or 0) - (webStampOld or 0))/60 )),
                        "webStampNew - webStampOld (HOURS)",         tostring(math.floor(((webStampNew or 0) - (webStampOld or 0))/(60*60) ))  );

      return false;

   --Make sure our web stamp move forward... if not, we have a problem
   elseif webStampOld and webStampNew and (webStampNew < webStampOld) then

      Debug.assert( webStampNew >= webStampOld, "TimeMgr:_checkWebStampMoveForward -> webStampNew >= webStampOld");

      MOAIFlurry:event( "TimeMgr-bad-webStampNew",
                        "Type",                                      "webStampOld and webStampNew and (webStampNew > webStampOld)",
                        "Web-Stamp-New",                             tostring(webStampNew),
                        "Web-Stamp-Old",                             tostring(webStampOld),
                        "Web-Date-New",                              tostring(webDateNew),
                        "Web-Date-Old",                              tostring(webDateOld),
                        "OS.Date-Local",                             tostring(os.date("%c",  os.time())),
                        "OS.Date-GMT",                               tostring(os.date("!%c", os.time())),
                        "webStampNew - webStampOld",                 tostring(webStampNew - webStampOld ),
                        "webStampNew - webStampOld (MINUTES)",       tostring(math.floor((webStampNew - webStampOld)/60 )),
                        "webStampNew - webStampOld (HOURS)",         tostring(math.floor((webStampNew - webStampOld)/(60*60) ))  );

      return false;

   end

   return true;

end

----------------------------------------------------------------------

function TimeMgr:_onOnlineMgrStatusOnline( webDateNew, webStampNew )

   Debug.assert( webDateNew,  "TimeMgr:_onOnlineMgrStatusOnline -> webDateNew"  );
   Debug.assert( webStampNew, "TimeMgr:_onOnlineMgrStatusOnline -> webStampNew" );

   if  webStampNew then

      --We have fix the web stamp which will create time moving backward by up to 24 hours
      local forceResetDone = App.data:getValue("TimeMgr.Force.WebStamp.Reset.Done");
      local moveForward    = true;

      if forceResetDone then
         --We want to check if our webstamp move forward in time
         moveForward = self:_checkWebStampMoveForward( self._webStampGMT, webStampNew, self._webDate, webDateNew );
      end

      if moveForward then

         local webDateOld  =  self._webDate;
         local webStampOld =  self._webStampGMT;
         local clockOld    =  MOAISim.getDeviceTime()-(self._clockStamp or 0);
         local timeGMTOld  =  self:_calcTimeGMT();
         local deltaTime   =  -10000000;

         --Delta check to see if our old info still hold (as it should)
         if timeGMTOld and webStampNew then
            deltaTime = webStampNew - timeGMTOld;
         end

         --Because it was showed that sometimes the time received from the internet go back and forth in time
         --Try to make it less fluctuate by only adjusting only if we are not in the same ballpark
         local ok    = webStampOld and self._clockStamp and (self._clockStamp > 0 ) and (deltaTime >= TIME_INTERVAL_CHANGE_MIN and deltaTime <= TIME_INTERVAL_CHANGE_MAX);
         local reset = (self._clockWasReset) or (not ok) or (not forceResetDone) or (App.shouldResetWebTimestamp);
         print("TimeMgr:onlineStatus reset: "..tostring(reset));
         if reset then
            App.shouldResetWebTimestamp = false;
            Debug.assert( (not forceResetDone) or ((webStampNew or 0) >= (webStampOld or 0)),  "TimeMgr:_onOnlineMgrStatusOnline -> reset: (webStampNew or 0) >= (webStampOld or 0)" );

            --since we have a new time from web, reset with this one
            self:_resetWebStamp( webStampNew );

            self._webDate         = webDateNew;
            self._webSuccessCount = ( self._webSuccessCount or 0 ) + 1;

            self:_save();

            App.data:setValue("TimeMgr.Force.WebStamp.Reset.Done", true);

            Debug.assert( webStampNew == self._webStampGMT,          "TimeMgr:_onOnlineMgrStatusOnline -> reset: webStampNew == self._webStampGMT" );

         end

         local clockNew    =  MOAISim.getDeviceTime()-(self._clockStamp or 0);
         local timeGMTNew  =  self:_calcTimeGMT();
         local flurryType  =  self:_calcOnlineMgrStatusOnlineFlurryType( webStampOld, self._localStampGMT, self._clockStamp, deltaTime );
         local deltaTimeAbs=  math.abs(deltaTime);
         local deltaTimeSgn=  math.sign(deltaTime);

         MOAIFlurry:event( "TimeMgr-OnlineStatus-v5",
                           "Type",                                tostring(flurryType),
                           "reset",                               tostring(reset),
                           "reset-type",                          "self._clockWasReset:"..tostring(reset).. "   not-ok:"..tostring(not ok),
                           "Web-Date-New",                        tostring(webDateNew),
                           "Web-Date-Old",                        tostring(webDateOld),
                           "Delta-TimeGMT-Seconds",               tostring(deltaTimeSgn*math.floor((deltaTimeAbs or 0))),
                           "Delta-TimeGMT-Minutes",               tostring(deltaTimeSgn*math.floor((deltaTimeAbs or 0)/60)),
                           "Delta-TimeGMT-Hours",                 tostring(deltaTimeSgn*math.floor((deltaTimeAbs or 0)/3600))    );

         if reset then

            MOAIFlurry:event( "TimeMgr-OnlineStatus-reset",
                              "Type",                                tostring(flurryType),
                              "Web-Date-New",                        tostring(webDateNew),
                              "Web-Date-Old",                        tostring(webDateOld),
                              "Web-Stamp-New",                       tostring(webStampNew),
                              "Web-Stamp-Old",                       tostring(webStampOld),
                              "Clock-New",                           tostring(clockNew),
                              "Clock-Old",                           tostring(clockOld),
                              "Time-GMT-New",                        tostring(timeGMTNew),
                              "Time-GMT-Old",                        tostring(timeGMTOld),
                              "Delta-TimeGMT-Minute",                tostring(deltaTimeSgn*math.floor((deltaTimeAbs or 0)/60))   );

         end
      end

   else

      Debug.assert( webStampNew, "TimeMgr:_onOnlineMgrStatusOnline -> webStampNew" );

   end
end

----------------------------------------------------------------------

function TimeMgr:_onOnlineMgrStatusOffline(  )

   local flurryType  =  nil;
   local webDateOld  =  self._webDate;
   local webStamp    =  self._webStampGMT;
   local timeGMTOld  =  self:_calcTimeGMT();
   local localGMT    =  self:getTimeLocalGMT();

   --This is our first time getting web
   if (self._webStampGMT == nil) and (self._localStampGMT == nil) then

      flurryType = "offline-local:reset-first-time";
      self:_resetLocalStamp( localGMT );

   else

      --If the time mgr return a value, keep using it
      if timeGMTOld then

         if self._webStampGMT then
            flurryType = "offline-local:use-cached-web";
         else
            Debug.assert(self._localStampGMT);
            flurryType = "offline-local:use-cached-local";
         end

      else

         --This should never happens because we are adjusting the clockStamp at init if not valid
         Debug.assert( timeGMTOld, "TimeMgr:_onOnlineMgrStatusOffline -> timeGMTOld" );

         if self._webStampGMT then
            flurryType = "offline-local:invalid-time-was-web";
         else
            Debug.assert(self._localStampGMT);
            flurryType = "offline-local:invalid-time-was-local";
         end

         --The time is not valid
         self:_resetLocalStamp( localGMT );

      end

   end

   self._webFailureCount = ( self._webFailureCount or 0 ) + 1;

   self:_save();

   MOAIFlurry:event( "TimeMgr-OfflineStatus",
                     "Type",                                tostring(flurryType),
                     "Web-Date-Old",                        tostring(webDateOld),
                     "WebStamp",                            tostring(webStamp),
                     "TimeGMTOld",                          tostring(timeGMTOld),
                     "TimeGMTNew",                          tostring(self:_calcTimeGMT()) );

end

----------------------------------------------------------------------

function TimeMgr:_isClockStampValid()

   --If the clockStamp is > than the current stamp, it means the phone was reset
   return      self._clockStamp
         and   self._clockStamp <=  MOAISim.getDeviceTime();

end

----------------------------------------------------------------------

function TimeMgr:_resetWebStamp( stamp )

   Debug.assert(stamp);

   self._clockStamp     = MOAISim.getDeviceTime();
   self._webStampGMT    = stamp;
   self._localStampGMT  = nil;

   Debug.assert(self:_calcTimeGMT());

end

----------------------------------------------------------------------

function TimeMgr:_resetLocalStamp( stamp )

   Debug.assert(stamp);

   --we are offline... we need to make sure we have a time
   --we need to use local until we connect to the web
   self._clockStamp     = MOAISim.getDeviceTime();
   self._localStampGMT  = stamp;
   self._webStampGMT    = nil;

   Debug.assert(self:_calcTimeGMT());

end

----------------------------------------------------------------------

function TimeMgr:convertWebDateToTimeGMT( date )

   if not Debug.assertType(date, "string", "TimeMgr:convertWebDateToTimeTableGMT -> date") then
      return;
   end

   -- Dont use a cache value and recalculate in case the user change clock as while playing
   local localDeltaGMT  = self:calcLocalDeltaGMT();
   local tblGMT         = self:_convertWebDateToTimeTableGMT( date );

   if tblGMT and localDeltaGMT then

      -- os.time assume the tbl is local and will convert to GMT
      -- however, our tbl is already in GMT = the adjustement will be done twice
      -- so we need to undo the adjustment
      local time        =  os.time(tblGMT);
      local timeGMT     =  time  + localDeltaGMT;

      return timeGMT;

   else

      Debug.assertType( tblGMT, "table", "TimeMgr:convertWebDateToTimeGMT -> tblGMT" );
      Debug.assertType( localDeltaGMT, "number", "TimeMgr:convertWebDateToTimeGMT -> localDeltaGMT" );

   end

end

----------------------------------------------------------------------

function TimeMgr:_convertWebDateToTimeTableGMT( date )

   if not Debug.assertType(date, "string", "TimeMgr:_convertWebDateToTimeTableGMT -> date") then
      return;
   end

   local day, month, year, hour, min, sec, tz = date:match("%a+, (%d+) (%a+) (%d+) (%d+):(%d+):(%d+) (%a+)");

   Debug.assert(tz and tz:upper() == "GMT", "TimeMgr:_convertWebDateToTimeTableGMT-> tz == GMT" );

   if day and month and year and hour and min and sec then

      return {
         year  = tonumber(year),
         month = tonumber(MONTHS[month:lower()]),
         day   = tonumber(day),
         hour  = tonumber(hour),
         min   = tonumber(min),
         sec   = tonumber(sec),
         isdst = false,
      };

   else

      Debug.assertType( day,   "number", "TimeMgr:_convertWebDateToTimeTableGMT -> day"   );
      Debug.assertType( month, "number", "TimeMgr:_convertWebDateToTimeTableGMT -> month" );
      Debug.assertType( year,  "number", "TimeMgr:_convertWebDateToTimeTableGMT -> year"  );
      Debug.assertType( hour,  "number", "TimeMgr:_convertWebDateToTimeTableGMT -> hour"  );
      Debug.assertType( min,   "number", "TimeMgr:_convertWebDateToTimeTableGMT -> min"   );
      Debug.assertType( sec,   "number", "TimeMgr:_convertWebDateToTimeTableGMT -> sec"   );

   end

end

----------------------------------------------------------------

function TimeMgr:calcLocalDeltaGMT()

   local t = os.time();
   return os.time(os.date("*t", t)) - os.time(os.date("!*t", t));

end

----------------------------------------------------------------------
