----------------------------------------------------------------

local UPDATE_MANDATORY_VERSION_ID = "udpate-mandatory-version";

----------------------------------------------------------------

UpdateMgr =
{
   _active        = false,
   _myVersion     = nil,
   _storeVersion  = nil,
   _needUpdate    = false,
   _mandatory     = false,
};

----------------------------------------------------------------

function UpdateMgr:activate()

   if not self._active then
      self._active         = true;
      self._myVersion      =  App:versionStringToNumber( App.info.version );
      self._needUpdate     = false;
      EventMgr:addListener( self, self._onCloudMasterData, "cloud:master-data-app-store-update" );

      UpdateMgr:_checkMandatoryUpdate();
   end

end

----------------------------------------------------------------

function UpdateMgr:deactivate()

   if  self._active then
      self._active = false;
      EventMgr:rmvListener( self );
   end

end

----------------------------------------------------------------

function UpdateMgr:isActive()
   return self._active == true;
end

----------------------------------------------------------------

function UpdateMgr:shouldUpdate()

   return self._active and self._needUpdate;

end

----------------------------------------------------------------

function UpdateMgr:isUpdateMandatory()

   return self._active and self._needUpdate and self._mandatory;

end

----------------------------------------------------------------

function UpdateMgr:SetMandatoryVersionNumber(version)
    print("UpdateMgr:SetMandatoryVersionNumber : ",tostring(version), " current version = ", tostring(self._myVersion));
    App.data:setValue( UPDATE_MANDATORY_VERSION_ID, version );
    UpdateMgr:_checkMandatoryUpdate();
end

----------------------------------------------------------------

function UpdateMgr:_checkMandatoryUpdate()
    print("data.mandatoryVersion = ", tostring(App.data:getValue(UPDATE_MANDATORY_VERSION_ID) or "nil"), " my+version = ", tostring(self._myVersion or "nil"));
    if (App.data:getValue(UPDATE_MANDATORY_VERSION_ID) or 0) > self._myVersion then
        self._needUpdate  = true;
        self._mandatory   = true;
    end
end

----------------------------------------------------------------

function UpdateMgr:_onCloudMasterData( data )

   local header      = (data or {}).header;
   local content     = (data or {}).content;

   Debug.assert( data and content );

   --Only care if we downloaded successfully
   if content and header and header.upToDate then

      self._needUpdate  = false;
      self._mandatory   = false;
      self._storeVersion = nil;

      local tbl = content.iOS or {};

      if MOAIEnvironment.osBrand == "Android" then
         tbl = content.android or {};
      end

      if  tbl then

         self._storeVersion  = App:versionStringToNumber( tbl.number );

         if       self._myVersion
            and   self._storeVersion  then

            if       type(self._myVersion)    == "number"
               and   type(self._storeVersion) == "number"   then

               self._needUpdate  = self._storeVersion > self._myVersion;
               self._mandatory   = self._needUpdate and tbl.mandatory;

               --When we have a mandatory update, flag it to make sure we always trigger it evern if we are offline
               if self._mandatory  then
                  App.data:setValue( UPDATE_MANDATORY_VERSION_ID, self._myVersion );
               end


            end

         end
      end
   end

end

----------------------------------------------------------------
