-------------------------------------------------------------------------------------

require('engine/object/object');
require('flash/flash-manager');
require('engine/utility/math');

-------------------------------------------------------------------------------------

local Base				= Object;

FlashAnim				= Base:new();

-------------------------------------------------------------------------------------

function FlashAnim:new( layer, textureSheet, id, x, y, rot, priority, fps, parent, attrLink )

	Debug.assert(layer, "FlashAnim:new -> invalid layer");
	Debug.assert(textureSheet and type(textureSheet) == "string", "FlashAnim:new -> invalid texture sheet");
	Debug.assert(id and type(id) == "string", "FlashAnim:new -> invalid anim id");

	x			= x or 0;
	y			= y or 0;
	rot		= rot or 0;
	priority	= priority or 0;
	fps		= fps or 30;

	local texture	= FlashMgr:getTextureSheet( textureSheet..".lua", textureSheet..".png", fps);
   local a			= Base.new( self );

   a.anim			= texture:newAnimPlayer(id);

	for _, v in pairs(a.anim.props) do
		v:setPriority(priority);
	end

	a.anim.root:setLoc(x, y);
	a.anim.root:setRot(rot);
	a:insertProps(layer);
	a.anim:setMode( MOAITimer.NORMAL )

	if parent then
      if attrLink == "location-only" then
         a.anim.root:setAttrLink( MOAIProp2D.INHERIT_LOC, parent, MOAIProp2D.TRANSFORM_TRAIT );
      else
         a.anim.root:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, parent, MOAIProp2D.TRANSFORM_TRAIT );
      end
	end

	a.animId			= id;
   a.layer			= layer;
   a.textureSheet	= texture;
	a.priority		= priority;

   return a;

end

----------------------------------------------------------------------

function FlashAnim:delete( )

	if self.anim and self.layer then

		self.anim:removeProps(self.layer);

		if self ~= nil and type(self) == "table" then
			for k,v in pairs(self) do
				self[k]	= nil;
			end
		end
	end

end


----------------------------------------------------------------------

function FlashAnim:setTime( time )

   if self.anim then
      self.anim:setTime(time);
   end

end

----------------------------------------------------------------------

function FlashAnim:getTime()

	if self.anim then
		return self.anim:getTime();
	end

end

----------------------------------------------------------------------

function FlashAnim:apply( time )

	if self.anim and self.anim.root then
		self.anim:apply(time);
	end

end

----------------------------------------------------------------------

function FlashAnim:setAnimFrame( frame )

	if self.anim then

		frame = frame or 0;

		local time = frame/fps;

		if time >= self.anim.sym.span then
			self:setAnimLastFrame();
		else
			self:stop();
			self:apply(time);
		end
	end

end

----------------------------------------------------------------------

function FlashAnim:setAnimLastFrame( )

	if self.anim then

		local time = self.anim.sym.span - self.anim.sym.frame;

		self:stop();
		self:apply(time);
	end

end

----------------------------------------------------------------------

function FlashAnim:resetAnim()

	if self.anim and self.anim.root then
		self:stop();
		self:apply(0);
	end

end

----------------------------------------------------------------------

function FlashAnim:setSpan( startTime, endTime )

	if self.anim then
		if endTime then
			self.anim:setSpan(startTime, endTime);
		else
			self.anim:setSpan(startTime);
		end
	end

end

----------------------------------------------------------------------

function FlashAnim:start( restart )

   if self.anim then

      if restart then
         self.anim:setTime(0);
      end
      self.anim:start();

   end

end

----------------------------------------------------------------------

function FlashAnim:pause()

   if self.anim then
      self.anim:pause();
   end

end

----------------------------------------------------------------------

function FlashAnim:stop()

   if self.anim then
      self.anim:stop();
   end

end

----------------------------------------------------------------------

function FlashAnim:setSpeed( s )

   if self.anim then
      self.anim:setSpeed(s or 1 );
   end

end

-------------------------------------------------------------------------------------

function FlashAnim:insertProps( layer )

   if self.anim and self.anim.root then
      self.anim:insertProps(layer);
   end
end

----------------------------------------------------------------------

function FlashAnim:setMode( mode )

	if self.anim then
		if mode then
			self.anim:setMode(mode);
		else
			self.anim:setMode(MOAITimer.NORMAL);
		end
	end

end

-------------------------------------------------------------------------------------

function FlashAnim:setLoop( loop )

	if loop then
		self:setMode( MOAITimer.LOOP );
	else
		self:setMode( MOAITimer.NORMAL );
	end

end

-------------------------------------------------------------------------------------

function FlashAnim:setLoc( x, y )

   if self.anim and self.anim.root then
      self.anim.root:setLoc( x, y );
   end

end

----------------------------------------------------------------------

function FlashAnim:setLocX( x )

   if self.anim and self.anim.root then

		local x2, y = self.anim.root:getLoc();
		self.anim.root:setLoc( x, y );
   end

end

----------------------------------------------------------------------

function FlashAnim:setLocY( y )

   if self.anim and self.anim.root then

		local x = self.anim.root:getLoc();
		self.anim.root:setLoc( x, y );
   end

end

----------------------------------------------------------------------

function FlashAnim:moveLoc( x, y, time, ease )

	local action;

   if self.anim then

		if self.anim.root then
			action = self.anim.root:moveLoc(x, y, 0, time, ease);
		else
			action = self.anim:moveLoc(x, y, 0, time, ease);
		end

		return action;
   end

end

----------------------------------------------------------------------

function FlashAnim:seekLoc( x, y, time, ease )

	local action = nil;

   if self.anim then
		if self.anim.root then
			action = self.anim.root:seekLoc(x, y, time, ease);
		else
			action = self.anim:seekLoc(x, y, time, ease);
		end
   end

	return action;

end

-------------------------------------------------------------------------------------

function FlashAnim:getLoc( world )

   if self.anim and self.anim.root then

		if world then
			return self.anim.root:modelToWorld();
		else
			return self.anim.root:getLoc();
		end
   end

end

-------------------------------------------------------------------------------------

function FlashAnim:setColor ( r,g,b,a )

   if self.anim and self.anim.props then

		for _, v in pairs(self.anim.props) do
			v:setColor(r,g,b,a);
		end
   end

end

-------------------------------------------------------------------------------------

function FlashAnim:setPriority ( p )

   if self.anim and self.anim.props then

		p					= p or 0;
		self.priority	= p;

      for _, v in pairs(self.anim.props) do
         v:setPriority(p);
      end
   end
end

-------------------------------------------------------------------------------------

function FlashAnim:getPriority ()

	if self.priority then
		return self.priority;
	end

	return 0;

end

-------------------------------------------------------------------------------------

function FlashAnim:setBlendMode( src, dest )

   if self.anim and self.anim.props then

      for _, v in pairs(self.anim.props) do
         v:setBlendMode(src, dest);
      end
   end

end

-------------------------------------------------------------------------------------

function FlashAnim:setScale( sx, sy )

   if self.anim and self.anim.root then
      self.anim.root:setScl( sx, sy );
   end

end

----------------------------------------------------------------------

function FlashAnim:seekScale( sx, sy, time, ease )

	local action	= nil;

   if self.anim and self.anim.root then

		ease		= ease or MOAIEaseType.SOFT_SMOOTH;
		action	= self.anim.root:seekScl(sx, sy, time, ease);
   end

	return action;

end

-------------------------------------------------------------------------------------

function FlashAnim:getScl( )

   if self.anim and self.anim.root then
      return self.anim.root:getScl();
   end

end

-------------------------------------------------------------------------------------

function FlashAnim:setRot( a )

   if self.anim and self.anim.root then
      self.anim.root:setRot( a);
   end

end

-------------------------------------------------------------------------------------

function FlashAnim:getRot( )

   if self.anim and self.anim.root then
      return self.anim.root:getRot();
   end

end

-------------------------------------------------------------------------------------

function FlashAnim:setAttrLink( attrID, parent,sourceAttrID )

   if self.anim and self.anim.root then
      self.anim.root:setAttrLink( attrID , parent, sourceAttrID);
   end

end

-------------------------------------------------------------------------------------

function FlashAnim:clearAttrLink( attrID )

	if self.anim and self.anim.root then
		self.anim.root:clearAttrLink( attrID );
   end

end

-------------------------------------------------------------------------------------

function FlashAnim:setParent( parent )

   if self.anim and self.anim.root then
      self.anim.root:setParent( parent );
   end

end

----------------------------------------------------------------------

function FlashAnim:setListener( event, func )

	if self.anim then
		self.anim:setListener(event, func);
	end

end

----------------------------------------------------------------------

function FlashAnim:setVisible( visible )

   if self.anim and self.anim.props then

		for _, v in pairs(self.anim.props) do
         v:setVisible(visible);
      end
   end

end

----------------------------------------------------------------------

function FlashAnim:hide()
	self:pause();
   self:setVisible(false);
end

----------------------------------------------------------------------

function FlashAnim:show( )
   self:setVisible(true);
end

----------------------------------------------------------------------

function FlashAnim:getName()

	if self.anim and self.animId then
		return self.animId;
	end

end

----------------------------------------------------------------------

function FlashAnim:forceUpdate()

   if self.anim and self.anim.props and self.anim.root then

		self.anim.root:forceUpdate();

	  for _, v in ipairs(self.anim.props) do
		 v:forceUpdate();
	  end
   end

end

-------------------------------------------------------------------------------------
