require('engine/audio/audio-object')


----------------------------------------------------------------

local Base = AudioObject;
AudioLayer = Base:new();

----------------------------------------------------------------

function AudioLayer:new()

   local l = Base.new( self, nil, AUDIO_TYPE_LAYER, true );
   
   l.parentMusicLevel = 1;
   l.parentSoundLevel = 1;
   l:play();

   return l;
   
end

----------------------------------------------------------------

function AudioLayer:setMusicVolume( v )

   self.parentMusicLevel = v;
   
   for i=1,#self.childList do
   
      local child = self.childList[i];
      if child.type == AUDIO_TYPE_MUSIC then
      
         child:setParentVolume( v );

      elseif child.type == AUDIO_TYPE_LAYER then
      
         child:setMusicVolume( v );
      
      end
   end

end

----------------------------------------------------------------

function AudioLayer:getFinalMusicVolume()

   return (self.parentMusicLevel or 1)*self:getFinalVolume();

end

----------------------------------------------------------------

function AudioLayer:setSoundVolume( v  )
   
   self.parentSoundLevel = v;
   
   for i=1,#self.childList do
   
      local child = self.childList[i];
      if child.type == AUDIO_TYPE_SOUND then
      
         child:setParentVolume( v );

      elseif child.type == AUDIO_TYPE_LAYER then
      
         child:setSoundVolume( v );
      
      end
   end

end

----------------------------------------------------------------

function AudioLayer:getFinalSoundVolume()
   
   return (self.parentSoundLevel or 1)*self:getFinalVolume();

end

----------------------------------------------------------------

function AudioLayer:addSound( filename, memory, loop, priority )

   local sound = AudioSound:new( filename, memory, loop, priority );

   self:_addChild( sound );

   sound:setParentVolume(  self:getFinalSoundVolume()  );

   return sound;
   
end

----------------------------------------------------------------

function AudioLayer:addMusic( filename, memory, loop )

   local music = AudioMusic:new( filename, memory, loop );

   self:_addChild( music );

   music:setParentVolume( self:getFinalMusicVolume() );

   return music;
   
end

----------------------------------------------------------------

function AudioLayer:onChildAudioDone( audio )

   for i=1,#self.childList do
   
      local child = self.childList[i];
      if child == audio then
         audio.parent = nil;
         table.remove(self.childList,i);
      end
   end
   
end

----------------------------------------------------------------

function AudioLayer:_updateVolume( v )

   Base._updateVolume( self, v );
   
   local finalSound = self:getFinalSoundVolume();
   local finalMusic = self:getFinalMusicVolume();
         
   for i=1,#self.childList do

      local child = self.childList[i];
      if child.type == AUDIO_TYPE_SOUND then
      
         child:setParentVolume( finalSound );

      elseif child.type == AUDIO_TYPE_MUSIC then
      
         child:setParentVolume( finalMusic );
      
      end
   end
end

----------------------------------------------------------------

function AudioLayer:_addChild( child )

   child.parent = self;
   table.insert( self.childList, child );

end
----------------------------------------------------------------