
local CATCH_ALL   = "__CATCH_ALL__";
local ADD_LISTENER= 1;
local RMV_LISTENER= 2;
local POST_EVENT  = 3;

local DEBUG_EVENT = false;

----------------------------------------------------------------

EventMgr =
{
   listeners   = {},
   guard       = false,
   requests    = { {}, {} },
   requestIdx  = 1,
};

----------------------------------------------------------------

function EventMgr:update( dt )

   local requests = self.requests[self.requestIdx];

   if #requests > 0 then

      local index = self.requestIdx;

      if self.requestIdx == 1 then
         self.requestIdx = 2;
      else
         self.requestIdx = 1;
      end

      for i=1,#requests do

         local request = requests[i];
         local type    = request[1];

         if type == ADD_LISTENER then

            self:_addListener( request[2], request[3], request[4]  );

         elseif type == RMV_LISTENER then

            self:_rmvListener( request[2], request[3]);

         elseif type == POST_EVENT then

            self:_postEvent( request[2], request[3]);

         else

            print("Error... wrong event request type", type);

         end
      end

      self.requests[index] = {};


   end
end

----------------------------------------------------------------

function EventMgr:addListener( listener, callback, type )

   Debug.assert( callback, "EventMgr:addListener -> callback : " .. tostring(type) );

   table.insert( self.requests[self.requestIdx], { ADD_LISTENER, listener, callback, type } );

end

----------------------------------------------------------------

function EventMgr:rmvListener( listener, type )

   table.insert( self.requests[self.requestIdx], { RMV_LISTENER, listener, type  } );

end


----------------------------------------------------------------

function EventMgr:postEvent( event, type )

   Debug.assert( type ~= nil, "EventMgr:postEvent -> type ~= nil" );

   if DEBUG_EVENT then
      print("EventMgr:postEvent", type);
   end

   table.insert( self.requests[self.requestIdx], { POST_EVENT, event, type  } );

end

----------------------------------------------------------------

function EventMgr:_addListener( listener, callback, type  )

   if type == nil then

      type = CATCH_ALL;

   end

   if self.listeners[type] == nil then

      self.listeners[type] = {};

   end

   table.insert( self.listeners[type], { listener=listener,callback=callback} );

end

----------------------------------------------------------------

function EventMgr:_rmvListener( listener, type )

   if type then

      self:_rmvListenerFromList(listener, self.listeners[type]);

   else

      for k,list in pairs(self.listeners) do

         self:_rmvListenerFromList( listener, list );
      end
   end
end

----------------------------------------------------------------

function EventMgr:_rmvListenerFromList( listener, list )

   if list and #list > 0 then

      for i=#list,1,-1  do

         if list[i].listener == listener then

            table.remove(list,i);

         end
      end
   end
end

----------------------------------------------------------------

function EventMgr:_postEvent( event, type )

   self:_postEventToList( event, type, self.listeners[type]      );
   self:_postEventToList( event, type, self.listeners[CATCH_ALL] );

end

----------------------------------------------------------------

function EventMgr:_postEventToList( event, type, list  )

   if list and #list > 0 then

      for i=1, #list do

         local entry    = list[i];
         local listener = entry.listener;
         local callback = entry.callback;

         if callback then
            callback( listener, event, type );
         else
            Debug.assert(callback, "EventMgr:_postEventToList -> callback : " .. tostring(type) );
         end

      end
   end
end


----------------------------------------------------------------
