
FontMgr =
{
   fonts = {};
};

----------------------------------------------------------------

local DPI = 163;

----------------------------------------------------------------

-- getMetaFont differs from getFont in the sense that you will often not get the exact font you want.
-- It returns a table with two variables: font and scale
-- The second returned variable is a factor representing the percentage you need to scale by. 1.0 == 100%
-- This is primarily used in ui-text-box.lua
function FontMgr:getMetaFont(fontName, size, lettersToCache)

   assert(fontName and size, "fontName or size cannot be nil.");
   assert(size >= 0 and size < 256);

   local origSize = size;

   -- Find the closest acceptable match for font size.
   if size <= 10 then
      size = 10.0;
   elseif size <= 20 then
	  -- XXX
	  -- This was changed from 15 to 16 because state-signin-tango prerenders username glyphs at 16 point font size.
	  -- However, the rest of the game was using 15, causing non-Latin glyphs to not appear.
	  -- The ideal solution would be to make the state request that the font manager do the glyph prerendering.
	  -- The font manager would then prerender them at the appropriate size.
      size = 16.0;
   elseif size <= 30 then
      size = 22.5;
   elseif size <= 40 then
      size = 30.0;
   else
      size = 50;
   end

   -- Calculate required scale factor
   local scaleFactor = origSize / size;

   -- These characters will always be rendered.
   local defaultCharSet = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789,.'?!: -$+/*%•";

   if lettersToCache then
	  lettersToCache = lettersToCache .. defaultCharSet;
   else
	  lettersToCache = defaultCharSet;
   end

   return { font=self:_getFont(fontName, size, lettersToCache),
            scale=scaleFactor };

end

----------------------------------------------------------------

function FontMgr:_getFont( fontName, size, letters )

   local id    = fontName..size;
   local font  = self.fonts[id];

   if font == nil then

      font = MOAIFont.new ()

      font.oldPreloadGlyph = font.preloadGlyphs;
      font.preloadGlyphs   =
         function (  self, letters, size )
            self:oldPreloadGlyph( letters, size or self._size, DPI );
         end

      if MOAIFileSystem.checkFileExists(fontName) then
         font:load( fontName  );
      elseif MOAIEnvironment.obbDirectory and MOAIFileSystem.checkFileExists(createFullFilename(MOAIEnvironment.obbDirectory, fontName)) then
         font:load(createFullFilename(MOAIEnvironment.obbDirectory, fontName));
      else
         Debug.error("Missing font: " .. tostring(fontName));
      end

      font:setCache( MOAIGlyphCache.new () )
      font:setReader( MOAIFreeTypeFontReader.new () )
      font:setDefaultSize( size, DPI );
      font._size = size;

      self.fonts[id] = font;

   end

   if font then
      font:preloadGlyphs( letters, size );
   end

   return font;

end

----------------------------------------------------------------
