
local TOUCH_STATE_PRESSED  = 1;
local TOUCH_STATE_DOWN     = 2;
local TOUCH_STATE_RELEASED = 3;

InputMgr =
{
   --For now, we only track one touch... 
   -- We will move to multi-touch later on
   enable       = true,
   touch        = nil,
   touchState   = nil,
   touchLoc     = { 0, 0 },
};

-----------------------------------------------------------------------

local DEFAULT_FAKE_TOUCH_HOLD_TIME = .1;

local FakeTouch = {}
FakeTouch.__index = FakeTouch;

function FakeTouch:isDown()
   Debug.assert(self.fakeTouchHoldTimer);
   Debug.assert(self.fakeTouchTimeLimit);

   return self.fakeTouchHoldTimer < self.fakeTouchTimeLimit;
end

function FakeTouch:getLoc()
   return self.x, self.y;
end

function InputMgr:_createFakeTouchUserdatum()
   local touch = {};
   setmetatable(touch, FakeTouch);

   touch.x = 0;
   touch.y = 0;

   return touch;
end

-----------------------------------------------------------------------

function InputMgr:update( dt )

	if not self.enable then
		return;
	end

   local touch = MOAIRoadRiot.getInputManager():getTouch();

   -- Handle fake taps/swipes
   if self.touch and self.touch.fakeTouchTimeLimit then

	  if self.touch:isDown() then

		 if self.touch.swipe then
			-- Update x/y
			self.touch.x = math.lerp(self.touch.startX, self.touch.endX, self.touch.fakeTouchHoldTimer);
			self.touch.y = math.lerp(self.touch.startY, self.touch.endY, self.touch.fakeTouchHoldTimer);
            print("Update swipe. Position is now: ", self.touch.x, self.touch.y);
		 end

		 self:_holdDownTouch();
		 self.touch.fakeTouchHoldTimer = self.touch.fakeTouchHoldTimer + dt;
	  else
		 self:_releaseTouch();
	  end

   elseif touch == self.touch then

      if self.touch then
         if self.touch:isDown() then
			self:_holdDownTouch();
         else
			self:_releaseTouch();
         end

      else
         self.touchState = nil;
      end

   else

      --Make sure we get the message about releasing the press
      if self.touch then
		 self:_releaseTouch();
      else
         if touch and touch:isDown() then
			self:_pressTouch(touch)
         end
      end
   end

end

-----------------------------------------------------------------------

function InputMgr:_releaseTouch()

   Debug.assert(self.touch, "Failed to release touch!");

   self.touchState = TOUCH_STATE_RELEASED;
   self.touchLoc   = { self.touch:getLoc() };
   self.touch      = nil;

   -- print("Releasing touch at: " .. self.touchLoc[1] .. ", " .. self.touchLoc[2]);

end

-----------------------------------------------------------------------

function InputMgr:_pressTouch(touch)
   self.touchState = TOUCH_STATE_PRESSED;
   self.touchLoc   = { touch:getLoc() };
   self.touch      = touch;
end

-----------------------------------------------------------------------

function InputMgr:_holdDownTouch()
   self.touchState = TOUCH_STATE_DOWN;
   self.touchLoc   = { self.touch:getLoc() };
end

-----------------------------------------------------------------------

function InputMgr:generateTouch(x, y, holdTime)

   self.touchState = TOUCH_STATE_PRESSED;
   self.touchLoc = { x, y };

   self.touch = self:_createFakeTouchUserdatum();
   self.touch.x = x;
   self.touch.y = y;
   self.touch.fakeTouchTimeLimit = holdTime or DEFAULT_FAKE_TOUCH_HOLD_TIME;
   self.touch.fakeTouchHoldTimer = 0;
   self.touch.swipe = false;

   print("Generating touch at: " .. x .. ", " .. y);

end

function InputMgr:generateSwipe(startX, startY, endX, endY, swipeTime)

   self.touchState = TOUCH_STATE_PRESSED;
   self.touchLoc = { startX, startY };

   self.touch = self:_createFakeTouchUserdatum();
   self.touch.startX = startX;
   self.touch.startY = startY;
   self.touch.endX   = endX;
   self.touch.endY   = endY;
   self.touch.x = startX;
   self.touch.y = startY;
   self.touch.fakeTouchTimeLimit = swipeTime;
   self.touch.fakeTouchHoldTimer = 0;
   self.touch.swipe = true;

end

-----------------------------------------------------------------------

function InputMgr:getTouchLoc()

   return self.touchLoc[X],self.touchLoc[Y];

end

-----------------------------------------------------------------------

function InputMgr:wasTouchPressed()

   return self.touchState == TOUCH_STATE_PRESSED;

end

-----------------------------------------------------------------------

function InputMgr:wasTouchReleased()

   return self.touchState == TOUCH_STATE_RELEASED;

end

-----------------------------------------------------------------------

function InputMgr:isTouchDown()

   return self.touchState == TOUCH_STATE_DOWN;

end

-----------------------------------------------------------------------

function InputMgr:isInputEnabled()

	return self.enable;

end

-----------------------------------------------------------------------

function InputMgr:disable()

	self.enable	= false;

end

-----------------------------------------------------------------------

function InputMgr:enable()

	self.enable	= true;

end

-----------------------------------------------------------------------
