require('engine/object/object')
require('engine/mission/mission-goal')

----------------------------------------------------------------------

local MISSION_DATA_VERSION = 1;

----------------------------------------------------------------------

Mission =
{
};

----------------------------------------------------------------------

function Mission:new( template  )

   local mission     = Object.new( self );

   mission.template  = template;
   mission.active    = false;
   mission.completed = false;

   mission.goals     = {};

   for i=1,#template.goals do

      local tplGoal = template.goals[i];

      mission.goals[i] = tplGoal.factory:new( mission, i, tplGoal );

   end

   return mission;

end

----------------------------------------------------------------------

function Mission:delete()

end

----------------------------------------------------------------------

function Mission:activate()

   if not self.active then

      self.active = true;

      for i=1,#self.goals do

         if not App.data:isCurrentMissionGoalCompleted( i ) then

            self.goals[i]:activate();
            
         else
         
            self.goals[i]:markAsCompleted();

         end
      end
   end

end

----------------------------------------------------------------------

function Mission:deactivate()

   if self.active then

      self.active = false;

      for i=1,#self.goals do
         if self.goals[i]:isActive() then
            self.goals[i]:deactivate();
         end
      end
   end

end

----------------------------------------------------------------------

function Mission:isActive()

   return self.active;

end

----------------------------------------------------------------------

function Mission:onGoalCompleted( goal )

   EventMgr:postEvent( {  goal=goal }, "mission-goal-completed" );
   
   if self:_IsCompleted() then
		
		print("Mission", self.template.name, "Completed");
      self.completed = true;
	
      EventMgr:postEvent( { mission=self }, "mission-completed" );

   end
   
   
end

----------------------------------------------------------------------

function Mission:isCompleted()

   return self.completed;

end

----------------------------------------------------------------------

function Mission:_IsCompleted()

   for i=1,#self.goals do
      if not self.goals[i]:isCompleted() then
         return false;
      end
   end

   return true;

end

----------------------------------------------------------------------