require("engine/network/connection-queue")

require("engine/network/web_request")
require("engine/network/push_web_request")
require("engine/network/tango_request")

-- Use this manager to make HTTP requests
-- This class in turn uses the ConnectionPool class to prevent opening too many connections at once.

-- TODO This would be a good place to move the OnlineMgr functionality to.

NetworkMgr = {}


function NetworkMgr:init()

   self._connectionQueue = ConnectionQueue.new();

   Debug.assert(self._connectionQueue);

end


function NetworkMgr:update(dt)

   self._connectionQueue:update(dt);

end

-- Request should contain:
-- * URL
-- * Retry count
-- * Priority
-- Request can optionally contain:
-- * timeoutTime
-- * callHandler (required for the next two entries)
-- * successFunc
-- * failureFunc
function NetworkMgr:sendWebRequest(request)

   print("NetworkMgr:sendWebRequest");

   Debug.assertType(request, {"table", "userdata"}, "NetworkMgr:sendWebRequest -> request");
   Debug.assert(request.typeName == "WebRequest");

   self._connectionQueue:queueRequest(request);

end

function NetworkMgr:sendPushWebRequest(request)

   print("NetworkMgr:sendPushWebRequest");

   Debug.assertType(request, {"table", "userdata"}, "NetworkMgr:sendPushWebRequest -> request");
   Debug.assert(request.typeName == "PushWebRequest");

   self._connectionQueue:queueRequest(request);

end

function NetworkMgr:sendTangoRequest(request)

   Debug.assertType(request, {"table", "userdata"}, "NetworkMgr:sendTangoRequest -> request");
   Debug.assert(request.typeName == "TangoRequest");

   self._connectionQueue:queueRequest(request);

end
