require("engine/utility/stringUtils");

TextureMgr =
{
   textures = {};
};

----------------------------------------------------------------

function TextureMgr:_createMiniFilename( fileName )

   return fileName .. "@mini.png";

end

----------------------------------------------------------------

function TextureMgr:getFinalFileName( fileName )

   local miniFilename = self:_createMiniFilename( fileName );

   if  App.useMiniTexture and MOAIFileSystem.checkFileExists ( createFullFilename(MOAIFileSystem.getWorkingDirectory (),miniFilename) ) then
      return miniFilename;
   else
      return fileName;
   end

end

----------------------------------------------------------------

function TextureMgr:_createTextureFromUserImage( filename, linear, isPathAbsolute )

   local workingDir;
   if not isPathAbsolute then
	  workingDir = MOAIFileSystem.getWorkingDirectory ()
	  MOAIFileSystem.setWorkingDirectory ( MOAIEnvironment.documentDirectory )
   end

   local image = MOAIImage.new ()
   image:load(filename);

   if not isPathAbsolute and workingDir then
	  MOAIFileSystem.setWorkingDirectory ( workingDir )
   end

   if image then

      local x1, y1 = image:getSize();

      image = image:padToPow2();

      local x2, y2 = image:getSize();

      local texture = MOAITexture.new ()

      if linear then
         texture:setFilter( MOAITexture.GL_LINEAR,  MOAITexture.GL_LINEAR);
      end

      texture:load(image);
      texture._uv = { xMin = 0, yMin = 0, xMax = x1/x2, yMax = y1/y2 };

      return texture;

   end

end

----------------------------------------------------------------

function TextureMgr:_createTextureTEST( filename, linear, isPathAbsolute )

   local workingDir;
   if not isPathAbsolute then
	  workingDir = MOAIFileSystem.getWorkingDirectory ()
	  MOAIFileSystem.setWorkingDirectory ( MOAIEnvironment.documentDirectory )
   end

   local texture = MOAITexture.new ()

   if linear then
      texture:setFilter( MOAITexture.GL_LINEAR,  MOAITexture.GL_LINEAR);
   end

   texture:load(filename);

   if not isPathAbsolute and workingDir then
	  MOAIFileSystem.setWorkingDirectory ( workingDir )
   end

   return texture;

end

----------------------------------------------------------------

function TextureMgr:getTexture( fileName, linear )

   local texture = self.textures[fileName]

   if texture == nil then

      if linear == nil then
         linear = true;
      end

      local finalFileName = self:getFinalFileName( fileName );

      if MOAIFileSystem.checkFileExists ( createFullFilename(MOAIFileSystem.getWorkingDirectory (),finalFileName) ) then

         texture = MOAITexture.new();

         --This should work  but doesnt... if we need to switch Moai source
         -- and we cannot "changed" the global behavior of the code, this should fix it (assuming you can make this work)
         if linear then
            texture:setFilter( MOAITexture.GL_LINEAR,  MOAITexture.GL_LINEAR);
         end

         texture:load(finalFileName);

      elseif MOAIFileSystem.checkFileExists ( createFullFilename(MOAIEnvironment.documentDirectory, fileName ) ) then

		 texture = self:_createTextureFromUserImage( fileName, linear, false );

	  elseif MOAIFileSystem.checkFileExists ( fileName ) then

		 -- Last ditch effort: don't prepend a path
		 texture = self:_createTextureFromUserImage( fileName, linear, true );

      elseif MOAIEnvironment.obbDirectory and MOAIFileSystem.checkFileExists ( createFullFilename(MOAIEnvironment.obbDirectory, fileName ) ) then

         texture = self:_createTextureTEST( createFullFilename(MOAIEnvironment.obbDirectory, fileName), linear, true );

	  else

         -- Filter out certain paths to avoid spamming Flurry.
         local blacklistedPatterns = {
            "/private/var/mobile/Containers/Data/Application/",
            "/private/var/mobile/Applications",
            "picture-__Guest_Friend_",
         }

         if StringUtils.containsAny(tostring(fileName), blacklistedPatterns) then
            Debug.error("Failed to load texture from any source. (fileName redacted to reduce spam)");
         else
            Debug.error("Failed to load texture from any source. fileName: " .. tostring(fileName));
         end
      end

      self.textures[fileName] = texture;

   end

   return texture;

end

----------------------------------------------------------------
