-- Taken from Nick Caplinger's personal Lua files. :)

-- An implementation of map, filter, and reduce - three higher-order functions core to functional programming.
-- https://en.wikipedia.org/wiki/Map_%28higher-order_function%29
-- https://en.wikipedia.org/wiki/Filter_%28higher-order_function%29
-- https://en.wikipedia.org/wiki/Fold_%28higher-order_function%29

function map(func, items)
   Debug.assert(type(func) == "function")

   local result = {}

   if items then
      -- Avoid table lookup
      local ins = table.insert

      for _, v in ipairs(items) do
         ins(result, func(v))
      end
   end

   return result
end

function filter(func, items)
   Debug.assert(type(func) == "function")

   local result = {}

   if items then
      -- Avoid table lookup
      local ins = table.insert

      for _, v in ipairs(items) do
         if func(v) then
            ins(result, v)
         end
      end
   end

   return result

end

function reduce(func, items)
   Debug.assert(type(func) == "function")

   if items == nil or #items == 0 then
      return nil
   end

   local result = items[1]

   for i = 2, #items do
      result = func(result, items[i])
   end

   return result
end
