
-------------------------------------------------------------------------------------

math.sign = function( v )

   if v < 0 then

      return -1;

   end

   return 1;

end

-------------------------------------------------------------------------------------

math.square = function( v )

   return v*v;

end

-------------------------------------------------------------------------------------

math.clamp = function( v, min, max )

   if v < min then

      return min;

   elseif v > max then

      return max;

   else

      return v;

   end
end

-------------------------------------------------------------------------------------

math.clampAngle180 = function( a  )

   while  a>=180.0  do
      a = a-360.0;
   end

   while a < -180.0 do
      a = a + 360.0;
   end

   return a;

end

-------------------------------------------------------------------------------------

math.clampAngle360 = function( a  )

   while  a>=360  do
      a = a-360.0;
   end

   while a < 0 do
      a = a + 360.0;
   end

   return a;

end

-------------------------------------------------------------------------------------

math.distance = function (x1, y1, x2, y2)

	local deltax = math.square(x2 - x1);
	local deltay =	math.square(y2- y1);
	
	return math.sqrt(deltax + deltay);

end

-------------------------------------------------------------------------------------

math.distancesqr = function (x1, y1, x2, y2)

	local deltax = math.square(x2 - x1);
	local deltay =	math.square(y2- y1);
	
	return deltax + deltay;

end

-------------------------------------------------------------------------------------

math.isInBoundary = function ( x, y, xMin, xMax, yMin, yMax )

   return      ( y >= yMin and y <= yMax )
         and   ( x >= xMin and x <= xMax );


end

-------------------------------------------------------------------------------------

math.lerp = function( v1, v2, k )

   return v1*(1-k) + v2*k;

end

-------------------------------------------------------------------------------------

math.roundToDecimal = function( num, idp )

	local mult = 10^(idp or 0);

	return math.floor( num * mult + 0.5 ) / mult;

end

-------------------------------------------------------------------------------------

math.smoothLerp = function( edge0,  edge1,  x )

   if edge0 == edge1 then

      return edge1;

   else

      x = math.lerp( edge0,  edge1,  x );
      -- Scale, and saturate x to 0..1 range
      x = math.clamp((x - edge0)/(edge1 - edge0),0,1);
      -- Evaluate polynomial
      x = (x * x * (3.0 - 2.0 * x));

      return math.lerp( edge0,  edge1,  x );
      
   end
end

-------------------------------------------------------------------------------------

math.smootherLerp = function( edge0,  edge1,  x )
  
   if edge0 == edge1 then

      return edge1;

   else

      x = math.lerp( edge0,  edge1,  x );
      -- Scale, and saturate x to 0..1 range
      x = math.clamp((x - edge0)/(edge1 - edge0),0,1);
      -- Evaluate polynomial
      x = x*x*x*(x*(x*6 - 15) + 10);

      return math.lerp( edge0,  edge1,  x );
      
   end
end

-------------------------------------------------------------------------------------

math.smoothStep = function( a,  b,  x )

   if a == b then

      return a;

   else

      local x2 = math.clamp((x-a)/(b-a),0,1);
   
      return (x2 * x2 * (3.0 - 2.0 * x2));
   
   end
end

-------------------------------------------------------------------------------------

-- v == time/duration

math.easeInQuad = function( v, edge0, edge1 )

   local delta = edge1-edge0;

   return delta * math.pow(v, 2) + edge0;

end

-------------------------------------------------------------------------------------

-- v == time/duration

math.easeOutQuad = function( v, edge0, edge1 )

   local delta = edge1-edge0;

   return -delta * v * (v - 2) + edge0;

end

-------------------------------------------------------------------------------------

-- v == time/duration

math.easeInOutQuad = function( v, edge0, edge1 )

   local delta = edge1-edge0;
   local time  = v * 2;

   if time < 1 then
      return delta / 2 * math.pow(time, 4) + edge0;
   else
      time = time - 2;

      return -delta / 2 * (math.pow(time, 4) - 2) + edge0;
   end

end

-------------------------------------------------------------------------------------

math.mapClamped = function( in1, out1, in2, out2, v )

   local rel   = 0;

   if in2 > in1 then

      local rel = math.clamp( (v-in1) / (in2-in1) , 0, 1 );

      return math.lerp( out1, out2, rel);

   elseif in2 < in1 then

      local rel = math.clamp( (v-in2) / (in1-in2) , 0, 1 );

      return math.lerp( out2, out1, rel);

   else
      return out1;
   end

end

-------------------------------------------------------------------------------------

--return time (given in seconds) as total hours, minutes, and seconds
math.getTimeTable = function( time )

   local t  =
   {
      hour  = 0,
      min   = 0,
      sec   = 0,
   };

   if time then
      t.hour   = math.floor( time / (60*60) );
      t.min    = math.floor( (time - (t.hour*60*60))/60 );
      t.sec    = math.floor( time - (t.hour*60*60) - (t.min*60) );
   end

   return t;

end

-------------------------------------------------------------------------------------
