
-------------------------------------------------------------------------------------

function areTblKeysMatch( tblA, tblB )

	Debug.assert(tblA and type(tblA) == "table");
	Debug.assert(tblB and type(tblB) == "table");

	for k,v in pairs(tblA) do
		if tblB[k] == nil then
			return false;
		end
	end

	for k,v in pairs(tblB) do
		if tblA[k] == nil then
			return false;
		end
	end

	return true;

end

-------------------------------------------------------------------------------------

function removeIf(array, predicate)
   for i=#array, 1, -1 do
	  if predicate(array[i]) then
		 table.remove(array, i)
	  end
   end
end

-------------------------------------------------------------------------------------

function isTableEmpty(t)
   return next(t) == nil;
end

-------------------------------------------------------------------------------------

-- Returns a shuffled version of table t
-- Because this is not an in-place shuffle, this may be slow for larger tables
-- Keys must be numbers (i.e. the table must be an array)
-- http://en.wikipedia.org/wiki/Fisher%E2%80%93Yates_shuffle#The_.22inside-out.22_algorithm
function makeShuffled(t)

   local result = {};

   -- Store the function to avoid lookups
   local rand = math.random;

   for i = 1, #t do
	  local j = rand(i);

	  if j ~= i then
		 result[i] = result[j];
	  end

	  result[j] = t[i];
   end

   return result;
end

-------------------------------------------------------------------------------------

function pickWeightedRandom(valueTable, weightTable)

   Debug.assert(valueTable and type(valueTable) == "table");
   Debug.assert(weightTable and type(weightTable) == "table");

   local pick = nil;
   local sum = 0;

   -- Optimization: store the function to avoid repeated table lookups
   local randFunc = math.random;

   for k, value in pairs(valueTable) do

      local weight = weightTable[k];

      if weight and weight > 0 then
         if value then

			local randNum = randFunc();

			sum = sum + weight;

			if (sum * randNum) <= weight then
			   pick = value;
			end
         end
      end
   end

   return pick;

end

-------------------------------------------------------------------------------------

function getTableKeys(tbl)

   local result = {};

   for k, _ in pairs(tbl) do
	  table.insert(result, k);
   end

   return result;
end

-------------------------------------------------------------------------------------

local function _countTableDifferences(tbl1, tbl2)

   Debug.assertType(tbl1, {"table", "nil"}, "tbl1");
   Debug.assertType(tbl2, {"table", "nil"}, "tbl2");

   if type(tbl1) ~= "table" or type(tbl2) ~= "table" then
      return 1;
   end

   local diffCount = 0;

   for k, v in pairs ( tbl1 ) do

      if type(v) == "table" then
         diffCount = diffCount + ( _countTableDifferences(v, tbl2[k]) or 0 );
      elseif v ~= tbl2[k] then
         diffCount = diffCount + 1;
      end
   end

   return diffCount;

end

function countTableDifferences(tbl1, tbl2)

   Debug.assertType(tbl1, "table", "tbl1");
   Debug.assertType(tbl2, "table", "tbl2");

   if type(tbl1) ~= "table" or type(tbl2) ~= "table" then
      return nil;
   end

   return _countTableDifferences(tbl1, tbl2) + _countTableDifferences(tbl2, tbl1);

end

-------------------------------------------------------------------------------------

function mixItemsIntoListAtInterval(items, list, interval)

   Debug.assert(interval and interval > 0, "Interval must be greater than zero.");
   Debug.assert(items and #items > 0, "No items to mix into list.");
   Debug.assert(list and #list > 0, "The list was empty and therefore no items can be mixed in.");

   local numItemsInserted = 0;

   if interval < #list then

	  -- Go one past the end to add one to the end if the interval allows for it
	  local i = interval;

	  while i <= #list + 1 do

		 table.insert(list, i, items[numItemsInserted + 1]);
		 numItemsInserted = numItemsInserted + 1

		 -- Stop when we don't have more items to mix in
		 if numItemsInserted == #items then
			break;
		 end

		 i = i + interval;

	  end

   else

	  table.insert(list, items[1]);
   end

end

----------------------------------------------------------------------

function copyTable( tbl, visited )

   visited = visited or {};
   local result = {};

   for k, v in pairs (tbl) do

      if type(v) == "table" and (not visited[v]) then

         visited[v] = true;
         result[k]  = copyTable(v, visited);
      else
         result[k] = v
      end

   end

   return result;
end

function hasValue(tbl,val)
    for index, value in ipairs(tbl) do
        if value == val then
            return true
        end
    end

    return false
end
