local AnimPlayer = {}

local function insertProps ( self, layer )

   if layer then
      for _, v in ipairs ( self.props ) do
         
         layer:insertProp ( v )
      end
      
      for _, layerOps in pairs( self.embSyms ) do
         for _, sym in pairs ( layerOps ) do
            sym.player:insertProps( layer )
         end
      end
   end
end

local function removeProps ( self, layer )

   if layer then
      for _, v in ipairs ( self.props ) do
         layer:removeProp ( v )
      end
      
      for _, layerOps in pairs( self.embSyms ) do
         for _, sym in pairs ( layerOps ) do
            sym.player:removeProps( layer )
         end
      end
   end
   
end

local function pauseLastFrame (self, pause)

	if pause then
		self:setSpan ( self.sym.span - self.sym.frame );
	else
		self:setSpan ( self.sym.span );
	end

end

local function addLayers ( self, player, parent, sym, props )
	
	for lid, layer in pairs ( sym.layers ) do
		
    local curveSet = layer.curves
		local prop
		
		if ( layer.isEmbeddedSymbol ) then
			prop = MOAIProp2D.new() -- MOAITransform2D.new ()
						
		else
			prop = MOAIProp2D.new ()
			
			prop:setDeck ( self.brushDeck.quadDeck )
			prop:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA )
			player:setLink ( player.nextLinkIdx, curveSet.id, prop, MOAIProp2D.ATTR_INDEX )		
			player.nextLinkIdx = player.nextLinkIdx + 1
		end
		
		prop:setParent ( parent )
		props [ lid ] = prop
		
      local curIndex = 0
		
      if curveSet.x then
         player:setLink ( player.nextLinkIdx + curIndex, curveSet.x,  prop, MOAITransform.ATTR_X_LOC )
         curIndex = curIndex + 1
      end

      if curveSet.y then
         player:setLink ( player.nextLinkIdx + curIndex, curveSet.y,  prop, MOAITransform.ATTR_Y_LOC )
         curIndex = curIndex + 1
      end

      if curveSet.r then
         player:setLink ( player.nextLinkIdx + curIndex, curveSet.r,  prop, MOAITransform.ATTR_Z_ROT )
         curIndex = curIndex + 1
      end

      if curveSet.sx then
         player:setLink ( player.nextLinkIdx + curIndex, curveSet.sx, prop, MOAITransform.ATTR_X_SCL )
         curIndex = curIndex + 1
      end

      if curveSet.sy then
         player:setLink ( player.nextLinkIdx + curIndex, curveSet.sy, prop, MOAITransform.ATTR_Y_SCL )
         curIndex = curIndex + 1
      end

      if curveSet.a then
         player:setLink ( player.nextLinkIdx + curIndex, curveSet.a,  prop, MOAIColor.ATTR_A_COL )
         curIndex = curIndex + 1
      end

      if curveSet.vis then
         player:setLink ( player.nextLinkIdx + curIndex, curveSet.vis,  prop, MOAIProp.ATTR_VISIBLE )
         curIndex = curIndex + 1
      end

		if curveSet.px then
			player:setLink ( player.nextLinkIdx + curIndex, curveSet.px,  prop, MOAITransform.ATTR_X_PIV )
         curIndex = curIndex + 1
      end
      
      if curveSet.py then
         player:setLink ( player.nextLinkIdx + curIndex, curveSet.py,  prop, MOAITransform.ATTR_Y_PIV )
         curIndex = curIndex + 1
      end

		player.nextLinkIdx = player.nextLinkIdx + curIndex

	end
end

local function getElementsInTable ( tab )
	local count = 0
	for _ in pairs( tab ) do count = count + 1 end
	return count
end

local function countAnimationLinks ( self, sym )
	
	local animLinksCount = 0
	
	for _, layer in pairs ( sym.layers ) do
		animLinksCount = animLinksCount + getElementsInTable ( layer.curves )
	end
	
	return animLinksCount
end

local function onKeyframe ( self, _, _, time )
--	require('mobdebug').on() 
--	printf ( "%s: keyframe %d: timesExecuted=%f, time=%f, value=%f\n", self.test, keyframe, timesExecuted, time, value )
	local ops = self.embSymOp [ self.resBundle.roundTime( time ) ]
	
	if ops == nil and time > 0.01 then
		print ('WARNING: op not found')
	end
	
	if ops ~= nil then
		for _, op in ipairs( ops ) do
			local player = self.embSyms [ op.symId ] [ op.layerId ].player
			if op.vis then
				player:setTime ( 0 )
				player:apply ( 0 )
				
				player:start ( self )				
			else
				player:stop( )
			end
		end
	end
end


AnimPlayer.new = function ( resBundle, sid, parentTransform )
  
	local sym = resBundle.syms [ sid ]

	local player = MOAIAnim.new ()
	if sym.embSymOp ~= nil then
		player:setCurve ( sym.embSymsCurve )		
		player:setListener ( MOAITimer.EVENT_TIMER_KEYFRAME, onKeyframe )
	end
	
	player.nextLinkIdx = 1
	
	local animLinksCount = countAnimationLinks ( resBundle, sym )
	player:reserveLinks ( animLinksCount )	
	
	local props = {}
	addLayers ( resBundle, player, parentTransform, sym, props )

	player.root = parentTransform
	player.props = props
	player.resBundle = resBundle
	player.sym   = sym
	player.test = sym.name
	player.embSymOp = sym.embSymOp
	player.embSyms = {}
	
	if player.embSymOp ~= nil then
		for _, ops in pairs ( player.embSymOp ) do
			for _, op in ipairs ( ops ) do
				if player.embSyms [ op.symId ] == nil then
					player.embSyms [ op.symId ] = {}
				end
				if player.embSyms [ op.symId ] [ op.layerId ] == nil then
					player.embSyms [ op.symId ] [ op.layerId ] = {}
					if ( props [ op.layerId ] == nil ) then 
						error ( 'Parent prop is nil' )
					end
						
					local embPlayer = AnimPlayer.new ( resBundle, op.symId, props [ op.layerId ] )
					if op.loop == 1 then
						embPlayer:setMode( MOAITimer.LOOP )
					end
					player.embSyms [ op.symId ] [ op.layerId ].player = embPlayer
				end
			end
		end
	end
	
	player.insertProps = insertProps
	player.removeProps = removeProps
	player.pauseLastFrame = pauseLastFrame
	
	player:setSpan ( sym.span )
	player:apply ( 0 )
	
	return player
end



return AnimPlayer