require('game/entity/avatar/avatar-armor');
require('game/entity/avatar/avatar-main-weapon');
require('game/entity/avatar/avatar-paint');
require('game/entity/avatar/avatar-side-weapon');
require('game/entity/turret/turret');

----------------------------------------------------------------------

local HACK_CANT_SHOOT                        =  false;
local HACK_ALWAYS_SHOOT                      =  false;
local HACK_CANT_DIE                          =  false;
local HACK_AVATAR_INVICIBLE                  =  false;
local AVATAR_MOVE_K                          =  { 1.00, 2.00 };
local DEAD_SPEED_K                           =  30*0.90;
local AVATAR_HIT_LOOSE_CTRL                  =  0.05;
local AVATAR_HIT_REGAIN_CTRL                 =  0.55;
local AVATAR_COLLISION_EXPEL_K               =  { 2.00,      2.00     };
local AVATAR_COLLISION_IMPULSE_K             =  { 1.00,      1.00     };
local AVATAR_COLLISION_IMPULSE_INERTIA_K     =  { 0.75*30,   0.825*30 };
local AVATAR_COLLISION_IMPULSE_DELTA_K       =  { 0.05,      0.10     };
local AVATAR_COLLISION_MOVE_K                =  { 1.00,      1.00     };
local AVATAR_COLLISION_MOVE_BARRICADE_K      =  { 1.00,      1.00     };
local AVATAR_COLLISION_MOVE_BARRICADE_EXTRA  =  {20.00,      0.00     };
local AVATAR_COLLISION_MOVE_INERTIA_K        =  { 30*0.975,  30*0.95  };
local AVATAR_MIN_TIME_BETWEEN_COLLIDE        =  0.05;
local AVATAR_FLYOUT_INTERVAL_MIN             =  0.25;
local AVATAR_TOUCH_DELTA_Y                   =  60;

local MOUNT_POINT_LOCKED							= -1;
local MOUNT_POINT_EMPTY								= 0;
local MOUNT_POINT_LEFT								= "left";
local MOUNT_POINT_RIGHT								= "right";

local SHIELD_DRAIN_TICK_TIME                 = 0.5;  --seconds to tick shield drain
local SHIELD_DRAIN_TICK_DAMAGE               = 1/60; --damage scale of max shield hp to do per tick (1/60 health per 0.5s tick will remove shields in 30s)
local SHIELD_DRAIN_DELAY                     = 1; --delay before beginning shield drain

----------------------------------------------------------------------

local Base  = Entity;
Avatar      = Base:new();

----------------------------------------------------------------------

function Avatar:new( game, layers, template, x, y, a, boundary, tplPaint, tplMainWeapon, tplArmor, tplSideWeapon, tplTurret )

   local a = Base.new( self, game, layers[template.layer or 3], template, x, y );

	a.isAvatar				= true;

   a.mainWeapon         = AvatarMainWeapon:new( game, layers[template.layer or 3], tplMainWeapon, self);
   a.paint              = AvatarPaint:new(      game, layers[tplPaint.layer or 3], tplPaint,      self);
   a.armor              = AvatarArmor:new(      game, layers[tplArmor.layer or 3], tplArmor,      self);

   if template.shadow then
      a.shadow = Entity:new( game, layers[template.shadow.layer], template.shadow, x, y, 0 );
   end

   a.secretScoreK       = template.secretScoreK or 1;
   a.powerups           = {};
   a.impulseDelta       = { 0, 0 }
   a.boundary           = boundary;
   a.input              = { down = false, myAnchorPos, inSmoothPos };
   a.hitSlowTimer       = 100000;
   a.timeSinceCollide   = 0;
   a.speedBurst         = 0;
   a.massBurst          = 0;
   a.speedScale         = 1;
   a.timeSinceDamage    = 100000;
   a.collideMove        = { 0, 0 }
   a.targetY            = y;
   a.flyoutTime         = 0;
   a.touchDeltaY        = 0;
   a.firingLoop         = false;
   a.prevX              = 0;

	a.noCollide				= false

   a.hitPoints           = tplArmor.hitPoints;
   a.controlMoveK        = math.lerp(AVATAR_MOVE_K[1],AVATAR_MOVE_K[2], App.data:getControlSensitivity() );

   if tplSideWeapon then
      a.sideWeapon       = AvatarSideWeapon:new( game, layers[tplSideWeapon.layer or 3], tplSideWeapon, self, a.template.sideweaponOffset );
   else
      a.sideWeapon      = nil;
   end

   if tplTurret then
      local factory = tplTurret.factory or Turret;
      a.turret = factory:new( game, layers[tplTurret.layer or 3], tplTurret,  x, y, 0, a, a.template.turretOffset );
   else
      a.turret = nil;
   end

	local droneIndexL		= DroneMgr:isDroneMountLeftAssigned();
	local droneIndexR		= DroneMgr:isDroneMountRightAssigned();

	if droneIndexL and DroneMgr:isEnabled() then
		a.droneLeft = a:_createDroneLeft( game, layers, x, y, 0, a, droneIndexL, MOUNT_POINT_LEFT );
	end

	if droneIndexR and DroneMgr:isEnabled() then
		a.droneRight = a:_createDroneRight( game, layers, x, y, 0, a, droneIndexR, MOUNT_POINT_RIGHT );
	end

	if template.shieldVFX then

		a.shieldVFX		= {};

		for k,shTmplt in ipairs(template.shieldVFX) do
			a.shieldVFX[k]	= Entity:new( game, layers[shTmplt.layer or 3], shTmplt, shTmplt.offset[1], shTmplt.offset[2], shTmplt.rotOffset or 0 );

			a.shieldVFX[k].prop:setPriority(10);
			a.shieldVFX[k].prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, a.prop, MOAIProp2D.TRANSFORM_TRAIT );

			if k == 2 then
				a.shieldVFX[k]:setScale(-1, 1);
			end
		end
	end

   a:debugCollision();

   --reset again to make sure our child are reset as well
   a:reset(x, y, 0 );

   return a;

end

----------------------------------------------------------------------

function Avatar:reset( x, y, angle )

   --Hack.. please remove speed

   Base.reset(self, x, y, angle );

   self.speed              = { 0, self.template.speed[Y] };
	self.shieldDrones       = { count = 0, rarity = 1 };
   self.shieldDrainTimer   = 0;
   self.shieldDrainDelay   = 0;

   if self.mainWeapon then
      self.mainWeapon:reset( x, y, angle, self );
   end

   if self.sideWeapon then
      self.sideWeapon:reset( x, y, angle, self, self.template.sideweaponOffset );
   end

   if self.shadow then
      self.shadow:reset(x+self.template.shadow.offset[1], y+self.template.shadow.offset[2], 0  );
   end

   if self.paint then
      self.paint:reset( x, y, angle, self );
   end

   if self.armor then
      self.armor:reset( x, y, angle, self );
   end

   if self.turret then
      self.turret:reset( x, y, angle, self, self.template.turretOffset   );
   end

	if self.droneLeft then
		self.droneLeft:reset(x, y, angle, self );
	end

	if self.droneRight then
		self.droneRight:reset(x, y, angle, self );
	end

	if self.shieldVFX then
		for _,v in pairs(self.shieldVFX) do
			v:hide();
		end
	end

   self.savePrompt   = App.data:isReviveUnlocked();

	EventMgr:rmvListener( self );

	EventMgr:addListener( self,	self.onHeal,               "player-avatar-heal"          );
   EventMgr:addListener( self,	self.onTimedConsumable,    "active-timed-consumable"     );

end

----------------------------------------------------------------------

function Avatar:delete()

	EventMgr:rmvListener( self );

	if self.shadow then
      self.shadow:delete();
   end

	if self.shieldVFX then
		for _,v in pairs(self.shieldVFX) do
			v:delete();
		end
	end

   if self.mainWeapon then
      self.mainWeapon:delete();
   end

   if self.sideWeapon then
      self.sideWeapon:delete();
   end

   if self.paint then
      self.paint:delete();
   end

   if self.armor then
      self.armor:delete();
   end
   
   if self.turret then
      self.turret:delete();
   end

	if self.droneLeft then
		self.droneLeft:delete();
	end

	if self.droneRight then
		self.droneRight:delete();
	end

	Base.delete(self);

end

----------------------------------------------------------------------

function Avatar:hide( )

   Base.hide( self );

   if self.shadow then
      self.shadow:hide();
   end

	if self.shieldVFX then
		for _,v in pairs(self.shieldVFX) do
			v:hide();
		end
	end

   if self.mainWeapon then
      self.mainWeapon:hide();
   end

   if self.sideWeapon then
      self.sideWeapon:hide();
   end

   if self.paint then
      self.paint:hide();
   end

   if self.armor then
      self.armor:hide();
   end

   if self.turret then
      self.turret:hide();
   end

	if self.droneLeft then
		self.droneLeft:hide();
	end

	if self.droneRight then
		self.droneRight:hide();
	end

end

----------------------------------------------------------------------

function Avatar:show( )

   Base.show( self );

   if self.shadow then
      self.shadow:show();
   end

	if self.shieldVFX then
		for _,v in pairs(self.shieldVFX) do
			v:hide();
		end
	end

   if self.mainWeapon then
      self.mainWeapon:show();
   end

   if self.sideWeapon then
      self.sideWeapon:show();
   end

   if self.paint then
      self.paint:show();
   end

   if self.armor then
		if self.armor then
			self.armor:hide();
		end

      self.armor:show();
   end

   if self.turret then
      self.turret:show();
   end

	if self.droneLeft then
		self.droneLeft:show();
	end

	if self.droneRight then
		self.droneRight:show();
	end

end

----------------------------------------------------------------------

function Avatar:_createDroneLeft( game, layers, x, y, a, parent, index, mountPoint )

	local template, visData, stats = DroneMgr:getDroneTemplateLeft(index);

	if template and template.factory then

		if mountPoint == MOUNT_POINT_RIGHT then
			template.offset[1] = -1*template.offset[1];
		end

		return template.factory:new( game, layers, template,  x, y, a, parent, template.offset, mountPoint, index, visData, stats );
	end

end

----------------------------------------------------------------------

function Avatar:_createDroneRight( game, layers, x, y, a, parent, index, mountPoint )

	local template, visData, stats = DroneMgr:getDroneTemplateRight(index);

	if template and template.factory then

		if mountPoint == MOUNT_POINT_RIGHT then
			template.offset[1] = -1*template.offset[1];
		end

		return template.factory:new( game, layers, template,  x, y, a, parent, template.offset, mountPoint, index, visData, stats );
	end

end

----------------------------------------------------------------------

function Avatar:getMagnetRadius( )

   local magnetRad      = 0;
   local powerupMagnet  = self.powerups["powerup_magnet"];

   if powerupMagnet then
      magnetRad = powerupMagnet.template.radius;
   end

   if self.turret then
      magnetRad = magnetRad + self.turret:getMagnetRadius();
   end

   return magnetRad;

end

----------------------------------------------------------------------

function Avatar:getScoreK( )

   local powerup = self.powerups["powerup_scoreK"];
   local double  = self.powerups["powerup_double_score"];
   local k = 1;

   if powerup then
      k = powerup.template.scoreK;
   end

   if double then
      k = k*2;
   end

	if self.droneLeft then
		k = k+self.droneLeft:getScoreK();
	end

	if self.droneRight then
		k = k+self.droneRight:getScoreK();
	end

   return k;

end

----------------------------------------------------------------------

function Avatar:getSecretScoreK( )
   return self.secretScoreK;
end

----------------------------------------------------------------------

function Avatar:getCurrentPickupK( )

   local powerup = self.powerups["powerup_scoreK"];

   if powerup then
      return powerup.template.scoreK;
   end

   return 1;

end

----------------------------------------------------------------------

function Avatar:isSpeedBurstOn( )

   return self.powerups["powerup_speed_burst"] ~= nil;

end

----------------------------------------------------------------------

function Avatar:isInvicible( )

   if HACK_AVATAR_INVICIBLE then
      return true;
   end

   for k,p in pairs(self.powerups) do

      if p:isInvicible( ) then

         return true;

      end
   end

   return false;

end

----------------------------------------------------------------------

function Avatar:canTurretShoot()

	if HACK_ALWAYS_SHOOT
		or (self.input.down and self.input.validArea and (not self:isDead()) and (not HACK_CANT_SHOOT)) then

		return true;
	end

	return false;

end

----------------------------------------------------------------------

function Avatar:addShieldDrone( drone, rarity, hitMarker )

	if drone and rarity and self.shieldDrones[drone] == nil then
		self.shieldDrones[drone]		= drone;
		self.shieldDrones.count			= self.shieldDrones.count + 1;

		if rarity >= self.shieldDrones.rarity then
			self.shieldDrones.rarity		= rarity;
			self.shieldDrones.hitMarker	= hitMarker;
		end
	end

end

----------------------------------------------------------------------

function Avatar:isShieldEnabled()

	if self.shieldVFX and self:getShieldDroneCount() > 0 then
		return true;
	end

	return false;

end

----------------------------------------------------------------------

function Avatar:isShieldActive()

	if self:isShieldEnabled() then
		local ratio = self:getShieldHPRatio();

		if ratio >= 0 then
			return ratio;
		end
	end

end

----------------------------------------------------------------------

function Avatar:getShieldHPRatio()

	if self:getShieldDroneCount() > 0 then

		local hp		= 0;
		local dmg	= 0;

		for k,v in pairs(self.shieldDrones) do
			if type(v) == "table" and v.getShieldHPMax then
				hp		= hp + v:getShieldHPMax();
				dmg	= dmg + v:getShieldDamage();
			end
		end

		if hp <= 0 then
			return 0;
		else
			return 1 - math.clamp((dmg/hp), 0, 1);
		end
	end

	return 0;

end

----------------------------------------------------------------------

function Avatar:getShieldMaxHP()

	if self:getShieldDroneCount() > 0 then

		local hp		= 0;

		for k,v in pairs(self.shieldDrones) do
			if type(v) == "table" and v.getShieldHPMax then
				hp		= hp + v:getShieldHPMax();
			end
		end

      return hp
	end

   return 0;

end

----------------------------------------------------------------------

function Avatar:getShieldDroneCount()
	return self.shieldDrones.count;
end

----------------------------------------------------------------------

function Avatar:getShieldDroneHitMarker()
	return self.shieldDrones.hitMarker;
end

----------------------------------------------------------------------

function Avatar:_createShieldHitMarker( hitMarker )

	if hitMarker then
		if not self.hitMarkerCooldown then
			self.hitMarkerCooldown	= 0;
		end

		if self.hitMarkerCooldown <= 0 then
			self.hitMarkerCooldown = 0.5;
			self.game.entityMgr:spawnExplosion(  hitMarker, 0, 0, 0, self );
		end
	end

end

----------------------------------------------------------------------

function Avatar:onDamageShield( damage, disableHitVFX )

	local shieldRatio	= self:isShieldActive();

	if shieldRatio and shieldRatio > 0 and damage and damage > 0 then

		local dmg	= damage;
		local hp		= 0;
		local ddmg	= 0;

		for k,v in pairs(self.shieldDrones) do
			if type(v) == "table" and v.onDamage then
				hp		= v:getShieldHPMax();
				ddmg	= v:getShieldDamage();

				if ddmg < hp then
					local d = math.max( math.min( dmg, hp-ddmg ), 0 );

					dmg = dmg - d;
					v:onDamage( d );
				end
			end
		end

      if not disableHitVFX then
         local hitMarker	= self:getShieldDroneHitMarker();

         self:_createShieldHitMarker(hitMarker);
      end

      if self:isShieldEnabled() and self:getShieldHPRatio() <= 0 then
         EventMgr:postEvent(	{	text="Shield Down", color={1,0,0,1}	}, "hud-alert-text"	);
      end
	end

end

----------------------------------------------------------------------

function Avatar:updateShield( dt )

	if self.shieldVFX then

		for k,v in ipairs(self.shieldVFX) do

			local a = v:getRot();

			if k == 1 then
				a = math.clampAngle360(a - 840 * dt);
			elseif k == 2 then
				a = math.clampAngle360(a + 284 * dt);
			else
				a = math.clampAngle360(a - 432 * dt);
			end

			v:setRot(a);
		end

		local shieldRatio	= self:isShieldActive();
		local hitMarker	= self:getShieldDroneHitMarker();

		if shieldRatio and shieldRatio > 0 then

			if shieldRatio >= 2/3 then
				self.shieldVFX[1].prop:setVisible(true);
				self.shieldVFX[2].prop:setVisible(true);
				self.shieldVFX[3].prop:setVisible(true);
			elseif shieldRatio >= 1/3 then
				self.shieldVFX[1].prop:setVisible(true);
				self.shieldVFX[2].prop:setVisible(true);
				self.shieldVFX[3].prop:setVisible(false);
			else
				self.shieldVFX[1].prop:setVisible(true);
				self.shieldVFX[2].prop:setVisible(false);
				self.shieldVFX[3].prop:setVisible(false);
			end

			if not self._ShieldUp then
				self._ShieldUp		= true;

				self:_createShieldHitMarker(hitMarker);
			end
		else
			for k,v in ipairs(self.shieldVFX) do
				v.prop:setVisible(false);
			end

			if self._ShieldUp then
				self._ShieldUp		= false;

				self:_createShieldHitMarker(hitMarker);
			end
		end
	end

end

----------------------------------------------------------------------

function Avatar:updateShieldDrain( dt )

	if self.shieldVFX then

      local shieldRatio = self:isShieldActive();

      if self.shieldDrainTimer == nil then
         self.shieldDrainTimer   = 0;
      end

      if self.shieldDrainDelay == nil then
         self.shieldDrainDelay   = 0;
      end

      if shieldRatio and shieldRatio > 0 then

         if self.shieldDrainDelay >= SHIELD_DRAIN_DELAY then

            self.shieldDrainTimer   = self.shieldDrainTimer + dt;

            if self.shieldDrainTimer >= SHIELD_DRAIN_TICK_TIME then

               self.shieldDrainTimer   = 0;

               self:onDamageShield( SHIELD_DRAIN_TICK_DAMAGE*self:getShieldMaxHP(), true );
            end
         else
            self.shieldDrainDelay   = self.shieldDrainDelay + dt;
         end
      else
         self.shieldDrainTimer   = 0;
         self.shieldDrainDelay   = 0;
      end
   end

end

----------------------------------------------------------------------

function Avatar:activateDroneBoost( tplPowerup, side )

	if not self:isSpeedBurstOn() then
		if side == MOUNT_POINT_LEFT and self.droneLeft and self.droneLeft:isBoostCharged() then
			self:_activatePowerup(tplPowerup);
			self.droneLeft:resetCharge();
		end

		if side == MOUNT_POINT_RIGHT and self.droneRight and self.droneRight:isBoostCharged() then
			self:_activatePowerup(tplPowerup);
			self.droneRight:resetCharge();
		end
	end

end

----------------------------------------------------------------------

function Avatar:_isBoostDroneCharged( side )

	if side == MOUNT_POINT_LEFT and self.droneLeft then
		return self.droneLeft:isBoostCharged();
	end

	if side == MOUNT_POINT_RIGHT and self.droneRight then
		return self.droneRight:isBoostCharged();
	end

end

----------------------------------------------------------------------

function Avatar:isBoostDroneChargedLeft()
	return self:_isBoostDroneCharged(MOUNT_POINT_LEFT);
end

----------------------------------------------------------------------

function Avatar:isBoostDroneChargedRight()
	return self:_isBoostDroneCharged(MOUNT_POINT_RIGHT);
end

----------------------------------------------------------------------

function Avatar:_getDroneTarget( side )

	if side == MOUNT_POINT_LEFT and self.droneLeft then
		return self.droneLeft:getTarget();
	end

	if side == MOUNT_POINT_RIGHT and self.droneRight then
		return self.droneRight:getTarget();
	end

end

----------------------------------------------------------------------

function Avatar:getDroneTargetLeft()
	return self:_getDroneTarget(MOUNT_POINT_LEFT);
end

----------------------------------------------------------------------

function Avatar:getDroneTargetRight()
	return self:_getDroneTarget(MOUNT_POINT_RIGHT);
end

----------------------------------------------------------------------

function Avatar:hasChargeRequiredDrone()

	local left	= self.droneLeft;
	local right	= self.droneRight;

	if left then
		left	= self.droneLeft:isChargeRequired();
	end

	if right then
		right	= self.droneRight:isChargeRequired();
	end

	return left or right;

end

----------------------------------------------------------------------

function Avatar:hasManualActivatedDrone()

	local left	= self.droneLeft;
	local right	= self.droneRight;

	if left then
		left	= self.droneLeft:isActivationManual();
	end

	if right then
		right	= self.droneRight:isActivationManual();
	end

	return left or right;

end

----------------------------------------------------------------------

function Avatar:hasChargeRequiredDrone()

	local left	= self.droneLeft;
	local right	= self.droneRight;

	if left then
		left	= self.droneLeft:isChargeRequired();
	end

	if right then
		right	= self.droneRight:isChargeRequired();
	end

	return left or right;

end

----------------------------------------------------------------------

function Avatar:setDroneChargeLeft( k )

	if self.droneLeft then
		return self.droneLeft:setCharge(k);
	end

end

----------------------------------------------------------------------

function Avatar:setDroneChargeRight( k )

	if self.droneRight then
		return self.droneRight:setCharge(k);
	end

end

----------------------------------------------------------------------

function Avatar:setDroneChargeFullLeft()

	if self.droneLeft then
		return self.droneLeft:setChargeFull();
	end

end

----------------------------------------------------------------------

function Avatar:setDroneChargeFullRight()

	if self.droneRight then
		return self.droneRight:setChargeFull();
	end

end

----------------------------------------------------------------------

function Avatar:forceDroneActivationLeft()

	if self.droneLeft then
		return self.droneLeft:forceAbilityActivation();
	end

end

----------------------------------------------------------------------

function Avatar:forceDroneActivationRight()

	if self.droneRight then
		return self.droneRight:forceAbilityActivation();
	end

end

----------------------------------------------------------------------

function Avatar:update( dt, input )

   Base.update( self, dt );

   local userAccel = (self.input.down and self.input.validArea and (not self:isDead()));
   local userShoot = (self.input.down and self.input.validArea and (not self:isDead()) and (not HACK_CANT_SHOOT));

   --The user can change the sensitivity in the option menu
   self.controlMoveK     = math.lerp(AVATAR_MOVE_K[1],AVATAR_MOVE_K[2], App.data:getControlSensitivity() );

   self.timeSinceDamage  = self.timeSinceDamage  + dt;

   self:updateMove( dt, input );

   self:updatePowerups( dt );

   self:updatePostCollide( dt );

   self:updateSpeed(dt, self.game.gameSpeedK, userAccel );

	self:updateShield( dt );
   self:updateShieldDrain( dt );

   local burstOn = self:isSpeedBurstOn();

   if HACK_ALWAYS_SHOOT or userShoot then

      if self.mainWeapon then
         self.mainWeapon:update( dt, burstOn );
      end

      if self.sideWeapon then
         self.sideWeapon:update( dt, burstOn );
      end

   else

      if self.mainWeapon and self.firingLoop then

         EventMgr:postEvent( {sounds=self.mainWeapon.sounds, name="avatar update",useAlt=true}, "loop-stop" );
         self.firingLoop = false;
         if HACK_GUN_OR_IMPACT == 0 then
            EventMgr:postEvent( {sounds=self.mainWeapon.sounds, name="avatar update",}, "loop-stop" );
         end

      end
   end

   if self.turret then
	  self.turret:update(dt, nil, burstOn, self:canTurretShoot());
   end

   if self.shadow then
      local x,y = self:getLoc();
      self.shadow:setLoc( x+self.template.shadow.offset[1], y+self.template.shadow.offset[2] );
   end

   if self.hitMarkerCooldown and self.hitMarkerCooldown > 0 then
	  self.hitMarkerCooldown = self.hitMarkerCooldown - dt;
   end

	if self.droneLeft then
		self.droneLeft:update(dt, nil, burstOn, self:canTurretShoot());
	end

	if self.droneRight then
		self.droneRight:update(dt, nil, burstOn, self:canTurretShoot());
	end

end

----------------------------------------------------------------------

function  Avatar:getSpeed( addImpulse )

   if self.speed then

      local spdX, spdY =  self.speed[X],self.speed[Y];

      if self.speedBurst then

         spdY = spdY + self.speedBurst;

      end


      if addImpulse and self.collisionImpulse then

         spdX, spdY  = spdX+self.collisionImpulse[X]
                     , spdY+self.collisionImpulse[Y];
      end

      return  spdX, spdY;
   end

end

----------------------------------------------------------------------

function Avatar:updateSpeed( dt, gameSpeedK, userAccel )

   if self:isDead() then

      self.speed[X]  =  self.speed[X]*math.min(dt*DEAD_SPEED_K,0.90);
      self.speed[Y]  =  self.speed[Y]*math.min(dt*DEAD_SPEED_K,0.90);
      self.speedBurst= 0;

   else

      local x        = self:getLocX();

      self.speed[X]  =  math.lerp( self.speed[X], (x-self.prevX)/dt,  0.10 );
      self.prevX     = x;

      --This will help for consumable screen where we do not want to accelerate
      if userAccel then
         self.speed[Y]  =  self.speed[Y] + self:getAccelY()*dt*gameSpeedK;
      end

      if self.speedMin then

         self.speed[X] =  math.sign(self.speed[X])*math.max( math.abs(self.speed[X]), self.speedMin[X]*gameSpeedK );
         self.speed[Y] =  math.max( self.speed[Y], self.speedMin[Y]*gameSpeedK );

      end

      if self.speedMax then

         self.speed[X] =  math.sign(self.speed[X])*math.min( math.abs(self.speed[X]), self.speedMax[X]*gameSpeedK );
         self.speed[Y] =  math.min( self.speed[Y], self.speedMax[Y]*gameSpeedK );

      end
   end
end

----------------------------------------------------------------------

function Avatar:updatePostCollide( dt )

   self.timeSinceCollide = self.timeSinceCollide + dt;
   self.hitSlowTimer     = self.hitSlowTimer     + dt;

   --This should never happens, but just i ncase
   if self.collisionImpulse[Y] > 0 then
      self.collisionImpulse[Y] = 0;
   end

   if self.collideMove[Y] > 0 then
      self.collideMove[Y] = 0;
   end

   for i=1,2 do

      self.collisionImpulse[i]   =  self.collisionImpulse[i]*math.min(dt*AVATAR_COLLISION_IMPULSE_INERTIA_K[i],0.99);

      self.collideMove[i]        =  self.collideMove[i]
                                 +  AVATAR_COLLISION_IMPULSE_DELTA_K[i]*self.collisionImpulse[i];

      self.collideMove[i]        = self.collideMove[i]* math.min(dt*AVATAR_COLLISION_MOVE_INERTIA_K[i],0.99);



   end
end

----------------------------------------------------------------------

function Avatar:updateMove( dt, input )

  if (not self:isDead()) then

      local myX, myY    = self:getLocX(), self.targetY;
      local ix,   iy    = input.lastDownPt.x, input.lastDownPt.y;

      --first time, only initialize
      if self.input.inSmoothPos == nil or self.forceResetMove  then

         self.input.down              = false;
         self.input.inSmoothPos       = { x = ix,    y = iy  };
         self.input.inAnchorPos       = { x = ix,    y = iy  };
         self.input.myAnchorPos       = { x = myX,   y = myY };
         self.input.lastValid         = { x = ix,    y = iy  };
         self.input.validArea         = false;
         self.forceResetMove          = false;

      else

         self.input.validArea = iy < 0 and input.down;

         --This need to be call before updating self.input.inSmoothPos
         -- in case we revert ix,iy
         if self.input.validArea then
            self.input.lastValid.x, self.input.lastValid.y = ix, iy;
            --Moving Up
            self.touchDeltaY = math.lerp(self.touchDeltaY, AVATAR_TOUCH_DELTA_Y, 0.075 );
         else
            ix, iy = self.input.lastValid.x, self.input.lastValid.y;
            --Moving Down
            self.touchDeltaY        = math.lerp(self.touchDeltaY, 0, 0.10 );
         end

         --When hitting another car, loose control for a little while
         local hitMoveK =  math.clamp(  (self.hitSlowTimer-AVATAR_HIT_LOOSE_CTRL) / AVATAR_HIT_REGAIN_CTRL, 0 , 1 );

         --To give the illusion of loosing control, delay the input
         self.input.inSmoothPos.x  = self.input.inSmoothPos.x*(1-hitMoveK) + ix*hitMoveK;
         self.input.inSmoothPos.y  = self.input.inSmoothPos.y*(1-hitMoveK) + iy*hitMoveK;

         if  input.down then

            --Just pressed down
            if (not self.input.down) then
               self.input.down         = true;
               self.input.inAnchorPos  = { x = ix,    y = iy  };
               self.input.myAnchorPos  = { x = myX-self.collideMove[X],  y = self.targetY };
            end
         else
            self.input.down         = false;
         end
		end

   end

   self:updateLoc();

end

----------------------------------------------------------------

function  Avatar:updateLoc( )

   local dx, dy      = self.input.inSmoothPos.x - self.input.inAnchorPos.x
                     , self.input.inSmoothPos.y - self.input.inAnchorPos.y;

   local newX        = self.input.myAnchorPos.x
                     + self.controlMoveK*dx
                     + self.collideMove[X];

   local newY        = self.input.myAnchorPos.y
                     + self.touchDeltaY
                     + self.collideMove[Y];

   newX, newY        = self:keepInsideBoundary( newX, newY, self.controlMoveK*dx,self.collideMove[X] );

   self.x = newX;
   self.y = newY;

   self:setLoc( newX, newY );

end

----------------------------------------------------------------

function  Avatar:updatePowerups( dt )

   for k,p in pairs(self.powerups) do

      p:update( dt );

      if p:isDone() then

         local tpl = p.tplPowerup;
         if tpl then
            EventMgr:postEvent( {sounds=tpl.sounds, name="avatar updatePowerups",}, "loop-stop" );
            if p.type == "powerup_double" and self.firingLoop and HACK_GUN_OR_IMPACT == 0 then
               EventMgr:postEvent( {sounds=self.mainWeapon.sounds, name="update powerups gun swap",}, "add-sfx" );
            end

         else
            EventMgr:postEvent( {sounds=p.sounds, name="avatar updatePowerups",}, "loop-stop" );
            if p.type == "powerup_double" and self.firingLoop and HACK_GUN_OR_IMPACT == 0 then
               EventMgr:postEvent( {sounds=self.mainWeapon.sounds, name="update powerups gun swap",}, "add-sfx" );
            end
         end

         self.powerups[k].deleteMe  = true;
         self.powerups[k]           = nil;

      end
   end

   --Not the most eloquent code in the world
   --
   local burst = self.powerups["powerup_speed_burst"];
   if burst then

      if self.backupCollision == nil then
         self.backupCollision= { self:getCollisionWidth(), self:getCollisionHeight() };
      end

      self:setCollisionSize( burst:getCollisionWidth(), burst:getCollisionHeight() );

   elseif self.backupCollision then

      self:setCollisionSize( self.backupCollision[X], self.backupCollision[Y] );

      self.backupCollision = nil;

   end

end

----------------------------------------------------------------------

function  Avatar:onHitHazardMissile()

   if (not self:isDead())  then

		local shieldRatio = self:isShieldActive();

		if shieldRatio and shieldRatio > 0 and not self:isSpeedBurstOn() then
			self:onDamageShield(9000000);
      elseif (not self:isInvicible()) then
         self:onDamage(9000000, "Missile");
		end
   end

end

----------------------------------------------------------------------

function  Avatar:onHitBarricade( collision )

   if (not self:isDead())  then

		local shieldRatio = self:isShieldActive();

		if shieldRatio and shieldRatio > 0 and not self:isSpeedBurstOn() then
			self:onDamageShield(9000000);
      elseif (not self:isInvicible()) then

         local other         = collision:getOther(self);

         local impX, impY     = collision:getImpulse(self);
         local ex,   ey       = collision:getExpel(self);

         local type           = collision.type;

         --Hack for bad collision detection at high speed
         if type == X and math.abs(self.speed[Y]) > 2000 then

            --Look if 1 frame behind the car was "under" the barricade
            local ox,oy = other:getLoc();
            local oBot  = (oy + self.speed[Y]/30) - other:getCollisionHeight() / 2;
            local ax,ay = self:getLoc();
            local aTop  = ay+self:getCollisionHeight()/2;

            if aTop < oBot then
               type = Y;
            end
         end

         --First make sure to expel from the other vehicle
         self.collideMove[X]   = self.collideMove[X]
                               + AVATAR_COLLISION_MOVE_BARRICADE_K[X]*ex
                               + AVATAR_COLLISION_MOVE_BARRICADE_EXTRA[X]*math.sign(ex);
         self.collideMove[Y]   = self.collideMove[Y]
                               + AVATAR_COLLISION_MOVE_BARRICADE_K[Y]*ey
                               + AVATAR_COLLISION_MOVE_BARRICADE_EXTRA[Y]*math.sign(ey);

         -- If we dont have much speed, dont penalize the player too much
         if math.abs(self.speed[type]) > 20 then

            --Create a little damage
            self:onDamage(other.template.damageToAvatar[type], "Barricade-"..type);

            self.hitSlowTimer     = 0;

         end

         EventMgr:postEvent( { vehicle="barricade" }, "avatar-sideswipe" );

         self.speed[X] = 0;

         self:updateLoc();
      end
   end

end

----------------------------------------------------------------------

function  Avatar:onCollision( collision, otherCollider )

   if     (not self:isDead() )
      and ( self.timeSinceCollide > AVATAR_MIN_TIME_BETWEEN_COLLIDE ) then

      if not self:isInvicible()  then

         local other         = collision:getOther(self);
         local impX, impY    = collision:getImpulse(self);
         local ex,   ey      = collision:getExpel( self );

         --First make sure to expel from the other vehicle
         self.collideMove[X]     = self.collideMove[X] + AVATAR_COLLISION_MOVE_K[X]*ex;
         self.collideMove[Y]     = self.collideMove[Y] + AVATAR_COLLISION_MOVE_K[Y]*ey;

         if collision.type == X then

            self.collisionImpulse[X] = math.clamp( self.collisionImpulse[X] + AVATAR_COLLISION_IMPULSE_K[X]*impX
                                              , -0.50*self.speedMax[X]
                                              ,  0.50*self.speedMax[X] );

            EventMgr:postEvent( { vehicle=other }, "avatar-sideswipe" );

         else
            --Make sure the impulse slow us down
            -- if so, make sure it will be noticeable
            if impY < 0 then

               self.collisionImpulse[Y] = math.clamp( self.collisionImpulse[Y] + AVATAR_COLLISION_IMPULSE_K[Y]*impY
                                                 , -0.50*self.speed[Y]
                                                 , -0.25*self.speed[Y]                       );
            end
         end

			local shieldRatio = self:isShieldActive();

			if shieldRatio and shieldRatio > 0 and otherCollider and other.template.damageToAvatar[collision.type] > 0 and not self:isSpeedBurstOn() then
				self:onDamageShield( other.template.damageToAvatar[collision.type] );
			else

         local dying = self:onDamage( other.template.damageToAvatar[collision.type], "Collision-"..collision.type );

         if collision.type == Y and not dying then
            EventMgr:postEvent( {sounds=other.template.damageTypeSound[collision.type], name="collision",}, "add-sfx" );
         elseif collision.type == X and not dying then
				EventMgr:postEvent( {sounds=other.template.damageTypeSound[collision.type], name="collision",}, "add-sfx" );
			end

         self.timeSinceCollide   = 0;
         self.hitSlowTimer       = 0;
			end
      end
   end

   self:updateLoc();

end

----------------------------------------------------------------------

function Avatar:tryFlyout( tpl, x, y )

   if self.game.duration - self.flyoutTime > AVATAR_FLYOUT_INTERVAL_MIN then

      self.game.entityMgr:spawnFlyout( tpl, x,  y, true );

      self.flyoutTime = self.game.duration;

   end
end

----------------------------------------------------------------------

function Avatar:onBulletHit( d, x, y, owner)

   if     ( not self:isDead() )
      and ( d ~= 0 )
      and ( not self:isInvicible() ) then

      if owner and owner.template and type(owner.template.type) == "string" and owner.template.type:find("boss") then
         self:onDamage(d, "Bullet-Boss");
      else
         self:onDamage(d, "Bullet");
      end

   end
end


----------------------------------------------------------------------

function Avatar:onRevive()

   self:show();
	self.damageValue  = 0;
   self.collideMove[X]        = 0;
   self.collideMove[Y]        = 0;
   self.touchDeltaY           = 0;
   self.hitSlowTimer          = 1000;
   self.forceResetMove        = true;
   self.savePrompt            = App.reviveCount < App.data:getMaxRevive();
   self.deathExplosion        = nil;

   if self.revivePowerups then
      for i,v in ipairs(self.revivePowerups) do
         self:_activatePowerup( v );
      end
   end

end

----------------------------------------------------------------------

function Avatar:_activatePowerup( tplPowerup )

print("_activatePowerup", tplPowerup, tplPowerup.factory );

   local px, py  = self:getLoc();

   if self.powerups[tplPowerup.type] then

      if self.powerups[tplPowerup.type].template == tplPowerup then

         self.powerups[tplPowerup.type]:reset ( px, py, 0, self );

      else

         self.powerups[tplPowerup.type].deleteMe = true;
         self.powerups[tplPowerup.type] = self.game.entityMgr:spawnPowerup( tplPowerup.factory or Powerup, tplPowerup, px, py, 0, self );

      end

   else

      self.powerups[tplPowerup.type] = self.game.entityMgr:spawnPowerup( tplPowerup.factory or Powerup, tplPowerup, px, py, 0, self );

   end

end

----------------------------------------------------------------------

function Avatar:onPickup( p )

   local px,py = p:getLoc();

   if p.pickupType == "currency" then

      if self._currencyValueK == nil then
         self.game.currency =  self.game.currency + p.value;
      else
         local value = p.value;

         --Note: must check value due to different values from pickup template
         if value == 2 then
            value = 1;
         elseif value == 20 then
            value = 10;
         end

         self.game.currency =  self.game.currency + (value * self._currencyValueK);
      end
   elseif p.pickupType == "wrench" then

      self:_onPickupWrench( p );

	elseif p.pickupType == "bounty" then

		if p.tplPowerup then
			self.game.entityMgr:spawnExplosion(p.tplPowerup, 0, 0, 0, self);
		end
   else
      local tplPowerup = p.tplPowerup;

      if tplPowerup then

         self:_activatePowerup( tplPowerup );

      end
   end

   if self.turret then
      if self.turret:onPickup() then
         self.turret:onPickup(p);
      end
   end

end

----------------------------------------------------------------------

function Avatar:_onPickupWrench( p )

   self.damageValue =  math.max( self.damageValue - (p.value/100)*self.hitPoints, 0 );

end

----------------------------------------------------------------------

function Avatar:onHeal( event )

	if event then

		if event.heal and event.heal > 0 then
			self.damageValue =  math.max( self.damageValue - (event.heal/100)*self.hitPoints, 0 );
		end

		if event.avatarVFX then
			self.game.entityMgr:spawnExplosion(event.avatarVFX, 0, 0, 0, self);
		end
	end

end

----------------------------------------------------------------------

function Avatar:onConsumable( tpl )

   Debug.assert( tpl );
   Debug.assert( tpl.tplPowerup );

   self:_activatePowerup( tpl.tplPowerup );

   if not self.revivePowerups then
      self.revivePowerups ={};
   end

   if tpl.reviveReapply then
      table.insert(self.revivePowerups, tpl.tplPowerup);
   end

end

----------------------------------------------------------------------

function Avatar:onTimedConsumable( event )

   if not event then
      return;
   end

   --check currency scaling timed consumables
   if    event.index == CONSUMABLE_150PCT_INDEX
      or event.index == CONSUMABLE_300PCT_INDEX
      or event.index == CONSUMABLE_400PCT_INDEX
      or event.index == CONSUMABLE_500PCT_INDEX then

      local tpl   = ConsumableMgr:getTemplate(event.index).tplPowerup;
      local level = App.data:getPickupLevel(PICKUP_COIN_INDEX);

      if tpl and tpl.valueK and tpl.valueK > level then
         self._currencyValueK = tpl.valueK;
      end
   end

end

----------------------------------------------------------------------

function Avatar:onDamage( d, type )

   if not self:isDead() then

      if not self:isInvicible() then


			if d > 0 then
         	self.timeSinceDamage  = 0;
         	EventMgr:postEvent({amount=d}, "avatar-damaged");
			end

         if Base.onDamage( self, d ) then
            MOAIFlurry:event("Player-Death", "last-damage", tostring(d), "caused", type);
            return self:onDie();
         end

         return false;

      end
   end

   return true;

end

----------------------------------------------------------------------

function Avatar:onDie( )

   if not self:isInvicible() then

      if HACK_CANT_DIE then

         self.damageValue = 0;

		elseif self.savePrompt then

         self:hide();
			self.noCollide = true;

         if self.template.explosion then

            self.deathExplosion = self.game.entityMgr:spawnExplosion( self.template.explosion, self:getLoc() );

         end
         for k,p in pairs(self.powerups) do

            self.powerups[k] = nil;
            p.deleteMe = true;

         end

			self:_onDroneDie();

			self.savePrompt = false;
			EventMgr:postEvent( {state="entity\avatar\avatar.lua"}, "death-save" );

      else

         self:hide();

         EventMgr:postEvent( {name="player died", time=self.game.duration}, "avatar-death" );

         if self.template.explosion and not self.deathExplosion then

            self.game.entityMgr:spawnExplosion( self.template.explosion, self:getLoc() );

         end

         for k,p in pairs(self.powerups) do

            self.powerups[k] = nil;
            p.deleteMe = true;

         end

			self:_onDroneDie();

         return true;

      end
   end

   return false;
end

----------------------------------------------------------------------

function Avatar:_onDroneDie()

	if self.droneLeft then
		self.droneLeft:onDie();
	end

	if self.droneRight then
		self.droneRight:onDie();
	end

end


----------------------------------------------------------------------



