
require('game/entity/entity');

----------------------------------------------------------------------

local Base = Entity;
Background = Base:new();

----------------------------------------------------------------------

function Background:new( game, layer, template,  x, y, a )

   local e    = Base.new( self, game, layer, template,  x, y, a );
   local y    = e.game.boundary.min.y;

   
   local height = 0;

   e.tiles = {};

   repeat

      local th = 0;

      for i=1,#template.tile do
         local tile = Entity:new( game, layer, template.tile[i],  x, y, a );

         th = tile:getHeight();

         tile:setLocY( y - th/2 );

         table.insert(e.tiles, tile);

         height = height + th;

         y = y + th;
      end

   -- do (y-th) to make sure we have more than enough
   -- if the size would fit the screen, we would only have one = trouble
   until (y-th) > 2*e.game.boundary.max.y

   e:setHeight(height);
   
   e.cache = { decal = {}, foreground = {} };

   if template.decal then
      for i=1,#template.decal do
         e.cache.decal[i] = Entity:new( game, e.layer, template.decal[i],  0, 1000, 0 );
         e.cache.decal[i].xRnd = { 0, 0 };
         if template.decal[i].xRnd then
            e.cache.decal[i].xRnd[1] = template.decal[i].xRnd[1];
            e.cache.decal[i].xRnd[2] = template.decal[i].xRnd[2];
         end
         e.cache.decal[i].sizeRnd = { 1, 1 };
         if template.decal[i].sizeRnd then
            e.cache.decal[i].sizeRnd[1] = template.decal[i].sizeRnd[1];
            e.cache.decal[i].sizeRnd[2] = template.decal[i].sizeRnd[2];
         end
         e.cache.decal[i]:hide();
      end
   end

   if template.foreground then
      for i=1,#template.foreground do
         e.cache.foreground[i] = Entity:new( game, e.game.parent.layer.foreground[1], template.foreground[i],  0, 1000, 0 );
         e.cache.foreground[i]:hide();
      end
   end

   e:reset();

   return e;

end

----------------------------------------------------------------------

function Background:reset( x, y, a )

   Base.reset( self, x, y , a );

   if self.template.decalSpawnTime then
      self.decalTime       = math.random(self.template.decalSpawnTime[1]*1000,self.template.decalSpawnTime[2]*1000)/1000;
   else
      self.decalTime       = nil;
   end

   if self.template.foregroundSpawnTime then
      self.foregroundTime  = math.random(self.template.foregroundSpawnTime[1]*1000,self.template.foregroundSpawnTime[2]*1000)/1000;
   else
      self.foregroundTime = nil;
   end

   local height = 0;
   
   if self.tiles then

      local y    = self.game.boundary.min.y;

      for i=1,#self.tiles do

         local tile = self.tiles[i];
         local th   = tile.austin:getHeight();

         --stretch a bit to avoid a bug creating an artifact of a line in between tiles
         tile:setScale(1, (th+4)/th );

         tile:setLocY( y - th/2 );

         height = height + th;

         y = y + th;

      end
   end
   
   self:setHeight(height);
   
end

----------------------------------------------------------------------

function Background:delete()

   Base.delete(self);

   for i=1,#self.tiles do

      self.tiles[i]:delete();

   end
end

----------------------------------------------------------------------

function Background:update( dt, avatarSpeedY )

   Base.update(self,dt);

   local dark = 1;--math.clamp(1-self:getAge() / 60, 0.2, 1);

   if self.template.avatarSpeedMax then
      avatarSpeedY = math.min(avatarSpeedY, self.template.avatarSpeedMax);
   end

   self:updateDecal(       dt, avatarSpeedY, dark );
   self:updateForeground(  dt, avatarSpeedY, dark );
   self:updateTile(        dt, avatarSpeedY, dark );

end

----------------------------------------------------------------------

function Background:updateTile( dt, avatarSpeedY, dark )

   if self.tiles then

      local minY   = self.game.boundary.min.y;
      local tiles  = self.tiles;
      local count  = #tiles;
      local height = self:getHeight();

      for i=1,count do

         local tile = tiles[i];

         tile:update(dt,avatarSpeedY);

         local y = tile.austin:getLocY();
         local h = tile.austin:getHeight();

         tile:setColor( dark, dark, dark, 1 );

         if (y+h/2) < minY then

            tile:setLocY( y+height );

         end
      end
   end
end

----------------------------------------------------------------------

function Background:updateDecal( dt, avatarSpeedY )

   if self.decalTime then

      if self.decal then

         self.decal:update(dt,avatarSpeedY);

         if (self.decal:getLocY()+self.decal:getHeight()) < self.game.boundary.min.y  then

            self.decalTime = math.random(self.template.decalSpawnTime[1]*1000,self.template.decalSpawnTime[2]*1000)/1000;
            self.decal:hide();
            self.decal     = nil;

         end
      else

         self.decalTime = self.decalTime - dt;

         if self.decalTime < 0 and #self.cache.decal > 0 then

            self.decal = self.cache.decal[math.random(1,#self.cache.decal)]
            self.decal:reset(math.random(self.decal.xRnd[1],self.decal.xRnd[2]), self.game.boundary.max.y + 2*self.decal:getHeight(), 0);
            self.decal:setScale( math.random(self.decal.sizeRnd[1]*1000,self.decal.sizeRnd[2]*1000)/1000);

            self.decal:show();

         end
      end
   end
end

----------------------------------------------------------------------

function Background:updateForeground( dt, avatarSpeedY )

   if self.foregroundTime then

      if self.foreground then

         self.foreground:update(dt,avatarSpeedY);

         if (self.foreground:getLocY()+self.foreground:getHeight()) < self.game.boundary.min.y  then

            self.foregroundTime = math.random(self.template.foregroundSpawnTime[1]*1000,self.template.foregroundSpawnTime[2]*1000)/1000;
            self.foreground:hide();
            self.foreground   = nil;

         end
      else

         self.foregroundTime = self.foregroundTime - dt;

         if self.foregroundTime < 0 and #self.cache.foreground > 0 then

            self.foreground = self.cache.foreground[math.random(1,#self.cache.foreground)]
            self.foreground:reset(0, self.game.boundary.max.y + 2*self.foreground:getHeight(), 0);
            self.foreground:show();

         end
      end
   end
end

----------------------------------------------------------------------

