require('game/entity/entity');
require('game/entity/barricade/barricadeDecal');
require('game/entity/barricade/barricadeObstacle');
----------------------------------------------------------------------

local Base = Entity;

Barricade = Base:new();

----------------------------------------------------------------------

function Barricade:new( game, layer, template,  x, y, a, lane )

   local e = Base.new( self, game, layer, template, x, y, a, lane );

   e.obstacles     = {};
   for i=1,#template.obstacles do

      local obstacle  = BarricadeObstacle:new( game,  layer,  template.obstacles[i],  x, y, 0, e );
      table.insert( e.obstacles, obstacle);

   end

   e.decals        = {};
   for i=1,#template.decals do

      local decal = BarricadeDecal:new( game,  game.parent.layer.background[5],  template.decals[i],  x, y, 0, e );
      table.insert( e.decals, decal);

   end


   --reset again because the first time, we didnt have decals or obstacles
   e:reset( x, y, a, lane );
   
   return e;

end

----------------------------------------------------------------------

function Barricade:reset( x, y, angle, lane )

   Base.reset( self, x, y, angle );

   self.done = false;
   self.maxY = -1000;
   self.lane = lane;

   if self.obstacles then
      for i=1, #self.obstacles do
         self.obstacles[i]:reset( x, y, angle, self );
         self.maxY = math.max(self.maxY,self.obstacles[i]:getTop());
      end
   end

   if self.decals then
      for i=1, #self.decals do
         self.decals[i]:reset( x, y, angle, self );
         self.maxY = math.max(self.maxY,self.decals[i]:getTop());
      end
   end

   if App then
      self._screenTop     = App.viewportHeight/2;
      self._screenBottom  = -App.viewportHeight/2;
   end

   if self.template.waitLaneClear and self.lane then
      self.move = false;
   else
      self.move = true;
   end

end

----------------------------------------------------------------------

function Barricade:update( dt, avatarSpeedY )

   if not self.done then

      local destroyed = false;

      self.game.gameSpawner:disableLaneFor(self.lane, 1);

      if self.move then
         
         Base.update( self, dt, avatarSpeedY );

         self.done = true;
         self.maxY = -1000;

         for i=1, #self.obstacles do

            local o = self.obstacles[i];

            o:update( dt, avatarSpeedY  );
            
            self.maxY = math.max(self.maxY,o:getTop());

            if not o:isDone() then
               self.done = false;
            end

            if o:isDestroyed() then
               destroyed = true;
            end
         end

         for i=1, #self.decals do

            local d = self.decals[i];
      
            d:update( dt, avatarSpeedY  );

            self.maxY = math.max(self.maxY,d:getTop());

            if not d:isDone() then
               self.done = false;
            end

         end

         if destroyed then
            self:onObjectDestroyed();
         end
      else

         if #self.game.lanes[self.lane].vehicle == 0 then
            self.move = true;
         end
      
      end
   end
end

----------------------------------------------------------------------

function Barricade:onObjectDestroyed( object, impactLocX, impactLocY )

   for i=1, #self.obstacles do

      local o = self.obstacles[i];

      if o and o ~= object then
         o:onObjectDestroyed( object, impactLocX, impactLocY );
      end

   end

end

----------------------------------------------------------------------

function Barricade:isObstacleOnScreen( timeBuffer )

   if self._screenTop and self._screenBottom then

      local time = timeBuffer or 0; --setting to > 0 will test if obstacle will be on screen in the time set

      for _,v in ipairs(self.obstacles) do

         local y     = v:getBottom();
         local y2    = v:getTop();
         local sx,sy = v:getSpeed(); --speed in pixels per sec
         local ay    = self.game.avatar:getSpeedY() or 0;

         y  = y - time * ( ay - (sy or 0) );
         y2 = y2 - time*( ay - (sy or 0) );

         if not v:isDone() and not v:isDestroyed() and y <= self._screenTop and y2 >= self._screenBottom then
            return true;
         end
      end
   end

   return false;

end

----------------------------------------------------------------------

function Barricade:getMaxY()
   return self.maxY;
end

----------------------------------------------------------------------

function Barricade:isDone()
   return self.done;
end

----------------------------------------------------------------------