require('game/entity/entity');

----------------------------------------------------------------------

local Base = Entity;

BarricadeObstacle = Base:new();

----------------------------------------------------------------------

function BarricadeObstacle:new( game, layer, template,  x, y, a, owner )

   local e = Base.new(self, game, layer, template, x, y, a, owner );

   e.mass  = 1000000;
   e:debugCollision();

   if template.barricade then
   	e.isBarricade = true;
   end

	
	

   return e;

end

----------------------------------------------------------------------

function BarricadeObstacle:reset( x, y, angle, owner )

   Base.reset( self, x, y, angle );

   self.owner = owner;
   self.done = false;
   self.destroyed = false;

   local x,y = self:getLoc();

   x = x + self.template.delta[1];
   y = y + self.template.delta[2];

   self:setLoc(x,y);

   self.done = false;

end

----------------------------------------------------------------------

function BarricadeObstacle:update( dt, avatarSpeedY )

   if not self.done then

      Base.update( self, dt, avatarSpeedY );

      if self:getCollisionWidth() > 0 and self:getCollisionHeight() > 0 and not self.game.avatar.noCollide then
         local collision = Collision:checkCollision( self, self.game.avatar )

         if collision then

				local shieldRatio = self.game.avatar:isShieldActive();

            self.game.avatar:onHitBarricade(collision);

            if self.game.avatar:isInvicible() or (shieldRatio and shieldRatio > 0) then
					local impact = collision:getImpactLoc()
					self:onDestroyed( impact );
            end
         end
      end

      if self:getTop() < self.game.boundary.min.y then 
      
         self.done = true;
			
			if self.isBarricade then
				EventMgr:postEvent( {}, "barricade-avoided" );
			end
         
      end

   end

   return self.done;

end
----------------------------------------------------------------------

function BarricadeObstacle:onDestroyed( impactLocX, impactLocY )

   if not self.done then

      self.done = true;
      self:hide();

      if self.isBarricade then
         self.destroyed = true;
      end

      if self.template.explosion then
         self.game.entityMgr:spawnExplosion(  self.template.explosion, impactLocX or self:getLocX(), impactLocY or self:getLocY() );
      end

      if self.template.specialDropList and self.template.specialDropSpeedK then
         self.game:spawnPickupFromList( self.template.specialDropList, {self:getLocX(), self:getLocY()}, self.template.specialDropSpeedK  );
      end

      EventMgr:postEvent( {killByPlayer=true}, "barricade-destroyed" );

      if self.owner and self.owner.onObjectDestroyed then
         self.owner:onObjectDestroyed( self, impactLocX or self:getLocX(), impactLocY or self:getLocY() );
      end
   end

end


----------------------------------------------------------------------

function BarricadeObstacle:onObjectDestroyed( object, impactLocX, impactLocY )

   self.done = true;
   self:hide();

end

----------------------------------------------------------------------

function  BarricadeObstacle:onDamage( d )

   local d2 = self:calcDamageK()*d;

   Entity.onDamage(self, d2);

   return self.damageValue >= self.hitPoints;

end


----------------------------------------------------------------------

function BarricadeObstacle:isDone()
   return self.done;
end

----------------------------------------------------------------------

function BarricadeObstacle:isDestroyed()
   return self.destroyed;
end

----------------------------------------------------------------------