require('game/entity/drone/drone');

local MOUNT_POINT_LEFT			= "left";
local MOUNT_POINT_RIGHT			= "right";

local DRONE_DISABLED				= "disabled";
local DRONE_RESPAWN_COOLDOWN	= "respawn-cooldown";
local DRONE_RESPAWN				= "respawn";
local DRONE_READY					= "ready";
local DRONE_CHARGED_INIT		= "charged-init"
local DRONE_CHARGED_BOOSTING	= "charged-boosting";
local DRONE_CHARGED_STANDBY	= "charged-standby";

local MANUAL_ACTIVATION			= false;

local PULSE_TIME					= 0.3;

----------------------------------------------------------------------

local Base	= Drone;
DroneBoost	= Base:new();

----------------------------------------------------------------------


function DroneBoost:new( game, layers, template,  x, y, angle, parent, offset, mountPoint, garage_index, visData, stats )

	-- modify base template to fit player vehicle
	if template and garage_index and visData then
		local rarity;
		local level;

		if mountPoint == MOUNT_POINT_LEFT then
			rarity	= math.min( DroneMgr:getDroneRarityLeft(garage_index), DroneMgr:getDroneMaxRarity() );
			level		= math.min( DroneMgr:getDroneLevelLeft(garage_index), DroneMgr:getDroneMaxLevel());
		else
			rarity	= math.min( DroneMgr:getDroneRarityRight(garage_index), DroneMgr:getDroneMaxRarity() );
			level		= math.min( DroneMgr:getDroneLevelRight(garage_index), DroneMgr:getDroneMaxLevel());
		end

		template.tplPowerup.duration	= stats.rarity[rarity];
		template.chargeThreshold		= stats.level[level];
		template.vfx.color				= visData.off;
	end

	local e = Base.new( self, game, layers, template, x, y, angle, parent, offset, mountPoint, garage_index );

   if template and mountPoint then
		e._chevColorOff	= { r=visData.off[1], g=visData.off[2], b=visData.off[3], a=visData.off[4] };
		e._chevColorOn		= { r=visData.on[1], g=visData.on[2], b=visData.on[3], a=visData.on[4] };
		e._chevColorStdby	= { r=visData.standby[1], g=visData.standby[2], b=visData.standby[3], a=visData.standby[4] };
		e._chevrons			= {};

		for i=1,#template.vfx.offset do
			e._chevrons[i]	= e:_createVisProp( layers[template.vfx.layer or template.layer or 5], template.vfx.priority, template.vfx.offset[i][1], template.vfx.offset[i][2], 0, template.vfx.width, template.vfx.height, template.vfx.texture[i], template.vfx.scale, template.vfx.color, template.vfx.blend );
			e._chevrons[i]:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
		end

		e._readyVFX	= e:_createVisProp( layers[template.readyvfx.layer or template.layer or 5], template.readyvfx.priority, template.readyvfx.offset[1], template.readyvfx.offset[2], 0, template.readyvfx.width, template.readyvfx.height, template.readyvfx.texture, template.readyvfx.scale, template.readyvfx.color, template.readyvfx.blend );
		e._readyVFX:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );

		e._pipOn				= {};
		e._pipOff			= {};

		for i=1,#template.pip_on.offset do
			e._pipOn[i]		= e:_createVisProp( layers[template.pip_on.layer or template.layer or 5], template.pip_on.priority, template.pip_on.offset[i][1], template.pip_on.offset[i][2], 0, template.pip_on.width, template.pip_on.height, template.pip_on.texture );
			e._pipOff[i]	= e:_createVisProp( layers[template.pip_off.layer or template.layer or 5], template.pip_off.priority, template.pip_off.offset[i][1], template.pip_off.offset[i][2], 0, template.pip_off.width, template.pip_off.height, template.pip_off.texture );
			e._pipOn[i]:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e._pipOff[i]:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
		end

      --call here because avatar doesnt get called yet
      e:reset( x, y, angle, parent, offset  );
   end

   return e;

end

----------------------------------------------------------------------

function DroneBoost:reset( x, y, angle, parent, offset )

   Base.reset(self, x, y, angle, parent, offset);

	self._chargeCounter	= 0;
	self._initTimer		= 0;
	self._vfxTimer			= 0;
	self._vfxBlinkTimer	= 0;
	self._cooldownTimer	= 0;
	self._pipTracker		= {false, false, false};
	self._pipTimer			= 0;

	if self.game then
		self.game:disablePickup( PICKUP_BOOST_INDEX );
	end

end

----------------------------------------------------------------------

function DroneBoost:update( dt, avatarSpeedY, burstOn, canShoot )

	Base.update(self, dt, avatarSpeedY);

	self:_chargeUpdate(dt);
	self:_vfxUpdate(dt);
	self:_updatePipPulse(dt);

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneBoost:_chargeUpdate( dt )

   Debug.assert(self, "DroneBoost:_chargeUpdate -> self nil");
   Debug.assert(self.game, "DroneBoost:_chargeUpdate -> self.game nil");
   Debug.assert(self.game.avatar, "DroneBoost:_chargeUpdate -> self.game.avatar nil");

   if    not self
      or not self.game
      or not self.game.avatar
      or     self.game.avatar:isDead() then

      return;
   end

   if self.state == DRONE_CHARGED_INIT then
      self._initTimer = self._initTimer + dt;

      if not MANUAL_ACTIVATION and self._initTimer >= self.template.initTime then
         self:_initBoost();
      end
   elseif self.state == DRONE_CHARGED_STANDBY then

      local otherDroneChargeReady;

      if self._mountPosition == MOUNT_POINT_LEFT then
         otherDroneChargeReady = self.game.avatar:isBoostDroneChargedRight();
      else
         otherDroneChargeReady = self.game.avatar:isBoostDroneChargedLeft();
      end

      if (not otherDroneChargeReady or self._mountPosition == MOUNT_POINT_LEFT ) and not self.game.avatar:isSpeedBurstOn() then

         self._cooldownTimer = self._cooldownTimer + dt;

         if		self._cooldownTimer >= self.template.abilityCooldown
         or	( MANUAL_ACTIVATION and self._cooldownTimer >= self.template.abilityCooldown/2 ) then

            if MANUAL_ACTIVATION then
               self.state			= DRONE_CHARGED_INIT;
            else
               self:_initBoost();
            end
         end
      end
   end

end

----------------------------------------------------------------------

function DroneBoost:_vfxUpdate( dt )

   Debug.assert(self, "DroneBoost:_vfxUpdate -> self nil");
   Debug.assert(self.game, "DroneBoost:_vfxUpdate -> self.game nil");
   Debug.assert(self.game.avatar, "DroneBoost:_vfxUpdate -> self.game.avatar nil");

   if    not self
      or not self.game
      or not self.game.avatar then

      return;
   end

	if self.state == DRONE_CHARGED_INIT and not self.game.avatar:isDead() then

		local tbl	= { self._chevColorOn, self._chevColorOn, self._chevColorOn, self._chevColorOn };

		self._vfxBlinkTimer	= self._vfxBlinkTimer + dt;

		if self._vfxBlinkTimer > PULSE_TIME then
         EventMgr:postEvent( {sounds = {start={"BoostDroneCharge",},}, name="boost pulse"}, "add-sfx" );
			self._vfxBlinkTimer = 0;
		end

		if self._vfxBlinkTimer >= PULSE_TIME/2 then
			tbl	= { self._chevColorOff, self._chevColorOff, self._chevColorOff, self._chevColorOff };
		end

		for i=1,#tbl do
			self._chevrons[i]:setColor( tbl[i].r, tbl[i].g, tbl[i].b, 1 );
		end

		local k	= math.max((self._vfxBlinkTimer/2) / (PULSE_TIME/2), 0);

		if k > 1 then
			k = k-1;
		end

		local a	= math.max(1-k, 0);

		self._readyVFX:setVisible(true);
		self._readyVFX:setScl(k,k);
		self._readyVFX:setColor(self._chevColorOn.r, self._chevColorOn.g, self._chevColorOn.b, a);
	elseif self.state == DRONE_CHARGED_BOOSTING and not self.game.avatar:isDead() then

		self:_updateChevrons( math.max( 1 - self._vfxTimer/self.template.tplPowerup.duration, 0 ) );
		self._readyVFX:setVisible(false);

		self._vfxTimer = self._vfxTimer + dt;

		if self._vfxTimer >= self.template.tplPowerup.duration then
			self.state = DRONE_READY;

			self:resetCharge();
			self:_updateChevrons();
		end
	elseif self.state == DRONE_CHARGED_STANDBY and not self.game.avatar:isDead() then

		local tbl	= { self._chevColorStdby, self._chevColorStdby, self._chevColorStdby, self._chevColorStdby };

		local otherDroneChargeReady = nil;

		if self._mountPosition == MOUNT_POINT_LEFT then
			otherDroneChargeReady = self.game.avatar:isBoostDroneChargedRight();
		else
			otherDroneChargeReady = self.game.avatar:isBoostDroneChargedLeft();
		end

		if not otherDroneChargeReady and not MANUAL_ACTIVATION then
			tbl	= { self._chevColorOn, self._chevColorOn, self._chevColorOn, self._chevColorOn };
		end

		self._vfxBlinkTimer	= self._vfxBlinkTimer + dt;

		if self._vfxBlinkTimer > PULSE_TIME then
         EventMgr:postEvent( {sounds = {start={"BoostDroneCharge",},}, name="boost pulse"}, "add-sfx" );
			self._vfxBlinkTimer = 0;
		end

		if self._vfxBlinkTimer >= PULSE_TIME/2 then
			tbl	= { self._chevColorOff, self._chevColorOff, self._chevColorOff, self._chevColorOff };
		end

		for i=1,#tbl do
			self._chevrons[i]:setColor( tbl[i].r, tbl[i].g, tbl[i].b, 1 );
		end

		local k	= math.max((self._vfxBlinkTimer/2) / (PULSE_TIME/2), 0);

		if k > 1 then
			k = k-1;
		end

		local a	= math.max(1-k, 0);

		self._readyVFX:setVisible(true);
		self._readyVFX:setScl(k,k);
		if not otherDroneChargeReady and not MANUAL_ACTIVATION then
			self._readyVFX:setColor(self._chevColorOn.r, self._chevColorOn.g, self._chevColorOn.b, a);
		else
			self._readyVFX:setColor(self._chevColorStdby.r, self._chevColorStdby.g, self._chevColorStdby.b, a);
		end
	elseif self.state ~= DRONE_READY and not self.game.avatar:isDead() then
		self._readyVFX:setVisible(false);
		self:resetCharge();
	elseif self.game.avatar:isDead() then
		self._readyVFX:setVisible(false);
		self:resetCharge();

		for i=1,#self._pipOn do
			self._pipOn[i]:setVisible(false);
			self._pipOff[i]:setVisible(false);
		end
	end

end

----------------------------------------------------------------------

function DroneBoost:isBoostCharged()
   return self._chargeCounter >= self.template.chargeThreshold;
end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneBoost:_updateChevrons( percentage )

	local tbl	= { self._chevColorOff, self._chevColorOff, self._chevColorOff, self._chevColorOff };
	local c		= percentage or math.min( self._chargeCounter/self.template.chargeThreshold, 1 );

	if percentage then
		if c <= 0 then
			tbl	= { self._chevColorOff, self._chevColorOff, self._chevColorOff, self._chevColorOff };
		elseif c <= 0.25 then
			tbl	= { self._chevColorOn, self._chevColorOff, self._chevColorOff, self._chevColorOff };
		elseif c <= 0.5 then
			tbl	= { self._chevColorOn, self._chevColorOn, self._chevColorOff, self._chevColorOff };
		elseif c <= 0.75 then
			tbl	= { self._chevColorOn, self._chevColorOn, self._chevColorOn, self._chevColorOff };
		else
			tbl	= { self._chevColorOn, self._chevColorOn, self._chevColorOn, self._chevColorOn };
		end
	else
		if c < 0.25 then
			tbl	= { self._chevColorOff, self._chevColorOff, self._chevColorOff, self._chevColorOff };
		elseif c < 0.5 then
			tbl	= { self._chevColorOn, self._chevColorOff, self._chevColorOff, self._chevColorOff };
		elseif c < 0.75 then
			tbl	= { self._chevColorOn, self._chevColorOn, self._chevColorOff, self._chevColorOff };
		else
			tbl	= { self._chevColorOn, self._chevColorOn, self._chevColorOn, self._chevColorOff };
		end
	end

	for i=1,#tbl do
		self._chevrons[i]:setColor( tbl[i].r, tbl[i].g, tbl[i].b, 1 );
	end

	self._pipTracker	= {	false, false, false	};

	if percentage then
		if c <= 0 then
			self._pipTracker	= {	false, false, false	};
		elseif c < 1/3 then
			self._pipTracker	= { true, false, false};
		elseif c < 2/3 then
			self._pipTracker	= { true, true, false};
		else
			self._pipTracker	= { true, true, true};
		end
	else
		if c < 1/3 then
			self._pipTracker	= { false, false, false};
		elseif c < 2/3 then
			self._pipTracker	= { true, false, false};
		elseif c < 1 then
			self._pipTracker	= { true, true, false};
		else
			self._pipTracker	= { true, true, true};
		end
	end

	for i=1,#self._pipOn do
		if self._pipTracker[i] then
			self._pipOn[i]:setVisible(true);
			self._pipOff[i]:setVisible(false);
		else
			self._pipOn[i]:setVisible(false);
			self._pipOff[i]:setVisible(true);
		end
	end

end

----------------------------------------------------------------------

function DroneBoost:_updatePipPulse( dt )

	if self._pipTracker then

		if self._pipTimer > PULSE_TIME then
			self._pipTimer	= 0;
		end

		local k;

		if self._pipTimer < PULSE_TIME/2 then
			k	= math.mapClamped(0, 1, PULSE_TIME/2, 1.5, self._pipTimer);
		else
			k	= math.mapClamped(PULSE_TIME/2, 1.5, PULSE_TIME, 1, self._pipTimer);
		end

		for i=1,#self._pipOn do
			if self._pipTracker[i] then
				self._pipOn[i]:setScl(k, k);
			else
				self._pipOn[i]:setScl(1, 1);
			end
		end

		self._pipTimer	= self._pipTimer + dt;
	end

end


----------------------------------------------------------------------
----------------------------------------------------------------------

function DroneBoost:_initBoost()

   Debug.assert(self, "DroneBoost:_initBoost -> self nil");
   Debug.assert(self.game, "DroneBoost:_initBoost -> self.game nil");
   Debug.assert(self.game.avatar, "DroneBoost:_initBoost -> self.game.avatar nil");

   if    not self
      or not self.game
      or not self.game.avatar then

      return;
   end

   if not self.game.avatar:isDead() and not self.game.avatar:isSpeedBurstOn() then

      local x,y				= self.game.avatar:getLoc();

      self.state				= DRONE_CHARGED_BOOSTING;
      self._initTimer		= 0;
      self._vfxTimer			= 0;
      self._vfxBlinkTimer	= 0;
      self._cooldownTimer	= 0;

      self.game.avatar:activateDroneBoost(self.template.tplPowerup, self._mountPosition);
   end

end

----------------------------------------------------------------------

function DroneBoost:onEnemyKilled( event, type )

   Debug.assert(self, "DroneBoost:onEnemyKilled -> self nil");
   Debug.assert(self.game, "DroneBoost:onEnemyKilled -> self.game nil");
   Debug.assert(self.game.avatar, "DroneBoost:onEnemyKilled -> self.game.avatar nil");

   if    not self
      or not self.game
      or not self.game.avatar then

      return;
   end

	if not self.game.avatar:isSpeedBurstOn() and self.state == DRONE_READY and not self:isBoostCharged() then

		local otherDroneChargeReady;

		if self._mountPosition == MOUNT_POINT_LEFT then
			otherDroneChargeReady = self.game.avatar:isBoostDroneChargedRight();
		else
			otherDroneChargeReady = self.game.avatar:isBoostDroneChargedLeft();
		end

		self._chargeCounter = self._chargeCounter + 1;

		self:_updateChevrons();

		if	self:isBoostCharged() and not otherDroneChargeReady then

			self.state				= DRONE_CHARGED_INIT;
			self._initTimer		= 0;
			self._vfxTimer			= 0;
			self._vfxBlinkTimer	= 0;
			self._cooldownTimer	= 0;

			EventMgr:postEvent(	{	text="Boost Drone Activated"	}, "hud-alert-text"	);
		elseif self:isBoostCharged() and otherDroneChargeReady == true then

			self.state				= DRONE_CHARGED_STANDBY;
			self._initTimer		= 0;
			self._vfxTimer			= 0;
			self._vfxBlinkTimer	= 0;
			self._cooldownTimer	= 0;

			EventMgr:postEvent(	{	text="Boost Drone Charged"	}, "hud-alert-text"	);
		end
	end

end

----------------------------------------------------------------------

function DroneBoost:resetCharge()

	local tbl	= { self._chevColorOff, self._chevColorOff, self._chevColorOff, self._chevColorOff };

	self._chargeCounter	= 0;
	self._initTimer		= 0;
	self._vfxTimer			= 0;
	self._vfxBlinkTimer	= 0;
	self._cooldownTimer	= 0;
	self._pipTracker		= {false, false, false};
	self._pipTimer			= 0;

	for i=1,#tbl do
		self._chevrons[i]:setColor( tbl[i].r, tbl[i].g, tbl[i].b, 1 );
	end

	for i=1,#self._pipOn do
		self._pipOn[i]:setVisible(false);
		self._pipOff[i]:setVisible(true);
	end

end

----------------------------------------------------------------------

function DroneBoost:onInputSwipeUp()

	if			MANUAL_ACTIVATION
		and	self.state == DRONE_CHARGED_INIT
		and	not self.game.avatar:isDead()
		and	not self.game.avatar:isSpeedBurstOn() then

		self:_initBoost();
	end

end

----------------------------------------------------------------------

function DroneBoost:isActivationManual()
	return MANUAL_ACTIVATION;
end

----------------------------------------------------------------------

function DroneBoost:isChargeRequired()
	return true;
end

----------------------------------------------------------------------

function DroneBoost:setCharge( k )

	self._chargeCounter = math.clamp( k*(self.template.chargeThreshold - 1), 0, self.template.chargeThreshold - 1 );

   self:onEnemyKilled();

	return true;

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneBoost:delete()

	if self._chevrons then
		for k,v in pairs(self._chevrons) do
			v:delete();
		end
	end

	if self._readyVFX then
		self._readyVFX:delete();
	end

	if self._pipOn then
		for k,v in pairs(self._pipOn) do
			v:delete();
		end
	end

	if self._pipOff then
		for k,v in pairs(self._pipOff) do
			v:delete();
		end
	end

	Base.delete(self);

end

----------------------------------------------------------------------

function DroneBoost:hide()

	Base.hide(self);

	if self._chevrons then
		for k,v in pairs(self._chevrons) do
			v:setVisible(false);
		end
	end

	if self._readyVFX then
		self._readyVFX:setVisible(false);
	end

	if self._pipOn then
		for k,v in pairs(self._pipOn) do
			v:setVisible(false);
		end
	end

	if self._pipOff then
		for k,v in pairs(self._pipOff) do
			v:setVisible(false);
		end
	end

end

----------------------------------------------------------------------

function DroneBoost:show()

	Base.show(self);

	if self._chevrons then
		for k,v in pairs(self._chevrons) do
			v:setVisible(true);
		end
	end

	if self._readyVFX then
		self._readyVFX:setVisible(false);
	end

	if self._pipOn then
		for k,v in pairs(self._pipOn) do
			v:setVisible(false);
		end
	end

	if self._pipOff then
		for k,v in pairs(self._pipOff) do
			v:setVisible(true);
		end
	end

end


----------------------------------------------------------------------
----------------------------------------------------------------------
