require('game/entity/drone/drone');

local MOUNT_POINT_LEFT			= "left";
local MOUNT_POINT_RIGHT			= "right";

local DRONE_DISABLED				= "disabled";
local DRONE_RESPAWN_COOLDOWN	= "respawn-cooldown";
local DRONE_RESPAWN				= "respawn";
local DRONE_READY					= "ready";

----------------------------------------------------------------------

local Base	= Drone;
DroneGun		= Base:new();

----------------------------------------------------------------------


function DroneGun:new( game, layers, template,  x, y, angle, parent, offset, mountPoint, garage_index, visData, stats )

	-- modify base template to fit player vehicle
	if template and garage_index and visData then
		local rarity;
		local level;

		if mountPoint == MOUNT_POINT_LEFT then
			rarity	= math.min( DroneMgr:getDroneRarityLeft(garage_index), DroneMgr:getDroneMaxRarity() );
			level		= math.min( DroneMgr:getDroneLevelLeft(garage_index), DroneMgr:getDroneMaxLevel() );
		else
			rarity	= math.min( DroneMgr:getDroneRarityRight(garage_index), DroneMgr:getDroneMaxRarity() );
			level		= math.min( DroneMgr:getDroneLevelRight(garage_index), DroneMgr:getDroneMaxLevel() );
		end

		template.projectile.texture	= visData.texture[rarity][level];
		template.shootDelay				= visData.shootDelay[level];
		template.bulletCount				= visData.bulletCount[level];
		template.hitpoints				= parent.hitPoints;

		local count = (template.bulletCount or 1) / ( (template.bulletCount or 1) * (template.bulletDelay or 0)  + (template.shootDelay or 0) );

		template.projectile.damage		= template.projectile.damage * ( App.data:getCurrentPlayerMainWeaponMaxDPS() / ( stats.rarity[rarity] - stats.level[level] ) ) / count;
	end

	local e = Base.new( self, game, layers, template, x, y, angle, parent, offset, mountPoint, garage_index );

   if template and mountPoint then

      --call here because avatar doesnt get called yet
      e:reset( x, y, angle, parent, offset  );
   end

   return e;

end

----------------------------------------------------------------------

function DroneGun:reset( x, y, angle, parent, offset )

   Base.reset(self, x, y, angle, parent, offset);

	self._shootTimer		= 0;
	self._bulletTimer		= random.mapClamped(0, self.template.bulletDelay);
	self._bulletCounter	= self.template.bulletCount or 1;
	self._bulletOffset	= self.template.bulletDelta or { { x=0, y=0} };
	self._offsetIndex		= 1;

end

----------------------------------------------------------------------

function DroneGun:update( dt, avatarSpeedY, burstOn, canShoot )

	Base.update(self, dt, avatarSpeedY);

	self:_updateShoot(dt, burstOn, canShoot)

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneGun:_updateShoot( dt, burstOn, canShoot )

	if self.state == DRONE_READY and canShoot then

		self._shootTimer = self._shootTimer - dt;

		if self._shootTimer < 0 then
			self._bulletTimer = self._bulletTimer - dt;

			if self._bulletTimer < 0 then
				self:_shoot(burstOn);
			end
		end
	end

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneGun:_getBulletOffset()

   local x,y	= self:getLoc(true);

	local delta = self._bulletOffset[self._offsetIndex];
	local dX,dY	= (delta.x or 0), (delta.y or 0);

	self._offsetIndex = self._offsetIndex + 1;

	if self._offsetIndex > #self._bulletOffset then
		self._offsetIndex		= 1;
	end

   return x+dX, y+dY;

end

----------------------------------------------------------------------

function DroneGun:_shoot( burstOn )

	self._bulletCounter		= self._bulletCounter - 1;
	self._bulletTimer			= self.template.bulletDelay;

	if self._bulletCounter < 1 then
		self._shootTimer		= self.template.shootDelay;
		self._bulletCounter	= self.template.bulletCount or 1;
	end

	local x,y = self:_getBulletOffset();
	local p;

	p = self.game.entityMgr:spawnProjectile( self.template.projectile, x, y, self.angle, self.parent );
   EventMgr:postEvent( {sounds = {start={"MachinegunDrone1", "MachinegunDrone2", "MachinegunDrone3",},}, name="machinegun drone fire"}, "add-sfx" );

	p.speedBurst = burstOn;

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneGun:hide()

	Base.hide(self);

end

----------------------------------------------------------------------

function DroneGun:show()

	Base.show(self);

end


----------------------------------------------------------------------
----------------------------------------------------------------------