require('game/entity/drone/drone');

local MOUNT_POINT_LEFT			= "left";
local MOUNT_POINT_RIGHT			= "right";

local DRONE_DISABLED				= "disabled";
local DRONE_RESPAWN_COOLDOWN	= "respawn-cooldown";
local DRONE_RESPAWN				= "respawn";
local DRONE_READY					= "ready";

local BEAM_EXTEND_TIME			= 10/30;
local BEAM_RETRACT_TIME			= 3/30;
local BEAM_PULSE_FREQ			= 1.0;
local BEAM_PULSE_WIDTH			= 0.4;
local BEAM_PULSE_ALPHA			= 0.6;
local BEAM_HOLD_TIME				= 1/30;
local BEAM_COOLDOWN				= 1.25;

local BEAM_STATE_ALL_OFF		= "all-off";
local BEAM_STATE_MINOR_EXTEND	= "minor-extend";
local BEAM_STATE_MINOR_ROT		= "minor-rotate";
local BEAM_STATE_MINOR_RETURN	= "minor-retract";
local BEAM_STATE_ALL_PAUSE		= "all-pause";
local BEAM_STATE_MAJOR_EXTEND	= "major-extend";
local BEAM_STATE_MAJOR_PULSE	= "major-pulse";
local BEAM_STATE_MAJOR_FADE	= "major-fade";

----------------------------------------------------------------------

local Base	= Drone;
DroneLaser	= Base:new();

----------------------------------------------------------------------


function DroneLaser:new( game, layers, template,  x, y, angle, parent, offset, mountPoint, garage_index, visData, stats )

	-- modify base template to fit player vehicle
	if template and garage_index and visData then

		local rarity;
		local level;

		if mountPoint == MOUNT_POINT_LEFT then
			rarity	= math.min( DroneMgr:getDroneRarityLeft(garage_index), #visData.projectile );
			level		= math.min( DroneMgr:getDroneLevelLeft(garage_index), #visData.projectile );
		else
			rarity	= math.min( DroneMgr:getDroneRarityRight(garage_index), #visData.projectile );
			level		= math.min( DroneMgr:getDroneLevelRight(garage_index), #visData.projectile );
		end

		template.bulletCount							= visData.beamCount[rarity]
		template.beamMinor.texture					= visData.beamMinor[rarity];
		template.beamMinor.offset					= visData.beamOffset[rarity];
		template.beamMajor.texture					= visData.beamMajor[rarity];
		template.beamMajor.width					= visData.beamMajorWidth[rarity] * template.beamMajor.width;
		template.muzzleGlow.texture				= visData.glow[rarity];
		template.muzzleFlash.texture				= visData.muzzleFlash[rarity];
		template.muzzleFlash.scale					= visData.muzzleFlashScale[rarity];
		template.projectile.texture				= visData.projectile[rarity];
		template.projectile.hitMarker.texture	= visData.impact[rarity];
		template.hitpoints							= parent.hitPoints;
		template.mainBeamDuration					= visData.beamMajorDuration[level];

		local count = (template.bulletCount or 1) / ( (template.bulletCount or 1) * (template.bulletDelay or 0)  + (template.shootDelay or 0) );

		template.projectile.damage		= template.projectile.damage * ( App.data:getCurrentPlayerMainWeaponMaxDPS() / ( stats.rarity[rarity] - stats.level[level] ) ) / count;

		template.projectileM						= copyTable(template.projectile);
		template.projectileM.damage			= template.projectile.damage * template.bulletCount * 3;
		template.projectileM.width				= template.beamMajor.width*0.3;
		template.projectileM.height			= template.beamMajor.width*0.15;
		template.projectileM.collisionWidth	= template.projectileM.width;
	end

	local e = Base.new( self, game, layers, template, x, y, angle, parent, offset, mountPoint, garage_index );

   if template and mountPoint then

		e.beamMinor	= {};

		for i=1,template.bulletCount do
			e.beamMinor[i]	= e:_createVisProp( layers[template.beamMinor.layer or 5], template.beamMinor.priority, template.beamMinor.offset[i][1], template.beamMinor.offset[i][2], 0, template.beamMinor.width, template.beamMinor.height, template.beamMinor.texture, template.beamMinor.scale,  template.beamMinor.color, template.beamMinor.blend );
			e.beamMinor[i]:setRot(template.beamMinor.offset[i][3]);
			e.beamMinor[i]:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e.beamMinor[i]:setVisible(false);
		end

		e.beamMajor = e:_createVisProp( layers[template.beamMajor.layer or 5], template.beamMajor.priority, template.beamMajor.offset[1], template.beamMajor.offset[2], 0, template.beamMajor.width, template.beamMajor.height, template.beamMajor.texture, template.beamMajor.scale,  template.beamMajor.color, template.beamMajor.blend );
		e.beamMajor:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
		e.beamMajor:setVisible(false);

		e.glow	= e:_createVisProp( layers[template.muzzleGlow.layer or template.layer or 5], template.muzzleGlow.priority, template.muzzleGlow.offset[1], template.muzzleGlow.offset[2], 0, template.muzzleGlow.width, template.muzzleGlow.height, template.muzzleGlow.texture, nil, nil, template.muzzleGlow.blend );
		e.glow:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );

		e.spark	= e:_createVisProp( layers[template.muzzleSpark.layer or template.layer or 5], template.muzzleSpark.priority, template.muzzleSpark.offset[1], template.muzzleSpark.offset[2], 0, template.muzzleSpark.width, template.muzzleSpark.height, template.muzzleSpark.texture, nil, nil, template.muzzleSpark.blend );
		e.spark:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );

		e.muzzleFlash = {};

		for i=1,#template.muzzleFlash.offset do
			e.muzzleFlash[i]	= e:_createVisProp( layers[template.muzzleFlash.layer or 5], template.muzzleFlash.priority, template.muzzleFlash.offset[i][1], template.muzzleFlash.offset[i][2], 0, template.muzzleFlash.width, template.muzzleFlash.height, template.muzzleFlash.texture, nil,  template.muzzleFlash.color, template.muzzleFlash.blend );
			e.muzzleFlash[i]:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e.muzzleFlash[i]:setVisible(false);
		end

      --call here because avatar doesnt get called yet
      e:reset( x, y, angle, parent, offset  );
   end

   return e;

end

----------------------------------------------------------------------

function DroneLaser:reset( x, y, angle, parent, offset )

   Base.reset(self, x, y, angle, parent, offset);

	self._shootTimer		= (self.template.shootDelay or 0) + BEAM_EXTEND_TIME*2 + BEAM_HOLD_TIME + BEAM_COOLDOWN;
	self._bulletTimer		= self.template.bulletDelay or 0;
	self._bulletCounter	= self.template.bulletCount or 1;
	self._bulletOffset	= self.template.bulletDelta or { { x=0, y=0} };
	self._offsetIndex		= 1;
	self._beamWTimer		= BEAM_PULSE_FREQ;
	self._beamLTimer		= 0;
	self._beamMajorTimer	= self.template.mainBeamDuration;
	self._muzzleGlowFlip	= false;
	self._muzzleFlashTimer	= 0;

end

----------------------------------------------------------------------

function DroneLaser:update( dt, avatarSpeedY, burstOn, canShoot )

	Base.update(self, dt, avatarSpeedY);

	self:_updateShoot(dt, burstOn, canShoot);
	self:_updateBeam(dt, burstOn, canShoot);
	self:_updateGlow(dt, canShoot);
	self:_updateMuzzleFlash(dt)

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneLaser:_updateShoot( dt, burstOn, canShoot )

	if			self.state == DRONE_READY
		and	(	canShoot
		or			(self._beamMajorTimer < self.template.mainBeamDuration)	) then

		if self._beamState == BEAM_STATE_MINOR_ROT then

			self._bulletTimer = math.max(self._bulletTimer - dt, 0);

			if self._bulletTimer <= 0 then
				self:_shoot(dt, burstOn, false);
			end
		elseif		self._beamState == BEAM_STATE_MAJOR_EXTEND
					or	self._beamState == BEAM_STATE_MAJOR_PULSE then

			self._bulletTimer		= math.max(self._bulletTimer - dt, 0);
			self._beamMajorTimer	= self._beamMajorTimer - dt;

			if self._bulletTimer <= 0 then
				self:_shoot(dt, burstOn, true);
			end
		elseif self._beamState == BEAM_STATE_MAJOR_FADE then
			self._beamMajorTimer	= self._beamMajorTimer - dt;

			if self._beamMajorTimer <= 0 then
				self._shootTimer		= (self.template.shootDelay or 0) + BEAM_EXTEND_TIME*3 + BEAM_COOLDOWN;
				self._bulletTimer		= self.template.bulletDelay or 0;
				self._bulletCounter	= self.template.bulletCount or 1;
				self._offsetIndex		= 1;
				self._beamMajorTimer	= self.template.mainBeamDuration;
			end
		end

		self._shootTimer = math.max(self._shootTimer - dt, 0);
	else
		self._shootTimer		= self.template.shootDelay + BEAM_EXTEND_TIME*3 + BEAM_COOLDOWN;
		self._bulletTimer		= self.template.bulletDelay or 0;
		self._bulletCounter	= self.template.bulletCount or 1;
		self._offsetIndex		= 1;
		self._beamWTimer		= BEAM_PULSE_FREQ;
		self._beamLTimer		= 0;
		self._beamMajorTimer	= self.template.mainBeamDuration;
		self._muzzleGlowFlip	= false;
		self._muzzleFlashTimer	= 0;
	end

end

----------------------------------------------------------------------

function DroneLaser:_updateBeam( dt, burstOn, canShoot )

	self._beamState	= BEAM_STATE_ALL_OFF;

	if			self.state == DRONE_READY
		and	(	canShoot
		or			(self._beamMajorTimer < self.template.mainBeamDuration)	) then

		if self._shootTimer > (self.template.shootDelay + BEAM_EXTEND_TIME + BEAM_RETRACT_TIME + BEAM_HOLD_TIME)
			or self.parent:isDead() then

         if self.beamLoopSFX then
            EventMgr:postEvent( {sounds={ loop={"LaserDroneLoop",},}, name="laser drone loop",}, "loop-stop" );
            self.beamLoopSFX = false;
         end
         self.beamLoopSFX = false;
			self._beamState		= BEAM_STATE_ALL_OFF;
		elseif self._shootTimer > (self.template.shootDelay + BEAM_RETRACT_TIME + BEAM_HOLD_TIME) then
			self._beamState		= BEAM_STATE_MINOR_EXTEND;
         EventMgr:postEvent( {sounds = {start={"LaserDroneWarmup",},}, name="laser drone warmup"}, "add-sfx" );
         if self.beamLoopSFX then
            EventMgr:postEvent( {sounds={ loop={"LaserDroneLoop",},}, name="laser drone loop",}, "loop-stop" );
            self.beamLoopSFX = false;
         end
         self.beamLoopSFX = false;
		elseif self._shootTimer > BEAM_RETRACT_TIME + BEAM_HOLD_TIME then
			self._beamState		= BEAM_STATE_MINOR_ROT;
         if self.beamLoopSFX then
            EventMgr:postEvent( {sounds={ loop={"LaserDroneLoop",},}, name="laser drone loop",}, "loop-stop" );
            self.beamLoopSFX = false;
         end
         self.beamLoopSFX = false;
		elseif self._shootTimer > BEAM_HOLD_TIME then
			self._beamState		= BEAM_STATE_MINOR_RETURN;
         if self.beamLoopSFX then
            EventMgr:postEvent( {sounds={ loop={"LaserDroneLoop",},}, name="laser drone loop",}, "loop-stop" );
            self.beamLoopSFX = false;
         end
         self.beamLoopSFX = false;
		elseif self._shootTimer > 0 then
			self._beamState		= BEAM_STATE_ALL_PAUSE;
         if self.beamLoopSFX then
            EventMgr:postEvent( {sounds={ loop={"LaserDroneLoop",},}, name="laser drone loop",}, "loop-stop" );
            self.beamLoopSFX = false;
         end
         self.beamLoopSFX = false;
		else
			if self._beamMajorTimer > (self.template.mainBeamDuration - BEAM_EXTEND_TIME) then
				self._beamState	= BEAM_STATE_MAJOR_EXTEND;
            if not self.beamLoopSFX then
               EventMgr:postEvent( {sounds = {start={"LaserDroneFire",},}, name="laser drone fire"}, "add-sfx" );
               EventMgr:postEvent( {sounds = {loop={"LaserDroneLoop",},}, name="laser drone loop"}, "add-sfx" );
               self.beamLoopSFX = true;
            end
			elseif self._beamMajorTimer > BEAM_EXTEND_TIME then
				self._beamState	= BEAM_STATE_MAJOR_PULSE;
			else
				self._beamState	= BEAM_STATE_MAJOR_FADE
            if self.beamLoopSFX then
               EventMgr:postEvent( {sounds={ loop={"LaserDroneLoop",},}, name="laser drone loop",}, "loop-stop" );
               self.beamLoopSFX = false;
            end
			end
		end
	end

	self:_updateBeamLength(dt, self._beamState);
	self:_updateBeamPulse(dt, self._beamState);
	self:_updateBeamRotation(dt, self._beamState);

end

----------------------------------------------------------------------

function DroneLaser:_updateGlow( dt, canShoot )

	if			self.state == DRONE_READY
		and	not self.parent:isDead()
		and	canShoot
		and	self._shootTimer < (self.template.shootDelay + BEAM_RETRACT_TIME + BEAM_HOLD_TIME) then

		local k	= math.mapClamped(0, 1, (self.template.shootDelay + BEAM_RETRACT_TIME + BEAM_HOLD_TIME), 0, self._shootTimer);
		local sk	= math.mapClamped(0, 0, (self.template.shootDelay + BEAM_RETRACT_TIME + BEAM_HOLD_TIME), 1, self._shootTimer);
		local a	= math.clampAngle360( self.spark:getRot() + 360*dt)
		local f	= 1;

		if self._muzzleGlowFlip then
			self._muzzleGlowFlip	= false;
			f							= -1;
		else
			self._muzzleGlowFlip	= true;
			f							= 1;
		end

		if self._shootTimer <= 0 then
			self.glow:setVisible(false);
			self.spark:setVisible(false);
		else
			self.glow:setVisible(true);
			self.glow:setScl( k*self.template.muzzleGlow.scale[1]*f, k*self.template.muzzleGlow.scale[2] );
			self.spark:setVisible(true);
			self.spark:setScl( sk*self.template.muzzleSpark.scale[1], sk*self.template.muzzleSpark.scale[2] );
			self.spark:setRot(a);
		end
	else
		self.glow:setVisible(false);
		self.spark:setVisible(false);
	end

end

----------------------------------------------------------------------

function DroneLaser:_updateMuzzleFlash( dt )

	if		self._beamState == BEAM_STATE_MAJOR_EXTEND
		or	self._beamState == BEAM_STATE_MAJOR_PULSE
		or	self._beamState == BEAM_STATE_MAJOR_FADE then

		local hFreq	= 1/3;
		local fFreq	= 2/3

		self._muzzleFlashTimer		= self._muzzleFlashTimer + dt;

		if self._muzzleFlashTimer > fFreq then
			self._muzzleFlashTimer	= 0;
		end

		if self._muzzleFlashTimer > hFreq then
			local w	= math.mapClamped(hFreq, 1, fFreq, self.template.muzzleFlash.scale[1], self._muzzleFlashTimer);
			local h	= math.mapClamped(hFreq, 1, fFreq, self.template.muzzleFlash.scale[2], self._muzzleFlashTimer);
			local a	= math.mapClamped(hFreq, 1, fFreq, 0, self._muzzleFlashTimer);

			self.muzzleFlash[1]:setVisible(false);
			self.muzzleFlash[2]:setVisible(true);
			self.muzzleFlash[2]:setScl(w, h);
			self.muzzleFlash[2]:setColor(1, 1, 1, a);
		else
			local w	= math.mapClamped(0, 1, hFreq, self.template.muzzleFlash.scale[1], self._muzzleFlashTimer);
			local h	= math.mapClamped(0, 1, hFreq, self.template.muzzleFlash.scale[2], self._muzzleFlashTimer);
			local a	= math.mapClamped(0, 1, hFreq, 0, self._muzzleFlashTimer);

			self.muzzleFlash[1]:setVisible(true);
			self.muzzleFlash[1]:setScl(w, h);
			self.muzzleFlash[1]:setColor(1, 1, 1, a);
			self.muzzleFlash[2]:setVisible(false);
		end
	else
		self._muzzleFlashTimer	= 0;

		if self.muzzleFlash then
			for _,v in pairs(self.muzzleFlash) do
				v:setVisible(false);
			end
		end
	end

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneLaser:_getBulletOffset()

	local offset	= { x=0,	y=0,	angle=0 };
	local delta		= self._bulletOffset[self._offsetIndex];
	local dX,dY		= (delta.x or 0), (delta.y or 0);

	self._offsetIndex = self._offsetIndex + 1;

	if self._offsetIndex > #self._bulletOffset then
		self._offsetIndex		= 1;
	end

	offset.x = dX;
	offset.y	= dY;

   return offset;

end

----------------------------------------------------------------------

function DroneLaser:_shoot( dt, burstOn, majorBeam )

	self._bulletCounter		= self._bulletCounter - 1;
	self._bulletTimer			= self.template.bulletDelay or 0;

	if majorBeam then

		local offset = self:_getBulletOffset();
		local p;

		p = self.game.entityMgr:spawnProjectile( self.template.projectileM, offset.x, offset.y, offset.angle, self.parent, nil, self.beamMajor, offset );

		p.speedBurst = burstOn;
	else

		local offset = self:_getBulletOffset();

		for i=1,#self.beamMinor do
			local p;

			p = self.game.entityMgr:spawnProjectile( self.template.projectile, offset.x, offset.y, offset.angle, self.parent, nil, self.beamMinor[i], offset );

			p.speedBurst = burstOn;
		end
	end

end

----------------------------------------------------------------------

function DroneLaser:_updateBeamLength( dt, beamState )

	if beamState == BEAM_STATE_ALL_OFF then

		self._beamLTimer	= 0;

		for i=1,#self.beamMinor do
			self.beamMinor[i]:setScl(1,1);
			self.beamMinor[i]:setVisible(false);
		end

		self.beamMajor:setScl(1,1);
		self.beamMajor:setVisible(false);
	elseif beamState == BEAM_STATE_MINOR_EXTEND then

		self._beamLTimer	= math.min(self._beamLTimer + dt, BEAM_EXTEND_TIME);

		for i=1,#self.beamMinor do
			local w,l = self.beamMinor[i]:getScl();

			self.beamMinor[i]:setVisible(true);
			self.beamMinor[i]:setScl(w, self._beamLTimer/BEAM_EXTEND_TIME);
		end

		self.beamMajor:setScl(1,1);
		self.beamMajor:setVisible(false);
	elseif beamState == BEAM_STATE_MINOR_ROT then

		self._beamLTimer	= BEAM_RETRACT_TIME;

		for i=1,#self.beamMinor do
			local w,l = self.beamMinor[i]:getScl();

			self.beamMinor[i]:setVisible(true);
			self.beamMinor[i]:setScl(w, 1);
		end

		self.beamMajor:setScl(1,1);
		self.beamMajor:setVisible(false);
	elseif beamState == BEAM_STATE_MINOR_RETURN then

		self._beamLTimer	= math.max(self._beamLTimer - dt, 0);

		for i=1,#self.beamMinor do
			local w,l = self.beamMinor[i]:getScl();

			self.beamMinor[i]:setVisible(true);
			self.beamMinor[i]:setScl(w, self._beamLTimer/BEAM_RETRACT_TIME);
		end

		self.beamMajor:setScl(1,1);
		self.beamMajor:setVisible(false);
	elseif beamState == BEAM_STATE_ALL_PAUSE then

		self._beamLTimer	= 0;

		for i=1,#self.beamMinor do
			self.beamMinor[i]:setScl(1,1);
			self.beamMinor[i]:setVisible(false);
		end

		self.beamMajor:setScl(1,1);
		self.beamMajor:setVisible(false);
	elseif beamState == BEAM_STATE_MAJOR_EXTEND then

		self._beamLTimer	= math.min(self._beamLTimer + dt, BEAM_EXTEND_TIME);

		for i=1,#self.beamMinor do
			self.beamMinor[i]:setScl(1, 1);
			self.beamMinor[i]:setVisible(false);
		end

		local w,l = self.beamMajor:getScl();

		self.beamMajor:setVisible(true);
		self.beamMajor:setScl(w, self._beamLTimer/BEAM_EXTEND_TIME);
	elseif beamState == BEAM_STATE_MAJOR_PULSE then

		self._beamLTimer	= 0;

		for i=1,#self.beamMinor do
			self.beamMinor[i]:setScl(1, 1);
			self.beamMinor[i]:setVisible(false);
		end

		local w,l = self.beamMajor:getScl();

		self.beamMajor:setVisible(true);
		self.beamMajor:setScl(w, 1);
	elseif beamState == BEAM_STATE_MAJOR_FADE then

		self._beamLTimer	= math.min(self._beamLTimer + dt, BEAM_EXTEND_TIME);

		for i=1,#self.beamMinor do
			self.beamMinor[i]:setScl(1, 1);
			self.beamMinor[i]:setVisible(false);
		end

		local w	= math.mapClamped(0, 1, BEAM_EXTEND_TIME, 0, self._beamLTimer);
		local a	= math.mapClamped(0, 0.8, BEAM_EXTEND_TIME, 0, self._beamLTimer);

		self.beamMajor:setVisible(true);
		self.beamMajor:setScl(w, 1);
		self.beamMajor:setColor(1,1,1, a);
	end

end

----------------------------------------------------------------------

function DroneLaser:_updateBeamPulse( dt, beamState )

	local hfreq	= BEAM_PULSE_FREQ/2;

	if beamState == BEAM_STATE_ALL_OFF then

		self._beamWTimer = 0;

		for i=1,#self.beamMinor do
			self.beamMinor[i]:setScl(1,1);
			self.beamMinor[i]:setColor(1,1,1,1);
			self.beamMinor[i]:setVisible(false);
		end

		self.beamMajor:setScl(1,1);
		self.beamMajor:setColor(1,1,1,1);
		self.beamMajor:setVisible(false);
	elseif		beamState == BEAM_STATE_MINOR_EXTEND
				or	beamState == BEAM_STATE_MINOR_ROT
				or	beamState == BEAM_STATE_MINOR_RETURN then

		self._beamWTimer	= self._beamWTimer + dt;

		for i=1,#self.beamMinor do
			local w,l	= self.beamMinor[i]:getScl();
			local a		= 1;

			if self._beamWTimer > BEAM_PULSE_FREQ then
				self._beamWTimer = 0;
			end

			if self._beamWTimer < hfreq then
				w	= math.mapClamped(0, 1, hfreq, BEAM_PULSE_WIDTH, self._beamWTimer);
				a	= math.mapClamped(0, 1, hfreq, BEAM_PULSE_ALPHA, self._beamWTimer);
			else
				w	= math.mapClamped(hfreq, BEAM_PULSE_WIDTH, BEAM_PULSE_FREQ, 1, self._beamWTimer);
				a	= math.mapClamped(hfreq, BEAM_PULSE_ALPHA, BEAM_PULSE_FREQ, 1, self._beamWTimer);
			end

			self.beamMinor[i]:setVisible(true);
			self.beamMinor[i]:setScl(w, l);
			self.beamMinor[i]:setColor(1,1,1,a);
		end

		self.beamMajor:setScl(1,1);
		self.beamMajor:setColor(1,1,1,1);
		self.beamMajor:setVisible(false);
	elseif beamState == BEAM_STATE_ALL_PAUSE then
		self._beamWTimer = 0;

		for i=1,#self.beamMinor do
			self.beamMinor[i]:setScl(1,1);
			self.beamMinor[i]:setColor(1,1,1,1);
			self.beamMinor[i]:setVisible(false);
		end

		self.beamMajor:setScl(1,1);
		self.beamMajor:setColor(1,1,1,1);
		self.beamMajor:setVisible(false);
	elseif		beamState == BEAM_STATE_MAJOR_EXTEND
				or	beamState == BEAM_STATE_MAJOR_PULSE then

		self._beamWTimer	= self._beamWTimer + dt;

		for i=1,#self.beamMinor do
			self.beamMinor[i]:setScl(1, 1);
			self.beamMinor[i]:setColor(1,1,1,1);
			self.beamMinor[i]:setVisible(false);
		end

		if self._beamWTimer > BEAM_PULSE_FREQ then
			self._beamWTimer = 0;
		end

		local w,l	= self.beamMajor:getScl();
		local a		= 1;

		if self._beamWTimer < hfreq then
			w	= math.mapClamped(0, 0.8, hfreq, BEAM_PULSE_WIDTH, self._beamWTimer);
			a	= math.mapClamped(0, 0.8, hfreq, BEAM_PULSE_ALPHA, self._beamWTimer);
		else
			w	= math.mapClamped(hfreq, BEAM_PULSE_WIDTH, BEAM_PULSE_FREQ, 0.8, self._beamWTimer);
			a	= math.mapClamped(hfreq, BEAM_PULSE_ALPHA, BEAM_PULSE_FREQ, 0.8, self._beamWTimer);
		end

		self.beamMajor:setVisible(true);
		self.beamMajor:setScl(w,l);
		self.beamMajor:setColor(1,1,1,a);
	end

end

----------------------------------------------------------------------

function DroneLaser:_updateBeamRotation( dt, beamState )

	if beamState == BEAM_STATE_MINOR_EXTEND then

		for i=1,#self.beamMinor do
			self.beamMinor[i]:setRot(self.template.beamMinor.offset[i][3]);
		end
	elseif beamState == BEAM_STATE_MINOR_ROT then

		for i=1,#self.beamMinor do
			local a	= math.mapClamped( (self.template.shootDelay+BEAM_RETRACT_TIME+BEAM_HOLD_TIME), self.template.beamMinor.offset[i][3], BEAM_RETRACT_TIME+BEAM_HOLD_TIME, 0, self._shootTimer )

			self.beamMinor[i]:setRot(a);
		end
	else

		for i=1,#self.beamMinor do
			self.beamMinor[i]:setRot(0);
		end
	end

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneLaser:delete()

	if self.beamMinor then
		for i=#self.beamMinor,1,-1 do
			self.beamMinor[i]:delete();
		end
	end

	if self.beamMajor then
		self.beamMajor:delete();
	end

	if self.glow then
		self.glow:delete();
	end

	if self.spark then
		self.spark:delete();
	end

	if self.muzzleFlash then
		for i=#self.muzzleFlash,1,-1 do
			self.muzzleFlash[i]:delete();
		end
	end

	Base.delete(self);

end

----------------------------------------------------------------------

function DroneLaser:hide()

	Base.hide(self);

	if self.beamMinor then
		for _,v in pairs(self.beamMinor) do
			v:setVisible(false);
		end
	end

	if self.beamMajor then
		self.beamMajor:setVisible(false);
	end

	if self.glow then
		self.glow:setVisible(false);
	end

	if self.spark then
		self.spark:setVisible(false);
	end

	if self.muzzleFlash then
		for _,v in pairs(self.muzzleFlash) do
			v:setVisible(false);
		end
	end

end

----------------------------------------------------------------------

function DroneLaser:show()

	Base.show(self);

	if self.beamMinor then
		for _,v in pairs(self.beamMinor) do
			v:setVisible(false);
		end
	end

	if self.beamMajor then
		self.beamMajor:setVisible(false);
	end

	if self.glow then
		self.glow:setVisible(false);
	end

	if self.spark then
		self.spark:setVisible(false);
	end

	if self.muzzleFlash then
		for _,v in pairs(self.muzzleFlash) do
			v:setVisible(false);
		end
	end

end


----------------------------------------------------------------------
----------------------------------------------------------------------