require('game/entity/drone/drone');

local MOUNT_POINT_LEFT			= "left";
local MOUNT_POINT_RIGHT			= "right";

local DRONE_DISABLED				= "disabled";
local DRONE_RESPAWN_COOLDOWN	= "respawn-cooldown";
local DRONE_RESPAWN				= "respawn";
local DRONE_READY					= "ready";

----------------------------------------------------------------------

local Base	= Drone;
DroneMagnet	= Base:new();

----------------------------------------------------------------------


function DroneMagnet:new( game, layers, template,  x, y, angle, parent, offset, mountPoint, garage_index, visData, stats )

	-- modify base template to fit player vehicle
	if template and garage_index and visData then
		local rarity;
		local level;

		if mountPoint == MOUNT_POINT_LEFT then
			rarity	= math.min( DroneMgr:getDroneRarityLeft(garage_index), DroneMgr:getDroneMaxRarity() );
			level		= math.min( DroneMgr:getDroneLevelLeft(garage_index), DroneMgr:getDroneMaxLevel());
		else
			rarity	= math.min( DroneMgr:getDroneRarityRight(garage_index), DroneMgr:getDroneMaxRarity() );
			level		= math.min( DroneMgr:getDroneLevelRight(garage_index), DroneMgr:getDroneMaxLevel());
		end

		template.tplPowerup.radius		= stats.rarity[rarity] + stats.level[level];
		template.tplPowerup.width		= template.tplPowerup.radius * 2;
		template.tplPowerup.height		= template.tplPowerup.width;
		template.tplPowerup.color		= visData.color[rarity];
		template.tplPowerup.pulseTime	= visData.pulseTime[rarity];
	end

	local e = Base.new( self, game, layers, template, x, y, angle, parent, offset, mountPoint, garage_index );

   if template and mountPoint then

		e.powerup	= game.entityMgr:spawnPowerup( template.tplPowerup.factory, template.tplPowerup, x, y, 0, self );

      --call here because avatar doesnt get called yet
      e:reset( x, y, angle, parent, offset  );
   end

   return e;

end

----------------------------------------------------------------------

function DroneMagnet:reset( x, y, angle, parent, offset )

   Base.reset(self, x, y, angle, parent, offset);

   if self.powerup then

      self.powerup:reset(  x, y, angle, self );

      if self.game then
         self.game:disablePickup( PICKUP_MAGNET_INDEX );
      end
   end

end

----------------------------------------------------------------------

function DroneMagnet:update( dt, avatarSpeedY, burstOn, canShoot )

	Base.update(self, dt, avatarSpeedY);

	if self.powerup then
      self.powerup:update( dt );
   end

end

----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneMagnet:getMagnetRadius()

   return self.powerup.template.radius;

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneMagnet:delete()

	if self.powerup then
		self.powerup:delete();
	end

	Base.delete(self);

end

----------------------------------------------------------------------

function DroneMagnet:hide()

	Base.hide(self);

	if self.powerup then
		self.powerup:hide();
	end

end

----------------------------------------------------------------------

function DroneMagnet:show()

	Base.show(self);

	if self.powerup then
		self.powerup:show();
	end

end


----------------------------------------------------------------------
----------------------------------------------------------------------