require('game/entity/drone/drone');

local MOUNT_POINT_LEFT			= "left";
local MOUNT_POINT_RIGHT			= "right";

local DRONE_DISABLED				= "disabled";
local DRONE_RESPAWN_COOLDOWN	= "respawn-cooldown";
local DRONE_RESPAWN				= "respawn";
local DRONE_READY					= "ready";

----------------------------------------------------------------------

local Base		= Drone;
DroneMissile	= Base:new();

----------------------------------------------------------------------


function DroneMissile:new( game, layers, template,  x, y, angle, parent, offset, mountPoint, garage_index, visData, stats )

	-- modify base template to fit player vehicle
	if template and garage_index and visData then
		local rarity;
		local level;

		if mountPoint == MOUNT_POINT_LEFT then
			rarity	= math.min( DroneMgr:getDroneRarityLeft(garage_index), DroneMgr:getDroneMaxRarity() );
			level		= math.min( DroneMgr:getDroneLevelLeft(garage_index), DroneMgr:getDroneMaxLevel() );
		else
			rarity	= math.min( DroneMgr:getDroneRarityRight(garage_index), DroneMgr:getDroneMaxRarity() );
			level		= math.min( DroneMgr:getDroneLevelRight(garage_index), DroneMgr:getDroneMaxLevel() );
		end

		template.vfx.texture										= visData.glow[rarity];
		template.proxy.texture									= template.texture;
		template.proxy.offset									= visData.pos[rarity];
		template.bulletCount										= #template.proxy.offset;
		template.projectile.texture							= template.texture;
		template.hitpoints										= parent.hitPoints;

		if mountPoint == MOUNT_POINT_RIGHT then
			for i=1,#template.proxy.offset do
				template.proxy.offset[i][1] = -1 * template.proxy.offset[i][1];
			end
		end

		local tmp	= {};

		for k,v in ipairs(template.proxy.offset) do
			local tbl	= {}

			tbl.x	= v[1];
			tbl.y	= v[2];

			table.insert(tmp, tbl);
		end

		template.bulletDelta = tmp;

		local count = ( template.bulletCount or 1 )/ ( (template.bulletCount or 1) * (template.bulletDelay or 0)  + (template.respawnCDRange[1] or 0) );

		template.projectile.damage	= template.projectile.damage * ( App.data:getCurrentPlayerMainWeaponMaxDPS() / ( stats.rarity[rarity] - stats.level[level] ) ) / count;
		template.projectile.explosion.damage.vehicleVal	= template.projectile.damage;
	end

	local e = Base.new( self, game, layers, template, x, y, angle, parent, offset, mountPoint, garage_index );

   if template and mountPoint then
		e.proxyL	= {};
		e.proxyR	= {};
		e.proxyS	= {};
		e.vfx		= {};

		local dx = 0;

		for i=1,#template.proxy.offset do
			e.proxyL[i]	= e:_createVisProp( layers[template.proxy.layer or template.proxy.layer or 5], template.proxy.priority, template.proxy.offset[i][1]-template.proxy.width/4, template.proxy.offset[i][2], 0, template.proxy.width/2, template.proxy.height, template.proxy.texture );
			e.proxyL[i]:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e.proxyR[i]	= e:_createVisProp( layers[template.proxy.layer or template.proxy.layer or 5], template.proxy.priority, template.proxy.offset[i][1]+template.proxy.width/4, template.proxy.offset[i][2], 0, template.proxy.width/2, template.proxy.height, template.proxy.texture );
			e.proxyR[i]:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e.proxyR[i]:setScl(-1,1);
			e.vfx[i]		= e:_createVisProp( layers[template.vfx.layer or 5], template.vfx.priority, template.vfx.offset[1]+template.proxy.width/4, template.vfx.offset[2], 0, template.vfx.width, template.vfx.height, template.vfx.texture, nil, template.vfx.color, template.vfx.blend );
			e.vfx[i]:setAttrLink( MOAIProp2D.INHERIT_LOC, e.proxyL[i], MOAIProp2D.TRANSFORM_TRAIT );
			e.proxyS[i]	= e:_createVisProp( layers[template.proxyShadow.layer or 5], template.proxyShadow.priority, template.proxyShadow.offset[1]+template.proxy.width/4, template.proxyShadow.offset[2], 0, template.proxyShadow.width, template.proxyShadow.height, template.proxyShadow.texture, nil, template.proxyShadow.color, template.proxyShadow.blend );
			e.proxyS[i]:setAttrLink( MOAIProp2D.INHERIT_LOC, e.proxyL[i], MOAIProp2D.TRANSFORM_TRAIT );

			if dx < math.abs(template.proxy.offset[i][1]) then
				dx = math.abs(template.proxy.offset[i][1]);
			end
		end

		if mountPoint == MOUNT_POINT_LEFT then
			e._screenWidthBound[1] = e._screenWidthBound[1] + dx;
		else
			e._screenWidthBound[2] = e._screenWidthBound[2] - dx;
		end

      --call here because avatar doesnt get called yet
      e:reset( x, y, angle, parent, offset  );
   end

   return e;

end

----------------------------------------------------------------------

function DroneMissile:reset( x, y, angle, parent, offset )

   Base.reset(self, x, y, angle, parent, offset);

	self._target			= nil;
	self._targetTimer		= random.mapClamped(0, 3/30);
	self._shootTimer		= self.template.shootDelay or 0;
	self._bulletTimer		= self.template.bulletDelay;
	self._bulletCounter	= self.template.bulletCount or 1;
	self._bulletOffset	= self.template.bulletDelta or { { x=0, y=0} };
	self._targetCount		= #self._bulletOffset;
	self._offsetIndex		= 1;
	self._flicker			= true;

	if self.template and self.template.shootDelay and self.template.bulletDelay then
		self._vfxLength	= self.template.shootDelay + self.template.bulletDelay;
	else
		self._vfxLength	= 1;
	end

end

----------------------------------------------------------------------

function DroneMissile:update( dt, avatarSpeedY, burstOn, canShoot )

	Base.update(self, dt, avatarSpeedY);

	self:_updateTarget(dt, canShoot);
	self:_updateShoot(dt, burstOn, canShoot)
	self:_updateVFX();

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneMissile:_updateShoot( dt, burstOn, canShoot )

	if self.state == DRONE_READY and canShoot and self:_isTargetValid(self._target) then

		self._shootTimer = math.max(self._shootTimer - dt, 0);

		if self._shootTimer <= 0 then
			self._bulletTimer = math.max(self._bulletTimer - dt, 0);

			if self._bulletTimer <= 0 then
				self:_shoot(burstOn);
			end
		end
	elseif self.state == DRONE_READY and ( not canShoot or not self:_isTargetValid(self._target) ) then
		self._shootTimer	= math.min(self._shootTimer + 2*dt, self.template.shootDelay);
		self._bulletTimer	= math.min(self._bulletTimer + 2*dt, self.template.bulletDelay);
	end

end

----------------------------------------------------------------------

function DroneMissile:_updateTarget( dt, canShoot )

	if		self.state == DRONE_DISABLED
		or	self.state == DRONE_RESPAWN_COOLDOWN
		or self.state == DRONE_RESPAWN
		or not canShoot
		or	#self.game.entityMgr.vehicles == 0 then

		self._targetTimer		= random.mapClamped(0, 2/5);
		self._target			= nil;

		if		self.state == DRONE_DISABLED
			or	self.state == DRONE_RESPAWN_COOLDOWN
			or self.state == DRONE_RESPAWN then

			self._shootTimer		= self.template.shootDelay;
			self._bulletTimer		= self.template.bulletDelay;
			self._bulletCounter	= self.template.bulletCount or 1;
			self._offsetIndex		= 1;
		end
      
      EventMgr:postEvent( {sounds={ loop={"HunterFlameLoop",},}, name="hunter flame loop",}, "loop-stop" );
      self.flameLoopSFX = false;

	else

		self._targetTimer = math.max(self._targetTimer - dt, 0);

		if self._targetTimer <= 0 then

			local vehicles    = self.game.entityMgr.vehicles;
			local x,y			= self:getLoc(true);
			local best        = 1;
			local bestScore   = 9000000000;

			if self._target == nil then
				self._target	= {};
			end

			if self._target and #self._target > 0 then
				for i=#self._target,1,-1 do
					if not self:_isTargetValid(self._target[i]) then
						table.remove(self._target, i);
					end
				end
			end

			for i=1,#vehicles do

				local tgt   = vehicles[i];

				if self:_isTargetValid( tgt ) then

					local tx,ty	= tgt:getLoc(true);
					local score = 10*tgt:getHealthMax()

					score = score + math.distance(x, y, tx, ty);

					if self:_isPreviousTarget(tgt) then
						score = score * 10;
					end

					if score < bestScore then
						bestScore = score;
						best      = i;
					end
				end
			end

			table.insert(self._target, vehicles[best]);

			self._targetTimer	= random.mapClamped(0, 3/30);
		end
	end

end

----------------------------------------------------------------------

function DroneMissile:_updateVFX()

	if self.state == DRONE_READY and not self.game.avatar:isDead() then

		local time	= math.max(self._shootTimer + self._bulletTimer, 0);

		if time >= self._vfxLength then
			for i=1,#self.vfx do
				self.vfx[i]:setVisible(false);
			end
		else

			local k	= math.max( (1 - time/self._vfxLength) , 0 );
			local f	= 1;

			if self._flicker then
				f					= -1;
				self._flicker	= false;
			else
				f					= 1;
				self._flicker	= true;
			end

         if not self.flameLoopSFX then
            EventMgr:postEvent( {sounds = {loop={"HunterFlameLoop",},}, name="hunter flame loop"}, "add-sfx" );
            self.flameLoopSFX = true;
         end
         
			for i=1,#self.vfx do
				if i >= self._offsetIndex then
					self.vfx[i]:setVisible(true);
					self.vfx[i]:setScl( f * self.template.vfx.scale[1]*k, self.template.vfx.scale[2]*k);
					self.vfx[i]:setColor(1, 1, 1, k);
				else
               EventMgr:postEvent( {sounds={ loop={"HunterFlameLoop",},}, name="hunter flame loop",}, "loop-stop" );
               self.flameLoopSFX = false;
					self.vfx[i]:setVisible(false);
				end
			end
		end
	else
		for i=1,#self.vfx do
         EventMgr:postEvent( {sounds={ loop={"HunterFlameLoop",},}, name="hunter flame loop",}, "loop-stop" );
         self.flameLoopSFX = false;
			self.vfx[i]:setVisible(false);
		end
	end

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneMissile:_isTargetValid( target )

	local dx,dy	= self:getLoc(true);

	dy = dy+256;

	if			target
		and	target == self._target then

		local valid	= true;

		for i=1,#target do

			if not target[i] then

				valid	= false;
			elseif		target[i].deleteMe
						or	target[i]:isDead() then

				valid	= false;
			else
				local tx,ty	= target[i]:getLoc(true);

				if dy > ty then
					valid = false;
				end
			end
		end

		return valid;
   elseif	target
		and	not target.deleteMe
      and	not target:isDead() then

		local tx,ty	= target:getLoc(true);

		if ty > dy then
			return true;
		end
   end

   return false;

end

----------------------------------------------------------------------

function DroneMissile:_isPreviousTarget( target )

	if target and self._target then
		for k,v in pairs(self._target) do
			if v == target then
				return true;
			end
		end
	end

end

----------------------------------------------------------------------

function DroneMissile:_getBulletOffset()

   local x,y	= self:getLoc(true);

	local delta = self._bulletOffset[self._offsetIndex];
	local dX,dY	= (delta.x or 0), (delta.y or 0);
	local index	= self._offsetIndex;

	if			self.proxyL and self.proxyL[self._offsetIndex]
		and	self.proxyR and self.proxyR[self._offsetIndex]
		and	self.proxyS and self.proxyS[self._offsetIndex] then

		self.proxyL[self._offsetIndex]:setVisible(false);
		self.proxyR[self._offsetIndex]:setVisible(false);
		self.proxyS[self._offsetIndex]:setVisible(false);
	end

	self._offsetIndex = self._offsetIndex + 1;

	if self._offsetIndex > #self._bulletOffset then
		self._offsetIndex		= 1;
	end

   return x+dX, y+dY, index;

end

----------------------------------------------------------------------

function DroneMissile:_shoot( burstOn )

	if self._target then

		local reset	= false;

		self._bulletCounter		= self._bulletCounter - 1;
		self._bulletTimer			= self.template.bulletDelay;

		if self._bulletCounter < 1 then
			self._shootTimer		= self.template.shootDelay;
			self._bulletCounter	= self.template.bulletCount or 1;
			reset						= true;
		end

		local x,y,i = self:_getBulletOffset();
		local p;

		p = self.game.entityMgr:spawnProjectile( self.template.projectile, x, y, 0, self.parent, self._target[i] );
      EventMgr:postEvent( {sounds = {start={"HunterFlameStart",},}, name="hunter flame start"}, "add-sfx" );
      

		p.speedBurst = burstOn;

		if reset then
			self:onRespawnBegin();
			self:onRespawnEnd();
		end
	end

end

----------------------------------------------------------------------

function DroneMissile:getTarget()

	if self._target and #self._target > 0 then
		return self._target[1];
	end

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneMissile:delete()

	if self.proxyL then
		for i=1,#self.proxyL do
			self.proxyL[i]:delete();
		end
	end

	if self.proxyR then
		for i=1,#self.proxyR do
			self.proxyR[i]:delete();
		end
	end

	if self.proxyS then
		for i=1,#self.proxyS do
			self.proxyS[i]:delete();
		end
	end

	if self.vfx then
		for i=1,#self.vfx do
			self.vfx[i]:delete();
		end
	end

	Base.delete(self);

end

----------------------------------------------------------------------

function DroneMissile:hide()

	Base.hide(self);

	if self.proxyL then
		for i=1,#self.proxyL do
			self.proxyL[i]:setVisible(false);
		end
	end

	if self.proxyR then
		for i=1,#self.proxyR do
			self.proxyR[i]:setVisible(false);
		end
	end

	if self.proxyS then
		for i=1,#self.proxyS do
			self.proxyS[i]:setVisible(false);
		end
	end


	if self.vfx then
		for i=1,#self.vfx do
			self.vfx[i]:setVisible(false);
		end
	end

end

----------------------------------------------------------------------

function DroneMissile:show()

	Base.show(self);

	if self.prop then
		self.prop:setVisible(false);
	end

	if self.proxyL then
		for i=1,#self.proxyL do
			self.proxyL[i]:setVisible(true);
		end
	end

	if self.proxyR then
		for i=1,#self.proxyR do
			self.proxyR[i]:setVisible(true);
		end
	end

	if self.proxyS then
		for i=1,#self.proxyS do
			self.proxyS[i]:setVisible(true);
		end
	end


	if self.vfx then
		for i=1,#self.vfx do
			self.vfx[i]:setVisible(false);
		end
	end

end


----------------------------------------------------------------------
----------------------------------------------------------------------