require('game/entity/drone/drone');

local MOUNT_POINT_LEFT			= "left";
local MOUNT_POINT_RIGHT			= "right";

local DRONE_DISABLED				= "disabled";
local DRONE_RESPAWN_COOLDOWN	= "respawn-cooldown";
local DRONE_RESPAWN				= "respawn";
local DRONE_READY					= "ready";
local DRONE_CHARGED				= "drone-charged";

local VFX_SCALE_UP_TIME			= 10/30;
local PULSE_TIME					= 6/30;

----------------------------------------------------------------------

local Base		= Drone;
DroneScoreMult	= Base:new();

----------------------------------------------------------------------


function DroneScoreMult:new( game, layers, template,  x, y, angle, parent, offset, mountPoint, garage_index, visData, stats )

	-- modify base template to fit player vehicle
	if template and garage_index and visData then
		local rarity;
		local level;

		if mountPoint == MOUNT_POINT_LEFT then
			rarity	= math.min( DroneMgr:getDroneRarityLeft(garage_index), DroneMgr:getDroneMaxRarity());
			level		= math.min( DroneMgr:getDroneLevelLeft(garage_index), DroneMgr:getDroneMaxLevel());
		else
			rarity	= math.min( DroneMgr:getDroneRarityRight(garage_index), DroneMgr:getDroneMaxRarity());
			level		= math.min( DroneMgr:getDroneLevelRight(garage_index), DroneMgr:getDroneMaxLevel());
		end

		template.scoreK				= stats.rarity[rarity];
		template.duration				= stats.level.duration[level];
		template.chargeThreshold	= stats.level.chargeThreshold[level];
		template.readyvfx.color		= visData.color[rarity];
	end

	local e = Base.new( self, game, layers, template, x, y, angle, parent, offset, mountPoint, garage_index );

   if template and mountPoint then
		e._chevColorOff	= { r=visData.chevronColor.off[1], g=visData.chevronColor.off[2], b=visData.chevronColor.off[3], a=visData.chevronColor.off[4] };
		e._chevColorOn		= { r=visData.chevronColor.on[1], g=visData.chevronColor.on[2], b=visData.chevronColor.on[3], a=visData.chevronColor.on[4] };
		e._chevronL			= {};
		e._chevronR			= {};

		for i=1,#template.vfx.offset do
			e._chevronL[i]	= e:_createVisProp( layers[template.vfx.layer or template.layer or 5], template.vfx.priority, template.vfx.offset[i][1], template.vfx.offset[i][2], 0, template.vfx.width, template.vfx.height, template.vfx.texture[i], template.vfx.scale, visData.chevronColor.off, template.vfx.blend );
			e._chevronR[i]	= e:_createVisProp( layers[template.vfx.layer or template.layer or 5], template.vfx.priority, -1*template.vfx.offset[i][1], template.vfx.offset[i][2], 0, template.vfx.width, template.vfx.height, template.vfx.texture[i], template.vfx.scale, visData.chevronColor.off, template.vfx.blend );
			e._chevronL[i]:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e._chevronR[i]:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e._chevronR[i]:setScl(-1, 1);
		end

		e._readyVFX	= e:_createVisProp( layers[template.readyvfx.layer or template.layer or 5], template.readyvfx.priority, template.readyvfx.offset[1], template.readyvfx.offset[2], 0, template.readyvfx.width, template.readyvfx.height, template.readyvfx.texture, nil, template.readyvfx.color, template.readyvfx.blend );
		e._readyVFX:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );

		e._pipOn				= {};
		e._pipOff			= {};

		for i=1,#template.pip_on.offset do
			e._pipOn[i]		= e:_createVisProp( layers[template.pip_on.layer or template.layer or 5], template.pip_on.priority, template.pip_on.offset[i][1], template.pip_on.offset[i][2], 0, template.pip_on.width, template.pip_on.height, template.pip_on.texture );
			e._pipOff[i]	= e:_createVisProp( layers[template.pip_off.layer or template.layer or 5], template.pip_off.priority, template.pip_off.offset[i][1], template.pip_off.offset[i][2], 0, template.pip_off.width, template.pip_off.height, template.pip_off.texture );
			e._pipOn[i]:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e._pipOff[i]:setAttrLink( MOAIProp2D.INHERIT_LOC, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
		end

      --call here because avatar doesnt get called yet
      e:reset( x, y, angle, parent, offset  );
   end

   return e;

end

----------------------------------------------------------------------

function DroneScoreMult:reset( x, y, angle, parent, offset )

   Base.reset(self, x, y, angle, parent, offset);

	self._chargeCounter	= 0;
	self._scaleTimer		= 0;
	self._vfxPulseTimer	= 0;
	self._exitTimer		= 0;
	self._exitAnim			= false;
	self._durationTimer	= self.template.duration or 1;

end

----------------------------------------------------------------------

function DroneScoreMult:update( dt, avatarSpeedY, burstOn, canShoot )

	Base.update(self, dt, avatarSpeedY);

	self:_chargeUpdate(dt);
	self:_vfxUpdate(dt);

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneScoreMult:_chargeUpdate( dt )

	if			self.state == DRONE_CHARGED
		and	self._scaleTimer >= VFX_SCALE_UP_TIME
		and	not self.game.avatar:isDead() then

		self._durationTimer = math.max( self._durationTimer - dt, 0 );

		self:_updateChevrons(self._durationTimer/self.template.duration);

		if self._durationTimer <= 0 then
			self:resetCharge();

			self.state			= DRONE_READY;
			self._exitAnim		= true;
		end
	end

end

----------------------------------------------------------------------

function DroneScoreMult:_vfxUpdate( dt )

	if self.state == DRONE_READY and self._exitAnim and not self.parent:isDead() then
		self._exitTimer = self._exitTimer + dt;

		local k, alpha;

		if self._exitTimer <= VFX_SCALE_UP_TIME then
			k		= math.mapClamped(0, 1, VFX_SCALE_UP_TIME, 0, self._exitTimer);
			alpha	= self.template.readyvfx.color[4];
		else
			k		= math.mapClamped(VFX_SCALE_UP_TIME, 0, VFX_SCALE_UP_TIME*2, 10, self._exitTimer);
			alpha	= math.mapClamped(VFX_SCALE_UP_TIME, self.template.readyvfx.color[4], VFX_SCALE_UP_TIME*2, 0, self._exitTimer);
		end

		if alpha <= 0 then
			self._readyVFX:setColor(self.template.readyvfx.color[1], self.template.readyvfx.color[2], self.template.readyvfx.color[3], self.template.readyvfx.color[4]);
			self._readyVFX:setVisible(false);

			self._exitAnim = false;
		else
			local a = self._readyVFX:getRot();

			self._readyVFX:setVisible(true);
			self._readyVFX:setRot( math.clampAngle360( a + ( self.template.readyvfx.spin * dt ) ) );
			self._readyVFX:setColor(self.template.readyvfx.color[1], self.template.readyvfx.color[2], self.template.readyvfx.color[3], alpha);
			self._readyVFX:setScl( k, k );
		end
	elseif self.state == DRONE_CHARGED and self._scaleTimer < VFX_SCALE_UP_TIME and not self.parent:isDead() then

		self._scaleTimer = math.min(self._scaleTimer + dt, VFX_SCALE_UP_TIME);

		local k = self._scaleTimer/VFX_SCALE_UP_TIME;

		if k <= 0 then
			self._readyVFX:setVisible(false);
		else
			local a = self._readyVFX:getRot();

			self._readyVFX:setVisible(true);
			self._readyVFX:setRot( math.clampAngle360( a + ( self.template.readyvfx.spin * dt ) ) );
			self._readyVFX:setScl( k, k );
			self._readyVFX:setColor(self.template.readyvfx.color[1], self.template.readyvfx.color[2], self.template.readyvfx.color[3], self.template.readyvfx.color[4]);
		end
	elseif self.state == DRONE_CHARGED and self._scaleTimer >= VFX_SCALE_UP_TIME and not self.parent:isDead() then

		self._vfxPulseTimer	= self._vfxPulseTimer + dt;

		if self._vfxPulseTimer > PULSE_TIME then
			self._vfxPulseTimer = 0;
		end

		local a = self._readyVFX:getRot();
		local k;

		if self._vfxPulseTimer <= PULSE_TIME/2 then
			k = math.mapClamped(0, 1, PULSE_TIME/2, 0.7, self._vfxPulseTimer);
		else
			k = math.mapClamped(PULSE_TIME/2, 0.7, PULSE_TIME, 1.0, self._vfxPulseTimer);
		end

		self._readyVFX:setVisible(true);
		self._readyVFX:setRot( math.clampAngle360( a + ( self.template.readyvfx.spin * dt ) ) );
		self._readyVFX:setScl( k, k );
	elseif self.parent:isDead() then

		self:resetCharge();
		self._readyVFX:setVisible(false);

		for i=1,#self._chevronL do
			self._chevronL[i]:setVisible(false);
			self._chevronR[i]:setVisible(false);
		end

		for i=1,#self._pipOn do
			self._pipOn[i]:setVisible(false);
			self._pipOff[i]:setVisible(false);
		end
	elseif self.state == DRONE_RESPAWN or self.state == DRONE_RESPAWN_COOLDOWN then

		self:resetCharge();
		self._readyVFX:setVisible(false);

		for i=1,#self._chevronL do
			self._chevronL[i]:setVisible(true);
			self._chevronR[i]:setVisible(true);
		end

		for i=1,#self._pipOn do
			self._pipOn[i]:setVisible(false);
			self._pipOff[i]:setVisible(true);
		end
	end

end

----------------------------------------------------------------------

function DroneScoreMult:_updateChevrons( percentage )

	local tbl	= { self._chevColorOff, self._chevColorOff, self._chevColorOff };
	local c		= percentage or math.min( self._chargeCounter/self.template.chargeThreshold, 1 );

	if percentage then
		if c < 1/3 then
			tbl	= { self._chevColorOn, self._chevColorOff, self._chevColorOff };
		elseif c < 2/3 then
			tbl	= { self._chevColorOn, self._chevColorOn, self._chevColorOff };
		else
			tbl	= { self._chevColorOn, self._chevColorOn, self._chevColorOn };
		end
	else
		if c < 1/3 then
			tbl	= { self._chevColorOff, self._chevColorOff, self._chevColorOff };
		elseif c < 2/3 then
			tbl	= { self._chevColorOn, self._chevColorOff, self._chevColorOff };
		elseif c < 1 then
			tbl	= { self._chevColorOn, self._chevColorOn, self._chevColorOff };
		else
			tbl	= { self._chevColorOn, self._chevColorOn, self._chevColorOn };
		end
	end

	for i=1,#tbl do
		self._chevronL[i]:setVisible(true);
		self._chevronR[i]:setVisible(true);
		self._chevronL[i]:setColor( tbl[i].r, tbl[i].g, tbl[i].b, 1 );
		self._chevronR[i]:setColor( tbl[i].r, tbl[i].g, tbl[i].b, 1 );
	end

	local pip	= {	false, false, false	};

	if percentage then
		if c <= 0 then
			pip	= {	false, false, false	};
		elseif c < 1/3 then
			pip	= { true, false, false};
		elseif c < 2/3 then
			pip	= { true, true, false};
		else
			pip	= { true, true, true};
		end
	else
		if c < 1/3 then
			pip	= { false, false, false};
		elseif c < 2/3 then
			pip	= { true, false, false};
		elseif c < 1 then
			pip	= { true, true, false};
		else
			pip	= { true, true, true};
		end
	end

	for i=1,#self._pipOn do
		if pip[i] then
			self._pipOn[i]:setVisible(true);
			self._pipOff[i]:setVisible(false);
		else
			self._pipOn[i]:setVisible(false);
			self._pipOff[i]:setVisible(true);
		end
	end

end

----------------------------------------------------------------------

function DroneScoreMult:getScoreK()

	if self.state == DRONE_CHARGED then
		return self.template.scoreK;
	end

	return 0;

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneScoreMult:onPickup( event )

	if self.state == DRONE_READY and event and event.pickup and event.pickup.pickupType == "currency" then

		self._chargeCounter = self._chargeCounter + math.min( event.pickup.value or 1, 2);

		self:_updateChevrons();

		if self:isScoreKCharged() then
			self.state				= DRONE_CHARGED;
			self._scaleTimer		= 0;
			self._vfxPulseTimer	= 0;
			self._exitTimer		= 0;
			self._exitAnim			= false;
			self._durationTimer	= self.template.duration or 1;

			EventMgr:postEvent(	{	text="Bonus Drone Activated"	}, "hud-alert-text"	);
		end
	end

end

----------------------------------------------------------------------

function DroneScoreMult:resetCharge()

	local tbl	= { self._chevColorOff, self._chevColorOff, self._chevColorOff };

	self._chargeCounter	= 0;
	self._scaleTimer		= 0;
	self._vfxPulseTimer	= 0;
	self._exitTimer		= 0;
	self._exitAnim			= false;
	self._durationTimer	= self.template.duration or 1;

	for i=1,#tbl do
		self._chevronL[i]:setColor( tbl[i].r, tbl[i].g, tbl[i].b, 1 );
		self._chevronR[i]:setColor( tbl[i].r, tbl[i].g, tbl[i].b, 1 );
	end

	for i=1,#self._pipOn do
		self._pipOn[i]:setVisible(false);
		self._pipOff[i]:setVisible(true);
	end

end

----------------------------------------------------------------------

function DroneScoreMult:isScoreKCharged()
	return self._chargeCounter >= self.template.chargeThreshold;
end

----------------------------------------------------------------------

function DroneScoreMult:isChargeRequired()
	return true;
end

----------------------------------------------------------------------

function DroneScoreMult:setCharge( k )

	self._chargeCounter = math.clamp( k*self.template.chargeThreshold, 0, self.template.chargeThreshold );

   self:_updateChevrons();

	if self:isScoreKCharged() then
		self.state				= DRONE_CHARGED;
		self._scaleTimer		= 0;
		self._vfxPulseTimer	= 0;
		self._exitTimer		= 0;
		self._exitAnim			= false;
		self._durationTimer	= self.template.duration or 1;

		return true;
	end

end


----------------------------------------------------------------------
----------------------------------------------------------------------


function DroneScoreMult:delete()

	if self._chevronL then
		for k,v in pairs(self._chevronL) do
			v:delete();
		end
	end

	if self._chevronR then
		for k,v in pairs(self._chevronR) do
			v:delete();
		end
	end

	if self._pipOn then
		for k,v in pairs(self._pipOn) do
			v:delete();
		end
	end

	if self._pipOff then
		for k,v in pairs(self._pipOff) do
			v:delete();
		end
	end

	if self._readyVFX then
		self._readyVFX:delete();
	end

	Base.delete(self);

end

----------------------------------------------------------------------

function DroneScoreMult:hide()

	Base.hide(self);

	if self._chevronL then
		for k,v in pairs(self._chevronL) do
			v:setVisible(false);
		end
	end

	if self._chevronR then
		for k,v in pairs(self._chevronR) do
			v:setVisible(false);
		end
	end

	if self._pipOn then
		for k,v in pairs(self._pipOn) do
			v:setVisible(false);
		end
	end

	if self._pipOff then
		for k,v in pairs(self._pipOff) do
			v:setVisible(false);
		end
	end

	if self._readyVFX then
		self._readyVFX:setVisible(false);
	end

end

----------------------------------------------------------------------

function DroneScoreMult:show()

	Base.show(self);

	if self._chevronL then
		for k,v in pairs(self._chevronL) do
			v:setVisible(true);
		end
	end

	if self._chevronR then
		for k,v in pairs(self._chevronR) do
			v:setVisible(true);
		end
	end

	if self._pipOn then
		for k,v in pairs(self._pipOn) do
			v:setVisible(false);
		end
	end

	if self._pipOff then
		for k,v in pairs(self._pipOff) do
			v:setVisible(true);
		end
	end

	if self._readyVFX then
		self._readyVFX:setVisible(false);
	end

end


----------------------------------------------------------------------
----------------------------------------------------------------------