require('engine/anim/anim');
require('engine/utility/math');
require('engine/font/font-manager')
require('engine/object/object')
require('engine/texture/texture-manager')
require('flash/flash-manager')
require('game/collision');

----------------------------------------------------------------------

local ENTITY_DEBUG_COLLISION_ENABLED	= false;
local ROOT_SPLIT_PROP						= 'resources/square.png';
local ROOT_SPLIT_PROP_WIDTH				= 1;
local ROOT_SPLIT_PROP_HEIGHT				= 1;

--Use a manager to handle that

----------------------------------------------------------------------

local Base           = Object;

Entity               = Base:new();

----------------------------------------------------------------------

function Entity:new( game, layer, template, x, y, angle, ... )

   local e           = Base.new( self );

   e.__marker        = "entity";

   e.game            = game;
   e.template        = template;

   e.austin          = MOAIRoadRiotEntity.new();

   if e.template then

      e.damageValue     = 0;
      e.hitPoints       = template.hitPoints       or 1;
      e.score           = template.score           or 0;
      e.damage          = template.damage          or 0;
      e.mass            = template.mass            or 1;
      e.layer           = layer;
      e.texture         = template.texture;
		e.splitProp			= template.splitProp			or false;
      e.ausToProp       = true;

      e.collisionImpulse = { 0, 0 };

      local gfxQuad = nil;

      if e.texture then

         local gfxQuad  = MOAIGfxQuad2D.new ();

			if not e.splitProp then
				gfxQuad:setTexture ( game:getTexture(e.texture))
				gfxQuad:setRect ( -(template.width or 0)/2, -(template.height or 0)/2, (template.width or 0)/2, (template.height or 0)/2 )
			else
				gfxQuad:setTexture ( game:getTexture(ROOT_SPLIT_PROP))
				gfxQuad:setRect ( -ROOT_SPLIT_PROP_WIDTH/2, -ROOT_SPLIT_PROP_HEIGHT/2, ROOT_SPLIT_PROP_WIDTH/2, ROOT_SPLIT_PROP_HEIGHT/2 )
			end

         e.prop  = MOAIProp2D.new ()
         e.prop:setDeck ( gfxQuad )

      elseif e.template.text then

         e.textBox = MOAITextBox.new ();
         e.prop    = e.textBox;

         local metaFont = FontMgr:getMetaFont( e.template.text.font, e.template.text.size, e.template.text.letters )
         e.textBox:setFont ( metaFont.font )
         e.textBox:setGlyphScale ( metaFont.scale )
         e.textBox:setAlignment ( MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY )
         e.textBox:setYFlip ( true )
         e.textBox:setRect ( -200, -200, 200, 200 )
         if e.template.text.string  then
            e.textBox:setString ( e.template.text.string )
         end

      else

         e.prop  = MOAIProp2D.new ()

      end

      e.prop:setPriority ( 0 )
      e.prop:setLoc(x,y)

      if e.template.blend and  e.template.blend =="add" then
         e.prop:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE );
		elseif e.template.blend and e.template.blend == "mult" then
			e.prop:setBlendMode( MOAIProp.GL_DST_COLOR, MOAIProp.GL_ONE_MINUS_SRC_ALPHA );
      else
         e.prop:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA );
      end

      if e.prop.animate then
         e.prop:insertProps(layer);
      else
         layer:insertProp ( e.prop );
      end

      if e.splitProp then
         local width    = template.width/2;
         local height   = template.height;
         local xOffset  = template.width/4;

         e.splitRight   = e:_createSplitProp(layer, xOffset, 0, width, height, -1, 1, e.texture);
         e.splitLeft    = e:_createSplitProp(layer, -xOffset, 0, width, height, 1, 1, e.texture);

         if BUILD_DEBUG then
            e.splitRight.__marker = "splitRightProp";
            e.splitLeft.__marker = "splitLeftProp";
         end

         e.splitRight:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
         e.splitLeft:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
      end

      if e.template.flashAnim then
         e.isFlashAnim  = function() return true end;
         e.flashAnim    = e:newFlashAnim(layer, template.flashAnim, e.prop);
      else
         e.isFlashAnim  = function() return false end;
      end

      e:reset( x, y, angle, ... );

   end

   if BUILD_DEBUG and e.prop then
      e.prop.__marker = "entityProp";
   end

   return e;
end

----------------------------------------------------------------------

function Entity:delete()

   EventMgr:rmvListener( self );

   if self.layer then

      if self:isFlashAnim() then
         self.flashAnim:delete();
      end

      if self.splitRight then
         self.layer:removeProp( self.splitRight );
         self.splitRight = nil;
      end

      if self.splitLeft then
         self.layer:removeProp( self.splitLeft );
         self.splitLeft = nil;
      end

      if self.prop then
         self.layer:removeProp( self.prop );
         self.prop = nil;
      end
   end

end

----------------------------------------------------------------------

--called when "cached" and not use anymore... it will not get delete
function Entity:deactivate()

   if self.prop and self.prop.animate then
      self.prop:stop();
   end

end


----------------------------------------------------------------------

function Entity:reset( x, y, angle )

   self.damageValue  = 0;
   self.deleteMe     = false;
   self.avatarSpeedK = 1;

   self.collisionImpulse = { 0, 0 };

   angle             = angle or 0;

   if self.template then
      angle = angle + math.random() * (self.template.angleRnd or 0);
   end

   self.ausToProp = true;

   if  self.prop then

      self.prop:setLoc( x or 0, y or 0 );
      self.prop:setRot( angle);

      if  self.prop.animate then
         self.prop:start( true );
         self.prop:setSpeed( self.template.animSpeed );
      end

   end

   local template = self.template;

   if template then

      if template.scale then
         self:setScale(template.scale[X],template.scale[Y]);
      else
         self:setScale(1,1);
      end

      self.avatarSpeedK = 1;

      if template.avatarSpeedK then

         if template.avatarSpeedKRnd then

            self.avatarSpeedK = template.avatarSpeedK + math.random()*template.avatarSpeedKRnd;

         else

            self.avatarSpeedK = template.avatarSpeedK;

         end
      end

      local accel    = { 0, 0 };
      if template.accel then
         accel  = { template.accel[1], template.accel[2] } ;
      end

      local speed = {0,0};

      if template.speed then

         if template.speedRnd then

            speed = {   template.speed[1] + math.random(0,template.speedRnd[1])
                     ,  template.speed[2] + math.random(0,template.speedRnd[2]) };


         else

            speed  = { template.speed[1], template.speed[2] } ;

         end


      end

      if template.speedMin then
         self.speedMin  = { template.speedMin[1],   template.speedMin[2] };
      else
         self.speedMin  = { -10000, -10000 };
      end

      if template.speedMinRnd then
         self.speedMin[X]  = self.speedMin[X] + math.random( 0,template.speedMinRnd[X] );
         self.speedMin[Y]  = self.speedMin[Y] + math.random( 0,template.speedMinRnd[Y] );
      end

      if template.speedMax then
         self.speedMax     = { template.speedMax[1],   template.speedMax[2] };
      else
         self.speedMax     = { 10000, 10000 };
      end

      if template.speedMaxRnd then
         self.speedMax[X]  = self.speedMax[X] + math.random( 0,template.speedMaxRnd[X] );
         self.speedMax[Y]  = self.speedMax[Y] + math.random( 0,template.speedMaxRnd[Y] );
      end



      if template.color then

         local r,g,b,a  = template.color[1]
                        , template.color[2]
                        , template.color[3]
                        , template.color[4];

         if template.colorRnd then

            r,g,b,a  = r + math.random()*template.colorRnd[1]
                     , g + math.random()*template.colorRnd[2]
                     , b + math.random()*template.colorRnd[3]
                     , a + math.random()*template.colorRnd[4];

         end

         self.color = {r,g,b,a};
         self:setColor(r,g,b,a);

      else

         self.color = {1,1,1,1};
         self:setColor(1,1,1,1);

      end

      if self.austin then

         self.austin:reset();

         self.austin:setLoc(x or 0, y or 0);
         self.austin:setFaceAngle(angle);

         self.austin:setSpeed( speed[1], speed[2]   );

         self.austin:setSpeedMin(      self.speedMin[1], self.speedMin[2]  );

         self.austin:setSpeedMax(      self.speedMax[1], self.speedMax[2]  );
         self.austin:setAvatarSpeedK(  0,                self.avatarSpeedK );
         self.austin:setAccel(         accel[1],         accel[2]          );
         self.austin:setImpulseStopK(  0.9,              0.99              );
         self.austin:setSpin(   (template.spin or 0)
                              + (math.random()*( template.spinRnd or 0)) );

         self.austin:setSize( template.width    or 0
                            , template.height  or 0);

         self.austin:setCollisionSize(template.collisionWidth  or (template.width   or 0)
                                    , template.collisionHeight or (template.height  or 0));

         self.austin:setMass( template.mass );

      end

      if self:isFlashAnim() then

         if self.flashAnim.template.scale then
            self.flashAnim:setScale(self.flashAnim.template.scale[1], self.flashAnim.template.scale[2]);
         else
            self.flashAnim:setScale(1, 1);
         end
      end

      self:show();

   end
end

----------------------------------------------------------------------

function Entity:onDeleteMe()
   self.deleteMe = true;
end
----------------------------------------------------------------------

function Entity:debugCollision( layer )

   if ENTITY_DEBUG_COLLISION_ENABLED and self.collisionProp == nil then

      self:createDebugCollision( layer );
   end

end

----------------------------------------------------------------------

function Entity:createDebugCollision( layer )

   local gfxQuad = MOAIGfxQuad2D.new ()
   gfxQuad:setTexture ( TextureMgr:getTexture("resources/square.png") );
   gfxQuad:setRect (   -self:getCollisionWidth()/2, -self:getCollisionHeight()/2,
                        self:getCollisionWidth()/2,  self:getCollisionHeight()/2 )


   self.collisionProp = MOAIProp2D.new ();
   self.collisionProp:setDeck ( gfxQuad );
   self.collisionProp:setColor(1,0,0,0.25);
   self.collisionProp:setPriority ( 0 )
   self.collisionProp:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA );

   if self.prop then
      self.collisionProp:setAttrLink ( MOAIProp2D.INHERIT_TRANSFORM, self.prop, MOAIProp2D.TRANSFORM_TRAIT )
   end

   layer = layer or  self.game.parent.layer.game[9];

   layer:insertProp ( self.collisionProp );


end

----------------------------------------------------------------------

function Entity:hide()

   if self.prop then
      self.prop:setVisible( false );
   end

	if self.splitRight then
		self.splitRight:setVisible(false);
	end

	if self.splitLeft then
		self.splitLeft:setVisible(false);
	end

   if self.collisionProp then
      self.collisionProp:setVisible( false );
   end

   if self:isFlashAnim() then
      self.flashAnim:hide();
   end

end

----------------------------------------------------------------------

function Entity:show()

   if self.prop then
      self.prop:setVisible( true );
   end

	if self.splitRight then
		self.splitRight:setVisible(true);
	end

	if self.splitLeft then
		self.splitLeft:setVisible(true);
	end

   if self.collisionProp then
      self.collisionProp:setVisible( true );
   end

   if self:isFlashAnim() then
      self.flashAnim:show();
   end

end

----------------------------------------------------------------------

function Entity:update( dt, avatarSpeedY )

   if self.austin then

      self.austin:setAvatarSpeed(0,avatarSpeedY);
      self.austin:update( dt );

      if self.prop then

         if self.ausToProp then
            self.prop:setLoc(self.austin:getLoc());
         else
            self.austin:setLoc(self.prop:getLoc());
         end

         self.prop:setRot(self.austin:getFaceAngle());
      end
   end

   --if self.collisionProp then
   --   self.collisionProp:setLoc(self.austin:getLoc());
   --end

end

----------------------------------------------------------------------

function Entity:newFlashAnim( layer, template, parent )

   Debug.assertType(template, "table", "Entity:newFlashAnim -> invalid template");

	local anim;

	if template and type(template) == "table" then


		template.color = color or {1,1,1,1};
		anim				= FlashAnim:new(	layer,
													template.flashId,
													template.animId,
													template.offset[1],
													template.offset[2],
													template.rotOffset,
													template.priority,
													template.fps,
													parent,
                                       template.attrLink);
		anim.template	= template;

		anim:setColor(template.color[1], template.color[2], template.color[3], template.color[4]);

      if template.blend =="add" then
         anim:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE );
		elseif template.blend == "mult" then
			anim:setBlendMode( MOAIProp.GL_DST_COLOR, MOAIProp.GL_ONE_MINUS_SRC_ALPHA );
      end

      if template.speed then
         anim:setSpeed(template.speed);
      end
	end

	return anim;

end

----------------------------------------------------------------------

function Entity:onDamage( d )

   if d and d ~= 0 then

      self.damageValue = math.clamp( self.damageValue + d, 0, self.hitPoints );

   end

   return self.damageValue >= self.hitPoints;

end


----------------------------------------------------------------------

function Entity:isDone()

   return false;

end

----------------------------------------------------------------------

function Entity:isDead()

   return self.damageValue >= self.hitPoints;

end

----------------------------------------------------------------------

function Entity:getHealth()

	return math.max(self.hitPoints - self.damageValue, 0 );

end

----------------------------------------------------------------------

function Entity:getHealthMax()

   return self.hitPoints;

end

----------------------------------------------------------------------

function Entity:getDamageReceived()

	return self.damageValue or 0;

end

----------------------------------------------------------------------

function  Entity:setColor ( r, g, b, a )

   if self.prop then
      self.prop:setColor( r, g, b, a );
   end

end

----------------------------------------------------------------------

function  Entity:getAge ()

   if self.austin then
      return self.austin:getAge();
   end

   return 0;

end

----------------------------------------------------------------------

function  Entity:setAge ( age )

   if self.austin then
      self.austin:setAge(age);
   end

end

----------------------------------------------------------------------

function  Entity:getLoc ( world )

   if self.prop then

      if world then

         return self.prop:modelToWorld();

      end

      return self.prop:getLoc();

   else

      return 0,0;

   end
end

----------------------------------------------------------------------

function  Entity:getLocX ( world )

   local x = self:getLoc(world);
   return x;

end

----------------------------------------------------------------------

function  Entity:getLocY ( world)

   local x,y = self:getLoc(world);
   return y;

end

----------------------------------------------------------------------

function  Entity:getTop( world )

   local x,y = self:getLoc(world);

   return y+self:getHeight()/2;

end

----------------------------------------------------------------------

function  Entity:getBottom( world )

   local x,y = self:getLoc(world);
   return y-self:getHeight()/2;

end

----------------------------------------------------------------------

function  Entity:getLeft( world )

   local x= self:getLoc(world);
   return x-self:getWidth()/2;

end

----------------------------------------------------------------------

function  Entity:getRight( world )

   local  x = self:getLoc(world);
   return x+self:getWidth()/2;

end

----------------------------------------------------------------------

function  Entity:setLoc ( x, y )

   if self.prop then
      self.prop:setLoc( x, y );
   end

   if self.austin then
      self.austin:setLoc(x, y);
   end

end

----------------------------------------------------------------------

function  Entity:setLocX ( x )

   if self.prop then
      local myX, myY = self.prop:getLoc();

      self.prop:setLoc( x,  myY );
   end

   if self.austin then
      self.austin:setLocX(x);
   end

end

----------------------------------------------------------------------

function  Entity:setLocY ( y )

   if self.prop then
      local myX, myY = self.prop:getLoc();
      self.prop:setLoc( myX, y );
   end

   if self.austin then
      self.austin:setLocY(y);
   end

end

----------------------------------------------------------------------

function  Entity:moveLoc ( dx, dy )

   if self.prop then
      self.prop:moveLoc(dx, dy);
   end

   if self.austin then
      self.austin:moveLoc(dx, dy);
   end

end

----------------------------------------------------------------------

function  Entity:setRot( a )

   if self.prop then
      self.prop:setRot( a );
   end

   if self.austin then
      self.austin:setFaceAngle(a);
   end

end

----------------------------------------------------------------------

function  Entity:getRot()

   if self.prop then
      return self.prop:getRot();
   end

   return 0;

end

----------------------------------------------------------------------

function  Entity:setMoveAngle ( a )

   if self.austin then
      self.austin:setMoveAngle( a );
   end

end


----------------------------------------------------------------------

function  Entity:getScale ()

   if self.prop then
      local sx, sy = self.prop:getScl();
      return sx, sy;
   end

   return 1,1;

end

----------------------------------------------------------------------

function  Entity:setScale (sx, sy, scaleCol)

   if self.prop then
      self.prop:setScl( sx, sy );
   end

	if scaleCol then

		local w = (self.template.collisionWidth or ( self.template.width or 0 )) * sx;
		local h = (self.template.collisionHeight or ( self.template.height or 0 )) * sy;

		self:setCollisionSize(w, h);
	end

end

----------------------------------------------------------------------

function  Entity:getSize( scaled )

   if self.austin then
      if scaled then

         local kx, ky = self:getScale();
         local sx, sy = self.austin:getSize();

         return kx*sx, ky*sy;

      else

         return self.austin:getSize();
      end
   end

   return self:getScale();

end

----------------------------------------------------------------------

function  Entity:setHeight( h )

   if self.austin then
      return self.austin:setHeight(h );
   end

   return 0;

end

----------------------------------------------------------------------

function  Entity:setWidth( w )

   if self.austin then
      return self.austin:setWidth( w );
   end

   return 0;

end

----------------------------------------------------------------------

function  Entity:getWidth()

   if self.austin then
      return self.austin:getWidth();
   end

   return 0;

end

----------------------------------------------------------------------

function  Entity:getHeight()

   if self.austin then
      return self.austin:getHeight();
   end

   return 0;

end

----------------------------------------------------------------------

function  Entity:setCollisionSize( w, h )

	if self.austin then
   	self.austin:setCollisionSize( w, h );
   end

end

----------------------------------------------------------------------

function  Entity:getCollisionWidth()

   if self.austin then
      return self.austin:getCollisionWidth();
   end

   return 0;

end

----------------------------------------------------------------------

function  Entity:getCollisionHeight()

   if self.austin then
      return self.austin:getCollisionHeight();
   end

   return 0;

end

----------------------------------------------------------------------

function  Entity:calcFinalSpeed ()

   if self.austin then
      return self.austin:calcFinalSpeed();
   end

   return 0;

end

----------------------------------------------------------------------

function  Entity:calcFinalSpeedX ()

   if self.austin then
      return self.austin:calcFinalSpeedX();
   end

   return 0;

end

----------------------------------------------------------------------

function  Entity:calcFinalSpeedY ()

   if self.austin then
      return self.austin:calcFinalSpeedY();
   end

   return 0;

end

----------------------------------------------------------------------

function  Entity:setSpeed (sx, sy)

   if self.austin then
      return self.austin:setSpeed(sx,sy);
   end

   return 0,0;

end

----------------------------------------------------------------------

function  Entity:setSpeedX( sx )

   if self.austin then
      return self.austin:setSpeedX(sx);
   end
   return 0;

end

----------------------------------------------------------------------

function  Entity:setSpeedY( sy )

   if self.austin then
      return self.austin:setSpeedY(sy);
   end
   return 0;

end

----------------------------------------------------------------------

function  Entity:getSpeed( compounded )

   if self.austin then
      return self.austin:getSpeed( compounded );
   end
   return 0,0;

end

----------------------------------------------------------------------

function  Entity:getSpeedX( compounded )

   local x,y = self:getSpeed(compounded);

   return x;

end

----------------------------------------------------------------------

function  Entity:getSpeedY( compounded )

   local x,y = self:getSpeed(compounded);
   return y;

end

----------------------------------------------------------------------

function  Entity:getSpeedMaxX(  )

   if self.austin then
      return self.austin:getSpeedMaxX(  );
   end
   return 0;

end

----------------------------------------------------------------------

function  Entity:getSpeedMaxY(  )

   if self.austin then
      return self.austin:getSpeedMaxY(  );
   end
   return 0;

end


----------------------------------------------------------------------

function  Entity:setAccel (ax, ay)

   if self.austin then
      self.austin:setAccel(ax,ay);
   end

end

----------------------------------------------------------------------

function  Entity:setAccelX( ax )

   if self.austin then
      self.austin:setAccelX(ax);
   end

end

----------------------------------------------------------------------

function  Entity:setAccelY( ay )

   if self.austin then
      self.austin:setAccelY(ay);
   end

end


----------------------------------------------------------------------

function  Entity:getAccel(  )

   if self.austin then
      return self.austin:getAccel();
   end

   return 0,0;

end

----------------------------------------------------------------------

function  Entity:getAccelX()

   local x,y = self:getAccel();
   return x;

end

----------------------------------------------------------------------

function  Entity:getAccelY()

   local x,y = self:getAccel();
   return y;

end

----------------------------------------------------------------------

function  Entity:getMass()

   local m =  self.mass;

   if self.massBurst then

      m = m + self.massBurst;

   end

   return m;
end

----------------------------------------------------------------------

function Entity:isCollideRadius(x, y, rad)

   if self.austin then
      return self.austin:isCollideRadius(x, y, rad);
   end
   return false;

end

----------------------------------------------------------------------

function Entity:isCollideEntity( other )

   if self.austin then
      return self.austin:isCollideEntity( other.austin );
   end

   return false;

end

----------------------------------------------------------------------

function  Entity:getCollisionSize()

   if self.austin then
      return self.austin:getCollisionSize();
   end
   return 0,0;

end

----------------------------------------------------------------------

function  Entity:addImpulse( ix, iy  )

   if self.austin then
      self.austin:addImpulse( ix, iy );
   end

end

----------------------------------------------------------------------

function  Entity:addImpulseX( ix, iy  )

   if self.austin then
      self.austin:addImpulseX( ix, iy );
   end

end

----------------------------------------------------------------------

function  Entity:addImpulseY( ix, iy  )

   if self.austin then
      self.austin:addImpulseY( ix, iy );
   end

end

----------------------------------------------------------------------

function  Entity:setImpulse( ix, iy  )

   if self.austin then
      self.austin:setImpulse( ix, iy );
   end

end

----------------------------------------------------------------------

function  Entity:setImpulseX( ix, iy  )

   if self.austin then
      self.austin:setImpulseX( ix, iy );
   end

end

----------------------------------------------------------------------

function  Entity:setImpulseY( ix, iy  )

   if self.austin then
      self.austin:setImpulseY( ix, iy );
   end

end

----------------------------------------------------------------------

function  Entity:getImpulse()

   if self.austin then
      return self.austin:getImpulse();
   end
   return 0,0;

end

----------------------------------------------------------------------

function  Entity:getImpulseX()

   if self.austin then
      return self.austin:getImpulseX();
   end
   return 0;

end

----------------------------------------------------------------------

function  Entity:getImpulseY()

   if self.austin then
      return self.austin:getImpulseY();
   end
   return 0;

end


----------------------------------------------------------------------

function  Entity:isColliding( x, y, cx, cy )

   local vx, vy   = self:getLoc();
   local dx, dy	= math.abs(vx-x),math.abs(vy-y);
   local vcx, vcy	= self:getCollisionSize();

   return      (dx < 0.5*(vcx+cx))
         and	(dy < 0.5*(vcy+cy));

end


----------------------------------------------------------------------

function  Entity:onCollision( collision  )

   local newX, newY  = collision:getNewLoc(self);
   local impX, impY  = collision:getImpulse(self);

   self:setLoc( newX, newY  );

   self:addImpulse(impX, impY);
   self.collisionImpulse[1] = self.collisionImpulse[1] + impX;
   self.collisionImpulse[2] = self.collisionImpulse[2] + impY;

end

----------------------------------------------------------------------

function Entity:keepInsideBoundary( x, y )

   local myWidth,       myHeight = self:getSize();
   local boundWidth, boundHeight = self.boundary.size.x, self.boundary.size.y;

   if ( x - 0.5*myWidth ) < (-0.5*boundWidth) then

      x = -0.5*boundWidth + 0.5*myWidth;

   elseif ( x + 0.5*myWidth ) > (0.5*boundWidth ) then

      x = 0.5*boundWidth - 0.5*myWidth;

   end

   if ( y - 0.5*myHeight ) < (-0.5*boundHeight) then

      y = -0.5*boundHeight + 0.5*myHeight;

   elseif ( y + 0.5*myHeight ) > (0.5*boundHeight ) then

      y = 0.5*boundHeight - 0.5*myHeight;

   end

   return  x, y;

end

----------------------------------------------------------------------

function Entity:_createSplitProp( layer, x, y, w, h, sclX, sclY, name )

   local gfxQuad = MOAIGfxQuad2D.new();
   local texture = self.game:getTexture(name, true);

   gfxQuad:setTexture( texture );
   gfxQuad:setRect( -w/2, -h/2, w/2, h/2 );

   if texture and texture._uv then

      --setUVRect inverse the image for us... go figure!?
      gfxQuad:setUVRect(      texture._uv.xMin or 0
                           ,  texture._uv.yMax or 1
                           ,  texture._uv.xMax or 1
                           ,  texture._uv.yMin or 0 );
   end

   local prop = MOAIProp2D.new();

   prop:setDeck( gfxQuad );
   prop:setPriority( 0 );
   prop:setLoc(x,y);
	prop:setScl(sclX, sclY);
   prop:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA );

   layer:insertProp ( prop )

   return prop;

end

----------------------------------------------------------------------

function Entity:setAustinToProp( flag )

	self.ausToProp = flag;

end

----------------------------------------------------------------------
