require('game/entity/entity');



----------------------------------------------------------------------
local Base = Entity;

Explosion = Base:new();

----------------------------------------------------------------------

function Explosion:new( game, layer, template,  x, y, a, flipHoriz )

   local e = Base.new(self, game, layer, template, x, y, a );

   e.game            = game;
   e.fadeStart       = template.fadeStart          or 0;
   e.avatarSpeedK    = e.template.avatarSpeedK     or 0;
   e.maxDamageRadius = e.template.maxDamageRadius  or 100000000;
   e.sounds          = e.template.sounds;
	e._flipHoriz		= flipHoriz or false;

   --make sure we call reset after we created the animation
   e:reset( x, y, a );

   return e;

end

----------------------------------------------------------------------

function Explosion:delete( )

   if self:isFlashAnim() then
      self.flashAnim:delete();
   end

   Base.delete( self );

end

----------------------------------------------------------------------

function Explosion:hide( )

   Base.hide( self );

   if self:isFlashAnim() then
      self.flashAnim:hide();
   end

end


----------------------------------------------------------------------

function Explosion:show( )

   Base.show( self );

   if self:isFlashAnim() then
      self.flashAnim:show();
      self.flashAnim:setTime(0);
      self.flashAnim:start();
      self.prop:setVisible(false);
   end

end


----------------------------------------------------------------------

function Explosion:reset( x, y, angle )

   Base.reset( self, x, y, angle );

   self.prop:setPriority(self.template.priority or 100);

   self.damageDone	= false;

   if self:isFlashAnim() then
      self.flashAnim:setListener(   MOAIAction.EVENT_STOP,
                                    function()
                                       if self and self.flashAnim then
                                          self.flashAnim:hide();
                                       end
                                    end );
   end

	if self.template.animate and not self.template.flipHoriz then
		self._animFlipHorizontal = false;
	end

end

----------------------------------------------------------------------

function Explosion:update( dt, avatarSpeedY )

   Base.update( self, dt, avatarSpeedY );

   local age = self:getAge();
   local tpl = self.template;

   if       ( not self.damageDone  )
      and   ( tpl.damage )   then

      if age > tpl.damage.start then
      
         if age < tpl.damage.stop then

            local x, y = self:getLoc();
            
            local ssx = self:getSize( true );
						ssx = math.abs(ssx);
            
            local rad = math.min(ssx/2,self.maxDamageRadius or 100000000);

            self.game:areaDamageVehicle( x,y, rad, tpl.damage.vehicleVal, tpl.damage.avatarVal, tpl.damage.hazardVal, tpl.damage.projVal );
         
         else 
            self.damageDone = true;
         end
      end
   end

   local a = 1-math.max(    (age-self.fadeStart)
                           /(tpl.duration),    0  );

   if a > 0 then

      self:setColor( self.color[1],self.color[2],self.color[3], self.color[4]*a);

      if tpl.growth then

         local sx, sy = self:getScale();

         sx = math.abs(sx) + tpl.growth[1]*dt;
         sy = sy + tpl.growth[2]*dt;

			if self._animFlipHorizontal == true then
				self._animFlipHorizontal	= false;
				sx									= -sx;
			elseif self._animFlipHorizontal == false then
				self._animFlipHorizontal	= true;
				sx									= math.abs(sx);
			elseif self._flipHoriz then
				sx									= -sx;
			end

         self:setScale( sx, sy );
      end

   elseif ( (self.damageDone ) 
         or (not tpl.damage) ) then

      self.deleteMe = true;
   end

end

----------------------------------------------------------------------