require('game/entity/entity');

----------------------------------------------------------------------

Flyout = Entity:new();

----------------------------------------------------------------------

function Flyout:new( game, layer, tpl,  x, y, a, ... )

   local e = Entity.new(self, game, layer, tpl, x, y, a, ... );

   if tpl then

      e.duration = tpl.duration;

      e.keyIndex = 1;
      e.keys     = {};

      for i=1, #tpl.keys do

         local tplKey      = tpl.keys[i];

         e.keys[i]         = {};
         e.keys[i].time    = tplKey.time;
         e.keys[i].scale   = tplKey.scale;
         e.keys[i].alpha   = tplKey.alpha;
         e.keys[i].speed   = tplKey.speed;

      end
   end

   return e;

end

----------------------------------------------------------------------

function Flyout:reset( x, y, angle )

   Entity.reset( self, x, y, angle );

   self.keyIndex = 1;

end

----------------------------------------------------------------------

function Flyout:update( dt, avatarSpeedY )

   local color = self.color;
   local key   = self.keys[self.keyIndex];
   local next  = self.keys[self.keyIndex+1];
   local age   = self:getAge();
   
   while next and age > next.time do

      self.keyIndex = self.keyIndex+1;
      key           = self.keys[self.keyIndex];
      next          = self.keys[self.keyIndex+1];

   end

   if key then

      local alpha    = key.alpha;
      local scale    = key.scale;
      local speed    = key.speed;

      if next then

         local timeRel  = math.clamp((age-key.time)/(next.time-key.time),0,1);

         alpha = math.lerp(key.alpha,next.alpha,timeRel);
         scale = math.lerp(key.scale,next.scale,timeRel);
         speed = math.lerp(key.speed,next.speed,timeRel);

      end

      self:setColor( color[1],color[2],color[3], color[4]*alpha );
      self:setScale( scale, scale );
      self:setSpeed( 0,     speed );

		if self.dropShadow and self.dropShadow.textBox then
			self.dropShadow.textBox:setColor(0, 0, 0, color[4]*alpha);
		end
   end




   Entity.update( self, dt, avatarSpeedY );

   if age > self.duration then

      self.deleteMe = true;

   end
end

----------------------------------------------------------------------