require('game/entity/entity');

----------------------------------------------------------------------

local AIR_TARGET_STATE_WAIT   = 1;
local AIR_TARGET_STATE_GOING  = 2;
local AIR_TARGET_STATE_DONE   = 3;

----------------------------------------------------------------------

AirStrikeTarget = Entity:new();

----------------------------------------------------------------------

function AirStrikeTarget:new( game, layer, template, x, y, a, owner )

   local e = Entity.new( self, game, layer, template, x, y, a, owner );

   return e; 
   
end

----------------------------------------------------------------------

function AirStrikeTarget:reset( x, y, angle , owner )

   Entity.reset( self, x, y, angle , owner );

   self.age          = 0;
   self.state        = AIR_TARGET_STATE_WAIT;
   self.stateTime    = 0;
   self.owner        = owner;
   self.delay        = self.template.delay;
   self.readyTime    = self.template.readyTime;
   self.timeToExplode= self.template.timeToExplode;

   self:hide();

end

----------------------------------------------------------------------

function AirStrikeTarget:update ( dt )

   Entity.update( self, dt );

   self.stateTime = self.stateTime + dt;

   if self.state == AIR_TARGET_STATE_WAIT  then

      self:setLocX( self.owner:getLocX() );

      if self.age > self.delay then
         self.state     = AIR_TARGET_STATE_GOING;
         self.stateTime = 0;
      end

   elseif self.state == AIR_TARGET_STATE_GOING then

      if self.age > self.timeToExplode then

         self:hide();
         self.state = AIR_TARGET_STATE_DONE;

         self.game.entityMgr:spawnExplosion(self.template.explosionTpl, self:getLoc());

      else

         local timeK = math.clamp(       self.stateTime
                                    /  ( self.readyTime-self.delay ), 0, 1);
         local alpha = math.smoothStep(0,1,timeK);
         local scale = math.lerp( self.template.startScale, 1, timeK)

         self:setColor(1,1,1, alpha*alpha);
         self:setScale( scale, scale);

         self:show();

      end
   end

   return self.state == AIR_TARGET_STATE_DONE;

end

----------------------------------------------------------------------

function AirStrikeTarget:isDone (  )

   return self.state == AIR_TARGET_STATE_DONE;

end

----------------------------------------------------------------------