require('game/entity/entity');

----------------------------------------------------------------------

local HAZARD_MISSILE_LINE_STATE_WAIT   = 1;
local HAZARD_MISSILE_LINE_STATE_GOING  = 2;
local HAZARD_MISSILE_LINE_STATE_DONE   = 3;

----------------------------------------------------------------------

local Base         = Entity;
HazardMissileFlame = Base:new();

----------------------------------------------------------------------

function HazardMissileFlame:new( game, layers, template, x, y, a, owner )

print("HazardMissileFlame:new", game, layers, template, x, y, a, owner);

   local e = Base.new( self, game, layers[template.layer], template, x, y, a, owner );

   e.prop:setAttrLink ( MOAIProp2D.INHERIT_TRANSFORM, owner.prop, MOAIProp2D.TRANSFORM_TRAIT );
	e.prop:setAttrLink ( MOAIProp2D.ATTR_VISIBLE, owner.prop, MOAIProp2D.ATTR_VISIBLE );

   return e; 
   
end

----------------------------------------------------------------------

function HazardMissileFlame:reset( x, y, angle , owner )

   Base.reset( self, x, y, angle , owner );

	self._launchTimer	= 0;

   if self.template.offset then
      self:setLoc( self.template.offset[X], self.template.offset[Y] );
   end

	if self.template.animate then
		self._animFlipHorizontal = false;
	end

	if			self.template.launchTime
		and	self.template.launchScale
		and	self.template.launchFade then

		self._launchAnim	= true;
	end

   self.changeColor = math.random() * 1;

end

----------------------------------------------------------------------

function HazardMissileFlame:delete()

   if self.shadow then
      self.shadow:delete();
   end

   if self.fire then
      self.fire:delete();
   end

   Base.delete(self);

end

----------------------------------------------------------------------

function HazardMissileFlame:update ( dt )

   Base.update( self, dt );
   
   self.changeColor = self.changeColor - dt;
   
   if self.changeColor < 0 then
   
      self.changeColor = math.lerp(0.05,0.10,math.random());
      
      self:setColor( 1,1,1, math.lerp(0.75,1.00,math.random()) );
      
      self._scaleX = math.lerp(0.85,1.05,math.random());
		self._scaleY = math.lerp(0.85,1.05,math.random());
      self:setScale( self._scaleX, self._scaleY );
   end

	if self._launchAnim == true then
		self:updateLaunchAnim(dt);
	end

	if self._animFlipHorizontal == false then
		local scaleX = self._scaleX or 1;
		local scaleY = self._scaleY or 1;

		self._animFlipHorizontal	= true;

		self:setScale(1*scaleX, 1*scaleY);
	elseif self._animFlipHorizontal == true then
		local scaleX = self._scaleX or 1;
		local scaleY = self._scaleY or 1;

		self._animFlipHorizontal	= false;

		self:setScale(-1*scaleX, 1*scaleY);
	end

end

----------------------------------------------------------------------

function HazardMissileFlame:updateLaunchAnim( dt )

	if self._launchTimer <= self.template.launchTime then

		if self._launchTimer < self.template.launchFade then
			self._scaleX = math.mapClamped(0, 1, self.template.launchFade, self.template.launchScale[1], self._launchTimer);
			self._scaleY = math.mapClamped(0, 1, self.template.launchFade, self.template.launchScale[2], self._launchTimer);
		else
			self._scaleX = math.mapClamped(self.template.launchFade, self.template.launchScale[1], self.template.launchTime, 1, self._launchTimer);
			self._scaleY = math.mapClamped(self.template.launchFade, self.template.launchScale[2], self.template.launchTime, 1, self._launchTimer);
		end

		self:setScale(self._scaleX, self._scaleY);
	else
		if self._scaleX > 1.05 or self._scaleX < 0.85 then
			self._scaleX	= 1;
		end

		if self._scaleY > 1.05 or self._scaleY < 0.85 then
			self._scaleY	= 1;
		end

		self:setScale(self._scaleX, self._scaleY);
	end

	self._launchTimer = self._launchTimer + dt;

end

----------------------------------------------------------------------