require('game/entity/entity');

----------------------------------------------------------------------

local HAZARD_MISSILE_LINE_STATE_WAIT   = 1;
local HAZARD_MISSILE_LINE_STATE_GOING  = 2;
local HAZARD_MISSILE_LINE_STATE_DONE   = 3;

----------------------------------------------------------------------

HazardMissileLine = Entity:new();

----------------------------------------------------------------------

function HazardMissileLine:new( game, layers, template, x, y, a, owner )

   local e = Entity.new( self, game, layers[template.layer], template, x, y, a, owner );
   
   e.sounds = e.template.sounds;
   
   
   return e; 
   
end

----------------------------------------------------------------------

function HazardMissileLine:reset( x, y, angle , owner )

   Entity.reset( self, x, y, angle , owner );

   self.state        = HAZARD_MISSILE_LINE_STATE_WAIT;
   self.stateTime    = 0;
   self.owner        = owner;
   self.delay        = self.template.delay;
   self.readyTime    = self.template.readyTime;
   self.fadeoutTime  = self.template.fadeoutTime;
   self.overTime     = self.template.overTime;
   self.tplColor     = self.template.color or {1,1,1,1};

   self:hide();

end

----------------------------------------------------------------------

function HazardMissileLine:update ( dt )

   Entity.update( self, dt );

   self.stateTime = self.stateTime + dt;
   
   local age = self:getAge();

   if self.state == HAZARD_MISSILE_LINE_STATE_WAIT  then

      self:setLocX( self.owner:getLocX() );

      if age > self.delay then
         self.state     = HAZARD_MISSILE_LINE_STATE_GOING;
         self.stateTime = 0;

         --EventMgr:postEvent( {sounds=self.sounds, name="HazardMissileLine"}, "add-sfx" );

      end

   elseif self.state == HAZARD_MISSILE_LINE_STATE_GOING then

      if age > self.overTime then
      
         --EventMgr:postEvent( {sounds=self.sounds, name="HazardMissileLine:self.age > self.overTime",}, "loop-stop" );

         self:hide();
         self.state = HAZARD_MISSILE_LINE_STATE_DONE;

      elseif age > self.fadeoutTime then

         local timeK = math.clamp(     ( age-self.fadeoutTime )
                                    /  ( self.overTime-self.fadeoutTime   ), 0, 1 );
                                    
         local alpha = math.smoothStep(1,0,timeK);
         
         self:setColor(self.tplColor[1],self.tplColor[2],self.tplColor[3], alpha*self.tplColor[4]);
         
      else

         local timeK = math.clamp(       self.stateTime
                                    /  ( self.readyTime-self.delay ), 0, 1);
         local alpha = math.smoothStep(0,1,timeK);
         local scale = math.lerp( self.template.startScale, 1, timeK)

         self:setColor(self.tplColor[1],self.tplColor[2],self.tplColor[3], alpha*alpha*self.tplColor[4]);
         self:setScale( scale, scale);

         self:show();

      end
   end

   return self.state == HAZARD_MISSILE_LINE_STATE_DONE;

end

----------------------------------------------------------------------

function HazardMissileLine:stop()

	if self then
		self.state = HAZARD_MISSILE_LINE_STATE_DONE;
		self:setAge(self.overTime+1);
		self:hide();
	end

end
----------------------------------------------------------------------

function HazardMissileLine:isDone (  )

   return self.state == HAZARD_MISSILE_LINE_STATE_DONE;

end

----------------------------------------------------------------------