require('game/entity/entity');

----------------------------------------------------------------------

local Base  = Entity;
HealthBar   = Base:new();

----------------------------------------------------------------------

function HealthBar:new( game, layer, template,  x, y, a, owner, ownerOffset )

   local e	= Base.new(self, game, layer, template, x, y, a, owner, ownerOffset );

	e.prop:setPriority(template.priority or 1);

	if template.frame and template.background then
		e.background	= Base.new(self, game, layer, template.background, x, y, a, owner, ownerOffset )
		e.frame			= Base.new(self, game, layer, template.frame, x, y, a, owner, ownerOffset );

		e.frame.prop:setPriority(math.max((template.priority or 1)+2, 2));
		e.prop:setPriority(math.max((template.priority or 1)+1, 1));
		e.background.prop:setPriority(math.max((template.priority or 0)-1, 0));
	end

   return e;

end

----------------------------------------------------------------------

function HealthBar:reset( x, y, a, owner, ownerOffset )

   Entity.reset( self, x, y, a );
   
   self.owner        = owner;
   self.ownerOffset  = ownerOffset;
   self.damageOffset = 0;
      
   self:hide();

end


----------------------------------------------------------------------

function  HealthBar:update( dt, avatarSpeedY )

   if self.owner then
   
      local x,y  = self.owner:getLoc();
		local bx	  = x;

      if self.ownerOffset then
         x  = x+self.ownerOffset[1]+self.damageOffset;
         y  = y+self.ownerOffset[2];
			bx = bx+self.ownerOffset[1];
      end
   
      self:setLoc( x, y );

		if self.frame then
			self.frame:setLoc(bx, y);
		end

		if self.background then
			self.background:setLoc(bx, y);
		end
   end

end

----------------------------------------------------------------------

function  HealthBar:onDamage( d )

   local ratio = self.owner.damageValue/self.owner.hitPoints
   local ratio2= 1-ratio;

   self:show();
   
   self.damageOffset = -0.5*ratio*self:getWidth();
   self:setScale(ratio2,1);

   local color;

   if ratio2 > 0.5 then
      color = self.template.colorFull;
   elseif ratio2 > 0.25 then
      color = self.template.colorHalf;
   else
      color = self.template.colorEmpty;
   end
   
   color = color or self.template.color;
   
   self:setColor(color[1],color[2],color[3],color[4]);
      

end

----------------------------------------------------------------------

function HealthBar:delete()

	if self.frame then
		self.frame:delete();
	end

	if self.background then
		self.background:delete();
	end

	Base.delete(self);

end

----------------------------------------------------------------------

function HealthBar:hide()

	if self.frame then
		self.frame:hide();
	end

	if self.background then
		self.background:hide();
	end

	Base.hide(self);

end

----------------------------------------------------------------------

function HealthBar:show()

	if self.frame then
		self.frame:show();
	end

	if self.background then
		self.background:show();
	end

	Base.show(self);

end

----------------------------------------------------------------------