require('game/entity/entity');

----------------------------------------------------------------------

local Base = Entity
Pickup = Base:new();

----------------------------------------------------------------------

function Pickup:new( game, layer, template,  x, y, a, boundary, index, count, speedK  )

   local p = Base.new( self, game, layer, template, x, y, a, boundary, index, count, speedK );

   if template then

		p.prop:setPriority(10);

      p.pickupType   = template.type;
      p.sounds       = template.sounds;
      p.currency     = template.currency;
      p.value        = template.value;
      p.tplPowerup   = p.template.tplPowerup;
      p.pickableDelay= p.template.pickableDelay or 0;
		p.tangoId		= template.tangoId or nil;

		if template.element1 then
			p.element1				= Entity:new( game, layer, template.element1, x, y, a );
			p.element1.spinRate	= (template.element1.spin or 0) + (math.random()*( template.element1.spinRnd or 0));

			if p.element1.spinRate ~= 0 then
				p.element1.spinRate = 180/p.element1.spinRate;
			end

			p.element1.prop:setPriority(template.element1.priority or 0);
			p.element1.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, p.prop, MOAIProp2D.TRANSFORM_TRAIT );
			p.element1:setLoc( template.element1.offset[1] or 0, template.element1.offset[2] or 0 );
		end

		if template.element2 then
			p.element2				= Entity:new( game, layer, template.element2, x, y, a );
			p.element2.spinRate	= (template.element2.spin or 0) + (math.random()*( template.element2.spinRnd or 0));

			if p.element2.spinRate ~= 0 then
				p.element2.spinRate = 180/p.element2.spinRate;
			end

			p.element2.prop:setPriority(template.element2.priority or 0);
			p.element2.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, p.prop, MOAIProp2D.TRANSFORM_TRAIT );
			p.element2:setLoc( template.element2.offset[1] or 0, template.element2.offset[2] or 0 );
		end

      p:debugCollision();

   end

   return p;

end

----------------------------------------------------------------------

function Pickup:reset( x, y, a, boundary, index, count, speedK  )

   Base.reset( self, x, y, a );

   if not speedK then
      speedK = {1,1};
   end

   self.pickable     = false;
   self.boundary     = boundary;
   self.magnetized   = false;
	self.droneMount	= nil;
   self.magnetDelta  = { 10000, 10000 };

	if self.element1 then
		self.element1.spinRate	= (self.template.element1.spin or 0) + (math.random()*( self.template.element1.spinRnd or 0));

		if self.element1.spinRate ~= 0 then
			self.element1.spinRate = 180/self.element1.spinRate;
		end
	end

	if self.element2 then
		self.element2.spinRate	= (self.template.element2.spin or 0) + (math.random()*( self.template.element2.spinRnd or 0));

		if self.element2.spinRate ~= 0 then
			self.element2.spinRate = 180/self.element2.spinRate;
		end
	end

   if count ~= 1 then

      local speedMin = self.template.speed[1]*speedK[1];
      local speedMax = (self.template.speed[1]+self.template.speedRnd[1])*speedK[1];
      
      if x < (7/10*boundary.min.x) then

         speedMin = math.max(2*(boundary.min.x-x), speedMin);
         speedMax = speedMax*1.5;

      elseif x > (7/10*boundary.max.x) then

         speedMin = speedMin*1.5;
         speedMax = math.min(2*(boundary.max.x-x), speedMax);

      end

      local k       = (index + 0.80*math.random() - 1)/count;

      self:setSpeed( math.lerp( speedMin, speedMax, k )
                   , self:getSpeedY()*speedK[2]             );
      
   else

      self:setSpeed( self:getSpeedX()*speedK[1]
                   , self:getSpeedY()*speedK[2] );

   end

end

----------------------------------------------------------------------

function Pickup:update( dt )

   Base.update( self, dt );

   local myW  = self:getWidth();

   local x, y = self:getLoc();

   if not self.pickable then
      self.pickable =      self:getAge() > self.pickableDelay
                     or    self:getSpeedY() < 0;
   end

	if self.element1 then
		self:_spinElement(dt, self.element1, self.element1.spinRate);
	end

	if self.element2 then
		self:_spinElement(dt, self.element2, self.element2.spinRate);
	end

   if self.boundary then

      local boundMinX = self.boundary.min.x;
      --We are getting close to the edge...
      if (x-2*myW) <  boundMinX  then

         --We pass the boundary
         if (x-0.5 * myW) < boundMinX then

            x = self.boundary.min.x+0.5*myW;
            self:setSpeedX( 0.5* math.abs( self:getSpeedX() ) );

         else

            --Slow down..
            self:setSpeedX( self:getSpeedX()*0.975 );

         end

      else

         local boundMaxX = self.boundary.max.x;

         if (x+2*myW) > boundMaxX then

            if (x+0.5*myW) > boundMaxX then

               x = self.boundary.max.x-0.5*myW;
               self:setSpeedX( -0.5* math.abs( self:getSpeedX() ) );

            else

               --Slow down..
               self:setSpeedX( self:getSpeedX()*0.975 );

            end
         end
      end

      self:setLoc(x,y);
   end

end

----------------------------------------------------------------------

function Pickup:delete()

	Base.delete(self);

	if self.element1 then
		self.element1:delete();
	end

	if self.element2 then
		self.element2:delete();
	end

end

----------------------------------------------------------------------

function Pickup:hide()

	Base.hide(self);

	if self.element1 then
		self.element1:hide();
	end

	if self.element2 then
		self.element2:hide();
	end

end

----------------------------------------------------------------------

function Pickup:show()

	Base.show(self);

	if self.element1 then
		self.element1:show();
	end

	if self.element2 then
		self.element2:show();
	end

end

----------------------------------------------------------------------

function Pickup:_spinElement( dt, element, rate )

	if not rate or rate == 0 then
		return false;
	end

	if not element.spinTime then
		element.spinTime = 0;
	end

	local absRate	= math.abs(rate);
	local spin		= 360;

	if rate < 0 then
		spin = -360;
	end

	element.spinTime = element.spinTime + dt;

	if element.spinTime > absRate then
		element.spinTime = 0;
	end

	element.prop:setRot(spin*(element.spinTime/absRate));

end

----------------------------------------------------------------------

