require('game/entity/powerup/powerup');

----------------------------------------------------------------------

local STATE_ACCEL = 1;
local STATE_DECEL = 2;

----------------------------------------------------------------------

PowerupDash = Powerup:new();

----------------------------------------------------------------------

function PowerupDash:new( game, layer, template,  x, y, a, owner )

   local p = Powerup.new( self, game, layer, template,  x, y, a, owner );

   p.warningDuration = template.warningDuration or 0;
   p.warningCount    = template.warningCount    or 1;
   p.warningColor    = template.warningColor    or { 1,1,1,0 };
   p.warningScale    = template.warningScale    or 1;

   return p;

end

----------------------------------------------------------------------

function PowerupDash:reset( x, y, angle, owner )
   
   Powerup.reset( self, x, y, angle, owner );
   self.state = STATE_ACCEL;

end

----------------------------------------------------------------------

function PowerupDash:onHitBarricade()

   if self.state == STATE_ACCEL then
   
      --this will stop us and throw projectile
      self:setAge( self.duration );
      
   end
end

----------------------------------------------------------------------

function PowerupDash:onHitHazardMissile()

   if self.state == STATE_ACCEL then
   
      --this will stop us and throw projectile
      self:setAge( self.duration );
      
   end
end

----------------------------------------------------------------------

function PowerupDash:update( dt )
   
   Powerup.update( self, dt );

   local age = self:getAge();
   
   if self.owner:isDead() then

      self:hide();
      self.owner.speedBurst = 0;

   elseif self.state == STATE_ACCEL then

      self.owner.speedBurst  = 0.90*self.owner.speedBurst+0.10*self.template.speedBurst;

      if age > self.duration then

         local x,y  = self.owner:getLoc();
         self:hide();

			local p;

			if self.game.gameDirector:isBossScenarioActive() then
				local tpl	= Template.clone( self.template.tplProjectile );

				tpl.damage	= 0;
				p				= self.game.entityMgr:spawnProjectile(  tpl, x, y, 0, self.owner );
			else
				p	= self.game.entityMgr:spawnProjectile(  self.template.tplProjectile, x, y, 0, self.owner );
			end


         p:setSpeedY( math.max(self.owner:getSpeedY(),p:getSpeedY()) );

         self.state = STATE_DECEL;

         if self.template.disableBoostPickup then
            self.game:enablePickup( PICKUP_BOOST_INDEX );
         end

      elseif age > ( self.duration - self.warningDuration ) then

         local rel = math.clamp((self.duration-age)/self.warningDuration,0,1);
         local k   = 0.5*(math.cos(2*math.pi*rel*self.warningCount)+1);
         local k2  = 1-math.smoothStep(0,1,k);

         local r   = math.lerp(self.color[1],self.warningColor[1], k2);
         local g   = math.lerp(self.color[2],self.warningColor[2], k2);
         local b   = math.lerp(self.color[3],self.warningColor[3], k2);
         local a   = math.lerp(self.color[4],self.warningColor[4], k2);

         self:setColor(r,g,b,a);

         local scale = math.lerp(self.warningScale, 1, k2);
         self:setScale(scale);
         self:setCollisionSize( self:getWidth()*scale, self:getHeight()*scale );
         if self.template.disableBoostPickup then
            self.game:disablePickup( PICKUP_BOOST_INDEX );
         end

      end

   else

      self.owner.speedBurst  = math.max(self.owner.speedBurst-self.template.speedBurst/6, 0);

   end
end

----------------------------------------------------------------------

function PowerupDash:isDone( )

   if self.owner:isDead() then

      return true;

   elseif      self.state == STATE_DECEL
         and   self.owner.speedBurst == 0  then

      return Powerup.isDone( self );

   end

   return false;

end
----------------------------------------------------------------------
