require('game/collision');
require('game/entity/entity');
require('game/entity/hazard/hazard-missile-flame');

----------------------------------------------------------------------

local Base  = Entity;
Projectile  = Entity:new();


----------------------------------------------------------------------

function Projectile:new( game, layers, template, x, y, a, owner, target, parent, pOffset )

	if template then

		local p = Base.new( self, game, (layers or {})[(template or {}).layer or 6], template, x, y, a, owner, target, parent, pOffset );

		if template.fire then
			p.fire = HazardMissileFlame:new( game, layers, template.fire, x, y, a, p );
		end

		p:debugCollision();

		return p;
	end

	return Base.new( self,  game );

end

----------------------------------------------------------------------

function Projectile:reset( x, y, a, owner, target, parent, pOffset )

   Base.reset( self, x, y, a );

   self.owner				= owner;
	self.damageAvatar		= self.template.damageAvatar;
   self.damageBarricade = self.template.damageBarricade;
   self.damageMissile   = self.template.damageMissile;
   self.invicible			= self.template.invicible;
   self.speedBurst		= self.template.speedBurst;
	self.curScale			= self.template.scale or {1, 1};
	self.scaleRate			= self.template.scaleRate or {0, 0};
	self.hitMarker			= self.template.hitMarker;
	self.noHitRepeat		= self.template.noHitRepeat or false;
	self.hitList			= {};
	self.clearLinkTimer	= self.template.projDetachTime or false;
   self._lifeTime       = 0;

	self:setMoveAngle(a);

	if parent then

      self.hasParent		= true;
		self.pOffset		= pOffset

		if parent.prop then
			self.parentProp	= parent.prop;
			self.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, parent.prop, MOAIProp2D.TRANSFORM_TRAIT );
		else
			self.parentProp	= parent;
			self.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, parent, MOAIProp2D.TRANSFORM_TRAIT );
		end
   else
      self.hasParent		= false;
		self.parentProp	= nil;
		self.pOffset		= nil;
      self.prop:clearAttrLink (MOAIProp2D.INHERIT_TRANSFORM);
	end

	if self.fire then
		self.fire:reset();
	end

end

----------------------------------------------------------------------

function Projectile:update( dt, avatarSpeedY )

	Base.update( self, dt, avatarSpeedY );

   self:updateLifeTime(dt);
	self:updateScale(dt);

	if self.fire then
		self.fire:update(dt);
	end

end

----------------------------------------------------------------------

function Projectile:updateLifeTime( dt )

   if self.template.maxLifeTime == nil then
      return;
   end

   if self.template.maxLifeTime <= 0 then
      return;
   end

   self._lifeTime = self._lifeTime + dt;

   if self._lifeTime > self.template.maxLifeTime then
      self:onDeleteMe();
   end

end

----------------------------------------------------------------------

function Projectile:updateScale( dt )

	if	self.scaleRate[1] ~= 0 or self.scaleRate[2] ~= 0 then

		self.curScale[1] = math.max(self.curScale[1] + self.scaleRate[1]*dt, 0);
		self.curScale[2] = math.max(self.curScale[2] + self.scaleRate[2]*dt, 0);

		self:setScale(self.curScale[1], self.curScale[2], true);
	end

end

----------------------------------------------------------------------

function Projectile:addHitEntity( target )

	if target then
		self.hitList[target] = true;
	end

end

----------------------------------------------------------------------

function Projectile:hasHit( target )

	if self.hitList and target then

		return self.hitList[target];
	end

end

----------------------------------------------------------------------

function Projectile:onDeleteMe()

   if self.fire then
      self.fire:onDeleteMe();
   end

   Base.onDeleteMe(self);

end

----------------------------------------------------------------------

function Projectile:delete()

   if self.fire then
      self.fire:delete();
   end

   Base.delete(self);

end

----------------------------------------------------------------------

function Projectile:hide( )

   Base.hide(self);

   if self.fire then
      self.fire:hide();
   end

end

----------------------------------------------------------------------

function Projectile:show( )

   Base.show(self);

   if self.fire then
      self.fire:show();
   end

end

----------------------------------------------------------------------
