require('game/collision');
require('game/entity/projectile/projectile');

----------------------------------------------------------------------

local Base  = Projectile;
Rocket      = Base:new();

----------------------------------------------------------------------

function Rocket:new( game, layers, template, x, y, a, owner )

   local e = Base.new( self, game, layers, template, x, y, a, owner );

   return e;

end

----------------------------------------------------------------------

function Rocket:reset( x, y, a, owner, target )

   Base.reset( self, x, y, a, owner, target );

   self.angle        = a;
   self:setMoveAngle(a);

   self.maxTurnRate  = self.template.maxTurnRate or 0;
   self.turnDelay    = self.template.turnDelay or 0;

   self.accelDelay   = self.template.accelDelay or 0;

   if self.accelDelay > 0 then
      self:setAccel(0,0);
   end

   self:setTarget( target );
   self.homing    = self.template.homing;
   self.homingDelay  = self.template.homingDelay or 0;

   if self.template.overrideTarget then
      self:setTarget( nil );
   end

	if			not self.target
		and	owner
		and	owner ~= self.game.avatar then

      self:setTarget( self.game.avatar );

	end

end

----------------------------------------------------------------------

function Rocket:update( dt, avatarSpeedY )

   Base.update( self, dt, avatarSpeedY );

   self:updateTarget(dt);

   if self.target and self.homing then

      self.turnDelay = (self.turnDelay or 0) - dt;

      if self.turnDelay < 0 then
         self:updateFace(dt);
      end

   end

   if self.accelDelay > 0 then

      self.accelDelay = self.accelDelay - dt;

      if self.accelDelay <= 0 then

         self:setAccel(self.template.accel[1],self.template.accel[2])

      end
   end

end

----------------------------------------------------------------------

function Rocket:isTargetValid( target )

   if     (target)
      and (not target.deleteMe )
      and (not target:isDead() ) then

      return true;

   end

end

----------------------------------------------------------------------

function Rocket:updateFace( dt  )

   if       self.game
      and   self.target
      and   self:isTargetValid(self.target)  then

      local x, y   = self:getLoc(true);
      local tx, ty = self.target:getLoc();
      local dx, dy = tx-x, ty-y-self.target:getHeight()/4;

      --Calculate around Y
      local rad   = -math.atan2(dx,dy);

      local a     = ( rad*180/math.pi );
      local da    = math.clampAngle180( a-self.angle );

      local da2   = math.clamp(da, -self.maxTurnRate*dt,self.maxTurnRate*dt);

      self.angle  = self.angle+da2;
      self.da     = da2;

      self:setRot( self.angle );
      self:setMoveAngle(self.angle);

   end
end

----------------------------------------------------------------------

function Rocket:updateTarget(dt)

   if not self:isTargetValid(self.target) and self.homing and self.target ~= self.game.avatar then

      self:setTarget( nil );

      self.homingDelay  = (self.homingDelay or 0) - dt;

      if  self.homingDelay < 0 then
         local vehicles    = self.game.entityMgr.vehicles;
         local best        = nil;
         local bestScore   = nil;
         local myX, myY    = self:getLoc();
         local owX, owY    = (self.owner or self):getLoc();
         local deltaOwner  = myX - owX;

         for i=1,#vehicles do

            local tgt   = vehicles[i];

            if self:isTargetValid( tgt ) then

               local tgtX, tgtY  = tgt:getLoc(true);
               local dX, dY      = math.abs(tgtX-myX),
                                   math.abs(tgtY);

               if  tgtY>myY then
                  local deltaOwner2 = tgt:getLocX() - owX;
                  local costOwner   = -(math.sign(deltaOwner)*deltaOwner2);
                  local score       = math.abs(tgtY) + 10*math.random() + 10*costOwner + (tgt.rocketTargetPenalty or 0);

                  if bestScore == nil or score < bestScore then
                     bestScore = score;
                     best      = tgt;
                  end
               end

            end
         end


         if best then
            self:setTarget( best );
         end

      end
   end
end


----------------------------------------------------------------------

function Rocket:setTarget( tgt )

   if tgt ~= self.target then

      if self.target then
         self.target.rocketTargetPenalty = (self.target.rocketTargetPenalty or 0 ) - 1000;
      end

      if tgt then
         tgt.rocketTargetPenalty = (tgt.rocketTargetPenalty or 0) + 1000;
      end

      self.target = tgt;

   end

end


----------------------------------------------------------------------
