require('game/entity/turret/turret');

----------------------------------------------------------------------
local BOSS_INDEX											= 2;

local TURRET_SHOOT_DISABLED                     = 1;
local TURRET_SHOOT_SLEEP                        = 2;
local TURRET_SHOOT_AIM                          = 3;
local TURRET_SHOOT_SHOOT                        = 4;
local TURRET_SHOOT_ON_SCREEN_DELTA              = 25;
local TURRET_SHOOT_TARGET_AVATER_VALID_OFFSET   = 50;
local TURRET_SHOOT_TARGET_AI                    = 1;
local TURRET_SHOOT_TARGET_AVATAR                = 2;

local TURRET_WEAPON_GUN									= 1;
local TURRET_WEAPON_ROCKET								= 2;

local TURRET_GLOW_VFX_SLEEP							= 1;
local TURRET_GLOW_VFX_START							= 2;
local TURRET_GLOW_VFX_STOP								= 3;

local ROTOR_RATE											= 0.5;
local	ROTOR_WASH_RATE									= 2.0;
local ROTOR_WASH_ADD_SCALE								= 0.5;

local ANIM_STATE_IDLE									= 1;
local ANIM_STATE_STRAFE									= 2;
local ANIM_STATE_STRAFE_EASE							= 3;

----------------------------------------------------------------------

local Base  = Turret;
TurretBossGunship = Base:new();

----------------------------------------------------------------------

function TurretBossGunship:new(  game, layer, template,  x, y, angle, parent, offset )

   local e = Base.new( self, game, layer, template, x, y, angle, parent, offset );

	e.prop:setPriority(10);

	if e.splitRight and e.splitLeft then
		e.splitRight:setPriority(11);
		e.splitLeft:setPriority(11);
	end

	if template.armor then
		e.armor = Entity.new( Entity, game, layer, template.armor, 0, 0, 0, e, {template.armor.offset[1], template.armor.offset[2]}  );
		e.armor.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
		e.armor.prop:setLoc( template.armor.offset[1], template.armor.offset[2] );
		e.armor.prop:setPriority(12);
	end

	if template.turretShadow then
		e.turretShadow = Entity.new( Entity, game, layer, template.turretShadow, 0, 0, 0, parent.shadow, {template.turretShadow.offset[1], template.turretShadow.offset[2]}  );
		e.turretShadow.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, parent.shadow.prop, MOAIProp2D.TRANSFORM_TRAIT );
		e.turretShadow.prop:setLoc( template.turretShadow.offset[1], template.turretShadow.offset[2] );
		e.turretShadow.prop:setPriority(template.turretShadow or 0);
	end

	if template.barrel then
		e.barrel = {};

		for i=1,#template.barrel do
			e.barrel[i] = Entity.new( Entity, game, layer, template.barrel[i], 0, 0, 0, e, {template.barrel[i].offset[1], template.barrel[i].offset[2]}  );
			e.barrel[i].prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e.barrel[i].prop:setLoc( template.barrel[i].offset[1], template.barrel[i].offset[1] );
			e.barrel[i].prop:setPriority(template.barrel[i].priority or 9);
		end

		if template.muzzleVFX then
			e.muzzleVFX = {};

			for i=1,#template.muzzleVFX do
				e.muzzleVFX[i] = Entity.new( Entity, game, e.game.entityMgr.layers[template.muzzleVFX[i].layer or 1], template.muzzleVFX[i], 0, 0, 0, e, {template.muzzleVFX[i].offset[1], template.muzzleVFX[i].offset[2]}  );
				e.muzzleVFX[i].prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
				e.muzzleVFX[i].prop:setLoc( template.muzzleVFX[i].offset[1], template.muzzleVFX[i].offset[2] );
				e.muzzleVFX[i].prop:setPriority(template.muzzleVFX[i].priority or 20);
				e.muzzleVFX[i]:hide();
			end
		end

		if template.barrelShadow then
			e.barrelShadow = {};

			for i=1,#template.barrelShadow do
				e.barrelShadow[i] = Entity.new( Entity, game, e.game.entityMgr.layers[template.barrelShadow[i].layer or 1], template.barrelShadow[i], 0, 0, 0, e.turretShadow, {template.barrelShadow[i].offset[1], template.barrelShadow[i].offset[2]}  );
				e.barrelShadow[i].prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, e.turretShadow.prop, MOAIProp2D.TRANSFORM_TRAIT );
				e.barrelShadow[i].prop:setLoc( template.barrelShadow[i].offset[1], template.barrelShadow[i].offset[2] );
				e.barrelShadow[i].prop:setPriority(template.barrelShadow[i].priority or 1);
			end
		end
	end

	if template.rocketPod then
		e.rocketPod = {};

		for i=1,#template.rocketPod do
			e.rocketPod[i] = Entity.new( Entity, game, e.game.entityMgr.layers[template.rocketPod[i].layer or 3], template.rocketPod[i], 0, 0, 0, parent, {template.rocketPod[i].offset[1], template.rocketPod[i].offset[2]}  );
			e.rocketPod[i].prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, parent.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e.rocketPod[i].prop:setLoc( template.rocketPod[i].offset[1], template.rocketPod[i].offset[2] );
			e.rocketPod[i].prop:setPriority(template.rocketPod[i].priority or 9);

			if i%2 == 0 then
				e.rocketPod[i]:setScale(-1,1);
			end
		end

		if template.rocketPodShadow then
			e.rocketPodShadow = {};

			for i=1,#template.rocketPodShadow do
				e.rocketPodShadow[i] = Entity.new( Entity, game, layer, template.rocketPodShadow[i], 0, 0, 0, parent.shadow, {template.rocketPodShadow[i].offset[1], template.rocketPodShadow[i].offset[2]}  );
				e.rocketPodShadow[i].prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, parent.shadow.prop, MOAIProp2D.TRANSFORM_TRAIT );
				e.rocketPodShadow[i].prop:setLoc( template.rocketPodShadow[i].offset[1], template.rocketPodShadow[i].offset[2] );
				e.rocketPodShadow[i].prop:setPriority(template.rocketPodShadow[i].priority or 1);
			end
		end
	end

	if template.frontRotor then
		e.frontRotor = {}

		for i=1,#template.frontRotor do
			e.frontRotor[i] = Entity.new( Entity, game, e.game.entityMgr.layers[template.frontRotor[i].layer or 3], template.frontRotor[i], 0, 0, 0, parent, {template.frontRotor[i].offset[1], template.frontRotor[i].offset[2]}  );
			e.frontRotor[i].prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, parent.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e.frontRotor[i].prop:setLoc( template.frontRotor[i].offset[1], template.frontRotor[i].offset[2] );
			e.frontRotor[i].prop:setPriority(template.frontRotor[i].priority or 15);
		end
	end

	if template.rearRotor then
		e.rearRotor = {}

		for i=1,#template.rearRotor do
			e.rearRotor[i] = Entity.new( Entity, game, e.game.entityMgr.layers[template.rearRotor[i].layer or 3], template.rearRotor[i], 0, 0, 0, parent, {template.rearRotor[i].offset[1], template.rearRotor[i].offset[2]}  );
			e.rearRotor[i].prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, parent.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e.rearRotor[i].prop:setLoc( template.rearRotor[i].offset[1], template.rearRotor[i].offset[2] );
			e.rearRotor[i].prop:setPriority(template.rearRotor[i].priority or 15);
		end
	end

	if template.frontRotorWash then
		e.frontRotorWash = {}

		for i=1,#template.frontRotorWash do
			e.frontRotorWash[i] = Entity.new( Entity, game, e.game.entityMgr.layers[template.frontRotorWash[i].layer or 3], template.frontRotorWash[i], 0, 0, 0, parent, {template.frontRotorWash[i].offset[1], template.frontRotorWash[i].offset[2]}  );
			e.frontRotorWash[i].prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, parent.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e.frontRotorWash[i].prop:setLoc( template.frontRotorWash[i].offset[1], template.frontRotorWash[i].offset[2] );
			e.frontRotorWash[i].prop:setPriority(template.frontRotorWash[i].priority or 15);
		end
	end

	if template.rearRotorWash then
		e.rearRotorWash = {}

		for i=1,#template.rearRotorWash do
			e.rearRotorWash[i] = Entity.new( Entity, game, e.game.entityMgr.layers[template.rearRotorWash[i].layer or 3], template.rearRotorWash[i], 0, 0, 0, parent, {template.rearRotorWash[i].offset[1], template.rearRotorWash[i].offset[2]}  );
			e.rearRotorWash[i].prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, parent.prop, MOAIProp2D.TRANSFORM_TRAIT );
			e.rearRotorWash[i].prop:setLoc( template.rearRotorWash[i].offset[1], template.rearRotorWash[i].offset[2] );
			e.rearRotorWash[i].prop:setPriority(template.rearRotorWash[i].priority or 15);
		end
	end

	if template.glowVFX then
		e.glowVFX = Entity.new( Entity, game, layer, template.glowVFX, 0, 0, 0, e, {template.glowOffset.x or 0, template.glowOffset.y or 0}  );
		e.glowVFX.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
		e.glowVFX.prop:setLoc( template.glowOffset.x or 0, template.glowOffset.y or 0 );
		e.glowVFX.prop:setPriority(20);
		e.glowVFX:hide();

		if template.rocketGlowOffset then
			e.glowRocketVFX = {};

			for i=1,#template.rocketGlowOffset do
				e.glowRocketVFX[i] = Entity.new( Entity, game, e.game.entityMgr.layers[template.glowVFX.layer or template.layer or 3], template.glowVFX, 0, 0, 0, parent, {template.rocketGlowOffset[i].x or 0, template.rocketGlowOffset[i].y or 0}  );
				e.glowRocketVFX[i].prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, parent.prop, MOAIProp2D.TRANSFORM_TRAIT );
				e.glowRocketVFX[i].prop:setLoc( template.rocketGlowOffset[i].x or 0, template.rocketGlowOffset[i].y or 0 );
				e.glowRocketVFX[i]:hide();
			end
		end
	end

   if template then

		--call here because avatar doesnt get called yet
      e:reset( x, y, angle, parent, offset  );
   end

   return e;

end

----------------------------------------------------------------------

function TurretBossGunship:reset( x, y, angle, parent, offset )

   Base.reset( self, x, y, angle, parent, offset );

	parent:setRot(0);

	local x,y	= self.prop:modelToWorld();
	local X,Y	= self.parent:getLoc();

   self.state				= TURRET_SHOOT_SLEEP;
	self.weaponState		= TURRET_WEAPON_GUN;
	self.attackDelay		= 0;
	self.projDelay			= 0;
   self.bulletCount		= 1;
	self.rocketCount		= 1;
	self.rocketPodIndex	= 1;
	self.recoilGTime		= {0,0};
	self.recoilGOffset	= self.template.recoilGunOffset or 25;
	self.recoilRTime		= 0;
	self.recoilROffset	= self.template.recoilRocketOffset or 25;
   self.shootAngleMax	= self.template.shootAngleMax or 1;
   self.faceAndShoot		= self.template.faceAndShoot or true;
	self.type				= self.template.type or "";
   self.bulletDelta		= self.template.bulletDelta;
	self.rocketDelta		= self.template.rocketOffset;
	self.parentProj		= self.template.parentProj or false;
	self.projOffset		= self.template.projOffset or {{ x=0, y=0, angle=0}};
	self.glowGunState		= TURRET_GLOW_VFX_SLEEP;
	self.glowGunTime		= math.min( (self.template.shootDelay or 0)+(self.template.bulletDelay or 0), self.template.glowGTime or 0.25);
	self.glowRocketState	= TURRET_SHOOT_SLEEP;
	self.glowRocketTime	= math.min( (self.template.launchDelay or 0)+(self.template.rocketDelay or 0), self.template.glowRTime or 0.25);
	self.glowGScale		= self.template.glowGScale or {w=3,	h=3};
	self.glowRScale		= self.template.glowRScale or {w=3,	h=3};
	self.animState			= nil;
	self.idleK				= self.template.idleAnimSpeedK or 1;
	self.strafeK			= self.template.strafeAnimSpeedK or 1;
	self.strafeEaseK		= self.template.strafeEaseAnimSpeedK or 1;
	self.spinTime			= 0;
	self.deathSpin			= false;

	if self.parent.shadow then
		self.parent.shadow:setLoc(X+self.parent.template.shadow.offset[1], Y+self.parent.template.shadow.offset[2]);
		self.parent.shadow:setRot(0);
	end

	self.parent:setAustinToProp(true);

	EventMgr:rmvListener( self );

	EventMgr:addListener( self, self._setIdlePosition,			"boss-strafe-idle");
	EventMgr:addListener( self, self._setStrafePosition,		"boss-strafe-left");
	EventMgr:addListener( self, self._setStrafePosition,		"boss-strafe-right");
	EventMgr:addListener( self, self._setStrafeEasePosition, "boss-strafe-left-ease");
	EventMgr:addListener( self, self._setStrafeEasePosition, "boss-strafe-right-ease");
	EventMgr:addListener( self, self._splosionLeft,				"gunship-splosion-left" );
	EventMgr:addListener( self, self._splosionRight,			"gunship-splosion-right" );
	EventMgr:addListener( self, self.hideWing,					"gunship-wing-hide" );
   EventMgr:addListener( self, self.introSFX,					"boss-spawn" );
   EventMgr:addListener( self, self.extroSFX,					"boss-kill" );


end

----------------------------------------------------------------------

function TurretBossGunship:onFocus()

	if self.strafeAnimOffset then
		for i=1,#self.strafeAnimOffset do
			self.strafeAnimOffset[i]:start();
		end
	end

	if self.strafeAnimScale then
		self.strafeAnimScale:start();
	end

end

----------------------------------------------------------------------

function TurretBossGunship:onLoseFocus()

	if self.strafeAnimOffset then
		for i=1,#self.strafeAnimOffset do
			self.strafeAnimOffset[i]:pause();
		end
	end

	if self.strafeAnimScale then
		self.strafeAnimScale:pause();
	end

end

----------------------------------------------------------------------

function TurretBossGunship:update( dt, avatarSpeedY, burstOn, canShoot )

   --Base.update( self, dt, avatarSpeedY );

   self.burstOn = burstOn;

   local x,y    = self.prop:modelToWorld();
   local X,Y    = self.parent:getLoc();
   local a      = self.angle or self:getRot();
   local A      = math.clampAngle180(self.parent:getRot());

   self:updateTarget();
   self:updateVFXGun(dt);
   self:updateVFXRocket(dt);
   self:updateRecoilGun(dt);
   self:updateRecoilRocket(dt);
   self:updateRotorSpin(dt);
   self:updateRotorWash(dt);

   if self.state ~= TURRET_SHOOT_DISABLED and not self.deathSpin then
	  self:updateIdleAnim(dt);
	  self:updateStrafeAnim(dt);
   end

   if self.target and self.template.projectile and canShoot then
      self:updateFace( dt );
      self:updateShoot( dt, burstOn );
   end

   if self.parent.shadow then
	  self.parent.shadow:show();
	  self.parent.shadow:setLoc(X+self.parent.template.shadow.offset[1], Y+self.parent.template.shadow.offset[2]);
	  self.parent.shadow:setRot(A);
	  self.parent.shadow:setScale(self.parent.prop:getScl() or 1);
   end

   if self.turretShadow then
	  self.turretShadow:setRot(a);
   end

   if self.deathSpin then
	  self:updateDeathSpin(dt);
   end

end

----------------------------------------------------------------------

function TurretBossGunship:isTargetValid( target )

   if     (target)
      and (not target.deleteMe )
      and (not target:isDead() ) then

      if self.targetType == TURRET_SHOOT_TARGET_AI then

         return self:getLocY(true) < (target:getLocY()-target:getHeight()/2);
      else

         return self:getLocY(true) > ( self.game.avatar:getLocY() + TURRET_SHOOT_TARGET_AVATER_VALID_OFFSET );
      end
   end

   return false;

end

----------------------------------------------------------------------

function TurretBossGunship:updateTarget()

   --cant put in reset because self.game.avatar is not set yet
   if self.game.avatar == self.parent then
      self.targetType    = TURRET_SHOOT_TARGET_AI;
   else
      self.targetType    = TURRET_SHOOT_TARGET_AVATAR;
   end

   if self.targetType == TURRET_SHOOT_TARGET_AVATAR then

      self.target = self.game.avatar;
   else

      if #self.game.entityMgr.vehicles == 0 then

         self.target = nil;

      elseif self.target == nil then

         local vehicles    = self.game.entityMgr.vehicles;
         local best        = 1;
         local bestScore   = 1000000000;
         for i=1,#vehicles do

            local tgt   = vehicles[i];
            if self:isTargetValid( tgt ) then

               local tgtY  = tgt:getLocY(true);
               local score = 10*tgt:getHealthMax()

               - tgtY;
               if score < bestScore then

                  bestScore = score;
                  best      = i;

               end
            end
         end

         self.target = self.game.entityMgr.vehicles[best];

      elseif not self:isTargetValid( self.target ) then

         self.target = nil;

      end
   end
end

----------------------------------------------------------------------

function TurretBossGunship:isAngleOk( a )

   local limit = self.angleLimit;

   if limit then

      if limit.min < 0 then
         a = math.clampAngle180( a );
      else
         a = math.clampAngle360( a );
      end

      return a >= limit.min and a <= limit.max;
   else

      return true;
   end
end

----------------------------------------------------------------------

function TurretBossGunship:updateFace( dt )

   if       self.game
      and   self.target
		and	self.state ~= TURRET_SHOOT_DISABLED
      and   ( self.faceAndShoot or self.state ~= TURRET_SHOOT_SHOOT ) then

      local x, y   = self:getLoc(true);
      local tx, ty = self.target:getLoc();
      local dx, dy = tx-x, ty-y-self.target:getHeight()/4;

      --Calculate around Y
      local rad   = -math.atan2(dx,dy);
		local pr		= self.parent:getRot();
      local a     = ( rad*180/math.pi );
      local da    = math.clampAngle180( a-self.angle-math.clampAngle180(pr) );

      if self:isAngleOk( a ) then

         local da2   = math.clamp(da, -self.maxTurnRate*dt,self.maxTurnRate*dt);

         self.angle  = self.angle+da2;
         self.da     = da2;

         self:setRot( self.angle );

      else

         self.da = da;
      end

		if	self.template.bodyFaceShoot then

			local fa = math.clampAngle180(a);
			local tk	= 1;
			local d	= 0;

			pr = math.clampAngle180(pr);

			if fa > 0 then
				fa = fa - 180;
			elseif fa < 0 then
				fa = 180 + fa;
			else
				fa = 0;
			end

			if fa > pr then
				d	= fa - pr;

				if d <= self.template.bodyTargetCone then
					return false;
				elseif d >= self.template.bodyAngleDelta then
					tk = self.template.bodyTurnRateK;
				end

				pr = math.clamp(pr + self.template.bodyTurnRate*tk*dt, self.template.bodyAngleLimit[1], self.template.bodyAngleLimit[2]);
			elseif fa < pr then
				d	= pr - fa;

				if d <= self.template.bodyTargetCone then
					return false;
				elseif d >= self.template.bodyAngleDelta then
					tk = self.template.bodyTurnRateK;
				end

				pr = math.clamp(pr - self.template.bodyTurnRate*tk*dt, self.template.bodyAngleLimit[1], self.template.bodyAngleLimit[2]);
			else

				return false;
			end

			self.parent:setRot(pr);
		end
   end

end

----------------------------------------------------------------------

function TurretBossGunship:disable()

   self.state			= TURRET_SHOOT_DISABLED;
	self.weaponState	= TURRET_WEAPON_GUN;
   self.attackDelay	= self.template.shootDelay or 0;
   self.projDelay		= self.template.bulletDelay or 0;

end

----------------------------------------------------------------------

function TurretBossGunship:enable()

   if self.state == TURRET_SHOOT_DISABLED then
      self.state = TURRET_SHOOT_AIM;
   end

end

----------------------------------------------------------------------

function TurretBossGunship:introSFX()

   EventMgr:postEvent( { sounds={ loop={"GunshipRotor",}, }, name="Gunship Start",}, "add-sfx" );

end

----------------------------------------------------------------------

function TurretBossGunship:extroSFX()

   EventMgr:postEvent( { sounds={ loop={"GunshipRotor",}, }, name="Gunship Stop",}, "loop-stop" );

end

----------------------------------------------------------------------

function TurretBossGunship:calcBulletLoc()

   local x,y		= self:getLoc(true);
   local dX,dY		= 0,0;

   if self.bulletDelta then

      local delta = self.bulletDelta[self.bulletCount+1] or {};

      dX = delta.x or 0;
      dY = delta.y or 0;
   end

   return x+dX, y+dY;

end

----------------------------------------------------------------------

function TurretBossGunship:calcParentBulletOffset()

	local offset = { x=0,	y=0,	angle=0 };

   if self.projOffset then
		offset = self.projOffset[self.bulletCount+1] or self.projOffset[1] or offset;
   end

   return offset;

end

----------------------------------------------------------------------

function TurretBossGunship:calcRocketLoc( index )

   local x,  y = self:getLoc(true);
   local dX,dY = 0,0;

   if self.rocketDelta then

		local delta = self.rocketDelta[math.min(index, #self.rocketDelta)] or {};

      dX = delta.x or 0;
      dY = delta.y or 0;
   end

   return x+dX, y+dY;

end

----------------------------------------------------------------------

function TurretBossGunship:updateShoot( dt, burstOn  )

   if self.state == TURRET_SHOOT_DISABLED then

   elseif self.state == TURRET_SHOOT_SLEEP then

      if (self:getLocY(true) + TURRET_SHOOT_ON_SCREEN_DELTA) < ( self.game.boundary.max.y ) then
         self.state = TURRET_SHOOT_AIM;
      end

   elseif self.state == TURRET_SHOOT_AIM then

      self.attackDelay = self.attackDelay - dt;

      if       self.attackDelay      < 0
			and	self.da
         and   math.abs(self.da)    < self.shootAngleMax
         and   (not self.template.shootPosMax  or self:getLocY(true)   < self.template.shootPosMax) then

         self.state			= TURRET_SHOOT_SHOOT;
         self.bulletCount	= 1;
			self.rocketCount	= 1;

			if self.weaponState == TURRET_WEAPON_GUN then
				self.attackDelay	= self.template.launchDelay;
			else
				self.attackDelay	= self.template.shootDelay;
			end
      end

   else

      if self.target == nil then
         self.state = TURRET_SHOOT_AIM;
      else

         self.projDelay = self.projDelay-dt;

         if self.projDelay < 0.001 then
				if self.weaponState == TURRET_WEAPON_GUN then
					self:shoot();
				else
					self:launch();
				end
         end
      end
   end
end

----------------------------------------------------------------------

function TurretBossGunship:shoot()

	self.projDelay = self.template.bulletDelay;

	if self.bulletCount < self.template.bulletCount+1 then
		local x,y = self:calcBulletLoc();
		local p;

		if self.parentProj then
			local poffset = self:calcParentBulletOffset();

			p = self.game.entityMgr:spawnProjectile( self.template.projectile, poffset.x, poffset.y, poffset.angle, self.parent, self.target, self, poffset );
		else
			p = self.game.entityMgr:spawnProjectile( self.template.projectile, x, y, self.angle, self.parent, self.target );
		end

		p.speedBurst		= self.burstOn;
		p.damageAvatar		= (p.damageAvatar or 0) * (AbTestMgr:getBossDamageScale(BOSS_INDEX) or 1);

		EventMgr:postEvent( {sounds = {start={	"CannonFire",
															},}, name="turret shoot"}, "add-sfx" );
	end

	self.bulletCount = self.bulletCount + 1;

	if self.bulletCount >= self.template.bulletCount+1 then
		self.state			= TURRET_SHOOT_AIM;
		self.weaponState	= TURRET_WEAPON_ROCKET;
		self.projDelay		= self.template.rocketDelay;
	end

end

----------------------------------------------------------------------

function TurretBossGunship:launch()

	self.projDelay = self.template.rocketDelay;

	if self.rocketCount < self.template.rocketCount+1 then
		local x,y	= self:calcRocketLoc(self.rocketCount);
		local a		= math.clampAngle180(self.parent:getRot());
		local p;

		p = self.game.entityMgr:spawnProjectile( self.template.rocket, x, y, 180+a, self.parent, self.game.avatar );

		p.speedBurst		= self.burstOn;
		p.damageAvatar		= (p.damageAvatar or 0) * (AbTestMgr:getBossDamageScale(BOSS_INDEX) or 1);

		EventMgr:postEvent( {sounds = {start={	"GunshipMissile", }, }, name="turret shoot"}, "add-sfx" );
	end

	self.rocketCount = self.rocketCount + 1;

	if self.rocketCount >= self.template.rocketCount+1 then
		self.state			= TURRET_SHOOT_AIM;
		self.weaponState	= TURRET_WEAPON_GUN;
		self.projDelay		= self.template.bulletDelay;
	end

end

----------------------------------------------------------------------

function TurretBossGunship:updateVFXGun( dt )

	if self.glowVFX then

		local countdown;
		local scale			= 0;

		if	self.state and self.state == TURRET_SHOOT_AIM then
			countdown = self.attackDelay + self.projDelay;
		else
			countdown = self.projDelay;
		end

		if		(self.state and self.state == TURRET_SHOOT_DISABLED or self.state == TURRET_SHOOT_GUN_COOLDOWN)
			or	(self.weaponState and self.weaponState	== TURRET_WEAPON_ROCKET)
			or	not self.attackDelay
			or not self.projDelay
			or	not self.glowGScale
			or	countdown >= self.glowGunTime then

			self.glowVFX:hide();

			if self.glowGunState then
				if self.glowGunState == TURRET_GLOW_VFX_STOP then
					self.glowGunState = TURRET_GLOW_VFX_SLEEP;
				elseif self.glowGunState == TURRET_GLOW_VFX_START then
					self.glowGunState = TURRET_GLOW_VFX_STOP;
				else
					self.glowGunState = TURRET_GLOW_VFX_SLEEP;
				end
			end
		else
			if self.bulletCount <= 1 then
				scale = math.min((self.glowGunTime - countdown)/self.glowGunTime, 1);
				scale = math.max(0, scale);

				self.glowVFX:show();
				self.glowVFX:setScale(scale*self.glowGScale.w, scale*self.glowGScale.h);

				if self.glowState then
					if self.glowGunState ~= TURRET_GLOW_VFX_START then
						self.glowGunState = TURRET_GLOW_VFX_START;
					end
				end
			else
				self.glowVFX:hide();
			end

			local barrelIndex;

			if self.bulletCount%2 == 0 then
				barrelIndex = 2;
			else
				barrelIndex = 1;
			end

			if barrelIndex == 1 and countdown <= 0.10 and self.recoilGTime[1] <= 0 then
				self.recoilGTime[1] = self.template.recoilGunTime + self.template.recoilGunReturnTime;
			elseif barrelIndex == 2 and countdown <= 0.10 and self.recoilGTime[2] <= 0 then
				self.recoilGTime[2] = self.template.recoilGunTime + self.template.recoilGunReturnTime;
			end
		end
	end

end

----------------------------------------------------------------------

function TurretBossGunship:updateVFXRocket( dt )

	if self.glowRocketVFX then

		local countdown;
		local scale			= 0;

		if	self.state and self.state == TURRET_SHOOT_AIM then
			countdown = self.attackDelay + self.projDelay;
		else
			countdown = self.projDelay;
		end

		if		(self.state and self.state == TURRET_SHOOT_DISABLED or self.state == TURRET_SHOOT_GUN_COOLDOWN)
			or	(self.weaponState and self.weaponState	== TURRET_WEAPON_GUN)
			or	not self.attackDelay
			or not self.projDelay
			or	not self.glowRScale
			or	countdown >= self.glowRocketTime then

			for i=1,#self.glowRocketVFX do
				self.glowRocketVFX[i]:hide();
			end

			if self.glowRocketState then
				if self.glowRocketState == TURRET_GLOW_VFX_STOP then
					self.glowRocketState = TURRET_GLOW_VFX_SLEEP;
				elseif self.glowRocketState == TURRET_GLOW_VFX_START then
					self.glowRocketState = TURRET_GLOW_VFX_STOP;
				else
					self.glowRocketState = TURRET_GLOW_VFX_SLEEP;
				end
			end
		else
			scale = math.min((self.glowRocketTime - countdown)/self.glowRocketTime, 1);
			scale = math.max(0, scale);

			if self.rocketCount%2 == 0 then
				self.glowRocketVFX[1]:hide();
				self.glowRocketVFX[2]:show();
				self.glowRocketVFX[2]:setScale(scale*self.glowRScale.w, scale*self.glowRScale.h);
			else
				self.glowRocketVFX[1]:show();
				self.glowRocketVFX[2]:hide();
				self.glowRocketVFX[1]:setScale(scale*self.glowRScale.w, scale*self.glowRScale.h);
			end

			if self.glowRocketState then
				if self.glowRocketState ~= TURRET_GLOW_VFX_START then
					self.glowRocketState = TURRET_GLOW_VFX_START;
				end
			end

			if countdown <= 0.15 and self.recoilRTime <= 0 then
				self.recoilRTime = self.template.recoilRocketTime + self.template.recoilRocketReturnTime;

				if self.rocketCount%2 == 0 then
					self.rocketPodIndex = 2;
				else
					self.rocketPodIndex = 1;
				end
			end
		end
	end

end

----------------------------------------------------------------------

function TurretBossGunship:updateRotorSpin( dt )

	if not self.rotorTime then
		self.rotorTime = 0;
	elseif self.rotorTime >= ROTOR_RATE then
		self.rotorTime = 0;
	end

	self.rotorTime = self.rotorTime + dt;

	local angle = 360*(self.rotorTime/ROTOR_RATE);

	if self.frontRotor then
		for i=1,#self.frontRotor do
			local direction = -1;

			if i > 1 then
				direction = 1;
			end

			self:_spinRotor(self.frontRotor[i], direction*angle);
		end
	end

	if self.rearRotor then
		for i=1,#self.rearRotor do
			local direction = -1;

			if i > 1 then
				direction = 1;
			end

			self:_spinRotor(self.rearRotor[i], direction*angle);
		end
	end

	if self.frontRotorWash then
		for i=1,#self.frontRotorWash do
			local direction = 1;

			if i > 1 then
				direction = -1;
			end

			self:_spinRotor(self.frontRotorWash[i], direction*angle);
		end
	end

	if self.rearRotorWash then
		for i=1,#self.rearRotorWash do
			local direction = 1;

			if i > 1 then
				direction = -1;
			end

			self:_spinRotor(self.rearRotorWash[i], direction*angle);
		end
	end

end

----------------------------------------------------------------------

function TurretBossGunship:_spinRotor( rotor, angle )

	rotor:setRot(angle);

end

----------------------------------------------------------------------

function TurretBossGunship:updateRotorWash( dt )

	if not self.rotorWashTime then
		self.rotorWashTime = 0;
	elseif self.rotorWashTime >= ROTOR_WASH_RATE or self.animState~=ANIM_STATE_IDLE then
		self.rotorWashTime = ROTOR_WASH_RATE/4;
	end

	if not self.animState then
		return false;
	end

	self.rotorWashTime = self.rotorWashTime + dt;

	local scale = 1;

	if self.rotorWashTime > ROTOR_WASH_RATE/2 then
		scale = scale + ROTOR_WASH_ADD_SCALE*(ROTOR_WASH_RATE - self.rotorWashTime/(ROTOR_WASH_RATE/2));
	else
		scale = scale + ROTOR_WASH_ADD_SCALE*(self.rotorWashTime/(ROTOR_WASH_RATE/2));
	end

	if self.frontRotorWash then
		for i=1,#self.frontRotorWash do
			self.frontRotorWash[i]:setScale(scale, scale);
		end
	end

	if self.rearRotorWash then
		for i=1,#self.rearRotorWash do
			self.rearRotorWash[i]:setScale(scale, scale);
		end
	end

end

----------------------------------------------------------------------

function TurretBossGunship:updateRecoilGun( dt )

	if self.barrel then

		local count = 0;

		for i=1,#self.barrel do

			if	self.recoilGTime[i] <= 0 then
				count = count + 1

				self.barrel[i]:setLoc(self.template.barrel[i].offset[1], self.template.barrel[i].offset[2]);

				if self.muzzleVFX then
					self.muzzleVFX[i]:hide();
				end

				if self.barrelShadow then
					self.barrelShadow[i]:setLoc(self.template.barrelShadow[i].offset[1], self.template.barrelShadow[i].offset[2]);
				end
			end

			if count == #self.barrel then
				return false;
			end
		end

		for i=1,#self.barrel do

			self.recoilGTime[i] = math.max(self.recoilGTime[i] - dt, 0);

			if self.muzzleVFX then
				self.muzzleVFX[i]:show();
			end

			if self.recoilGTime[i] > self.template.recoilGunReturnTime then
				local t	= self.recoilGTime[i] - self.template.recoilGunReturnTime;
				local y	= self.recoilGOffset * math.clamp(self.template.recoilGunTime - t/self.template.recoilGunTime, 0, 1);

				y = self.template.barrel[i].offset[2] - y;

				self.barrel[i]:setLoc(self.template.barrel[i].offset[1], y);

				if self.barrelShadow then
					self.barrelShadow[i]:setLoc(self.template.barrelShadow[i].offset[1], y);
				end
			else
				local y	= self.recoilGOffset * math.clamp(self.recoilGTime[i]/self.template.recoilGunReturnTime, 0, 1);

				y = self.template.barrel[i].offset[2] - y;

				self.barrel[i]:setLoc(self.template.barrel[i].offset[1], y);

				if self.muzzleVFX then
					self.muzzleVFX[i]:hide();
				end

				if self.barrelShadow then
					self.barrelShadow[i]:setLoc(self.template.barrelShadow[i].offset[1], y);
				end
			end
		end
	end

end

----------------------------------------------------------------------

function TurretBossGunship:updateRecoilRocket( dt )

	if self.rocketPod then

		if	self.recoilRTime <= 0 then

			for i=1,#self.rocketPod do
				self.rocketPod[i]:setLoc(self.template.rocketPod[i].offset[1], self.template.rocketPod[i].offset[2]);
			end

			return true;
		end

		if self.rocketPodIndex == 1 then
			self.rocketPod[2]:setLoc(self.template.rocketPod[2].offset[1], self.template.rocketPod[2].offset[2]);
		else
			self.rocketPod[1]:setLoc(self.template.rocketPod[1].offset[1], self.template.rocketPod[1].offset[2]);
		end

		self.recoilRTime = self.recoilRTime - dt;

		if self.recoilRTime > self.template.recoilRocketReturnTime then
			local t	= self.recoilRTime - self.template.recoilRocketReturnTime;
			local y	= self.recoilGOffset * math.clamp(self.template.recoilRocketTime - t/self.template.recoilRocketTime, 0, 1);

			y = self.template.rocketPod[self.rocketPodIndex].offset[2] + y;

			self.rocketPod[self.rocketPodIndex]:setLoc(self.template.rocketPod[self.rocketPodIndex].offset[1], y);
		else
			local y	= self.recoilGOffset * math.clamp(self.recoilRTime/self.template.recoilRocketReturnTime, 0, 1);

			y = self.template.rocketPod[self.rocketPodIndex].offset[2] + y;

			self.rocketPod[self.rocketPodIndex]:setLoc(self.template.rocketPod[self.rocketPodIndex].offset[1], y);
		end
	end

end

----------------------------------------------------------------------

function TurretBossGunship:updateIdleAnim( dt )

	if self.animState and self.animState == ANIM_STATE_IDLE then
		self:_updateIdleRotation(dt);
		self:_updateIdleOffset(dt);
		self:_updateIdleScale(dt);
	end

end

----------------------------------------------------------------------

function TurretBossGunship:_setIdlePosition( event )

	if	self.animState == nil or self.animState == ANIM_STATE_STRAFE_EASE then

		if not self.idleAnimRoot then
			self.idleAnimRoot		= {event.X, event.Y};
		else
			self.idleAnimRoot		= {event.X, self.idleAnimRoot[2]};
		end

		if self.strafeAnimOffset then
			for i=1,#self.strafeAnimOffset do
				self.strafeAnimOffset[i]:stop();
			end
		end

		if self.strafeAnimScale then
			self.strafeAnimScale:stop();
		end

		self.animState					= ANIM_STATE_IDLE;
		self.idleAnimAngle			= nil;
		self.idleAnimRotIndex		= 1;
		self.idleAnimRotTime			= 0;
		self.idleAnimOffsetIndex	= 1;
		self.idleAnimOffsetTime		= 0;
		self.idleAnimScaleIndex		= 1;
		self.idleAnimScaleTime		= 0;

		self.parent:setAustinToProp(false);
	end

end

----------------------------------------------------------------------

function TurretBossGunship:_updateIdleRotation( dt )

	if not self.template.idleAnim.timeRot then
		return false;
	end

	if self.idleAnimRotTime*self.idleK > self.template.idleAnim.timeRot[self.idleAnimRotIndex] then
		self.idleAnimRotTime = 0;

		if self.idleAnimRotIndex >= #self.template.idleAnim.timeRot then
			self.idleAnimRotIndex = 1;
		else
			self.idleAnimRotIndex = self.idleAnimRotIndex + 1;
		end
	end

	self.idleAnimRotTime = self.idleAnimRotTime + dt;

	self.idleAnimAngle	= self.template.idleAnim.angle[self.idleAnimRotIndex]

--[[
	local fa = self.template.idleAnim.angle[self.idleAnimRotIndex-1] or self.strafeAnimAngle or 0;

	self.idleAnimAngle = math.smoothLerp(fa, self.template.idleAnim.angle[self.idleAnimRotIndex], (self.idleAnimRotTime*self.idleK)/self.template.idleAnim.timeRot[self.idleAnimRotIndex]);

	self.parent:setRot(self.idleAnimAngle);
]]

end

----------------------------------------------------------------------

function TurretBossGunship:_updateIdleOffset( dt )

	if not self.template.idleAnim.timeOffset then
		return false;
	end

	if self.idleAnimOffsetTime*self.idleK > self.template.idleAnim.timeOffset[self.idleAnimOffsetIndex] then
		self.idleAnimOffsetTime = 0;

		if self.idleAnimOffsetIndex >= #self.template.idleAnim.timeOffset then
			self.idleAnimOffsetIndex = 1;
		else
			self.idleAnimOffsetIndex = self.idleAnimOffsetIndex + 1;
		end
	end

	if self.idleAnimOffsetTime == 0 then

		if not self.strafeAnimOffset then
			self.strafeAnimOffset = {};

			for i=1,#self.template.idleAnim.timeOffset do
				self.strafeAnimOffset[i] = nil;
			end
		end

		self.strafeAnimOffset[self.idleAnimOffsetIndex] = self.parent.prop:seekLoc(
																												self.idleAnimRoot[1]+self.template.idleAnim.offset[self.idleAnimOffsetIndex][1],
																												self.idleAnimRoot[2]+self.template.idleAnim.offset[self.idleAnimOffsetIndex][2],
																												self.template.idleAnim.timeOffset[self.idleAnimOffsetIndex]*2.0*(1/self.idleK),
																												MOAIEaseType.SOFT_SMOOTH	);
	end

	self.idleAnimOffsetTime = self.idleAnimOffsetTime + dt;

end

----------------------------------------------------------------------

function TurretBossGunship:_updateIdleScale( dt )

	if not self.template.idleAnim.timeScale then
		return false;
	end

	if self.idleAnimScaleTime*self.idleK > self.template.idleAnim.timeScale[self.idleAnimScaleIndex] then
		self.idleAnimScaleTime = 0;

		if self.idleAnimScaleIndex >= #self.template.idleAnim.timeScale then
			self.idleAnimScaleIndex = 1;
		else
			self.idleAnimScaleIndex = self.idleAnimScaleIndex + 1;
		end
	end

	if self.idleAnimScaleTime == 0 then

		if self.strafeAnimScale then
			self.strafeAnimScale:stop();
		end

		self.strafeAnimScale = self.parent.prop:seekScl(
																			self.template.idleAnim.scale[self.idleAnimScaleIndex][1],
																			self.template.idleAnim.scale[self.idleAnimScaleIndex][2],
																			self.template.idleAnim.timeScale[self.idleAnimScaleIndex]*(1/self.idleK),
																			MOAIEaseType.LINEAR	);
	end

	self.idleAnimScaleTime = self.idleAnimScaleTime + dt;

end

----------------------------------------------------------------------

function TurretBossGunship:updateStrafeAnim( dt )

	if self.animState and self.animState ~= ANIM_STATE_IDLE then
		self:_updateStrafeRotation(dt);
		self:_updateStrafeOffset(dt);
		self:_updateStrafeScale(dt);
	end

end

----------------------------------------------------------------------

function TurretBossGunship:_setStrafePosition( event )

	if	self.animState == nil or self.animState == ANIM_STATE_IDLE then

		if not self.idleAnimRoot then
			self.idleAnimRoot		= {event.X, event.Y};
		else
			self.idleAnimRoot		= {event.X, self.idleAnimRoot[2]};
		end

		if event.direction == "left" then
			self.strafeAnim = self.template.strafeLAnim;
		else
			self.strafeAnim = self.template.strafeRAnim;
		end

		if self.strafeAnimOffset then
			for i=1,#self.strafeAnimOffset do
				self.strafeAnimOffset[i]:stop();
			end
		end

		if self.strafeAnimScale then
			self.strafeAnimScale:stop();
		end

		self.animState					= ANIM_STATE_STRAFE;
		self.strafeAnimRotIndex		= 1;
		self.strafeAnimRotTime		= 0;
		self.strafeAnimOffsetIndex	= 1;
		self.strafeAnimOffsetTime	= 0;
		self.strafeAnimScaleIndex	= 1;
		self.strafeAnimScaleTime	= 0;
		self.strafeAnimK				= self.strafeK;

		self.parent:setAustinToProp(true);
	end

end

----------------------------------------------------------------------

function TurretBossGunship:_setStrafeEasePosition( event )

	if	(self.animState == nil or self.animState == ANIM_STATE_STRAFE) then

		if not self.idleAnimRoot then
			self.idleAnimRoot		= {event.X, event.Y};
		else
			self.idleAnimRoot		= {event.X, self.idleAnimRoot[2]};
		end

		self.idleAnimAngle = math.clampAngle180(self.parent:getRot());

		if self.idleAnimAngle > 0 then
			self.strafeAnim = self.template.strafeLEaseAnim;
		elseif self.idleAnimAngle < 0 then
			self.strafeAnim = self.template.strafeREaseAnim;
		elseif event.direction == "left" then
			self.strafeAnim = self.template.strafeLEaseAnim;
		else
			self.strafeAnim = self.template.strafeREaseAnim;
		end

		if self.strafeAnimOffset then
			for i=1,#self.strafeAnimOffset do
				self.strafeAnimOffset[i]:stop();
			end
		end

		if self.strafeAnimScale then
			self.strafeAnimScale:stop();
		end

		self.animState					= ANIM_STATE_STRAFE_EASE;
		self.strafeAnimRotIndex		= 1;
		self.strafeAnimRotTime		= 0;
		self.strafeAnimOffsetIndex	= 1;
		self.strafeAnimOffsetTime	= 0;
		self.strafeAnimScaleIndex	= 1;
		self.strafeAnimScaleTime	= 0;
		self.strafeAnimK				= self.strafeEaseK;

	end

end

----------------------------------------------------------------------

function TurretBossGunship:_updateStrafeRotation( dt )

	if not self.strafeAnim.timeRot
		or (self.strafeAnim.timeRot and not self.strafeAnim.timeRot[self.strafeAnimRotIndex]) then
		return false;
	end

	if self.strafeAnimRotTime*self.strafeAnimK > self.strafeAnim.timeRot[self.strafeAnimRotIndex] then
		self.strafeAnimRotTime	= 0;
		self.strafeAnimRotIndex	= self.strafeAnimRotIndex + 1;

		if not self.strafeAnim.timeRot[self.strafeAnimRotIndex] then

			if self.animState == ANIM_STATE_STRAFE_EASE then
				self.animState = nil;
			end

			return false;
		end
	end

	self.strafeAnimRotTime	= self.strafeAnimRotTime + dt;

--[[
	local offset	= self.idleAnimAngle or math.clampAngle180(self.parent:getRot()) or 0;
	local fa			= offset;

	if self.strafeAnim.angle[self.strafeAnimRotIndex-1] then
		fa = self.strafeAnim.angle[self.strafeAnimRotIndex-1] + offset;
	end

	self.strafeAnimAngle = math.smoothLerp(fa, offset+self.strafeAnim.angle[self.strafeAnimRotIndex], (self.strafeAnimRotTime*self.strafeAnimK)/(self.strafeAnim.timeRot[self.strafeAnimRotIndex]*1.2));

	self.parent:setRot(self.strafeAnimAngle);
]]

end

----------------------------------------------------------------------

function TurretBossGunship:_updateStrafeOffset( dt )

	if self.animState == ANIM_STATE_STRAFE	then

		local x,y = self.parent:getLoc();

		if y < self.idleAnimRoot[2]+10 then
			self.parent:setSpeedY(15);
		else
			self.parent:setSpeedY(0);
		end
	else
		self.parent:setSpeedY(0);
	end

	if not self.strafeAnim.timeOffset
		or (self.strafeAnim.timeOffset and not self.strafeAnim.timeOffset[self.strafeAnimOffsetIndex]) then
		return false;
	end

	if self.strafeAnimOffsetTime*self.strafeAnimK > self.strafeAnim.timeOffset[self.strafeAnimOffsetIndex] then
		self.strafeAnimOffsetTime = 0;

		self.strafeAnimOffsetIndex = self.strafeAnimOffsetIndex + 1;

		if not self.strafeAnim.timeOffset[self.strafeAnimOffsetIndex] then

			if self.animState == ANIM_STATE_STRAFE_EASE then
				self.animState = nil;
			end

			return false;
		end
	end

	if self.strafeAnimOffsetTime == 0 then

		if not self.strafeAnimOffset then
			self.strafeAnimOffset = {};

			for i=1,#self.strafeAnim.timeOffset do
				self.strafeAnimOffset[i] = nil;
			end
		end

		self.strafeAnimOffset[self.strafeAnimOffsetIndex] = self.parent.prop:seekLoc(
																												self.idleAnimRoot[1]+self.strafeAnim.offset[self.strafeAnimOffsetIndex][1],
																												self.idleAnimRoot[2]+self.strafeAnim.offset[self.strafeAnimOffsetIndex][2],
																												self.strafeAnim.timeOffset[self.strafeAnimOffsetIndex]*2.0*(1/self.strafeAnimK),
																												MOAIEaseType.SOFT_SMOOTH	);
	end

	self.strafeAnimOffsetTime = self.strafeAnimOffsetTime + dt;

end

----------------------------------------------------------------------

function TurretBossGunship:_updateStrafeScale( dt )

	if not self.strafeAnim.timeScale
		or (self.strafeAnim.timeScale and not self.strafeAnim.timeScale[self.strafeAnimScaleIndex]) then
		return false;
	end

	if self.strafeAnimScaleTime > self.strafeAnim.timeScale[self.strafeAnimScaleIndex]*self.strafeAnimK then
		self.strafeAnimScaleTime	= 0;
		self.strafeAnimScaleIndex	= self.strafeAnimScaleIndex + 1;

		if not self.strafeAnim.timeScale[self.strafeAnimScaleIndex] then

			if self.animState == ANIM_STATE_STRAFE_EASE then
				self.animState = nil;
			end

			return false;
		end
	end

	if self.strafeAnimScaleTime == 0 then

		if self.strafeAnimScale then
			self.strafeAnimScale:stop();
		end

		self.strafeAnimScale = self.parent.prop:seekScl(
																			self.strafeAnim.scale[self.strafeAnimScaleIndex][1],
																			self.strafeAnim.scale[self.strafeAnimScaleIndex][2],
																			self.strafeAnim.timeScale[self.strafeAnimScaleIndex]*(1/self.strafeAnimK),
																			MOAIEaseType.LINEAR	);
	end

	self.strafeAnimScaleTime = self.strafeAnimScaleTime + dt;

end

----------------------------------------------------------------------

function TurretBossGunship:_splosionLeft()

	self.parent:setAustinToProp(true);
	self.parent:addImpulse(-5, 3);

	self.deathSpin = true;

end

----------------------------------------------------------------------

function TurretBossGunship:_splosionRight()

	self.parent:setAustinToProp(true);
	self.parent:addImpulse(5, -3);

	self.deathSpin = true;

end

----------------------------------------------------------------------

function TurretBossGunship:hideWing()

	if self.armor then
		self.armor:hide();
	end

	if self.frontRotor then
		for i=1,#self.frontRotor do
			self.frontRotor[i]:hide();
		end
	end

	if self.frontRotorWash then
		for i=1,#self.frontRotorWash do
			self.frontRotorWash[i]:hide();
		end
	end

	if self.parent.visElement then
		for i=1,#self.parent.visElement do
			self.parent.visElement[i]:hide();
		end
	end

	if self.parent.armor then
		self.parent.armor:hide();
	end

	self.deathSpin = true;

end

----------------------------------------------------------------------

function TurretBossGunship:updateDeathSpin( dt )

	local a		= math.clampAngle360(self.parent:getRot());

	self.spinTime = self.spinTime + dt;

	local rate	= math.smootherLerp(0, 720, self.spinTime/2.1);

	a = math.clampAngle360(a + rate*dt);

	self.parent:setRot(a);

	if self.parent:getSpeedY() <= 0 then

		local x,y	= self.parent:getLoc();
		local lane	= self.game:getClosestLaneTo(x);

		self.parent:setAustinToProp(true);
		self.parent:setSpeedY(30);

		if lane <= 3 then
			self.parent:setSpeedX(90);
		else
			self.parent:setSpeedX(-90);
		end
	end

end

----------------------------------------------------------------------

function TurretBossGunship:delete()

	Base.delete(self);

	if self.armor then
		self.armor:delete();
	end

	if self.turretShadow then
		self.turretShadow:delete();
	end

	if self.barrel then
		for i=1,#self.barrel do
			self.barrel[i]:delete();
		end
	end

	if self.rocketPod then
		for i=1,#self.rocketPod do
			self.rocketPod[i]:delete();
		end
	end

	if self.barrelShadow then
		for i=1,#self.barrelShadow do
			self.barrelShadow[i]:delete();
		end
	end

	if self.frontRotor then
		for i=1,#self.frontRotor do
			self.frontRotor[i]:delete();
		end
	end

	if self.rearRotor then
		for i=1,#self.rearRotor do
			self.rearRotor[i]:delete();
		end
	end

	if self.frontRotorWash then
		for i=1,#self.frontRotorWash do
			self.frontRotorWash[i]:delete();
		end
	end

	if self.rearRotorWash then
		for i=1,#self.rearRotorWash do
			self.rearRotorWash[i]:delete();
		end
	end

	if self.muzzleVFX then
		for i=1,#self.muzzleVFX do
			self.muzzleVFX[i]:delete();
		end
	end

	if self.glowVFX then
		self.glowVFX:delete();
	end

	if self.glowRocketVFX then
		for i=1,#self.glowRocketVFX do
			self.glowRocketVFX[i]:delete();
		end
	end

end

----------------------------------------------------------------------

function TurretBossGunship:hide()

	Base.hide(self);

	if self.armor then
		self.armor:hide();
	end

	if self.turretShadow then
		self.turretShadow:hide();
	end

	if self.barrel then
		for i=1,#self.barrel do
			self.barrel[i]:hide();
		end
	end

	if self.rocketPod then
		for i=1,#self.rocketPod do
			self.rocketPod[i]:hide();
		end
	end

	if self.barrelShadow then
		for i=1,#self.barrelShadow do
			self.barrelShadow[i]:hide();
		end
	end

	if self.frontRotor then
		for i=1,#self.frontRotor do
			self.frontRotor[i]:hide();
		end
	end

	if self.rearRotor then
		for i=1,#self.rearRotor do
			self.rearRotor[i]:hide();
		end
	end

	if self.frontRotorWash then
		for i=1,#self.frontRotorWash do
			self.frontRotorWash[i]:hide();
		end
	end

	if self.rearRotorWash then
		for i=1,#self.rearRotorWash do
			self.rearRotorWash[i]:hide();
		end
	end

	if self.muzzleVFX then
		for i=1,#self.muzzleVFX do
			self.muzzleVFX[i]:hide();
		end
	end

	if self.glowVFX then
		self.glowVFX:hide();
	end

	if self.glowRocketVFX then
		for i=1,#self.glowRocketVFX do
			self.glowRocketVFX[i]:hide();
		end
	end

	if self.strafeAnimOffset then
		for i=1,#self.strafeAnimOffset do
			self.strafeAnimOffset[i]:pause();
		end
	end

	if self.strafeAnimScale then
		self.strafeAnimScale:pause();
	end

end

----------------------------------------------------------------------

function TurretBossGunship:show()

	Base.show(self);

	if self.armor then
		self.armor:show();
	end

	if self.turretShadow then
		self.turretShadow:show();
	end

	if self.barrel then
		for i=1,#self.barrel do
			self.barrel[i]:show();
		end
	end

	if self.rocketPod then
		for i=1,#self.rocketPod do
			self.rocketPod[i]:show();
		end
	end

	if self.barrelShadow then
		for i=1,#self.barrelShadow do
			self.barrelShadow[i]:show();
		end
	end

	if self.frontRotor then
		for i=1,#self.frontRotor do
			self.frontRotor[i]:show();
		end
	end

	if self.rearRotor then
		for i=1,#self.rearRotor do
			self.rearRotor[i]:show();
		end
	end

	if self.frontRotorWash then
		for i=1,#self.frontRotorWash do
			self.frontRotorWash[i]:show();
		end
	end

	if self.rearRotorWash then
		for i=1,#self.rearRotorWash do
			self.rearRotorWash[i]:show();
		end
	end

	if self.muzzleVFX then
		for i=1,#self.muzzleVFX do
			self.muzzleVFX[i]:show();
		end
	end

	if self.glowVFX then
		self.glowVFX:show();
	end

	if self.glowRocketVFX then
		for i=1,#self.glowRocketVFX do
			self.glowRocketVFX[i]:show();
		end
	end

	if self.strafeAnimOffset then
		for i=1,#self.strafeAnimOffset do
			self.strafeAnimOffset[i]:start();
		end
	end

	if self.strafeAnimScale then
		self.strafeAnimScale:start();
	end

end

----------------------------------------------------------------------
