require('game/entity/turret/turret');

----------------------------------------------------------------------

local TURRET_SHOOT_DISABLED                     = 1;
local TURRET_SHOOT_SLEEP                        = 2;
local TURRET_SHOOT_AIM                          = 3;
local TURRET_SHOOT_SHOOT                        = 4;
local TURRET_SHOOT_ON_SCREEN_DELTA              = 25;
local TURRET_SHOOT_TARGET_AVATER_VALID_OFFSET   = 50;
local TURRET_SHOOT_TARGET_AI                    = 1;
local TURRET_SHOOT_TARGET_AVATAR                = 2;

local TURRET_BEAM_INACTIVE								= 1;
local TURRET_BEAM_POWERUP								= 2;
local TURRET_BEAM_ACTIVE								= 3;
local TURRET_BEAM_POWERDOWN							= 4;

local SCREEN_HEIGHT										= 1136;


----------------------------------------------------------------------

local Base  = Turret;
TurretLaser = Base:new();

----------------------------------------------------------------------

function TurretLaser:new(  game, layer, template,  x, y, angle, parent, offset )

   local e = Base.new( self, game, layer, template, x, y, angle, parent, offset );

	e.beamVFX = Base.new( self, game, layer, template.beamVFX, 0, 0, template.projOffset.angle, e, {template.projOffset.x, template.projOffset.y}  );
	e.beamVFX:hide();

	e.projSpeed = template.projectile.speed[2] or 1500;

   if template then

      --call here because avatar doesnt get called yet
      e:reset( x, y, angle, parent, offset  );

   end

   return e;

end

----------------------------------------------------------------------

function TurretLaser:reset( x, y, angle, parent, offset )

   Base.reset( self, x, y, angle, parent, offset );

   self.state         = TURRET_SHOOT_SLEEP;
   self.shootDelay    = 0;
   self.bulletDelay   = 0;
   self.bulletCount   = 0;
	self.alwaysShoot	 = self.template.alwaysShoot or false;
   self.shootAngleMax = self.template.shootAngleMax or 1;
   self.faceAndShoot  = self.template.faceAndShoot or true;
	self.type			 = self.template.type or "";
   self.bulletDelta   = self.template.bulletDelta;
	self.parentProj	 = self.template.parentProj or false;
	self.projOffset	 = self.template.projOffset or {{ x=0, y=0, angle=0}};
	self.sweepTime		 = self.template.sweepTime or 2.0;
	self.sweepTimer	 = 0;
	self.sweepArc		 = self.template.sweepArc or 30;

	self.beamCooldownTime		= self.template.beamCooldown or 1.0;
	self.beamPulseFreq			= self.template.beamPulseFreq or 1.0;
	self.beamPulseWidth			= self.template.beamPulseWidth or 0.4;
	self.beamPulseAlpha			= self.template.beamPulseAlpha or 0.6;
	self.beamTarget				= false;
	self.beamState					= TURRET_BEAM_INACTIVE;
	self.beamGrowTime				= math.clamp(SCREEN_HEIGHT/(self.projSpeed or 1500), 0.35, 0.9);
	self.beamRetTime				= math.clamp(self.beamGrowTime, 0.20, 1/3);
	self.beamCooldown				= 0;
	self.beamPulseTime			= 0;
	self.beamStartTimer			= 0;
	self.beamStopTimer			= 0;
   
   self.beamLoopOn            = false;

	self:_hideBeamVFX();

end

----------------------------------------------------------------------

function TurretLaser:update( dt, avatarSpeedY, burstOn, canShoot )

   --Base.update( self, dt, avatarSpeedY );

   self:updateTarget();

	if canShoot and self.template.projectile then
		self:updateFace( dt );
	end

   if (self.target or self.alwaysShoot)
		and self.template.projectile
		and canShoot then

		self:updateShoot( dt, burstOn );
	else
		self.beamTarget = false;
	end

	self:updateBeamVFX(dt);

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function TurretLaser:isTargetValid( target )

   if     (target)
      and (not target.deleteMe )
      and (not target:isDead() ) then

      if self.targetType == TURRET_SHOOT_TARGET_AI then

         return self:getLocY(true) < (target:getLocY()-target:getHeight()/2);

      else

         return self:getLocY(true) > ( self.game.avatar:getLocY() + TURRET_SHOOT_TARGET_AVATER_VALID_OFFSET );

      end
   end

   return false;

end

----------------------------------------------------------------------

function TurretLaser:updateTarget()

   --cant put in reset because self.game.avatar is not set yet
   if self.game.avatar == self.parent then
      self.targetType    = TURRET_SHOOT_TARGET_AI;
   else
      self.targetType    = TURRET_SHOOT_TARGET_AVATAR;
   end

   if self.targetType == TURRET_SHOOT_TARGET_AVATAR then

      self.target = self.game.avatar;

   else

      if #self.game.entityMgr.vehicles == 0 then

         self.target = nil;

      elseif self.target == nil then

         local vehicles    = self.game.entityMgr.vehicles;
         local best        = 1;
         local bestScore   = 1000000000;
         for i=1,#vehicles do

            local tgt   = vehicles[i];
            if self:isTargetValid( tgt ) then

               local tgtY  = tgt:getLocY(true);
               local score = 10*tgt:getHealthMax()

               - tgtY;
               if score < bestScore then

                  bestScore = score;
                  best      = i;

               end
            end
         end

         self.target = self.game.entityMgr.vehicles[best];

      elseif not self:isTargetValid( self.target ) then

         self.target = nil;

      end
   end
end

----------------------------------------------------------------------

function TurretLaser:isAngleOk( a )

   local limit = self.angleLimit;

   if limit then

      if limit.min < 0 then
         a = math.clampAngle180( a );
      else
         a = math.clampAngle360( a );
      end

      return a >= limit.min and a <= limit.max;

   else

      return true;

   end
end

----------------------------------------------------------------------

function TurretLaser:updateFace( dt  )

   if       self.game
      and   ( self.faceAndShoot or self.state ~= TURRET_SHOOT_SHOOT ) then

		if self.sweepTimer > self.sweepTime then
			self.sweepTimer = 0;
		end

		self.sweepTimer = self.sweepTimer + dt;

		if self.sweepTimer <= self.sweepTime/4 then

			self.angle = ( self.sweepTimer/(self.sweepTime/4) ) * ( self.sweepArc/-2 );

		elseif (self.sweepTimer > self.sweepTime/4) and (self.sweepTimer <= 3*(self.sweepTime/4)) then

			self.angle = ( self.sweepTimer - self.sweepTime/4 ) / ( self.sweepTime/2 );
			self.angle = ( self.sweepArc * self.angle ) - ( self.sweepArc/2 )

		else

			self.angle = ( self.sweepTimer - 3*(self.sweepTime/4) ) / ( self.sweepTime/4 );
			self.angle = (self.sweepArc/2) - (self.sweepArc/2)*self.angle;

		end

		self.da = 0;
		self:setRot(self.angle);

   end

end

----------------------------------------------------------------------

function TurretLaser:disable()

   self.state				= TURRET_SHOOT_DISABLED;
	self.beamTarget		= false;
	self.beamStartTimer	= 0;

end

----------------------------------------------------------------------

function TurretLaser:enable()

   if self.state == TURRET_SHOOT_DISABLED then
      self.state = TURRET_SHOOT_AIM;
   end
end


----------------------------------------------------------------------

function TurretLaser:calcBulletLoc()

   local x,  y = self:getLoc(true);
   local dX,dY = 0,0;

   if self.bulletDelta then

      local delta = self.bulletDelta[self.bulletCount+1] or {};

      dX = delta.x or 0;
      dY = delta.y or 0;

   end

   return x+dX, y+dY;

end

----------------------------------------------------------------------

function TurretLaser:calcParentBulletOffset()

	local offset = { x=0,	y=0,	angle=0 };

   if self.projOffset then
		offset = self.projOffset[self.bulletCount+1] or self.projOffset[1] or offset;
   end

   return offset;

end

----------------------------------------------------------------------

function TurretLaser:updateShoot( dt, burstOn  )

   if self.state == TURRET_SHOOT_DISABLED then

		self.beamTarget = false;

   elseif self.state == TURRET_SHOOT_SLEEP then

      if (self:getLocY(true) + TURRET_SHOOT_ON_SCREEN_DELTA) < ( self.game.boundary.max.y ) then

         self.state = TURRET_SHOOT_AIM;

      end

		self.beamTarget = false;

   elseif self.state == TURRET_SHOOT_AIM then

      self.shootDelay = self.shootDelay - dt;

      if       self.shootDelay      < 0
			and	self.da
         and   math.abs(self.da)    < self.shootAngleMax
         and   (not self.template.shootPosMax  or self:getLocY(true)   < self.template.shootPosMax) then

         self.state       = TURRET_SHOOT_SHOOT;
         self.shootDelay  = self.template.shootDelay;
         self.bulletCount = 0;
      end

		self.beamTarget = false;

   else

      if self.target == nil and not self.alwaysShoot then

         self.state = TURRET_SHOOT_AIM;

			self.beamTarget = false;

      else

			self.beamTarget = true;

			self:_shootProj(dt, burstOn);
      end
   end
end

----------------------------------------------------------------------

function TurretLaser:_shootProj( dt, burstOn )

	self.bulletDelay = self.bulletDelay-dt;

	if self.bulletDelay < 0.001    then

		local x,y = self:calcBulletLoc();

		self.bulletDelay = self.template.bulletDelay;

		self.bulletCount = self.bulletCount + 1;

		local p;

		if self.parentProj then
			local poffset = self:calcParentBulletOffset();

			p = self.game.entityMgr:spawnProjectile( self.template.projectile, poffset.x, poffset.y, poffset.angle, self.parent, self.target, self, poffset );
		else
			p = self.game.entityMgr:spawnProjectile( self.template.projectile, x, y, self.angle, self.parent, self.target );
		end

		p.speedBurst = burstOn;

		if self.bulletCount  >= self.template.bulletCount  then

			self.state = TURRET_SHOOT_AIM;

		end
	end

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function TurretLaser:hide()

	Base.hide(self);
	self:_hideBeamVFX();

	self.beamTarget		= false;
	self.beamState			= TURRET_BEAM_INACTIVE;
	self.beamStartTimer	= 0;
	self.beamStopTimer	= 0;
	self.beamCooldown		= 0;
	self.beamPulseTime	= 0;

end

----------------------------------------------------------------------

function TurretLaser:show()

	Base.show(self);

end

----------------------------------------------------------------------

function TurretLaser:_hideBeamVFX()

	if self.beamVFX then
		self.beamVFX:hide();
	end

end

----------------------------------------------------------------------

function TurretLaser:_showBeamVFX()

	if self.beamVFX then
		self.beamVFX:show();
	end

end

----------------------------------------------------------------------

function TurretLaser:_isBeamOnCooldown()

	if (self.beamState == TURRET_BEAM_ACTIVE or self.beamState == TURRET_BEAM_POWERUP)
		and (self.beamCooldown < self.beamCooldownTime) then
			return true;
	end

	return false;

end

----------------------------------------------------------------------

function TurretLaser:updateBeamVFX( dt )

	if self
		and self.game
		and self.game.avatar
		and self.beamVFX then

		if self.beamTarget then
			if self.beamState == TURRET_BEAM_INACTIVE
				or self.beamState == TURRET_BEAM_POWERDOWN then
					self.beamState			= TURRET_BEAM_POWERUP;
					self.beamStopTimer	= 0;
					self.beamCooldown		= 0;
					self.beamPulseTime	= 0;
			end
		else
			if self.beamState == TURRET_BEAM_ACTIVE
				or self.beamState == TURRET_BEAM_POWERUP then
					if self.beamCooldown >= self.beamCooldownTime then
						self.beamState			= TURRET_BEAM_POWERDOWN;
						self.beamStartTimer	= 0;
						self.beamCooldown		= 0;
						self.beamPulseTime	= 0;
					else
						self.beamCooldown = self.beamCooldown + dt;
					end
			end
		end

		if self.beamState == TURRET_BEAM_POWERUP then
         if not self.beamLoopOn then
            local lSounds = { loop={"LaserLoop",},};
            EventMgr:postEvent( {sounds=lSounds, name="laser",}, "add-sfx" );
            self.beamLoopOn = true;
         end
			self:updateBeamPowerUp(dt);
		elseif self.beamState == TURRET_BEAM_POWERDOWN then
         if self.beamLoopOn then
            local lSounds = { loop={"LaserLoop",},};
            EventMgr:postEvent( {sounds=lSounds, name="laser",}, "loop-stop" );
            self.beamLoopOn = false;
         end
			self:updateBeamPowerDown(dt);
		elseif self.beamState == TURRET_BEAM_ACTIVE then
			self:updateBeamActivePulse(dt);
		end

	end

end

----------------------------------------------------------------------

function TurretLaser:updateBeamPowerUp( dt )

	self.beamVFX:show();
	self.beamVFX:setColor(1,1,1,1);
	self.beamVFX:setLoc(self.projOffset.x, self.projOffset.y);

	self.beamStartTimer = self.beamStartTimer + dt;

	local scaleX = math.min(self.beamStartTimer/(self.beamGrowTime*0.45), 1);
	local scaleY = math.min(self.beamStartTimer/(self.beamGrowTime*0.75), 1);

	self.beamVFX:setScale(scaleX, scaleY);

	if scaleY == 1 then
		self.beamState			= TURRET_BEAM_ACTIVE;
		self.beamStartTimer	= 0;
	end

end

----------------------------------------------------------------------

function TurretLaser:updateBeamPowerDown( dt )

	self.beamStopTimer = self.beamStopTimer + dt;

--[[
	--beam cut off efect
	local maxY	= (SCREEN_HEIGHT*2) + self.projOffset.y;
	local y		= (self.beamStopTimer/(self.beamGrowTime*2.0))*(SCREEN_HEIGHT*2) + self.projOffset.y;
	local a		= 1 - (self.beamStopTimer/self.beamGrowTime);

	y = math.min(y, maxY);
	a = math.max(a, 0.3);

	self.beamVFX:setLoc(self.projOffset.x, y);
	self.beamVFX:setColor(1, 1, 1, a);

	if y >= maxY then
		self.beamState			= TURRET_BEAM_INACTIVE;
		self.beamStopTimer	= 0;
	end
]]

	--beam retract effect
	self.beamVFX:setLoc(self.projOffset.x, self.projOffset.y);

	local scaleX = math.max(1-(self.beamStopTimer/self.beamRetTime), 0.5);
	local scaleY = math.max(1-(self.beamStopTimer/self.beamRetTime), 0.01);
	local a		= 1 - (self.beamStopTimer/self.beamRetTime);

	a = math.max(a, 0.3);

	self.beamVFX:setScale(scaleX, scaleY);
	self.beamVFX:setColor(1, 1, 1, a);

	if scaleY <= 0.01 then
		self.beamState			= TURRET_BEAM_INACTIVE;
		self.beamStopTimer	= 0;
	end

end

----------------------------------------------------------------------

function TurretLaser:updateBeamActivePulse( dt )

	self.beamPulseTime = self.beamPulseTime + dt;

	if self.beamPulseTime >= self.beamPulseFreq then
		self.beamPulseTime = 0;
	end

	local scaleX = math.clamp(self.beamPulseTime/(self.beamPulseFreq/2), 0, 2) * self.beamPulseWidth;
	local scaleA = math.clamp(self.beamPulseTime/(self.beamPulseFreq/2), 0, 2) * self.beamPulseAlpha;

	if self.beamPulseTime >= 0 and self.beamPulseTime <= (self.beamPulseFreq/2) then
		scaleX = 1 + scaleX;
		scaleA = 1 - self.beamPulseAlpha + scaleA;
	else
		scaleX = 1 + self.beamPulseWidth - scaleX;
		scaleA = 1 + self.beamPulseAlpha - scaleA;
	end

	self.beamVFX:setScale(scaleX, 1);
	self.beamVFX:setColor(1, 1, 1, scaleA);

end

----------------------------------------------------------------------


