require('game/entity/turret/turret');

----------------------------------------------------------------------

local TURRET_MINE_DISABLED								= 0;
local TURRET_MINE_SLEEP									= 1;
local TURRET_MINE_SHOOT									= 2;

local TURRET_GLOW_VFX_SLEEP							= 1;
local TURRET_GLOW_VFX_START							= 2;
local TURRET_GLOW_VFX_STOP								= 3;

----------------------------------------------------------------------

local Base  = Turret;
TurretMine = Base:new();

----------------------------------------------------------------------

function TurretMine:new(  game, layer, template,  x, y, angle, parent, offset )

   local e = Base.new( self, game, layer, template, x, y, angle, parent, offset );

	if template.glowVFX then
		e.glowVFX = Base.new( self, game, layer, template.glowVFX, 0, 0, 0, e, {template.glowOffset.x, template.glowOffset.y}  );
		e.glowVFX:hide();
	end

   if template then

      --call here because avatar doesnt get called yet
      e:reset( x, y, angle, parent, offset  );

   end

   return e;

end

----------------------------------------------------------------------

function TurretMine:reset( x, y, angle, parent, offset )

   Base.reset( self, x, y, angle, parent, offset );

   self.state         = TURRET_MINE_SLEEP;
   self.shootDelay    = self.template.shootDelay or 0;
   self.bulletDelay   = self.template.bulletDelay or 0;
   self.bulletCount   = 0;
	self.glowState		 = TURRET_GLOW_VFX_SLEEP;
	self.glowTime		 = math.min( (self.template.shootDelay or 0) + (self.template.bulletDelay or 0), self.template.glowTime or 0.25);
	self.glowScale		 = self.template.glowScale or {w=3,	h=3};

end

----------------------------------------------------------------------

function TurretMine:update( dt, avatarSpeedY, burstOn, canShoot )

   --Base.update( self, dt, avatarSpeedY );

   self:updateShoot(dt);
	self:updateVFX(dt);

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function TurretMine:disable()

   self.state = TURRET_MINE_DISABLED;

end

----------------------------------------------------------------------

function TurretMine:enable()

   if self.state == TURRET_MINE_DISABLED then

      self.state			= TURRET_MINE_SLEEP;
		self.bulletCount	= 0;
		self.bulletDelay	= self.template.bulletDelay or 0;
   end

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function TurretMine:updateShoot( dt )

	if self.state == TURRET_MINE_DISABLED then

		return false;
	end

	if self.state == TURRET_MINE_SLEEP then

		self.shootDelay = self.shootDelay - dt;

		if self.shootDelay <= 0 then

			self.shootDelay	= self.template.shootDelay or 0;
			self.state			= TURRET_MINE_SHOOT;
		end
	else

		self.bulletDelay = self.bulletDelay-dt;

		if self.bulletDelay <= 0 then

			self:_layMine();
			self.bulletDelay	= self.template.bulletDelay or 0;
			self.bulletCount	= self.bulletCount + 1;
		end

		if self.bulletCount  >= self.template.bulletCount then

			self.bulletCount	= 0;
			self.state			= TURRET_MINE_SLEEP;
		end
	end

end

----------------------------------------------------------------------

function TurretMine:updateVFX( dt )

	if self.glowVFX then

		local countdown;
		local scale			= 0;

		if	self.state and self.state == TURRET_MINE_SLEEP then
			countdown = self.shootDelay + self.bulletDelay;
		else
			countdown = self.bulletDelay;
		end

		if		(self.state and self.state == TURRET_MINE_DISABLED)
			or	not self.shootDelay
			or	not self.glowScale
			or	countdown >= self.glowTime then

			self.glowVFX:hide();

			if self.glowState then
				if self.glowState == TURRET_GLOW_VFX_STOP then
					self.glowState = TURRET_GLOW_VFX_SLEEP;
				elseif self.glowState == TURRET_GLOW_VFX_START then
					self.glowState = TURRET_GLOW_VFX_STOP;
				else
					self.glowState = TURRET_GLOW_VFX_SLEEP;
				end
			end
		else
			scale = math.min((self.glowTime - countdown)/self.glowTime, 1);

			self.glowVFX:show();
			self.glowVFX:setScale(scale*self.glowScale.w, scale*self.glowScale.h);

			if self.glowState then
				if self.glowState ~= TURRET_GLOW_VFX_START then
					self.glowState = TURRET_GLOW_VFX_START;
				end
			end
		end
	end

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function TurretMine:hide()

	Base.hide(self);

	if self.glowVFX then
		self.glowVFX:hide();
	end

end

----------------------------------------------------------------------

function TurretMine:show()

	Base.show(self);

	if self.glowVFX then
		self.glowVFX:show();
	end

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function TurretMine:_layMine()

	local x,y	= self.parent:getLoc();
	local lane	= self.game:getClosestLaneTo(x);

	self.game.gameSpawner:spawnBarricade(	self.template.barricade,
														x+self.template.barrOffset.x,
														y+self.template.barrOffset.y,
                                          lane
													);

end

----------------------------------------------------------------------