require('game/entity/turret/turret');

----------------------------------------------------------------------

local TURRET_SHOOT_DISABLED                     = 1;
local TURRET_SHOOT_SLEEP                        = 2;
local TURRET_SHOOT_AIM                          = 3;
local TURRET_SHOOT_SHOOT                        = 4;
local TURRET_SHOOT_ON_SCREEN_DELTA              = 25;
local TURRET_SHOOT_TARGET_AVATER_VALID_OFFSET   = 50;
local TURRET_SHOOT_TARGET_AI                    = 1;
local TURRET_SHOOT_TARGET_AVATAR                = 2;

local TURRET_GLOW_VFX_SLEEP							= 1;
local TURRET_GLOW_VFX_START							= 2;
local TURRET_GLOW_VFX_STOP								= 3;

----------------------------------------------------------------------

local Base  = Turret;
TurretNova = Base:new();

----------------------------------------------------------------------

function TurretNova:new(  game, layer, template,  x, y, angle, parent, offset )

   local e = Base.new( self, game, layer, template, x, y, angle, parent, offset );

	e.glowVFX = Base.new( self, game, layer, template.glowVFX, 0, 0, 0, e, {template.glowOffset.x, template.glowOffset.y}  );
	e.glowVFX:hide();

   if template then
      --call here because avatar doesnt get called yet
      e:reset( x, y, angle, parent, offset  );
   end

   return e;

end

----------------------------------------------------------------------

function TurretNova:reset( x, y, angle, parent, offset )

   Base.reset( self, x, y, angle, parent, offset );

   self.state         = TURRET_SHOOT_SLEEP;
	self.shotCooldown	 = false;
   self.shootDelay    = 0;
   self.bulletDelay   = self.template.bulletDelay;
   self.bulletCount   = 0;
	self.glowState		 = TURRET_GLOW_VFX_SLEEP;
	self.glowTime		 = math.min(self.template.bulletDelay or 0, self.template.glowTime or 0.25);
	self.glowScale		 = self.template.glowScale or {w=3,	h=3};
	self.alwaysShoot	 = self.template.alwaysShoot or false;
   self.shootAngleMax = self.template.shootAngleMax or 1;
   self.faceAndShoot  = self.template.faceAndShoot or true;
	self.type			 = self.template.type or "";
   self.bulletDelta   = self.template.bulletDelta;
	self.parentProj	 = self.template.parentProj or false;
	self.projOffset	 = self.template.projOffset or {{ x=0, y=0, angle=0 }};

end

----------------------------------------------------------------------

function TurretNova:update( dt, avatarSpeedY, burstOn, canShoot )

   --Base.update( self, dt, avatarSpeedY );

   self:updateTarget();
	self:updateVFX();

   if			(self.target or self.alwaysShoot)
		and	self.template.projectile
		and	canShoot then

      self:updateFace( dt );
      self:updateShoot( dt, burstOn );
	else
		self.bulletDelay	= self.template.bulletDelay;
	end

end

----------------------------------------------------------------------

function TurretNova:isTargetValid( target )

   if     (target)
      and (not target.deleteMe )
      and (not target:isDead() ) then

      if self.targetType == TURRET_SHOOT_TARGET_AI then

         return self:getLocY(true) < (target:getLocY()-target:getHeight()/2);

      else

         return self:getLocY(true) > ( self.game.avatar:getLocY() + TURRET_SHOOT_TARGET_AVATER_VALID_OFFSET );

      end
   end

   return false;

end

----------------------------------------------------------------------

function TurretNova:updateTarget()

   --cant put in reset because self.game.avatar is not set yet
   if self.game.avatar == self.parent then
      self.targetType    = TURRET_SHOOT_TARGET_AI;
   else
      self.targetType    = TURRET_SHOOT_TARGET_AVATAR;
   end

   if self.targetType == TURRET_SHOOT_TARGET_AVATAR then

      self.target = self.game.avatar;

   else

      if #self.game.entityMgr.vehicles == 0 then

         self.target = nil;

      elseif self.target == nil then

         local vehicles    = self.game.entityMgr.vehicles;
         local best        = 1;
         local bestScore   = 1000000000;
         for i=1,#vehicles do

            local tgt   = vehicles[i];
            if self:isTargetValid( tgt ) then

               local tgtY  = tgt:getLocY(true);
               local score = 10*tgt:getHealthMax()

               - tgtY;
               if score < bestScore then

                  bestScore = score;
                  best      = i;

               end
            end
         end

         self.target = self.game.entityMgr.vehicles[best];

      elseif not self:isTargetValid( self.target ) then

         self.target = nil;

      end
   end
end

----------------------------------------------------------------------

function TurretNova:isAngleOk( a )

   local limit = self.angleLimit;

   if limit then

      if limit.min < 0 then
         a = math.clampAngle180( a );
      else
         a = math.clampAngle360( a );
      end

      return a >= limit.min and a <= limit.max;

   else

      return true;

   end
end

----------------------------------------------------------------------

function TurretNova:updateFace( dt  )

   if       self.game
      and   self.target
      and   ( self.faceAndShoot or self.state ~= TURRET_SHOOT_SHOOT ) then

      local x, y   = self:getLoc(true);
      local tx, ty = self.target:getLoc();
      local dx, dy = tx-x, ty-y-self.target:getHeight()/4;

      --Calculate around Y
      local rad   = -math.atan2(dx,dy);

      local a     = ( rad*180/math.pi );
      local da    = math.clampAngle180( a-self.angle );

      if self:isAngleOk( a ) then

         local da2   = math.clamp(da, -self.maxTurnRate*dt,self.maxTurnRate*dt);

         self.angle  = self.angle+da2;
         self.da     = da2;

         self:setRot( self.angle );

      else

         self.da = da;

      end

   end
end

----------------------------------------------------------------------

function TurretNova:disable()

   self.state			= TURRET_SHOOT_DISABLED;
	self.bulletDelay	= self.template.bulletDelay;

end

----------------------------------------------------------------------

function TurretNova:enable()

   if self.state == TURRET_SHOOT_DISABLED then
      self.state			= TURRET_SHOOT_AIM;
		self.shotCooldown	= false;
		self.bulletDelay	= self.template.bulletDelay;
   end
end


----------------------------------------------------------------------

function TurretNova:calcBulletLoc()

   local x,  y = self:getLoc(true);
   local dX,dY = 0,0;

   if self.bulletDelta then

      local delta = self.bulletDelta[self.bulletCount+1] or {};

      dX = delta.x or 0;
      dY = delta.y or 0;

   end

   return x+dX, y+dY;

end

----------------------------------------------------------------------

function TurretNova:calcParentBulletOffset()

	local offset = { x=0,	y=0,	angle=0 };

   if self.projOffset then
		offset = self.projOffset[self.bulletCount+1] or self.projOffset[1] or offset;
   end

   return offset;

end

----------------------------------------------------------------------

function TurretNova:updateShoot( dt, burstOn  )

   if self.state == TURRET_SHOOT_DISABLED then

   elseif self.state == TURRET_SHOOT_SLEEP then

      if (self:getLocY(true) + TURRET_SHOOT_ON_SCREEN_DELTA) < ( self.game.boundary.max.y ) then

         self.state = TURRET_SHOOT_AIM;

      end

   elseif self.state == TURRET_SHOOT_AIM then

      self.shootDelay = self.shootDelay - dt;

      if       self.shootDelay      < 0
			and	self.da
         and   math.abs(self.da)    < self.shootAngleMax
         and   (not self.template.shootPosMax  or self:getLocY(true)   < self.template.shootPosMax) then

         self.state       = TURRET_SHOOT_SHOOT;
         self.shootDelay  = self.template.shootDelay;
         self.bulletCount = 0;

      end

   else

      if self.target == nil and not self.alwaysShoot then

         self.state = TURRET_SHOOT_AIM;

      else

         self.bulletDelay = self.bulletDelay-dt;

         if self.bulletDelay <= 0 then

				if not self.shotCooldown then

					local x,y				= self:calcBulletLoc();

					self.shotCooldown		= true;
					self.bulletCount		= self.bulletCount + 1;

					local p;

					if self.parentProj then
						local poffset = self:calcParentBulletOffset();

						p = self.game.entityMgr:spawnProjectile( self.template.projectile, poffset.x, poffset.y, poffset.angle, self.parent, self.target, self, poffset );
					else
						p = self.game.entityMgr:spawnProjectile( self.template.projectile, x, y, self.angle, self.parent, self.target );
					end

					p.speedBurst = burstOn;
			
					if self.bulletCount  >= self.template.bulletCount  then
               	self.state = TURRET_SHOOT_AIM;
					end
				end

				if self.bulletDelay <= 0 then
					self.shotCooldown		= false;
					self.bulletDelay		= self.template.bulletDelay;
            end
         end
      end
   end
end

----------------------------------------------------------------------

function TurretNova:hide()

	Base.hide(self);

	if self.glowVFX then
		self.glowVFX:hide();
	end

end

----------------------------------------------------------------------

function TurretNova:show()

	Base.show(self);

	if self.glowVFX then
		self.glowVFX:show();
	end

end

----------------------------------------------------------------------

function TurretNova:updateVFX( dt )

	if		not self.bulletDelay
		or	not self.glowScale
		or	self.bulletDelay >= self.glowTime then

		self.glowVFX:hide();

		if self.glowState then
			if self.glowState == TURRET_GLOW_VFX_STOP then
				self.glowState = TURRET_GLOW_VFX_SLEEP;
			elseif self.glowState == TURRET_GLOW_VFX_START then
				self.glowState = TURRET_GLOW_VFX_STOP;
				EventMgr:postEvent( {sounds = {start={	"NovaFire",},}, name="nova shoot"}, "add-sfx" );
			else
				self.glowState = TURRET_GLOW_VFX_SLEEP;
			end
		end
	else
		local scale = math.min((self.glowTime - self.bulletDelay)/self.glowTime, 1);

		self.glowVFX:show();
		self.glowVFX:setScale(scale*self.glowScale.w, scale*self.glowScale.h);

		if self.glowState then
			if self.glowState ~= TURRET_GLOW_VFX_START then
				self.glowState = TURRET_GLOW_VFX_START;
			end
		end
	end

end

----------------------------------------------------------------------
