require('game/entity/turret/turret');

----------------------------------------------------------------------

local TURRET_SHOOT_DISABLED                     = 1;
local TURRET_SHOOT_SLEEP                        = 2;
local TURRET_SHOOT_AIM                          = 3;
local TURRET_SHOOT_SHOOT                        = 4;
local TURRET_SHOOT_ON_SCREEN_DELTA              = 25;
local TURRET_SHOOT_TARGET_AVATER_VALID_OFFSET   = 50;
local TURRET_SHOOT_TARGET_AI                    = 1;
local TURRET_SHOOT_TARGET_AVATAR                = 2;

local TURRET_GLOW_VFX_SLEEP							= 1;
local TURRET_GLOW_VFX_START							= 2;
local TURRET_GLOW_VFX_STOP								= 3;

----------------------------------------------------------------------

local Base  = Turret;
TurretShoot = Base:new();

----------------------------------------------------------------------

function TurretShoot:new(  game, layer, template,  x, y, angle, parent, offset )

   local e = Base.new( self, game, layer, template, x, y, angle, parent, offset );

	if template.glowVFX then
		e.glowVFX = Base.new( self, game, layer, template.glowVFX, 0, 0, 0, e, {template.glowOffset.x, template.glowOffset.y}  );
		e.glowVFX:hide();
	end

   if template then
      --call here because avatar doesnt get called yet
      e:reset( x, y, angle, parent, offset  );
   end

   return e;

end

----------------------------------------------------------------------

function TurretShoot:reset( x, y, angle, parent, offset )

   Base.reset( self, x, y, angle, parent, offset );

   self.state         = TURRET_SHOOT_SLEEP;
   self.shootDelay    = 0;
   self.bulletDelay   = self.template.bulletDelay or 0;
   self.bulletCount   = 0;
	self.glowState		 = TURRET_GLOW_VFX_SLEEP;
	self.glowTime		 = math.min( (self.template.shootDelay or 0) + (self.template.bulletDelay or 0), self.template.glowTime or 0.25);
	self.glowScale		 = self.template.glowScale or {w=3,	h=3};
   self.shootAngleMax = self.template.shootAngleMax or 1;
   self.faceAndShoot  = self.template.faceAndShoot or true;
	self.type			 = self.template.type or "";
   self.bulletDelta   = self.template.bulletDelta;
	self.parentProj	 = self.template.parentProj or false;
	self.projOffset	 = self.template.projOffset or {{ x=0, y=0, angle=0}};

end

----------------------------------------------------------------------

function TurretShoot:update( dt, avatarSpeedY, burstOn, canShoot )

   --Base.update( self, dt, avatarSpeedY );

   self:updateTarget();
	self:updateVFX(dt);

   if self.target and self.template.projectile and canShoot then

      self:updateFace( dt );
      self:updateShoot( dt, burstOn );

   end
end

----------------------------------------------------------------------

function TurretShoot:isTargetValid( target )

   if     (target)
      and (not target.deleteMe )
      and (not target:isDead() ) then

      if self.targetType == TURRET_SHOOT_TARGET_AI then

         return self:getLocY(true) < (target:getLocY()-target:getHeight()/2);

      else

         return self:getLocY(true) > ( self.game.avatar:getLocY() + TURRET_SHOOT_TARGET_AVATER_VALID_OFFSET );

      end
   end

   return false;

end

----------------------------------------------------------------------

function TurretShoot:updateTarget()

   --cant put in reset because self.game.avatar is not set yet
   if self.game.avatar == self.parent then
      self.targetType    = TURRET_SHOOT_TARGET_AI;
   else
      self.targetType    = TURRET_SHOOT_TARGET_AVATAR;
   end

   if self.targetType == TURRET_SHOOT_TARGET_AVATAR then

      self.target = self.game.avatar;

   else

      if #self.game.entityMgr.vehicles == 0 then

         self.target = nil;

      elseif self.target == nil then

         local vehicles    = self.game.entityMgr.vehicles;
         local best        = 1;
         local bestScore   = 1000000000;
         for i=1,#vehicles do

            local tgt   = vehicles[i];
            if self:isTargetValid( tgt ) then

               local tgtY  = tgt:getLocY(true);
               local score = 10*tgt:getHealthMax()

               - tgtY;
               if score < bestScore then

                  bestScore = score;
                  best      = i;

               end
            end
         end

         self.target = self.game.entityMgr.vehicles[best];

      elseif not self:isTargetValid( self.target ) then

         self.target = nil;

      end
   end
end

----------------------------------------------------------------------

function TurretShoot:isAngleOk( a )

   local limit = self.angleLimit;

   if limit then

      if limit.min < 0 then
         a = math.clampAngle180( a );
      else
         a = math.clampAngle360( a );
      end

      return a >= limit.min and a <= limit.max;

   else

      return true;

   end
end

----------------------------------------------------------------------

function TurretShoot:updateFace( dt  )

   if       self.game
      and   self.target
      and   ( self.faceAndShoot or self.state ~= TURRET_SHOOT_SHOOT ) then

      local x, y   = self:getLoc(true);
      local tx, ty = self.target:getLoc();
      local dx, dy = tx-x, ty-y-self.target:getHeight()/4;

      --Calculate around Y
      local rad   = -math.atan2(dx,dy);

      local a     = ( rad*180/math.pi );
      local da    = math.clampAngle180( a-self.angle );

      if self:isAngleOk( a ) then

         local da2   = math.clamp(da, -self.maxTurnRate*dt,self.maxTurnRate*dt);

         self.angle  = self.angle+da2;
         self.da     = da2;

         self:setRot( self.angle );

      else

         self.da = da;

      end

   end
end

----------------------------------------------------------------------

function TurretShoot:disable()

   self.state = TURRET_SHOOT_DISABLED;

end

----------------------------------------------------------------------

function TurretShoot:enable()

   if self.state == TURRET_SHOOT_DISABLED then
      self.state = TURRET_SHOOT_AIM;
   end
end


----------------------------------------------------------------------

function TurretShoot:calcBulletLoc()

   local x,  y = self:getLoc(true);
   local dX,dY = 0,0;

   if self.bulletDelta then

      local delta = self.bulletDelta[self.bulletCount+1] or {};

      dX = delta.x or 0;
      dY = delta.y or 0;

   end

   return x+dX, y+dY;

end

----------------------------------------------------------------------

function TurretShoot:calcParentBulletOffset()

	local offset = { x=0,	y=0,	angle=0 };

   if self.projOffset then
		offset = self.projOffset[self.bulletCount+1] or self.projOffset[1] or offset;
   end

   return offset;

end

----------------------------------------------------------------------

function TurretShoot:updateShoot( dt, burstOn  )

   if self.state == TURRET_SHOOT_DISABLED then

   elseif self.state == TURRET_SHOOT_SLEEP then

      if (self:getLocY(true) + TURRET_SHOOT_ON_SCREEN_DELTA) < ( self.game.boundary.max.y ) then

         self.state = TURRET_SHOOT_AIM;

      end

   elseif self.state == TURRET_SHOOT_AIM then

      self.shootDelay = self.shootDelay - dt;

      if       self.shootDelay      < 0
			and	self.da
         and   math.abs(self.da)    < self.shootAngleMax
         and   (not self.template.shootPosMax  or self:getLocY(true)   < self.template.shootPosMax) then

         self.state       = TURRET_SHOOT_SHOOT;
         self.shootDelay  = self.template.shootDelay;
         self.bulletCount = 0;

      end

   else

      if self.target == nil then

         self.state = TURRET_SHOOT_AIM;

      else

         self.bulletDelay = self.bulletDelay-dt;

         if self.bulletDelay < 0.001    then

            local x,y = self:calcBulletLoc();

            self.bulletDelay = self.template.bulletDelay;

            self.bulletCount = self.bulletCount + 1;

				local p;

				if self.parentProj then
					local poffset = self:calcParentBulletOffset();

					p = self.game.entityMgr:spawnProjectile( self.template.projectile, poffset.x, poffset.y, poffset.angle, self.parent, self.target, self, poffset );
				else
					p = self.game.entityMgr:spawnProjectile( self.template.projectile, x, y, self.angle, self.parent, self.target );
				end

            p.speedBurst = burstOn;

				if not self.parent.isAvatar then
					EventMgr:postEvent( {sounds = {start={	"GunTruckFire1",},}, name="turret shoot"}, "add-sfx" );
				else
					if self.type == "rocket" then
					
						EventMgr:postEvent( {sounds = {start={	"PlayerMissile",
																			},}, name="turret shoot"}, "add-sfx" );
					end
					
				end
					

            if self.bulletCount  >= self.template.bulletCount  then

               self.state = TURRET_SHOOT_AIM;

            end
         end
      end
   end
end

----------------------------------------------------------------------

function TurretShoot:updateVFX( dt )

	if self.glowVFX then

		local countdown;
		local scale			= 0;

		if	self.state and self.state == TURRET_SHOOT_AIM then
			countdown = self.shootDelay + self.bulletDelay;
		else
			countdown = self.bulletDelay;
		end

		if		(self.state and self.state <= TURRET_SHOOT_SLEEP)
			or	not self.shootDelay
			or not self.bulletDelay
			or	not self.glowScale
			or	countdown >= self.glowTime then

			self.glowVFX:hide();

			if self.glowState then
				if self.glowState == TURRET_GLOW_VFX_STOP then
					self.glowState = TURRET_GLOW_VFX_SLEEP;
				elseif self.glowState == TURRET_GLOW_VFX_START then
					self.glowState = TURRET_GLOW_VFX_STOP;
				else
					self.glowState = TURRET_GLOW_VFX_SLEEP;
				end
			end
		else
			scale = math.min((self.glowTime - countdown)/self.glowTime, 1);

			self.glowVFX:show();
			self.glowVFX:setScale(scale*self.glowScale.w, scale*self.glowScale.h);

			if self.glowState then
				if self.glowState ~= TURRET_GLOW_VFX_START then
					self.glowState = TURRET_GLOW_VFX_START;
				end
			end
		end
	end

end

----------------------------------------------------------------------

function TurretShoot:hide()

	Base.hide(self);

	if self.glowVFX then
		self.glowVFX:hide();
	end

end

----------------------------------------------------------------------

function TurretShoot:show()

	Base.show(self);

	if self.glowVFX then
		self.glowVFX:show();
	end

end

----------------------------------------------------------------------