require('game/entity/turret/turret');

----------------------------------------------------------------------

local BOSS_INDEX											= 1;

local TURRET_SHOOT_DISABLED                     = 1;
local TURRET_SHOOT_SLEEP                        = 2;
local TURRET_SHOOT_AIM                          = 3;
local TURRET_SHOOT_SHOOT                        = 4;
local TURRET_SHOOT_ON_SCREEN_DELTA              = 25;
local TURRET_SHOOT_TARGET_AVATER_VALID_OFFSET   = 50;
local TURRET_SHOOT_TARGET_AI                    = 1;
local TURRET_SHOOT_TARGET_AVATAR                = 2;

local TURRET_GLOW_VFX_SLEEP							= 1;
local TURRET_GLOW_VFX_START							= 2;
local TURRET_GLOW_VFX_STOP								= 3;

local STRAFE_STATE_IDLE									= 1;
local STRAFE_STATE_STRAFE								= 2;
local STRAFE_STATE_STRAFE_EASE						= 3;

----------------------------------------------------------------------

local Base  = Turret;
TurretTank = Base:new();

----------------------------------------------------------------------

function TurretTank:new(  game, layer, template,  x, y, angle, parent, offset )

   local e = Base.new( self, game, layer, template, x, y, angle, parent, offset );

   if BUILD_DEBUG then
      e.__marker = "TurretTank";
   end

   e.prop:setPriority(10);

   if e.splitRight and e.splitLeft then
      e.splitRight:setPriority(11);
      e.splitLeft:setPriority(11);
   end

   if template.armor then
      e.armor = Entity.new( Entity, game, layer, template.armor, 0, 0, 0, e, {template.armor.offset[1], template.armor.offset[2]}  );
      e.armor.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
      e.armor.prop:setLoc( template.armor.offset[1], template.armor.offset[2] );
      e.armor.prop:setPriority(12);
   end

   if template.turretShadow then
      e.turretShadow = Entity.new( Entity, game, layer, template.turretShadow, 0, 0, 0, e, {template.turretShadow.offset[1], template.turretShadow.offset[2]}  );
      e.turretShadow.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
      e.turretShadow.prop:setLoc( template.turretShadow.offset[1], template.turretShadow.offset[2] );
      e.turretShadow.prop:setPriority(5);
   end

   if template.barrel then
      e.barrel = Entity.new( Entity, game, layer, template.barrel, 0, 0, 0, e, {template.barrel.offset[1], template.barrel.offset[2]}  );
      e.barrel.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
      e.barrel.prop:setLoc( template.barrel.offset[1], template.barrel.offset[2] );
      e.barrel.prop:setPriority(15);

      if template.barrelShadow then
         e.barrelShadow = Entity.new( Entity, game, layer, template.barrelShadow, 0, 0, 0, e.barrel, {template.barrelShadow.offset[1], template.barrelShadow.offset[2]}  );
         e.barrelShadow.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, e.barrel.prop, MOAIProp2D.TRANSFORM_TRAIT );
         e.barrelShadow.prop:setLoc( template.barrelShadow.offset[1], template.barrelShadow.offset[2] );
         e.barrelShadow.prop:setPriority(1);
      end
   end

   if template.glowVFX and e.barrel then
      e.glowVFX = Entity.new( Entity, game, layer, template.glowVFX, 0, 0, 0, e.barrel, {template.glowOffset.x, template.glowOffset.y}  );
      e.glowVFX.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, e.barrel.prop, MOAIProp2D.TRANSFORM_TRAIT );
      e.glowVFX.prop:setLoc( template.glowOffset.x, template.glowOffset.y );
      e.glowVFX.prop:setPriority(20);
      e.glowVFX:hide();
   end

   if template then

      --call here because avatar doesnt get called yet
      e:reset( x, y, angle, parent, offset  );

   end

   return e;

end

----------------------------------------------------------------------

function TurretTank:reset( x, y, angle, parent, offset )

   Base.reset( self, x, y, angle, parent, offset );

   self.state           = TURRET_SHOOT_SLEEP;
   self.shootDelay      = self.template.shootDelay or 0;
   self.bulletDelay     = self.template.bulletDelay or 0;
   self.bulletCount     = 0;
   self.glowState       = TURRET_GLOW_VFX_SLEEP;
   self.glowTime        = math.min( (self.template.shootDelay or 0) + (self.template.bulletDelay or 0), self.template.glowTime or 0.25);
   self.glowScale       = self.template.glowScale or {w=3,     h=3};
   self.recoilTime      = 0;
   self.recoilOffset    = self.template.recoilOffset or 25;
   self.shootAngleMax   = self.template.shootAngleMax or 1;
   self.faceAndShoot    = self.template.faceAndShoot or true;
   self.type            = self.template.type or "";
   self.bulletDelta     = self.template.bulletDelta;
   self.parentProj      = self.template.parentProj or false;
   self.projOffset      = self.template.projOffset or {{ x=0, y=0, angle=0}};
   self.strafeState     = nil;
   self.strafeAngle     = nil;

   EventMgr:rmvListener( self );

--[[
      EventMgr:addListener( self, self._setStrafeIdle, "boss-strafe-idle");
      EventMgr:addListener( self, self._setStrafeDirection, "boss-strafe-left");
      EventMgr:addListener( self, self._setStrafeDirection, "boss-strafe-right");
      EventMgr:addListener( self, self._setStrafeEase, "boss-strafe-left-ease");
      EventMgr:addListener( self, self._setStrafeEase, "boss-strafe-right-ease");
]]
   EventMgr:addListener( self, self.hide, "vehicle-turret-hide" );

end

----------------------------------------------------------------------

function TurretTank:update( dt, avatarSpeedY, burstOn, canShoot )

   --Base.update( self, dt, avatarSpeedY );

   self.burstOn = burstOn;

   self:updateTarget();
   self:updateVFX(dt);
   self:updateRecoil(dt);

   if self.target and self.template.projectile and canShoot then

      self:updateFace( dt );
      self:updateShoot( dt, burstOn );
   end

end

----------------------------------------------------------------------

function TurretTank:isTargetValid( target )

   if     (target)
      and (not target.deleteMe )
      and (not target:isDead() ) then

      if self.targetType == TURRET_SHOOT_TARGET_AI then

         return self:getLocY(true) < (target:getLocY()-target:getHeight()/2);

      else

         return self:getLocY(true) > ( self.game.avatar:getLocY() + TURRET_SHOOT_TARGET_AVATER_VALID_OFFSET );

      end
   end

   return false;

end

----------------------------------------------------------------------

function TurretTank:updateTarget()

   --cant put in reset because self.game.avatar is not set yet
   if self.game.avatar == self.parent then
      self.targetType    = TURRET_SHOOT_TARGET_AI;
   else
      self.targetType    = TURRET_SHOOT_TARGET_AVATAR;
   end

   if self.targetType == TURRET_SHOOT_TARGET_AVATAR then

      self.target = self.game.avatar;

   else

      if #self.game.entityMgr.vehicles == 0 then

         self.target = nil;

      elseif self.target == nil then

         local vehicles    = self.game.entityMgr.vehicles;
         local best        = 1;
         local bestScore   = 1000000000;
         for i=1,#vehicles do

            local tgt   = vehicles[i];
            if self:isTargetValid( tgt ) then

               local tgtY  = tgt:getLocY(true);
               local score = 10*tgt:getHealthMax()

               - tgtY;
               if score < bestScore then

                  bestScore = score;
                  best      = i;

               end
            end
         end

         self.target = self.game.entityMgr.vehicles[best];

      elseif not self:isTargetValid( self.target ) then

         self.target = nil;

      end
   end
end

----------------------------------------------------------------------

function TurretTank:isAngleOk( a )

   local limit = self.angleLimit;

   if limit then

      if limit.min < 0 then
         a = math.clampAngle180( a );
      else
         a = math.clampAngle360( a );
      end

      return a >= limit.min and a <= limit.max;

   else

      return true;

   end
end

----------------------------------------------------------------------

function TurretTank:updateFace( dt )

   if       self.game
      and   self.target
		and	self.state ~= TURRET_SHOOT_DISABLED
      and   ( self.faceAndShoot or self.state ~= TURRET_SHOOT_SHOOT ) then

      local x, y   = self:getLoc(true);
      local tx, ty = self.target:getLoc();
      local dx, dy = tx-x, ty-y-self.target:getHeight()/4;

      --Calculate around Y
      local rad   = -math.atan2(dx,dy);
		local pr		= self.parent:getRot();
      local a     = ( rad*180/math.pi );
      local da    = math.clampAngle180( a-self.angle-math.clampAngle180(pr) );

      if self:isAngleOk( a ) then

         local da2   = math.clamp(da, -self.maxTurnRate*dt,self.maxTurnRate*dt);

         self.angle  = self.angle+da2;
         self.da     = da2;

         self:setRot( self.angle );

      else

         self.da = da;

      end

		if	self.template.bodyTurnStrafe and self.strafeState and self.strafeAngle then

			local fa	= math.clampAngle180(pr);

			if fa > self.strafeAngle then
				fa = fa - self.template.bodyTurnRate*dt;
			elseif fa < self.strafeAngle then
				fa = fa + self.template.bodyTurnRate*dt;
			else
				fa = self.strafeAngle;
			end

			fa = math.clamp(fa, self.template.bodyAngleLimit[1], self.template.bodyAngleLimit[2]);

			self.parent:setRot(fa);
		end
   end

end

----------------------------------------------------------------------

function TurretTank:disable()

   self.state			= TURRET_SHOOT_DISABLED;
   self.shootDelay	= self.template.shootDelay or 0;
   self.bulletDelay	= self.template.bulletDelay or 0;

end

----------------------------------------------------------------------

function TurretTank:enable()

   if self.state == TURRET_SHOOT_DISABLED then
      self.state = TURRET_SHOOT_AIM;
   end

end


----------------------------------------------------------------------

function TurretTank:calcBulletLoc()

   local x,  y = self:getLoc(true);
   local dX,dY = 0,0;

   if self.bulletDelta then

      local delta = self.bulletDelta[self.bulletCount+1] or {};

      dX = delta.x or 0;
      dY = delta.y or 0;

   end

   return x+dX, y+dY;

end

----------------------------------------------------------------------

function TurretTank:calcParentBulletOffset()

	local offset = { x=0,	y=0,	angle=0 };

   if self.projOffset then
		offset = self.projOffset[self.bulletCount+1] or self.projOffset[1] or offset;
   end

   return offset;

end

----------------------------------------------------------------------

function TurretTank:updateShoot( dt, burstOn  )

   if self.state == TURRET_SHOOT_DISABLED then

   elseif self.state == TURRET_SHOOT_SLEEP then

      if (self:getLocY(true) + TURRET_SHOOT_ON_SCREEN_DELTA) < ( self.game.boundary.max.y ) then
         self.state = TURRET_SHOOT_AIM;
      end

   elseif self.state == TURRET_SHOOT_AIM then

      self.shootDelay = self.shootDelay - dt;

      if       self.shootDelay      < 0
			and	self.da
         and   math.abs(self.da)    < self.shootAngleMax
         and   (not self.template.shootPosMax  or self:getLocY(true)   < self.template.shootPosMax) then

         self.state       = TURRET_SHOOT_SHOOT;
         self.shootDelay  = self.template.shootDelay;
         self.bulletCount = 0;

      end

   else

      if self.target == nil then
         self.state = TURRET_SHOOT_AIM;
      else

         self.bulletDelay = self.bulletDelay-dt;

         if self.bulletDelay < 0.001    then
				self:shoot();
         end
      end
   end
end

----------------------------------------------------------------------

function TurretTank:shoot()

	local x,y = self:calcBulletLoc();

	self.bulletDelay = self.template.bulletDelay;
	self.bulletCount = self.bulletCount + 1;

	local p;

	if self.parentProj then
		local poffset = self:calcParentBulletOffset();

		p = self.game.entityMgr:spawnProjectile( self.template.projectile, poffset.x, poffset.y, poffset.angle, self.parent, self.target, self, poffset );
	else
		p = self.game.entityMgr:spawnProjectile( self.template.projectile, x, y, self.angle, self.parent, self.target );
	end

	p.speedBurst   = self.burstOn;
	p.damageAvatar = (p.damageAvatar or 0) * (AbTestMgr:getBossDamageScale(BOSS_INDEX) or 1);

   local bx,by = self.barrel:getLoc(true);

   self.game.entityMgr:spawnExplosion(self.template.muzzleBlastVFX, self.template.muzzleBlastVFX.offset[1], self.template.muzzleBlastVFX.offset[2], 0, self);

	EventMgr:postEvent( {sounds = {start={	"CannonFire",
                                         },}, name="turret shoot"}, "add-sfx" );

	if self.bulletCount  >= self.template.bulletCount  then
		self.state = TURRET_SHOOT_AIM;
	end

end

----------------------------------------------------------------------

function TurretTank:updateVFX( dt )

	if self.glowVFX then

		local countdown;
		local scale			= 0;

		if	self.state and self.state == TURRET_SHOOT_AIM then
			countdown = self.shootDelay + self.bulletDelay;
		else
			countdown = self.bulletDelay;
		end

		if		(self.state and self.state <= TURRET_SHOOT_SLEEP)
			or	not self.shootDelay
			or not self.bulletDelay
			or	not self.glowScale
			or	countdown >= self.glowTime then

			self.glowVFX:hide();

         local lSounds = { loop={"CannonCharge",},};
         EventMgr:postEvent( {sounds=lSounds, name="tankcharge",}, "loop-stop" );

			if self.glowState then
				if self.glowState == TURRET_GLOW_VFX_STOP then
					self.glowState = TURRET_GLOW_VFX_SLEEP;
				elseif self.glowState == TURRET_GLOW_VFX_START then
					self.glowState = TURRET_GLOW_VFX_STOP;
				else
					self.glowState = TURRET_GLOW_VFX_SLEEP;
				end
			end
		else
			scale = math.min((self.glowTime - countdown)/self.glowTime, 1);

			self.glowVFX:show();
			self.glowVFX:setScale(scale*self.glowScale.w, scale*self.glowScale.h);

			if self.glowState then
				if self.glowState ~= TURRET_GLOW_VFX_START then
					self.glowState = TURRET_GLOW_VFX_START;

               local lSounds = { loop={"CannonCharge",},};
               EventMgr:postEvent( {sounds=lSounds, name="tankcharge",}, "add-sfx" );


				end
			end

			if countdown <= 0.15 and self.recoilTime <= 0 then
				self.recoilTime = self.template.recoilTime + self.template.recoilReturnTime;
			end
		end
	end

end

----------------------------------------------------------------------

function TurretTank:updateRecoil( dt )

	if self.barrel then

		if self.recoilTime <= 0 then
			self.barrel:setLoc(self.template.barrel.offset[1], self.template.barrel.offset[2]);

			return true;
		end

		self.recoilTime = self.recoilTime - dt;

		if self.recoilTime > self.template.recoilReturnTime then

			local t	= self.recoilTime - self.template.recoilReturnTime;
			local y	= self.recoilOffset * math.clamp(self.template.recoilTime - t/self.template.recoilTime, 0, 1);

			y = self.template.barrel.offset[2] - y;

			self.barrel:setLoc(self.template.barrel.offset[1], y);
		else
			local y	= self.recoilOffset * math.clamp(self.recoilTime/self.template.recoilReturnTime, 0, 1);

			y = self.template.barrel.offset[2] - y;

			self.barrel:setLoc(self.template.barrel.offset[1], y);
		end

	end

end

----------------------------------------------------------------------

function TurretTank:_setStrafeDirection( event )

	if	self.strafeState and self.strafeState == STRAFE_STATE_IDLE then

		if event.direction == "left" then
			self.strafeAngle = self.template.bodyAngleLimit[2];
		else
			self.strafeAngle = self.template.bodyAngleLimit[1];
		end

		self.strafeState = STRAFE_STATE_STRAFE;
	end

end

----------------------------------------------------------------------

function TurretTank:_setStrafeEase( event )

	if self.strafeState and self.strafeState == STRAFE_STATE_STRAFE then
		self.strafeAngle = 0;
		self.strafeState = STRAFE_STATE_STRAFE_EASE;
	end

end

----------------------------------------------------------------------

function TurretTank:_setStrafeIdle( event )

	if	self.strafeState == nil or self.strafeState == STRAFE_STATE_STRAFE_EASE then
		self.strafeAngle = 0;
		self.strafeState = STRAFE_STATE_IDLE;
	end

end

----------------------------------------------------------------------

function TurretTank:delete()

	Base.delete( self );

	if self.armor then
		self.armor:delete();
	end

	if self.turretShadow then
		self.turretShadow:delete();
	end

	if self.barrel then
		self.barrel:delete();
	end

	if self.barrelShadow then
		self.barrelShadow:delete();
	end

	if self.glowVFX then
		self.glowVFX:delete();
	end

end

----------------------------------------------------------------------

function TurretTank:hide()

	Base.hide(self);

	if self.armor then
		self.armor:hide();
	end

	if self.turretShadow then
		self.turretShadow:hide();
	end

	if self.barrel then
		self.barrel:hide();
	end

	if self.barrelShadow then
		self.barrelShadow:hide();
	end

	if self.glowVFX then
		self.glowVFX:hide();
	end

end

----------------------------------------------------------------------

function TurretTank:show()

	Base.show(self);

	if self.armor then
		self.armor:show();
	end

	if self.turretShadow then
		self.turretShadow:show();
	end

	if self.barrel then
		self.barrel:show();
	end

	if self.barrelShadow then
		self.barrelShadow:show();
	end

	if self.glowVFX then
		self.glowVFX:show();
	end

end

----------------------------------------------------------------------
