require('game/entity/entity');
require('game/entity/healthBar');
require('app/template/bounty/bounty_reticle');
require('app/template/bounty/bounty_portrait');
require('game/entity/turret/turret-shoot');
require('game/entity/turret/turret-tank');
require('game/entity/turret/turret-mine');
require('game/entity/turret/turret-gunship');

----------------------------------------------------------------------

local VEHICLE_HIT_DAMAGE_K = 1/100;

----------------------------------------------------------------------

local Base  = Entity;
Vehicle     = Base:new();

----------------------------------------------------------------------

function Vehicle:new( game, layers, template, x, y, a, type, ... )

print(game, layers);

   if template then

      local m	= Base.new( self,  game, layers[template.layer], template, x, y, a, type, ... );
		local t	= Template.clone(TEMPLATE_BOUNTY_PORTRAIT);
		local f	= Template.clone(TEMPLATE_BOUNTY_PORTRAIT_FRAME);

      m.game  = game;
      m.type  = type;

      if template.shadow then
         m.shadow = Entity:new( game, layers[template.shadow.layer], template.shadow, x, y, a, ... );

			if not template.shadow.noParent then
				m.shadow.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, m.prop, MOAIProp2D.TRANSFORM_TRAIT );
				m.shadow:setLoc( template.shadow.offset[1], template.shadow.offset[2] );
			else
				m.shadow:setLoc( x+template.shadow.offset[1], y+template.shadow.offset[2] );
				m.shadow:hide();
			end

			m.shadow.prop:setPriority(template.shadow.priority or 0);
      end

      if template.collideWithAvatar ~= nil then
         m.collideWithAvatar  =  template.collideWithAvatar;
      else
         m.collideWithAvatar  =  true;
      end

      if template.collideWithVehicle ~= nil then
         m.collideWithVehicle  =  template.collideWithVehicle;
      else
         m.collideWithVehicle  =  true;
      end

		if template.visElement then
			m.visElement = {};

			for i=1,#template.visElement do
				m.visElement[i] = Entity:new(game, layers[template.visElement[i].layer or 3], template.visElement[i], 0, 0, 0, m);
				m.visElement[i].prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, m.prop, MOAIProp2D.TRANSFORM_TRAIT );
				m.visElement[i].prop:setPriority(template.visElement[i].priority or 0);
				m.visElement[i]:setLoc(template.visElement[i].offset[1], template.visElement[i].offset[2]);
			end
		end

		if template.armor then
			m.armor = Entity:new(game, layers[template.armor.layer or template.layer], template.armor, x, y, 0, m);
			m.armor.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, m.prop, MOAIProp2D.TRANSFORM_TRAIT );
			m.armor.prop:setPriority(template.armor.priority or 0);
			m.armor:setLoc( template.armor.offset[1], template.armor.offset[2] );
		end

      if template.turret then
			local factory = template.turret.factory or TurretShoot;

         m.turret = factory:new( game, layers[template.turret.template.layer], template.turret.template, x, y, 180, m, template.turret.offset );
      else
         m.turret = nil;
      end

      if template.healthBar then
         m.healthBar = HealthBar:new( game, layers[template.healthBar.template.layer], template.healthBar.template, x, y, 0, m, template.healthBar.offset );
      else
         m.healthBar = nil;
      end

		if template.bountyPic then
			t.texture = template.bountyPic;

			m.portrait = Entity:new( game, layers[t.layer], t, x, y, a );
			m.portrait.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, m.prop, MOAIProp2D.TRANSFORM_TRAIT );
			m.portrait:setLoc(template.picOffset[1]+t.offset[1], template.picOffset[2]+t.offset[2]);
			m.portrait:setColor(t.color[1], t.color[2], t.color[3], t.color[4]);

			m.frame = Entity:new( game, layers[f.layer], f, x, y, a );
			m.frame.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, m.prop, MOAIProp2D.TRANSFORM_TRAIT );
			m.frame:setLoc(template.picOffset[1]+f.offset[1], template.picOffset[2]+f.offset[2]);
			m.frame:setColor(f.color[1], f.color[2], f.color[3], f.color[4]);

			m.reticle = Entity:new( game, layers[TEMPLATE_BOUNTY_RETICLE.layer], TEMPLATE_BOUNTY_RETICLE, x, y, a );
			m.reticle.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, m.prop, MOAIProp2D.TRANSFORM_TRAIT );
			m.reticle:setLoc(TEMPLATE_BOUNTY_RETICLE.offset[1], TEMPLATE_BOUNTY_RETICLE.offset[2]);
			m.reticle:setColor(TEMPLATE_BOUNTY_RETICLE.color[1], TEMPLATE_BOUNTY_RETICLE.color[2], TEMPLATE_BOUNTY_RETICLE.color[3], TEMPLATE_BOUNTY_RETICLE.color[4]);
		end

 --  m.debugText = game.parent:addWidget( TextBox,               game.parent.layer.ui[1],
 --                                         x,y, 200,  100,
 --                                         "Guest", "resources/font/TangoLatin.otf", 20, nil,
 --                       MOAITextBox.CENTER_JUSTIFY,   MOAITextBox.CENTER_JUSTIFY,   {1,1,1}      );

      m:debugCollision();

      --reset again to make every object are resetted
      m:reset( x, y, a, type, ... );

      return m;
   else

      return Base.new( self,  game );
   end

end

----------------------------------------------------------------------

function Vehicle:delete()

   if self.healthBar then
      self.healthBar:delete();
   end

   if self.shadow then
      self.shadow:delete();
   end

	if self.visElement then
		for i=1,#self.visElement do
			self.visElement[i]:delete();
		end
	end

	if self.armor then
		self.armor:delete();
	end

   if self.turret then
      self.turret:delete();
   end

	if self.reticle then
		self.reticle:delete();
	end

	if self.portrait then
		self.portrait:delete();
	end

	if self.frame then
		self.frame:delete();
	end

   Base.delete(self);

end

----------------------------------------------------------------------

function Vehicle:reset( x, y, angle )

   Base.reset( self, x, y, angle );

   local w = self:getCollisionWidth();

   self.damageTimer				= nil;
   self.minX						= self.game.boundary.min.x + w /2;
   self.maxX						= self.game.boundary.max.x - w /2;
   self.avatarImpulseMinY		= self.template.avatarImpulseMinY or 150;
   self.avatarImpulseMaxY		= self.template.avatarImpulseMaxY or 300
   self.rocketTargetPenalty	= 0;
	self.hitMarkerCooldown		= 0;
	self.flyoutMult				= self.template.flyoutMult;
	self.flyoutScore				= self.template.flyoutScore;
	self.drop						= self.template.drop;
	self.dropIndex					= self.template.dropIndex;
	self.specialDrop				= self.template.specialDrop;
	self.specialDropOdds			= self.template.specialDropOdds;
	self.specialDropIndex		= self.template.specialDropIndex;
	self.specialAdditive			= self.template.specialAdditive;
	self.playDeathAnim			= self.template.playDeathAnim or false;
	self.offScreenInvicible		= self.template.offScreenInvc or false;

   if self.shadow then
		if self.template.shadow.noParent then
			self.shadow:setLoc( x+self.template.shadow.offset[1], y+self.template.shadow.offset[2] );
			self.shadow:hide();
		else
			self.shadow:setLoc( self.template.shadow.offset[1], self.template.shadow.offset[2] );
		end
   end

   if self.healthBar then
      self.healthBar:reset( x, y, 0, self, self.template.healthBar.offset  );
   end

	if self.armor then
		self.armor:setLoc( self.template.armor.offset[1], self.template.armor.offset[2] );
	end

   if self.turret then
      self.turret:reset( x, y, 180, self, self.template.turret.offset  );
   end

end

----------------------------------------------------------------------

function Vehicle:hide()

   Base.hide(self);

   if self.debugText then
      self.debugText:setText("");
   end

   if self.shadow then
      self.shadow:hide();
   end

	if self.visElement then
		for i=1,#self.visElement do
			self.visElement[i]:hide();
		end
	end

	if self.armor then
		self.armor:hide();
	end

   if self.healthBar then
      self.healthBar:hide();
   end

   if self.turret then
      self.turret:hide();
   end

	if self.reticle then
		self.reticle:hide();
	end

	if self.portrait then
		self.portrait:hide();
	end

	if self.frame then
		self.frame:hide();
	end

end

----------------------------------------------------------------------

function Vehicle:show()

   Base.show(self);

   if self.shadow then
      self.shadow:show();
   end

	if self.visElement then
		for i=1,#self.visElement do
			self.visElement[i]:show();
		end
	end

	if self.armor then
		self.armor:show();
	end

   if self.healthBar then
      self.healthBar:show();
   end

   if self.turret then
      self.turret:show();
   end

	if self.reticle then
		self.reticle:show();
	end

	if self.portrait then
		self.portrait:show();
	end

	if self.frame then
		self.frame:show();
	end

end

----------------------------------------------------------------------

function  Vehicle:pause()
end

----------------------------------------------------------------------

function  Vehicle:resume()
end

----------------------------------------------------------------------

function Vehicle:setDeathAnimActive()

	self.playDeathAnim = true;

end

----------------------------------------------------------------------

function Vehicle:setDeathAnimInactive()

	self.playDeathAnim = false;

end

----------------------------------------------------------------------

function  Vehicle:setLoc ( x, y )

   Base.setLoc( self, x, y );

end

----------------------------------------------------------------------

function  Vehicle:setLocX( x )

   Base.setLocX( self, x );

end


----------------------------------------------------------------------

function  Vehicle:setLocY ( y )

   Base.setLocY( self, y );

end


----------------------------------------------------------------------

function  Vehicle:update( dt, avatarSpeedY )

   --hack to make sure the fuel truck keep moving on death
   Base.update( self, dt, avatarSpeedY );

   local x,y = self:getLoc();


   if self.healthBar then
      self.healthBar:update( dt, avatarSpeedY );
   end

   if self.turret then
      self.turret:update( dt, avatarSpeedY, false, true  );
   end
   

   if  x > self.maxX then

      self:setLocX( self.maxX );
      self:setSpeedX(0);
      --self:onDamage( math.abs(self:getImpulseX())*VEHICLE_HIT_DAMAGE_K);
      self:setImpulseX( 0 );

   elseif x < self.minX then

      self:setLocX( self.minX );
      self:setSpeedX(0);
      --self:onDamage( math.abs(self:getImpulseX())*VEHICLE_HIT_DAMAGE_K);
      self:setImpulseX( 0 );

   end

   if ( y + self:getHeight()/2) < self.game.killVehicleY then

      if    (  not self.turret )
         or ( (self.turret:getLocY(true) + self.turret:getHeight()/2) < self.game.killVehicleY) then
         
         EventMgr:postEvent({ enemy=self, x=self:getLocX() }, "enemy-missed");

			if self.template.tangoId then
				EventMgr:postEvent({ Id=self.template.tangoId}, "bounty-vehicle-removed");
			end

         self.deleteMe = true;

      end
      
   elseif ( y - self:getHeight()/2) > self.game.stopVehicleY then

      self:setLoc( x, self.game.stopVehicleY + self:getHeight()/2  );

   elseif y < 0 and y < self.game.avatar:getLocY() then

      --once we are behind the player... just leave (to avoid difficult to see collision and more)
      self:setSpeedY( self:getSpeedY()*0.5 );

   end


   if self.damageTimer then

      self.damageTimer = self.damageTimer + dt;

   end

	if self.reticle and not self:isDead() then
		self:updateReticleSpin(dt);
		self:updateReticleStamp(dt);
	end

	if self.offScreenInvicible then
		self:updateVehcOffScreenInvicible(y);
	end

 --  if self.debugText then
 --     self.debugText:setLoc(x,y);
 --     self.debugText:setText(""..math.floor(self:calcFinalSpeedY ()));
 --  end

end

----------------------------------------------------------------------

function Vehicle:updateVehcOffScreenInvicible( y )

	if not self.spawnSpeed then
		self.spawnSpeed = self:getSpeedY();
	end

	if (y + self.template.height/2) < (App.viewportHeight/2) then
		self.offScreenInvicible		= false;
		self.invicible					= false;
		self.stampTime					= TEMPLATE_BOUNTY_RETICLE.stampTime;

		self:setSpeedY(self.spawnSpeed);

		if self.reticle then
			self.reticle:show();
		end
	else
		self.invicible					= true;

		self:setSpeedY(-250);

		if self.reticle then
			self.reticle:hide();
		end
	end

end

----------------------------------------------------------------------

function Vehicle:updateReticleStamp( dt )

	if self.stampTime and self.reticle then

		self.reticle:show();

		self.stampTime = self.stampTime - dt;

		local scale = self.stampTime/TEMPLATE_BOUNTY_RETICLE.stampTime;

		self.reticle:setScale( 1 + TEMPLATE_BOUNTY_RETICLE.stampScale[1]*scale, 1 + TEMPLATE_BOUNTY_RETICLE.stampScale[1]*scale );

		if self.stampTime <= 0 then
			self.reticle:setScale(1,1);
			self.stampTime = nil;
		end

	end

end

----------------------------------------------------------------------

function Vehicle:updateReticleSpin( dt )

	if self.offScreenInvicible then
		return true;
	end

	if not self.reticleTime then
		self.reticleTime = 0;
	end

	self.reticleTime			= self.reticleTime + dt;

	if self.reticleTime > TEMPLATE_BOUNTY_RETICLE.rotationTime then
		self.reticleTime = 0;
	end

	local rot	= 360 * (self.reticleTime/TEMPLATE_BOUNTY_RETICLE.rotationTime);

	if self.reticle then
		self.reticle:setRot(rot);
	end

end


----------------------------------------------------------------------

function Vehicle:createDebugCollision( layer )

   if self.template.collisionObjects then

      self.collisionProp = MOAIProp2D.new ();

      if self.prop then
         self.collisionProp:setAttrLink ( MOAIProp2D.INHERIT_TRANSFORM, self.prop, MOAIProp2D.TRANSFORM_TRAIT )
      end

      for i,co in ipairs(self.template.collisionObjects) do
         local gfxQuad = MOAIGfxQuad2D.new ()

         gfxQuad:setTexture ( TextureMgr:getTexture("resources/square.png") );
         gfxQuad:setRect (   -co.size[X]/2, -co.size[Y]/2,  co.size[X]/2,  co.size[Y]/2 )

         local prop = MOAIProp2D.new ();

         prop:setLoc(co.delta[X],co.delta[Y]);
         prop:setDeck ( gfxQuad );
         prop:setColor(1,0,0,0.25);
         prop:setPriority ( 0 )
         prop:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA );

         if self.collisionProp then
            prop:setAttrLink ( MOAIProp2D.INHERIT_TRANSFORM, self.collisionProp, MOAIProp2D.TRANSFORM_TRAIT )
         end

         layer = layer or  self.game.parent.layer.game[9];

         layer:insertProp ( prop );

      end
   else

      Base.createDebugCollision( self, layer );

   end
end


----------------------------------------------------------------------

function  Vehicle:isColliding( x, y, cx, cy )

   if self.template.collisionObjects then

      local myX, myY = self:getLoc();
      local x2,  y2  = x-myX, y-myY;

      for i,co in ipairs(self.template.collisionObjects) do

         local coX, coY   = co.delta[X],        co.delta[Y];
         local coCX, coCY	= co.size[X],        co.size[Y];
         local dx,   dy    = math.abs(coX-x2),  math.abs(coY-y2);

         if       (dx < 0.5*(coCX+cx))
            and	(dy < 0.5*(coCY+cy)) then

print("Vehicle:isColliding", myX, myY, x2,  y2, coX, coY, coCX, coCY, dx,   dy );

            return true;
         end

      end

   else

      return Base.isColliding(self, x, y, cx, cy);

   end

end


----------------------------------------------------------------------

function  Vehicle:onCollision( collision )

   Base.onCollision( self, collision  );

   local impX, impY    = collision:getImpulse(self);

   if collision:getOther(self) == self.game.avatar    then

      if  self.game.avatar:isInvicible() then

         self:onDamage(10000000);
         EventMgr:postEvent( {enemy=self}, "boost-kill" );

      else

         local damK = 1;
         if self.template.avatarHitDamage then
            damK = self.template.avatarHitDamage[self.game.armorLevel] or 1;
         end

         self:onDamage(damK*self.hitPoints);

      end
   
      if collision.type == Y then

         if impY > 0 then

print("Vehicle:onCollision-2", math.clamp(impY, self.avatarImpulseMinY,self.avatarImpulseMaxY), self.avatarImpulseMinY,self.avatarImpulseMaxY);
            self.austin:addImpulseY(math.clamp(impY, self.avatarImpulseMinY,self.avatarImpulseMaxY));
         else
print("Vehicle:onCollision-3", math.clamp(impY,-250,-100));
            self.austin:addImpulseY(math.clamp(impY,-250,-100));
         end
         
      else
         
         self.austin:addImpulseX( impX );

      end

   else

      if collision.type == Y then

         if impY > 0 then
            self.austin:addImpulseY( math.clamp(impY,50,250));
         else
            self.austin:addImpulseY( math.clamp(impY,-250,-50));
         end
         
      else
         self.austin:addImpulseX( impX );
      end
      
      self:onDamage( (math.abs(impY)+math.abs(impX))*VEHICLE_HIT_DAMAGE_K );

   end
end

----------------------------------------------------------------------

function  Vehicle:calcDamageK()

   local y     = self:getLocY();
   local h     = self:getHeight();
   local max   = self.game.boundary.max.y;

   return 1-math.clamp((y-max)/(h/2),0,1);

end

----------------------------------------------------------------------

function  Vehicle:onDamage( d )
	
   if not self.invincible then

      local d2 = self:calcDamageK()*d;

      self.damageTimer = self.damageTimer or 0;

      Base.onDamage(self, d2);

      if self.healthBar then
      
         self.healthBar:onDamage( d2 );
      
      end

      return self.damageValue >= self.hitPoints;
   else
      return false;
   end

end

----------------------------------------------------------------------
