require('game/entity/vehicle');

----------------------------------------------------------------------

local Base     = Vehicle;
VehicleChopper = Base:new();

----------------------------------------------------------------------

function VehicleChopper:new( game, layers, template, x, y, a, type, rail )

   local e = Base.new( self,  game, layers, template, x, y, a, type, rail );

   e.blade = Entity:new(   game, layers[template.blade.layer], template.blade, 0,0, 0 );

   e.blade.prop:setAttrLink ( MOAIProp2D.INHERIT_TRANSFORM, e.prop, MOAIProp2D.TRANSFORM_TRAIT );
   e.blade:setLoc(template.blade.delta[1], template.blade.delta[2]);

   if template.shadowBlade then
      e.shadowBlade = Entity:new(   game, layers[template.shadowBlade.layer], template.shadowBlade, 0,0, 0 );
   end

   --reset again, so the blade and everything else is reset
   e:reset( x, y, a, type, rail);

   return e;

end


----------------------------------------------------------------------

function VehicleChopper:reset( x, y, angle, type, rail )

   Base.reset(self, x, y, angle, type, rail);


   if self.blade then
      self.blade:reset();
   end
   
   if self.railActionLoc then
      self.railActionLoc:stop();
   end
   if self.railActionRot then
      self.railActionRot:stop();
   end

   self.railTime     = 0;
   self.rail         = rail;
   self.railIndex    = 1;

   if rail then

      self:setLoc(self.rail[1].x, self.rail[1].y );
      self:setRot(self.rail[1].a );

      self:nextRailPoint();

   end

   if self.shadow then

      self.shadow:setLoc(x+self.template.shadow.offset[1], y+self.template.shadow.offset[2]);
		self.shadow:hide();

      if self.shadowBlade then
         self.shadowBlade.prop:setAttrLink ( MOAIProp2D.INHERIT_TRANSFORM, self.shadow.prop, MOAIProp2D.TRANSFORM_TRAIT );
         self.shadowBlade.prop:setLoc(self.template.shadowBlade.delta[1], self.template.shadowBlade.delta[2]);
			self.shadowBlade:hide();
      end

   end

end


----------------------------------------------------------------------

function VehicleChopper:hide()

   Base.hide(self);

   if self.blade then
      self.blade:hide();
   end

   if self.shadowBlade then
      self.shadowBlade:hide();
   end

end
----------------------------------------------------------------------

function VehicleChopper:show()

   Base.show(self);

   if self.blade then
      self.blade:show();
   end

   if self.shadowBlade then
      self.shadowBlade:show();
   end

end

----------------------------------------------------------------------

function VehicleChopper:pause()

   if self.railActionLoc then
      self.railActionLoc:pause();
   end
   if self.railActionRot then
      self.railActionRot:pause();
   end

end

----------------------------------------------------------------------

function VehicleChopper:resume()

   if self.railActionLoc then
      self.railActionLoc:start();
   end
   if self.railActionRot then
      self.railActionRot:start();
   end
end


----------------------------------------------------------------------

function  VehicleChopper:nextRailPoint()

   self.railTime  = 0;
   self.railIndex = self.railIndex + 1;

   if self.railIndex <= #self.rail then

      local current = self.rail[self.railIndex-1];
      local target  = self.rail[self.railIndex];
      local time    = target.time - current.time;

      self.railActionLoc = self.prop:moveLoc(target.x-self:getLocX(), target.y-self:getLocY(),  time,    target.ease or MOAIEaseType.LINEAR );
      self.railActionRot = self.prop:moveRot(math.clampAngle180(target.a-self:getRot()),        time,    target.ease or MOAIEaseType.LINEAR );

      return true;

   else

      return false;

   end

end

----------------------------------------------------------------------

function  VehicleChopper:update( dt, avatarSpeedY )

   --hack because when updating the entity, it messed with the animation
  -- Base.update( self, dt, avatarSpeedY );

   --Michel... use roadriot to drive the process instead of thi
   local x,y = self.prop:getLoc();
	local rot = self.prop:getRot();

   self.austin:setLoc(x, y);

   if self.healthBar then
      self.healthBar:update( dt, avatarSpeedY );
   end

   if self.blade then

      self.blade:update(dt);

      if self.shadowBlade then
         self.shadowBlade:setRot(self.blade.prop:getRot());
      end
   end

   self.railTime = self.railTime + dt;

   if self.railActionLoc:isDone () and self.railActionRot:isDone () then
      
      if not self:nextRailPoint() then
         self.deleteMe = true;
      end
   end

   if self.collisionProp then
      self.collisionProp:setLoc(self.austin:getLoc());
   end

	if self.shadow then
		self.shadow:show();
		self.shadowBlade:show();
		self.shadow:setLoc(x+self.template.shadow.offset[1], y+self.template.shadow.offset[2]);
		self.shadow.prop:setRot(rot);
	end

end


----------------------------------------------------------------------
