require('game/entity/entity');

----------------------------------------------------------------------

local Base = Entity;
Warning     = Entity:new();

----------------------------------------------------------------------

function Warning:new( game, layers, template, x, y, a, owner, target )

   local e = Base.new( self, game, layers[template.layer], template, x, y, a, owner, target );

   e.delay                 = template.delay                  or 0;
   e.duration              = template.duration               or 10;
   e.changeStateTimeAccelK = template.changeStateTimeAccelK  or 0.9;
   e.changeStateTimeMin    = template.changeStateTimeMin     or 0.2;
   e.sounds                = template.sounds;
   e.soundStarted           = false;

   return e;
   
end

----------------------------------------------------------------------

function Warning:reset( x, y, angle, owner, target )

   Base.reset( self, x, y, angle , owner );

   self.stateTime             = 0;
   self.changeStateTime       = self.template.changeStateTime or 1;
   self.direction             = 1;
   self.owner                 = owner;
   self.target                = target;
   self:setColor(1,1,1,0);
   self.soundStarted           = false;

end

----------------------------------------------------------------------

function Warning:updateColor( dt )

   local downK = self.stateTime/self.changeStateTime;

   if self.direction > 0 then
      downK = 1-downK;
   end

   local colorUp   = self.template.colorUp;
   local colorDown = self.template.colorDown;

   if self.target  then

      colorUp   = self.template.targetColorUp   or self.template.colorUp;
      colorDown = self.template.targetColorDown or self.template.colorDown;

   end

   local r = math.smoothLerp( colorUp[1], colorDown[1], downK );
   local g = math.smoothLerp( colorUp[2], colorDown[2], downK );
   local b = math.smoothLerp( colorUp[3], colorDown[3], downK );
   local a = math.smoothLerp( colorUp[4], colorDown[4], downK );
   
   self:setColor(r,g,b,a);
   
   local scale =  math.smoothLerp( self.template.scaleUp, self.template.scaleDown, downK );
   self:setScale(scale);
   
   self.stateTime = self.stateTime + dt;

   if self.stateTime > self.changeStateTime then

      self.stateTime       = 0;
      self.direction       = -self.direction;
      self.changeStateTime = self.changeStateTime*self.changeStateTimeAccelK;

      self:setLocX(self.owner:getLocX());

      self.changeStateTime = math.max( self.changeStateTime, self.changeStateTimeMin );

   end
end

----------------------------------------------------------------------

function Warning:update ( dt )

   Base.update(self, dt);

   local age = self:getAge();

   if age < self.delay then

      self:hide();

   elseif age < self.duration then

      self:show();

      self:updateColor(dt);

   else

      self:hide();

   end

   return age >= self.duration;

end

----------------------------------------------------------------------

function Warning:stop()

	if self then
		self:setAge(self.duration);
	end

end

----------------------------------------------------------------------

function Warning:isDone()

   return self:getAge() >= self.duration;

end

----------------------------------------------------------------------