----------------------------------------------------------------------
require('engine/object/object')
require('game/game-scenario/game-scenario-tutorial');
require('game/game-scenario/game-scenario-drone-charge-tutorial');
require('game/maps/map01/backgroundCharts');
require('game/maps/map01/hazardCharts');
require('game/maps/map01/projectileCharts');
require('game/maps/map01/scenarioCharts');
require('game/maps/map01/pickupCharts');
require('game/maps/map01/vehicleCharts');
require('app/template/scenario/scenario-tutorial');
require('engine/utility/tableUtil');

----------------------------------------------------------------------

local REVIVE_SLOW_GAMESPEED_K        = 0.75;
local REVIVE_SLOW_GAMESPEED_K_TIME   = 15.0;
local REVIVE_SLOW_GAMESPEED_K_ACCEL  = (1-REVIVE_SLOW_GAMESPEED_K)/REVIVE_SLOW_GAMESPEED_K_TIME;

local HACK_PICK_BOSS_SCENARIO = false;
local HACK_DISABLE_BOSS_SPAWN = false;

-- Modify above, not below.
if not BUILD_DEBUG then
   HACK_PICK_BOSS_SCENARIO = false;
   HACK_DISABLE_BOSS_SPAWN = false;
end

----------------------------------------------------------------------

GameDirector =
{
};

----------------------------------------------------------------------

function GameDirector:new( game )

   local gd = Object.new( self );

   gd.__marker                  =  "game-director";

   gd.game                      = game;
   gd.entityMgr                 = game.entityMgr;
   gd.scenario                  = nil;
   gd.playerLocX                = 0;
   gd.playerLocXTime            = 0;
   gd.updateGameSpeedKEnabled   = false;
   gd.enabled                   = false;

   local level                  = App.data:getCurrentPlayerLevel();
   local game_var               = App.data:getGameDirectorSettings(level);
   local dist_var               = App.data:getGameDistanceScore(level);

   gd.game_speed_k_start        = game_var.GAME_SPEED_START;
   gd.game_speed_k_accel        = game_var.GAME_SPEED_ACCEL;
   gd.game_speed_k_max          = game_var.GAME_SPEED_MAX;

   gd.vehicle_speed_drop        = game_var.VEHICLE_SPEED_DROP;

   gd.vehicle_chart_catchup_k   = game_var.VEHICLE_CHART_CATCHUP;
   gd.vehicle_chart_slow_k      = game_var.VEHICLE_CHART_SLOW;
   gd.vehicle_chart_target      = game_var.VEHICLE_CHART_TARGET;
   gd.vehicle_chart_max         = game_var.VEHICLE_CHART_MAX;

   gd.scenario_chart_k          = game_var.SCENARIO_CHART;

   gd.min_daily_interval        = game_var.MIN_DAILY_INTERVAL;
   gd.min_boss_interval         = game_var.MIN_BOSS_INTERVAL;
   gd.game_distance_score_k     = dist_var.GAME_DISTANCE_SCORE;

   gd.scenario_hazard_timer_min = game_var.SCENARIO_HAZARD_TIMER_MIN;
   gd.scenario_interval_min     = game_var.SCENARIO_INTERVAL_MIN;
   gd.scenario_interval_max     = game_var.SCENARIO_INTERVAL_MAX;

   return gd;

end

----------------------------------------------------------------------

function GameDirector:onActivate()

end

----------------------------------------------------------------------

function GameDirector:onDeactivate()

	if self.scenario then
		self.scenario:onDeactivate();
	end

   self.game            = nil;
   self.entityMgr       = nil;
   self.scenario        = nil;

   EventMgr:rmvListener( self );

end

----------------------------------------------------------------------

function GameDirector:onStarted()

   self.gameSpawner              = self.game.gameSpawner;

   self.scenarioTimer            = 0;
   self.dailyTimer               = 100000;
   self.bossTimer						= 100000;
   self.enabled                  = true;
   self.updateGameSpeedKEnabled  = true;

   local difficulty     = math.clamp(  (App.data:getPlayerWeaponLevel()-1)/14 + DroneMgr:getDroneDifficultyMod(), 0, 1  );

   self.gameSpeedAbK    = ( AbTestMgr:getGameDifficultySpeedK( ) or 100 ) / 100;
   self.playerDamageAbK = ( AbTestMgr:getGameDifficultyDamageK( ) or 100 ) / 100;

   self.game:setPlayerDamageK( self.playerDamageAbK );

   self.reviveSlowGameSpeedK = 1;

   self.gameSpeedKStart    = math.lerp( self.game_speed_k_start[1],    self.game_speed_k_start[2], difficulty );
   self.gameSpeedK         = self.gameSpeedKStart;
   self.gameSpeedKAccel    = math.lerp( self.game_speed_k_accel[1],    self.game_speed_k_accel[2], difficulty );
   self.gameSpeedMax       = math.lerp( self.game_speed_k_max[1],      self.game_speed_k_max[2], difficulty );
   --set it here right away in case we go in tuturial more or something else that would disable
   self.game:setGameSpeedK( self.gameSpeedK*self.gameSpeedAbK );

   local distScoreK        = math.lerp( self.game_distance_score_k[1], self.game_distance_score_k[2], difficulty );
   self.game:setDistanceScoreK( distScoreK );

   self.scenarioHazardTimerMin = math.lerp( self.scenario_hazard_timer_min[1],      self.scenario_hazard_timer_min[2], difficulty );
   self.scenarioIntervalMin    = math.lerp( self.scenario_interval_min[1],          self.scenario_interval_min[2],        difficulty );
   self.scenarioIntervalMax    = math.lerp( self.scenario_interval_max[1],          self.scenario_interval_max[2],        difficulty );
   self.scenarioInterval       = math.lerp(self.scenarioIntervalMin,self.scenarioIntervalMax, math.random());

   self.vehicleSpeedDrop       = math.lerp( self.vehicle_speed_drop[1],   self.vehicle_speed_drop[2],   difficulty );

   self.vehicleChartCatchupK   = math.lerp( self.vehicle_chart_catchup_k[1],   self.vehicle_chart_catchup_k[2],   difficulty );
   self.vehicleChartSlowK      = math.lerp( self.vehicle_chart_slow_k[1],      self.vehicle_chart_slow_k[2],   difficulty );

   self.vehicleChartTarget     =  math.lerp( self.vehicle_chart_target[1],      self.vehicle_chart_target[2],      difficulty );
   self.vehicleChartMax        =  math.lerp( self.vehicle_chart_max[1],         self.vehicle_chart_max[2],         difficulty );

   print("DIRCTOR-CHART", difficulty, self.vehicleChartSkip, self.vehicleChartCatchupK, self.vehicleChartSlowK, self.vehicleChartTarget, self.vehicleChartMax);

   self.gameSpawner:setVehicleSpeedDropStart (   self.vehicleSpeedDrop );

   self.gameSpawner:setVehicleChartCatchupK (   self.vehicleChartCatchupK  );
   self.gameSpawner:setVehicleChartSlowK (      self.vehicleChartSlowK     );
   self.gameSpawner:setVehicleChartTarget (     self.vehicleChartTarget    );
   self.gameSpawner:setVehicleChartMax (        self.vehicleChartMax       );


   self.dailyIntervalMin = math.lerp( self.min_daily_interval[1], self.min_daily_interval[2], difficulty );
   self.bossIntervalMin	 = math.lerp( self.min_boss_interval[1],  self.min_boss_interval[2],  difficulty );

   self.scenarioChartK   = math.lerp( self.scenario_chart_k[1],   self.scenario_chart_k[2],   difficulty );

   print("DAILY", App.data:isDailyActive(),
         App.data:getDailyChallengeGoal(),
         App.data:getDailyChallengeProgress(),
         App.data:getDailyChallengeStreak());


   if App.data:getValue("tutorialDone") ~= TUTORIAL_DONE_NUMBER then

      local version = 2;

      self.scenario = GameScenarioTutorial:new(    self,
                                                   self.game,
                                                   TEMPLATE_SCENARIO_TUTORIAL[version] );
	elseif		self.game.avatar:hasChargeRequiredDrone()
			and	not self.game.avatar:hasManualActivatedDrone()
			and	not App.data:getValue("droneChargeTutorialFinished", false) then

		self.scenario = GameScenarioDroneChargeTutorial:new( self, self.game, {} );

	elseif			self.game.avatar:hasManualActivatedDrone()
				and	not App.data:getValue("droneActivationTutorialFinished", false) then

   end

end

----------------------------------------------------------------------

function GameDirector:onAvatarRevive()

   self.reviveSlowGameSpeedK = REVIVE_SLOW_GAMESPEED_K;

end

----------------------------------------------------------------------

function GameDirector:onAvatarPickedUp( p )

   local tpl = p.tplPowerup;
   if tpl then

      if p.pickupType ~= "powerup_double" then
         EventMgr:postEvent( {sounds=tpl.sounds, name="onAvatarPickedUp:".. p.pickupType,}, "add-sfx" );
      elseif self.game.avatar.firingLoop then

         if HACK_GUN_OR_IMPACT == 0 then
            EventMgr:postEvent( {sounds=self.game.avatar.mainWeapon.sounds, name="Gun Out".. p.pickupType,}, "loop-stop" );
         end
         EventMgr:postEvent( {sounds=tpl.sounds, name="Double In"}, "add-sfx" );
         local tSounds = {start={"DoubleFirePickup",},loop={"DoubleGunLoop",},};
         EventMgr:postEvent( {sounds=tSounds, name="double in",}, "add-sfx" );

      end

   else
      if p.pickupType ~= "powerup_double" then
         EventMgr:postEvent( {sounds=p.sounds, name="onAvatarPickedUp:".. p.pickupType,}, "add-sfx" );
      elseif self.game.avatar.firingLoop then

         if HACK_GUN_OR_IMPACT == 0 then
            EventMgr:postEvent( {sounds=self.game.avatar.mainWeapon.sounds, name="Gun Out".. p.pickupType,}, "loop-stop" );
         end
         EventMgr:postEvent( {sounds=p.sounds, name="Double In".. p.pickupType,}, "add-sfx" );
         local tSounds = {start={"DoubleFirePickup",},loop={"DoubleGunLoop",},};
         EventMgr:postEvent( {sounds=tSounds, name="double in",}, "add-sfx" );
      end

   end
end

----------------------------------------------------------------------

function GameDirector:enable(  )

   self.enabled = true;
end

----------------------------------------------------------------------

function GameDirector:disable(  )

   self.enabled = false;
end

----------------------------------------------------------------------

function GameDirector:enableUpdateSpeedK(  )

   self.updateGameSpeedKEnabled = true;

end

----------------------------------------------------------------------

function GameDirector:disableUpdateSpeedK(  )

   self.updateGameSpeedKEnabled = false;

end

----------------------------------------------------------------------

function GameDirector:updateSpeedK( dt )

   --To avoid slowing down too long, we use a separate k factor
   -- this also avoid reviving multiple time in a row to slow down the game too much
   if self.reviveSlowGameSpeedK < 1 then

      self.reviveSlowGameSpeedK = math.min(self.reviveSlowGameSpeedK + dt*REVIVE_SLOW_GAMESPEED_K_ACCEL,1);

   end

   if self.updateGameSpeedKEnabled then

      local rel      = math.clamp((self.gameSpeedK-self.gameSpeedKStart)
                                 /(self.gameSpeedMax-self.gameSpeedKStart),0,1);

      local timeK    = math.lerp( 1, 0.50, rel);

      self.gameSpeedK = math.min( self.gameSpeedK + timeK*dt*self.gameSpeedKAccel,   self.gameSpeedMax  );
      self.game:setGameSpeedK( self.gameSpeedK*self.gameSpeedAbK*self.reviveSlowGameSpeedK );

   end
end

----------------------------------------------------------------------

function GameDirector:update( dt )

   if self.enabled then

      if self.updateGameSpeedKEnabled then
         self:updateSpeedK( dt );
      end

      if self.scenario then

         self.scenario:update( dt );

         if self.scenario:isDone() then

            self.scenario:onDone();

            self.scenarioTimer = 0;

            if self.scenario.template.factory == GameScenarioDaily then
               self.dailyTimer = 0;
            end

            if self.scenario.template.factory == GameScenarioBoss then
               self.bossTimer = 0;
            end

            self.scenario = nil;

         end

      else

         self.scenarioTimer = self.scenarioTimer+dt;
         self.dailyTimer    = self.dailyTimer+dt;
         self.bossTimer     = self.bossTimer+dt;

         if       self.scenarioTimer                > self.scenarioInterval
            and   self.gameSpawner:getHazardTimer() > self.scenarioHazardTimerMin
            and   (not self.game:isSpeedBurstOn() )                                  then

            local ran = math.random();
            self.scenarioInterval = math.lerp(self.scenarioIntervalMin,self.scenarioIntervalMax, math.random());

            local tplScenario = self:pickScenario();

            if tplScenario then

               --If we pick the daily challenge but the daily challenge is done for today
               -- Or if we had the daily really recently
               -- We are skipping
               if     tplScenario.factory==GameScenarioDaily
                  and ( (not App.data:isDailyActive() ) or (self.dailyTimer< self.dailyIntervalMin)) then
                  print("DAILY CANCELLED", App.data:isDailyActive(), self.dailyTimer, self.dailyIntervalMin);
                  tplScenario = nil;
               end

               -- check if boss has been unlocked and boss not already spawned
               if not HACK_PICK_BOSS_SCENARIO
                  and tplScenario
                  and tplScenario.factory == GameScenarioBoss
                  and (not App.data:isBossUnlocked()
                       or not App.data:isBossSpawnRateValid()
                       or self.bossTimer < self.bossIntervalMin) then

                     tplScenario = nil;
               end

               if     HACK_DISABLE_BOSS_SPAWN
                  and tplScenario
                  and tplScenario.factory == GameScenarioBoss then

                  tplScenario = nil;
               end

               if tplScenario then

                  self.scenario = tplScenario.factory:new( self, self.game, tplScenario );

               end
            end
         end
      end

   end

end

----------------------------------------------------------------

function GameDirector:pickScenario()

   local chart = self:getScenarioChart();

   if (not chart) or (not chart.template) then
      -- Where is our data?!
      return nil;
   end

   if HACK_PICK_BOSS_SCENARIO then
      return chart.template[6][1]
   end

   local chosen = pickWeightedRandom(chart.template, chart.odds);

   if chosen then
      return chosen[math.random(1,#chosen)];
   end

end

----------------------------------------------------------------

function GameDirector:getScenarioChart (  )

   local duration = self.game.duration*self.scenarioChartK;

   for i = 1, #MAP01_SCENARIO_CHART do

      local entry = MAP01_SCENARIO_CHART[i];

      if duration < entry.duration then
         return entry;
      end
   end

   return nil;

end

----------------------------------------------------------------------

function GameDirector:isBossScenarioActive()

   if       self.scenario
      and   self.scenario.template
      and   self.scenario.template.factory == GameScenarioBoss then

      return true;
   end

   return false;

end

----------------------------------------------------------------------
