----------------------------------------------------------------------
require('game/game-scenario/game-scenario');
----------------------------------------------------------------------

local Base = GameScenario;
GameScenarioBarricade = Base:new();

----------------------------------------------------------------------

function GameScenarioBarricade:new( director, game, template )

   local gs = Base.new( self, director, game, template );

   gs.bottomScreen = game.boundary.min.y;

   if template.waitNoVehicle then
      gs.waiting = true;
      gs.spawner:disableSpawn();

   else

      gs:createBarricades();
      gs.waiting = false;
   end


   return gs;

end

----------------------------------------------------------------------

function GameScenarioBarricade:createBarricades(  )
 
   self.barricades = {};

   for i=1,#self.template.barricades do

      local tpl = self.template.barricades[i];

      local b   = self.game.gameSpawner:spawnBarricade( tpl.template, 
                                                        self.game:getLaneLocX(tpl.lane), self.game.boundary.max.y+tpl.deltaY,
                                                        tpl.lane                                                                );

      self.barricades[i] = b;
      
   end
end

----------------------------------------------------------------------

function GameScenarioBarricade:update( dt )

   if not self.done then

      Base.update( self, dt );

      if self.waiting then

         local spawn    = true;
         local vehicles = self.game.entityMgr.vehicles;

         for i=1,#vehicles do

            local v      = vehicles[i];
            local deltaY = self.bottomScreen-v:getTop();
            local speedY = v:calcFinalSpeedY();
            local speedY2= math.sign(speedY)*math.max(math.abs(speedY), 0.01);
            local time   = deltaY/speedY2;

            if v:getTop() > 0 or time > 3.5 then
               spawn = false;
            end
         end

         if spawn then

            self:createBarricades();
            self.waiting  = false;

         end
      else

         self.done = true;

         for i=1,#self.barricades do

            if       (not self.barricades[i]:isDone())
               and   (self.barricades[i]:getMaxY() > self.game.boundary.max.y) then

               self.done = false;

            end
         end
      end
   end
end

----------------------------------------------------------------------

function GameScenarioBarricade:isDone()
   return self.done;
end

----------------------------------------------------------------------

function GameScenarioBarricade:onDeactivate()

end

----------------------------------------------------------------------

function GameScenarioBarricade:onDone()

	self:onDeactivate();

   Base.onDone( self );
   
   if #self.barricades > 2 then
   
      local barricadePass = "RoadBlockPass"
      
      EventMgr:postEvent( {sounds = {start={barricadePass,},}, name="stategame update"}, "add-sfx" );
      
   end

   self.spawner:enableSpawn();
   
   self.spawner:enableAllLanes();

end


----------------------------------------------------------------------
