----------------------------------------------------------------------
require('game/game-scenario/game-scenario');
require('engine/event/event-manager')
require('app/template/boss-vehicles/stats_template_boss')
require('app/template/barricade/barricade_01')
require('app/template/barricade/barricade_02')
require('app/template/barricade/barricade_03')
require('app/template/hazard/hazard-missile-boss')
----------------------------------------------------------------------

local Base        = GameScenario;
GameScenarioBoss	= GameScenario:new();

----------------------------------------------------------------------

local HACK_BOSS_INDEX                  = nil;
local HACK_BOSS_LEVEL                  = nil;
local HACK_BOSS_INVICIBLE              = false;
local HACK_BOSS_USE_TARGET_HITPOINTS   = false;

if not BUILD_DEBUG then
   HACK_BOSS_INDEX                  = nil;
   HACK_BOSS_LEVEL                  = nil;
   HACK_BOSS_USE_TARGET_HITPOINTS   = nil;
   HACK_BOSS_INVICIBLE              = nil;
end

local TOP_OF_SCREEN                 = App.viewportHeight/2;
local DESCEND_SPEED                 = -250;
local DECCELEARTION_TIME            = 1/3;
local BARRICADE_WIDTH_BUFFER        = 64;

local MYSTERY_DROP_KILL_MIN     = 2;
local MYSTERY_DROP_KILL_MAX     = 5;

local WARNING_STATE_START       = 0;
local WARNING_STATE_PHASE_1     = 1;
local WARNING_STATE_PHASE_2     = 2;
local WARNING_STATE_PHASE_3     = 3;
local WARNING_STATE_PHASE_4     = 4;
local WARNING_STATE_PHASE_5     = 5;
local WARNING_STATE_CLEAR       = 6;
local WARNING_PULSE_RATE        = 0.5;
local WARNING_PULSE_TIME        = 0.50;
local WARNING_OVERLAY_PULSE     = 1.5;
local WARNING_OVERLAY_R         = 1.0;
local WARNING_OVERLAY_G         = 0.75;
local WARNING_OVERLAY_B         = 0.0;
local WARNING_OVERLAY_A         = 0.6;
local WARNING_START_TIME        = 0.0;
local WARNING_EXIT_TIME         = 0.25;
local WAIT_CAR_TIME             = 2.00;


local WARNING_START =
   {
      -- stripe
      {x=640,           y=TOP_OF_SCREEN-(TOP_OF_SCREEN/2),              dx =-640,               dy=(TOP_OF_SCREEN/2)*1.5,               time=0.2,},
      
      -- danger text
      {x=-543,          y=TOP_OF_SCREEN-(TOP_OF_SCREEN/2),              dx =466,                dy=(TOP_OF_SCREEN/2)*1.5,               time=0.2,},
      
      -- skull
      {x=404,           y=TOP_OF_SCREEN-(TOP_OF_SCREEN/2),              dx =-188,               dy=(TOP_OF_SCREEN/2)*1.5,               time=0.2,},
      
      -- boss title
      {x=-640,          y=0,                                            dx =640,                dy=TOP_OF_SCREEN*1.5,                   time=0.2,},
   };

local NUMBER_TEXTURE                            =
   {
      "resources/UI/Boss/Decrepit_1.png",
      "resources/UI/Boss/Decrepit_2.png",
      "resources/UI/Boss/Decrepit_3.png",
      "resources/UI/Boss/Decrepit_4.png",
      "resources/UI/Boss/Decrepit_5.png",
      "resources/UI/Boss/Decrepit_6.png",
      "resources/UI/Boss/Decrepit_7.png",
      "resources/UI/Boss/Decrepit_8.png",
      "resources/UI/Boss/Decrepit_9.png",
      "resources/UI/Boss/Decrepit_0.png",
   };

local LEVEL_DIMENSION                   = { x= 154,     y= 38, };
local NUMBER_DIMENSION                  = { x= 42,      y= 38, };

local BOSS_STRAFE_SLEEP                         = 0;
local BOSS_STRAFE_L                             = 1;
local BOSS_STRAFE_R                             = 2;
local BOSS_STRAFE_IDLE                  = 3;
local BOSS_FIGHT_TARGET_TIME  = { 10.00, 15.00 };


----------------------------------------------------------------------

function GameScenarioBoss:new( director, game, template )

   local gs     = GameScenario.new( self, director, game, template );
   local m      = AbTestMgr:getBossMysteryDropRate() or {MYSTERY_DROP_KILL_MIN, MYSTERY_DROP_KILL_MAX};

   if BUILD_DEBUG then
      gs.__marker = "boss-scenario";
   end

   gs.game:disableDurationCounter();

   --make sure we cant spawn anything anymore
   gs.game.gameSpawner:disableSpawn();

   gs.waiting          = true;
   gs.topScreen        = game.boundary.max.y;
   gs.bottomScreen     = game.boundary.min.y;

   local dps           = App.data:calcCurrentPlayerDPS( 1.00, 0.30, 1.00, 1.00 );
   local time          = math.lerp( BOSS_FIGHT_TARGET_TIME[1],BOSS_FIGHT_TARGET_TIME[2], math.random());
   gs.bossHitPointsTgt = dps * time;


   gs.bossIndex    = gs:_pickBoss(template.spawnOdds);
   gs.bossCurLevel = App.data:getBossLevel(gs.bossIndex, gs.bossHitPointsTgt);

   if HACK_BOSS_INDEX then
      gs.bossIndex = HACK_BOSS_INDEX;
   end

   if HACK_BOSS_LEVEL then
      gs.bossCurLevel = HACK_BOSS_LEVEL;
   end

   gs.bossTemplate              = App.data:getBossTemplate(gs.bossIndex, gs.bossCurLevel);
   gs.hitPointsK                = AbTestMgr:getBossHealthScale(gs.bossIndex) or 1;
   gs.targetGameSpeedK          = gs.bossTemplate.targetGameSpeedK or 1.00;
   gs.clearScreenTimer          = gs.bossTemplate.spawnDelay or template.clearScreenTime or 2.5;
   gs.exitScenarioTimer         = template.exitScenarioTime or 2.5;
   gs.spawnLaneClamp            = gs.bossTemplate.spawnLaneLimit or {3, 3};
   gs.stopHeight                = TOP_OF_SCREEN - (gs.bossTemplate.screenClearance or TOP_OF_SCREEN/5);
   gs.gameSpeedRestoreAccel     = gs.exitScenarioTimer/30;
   gs.specialDropMin            = m[1];
   gs.specialDropMax            = m[2];
   gs.encounterTimer            = 0;
   gs.encounterTotalTimer       = 0;

   -- strafe
   gs.strafeState       = BOSS_STRAFE_SLEEP;
   gs.strafeLaneClamp   = gs.bossTemplate.strafeLaneLimit or {3, 3};
   gs.strafeDelayClamp  = gs.bossTemplate.strafeDelayLimit or {1, 1};
   gs.strafeSpeedClamp  = gs.bossTemplate.strafeSpeedMax or {-128, 128};
   gs.strafeAccel       = (gs.bossTemplate.strafeAccel or 128)/30;
   gs.strafeDeccel      = (gs.bossTemplate.strafeDeccel or 128)/30;
   gs.strafeMinToMax    = gs.bossTemplate.strafeMinToMax or false;
   gs.strafeSpeed       = 0;
   gs.strafeTimer       = 0;
   gs.strafeLocClamp    = gs:_getStrafeLocClamp();
   gs.strafeDelayMax    = gs:_getStrafeDelayMax();
   gs.strafeTargetLoc   = gs.game:getLaneLocX(3);

   -- death multiSplosions
   gs.deathTimer        = 0;
   gs.deathVFXCount     = 1;
   gs.deathVFXDesc      = gs.bossTemplate.deathVFX or 
      {
         {
            texture             = 'resources/Explosion/Explosion_04.png',
            width               = 64,
            height              = 64,
            avatarSpeedK        = 0.00,
            angleRnd            = 360,
            growth              = { 8, 8 };
            duration            = 0.5,
            spin                = 180,
            spinRnd             = 90,
            color               = { 0.90, 0.75, 0.00, 1.0 },
            colorRnd            = { 0.10, 0.25, 0.00, 0.5 },
            x                   = 0,
            y                   = 0,
            trigTime            = 0.00,
            endTime             = 0.50,
         },
      };

   gs.deathTimeEnd              = gs:_getDeathAnimEndTime(gs.deathVFXDesc);

   -- boss incoming warning
   gs.warningTimer      = 0;
   gs.overlayTimer      = 0;
   gs.pulseTimer        = 0;
   gs.warningState      = nil;
   gs.warningOverlay    = game.parent:addWidget( Image, game.parent.layer.ui[1], 0,0, App.viewportWidth, App.viewportHeight, "resources/square.png" );
   gs.warningStrip      = game.parent:addWidget( Image, game.parent.layer.ui[1], WARNING_START[1].x, WARNING_START[1].y, 641, 108, "resources/UI/Boss/Boss_Warning_Strip.png" );
   gs.warningText       = game.parent:addWidget( Image, game.parent.layer.ui[1], WARNING_START[2].x, WARNING_START[2].y, 382, 70, "resources/UI/Boss/Boss_Danger_Sign.png" );
   gs.warningSkull      = game.parent:addWidget( Image, game.parent.layer.ui[1], WARNING_START[3].x, WARNING_START[3].y, 167, 157, "resources/UI/Boss/Boss_Skull_Crossbones.png" );
   gs:_setBossText(game);
   gs.warningOverlay:setVisible(false);

   -- boss barricade
   gs.barricade         = {nil, nil, nil, nil, nil};
   gs.barrStartTime     = gs.bossTemplate.barrStartTime or {10, 10};
   gs.barrRepeatDelay   = gs.bossTemplate.barrRepeatDelay or {10, 10};
   gs.barrSpawnInterval = gs:_copyTable(gs.bossTemplate.barrSpawnDelay or {0.25, 0.75});
   gs.barrCountClamp    = gs:_copyTable(gs.bossTemplate.barrCountLimit or {0, 0});
   gs.barrLaneClamp     = gs:_copyTable(gs.bossTemplate.barrLaneLimit or {true, true, true, true, true});
   gs.barrTemplate      = gs.bossTemplate.barrTemplate or TEMPLATE_BARRICADE_01;
   gs.barrLaneBlock     = gs.bossTemplate.barrBossLaneBlock or false;
   gs.barrList          = nil;
   gs.barrTimer         = gs:_getRandomMapped(gs.barrStartTime[1], gs.barrStartTime[2]);
   gs.barrProgressTime  = gs.bossTemplate.barrProgressTime or 30;

   -- boss missiles
   gs.missStartTime     = gs.bossTemplate.missStartTime or {10, 10};
   gs.missRepeatDelay   = gs.bossTemplate.missRepeatDelay or {10, 10};
   gs.missSpawnInterval = gs:_copyTable(gs.bossTemplate.missSpawnDelay or {0.25, 0.75});
   gs.missCountClamp    = gs:_copyTable(gs.bossTemplate.missCountLimit or {0, 0});
   gs.missLaneClamp     = gs:_copyTable(gs.bossTemplate.missLaneLimit or {true, true, true, true, true});
   gs.missTemplate      = gs.bossTemplate.missTemplate or TEMPLATE_BOSS_MISSILE_TEMPLATES[1];
   gs.missList          = nil;
   gs.missTimer         = gs:_getRandomMapped(gs.missStartTime[1], gs.missStartTime[2]);
   gs.missProgressTime  = gs.bossTemplate.missProgressTime or 30;

   gs.reviveCount       = 0;

   EventMgr:addListener( gs, gs.onAvatarKill, "avatar-kill" );
   EventMgr:addListener( gs, gs.onReviveStart, "revive-start" );
   EventMgr:addListener( gs, gs.onReviveStop, "revive-stop" );
   EventMgr:addListener( gs, gs.onEnableDistanceCounter, "enable-distance-counter" );
   EventMgr:addListener( gs, gs.onMysteryTokenSpawned, "pickup-mystery-spawned" );

   return gs;

end

----------------------------------------------------------------------

function GameScenarioBoss:isDone()

   return self.done or false;

end

----------------------------------------------------------------------

function GameScenarioBoss:onDeactivate()

   self:_deleteDeathOverlay();
   self:_deleteWarning();
   self.director:enableUpdateSpeedK();
   self.game.gameSpawner:enableSpawn();
   self.game:enableDistanceCounter();
   self.game:enableDurationCounter();

	self.bossVehicle = nil;

   EventMgr:rmvListener( self );

end

----------------------------------------------------------------------

function GameScenarioBoss:onDone( )

	self:onDeactivate();

	Base.onDone( self );

end

----------------------------------------------------------------------

function GameScenarioBoss:update( dt )

   GameScenario.update( self, dt );

   if self.warningState then
      self:_pulseWarningOverlay(dt);
   end

   if self.warningState and self.warningState ~= WARNING_STATE_CLEAR then
		self:_updateWarning(dt);
   elseif self.warningState and self.warningState == WARNING_STATE_CLEAR then
		self:_exitWarning(dt);
   end

   if not self.game.avatar:isSpeedBurstOn() and not self.bossVehicle then
      self:_updateClearScreen(dt);
   elseif self.game.avatar:isSpeedBurstOn() and not self.bossVehicle then
      self.game.gameSpawner:enableSpawn();
      self.waiting = true;
   end

   if self.bossVehicle then
      self:_updateEncounterTime(dt);
      self:_updateBarricade(dt);
      self:_updateMissile(dt);
      self:_updateBossGameSpeedClamp(dt);
      self:_updateBossSpeed(dt);
      self:_updateBossStrafe(dt);
      self:_updateBossDeathAnim(dt);
      self:_updateBossDeathCount(dt);
   end

   if self.exitScenario then
      self:_updateBossGameSpeedRestore(dt);
      self:_updateExitScenario(dt);
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_updateBossDeathAnim( dt )

   if self.bossVehicle:isDead() and self.bossVehicle.playDeathAnim then

      self.bossVehicle.invicible = true;
      
      if self.bossVehicle.turret then
         self.bossVehicle.turret:disable();
      end

      self:_playDeathVFX();

      self.deathTimer =  self.deathTimer + dt;

      if self.deathTimer >= self.deathTimeEnd then
         self.bossVehicle.invicible = false;
         self.exitScenario          = true;
         self.deathTimer            = 0;

         self.bossVehicle:setDeathAnimInactive();
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_updateBossDeathCount( dt )

   if not self.countDeath and self.bossVehicle:isDead() then

      local count = App.data:getValue("boss-kill-count") or 0;

      self.countDeath = true;

      App.data:setValue("boss-kill-count", count+1);

      EventMgr:postEvent( {}, "boss-kill" );

      self.game:enableDistanceCounter();

      if self.bossVehicle and self.bossVehicle.turret then
         self.bossVehicle.turret:disable();
      end

      if self.game.entityMgr.hazards and #self.game.entityMgr.hazards > 0 then
         for i=1, #self.game.entityMgr.hazards do
            self.game.entityMgr.hazards[i]:selfDestruct();
         end
      end

      local eventGroup = math.floor(self.bossCurLevel / 5);
      MOAIFlurry:eventStop("Boss-Group-"..eventGroup,
                           "level-"..self.bossCurLevel.."-ending",   "boss-death:"..tostring(self.bossTemplate.type),
                           "level-"..self.bossCurLevel.."-time",     tostring(math.floor(self.encounterTotalTimer/5)*5)..":"..tostring(self.bossTemplate.type)   );

   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_updateBossGameSpeedClamp( dt )

   if not self.bossVehicle:isDead() then

      local speed = self.game.gameSpeedK - self.gameSpeedClampAccel;

      if self.game.gameSpeedK > self.targetGameSpeedK then
         speed = math.clamp( speed, self.targetGameSpeedK, self.savedGameSpeedK);
      elseif self.game.gameSpeedK < self.targetGameSpeedK then
         speed = math.clamp( speed, self.savedGameSpeedK, self.targetGameSpeedK);
      else
         speed = self.targetGameSpeedK;
      end

      self.game:setGameSpeedK(speed);
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_updateBossGameSpeedRestore( dt )

   local speed = self.game.gameSpeedK + self.gameSpeedRestoreAccel;

   if self.game.gameSpeedK < self.savedGameSpeedK then
      speed = math.clamp( speed, self.targetGameSpeedK, self.savedGameSpeedK);
   elseif self.game.gameSpeedK > self.savedGameSpeedK then
      speed = math.clamp( speed, self.savedGameSpeedK, self.targetGameSpeedK);
   else
      speed = self.savedGameSpeedK;
   end

   self.game:setGameSpeedK(speed);

end

----------------------------------------------------------------------

function GameScenarioBoss:_updateBossSpeed( dt )

   if not self.bossVehicle:isDead() and self.strafeState == BOSS_STRAFE_SLEEP then

      local xLoc, yLoc  = self.bossVehicle:getLoc();

      if yLoc > self.stopHeight then
         self.bossVehicle:setSpeedY(DESCEND_SPEED);

         if self.bossVehicle.turret then
            self.bossVehicle.turret:disable();
         end
      else
         self.bossVehicle:setSpeedY(0);
         self:onReviveStop();
         self:_initBossTurret();

         self.strafeState = BOSS_STRAFE_IDLE;
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_updateBossStrafe( dt )

   if           self.strafeLaneClamp[1] >= self.strafeLaneClamp[2]
      or self.bossVehicle:isDead()
   or   self.strafeState == BOSS_STRAFE_SLEEP then

      return false;
   end

   local x,y = self.bossVehicle:getLoc();

   self:_checkBarricades(x);

   if self.strafeState == BOSS_STRAFE_L then

      EventMgr:postEvent({X=(self.strafeTargetLoc or x), Y=y, direction="left"}, "boss-strafe-left");

      if (self.strafeTargetLoc - x) >= -1.5 then

         self.strafeSpeed = 0;
         self.strafeState = BOSS_STRAFE_IDLE;
      elseif self:_isInDeccelerationRange(x - self.strafeTargetLoc) then

         EventMgr:postEvent({X=(self.strafeTargetLoc or x), Y=y, direction="left"},"boss-strafe-left-ease");

         self.strafeSpeed = math.clamp(self.strafeSpeed+self.strafeDeccel, self.strafeSpeedClamp[1], -self.strafeDeccel);
      else

         self.strafeSpeed = math.max(self.strafeSpeed-self.strafeAccel, self.strafeSpeedClamp[1]);
      end

      self.bossVehicle:setSpeedX(self.strafeSpeed);
   elseif self.strafeState == BOSS_STRAFE_R then

      EventMgr:postEvent({X=(self.strafeTargetLoc or x), Y=y, direction="right"}, "boss-strafe-right");

      if (self.strafeTargetLoc - x) <= 1.5 then

         self.strafeSpeed = 0;
         self.strafeState = BOSS_STRAFE_IDLE;
      elseif self:_isInDeccelerationRange(self.strafeTargetLoc - x) then

         EventMgr:postEvent({X=(self.strafeTargetLoc or x), Y=y, direction="right"}, "boss-strafe-right-ease");

         self.strafeSpeed = math.clamp(self.strafeSpeed-self.strafeDeccel, self.strafeDeccel, self.strafeSpeedClamp[2]);
      else

         self.strafeSpeed = math.min(self.strafeSpeed+self.strafeAccel, self.strafeSpeedClamp[2]);
      end

      self.bossVehicle:setSpeedX(self.strafeSpeed);
   else

      EventMgr:postEvent({X=(self.strafeTargetLoc or x), Y=y}, "boss-strafe-idle");

      self.strafeTimer = self.strafeTimer + dt;

      if self.strafeTimer >= self.strafeDelayMax then

         self.strafeTargetLoc           = self:_getStrafeTargetLoc();
         self.strafeDelayMax            = self:_getStrafeDelayMax();
         self.strafeTimer                       = 0;
         self.strafeSpeed                       = 0;
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_updateClearScreen( dt )

   if not self:isDone() and self.waiting then

      self.game.gameSpawner:disableSpawn();
      self.director:disableUpdateSpeedK();

      local spawn    = true;
      local vehicles = self.game.entityMgr.vehicles;
      local markers     = MarkerMgr.markers;

      for i=1,#vehicles do

         local v      = vehicles[i];
         local deltaY = self.bottomScreen-v:getTop();
         local speedY = v:calcFinalSpeedY();
         local speedY2= math.sign(speedY)*math.max(math.abs(speedY), 0.01);
         local time   = deltaY/speedY2;

         if time > WAIT_CAR_TIME then
            return false;
         end
      end

      if markers then

         for i=1,#markers do

            local m = markers[i];

            if m:getTop() >= self.bottomScreen and m:getTop() <= self.topScreen then
               return false;
            end
         end
      end

      if not self.warningState then
         self.warningState = WARNING_STATE_START;
      end

      self.clearScreenTimer = self.clearScreenTimer - dt;

      if spawn and not self.game.avatar:isSpeedBurstOn() then

         if self.clearScreenTimer <= 0 then

            self:_spawnBoss();

            self.canSpawn               = self.warningTimer + (self.bossTemplate.warningExitDelay or WARNING_PULSE_TIME);
            self.savedGameSpeedK        = self.game.gameSpeedK;
            self.gameSpeedClampAccel    = (self.savedGameSpeedK - self.targetGameSpeedK)/30;
            self.gameSpeedRestoreAccel  = (self.savedGameSpeedK - self.targetGameSpeedK)*self.gameSpeedRestoreAccel;
            self.waiting                = false;
            self.clearScreenTimer       = nil;
         end
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_updateEncounterTime( dt )

   self.encounterTimer        = self.encounterTimer      + dt;

   if not self.bossVehicle:isDead() then
      self.encounterTotalTimer   = self.encounterTotalTimer + dt;
   end

   if self.encounterTimer >= self.barrProgressTime then
      self:advanceBarrierCount();
   end

   if self.encounterTimer >= self.missProgressTime then
      self:advanceMissileCount();
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_updateExitScenario( dt )

   self.exitScenarioTimer = self.exitScenarioTimer - dt;

   if self.exitScenarioTimer <= 0 then
      self:_exitScenario();
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_updateBarricade( dt )

   if self:_isBarricadeValid() and not self.bossVehicle:isDead() then

      self.barrTimer = self.barrTimer - dt;

      if self.barrTimer <= 0 and (self.barrCountClamp[1]+self.barrCountClamp[2] > 0) then

         if self.barrList == nil
         or (self.barrList and #self.barrList <= 0) then

            self.barrList = self:_createLaneList(self.barrCountClamp, self.barrLaneClamp);
         end

         self:_spawnBarricade();
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_updateMissile( dt )

   if self:_isMissileValid() and not self.bossVehicle:isDead() then

      self.missTimer = self.missTimer - dt;

      if self.missTimer <= 0 and (self.missCountClamp[1]+self.missCountClamp[2] > 0) then

         if             self.missList == nil
         or (self.missList and #self.missList <= 0) then

            self.missList = self:_createLaneList(self.missCountClamp, self.missLaneClamp);

            EventMgr:postEvent( { sounds={ start={"MissileWarning",}, }, name="Game Scenario Hazard:update",}, "add-sfx" );
         end

         self:_spawnMissile();
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_updateWarning( dt )

   if self.warningState == WARNING_STATE_START then
      self:_initWarningState(dt);
      EventMgr:postEvent( { sounds={ start={"BossIntroWarning",}, }, name="Tank Intro",}, "add-sfx" );
   elseif self.warningState == WARNING_STATE_PHASE_1 then
      self:_moveWarningStrip(dt);
   elseif self.warningState == WARNING_STATE_PHASE_2 then
      self:_moveWarningSkull(dt);
   elseif self.warningState == WARNING_STATE_PHASE_3 then
      self:_moveWarningText(dt);
   elseif self.warningState == WARNING_STATE_PHASE_4 then
      self:_moveBossLogo(dt);
   else
      self:_pulseWarningText(dt);
   end

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function GameScenarioBoss:_isBarricadeValid()

   if           self.barrStartTime[1] < 0 or self.barrStartTime[1] > self.barrStartTime[2]
      or        self.barrRepeatDelay[1] < 0 or self.barrRepeatDelay[1] > self.barrRepeatDelay[2]
      or self.barrSpawnInterval[1] < 0 or self.barrSpawnInterval[1] > self.barrSpawnInterval[2]
   or self.barrCountClamp[1] < 0 or self.barrCountClamp[1] > self.barrCountClamp[2] then

      return false;
   end

   for i=1,5 do

      if self.barrLaneClamp[i] then
         return true;
      end
   end

   return false;

end

----------------------------------------------------------------------

function GameScenarioBoss:_isMissileValid()

   if           self.missStartTime[1] < 0 or self.missStartTime[1] > self.missStartTime[2]
      or        self.missRepeatDelay[1] < 0 or self.missRepeatDelay[1] > self.missRepeatDelay[2]
      or self.missSpawnInterval[1] < 0 or self.missSpawnInterval[1] > self.missSpawnInterval[2]
   or self.missCountClamp[1] < 0 or self.missCountClamp[1] > self.missCountClamp[2] then

      return false;
   end

   for i=1,5 do

      if self.missLaneClamp[i] then
         return true;
      end
   end

   return false;

end

----------------------------------------------------------------------

function GameScenarioBoss:_spawnBarricade()

   if self.barrList and #self.barrList > 0 then

      if self.barrLaneBlock then

         local loc                              = self.game:getLaneLocX(self.barrList[1]);
         local blockedLanes     = self:_getBlockedLanes();

         if loc >= blockedLanes[1] and loc <= blockedLanes[2] then
            return false;
         end

         if self.strafeState == BOSS_STRAFE_L then
            if loc < self.strafeTargetLoc then
               return false;
            end
         elseif self.strafeState == BOSS_STRAFE_R then
            if loc > self.strafeTargetLoc then
               return false;
            end
         end
      end

      if not self.countDeath then
         self.barricade[self.barrList[1]] = self.game.gameSpawner:spawnBarricade(       self.barrTemplate,
                                                                                        self.game:getLaneLocX(self.barrList[1]),
                                                                                        self.game.boundary.max.y,
                                                                                        self.barrList[1]
         );
      end

      table.remove(self.barrList, 1);

      if        #self.barrList <= 0 then
         self.barrTimer = self:_getRandomMapped(self.barrRepeatDelay[1], self.barrRepeatDelay[2]);
      else
         self.barrTimer = self:_getRandomMapped(self.barrSpawnInterval[1], self.barrSpawnInterval[2]);
      end
   else
      self.barrList = nil;
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_spawnMissile()

   if self.missList and #self.missList > 0 then

      if not self.countDeath then
         self.game.gameSpawner:spawnHazard(     self.missTemplate,
                                                self.game:getLaneLocX(self.missList[1]),
                                                0,
                                                nil
         );
      end

      table.remove(self.missList, 1);

      if        self.missList and #self.missList <= 0 then
         self.missTimer = self:_getRandomMapped(self.missRepeatDelay[1], self.missRepeatDelay[2]);
      else
         self.missTimer = self:_getRandomMapped(self.missSpawnInterval[1], self.missSpawnInterval[2]);
      end
   else
      self.missList = nil;
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:advanceBarrierCount()

   self.barrRepeatDelay[1]              = math.max(self.barrRepeatDelay[1] - 1, 3.5);
   self.barrRepeatDelay[2]              = math.max(self.barrRepeatDelay[2] - 1, 3.5);
   self.barrSpawnInterval[1]    = math.max(self.barrSpawnInterval[1] - 0.15, 0.30);
   self.barrSpawnInterval[2]    = math.max(self.barrSpawnInterval[2] - 0.15, 0.50);
   self.barrCountClamp[1]               = math.min(self.barrCountClamp[1] + 1, 5);
   self.barrCountClamp[2]               = math.min(self.barrCountClamp[2] + 1, 5);
   self.barrLaneClamp                   = {true, true, true, true, true};
   if self.encounterTotalTimer <= 20 then
      self.barrProgressTime             = math.max(self.barrProgressTime - 1, 4);
   else
      self.barrProgressTime             = math.max(self.barrProgressTime - 1, 5);
   end
   self.encounterTimer                  = 0;

end

----------------------------------------------------------------------

function GameScenarioBoss:resetBarrierCount()

   self.barrRepeatDelay                         = self:_copyTable(self.bossTemplate.barrRepeatDelay or {10, 10});
   self.barrSpawnInterval               = self:_copyTable(self.bossTemplate.barrSpawnDelay or {0.25, 0.75});
   self.barrCountClamp                  = self:_copyTable(self.bossTemplate.barrCountLimit or {0, 0});
   self.barrLaneClamp                   = self:_copyTable(self.bossTemplate.barrLaneLimit or {true, true, true, true, true});
   self.barrProgressTime                = self.bossTemplate.barrProgressTime or 30;
   self.encounterTimer                  = 0;

end

----------------------------------------------------------------------

function GameScenarioBoss:advanceMissileCount()

   self.missRepeatDelay[1]              = math.max(self.missRepeatDelay[1] - 1, 3.5);
   self.missRepeatDelay[2]              = math.max(self.missRepeatDelay[2] - 1, 3.5);
   self.missSpawnInterval[1]    = math.max(self.missSpawnInterval[1] - 0.15, 0);
   self.missSpawnInterval[2]    = math.max(self.missSpawnInterval[2] - 0.15, 0.35);
   self.missCountClamp[1]               = math.min(self.missCountClamp[1] + 1, 5);
   self.missCountClamp[2]               = math.min(self.missCountClamp[2] + 1, 5);
   self.missLaneClamp                   = {true, true, true, true, true};
   if self.encounterTotalTimer <= 20 then
      self.missProgressTime             = math.max(self.missProgressTime - 1, 4);
   else
      self.missProgressTime             = math.max(self.missProgressTime - 1, 5);
   end
   self.encounterTimer                  = 0;

end

----------------------------------------------------------------------

function GameScenarioBoss:resetMissileCount()

   self.missRepeatDelay                         = self:_copyTable(self.bossTemplate.missRepeatDelay or {10, 10});
   self.missSpawnInterval               = self:_copyTable(self.bossTemplate.missSpawnDelay or {0.25, 0.75});
   self.missCountClamp                  = self:_copyTable(self.bossTemplate.missCountLimit or {0, 0});
   self.missLaneClamp                   = self:_copyTable(self.bossTemplate.missLaneLimit or {true, true, true, true, true});
   self.missProgressTime                = self.bossTemplate.missProgressTime or 30;
   self.encounterTimer                  = 0;

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function GameScenarioBoss:_checkBarricades( loc )

   if not self.barrLaneBlock then
      return false;
   end

   local startLane = self.game:getClosestLaneTo(loc);

   if self.strafeState == BOSS_STRAFE_L then

      startLane = math.clamp(startLane-1, 1, 5);

      for i=startLane,1,-1 do

         if                     self.barricade
            and         self.barricade[i]
            and         not self.barricade[i]:isDone()
         and    math.clamp(self.game:getLaneLocX(i), self.strafeLocClamp[1], self.strafeLocClamp[2]) >= self.strafeTargetLoc then

            self.strafeTargetLoc = math.clamp(self.game:getLaneLocX(math.clamp(i+1,1,5)), self.strafeLocClamp[1], self.strafeLocClamp[2]);
         end
      end
   elseif self.strafeState == BOSS_STRAFE_R then

      startLane = math.clamp(startLane+1, 1, 5);

      for i=startLane,5 do

         if                     self.barricade
            and         self.barricade[i]
            and         not self.barricade[i]:isDone()
         and    math.clamp(self.game:getLaneLocX(i), self.strafeLocClamp[1], self.strafeLocClamp[2]) <= self.strafeTargetLoc then

            self.strafeTargetLoc = math.clamp(self.game:getLaneLocX(math.clamp(i-1,1,5)), self.strafeLocClamp[1], self.strafeLocClamp[2]);
         end
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_copyTable( tbl )

   local temp = {};

   for k,v in pairs(tbl) do
      temp[k] = v;
   end

   return temp;

end

----------------------------------------------------------------------

function GameScenarioBoss:_createLaneList( objects, lanes )

   local objCount               = math.random(objects[1], objects[2]);
   local list                   = {};
   local temp                   = {}
   local count                  = 0;

   -- make list of lanes
   for i=1,5 do

      if lanes[i] then
         table.insert(list,i);
      end
   end

   local maxCount               = math.min(objCount, #list);

   -- shuffle list of lanes
   for i=1,#list do

      local rand        = math.random(i,#list);
      local temp        = list[rand];

      list[rand]        = list[i];
      list[i]           = temp;
   end

   -- return the max count
   for i=1,maxCount do
      temp[i] = list[i];
   end

   return temp;

end

----------------------------------------------------------------------

function GameScenarioBoss:_exitScenario()

   self.done = true;

end

----------------------------------------------------------------------

function GameScenarioBoss:_getBlockedLanes()

   local x,y = self.bossVehicle:getLoc();
   local w   = self.bossTemplate.collisionWidth/2;

   if self.strafeState ~= BOSS_STRAFE_SLEEP then
      w = w + math.max(math.abs(self.strafeSpeed), BARRICADE_WIDTH_BUFFER)/self.targetGameSpeedK;
   end

   return {x-w, x+w};

end

----------------------------------------------------------------------

function GameScenarioBoss:_getRandomMapped( start, stop )

   return math.mapClamped( 0, start, 1, stop, math.random() );

end

----------------------------------------------------------------------

function GameScenarioBoss:_getSpawnLaneLoc()

   local lane   = math.random(self.spawnLaneClamp[1], self.spawnLaneClamp[2]);
   local loc    = math.clamp(self.game:getLaneLocX(lane), self.strafeLocClamp[1], self.strafeLocClamp[2]);

   return loc;

end

----------------------------------------------------------------------

function GameScenarioBoss:_pickBoss( tbl )

   local choicesTbl = tbl;

   if choicesTbl then

      local pick  = nil;
      local total = 0;

      for i,choice in ipairs(choicesTbl) do
         local value = i;
         local odd   = choice;

         if value and odd and odd > 0 then
            total = total + odd;

            if math.random() <= math.min(odd/total, 1) then
               pick = value;
            end
         end
      end

      if not pick then
         self:_exitScenario();

         return 1;
      else
         return pick;
      end
   else
      self:_exitScenario();

      return 1;
   end

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function GameScenarioBoss:_isInDeccelerationRange( dist )

   if dist <= 0 then

      return false;
   end

   local vi     = math.abs(self.strafeSpeed);
   local        d       = (vi*DECCELEARTION_TIME) + 0.5*( -self.strafeDeccel*(DECCELEARTION_TIME*DECCELEARTION_TIME) );

   if dist <= d then

      return true;
   end

   return false;

end

----------------------------------------------------------------------

function GameScenarioBoss:_getStrafeDelayMax()

   return math.mapClamped(0, self.strafeDelayClamp[1], 1, self.strafeDelayClamp[2], math.random());

end

----------------------------------------------------------------------

function GameScenarioBoss:_getStrafeLocClamp()

   local width  = self.bossTemplate.collisionWidth/2;

   if self.bossTemplate.widthClearance then
      width = self.bossTemplate.widthClearance;
   end

   local left   = self.game:getLaneLocX(1)-width;
   local right  = self.game:getLaneLocX(5)+width;

   if left < -App.viewportWidth/2 then
      left = -App.viewportWidth/2+width;
   end

   if right > App.viewportWidth/2 then
      right = App.viewportWidth/2-width;
   end

   left         = math.max(self.game:getLaneLocX(self.strafeLaneClamp[1]), left);
   right        = math.min(self.game:getLaneLocX(self.strafeLaneClamp[2]), right);

   if           left >= right
   or right <= left then

      self.strafeLaneClamp[1] = self.strafeLaneClamp[2];
   end

   return {left, right};

end

----------------------------------------------------------------------

function GameScenarioBoss:_getStrafeTargetLoc( )

   local lane   = math.random(self.strafeLaneClamp[1], self.strafeLaneClamp[2]);
   local loc    = math.clamp(self.game:getLaneLocX(lane), self.strafeLocClamp[1], self.strafeLocClamp[2]);

   if self.strafeMinToMax then
      loc = math.clamp(self.game:getLaneLocX(self.strafeLaneClamp[1]), self.strafeLocClamp[1], self.strafeLocClamp[2]);

      if loc == self.strafeTargetLoc then
         loc = math.clamp(self.game:getLaneLocX(self.strafeLaneClamp[2]), self.strafeLocClamp[1], self.strafeLocClamp[2]);
      end
   end

   while loc == self.strafeTargetLoc do
      lane      = math.random(self.strafeLaneClamp[1], self.strafeLaneClamp[2]);
      loc       = math.clamp(self.game:getLaneLocX(lane), self.strafeLocClamp[1], self.strafeLocClamp[2]);
   end

   if loc > self.strafeTargetLoc then

      self.strafeState = BOSS_STRAFE_R;
   else

      self.strafeState = BOSS_STRAFE_L;
   end

   return loc;

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function GameScenarioBoss:_deleteDeathOverlay()

   if self.deathOverlay then
      self.deathOverlay:delete();
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_getDeathAnimEndTime( template )

   return template[#template].endTime;

end

----------------------------------------------------------------------

function GameScenarioBoss:_playDeathVFX()

   if self.deathVFXCount <= #self.deathVFXDesc then

      for i=self.deathVFXCount,#self.deathVFXDesc do

         if self.deathTimer >= self.deathVFXDesc[i].trigTime then
            self.deathVFXCount = self.deathVFXCount + 1;

            local splosion;

            if self.deathVFXDesc[i].disableParent then
               local x,y = self.bossVehicle:getLoc();

               splosion = self.game.entityMgr:spawnExplosion( self.deathVFXDesc[i], self.deathVFXDesc[i].x+x, self.deathVFXDesc[i].y+y, 0 );
            else
               splosion = self.game.entityMgr:spawnExplosion( self.deathVFXDesc[i], self.deathVFXDesc[i].x, self.deathVFXDesc[i].y, 0, self.bossVehicle );
            end

            if                  self.deathVFXDesc[i].impulseVector
               and      self.deathVFXDesc[i].impulseVector.x
            and         self.deathVFXDesc[i].impulseVector.y then

               splosion:addImpulse(self.deathVFXDesc[i].impulseVector.x, self.deathVFXDesc[i].impulseVector.y);
            end

            if self.deathVFXDesc[i].postEvent then
               EventMgr:postEvent({}, self.deathVFXDesc[i].postEvent);
            end
         end
      end
   end

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function GameScenarioBoss:_deleteWarning()

   if self.warningState then
      self.warningState = WARNING_STATE_CLEAR;
   end

   if self.warningStrip then
      self.warningStrip:delete();
   end

   if self.warningText then
      self.warningText:delete();
   end

   if self.warningSkull then
      self.warningSkull:delete();
   end

   if self.warningLogo then
      self.warningLogo:delete();
   end

   if self.warningLvlTxt then
      self.warningLvlTxt:delete();
   end

   if self.warningLevel then
      for i=1,#self.warningLevel do
         self.warningLevel[i]:delete();
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_exitWarning( dt )

   local dy     = 0;
   local y      = 0;

   if self.warningTimer then
      self.warningTimer = self.warningTimer + dt;

      dy = WARNING_START[1].y + (self.warningTimer/WARNING_EXIT_TIME)*WARNING_START[1].dy;
      y         = WARNING_START[1].y-((self.bossTemplate.warningLogo.height/2)-54-28)+ (self.warningTimer/WARNING_EXIT_TIME)*WARNING_START[4].dy;
   end

   if self.warningStrip then
      self.warningStrip:setLocY(dy);
      self.warningStrip:setScale(1,1);
   end

   if self.warningText then
      self.warningText:setLocY(dy);
      self.warningText:setScale(1,1);
   end

   if self.warningSkull then
      self.warningSkull:setLocY(dy);
      self.warningSkull:setScale(1,1);
   end

   if self.warningLogo then
      self.warningLogo:setLocY(y);
      self.warningLogo:setScale(1,1);
   end

   if self.warningTimer and self.warningTimer > WARNING_EXIT_TIME then
      self:_deleteWarning();
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_initWarningState( dt )

   self.warningTimer = self.warningTimer + dt;

   if self.warningTimer >= WARNING_START_TIME then
      self.warningState = WARNING_STATE_PHASE_1;
      self.warningTimer = 0;
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_moveBossLogo(dt)

   if self.warningLogo then

      if self.warningTimer < WARNING_START[4].time then
         self.warningTimer = self.warningTimer + dt;

         self.warningLogo:setVisible(true);
         self.warningLvlTxt:setVisible(true);

         if self.warningLevel then
            for i=1,#self.warningLevel do
               self.warningLevel[i]:setVisible(true);
            end
         end

         local time     = self.warningTimer/WARNING_START[4].time;
         local scale    = 11-math.lerp(1,10, time);
         local alpha    = math.clamp(time,0,1);

         self.warningLogo:setScale(scale, scale);
         self.warningLogo:setColor(1,1,1, alpha);
      else
         self.warningState = WARNING_STATE_PHASE_5;
         self.warningTimer = 0;

         self.warningLogo:setScale(1, 1);
         self.warningLogo:setColor(1,1,1,1);
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_moveWarningStrip(dt)

   if self.warningStrip then

      if self.warningTimer < WARNING_START[1].time then                         
         self.warningTimer = self.warningTimer + dt;

         local dx = (self.warningTimer/WARNING_START[1].time)*WARNING_START[1].dx;

         self.warningStrip:setLocX(math.clamp(WARNING_START[1].x+dx, WARNING_START[1].x, WARNING_START[1].x+WARNING_START[1].dx));
      else
         self.warningState = WARNING_STATE_PHASE_2;
         self.warningTimer = 0;

         self.warningStrip:setLocX(WARNING_START[1].x+WARNING_START[1].dx);
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_moveWarningText(dt)

   if self.warningText then

      if self.warningTimer < WARNING_START[2].time then
         self.warningTimer = self.warningTimer + dt;

         local dx = (self.warningTimer/WARNING_START[2].time)*WARNING_START[2].dx;

         self.warningText:setLocX(math.clamp(WARNING_START[2].x+dx, WARNING_START[2].x, WARNING_START[2].x+WARNING_START[2].dx));
      else
         self.warningState = WARNING_STATE_PHASE_4;
         self.warningTimer = 0;

         self.warningText:setLocX(WARNING_START[2].x+WARNING_START[2].dx);
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_moveWarningSkull(dt)

   if self.warningSkull then

      if self.warningTimer < WARNING_START[3].time then
         self.warningTimer = self.warningTimer + dt;

         local dx = (self.warningTimer/WARNING_START[3].time)*WARNING_START[3].dx;

         self.warningSkull:setLocX(math.clamp(WARNING_START[3].x+dx, WARNING_START[3].x+WARNING_START[3].dx, WARNING_START[3].x));
      else
         self.warningState = WARNING_STATE_PHASE_3;
         self.warningTimer = 0;

         self.warningSkull:setLocX(WARNING_START[3].x+WARNING_START[3].dx);
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_pulseWarningOverlay(dt)

   if self.warningOverlay then

      local scale = 1.0;
      local t           = WARNING_OVERLAY_PULSE/2;

      self.overlayTimer = self.overlayTimer + dt;
      self.warningOverlay:setVisible(true);

      if self.overlayTimer < t then
         scale = (self.overlayTimer/t)*WARNING_OVERLAY_A;

      else
         scale = WARNING_OVERLAY_A - ((self.overlayTimer-t)/t)*WARNING_OVERLAY_A;

         if self.overlayTimer >= WARNING_OVERLAY_PULSE then
            self.overlayTimer = 0;

            if self.warningState == WARNING_STATE_CLEAR then
               self.warningOverlay:delete();
               self.warningOverlay = nil;

               return false;
            end
         end
      end

      scale = math.max(scale, 0);
      self.warningOverlay:setColor(WARNING_OVERLAY_R, WARNING_OVERLAY_G, WARNING_OVERLAY_B, scale);
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_pulseWarningText(dt)

   if self.warningText then

      local scale       = 1.0;
      local t           = WARNING_PULSE_RATE/2;

      self.pulseTimer           = self.pulseTimer + dt;
      self.warningTimer                 = self.warningTimer + dt;

      if self.canSpawn and self.warningTimer > self.canSpawn then
         self.warningState = WARNING_STATE_CLEAR;
         self.warningTimer = 0;

         return false;
      end

      if self.pulseTimer < t then

         scale = scale - ( (self.pulseTimer/t)*0.5);
      else
         scale = 0.5 + ( ((self.pulseTimer-t)/t)*0.5);

         if self.pulseTimer >= WARNING_PULSE_RATE then
            self.pulseTimer = 0;
         end
      end

      self.warningText:setScale(scale, scale);
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_setBossText( game )

   local level          = self.bossCurLevel;
   local count          = 1;
   local div            = 1;
   local digit          = {};
   local offsetY        = self.bossTemplate.levelTxtOffsetY or LEVEL_DIMENSION.y;

   if level > 999 then
      count     = 4;
      div       = 1000;
   elseif level > 99 then
      count     = 3;
      div       = 100;
   elseif level > 9 then
      count     = 2;
      div       = 10;
   end

   for i=1,count do
      digit[i]  = math.max(math.floor(level/div), 0);
      level             = level - (digit[i]*div);
      div               = math.max(1, div/10);
   end

   local tX = (-(LEVEL_DIMENSION.x + NUMBER_DIMENSION.x*(count+1))/2) + (LEVEL_DIMENSION.x/2);
   local nX = tX + (LEVEL_DIMENSION.x/2) + (NUMBER_DIMENSION.x*1.5);

   self.warningLogo             = game.parent:addWidget(Image, game.parent.layer.ui[1], 0, WARNING_START[1].y-(self.bossTemplate.warningLogo.height+108+64)/2, self.bossTemplate.warningLogo.width, self.bossTemplate.warningLogo.height, self.bossTemplate.warningLogo.texture);
   self.warningLvlTxt   = game.parent:addWidget(Image, game.parent.layer.ui[1], tX, (-self.bossTemplate.warningLogo.height/2)-offsetY, LEVEL_DIMENSION.x, LEVEL_DIMENSION.y, "resources/UI/Boss/Decrepit_Level.png");

   self.warningLvlTxt.prop:setAttrLink(MOAIProp2D.INHERIT_TRANSFORM, self.warningLogo.prop, MOAIProp2D.TRANSFORM_TRAIT);
   self.warningLogo:setVisible(false);
   self.warningLvlTxt:setVisible(false);

   self.warningLevel    = {};

   for i=1,count do
      if digit[i] > 0 then
         self.warningLevel[i]   = game.parent:addWidget(Image, game.parent.layer.ui[1], nX+((i-1)*NUMBER_DIMENSION.x), (-self.bossTemplate.warningLogo.height/2)-offsetY, NUMBER_DIMENSION.x, NUMBER_DIMENSION.y, NUMBER_TEXTURE[digit[i]]);
      else
         self.warningLevel[i]   = game.parent:addWidget(Image, game.parent.layer.ui[1], nX+((i-1)*NUMBER_DIMENSION.x), (-self.bossTemplate.warningLogo.height/2)-offsetY, NUMBER_DIMENSION.x, NUMBER_DIMENSION.y, NUMBER_TEXTURE[10]);
      end

      self.warningLevel[i].prop:setAttrLink(MOAIProp2D.INHERIT_TRANSFORM, self.warningLogo.prop, MOAIProp2D.TRANSFORM_TRAIT);
      self.warningLevel[i]:setVisible(false);
   end

end

----------------------------------------------------------------------
----------------------------------------------------------------------

function GameScenarioBoss:onAvatarKill( event, type )

   if event.monster == self.bossVehicle then
      EventMgr:rmvListener( self.bossVehicle.turret );

      if not self.bossVehicle.template.playDeathAnim then
         self.exitScenario = true;
      end
   end
   
end

----------------------------------------------------------------------

function GameScenarioBoss:onEnableDistanceCounter()

   self.game:enableDistanceCounter();

end

----------------------------------------------------------------------

function GameScenarioBoss:_resetBossEncounter()

   if self.bossVehicle then
      self.bossVehicle.invicible = true;
      self:resetBarrierCount();
      self:resetMissileCount();

      if self.bossVehicle.turret then
         self.bossVehicle.turret:disable();
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:onReviveStart()

   self.reviveCount = self.reviveCount + 1;
   MOAIFlurry:event( "Boss-Revive",
                     "count",          tostring(self.reviveCount),
                     "level",          tostring(self.bossCurLevel),
                     "time",           tostring(math.floor(self.encounterTotalTimer)));

   self:_resetBossEncounter();

end

----------------------------------------------------------------------

function GameScenarioBoss:onReviveStop()

   if self.bossVehicle then
      self.bossVehicle.invicible = false;

      if self.bossVehicle.turret and not self.bossVehicle:isDead() then
         self.bossVehicle.turret:enable();
      end
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:onMysteryTokenSpawned()

   App.data:setValue("boss-kill-count", 0);

end

----------------------------------------------------------------------

function GameScenarioBoss:_initBossTurret()

   if                   not self.initTurret
      and       self.bossVehicle
   and  not self.bossVehicle:isDead() then

      self.initTurret = true;
      App.data:setValue("boss-games-since-encounter", 0);

      if self.bossVehicle.turret.shootDelay then
         self.bossVehicle.turret.shootDelay = self.bossTemplate.initShootDelay or 0;
      end

      if self.bossVehicle.turret.attackDelay then
         self.bossVehicle.turret.attackDelay = self.bossTemplate.initShootDelay or 0;
      end

   end

end

----------------------------------------------------------------------

function GameScenario:_setMysteryToken()

   local count = App.data:getValue("boss-kill-count") or 0;

   if count < self.specialDropMin then
      self.bossVehicle.specialDropOdds   = 0;
   end

   if count >= self.specialDropMax then
      self.bossVehicle.specialDropOdds   = 1;
   end

end

----------------------------------------------------------------------

function GameScenarioBoss:_spawnBoss()

   self.game:disableDistanceCounter();

   self.bossVehicle = self.game.entityMgr:spawnVehicle( self.bossTemplate,
                                                        self.game:getLaneLocX(3),
                                                        self.game.spawnVehicleY + self.bossTemplate.height/2,
                                                        0,
                                                        2,
                                                        3 );

   self.bossVehicle:setLoc(self:_getSpawnLaneLoc(), self.game.spawnVehicleY + self.bossTemplate.height/2);
   self.bossVehicle.hitPoints		= self.bossVehicle.hitPoints*self.hitPointsK;
	self.bossVehicle.onCollision	= function() end;

   if HACK_BOSS_INVICIBLE then
      self.bossVehicle.hitPoints        =  1000000000;
   elseif HACK_BOSS_USE_TARGET_HITPOINTS then
      self.bossVehicle.hitPoints        = self.bossHitPointsTgt;
   end

   self:_setMysteryToken();
   self:_resetBossEncounter();

   local eventGroup = math.floor(self.bossCurLevel / 5);
   MOAIFlurry:eventStart(   "Boss-Group-"..eventGroup,
                            "level-"..self.bossCurLevel.."-ending",   "player-death:"..tostring(self.bossTemplate.type)  );

   EventMgr:postEvent( {}, "boss-spawn" );

end

----------------------------------------------------------------------
