----------------------------------------------------------------------
require('game/game-scenario/game-scenario');
require('engine/event/event-manager')
----------------------------------------------------------------------

local STATE_START_TUTORIAL			= "tutorial-start";
local STATE_INTRO_CHARGING			= "tutorial-charging";
local STATE_INTRO_ACTIVATION		= "tutorial-activation";
local STATE_EXIT_TUTORIAL			= "tutorial-exit";

local TIME_START						= 4;
local TIME_CHARGE_START				= 9;
local TIME_ACTIVATION				= 14;
local TIME_EXIT						= 17;

local TEXT_Y							= App.viewportHeight/2 - 440;

----------------------------------------------------------------------

local Base								= GameScenario;
GameScenarioDroneChargeTutorial	= Base:new();

----------------------------------------------------------------------

function GameScenarioDroneChargeTutorial:new( director, game, template )



   local gs = GameScenario.new( self, director, game, template );
   
   gs.spawner:disableSpawn();
   gs.screenWidth			= App.viewportWidth;
   gs.screenHeight		= App.viewportHeight;
	gs.defaultFontName	= "resources/font/TangoLatin.otf";

	--this is to make sure the tutorial continues updating when the game is paused. Should always be true.
	gs.tutorialActive = true;
	gs.done				= false;
	gs.timer				= 0;
	gs._state			= STATE_START_TUTORIAL;

	-- swipe up image for manual activation
--[[
	gs.tt_HoldFinger	= game.parent:addWidget(				Image, game.parent.layer.ui[1],
																			0, 0,
																			256,256, 
																			"resources/UI/pointericon.png" );
	gs.tt_HoldFinger:hide();
]]

	-- tutorial text
   gs.startTextShdw	= game.parent:addWidget(            TextBox, game.parent.layer.ui[1],
                                                         3, TEXT_Y-3, gs.screenWidth, gs.screenHeight,
                                                         "Your drone\nneeds to\nbe charged.",
                                                         gs.defaultFontName, 24, nil,
                                                         MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
   gs.startText		= game.parent:addWidget(            TextBox, game.parent.layer.ui[1],
                                                         0, TEXT_Y, gs.screenWidth, gs.screenHeight,
                                                         "Your drone\nneeds to\nbe charged.",
                                                         gs.defaultFontName, 24, nil,
                                                         MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   gs.chargeTextShdw	= game.parent:addWidget(            TextBox, game.parent.layer.ui[1],
                                                         3, TEXT_Y-3, gs.screenWidth, gs.screenHeight,
                                                         "Dots indicate\nthe charge level.",
                                                         gs.defaultFontName, 24, nil,
                                                         MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
   gs.chargeText		= game.parent:addWidget(            TextBox, game.parent.layer.ui[1],
                                                         0, TEXT_Y, gs.screenWidth, gs.screenHeight,
                                                         "Dots indicate\nthe charge level.",
                                                         gs.defaultFontName, 24, nil,
                                                         MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   gs.actvTextShdw	= game.parent:addWidget(            TextBox, game.parent.layer.ui[1],
                                                         3, TEXT_Y-3, gs.screenWidth, gs.screenHeight,
                                                         "Fully charged\ndrones will\nauto-activate",
                                                         gs.defaultFontName, 24, nil,
                                                         MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
   gs.actvText			= game.parent:addWidget(            TextBox, game.parent.layer.ui[1],
                                                         0, TEXT_Y, gs.screenWidth, gs.screenHeight,
                                                         "Fully charged\ndrones will\nauto-activate",
                                                         gs.defaultFontName, 24, nil,
                                                         MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   gs.exitTextShdw	= game.parent:addWidget(            TextBox, game.parent.layer.ui[1],
                                                         3, TEXT_Y-3, gs.screenWidth, gs.screenHeight,
                                                         "Good Hunting!",
                                                         gs.defaultFontName, 24, nil,
                                                         MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
   gs.exitText			= game.parent:addWidget(            TextBox, game.parent.layer.ui[1],
                                                         0, TEXT_Y, gs.screenWidth, gs.screenHeight,
                                                         "Good Hunting!",
                                                         gs.defaultFontName, 24, nil,
                                                         MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);

	gs.startTextShdw:setLineSpacing(20);
	gs.startText:setLineSpacing(20);
	gs.chargeTextShdw:setLineSpacing(20);
	gs.chargeText:setLineSpacing(20);
	gs.actvTextShdw:setLineSpacing(20);
	gs.actvText:setLineSpacing(20);
	gs.exitTextShdw:setLineSpacing(20);
	gs.exitText:setLineSpacing(20);
   gs.startTextShdw:hide();
	gs.startText:hide();
	gs.chargeTextShdw:hide();
	gs.chargeText:hide();
	gs.actvTextShdw:hide();
	gs.actvText:hide();
	gs.exitTextShdw:hide();
	gs.exitText:hide();

	-- disable speed increase and counters
   director:disableUpdateSpeedK();
   game:disableDistanceCounter();
   game:disableDurationCounter();

	-- deal one point of damage incase of health activation restrictions
	gs.game.avatar.damageValue = gs.game.avatar.damageValue + 1;

   return gs;

end


----------------------------------------------------------------------

function GameScenarioDroneChargeTutorial:onDeactivate()

	EventMgr:rmvListener( self );

	self.startTextShdw:hide();
	self.startText:hide();
	self.chargeTextShdw:hide();
	self.chargeText:hide();
	self.actvTextShdw:hide();
	self.actvText:hide();
	self.exitTextShdw:hide();
	self.exitText:hide();

	self.game.avatar.damageValue = 0;

   self.director:enableUpdateSpeedK();
   self.game.gameSpawner:enableSpawn();
   self.game:enableDistanceCounter();
   self.game:enableDurationCounter();

end

----------------------------------------------------------------------

function GameScenarioDroneChargeTutorial:onDone( )

	self:onDeactivate();

	Base.onDone( self );

end

----------------------------------------------------------------------

function GameScenarioDroneChargeTutorial:update( dt )

   GameScenario.update( self, dt );
	
	if not self.done and not self.game.avatar:isDead() then
		self:updatePhase( dt );
	end

end

----------------------------------------------------------------------

function GameScenarioDroneChargeTutorial:updatePhase( dt )

	self.timer	= self.timer + dt;

	if self._state == STATE_START_TUTORIAL then
		self:_startPhase(self.timer);
	elseif self._state == STATE_INTRO_CHARGING then
		self:_chargePhase(self.timer);
	elseif self._state == STATE_INTRO_ACTIVATION then
		self:_actvPhase(self.timer);
	elseif self._state == STATE_EXIT_TUTORIAL then
		self:_exitPhase(self.timer);
	end

end

----------------------------------------------------------------------

function GameScenarioDroneChargeTutorial:_startPhase( time )

	self.startTextShdw:show();
	self.startText:show();
	self.chargeTextShdw:hide();
	self.chargeText:hide();
	self.actvTextShdw:hide();
	self.actvText:hide();
	self.exitTextShdw:hide();
	self.exitText:hide();

	self.game.avatar:setDroneChargeLeft(0);
	self.game.avatar:setDroneChargeRight(0);

	if time >= TIME_START then
		self._state	= STATE_INTRO_CHARGING;
	end

end

----------------------------------------------------------------------

function GameScenarioDroneChargeTutorial:_chargePhase( time )

	self.startTextShdw:hide();
	self.startText:hide();
	self.chargeTextShdw:show();
	self.chargeText:show();
	self.actvTextShdw:hide();
	self.actvText:hide();
	self.exitTextShdw:hide();
	self.exitText:hide();

	local k	= (time-2-TIME_START)/(TIME_CHARGE_START-TIME_START-2);
			k	= math.clamp(k, 0, 0.95);

	self.game.avatar:setDroneChargeLeft(k);
	self.game.avatar:setDroneChargeRight(k);

	if time >= TIME_CHARGE_START then
		self._state	= STATE_INTRO_ACTIVATION;

		self.game.avatar:setDroneChargeFullLeft();
		self.game.avatar:setDroneChargeFullRight();
      self.game.avatar:forceDroneActivationLeft();
      self.game.avatar:forceDroneActivationRight();
	end

end

----------------------------------------------------------------------

function GameScenarioDroneChargeTutorial:_actvPhase( time )

	self.startTextShdw:hide();
	self.startText:hide();
	self.chargeTextShdw:hide();
	self.chargeText:hide();
	self.actvTextShdw:show();
	self.actvText:show();
	self.exitTextShdw:hide();
	self.exitText:hide();

	if time >= TIME_ACTIVATION then
		self._state	= STATE_EXIT_TUTORIAL;
	end

end

----------------------------------------------------------------------

function GameScenarioDroneChargeTutorial:_exitPhase( time )

	self.startTextShdw:hide();
	self.startText:hide();
	self.chargeTextShdw:hide();
	self.chargeText:hide();
	self.actvTextShdw:hide();
	self.actvText:hide();
	self.exitTextShdw:show();
	self.exitText:show();

	if time >= TIME_EXIT then
		self.done	= true;

		App.data:setValue("droneChargeTutorialFinished", true);
	end

end

----------------------------------------------------------------------

function GameScenarioDroneChargeTutorial:isDone()
    return self.done or false;
end

----------------------------------------------------------------------
