----------------------------------------------------------------------
require('game/game-scenario/game-scenario');
----------------------------------------------------------------------

local Base = GameScenario;
GameScenarioHazard = Base:new();

----------------------------------------------------------------------

function GameScenarioHazard:new( director, game, template )

   local gs = GameScenario.new( self, director, game, template );
   
   gs.startSounds = { start={"MissileWarning",}, };
   gs.startPlayed = false;

   gs.hazards = {};

   for i = 1, #template.hazards do

      local tpl = template.hazards[i];

      gs.hazards[i] = { template =  tpl };

      if  tpl.delayRnd  then
         gs.hazards[i].delay = tpl.delay + math.random()*tpl.delayRnd;
      else
         gs.hazards[i].delay = tpl.delay;
      end
      
   end


   gs.spawner:disableSpawn();

   return gs;

end

----------------------------------------------------------------------

function GameScenarioHazard:update( dt )

   Base.update( self, dt );
   
   if not self.startPlayed then
      EventMgr:postEvent( { sounds=self.startSounds, name="Game Scenario Hazard:update",}, "add-sfx" );
      self.startPlayed = true;
   end

 --  if self.started
   for i=#self.hazards,1,-1 do

      local as = self.hazards[i];

      as.delay = as.delay-dt;

      if as.delay <= 0 then

         self.game.gameSpawner:spawnHazard(  as.template.hazardTpl,
                                             self.game:getLaneLocX(as.template.lane)
                                             , 0
                                             , nil );

         table.remove(self.hazards,i);

      end
   end
end

----------------------------------------------------------------------

function GameScenarioHazard:isDone()

   return #self.hazards== 0 and #self.game.entityMgr.hazards == 0 ;

end

----------------------------------------------------------------------

function GameScenarioHazard:onDeactivate()

end

----------------------------------------------------------------------

function GameScenarioHazard:onDone()

	self:onDeactivate();

   Base.onDone( self );
   
   self.startPlayed = false;

   self.spawner:enableSpawn();

end

----------------------------------------------------------------------
