----------------------------------------------------------------------
require('game/game-scenario/game-scenario');
require('engine/event/event-manager')
----------------------------------------------------------------------


----------------------------------------------------------------------


GameScenarioTutorial = GameScenario:new();

----------------------------------------------------------------------

function GameScenarioTutorial:new( director, game, template )



   local gs = GameScenario.new( self, director, game, template );
   
   gs.spawner:disableSpawn();
   gs.screenWidth = App.viewportWidth;
   gs.screenHeight = App.viewportHeight;
	
	--this is to make sure the tutorial continues updating when the game is paused. Should always be true.
	gs.tutorialActive = true;

   --[[
	--temporary text
	-- TODO Replace explicit usage of MOAITextBox with a TextBox widget.
	local metaFont = FontMgr:getMetaFont( "resources/font/TangoLatin.otf", 28 );

	gs.textbox = MOAITextBox.new()
	gs.textbox:setRect(
								.5 * -App.viewportWidth, .25 * -App.viewportHeight,
								.5 * App.viewportWidth, .25 * App.viewportHeight
								)
	gs.textbox:setFont(metaFont.font)
   gs.textbox:setGlyphScale(metaFont.scale)
	game.parent.layer.ui[1]:insertProp(gs.textbox)
	gs.textbox:setYFlip(true)
	gs.textbox:setAlignment(MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY)
	----
	]]

	local arrowInset = 130;
   gs.arrowScale = 1;
	gs.arrowCollide = arrowInset + 135;
	


   gs.rightArrow = game.parent:addWidget( 
                                                        Image, game.parent.layer.background[1], 
                                                        gs.screenWidth/2 - arrowInset,
                                                        (gs.screenHeight/2 * -1) + 60,
																		  (122 * gs.arrowScale) * -1,
                                                        238 * gs.arrowScale, 
                                                        "resources/UI/TutorialArrow.png" );
	gs.rightArrow:hide();
                                                        
   gs.leftArrow = game.parent:addWidget( 
                                                        Image, game.parent.layer.background[1], 
                                                        (gs.screenWidth/2 * -1) + arrowInset,
                                                        (gs.screenHeight/2 * -1) + 60,
                                                        122 * gs.arrowScale,
                                                        238 * gs.arrowScale, 
                                                        "resources/UI/TutorialArrow.png" );
	gs.leftArrow:hide();
																		  
																		  
	gs.tt_HoldFinger = game.parent:addWidget(					Image, game.parent.layer.ui[1],
																			0, 0,
																			360,180, 
																			"resources/UI/TT_Hold_Finger.png" );
	gs.tt_HoldFinger:hide();
                                                        
   gs.tt_Swipe = game.parent:addWidget(								Image, game.parent.layer.ui[1],
																			0, 0,
																			439,118,
																			"resources/UI/TT_Swipe.png" );
	gs.tt_Swipe:hide();

   gs.tt_Great = game.parent:addWidget(							Image, game.parent.layer.ui[1],
																			0, 0,
																			190,73,
																			"resources/UI/TT_Great.png" );
   gs.tt_Great:hide();
   
   
   gs.getReady = game.parent:addWidget(							Image, game.parent.layer.ui[1],
																			0, 0,
																			328,78, 
																			"resources/UI/TT_Get_Ready.png" );
   gs.getReady:hide();
   
   
   gs.tt_GO = game.parent:addWidget(							Image, game.parent.layer.ui[1],
																			0, 0,
																			142,73,
																			"resources/UI/TT_GO.png" );
   gs.tt_GO:hide();
	
   gs.tt_CollectCoins = game.parent:addWidget(					Image, game.parent.layer.ui[1],
																			0, 0,
																			350,100,
																			"resources/UI/TT_Collect_Coins1.png" );
   gs.tt_CollectCoins:hide();
	
   gs.tt_DestroyCar = game.parent:addWidget(					Image, game.parent.layer.ui[1],
																			0, 0,
																			439,140,
																			"resources/UI/TT_Destroy_Car.png" );
   gs.tt_DestroyCar:hide();
	
	gs.coinPause = 1.75;
	gs.coinsSpawned = 0;
	gs.coinMissedLimit = 3;
	gs.carSpawnCount = 0;
	gs.carSpawnLimit = 3;
	gs.phase = 0;
	gs.phaseList = {
							{1,nil,nil,gs.startHoldDown,gs.updateHoldDown,nil,"Hold-Shoot"},
							{2,{0.7},{gs.tt_Great},nil,nil,nil,"Great"},
							{3,nil,nil,gs.startArrows,gs.updateArrows,nil,"Swipe-Move"},
							{4,{0.7},{gs.tt_Great},nil,nil,nil,"Great"},
							{5,{2},{gs.tt_DestroyCar},gs.startCarKill,gs.updateCarKill,nil,"Destroy-Car"},
							{6,nil,nil,nil,gs.updateCoinFall,nil,"Coin-Spawned"},
							{7,nil,nil,gs.startCoinPause,gs.updateCoinPause,nil,"Coin-Paused"},
							{8,nil,nil,gs.startCoinWait,gs.updateCoinWait,nil,"Collect-Coin"},
							{9,{1},{gs.tt_Great},nil,nil,nil,"Great"},
							{10,nil,nil,gs.startDataSave,nil,nil,"Data-Save"},
							{11,{1.5},nil,nil,nil,nil,"Banner-Delay"},
							{12,{2},{gs.getReady},nil,nil,nil,"Get-Ready"},
							{13,{1.5},{gs.tt_GO},nil,nil,nil,"Go"},
							{14,nil,nil,gs.startChoice,gs.updateChoice,nil,"Tutorial-Prompt"},
							{15,nil,nil,gs.startAutoHoldDown,gs.updateAutoHoldDown,nil,"Auto-Hold"},
							{16,nil,nil,gs.startAutoArrows,gs.updateAutoArrows,nil,"Auto-Arrows"},
							
							};

	gs.templateOrder = template.phaseOrder;
	gs.templateCount = 0;
	gs.tutorialID = template.id;

   director:disableUpdateSpeedK();

   game:disableDistanceCounter();
   game:disableDurationCounter();

   EventMgr:postEvent({}, "tutorial.start" );

	EventMgr:addListener( gs, gs.onAvatarPickup, "avatar-pickup" );
	EventMgr:addListener( gs, gs.onAvatarKill, "avatar-kill" );

   return gs;

end


----------------------------------------------------------------------

function GameScenarioTutorial:onDeactivate()

end

----------------------------------------------------------------------

function GameScenarioTutorial:onDone( )

	self:onDeactivate();

	EventMgr:rmvListener( self );

   self.director:enableUpdateSpeedK();

   self.game.gameSpawner:enableSpawn();
   self.game:enableDistanceCounter();
   self.game:enableDurationCounter();

   EventMgr:postEvent({ ending = "completed" }, "tutorial.finish" );

end

----------------------------------------------------------------------

function GameScenarioTutorial:update( dt )

   GameScenario.update( self, dt );
	
	if not self.done and not self.game.avatar:isDead() then
		self:updatePhase( dt );
	end

end

----------------------------------------------------------------------

function GameScenarioTutorial:setPhase( phaseNumber, displayTimers, displayObjects, startCall, updateCall, endCall, name, ABConditionals )

   self.phase = phaseNumber or ( self.phase + 1 );

   EventMgr:postEvent({ phase = self.phase }, "tutorial.progress");
	
	if ABConditionals then
		for i,v in ipairs(ABConditionals) do
			if self.tutorialID == v.id then
				self.templateCount = v.count;
			end
		end
	end
				
	if startCall then
		startCall(self);
	else
		self.startCall = nil;
	end
	
	if displayTimers then
		self.displayTimers = displayTimers;
	else
		self.displayTimers = nil;
	end
	
	if displayObjects then
		self.displayObjects = displayObjects;
	else
		self.displayObjects = nil;
	end
	
	if updateCall then
		self.updateCall = updateCall
	else
		self.updateCall = nil;
	end
	
	if endCall then
		self.endCall = endCall;
	else
		self.endCall = nil;
	end
	


end

----------------------------------------------------------------------

function GameScenarioTutorial:updatePhase( dt )

	-- display timers and objects are used to display text/images. the timer runs independently from the 
	-- update call, so if you want something on the screen for a phase that can vary in length, show and hide
	-- the object from the start/update/end calls, and leave the timer/object fields nil

	if self.displayTimers then
		for i,v in ipairs(self.displayTimers) do
			local zerocount = 0;
			if v > 0 then
				self.displayTimers[i] = self.displayTimers[i] - dt;
				if self.displayObjects then
					if self.displayObjects[i]:isHidden() then
						self.displayObjects[i]:show();
					end
				end
			elseif v <= 0 then
				self.displayTimers[i] = 0;
				zerocount = zerocount + 1;
				if self.displayObjects then
					if not self.displayObjects[i]:isHidden() then
						self.displayObjects[i]:hide();
					end
				end
				
				if #self.displayTimers == zerocount then
					self.displayTimers = nil;
					self.displayObjects = nil;
				end
			end
		end
	end
	
	if not self.displayTimers and not self.updateCall then
		if self.endCall then
			self.endCall(self);
		end
	
		if self.templateCount >= #self.templateOrder then
			self.done = true;
		else
			self.templateCount = self.templateCount + 1;
			local arg = {};
			for j=1,7 do
				arg[j] = self.phaseList[self.templateOrder[self.templateCount]][j];
			end
			self:setPhase( arg[1], arg[2], arg[3], arg[4], arg[5], arg[6], arg[7] );
		end
	elseif self.updateCall then
		self.updateCall(self, dt);
	end
end

----------------------------------------------------------------------

function GameScenarioTutorial:nextPhase()

	if self.displayTimers then
		for i,v in ipairs(self.displayTimers) do
			self.displayTimers[i] = 0;
		end
	end
	
	self.updateCall = nil;

end
----------------------------------------------------------------------

function GameScenarioTutorial:startHoldDown()

	self.holdTimer = nil;
	self.holdTarget = 1;
	self.tt_HoldFinger:show();

end

----------------------------------------------------------------------

function GameScenarioTutorial:updateHoldDown( dt )
 
	if self.game.avatar.input.down then
		if not self.holdTimer then
			self.holdTimer = 0;
		end
	end
               
	if self.holdTimer then
		self.holdTimer = self.holdTimer + dt;
                               
      if self.holdTimer >= self.holdTarget then
         self.tt_HoldFinger:hide();
			self.updateCall = nil;
		end

	end
 
end

----------------------------------------------------------------------

function GameScenarioTutorial:startAutoHoldDown()

	self.autoHoldTimer = nil;
	self.autoHoldTarget = 2;
	self.tt_HoldFinger:show();

end

----------------------------------------------------------------------

function GameScenarioTutorial:updateAutoHoldDown( dt )

	if not self.autoHoldTimer then
		self.autoHoldTimer = 0;
		self.autoHoldTarget = 2.5;
	end
	
	if self.autoHoldTimer then
		self.autoHoldTimer = self.autoHoldTimer + dt;
                               
      if self.autoHoldTimer >= self.autoHoldTarget then
         self.tt_HoldFinger:hide();
			self.updateCall = nil;
		end

	end
	
end

----------------------------------------------------------------------

function GameScenarioTutorial:startArrows()

	-- this prevents arrow from jumping when they're shown.
	local avX, avY = self.game.avatar:getLoc();
	self.leftArrow:setLocY(avY);
	self.rightArrow:setLocY(avY);
	
  	self.maxAvatarL = 0;
  	self.maxAvatarR = 0;
  	self.leftMoved = false;
  	self.rightMoved = false;
  	self.leftGone = false;
  	self.rightGone = false;

  	self.rArrowMoveRate = 0.04;
  	self.lArrowMoveRate = 0.04;
			
	self.rightArrow:show();
	self.leftArrow:show();
	self.tt_Swipe:show();

end

----------------------------------------------------------------------

function GameScenarioTutorial:updateArrows( dt )

	local avX, avY = self.game.avatar:getLoc();

	self.leftArrow:setLocY(avY);
	self.rightArrow:setLocY(avY);

	if avX+75 > self.maxAvatarR then
		self.maxAvatarR = avX+75;
	end
       
	if avX-75 < self.maxAvatarL then
		self.maxAvatarL = avX-75;
	end
		
	if self.leftMoved == false or self.rightMoved == false then
		if avX > self.screenWidth/2 - self.arrowCollide and self.rightMoved == false then
			self.rArrowMoveRate = math.max(self.rArrowMoveRate * self.game.avatar:getSpeedX(),10) 
			self.rightMoved = true;
		elseif avX < (self.screenWidth/2 * -1) + self.arrowCollide and self.leftMoved == false then
			self.lArrowMoveRate = math.min(self.lArrowMoveRate * self.game.avatar:getSpeedX(), -10); 
			self.leftMoved = true;
		end
	end
	
		if self.rightMoved == true and self.rightGone == false then
		--move the right arrow +x
			local x,y = self.rightArrow:getLoc();
			x = x + self.rArrowMoveRate;
			if self.maxAvatarR > x then
				x = self.maxAvatarR;
			end
			self.rightArrow:setLoc( x, y )
			if x > self.screenWidth/2 + 30 then
				--make it invisible
				self.rightArrow:hide()
				self.rightGone = true;
			end  
		end
		
		if self.leftMoved == true and self.leftGone == false then
		--move the left arrow -x
			local x,y = self.leftArrow:getLoc();
			x = x + self.lArrowMoveRate;
			if self.maxAvatarL < x then
				x = self.maxAvatarL;
			end
			self.leftArrow:setLoc( x, y )
			if x < (self.screenWidth/2 * -1) - 30 then
				--make it invisible
				self.leftArrow:hide()
				self.leftGone = true;
			end  
		end
	
	if self.rightGone and self.leftGone then
		self.tt_Swipe:hide();
		self.updateCall = nil;
	end


end

----------------------------------------------------------------------

function GameScenarioTutorial:startAutoArrows()

	-- this prevents arrow from jumping when they're shown.
	local avX, avY = self.game.avatar:getLoc();
	local xLeft = self.leftArrow:getLocX();
	local xRight = self.rightArrow:getLocX();
	
	self.leftArrow:setLocY(avY);
	self.rightArrow:setLocY(avY);
	
	self.leftArrow:setLocX(xLeft - 50);
	self.rightArrow:setLocX(xRight + 50);
			
	self.rightArrow:show();
	self.leftArrow:show();
	self.tt_Swipe:show();
	
	self.autoArrowTimer = 0;
	self.autoArrowTarget = 3.5;

end

----------------------------------------------------------------------

function GameScenarioTutorial:updateAutoArrows( dt )

	local avX, avY = self.game.avatar:getLoc();

	self.leftArrow:setLocY(avY);
	self.rightArrow:setLocY(avY);
	
	self.autoArrowTimer = self.autoArrowTimer + dt;
	
	if self.autoArrowTimer >= self.autoArrowTarget then
	
		self.rightArrow:hide()
		self.leftArrow:hide()
		self.tt_Swipe:hide();
		self.updateCall = nil;
	end


end

----------------------------------------------------------------------

function GameScenarioTutorial:startCarKill()

	self.carSpawnTimer = 1;
	self.carKilled = false;
	self.invincibleY = self.screenHeight/2 - 55;

end

----------------------------------------------------------------------

function GameScenarioTutorial:spawnTutorialCar()

	self.carSpawnCount = self.carSpawnCount + 1;
	
	if self.carSpawnCount >= self.carSpawnLimit then
		self.carSpawnCount = 0;
		self.tt_DestroyCar:show();
	end
		
		

	self.currentVehicle = self.game.entityMgr:spawnVehicle(   MAP01_VEHC_TMPLT01[2],
                                             self.game:getLaneLocX(3),
                                             self.game.spawnVehicleY + MAP01_VEHC_TMPLT01[2].height/2,
                                             0,
                                             2,
                                             3                                     );
															
	self.currentVehicle.invincible = true;
	

end

----------------------------------------------------------------------

function GameScenarioTutorial:updateCarKill( dt )

	if self.carKilled then
		self.updateCall = nil;
		return;
	end
	
	if self.carSpawnTimer > 0 then
		self.carSpawnTimer = self.carSpawnTimer - dt;
		if self.carSpawnTimer <= 0 then
			self.carSpawnTimer = 0;
			self:spawnTutorialCar();
		end
	end
	
	if self.carSpawnTimer <= 0 and self.game.entityMgr.vehicles then
		if #self.game.entityMgr.vehicles < 1 then
			self.carSpawnTimer = 1;
		else
			for i,v in ipairs(self.game.entityMgr.vehicles) do
				local x,y = v:getLoc();
				if y < self.invincibleY then
					v.invincible = false;
				end
			end
		end
	end
end

----------------------------------------------------------------------

function GameScenarioTutorial:updateCoinFall( dt )

	if self.game.entityMgr.pickups then
		local x,y;
		for i,v in ipairs(self.game.entityMgr.pickups) do
			if v.pickupType == "currency" then
				x,y = v:getLoc();
			end
			if y < self.screenHeight/4 then
				self.updateCall = nil;
			end
		end
	end
end

----------------------------------------------------------------------

function GameScenarioTutorial:startCoinPause()

	self.coinPauseTimer = self.coinPause;
	self.coinPickup     = nil;
   
	self.tt_CollectCoins:show();

	if self.game.entityMgr.pickups then
		local x,y;
		for i,v in ipairs(self.game.entityMgr.pickups) do
			if v.pickupType == "currency" then
				x,y = v:getLoc();
            self.coinPickup = v;
            self.coinPickup.austin:setTimeK( 0 );
            
			end
		end
		if x < 0 then
			self.leftArrow:show();
			self.leftArrow:setLoc( x + 125, y );
		else
			self.rightArrow:show();
			self.rightArrow:setLoc( x - 125, y );
		end
	end
end

----------------------------------------------------------------------

function GameScenarioTutorial:updateCoinPause( dt )

	self.coinPauseTimer = self.coinPauseTimer - dt;
	
	--if self.coinPauseTimer <= 0 or InputMgr:wasTouchPressed() then
	if self.coinPauseTimer <= 0 then

      if self.coinPickup then
         self.coinPickup.austin:setTimeK( 0.75 ); 
      end
      
		self.tt_CollectCoins:hide();
		self.leftArrow:hide();
		self.rightArrow:hide();
		self:nextPhase();
	end
		
end

----------------------------------------------------------------------

function GameScenarioTutorial:startCoinWait()
	self.coinCollected = false;
end

----------------------------------------------------------------------

function GameScenarioTutorial:updateCoinWait( dt )

	if self.coinCollected then
		self.updateCall = nil;
		return;
	end
	
	if self.game.entityMgr.pickups and self.game.entityMgr.vehicles then
		if #self.game.entityMgr.pickups < 1 and #self.game.entityMgr.vehicles < 1 then
			self:spawnTutorialCar();
		end
	end
	if self.game.entityMgr.vehicles then
		if #self.game.entityMgr.vehicles > 0 then
			for i,v in ipairs(self.game.entityMgr.vehicles) do
				local x,y = v:getLoc();
				if y < self.invincibleY then
					v.invincible = false;
				end
			end
		end
	end
end

----------------------------------------------------------------------

function GameScenarioTutorial:startDataSave()

	App.data:setValue( "tutorialDone", TUTORIAL_DONE_NUMBER );

	EventMgr:postEvent( {}, "tutorial-completed" );

end

----------------------------------------------------------------------

function GameScenarioTutorial:startChoice()

	self.textbox:setString("Play Tutorial?");

end

----------------------------------------------------------------------

function GameScenarioTutorial:updateChoice( dt )

	if not self.cTestTimer then
		self.cTestTimer = 3;
	end
	
	self.cTestTimer = self.cTestTimer - dt;

	if self.cTestTimer <= 0 then
		self:choseYes();
		self.cTestTimer = nil;
	end

end

----------------------------------------------------------------------

function GameScenarioTutorial:choseYes()
	self.textbox:setString("");
	self.updateCall = nil;
end

----------------------------------------------------------------------

function GameScenarioTutorial:choseNo()
	self.textbox:setString("");
	local ABConditionals = {{id = "Ask",count=11}};
	self:setPhase(10,nil,nil,self.startDataSave,nil,nil,"Data-Save",ABConditionals);
end



----------------------------------------------------------------------

function GameScenarioTutorial:onAvatarPickup( event, type )

	if App.data:getValue("tutorialDone") == TUTORIAL_DONE_NUMBER then
		EventMgr:rmvListener( self );
		return
	end

	if event.pickup.pickupType == "currency" then
		self.coinCollected = true;
	end
	 
end

----------------------------------------------------------------------

function GameScenarioTutorial:onAvatarKill( event, type )

	if App.data:getValue("tutorialDone") == TUTORIAL_DONE_NUMBER then
		EventMgr:rmvListener( self );
		return
	end
	
	self.carKilled = true;
	self.coinsSpawned = self.coinsSpawned + 1;
	self.carSpawnCount = 0;
	if not self.tt_DestroyCar:isHidden() then
		self.tt_DestroyCar:hide();
	end
	if self.coinsSpawned >= self.coinMissedLimit then
		self.coinsSpawned = 0;
		local ABConditionals = {{id = "Ask",count=7},{id="Full",count=6}};
		self:setPhase( 6, nil, nil, nil, self.updateCoinFall,nil,"Coin-Spawned", ABConditionals );
	end
end

----------------------------------------------------------------------

function GameScenarioTutorial:isDone()

    return self.done;

end

----------------------------------------------------------------------
