require('engine/mission/mission-goal')

----------------------------------------------------------------------

local Base = MissionGoal;

MissionGoalAvoidLanes = Base:new();

----------------------------------------------------------------------

function MissionGoalAvoidLanes:new( mission, index, template  )

print("MissionGoalAvoidLanes:new");

   local goal  = Base.new( self, mission, index, template );

   return goal;

end

----------------------------------------------------------------------

function MissionGoalAvoidLanes:activate()

   Base.activate( self );
   
   EventMgr:addListener( self, self.onAvatarScore, "avatar-score" );
   EventMgr:addListener( self, self.onGameOver, "game-over" );

end

----------------------------------------------------------------------

function MissionGoalAvoidLanes:onGameStart()

   Base.onGameStart( self);

   self.cumulativeScore  = 0;
   self.failed = false;


end

----------------------------------------------------------------------

function MissionGoalAvoidLanes:onAvatarScore( event, type )

   if (not self.failed) then

      --In some rare case, we get this event once the game got terminated
      if event.game and event.game.avatar then

         local laneWidth = event.game:getLaneLocX(4);
      
         local cWidth = event.game.avatar.template.collisionWidth
      
         local sep = (cWidth/2) + (laneWidth/2);
      
         for i,v in ipairs(self.template.lanes) do
      
            local deltaX = event.xLoc - event.game:getLaneLocX(v);
            if deltaX >= 0 then
            -- car is to the right of the lane center
               if deltaX < sep then
                  self.failed = true;
               end
            else
            --car is to the right of the lane center
               if deltaX > -sep then
                  self.failed = true;
               end
            end
         end
         

         if not self.completed and not self.failed then
         
            self.cumulativeScore = event.distance;
            
            if self.cumulativeScore >= self.template.objective then

               self:_OnCompleted();
               self.cumulativeScore = -1;
            end
         end
      end
   end
end

----------------------------------------------------------------------

function MissionGoalAvoidLanes:onGameOver(event)

   if not App.data:isCurrentMissionGoalCompleted( self.index ) and not self.completed then
      
      local curMis = App.data:getMissionIndex();

      App.data:updateMissionGoal(  curMis, self.index, self.cumulativeScore );
      
   end
end
