require('state/banner/banner')
----------------------------------------------------------------

BannerMgr = {};

----------------------------------------------------------------

function BannerMgr:init()

   self.layer           = nil;
   self.bannerPlaying   = nil;
   self.bannerTable     = {};
   self.bannerAdsTable  = {};

   EventMgr:addListener(self, self.onAdsEnable,           "ads-enabled" );
   EventMgr:addListener(self, self.onAdsDisable,          "ads-disabled" );

   EventMgr:addListener(self, self.onGoalComplete,        "mission-goal-completed" );
   EventMgr:addListener(self, self.onMissionComplete,     "mission-completed" );
   EventMgr:addListener(self, self.onDailyProgress,       "daily-progress" );
   EventMgr:addListener(self, self.onDailyComplete,       "daily-success" );
   EventMgr:addListener(self, self.onMissionReward,       "mission-reward" );
   EventMgr:addListener(self, self.onDailyReward,         "daily-reward" );
   EventMgr:addListener(self, self.onBuyGems,             "gems-bought" );
   EventMgr:addListener(self, self.onBuyGold,             "gold-bought" );
   EventMgr:addListener(self, self.onBuyKeys,             "keys-bought" );
   EventMgr:addListener(self, self.onBuyDoubleCoin,       "double-coin-bought" );
   EventMgr:addListener(self, self.onGiftedKeys,          "keys-gifted" );
   EventMgr:addListener(self, self.onGiftSent,            "gift-sent" );
   EventMgr:addListener(self, self.onWeeklyReward,        "weekly-reward" );
   EventMgr:addListener(self, self.onTangoBonus,          "tango-bonus" );
   EventMgr:addListener(self, self.onLevelReward,         "level-reward" );
   EventMgr:addListener(self, self.onPromoGift,           "promo-gift-reward" );
   EventMgr:addListener(self, self.onInviteGift,          "invite-tango-friend-success" );
   EventMgr:addListener(self, self.onBragGift,            "brag-friend-success" );
   EventMgr:addListener(self, self.onShowBannerAd,        "show-banner-ad" );
   EventMgr:addListener(self, self.onVideoThief,          "video-thief" );
   EventMgr:addListener(self, self.onVideoGold,           "video-gold" );
   EventMgr:addListener(self, self.onVideoKeys,           "video-key" );
   EventMgr:addListener(self, self.onMysteryReward,       "mystery-reward" );
   EventMgr:addListener(self, self.onAchievementComplete, "achievement-completed" );
   EventMgr:addListener(self, self.onAchievementReward,   "achievement-reward" );
   EventMgr:addListener(self, self.onBountyActivate,      "bounty-activated" );
   EventMgr:addListener(self, self.onBountyCollect,       "bounty-collected" );
   EventMgr:addListener(self, self.onBountyReset,         "bounty-reset" );
   EventMgr:addListener(self, self.onBountyPickup,        "bounty-pickup" );
   EventMgr:addListener(self, self.onBuyStarterPack,      "starterpack-bought" );
   EventMgr:addListener(self, self.onJuneGiftReceived,    "june-gift-received" );


end


----------------------------------------------------------------

function BannerMgr:clear()

   if self.bannerPlaying then

      self.bannerPlaying:deactivate();
      self.bannerPlaying:delete();
      self.bannerPlaying = nil;

   end

   self.bannerTable    = {};
   self.bannerAdsTable = {};

end

----------------------------------------------------------------

function BannerMgr:onInput()

   -- hack for now to only check input for ads...
   -- this should be move from within the banner itself
   if self.bannerPlaying and self.bannerPlaying._ads then

      local clicked = self.bannerPlaying:onInput();

      if clicked then

         if self.bannerPlaying._adsLink then

            EventMgr:postEvent({ link = self.bannerPlaying._adsLink }, "banner-ad-clicked");

            App:gotoPage(self.bannerPlaying._adsLink);

            self.bannerPlaying:deactivate();
            self.bannerPlaying:delete();
            self.bannerPlaying = nil;

         end
         return true;

      end
   end


   return false;

end

----------------------------------------------------------------

function BannerMgr:update( dt )

 --  print("BannerMgr:update", self.bannerTable, #self.bannerTable );

   if self.bannerPlaying then

      self.bannerPlaying:update( dt );

      if self.bannerPlaying:isDone() then

         if self.bannerPlaying._ads then
            EventMgr:postEvent({}, "banner-ad-done" );
         end

         self.bannerPlaying:deactivate();
         self.bannerPlaying:delete();
         self.bannerPlaying = nil;

      else

         if self.bannerPlaying._ads then

            if (not self._adsEnable) or #self.bannerTable > 0 then
               self.bannerPlaying:forceLeave();
            end

         end

      end

   else

      if self.layer ~= nil      then

         if #self.bannerTable > 0 then

            self.bannerPlaying = self.bannerTable[1];
            table.remove( self.bannerTable, 1 );

         end

         if self.bannerPlaying then
            self.bannerPlaying:setLayer( self.layer );
            self.bannerPlaying:activate();
         end

      end
   end
end


----------------------------------------------------------------

function BannerMgr:onAdsEnable( event )

   self._adsEnable = true;

end

----------------------------------------------------------------

function BannerMgr:onAdsDisable( event )

   self._adsEnable = false;

end

----------------------------------------------------------------

function BannerMgr:createBanner( params )

   local banner = Banner:new( params );

   if banner then

      table.insert( self.bannerTable, banner );

   end

end


----------------------------------------------------------------

function BannerMgr:setLayer( layer )

   print("BannerMgr:setLayer", layer );

   if self.layer ~= layer then

      self.layer = layer;
      if self.bannerPlaying then
         self.bannerPlaying:setLayer( layer );
      end

   end

end

----------------------------------------------------------------

function BannerMgr:onGoalComplete( event )


   local banner = {
                     title="GOAL COMPLETE:",
                     text=event.goal.template.bannerText,
                     sound="GoalAccomplished",
                  };

   self:createBanner( banner);


end

----------------------------------------------------------------

function BannerMgr:onMissionComplete( event )


   local banner = {
                     title="Great Job!",
                     text="Mission Complete",
                     sound="MissionAccomplished",

                  };

   self:createBanner( banner);

end

----------------------------------------------------------------

function BannerMgr:onDailyProgress( event )

   local progress  = event.curProg;
   local objective = event.curObj;

   if progress ~= objective and progress > 0 then


      local banner = {
                        title="DAILY PROGRESS:",
                        text="",
                        sound="DailyCompleteDay4",
                        starTotal = objective,
                        starCurrent = progress,
                     };

      self:createBanner( banner);

   end
end

----------------------------------------------------------------

function BannerMgr:onDailyComplete( event )

   local objective = event.curObj;


   local banner = {
                     title="DAILY COMPLETE:",
                     text="",
                     sound="DailyCompleteDay4",
                     starTotal = objective,
                     starCurrent = objective,

                  };

   self:createBanner( banner);

end

----------------------------------------------------------------

function BannerMgr:onMissionReward( event )

	local addon = " Gems";
	if event.gem == 1 then
		addon = " Gem";
	end


   local banner = {
                     title="MISSION REWARD:",
                     text="Received "..event.gem..addon,
                     background="resources/UI/Banner_Green.png",
                     sound="PurchaseGems",
                     time=3,
                  };

   self:createBanner( banner);

end

----------------------------------------------------------------

function BannerMgr:onDailyReward( event )

	if event.rewards.gem then

		if event.rewards.gem > 0 then

			local addon = " Gems";
			if event.rewards.gem == 1 then
				addon = " Gem";
			end

         local banner = {
                           title="DAILY REWARD:",
                           text="Received "..tostring(event.rewards.gem)..addon,
                           background="resources/UI/Banner_Green.png",
                           sound="PurchaseGems",
                           time=3,
                        };

         self:createBanner( banner);


		end
	end

	if event.rewards.gold then
		if event.rewards.gold > 0 then

			local addon = " Coins";
			if event.rewards.gold == 1 then
				addon = " Coin";
			end

         local banner = {
                           title="DAILY REWARD:",
                           text="Received "..tostring(event.rewards.gold)..addon,
                           background="resources/UI/Banner_Green.png",
                           sound="PurchaseGold",
                           time=3,
                        };

         self:createBanner( banner);

		end
	end

end

----------------------------------------------------------------

function BannerMgr:onBuyGems( event )


   local banner = {
                     title="PURCHASED:",
                     text=event.gem.." Gems!",
                     background="resources/UI/Banner_Green.png",
                     sound="PurchaseGems",
                     time=2,
                  };

   self:createBanner( banner);

end

----------------------------------------------------------------

function BannerMgr:onBuyGold( event )

   local banner = {
                     title="PURCHASED:",
                     text=event.gold.." Coins!",
                     background="resources/UI/Banner_Green.png",
                     sound="PurchaseGold",
                     time=2,
                  };

   self:createBanner( banner);

end

----------------------------------------------------------------

function BannerMgr:onVideoThief( event )

   if event.gold then

      self:onVideoGold( event );

   elseif event.keys then

      self:onVideoKeys( event );

   else

      Debug.assert(event.gold  or event.keys, "BannerMgr:onVideoThief->event.gold  or event.keys" );

   end
end


----------------------------------------------------------------

function BannerMgr:onVideoGold( event )

   local cId      = AbTestMgr:getVideoConsumableRewardIndex();
   local banner;

   if cId and AbTestMgr:isConsumableStoreEnabled() then

      local tpl   = ConsumableMgr:getTemplate(cId).tplPowerup;
      local val   = tostring(tpl.valueK or 1.5).."x";

      banner =
      {
         title="VIDEO REWARD:",
         text=val.." Coin Boost - 60 min!",
         background="resources/UI/Banner_Green.png",
         sound="PurchaseGold",
         time=2,
      };
   else
      banner =
      {
         title="VIDEO REWARD:",
         text=event.gold.." Coins!",
         background="resources/UI/Banner_Green.png",
         sound="PurchaseGold",
         time=2,
      };
   end

   self:createBanner( banner);

end

----------------------------------------------------------------

function BannerMgr:onBuyKeys( event )

   local banner = {
                     title="PURCHASED:",
                     text=event.keys.." Keys!",
                     background="resources/UI/Banner_Green.png",
                     sound="PurchaseKeys",
                     time=2,
                  };

   self:createBanner( banner);


end

----------------------------------------------------------------

function BannerMgr:onVideoKeys( event )


   local keyText = " Keys!";
   if event.keys then
      if event.keys == 1 then
         keyText = " Key!";
      end
   end


   local banner = {
                     title="VIDEO REWARD:",
                     text=event.keys..keyText,
                     background="resources/UI/Banner_Green.png",
                     sound="PurchaseKeys",
                     time=2,
                  };

   self:createBanner( banner);

end

----------------------------------------------------------------

function BannerMgr:onBuyDoubleCoin( event )


   local banner = {
                     title="PURCHASED:",
                     text="Double Coin!",
                     background="resources/UI/Banner_Green.png",
                     sound="PurchaseKeys",
                     time=2,
                  };

   self:createBanner( banner);

end


----------------------------------------------------------------

function BannerMgr:onGiftSent( event )

   if #self.bannerTable < 2 then

      local banner = {
                        title="Nice!",
                        text="Sending a Gift",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseKeys",
                        time=1.5,
                     };

      self:createBanner( banner);

   end
end

----------------------------------------------------------------

function BannerMgr:onGiftedKeys( event )

   local name    = "A Friend";
   local contact = ContactMgr:findContact(event.giver);

   if contact then
      name = contact:getFirstName() .. " " .. contact:getLastName() ;
   end

	local keyText = " key!";
	if event.keys > 1 then
		keyText = " keys!";
	end

   local banner = {
                     title=name,
                     text="Gifted you "..event.keys..keyText,
                     background="resources/UI/Banner_Green.png",
                     sound="PurchaseKeys",
                     time=3,
                     titleFontName="resources/font/Tango.otf",
                     titleFontSize=16,
                     preloadedGlyphs=name
                  };

   self:createBanner( banner);


end

----------------------------------------------------------------

function BannerMgr:onWeeklyReward( event )

	if event and event.gem and event.gem > 0 then

		local addon = " Gems";

		if event.gem == 1 then
			addon = " Gem";
		end

		local banner = {
								title="WEEKLY REWARD",
								text="Awarded "..event.gem..addon,
								background="resources/UI/Banner_Green.png",
								sound="PurchaseGems",
								time=3,
							};

		self:createBanner( banner);
	end

end

----------------------------------------------------------------

function BannerMgr:onAchievementComplete( event )

   local title = event.achievement:getTitle();
   if title then

      local banner = {
                        title="ACHIEVEMENT COMPLETE:",
                        text=title,
                        sound="GoalAccomplished",
                        time=3,
                     };

      self:createBanner( banner);

   end
end

----------------------------------------------------------------

function BannerMgr:onAchievementReward( event )

	local addon = " Gems";

	if event.gem == "1" then
		addon = " Gem";
	end

   local banner = {
                     title="ACHIEVEMENT COMPLETE:",
                     text="Awarded "..event.gem..addon,
                     background="resources/UI/Banner_Green.png",
                     sound="PurchaseGems",
                     time=2,
                  };

   self:createBanner( banner);

end

----------------------------------------------------------------

function BannerMgr:onTangoBonus( event )

   if event and type(event.bonus)=="number" then
      local banner = {
                        title="TANGO GIFT:",
                        text="Received "..event.bonus.." Gems",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseGems",
                        time=3,
                     };

      self:createBanner( banner);
   else
   		Debug.assert( event and type(event.bonus)=="number", "BannerMgr:onTangoBonus -> event and type(event.bonus)==number" );
   end

end

----------------------------------------------------------------

function BannerMgr:onLevelReward( event )

   --Dont stack too many banners
   -- it is probably because we are cheating and increasing our level
   if #self.bannerTable < 5 then

      if event.gold then

         local banner = {
                           title="LEVEL REWARD",
                           text=event.gold.." Coins!",
                           background="resources/UI/Banner_Green.png",
                           sound="PurchaseGold",
                           time=2,
                        };

         self:createBanner( banner);

      end

      if event.gem then

         local banner = {
                           title="LEVEL REWARD",
                           text=event.gem.." Gems!",
                           background="resources/UI/Banner_Green.png",
                           sound="PurchaseGems",
                           time=2,
                        };

         self:createBanner( banner);

      end
   end
end

----------------------------------------------------------------

function BannerMgr:onPromoGift( event )

	if event.gold then


      local banner = {
                        title="GIFT REWARD:",
                        text=event.gold.." Coins!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseGold",
                        time=2,
                     };

      self:createBanner( banner);

	end

	if event.gem then


      local banner = {
                        title="GIFT REWARD:",
                        text=event.gem.." Gems!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseGems",
                        time=2,
                     };

      self:createBanner( banner);

	end
end

----------------------------------------------------------------

function BannerMgr:onInviteGift( event )

   if event.gold then


      local banner = {
                        title="INVITE REWARD:",
                        text=event.gold.." Coins!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseGold",
                        time=2,
                     };

      self:createBanner( banner);

   end

   if event.gem then


      local banner = {
                        title="INVITE REWARD:",
                        text=event.gem.." Gems!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseGems",
                        time=2,
                     };

      self:createBanner( banner);

   end
end

----------------------------------------------------------------

function BannerMgr:onBragGift( event )

   if event.key and event.key > 0 then

      local banner = {
                        title="BRAG REWARD:",
                        text=event.key.." Keys!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseKeys",
                        time=3,
                     };

      self:createBanner( banner);

   else


      local banner = {
                        title="Nice!",
                        text="Bragging to Friend!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseKeys",
                        time=3,
                     };

      self:createBanner( banner);

   end

end


----------------------------------------------------------------

function BannerMgr:onShowBannerAd( event )

   if event.banner  then

print("BannerMgr:onShowBannerAd", event.banner, event.link, event.duration, event.width, event.height );

      local banner = {
                        title="",
                        text="",
                        background=event.banner,
                        sound="",
                        time=event.duration,
                        sWidth=event.width,
                        sHeight=event.height,
                        dHeight=0,
                        advert=true,
                        adLink=event.link,
                     };

      self:createBanner( banner);

   end

end

----------------------------------------------------------------

function BannerMgr:onMysteryReward( event )

	if event.gold then

      local amt = nil;
      if event.gold == 1500 then
         amt = "1,500";
      else
         amt = ""..event.gold;
      end

      local banner = {
                        title="MYSTERY REWARD:",
                        text=amt.." Coins!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseGold",
                        time=2,
                     };

      self:createBanner( banner);

	end

	if event.gems then

      local amt = nil;
      if event.gems == 5000 then
         amt = "5,000";
      else
         amt = ""..event.gems;
      end

      local banner = {
                        title="MYSTERY REWARD:",
                        text=amt.." Gems!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseGems",
                        time=2,
                     };

      self:createBanner( banner);

	end

   if event.keys and event.keys > 1 then

      local banner = {
                        title="MYSTERY REWARD:",
                        text=event.keys.." Keys!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseKeys",
                        time=3,
                     };

      self:createBanner( banner);


   elseif event.keys and event.keys == 1 then

      local banner = {
                        title="MYSTERY REWARD:",
                        text=event.keys.." Key!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseKeys",
                        time=3,
                     };

      self:createBanner( banner);

   end

   if event.consumable then

      if event.consumable == "revive" then

         local banner = {
                           title="MYSTERY REWARD:",
                           text=" Free Revive!",
                           background="resources/UI/Banner_Green.png",
                           sound="PurchaseKeys",
                           time=3,
                        };

         self:createBanner( banner);


      elseif event.consumable == "power-time" then

         local banner = {
                           title="MYSTERY REWARD:",
                           text=" Free Power Time!",
                           background="resources/UI/Banner_Green.png",
                           sound="PurchaseKeys",
                           time=3,
                        };

         self:createBanner( banner);


      elseif event.consumable == "double-score" then

         local banner = {
                           title="MYSTERY REWARD:",
                           text=" Free 2x Score!",
                           background="resources/UI/Banner_Green.png",
                           sound="PurchaseKeys",
                           time=3,
                        };

         self:createBanner( banner);


      elseif event.consumable == "head-start" then


         local banner = {
                           title="MYSTERY REWARD:",
                           text=" Free Head Start!",
                           background="resources/UI/Banner_Green.png",
                           sound="PurchaseKeys",
                           time=3,
                        };

         self:createBanner( banner);

      end

   end

end

----------------------------------------------------------------

function BannerMgr:onBountyActivate( event )

   local banner = {
                     title="New Challenge:",
                     text="BOUNTIES UNLOCKED!",
                     portrait="resources/UI/French_Driver_128x128.png",
                     background="resources/UI/Banner_Green.png",
                     sound="PurchaseGems",
                     timeIn=0.2,
                     time=3,
                     timeOut=0.2,
                  };

   self:createBanner( banner);

end

----------------------------------------------------------------

function BannerMgr:onBountyCollect( event )

   local pImage = "resources/UI/Claire_128x128.png"

   if event and event.id then
      if event.id == "__GHOST_01" then
         pImage = "resources/UI/French_Driver_Defeated_128x128.png"
      elseif event.id == "__GHOST_02" then
         pImage = "resources/UI/Irish_Driver_Defeated_128x128.png"
      elseif event.id == "__GHOST_03" then
         pImage = "resources/UI/Spanish_Driver_Defeated_128x128.png"
      end
   end

   --Dont stack too many banners
   -- it is probably because we are cheating and increasing our level
   if event and event.gem and #self.bannerTable < 5 then

      local banner = {
                        title="BOUNTY REWARD",
                        text=event.gem.." Gems!",
                        portrait = pImage;
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseGems",
                     };

      self:createBanner( banner);
   end

end

----------------------------------------------------------------

function BannerMgr:onBountyReset( event )

   local banner = {
                     title="Weekly Challenge:",
                     text="NEW BOUNTIES AVAILABLE!",
                     portrait="resources/UI/French_Driver_128x128.png",
                     background="resources/UI/Banner_Green.png",
                     sound="PurchaseGems",
                     timeIn=0.5,
                     time=1.5,
                     timeOut=0.5,
                  };

   self:createBanner( banner);

end

----------------------------------------------------------------

function BannerMgr:onBountyPickup( event )

	if not LeagueMgr:isLeagueEnabled() then

      local gID = "Bounties:";
      local pImage = "resources/UI/Claire_128x128.png";

      if event and event.id then
         if event.id == "__GHOST_01" then
            gID = "Bastian Zettel:";
            pImage = "resources/UI/French_Driver_Defeated_128x128.png"
         elseif event.id == "__GHOST_02" then
            gID = "Dana St.Patrick:";
            pImage = "resources/UI/Irish_Driver_Defeated_128x128.png"
         elseif event.id == "__GHOST_03" then
            gID = "Ricardo Orlando:";
            pImage = "resources/UI/Spanish_Driver_Defeated_128x128.png"
         end
      end

      local banner = {
                        title=gID,
                        text="BOUNTY COLLECTED!",
                        portrait = pImage;
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseGems",
                        timeIn=0.5,
                        time=1.5,
                        timeOut=0.5,
                     };

      self:createBanner( banner);
   end

end

----------------------------------------------------------------

function BannerMgr:onBuyStarterPack( event )

   if (not event) or (not event.source) then
      return;
   end

   local banner;

   if event.source == "head-start-popup" then

      banner = {
         title="PURCHASED:",
         text="Head Start Bundle!",
         background="resources/UI/Banner_Green.png",
         sound="PurchaseGems",
         time=3,
      };
   end

   if banner then
      self:createBanner(banner);
   end

end

----------------------------------------------------------------

function BannerMgr:onJuneGiftReceived( event )

	if event.gold then

      local amt = ""..event.gold;

      local banner = {
                        title="GIFT RECEIVED:",
                        text=amt.." Coins!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseGold",
                        time=2,
                     };

      self:createBanner( banner);

	end

	if event.gems then

      local amt = ""..event.gems;

      local banner = {
                        title="GIFT RECEIVED:",
                        text=amt.." Gems!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseGems",
                        time=2,
                     };

      self:createBanner( banner);

	end

   if event.keys and event.keys > 1 then

      local banner = {
                        title="GIFT RECEIVED:",
                        text=event.keys.." Keys!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseKeys",
                        time=3,
                     };

      self:createBanner( banner);


   elseif event.keys and event.keys == 1 then

      local banner = {
                        title="GIFT RECEIVED:",
                        text=event.keys.." Key!",
                        background="resources/UI/Banner_Green.png",
                        sound="PurchaseKeys",
                        time=3,
                     };

      self:createBanner( banner);

   end

   if event.consumable then
       local count = event.count or 1;

      if event.consumable == "revive" and count >= 1 then
          local textString = "";
          if count == 1 then textString = " Free Revive!"; else textString = count.." Free Revives!"; end
         local banner = {
                           title="GIFT RECEIVED:",
                           text=textString,
                           background="resources/UI/Banner_Green.png",
                           sound="PurchaseKeys",
                           time=3,
                        };

         self:createBanner( banner);


      elseif event.consumable == "power-time" then

          local textString = "";
          if count == 1 then textString = " Free Power Time!"; else textString = count.." Free Power Times!";  end

         local banner = {
                           title="GIFT RECEIVED:",
                           text=textString,
                           background="resources/UI/Banner_Green.png",
                           sound="PurchaseKeys",
                           time=3,
                        };

         self:createBanner( banner);


      elseif event.consumable == "double-score" then
          local textString = "";
          if count == 1 then textString = " Free 2x Score!"; else textString = count.." Free 2x Scores!"; end

         local banner = {
                           title="GIFT RECEIVED:",
                           text=textString,
                           background="resources/UI/Banner_Green.png",
                           sound="PurchaseKeys",
                           time=3,
                        };

         self:createBanner( banner);


      elseif event.consumable == "head-start" then

          local textString = "";
          if count == 1 then textString = " Free Head Start!"; else textString = count.." Free Head Starts!"; end

         local banner = {
                           title="GIFT RECEIVED:",
                           text=textString,
                           background="resources/UI/Banner_Green.png",
                           sound="PurchaseKeys",
                           time=3,
                        };

         self:createBanner( banner);

         elseif event.consumable == "mystery" then
          local textString = "";
          if count == 1 then textString = " Free Mystery Spin!"; else textString = count.." Free Mystery Spins!"; end


         local banner = {
                           title="GIFT RECEIVED:",
                           text=textString,
                           background="resources/UI/Banner_Green.png",
                           sound="PurchaseKeys",
                           time=3,
                        };

         self:createBanner( banner);

      end

   end

end

----------------------------------------------------------------
