require( 'engine/file/fileMgr' )
require('engine/object/object')
require('engine/utility/math')

----------------------------------------------------------------

local WIDTH          = 630;
local HEIGHT         = 144;
local S_WIDTH			= 429;
local TEXT_OFFSET    = { 33, -21 };
local TEXT_OFFSET1   = { 33, 43 };
local TEXT_OFFSET2   = { 33, -21 };
local STAR_OFFSET    = { 33, -21 };
local STAR_WIDTH     = 55;
local STAR_HEIGHT    = 54;
local STAR_SPACING   = 40;
local PORT_W         = 104;
local PORT_H         = 104;
local PORT_X_OFFSET  = -246;
local PORT_Y_OFFSET  = 2;

local portDefault    = "resources/UI/Claire_128x128.png";
local bgDefault      = "resources/UI/Banner_Blue.png";
local soundDefault   = "DailyCompleteDay4";
local timeInDefault  = 0.33;
local timeDefault    = 1;
local timeOutDefault = 0.33;

local titleFontDefaultName = "resources/font/TangoLatin.otf";
local titleFontDefaultSize = 16;
local msgFontDefaultName = "resources/font/TangoLatin.otf";
local msgFontDefaultSize = 19;

----------------------------------------------------------------

local Base           = Object;
Banner               = Base:new();

----------------------------------------------------------------

function Banner:new( params )

   local b  = Base.new(self);

   b.settings     =
   {
      text        = params.title       or "";
		text2			= params.text        or "";
      portrait    = params.portrait    or portDefault;
      background  = params.background  or bgDefault;
      sound       = params.sound       or soundDefault;
      timeIn      = params.timeIn      or timeInDefault;
      time        = params.time        or timeDefault;
      timeOut     = params.timeOut     or timeOutDefault;
      starTotal   = params.starTotal;
      starCurrent = params.starCurrent;
      titleFontName = params.titleFontName or titleFontDefaultName;
      titleFontSize = params.titleFontSize or titleFontDefaultSize;
      msgFontName = params.msgFontName or msgFontDefaultName;
      msgFontSize = params.msgFontSize or msgFontDefaultSize;
      tXOffset    = params.tXOffset    or 0;
      tYOffset    = params.tYOffset    or 0;
      width       = params.sWidth      or WIDTH;
      height      = params.sHeight     or HEIGHT;
      dHeight     = params.dHeight     or 10;
      advert      = params.advert      or false;
      adLink      = params.link;
      preloadedGlyphs = params.preloadedGlyphs;
   };

   b.startLoc    =
   {
      App.viewportWidth/2  + b.settings.width/2,
      App.viewportHeight/2 - b.settings.height/2 - b.settings.dHeight,
   };
   b.loc    =
   {
      b.startLoc[X],
      b.startLoc[Y],
   };
   b.time   = 0;

   b.layer           = nil;
   b.background      = self:createProp(   nil,
                                          nil,
                                          b.startLoc[X],
                                          b.startLoc[Y],
                                          b.settings.width,
                                          b.settings.height,
                                          b.settings.background );
   b.background:setPriority(300);
                                          
   b.portrait        = self:createProp(   nil,
                                          b.background,
                                          PORT_X_OFFSET,
                                          PORT_Y_OFFSET,
                                          PORT_W,
                                          PORT_H,
                                          b.settings.portrait );
                                          
   b.portrait:setPriority(301);
   
   if not b.settings.text2 then
	
      b.textShad        = self:createTextProp(  nil,
                                                b.background,
                                                ( b.settings.tXOffset )+TEXT_OFFSET[X]+3,
                                                ( b.settings.tYOffset )+TEXT_OFFSET[Y]-3,
                                                b.settings.width,
                                                b.settings.height,
                                                b.settings.text,
                                                b.settings.msgFontName,
                                                b.settings.msgFontSize,
                                                b.settings.preloadedGlyphs,
																true );
      b.textShad:setPriority(301);

      b.text            = self:createTextProp(  nil,
                                                b.background,
                                                ( b.settings.tXOffset )+TEXT_OFFSET[X],
                                                ( b.settings.tYOffset )+TEXT_OFFSET[Y],
                                                b.settings.width,
                                                b.settings.height,
                                                b.settings.text,
                                                b.settings.msgFontName,
                                                b.settings.msgFontSize,
                                                b.settings.preloadedGlyphs );
      b.text:setPriority(302);
																
   else
	
      b.textShad        = self:createTextProp(  nil,
                                                b.background,
                                                ( b.settings.tXOffset )+TEXT_OFFSET1[X]+3,
                                                ( b.settings.tYOffset )+TEXT_OFFSET1[Y]-3,
                                                S_WIDTH,
                                                b.settings.height,
                                                b.settings.text,
                                                b.settings.titleFontName,
                                                b.settings.titleFontSize,
                                                b.settings.preloadedGlyphs,
																true );
      b.textShad:setPriority(301);
	
      b.text            = self:createTextProp(  nil,
                                                b.background,
                                                ( b.settings.tXOffset )+TEXT_OFFSET1[X],
                                                ( b.settings.tYOffset )+TEXT_OFFSET1[Y],
                                                S_WIDTH,
                                                b.settings.height,
                                                b.settings.text,
                                                b.settings.titleFontName,
                                                b.settings.titleFontSize,
                                                b.settings.preloadedGlyphs );
      b.text:setPriority(302);
																
      b.text2Shad       = self:createTextProp(  nil,
                                                b.background,
                                                ( b.settings.tXOffset )+TEXT_OFFSET2[X]+3,
                                                ( b.settings.tYOffset )+TEXT_OFFSET2[Y]-3,
                                                b.settings.width,
                                                b.settings.height,
                                                b.settings.text2,
                                                b.settings.msgFontName,
                                                b.settings.msgFontSize,
                                                b.settings.preloadedGlyphs,
																true  );
      b.text2Shad:setPriority(301);

																
      b.text2           = self:createTextProp(  nil,
                                                b.background,
                                                ( b.settings.tXOffset )+TEXT_OFFSET2[X],
                                                ( b.settings.tYOffset )+TEXT_OFFSET2[Y],
                                                b.settings.width,
                                                b.settings.height,
                                                b.settings.text2,
                                                b.settings.msgFontName,
                                                b.settings.msgFontSize,
                                                b.settings.preloadedGlyphs );
      b.text2:setPriority(302);

	end
	
   
   b.stars = {};
   if b.settings.starTotal then
   
      local spacing  = STAR_SPACING;
      local starX    = -spacing * (b.settings.starTotal - 1);
      local starY    = 0;
      local texture = nil;
      for i=1,b.settings.starTotal do
         if i <= b.settings.starCurrent then
            texture = "resources/UI/Banner_DailyLitStar.png";
         else
         
            texture = "resources/UI/Banner_DailyUnlitStar.png";
         end

         b.stars[i]= self:createProp(     nil,
                                          b.background,
                                          starX+STAR_OFFSET[X],
                                          starY+STAR_OFFSET[Y],
                                          STAR_WIDTH, 
                                          STAR_HEIGHT,
                                          texture );
         b.stars[i]:setPriority(302);
                                          
         starX = starX + (spacing * 2);
      end
   end
   
   if b.settings.advert then
      b._ads       = true;
      b._adsLink   = b.settings.adLink;
   end


   return b;
   
end

----------------------------------------------------------------

function Banner:delete()

   self.text        = self:removeProp( self.text       );
	self.textShad    = self:removeProp( self.textShad   );
	if self.text2 then
		self.text2    = self:removeProp( self.text2      );
		self.text2Shad= self:removeProp( self.text2Shad  );
	end
   self.portrait    = self:removeProp( self.portrait );
   self.background  = self:removeProp( self.background );
   for i=1, #self.stars do
      self.stars[i] = self:removeProp( self.stars[i] );
   end
   
end

----------------------------------------------------------------

function Banner:activate()
   
   EventMgr:postEvent( {sounds= {volume=1,priority=2,start={self.settings.sound}}, name="updateBanner"}, "add-sfx" );

   self.time         = 0;
   self._forceLeave  = false;

end

----------------------------------------------------------------

function Banner:deactivate()

end


----------------------------------------------------------------

function Banner:onInput( )

   if self.layer then

      if  InputMgr:wasTouchPressed() then

         local x, y = self.layer:wndToWorld ( InputMgr:getTouchLoc() );

         if self.background then

            local bx, by = self.background:getLoc();
            local dx, dy = math.abs(x-bx), math.abs(y-by);

            return      dx < (self.background._width/2)
                  and   dy < (self.background._height/2);

         end

      end

   end
end

----------------------------------------------------------------

function Banner:createProp( layer, parent, x, y, w, h, texture )

   local gfxQuad  = MOAIGfxQuad2D.new ();
         
   gfxQuad:setTexture ( TextureMgr:getTexture(texture, true) );
   gfxQuad:setRect ( -w/2, -h/2, w/2, h/2 );

   local prop  = MOAIProp2D.new ();

   prop._width  = w;
   prop._height = h;

   prop:setDeck ( gfxQuad );
   prop:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA );
   
   if parent then
      prop:setAttrLink ( MOAIProp2D.INHERIT_TRANSFORM, parent, MOAIProp2D.TRANSFORM_TRAIT );
   end
   
   prop:setLoc(x,y);
   
   if layer then
      layer:insertProp ( prop );
   end
   
   return prop;

end

----------------------------------------------------------------

function Banner:createTextProp( layer, parent, x, y, w, h, text, fontName, fontSize, glyphsToLoad, black )

   local textProp  = MOAITextBox.new ();
   
	if black then
		textProp:setColor(0,0,0,1);
	end
   
   local metaFont = FontMgr:getMetaFont(fontName, fontSize, glyphsToLoad);

   textProp:setFont ( metaFont.font );
   textProp:setGlyphScale ( metaFont.scale );
   textProp:setYFlip ( true );
   textProp:setRect ( -w/2, -h/2, w/2, h/2 );
   textProp:setString ( text );
   textProp:setAlignment ( MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY );
   
   if parent then
      textProp:setAttrLink ( MOAIProp2D.INHERIT_TRANSFORM, parent, MOAIProp2D.TRANSFORM_TRAIT );
   end
   
   textProp:setLoc( x, y );
   
   if layer then
      layer:insertProp(textProp);
   end
      
   return textProp;

end

--[[
-- This function is ideal - it uses the the ui-text-box wrapper instead of using Moai's text prop directly.
function Banner:createTextBoxNew( layer, parent, x, y, w, h, text, fontName, fontSize, black )

	local textProp = state:addWidget(TextBox, layer, x, y, w, h, text, fontName, fontSize, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   
   if parent then
      textProp.moaiTextBox:setAttrLink(MOAIProp2D.INHERIT_TRANSFORM, parent, MOAIProp2D.TRANSFORM_TRAIT);
   end

   if black then
      textProp.moaiTextBox:setColor(0, 0, 0, 1);
   end
end
]]

----------------------------------------------------------------

function Banner:addProp( prop, priority )

   if prop and self.layer then
      self.layer:insertProp( prop );
      prop:setPriority(priority or 100 );
   end

end

----------------------------------------------------------------

function Banner:removeProp( prop )

   if prop and self.layer then
      self.layer:removeProp( prop );
   end

end

----------------------------------------------------------------

function Banner:setPropAlpha( prop, a, black )

   if prop then
		if not black then
			prop:setColor(1,1,1,a);
		else
			prop:setColor(0,0,0,a);
		end
   end
   
end

----------------------------------------------------------------

function Banner:setLayer( layer )
   
   if layer ~= self.layer then

      self:removeProp( self.text        );
		self:removeProp( self.textShad    );
		if self.text2 then
			self:removeProp( self.text2       );
			self:removeProp( self.text2Shad   );
		end
      self:removeProp( self.portrait  );
      self:removeProp( self.background  );
      for i=1, #self.stars do
         self:removeProp( self.stars[i] );
      end

      self.layer = layer;

      self:addProp( self.background, 300 );
      self:addProp( self.portrait,   304 );
      self:addProp( self.text,       303 );
		self:addProp( self.textShad,   302 );
		if self.text2 then
			self:addProp( self.text2,   303    );
			self:addProp( self.text2Shad,302   );
		end
      for i=1, #self.stars do
         self:addProp( self.stars[i],304 );
      end
      
   end
end


----------------------------------------------------------------

function Banner:forceLeave()

   self._forceLeave = true;

end

----------------------------------------------------------------

function Banner:update( dt )

   local time     = self.time;
   local moveRel  = 1-math.clamp((self.settings.timeIn-time)/self.settings.timeIn,0,1);
   local x,y      = math.smoothLerp( self.startLoc[X] , 0, moveRel )
                  , self.loc[Y];

   if self.background then
   
      self.background:setLoc( x,y );
      
   end
   
   if time > self.settings.timeIn then

      --If we are force to leave, dont stay idle
      if self._forceLeave and ((time-self.settings.timeIn) < self.settings.time) then

         --Stay at least 1/2 second to not look bad
         self.settings.time = math.max(time-self.settings.timeIn,0.5);

      end

      local time2 = time-self.settings.timeIn-self.settings.time;


      if time2 > 0 then
      
         local alpha = 1-math.clamp( time2/self.settings.timeOut,0,1);
         self:setPropAlpha( self.portrait,   alpha );
         self:setPropAlpha( self.background, alpha );
         self:setPropAlpha( self.text,       alpha );
			self:setPropAlpha( self.textShad,   alpha, true );
			if self.text2 then
				self:setPropAlpha( self.text2,      alpha );
				self:setPropAlpha( self.text2Shad,  alpha, true );
			end
         for i=1, #self.stars do
            self:setPropAlpha(self.stars[i], alpha);
         end

         self.done = alpha <= 0.01;
         
      end

   end

   self.loc    = {x,y};
   self.time   = time+dt;

end

----------------------------------------------------------------

function Banner:isDone()

   return self.done;

end

----------------------------------------------------------------

