require('state/guide/guide')
require('app/template/guide/guide-sets')
----------------------------------------------------------------

GuideMgr =	{};

----------------------------------------------------------------

function GuideMgr:init()

   self.layer   = nil;

   self.guide   = nil;

   self.active = false;
   self.busy   = false;

   self.guideList = {};

   self.lastStateStopped = "";

   self.defaultReminderSpacing  = 10;

   EventMgr:addListener( self, self.onCloudMasterData, "cloud:master-data-guide" );
   EventMgr:addListener( self, self.onUpgrade,         "upgrade-success" );
   EventMgr:addListener( self, self.onGameOver,        "game-over");

end

----------------------------------------------------------------

function GuideMgr:onCloudMasterData( data )

   local content     = (data or {}).content;
   if content then
      if content.defaultReminderSpacing then
         self.defaultReminderSpacing = content.defaultReminderSpacing;
      end
      if content.reminderSpacing then
         self.reminderSpacingTable = content.reminderSpacing;
      end
      if content.guideInfo then
         self.guideInfo = content.guideInfo;
      end
   end
end

----------------------------------------------------------------

function GuideMgr:endFocus( )

   if self.guideList then

      if #self.guideList > 0 and not self.active and not self.guide then
         if self:loadSet(self.guideList[1]) then
            self:start();
         end
      end

   end

end

----------------------------------------------------------------

function GuideMgr:forceGuide( name )

   self.busy = true;

   --this forces the current guide to run, by placing it at the top of the queue, and starting it up.
   if self.guideList and name then
      table.insert(self.guideList, 1, name);
   else
      self.guideList = {};
   end

   if not self.active and not self.guide then
      self:loadSet(self.guideList[1]);
      self:start();
   end

end

----------------------------------------------------------------

function GuideMgr:update( dt )

   if self.guide and self.active then

      --Not great but we are trying to avoid having an ads while the Guide is present
      --It would be better if the guide would send message when it is coming in and out (or waiting to come out)
      -- and have the banner listening for that instead
      EventMgr:postEvent({}, "ads-disabled" );

      self.guide:update(dt);
   end

end

----------------------------------------------------------------

function GuideMgr:start()

   if self.guide then
      self.active = true;
   end

end

----------------------------------------------------------------

function GuideMgr:stop()

   if self.guideList[1] then
      table.remove(self.guideList, 1);
   end
   self.active = false;
   self.busy   = false;
   self.guide  = nil;

   local curState = StateMgr:getCurState();
   self.lastStateStopped = curState.stateFilename;

end

----------------------------------------------------------------

function GuideMgr:isActive()

   return self.active;

end

----------------------------------------------------------------

function GuideMgr:isBusy()

   return self.busy;

end

----------------------------------------------------------------

function GuideMgr:addSet( name )

   -- print("GuideMgr:addSet( " .. tostring(name) .. " )")

   self.busy = true;
   table.insert(self.guideList, name);
   return true;

end

----------------------------------------------------------------

function GuideMgr:removeSet( name )

   for i,v in ipairs(self.guideList) do
      if name == v then
         print("GuideMgr:removeSet", name);
         table.remove(self.guideList, i);
      end
   end

end

----------------------------------------------------------------

function GuideMgr:loadSet( name )

   if name then

      -- "result-ranking" means garage first, "ranking" means old flow
      local append = "_a";

      -- "force-car-view" means they have to go see the new car, "car-reminder" means they just get a pop-up
      local newcarFlow = App.data:getValue( "ab-claire-newcar-flow" );
      local newcarAppend = "_1";
      if newcarFlow == "force-car-view" then
         newcarAppend = "_2";
      end

      if name == "car_unlocked" then

         name = name .. append .. newcarAppend;

      elseif name == "new_player"
         or name == "upgrade_weapon"
         or name == "daily_complete"
         or name == "mission_complete"
         or name == "upgrade_reminder" then

         name = name .. append;

      else
         --no append, if no difference between flow versions of guide.
      end

      for _, guideSet in ipairs(TEMPLATE_GUIDE_SETS) do

         if guideSet.name == name then
            print("GuideMgr:loadSet:", name);
            self.guide = Guide:new(guideSet, self.layer);

            if self.guide then
               App.data:setValue("guide_wait_games", 0);
               local timeStamp = TimeMgr:getTimeGMT();
               if timeStamp then
                  App.data:setValue("guide_wait_timestamp", timeStamp);
               end
               self.guide:load();
            end

            return true;
         end

      end

      Debug.error("No guide set found with name: " .. tostring(name));

      self:removeSet(name);
      return false;

   end

   return false;
   
end

----------------------------------------------------------------

function GuideMgr:setLayer( layer )

   print("GuideMgr:setLayer");

   self.layer = layer;
   if self.guide then
      self.guide:setLayer( layer, state );
   end

end

----------------------------------------------------------------

function GuideMgr:onUpgrade( event )

   --joe reset the games played between upgrades int
   App.data:setValue( "guide-games-since-upgrade", 0 );

end

----------------------------------------------------------------

function GuideMgr:onGameOver( event )

   --can upgrade check
   local veh = App.data:getCurrentPlayerVehicle();

   local weapon = App.data:canUpgradePlayerMainWeapon( veh );
   local armor  = App.data:canUpgradePlayerArmor( veh );
   local side   = nil;
   local turret = nil;
   if App.data:isPlayerSideWeaponUnlocked( vehicle_index ) then
      side = App.data:canUpgradePlayerSideWeapon( veh );
   end
   if App.data:isPlayerTurretWeaponUnlocked( veh ) then
      turret = App.data:canUpgradePlayerTurretWeapon( veh );
   end


   if weapon or armor or side or turret then

      local games = App.data:getValue( "guide-games-since-upgrade" );
      if games then
         games = games+1;
      else
         games = 0;
      end
      print("guide garage reminder incrementing:", games, weapon, armor, side, turret);
      App.data:setValue( "guide-games-since-upgrade", games );
   end

end

----------------------------------------------------------------

function GuideMgr:getReminderSpacing( name )

   if self.reminderSpacingTable and self.reminderSpacingTable[name] then
      return self.reminderSpacingTable[name];
   else
      return self.defaultReminderSpacing;
   end

end

----------------------------------------------------------------

function GuideMgr:getCurrentGuideInfo(  )

   return AbTestMgr:getGuideInfo() or self.guideInfo;

end

----------------------------------------------------------------

function GuideMgr:getGuideDisabled( name )

   if self:isGuidePaused( name ) then
      return true;
   else

      local guideInfo = self:getCurrentGuideInfo();

      if guideInfo and guideInfo[name] then
         return guideInfo[name].disabled;
      else
         return false;
      end
   end

end

----------------------------------------------------------------

function GuideMgr:isGuidePaused( name )

   local guideInfo = self:getCurrentGuideInfo();

   if guideInfo and guideInfo[name] then

      if guideInfo[name].time then
         local time = TimeMgr:getTimeGMT();
         local timestamp = App.data:getValue("guide_wait_timestamp");

         if time and timestamp then
            if (time - timestamp) <= guideInfo[name].time then
               return true;
            end
         end
      end

      if guideInfo[name].games then
         local games = App.data:getValue("guide_wait_games");
         if games and games < guideInfo[name].games then
            return true;
         end
      end

      if guideInfo[name].videoWaitTime then
         local time = TimeMgr:getTimeGMT();
         local timestamp = App.data:getValue("video_wait_timestamp");

         if time and timestamp then
            if (time - timestamp) <= guideInfo[name].videoWaitTime then
               return true;
            end
         end
      end

      if guideInfo[name].videoWaitGames then
         local games = App.data:getValue("video_wait_games");
         if games and games < guideInfo[name].videoWaitGames then
            return true;
         end
      end

   end

   return false;

end
