require( 'engine/file/fileMgr' )
require('engine/object/object')
require('engine/utility/math')

----------------------------------------------------------------

local Base           = Object;
Marker               = Base:new();

----------------------------------------------------------------

function Marker:new( state, game, template, fontName, fontSize, glyphsToPreload )

   local m  = Base.new(self);

   m.template     = template;
   m.state        = state;
   m.game         = game;
   m.distance     = 000;

   m.image        = m:createProp( state.layer.game[template.image.layer], nil, 0, 10000, template.image.width, template.image.height, template.image.texture );
   m.text         = {};
   m.text[1]      = m:createTextProp( state.layer.game[template.text.layer], nil, 0,10000, App.viewportWidth-50,  100,  "", template.text.fontName, template.text.fontSize, nil, template.text.color, MOAITextBox.LEFT_JUSTIFY,    MOAITextBox.CENTER_JUSTIFY )
   m.text[2]      = m:createTextProp( state.layer.game[template.text.layer], nil, 0,10000, App.viewportWidth-50,  100,  "", template.text.fontName, template.text.fontSize, nil, template.text.color, MOAITextBox.RIGHT_JUSTIFY,   MOAITextBox.CENTER_JUSTIFY )
 
   return m;
   
end

----------------------------------------------------------------

function Marker:createProp( layer, parent, x, y, w, h, texture )

   local gfxQuad  = MOAIGfxQuad2D.new ();
         
   gfxQuad:setTexture ( self.game:getTexture(texture, true) );
   gfxQuad:setRect ( -w/2, -h/2, w/2, h/2 );

   local prop  = MOAIProp2D.new ();
  
   prop:setDeck ( gfxQuad );
   prop:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA );
   
   if parent then
      prop:setAttrLink ( MOAIProp2D.INHERIT_TRANSFORM, parent, MOAIProp2D.TRANSFORM_TRAIT );
   end
   
   prop:setLoc(x,y);
   
   if layer then
      layer:insertProp ( prop );
   end
   
   return prop;

end

----------------------------------------------------------------

function Marker:createTextProp( layer, parent, x, y, w, h, text, fontName, fontSize, glyphsToPreload, color, alignHz, alignVt )

   local textProp  = MOAITextBox.new ();
   
   local metaFont = FontMgr:getMetaFont(fontName, fontSize, glyphsToPreload);

   textProp:setFont ( metaFont.font );
   textProp:setGlyphScale( metaFont.scale );
   textProp:setYFlip ( true );
   textProp:setRect ( -w/2, -h/2, w/2, h/2 );
   textProp:setString ( text );
   textProp:setAlignment ( alignHz or MOAITextBox.CENTER_JUSTIFY, alignVt or MOAITextBox.CENTER_JUSTIFY );
   textProp:setColor( color[1],color[2],color[3],color[4]);
   if parent then
      textProp:setAttrLink ( MOAIProp2D.INHERIT_TRANSFORM, parent, MOAIProp2D.TRANSFORM_TRAIT );
   end
   
   textProp:setLoc( x, y );
   
   if layer then
      layer:insertProp(textProp);
   end
      
   return textProp;

end

----------------------------------------------------------------------

function  Marker:getTop()

   local x,y = self.image:getLoc();

   return y+self.template.image.height/2;

end

----------------------------------------------------------------

function Marker:reset( dist )

   self.done      = false;
   self.distance  = dist;
   
   self.alpha = 1.00;
   self.scale = 1.00;
   self.deltaY= 0.00;

   self.image:setLoc(0,1000);

   for i=1,#self.text do
      self.text[i]:setLoc(0,1000);
      self.text[i]:setScl(1,1);
      self.text[i]:setString("" .. dist .. " m"); 
      self.text[i]:setColor( self.template.text.color[1], self.template.text.color[2], self.template.text.color[3], self.template.text.color[4]*self.alpha );
                 
   end

end

----------------------------------------------------------------

function Marker:update( dt )

  if self.game then

      --Because the game speed up and make it really hard to see the text passing by
      -- we slow it down using the game speed factor
      local delta   = (self.distance-self.game.distanceScore)/(self.game:getDistanceScoreK());
      local y       = delta+self.game.avatarPosY;
      
      if y < App.viewportHeight then

         local y2    = y;
         local yMin  = self.game.avatarPosY - 50;
         
         self.image:setLoc(  0, y );
         
         if y2 > yMin then
         
             for i=1,#self.text do
               self.text[i]:setLoc(0, y2 - 15);
            end

         else

            y2 = yMin;

            self.alpha   = math.max(self.alpha-1.50/255 * self.game.gameSpeedK);
            self.scale   = self.scale  * 1.0001;
            self.deltaY  = self.deltaY + 2.0000 * self.game.gameSpeedK;

            if self.alpha  < 0.00 then

               self.done = true;
            
            else
         
               for i=1,#self.text do
                  self.text[i]:setScl(self.scale,self.scale);
                  self.text[i]:setColor( self.template.text.color[1], self.template.text.color[2], self.template.text.color[3], self.template.text.color[4]*self.alpha );
                  self.text[i]:setLoc( 0, y2 + self.deltaY + 15 );
               end

            end
         end
      end
  end
  
end

----------------------------------------------------------------

