
require('engine/object/object')

----------------------------------------------------------------

local Base        = Object;
OverlayBusy       = Base:new();

local fontName    = "resources/font/TangoLatin.otf";
local fontSize    = 10;

----------------------------------------------------------------

function  OverlayBusy:new( )

print("OverlayBusy:new");

   local o           = Base.new( self );

	o.overlay         = o:createProp(      nil,
                                          nil,
                                          0,
                                          0,
                                          App.viewportWidth,
                                          App.viewportHeight,
                                          "resources/UI/blackbox.png",
                                          {1,1,1, 0.80}
                                          );

   o.wheel           = o:createProp(      nil,
                                          self.Overlay,
                                          0,
                                          0,
                                          180,
                                          180,
                                          "resources/UI/loading-wheel.png",
                                          {1,1,1,1.00}
                                          );

   o.message         = o:createTextProp(  nil,
                                          self.Overlay,
                                          0,
                                          -125,
                                          560,
                                          220,
                                          "",
                                          fontName,
                                          fontSize
                                          );

   return o;

end

----------------------------------------------------------------

function OverlayBusy:createProp( layer, parent, x, y, w, h, texture, color )

   local gfxQuad  = MOAIGfxQuad2D.new ();

   gfxQuad:setTexture ( TextureMgr:getTexture(texture) );
   gfxQuad:setRect ( -w/2, -h/2, w/2, h/2 );

   local prop  = MOAIProp2D.new ();

   prop:setDeck ( gfxQuad );
   prop:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA );

   if parent then
      prop:setAttrLink ( MOAIProp2D.INHERIT_TRANSFORM, parent, MOAIProp2D.TRANSFORM_TRAIT );
   end

   prop:setLoc(x,y);

   if color then
      prop:setColor( color[1], color[2], color[3], color[4] );
   end

   if layer then
      layer:insertProp ( prop );
   end


   return prop;

end

----------------------------------------------------------------

function OverlayBusy:createTextProp( layer, parent, x, y, w, h, text, fontName, fontSize, color )

   local textProp  = MOAITextBox.new ();

   local metaFont = FontMgr:getMetaFont(fontName, fontSize);

   textProp:setFont ( metaFont.font );
   textProp:setGlyphScale( metaFont.scale );
   textProp:setYFlip ( true );
   textProp:setRect ( -w/2, -h/2, w/2, h/2 );
   textProp:setString ( text );
   textProp:setAlignment ( MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY );

   if parent then
      textProp:setAttrLink ( MOAIProp2D.INHERIT_TRANSFORM, parent, MOAIProp2D.TRANSFORM_TRAIT );
   end

   textProp:setLoc( x, y );

   if color then
      textProp:setColor( color[1], color[2], color[3], color[4] );
   end

   if layer then
      layer:insertProp(textProp);
   end

   return textProp;

end

----------------------------------------------------------------

function OverlayBusy:addProp( prop, priority )

   if prop and self.layer then
      self.layer:insertProp( prop );
      prop:setPriority(priority or 100 );
   end

end

----------------------------------------------------------------

function OverlayBusy:removeProp( prop )

   if prop and self.layer then
      self.layer:removeProp( prop );
   end

end

----------------------------------------------------------------

function OverlayBusy:setPropAlpha( prop, a )

   if prop then
      prop:setColor(1,1,1,a);
   end

end

----------------------------------------------------------------

function OverlayBusy:setText( text )

   if self.message then
      self.message:setString( text or "" );
   end

end

----------------------------------------------------------------

function OverlayBusy:setLayer( layer )

   if layer ~= self.layer then

      self:removeProp( self.overlay    );
      self:removeProp( self.wheel      );
      self:removeProp( self.message    );

      self.layer = layer;

      self:addProp( self.overlay,    300 );
      self:addProp( self.wheel,      301 );
      self:addProp( self.message,     302 );

   end
end

----------------------------------------------------------------

function OverlayBusy:update( dt )

   --TODO: add rot functionality to widget instead of using prop
   self.wheel:moveRot(-10);

end

----------------------------------------------------------------
