
require('engine/object/object')

----------------------------------------------------------------

local PRIORTY_0				= 10;
local PRIORTY_1				= 20;
local PRIORTY_2				= 30;
local PRIORTY_3				= 40;
local PRIORTY_4				= 50;
local PRIORTY_5				= 60;
local PRIORTY_6				= 70;
local PRIORTY_7				= 80;
local PRIORTY_8				= 90;

----------------------------------------------------------------

local Base        = Object;
PauseTransition       = Base:new();

local screenWidth          = App.viewportWidth;
local screenHeight         = App.viewportHeight;

----------------------------------------------------------------

function  PauseTransition:new( game )

print("PauseTransition:new");

   local o           = Base.new( self );
	
	o.game = game;
   

	-- black overlay
   o.Overlay = self:createProp(o.game.layer.ui[1], nil, 0, 0, App.viewportWidth, App.viewportHeight, "resources/UI/blackbox.png", {1,1,1, 0.3});
	o.Overlay:setVisible(false);
	
	o.countImages = {};
	o.countStartX = ((App.viewportWidth/2)+39.5)*-1;
	o.countEndX = (App.viewportWidth)+39.5;
	
	for i=1,3 do
		o.countImages[i] = self:createProp(o.game.layer.ui[1], nil, o.countStartX, 0, 79, 64,
			"resources/UI/"..i..".png", {1,1,1,1});
	end
	
	o.goImage = self:createProp(o.game.layer.ui[1], nil, 0, 0, 149, 67, "resources/UI/GO.png", {1,1,1,1});
	o.goImage:setVisible(false);
                                          
   return o;
	
end

----------------------------------------------------------------

function PauseTransition:createProp( layer, parent, x, y, w, h, texture, color )

   local gfxQuad  = MOAIGfxQuad2D.new ();
         
   gfxQuad:setTexture ( TextureMgr:getTexture(texture) );
   gfxQuad:setRect ( -w/2, -h/2, w/2, h/2 );

   local prop  = MOAIProp2D.new ();
  
   prop:setDeck ( gfxQuad );
   prop:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA );
   
   if parent then
      prop:setAttrLink ( MOAIProp2D.INHERIT_TRANSFORM, parent, MOAIProp2D.TRANSFORM_TRAIT );
   end
   
   prop:setLoc(x,y);
   
   if color then
      prop:setColor( color[1], color[2], color[3], color[4] );
   end
   
   if layer then
      layer:insertProp ( prop );
   end

   
   return prop;

end

----------------------------------------------------------------

function PauseTransition:setLayers( layer )
	
	layer:insertProp( self.Overlay );
	layer:insertProp( self.goImage );
	for i=1,3 do
		layer:insertProp( self.countImages[i] );
	end
end
----------------------------------------------------------------

function PauseTransition:onStart()

	if self.goCount and self.count then
		if self.goCount ~= -1 or self.count > 0 then
			self:resetWidgets();
			self:resetVars();
		else
			self:resetVars();
		end
	else
		self:resetVars();
	end
end

----------------------------------------------------------------

function PauseTransition:countdownUpdate( dt )

	if self.count > 0 then

		self.count = self.count - dt;
		
		if self.count <= 0 then
			self.count = 0;
			self.goCount = 0;
			self.goImage:setVisible(true);
			return;
		end

		
		local num = math.ceil(self.count*2);
		local perc = num - (self.count*2);
		local pos;
		
		if perc <= 0.2 then
			pos = self.countStartX - (perc*5)*self.countStartX;
		elseif perc > 0.2 and perc < 0.75 then
			pos = 0;
		elseif perc >= 0.75 then
			pos = ((perc-0.74)*4)*self.countEndX;
		end
		
		self.countImages[num]:setLoc(pos,0);
		
	elseif self.goCount > -1 then
	
		self.goCount = self.goCount + dt;
		
		local pct = 0;
		local range = 0;
		
		if self.goCount < self.goTarTime[self.goTarget] then
		
			if self.goTarget == 1 then
		
				pct = self.goCount/self.goTarTime[self.goTarget];
				range = self.goTar[self.goTarget] - self.goSizeStart;
				self.goSize = self.goSizeStart + (pct * range);
				
			else
			
				pct = self.goCount/self.goTarTime[self.goTarget];
				range = self.goTar[self.goTarget] - self.goTar[self.goTarget-1];
				self.goSize = self.goTar[self.goTarget-1] + (pct * range);
				
			end
			

		elseif self.goCount >= self.goTarTime[self.goTarget] then
			
			self.goSize = self.goTar[self.goTarget]
			self.goTarget = self.goTarget + 1;
			self.goCount = 0;
			if self.goTarget > self.goTargetMax then
				self.goTarget = 0;
				self.count = -1;
				self.goCount = -1;
				self.goImage:setVisible(false);
				self.goImage:setScl( self.goSize, self.goSizeStart );
				self:countdownOver();
				return;
			end
		end
		
		self.goImage:setScl( self.goSize, self.goSize );
		
	end

end

----------------------------------------------------------------

function PauseTransition:countdownOver()

	self:resetWidgets();

   EventMgr:postEvent( {state="state/state-pause.lua"}, "game-unpaused" );
	GamePaused = false;
	self.game.isUnpausing = false;
	
end

----------------------------------------------------------------

function PauseTransition:resetWidgets()

	self.Overlay:setVisible(false);
	for i=1,3 do
		self.countImages[i]:setLoc( self.countStartX, 0 );
	end
	self.goImage:setVisible(false);

end

----------------------------------------------------------------

function PauseTransition:resetVars()

	self.goCount = -1;
	self.goTargetMax = 3;
	self.count = 1.5;
	self.goSize = 0.8
	self.goSizeStart = 0.8
	self.goTarget = 1;
	self.goTar = {};
	self.goTar[1] = 2;
	self.goTar[2] = 0.85;
	self.goTar[3] = 1;
	self.goTarTime = {};
	self.goTarTime[1] = 0.15;
	self.goTarTime[2] = 0.1;
	self.goTarTime[3] = 0.05;
	self.Overlay:setVisible(true);

end
----------------------------------------------------------------

return PauseTransition;

----------------------------------------------------------------
