----------------------------------------------------------------

require('flash/flash-manager')
require('engine/object/object')

----------------------------------------------------------------

local SHOP_TRAY_HOME_X		= 0;
local SHOP_TRAY_HOME_Y		= 0;
local SHOP_TRAY_SHIFT		= 300;
local COIN_COUNT_X			= 0;
local COIN_COUNT_Y			= 0;
local GEM_COUNT_X				= 218;
local GEM_COUNT_Y				= 0;
local KEY_COUNT_X				= -218;
local KEY_COUNT_Y				= 0;
local PRIORTY_0				= 10;
local PRIORTY_1				= 20;
local PRIORTY_2				= 30;
local PRIORTY_3				= 40;
local PRIORTY_4				= 50;
local PRIORTY_5				= 60;
local PRIORTY_6				= 70;
local PRIORTY_7				= 80;
local PRIORTY_8				= 90;
local PRIORTY_9				= 100;
local PRIORTY_100				= 1100;


----------------------------------------------------------------

local Base        = Object;
ResourceBar       = Base:new();

----------------------------------------------------------------

function  ResourceBar:new( state, x, y, layer, partition )

   local textureSheet   = FlashMgr:getTextureSheet("resources/flash-resource-bundle/MainScreen.lua", "resources/flash-resource-bundle/MainScreen.png", 30);

   local o  = Base.new( self );

   self.defaultFontName = "resources/font/TangoLatin.otf";

   o.x      = x;
   o.y      = y;

   o.state           = state;
	o.layer           = layer;
   o.partition       = partition;

   -- keys, coins, gems and shop buttons
	o.ShopBarPivot = MOAITransform2D.new();
   o.ShopBarPivot:setLoc(o.x, o.y);
   --o.Background = state:addWidget(Anim, o.layer, 0, 0, 0, 0, "ResourceBackGround", textureSheet);
   --o.Background:setPriority(PRIORTY_100);
   --o.Background:setColor(116/255*0.75,185/255*0.75,207/255*0.75,0.75);

   --o.Background:setParent(o.ShopBarPivot);

	o.CoinButton = state:addWidget(ButtonAnim, o.layer, o.partition, 0, 0, 0,0, "CoinsBuy", textureSheet, nil, o.goCoinStore, nil, nil, nil, nil);
	o.CoinButton:setPriority(PRIORTY_100);
	o.CoinButton:setParent(o.ShopBarPivot);
	o.CoinCounter = state:addWidget(TextBox, o.layer, 90, 0, 180, 81, App.data:getGoldCount()..'', self.defaultFontName, 15, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	o.CoinCounter.moaiTextBox:setPriority(PRIORTY_100);
	o.CoinCounter:setParent(o.CoinButton.anim.props[2]);

	o.GemButton = state:addWidget(ButtonAnim, o.layer, o.partition, 0,0, 0,0, "GemsBuy", textureSheet, nil, o.goGemStore, nil, nil, nil, nil);
	o.GemButton:setPriority(PRIORTY_100);
	o.GemButton:setParent(o.ShopBarPivot);
	o.GemCounter = state:addWidget(TextBox, o.layer, 70, 0, 180, 81, App.data:getGemCount()..'', self.defaultFontName, 15, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   o.GemCounter.moaiTextBox:setPriority(PRIORTY_100);
	o.GemCounter:setParent(o.GemButton.anim.props[2]);

   o.KeyButton = state:addWidget(ButtonAnim, o.layer, o.partition, 0,0, 0,0, "KeyBuyResource", textureSheet, nil, o.goKeyStore, nil, nil, nil, nil);
   o.KeyButton:setPriority(PRIORTY_100);
   o.KeyButton:setParent(o.ShopBarPivot);
   o.KeyCounter = state:addWidget(TextBox, o.layer, 70, 0, 180, 81, App.data:getKeyCount()..'', self.defaultFontName, 15, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   o.KeyCounter:setParent(o.ShopBarPivot);
   o.KeyCounter.moaiTextBox:setPriority(PRIORTY_100);
	o.KeyCounter:setParent(o.KeyButton.anim.props[2]);


   return o;

end


----------------------------------------------------------------

function ResourceBar:onUpdate( dt )

   self:_updateGoldCount();
   self:_updateGemCount();
   self:_updateKeyCount();
   --self:updateHolidayTag();

end

----------------------------------------------------------------

function ResourceBar:_updateGoldCount()

   local MAX_GOLD_DISPLAY = 9999999;

   local gold = App.data:getGoldCount();

   if gold ~= self.gold then

      self.gold = gold;

      gold = math.min(gold, MAX_GOLD_DISPLAY);

      if gold >= 1000000 then
         self.CoinCounter:setGlyphScale(0.80);
      else
         self.CoinCounter:setGlyphScale(1.00);
      end

      self.CoinCounter:setText("<c:d4e7f9>" .. getFormattedNumber(gold) .. "</>");

   end
end

----------------------------------------------------------------

function ResourceBar:_updateGemCount()

   local MAX_GEM_DISPLAY = 99999;

   local gem = App.data:getGemCount();

   if gem ~= self.gem then

      self.gem = gem;

      gem = math.min(gem, MAX_GEM_DISPLAY);

      if gem >= 10000 then
         self.GemCounter:setGlyphScale(0.80);
      else
         self.GemCounter:setGlyphScale(1.00);
      end

      self.GemCounter:setText("<c:d4e7f9>" .. getFormattedNumber(gem) .. "</>");

   end
end
----------------------------------------------------------------

function ResourceBar:_updateKeyCount()

   local MAX_KEY_DISPLAY = 99999;

   local key = App.data:getKeyCount();

   if key ~= self.key then

      self.key = key;

      key = math.min(key, MAX_KEY_DISPLAY);

      if key >= 10000 then
         self.KeyCounter:setGlyphScale(0.80);
      else
         self.KeyCounter:setGlyphScale(1.00);
      end

      self.KeyCounter:setText("<c:d4e7f9>" .. getFormattedNumber(key) .. "</>");

   end
end

----------------------------------------------------------------

function ResourceBar:updateHolidayTag()

   if StateMgr.headerAvailable and PromoMgr:getCurrentSales("holiday") then

      local s = StateMgr:getCurState();

      if s.mainLayer and not self.SaleTag then

         local textureSheet2 = FlashMgr:getTextureSheet("resources/flash-resource-bundle/Group2.lua", "resources/flash-resource-bundle/Group2.png", 30);
         self.SaleTag = s:addWidget(Anim, s.mainLayer, 35, -75, 0, 0, "S_RedTag", textureSheet2, nil, nil, true);
         self.SaleTag:setPriority(PRIORTY_100);
         self.SaleTag:setParent(self.GemButton.anim.props[2]);
         self.tagVisible = true;

      end

      if s.stateFilename and self.SaleTag then
         if s.stateFilename == "state/state-gem.lua" then
            if self.tagVisible then
               self.SaleTag:setVisible(false);
               self.tagVisible = false;
            end
         end
      end
   else
      if self.SaleTag then
         self.SaleTag:setParent(nil);
         self.SaleTag:setVisible(false);
      end
   end

end

----------------------------------------------------------------

function ResourceBar:createConsumableButton()

   if AbTestMgr:isConsumableStoreEnabled() and App.data:isConsumableUnlocked() and not self.ConsumableButton then

      local textureSheet      = FlashMgr:getTextureSheet("resources/flash-resource-bundle/MainScreen.lua", "resources/flash-resource-bundle/MainScreen.png", 30);

      --self.cBackground        = self.state:addWidget(Image, self.layer, -270, self.y-68, 100, 66, "resources/square.png", { 116/255*0.75, 185/255*0.75, 207/255*0.75, 0.75 });
      --self.cBackground:setPriority(PRIORTY_100);

      self.ConsumableButton   = self.state:addWidget(ButtonImage, self.layer, -262, self.y-68, 102, 68, "resources/UI/ResourceBar_ConsumableButton.png", nil, self.goConsumableStore, nil, nil);
      self.ConsumableButton:setPriority(PRIORTY_100);
      --self.ConsumableButton:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self.cBackground.prop, MOAIProp2D.TRANSFORM_TRAIT );

      self.ConsumableSparkle  = self.state:addWidget(Anim, self.layer, -262, self.y-68, 0,0, "NotificationSparkle", textureSheet);
      self.ConsumableSparkle:setPriority(PRIORTY_100-1);
      self.ConsumableSparkle:setScale(1.5, 1.5);
      self.ConsumableSparkle:setSpan(29/30);
      self.ConsumableSparkle:setMode(MOAITimer.LOOP);
      self.ConsumableSparkle:setVisible(false);
   end

end

----------------------------------------------------------------

function ResourceBar:setConsumableCallback( obj, func  )

   self.state._resourceBarConsumableCallObj  = obj;
   self.state._resourceBarConsumableCallFunc = func;

end

----------------------------------------------------------------

function ResourceBar.goConsumableStore( state )

   MOAIFlurry:event("ui-resource-store",      "type", "consumable",  "state", state.stateFilename );
   MOAIFlurry:event("ui-resource-store-consumable",                  "state", state.stateFilename );

   if state._resourceBarConsumableCallFunc then
      state._resourceBarConsumableCallFunc( state._resourceBarConsumableCallObj );
   else
      StateMgr:push("state/state-consumable.lua");
   end

end

----------------------------------------------------------------

function ResourceBar:setGoldCallback( obj, func  )

   self.state._resourceBarGoldCallObj  = obj;
   self.state._resourceBarGoldCallFunc = func;

end

----------------------------------------------------------------

function ResourceBar.goCoinStore( state )

   MOAIFlurry:event("ui-resource-store",      "type", "coin",  "state", state.stateFilename );
   MOAIFlurry:event("ui-resource-store-coin",                  "state", state.stateFilename );

   if state._resourceBarGoldCallFunc then

      state._resourceBarGoldCallFunc( state._resourceBarGoldCallObj );

   else

      StateMgr:push("state/state-coin.lua");

   end

end


----------------------------------------------------------------

function ResourceBar:setGemCallback( obj, func  )

   self.state._resourceBarGemCallObj  = obj;
   self.state._resourceBarGemCallFunc = func;

end

----------------------------------------------------------------

function ResourceBar.goGemStore( state )

   MOAIFlurry:event("ui-resource-store",        "type", "gem",    "state", state.stateFilename );
   MOAIFlurry:event("ui-resource-store-gem",                      "state", state.stateFilename );

   if state._resourceBarGemCallFunc then

      state._resourceBarGemCallFunc( state._resourceBarGemCallObj );

   else

      StateMgr:push("state/state-gem.lua");

   end

end

----------------------------------------------------------------

function ResourceBar:setKeyCallback( obj, func  )

   self.state._resourceBarKeyCallObj  = obj;
   self.state._resourceBarKeyCallFunc = func;

end

----------------------------------------------------------------

function ResourceBar.goKeyStore( state )

   MOAIFlurry:event("ui-resource-store",        "type", "key", "state", state.stateFilename);
   MOAIFlurry:event("ui-resource-store-key",                   "state", state.stateFilename );

   if state._resourceBarKeyCallFunc then

      state._resourceBarKeyCallFunc( state._resourceBarKeyCallObj );

   else

      StateMgr:push("state/state-key.lua");

   end

end

----------------------------------------------------------------

return ResourceBar;

----------------------------------------------------------------
