----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-scroll-list');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local ACH_HOME_X			= 0;
local ACH_HOME_Y			= 270;
local ACH_SPACING			= 182;

local SCROLL_WIND_MIN_X		= -App.viewportWidth/2;
local SCROLL_WIND_MAX_X		= App.viewportWidth/2;
local SCROLL_WIND_MIN_Y		= -352;
local SCROLL_WIND_MAX_Y		= 355;

local SWEET_SPOT_MIN_X		= -175;
local SWEET_SPOT_MAX_X		=  175;
local SWEET_SPOT_MIN_Y		= SCROLL_WIND_MIN_Y;
local SWEET_SPOT_MAX_Y		= SCROLL_WIND_MAX_Y;

local SWIPE_BUFFER_X			= 100;
local SWIPE_BUFFER_Y			=  50;

local COST_ICON_X				= 210;
local COST_ICON_Y				= 5;
local COST_X					= 230;
local COST_Y					= 5;

local PRIORTY_0				= 10;
local PRIORTY_1				= 20;
local PRIORTY_2				= 30;
local PRIORTY_3				= 40;
local PRIORTY_4				= 50;
local PRIORTY_5				= 60;
local PRIORTY_6				= 70;
local PRIORTY_7				= 80;
local PRIORTY_8				= 90;
local PRIORTY_9				= 100;
local PRIORTY_10				= 110;

local DEFAULT_ICON         = "resources/UI/Banner_DailyLitStar.png";

local TEXTURE_FPS          = 30;
local PROGRESS_SPEED       = 1.5;

local ACHIEVEMENT_TITLE_X = 22;


----------------------------------------------------------------

local Base						= State;
local StateAchievement				= Base:new();

----------------------------------------------------------------

function StateAchievement:onLoad()

   Base.onLoad( self );
   
   self.defaultFontName = "resources/font/TangoLatin.otf";

	self.mainTexture		= FlashMgr:getTextureSheet("resources/flash-resource-bundle/MainScreen.lua", "resources/flash-resource-bundle/MainScreen.png", TEXTURE_FPS);
	self.group3Texture		= FlashMgr:getTextureSheet("resources/flash-resource-bundle/Group3.lua", "resources/flash-resource-bundle/Group3.png", TEXTURE_FPS);
	self.popupsTexture		= FlashMgr:getTextureSheet("resources/flash-resource-bundle/PopUPs.lua", "resources/flash-resource-bundle/PopUPs.png", TEXTURE_FPS);
	self.mysteryTexture		= FlashMgr:getTextureSheet("resources/flash-resource-bundle/Mystery_Box_Centered.lua", "resources/flash-resource-bundle/Mystery_Box_Centered.png", TEXTURE_FPS);

	-- background, frame, and title
	self.BackDrop			= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "MainFrameBackground", self.mainTexture);
	self.Frame				= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "PU_ScrollFrame", self.mainTexture);
	self.Frame:setPriority(PRIORTY_5);

   if StateMgr.headerAvailable then
      self.Title			= self:addWidget(Anim, self.mainLayer, 0, 430, 0,0, "TitleAchievements", self.popupsTexture, nil, nil, true);
      self.Title:setPriority(PRIORTY_6);
      self.Title:setScale(0.9, 0.9);
   end
   
	-- resource bar
   self:_setupResourceBar();

	-- push/pop state buttons
	self.BackButton		= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "PU_DoneButton", self.mainTexture, nil, self.goBack, nil, nil, nil, nil, 40);
	self.BackButton:setPriority(PRIORTY_6);
	
	-- scroll list
	self.AchievementCount   = AchievementMgr:getAchievementCount() or 0;


	self.ScrollList		= self:addWidget(		ScrollList,
															self.mainLayer,
															ACH_HOME_X, ACH_HOME_Y,
															{ {SCROLL_WIND_MIN_X, SCROLL_WIND_MAX_X},	{SCROLL_WIND_MIN_Y, SCROLL_WIND_MAX_Y} },
															nil,
															nil,
															{ 0, (1/4)*ACH_SPACING },
															{ nil, ACH_SPACING },
															{ 0, self.AchievementCount },
															nil,
															{ SWIPE_BUFFER_X, SWIPE_BUFFER_Y },
															nil,
															nil,
															{ {SWEET_SPOT_MIN_X, SWEET_SPOT_MAX_X},	{SWEET_SPOT_MIN_Y, SWEET_SPOT_MAX_Y} },
															nil, nil, nil, nil, nil, nil,
															"ScrollBarButton", self.mainTexture, 12, 78, 135, PRIORTY_4 );



end

----------------------------------------------------------------

function StateAchievement:onActivate()

	Base.onActivate(self);
	
	self.ScrollList:setLoc(ACH_HOME_X, ACH_HOME_Y);

   if self.resourceBar then
      self.resourceBar:createConsumableButton();
   end

end

----------------------------------------------------------------

function StateAchievement:onFocus()

   Base.onFocus( self );

	StateMgr:setSyncBusyDelay(0);
   
	-- Achievement Entries
   self.AchievementCount   = AchievementMgr:getAchievementCount() or 0;
   self.ScrollList:setEntryCount( 0, self.AchievementCount )
	self:createEntryList();
   
   self:achievementFocusCalls();

	self:updateEntryList();

end

----------------------------------------------------------------

function StateAchievement:achievementFocusCalls()

	for i=1,#self.EntryList do

      local achievement = AchievementMgr:getAchievementByIndex(self.EntryList[i].pickIndex);
      
      if not achievement then
         return;
      end
      
      if not achievement.goal then
         return;
      end
      
      achievement.goal:achievementScreenFocus();
   end

end

----------------------------------------------------------------

function StateAchievement:onLoseFocus( )

   Base.onLoseFocus(self);
	
	StateMgr:setSyncBusyDelay(nil);
   
end

----------------------------------------------------------------

function StateAchievement:onUpdate( dt )

	Base.onUpdate(self, dt);
   
   self:updateRankupTransitions(dt);
   
   self:updateTickList(dt);

end

----------------------------------------------------------------------

function StateAchievement:isBusy()

   return StoreMgr:isBusy();

end

----------------------------------------------------------------
----------------------------------------------------------------

function StateAchievement:_setupResourceBar()

   self.resourceBar = ResourceBar:new( self, 0, self:_calcHeaderLocY(), self.mainLayer, self.mainPartition  );

   self.resourceBar:setGoldCallback(
         nil
      ,
         function()
				StateMgr:push("state/state-coin.lua");
         end
   );

   self.resourceBar:setGemCallback(
         nil
      ,
         function()
				StateMgr:push("state/state-gem.lua");
         end
   );

   self.resourceBar:setKeyCallback(
         nil
      ,
         function()
				StateMgr:push("state/state-key.lua");
         end
   );

end

----------------------------------------------------------------
----------------------------------------------------------------

function StateAchievement:createEntryList()

	local count                = 1;
	local AchievementCount		= self.AchievementCount + count - 1;
	local n                    = 1;
   if self.EntryList then
      for k,v in pairs(self.EntryList) do
         self:_deleteAchievementEntry( v )
      end
   end
   self.EntryList = {};
	for i=count,AchievementCount do

      local achievement = AchievementMgr:getAchievementByIndex(n);

      if achievement then
         table.insert(self.EntryList, self:_createAchievementEntry(i, n));
      end
		n = n + 1;
	end

end

----------------------------------------------------------------

function StateAchievement:_createAchievementEntry( index, achIndex )

	local tempTable         = {};
	local offset            = self.ScrollList:getEntryOffsetY(achIndex);
	
	tempTable.pickIndex     = achIndex;

   local achievement       = AchievementMgr:getAchievementByIndex(achIndex);
   local icon              = achievement:getIcon() or DEFAULT_ICON;

   tempTable.id            =  achievement.id;

	tempTable.plaque        = self:addWidget(Anim, self.mainLayer, 0, offset, 0,0, "AchievementScreen_MainPanel", self.mysteryTexture);

   tempTable.portrait      = self:addWidget(Image, self.mainLayer,-237.9,offset+17.9,105,105, icon );

   tempTable.checkMark     = self:addWidget(Anim, self.mainLayer, 0,offset,0,0, "AchievementScreen_CheckmarkAnimation", self.mysteryTexture);

   tempTable.rewardAnim    = self:addWidget(Anim, self.mainLayer, 0,offset,0,0, "AchievementScreen_RewardAnimation", self.mysteryTexture);

	tempTable.titleText     = self:addWidget(TextBox, self.mainLayer, ACHIEVEMENT_TITLE_X, offset+45, 382, 48, "", self.defaultFontName, 16, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);

	tempTable.desc          = self:addWidget(TextBox, self.mainLayer, ACHIEVEMENT_TITLE_X, offset-13, 382, 48, "", self.defaultFontName, 9.6, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);

   tempTable.progressBar   = self:addWidget(Anim, self.mainLayer, 0,offset,0,0, "AchievementScreen_ProgressBarAnimation", self.mysteryTexture);

	tempTable.progress      = self:addWidget(TextBox, self.mainLayer, 0, offset-63, 382, 48, "", self.defaultFontName, 9.6, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);

	tempTable.reward        = self:addWidget(TextBox, self.mainLayer, 262, offset-40, 382, 75, "", self.defaultFontName, 17.6, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   
   local numRanks = achievement:getMaxRank();

   tempTable.stars         = {};
   if numRanks then
      for i=1,numRanks do
         tempTable.stars[i]= self:addWidget(Anim, self.mainLayer, -273+(35*(i-1)), offset-55,0,0, "AchievementScreen_StarAnimation", self.mysteryTexture);
      end
   end

   tempTable.desc:setColor(0.62, 0.8, 0.95, 1);
	
	tempTable.plaque:setPriority(PRIORTY_0);
	tempTable.portrait:setPriority(PRIORTY_1);
   tempTable.checkMark:setPriority(PRIORTY_1+1);
   tempTable.rewardAnim:setPriority(PRIORTY_1+1);
	tempTable.titleText:setPriority(PRIORTY_1+1);
	tempTable.desc:setPriority(PRIORTY_1+1);
   tempTable.progressBar:setPriority(PRIORTY_1+1);
   tempTable.progress:setPriority(PRIORTY_1+1);
   if numRanks then
      for i=1,numRanks do
         tempTable.stars[i]:setPriority(PRIORTY_1+1);
      end
   end
   tempTable.reward:setPriority(PRIORTY_1+1);

	
	tempTable.plaque:setParent(self.ScrollList.root);
	tempTable.portrait:setParent(self.ScrollList.root);
   tempTable.checkMark:setParent(self.ScrollList.root);
   tempTable.rewardAnim:setParent(self.ScrollList.root);
	tempTable.titleText:setParent(self.ScrollList.root);
	tempTable.desc:setParent(self.ScrollList.root);
   tempTable.progressBar:setParent(self.ScrollList.root);
   tempTable.progress:setParent(self.ScrollList.root);
   if numRanks then
      for i=1,numRanks do
         tempTable.stars[i]:setParent(self.ScrollList.root);
      end
   end
   tempTable.reward:setParent(self.ScrollList.root);
   
   tempTable.progressBar:setSpeed(PROGRESS_SPEED);
	
	return tempTable;

end

----------------------------------------------------------------

function StateAchievement:_deleteAchievementEntry( entry )


	self:deleteWidget(entry.plaque);
   self:deleteWidget(entry.portrait);
   self:deleteWidget(entry.checkMark);
   self:deleteWidget(entry.rewardAnim);
	self:deleteWidget(entry.titleText);
	self:deleteWidget(entry.desc);
   self:deleteWidget(entry.progressBar);
	self:deleteWidget(entry.progress);
   self:deleteWidget(entry.reward);

   for k,v in pairs(entry.stars) do
      self:deleteWidget(v);
   end


end

----------------------------------------------------------------

function StateAchievement:updateEntryList()

	for i=1,#self.EntryList do

      local achievement = AchievementMgr:getAchievementByIndex(self.EntryList[i].pickIndex);
      
      if not achievement then
         return;
      end
      
      if not achievement.goal then
         return;
      end

      local finished    = achievement:isFinished();
      local completed   = achievement:isCompleted();
      local objective   = achievement:getObjective();
      local progress    = achievement:getProgress();
      local title       = achievement:getTitle();
      local desc        = achievement:getDesc();
      local icon        = achievement:getIcon();
      local rank        = achievement:getRank();
      local rewardTxt   = achievement:getRewardTxt();
      
      if completed and not finished and not self.inputDisabled then
         self:disableAchievementInput();
      end

      
      local percent = nil;
      if progress and objective then
         percent = self:roundNum( (progress/objective)*100, 0 );
      end
      if not percent then
         percent = 0;
      end
      if percent < 1 or percent > 100 then
         percent = 0;
      end

      local oldPercent  = App.data:getValue("lastSeenPercentEntry"..i);
      if not oldPercent then
         oldPercent = 0;
      elseif not completed and oldPercent > percent then
         App.data:setValue("lastSeenPercentEntry"..i, 0);
         oldPercent = 0;
      end
      if oldPercent < 1 or oldPercent > 100 then
         oldPercent = 0;
      end
      
      if not completed then
         App.data:setValue("lastSeenPercentEntry"..i, percent);
      else
         App.data:setValue("lastSeenPercentEntry"..i, 0);
      end
      
      if rank and rank > 1 then
         for j=1,rank-1 do
            self.EntryList[i].stars[j]:setAnimLastFrame();
         end
      end

      self.EntryList[i].titleText:setText(title);
      self.EntryList[i].reward:setText(rewardTxt);
      self.EntryList[i].desc:setText(desc);
      
      if completed and not finished then

         self:addRankupTransition( i, oldPercent, rewardTxt, rank+1, finished  );

      else
      
         if not finished then
            self.EntryList[i].progress:setText(percent.."%");
            self.EntryList[i].progressBar:setAnimFrame( percent, TEXTURE_FPS );
         else
            self.EntryList[i].progress:setText("");
            self.EntryList[i].checkMark:setAnimLastFrame();
            self.EntryList[i].progressBar:setVisible(false);
         end
      
      end

	end

end

----------------------------------------------------------------

function StateAchievement:disableAchievementInput()

   self.BackButton:setColor(1,1,1,0.25);
   self.inputDisabled = true;
   disableButtonInput();

end

----------------------------------------------------------------

function StateAchievement:enableAchievementInput()

   self.BackButton:setColor(1,1,1,1);
   self.inputDisabled = nil;
   enableButtonInput();

end

----------------------------------------------------------------

function StateAchievement:addProgressBarUptick( e, old, current )

   local t1 = old/TEXTURE_FPS;
   local t2 = current/TEXTURE_FPS;
   self.EntryList[e].progressBar:stop();
   self.EntryList[e].progressBar:setSpan(t1, t2);
   self.EntryList[e].progressBar:setTime(t1);
   self.EntryList[e].progressBar:start();
   if not self.tickList then
      self.tickList = {};
   end
   if old < 1 then
      old = 0;
   end
   if current < 1 then
      current = 0;
   end
   table.insert(self.tickList, {entry=e, totalTime=(t2-t1)/PROGRESS_SPEED, pStart=old, pEnd=current} );

end

----------------------------------------------------------------

function StateAchievement:updateTickList( dt )

   if self.tickList then
      for i,v in pairs(self.tickList) do
         if not v.time then 
            v.time = 0;
            self.EntryList[v.entry].progress:setText(v.pStart.."%");
            if not self.uptickSFX then
               self.uptickSFX = true;
               local cSounds = {loop={"ScoreCountUp",},};
               EventMgr:postEvent( { sounds=cSounds, name="progress loop",}, "add-sfx" );
            end
         else
            v.time = v.time + dt;
            local progress = v.time/v.totalTime;
            local tick = self:roundNum( ((v.pEnd-v.pStart)*progress)+v.pStart, 0 );

            if tick >= v.pStart and tick <= v.pEnd then
               self.EntryList[v.entry].progress:setText(tick.."%");
            elseif tick > v.pEnd then
               self.EntryList[v.entry].progress:setText(v.pEnd.."%");
            end
            if v.time >= v.totalTime then
               table.remove(self.tickList, i);
               if #self.tickList < 1 then
                  self.uptickSFX = nil;
                  local cSounds = {loop={"ScoreCountUp",},};
                  EventMgr:postEvent( {sounds=cSounds, name="progress loop stop",}, "loop-stop" );
               end
            end
         end
      end
   end

end


----------------------------------------------------------------

function StateAchievement:roundNum( val, decimal )

  local exp = decimal and 10^decimal or 1
  return math.ceil(val * exp - 0.5) / exp

end



----------------------------------------------------------------
----------------------------------------------------------------

function StateAchievement:addRankupTransition( i, sPerc, reward, nRank, finished  )

   if not self.rankUpTransitions then
      self.rankUpTransitions = {};
   end
   
   local bt = ((100-sPerc)/TEXTURE_FPS)/PROGRESS_SPEED;
   
   local rTable =    {
                        entry          = i,
                        start          = sPerc,
                        nextRank       = nRank,
                        timer          = 0,
                        barEnd         = bt+0.2,
                        gemTime        = bt+0.2,
                        rankUpTime     = bt+0.3,
                        bannerGems     = reward,
                        gemReset       = bt+1.3,
                        starTime       = bt+0.2,
                        transitionEnd  = bt+2,
                        failsafeTimer  = 6,

                     };
   if finished then
      rTable.checkTime = bt;
   else
      rTable.checkTime = nil;
   end
   
   table.insert(self.rankUpTransitions, rTable);


end

----------------------------------------------------------------

function StateAchievement:updateRankupTransitions( dt )

   if self.rankUpTransitions then

      if self.rankUpTransitions[1] then
         
         local c = self.rankUpTransitions[1];
         
         if c.entry then

            if c.started then
               c.timer = c.timer + dt;
               if c.timer >= c.barEnd and c.barEnd > 0 then
                  c.barEnd = -1;
                  --transition to new goal
                  --moved to accomodate the new rankup
                  --self:updateSpecificEntry( c.entry );
               end
               if c.timer >= c.starTime and c.starTime > 0 then
                  c.starTime = -1;
                  --play current star rank
                  if c.nextRank and self.EntryList[c.entry].stars[c.nextRank-1] then
                     self.EntryList[c.entry].stars[c.nextRank-1]:start();
                  end
               end
               if c.timer >= c.gemTime and c.gemTime > 0 then
                  c.gemTime = -1;
                  --play gem flash
                  self.EntryList[c.entry].rewardAnim:start();
                  self.EntryList[c.entry].reward:setColor(1,1,1,0.3);
               end
               if c.timer >= c.rankUpTime and c.rankUpTime > 0 then
                  c.rankUpTime = -1;
                  local ach = AchievementMgr:getAchievementByIndex(self.EntryList[c.entry].pickIndex);
                  if ach then
                     ach:nextRank();
                     c.rankUpComplete = true;
                     self:updateSpecificEntry( c.entry );
                     EventMgr:postEvent( { gem=c.bannerGems }, "achievement-reward" );
                  end
               end
               if c.timer >= c.gemReset and c.gemReset > 0 and c.rankUpComplete then
                  c.gemReset = -1;
                  --reset gem animation
                  self.EntryList[c.entry].rewardAnim:stop();
                  self.EntryList[c.entry].rewardAnim:setSpan(0,0);
                  self.EntryList[c.entry].rewardAnim:start();
                  self.EntryList[c.entry].reward:setColor(1,1,1,1);

                  local ach = AchievementMgr:getAchievementByIndex(self.EntryList[c.entry].pickIndex);

                  if ach then
                     local newGemText = ach:getRewardTxt();
                     if newGemText then
                        self.EntryList[c.entry].reward:setText(newGemText);
                     end
                  end

               end
               if c.checkTime and c.timer >= c.checkTime and c.checkTime > 0 then
                  c.checkTime = -1;
                  --play checkmark
                  --self.EntryList[c.entry].checkMark:start();
               end
               if c.timer >= c.transitionEnd and c.transitionEnd > 0 then
                  --remove this table entry
                  c.transitionEnd = -1;
                  table.remove(self.rankUpTransitions, 1);
                  if #self.rankUpTransitions < 1 then
                     self:enableAchievementInput();
                  end
               end

            elseif self.ScrollList:isEntryInView(c.entry) and not c.started then

               c.started = true;
               self:addProgressBarUptick( c.entry, c.start, 100 );

            elseif not c.seeking then
               --scroll to entry
               c.seeking = true;
               self.ScrollList:advanceToEntry( c.entry, 1.0);

            elseif c.failsafeTimer > 0 then
               c.failsafeTimer = c.failsafeTimer - dt;
               if c.failsafeTimer <= 0 then
                  c.failsafeTimer = -1
                  self:addProgressBarUptick( c.entry, c.start, 100 );
                  c.started = true;
               end
            end
            
         end

      end

   end


end

----------------------------------------------------------------

function StateAchievement:updateSpecificEntry( entry )


   local achievement = AchievementMgr:getAchievementByIndex(self.EntryList[entry].pickIndex);
   local offset      = self.ScrollList:getEntryOffsetY(self.EntryList[entry].pickIndex);
   
   if not achievement then
      return;
   end
   
   if not achievement.goal then
      return;
   end

   local finished    = achievement:isFinished();
   local title       = achievement:getTitle();
   local desc        = achievement:getDesc();
   local icon        = achievement:getIcon();
   local rank        = achievement:getRank();
   local rewardTxt   = achievement:getRewardTxt();


   self.EntryList[entry].portrait:swapImage(icon);
   self.EntryList[entry].portrait:setLoc(-237.9,offset+17.9);
   self.EntryList[entry].portrait:setParent(self.ScrollList.root);
   self.EntryList[entry].titleText:setText(title);
   self.EntryList[entry].desc:setText(desc);
   
   local sSound = {start={"Applause",},};
   EventMgr:postEvent( { sounds=sSound, name="achievement applause",}, "add-sfx" );
   
   local sSound = {start={"PurchaseGold",},};
   EventMgr:postEvent( { sounds=sSound, name="achievement sting",}, "add-sfx" );
   
   if finished then
      self.EntryList[entry].progress:setText("");
      self.EntryList[entry].progressBar:setVisible(false);
      self.EntryList[entry].checkMark:start();
   else

      self.EntryList[entry].progress:setText("0%");
      self.EntryList[entry].progressBar:stop();
      self.EntryList[entry].progressBar:setSpan( 0, 0 );
      self.EntryList[entry].progressBar:start();
   end






end

----------------------------------------------------------------
----------------------------------------------------------------

function StateAchievement:showErrorPopup( message )
	
	local text = message or LocalizationMgr:getString( "NATIVE_TRY_AGAIN" );

   MOAIDialogIOS.showDialog(	LocalizationMgr:getString( "NATIVE_TRANS_FAIL" )
                              ,	text
                              ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                              ,  nil
                              ,  nil
                              ,  false
                              ,  function()
                                 end
                           );
																					
end

----------------------------------------------------------------
----------------------------------------------------------------

function StateAchievement:goBack()
	
	StateMgr:pop();

end

----------------------------------------------------------------

return StateAchievement;

----------------------------------------------------------------
