----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local PRIORTY_0				= 10;
local PRIORTY_1				= 20;
local PRIORTY_2				= 30;
local PRIORTY_3				= 40;
local PRIORTY_4				= 50;
local PRIORTY_5				= 60;
local PRIORTY_6				= 70;
local PRIORTY_7				= 80;
local PRIORTY_8				= 90;
local PRIORTY_9				= 100;
local PRIORTY_10				= 110;



----------------------------------------------------------------

local Base									= State;
local StateAuthenticateConfirm		= Base:new();

----------------------------------------------------------------

function StateAuthenticateConfirm:onLoad()

   State.onLoad( self );

   local textureSheet = FlashMgr:getTextureSheet("resources/flash-resource-bundle/Group2.lua",
                                                 "resources/flash-resource-bundle/Group2.png",
                                                 30);

   self.IS_POPUP = true;

   self.background = self:addWidget( Image,
                                     self.mainLayer,
                                     0,
                                     0,
                                     640,
                                     1136,
                                     "resources/UI/Splash.png" );

   self.tangoBubble = self:addWidget( Image,
                                      self.mainLayer,
                                      200,
                                      290,
                                      70,
                                      61,
                                      "resources/UI/Splash_Tango.png" );

end


----------------------------------------------------------------

function StateAuthenticateConfirm:onFocus()

   State.onFocus(self);

   disableButtonInput();

   StateMgr:setSyncBusyDelay(1);

   EventMgr:postEvent({}, "will-authenticate" )

   App.data:setValue( "cheater.tango.authentication.inprogress", true );

   delay( 0.1,
          function()
             self:goAuthenticate();
          end
   );

end

----------------------------------------------------------------

function StateAuthenticateConfirm:onLoseFocus( )

   Base.onLoseFocus(self);

   StateMgr:setSyncBusyDelay(nil);
end

----------------------------------------------------------------------

function StateAuthenticateConfirm:isBusy()
   return true;
end

----------------------------------------------------------------------

function StateAuthenticateConfirm:getBusyName()
   return "Signing in to Tango";
end

----------------------------------------------------------------

function StateAuthenticateConfirm:goAuthenticate()

   local tangoState = "";
   if MOAITangoSdk:tangoHasSdkSupport() then
      tangoState = "yes-sdk-supported";
   elseif MOAITangoSdk:tangoIsInstalled() then
      tangoState = "yes-sdk-unsupported";
   else
      tangoState = "no";
   end

   MOAIFlurry:eventStart(  "TangoFirstAuthentication-2"
                        ,  "tangoState",                 tostring(tangoState)
                        ,  "game-played",                tostring(App.data:getGamePlayed() or 0)
                        ,  "result",                     "processing"
                        ,  "error",                      "processing"                           );

   if OnlineMgr:isOnline() then

      local requestSuccessFunc = function()
         print("Tango auth success");
         StateAuthenticateConfirm:_onAuthSuccess(tangoState);
      end

      local requestFailureFunc = function(code, message)
         print("Tango auth failure");
         StateAuthenticateConfirm:_onAuthFailure(code, message);
      end

      local requestOptions = {
         requestClass = MOAITangoAuthenticateRequest,
         requestData = { },
         successFunc = requestSuccessFunc,
         failureFunc = requestFailureFunc,
         priority = Request.PRIORITY_HIGH,
      }

      NetworkMgr:sendTangoRequest(TangoRequest.newFromBase(requestOptions));

   else

      MOAIFlurry:event( "TangoFirstAuthentication-Error",
                        "code",  "offline",
                        "error", "offline" );

      MOAIFlurry:eventStop( "TangoFirstAuthentication-2",
                            "result",  "offline",
                              "error", "offline" );

      MOAIDialogIOS.showDialog(	LocalizationMgr:getString( "NATIVE_NOT_ONLINE_TITLE" )
                              ,  LocalizationMgr:getString( "NATIVE_NOT_ONLINE_MSG" )
                              ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                              ,  nil
                              ,  nil
                              ,  false
                              ,  function()
                                    enableButtonInput();
                                    StateMgr:pop();
                                 end
                              );
   end

end

----------------------------------------------------------------

function StateAuthenticateConfirm:goBack()

	print("Go Back");

	StateMgr:pop();
end

----------------------------------------------------------------

function StateAuthenticateConfirm:isUsingCameraHack()
   return false;
end

----------------------------------------------------------------

function StateAuthenticateConfirm:_onAuthSuccess(tangoState)

   print("Authentification Success");


   MOAIFlurry:eventStop(   "TangoFirstAuthentication-2"
                           ,  "result",                     "success"
                           ,  "error",                      "none"         );

   MOAIFlurry:event(   "TangoFirstAuthentication-Success"
                       ,  "tangoState",                     tostring(tangoState)     );

   enableButtonInput();

   HACK_NO_AUTHENTICATION = false;

   StateMgr:clear('state/state-splash.lua');

end

function StateAuthenticateConfirm:_onAuthFailure(code, message)

   print("error", code, message);

   local title = LocalizationMgr:getString( "NATIVE_AUTH_ERROR_TITLE_1" );

   if code == MOAITangoSdk.TANGO_SDK_TANGO_DEVICE_NOT_VALIDATED then
      title = LocalizationMgr:getString( "NATIVE_AUTH_ERROR_TITLE_2" );
   end

   MOAIFlurry:eventStop( "TangoFirstAuthentication-2",
                         "result", "failure",
                         "error",  tostring(message) );



   MOAIFlurry:event( "TangoFirstAuthentication-Error",
                     "code",  tostring(code),
                     "error", tostring(message) );

   MOAIDialogIOS.showDialog( title
                             , message
                             ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                             ,  nil
                             ,  nil
                             ,  false
                             ,  function()
                                   enableButtonInput();
                                   StateMgr:pop();
                                end
   );

end

----------------------------------------------------------------

return StateAuthenticateConfirm;

----------------------------------------------------------------
