----------------------------------------------------------------

require('app/contact/contact-manager')
require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

BragData							= nil;

----------------------------------------------------------------

local SLIDE_MIN				= -App.viewportHeight/2;
local SLIDE_MAX				= App.viewportHeight/2;
local SWIPE_WIN_MIN_X		= -App.viewportWidth/2;
local SWIPE_WIN_MAX_X		= App.viewportWidth/2;
local SWIPE_WIN_MIN_Y		= -245;
local SWIPE_WIN_MAX_Y		= 285;
local ROLLOUT_X				= 0;
local ROLLOUT_Y				= 0;
local ROLLOUT_SHIFT			= 570;
local RANK_LiST_HOME_X		= 0;
local RANK_LiST_HOME_Y		= 230;
local RANK_LiST_SPACING		= 105;
local REWARD_COUNT_X			= -170;
local REWARD_COUNT_Y			= 324;

local PRIORTY_0				= 10;
local PRIORTY_1				= 20;
local PRIORTY_2				= 30;
local PRIORTY_3				= 40;
local PRIORTY_4				= 50;
local PRIORTY_5				= 60;
local PRIORTY_6				= 70;
local PRIORTY_7				= 80;

----------------------------------------------------------------

local Base						= State;
local StateBrag				= Base:new();
local textureSheet			= FlashMgr:getTextureSheet("resources/flash-resource-bundle/MainScreen.lua", "resources/flash-resource-bundle/MainScreen.png", 30);

----------------------------------------------------------------

function StateBrag:onLoad()

	State.onLoad( self );

   self.defaultFontName = "resources/font/TangoLatin.otf";

	-- background and frames
	self.BackDrop			= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "IBBackDrop", textureSheet);
	self.BackDrop:setPriority(1);
	self.BackOverlay		= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "IBBackDrop", textureSheet);
	self.BackOverlay:setPriority(2);
	self.Frame				= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "IBScrollFrame", textureSheet);
	self.Frame:setPriority(PRIORTY_4);
	self.Title				= self:addWidget(Anim, self.mainLayer, 0,430,0,0, "TitleBrag", textureSheet, nil, nil, true);
	self.Title:setPriority(PRIORTY_5);

	-- Select All/Send buttons
	self.SelectAllTitle	= self:addWidget(TextBox, self.mainLayer, 110, 324, 540, 140, '<c:1b354f>Select All<c>', self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.SelectAllTitle:setPriority(PRIORTY_6);
	self.SelectAllButton	= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "IBChackAllBoxButton", textureSheet, nil, self.addAllSelection, nil, nil, nil, nil, 20);
	self.SelectAllButton:setPriority(PRIORTY_6);

   self.BackButton		= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "IBBackButton", textureSheet, nil, self.goBack, nil, nil);
   self.SendButton		= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "IBSendButton", textureSheet, nil, self.sendBrag, nil, nil);

   self.SendButton:setPriority(PRIORTY_6);
	self.BackButton:setPriority(PRIORTY_6);

	-- reward counter
	self.RewardTitle		= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "IBDailyReward", textureSheet, nil, nil, true);
	self.RewardTitle:setPriority(PRIORTY_6);
	self.KeyTray			= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "IBKayTry", textureSheet, nil, nil, true);
	self.KeyTray:setPriority(PRIORTY_6-1);
	self.RewardText		= self:addWidget(TextBox, self.mainLayer, REWARD_COUNT_X, REWARD_COUNT_Y, 540, 140, ' ', self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.RewardText:setPriority(PRIORTY_6);

   self:_setupBasicResourceBar();

	-- scroll list
	self.ScrollList		= self:addWidget(		ScrollList,
															self.mainLayer,
															RANK_LiST_HOME_X, RANK_LiST_HOME_Y,
															{ {SWIPE_WIN_MIN_X, SWIPE_WIN_MAX_X},	{SWIPE_WIN_MIN_Y, SWIPE_WIN_MAX_Y} },
															nil,
															nil,
															{ 0, (1/40)*RANK_LiST_SPACING },
															{ nil, RANK_LiST_SPACING },
															{ 0, 1 },
															nil,
															{ 100, 50 },
															{0, RANK_LiST_SPACING},
															nil,
															{ {-320, 200},	{SLIDE_MIN, SLIDE_MAX} },
															nil, nil, nil, nil, nil, nil,
															"ScrollBarButton", textureSheet, 12, 0, 145, PRIORTY_5 );
	self:createEntryCache();

end

----------------------------------------------------------------

function StateBrag:onActivate()

	Base.onActivate(self);

   if self.resourceBar then
      self.resourceBar:createConsumableButton();
   end

	self:setPlayerName();

end

----------------------------------------------------------------

function StateBrag:onFocus()

	State.onFocus( self );
	
   self:createEntryList();
	self:createTangoMessage();
	self.ScrollList:setEntryCount(nil, #self.EntryList);
	self.ScrollList:setEntryData(self.EntryList);
	self.ScrollList:goToEntry(1);

   if self:isValidFriendAvailable() then
      self.SelectAllButton:setAnimFrame();

      self:enableSelectAll(true);

      if LoginMgr:isTango() then

			if AbTestMgr:getBragAutoSelectAll() == true then
				self.SelectAllMode = "off";

				self:enableSend(true);
				self:addAllSelection();
			elseif			AbTestMgr:getBragAutoSelectAll() == nil
						and	string.lower(App.data:getValue("App.settings.brag-select-all", "auto")) == "auto" then

				self.SelectAllMode = "off";

				self:enableSend(true);
				self:addAllSelection();
			else
				self:enableSend(false);
				self:updateRewardCount();
			end
      else
			self:enableSend(false);
			self:updateRewardCount();
		end
   else
      self:enableSelectAll(false);
      self:enableSend(false);
   end

	EventMgr:addListener( self, self.onNewPofilePicture, "new-profile-picture");
	
end

----------------------------------------------------------------

function StateBrag:onLoseFocus( )

   Base.onLoseFocus(self);

	EventMgr:rmvListener( self );

end

----------------------------------------------------------------

function StateBrag:onUpdate( dt )

   Base.onUpdate( self, dt );

end

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

function StateBrag:enableSelectAll( mode )

	if mode then
		self.SelectAllTitle:setVisible(true);
		self.SelectAllButton:setVisible(true);
		
		self.SelectAllButton:enable();
	else
		self.SelectAllTitle:setVisible(false);
		self.SelectAllButton:setVisible(false);
		
		self.SelectAllButton:disable();
	end

end

----------------------------------------------------------------

function StateBrag:enableSend( mode )

	if mode then
		self.SendButton:enable();
		self.SendButton:setColor(1.0, 1.0, 1.0, 1.0);
	else
		self.SendButton:disable();
		self.SendButton:setColor(0.2, 0.2, 0.2, 1.0);
	end

end

----------------------------------------------------------------

function StateBrag:isValidFriendAvailable()

	if ContactMgr:getFriendsCount() < 1 then
	
		return false;
	end
	
	if not self.EntryList then
	
		return false;
	else

		for i=1,#self.EntryList do
		
			if self.EntryList[i].used == 'Selected' or self.EntryList[i].used == false then
			
				return true;
			end
		end
	end
	
	return false;

end

----------------------------------------------------------------

function StateBrag:createTangoMessage()

	if (not LoginMgr:isTango() and ( MOAITangoSdk:tangoHasSdkSupport() or MOAIEnvironment.osBrand == "Android")) then

		if not self.EntryList then
			self.EntryList = {};
		end

		local t = {};

		t.tangoID		= "ConnectToTango";
		t.portrait		= "resources/UI/PlayerPortrait.png";
		t.name			= "";
		t.used			= true;

		table.insert(self.EntryList, t);
	end

end

----------------------------------------------------------------

function StateBrag:goTangoConnect()

   if MOAITangoSdk:tangoHasSdkSupport() then
      StateMgr:push("state/state-authenticate-confirm.lua");
   else
		installTango();
   end

end

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

function StateBrag:updateRewardCount()

	local count				= 0;
	local reward			= 0;
	local key				= App.data:getValue('BragRewardTracker') or 0;
   local inTimeLimit		= isTimeInLim('BragRewardTimer2', '', App.data:getValue("App.settings.brag-cooldown") or BRAG_REWARD_TIME_LIMIT, TimeMgr:getTimeGMT()) or false;
	local max				= App.data:getKeyMaxReward();

	if not inTimeLimit then
		key = 0;
	end

	for i=1,#self.EntryList do
		if self.EntryList[i].used == 'Selected' then
			count = count + 1;
		end
	end
	
	if (key >= max) or ((key + count) >= max) then
		reward = max;
	else
		reward = key + count;
	end
		
	self.RewardText:setText(reward..'/'..max);

end

----------------------------------------------------------------

function StateBrag:setPlayerName( )

	self:createMsgTable();
	
	if App.data:getValue('BragMsgIndex2') == nil then
		App.data:setValue('BragMsgIndex2', math.random( 1, #self.Message_List ));
	end

end

----------------------------------------------------------------

function StateBrag:createMsgTable()

   self.Message_List       = {};

	local score;
	
	if not App.lastGameScore then
		score = 0;
	else
		score = App.lastGameScore;
	end

   local bragMessageTemplates = MessageMgr:getBragTemplateList();

   for i=1, #bragMessageTemplates do
      self.Message_List[i]           = {};

      self.Message_List[i].title     = string.gsub( bragMessageTemplates[i].title, "@MYSCORE", score  );
      self.Message_List[i].body      = string.gsub( bragMessageTemplates[i].body,  "@MYSCORE", score  );
      self.Message_List[i].action    = bragMessageTemplates[i].action;
      self.Message_List[i].url       = bragMessageTemplates[i].url;
      self.Message_List[i].thumbnail = bragMessageTemplates[i].thumbnail;
   end

end

----------------------------------------------------------------

function StateBrag:updateBragMsg()

   local idx = App.data:getValue('BragMsgIndex2');

   if self.Message_List and idx and idx >=1 and idx <= #self.Message_List and self.Message_List[idx] then
      self.BragText:setText(self.Message_List[idx].title..'...');
      self.BragTextShadow:setText(self.Message_List[idx].title..'...');
   else
      Debug.assert(self.Message_List and idx and idx >=1 and idx <= #self.Message_List and self.Message_List[idx],
                  "StateBrag:updateBragMsg ->self.Message_List and idx and idx >=1 and idx <= #self.Message_List and self.Message_List[idx]");
   end

end

----------------------------------------------------------------

function StateBrag:createEntryCache()

	local connectText = '<c:6abffd>Play with your Tango\nfriends and get<c>';
   local bonus = App.data:getTangoBonus();

	self.ScrollList:setEntryObjectDataUpdateFunction(	function( data, object )

																			object.banner:setVisible(true);
																			object.picFrame:setVisible(true);
																			object.portrait:setVisible(true);
																			object.userName:setVisible(true);

																			object.userName:setMetaFont("resources/font/Tango.otf", 18, data.name);
																			object.userName:setText(data.name);

																			if object.portrait:getImageName() ~= data.portrait then
																				object.portrait:swapImage(data.portrait);
																				object.portrait:setParent(object.banner.anim.root);
																				object.portrait:setLoc(-240, 0);
																				object.portrait:setPriority(PRIORTY_1);
																			end

																			if data.used then
																				if data.used == 'Selected' then
																					object.clock:setVisible(false);
																					object.inputEmpty:setVisible(false);
																					object.inputCheck:setVisible(true);
																					object.inputEmpty:disable();
																					object.inputCheck:enable();
																				else
																					object.clock:setVisible(true);
																					object.inputEmpty:setVisible(false);
																					object.inputCheck:setVisible(false);
																					object.inputEmpty:disable();
																					object.inputCheck:disable();
																				end
																			else
																				object.clock:setVisible(false);
																				object.inputEmpty:setVisible(true);
																				object.inputCheck:setVisible(false);
																				object.inputEmpty:enable();
																				object.inputCheck:disable();
																			end

																			object.inputEmpty.onReleasedFunc =
																					function ()
																						self:addSelection(data, object)
																					end;

																			object.inputCheck.onReleasedFunc =
																					function ()
																						self:addSelection(data, object)
																					end;

																			if data.tangoID == "ConnectToTango" then
																				object.picFrame:setVisible(false);
																				object.portrait:setVisible(false);
																				object.clock:setVisible(false);
																				object.inputEmpty:setVisible(false);
																				object.inputCheck:setVisible(false);
																				object.tangoConnectText:setVisible(true);
																				object.tangoGemIcon:setVisible(true);
																				object.tangoConnectButton:setVisible(true);

																				object.inputEmpty:disable();
																				object.inputCheck:disable();
																				object.tangoConnectButton:enable();
																			else
																				object.picFrame:setVisible(true);
																				object.portrait:setVisible(true);
																				object.tangoConnectText:setVisible(false);
																				object.tangoGemIcon:setVisible(false);
																				object.tangoConnectButton:setVisible(false);

																				object.tangoConnectButton:disable();
																			end

																		end
																		);

	self.ScrollList:setEntryObjectLocUpdateFunction(	function( x, y, object )
																			object.banner:setLoc(x,y);
																		end
																	);

	self.ScrollList:setEntryObjectParentUpdateFunction(	function( target, object )
																				object.banner:setParent(target);
																			end
																		);

	self.ScrollList:setEntryCacheObjects(	nil,
														40,
														function()
															local tempTable = {};

															tempTable.banner		= self:addWidget(Anim, self.mainLayer, 0,0, 0,0, "Ranking", textureSheet);
															tempTable.picFrame	= self:addWidget(Image, self.mainLayer, -240, 0, PORTRAIT_FRAME_WIDTH, PORTRAIT_FRAME_HEIGHT, "resources/UI/PlayerPortraitRing.png");
															tempTable.portrait	= self:addWidget(Image, self.mainLayer, -240, 0, PORTRAIT_WIDTH, PORTRAIT_HEIGHT, "resources/UI/PlayerPortrait.png");
															tempTable.userName	= self:addWidget(TextBox, self.mainLayer, 10,0, 390, 96, '', self.defaultFontName, 20, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
															tempTable.clock		= self:addWidget(Anim, self.mainLayer, 0,0, 0,0, "BragGrayButton", textureSheet);
															tempTable.inputEmpty	= self:addWidget(ButtonImage, self.mainLayer, 240, 0, 72, 68, "resources/UI/CheckBoxEmpty.png");
															tempTable.inputCheck	= self:addWidget(ButtonImage, self.mainLayer, 240, 0, 72, 68, "resources/UI/CheckBoxFilled.png");

															tempTable.tangoConnectText		= self:addWidget(	TextBox,
																															self.mainLayer,
																															0,
																															0,
																															540,
																															140,
																															connectText..'         '..bonus,
																															self.defaultFontName, 16, nil,
																															MOAITextBox.LEFT_JUSTIFY,
																															MOAITextBox.CENTER_JUSTIFY
																															);
															tempTable.tangoGemIcon			= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "BuyButtonGem", textureSheet, nil, nil, true);
															tempTable.tangoConnectButton	= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "TangoButton", textureSheet, nil, self.goTangoConnect, nil, nil);

															tempTable.tangoGemIcon:setScale(0.6, 0.6)
															tempTable.inputEmpty:setInputXtra(20, 10);
															tempTable.inputCheck:setInputXtra(20, 10);
															tempTable.inputEmpty:disable();
															tempTable.inputCheck:disable();

															tempTable.inputEmpty:setInViewTest(	function()
																												return self.ScrollList:isEntryCacheObjInView(tempTable.inputEmpty.prop);
																											end
																											);
															tempTable.inputCheck:setInViewTest(	function()
																												return self.ScrollList:isEntryCacheObjInView(tempTable.inputCheck.prop);
																											end
																											);

															tempTable.picFrame:setParent(tempTable.banner.anim.root);
															tempTable.portrait:setParent(tempTable.banner.anim.root);
															tempTable.userName:setParent(tempTable.banner.anim.root);
															tempTable.clock:setParent(tempTable.banner.anim.root);
															tempTable.inputEmpty:setParent(tempTable.banner.anim.root);
															tempTable.inputCheck:setParent(tempTable.banner.anim.root);
															tempTable.tangoConnectText:setParent(tempTable.banner.anim.root);
															tempTable.tangoGemIcon:setParent(tempTable.banner.anim.root);
															tempTable.tangoConnectButton:setParent(tempTable.banner.anim.root);

															tempTable.picFrame:setLoc(-240, 0);
															tempTable.portrait:setLoc(-240, 0);
															tempTable.userName:setLoc(10, 0);
															tempTable.clock:setLoc(240, 0);
															tempTable.inputEmpty:setLoc(240, 0);
															tempTable.inputCheck:setLoc(240, 0);
															tempTable.tangoConnectText:setLoc(0, 0);
															tempTable.tangoGemIcon:setLoc(-20, -20);
															tempTable.tangoConnectButton:setLoc(240, 0);

															tempTable.banner:setPriority(PRIORTY_0);
															tempTable.picFrame:setPriority(PRIORTY_1+1);
															tempTable.portrait:setPriority(PRIORTY_1);
															tempTable.userName:setPriority(PRIORTY_1);
															tempTable.clock:setPriority(PRIORTY_1);
															tempTable.inputEmpty:setPriority(PRIORTY_1);
															tempTable.inputCheck:setPriority(PRIORTY_1);
															tempTable.tangoConnectText:setPriority(PRIORTY_1);
															tempTable.tangoGemIcon:setPriority(PRIORTY_1);
															tempTable.tangoConnectButton:setPriority(PRIORTY_1);

															return tempTable;
														end
													);

end

----------------------------------------------------------------

function StateBrag:onNewPofilePicture()

   self:updatePortait();
   self.ScrollList:forceUpdateCacheToMap();

end

----------------------------------------------------------------

function StateBrag:updatePortait()

   if self.EntryList then
      for i,entry in ipairs(self.EntryList) do

         local profile = ContactMgr:findContact(entry.tangoID);
         if profile then
            local before   = entry.portrait;
            entry.portrait	=  ContactMgr:getContactPicture(profile)
                           or "resources/UI/PlayerPortrait.png";

            if before ~= entry.portrait then

            end
         end
      end
   end

end

----------------------------------------------------------------

function StateBrag:createEntryList()

	self.EntryList	= {};

   local list		= {};
   
	for i=1,ContactMgr:getFriendsCount() do
   
      local contact = ContactMgr:getFriend(i);

      if			contact
			and	( contact:isAppInstalled()
			or		string.lower(App.data:getValue("App.settings.brag-select", "road-riot-installed")) == "everyone" ) then

         table.insert(list, contact);
      end
   end

	local ladderSize = #list;
	
	self.ListSize = ladderSize;

	for i=1,ladderSize do
		
		local tempTable = {};

		if i <= #list then
			tempTable.tangoID		= list[i]:getAccountId();

         local profile			= ContactMgr:findContact(tempTable.tangoID);

         tempTable.portrait	=  ContactMgr:getContactPicture(profile)
                              or "resources/UI/PlayerPortrait.png";
			tempTable.name			= '<c:d4e7f9>'..createUserNameDisplay(list[i]:getFirstName(), list[i]:getLastName(), true, "Friend", 19)..'<c>';
		else
			tempTable.tangoID		= nil;
			tempTable.portrait	= "resources/UI/PlayerPortrait.png";
			tempTable.name			= i..'';
		end
		
		if tempTable.tangoID and isNewHiScore(tempTable.tangoID) then
			tempTable.used = false;
		else
			tempTable.used = true;
		end
		
		self.EntryList[i] = tempTable;
	end

end

----------------------------------------------------------------

function StateBrag:_updateSendSelectAllButton()

	if validSelections then
		self:enableSend(true);
	else
		self:enableSend(false);
	end

	local validSelections = false;

	for k,v in ipairs(self.EntryList) do
		if v.used == false then
			self.SelectAllButton:setAnimFrame();
			self.SelectAllMode = "off";
		elseif v.used == "Selected" then
			validSelections = true;
		end
	end

	if validSelections and self.SelectAllMode ~= "off" then
		self.SelectAllButton:setAnimLastFrame();
		self.SelectAllMode = 'on';
	end

	if validSelections then
		self:enableSend(true);
	else
		self:enableSend(false);
	end

   if not self:isValidFriendAvailable() then
      self:enableSelectAll(false);
      self:enableSend(false);
   end

	self.ScrollList:forceUpdateCacheToMap();

end

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

function StateBrag:addSelection( data, object )

	if data.used == 'Selected' then
		data.used				= false;
		self.SelectAllMode	= 'off';

		object.inputEmpty:setVisible(true);
		object.inputCheck:setVisible(false);
		object.inputEmpty:enable();
		object.inputCheck:disable();
		self.SelectAllButton:setAnimFrame();
	else
		data.used				= 'Selected';

		object.inputEmpty:setVisible(false);
		object.inputCheck:setVisible(true);
		object.inputEmpty:disable();
		object.inputCheck:enable();
	end

	self:updateRewardCount();
	self:_updateSendSelectAllButton();

end

----------------------------------------------------------------

function StateBrag:addAllSelection( )
	
	local count = 0;

	if not self.EntryList then
		return false;
	end

	if self.SelectAllMode == 'off' then
		for i=1,#self.EntryList do
			if self.EntryList[i].used == false then
				self.EntryList[i].used = 'Selected';
				count = count + 1;
			end
		end

		self.SelectAllButton:setAnimLastFrame();
		self.SelectAllMode = 'on';
	else
		for i=1,#self.EntryList do
			if self.EntryList[i].used == 'Selected' then
				self.EntryList[i].used = false;
			end
		end

		self.SelectAllButton:setAnimFrame();
		self.SelectAllMode = 'off'
	end

	self:updateRewardCount();
	self:_updateSendSelectAllButton();

end

----------------------------------------------------------------

function StateBrag:sendBrag( tangoID, index, button )

   local  inviteTemplates  = MessageMgr:getInviteTemplateList();
   local  bragTemplates    = MessageMgr:getBragTemplateList();

   if (not OnlineMgr:isOnline()) or (not inviteTemplates) or (not bragTemplates) then

      MOAIDialogIOS.showDialog(	LocalizationMgr:getString( "NATIVE_CONNECT_ERROR" )
                              ,  LocalizationMgr:getString( "DEFAULT_ERROR_NETWORK_MESSAGE" )
                              ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                              ,  nil
                              ,  nil
                              ,  false
                              ,  function()
                                 end
                              );

   else
      local tracker        = App.data:getValue('BragTracker') or {};
      local keyCount       = 0;
      local bragCount      = 0;
      self.SelectAllMode   = 'off'

      local bragList         = {};
      local inviteList       = {};
      local inviteIndex      = math.random( 1, #inviteTemplates );

      for i=1,#self.EntryList do
         if self.EntryList[i].used == 'Selected' then
            tracker['HiScore-'..self.EntryList[i].tangoID] = true;
            self.EntryList[i].used = true;

            local inTimeLimit		= isTimeInLim('BragRewardTimer2', '', App.data:getValue("App.settings.brag-cooldown") or BRAG_REWARD_TIME_LIMIT, TimeMgr:getTimeGMT()) or false;
            local keyTracker		= App.data:getValue('BragRewardTracker') or 0;
            local maxReward		= App.data:getKeyMaxReward();
                                       
            if not inTimeLimit then
               App.data:setValue('BragRewardTimer2-', TimeMgr:getTimeGMT());
               App.data:setValue('BragRewardTracker', 0);
               keyTracker = 0;
            end
            
            if (keyTracker < maxReward) then
               keyTracker = keyTracker + 1;
               keyCount   = keyCount + 1;
               App.data:setValue('BragRewardTracker', keyTracker);
            end

            bragCount = bragCount + 1;

            local contact = ContactMgr:findContact(self.EntryList[i].tangoID);

            if contact then

               if contact:isAppInstalled() then
						table.insert( bragList, contact );
               else
						table.insert( inviteList, contact );
               end
            end
         end
      end

      local sent = false;

      if bragList and #bragList > 0 then
         local idx = App.data:getValue('BragMsgIndex2');

         if self.Message_List and idx >=1 or idx <= #self.Message_List then
         	sent = true;
            MessageMgr:sendBrag( self.Message_List[idx], bragList );
         else
            Debug.assert(self.Message_List and idx >=1 or idx <= #self.Message_List, "StateBrag:sendBrag -> idx < 0 or idx > #bragTemplates");
         end
      end

      if inviteList and #inviteList > 0 then
			sent = true;
         MessageMgr:sendInvite(  inviteTemplates[inviteIndex], inviteList );
      end

		if sent then
			EventMgr:postEvent( {key=keyCount, bragCount=bragCount}, "brag-friend-success" );
		end

      if keyCount and keyCount>0 then
         App.data:addKey(keyCount, "Brag");
      end

		self:updateRewardCount();
		self:_updateSendSelectAllButton();

      if self:isValidFriendAvailable() then
         self.SelectAllButton:setAnimFrame();

         self:enableSelectAll(true);
         self:enableSend(true);
      else
         self:enableSelectAll(false);
         self:enableSend(false);
      end
   end

end

----------------------------------------------------------------

function StateBrag:goBragMsg()
	
	StateMgr:push("state/state-brag-msg-popup.lua");

end

----------------------------------------------------------------

function StateBrag:goBack()

	GamePaused	= false;
   EventMgr:postEvent( {state="state/state-brag.lua"}, "game-unpaused" );
	StateMgr:pop();

end

----------------------------------------------------------------

return StateBrag;

----------------------------------------------------------------
