----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-scroll-list');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local PRIORITY_0         = 10;
local PRIORITY_1         = 20;
local PRIORITY_2         = 30;
local PRIORITY_3         = 40;
local PRIORITY_4         = 50;
local PRIORITY_5         = 60;
local PRIORITY_6         = 70;
local PRIORITY_7         = 80;
local PRIORITY_8         = 90;
local PRIORITY_9         = 100;
local PRIORITY_10        = 110;

local SQUARE_TEXTURE     = "resources/square.png";
local DRONE_LEFT         = "left";  -- mirror in drone manager
local DRONE_RIGHT        = "right"; -- mirror in drone manager

----------------------------------------------------------------

local STATE_VEHICLE_DIFF = "diff-vehicle";
local STATE_DRONE_DIFF   = "diff-drone";
local STATE_GEM_DIFF     = "diff-gem";
local STATE_COIN_DIFF    = "diff-coin";
local STATE_LEVEL_DIFF   = "diff-level";
local STATE_GAMES_DIFF   = "diff-games";

local HACK_DEBUG_LAYOUT  = false;

if not BUILD_DEBUG then
   HACK_DEBUG_LAYOUT = false;
end

----------------------------------------------------------------

local Base = State;
local StateCloudResolve = Base:new();

----------------------------------------------------------------

function StateCloudResolve:onLoad()

   State.onLoad( self );

   self.defaultFontName = "resources/font/TangoLatin.otf";
   self.IS_POPUP  = true;

   -- Background
   self.Background = self:addWidget(Image, self.mainLayer, 0,0, 1200, 1400, SQUARE_TEXTURE, {0,0,0, 0.75});
   self.Background:setPriority(0);

   -- Frame
   self.Frame = self:addWidget(Image, self.mainLayer, 0, 0, 540, 850, "resources/UI/Sync/sync_frame.png");
   self.Frame:setPriority(1);

   -- data text
   self:_createCompText();

   -- Cloud/Local Button
   self.CloudButton	= self:addWidget(ButtonImage, self.mainLayer, -540/4, -850/2+80, 170, 100, "resources/UI/Sync/sync_button.png", nil, self.takeCloud, nil, nil);
   self.LocalButton	= self:addWidget(ButtonImage, self.mainLayer, 540/4, -850/2+80, 170, 100, "resources/UI/Sync/sync_button.png", nil, self.takeLocal, nil, nil);

   self.CloudButton:setPriority(PRIORITY_2);
   self.LocalButton:setPriority(PRIORITY_2);

end

----------------------------------------------------------------

function StateCloudResolve:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StateCloudResolve:onFocus( data )

   if HACK_DEBUG_LAYOUT then
      State.onFocus(self);
      self:_testPopup();

      return;
   end

   Debug.assert(data, "StateCloudResolve:onFocus -> no data");
   Debug.assert(data.device, "StateCloudResolve:onFocus -> no local data");
   Debug.assert(data.cloud, "StateCloudResolve:onFocus -> no remote data");

   State.onFocus(self);

   local dataMatch = self:_updateCompData(data.device, data.cloud);

   if dataMatch then
      -- If there's nothing to display, there isn't a very meaningful choice for the player.
      TransactionMgr:chooseCloudData(true);
      StateMgr:pop();
   end

end

----------------------------------------------------------------

function StateCloudResolve:onUpdate( dt )

   Base.onUpdate(self, dt);

end

----------------------------------------------------------------

function StateCloudResolve:onNeedDataResolution()
   -- We shouldn't do anything here
end

------------------------------------------------------------------------------------
-- Comparison
------------------------------------------------------------------------------------


function StateCloudResolve:_createCompText()

   self.GemsCloud    = self:addWidget(TextBox, self.mainLayer, -179, 102, 500, 100, "", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.GemsLocal    = self:addWidget(TextBox, self.mainLayer, 70, 102, 500, 100, "", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.CoinsCloud   = self:addWidget(TextBox, self.mainLayer, -179, 33, 500, 100, "", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.CoinsLocal   = self:addWidget(TextBox, self.mainLayer, 70, 33, 500, 100, "", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.CarsCloud    = self:addWidget(TextBox, self.mainLayer, -179, -35, 500, 100, "", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.CarsLocal    = self:addWidget(TextBox, self.mainLayer, 70, -35, 500, 100, "", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.DronesCloud  = self:addWidget(TextBox, self.mainLayer, -179, -102, 500, 100, "", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.DronesLocal  = self:addWidget(TextBox, self.mainLayer, 70, -102, 500, 100, "", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.LevelCloud   = self:addWidget(TextBox, self.mainLayer, -179, -170, 500, 100, "", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.LevelLocal   = self:addWidget(TextBox, self.mainLayer, 70, -170, 500, 100, "", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.GamesCloud   = self:addWidget(TextBox, self.mainLayer, -179, -235, 500, 100, "", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.GamesLocal   = self:addWidget(TextBox, self.mainLayer, 70, -235, 500, 100, "", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);

   self.GemsCloud:setPriority(PRIORITY_0);
   self.GemsLocal:setPriority(PRIORITY_0);
   self.CoinsCloud:setPriority(PRIORITY_0);
   self.CoinsLocal:setPriority(PRIORITY_0);
   self.CarsCloud:setPriority(PRIORITY_0);
   self.CarsLocal:setPriority(PRIORITY_0);
   self.DronesCloud:setPriority(PRIORITY_0);
   self.DronesLocal:setPriority(PRIORITY_0);
   self.LevelCloud:setPriority(PRIORITY_0);
   self.LevelLocal:setPriority(PRIORITY_0);
   self.GamesCloud:setPriority(PRIORITY_0);
   self.GamesLocal:setPriority(PRIORITY_0);

end

----------------------------------------------------------------

function StateCloudResolve:_updateCompData( device, cloud )

   local datamatch   = true;
   local green       = "<c:009c21>";
   local blue        = "<c:1e3645>";

   --gems
   local temp, cVal, dVal = self:_compareDataField(device, cloud, "gem", 0);
   if temp then
      datamatch = false

      if cVal > dVal then
         self.GemsCloud:setText(green..getFormattedNumber(cVal));
         self.GemsLocal:setText(blue..getFormattedNumber(dVal));
      else
         self.GemsCloud:setText(blue..getFormattedNumber(cVal));
         self.GemsLocal:setText(green..getFormattedNumber(dVal));
      end
   else
      self.GemsCloud:setText(blue..getFormattedNumber(cVal));
      self.GemsLocal:setText(blue..getFormattedNumber(dVal));
   end

   self:_scaleText(self.GemsCloud, cVal);
   self:_scaleText(self.GemsLocal, dVal);


   --coins
   temp, cVal, dVal = self:_compareDataField(device, cloud, "gold", 0);
   if temp then
      datamatch = false

      if cVal > dVal then
         self.CoinsCloud:setText(green..getFormattedNumber(cVal));
         self.CoinsLocal:setText(blue..getFormattedNumber(dVal));
      else
         self.CoinsCloud:setText(blue..getFormattedNumber(cVal));
         self.CoinsLocal:setText(green..getFormattedNumber(dVal));
      end
   else
      self.CoinsCloud:setText(blue..getFormattedNumber(cVal));
      self.CoinsLocal:setText(blue..getFormattedNumber(dVal));
   end

   self:_scaleText(self.CoinsCloud, cVal);
   self:_scaleText(self.CoinsLocal, dVal);

   --cars
   temp, cVal, dVal = self:_compareVehicles(device, cloud);
   if temp then
      datamatch = false

      if cVal > dVal then
         self.CarsCloud:setText(green..tostring(cVal));
         self.CarsLocal:setText(blue..tostring(dVal));
      else
         self.CarsCloud:setText(blue..tostring(cVal));
         self.CarsLocal:setText(green..tostring(dVal));
      end
   else
      self.CarsCloud:setText(blue..tostring(cVal));
      self.CarsLocal:setText(blue..tostring(dVal));
   end

   --drones
   temp, cVal, dVal = self:_compareDrones(device, cloud);
   if temp then
      datamatch = false

      if cVal > dVal then
         self.DronesCloud:setText(green..tostring(cVal));
         self.DronesLocal:setText(blue..tostring(dVal));
      else
         self.DronesCloud:setText(blue..tostring(cVal));
         self.DronesLocal:setText(green..tostring(dVal));
      end
   else
      self.DronesCloud:setText(blue..tostring(cVal));
      self.DronesLocal:setText(blue..tostring(dVal));
   end

   --levels
   temp, cVal, dVal = self:_compareDataField(device, cloud, "currentLevel", 1);
   if temp then
      datamatch = false

      if cVal > dVal then
         self.LevelCloud:setText(green..tostring(cVal));
         self.LevelLocal:setText(blue..tostring(dVal));
      else
         self.LevelCloud:setText(blue..tostring(cVal));
         self.LevelLocal:setText(green..tostring(dVal));
      end
   else
      self.LevelCloud:setText(blue..tostring(cVal));
      self.LevelLocal:setText(blue..tostring(dVal));
   end

   self:_scaleText(self.LevelCloud, cVal);
   self:_scaleText(self.LevelLocal, dVal);

   --games played
   temp, cVal, dVal = self:_compareDataField(device, cloud, "gamePlayed", 0);
   if temp then
      datamatch = false

      if cVal > dVal then
         self.GamesCloud:setText(green..tostring(cVal));
         self.GamesLocal:setText(blue..tostring(dVal));
      else
         self.GamesCloud:setText(blue..tostring(cVal));
         self.GamesLocal:setText(green..tostring(dVal));
      end
   else
      self.GamesCloud:setText(blue..tostring(cVal));
      self.GamesLocal:setText(blue..tostring(dVal));
   end

   return datamatch;

end


------------------------------------------------------------------------------------
-- Util
------------------------------------------------------------------------------------


function StateCloudResolve:isUsingCameraHack()
   return false;
end

------------------------------------------------------------------------------------

function StateCloudResolve:_compareDataField( device, cloud, fieldName, defaultValue )

   local data = {};
   data.device = device[fieldName] or defaultValue;
   data.cloud  = cloud[fieldName] or defaultValue;

   if data.device ~= data.cloud then
      return true, data.cloud, data.device;
   else
      return false, data.cloud, data.device;
   end

end

------------------------------------------------------------------------------------

function StateCloudResolve:_compareVehicles( device, cloud )

   local data = {};
   local deviceCount = 0;
   local cloudCount = 0;

   for i = 1, TemplateMgr:getPlayerVehicleCount() do

      local id = App.data:createMainWeaponId(i);

      if (device[id] or 0) > 0 then
         deviceCount = deviceCount + 1;
      end

      if (cloud[id] or 0) > 0 then
         cloudCount = cloudCount + 1;
      end
   end

   data.device = math.max(deviceCount, 1);
   data.cloud  = math.max(cloudCount, 1);

   if data.device ~= data.cloud then
      return true, data.cloud, data.device;
   else
      return false, data.cloud, data.device;
   end

end

------------------------------------------------------------------------------------

function StateCloudResolve:_compareDrones( device, cloud )

   if not DroneMgr:isEnabled() then
      return;
   end

   local data = {};
   local deviceCount = 0;
   local cloudCount = 0;

   for i = 1, App.data:getDroneCount() do

      local idLeft  = App.data:createDroneId(i, DRONE_LEFT);
      local idRight = App.data:createDroneId(i, DRONE_RIGHT);

      if (device[idLeft] or 0) > 0 then
         deviceCount = deviceCount + 1;
      end

      if (device[idRight] or 0) > 0 then
         deviceCount = deviceCount + 1;
      end

      if (cloud[idLeft] or 0) > 0 then
         cloudCount = cloudCount + 1;
      end

      if (cloud[idRight] or 0) > 0 then
         cloudCount = cloudCount + 1;
      end
   end

   data.device = deviceCount;
   data.cloud  = cloudCount;

   if data.device ~= data.cloud then
      return true, data.cloud, data.device;
   else
      return false, data.cloud, data.device;
   end

end

------------------------------------------------------------------------------------

function StateCloudResolve:_scaleText( textbox, val )

   if val > 999999999 then  --10 digit
      textbox:setScale(0.45, 0.45);
   elseif val > 99999999 then  --9 digit
      textbox:setScale(0.48, 0.48);
   elseif val > 9999999 then  --8 digit
      textbox:setScale(0.58, 0.58);
   elseif val > 999999 then  --7 digit
      textbox:setScale(0.68, 0.68);
   elseif val > 99999 then  --6 digit
      textbox:setScale(0.8, 0.8);
   elseif val > 9999 then  --5 digit
      textbox:setScale(0.9, 0.9);
   else
      textbox:setScale(1, 1);
   end

end

------------------------------------------------------------------------------------

function StateCloudResolve:_testPopup()

   local list =
   {
      {state=STATE_GEM_DIFF,        left="12",   right="12"};
      {state=STATE_COIN_DIFF,       left="12",   right="12"};
      {state=STATE_VEHICLE_DIFF,    left="12",   right="12"};
      {state=STATE_DRONE_DIFF,      left="12",   right="12"};
      {state=STATE_LEVEL_DIFF,      left="12",   right="12"};
      {state=STATE_GAMES_DIFF,      left="12",   right="12"};
   };

end


------------------------------------------------------------------------------------
-- Resolve Data
------------------------------------------------------------------------------------


function StateCloudResolve:takeCloud()

   self:goConfirm( {
         msg = "This will overwrite your Local save with the Cloud save!",
         callback = function()

            TransactionMgr:chooseCloudData(false);
            EventMgr:postEvent({data = "cloud"}, "player-choose-data");

         end,
   } );

end

------------------------------------------------------------------------------------

function StateCloudResolve:takeLocal()

   self:goConfirm( {
         msg = "This will overwrite your Cloud save with the Local save!",
         callback = function()

            TransactionMgr:chooseLocalData(false);
            EventMgr:postEvent({data = "local"}, "player-choose-data");

         end,
   } );

end


------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------


function StateCloudResolve:goConfirm( data )

   StateMgr:swap("state/state-data-resolve-confirm.lua", data);

end


------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------


return StateCloudResolve;
