----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local Base                       = State;
local StateConsumableOverlayNew  = Base:new();


local BUTTON_START_Y             = App.viewportHeight/2-300;
local BUTTON_WIDTH               = 128;
local BUTTON_HEIGHT              = 128;
local BUTTON_SPACING_X           = 160;
local BUTTON_SPACING_Y           = 160;

local EXIT_TIME                  = 1.5;

local PRIORITY_0				= 10;
local PRIORITY_1				= 20;
local PRIORITY_2				= 30;
local PRIORITY_3				= 40;
local PRIORITY_4				= 50;
local PRIORITY_5				= 60;
local PRIORITY_6				= 70;
local PRIORITY_7				= 80;
local PRIORITY_8				= 90;
local PRIORITY_9				= 100;
local PRIORITY_10				= 110;

----------------------------------------------------------------

local CONSUMABLE_ICON   =
{
   "resources/UI/Consumable/Icons/Consumable_HeadStart.png",
   "resources/UI/Consumable/Icons/Consumable_PowerTime.png",
   "resources/UI/Consumable/Icons/Consumable_DoubleScore.png",
   "resources/UI/Consumable/Icons/Consumable_1_5xGold.png",
   "resources/UI/Consumable/Icons/Consumable_3xGold.png",
   "resources/UI/Consumable/Icons/Consumable_4xGold.png",
   "resources/UI/Consumable/Icons/Consumable_5xGold.png",
};

local TIMER_CLOCK =
{
   "resources/UI/Consumable/Countdown/Consumable_05PctTimer.png",
   "resources/UI/Consumable/Countdown/Consumable_25PctTimer.png",
   "resources/UI/Consumable/Countdown/Consumable_50PctTimer.png",
   "resources/UI/Consumable/Countdown/Consumable_75PctTimer.png",
   "resources/UI/Consumable/Countdown/Consumable_95PctTimer.png",
};

local BUTTON_POSITION_Y =
{
   BUTTON_START_Y,
   BUTTON_START_Y-BUTTON_SPACING_Y,
   BUTTON_START_Y-2*BUTTON_SPACING_Y,
};

local TAP_TEXT_POS      = {x=0, y=-200};

----------------------------------------------------------------
----------------------------------------------------------------


function  StateConsumableOverlayNew:onLoad(  )

   Base.onLoad(self);

   self.defaultFontName = "resources/font/TangoLatin.otf";
   self.IS_POPUP        = true;

   self:_setupBasicResourceBar( true );

   self.Title = self:addWidget(Image, self.mainLayer, 0, BUTTON_START_Y+BUTTON_SPACING_Y, 352*0.9, 75*0.9, "resources/UI/Consumable/Consumable_OverlayTitle.png");
   self.Title:setPriority(PRIORITY_10);

   local buttonDeltaY      = math.min((App.viewHeight - 1136 )/4,0);
   local tapToStartDeltaY  = -buttonDeltaY;

   self.TapText      = self:addWidget(TextBox, self.mainLayer, TAP_TEXT_POS.x, TAP_TEXT_POS.y+tapToStartDeltaY, 400, 400, "TAP TO START", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.TapTextShdw  = self:addWidget(TextBox, self.mainLayer, TAP_TEXT_POS.x+2, TAP_TEXT_POS.y+tapToStartDeltaY-2, 400, 400, "TAP TO START", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});

   self.TapText:setPriority(PRIORITY_10);
   self.TapTextShdw:setPriority(PRIORITY_10-1);
   self.TapText:setVisible(false);
   self.TapTextShdw:setVisible(false);

   --guide screen button
   self.ScreenButton = self:addWidget(ButtonImage, self.mainLayer, 0,0, 1400, 1200, "resources/square.png", nil, nil, nil, nil);
   self.ScreenButton:setPriority(PRIORITY_0);
   self.ScreenButton:setColor(1,1,1,0);
   self.ScreenButton:disable();

end

----------------------------------------------------------------

function  StateConsumableOverlayNew:onActivate(  )

   Base.onActivate(self);

   if self.resourceBar then
      self.resourceBar:createConsumableButton();
   end

end

----------------------------------------------------------------

function  StateConsumableOverlayNew:onFocus( data )

   Base.onFocus(self);

   self._exitTimer   = 0;

   self.ScreenButton:disable();
   self:_updateConsumableButton();
   self:_checkGuide();

end

----------------------------------------------------------------

function  StateConsumableOverlayNew:onDeactivate(  )

   Base.onDeactivate(self);

end

----------------------------------------------------------------

function StateConsumableOverlayNew:onLoseFocus( )

   Base.onLoseFocus(self);

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateConsumableOverlayNew:onUpdate( dt )

   if App.game then
      App.game:onUpdate( dt );
   end

	self:_suppressDroneCharge();
   self:_updateExitTimer(dt);

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateConsumableOverlayNew:_checkGuide()

   self._guideActive       = false;
   self._guideScreenButton = self.ScreenButton;
   self._guideStoreButton  = self.resourceBar.ConsumableButton;
   self._guideSparkle      = self.resourceBar.ConsumableSparkle;
   self._guideBlueButton   = self._consumableButtons[1].FrameButtonBlue;
   self._guideIcon         = self._consumableButtons[1].ConsumableIcon;
   self._guideInvFrame     = self._consumableButtons[1].InventoryFrame;
   self._guideInvText      = self._consumableButtons[1].InventoryText;

   if self._guideSparkle then
      self._guideSparkle:stop();
      self._guideSparkle:setVisible(false);
   end

	if       not App.data:getValue("guide-consumable-store", false)
      and   App.data:getConsumableUnlockLevel() == App.data:getCurrentPlayerLevel() then

      self._guideActive = true;

      GuideMgr:addSet("consumable_store");
		App.data:setValue("guide-consumable-store", true);
	end

end

----------------------------------------------------------------

function StateConsumableOverlayNew:_updateConsumableButton()

   self:_createConsumableData();
   self:_createConsumableButtons();

end

----------------------------------------------------------------

function StateConsumableOverlayNew:_updateExitTimer( dt )

   if not self._exitTimer then
      self._exitTimer = 0;
   else
      self._exitTimer = math.min(self._exitTimer+dt, EXIT_TIME);
   end

   if self:_canExitOverlay() then
      self.TapText:setVisible(true);
      self.TapTextShdw:setVisible(true);
   else
      self.TapText:setVisible(false);
      self.TapTextShdw:setVisible(false);
   end

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateConsumableOverlayNew:_createConsumableData()

   self._consumableData = {};
   self._pulseList      = {};
   self._pulseTimer     = 0;

   self:_createSingleUseData();
   self:_createCoinKData();

end

----------------------------------------------------------------

function StateConsumableOverlayNew:_createSingleUseData()

   for i=1, ConsumableMgr:getTotalTypes() do

      if ConsumableMgr:isSingleUse(i) then
         local t     = {};

         t.index     = i;
         t.singleUse = true;
         t.freebie   = false;
         t.active    = false;
         t.timeLeftK = -1;    --timeleft/max duration
         t.inventory = ConsumableMgr:getInventoryCount(i);

         local freebie;

         if i == CONSUMABLE_HEADSTART_INDEX then
            freebie = App.data:getFreebieCount("head-start");
         elseif i == CONSUMABLE_POWERTIME_INDEX then
            freebie = App.data:getFreebieCount("power-time");
         elseif i == CONSUMABLE_DOUBLESCORE_INDEX then
            freebie = App.data:getFreebieCount("double-score");
         end

         if freebie and freebie > 0 then
            t.inventory = t.inventory + freebie;
            t.freebie   = true;
         end

         table.insert(self._consumableData, t);
      end
   end

end

----------------------------------------------------------------

function StateConsumableOverlayNew:_createCoinKData()

   local coinIndex,timeLeft = ConsumableMgr:getHighestActiveCoinK();

   if coinIndex then

      local t     = {};

      t.index     = coinIndex;
      t.singleUse = false;
      t.freebie   = false;
      t.active    = true;
      t.timeLeftK = math.max(timeLeft/ConsumableMgr:getDuration(coinIndex), 0.05);  --timeleft/max duration
      t.inventory = 1;

      table.insert(self._consumableData, t);
   end

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateConsumableOverlayNew:_createConsumableButtons()

   if not self._consumableData or #self._consumableData < 1 then
      self:goBack();
   end

   self:_deleteConsumableButtons();

   for k,v in ipairs(self._consumableData) do
      self:_addConsumableButton(k);
   end

end

----------------------------------------------------------------

function StateConsumableOverlayNew:_addConsumableButton( index )

   if not self._consumableButtons then
      self._consumableButtons = {};
   end

   local x,y   = self:_getButtonPos(index);
   local t     = {};

   t.FrameButtonBlue    = self:addWidget(ButtonImage, self.mainLayer, x, y,  BUTTON_WIDTH, BUTTON_HEIGHT, "resources/UI/Consumable/Consumable_OverlayFrameBlue.png");
   t.FrameButtonGreen   = self:addWidget(ButtonImage, self.mainLayer, x, y,  BUTTON_WIDTH, BUTTON_HEIGHT, "resources/UI/Consumable/Consumable_OverlayFrameGreen.png");
   t.ConsumableIcon     = self:addWidget(Image, self.mainLayer, 0, 0, 0.65*BUTTON_WIDTH, 0.65*BUTTON_HEIGHT, CONSUMABLE_ICON[self._consumableData[index].index]);
   t.InventoryFrame     = self:addWidget(Image, self.mainLayer, -BUTTON_WIDTH*0.5, BUTTON_HEIGHT*0.5, 48, 48, "resources/UI/Consumable/Consumable_Counter.png");
   t.InventoryText      = self:addWidget(TextBox, self.mainLayer, 0, 0, 100, 100, "0", self.defaultFontName, 12, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);

   if self._consumableData[index].timeLeftK < 0.05 then
      t.TimeClock = self:addWidget(Image, self.mainLayer, -BUTTON_WIDTH*0.5, BUTTON_HEIGHT*0.5, 48, 48, TIMER_CLOCK[1]);
   elseif self._consumableData[index].timeLeftK < 0.25 then
      t.TimeClock = self:addWidget(Image, self.mainLayer, -BUTTON_WIDTH*0.5, BUTTON_HEIGHT*0.5, 48, 48, TIMER_CLOCK[2]);
   elseif self._consumableData[index].timeLeftK < 0.5 then
      t.TimeClock = self:addWidget(Image, self.mainLayer, -BUTTON_WIDTH*0.5, BUTTON_HEIGHT*0.5, 48, 48, TIMER_CLOCK[3]);
   elseif self._consumableData[index].timeLeftK < 0.75 then
      t.TimeClock = self:addWidget(Image, self.mainLayer, -BUTTON_WIDTH*0.5, BUTTON_HEIGHT*0.5, 48, 48, TIMER_CLOCK[4]);
   else
      t.TimeClock = self:addWidget(Image, self.mainLayer, -BUTTON_WIDTH*0.5, BUTTON_HEIGHT*0.5, 48, 48, TIMER_CLOCK[5]);
   end

   t.FrameButtonBlue:setPriority(PRIORITY_1);
   t.FrameButtonGreen:setPriority(PRIORITY_1);
   t.ConsumableIcon:setPriority(PRIORITY_2);
   t.InventoryFrame:setPriority(PRIORITY_3);
   t.InventoryText:setPriority(PRIORITY_4);
   t.TimeClock:setPriority(PRIORITY_3);

   t.ConsumableIcon:setAttrLink(    MOAIProp2D.INHERIT_TRANSFORM,		t.FrameButtonBlue.prop, MOAIProp2D.TRANSFORM_TRAIT	);
   t.InventoryFrame:setAttrLink(    MOAIProp2D.INHERIT_TRANSFORM,		t.FrameButtonBlue.prop, MOAIProp2D.TRANSFORM_TRAIT	);
   t.InventoryText:setAttrLink(     MOAIProp2D.INHERIT_TRANSFORM,		t.InventoryFrame.prop, MOAIProp2D.TRANSFORM_TRAIT	);
   t.TimeClock:setAttrLink(         MOAIProp2D.INHERIT_TRANSFORM,		t.FrameButtonBlue.prop, MOAIProp2D.TRANSFORM_TRAIT	);

   if self._consumableData[index].singleUse then
      t.FrameButtonGreen:setVisible(false);
      t.FrameButtonGreen:disable();
      t.InventoryText:setText(tostring(self._consumableData[index].inventory));
      t.TimeClock:setVisible(false);

      if self._consumableData[index].inventory > 99 then
         t.InventoryText:setScale(0.75, 0.75);
      elseif self._consumableData[index].inventory > 9 then
         t.InventoryText:setScale(0.9, 0.9);
      else
         t.InventoryText:setScale(1, 1);
      end

      if self._consumableData[index].inventory > 0 then
         t.FrameButtonBlue.onReleasedFunc =  function()
                                                self:_setActive(self._consumableData[index], t);
                                             end
         t.FrameButtonGreen.onReleasedFunc =  function()
                                                self:_setInactive(self._consumableData[index], t);
                                             end
      else
         t.FrameButtonBlue.onReleasedFunc =  function()
                                                StateMgr:push("state/state-consumable.lua");
                                             end
      end
   else
      t.FrameButtonBlue:setVisible(false);
      t.FrameButtonBlue:disable();
      t.FrameButtonGreen:disable();
      t.InventoryFrame:setVisible(false);
      t.InventoryText:setVisible(false);

      self._pulseList[t.FrameButtonGreen] = t.FrameButtonGreen;
   end

   table.insert(self._consumableButtons, t);

end

----------------------------------------------------------------

function StateConsumableOverlayNew:_deleteConsumableButtons()

   if self._consumableButtons then

      for _,v in ipairs(self._consumableButtons) do
         self:deleteWidget(v.FrameButtonBlue);
         self:deleteWidget(v.FrameButtonGreen);
         self:deleteWidget(v.ConsumableIcon);
         self:deleteWidget(v.InventoryFrame);
         self:deleteWidget(v.InventoryText);
         self:deleteWidget(v.TimeClock);
      end
   end

   self._consumableButtons = {};

end

----------------------------------------------------------------

function StateConsumableOverlayNew:_getButtonPos( index )

   if not self._consumableData or not self._consumableData[index] then
      return 0,0;
   end

   local count = #self._consumableData;
   local x,y;
   local yId;

   if index <= 3 then
      yId   = 1;
      y     = BUTTON_POSITION_Y[yId];
   elseif index <= 6 then
      yId   = 2;
      y     = BUTTON_POSITION_Y[yId];
   else
      yId   = 3;
      y     = BUTTON_POSITION_Y[yId];
   end

   local modulo = index%3;

   if modulo == 1 then
      x  = -BUTTON_SPACING_X;
   elseif modulo == 2 then
      x  = 0;
   else
      x  = BUTTON_SPACING_X;
   end

   local special = false;

   if    (yId == 1 and count > 0 and count < 3)
      or (yId == 2 and count > 3 and count < 6)
      or (yId == 3 and count > 6 and count < 9) then

      special = true;
   end

   if special then
      if count%3 == 1 then
         x = 0
      elseif modulo == 1 then
         x = -BUTTON_SPACING_X/2;
      else
         x = BUTTON_SPACING_X/2;
      end
   end

   return x,y;

end

----------------------------------------------------------------

function StateConsumableOverlayNew:_setActive( data, button )

   if not data or not button then
      return;
   end

   data.active = true;

   button.FrameButtonBlue:setVisible(false);
   button.FrameButtonBlue:disable();
   button.FrameButtonGreen:setVisible(true);
   button.FrameButtonGreen:enable();

   button.ConsumableIcon:setAttrLink(  MOAIProp2D.INHERIT_TRANSFORM,    button.FrameButtonGreen.prop,    MOAIProp2D.TRANSFORM_TRAIT	);
   button.InventoryFrame:setAttrLink(  MOAIProp2D.INHERIT_TRANSFORM,		button.FrameButtonGreen.prop,    MOAIProp2D.TRANSFORM_TRAIT	);

   button.FrameButtonGreen:setScale(1,1);
   button.ConsumableIcon:setScale(1,1);
   button.InventoryFrame:setScale(1,1);

   self._pulseList[button.FrameButtonGreen]  = button.FrameButtonGreen;

end

----------------------------------------------------------------

function StateConsumableOverlayNew:_setInactive( data, button )

   if not data or not button then
      return;
   end

   data.active = false;

   button.FrameButtonBlue:setVisible(true);
   button.FrameButtonBlue:enable();
   button.FrameButtonGreen:setVisible(false);
   button.FrameButtonGreen:disable();

   button.ConsumableIcon:setAttrLink(  MOAIProp2D.INHERIT_TRANSFORM,    button.FrameButtonBlue.prop,    MOAIProp2D.TRANSFORM_TRAIT	);
   button.InventoryFrame:setAttrLink(  MOAIProp2D.INHERIT_TRANSFORM,		button.FrameButtonBlue.prop,    MOAIProp2D.TRANSFORM_TRAIT	);

   button.FrameButtonGreen:setScale(1,1);
   button.ConsumableIcon:setScale(1,1);
   button.InventoryFrame:setScale(1,1);

   self._pulseList[button.FrameButtonGreen]  = nil;

end

----------------------------------------------------------------
----------------------------------------------------------------


function StateConsumableOverlayNew:_activateConsumables()

   if not self._consumableData or #self._consumableData < 1 then
      return;
   end

   for _,v in ipairs(self._consumableData) do

      if v.active and v.singleUse then

         if v.freebie then
            if v.index == CONSUMABLE_HEADSTART_INDEX then
               App.data:spendFreebie("head-start", 1, "head-start freebie consumed");
            elseif v.index == CONSUMABLE_POWERTIME_INDEX then
               App.data:spendFreebie("power-time", 1, "power-time freebie consumed");
            elseif v.index == CONSUMABLE_DOUBLESCORE_INDEX then
               App.data:spendFreebie("double-score", 1, "double-score freebie consumed");
            end
         else
            ConsumableMgr:consume(v.index);
         end
      elseif v.active and not v.singleUse then
         EventMgr:postEvent(  {  index=v.index  }, "active-timed-consumable"   );
      end
   end

end

----------------------------------------------------------------

function StateConsumableOverlayNew:_canExitOverlay()

   if self._exitTimer then
      return self._exitTimer >= EXIT_TIME;
   end

   return false;

end

----------------------------------------------------------------

function StateConsumableOverlayNew:_suppressDroneCharge()

	if self.game and self.game.avatar then
		self.game.avatar:setDroneChargeLeft(0);
		self.game.avatar:setDroneChargeRight(0);
	end

end

----------------------------------------------------------------

function StateConsumableOverlayNew:onInput()

   Base.onInput( self );

   if InputMgr:wasTouchPressed() and self:_canExitOverlay() and not self._guideActive then

      local x, y = self:wndToWorld (  InputMgr:getTouchLoc() );
      local minY;

      if #self._consumableData <= 3 then
         minY = BUTTON_POSITION_Y[1] - BUTTON_SPACING_Y*0.75;
      elseif #self._consumableData <= 6 then
         minY = BUTTON_POSITION_Y[2] - BUTTON_SPACING_Y*0.75;
      else
         minY = BUTTON_POSITION_Y[3] - BUTTON_SPACING_Y*0.75;
      end

      if y <= minY then
         self:goBack();
      end

   end

   if self:_canExitOverlay() and App.game and not self._guideActive then
      App.game:onInput();
   end

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateConsumableOverlayNew:onDisableGuideScrnButton()
   self.ScreenButton:disable();
end

----------------------------------------------------------------

function StateConsumableOverlayNew:onDisablGuideConsmButton()
   self._guideBlueButton:disable();
end

----------------------------------------------------------------

function StateConsumableOverlayNew:onShowGuideSparkle()

   if self._guideSparkle then
      self._guideSparkle:setVisible(true);
      self._guideSparkle:start();
   end

end

----------------------------------------------------------------

function StateConsumableOverlayNew:isUsingCameraHack()

   return false;

end

----------------------------------------------------------------

function StateConsumableOverlayNew:onDataServerError()

   print("StateConsumableOverlayNew:onDataServerError()");

   if not StateMgr._showErrorDialog then
      --Give back the key
      Base.onDataServerError(self);
      App.data:addKey(1, "Data Server Error");
   end

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateConsumableOverlayNew:goBack()

   self:_activateConsumables();
   StateMgr:pop();

end


----------------------------------------------------------------
----------------------------------------------------------------


return StateConsumableOverlayNew;
