----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-scroll-list');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local GUIDE_HOME_X			= 0;
local GUIDE_HOME_Y			= 270;
local GUIDE_SPACING			= 167;
local GUIDE_OFFSET         = 68;

local SCROLL_WIND_MIN_X		= -App.viewportWidth/2;
local SCROLL_WIND_MAX_X		= App.viewportWidth/2;
local SCROLL_WIND_MIN_Y		= -352;
local SCROLL_WIND_MAX_Y		= 355;

local SWEET_SPOT_MIN_X		= -175;
local SWEET_SPOT_MAX_X		=  175;
local SWEET_SPOT_MIN_Y		= SCROLL_WIND_MIN_Y;
local SWEET_SPOT_MAX_Y		= SCROLL_WIND_MAX_Y;

local SWIPE_BUFFER_X			= 100;
local SWIPE_BUFFER_Y			=  50;

local ICON_WIDTH				= 100;
local ICON_HEIGHT				= 100;

local PRIORITY_0				= 10;
local PRIORITY_1				= 20;
local PRIORITY_2				= 30;
local PRIORITY_3				= 40;
local PRIORITY_4				= 50;
local PRIORITY_5				= 60;
local PRIORITY_6				= 70;
local PRIORITY_7				= 80;
local PRIORITY_8				= 90;
local PRIORITY_9				= 100;
local PRIORITY_10				= 110;
local PRIORITY_TOP			= 3000;

----------------------------------------------------------------

local CONSUMABLE_ICON =
{
   "resources/UI/Consumable/Icons/Consumable_HeadStart.png",
   "resources/UI/Consumable/Icons/Consumable_PowerTime.png",
   "resources/UI/Consumable/Icons/Consumable_DoubleScore.png",
   "resources/UI/Consumable/Icons/Consumable_1_5xGold.png",
   "resources/UI/Consumable/Icons/Consumable_3xGold.png",
   "resources/UI/Consumable/Icons/Consumable_4xGold.png",
   "resources/UI/Consumable/Icons/Consumable_5xGold.png",
};

local PICK_UP_ICON =
{
   "resources/UI/Consumable/Icons/Upgrade_Boost.png",
   "resources/UI/Consumable/Icons/Upgrade_Magnet.png",
   "resources/UI/Consumable/Icons/Upgrade_DoubleShot.png",
   "resources/UI/Consumable/Icons/Upgrade_ScoreMod.png",
   "resources/UI/Consumable/Icons/Upgrade_Repair.png",
   "resources/UI/GR_Icon_Coin.png",
};

local PICK_UP_NAME =
{
   "Boost",
   "Magnet",
   "Double Shot",
   "Score Mod",
   "Repair",
   "Double Coin",
};

----------------------------------------------------------------

local Base						= State;
local StateConsumable		= Base:new();

----------------------------------------------------------------

function StateConsumable:onLoad()

   Base.onLoad( self );

   self.defaultFontName = "resources/font/TangoLatin.otf";
	self.mainTexture		= FlashMgr:getTextureSheet("resources/flash-resource-bundle/MainScreen.lua", "resources/flash-resource-bundle/MainScreen.png", 30);

	-- background, frame, and title
	self.BackDrop			= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "MainFrameBackground", self.mainTexture);
	self.Frame				= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "PU_ScrollFrame", self.mainTexture);
	self.Frame:setPriority(PRIORITY_5);

   if StateMgr.headerAvailable then
		self.Title		= self:addWidget(TextBox, self.mainLayer, 0, 430, 640, 300, "BOOSTERS", self.defaultFontName, 28, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
		self.Title:setPriority(PRIORITY_6);
		self.TitleShdw	= self:addWidget(TextBox, self.mainLayer, 0+3, 430-3, 640, 300, "BOOSTERS", self.defaultFontName, 28, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
		self.TitleShdw:setPriority(PRIORITY_6-1);
   end

	-- resource bar
   self.resourceBar = ResourceBar:new( self, 0, self:_calcHeaderLocY(), self.mainLayer, self.mainPartition );
   self.resourceBar:setGoldCallback(   nil,
                                       function()
                                          StateMgr:swap("state/state-coin.lua");
                                       end
                                    );
   self.resourceBar:setGemCallback(    nil,
                                       function()
                                          StateMgr:swap("state/state-gem.lua");
                                       end
                                    );

   self.resourceBar:setKeyCallback(    nil,
                                       function()
                                          StateMgr:swap("state/state-key.lua");
                                       end
                                    );
	-- scroll list
	self.ScrollList		= self:addWidget(		ScrollList,
															self.mainLayer,
															GUIDE_HOME_X, GUIDE_HOME_Y,
															{ {SCROLL_WIND_MIN_X, SCROLL_WIND_MAX_X},	{SCROLL_WIND_MIN_Y, SCROLL_WIND_MAX_Y} },
															nil,
															nil,
															{ 0, 0 },
															{ nil, GUIDE_SPACING },
															{ 0, 5 },
															nil,
															{ SWIPE_BUFFER_X, SWIPE_BUFFER_Y },
															nil,
															nil,
															{ {SWEET_SPOT_MIN_X, SWEET_SPOT_MAX_X},	{SWEET_SPOT_MIN_Y, SWEET_SPOT_MAX_Y} },
															nil, nil, nil, nil, nil, nil,
															"ScrollBarButton", self.mainTexture, 12, 78, 135, PRIORITY_4 );

	self:_createGuideCache();

	-- push/pop state buttons
	self.BackButton		= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "PU_DoneButton", self.mainTexture, nil, self.goBack, nil, nil, nil, nil, 40);
	self.BackButton:setPriority(PRIORITY_6);

end

----------------------------------------------------------------

function StateConsumable:onActivate()

	Base.onActivate(self);

   if self.resourceBar then
      self.resourceBar:createConsumableButton();
      self.resourceBar:setConsumableCallback( nil, function() end );
   end

end

----------------------------------------------------------------

function StateConsumable:onFocus()

   Base.onFocus( self );

	StateMgr:setSyncBusyDelay(0);
   self:_updateList();
   self.ScrollList:goToEntry(1);

end

----------------------------------------------------------------

function StateConsumable:onLoseFocus( )

   Base.onLoseFocus(self);
	
	StateMgr:setSyncBusyDelay(nil);

end

----------------------------------------------------------------

function StateConsumable:onUpdate( dt )

	Base.onUpdate(self, dt);

   self:_updateTime(dt);

end

----------------------------------------------------------------

function StateConsumable:isBusy()
   return StoreMgr:isBusy();
end


----------------------------------------------------------------
----------------------------------------------------------------


function StateConsumable:_updateTime( dt )

   if self._timeStamp == nil then
      self._timeStamp = MOAISim.getElapsedTime();
   end

   if MOAISim.getElapsedTime() >= self._timeStamp+1 then
      self:_updateList();
   end

end

----------------------------------------------------------------

function StateConsumable:_updateList()

	self:_createGuideData();
	self.ScrollList:setEntryCount(nil, #self._listData);
	self.ScrollList:setEntryData(self._listData);
   self.ScrollList:modTraversalY(-GUIDE_SPACING*1.2);

   self._timeStamp = MOAISim.getElapsedTime();

end

----------------------------------------------------------------

function StateConsumable:forceUpdateList()

   self._timeStamp = MOAISim.getElapsedTime()-1;

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateConsumable:_createGuideCache()

	self.ScrollList:setEntryObjectDataUpdateFunction(	function( data, object )
                                                         self:_hideObjects(object);
                                                         self:updateHeader(data, object);
                                                         self:updateSingleUse(data, object);
                                                         self:updateTimedUse(data, object);
                                                         self:updateUpgrade(data, object);
																		end
																	);

	self.ScrollList:setEntryObjectLocUpdateFunction(	function( x, y, object )
																			object.root:setLoc(x,y);
																		end
																	);

	self.ScrollList:setEntryObjectParentUpdateFunction(	function( target, object )
																				object.root:setParent(target);
																			end
																		);

	self.ScrollList:setEntryCacheObjects(	nil,
														10,
														function()
															local temp				= {};

															temp.root				= MOAITransform2D.new();

                                             --category
                                             temp.category        = self:addWidget(Image, self.mainLayer, 0, 0, 844, 167, "resources/UI/League/LB_playerbanner.png");
                                             temp.categoryText    = self:addWidget(TextBox, self.mainLayer, 0, -GUIDE_OFFSET*0.5, 844, 167, "Categroy Text", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);

                                             --boosters
                                             temp.banner          = self:addWidget(Image, self.mainLayer, 0, 0, 844, 167, "resources/UI/League/LB_rankingbanner.png", {0.9, 0.9, 0.9, 1});
                                             temp.iconFrame       = self:addWidget(Image, self.mainLayer, -240, 0, 100, 100, "resources/UI/Consumable/Consumable_IconFrame.png");
                                             temp.icon            = self:addWidget(Image, self.mainLayer, 0, 0, 100*0.85, 100*0.85, "resources/UI/Consumable/Icons/Consumable_HeadStart.png");
                                             temp.counterFrame    = self:addWidget(Image, self.mainLayer, -40, 40, 48, 48, "resources/UI/Consumable/Consumable_Counter.png");
                                             temp.counterText     = self:addWidget(TextBox, self.mainLayer, 0, 0, 100, 100, "Counter Text", self.defaultFontName, 12, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
                                             temp.boosterName     = self:addWidget(TextBox, self.mainLayer, -22, 50, 294, 100, "Booster Name", self.defaultFontName, 18, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.RIGHT_JUSTIFY, {5/255, 20/255, 31/255, 1});
                                             temp.boosterDesc     = self:addWidget(TextBox, self.mainLayer, -22, -40, 294, 100, "Booster Desc", self.defaultFontName, 12, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.LEFT_JUSTIFY);
                                             temp.boosterDescShdw = self:addWidget(TextBox, self.mainLayer, -22+2, -40-2, 294, 100, "Booster Desc", self.defaultFontName, 12, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.LEFT_JUSTIFY, {0,0,0,1});
                                             temp.timeFrame       = self:addWidget(Image, self.mainLayer, 210, 0, 179, 100, "resources/UI/Consumable/Consumable_TimeFrame.png");
                                             temp.timeText        = self:addWidget(TextBox, self.mainLayer, 0, 0, 179, 100, "Time Left", self.defaultFontName, 19, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
                                             temp.pipBar          = self:_buildPipBar( temp.banner.prop );
                                             temp.blueButton      = self:addWidget(ButtonImage, self.mainLayer, 210, 0, 179, 100, "resources/UI/Consumable/Consumable_BlueButton.png", nil, nil, nil, nil, true);
                                             temp.greenButton     = self:addWidget(ButtonImage, self.mainLayer, 210, 0, 179, 100, "resources/UI/Consumable/Consumable_GreenButton.png", nil, nil, nil, nil, true);
                                             temp.gemIcon         = self:addWidget(Image, self.mainLayer, -42, 2, 64, 64, "resources/UI/GR_Icon_Gem.png");
                                             temp.coinIcon        = self:addWidget(Image, self.mainLayer, -42, 2, 64, 64, "resources/UI/GR_Icon_Coin.png");
                                             temp.costText        = self:addWidget(TextBox, self.mainLayer, -28, 2, 179, 90, "Cost Text", self.defaultFontName, 18, nil, MOAITextBox.RIGHT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
                                             temp.costTextShdw    = self:addWidget(TextBox, self.mainLayer, -28+2, 2-2, 179, 90, "Cost Text", self.defaultFontName, 18, nil, MOAITextBox.RIGHT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
                                             temp.infoButton      = self:addWidget(ButtonImage, self.mainLayer, -45, 45, 40, 40, "resources/UI/info_button.png", nil, nil, nil, nil, true);

															temp.category:setAttrLink(          MOAIProp2D.INHERIT_TRANSFORM,		temp.root,                 MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.categoryText:setAttrLink(      MOAIProp2D.INHERIT_TRANSFORM,		temp.category.prop,        MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.banner:setAttrLink(            MOAIProp2D.INHERIT_TRANSFORM,		temp.root,                 MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.iconFrame:setAttrLink(         MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,          MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.icon:setAttrLink(              MOAIProp2D.INHERIT_TRANSFORM,		temp.iconFrame.prop,       MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.counterFrame:setAttrLink(      MOAIProp2D.INHERIT_TRANSFORM,		temp.iconFrame.prop,       MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.counterText:setAttrLink(       MOAIProp2D.INHERIT_TRANSFORM,		temp.counterFrame.prop,    MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.boosterName:setAttrLink(       MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,          MOAIProp2D.TRANSFORM_TRAIT	);
															temp.boosterDesc:setAttrLink(       MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,          MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.boosterDescShdw:setAttrLink(   MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,          MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.timeFrame:setAttrLink(         MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,          MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.timeText:setAttrLink(          MOAIProp2D.INHERIT_TRANSFORM,		temp.timeFrame.prop,       MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.blueButton:setAttrLink(        MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,          MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.greenButton:setAttrLink(       MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,          MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.gemIcon:setAttrLink(           MOAIProp2D.INHERIT_TRANSFORM,		temp.blueButton.prop,      MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.coinIcon:setAttrLink(          MOAIProp2D.INHERIT_TRANSFORM,		temp.blueButton.prop,      MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.costText:setAttrLink(          MOAIProp2D.INHERIT_TRANSFORM,		temp.blueButton.prop,      MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.costTextShdw:setAttrLink(      MOAIProp2D.INHERIT_TRANSFORM,		temp.blueButton.prop,      MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.infoButton:setAttrLink(        MOAIProp2D.INHERIT_TRANSFORM,		temp.iconFrame.prop,       MOAIProp2D.TRANSFORM_TRAIT	);

                                             temp.category:setPriority(PRIORITY_1);
                                             temp.categoryText:setPriority(PRIORITY_1+1);
                                             temp.banner:setPriority(PRIORITY_2);
                                             temp.iconFrame:setPriority(PRIORITY_3);
                                             temp.icon:setPriority(PRIORITY_3+1);
                                             temp.counterFrame:setPriority(PRIORITY_3+2);
                                             temp.counterText:setPriority(PRIORITY_3+3);
                                             temp.boosterName:setPriority(PRIORITY_3);
                                             temp.boosterDesc:setPriority(PRIORITY_3+1);
                                             temp.boosterDescShdw:setPriority(PRIORITY_3);
                                             temp.timeFrame:setPriority(PRIORITY_3);
                                             temp.timeText:setPriority(PRIORITY_3+1);
                                             temp.blueButton:setPriority(PRIORITY_3);
                                             temp.greenButton:setPriority(PRIORITY_3);
                                             temp.gemIcon:setPriority(PRIORITY_3+1);
                                             temp.coinIcon:setPriority(PRIORITY_3+1);
                                             temp.costText:setPriority(PRIORITY_3+3);
                                             temp.costTextShdw:setPriority(PRIORITY_3+2);
                                             temp.infoButton:setPriority(PRIORITY_3);

                                             temp.infoButton:setInputXtra(32, 32);

															temp.blueButton:setInViewTest(	function()
                                                                                 local x, y = temp.blueButton.prop:modelToWorld();

                                                                                 return y >= SCROLL_WIND_MIN_Y and y <= SCROLL_WIND_MAX_Y;
																										end
																									);
															temp.greenButton:setInViewTest(	function()
                                                                                 local x, y = temp.greenButton.prop:modelToWorld();

                                                                                 return y >= SCROLL_WIND_MIN_Y and y <= SCROLL_WIND_MAX_Y;
																										end
																									);
															temp.infoButton:setInViewTest(	function()
                                                                                 local x, y = temp.infoButton.prop:modelToWorld();

                                                                                 return y >= SCROLL_WIND_MIN_Y and y <= SCROLL_WIND_MAX_Y;
																										end
																									);

                                             return temp;
														end
													);

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateConsumable:_createGuideData()

	self._listData	= {};

   self:_createSingleUseData();
   self:_createTimedUseData();
   self:_createUpgradeData();

end

----------------------------------------------------------------

function StateConsumable:_createSingleUseData()

   --header
   local t =
   {
      boosterType    = "category-header",
      idIndex        = -1,
      inventory      = -1,
      curLevel       = -1,
      maxLevel       = -1,
      descName       = "SINGLE USE",
      descText       = "",
      currency       = "",
      cost           = -1,
      offset         = 1,
   };

   table.insert(self._listData, t);

   --consumable
   for i=1,ConsumableMgr:getTotalTypes() do

      if ConsumableMgr:isSingleUse(i) then

         local t     = {};
         local price = ConsumableMgr:getCost(i);

         t.boosterType  = "category-single-use";
         t.idIndex      = i;
         t.inventory    = math.max(ConsumableMgr:getInventoryCount(i) or 0, 0);
         t.curLevel     = -1;
         t.maxLevel     = -1;
         t.descName     = ConsumableMgr:getName(i);
         t.descText     = ConsumableMgr:getDesc(i);
         t.offset       = 1;

         if t.idIndex == CONSUMABLE_HEADSTART_INDEX then
            t.inventory = t.inventory + (App.data:getFreebieCount("head-start") or 0);
         elseif t.idIndex == CONSUMABLE_POWERTIME_INDEX then
            t.inventory = t.inventory + (App.data:getFreebieCount("power-time") or 0);
         elseif t.idIndex == CONSUMABLE_DOUBLESCORE_INDEX then
            t.inventory = t.inventory + (App.data:getFreebieCount("double-score") or 0);
         end

         if price and price.gem and price.gem > 0 then
            t.currency     = "gem";
            t.cost         = price.gem;
         else
            t.currency     = "gold";
            t.cost         = price.gold;
         end

         table.insert(self._listData, t);
      end
   end

end

----------------------------------------------------------------

function StateConsumable:_createTimedUseData()

   --header
   local t =
   {
      boosterType    = "category-header",
      idIndex        = -1,
      inventory      = -1,
      curLevel       = -1,
      maxLevel       = -1,
      descName       = "TIMED USE",
      descText       = "",
      currency       = "",
      cost           = -1,
      offset         = 2,
   };

   table.insert(self._listData, t);

   --consumable
   for i=1,ConsumableMgr:getTotalTypes() do

      if not ConsumableMgr:isSingleUse(i) then

         local t     = {};
         local price = ConsumableMgr:getCost(i);

         t.boosterType  = "category-timed-use";
         t.idIndex      = i;
         t.inventory    = math.max(ConsumableMgr:getTimeRemaining(i) or 0, 0);
         t.curLevel     = -1;
         t.maxLevel     = -1;
         t.descName     = ConsumableMgr:getName(i);
         t.descText     = ConsumableMgr:getDesc(i);
         t.offset       = 2;

         if price and price.gem and price.gem > 0 then
            t.currency     = "gem";
            t.cost         = price.gem;
         else
            t.currency     = "gold";
            t.cost         = price.gold;
         end

         table.insert(self._listData, t);
      end
   end

end

----------------------------------------------------------------

function StateConsumable:_createUpgradeData()

   --header
   local t =
   {
      boosterType    = "category-header",
      idIndex        = -1,
      inventory      = -1,
      curLevel       = -1,
      maxLevel       = -1,
      descName       = "UPGRADES",
      descText       = "",
      currency       = "",
      cost           = -1,
      offset         = 3,
   };

   table.insert(self._listData, t);

   --consumable
   for i=1,#PICK_UP_NAME do

      local t     = {};
      local price = App.data:getPickupUpgradeCost(i) or {gold=0, gem=0};

      t.boosterType  = "category-upgrade";
      t.idIndex      = i;
      t.inventory    = -1;
      t.curLevel     = App.data:getPickupLevel(i) or 1;
      t.maxLevel     = App.data:getPickupMaxLevel(i) or 15;
      t.descName     = PICK_UP_NAME[i] or "";
      t.descText     = -1;
      t.offset       = 3;

      if price and price.gem and price.gem > 0 then
         t.currency     = "gem";
         t.cost         = price.gem;
      else
         t.currency     = "gold";
         t.cost         = price.gold;
      end

      table.insert(self._listData, t);
   end

end

----------------------------------------------------------------

function StateConsumable:_hideObjects( element )

   if element then
      element.category:setVisible(false);
      element.categoryText:setVisible(false);
      element.banner:setVisible(false);
      element.iconFrame:setVisible(false);
      element.icon:setVisible(false);
      element.counterFrame:setVisible(false);
      element.counterText:setVisible(false);
      element.boosterName:setVisible(false);
      element.boosterDesc:setVisible(false);
      element.boosterDescShdw:setVisible(false);
      element.timeFrame:setVisible(false);
      element.timeText:setVisible(false);
      element.blueButton:setVisible(false);
      element.greenButton:setVisible(false);
      element.gemIcon:setVisible(false);
      element.coinIcon:setVisible(false);
      element.costText:setVisible(false);
      element.costTextShdw:setVisible(false);
      element.infoButton:setVisible(false);

      element.blueButton:disable();
      element.greenButton:disable();
      element.infoButton:disable();

      self:hidePipBar(element.pipBar);
   end

end

----------------------------------------------------------------

function StateConsumable:_updatePurchaseButton( currency, cost, object, buyFunc )

      local icon;
      local button;
      local bank;
      local money    = tostring(cost);
      local offset   = -28;
      local cOffset  = -42;

      if currency == "gem" then
         icon  = object.gemIcon;
         bank  = App.data:getGemCount();
      else
         icon  = object.coinIcon;
         bank  = App.data:getGoldCount()
      end

      if bank >= cost then
         button = object.greenButton;
      else
         button = object.blueButton;
      end

      icon:setVisible(true);
      button:setVisible(true);
      object.costText:setVisible(true);
      object.costTextShdw:setVisible(true);

      icon:setAttrLink(                MOAIProp2D.INHERIT_TRANSFORM,    button.prop,   MOAIProp2D.TRANSFORM_TRAIT);
      object.costText:setAttrLink(     MOAIProp2D.INHERIT_TRANSFORM,    button.prop,   MOAIProp2D.TRANSFORM_TRAIT);
      object.costTextShdw:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM,    button.prop,   MOAIProp2D.TRANSFORM_TRAIT);

      if cost > 9999 then
         object.costText:setScale(0.7, 0.7);
         object.costTextShdw:setScale(0.7, 0.7);
         object.costText:setLoc(offset+30, 2);
         object.costTextShdw:setLoc(offset+30+2, 2-2);
         icon:setLoc(cOffset, 2);
      elseif cost > 999 then
         object.costText:setScale(0.9, 0.9);
         object.costTextShdw:setScale(0.9, 0.9);
         object.costText:setLoc(offset+12, 2);
         object.costTextShdw:setLoc(offset+12+2, 2-2);
         icon:setLoc(cOffset, 2);
      elseif cost > 99 then
         object.costText:setScale(1, 1);
         object.costTextShdw:setScale(1, 1);
         object.costText:setLoc(offset, 2);
         object.costTextShdw:setLoc(offset+2, 2-2);
         icon:setLoc(cOffset, 2);
      elseif cost > 9 then
         object.costText:setScale(1, 1);
         object.costTextShdw:setScale(1, 1);
         object.costText:setLoc(offset-12, 2);
         object.costTextShdw:setLoc(offset-12+2, 2-2);
         icon:setLoc(cOffset+12, 2);
      else
         object.costText:setScale(1, 1);
         object.costTextShdw:setScale(1, 1);
         object.costText:setLoc(offset-20, 2);
         object.costTextShdw:setLoc(offset-20+2, 2-2);
         icon:setLoc(cOffset+20, 2);
      end

      object.costText:setText(money);
      object.costTextShdw:setText(money);

      button:enable();
      button.onReleasedFunc = buyFunc;

end

----------------------------------------------------------------

function StateConsumable:updateHeader( data, object )

   if not data or not object then
      return;
   end

   if data.boosterType == "category-header" then
      object.category:setVisible(true);
      object.categoryText:setVisible(true);
      object.categoryText:setText(data.descName);

      object.category:setLoc(0, data.offset*GUIDE_OFFSET);
   end

end

----------------------------------------------------------------

function StateConsumable:updateSingleUse( data, object )

   if not data or not object then
      return;
   end

   if data.boosterType == "category-single-use" then
      object.banner:setVisible(true);
      object.iconFrame:setVisible(true);
      object.icon:setVisible(true);
      object.boosterName:setVisible(true);
      object.boosterDesc:setVisible(true);
      object.boosterDescShdw:setVisible(true);

      object.banner:setLoc(0, data.offset*GUIDE_OFFSET);
      object.icon:swapImage(CONSUMABLE_ICON[data.idIndex]);
      object.icon:setAttrLink(MOAIProp2D.INHERIT_TRANSFORM, object.iconFrame.prop, MOAIProp2D.TRANSFORM_TRAIT	);
      object.boosterName:setText(data.descName);
      object.boosterDesc:setText(data.descText);
      object.boosterDescShdw:setText(data.descText);
      object.boosterName:setLoc(-22, 60);

      if data.inventory > 0 then
         object.counterFrame:setVisible(true);
         object.counterText:setVisible(true);
         object.counterText:setText(tostring(data.inventory));

         if data.inventory > 99 then
            object.counterText:setScale(0.75, 0.75);
         elseif data.inventory > 9 then
            object.counterText:setScale(0.9, 0.9);
         else
            object.counterText:setScale(1, 1);
         end
      end

      self:_updatePurchaseButton(   data.currency,
                                    data.cost,
                                    object,
                                    function ()
                                       self:buyConsumable(data.idIndex, ConsumableMgr:canBuy(data.idIndex), data.currency, data.cost);
                                    end
                                 );
   end

end

----------------------------------------------------------------

function StateConsumable:updateTimedUse( data, object )

   if not data or not object then
      return;
   end

   if data.boosterType == "category-timed-use" then
      object.banner:setVisible(true);
      object.iconFrame:setVisible(true);
      object.icon:setVisible(true);
      object.boosterName:setVisible(true);
      object.boosterDesc:setVisible(true);
      object.boosterDescShdw:setVisible(true);

      object.banner:setLoc(0, data.offset*GUIDE_OFFSET);
      object.icon:swapImage(CONSUMABLE_ICON[data.idIndex]);
      object.icon:setAttrLink(MOAIProp2D.INHERIT_TRANSFORM, object.iconFrame.prop, MOAIProp2D.TRANSFORM_TRAIT	);
      object.boosterName:setText(data.descName);
      object.boosterDesc:setText(data.descText);
      object.boosterDescShdw:setText(data.descText);
      object.boosterName:setLoc(-22, 60);

      if data.inventory > 0 then
         object.counterFrame:setVisible(true);
         object.counterText:setVisible(true);
         object.counterText:setText("1");
         object.counterText:setScale(1, 1);

         object.timeFrame:setVisible(true);
         object.timeText:setVisible(true);

         local time  = math.getTimeTable(data.inventory);
         local h,m,s;

         if time.hour > 9 then
            h  = tostring(time.hour);
         else
            h  = "0"..tostring(time.hour);
         end

         if time.min > 9 then
            m  = tostring(time.min);
         else
            m  = "0"..tostring(time.min);
         end

         if time.sec > 9 then
            s  = tostring(time.sec);
         else
            s  = "0"..tostring(time.sec);
         end

         object.timeText:setText(h..":"..m..":"..s);
      else
         self:_updatePurchaseButton(   data.currency,
                                       data.cost,
                                       object,
                                       function ()
                                          self:buyConsumable(data.idIndex, ConsumableMgr:canBuy(data.idIndex), data.currency, data.cost);
                                       end
                                    );
      end
   end

end

----------------------------------------------------------------

function StateConsumable:updateUpgrade( data, object )

   if not data or not object then
      return;
   end

   if data.boosterType == "category-upgrade" then
      object.banner:setVisible(true);
      object.iconFrame:setVisible(true);
      object.icon:setVisible(true);
      object.boosterName:setVisible(true);
      object.infoButton:setVisible(true);

      object.banner:setLoc(0, data.offset*GUIDE_OFFSET);
      object.icon:swapImage(PICK_UP_ICON[data.idIndex]);
      object.icon:setAttrLink(MOAIProp2D.INHERIT_TRANSFORM, object.iconFrame.prop, MOAIProp2D.TRANSFORM_TRAIT	);
      object.boosterName:setText(data.descName);
      object.boosterName:setLoc(-22, 50);

      object.infoButton:enable();
      object.infoButton.onReleasedFunc =  function()
                                             StateMgr:push( "state/state-pickup-info-popup.lua", {	index= data.idIndex, level= App.data:getPickupLevel(data.idIndex)	} );
                                          end

      self:showPipBar(data.curLevel, object.pipBar, data.maxLevel);

      if data.curLevel >= data.maxLevel then
         object.costText:setVisible(true);
         object.costTextShdw:setVisible(true);
         object.costText:setText("Maxed");
         object.costTextShdw:setText("Maxed");
         object.costText:setScale(1.25, 1.25);
         object.costTextShdw:setScale(1.25, 1.25);
         object.costText:setLoc(-30, 0);
         object.costTextShdw:setLoc(-30+2, -2);
      else
         self:_updatePurchaseButton(   data.currency,
                                       data.cost,
                                       object,
                                       function ()
                                          self:buyUpgrade(data.idIndex, App.data:canAffordUpgradePickup(data.idIndex), data.currency, data.cost);
                                       end
                                    );
      end
   end

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateConsumable:_buildPipBar( parent )

	local tempTable	= {};
	local pipWidth		= 16;
	local maxLevel		= 15;
	
	for i=1,maxLevel do
	
		local x = -168 + (pipWidth/2) + (pipWidth*(i - 1));

		tempTable[i] = self:addWidget(Anim, self.mainLayer, x, -25, 0,0, "PickupPips", self.mainTexture);
		tempTable[i]:setPriority(PRIORITY_3+1);
      tempTable[i]:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, parent, MOAIProp2D.TRANSFORM_TRAIT );
	end

	return tempTable;

end

----------------------------------------------------------------

function StateConsumable:hidePipBar( bar )

	for _,v in ipairs(bar) do
      v:stop();
      v:setVisible(false);
   end

end

----------------------------------------------------------------

function StateConsumable:showPipBar( level, bar, maxLevel )

   local max = maxLevel or 15;

	for k,v in ipairs(bar) do

      if k > max then
         v:setVisible(false);
      else
         v:setVisible(true);

         if k <= level then
            v:setSpan(5/30, 5/30);
            v:start();
         else
            v:setSpan(0, 0);
            v:start();
         end
      end
	end

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateConsumable:buyConsumable( index, canBuy, currency, cost )

   if canBuy then
      ConsumableMgr:buy(index);
      self:forceUpdateList();
   else
      AUTO_PURCHASE_DATA.state         = 'StateConsumable';
      AUTO_PURCHASE_DATA.carIndex      = ""..-1;
      AUTO_PURCHASE_DATA.reason1       = 'Buy-Consumable';
      AUTO_PURCHASE_DATA.reason2       = 'Buy-Consumable-'..index;
		AUTO_PURCHASE_DATA.price         = cost;
		
		if currency == "gem" then
         AUTO_PURCHASE_DATA.currency	= 'Gem';
		else
         AUTO_PURCHASE_DATA.currency	= 'Coin';
		end

		AUTO_PURCHASE_FUNC               =	function()
                                             self:buyConsumable(index);
                                          end

		StateMgr:push("state/state-auto-purchase-popup-maz.lua");

      return;
   end

end

----------------------------------------------------------------

function StateConsumable:buyUpgrade( index, canBuy, currency, cost )

   if canBuy then
      App.data:upgradePickup(index);
      self:forceUpdateList();
   else
      AUTO_PURCHASE_DATA.state		= 'StateConsumable';
      AUTO_PURCHASE_DATA.carIndex	= ""..-1;
      AUTO_PURCHASE_DATA.reason1		= 'Upgrade-Pickup';
      AUTO_PURCHASE_DATA.reason2		= 'Upgrade-Pickup-'..index;
		AUTO_PURCHASE_DATA.price		= cost;
		
		if currency == "gem" then
         AUTO_PURCHASE_DATA.currency	= 'Gem';
		else
         AUTO_PURCHASE_DATA.currency	= 'Coin';
		end

		AUTO_PURCHASE_FUNC               =	function()
                                             self:buyUpgrade(index);
                                          end

		StateMgr:push("state/state-auto-purchase-popup-maz.lua");

      return;
   end

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateConsumable:goBack()
	StateMgr:pop();
end


----------------------------------------------------------------
----------------------------------------------------------------


return StateConsumable;


----------------------------------------------------------------
