----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-scroll-list');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local GUIDE_HOME_X			= 0;
local GUIDE_HOME_Y			= 270;
local GUIDE_SPACING			= 215;

local SCROLL_WIND_MIN_X		= -App.viewportWidth/2;
local SCROLL_WIND_MAX_X		= App.viewportWidth/2;
local SCROLL_WIND_MIN_Y		= -352;
local SCROLL_WIND_MAX_Y		= 355;

local SWEET_SPOT_MIN_X		= -175;
local SWEET_SPOT_MAX_X		=  175;
local SWEET_SPOT_MIN_Y		= SCROLL_WIND_MIN_Y;
local SWEET_SPOT_MAX_Y		= SCROLL_WIND_MAX_Y;

local SWIPE_BUFFER_X			= 100;
local SWIPE_BUFFER_Y			=  50;

local ICON_WIDTH				= 100;
local ICON_HEIGHT				= 100;

local STATE_UNLOCKED			= "state-unlocked";
local STATE_LOCKED			= "state-locked";
local STATE_COMING_SOON		= "state-coming-soon";

local STAMP_ANIM_DELAY		= 0.5;
local STAMP_ANIM_TIME		= 1/4;
local STAMP_FLASH_TIME		= 1/3;
local STAMP_SCALE_SIZE		= 40;

local DRONE_ICON_LOC			=
{
	{x= (-ICON_WIDTH-10)*2,	y=12},
	{x= -ICON_WIDTH-10,		y=12},
	{x= 0,						y=12},
	{x= ICON_WIDTH+10,		y=12},
	{x= (ICON_WIDTH+10)*2,	y=12},
};

local DRONE_STARS					=
{
	"resources/UI/Drone_UI/codex1Star.png",
	"resources/UI/Drone_UI/codex2Star.png",
	"resources/UI/Drone_UI/codex3Star.png",
	"resources/UI/Drone_UI/codex4Star.png",
	"resources/UI/Drone_UI/codex5Star.png",
};

local PRIORITY_0				= 10;
local PRIORITY_1				= 20;
local PRIORITY_2				= 30;
local PRIORITY_3				= 40;
local PRIORITY_4				= 50;
local PRIORITY_5				= 60;
local PRIORITY_6				= 70;
local PRIORITY_7				= 80;
local PRIORITY_8				= 90;
local PRIORITY_9				= 100;
local PRIORITY_10				= 110;
local PRIORITY_TOP			= 3000;


----------------------------------------------------------------

local Base						= State;
local StateDroneCodex		= Base:new();

----------------------------------------------------------------

function StateDroneCodex:onLoad()

   Base.onLoad( self );

   self.defaultFontName = "resources/font/TangoLatin.otf";
	self.mainTexture		= FlashMgr:getTextureSheet("resources/flash-resource-bundle/MainScreen.lua", "resources/flash-resource-bundle/MainScreen.png", 30);

	-- background, frame, and title
	self.BackDrop			= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "MainFrameBackground", self.mainTexture);
	self.Frame				= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "PU_ScrollFrame", self.mainTexture);
	self.Frame:setPriority(PRIORITY_5);

   if StateMgr.headerAvailable then
		self.Title		= self:addWidget(TextBox, self.mainLayer, 0, 430, 640, 300, "DRONE CODEX", self.defaultFontName, 28, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
		self.Title:setPriority(PRIORITY_6);
		self.TitleShdw	= self:addWidget(TextBox, self.mainLayer, 0+3, 430-3, 640, 300, "DRONE CODEX", self.defaultFontName, 28, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
		self.TitleShdw:setPriority(PRIORITY_6-1);
   end

	-- resource bar
   self.resourceBar = ResourceBar:new( self, 0, self:_calcHeaderLocY(), self.mainLayer, self.mainPartition );
	
	-- scroll list
	self.ScrollList		= self:addWidget(		ScrollList,
															self.mainLayer,
															GUIDE_HOME_X, GUIDE_HOME_Y,
															{ {SCROLL_WIND_MIN_X, SCROLL_WIND_MAX_X},	{SCROLL_WIND_MIN_Y, SCROLL_WIND_MAX_Y} },
															nil,
															nil,
															{ 0, 0 },
															{ nil, GUIDE_SPACING },
															{ 0, 5 },
															nil,
															{ SWIPE_BUFFER_X, SWIPE_BUFFER_Y },
															nil,
															nil,
															{ {SWEET_SPOT_MIN_X, SWEET_SPOT_MAX_X},	{SWEET_SPOT_MIN_Y, SWEET_SPOT_MAX_Y} },
															nil, nil, nil, nil, nil, nil,
															"ScrollBarButton", self.mainTexture, 12, 78, 135, PRIORITY_4 );

	self:_createGuideCache();

	-- push/pop state buttons
	self.BackButton		= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "PU_DoneButton", self.mainTexture, nil, self.goBack, nil, nil, nil, nil, 40);
	self.BackButton:setPriority(PRIORITY_6);

end

----------------------------------------------------------------

function StateDroneCodex:onActivate()

	Base.onActivate(self);

   if self.resourceBar then
      self.resourceBar:createConsumableButton();
   end

end

----------------------------------------------------------------

function StateDroneCodex:onFocus()

   Base.onFocus( self );

	self._stampTimer		= 0;
	self._stampObject		= nil;
	self._stampLeft		= nil;
	self._stampRight		= nil;
	self._stampHalo		= nil;

	StateMgr:setSyncBusyDelay(0);
	self:_createGuideData();
	self.ScrollList:setEntryCount(nil, #self._guideData);
	self.ScrollList:setEntryData(self._guideData);
	self:_checkStampAnim();

end

----------------------------------------------------------------

function StateDroneCodex:onLoseFocus( )

   Base.onLoseFocus(self);
	
	StateMgr:setSyncBusyDelay(nil);

	self._stampTimer		= 0;
	self._stampObject		= nil;
	self._stampLeft		= nil;
	self._stampRight		= nil;
	self._stampHalo		= nil;

end

----------------------------------------------------------------

function StateDroneCodex:onUpdate( dt )

	Base.onUpdate(self, dt);

	self:_updateStampAnim(dt);

end

----------------------------------------------------------------------

function StateDroneCodex:isBusy()
   return StoreMgr:isBusy();
end


----------------------------------------------------------------
----------------------------------------------------------------


function StateDroneCodex:_updateStampAnim( dt )

	if self._stampObject then

		self._stampTimer = self._stampTimer + dt;

		if			self._stampTimer < STAMP_ANIM_DELAY
			and	self._stampLeft
			and	self._stampRight then

			self._stampLeft:setColor(0.35, 0.35, 0.35, 1);
			self._stampRight:setColor(0.35, 0.35, 0.35, 1);
		elseif self._stampTimer >= STAMP_ANIM_TIME+STAMP_ANIM_DELAY+STAMP_FLASH_TIME then
			self._stampObject:setScale(1, 1);
			self._stampLeft:setColor(1, 1, 1, 1);
			self._stampRight:setColor(1, 1, 1, 1);
			self._stampLeft:setPriority(PRIORITY_4+2);
			self._stampRight:setPriority(PRIORITY_4+2);
			self._stampHalo:setVisible(false);

			self._stampObject	= nil;
			self._stampLeft	= nil;
			self._stampRight	= nil;
			self._stampHalo	= nil;
		elseif self._stampTimer >= STAMP_ANIM_DELAY+STAMP_ANIM_TIME then
			self._stampObject:setScale(1, 1);
			self._stampLeft:setColor(1, 1, 1, 1);
			self._stampRight:setColor(1, 1, 1, 1);
			self._stampLeft:setPriority(PRIORITY_4+2);
			self._stampRight:setPriority(PRIORITY_4+2);
			self._stampHalo:setPriority(PRIORITY_4+1);
			self._stampHalo:setVisible(true);

			local k	= math.mapClamped(STAMP_ANIM_DELAY+STAMP_ANIM_TIME, 1, STAMP_ANIM_TIME+STAMP_ANIM_DELAY+STAMP_FLASH_TIME, STAMP_SCALE_SIZE, self._stampTimer );
			local a	= math.mapClamped(STAMP_ANIM_DELAY+STAMP_ANIM_TIME, 1, STAMP_ANIM_TIME+STAMP_ANIM_DELAY+STAMP_FLASH_TIME, 0, self._stampTimer );

			self._stampHalo:setScale(k, k);
			self._stampHalo:setAlpha(a);
		elseif self._stampTimer >= STAMP_ANIM_DELAY then
			local k	= math.mapClamped(STAMP_ANIM_DELAY, STAMP_SCALE_SIZE, STAMP_ANIM_TIME+STAMP_ANIM_DELAY, 1, self._stampTimer );
			local a	= math.mapClamped(STAMP_ANIM_DELAY, 0, STAMP_ANIM_TIME+STAMP_ANIM_DELAY, 1.0, self._stampTimer );

			self._stampObject:setScale(k, k);
			self._stampLeft:setColor(1, 1, 1, a);
			self._stampRight:setColor(1, 1, 1, a);
			self._stampLeft:setPriority(PRIORITY_6);
			self._stampRight:setPriority(PRIORITY_6);
		end
	end

end

----------------------------------------------------------------
----------------------------------------------------------------


function StateDroneCodex:_createGuideCache()

	self.ScrollList:setEntryObjectDataUpdateFunction(	function( data, object )
																			self:_hideObjects(object);
																			self:_showStars(object.stars, 0);

																			local drones	=
																			{
																				{	l=object.drone_1L,	r=object.drone_1R,	parent=object.droneRt_1,	halo=object.halo_1	};
																				{	l=object.drone_2L,	r=object.drone_2R,	parent=object.droneRt_2,	halo=object.halo_2	};
																				{	l=object.drone_3L,	r=object.drone_3R,	parent=object.droneRt_3,	halo=object.halo_3	};
																				{	l=object.drone_4L,	r=object.drone_4R,	parent=object.droneRt_4,	halo=object.halo_4	};
																				{	l=object.drone_5L,	r=object.drone_5R,	parent=object.droneRt_5,	halo=object.halo_5	};
																			};

																			object.root:setVisible(true);
																			object.infoButton:disable();

																			if self._stampAnimList == nil then
																				self._stampAnimList	= {};
																			end

																			if data.stamp and data.rarity > 0 then
																				local tbl		= {};

																				tbl.animObject	= drones[data.rarity].parent;
																				tbl.dType		= data.dType;
																				tbl.left			= drones[data.rarity].l;
																				tbl.right		= drones[data.rarity].r;
																				tbl.halo			= drones[data.rarity].halo;

																				table.insert(self._stampAnimList, tbl);
																			end

																			for k,v in ipairs(drones) do
																				v.parent:setScale(1, 1);
																			end

																			if data.state == STATE_LOCKED then
																				object.messagePlate:setVisible(true);
																				object.LockTxt:setVisible(true);
																				object.LockTxtShdw:setVisible(true);
																				object.LockTxt:setText(data.name);
																				object.LockTxtShdw:setText(data.name);

																				for k,v in ipairs(drones) do
																					v.l:setVisible(true);
																					v.r:setVisible(true);
																					v.l:swapImage(data.visTable[k]);
																					v.r:swapImage(data.visTable[k]);
																					v.l:setAttrLink(MOAIProp2D.INHERIT_TRANSFORM, v.parent.prop, MOAIProp2D.TRANSFORM_TRAIT);
																					v.r:setAttrLink(MOAIProp2D.INHERIT_TRANSFORM, v.parent.prop, MOAIProp2D.TRANSFORM_TRAIT);
																					v.l:setColor(0, 0, 0, 1);
																					v.r:setColor(0, 0, 0, 1);
																					v.l:setPriority(PRIORITY_4+1);
																					v.r:setPriority(PRIORITY_4+1);
																					v.r:setScale(-1,1);
																				end
																			elseif data.state == STATE_UNLOCKED then
																				object.namePlate:setVisible(true);
																				object.NameTxt:setVisible(true);
																				object.NameTxtShdw:setVisible(true);
																				object.NameTxt:setText(data.name);
																				object.NameTxtShdw:setText(data.name);
																				self:_showStars(object.stars, data.rarity);
																				object.infoButton:setVisible(true);
																				object.infoButton:enable(true);

																				object.infoButton.onReleasedFunc	=	function()
																																	StateMgr:push( "state/state-drone-info-popup.lua", {	texture = data.visTable[data.rarity], rarity = data.rarity, level = data.level, name = data.name, desc = data.desc  } );
																																end

																				for k,v in ipairs(drones) do
																					v.l:setVisible(true);
																					v.r:setVisible(true);
																					v.l:swapImage(data.visTable[k]);
																					v.r:swapImage(data.visTable[k]);
																					v.l:setAttrLink(MOAIProp2D.INHERIT_TRANSFORM, v.parent.prop, MOAIProp2D.TRANSFORM_TRAIT);
																					v.r:setAttrLink(MOAIProp2D.INHERIT_TRANSFORM, v.parent.prop, MOAIProp2D.TRANSFORM_TRAIT);
																					v.l:setPriority(PRIORITY_4+1);
																					v.r:setPriority(PRIORITY_4+1);
																					v.r:setScale(-1,1);

																					if k <= data.rarity then
																						v.l:setColor(1, 1, 1, 1);
																						v.r:setColor(1, 1, 1, 1);
																					else
																						v.l:setColor(0.35, 0.35, 0.35, 1);
																						v.r:setColor(0.35, 0.35, 0.35, 1);
																					end
																				end
																			else
																				object.messagePlate:setVisible(true);
																				object.icon:setVisible(true);
																				object.LockTxt:setVisible(true);
																				object.LockTxtShdw:setVisible(true);
																				object.LockTxt:setText("COMING SOON");
																				object.LockTxtShdw:setText("COMING SOON");
																			end
																		end
																	);

	self.ScrollList:setEntryObjectLocUpdateFunction(	function( x, y, object )
																			object.root:setLoc(x,y);
																		end
																	);

	self.ScrollList:setEntryObjectParentUpdateFunction(	function( target, object )
																				object.root:setParent(target);
																			end
																		);

	self.ScrollList:setEntryCacheObjects(	nil,
														10,
														function()
															local temp				= {};

															temp.root				= self:addWidget(Image, self.mainLayer, 0, 0, 600, 170, "resources/UI/Drone_UI/codexFrame.png");
															temp.namePlate			= self:addWidget(Image, self.mainLayer, 0, -80, 448, 48, "resources/UI/Drone_UI/codexNameStarFrame.png");
															temp.messagePlate		= self:addWidget(Image, self.mainLayer, 0, -80, 448, 48, "resources/UI/Drone_UI/codexLockFrame.png");
															temp.icon				= self:addWidget(Image, self.mainLayer, 0, 12, ICON_WIDTH, ICON_HEIGHT, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.NameTxt			= self:addWidget(TextBox, self.mainLayer, -120, -80, 640, 300, "NAME", self.defaultFontName, 15, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
															temp.NameTxtShdw		= self:addWidget(TextBox, self.mainLayer, -120+3, -80-3, 640, 300, "NAME", self.defaultFontName, 15, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
															temp.LockTxt			= self:addWidget(TextBox, self.mainLayer, 0, -80, 640, 300, "UNLOCK IN GARAGE", self.defaultFontName, 15, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
															temp.LockTxtShdw		= self:addWidget(TextBox, self.mainLayer, 0+3, -80-3, 640, 300, "UNLOCK IN GARAGE", self.defaultFontName, 15, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
															temp.infoButton		= self:addWidget(ButtonImage, self.mainLayer, -2, -80-2, 64, 64, "resources/UI/info_button.png", nil, nil, nil, nil, true);
															temp.droneRt_1			= self:addWidget(Image, self.mainLayer, DRONE_ICON_LOC[1].x, DRONE_ICON_LOC[1].y, 1, 1, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.droneRt_2			= self:addWidget(Image, self.mainLayer, DRONE_ICON_LOC[2].x, DRONE_ICON_LOC[2].y, 1, 1, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.droneRt_3			= self:addWidget(Image, self.mainLayer, DRONE_ICON_LOC[3].x, DRONE_ICON_LOC[3].y, 1, 1, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.droneRt_4			= self:addWidget(Image, self.mainLayer, DRONE_ICON_LOC[4].x, DRONE_ICON_LOC[4].y, 1, 1, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.droneRt_5			= self:addWidget(Image, self.mainLayer, DRONE_ICON_LOC[5].x, DRONE_ICON_LOC[5].y, 1, 1, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.halo_1				= self:addWidget(Image, self.mainLayer, 0, 0, ICON_WIDTH, ICON_HEIGHT, "resources/UI/Bounty/WhiteFlash_small.png");
															temp.halo_2				= self:addWidget(Image, self.mainLayer, 0, 0, ICON_WIDTH, ICON_HEIGHT, "resources/UI/Bounty/WhiteFlash_small.png");
															temp.halo_3				= self:addWidget(Image, self.mainLayer, 0, 0, ICON_WIDTH, ICON_HEIGHT, "resources/UI/Bounty/WhiteFlash_small.png");
															temp.halo_4				= self:addWidget(Image, self.mainLayer, 0, 0, ICON_WIDTH, ICON_HEIGHT, "resources/UI/Bounty/WhiteFlash_small.png");
															temp.halo_5				= self:addWidget(Image, self.mainLayer, 0, 0, ICON_WIDTH, ICON_HEIGHT, "resources/UI/Bounty/WhiteFlash_small.png");
															temp.drone_1L			= self:addWidget(Image, self.mainLayer, -ICON_WIDTH/4,	0, ICON_WIDTH/2, ICON_HEIGHT, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.drone_1R			= self:addWidget(Image, self.mainLayer, ICON_WIDTH/4,		0, ICON_WIDTH/2, ICON_HEIGHT, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.drone_2L			= self:addWidget(Image, self.mainLayer, -ICON_WIDTH/4,	0, ICON_WIDTH/2, ICON_HEIGHT, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.drone_2R			= self:addWidget(Image, self.mainLayer, ICON_WIDTH/4,		0, ICON_WIDTH/2, ICON_HEIGHT, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.drone_3L			= self:addWidget(Image, self.mainLayer, -ICON_WIDTH/4,	0, ICON_WIDTH/2, ICON_HEIGHT, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.drone_3R			= self:addWidget(Image, self.mainLayer, ICON_WIDTH/4,		0, ICON_WIDTH/2, ICON_HEIGHT, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.drone_4L			= self:addWidget(Image, self.mainLayer, -ICON_WIDTH/4,	0, ICON_WIDTH/2, ICON_HEIGHT, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.drone_4R			= self:addWidget(Image, self.mainLayer, ICON_WIDTH/4,		0, ICON_WIDTH/2, ICON_HEIGHT, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.drone_5L			= self:addWidget(Image, self.mainLayer, -ICON_WIDTH/4,	0, ICON_WIDTH/2, ICON_HEIGHT, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.drone_5R			= self:addWidget(Image, self.mainLayer, ICON_WIDTH/4,		0, ICON_WIDTH/2, ICON_HEIGHT, "resources/UI/Drone_UI/comingsoonicon.png");
															temp.stars				= self:_createRarityStars( temp.root.prop, 98, 38, 120, -80 );

															temp.drone_1R:setScale(-1, 1);
															temp.drone_2R:setScale(-1, 1);
															temp.drone_3R:setScale(-1, 1);
															temp.drone_4R:setScale(-1, 1);
															temp.drone_5R:setScale(-1, 1);
															temp.infoButton:disable();

															temp.namePlate:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.messagePlate:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.icon:setAttrLink(				MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.NameTxt:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.NameTxtShdw:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.LockTxt:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.LockTxtShdw:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.infoButton:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.droneRt_1:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.droneRt_2:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.droneRt_3:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.droneRt_4:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.droneRt_5:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.halo_1:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_1.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.halo_2:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_2.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.halo_3:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_3.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.halo_4:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_4.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.halo_5:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_5.prop,			MOAIProp2D.TRANSFORM_TRAIT	);
															temp.drone_1L:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_1.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.drone_1R:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_1.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.drone_2L:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_2.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.drone_2R:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_2.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.drone_3L:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_3.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.drone_3R:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_3.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.drone_4L:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_4.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.drone_4R:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_4.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.drone_5L:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_5.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.drone_5R:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.droneRt_5.prop,		MOAIProp2D.TRANSFORM_TRAIT	);

															temp.root:setPriority(PRIORITY_1);
															temp.namePlate:setPriority(PRIORITY_3);
															temp.messagePlate:setPriority(PRIORITY_3);
															temp.icon:setPriority(PRIORITY_2);
															temp.NameTxt:setPriority(PRIORITY_3+2);
															temp.NameTxtShdw:setPriority(PRIORITY_3+1);
															temp.LockTxt:setPriority(PRIORITY_3+2);
															temp.LockTxtShdw:setPriority(PRIORITY_3+1);
															temp.infoButton:setPriority(PRIORITY_4);
															temp.halo_1:setPriority(PRIORITY_4+3);
															temp.halo_2:setPriority(PRIORITY_4+3);
															temp.halo_3:setPriority(PRIORITY_4+3);
															temp.halo_4:setPriority(PRIORITY_4+3);
															temp.halo_5:setPriority(PRIORITY_4+3);
															temp.drone_1L:setPriority(PRIORITY_4+2);
															temp.drone_1R:setPriority(PRIORITY_4+2);
															temp.drone_2L:setPriority(PRIORITY_4+2);
															temp.drone_2R:setPriority(PRIORITY_4+2);
															temp.drone_3L:setPriority(PRIORITY_4+2);
															temp.drone_3R:setPriority(PRIORITY_4+2);
															temp.drone_4L:setPriority(PRIORITY_4+2);
															temp.drone_4R:setPriority(PRIORITY_4+2);
															temp.drone_5L:setPriority(PRIORITY_4+2);
															temp.drone_5R:setPriority(PRIORITY_4+2);

															self:_showStars(temp.stars, 0);

															temp.infoButton:setInViewTest(	function()
																											local x, y = temp.infoButton.prop:modelToWorld();

																											return y >= SCROLL_WIND_MIN_Y and y <= SCROLL_WIND_MAX_Y;
																										end
																									);

															return temp;
														end
													);

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateDroneCodex:_createGuideData()

	local pickTbl		= DroneMgr:getDroneTypeOddsTable();

	self._guideData	= {};

	for i=1,#pickTbl do

		local lRarity, lIndex, lLevel	= DroneMgr:isDroneTypeCollectedLeft(i);
		local rRarity, rIndex, rLevel	= DroneMgr:isDroneTypeCollectedRight(i);
		local rarity						= math.max(lRarity or 0, rRarity or 0);
		local level							= math.max(lLevel or 0, rLevel or 0);

		if pickTbl[i] > 0 or rarity > 0 then
			local tbl							= {};

			if rarity <= 0 then
				tbl.state	= STATE_LOCKED;
			else
				tbl.state	= STATE_UNLOCKED;
			end

			tbl.dType		= i;
			tbl.rarity		= rarity;
			tbl.level		= level;
			tbl.name			= DroneMgr:getDroneName(i, math.max(rarity, 1));
			tbl.desc			= DroneMgr:getDroneDesc(i);
			tbl.visTable	= DroneMgr:getDroneBodyVisTable(i);
			tbl.stamp		= App.data:getValue("new-drone-type-"..i, false);

			table.insert(self._guideData, tbl);
		end
	end

	local comingsoon		= {};

	comingsoon.state		= STATE_COMING_SOON;
	comingsoon.dType		= 0;
	comingsoon.rarity		= 0;
	comingsoon.level		= 0;
	comingsoon.name		= "COMING SOON"
	comingsoon.desc		= {};
	comingsoon.visTable	= {};
	comingsoon.stamp		= false;

	table.insert(self._guideData, comingsoon);

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateDroneCodex:_checkStampAnim()

	if self._stampAnimList and #self._stampAnimList > 0 then
		for k,v in pairs(self._stampAnimList) do
			App.data:setValue("new-drone-type-"..v.dType, false);
		end

		self.ScrollList:goToEntry( math.max(self._stampAnimList[1].dType-1, 1) );

		self._stampObject		= self._stampAnimList[1].animObject;
		self._stampLeft		= self._stampAnimList[1].left;
		self._stampRight		= self._stampAnimList[1].right;
		self._stampHalo		= self._stampAnimList[1].halo;
		self._stampAnimList	= {};

		self._stampLeft:setColor(0.35, 0.35, 0.35, 1);
		self._stampRight:setColor(0.35, 0.35, 0.35, 1);

		delay(	STAMP_ANIM_DELAY,
					function()
						EventMgr:postEvent( { sounds={start={"BountyCrosshairs",},}, name="achievement sting",}, "add-sfx" );
					end );
	end

end

------------------------------------------------------------------------------------

function StateDroneCodex:_createRarityStars( parent, w, h, x, y )

	if parent then
		local width		= w or 254;
		local height	= h or 100;
		local X			= x or 0;
		local Y			= y or 0;
		local temp		= {};

		for i=1,DroneMgr:getDroneMaxRarity() do
			temp[i]	= self:addWidget( Image, self.mainLayer, X, Y, width, height, DRONE_STARS[i] );

			temp[i]:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, parent, MOAIProp2D.TRANSFORM_TRAIT );
			temp[i]:setPriority( PRIORITY_3+2 );
		end

		return temp;
	end

end

------------------------------------------------------------------------------------

function StateDroneCodex:_hideObjects( object )

	if object and type(object) == "table" then
		for k,v in pairs(object) do
			if v and type(v) == "table" and v.stop then
				v:stop();
			end

			if v and type(v) == "table" and v.setVisible then
				v:setVisible(false);
			end
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneCodex:_showStars( stars, rarity )

	if stars and rarity then
		for k,v in ipairs(stars) do
			if k == rarity then
				v:setVisible(true);
			else
				v:setVisible(false);
			end
		end
	end

end


----------------------------------------------------------------
----------------------------------------------------------------


function StateDroneCodex:goBack()
	StateMgr:pop();
end


----------------------------------------------------------------
----------------------------------------------------------------


return StateDroneCodex;


----------------------------------------------------------------
