----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local DRONE_LEFT					= "left";  -- mirror in app data
local DRONE_RIGHT					= "right"; -- mirror in app data

local PRIORITY_0					= 10;
local PRIORITY_1					= 20;
local PRIORITY_2					= 30;
local PRIORITY_3					= 40;
local PRIORITY_4					= 50;
local PRIORITY_5					= 60;
local PRIORITY_6					= 70;
local PRIORITY_7					= 80;
local PRIORITY_8					= 90;
local PRIORITY_9					= 100;
local PRIORITY_10					= 110;

local SQUARE_TEXTURE				= "resources/square.png";
local INIT_TIME					= 0.5;
local GLOW_ROTATION_RATE		= 45;

local STATE_EVOLVE_START		= "evolve-start";

----------------------------------------------------------------

local DRONE_STARS					=
{
	"resources/UI/Drone_UI/1Star.png",
	"resources/UI/Drone_UI/2Star.png",
	"resources/UI/Drone_UI/3Star.png",
	"resources/UI/Drone_UI/4Star.png",
	"resources/UI/Drone_UI/5Star.png",
};

--CAUTION: The order of the shadow must match the drone index order
local DRONE_SHADOW				=
{
	"resources/Vehicle/Drones/DroneGun/DroneGun_Shadow.png",
	"resources/Vehicle/Drones/DroneLaser/DroneLaser_Shadow.png",
	"resources/Vehicle/Drones/DroneBoost/DroneBoost_Shadow.png",
	"resources/Vehicle/Drones/DroneMissile/DroneMissile_Shadow.png",
	"resources/Vehicle/Drones/DroneRocket/DroneRocket_Shadow.png",
	"resources/Vehicle/Drones/DroneRepair/DroneRepair_Shadow.png",
	"resources/Vehicle/Drones/DroneShield/DroneShield_Shadow.png",
	"resources/Vehicle/Drones/DroneBarricade/DroneBarricade_Shadow.png",
};

----------------------------------------------------------------

local Base						= State;
local StateDroneEvolve		= Base:new();

----------------------------------------------------------------

function StateDroneEvolve:onLoad()

   State.onLoad( self );

   self.defaultFontName = "resources/font/TangoLatin.otf";
	self.IS_POPUP  = true;

	-- text message
	self.TitleText = self:addWidget(TextBox, self.mainLayer, 0, 326, 540, 70, "Drone Name", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.TitleText:setScale(1.5, 1.5);
	self.TitleText:setPriority(PRIORITY_3);
	self.TitleTextShdw = self:addWidget(TextBox, self.mainLayer, 0+2, 326-2, 540, 70, "Drone Name", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.TitleTextShdw:setScale(1.5, 1.5);
	self.TitleTextShdw:setPriority(PRIORITY_3-1);
	self.MessageText = self:addWidget(TextBox, self.mainLayer, 0, -348, 500, 260, "Tap To Continue", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.MessageText:setPriority(PRIORITY_3);
	self.MessageText:setScale(1.2, 1.2);
	self.MessageTextShdw = self:addWidget(TextBox, self.mainLayer, 0+2, -348-2, 500, 260, "Tap To Continue", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.MessageTextShdw:setPriority(PRIORITY_3-1);
	self.MessageTextShdw:setScale(1.2, 1.2);

   -- reveal anim
   local droneSheet  = FlashMgr:getTextureSheet("resources/flash-resource-bundle/Drone_Effects.lua", "resources/flash-resource-bundle/Drone_Effects.png", 30);

   self._revealAnim  = self:addWidget(Anim, self.mainLayer, 0, -54, 0,0, "Dronereveal_Dronepopup", droneSheet);
   self._revealAnim:setVisible(false);
   self._revealAnim:setScale(1/11.14, 1/9.14);

   -- evolve anim
   self._evolveAnim  = self:addWidget(Anim, self.mainLayer, 0, -54, 0,0, "Drone_evolve", droneSheet);
   self._evolveAnim:setPriority(PRIORITY_10)
   self._evolveAnim:setVisible(false);
   self._evolveAnim:setListener( MOAIAction.EVENT_STOP,
                                 function()
                                    enableButtonInput();
                                    self.BackButton:enable();
                                    self.MessageText:setVisible(true);
                                    self.MessageTextShdw:setVisible(true);
                                    self._evolveAnim:setVisible(false);
                                 end );

	-- god rays
	self.GGlow = self:addWidget(Image, self.mainLayer, 0, -54, 640, 640, "resources/UI/Drone_UI/Backglow-01.png");
	self.GRays = self:addWidget(Image, self.mainLayer, 0, -54, 1200, 1200, "resources/UI/Drone_UI/LightRays-01.png");

	self.GGlow:setPriority(PRIORITY_2-2);
	self.GRays:setPriority(PRIORITY_2-1);
   self.GGlow:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );
   self.GRays:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );

	-- pic
	self.droneRevealL	= self:addWidget(Image, self.mainLayer, -96, 0, 192, 384, "resources/UI/Drone_UI/sill.png");
	self.droneRevealL:setPriority(PRIORITY_3);
	self.droneRevealR	= self:addWidget(Image, self.mainLayer, 96, 0, 192, 384, "resources/UI/Drone_UI/sill.png");
	self.droneRevealR:setPriority(PRIORITY_3);
	self.droneShadow	= self:addWidget(Image, self.mainLayer, 0+40, -40, 256, 256, "resources/UI/Drone_UI/sill.png");
	self.droneShadow:setPriority(PRIORITY_3-1);
	self.starsReveal	= self:_createDroneStars( 0, 210, 1.55, 1.45 );

	-- Exit Button
	self.BackButton = self:addWidget(ButtonImage, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE, self.goBack, nil, nil, nil);
	self.BackButton:setPriority(PRIORITY_0);
	self.BackButton:setColor(0,0,0, 0.9);

end

----------------------------------------------------------------

function StateDroneEvolve:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StateDroneEvolve:onFocus( data )

   State.onFocus(self);

	Debug.assert(data and data.garage_index, "StateDroneEvolve:onFocus -> drone garage index is nil");
	Debug.assert(data and data.side, "StateDroneEvolve:onFocus -> pool is nil");

	self._state				= STATE_EVOLVE_START;
	self._revealTimer		= 0;
	self._garage_index	= data.garage_index;
	self._side				= data.side;
	self._type				= nil;
	self._rarity			= nil;
	self._texture			= nil;
	self._name				= nil;

   self:_hideAll();
   self:_updateInitDrone(data.garage_index, data.side);

   if self._side == DRONE_LEFT then
      DroneMgr:evolveDroneLeft(self._garage_index);
   else
      DroneMgr:evolveDroneRight(self._garage_index);
   end

   if self._side == DRONE_LEFT then
      self._type		= DroneMgr:getDroneTypeLeft(		self._garage_index );
      self._rarity	= DroneMgr:getDroneRarityLeft(	self._garage_index );
      self._texture	= DroneMgr:getDroneBodyVisLeft(	self._garage_index );
      self._name		= DroneMgr:getDroneNameLeft(		self._garage_index );
   else
      self._type		= DroneMgr:getDroneTypeRight(		self._garage_index );
      self._rarity	= DroneMgr:getDroneRarityRight(	self._garage_index );
      self._texture	= DroneMgr:getDroneBodyVisRight(	self._garage_index );
      self._name		= DroneMgr:getDroneNameRight(		self._garage_index );
   end

   self._name	= string.upper(self._name.." Drone");

   local collected = math.min(DroneMgr:isDroneTypeCollectedLeft(self._type) or 6,  DroneMgr:isDroneTypeCollectedRight(self._type) or 6);

   if collected and collected > math.clamp(self._rarity-1, 1, 5) then
      App.data:setValue("new-drone-type-"..self._type, true);
   end

end

----------------------------------------------------------------

function StateDroneEvolve:onUpdate( dt )

	Base.onUpdate(self, dt);

	self:_updateFlashTime(dt);
	self:_updateGGlowRayRotate(dt);

end


------------------------------------------------------------------------------------
-- Anim Update
------------------------------------------------------------------------------------


function StateDroneEvolve:_updateFlashTime( dt )

	if self._state == STATE_EVOLVE_START then
		self._revealTimer	= self._revealTimer + dt;

		local k	= math.min( self._revealTimer/INIT_TIME, 1);

		if self._revealTimer >= INIT_TIME then
			self._state			= STATE_EVOLVE_END;
			self._revealTimer	= 0;

			self:_evolveDroneStart();
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneEvolve:_updateGGlowRayRotate( dt )

   local glowAngle	= math.clampAngle360( self.GGlow:getRot() + GLOW_ROTATION_RATE*dt );
   local rayAngle		= math.clampAngle360( self.GRays:getRot() - GLOW_ROTATION_RATE*dt );

   self.GGlow:setRot( glowAngle );
   self.GRays:setRot( rayAngle );

end


------------------------------------------------------------------------------------
-- Evolve Drone Anim
------------------------------------------------------------------------------------


function StateDroneEvolve:_evolveDroneStart()

   self._evolveAnim:setVisible(true);
   self._evolveAnim:setTime(0);
   self._evolveAnim:start();

   EventMgr:postEvent( {sounds = {start={"DroneMerge",},}, name="drone merge"}, "add-sfx" );

   delay(   23/30,
            function()

               if self.droneRevealL then
                  self.droneRevealL:setVisible(false);
               end

               if self.droneRevealR then
                  self.droneRevealR:setVisible(false);
               end

               if self.droneShadow then
                  self.droneShadow:setVisible(false);
               end

               if self._revealAnim then
                  self._revealAnim:setVisible(true);
               end

            end );

   delay(   70/30,
            function()
               if self then
                  self:_evolveDroneReveal();
               end
            end );

end

------------------------------------------------------------------------------------

function StateDroneEvolve:_evolveDroneReveal()

	self.GGlow:setVisible(true);
	self.GRays:setVisible(true);
	self.TitleText:setVisible(true);
	self.TitleTextShdw:setVisible(true);
	self.droneRevealL:setVisible(true);
	self.droneRevealR:setVisible(true);
	self.droneShadow:setVisible(true);
	self:_showStars(self.starsReveal, self._rarity);

	self.TitleText:setText(self._name);
	self.TitleTextShdw:setText(self._name);

	self.droneRevealL:swapImage(self._texture);
	self.droneRevealR:swapImage(self._texture);
	self.droneRevealR:setScale(-1, 1);
	self.droneShadow:swapImage(DRONE_SHADOW[self._type]);

   self.droneRevealL:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );
   self.droneRevealR:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );
   self.droneShadow:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );

   self._revealAnim:setVisible(true);
   self._revealAnim:setTime(0);
   self._revealAnim:start();

   EventMgr:postEvent( {sounds = {start={"DroneReveal",},}, name="drone reveal"}, "add-sfx" );

end


------------------------------------------------------------------------------------
-- Util
------------------------------------------------------------------------------------


function StateDroneEvolve:_createDroneStars(  x, y, sx, sy, priority )

	if x and y and sx and sy then
		local width		= 200 * sx;
		local height	= 100 * sy;
		local temp		= {};
		local p			= priority or PRIORITY_2;

		for i=1,DroneMgr:getDroneMaxRarity() do
			temp[i]		= self:addWidget(Image, self.mainLayer, x, y, width, height, DRONE_STARS[i]);

			temp[i]:setPriority(p);
		end

		return temp;
	end

end

------------------------------------------------------------------------------------

function StateDroneEvolve:_hideStars( stars )

	if stars then
		for k,v in ipairs(stars) do
			v:setVisible(false);
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneEvolve:_showStars( stars, rarity )

	if stars and rarity and stars[rarity] then
		for k,v in ipairs(stars) do
			if k == rarity then
				v:setVisible(true);
			else
				v:setVisible(false);
			end
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneEvolve:_hideAll()

   disableButtonInput();
	self.GGlow:setVisible(false);
	self.GRays:setVisible(false);
	self.TitleText:setVisible(false);
	self.TitleTextShdw:setVisible(false);
	self.MessageText:setVisible(false);
	self.MessageTextShdw:setVisible(false);
	self:_hideStars(self.starsReveal);
	self.BackButton:disable();
	self.BackButton:setVisible(true);
   self._revealAnim:setVisible(false);
   self._evolveAnim:setVisible(false);

end

------------------------------------------------------------------------------------

function StateDroneEvolve:_updateInitDrone( garageIndex, side )

   local texture, type;

   if side == DRONE_LEFT then
      texture  = DroneMgr:getDroneBodyVisLeft(  garageIndex );
      type		= DroneMgr:getDroneTypeLeft(     garageIndex );
   else
      texture  = DroneMgr:getDroneBodyVisRight( garageIndex );
      type		= DroneMgr:getDroneTypeRight(    garageIndex );
   end

   self._revealAnim:setAnimLastFrame();
	self.droneRevealL:swapImage(texture);
	self.droneRevealR:swapImage(texture);
	self.droneRevealR:setScale(-1, 1);
	self.droneShadow:swapImage(DRONE_SHADOW[type]);

   self.droneRevealL:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );
   self.droneRevealR:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );
   self.droneShadow:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );

	self.droneRevealL:setVisible(true);
	self.droneRevealR:setVisible(true);
	self.droneShadow:setVisible(true);

end

------------------------------------------------------------------------------------

function StateDroneEvolve:isUsingCameraHack()
   return false;
end


------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------


function StateDroneEvolve:goBack()
	StateMgr:pop();
end


------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------


return StateDroneEvolve;


