----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');
require('ui/ui-snap-list');

----------------------------------------------------------------

local SWIPE_MIN_X					= -App.viewportHeight/2;
local SWIPE_MAX_X					= App.viewportHeight/2;
local SWIPE_MIN_Y					= -App.viewportWidth/2;
local SWIPE_MAX_Y					= App.viewportWidth/2;
local SWIPE_MIN_DELTA_X			= 75;
local SWIPE_MIN_DELTA_Y			= 75;
local SQUARE_TEXTURE				= "resources/square.png";
local CROSS_TEXTURE				= "resources/UI/Drone_UI/addbutton.png";
local COMING_SOON_TEXTURE		= "resources/UI/Drone_UI/comingsoonicon.png";
local SILHOUETTE_TEXTURE		= "resources/UI/Drone_UI/sill.png";
local LOCK_TEXTURE				= "resources/UI/Drone_UI/lockicon.png";
local DRONE_SPACING				= 800;
local DRONE_WIDTH					= 192;
local DRONE_HEIGHT				= 192;
local DRONE_LEFT_X				= -145;
local DRONE_RIGHT_X				= 145;
local DRONE_Y						= 15;

local STATE_MOUNT_LOCKED		= "mount-locked";
local STATE_SPACE_EMPTY			= "space-empty";
local STATE_SPACE_READY			= "space-ready";
local STATE_SPACE_EVOLVE		= "space-evolve";
local STATE_SPACE_MAXED			= "space-maxed";
local STATE_BUILD_ONLY			= "build-only";
local STATE_COMING_SOON			= "coming-soon";

local LEFT							= "left";
local RIGHT							= "right";
local CURRENCY_GEM				= "gem";
local CURRENCY_GOLD				= "gold";

local VEHC_DIMENSIONS			= { w=160*1.1, h=640*1.1 };

local PULSE_TIME					= 24/30;

local PRIORITY_0					= 10;
local PRIORITY_1					= 20;
local PRIORITY_2					= 30;
local PRIORITY_3					= 40;
local PRIORITY_4					= 50;
local PRIORITY_5					= 60;
local PRIORITY_6					= 70;
local PRIORITY_7					= 80;
local PRIORITY_8					= 90;
local PRIORITY_9					= 100;
local PRIORITY_10					= 110;
local PRIORITY_11					= 120;
local PRIORITY_12					= 130;
local PRIORITY_13					= 140;
local PRIORITY_14					= 150;
local PRIORITY_15					= 160;
local PRIORITY_16					= 170;
local PRIORITY_17					= 180;
local PRIORITY_18					= 190;
local PRIORITY_19					= 200;

----------------------------------------------------------------

local DRONE_STARS					=
{
	"resources/UI/Drone_UI/1Star.png",
	"resources/UI/Drone_UI/2Star.png",
	"resources/UI/Drone_UI/3Star.png",
	"resources/UI/Drone_UI/4Star.png",
	"resources/UI/Drone_UI/5Star.png",
};

local DRONE_SILVER_STARS		=
{
	{x=-77.5,	y=(DRONE_HEIGHT*3+30)/2-127-18.5},
	{x=-39.5,	y=(DRONE_HEIGHT*3+30)/2-127+3.5},
	{x=0.5,		y=(DRONE_HEIGHT*3+30)/2-127+18},
	{x=40.5,		y=(DRONE_HEIGHT*3+30)/2-127+3.5},
	{x=79.5,		y=(DRONE_HEIGHT*3+30)/2-127-18.5},
};

local PLAYER_VEHC					=
{
	{
		"resources/Vehicle/Player_SportsCar/Sportscar_Blue_ArmorLevel01.png",
		"resources/Vehicle/Player_SportsCar/Sportscar_Black_ArmorLevel01.png",
		"resources/Vehicle/Player_SportsCar/Sportscar_Red_ArmorLevel01.png",
		"resources/Vehicle/Player_SportsCar/Sportscar_Silver_ArmorLevel01.png",
		"resources/Vehicle/Player_SportsCar/Sportscar_Pink_ArmorLevel01.png",
		"resources/Vehicle/Player_SportsCar/Sportscar_Yellow_ArmorLevel01.png",
	},
	{
		"resources/Vehicle/PlayerVehicle_Sandrail/Sandrail_Black_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_Sandrail/Sandrail_Blue_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_Sandrail/Sandrail_Red_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_Sandrail/Sandrail_Green_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_Sandrail/Sandrail_Pink_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_Sandrail/Sandrail_Teal_ArmorLevel01.png",
	},
	{
		"resources/Vehicle/PlayerVehicle_MissileCar/Missilecar_Red_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MissileCar/Missilecar_Black_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MissileCar/Missilecar_Blue_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MissileCar/Missilecar_Silver_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MissileCar/Missilecar_Purple_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MissileCar/Missilecar_Orange_ArmorLevel01.png",
	},
	{
		"resources/Vehicle/PlayerVehicle_LazerCar/LazerCar_Yellow.png",
		"resources/Vehicle/PlayerVehicle_LazerCar/LazerCar_Black.png",
		"resources/Vehicle/PlayerVehicle_LazerCar/LazerCar_Blue.png",
		"resources/Vehicle/PlayerVehicle_LazerCar/LazerCar_Green.png",
		"resources/Vehicle/PlayerVehicle_LazerCar/LazerCar_Pink.png",
		"resources/Vehicle/PlayerVehicle_LazerCar/LazerCar_Red.png",
	},
	{
		"resources/Vehicle/PlayerVehicle_BlastCar/BlastCar_Silver.png",
		"resources/Vehicle/PlayerVehicle_BlastCar/BlastCar_Black.png",
		"resources/Vehicle/PlayerVehicle_BlastCar/BlastCar_Red.png",
		"resources/Vehicle/PlayerVehicle_BlastCar/BlastCar_Green.png",
		"resources/Vehicle/PlayerVehicle_BlastCar/BlastCar_Blue.png",
		"resources/Vehicle/PlayerVehicle_BlastCar/BlastCar_Orange.png",
	},
	{
		"resources/Vehicle/Player_SportsCar_V2/Sportscar_Base_ArmorLevel01.png",
		"resources/Vehicle/Player_SportsCar_V2/Sportscar_Black_ArmorLevel01.png",
		"resources/Vehicle/Player_SportsCar_V2/Sportscar_Blue_ArmorLevel01.png",
		"resources/Vehicle/Player_SportsCar_V2/Sportscar_Green_ArmorLevel01.png",
		"resources/Vehicle/Player_SportsCar_V2/Sportscar_Orange_ArmorLevel01.png",
		"resources/Vehicle/Player_SportsCar_V2/Sportscar_Yellow_ArmorLevel01.png",
	},
	{
		"resources/Vehicle/PlayerVehicle_MagnetCar_V2/MagnetCar_Base_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MagnetCar_V2/MagnetCar_Black_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MagnetCar_V2/MagnetCar_Red_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MagnetCar_V2/MagnetCar_Yellow_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MagnetCar_V2/MagnetCar_Purple_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MagnetCar_V2/MagnetCar_Teal_ArmorLevel01.png",
	},
	{
		"resources/Vehicle/PlayerVehicle_MissileCar_V2/MissilecarV2_Blue_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MissileCar_V2/MissilecarV2_Black_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MissileCar_V2/MissilecarV2_Red_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MissileCar_V2/MissilecarV2_Silver_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MissileCar_V2/MissilecarV2_Green_ArmorLevel01.png",
		"resources/Vehicle/PlayerVehicle_MissileCar_V2/MissilecarV2_Yellow_ArmorLevel01.png",
	},
	{
		"resources/Vehicle/PlayerVehicle_LazerCarV2/LazerCarV2_Silver.png",
		"resources/Vehicle/PlayerVehicle_LazerCarV2/LazerCarV2_Black.png",
		"resources/Vehicle/PlayerVehicle_LazerCarV2/LazerCarV2_Blue.png",
		"resources/Vehicle/PlayerVehicle_LazerCarV2/LazerCarV2_Orange.png",
		"resources/Vehicle/PlayerVehicle_LazerCarV2/LazerCarV2_Yellow.png",
		"resources/Vehicle/PlayerVehicle_LazerCarV2/LazerCarV2_Red.png",
	},
	{
		"resources/Vehicle/PlayerVehicle_BlastCarV2/BlastCarV2_Orange.png",
		"resources/Vehicle/PlayerVehicle_BlastCarV2/BlastCarV2_Black.png",
		"resources/Vehicle/PlayerVehicle_BlastCarV2/BlastCarV2_Red.png",
		"resources/Vehicle/PlayerVehicle_BlastCarV2/BlastCarV2_Green.png",
		"resources/Vehicle/PlayerVehicle_BlastCarV2/BlastCarV2_Blue.png",
		"resources/Vehicle/PlayerVehicle_BlastCarV2/BlastCarV2_Silver.png",
	},
};

----------------------------------------------------------------

local Base							= State;
local StateDroneGarage			= Base:new();

local textureSheet 				= FlashMgr:getTextureSheet("resources/flash-resource-bundle/MainScreen.lua", "resources/flash-resource-bundle/MainScreen.png", 30);

----------------------------------------------------------------

function StateDroneGarage:onLoad()

   State.onLoad( self );

	self.defaultFontName = "resources/font/TangoLatin.otf";

	-- title, background and frames
--	self.Title = self:addWidget(Anim, self.mainLayer, 0,430,0,0, "TitleGarage", textureSheet, nil, nil, true);
	self.Title = self:addWidget(TextBox, self.mainLayer, 0, 430, 640, 300, "DRONE", self.defaultFontName, 28, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.Title:setPriority(PRIORITY_13);
	self.TitleShdw		= self:addWidget(TextBox, self.mainLayer, 0+3, 430-3, 640, 300, "DRONE", self.defaultFontName, 28, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.TitleShdw:setPriority(PRIORITY_13-1);
	self.BackDrop		= self:addWidget(Image, self.mainLayer, 0,0, 640, 800, "resources/UI/Drone_UI/dronegbg.png");
	self.BlackFilter	= self:addWidget(Image, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE);
	self.BlackFilter:setPriority(PRIORITY_0-6);
	self.BlackFilter:setColor(0,0,0, 0.65);
	self.FrameOverlay = self:addWidget(Anim, self.mainLayer, 0,0,0,0, "GarageScrollFrame", textureSheet);
	self.FrameOverlay:setPriority(PRIORITY_12);
	self.PlayerVehcL	= self:addWidget(Image, self.mainLayer, -VEHC_DIMENSIONS.w/2, 0, VEHC_DIMENSIONS.w, VEHC_DIMENSIONS.h, PLAYER_VEHC[1][1]);
	self.PlayerVehcL:setPriority(PRIORITY_0-7);
	self.PlayerVehcR	= self:addWidget(Image, self.mainLayer, VEHC_DIMENSIONS.w/2, 0, VEHC_DIMENSIONS.w, VEHC_DIMENSIONS.h, PLAYER_VEHC[1][1]);
	self.PlayerVehcR:setPriority(PRIORITY_0-7);
	self.PlayerVehcR:setScale(-1,1);

	-- guide fullscreen button
	self.ScreenButton	= self:addWidget(ButtonImage, self.mainLayer, 0,0, App.viewportWidth*2, App.viewportHeight*2, SQUARE_TEXTURE, nil, nil, nil, nil);
	self.ScreenButton:setColor(1,1,1,0);

	-- codex button
	self.InfoButton	= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, -180, -425, 0,0, "UpGradeLockedButton", textureSheet, nil, self.goCodex, nil, nil);
	self.InfoButton:setPriority(PRIORITY_13);
	self.InfoFace		= self:addWidget(Image, self.mainLayer, -5, 5, 166, 64, "resources/UI/Drone_UI/drone_codex-button_face.png");
	self.InfoFace:setPriority(PRIORITY_13+1);
	self.InfoFace:setParent(self.InfoButton.anim.props[1]);
	self.InfoButton:setScale(1.18, 1.16);

	-- go back
	self.BackButton = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "ReadyButton", textureSheet, nil, self.goBack, nil, nil);
	self.BackButton:setPriority(PRIORITY_13);

	-- resource bar
   self.resourceBar = ResourceBar:new( self, 0, self:_calcHeaderLocY(), self.mainLayer, self.mainPartition  );

   -- unlock anim
   local droneSheet  = FlashMgr:getTextureSheet("resources/flash-resource-bundle/Drone_Effects.lua", "resources/flash-resource-bundle/Drone_Effects.png", 30);

   self._unlockAnim  = self:addWidget(Anim, self.mainLayer, 0,0,0,0, "Dronebay_Unlocking", droneSheet);
   self._blackFilter = self:addWidget(Image, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE, {0,0,0, 0.75});

   self._blackFilter:setPriority(PRIORITY_19);
   self._unlockAnim:setPriority(PRIORITY_19+1);
   self._unlockAnim:setVisible(false);
   self._blackFilter:setVisible(false);

	-- drone left/right
	self._droneLeft	= self:addWidget(		SnapList,
														self.mainLayer,
														DRONE_LEFT_X, DRONE_Y,
														{ {-320, 0},	{-360, 360} },
														nil,
														{ {SWIPE_MIN_X, SWIPE_MAX_X},	{SWIPE_MIN_Y, SWIPE_MAX_Y} },
														{0, DRONE_SPACING},
														{0, 1},
														{SWIPE_MIN_DELTA_X, SWIPE_MIN_DELTA_Y},
														nil,
														false,
														{ {-320, 0},	{-360, 360} } );

	self._droneRight	= self:addWidget(		SnapList,
														self.mainLayer,
														DRONE_RIGHT_X, DRONE_Y,
														{ {0, 320},	{-360, 360} },
														nil,
														{ {SWIPE_MIN_X, SWIPE_MAX_X},	{SWIPE_MIN_Y, SWIPE_MAX_Y} },
														{0, DRONE_SPACING},
														{0, 1},
														{SWIPE_MIN_DELTA_X, SWIPE_MIN_DELTA_Y},
														nil,
														false,
														{ {0, 320},	{-360, 360} } );
	self:_createDroneCache();

	--arrows
	self._droneArrowUpLeft		= self:addWidget(ButtonImage, self.mainLayer, DRONE_LEFT_X,		345,	96, 32, "resources/UI/Drone_UI/arrow.png",	nil, self._regressLeft);
	self._droneArrowUpRight		= self:addWidget(ButtonImage, self.mainLayer, DRONE_RIGHT_X,	345,	96, 32, "resources/UI/Drone_UI/arrow.png",	nil, self._regressRight);
	self._droneArrowDownLeft	= self:addWidget(ButtonImage, self.mainLayer, DRONE_LEFT_X,		-320,	96, 32, "resources/UI/Drone_UI/arrow.png",	nil, self._advanceLeft);
	self._droneArrowDownRight	= self:addWidget(ButtonImage, self.mainLayer, DRONE_RIGHT_X,	-320,	96, 32, "resources/UI/Drone_UI/arrow.png",	nil, self._advanceRight);
	self._arrowPulseTimer		= 0;

	self._droneArrowDownLeft:setScale(1, -1);
	self._droneArrowDownRight:setScale(1, -1);
	self._droneArrowUpLeft:setPriority(PRIORITY_0-5);
	self._droneArrowUpRight:setPriority(PRIORITY_0-5);
	self._droneArrowDownLeft:setPriority(PRIORITY_0-5);
	self._droneArrowDownRight:setPriority(PRIORITY_0-5);
	self._droneArrowUpLeft:setInputXtra(40, 5);
	self._droneArrowUpRight:setInputXtra(40, 5);
	self._droneArrowDownLeft:setInputXtra(40, 5);
	self._droneArrowDownRight:setInputXtra(40, 5);
	self._droneArrowUpLeft:setPressedScale(1.15, 1.15);
	self._droneArrowUpRight:setPressedScale(1.15, 1.15);
	self._droneArrowDownLeft:setBaseScale(1, -1);
	self._droneArrowDownRight:setBaseScale(1, -1);
	self._droneArrowDownLeft:setPressedScale(1.15, -1.15);
	self._droneArrowDownRight:setPressedScale(1.15, -1.15);

end

----------------------------------------------------------------

function StateDroneGarage:onActivate()

   Base.onActivate(self);

   if self.resourceBar then
      self.resourceBar:createConsumableButton();
   end

end

----------------------------------------------------------------

function StateDroneGarage:onFocus()

   Base.onFocus(self);

	self._objPulseTimer	= 0;
	self._objPulseList	= {};
	self._objRotTimer		= 0;
	self._objRotList		= {};

	self.ScreenButton:disable();
	self:_updatePlayerVehc();
	self:_initDroneList();
   self._unlockAnim:setVisible(false);
   self._blackFilter:setVisible(false);

end

----------------------------------------------------------------

function StateDroneGarage:onDataMerged()

	Base.onDataMerged(self);
   self:goBack();
   
end

----------------------------------------------------------------

function StateDroneGarage:onLoseFocus( )

   Base.onLoseFocus(self);

   self._unlockAnim:setVisible(false);
   self._blackFilter:setVisible(false);

end

----------------------------------------------------------------

function StateDroneGarage:onUpdate( dt )

	Base.onUpdate(self, dt);

	self:updateArrowPulse(dt);
	self:updatePulse(dt);
	self:updateRot(dt);

end


------------------------------------------------------------------------------------
-- Create Drone List
------------------------------------------------------------------------------------


function StateDroneGarage:_updatePlayerVehc()

	local vIndex	= App.data:getCurrentPlayerVehicle();
	local pIndex	= App.data:getCurrentPlayerVehiclePaintVariant();

	if vIndex and pIndex and PLAYER_VEHC[vIndex] and PLAYER_VEHC[vIndex][pIndex] then
		self.PlayerVehcL:swapImage(PLAYER_VEHC[vIndex][pIndex]);
		self.PlayerVehcR:swapImage(PLAYER_VEHC[vIndex][pIndex]);
		self.PlayerVehcR:setScale(-1,1);
	end

end


------------------------------------------------------------------------------------
-- Create Drone List
------------------------------------------------------------------------------------


function StateDroneGarage:_createDroneCache()

	local temp = {self._droneLeft, self._droneRight};

	for k,v in ipairs(temp) do
		v:setEntryObjectDataUpdateFunction(	function( data, object )

															self:_setCacheObjectVisible( object, false );
															self:_hideButton(object.omniButtonLG, object.omniIconsLG);
															self:_hideButton(object.omniButtonLB, object.omniIconsLB);
															self:_hidePips( object.pips, object.pipFrame );
															self:_hideStars(object.stars);

															object.root:setVisible(true);
															object.root:setPriority(PRIORITY_0);
															object.drone:swapImage(data.drone_texture);
															object.droneL:swapImage(data.drone_texture);
															object.droneR:swapImage(data.drone_texture);
															object.drone.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, object.root.prop, MOAIProp2D.TRANSFORM_TRAIT );
															object.droneL.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, object.root.prop, MOAIProp2D.TRANSFORM_TRAIT );
															object.droneR.prop:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, object.root.prop, MOAIProp2D.TRANSFORM_TRAIT );
															object.droneR:setScale(-1, 1);
															object.drone:setVisible(true);
															object.droneL:setVisible(false);
															object.droneR:setVisible(false);

															if data.state == STATE_MOUNT_LOCKED then
																object.titleText:setVisible(true);
																object.titleText:setText("UNLOCK");
																object.upgradeText:setVisible(true);
																object.upgradeText:setText("Unlock + Drone");

																local cost = DroneMgr:getDroneMountUnlockCost()

																if cost.cost == 0 then
																	object.buttonTxtLG:setText("FREE");
																	object.buttonTxtLB:setText("FREE");
																	object.buttonSTxtLG:setText("FREE");
																	object.buttonSTxtLB:setText("FREE");
																else
																	object.buttonTxtLG:setText(""..cost.cost);
																	object.buttonTxtLB:setText(""..cost.cost);
																	object.buttonSTxtLG:setText(""..cost.cost);
																	object.buttonSTxtLB:setText(""..cost.cost);
																end

																if data.side == LEFT then
																	object.omniButtonLG.onReleasedFunc	=	self._unlockMountLeft;
																	object.omniButtonLB.onReleasedFunc	=	self._unlockMountLeft;

																	if DroneMgr:canUnlockDroneMountLeft() then
																		self:_showButton(object.omniButtonLG);
																		object.buttonTxtLG:setVisible(true);
																		object.buttonSTxtLG:setVisible(true);
																		self:_showCurrency(object.omniIconsLG, cost.currency, cost.cost, object.buttonTxtLG, object.buttonSTxtLG);
																	else
																		self:_showButton(object.omniButtonLB);
																		object.buttonTxtLB:setVisible(true);
																		object.buttonSTxtLB:setVisible(true);
																		self:_showCurrency(object.omniIconsLB, cost.currency, cost.cost, object.buttonTxtLB, object.buttonSTxtLB);
																	end
																else
																	object.omniButtonLG.onReleasedFunc	=	self._unlockMountRight;
																	object.omniButtonLB.onReleasedFunc	=	self._unlockMountRight;

																	if DroneMgr:canUnlockDroneMountRight() then
																		self:_showButton(object.omniButtonLG);
																		object.buttonTxtLG:setVisible(true);
																		object.buttonSTxtLG:setVisible(true);
																		self:_showCurrency(object.omniIconsLG, cost.currency, cost.cost, object.buttonTxtLG, object.buttonSTxtLG);
																	else
																		self:_showButton(object.omniButtonLB);
																		object.buttonTxtLB:setVisible(true);
																		object.buttonSTxtLB:setVisible(true);
																		self:_showCurrency(object.omniIconsLB, cost.currency, cost.cost, object.buttonTxtLB, object.buttonSTxtLB);
																	end
																end
															elseif data.state == STATE_COMING_SOON then
																object.titleText:setVisible(true);
																object.titleText:setText("Coming\nSoon");
															elseif data.state == STATE_BUILD_ONLY then
																local cost;

																object.titleText:setVisible(true);
																object.titleText:setText("BUILD");

																object.omniButtonLG.onReleasedFunc	=	function()
																														self:_buildDrone(data.garage_index, data.side);
																													end
																object.omniButtonLB.onReleasedFunc	=	function()
																														self:_buildDrone(data.garage_index, data.side);
																													end

																if data.side == LEFT then
																	cost = DroneMgr:getBuildCostLeft(false);

																	if DroneMgr:canBuildDroneLeft(data.garage_index, false) then
																		self:_showButton(object.omniButtonLG);
																		object.buttonTxtLG:setVisible(true);
																		object.buttonSTxtLG:setVisible(true);
																		self:_showCurrency(object.omniIconsLG, cost.currency, cost.cost, object.buttonTxtLG, object.buttonSTxtLG);
																	else
																		self:_showButton(object.omniButtonLB);
																		object.buttonTxtLB:setVisible(true);
																		object.buttonSTxtLB:setVisible(true);
																		self:_showCurrency(object.omniIconsLB, cost.currency, cost.cost, object.buttonTxtLB, object.buttonSTxtLB);
																	end
																else
																	cost = DroneMgr:getBuildCostRight(false);

																	if DroneMgr:canBuildDroneRight(data.garage_index, false) then
																		self:_showButton(object.omniButtonLG);
																		object.buttonTxtLG:setVisible(true);
																		object.buttonSTxtLG:setVisible(true);
																		self:_showCurrency(object.omniIconsLG, cost.currency, cost.cost, object.buttonTxtLG, object.buttonSTxtLG);
																	else
																		self:_showButton(object.omniButtonLB);
																		object.buttonTxtLB:setVisible(true);
																		object.buttonSTxtLB:setVisible(true);
																		self:_showCurrency(object.omniIconsLB, cost.currency, cost.cost, object.buttonTxtLB, object.buttonSTxtLB);
																	end
																end

																if cost.cost == 0 then
																	object.buttonTxtLG:setText("FREE");
																	object.buttonTxtLB:setText("FREE");
																	object.buttonSTxtLG:setText("FREE");
																	object.buttonSTxtLB:setText("FREE");
																else
																	object.buttonTxtLG:setText(""..cost.cost);
																	object.buttonTxtLB:setText(""..cost.cost);
																	object.buttonSTxtLG:setText(""..cost.cost);
																	object.buttonSTxtLB:setText(""..cost.cost);
																end
															elseif data.state == STATE_SPACE_EMPTY then
																local cost;

																object.titleText:setVisible(true);
																object.titleText:setText("BUILD");

																object.omniButtonLG.onReleasedFunc	=	function()
																														self:_buildDrone(data.garage_index, data.side);
																													end
																object.omniButtonLB.onReleasedFunc	=	function()
																														self:_buildDrone(data.garage_index, data.side);
																													end

																if data.side == LEFT then
																	cost = DroneMgr:getBuildCostLeft(false);

																	if DroneMgr:canBuildDroneLeft(data.garage_index, false) then
																		self:_showButton(object.omniButtonLG);
																		object.buttonTxtLG:setVisible(true);
																		object.buttonSTxtLG:setVisible(true);
																		self:_showCurrency(object.omniIconsLG, cost.currency, cost.cost, object.buttonTxtLG, object.buttonSTxtLG);
																	else
																		self:_showButton(object.omniButtonLB);
																		object.buttonTxtLB:setVisible(true);
																		object.buttonSTxtLB:setVisible(true);
																		self:_showCurrency(object.omniIconsLB, cost.currency, cost.cost, object.buttonTxtLB, object.buttonSTxtLB);
																	end
																else
																	cost = DroneMgr:getBuildCostRight(false);

																	if DroneMgr:canBuildDroneRight(data.garage_index, false) then
																		self:_showButton(object.omniButtonLG);
																		object.buttonTxtLG:setVisible(true);
																		object.buttonSTxtLG:setVisible(true);
																		self:_showCurrency(object.omniIconsLG, cost.currency, cost.cost, object.buttonTxtLG, object.buttonSTxtLG);
																	else
																		self:_showButton(object.omniButtonLB);
																		object.buttonTxtLB:setVisible(true);
																		object.buttonSTxtLB:setVisible(true);
																		self:_showCurrency(object.omniIconsLB, cost.currency, cost.cost, object.buttonTxtLB, object.buttonSTxtLB);
																	end
																end

																if cost.cost == 0 then
																	object.buttonTxtLG:setText("FREE");
																	object.buttonTxtLB:setText("FREE");
																	object.buttonSTxtLG:setText("FREE");
																	object.buttonSTxtLB:setText("FREE");
																else
																	object.buttonTxtLG:setText(""..cost.cost);
																	object.buttonTxtLB:setText(""..cost.cost);
																	object.buttonSTxtLG:setText(""..cost.cost);
																	object.buttonSTxtLB:setText(""..cost.cost);
																end
															else
																object.titleText:setVisible(true);
																object.titleText:setText(data.name);
																object.drone:setVisible(false);
																object.droneL:setVisible(true);
																object.droneR:setVisible(true);

																self:_showPips( object.pips, object.pipFrame, data.level, data.rarity );

																if			data.rarity < DroneMgr:getDroneMaxRarity()
																	and	not DroneMgr:isDroneDupeResolveModeAutoMerge() then
																	object.pipFrame:setLoc(-24, -(DRONE_HEIGHT*3+30)/2+205);
																	object.pipFrame:setScale(0.735, 0.735);
																else
																	object.pipFrame:setLoc(0, -(DRONE_HEIGHT*3+30)/2+205);
																	object.pipFrame:setScale(1, 1);
																end

																self:_showStars( object.stars, data.rarity );

																object.infoButton:setVisible(true);
																object.infoButton:enable();

																object.infoButton.onReleasedFunc	=	function()
																													StateMgr:push( "state/state-drone-info-popup.lua", {	texture = data.drone_texture, rarity = data.rarity, level = data.level, name = data.name, desc = data.desc  } );
																												end

																object.upgradeText:setVisible(true);

																if data.side == LEFT then
																	object.omniButtonLG.onReleasedFunc	=	function()
																															self:_upgradeDroneLeft(data.garage_index);
																														end

																	object.omniButtonLB.onReleasedFunc	=	function()
																															self:_upgradeDroneLeft(data.garage_index);
																														end
																else
																	object.omniButtonLG.onReleasedFunc	=	function()
																															self:_upgradeDroneRight(data.garage_index);
																														end

																	object.omniButtonLB.onReleasedFunc	=	function()
																															self:_upgradeDroneRight(data.garage_index);
																														end
																end

																local cost;

																if data.state == STATE_SPACE_READY then
																	if data.side == LEFT then

																		cost	= DroneMgr:getUpgradeCostLeft(data.garage_index);

																		if DroneMgr:canUpgradeDroneLeft(data.garage_index) then
																			self:_showButton(object.omniButtonLG);
																			object.buttonTxtLG:setVisible(true);
																			object.buttonSTxtLG:setVisible(true);
																			self:_showCurrency(object.omniIconsLG, cost.currency, cost.cost, object.buttonTxtLG, object.buttonSTxtLG);
																		else
																			self:_showButton(object.omniButtonLB);
																			object.buttonTxtLB:setVisible(true);
																			object.buttonSTxtLB:setVisible(true);
																			self:_showCurrency(object.omniIconsLB, cost.currency, cost.cost, object.buttonTxtLB, object.buttonSTxtLB);
																		end
																	else

																		cost	= DroneMgr:getUpgradeCostRight(data.garage_index);

																		if DroneMgr:canUpgradeDroneRight(data.garage_index) then
																			self:_showButton(object.omniButtonLG);
																			object.buttonTxtLG:setVisible(true);
																			object.buttonSTxtLG:setVisible(true);
																			self:_showCurrency(object.omniIconsLG, cost.currency, cost.cost, object.buttonTxtLG, object.buttonSTxtLG);
																		else
																			self:_showButton(object.omniButtonLB);
																			object.buttonTxtLB:setVisible(true);
																			object.buttonSTxtLB:setVisible(true);
																			self:_showCurrency(object.omniIconsLB, cost.currency, cost.cost, object.buttonTxtLB, object.buttonSTxtLB);
																		end
																	end

																	object.upgradeText:setText("UPGRADE");

																	if cost.cost == 0 then
																		object.buttonTxtLG:setText("FREE");
																		object.buttonTxtLB:setText("FREE");
																		object.buttonSTxtLG:setText("FREE");
																		object.buttonSTxtLB:setText("FREE");
																	else
																		object.buttonTxtLG:setText(""..cost.cost);
																		object.buttonTxtLB:setText(""..cost.cost);
																		object.buttonSTxtLG:setText(""..cost.cost);
																		object.buttonSTxtLB:setText(""..cost.cost);
																	end
																elseif data.state == STATE_SPACE_EVOLVE then

																	object.omniButtonLG.onReleasedFunc	=	function()
																															self:_evolveDrone(data.garage_index, data.side);
																														end

																	object.omniButtonLB.onReleasedFunc	=	function()
																															self:_evolveDrone(data.garage_index, data.side);
																														end

																	if data.side == LEFT then
																		cost	= DroneMgr:getEvolveCostLeft(data.garage_index);

																		if DroneMgr:canEvolveDroneLeft(data.garage_index) then
																			self:_showButton(object.omniButtonLG);
																			object.buttonTxtLG:setVisible(true);
																			object.buttonSTxtLG:setVisible(true);
																			self:_showCurrency(object.omniIconsLG, cost.currency, cost.cost, object.buttonTxtLG, object.buttonSTxtLG);
																			self:_addToPulseList(object.omniButtonLG, 1.1);

																			if self.droneEvolveButton == nil then
																				self.droneEvolveButton				= object.omniButtonLG;
																				self.droneEvolveButtonTxt			= object.buttonTxtLG;
																				self.droneEvolveButtonTxtShdow	= object.buttonSTxtLG;
																				self.droneEvolveIcon					= object.omniIconsLG.gem;
																			end
																		else
																			self:_showButton(object.omniButtonLB);
																			object.buttonTxtLB:setVisible(true);
																			object.buttonSTxtLB:setVisible(true);
																			self:_showCurrency(object.omniIconsLB, cost.currency, cost.cost, object.buttonTxtLB, object.buttonSTxtLB);
																			self:_addToPulseList(object.omniButtonLB, 1.1);

																			if self.droneEvolveButton == nil then
																				self.droneEvolveButton				= object.omniButtonLB;
																				self.droneEvolveButtonTxt			= object.buttonTxtLB;
																				self.droneEvolveButtonTxtShdow	= object.buttonSTxtLB;
																				self.droneEvolveIcon					= object.omniIconsLB.gem;
																			end
																		end
																	else
																		cost	= DroneMgr:getEvolveCostRight(data.garage_index);

																		if DroneMgr:canEvolveDroneRight(data.garage_index) then
																			self:_showButton(object.omniButtonLG);
																			object.buttonTxtLG:setVisible(true);
																			object.buttonSTxtLG:setVisible(true);
																			self:_showCurrency(object.omniIconsLG, cost.currency, cost.cost, object.buttonTxtLG, object.buttonSTxtLG);
																			self:_addToPulseList(object.omniButtonLG, 1.1);

																			if self.droneEvolveButton == nil then
																				self.droneEvolveButton				= object.omniButtonLG;
																				self.droneEvolveButtonTxt			= object.buttonTxtLG;
																				self.droneEvolveButtonTxtShdow	= object.buttonSTxtLG;
																				self.droneEvolveIcon					= object.omniIconsLG.gem;
																			end
																		else
																			self:_showButton(object.omniButtonLB);
																			object.buttonTxtLB:setVisible(true);
																			object.buttonSTxtLB:setVisible(true);
																			self:_showCurrency(object.omniIconsLB, cost.currency, cost.cost, object.buttonTxtLB, object.buttonSTxtLB);
																			self:_addToPulseList(object.omniButtonLB, 1.1);

																			if self.droneEvolveButton == nil then
																				self.droneEvolveButton				= object.omniButtonLB;
																				self.droneEvolveButtonTxt			= object.buttonTxtLB;
																				self.droneEvolveButtonTxtShdow	= object.buttonSTxtLB;
																				self.droneEvolveIcon					= object.omniIconsLB.gem;
																			end
																		end
																	end

																	object.upgradeText:setText("EVOLVE");

																	if cost.cost == 0 then
																		object.buttonTxtLG:setText("FREE");
																		object.buttonTxtLB:setText("FREE");
																		object.buttonSTxtLG:setText("FREE");
																		object.buttonSTxtLB:setText("FREE");
																	else
																		object.buttonTxtLG:setText(""..cost.cost);
																		object.buttonTxtLB:setText(""..cost.cost);
																		object.buttonSTxtLG:setText(""..cost.cost);
																		object.buttonSTxtLB:setText(""..cost.cost);
																	end

																	self:_addToPulseList(object.pips.starOn[1], 1.2);
																	self:_addToPulseList(object.pips.starText[1], 1.2);
																	self:_addToPulseList(object.pips.starTextShdw[1], 1.2);
																	self:_addToPulseList(object.stars.slv[data.rarity+1], 1.2);
																	self:_addToRotList(object.gRays);

																	if self.droneEvolveFrame == nil then
																		self.droneEvolveSide			= data.side;
																		self.droneEvolveFrame		= object.root;
																		self.droneEvolveUpgradeTxt	= object.upgradeText;
																		self.droneEvolveTitleTxt	= object.titleText;
																		self.droneEvolveStars		= object.stars.reg[data.rarity];
																		self.droneEvolveSStars		= object.stars.slv[data.rarity+1];
																		self.droneEvolveDroneL		= object.droneL;
																		self.droneEvolveDroneR		= object.droneR;
																		self.droneEvolveGRays		= object.gRays;
																		self.droneEvolvePipFrame	= object.pipFrame;
																		self.droneEvolvePStarOn		= object.pips.starOn[1];
																		self.droneEvolvePStarTxt	= object.pips.starText[1];
																		self.droneEvolvePStarTxtS	= object.pips.starTextShdw[1];

																		for i=1,15 do
																			self["droneEvolvePipOn"..i]	= object.pips.on[i];
																		end
																	end
																else
																	object.upgradeText:setVisible(false);
																	object.levelText:setVisible(true);
																	object.levelText:setText("Max Level");

																	if data.rarity < DroneMgr:getDroneMaxRarity() then

																		self:_addToPulseList(object.stars.slv[data.rarity+1], 1.2);
																		self:_addToRotList(object.gRays);

																		if self.droneEvolveFrame == nil then

																			self.droneEvolveSide			= data.side;
																			self.droneEvolveFrame		= object.root;
																			self.droneEvolveUpgradeTxt	= object.upgradeText;
																			self.droneEvolveTitleTxt	= object.titleText;
																			self.droneEvolveStars		= object.stars.reg[data.rarity];
																			self.droneEvolveSStars		= object.stars.slv[data.rarity+1];
																			self.droneEvolveDroneL		= object.droneL;
																			self.droneEvolveDroneR		= object.droneR;
																			self.droneEvolveGRays		= object.gRays;
																			self.droneEvolveLevelTxt	= object.levelText;
																			self.droneEvolvePipFrame	= object.pipFrame;
																			--self.droneEvolvePStarOn		= object.pips.starOn[1];
																			--self.droneEvolvePStarTxt	= object.pips.starText[1];
																			--self.droneEvolvePStarTxtS	= object.pips.starTextShdw[1];

																			for i=1,15 do
																				self["droneEvolvePipOn"..i]	= object.pips.on[i];
																			end
																		end
																	end
																end
															end
														end
														);

		v:setEntryObjectLocUpdateFunction(	function( x, y, object )
															object.root:setLoc(x,y);
														end
														);

		v:setEntryObjectParentUpdateFunction(	function( target, object )
																object.root:setParent(target);
															end
															);

		v:setEntryCacheObjects(		0,
											5,
											function()
												local tempTable = {};

												tempTable.root				= self:addWidget(Image, self.mainLayer, 0,0, DRONE_WIDTH+64, DRONE_HEIGHT*3+30, "resources/UI/Drone_UI/newframe.png", {1,1,1, 0.95});
												tempTable.upgradeText	= self:addWidget(TextBox, self.mainLayer, 0, -(DRONE_HEIGHT*3+30)/2+150, DRONE_WIDTH+48, 100, "UPGRADE", self.defaultFontName, 12, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
												tempTable.titleText		= self:addWidget(TextBox, self.mainLayer, 0, (DRONE_HEIGHT*3+30)/2-140, DRONE_WIDTH+48, 200, "TITLE", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.LEFT_JUSTIFY);
												tempTable.levelText		= self:addWidget(TextBox, self.mainLayer, 0, -(DRONE_HEIGHT*3+30)/2+80, DRONE_WIDTH+48, 100, "TIMER", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
												tempTable.pipFrame		= self:addWidget(Image, self.mainLayer, 0, -(DRONE_HEIGHT*3+30)/2+205, DRONE_WIDTH+8, 24, "resources/UI/Drone_UI/pipbar.png");
												tempTable.pips				= self:_createDronePips( tempTable.pipFrame.prop );
												tempTable.stars			= self:_createDroneStars( tempTable.root.prop );
												tempTable.drone			= self:addWidget(Image, self.mainLayer, 0, 10, DRONE_WIDTH, DRONE_HEIGHT, COMING_SOON_TEXTURE);
												tempTable.droneL			= self:addWidget(Image, self.mainLayer, -DRONE_WIDTH/4, 10, DRONE_WIDTH/2, DRONE_HEIGHT, COMING_SOON_TEXTURE);
												tempTable.droneR			= self:addWidget(Image, self.mainLayer, DRONE_WIDTH/4, 10, DRONE_WIDTH/2, DRONE_HEIGHT, COMING_SOON_TEXTURE);
												tempTable.gRays			= self:addWidget(Image, self.mainLayer, 0, 10, DRONE_HEIGHT*2, DRONE_HEIGHT*2, "resources/UI/Drone_UI/LightRays-01.png");

												tempTable.infoButton		= self:addWidget(ButtonImage, self.mainLayer, DRONE_WIDTH/2-20, DRONE_HEIGHT/2-4, 40, 40, "resources/UI/info_button.png", nil, nil, nil, nil, true);
												tempTable.sparkle			= self:addWidget(Anim, self.mainLayer, DRONE_WIDTH/2-25, DRONE_HEIGHT/2-4, 0,0, "NotificationSparkle", textureSheet);

												tempTable.omniButtonLG	= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0, -(DRONE_HEIGHT*3+30)/2+75, 0,0, "UpGradeButton", textureSheet, nil, nil, nil, nil);
												tempTable.omniButtonLB	= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0, -(DRONE_HEIGHT*3+30)/2+75, 0,0, "UpGradeLockedButton", textureSheet, nil, nil, nil, nil);
												tempTable.omniIconsLG	= self:_createCurrencyIcon( tempTable.omniButtonLG.anim.props[1] );
												tempTable.omniIconsLB	= self:_createCurrencyIcon( tempTable.omniButtonLB.anim.props[1] );
												tempTable.buttonTxtLG	= self:addWidget(TextBox, self.mainLayer, -5, 5, 185, 100, "L TEXT", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
												tempTable.buttonTxtLB	= self:addWidget(TextBox, self.mainLayer, -5, 5, 185, 100, "L TEXT", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
												tempTable.buttonSTxtLG	= self:addWidget(TextBox, self.mainLayer, -3, 3, 185, 100, "L TEXT", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
												tempTable.buttonSTxtLB	= self:addWidget(TextBox, self.mainLayer, -3, 3, 185, 100, "L TEXT", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});

												tempTable.drone:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		tempTable.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
												tempTable.droneL:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		tempTable.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
												tempTable.droneR:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		tempTable.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
												tempTable.gRays:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		tempTable.root.prop,	MOAIProp2D.TRANSFORM_TRAIT	);
												tempTable.upgradeText:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		tempTable.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
												tempTable.titleText:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		tempTable.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
												tempTable.levelText:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		tempTable.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
												tempTable.pipFrame:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		tempTable.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
												tempTable.infoButton:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		tempTable.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
												tempTable.sparkle:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		tempTable.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
												tempTable.omniButtonLG:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		tempTable.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
												tempTable.omniButtonLB:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		tempTable.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
												tempTable.buttonTxtLG:setParent(		tempTable.omniButtonLG.anim.props[1]	);
												tempTable.buttonTxtLB:setParent(		tempTable.omniButtonLB.anim.props[1]	);
												tempTable.buttonSTxtLG:setParent(	tempTable.omniButtonLG.anim.props[1]	);
												tempTable.buttonSTxtLB:setParent(	tempTable.omniButtonLB.anim.props[1]	);

												tempTable.root:setPriority(			PRIORITY_0	);
												tempTable.upgradeText:setPriority(	PRIORITY_2	);
												tempTable.titleText:setPriority(		PRIORITY_2	);
												tempTable.levelText:setPriority(		PRIORITY_2	);
												tempTable.pipFrame:setPriority(		PRIORITY_2	);
												tempTable.drone:setPriority(			PRIORITY_2	);
												tempTable.droneL:setPriority(			PRIORITY_2	);
												tempTable.droneR:setPriority(			PRIORITY_2	);
												tempTable.gRays:setPriority(			PRIORITY_2-1);
												tempTable.infoButton:setPriority(	PRIORITY_3	);
												tempTable.sparkle:setPriority(		PRIORITY_3-1);
												tempTable.omniButtonLG:setPriority(	PRIORITY_2	);
												tempTable.omniButtonLB:setPriority(	PRIORITY_2	);
												tempTable.buttonTxtLG:setPriority(	PRIORITY_4	);
												tempTable.buttonTxtLB:setPriority(	PRIORITY_4	);
												tempTable.buttonSTxtLG:setPriority(	PRIORITY_3	);
												tempTable.buttonSTxtLB:setPriority(	PRIORITY_3	);

												tempTable.omniButtonLG:disable()
												tempTable.omniButtonLB:disable()

												--tempTable.omniButtonLG:setScale(1, 0.9);
												--tempTable.omniButtonLB:setScale(1, 0.9);
												--tempTable.sparkle:setScale(0.9, 0.9);

												tempTable.infoButton:setInputXtra(64, 64);
												tempTable.infoButton:setPressedScale(1.20, 1.20);
												tempTable.infoButton:disable();

												tempTable.sparkle:setSpan(29/30);
												tempTable.sparkle:setMode(MOAITimer.LOOP);

												self:_hidePips(tempTable.pips, tempTable.pipFrame);
												self:_hideStars(tempTable.stars);

												tempTable.omniButtonLG:setInViewTest(	function()
																										return v:isEntryCacheObjInView(tempTable.omniButtonLG.anim.root, true);
																									end	);
												tempTable.omniButtonLB:setInViewTest(	function()
																										return v:isEntryCacheObjInView(tempTable.omniButtonLB.anim.root, true);
																									end	);
												tempTable.infoButton:setInViewTest(		function()
																										return v:isEntryCacheObjInView(tempTable.infoButton.prop, true);
																									end	);

												return tempTable;
											end
										);
	end

	self._droneLeft:setGoToEntryYStartCallback(	function( index )
																	if self._droneLeftData and self._droneLeftData[index] then
																		DroneMgr:setDroneMountLeft(index);
																		self:_updateDroneArrowLeft();
																		EventMgr:postEvent( {sounds={start={"SwipeIn"}}, name="swipe move y",}, "add-sfx" );
																	end
																end
																);

	self._droneRight:setGoToEntryYStartCallback(	function( index )
																	if self._droneRightData and self._droneRightData[index] then
																		DroneMgr:setDroneMountRight(index);
																		self:_updateDroneArrowRight();
																		EventMgr:postEvent( {sounds={start={"SwipeIn"}}, name="swipe move y",}, "add-sfx" );
																	end
																end
																);

   local leftECO = self._droneLeft:getEntryCacheObject(1);

   if leftECO and leftECO.omniButtonLG then
      --joe ref for guide
      self.leftBuildButtonRef       = leftECO.omniButtonLG;
      self.leftBuildButtonTextRef   = leftECO.buttonTxtLG;
      self.leftBuildButtonTextSRef  = leftECO.buttonSTxtLG;
      self.leftBuildButtonGemIcon   = leftECO.omniIconsLG.gem;
      self.leftStars                = leftECO.stars;
      self.leftPips                 = leftECO.pips;
      self.leftDrone                = leftECO.drone;
      self.leftPipFrame             = leftECO.pipFrame;
      self.leftTitleText            = leftECO.titleText;
      self.leftRoot                 = leftECO.root;
      self.leftInfoButton           = leftECO.infoButton;
		self.leftSparkle					= leftECO.sparkle;
   end

   local unlockObjectDroneBay = self._droneRight:getEntryCacheObject(1);

   if unlockObjectDroneBay and unlockObjectDroneBay.drone and self._unlockAnim then
      self._rightDroneBayLock = unlockObjectDroneBay.drone;

      if self._unlockAnim then
         self._unlockAnim:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._rightDroneBayLock.prop, MOAIProp2D.TRANSFORM_TRAIT );
      end
   end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_createDroneData()

	local countL		= math.max(DroneMgr:getDroneCountLeft(), 1);
	local countR		= math.max(DroneMgr:getDroneCountRight(), 1);
	local maxRarity	= DroneMgr:getDroneMaxRarity();

	if countL > DroneMgr:getDroneTypeCount() then
		countL = DroneMgr:getDroneTypeCount();
	end

	if countR > DroneMgr:getDroneTypeCount() then
		countR = DroneMgr:getDroneTypeCount();
	end

	self.droneEvolveFrame	= nil;
	self._droneLeftData		= {};
	self._droneRightData		= {};

	for i=1,countL do
		local temp = {};

		temp.garage_index		= i;
		temp.name				= "";
		temp.drone_texture	= LOCK_TEXTURE;
		temp.state				= STATE_MOUNT_LOCKED;
		temp.side				= LEFT;
		temp.rarity				= 1;
		temp.level				= 1;

		if DroneMgr:isDroneMountLeftLocked() then
			temp.state				= STATE_MOUNT_LOCKED;
			table.insert(self._droneLeftData, temp);

			break;
		elseif DroneMgr:isDroneGarageSpaceEmptyLeft(i) then
			temp.state				= STATE_SPACE_EMPTY;
			temp.drone_texture	= SILHOUETTE_TEXTURE;
		else
			temp.drone_texture	= DroneMgr:getDroneBodyVisLeft(i);
			temp.name				= DroneMgr:getDroneNameLeft(i);
			temp.rarity				= DroneMgr:getDroneRarityLeft(i);
			temp.level				= DroneMgr:getDroneLevelLeft(i);
			temp.desc				= DroneMgr:getDroneDescLeft(i);

			if DroneMgr:isDroneUpgradeMaxedLeft(i) then
				if temp.rarity < maxRarity and not DroneMgr:isDroneDupeResolveModeAutoMerge() then
					temp.state		= STATE_SPACE_EVOLVE;
				else
					temp.state		= STATE_SPACE_MAXED;
				end
			else
				temp.state			= STATE_SPACE_READY;
			end
		end

		table.insert(self._droneLeftData, temp);
	end

	if			countL <= DroneMgr:getDroneTypeCount()
		and	DroneMgr:isDroneGarageSpaceFilledLeft(countL)
		and	not DroneMgr:isDroneMountLeftLocked() then

		local temp = {};

		temp.garage_index		= countL + 1;
		temp.name				= "";
		temp.side				= LEFT;
		temp.state				= STATE_SPACE_EMPTY;
		temp.drone_texture	= SILHOUETTE_TEXTURE;

		if temp.garage_index > DroneMgr:getDroneTypeCount() then
			temp.garage_index		= self:_findEmptyBay( countL, temp.side ) or temp.garage_index;
			temp.state				= STATE_BUILD_ONLY;
			temp.drone_texture	= SILHOUETTE_TEXTURE;
		end

		if DroneMgr:isDroneCollectionCompleteLeft() then
			temp.state				= STATE_COMING_SOON;
			temp.drone_texture	= COMING_SOON_TEXTURE;
		end

		table.insert(self._droneLeftData, temp);
	end

	for i=1,countR do
		local temp = {};

		temp.garage_index		= i;
		temp.name				= "";
		temp.drone_texture	= LOCK_TEXTURE;
		temp.state				= STATE_MOUNT_LOCKED;
		temp.side				= RIGHT;
		temp.rarity				= 1;
		temp.level				= 1;

		if DroneMgr:isDroneMountRightLocked() then
			temp.state				= STATE_MOUNT_LOCKED;
			table.insert(self._droneRightData, temp);

			break;
		elseif DroneMgr:isDroneGarageSpaceEmptyRight(i) then
			temp.state				= STATE_SPACE_EMPTY;
			temp.drone_texture	= SILHOUETTE_TEXTURE;
		else
			temp.drone_texture	= DroneMgr:getDroneBodyVisRight(i);
			temp.name				= DroneMgr:getDroneNameRight(i);
			temp.rarity				= DroneMgr:getDroneRarityRight(i);
			temp.level				= DroneMgr:getDroneLevelRight(i);
			temp.desc				= DroneMgr:getDroneDescRight(i);

			if DroneMgr:isDroneUpgradeMaxedRight(i) then
				if temp.rarity < maxRarity and not DroneMgr:isDroneDupeResolveModeAutoMerge() then
					temp.state		= STATE_SPACE_EVOLVE;
				else
					temp.state		= STATE_SPACE_MAXED;
				end
			else
				temp.state			= STATE_SPACE_READY;
			end
		end

		table.insert(self._droneRightData, temp);
	end

	if			countR <= DroneMgr:getDroneTypeCount()
		and	DroneMgr:isDroneGarageSpaceFilledRight(countR)
		and	not DroneMgr:isDroneMountRightLocked() then

		local temp = {};

		temp.garage_index		= countR + 1;
		temp.name				= "";
		temp.side				= RIGHT;
		temp.state				= STATE_SPACE_EMPTY;
		temp.drone_texture	= SILHOUETTE_TEXTURE;
		temp.rarity				= 1;
		temp.level				= 1;

		if temp.garage_index > DroneMgr:getDroneTypeCount() then
			temp.garage_index		= self:_findEmptyBay( countR, temp.side ) or temp.garage_index;
			temp.state				= STATE_BUILD_ONLY;
			temp.drone_texture	= SILHOUETTE_TEXTURE;
		end

		if DroneMgr:isDroneCollectionCompleteRight() then
			temp.state				= STATE_COMING_SOON;
			temp.drone_texture	= COMING_SOON_TEXTURE;
		end

		table.insert(self._droneRightData, temp);
	end

--[[
	print("StateDroneGarage:_createDroneData - Left data")
	for k,v in ipairs(self._droneLeftData) do
		for key, val in pairs(v) do
			print(k, key, val)
		end
		print(k, "Type", DroneMgr:getDroneTypeLeft(k))
	end

	print("StateDroneGarage:_createDroneData - Right data")
	for k,v in ipairs(self._droneRightData) do
		for key, val in pairs(v) do
			print(k, key, val)
		end
		print(k, "Type", DroneMgr:getDroneTypeRight(k))
	end
]]

end

------------------------------------------------------------------------------------

function StateDroneGarage:_createDronePips( parent )

	if parent then
		local width		= 12;
		local height	= 16;
		local spacing	= math.ceil( DRONE_WIDTH / DroneMgr:getDroneMaxLevel() );
		local start		= -spacing * math.floor( DroneMgr:getDroneMaxLevel() / 2 );
		local offset	= start;
		local temp		= { on = {}, off = {}, starOn = {}, starOff = {}, starText = {}, starTextShdw = {}	};

		for i=1,DroneMgr:getDroneMaxLevel() do
			offset		= start + ( ( i - 1 ) * spacing);
			temp.on[i]	= self:addWidget(Image, self.mainLayer, offset, 0, width, height, "resources/UI/Drone_UI/pip.png");
			temp.off[i]	= self:addWidget(Image, self.mainLayer, offset, 0, width, height, "resources/UI/Drone_UI/pipdeselect.png");

			temp.on[i]:setParent(		parent	);
			temp.off[i]:setParent(		parent	);

			temp.on[i]:setPriority(		PRIORITY_3+1	);
			temp.off[i]:setPriority(	PRIORITY_3	);
		end

		temp.starOn[1]			= self:addWidget(Image, self.mainLayer, offset+40, 3, 64, 64, "resources/UI/Drone_UI/StarOn.png");
		temp.starOff[1]		= self:addWidget(Image, self.mainLayer, offset+40, 3, 64, 64, "resources/UI/Drone_UI/StarOff.png");
		temp.starText[1]		= self:addWidget(TextBox, self.mainLayer, offset+39, 1, 100, 100, "8", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
		temp.starTextShdw[1]	= self:addWidget(TextBox, self.mainLayer, offset+39+2, 1-2, 100, 100, "8", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});

		temp.starOn[1]:setParent(				parent	);
		temp.starOff[1]:setParent(				parent	);
		temp.starText[1]:setParent(			parent	);
		temp.starTextShdw[1]:setParent(		parent	);

		temp.starOn[1]:setPriority(			PRIORITY_3		);
		temp.starOff[1]:setPriority(			PRIORITY_3		);
		temp.starText[1]:setPriority(			PRIORITY_3+2	);
		temp.starTextShdw[1]:setPriority(	PRIORITY_3+1	);

		return temp;
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_createDroneStars( parent )

	if parent then
		local width		= 200;
		local height	= 100;
		local	swidth	= 40;
		local sheight	= 40;
		local temp		= { reg={}, slv={} };

		for i=1,DroneMgr:getDroneMaxRarity() do
			temp.reg[i]	= self:addWidget(Image, self.mainLayer, 0, (DRONE_HEIGHT*3+30)/2-127, width, height, DRONE_STARS[i]);
			temp.slv[i]	= self:addWidget(Image, self.mainLayer, DRONE_SILVER_STARS[i].x, DRONE_SILVER_STARS[i].y, swidth, sheight, "resources/UI/Drone_UI/Silver_Star.png");

			temp.reg[i]:setParent( parent );
			temp.slv[i]:setParent( parent );
			temp.reg[i]:setPriority( PRIORITY_3 );
			temp.slv[i]:setPriority( PRIORITY_3+1 );
		end

		return temp;
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_createCurrencyIcon( parent )

	if parent then
		local width		= 80;
		local height	= 80;
		local temp		= {};

		temp.gem			= self:addWidget(Image, self.mainLayer, -68, 5, width*1.05, height, "resources/UI/GR_Icon_Gem.png");
		temp.gold		= self:addWidget(Image, self.mainLayer, -68, 5, width, height, "resources/UI/GR_Icon_Coin.png");

		temp.gem:setParent( parent );
		temp.gold:setParent( parent );

		temp.gem:setPriority( PRIORITY_4 );
		temp.gold:setPriority( PRIORITY_4 );

		return temp;
	end

end

------------------------------------------------------------------------------------
-- Guide
------------------------------------------------------------------------------------


function StateDroneGarage:_checkGuide()

	if			not App.data:getValue("guide-drone-evolve-merge-complete", false)
		and	self.droneEvolveFrame ~= nil then

		self.ScreenButton:enable();

		if self.droneEvolveSide == LEFT then

			if DroneMgr:isDroneDupeResolveModeAutoMerge() then
				GuideMgr:addSet("drone_merge_left");
         elseif DroneMgr:isDroneDupeResolveModeEvolve() then
				GuideMgr:addSet("drone_evolve_left");
			else
            GuideMgr:addSet("drone_hybrid_left");
         end
		else

			if DroneMgr:isDroneDupeResolveModeAutoMerge() then
				GuideMgr:addSet("drone_merge_right");
			elseif DroneMgr:isDroneDupeResolveModeEvolve() then
				GuideMgr:addSet("drone_evolve_right");
			else
            GuideMgr:addSet("drone_hybrid_right");
         end
		end

		App.data:setValue("guide-drone-evolve-merge-complete", true);
		GuideMgr:endFocus();
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:onInfoButtonSparkle()

	if self.leftSparkle then
		self.leftSparkle:setVisible(true);
		self.leftSparkle:start();
		self.leftSparkle:setPriority(2002);
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:guidePullEvolveButton()

	self.droneEvolveButton:setPriority(2103);
	self.droneEvolveButtonTxt:setPriority(2105);
	self.droneEvolveButtonTxtShdow:setPriority(2104);
	self.droneEvolveIcon:setPriority(2104);

end

------------------------------------------------------------------------------------

function StateDroneGarage:guidePullHybridButton()

	self.droneEvolveButton:setPriority(2103);
	self.droneEvolveButtonTxt:setPriority(2105);
	self.droneEvolveButtonTxtShdow:setPriority(2104);
	self.droneEvolveIcon:setPriority(2104);
   self.droneEvolveButton:disable();

end

------------------------------------------------------------------------------------

function StateDroneGarage:guideSetLeftToBuildIndex()

   self._guideDroneLeftIndexInitial = DroneMgr:isDroneMountLeftAssigned();

   self._droneLeft:advanceToEntryY(#self._droneLeftData);

end

------------------------------------------------------------------------------------

function StateDroneGarage:guideSetRightToBuildIndex()

   self._guideDroneRightIndexInitial = DroneMgr:isDroneMountRightAssigned();

   self._droneRight:advanceToEntryY(#self._droneRightData);

end

------------------------------------------------------------------------------------

function StateDroneGarage:guideSetLeftToEnd()

   self:guideSetLeftToBuildIndex();
   self:guideDisableScreenButton();

end

------------------------------------------------------------------------------------

function StateDroneGarage:guideSetRightToEnd()

   self:guideSetRightToBuildIndex();
   self:guideDisableScreenButton();

end

------------------------------------------------------------------------------------

function StateDroneGarage:guideDisableScreenButton()

   if self._guideDroneLeftIndexInitial then
      self._droneLeft:advanceToEntryY(self._guideDroneLeftIndexInitial);

      self._guideDroneLeftIndexInitial = nil;
   end

   if self._guideDroneRightIndexInitial then
      self._droneRight:advanceToEntryY(self._guideDroneRightIndexInitial);

      self._guideDroneRightIndexInitial = nil;
   end

   self.ScreenButton:disable();

end


------------------------------------------------------------------------------------
-- Drone Left/Right
------------------------------------------------------------------------------------


function StateDroneGarage:_initDroneList()

	self:_createDroneData();
	self:_updateDroneLeft();
	self:_updateDroneRight();
	self:_updateDronePosition();
	self:_checkGuide();

	if			not DroneMgr:isDroneMountRightLocked()
		and	DroneMgr:isDroneGarageSpaceEmptyRight(1) then

		self:_buildDrone(1, RIGHT);
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_updateDroneLeft()

	self._droneLeft:setEntryCount(nil, #self._droneLeftData);
	self._droneLeft:setEntryData(self._droneLeftData);
	self:_updateDroneArrowLeft();

end

------------------------------------------------------------------------------------

function StateDroneGarage:_updateDroneRight()

	self._droneRight:setEntryCount(nil, #self._droneRightData);
	self._droneRight:setEntryData(self._droneRightData);
	self:_updateDroneArrowRight();

end

------------------------------------------------------------------------------------

function StateDroneGarage:_updateDronePosition()

	self._droneLeft:goToEntryY( math.clamp(DroneMgr:getDroneMountLeft(), 1, #self._droneLeftData), true, true );
	self._droneRight:goToEntryY( math.clamp(DroneMgr:getDroneMountRight(), 1, #self._droneRightData), true, true );

end

------------------------------------------------------------------------------------


function StateDroneGarage:_updateDroneArrowLeft()

	if			not DroneMgr:isDroneMountLeftLocked()
		and	self._droneLeftData
		and	#self._droneLeftData > 0 then

		local pos = DroneMgr:getDroneMountLeft();

		if pos - 1 <= 0 then
			self._droneArrowUpLeft:disable();
			self._droneArrowUpLeft:setVisible(false);

			self._droneArrowUpLeft._visible	= false;
		else
			self._droneArrowUpLeft:enable();
			self._droneArrowUpLeft:setVisible(true);

			self._droneArrowUpLeft._visible	= true;
		end

		if pos + 1 > #self._droneLeftData then
			self._droneArrowDownLeft:disable();
			self._droneArrowDownLeft:setVisible(false);

			self._droneArrowDownLeft._visible	= false;
		else
			self._droneArrowDownLeft:enable();
			self._droneArrowDownLeft:setVisible(true);

			self._droneArrowDownLeft._visible	= true;
		end
	else
		self._droneArrowUpLeft:disable();
		self._droneArrowDownLeft:disable();
		self._droneArrowUpLeft:setVisible(false);
		self._droneArrowDownLeft:setVisible(false);

		self._droneArrowUpLeft._visible		= false;
		self._droneArrowDownLeft._visible	= false;
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_updateDroneArrowRight()

	if			not DroneMgr:isDroneMountRightLocked()
		and	self._droneRightData
		and	#self._droneRightData > 0 then

		local pos = DroneMgr:getDroneMountRight();

		if pos - 1 <= 0 then
			self._droneArrowUpRight:disable();
			self._droneArrowUpRight:setVisible(false);

			self._droneArrowUpRight._visible	= false;
		else
			self._droneArrowUpRight:enable();
			self._droneArrowUpRight:setVisible(true);

			self._droneArrowUpRight._visible	= true;
		end

		if pos + 1 > #self._droneRightData then
			self._droneArrowDownRight:disable();
			self._droneArrowDownRight:setVisible(false);

			self._droneArrowDownRight._visible	= false;
		else
			self._droneArrowDownRight:enable();
			self._droneArrowDownRight:setVisible(true);

			self._droneArrowDownRight._visible	= true;
		end
	else
		self._droneArrowUpRight:disable();
		self._droneArrowDownRight:disable();
		self._droneArrowUpRight:setVisible(false);
		self._droneArrowDownRight:setVisible(false);

		self._droneArrowUpRight._visible		= false;
		self._droneArrowDownRight._visible	= false;
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_upgradeDroneLeft( index )

	if DroneMgr:canUpgradeDroneLeft(index) then
      EventMgr:postEvent( {sounds = {start={"SwipeSucceed",},}, name="upgrade drone"}, "add-sfx" );
		DroneMgr:upgradeDroneLeft(index);
		self:_initDroneList();
	else
		local cost = DroneMgr:getUpgradeCostLeft(index);

		if cost.currency == CURRENCY_GEM then
			AUTO_PURCHASE_DATA.currency		= 'Gem';
		else
			AUTO_PURCHASE_DATA.currency		= 'Coin';
		end

		AUTO_PURCHASE_DATA.state			= 'StateDroneGarage';
		AUTO_PURCHASE_DATA.droneIndex		= ""..index;
		AUTO_PURCHASE_DATA.reason1			= 'Upgrade-Drone';
		AUTO_PURCHASE_DATA.reason2			= 'Upgrade-Drone'..index;
		AUTO_PURCHASE_DATA.price			= cost.cost;
		AUTO_PURCHASE_FUNC					=	function()
															DroneMgr:upgradeDroneLeft(index);
														end

		StateMgr:push("state/state-auto-purchase-popup-maz.lua");
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_upgradeDroneRight( index )

	if DroneMgr:canUpgradeDroneRight(index) then
      EventMgr:postEvent( {sounds = {start={"SwipeSucceed",},}, name="upgrade drone"}, "add-sfx" );
		DroneMgr:upgradeDroneRight(index);
		self:_initDroneList();
	else
		local cost = DroneMgr:getUpgradeCostRight(index);

		if cost.currency == CURRENCY_GEM then
			AUTO_PURCHASE_DATA.currency		= 'Gem';
		else
			AUTO_PURCHASE_DATA.currency		= 'Coin';
		end

		AUTO_PURCHASE_DATA.state			= 'StateDroneGarage';
		AUTO_PURCHASE_DATA.droneIndex		= ""..index;
		AUTO_PURCHASE_DATA.reason1			= 'Upgrade-Drone';
		AUTO_PURCHASE_DATA.reason2			= 'Upgrade-Drone'..index;
		AUTO_PURCHASE_DATA.price			= cost.cost;
		AUTO_PURCHASE_FUNC					=	function()
															DroneMgr:upgradeDroneRight(index);
														end

		StateMgr:push("state/state-auto-purchase-popup-maz.lua");
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_advanceLeft()
	self._droneLeft:advanceY();
end

------------------------------------------------------------------------------------

function StateDroneGarage:_regressLeft()
	self._droneLeft:regressY();
end

------------------------------------------------------------------------------------

function StateDroneGarage:_advanceRight()
	self._droneRight:advanceY();
end

------------------------------------------------------------------------------------

function StateDroneGarage:_regressRight()
	self._droneRight:regressY();
end

------------------------------------------------------------------------------------

function StateDroneGarage:updateArrowPulse( dt )

	local alpha;

	self._arrowPulseTimer	= self._arrowPulseTimer + dt;

	if self._arrowPulseTimer > PULSE_TIME then
		self._arrowPulseTimer	= 0;
	end

	if self._arrowPulseTimer <= PULSE_TIME/2 then
		alpha	= math.mapClamped(0, 0.8, PULSE_TIME/2, 1/3, self._arrowPulseTimer);
	else
		alpha	= math.mapClamped(PULSE_TIME/2, 1/3, PULSE_TIME, 0.8, self._arrowPulseTimer);
	end

	if self._droneArrowUpLeft._visible then
		self._droneArrowUpLeft:setColor(1, 1, 1, alpha);
	end

	if self._droneArrowDownLeft._visible then
		self._droneArrowDownLeft:setColor(1, 1, 1, alpha);
	end

	if self._droneArrowUpRight._visible then
		self._droneArrowUpRight:setColor(1, 1, 1, alpha);
	end

	if self._droneArrowDownRight._visible then
		self._droneArrowDownRight:setColor(1, 1, 1, alpha);
	end

end


------------------------------------------------------------------------------------
-- Drone Mount
------------------------------------------------------------------------------------


function StateDroneGarage:_unlockMountLeft()

	if DroneMgr:isDroneMountLeftLocked() then

		if DroneMgr:canUnlockDroneMountLeft() then
			DroneMgr:unlockDroneMountLeft();
			DroneMgr:setDroneMountLeft(1);
			self:_initDroneList();
		else
			local cost = DroneMgr:getDroneMountUnlockCost();

			if cost.currency == CURRENCY_GEM then
				AUTO_PURCHASE_DATA.currency		= 'Gem';
			else
				AUTO_PURCHASE_DATA.currency		= 'Coin';
			end

			AUTO_PURCHASE_DATA.state			= 'StateDroneGarage';
			AUTO_PURCHASE_DATA.reason1			= 'Unlock-Mount-Left';
			AUTO_PURCHASE_DATA.price			= cost.cost;
			AUTO_PURCHASE_FUNC					=	function()
																DroneMgr:unlockDroneMountLeft();
																DroneMgr:setDroneMountLeft(1);
															end

			StateMgr:push("state/state-auto-purchase-popup-maz.lua");
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_unlockMountRight()

	if DroneMgr:isDroneMountRightLocked() then

		if DroneMgr:canUnlockDroneMountRight() then
			DroneMgr:unlockDroneMountRight();
			DroneMgr:setDroneMountRight(1);

         if self._unlockAnim and self._rightDroneBayLock then

            disableButtonInput();
            self._droneLeft:disable();
            self._droneRight:disable();
            self._rightDroneBayLock:setVisible(false);
            self._unlockAnim:setVisible(true);
            self._blackFilter:setVisible(true);
            self._unlockAnim:setListener( MOAIAction.EVENT_STOP,
                                          function()
                                             delay(   0.5,
                                                      function()
                                                         enableButtonInput();
                                                         self._droneLeft:enable();
                                                         self._droneRight:enable();
                                                         self:_initDroneList()
                                                      end );
                                          end );
            self._unlockAnim:start();
         else
            self:_initDroneList();
         end
		else
			local cost = DroneMgr:getDroneMountUnlockCost();

			if cost.currency == CURRENCY_GEM then
				AUTO_PURCHASE_DATA.currency		= 'Gem';
			else
				AUTO_PURCHASE_DATA.currency		= 'Coin';
			end

			AUTO_PURCHASE_DATA.state			= 'StateDroneGarage';
			AUTO_PURCHASE_DATA.reason1			= 'Unlock-Mount-Right';
			AUTO_PURCHASE_DATA.price			= cost.cost;
			AUTO_PURCHASE_FUNC					=	function()
																DroneMgr:unlockDroneMountRight();
																DroneMgr:setDroneMountRight(1);
															end

			StateMgr:push("state/state-auto-purchase-popup-maz.lua");
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_findEmptyBay( count, side )

	if side == LEFT then
		for i=1,count do
			if DroneMgr:isDroneGarageSpaceEmptyLeft(i) then
				return i;
			end
		end
	else
		for i=1,count do
			if DroneMgr:isDroneGarageSpaceEmptyRight(i) then
				return i;
			end
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_buildDrone( index, side )

	if		( side == LEFT and not DroneMgr:canBuildDroneLeft(index, false) )
		or	( side == RIGHT and not DroneMgr:canBuildDroneRight(index, false) ) then

		local cost;

		if side == LEFT then
			cost = DroneMgr:getBuildCostLeft(false);
		else
			cost = DroneMgr:getBuildCostRight(false);
		end

		if cost.currency == CURRENCY_GEM then
			AUTO_PURCHASE_DATA.currency		= 'Gem';
		else
			AUTO_PURCHASE_DATA.currency		= 'Coin';
		end

		AUTO_PURCHASE_DATA.state			= 'StateDroneGarage';
		AUTO_PURCHASE_DATA.droneIndex		= ""..index;
		AUTO_PURCHASE_DATA.reason1			= 'Build-Drone';
		AUTO_PURCHASE_DATA.reason2			= 'Build-Drone'..index;
		AUTO_PURCHASE_DATA.price			= cost.cost;
		AUTO_PURCHASE_FUNC					=	function()
															local rarity, target_index;

															if side == LEFT then
																DroneMgr:addDroneLeft(index, cost);
															else
																DroneMgr:addDroneRight(index, cost);
															end

															if side == LEFT then
																rarity, target_index = DroneMgr:isDroneTypeCollectedLeft( DroneMgr:getDroneTypeLeft(index) );
															else
																rarity, target_index = DroneMgr:isDroneTypeCollectedRight( DroneMgr:getDroneTypeRight(index) );
															end

															if rarity and target_index and target_index ~= index then
																StateMgr:swap( "state/state-drone-merge.lua", {	garage_index = index, target_index = target_index, side = side } );
															else
																StateMgr:swap( "state/state-drone-reveal.lua", { garage_index = index, side = side } );
															end
														end

		StateMgr:push("state/state-auto-purchase-popup-maz.lua");

		return false;
	else
		local rarity, target_index, cost;

		if side == LEFT then
			cost = DroneMgr:getBuildCostLeft(false);

			DroneMgr:addDroneLeft(index, cost);
		else
			cost = DroneMgr:getBuildCostRight(false);

			DroneMgr:addDroneRight(index, cost);
		end

		if side == LEFT then
			rarity, target_index = DroneMgr:isDroneTypeCollectedLeft( DroneMgr:getDroneTypeLeft(index) );
		else
			rarity, target_index = DroneMgr:isDroneTypeCollectedRight( DroneMgr:getDroneTypeRight(index) );
		end

      EventMgr:postEvent( {sounds = {start={"BuildDrone",},}, name="build drone"}, "add-sfx" );
		if rarity and target_index and target_index ~= index then
			StateMgr:push( "state/state-drone-merge.lua", {	garage_index = index, target_index = target_index, side = side  } );
		else
			StateMgr:push( "state/state-drone-reveal.lua", { garage_index = index, side = side } );
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_evolveDrone( garage_index, side )

	if side == LEFT then

		if DroneMgr:canEvolveDroneLeft(garage_index) then
			--EventMgr:postEvent( {sounds = {start={"BuildDrone",},}, name="build drone"}, "add-sfx" );
			StateMgr:push( "state/state-drone-evolve.lua", { garage_index = garage_index, side = side } );
		else
			local cost = DroneMgr:getEvolveCostLeft(garage_index);

			if cost.currency == CURRENCY_GEM then
				AUTO_PURCHASE_DATA.currency		= 'Gem';
			else
				AUTO_PURCHASE_DATA.currency		= 'Coin';
			end

			AUTO_PURCHASE_DATA.state			= 'StateDroneGarage';
			AUTO_PURCHASE_DATA.droneIndex		= ""..garage_index;
			AUTO_PURCHASE_DATA.reason1			= 'Evolve-Drone';
			AUTO_PURCHASE_DATA.reason2			= 'Evolve-Drone'..garage_index;
			AUTO_PURCHASE_DATA.price			= cost.cost;
			AUTO_PURCHASE_FUNC					=	function()
																--EventMgr:postEvent( {sounds = {start={"BuildDrone",},}, name="build drone"}, "add-sfx" );
																StateMgr:push( "state/state-drone-evolve.lua", { garage_index = garage_index, side = side } );
															end

			StateMgr:push("state/state-auto-purchase-popup-maz.lua");
		end
	else

		if DroneMgr:canEvolveDroneRight(garage_index) then
			--EventMgr:postEvent( {sounds = {start={"BuildDrone",},}, name="build drone"}, "add-sfx" );
			StateMgr:push( "state/state-drone-evolve.lua", { garage_index = garage_index, side = side } );
		else
			local cost = DroneMgr:getEvolveCostRight(garage_index);

			if cost.currency == CURRENCY_GEM then
				AUTO_PURCHASE_DATA.currency		= 'Gem';
			else
				AUTO_PURCHASE_DATA.currency		= 'Coin';
			end

			AUTO_PURCHASE_DATA.state			= 'StateDroneGarage';
			AUTO_PURCHASE_DATA.droneIndex		= ""..garage_index;
			AUTO_PURCHASE_DATA.reason1			= 'Evolve-Drone';
			AUTO_PURCHASE_DATA.reason2			= 'Evolve-Drone'..garage_index;
			AUTO_PURCHASE_DATA.price			= cost.cost;
			AUTO_PURCHASE_FUNC					=	function()
																--EventMgr:postEvent( {sounds = {start={"BuildDrone",},}, name="build drone"}, "add-sfx" );
																StateMgr:push( "state/state-drone-evolve.lua", { garage_index = garage_index, side = side } );
															end

			StateMgr:push("state/state-auto-purchase-popup-maz.lua");
		end
	end

end


------------------------------------------------------------------------------------
-- Utils
------------------------------------------------------------------------------------


function StateDroneGarage:_disableButton( button )

	if button then
		button:disable();
		button:setColor(0.3, 0.3, 0.3, 1);
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_enableButton( button )

	if button then
		button:enable();
		button:setColor(1,1,1,1);
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_hideButton( button, icons )

	if button then
		button:disable();
		button:setVisible(false);
	end

	if icons then
		for k,v in pairs(icons) do
			v:setVisible(false);
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_showButton( button )

	if button then
		button:enable();
		button:setVisible(true);
		button:setColor(1,1,1,1);
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_showCurrency( icons, currency, cost, text, textShdw )

	if icons and currency then

		local iconX	= -80;
		local textX	= 35;
		local	y		= 5;

		if cost == 0 then
			textX	= 0;
		elseif cost < 10 then
			iconX	= -30;
			textX	= 25;
		elseif cost < 100 then
			iconX	= -35;
			textX	= 30;
		elseif cost < 1000 then
			iconX	= -45;
			textX	= 30;
		elseif cost < 10000 then
			iconX	= -60;
			textX	= 25;
		elseif cost < 100000 then
			iconX	= -70;
			textX	= 25;
		elseif cost < 1000000 then
			iconX	= -75;
			textX	= 30;
		end

		for k,v in pairs(icons) do
			if k == currency then
				if cost > 0 then
					v:setVisible(true);
					v:setLoc(iconX, y);
				else
					v:setVisible(false);
				end
				text:setLoc(textX, y);
				textShdw:setLoc(textX+2, y-2);
			else
				v:setVisible(false);
			end
		end

		icons.gem:setPriority( PRIORITY_4 );
		icons.gold:setPriority( PRIORITY_4 );
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_hidePips( pips, background )

	if pips and background then
		background:setVisible(false);

		if self._objPulseList and self._objPulseList[background] then
			self:_removeFromPulseList(background);
		end

		for k,v in pairs(pips) do
			for key,val in pairs(v) do
				val:setVisible(false);

				if self._objPulseList and self._objPulseList[val] then
					self:_removeFromPulseList(val);
				end
			end
		end

		if pips.starOn and pips.starOn[1] then
			if self._objPulseList and self._objPulseList[pips.starOn[1]] then
				self:_removeFromPulseList(pips.starOn[1]);
			end
		end

		if pips.starOff and pips.starOff[1] then
			if self._objPulseList and self._objPulseList[pips.starOff[1]] then
				self:_removeFromPulseList(pips.starOff[1]);
			end
		end

		if pips.starText and pips.starText[1] then
			if self._objPulseList and self._objPulseList[pips.starText[1]] then
				self:_removeFromPulseList(pips.starText[1]);
			end
		end

		if pips.starTextShdw and pips.starTextShdw[1] then
			if self._objPulseList and self._objPulseList[pips.starTextShdw[1]] then
				self:_removeFromPulseList(pips.starTextShdw[1]);
			end
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_showPips( pips, background, level, rarity )

	if pips and background and level then
		local maxL = DroneMgr:getDroneMaxLevel();

		background:setVisible(true);

		for i=1,maxL do
			if i <= level then
				pips.on[i]:setVisible(true);
				pips.off[i]:setVisible(false);
			else
				pips.on[i]:setVisible(false);
				pips.off[i]:setVisible(true);
			end

			pips.on[i]:setPriority(		PRIORITY_3+1	);
			pips.off[i]:setPriority(	PRIORITY_3	);
		end

		if			rarity
			and	rarity < DroneMgr:getDroneMaxRarity()
			and	not DroneMgr:isDroneDupeResolveModeAutoMerge() then

			if level < maxL then
				pips.starOff[1]:setVisible(true);
			else
				pips.starOn[1]:setVisible(true);
			end

			pips.starText[1]:setVisible(true);
			pips.starTextShdw[1]:setVisible(true);
			pips.starText[1]:setText( tostring(rarity+1) );
			pips.starTextShdw[1]:setText( tostring(rarity+1) );
		end

		pips.starOn[1]:setPriority(			PRIORITY_3		);
		pips.starOff[1]:setPriority(			PRIORITY_3		);
		pips.starText[1]:setPriority(			PRIORITY_3+2	);
		pips.starTextShdw[1]:setPriority(	PRIORITY_3+1	);
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_hideStars( stars )

	if stars then
		for _,v in pairs(stars) do
			for __,val in pairs(v) do
				val:setVisible(false);
				val:setPriority( PRIORITY_3 );

				if self._objPulseList and self._objPulseList[val] then
					self:_removeFromPulseList(val);
				end
			end
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_showStars( stars, rarity )

	if stars and stars.reg and rarity and stars.reg[rarity] then
		stars.reg[rarity]:setVisible(true);

		if stars.slv and stars.slv[rarity+1] then
			stars.slv[rarity+1]:setVisible(true);
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_setCacheObjectVisible( object, flag )

	local temp = flag;

	if temp == nil then
		temp = true;
	end

	if object and type(object) == "table" and next(object) ~= nil then
		for k,v in pairs(object) do

			if type(v) == "table" then
				if v.stop and v.start then
					if temp then
						v:start();
					else
						v:stop();
					end
				end

				if v.setVisible then
					v:setVisible(temp);
				end

				if v.disable and v.enable then
					if temp then
						v:enable();
					else
						v:disable();
					end
				end

				if self._objPulseList and self._objPulseList[object] then
					self:_removeFromPulseList(object);
				end

				if self._objRotList and self._objRotList[object] then
					self:_removeFromRotList(object);
				end
			end
		end
	end

	object.root:setPriority(			PRIORITY_0	);
	object.upgradeText:setPriority(	PRIORITY_2	);
	object.titleText:setPriority(		PRIORITY_2	);
	object.levelText:setPriority(		PRIORITY_2	);
	object.pipFrame:setPriority(		PRIORITY_2	);
	object.drone:setPriority(			PRIORITY_2	);
	object.droneL:setPriority(			PRIORITY_2	);
	object.droneR:setPriority(			PRIORITY_2	);
	object.infoButton:setPriority(	PRIORITY_3	);
	object.sparkle:setPriority(		PRIORITY_3-1);
	object.omniButtonLG:setPriority(	PRIORITY_2	);
	object.omniButtonLB:setPriority(	PRIORITY_2	);
	object.buttonTxtLG:setPriority(	PRIORITY_4	);
	object.buttonTxtLB:setPriority(	PRIORITY_4	);
	object.buttonSTxtLG:setPriority(	PRIORITY_3	);
	object.buttonSTxtLB:setPriority(	PRIORITY_3	);

end

------------------------------------------------------------------------------------

function StateDroneGarage:_addToPulseList( obj, scl )

	if obj and scl then

		local t	= {};

		t.obj		= obj;
		t.scl		= scl;

		self._objPulseList[obj]	= t;
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_removeFromPulseList( obj )

	if obj and self._objPulseList[obj] then
		self._objPulseList[obj]	= nil;

		obj:setScale(1,1);
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:updatePulse( dt )

	local hFreq	= PULSE_TIME/2;

	self._objPulseTimer		= self._objPulseTimer + dt;

	if self._objPulseTimer > PULSE_TIME then
		self._objPulseTimer	= 0;
	end

	if self._objPulseList then

		for k,v in pairs(self._objPulseList) do

			if v.obj and v.scl then

				local k;

				if self._objPulseTimer > hFreq then
					k	= math.mapClamped(hFreq, v.scl, PULSE_TIME, 1, self._objPulseTimer);
				else
					k	= math.mapClamped(0, 1, hFreq, v.scl, self._objPulseTimer);
				end

				v.obj:setScale(k,k);
			end
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_addToRotList( obj )

	if self._objRotList and obj then
		self._objRotList[obj] = obj;

		obj:setVisible(true);
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:_removeFromRotList( obj )

	if self._objRotList and obj and self._objRotList[obj] then
		self._objRotList[obj] = nil;
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:updateRot( dt )

	if self._objRotList then
		for _,v in pairs(self._objRotList) do
			v:setRot(v:getRot()+45*dt);
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:isDroneEquipped()

	local left	= DroneMgr:isDroneMountLeftAssigned();
	local right	= DroneMgr:isDroneMountRightAssigned();

	if left and DroneMgr:isDroneGarageSpaceFilledLeft(left) then
		return true;
	end

	if right and DroneMgr:isDroneGarageSpaceFilledRight(right) then
		return true;
	end

	return false;

end


------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------


function StateDroneGarage:goBack()

	if self:isDroneEquipped() then
		StateMgr:pop();
	else
		StateMgr:push("state/state-quit-drone-confirm.lua");
	end

end

------------------------------------------------------------------------------------

function StateDroneGarage:goCodex()
	StateMgr:push("state/state-drone-codex.lua");
end


------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------


return StateDroneGarage;
