----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local DRONE_LEFT            = "left";  -- mirror in app data
local DRONE_RIGHT           = "right"; -- mirror in app data

local PRIORITY_0            = 10;
local PRIORITY_1            = 20;
local PRIORITY_2            = 30;
local PRIORITY_3            = 40;
local PRIORITY_4            = 50;
local PRIORITY_5            = 60;
local PRIORITY_6            = 70;
local PRIORITY_7            = 80;
local PRIORITY_8            = 90;
local PRIORITY_9            = 100;
local PRIORITY_10           = 110;

local SQUARE_TEXTURE        = "resources/square.png";
local WHITE_FLASH           = "resources/UI/Bounty/WhiteFlash_small.png";
local FLASH_TIME            = 0.5;
local REVEAL_LOCK_TIME      = 0.25;
local GLOW_ROTATION_RATE    = 45;

local STATE_REVEAL_START    = "reveal-start";
local STATE_REVEAL_END      = "reveal-end";
local STATE_REVEAL_COMPLETE = "reveal-complete";

----------------------------------------------------------------

local DRONE_STARS =
{
    "resources/UI/Drone_UI/1Star.png",
    "resources/UI/Drone_UI/2Star.png",
    "resources/UI/Drone_UI/3Star.png",
    "resources/UI/Drone_UI/4Star.png",
    "resources/UI/Drone_UI/5Star.png",
};

----------------------------------------------------------------

local Base = State;
local StateDroneStats = Base:new();

----------------------------------------------------------------

function StateDroneStats:onLoad()

   State.onLoad( self );

   local textureSheet = FlashMgr:getTextureSheet("resources/flash-resource-bundle/PopUPs.lua", "resources/flash-resource-bundle/PopUPs.png", 30);

   self.defaultFontName = "resources/font/TangoLatin.otf";
   self.IS_POPUP = true;

   -- background
   self.Background = self:addWidget(Image, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE, {0,0,0, 0.75});
   self.Background:setPriority(0);

   -- frame
   self.Frame = self:addWidget(Image, self.mainLayer, 0,0, 600, 500, "resources/UI/Drone_UI/infoframe.png");
   self.Frame:setPriority(PRIORITY_0-1);

   -- text message
   self.NameText = self:addWidget(TextBox, self.mainLayer, 112, 140, 600, 100, "Drone Name", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.NameText:setScale(1.15, 1.15);
   self.NameText:setPriority(PRIORITY_3);
   self.NameTextShdw = self:addWidget(TextBox, self.mainLayer, 112+2, 140-2, 600, 100, "Drone Name", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
   self.NameTextShdw:setScale(1.15, 1.15);
   self.NameTextShdw:setPriority(PRIORITY_3-1);
   self.LevelText = self:addWidget(TextBox, self.mainLayer, -155, -170, 300, 100, "Drone Level", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.LEFT_JUSTIFY);
   self.LevelText:setScale(0.85, 0.85);
   self.LevelText:setPriority(PRIORITY_3);
   self.LevelTextShdw = self:addWidget(TextBox, self.mainLayer, -155+2, -170-2, 300, 100, "Drone Level", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.LEFT_JUSTIFY, {0,0,0,1});
   self.LevelTextShdw:setScale(0.85, 0.85);
   self.LevelTextShdw:setPriority(PRIORITY_3-1);
   self.DescText = self:addWidget(TextBox, self.mainLayer, 112, -60, 460, 400, "Drone Desc", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.LEFT_JUSTIFY);
   self.DescText:setPriority(PRIORITY_3);
   self.DescText:setScale(0.7, 0.7);
   self.DescText:setLineSpacing(5);
   self.DescTextShdw = self:addWidget(TextBox, self.mainLayer, 112+2, -60-2, 460, 400, "Drone Desc", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.LEFT_JUSTIFY, {0,0,0,1});
   self.DescTextShdw:setPriority(PRIORITY_3-1);
   self.DescTextShdw:setScale(0.7, 0.7);
   self.DescTextShdw:setLineSpacing(5);
   self.InfoText = self:addWidget(TextBox, self.mainLayer, 112, -125, 460, 400, "Drone Info", self.defaultFontName, 18, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.LEFT_JUSTIFY);
   self.InfoText:setPriority(PRIORITY_3);
   self.InfoText:setScale(0.6, 0.6);
   self.InfoText:setLineSpacing(3);
   self.InfoTextShdw = self:addWidget(TextBox, self.mainLayer, 112+2, -125-2, 460, 400, "Drone Info", self.defaultFontName, 18, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.LEFT_JUSTIFY, {0,0,0,1});
   self.InfoTextShdw:setPriority(PRIORITY_3-1);
   self.InfoTextShdw:setScale(0.6, 0.6);
   self.InfoTextShdw:setLineSpacing(3);

   -- drone pic
   self.DroneL = self:addWidget(Image, self.mainLayer, -50-155, 0, 100, 200, "resources/UI/Drone_UI/sill.png");
   self.DroneL:setPriority(PRIORITY_2);
   self.DroneR = self:addWidget(Image, self.mainLayer, 50-155, 0, 100, 200, "resources/UI/Drone_UI/sill.png");
   self.DroneR:setPriority(PRIORITY_2);

   -- rarity stars
   self.Stars = self:_createDroneStars( -155, 140, 0.8, 0.8 );

   -- Exit Button
   self.BackButton = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 285, 240, 0,0, "CloseButton", textureSheet, nil, self.goBack, nil, nil, nil, nil, 40);
   self.BackButton:setPriority(PRIORITY_0);

end

----------------------------------------------------------------

function StateDroneStats:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StateDroneStats:onFocus( data )

   State.onFocus(self);

   Debug.assertType(data, "table", "StateDroneStats:onFocus -> data");

   if not self:_isDataValid(data) then
      self:goBack();
      return;
   end

	local lvlTxt;
	local desc	= "";
	local stats	= "";

   if data.level >= DroneMgr:getDroneMaxLevel() then
      lvlTxt = "Max Level";
   else
      lvlTxt = "Level "..data.level;
   end

   for i=1,#data.desc do
      if i==1 then
         desc = stats..data.desc[i];
      else
         stats = stats..data.desc[i].."\n\n";
      end
   end

   self.DroneL:swapImage(data.texture);
   self.DroneR:swapImage(data.texture);
   self.DroneR:setScale(-1, 1);
   self:_showStars(self.Stars, data.rarity);

   self.NameText:setText(data.name);
   self.NameTextShdw:setText(data.name);
   self.LevelText:setText(lvlTxt);
   self.LevelTextShdw:setText(lvlTxt);
   self.DescText:setText(desc);
   self.DescTextShdw:setText(desc);
   self.InfoText:setText(stats);
   self.InfoTextShdw:setText(stats);

end

----------------------------------------------------------------

function StateDroneStats:onUpdate( dt )

   Base.onUpdate(self, dt);

end


------------------------------------------------------------------------------------
-- Util
------------------------------------------------------------------------------------


function StateDroneStats:_createDroneStars(  x, y, sx, sy, priority )

   if x and y and sx and sy then
      local width = 200 * sx;
      local height = 100 * sy;
      local temp = {};
      local p = priority or PRIORITY_2;

      for i=1,DroneMgr:getDroneMaxRarity() do
         temp[i] = self:addWidget(Image, self.mainLayer, x, y, width, height, DRONE_STARS[i]);

         temp[i]:setPriority(p);
      end

      return temp;
   end

end

------------------------------------------------------------------------------------

function StateDroneStats:_hideStars( stars )

   if stars then
      for k,v in ipairs(stars) do
         v:setVisible(false);
      end
   end

end

------------------------------------------------------------------------------------

function StateDroneStats:_isDataValid( data )

   if data == nil then
      return false;
   end

   if next(data) == nil then
      return false;
   end

   if data.level == nil then
      return false;
   end

   if data.desc == nil then
      return false;
   end

   if data.rarity == nil then
      return false;
   end

   if data.texture == nil then
      return false;
   end

   if data.name == nil then
      return false;
   end

   return true;

end

------------------------------------------------------------------------------------

function StateDroneStats:_showStars( stars, rarity )

   if stars and rarity and stars[rarity] then
      for k,v in ipairs(stars) do
         if k == rarity then
            v:setVisible(true);
         else
            v:setVisible(false);
         end
      end
   end

end

------------------------------------------------------------------------------------

function StateDroneStats:isUsingCameraHack()
   return false;
end


------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------


function StateDroneStats:goBack()
   StateMgr:pop();
end


------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------


return StateDroneStats;
