----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local DRONE_LEFT					= "left";  -- mirror in app data
local DRONE_RIGHT					= "right"; -- mirror in app data

local PRIORITY_0					= 10;
local PRIORITY_1					= 20;
local PRIORITY_2					= 30;
local PRIORITY_3					= 40;
local PRIORITY_4					= 50;
local PRIORITY_5					= 60;
local PRIORITY_6					= 70;
local PRIORITY_7					= 80;
local PRIORITY_8					= 90;
local PRIORITY_9					= 100;
local PRIORITY_10					= 110;

local SQUARE_TEXTURE				= "resources/square.png";
local INIT_TIME					= 1.0;
local GLOW_ROTATION_RATE		= 45;

local DRONE_A_X					= -145;
local DRONE_B_X					= 145;
local DRONE_Y						= -40;

local STATE_MERGE_INTRO       = "merge-intro-built-drone";
local STATE_MERGE_START       = "merge-start-merge";

----------------------------------------------------------------

local DRONE_STARS					=
{
	"resources/UI/Drone_UI/1Star.png",
	"resources/UI/Drone_UI/2Star.png",
	"resources/UI/Drone_UI/3Star.png",
	"resources/UI/Drone_UI/4Star.png",
	"resources/UI/Drone_UI/5Star.png",
};

--CAUTION: The order of the shadow must match the drone index order
local DRONE_SHADOW				=
{
	"resources/Vehicle/Drones/DroneGun/DroneGun_Shadow.png",
	"resources/Vehicle/Drones/DroneLaser/DroneLaser_Shadow.png",
	"resources/Vehicle/Drones/DroneBoost/DroneBoost_Shadow.png",
	"resources/Vehicle/Drones/DroneMissile/DroneMissile_Shadow.png",
	"resources/Vehicle/Drones/DroneRocket/DroneRocket_Shadow.png",
	"resources/Vehicle/Drones/DroneRepair/DroneRepair_Shadow.png",
	"resources/Vehicle/Drones/DroneShield/DroneShield_Shadow.png",
   "resources/Vehicle/Drones/DroneBarricade/DroneBarricade_Shadow.png",
};

----------------------------------------------------------------

local Base						= State;
local StateDroneMerge		= Base:new();

----------------------------------------------------------------

function StateDroneMerge:onLoad()

   State.onLoad( self );

   self.defaultFontName = "resources/font/TangoLatin.otf";
	self.IS_POPUP  = true;

	-- text message
	self.TitleText = self:addWidget(TextBox, self.mainLayer, 0, 326, 540, 70, "Drone Name", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.TitleText:setScale(1.5, 1.5);
	self.TitleText:setPriority(PRIORITY_3);
	self.TitleTextShdw = self:addWidget(TextBox, self.mainLayer, 0+2, 326-2, 540, 70, "Drone Name", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.TitleTextShdw:setScale(1.5, 1.5);
	self.TitleTextShdw:setPriority(PRIORITY_3-1);
	self.InfoText = self:addWidget(TextBox, self.mainLayer, 0, 100, 400, 400, "Duplicate drone will be used to evolve current drone", self.defaultFontName, 14, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.LEFT_JUSTIFY);
	self.InfoText:setPriority(PRIORITY_3);
	self.InfoTextShdw = self:addWidget(TextBox, self.mainLayer, 0+2, 100-2, 400, 400, "Duplicate drone will be used to evolve current drone", self.defaultFontName, 14, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.LEFT_JUSTIFY, {0,0,0,1});
	self.InfoTextShdw:setPriority(PRIORITY_3-1);
	self.MessageText = self:addWidget(TextBox, self.mainLayer, 0, -348, 500, 260, "Tap To Evolve", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.MessageText:setPriority(PRIORITY_3);
	self.MessageText:setScale(1.2, 1.2);
	self.MessageTextShdw = self:addWidget(TextBox, self.mainLayer, 0+2, -348-2, 500, 260, "Tap To Evolve", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.MessageTextShdw:setPriority(PRIORITY_3-1);
	self.MessageTextShdw:setScale(1.2, 1.2);

   -- intro anim
   local droneSheet  = FlashMgr:getTextureSheet("resources/flash-resource-bundle/Drone_Effects.lua", "resources/flash-resource-bundle/Drone_Effects.png", 30);

   self._introAnim   = self:addWidget(Anim, self.mainLayer, 0, -54, 0,0, "Drone_duplicatedrones", droneSheet);
   self._introAnim:setPriority(PRIORITY_0+1);
   self._introAnim:setVisible(false);
   self._introAnim:setListener(  MOAIAction.EVENT_STOP,
                                 function()
                                    enableButtonInput();
                                    self.MergeButton:enable();
                                    self.MessageText:setVisible(true);
                                    self.MessageTextShdw:setVisible(true);
                                    self.MessageText:setText("Tap To Evolve");
                                    self.MessageTextShdw:setText("Tap To Evolve");
                                    self:_showStars(self.starsA, self._garage_rarity);
                                    self:_showStars(self.starsB, self._target_rarity);
                                    self.InfoText:setVisible(true);
                                    self.InfoTextShdw:setVisible(true);
                                 end );

   -- reveal anim
   self._revealAnim  = self:addWidget(Anim, self.mainLayer, 0, -54, 0,0, "Dronereveal_Dronepopup", droneSheet);
   self._revealAnim:setVisible(false);
   self._revealAnim:setScale(1/11.14, 1/9.14);
   self._revealAnim:setListener( MOAIAction.EVENT_STOP,
                                 function()
                                    if self._state == STATE_MERGE_INTRO then

                                       self._state = STATE_MERGE_START;

                                       delay(   45/30,
                                                function()
                                                   self.TitleText:setVisible(false);
                                                   self.TitleTextShdw:setVisible(false);
                                                   self:_hideStars(self.starsReveal);
                                                   self.droneRevealL:setVisible(false);
                                                   self.droneRevealR:setVisible(false);
                                                   self._revealAnim:setVisible(false);
                                                   self:_showMergeDrones();
                                                end )
                                    end
                                 end );

   -- evolve anim
   self._evolveAnim  = self:addWidget(Anim, self.mainLayer, 0, -54, 0,0, "Drone_evolve", droneSheet);
   self._evolveAnim:setPriority(PRIORITY_10)
   self._evolveAnim:setVisible(false);
   self._evolveAnim:setListener( MOAIAction.EVENT_STOP,
                                 function()
                                    enableButtonInput();
                                    self.BackButton:enable();
                                    self.MessageText:setVisible(true);
                                    self.MessageTextShdw:setVisible(true);
                                    self.MessageText:setText("Tap To Continue");
                                    self.MessageTextShdw:setText("Tap To Continue");
                                    self._evolveAnim:setVisible(false);
                                 end );

	-- god rays
	self.GGlow = self:addWidget(Image, self.mainLayer, 0, -54, 640, 640, "resources/UI/Drone_UI/Backglow-01.png");
	self.GRays = self:addWidget(Image, self.mainLayer, 0, -54, 1200, 1200, "resources/UI/Drone_UI/LightRays-01.png");

	self.GGlow:setPriority(PRIORITY_2-2);
	self.GRays:setPriority(PRIORITY_2-1);
   self.GGlow:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );
   self.GRays:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );

	-- pic
	self.droneRevealL	= self:addWidget(Image, self.mainLayer, -96, -54, 192, 384, "resources/UI/Drone_UI/sill.png");
	self.droneRevealL:setPriority(PRIORITY_3);
	self.droneRevealR	= self:addWidget(Image, self.mainLayer, 96, -54, 192, 384, "resources/UI/Drone_UI/sill.png");
	self.droneRevealR:setPriority(PRIORITY_3);
	self.droneShadow	= self:addWidget(Image, self.mainLayer, 0+40, -54-40, 256, 256, "resources/UI/Drone_UI/sill.png");
	self.droneShadow:setPriority(PRIORITY_3-1);
	self.starsReveal	= self:_createDroneStars( 0, 210, 1.55, 1.45 );
	self.droneLA	= self:addWidget(Image, self.mainLayer, -48, 0, 96, 192, "resources/UI/Drone_UI/sill.png");
	self.droneLA:setPriority(PRIORITY_3);
	self.droneRA	= self:addWidget(Image, self.mainLayer, 48, 0, 96, 192, "resources/UI/Drone_UI/sill.png");
	self.droneRA:setPriority(PRIORITY_3);
	self.starsA	= self:_createDroneStars( DRONE_A_X, DRONE_Y+150, 1.0, 1.0 );
	self.droneLB	= self:addWidget(Image, self.mainLayer, -48, 0, 96, 192, "resources/UI/Drone_UI/sill.png");
	self.droneLB:setPriority(PRIORITY_3);
	self.droneRB	= self:addWidget(Image, self.mainLayer, 48, 0, 96, 192, "resources/UI/Drone_UI/sill.png");
	self.droneRB:setPriority(PRIORITY_3);
	self.starsB	= self:_createDroneStars( DRONE_B_X, DRONE_Y+150, 1.0, 1.0 );

   -- Merge Button
	self.MergeButton = self:addWidget(ButtonImage, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE, self._mergeAnimStart, nil, nil, nil);
	self.MergeButton:setPriority(0);
	self.MergeButton:setColor(0,0,0,0);

	-- Exit Button
	self.BackButton = self:addWidget(ButtonImage, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE, self.goBack, nil, nil, nil);
	self.BackButton:setPriority(PRIORITY_0);
	self.BackButton:setColor(0,0,0, 0.9);

end

----------------------------------------------------------------

function StateDroneMerge:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StateDroneMerge:onFocus( data )

   State.onFocus(self);

	Debug.assert(data and data.garage_index, "StateDroneMerge:onFocus -> drone garage index is nil");
	Debug.assert(data and data.target_index, "StateDroneMerge:onFocus -> drone target index is nil");
	Debug.assert(data and data.side, "StateDroneBuild:onFocus -> pool is nil");

   self._state          = STATE_MERGE_INTRO
	self._garage_index	= data.garage_index;
	self._target_index	= data.target_index;
	self._side				= data.side;
	self._type				= nil;
	self._garage_rarity	= nil;
	self._target_rarity	= nil;
   self._final_rarity   = nil;
	self._level				= nil;
	self._garage_texture	= nil;
	self._target_texture	= nil;
   self._final_texture  = nil;
   self._final_name     = nil;

	self:_hideAll();

   if self._side == DRONE_LEFT then
      self._type				= DroneMgr:getDroneTypeLeft(				self._garage_index );
      self._garage_rarity	= DroneMgr:getDroneRarityLeft(			self._garage_index );
      self._target_rarity	= DroneMgr:getDroneRarityLeft(			self._target_index );
      self._level				= DroneMgr:getDroneLevelLeft(				self._garage_index );
      self._garage_texture	= DroneMgr:getDroneBodyVisLeft(			self._garage_index );
      self._target_texture	= DroneMgr:getDroneBodyVisLeft(			self._target_index );
   else
      self._type				= DroneMgr:getDroneTypeRight(				self._garage_index );
      self._garage_rarity	= DroneMgr:getDroneRarityRight(			self._garage_index );
      self._target_rarity	= DroneMgr:getDroneRarityRight(			self._target_index );
      self._level				= DroneMgr:getDroneLevelRight(			self._garage_index );
      self._garage_texture	= DroneMgr:getDroneBodyVisRight(			self._garage_index );
      self._target_texture	= DroneMgr:getDroneBodyVisRight(			self._target_index );
   end

   self:_mergeDrone();

   local collected;
   local rarity	= self._garage_rarity;

   if self._side == DRONE_LEFT then
      rarity		= DroneMgr:getDroneRarityLeft(self._garage_index);
      collected	= DroneMgr:isDroneTypeCollectedRight(self._type);
   else
      rarity		= DroneMgr:getDroneRarityRight(self._garage_index);
      collected	= DroneMgr:isDroneTypeCollectedLeft(self._type);
   end

   if		collected == nil
      or	collected < rarity then

      App.data:setValue("new-drone-type-"..self._type, true);
   end

   self:_introMerge();

end

----------------------------------------------------------------

function StateDroneMerge:onUpdate( dt )

	Base.onUpdate(self, dt);

	self:_updateGGlowRayRotate(dt);

end


------------------------------------------------------------------------------------
-- State Update
------------------------------------------------------------------------------------


function StateDroneMerge:_updateGGlowRayRotate( dt )

   local glowAngle	= math.clampAngle360( self.GGlow:getRot() + GLOW_ROTATION_RATE*dt );
   local rayAngle		= math.clampAngle360( self.GRays:getRot() - GLOW_ROTATION_RATE*dt );

   self.GGlow:setRot( glowAngle );
   self.GRays:setRot( rayAngle );

end


------------------------------------------------------------------------------------
-- Merge Drone
------------------------------------------------------------------------------------


function StateDroneMerge:_introMerge()

	self.TitleText:setVisible(true);
   self.TitleTextShdw:setVisible(true);
	self.TitleText:setText(self._final_name);
	self.TitleTextShdw:setText(self._final_name);
	self:_showStars(self.starsReveal, self._garage_rarity);
	self.droneRevealL:swapImage( self._garage_texture );
	self.droneRevealR:swapImage( self._garage_texture );
	self.droneShadow:swapImage(DRONE_SHADOW[self._type]);
	self.droneRevealR:setScale(-1, 1);
	self.droneRevealL:setVisible(true);
	self.droneRevealR:setVisible(true);
	self.droneShadow:setVisible(true);
   self.droneRevealL:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );
   self.droneRevealR:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );
   self.droneShadow:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );

	if self._side == DRONE_LEFT then
		DroneMgr:setDroneMountLeft( self._target_index );
	else
		DroneMgr:setDroneMountRight( self._target_index );
	end

   self._revealAnim:setVisible(true);
   self._revealAnim:setTime(0);
   self._revealAnim:start();

end

------------------------------------------------------------------------------------

function StateDroneMerge:_showMergeDrones()

   self._revealAnim:setVisible(false);
	self.droneLA:setVisible(true);
	self.droneRA:setVisible(true);
	self.droneLB:setVisible(true);
	self.droneRB:setVisible(true);
	self.droneLA:swapImage( self._garage_texture );
	self.droneRA:swapImage( self._garage_texture );
	self.droneRA:setScale(-1, 1);
   self.droneLA:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._introAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );
   self.droneRA:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._introAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );
	self.droneLB:swapImage( self._target_texture );
	self.droneRB:swapImage( self._target_texture );
	self.droneRB:setScale(-1, 1);
   self.droneLB:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._introAnim.anim.props[2], MOAIProp2D.TRANSFORM_TRAIT );
   self.droneRB:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._introAnim.anim.props[2], MOAIProp2D.TRANSFORM_TRAIT );

   self._introAnim:setVisible(true);
   self._introAnim:setTime(0);
   self._introAnim:start();

   EventMgr:postEvent( {sounds = {start={"DroneMergeNotify",},}, name="drone merge notify"}, "add-sfx" );

end

------------------------------------------------------------------------------------

function StateDroneMerge:_mergeAnimStart()

   disableButtonInput();
   self.MergeButton:disable();
   self.MergeButton:setVisible(false);
   self.InfoText:setVisible(false);
   self.InfoTextShdw:setVisible(false);
   self.MessageText:setVisible(false);
   self.MessageTextShdw:setVisible(false);
   self._introAnim:setVisible(false);

   self._evolveAnim:setVisible(true);
   self._evolveAnim:setTime(0);
   self._evolveAnim:start();

   EventMgr:postEvent( {sounds = {start={"DroneMerge",},}, name="drone merge"}, "add-sfx" );

   delay(   23/30,
            function()
               self.droneLA:setVisible(false);
               self.droneRA:setVisible(false);
               self.droneLB:setVisible(false);
               self.droneRB:setVisible(false);
               self:_hideStars(self.starsA);
               self:_hideStars(self.starsB);
            end );

   delay(   70/30,
            function()
               if self then
                  self:_mergeDroneReveal();
               end
            end );

end

------------------------------------------------------------------------------------

function StateDroneMerge:_mergeDroneReveal()

   if self._side == DRONE_LEFT then
      self._target_rarity	= DroneMgr:getDroneRarityLeft(			self._target_index );
      self._target_texture	= DroneMgr:getDroneBodyVisLeft(			self._target_index );
   else
      self._target_rarity	= DroneMgr:getDroneRarityRight(			self._target_index );
      self._target_texture	= DroneMgr:getDroneBodyVisRight(			self._target_index );
   end

   self.InfoText:setVisible(false);
   self.InfoTextShdw:setVisible(false);
	self.GGlow:setVisible(true);
	self.GRays:setVisible(true);
	self.TitleText:setVisible(true);
   self.TitleTextShdw:setVisible(true);
	self.TitleText:setText(self._final_name);
	self.TitleTextShdw:setText(self._final_name);
	self:_showStars(self.starsReveal, self._final_rarity);
	self.droneRevealL:swapImage( self._final_texture );
	self.droneRevealR:swapImage( self._final_texture );
	self.droneRevealR:setScale(-1, 1);
	self.droneRevealL:setVisible(true);
	self.droneRevealR:setVisible(true);
	self.droneShadow:setVisible(true);
   self.droneRevealL:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );
   self.droneRevealR:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );

   self._revealAnim:setVisible(true);
   self._revealAnim:setTime(0);
   self._revealAnim:start();

   EventMgr:postEvent( {sounds = {start={"DroneReveal",},}, name="drone reveal"}, "add-sfx" );

end


------------------------------------------------------------------------------------
-- Util
------------------------------------------------------------------------------------


function StateDroneMerge:_createDroneStars(  x, y, sx, sy, priority )

	if x and y and sx and sy then
		local width		= 200 * sx;
		local height	= 100 * sy;
		local temp		= {};
		local p			= priority or PRIORITY_2;

		for i=1,DroneMgr:getDroneMaxRarity() do
			temp[i]		= self:addWidget(Image, self.mainLayer, x, y, width, height, DRONE_STARS[i]);

			temp[i]:setPriority(p);
		end

		return temp;
	end

end

------------------------------------------------------------------------------------

function StateDroneMerge:_hideStars( stars )

	if stars then
		for k,v in ipairs(stars) do
			v:setVisible(false);
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneMerge:_showStars( stars, rarity )

	if stars and rarity and stars[rarity] then
		for k,v in ipairs(stars) do
			if k == rarity then
				v:setVisible(true);
			else
				v:setVisible(false);
			end
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneMerge:_hideAll()

   disableButtonInput();
	self.GGlow:setVisible(false);
	self.GRays:setVisible(false);
	self.droneRevealL:setVisible(false);
	self.droneRevealR:setVisible(false);
	self.droneShadow:setVisible(false);
	self.TitleText:setVisible(false);
	self.TitleTextShdw:setVisible(false);
	self.InfoText:setVisible(false);
	self.InfoTextShdw:setVisible(false);
	self.MessageText:setVisible(false);
	self.MessageTextShdw:setVisible(false);
	self.droneRevealL:setVisible(false);
	self.droneRevealR:setVisible(false);
	self.droneShadow:setVisible(false);
	self:_hideStars(self.starsReveal);
	self.droneLA:setVisible(false);
	self.droneRA:setVisible(false);
	self:_hideStars(self.starsA);
	self.droneLB:setVisible(false);
	self.droneRB:setVisible(false);
	self:_hideStars(self.starsB);
   self.MergeButton:disable();
   self.MergeButton:setVisible(false);
	self.BackButton:disable();
	self.BackButton:setVisible(true);
   self._introAnim:setVisible(false);
   self._revealAnim:setVisible(false);
   self._evolveAnim:setVisible(false);

end

------------------------------------------------------------------------------------

function StateDroneMerge:_mergeDrone()

	if self._side == DRONE_LEFT then
		self._mergeRarity		= math.min( self._garage_rarity + self._target_rarity, DroneMgr:getDroneMaxRarity() );

		DroneMgr:mergeDroneToGarageSpaceLeft( self._garage_index, self._target_index );
		DroneMgr:setDroneMountLeft( self._target_index );

      self._final_rarity	= DroneMgr:getDroneRarityLeft(			self._target_index );
      self._final_texture	= DroneMgr:getDroneBodyVisLeft(			self._target_index );
      self._final_name		= DroneMgr:getDroneNameLeft(				self._target_index ).." Drone";
	else
		self._mergeRarity		= math.min( self._garage_rarity + self._target_rarity, DroneMgr:getDroneMaxRarity() );

		DroneMgr:mergeDroneToGarageSpaceRight( self._garage_index, self._target_index );
		DroneMgr:setDroneMountRight( self._target_index );

      self._final_rarity	= DroneMgr:getDroneRarityRight(			self._target_index );
      self._final_texture	= DroneMgr:getDroneBodyVisRight(			self._target_index );
      self._final_name		= DroneMgr:getDroneNameRight(				self._target_index ).." Drone";
	end

end

------------------------------------------------------------------------------------

function StateDroneMerge:isUsingCameraHack()
   return false;
end


------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------


function StateDroneMerge:goBack()
	StateMgr:pop();
end


------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------


return StateDroneMerge;


