----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local DRONE_LEFT					= "left";  -- mirror in app data
local DRONE_RIGHT					= "right"; -- mirror in app data

local PRIORITY_0					= 10;
local PRIORITY_1					= 20;
local PRIORITY_2					= 30;
local PRIORITY_3					= 40;
local PRIORITY_4					= 50;
local PRIORITY_5					= 60;
local PRIORITY_6					= 70;
local PRIORITY_7					= 80;
local PRIORITY_8					= 90;
local PRIORITY_9					= 100;
local PRIORITY_10					= 110;

local SQUARE_TEXTURE				= "resources/square.png";
local WHITE_FLASH					= "resources/UI/Bounty/WhiteFlash_small.png";
local FLASH_TIME					= 0.5;
local REVEAL_LOCK_TIME			= 0.25;
local GLOW_ROTATION_RATE		= 45;

local STATE_REVEAL_START		= "reveal-start";
local STATE_REVEAL_END			= "reveal-end";
local STATE_REVEAL_COMPLETE	= "reveal-complete";

----------------------------------------------------------------

local DRONE_STARS					=
{
	"resources/UI/Drone_UI/1Star.png",
	"resources/UI/Drone_UI/2Star.png",
	"resources/UI/Drone_UI/3Star.png",
	"resources/UI/Drone_UI/4Star.png",
	"resources/UI/Drone_UI/5Star.png",
};

--CAUTION: The order of the shadow must match the drone index order
local DRONE_SHADOW				=
{
	"resources/Vehicle/Drones/DroneGun/DroneGun_Shadow.png",
	"resources/Vehicle/Drones/DroneLaser/DroneLaser_Shadow.png",
	"resources/Vehicle/Drones/DroneBoost/DroneBoost_Shadow.png",
	"resources/Vehicle/Drones/DroneMissile/DroneMissile_Shadow.png",
	"resources/Vehicle/Drones/DroneRocket/DroneRocket_Shadow.png",
	"resources/Vehicle/Drones/DroneRepair/DroneRepair_Shadow.png",
	"resources/Vehicle/Drones/DroneShield/DroneShield_Shadow.png",
   "resources/Vehicle/Drones/DroneBarricade/DroneBarricade_Shadow.png",
};

----------------------------------------------------------------

local Base						= State;
local StateDroneReveal		= Base:new();

----------------------------------------------------------------

function StateDroneReveal:onLoad()

   State.onLoad( self );

   self.defaultFontName = "resources/font/TangoLatin.otf";
	self.IS_POPUP  = true;

	-- screen flash
	self.WhiteFlash = self:addWidget(Image, self.mainLayer, 0,0, 8192, 8192, WHITE_FLASH);
	self.WhiteFlash:setPriority(PRIORITY_10);

	-- god rays
	self.GGlow = self:addWidget(Image, self.mainLayer, 0, -54, 640, 640, "resources/UI/Drone_UI/Backglow-01.png");
	self.GGlow:setPriority(PRIORITY_2-2);
	self.GRays = self:addWidget(Image, self.mainLayer, 0, -54, 1200, 1200, "resources/UI/Drone_UI/LightRays-01.png");
	self.GRays:setPriority(PRIORITY_2-1);

	-- text message
	self.TitleText = self:addWidget(TextBox, self.mainLayer, 0, 326, 540, 70, "Drone Name", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.TitleText:setScale(1.5, 1.5);
	self.TitleText:setPriority(PRIORITY_3);
	self.TitleTextShdw = self:addWidget(TextBox, self.mainLayer, 0+2, 326-2, 540, 70, "Drone Name", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.TitleTextShdw:setScale(1.5, 1.5);
	self.TitleTextShdw:setPriority(PRIORITY_3-1);
	self.MessageText = self:addWidget(TextBox, self.mainLayer, 0, -348, 500, 260, "Tap To Continue", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.MessageText:setPriority(PRIORITY_3);
	self.MessageText:setScale(1.2, 1.2);
	self.MessageTextShdw = self:addWidget(TextBox, self.mainLayer, 0+2, -348-2, 500, 260, "Tap To Continue", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.MessageTextShdw:setPriority(PRIORITY_3-1);
	self.MessageTextShdw:setScale(1.2, 1.2);

   -- reveal anim
   local droneSheet  = FlashMgr:getTextureSheet("resources/flash-resource-bundle/Drone_Effects.lua", "resources/flash-resource-bundle/Drone_Effects.png", 30);

   self._revealAnim  = self:addWidget(Anim, self.mainLayer, 0, -54, 0,0, "Dronereveal_Dronepopup", droneSheet);
   self._revealAnim:setVisible(false);
   self._revealAnim:setScale(1/11.14, 1/9.14);
   self._revealAnim:setListener( MOAIAction.EVENT_STOP,
                                 function()
                                    enableButtonInput();
                                    self.BackButton:enable();
                                    self.MessageText:setVisible(true);
                                    self.MessageTextShdw:setVisible(true);
                                    self._revealAnim:setVisible(false);
                                 end );

	-- pic
	self.droneRevealL	= self:addWidget(Image, self.mainLayer, -96, 0, 192, 384, "resources/UI/Drone_UI/sill.png");
	self.droneRevealL:setPriority(PRIORITY_3);
	self.droneRevealR	= self:addWidget(Image, self.mainLayer, 96, 0, 192, 384, "resources/UI/Drone_UI/sill.png");
	self.droneRevealR:setPriority(PRIORITY_3);
	self.droneShadow	= self:addWidget(Image, self.mainLayer, 0+40, -40, 256, 256, "resources/UI/Drone_UI/sill.png");
	self.droneShadow:setPriority(PRIORITY_3-1);
	self.starsReveal	= self:_createDroneStars( 0, 210, 1.55, 1.45 );

	-- Exit Button
	self.BackButton = self:addWidget(ButtonImage, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE, self.goBack, nil, nil, nil);
	self.BackButton:setPriority(PRIORITY_0);
	self.BackButton:setColor(0,0,0, 0.9);

end

----------------------------------------------------------------

function StateDroneReveal:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StateDroneReveal:onFocus( data )

   State.onFocus(self);

	Debug.assert(data and data.garage_index, "StateDroneReveal:onFocus -> drone garage index is nil");
	Debug.assert(data and data.side, "StateDroneReveal:onFocus -> pool is nil");

	self._state				= STATE_REVEAL_START;
	self._revealTimer		= 0;
	self._garage_index	= data.garage_index;
	self._side				= data.side;
	self._type				= nil;
	self._rarity			= nil;
	self._texture			= nil;
	self._name				= nil;

   self:_hideAll();

   if self._side == DRONE_LEFT then
      self._type		= DroneMgr:getDroneTypeLeft(		self._garage_index );
      self._rarity	= DroneMgr:getDroneRarityLeft(	self._garage_index );
      self._texture	= DroneMgr:getDroneBodyVisLeft(	self._garage_index );
      self._name		= DroneMgr:getDroneNameLeft(		self._garage_index );
   else
      self._type		= DroneMgr:getDroneTypeRight(		self._garage_index );
      self._rarity	= DroneMgr:getDroneRarityRight(	self._garage_index );
      self._texture	= DroneMgr:getDroneBodyVisRight(	self._garage_index );
      self._name		= DroneMgr:getDroneNameRight(		self._garage_index );
   end

   self._name	= string.upper(self._name.." Drone");

   App.data:setValue("new-drone-type-"..self._type, true);

end

----------------------------------------------------------------

function StateDroneReveal:onDataMerged()

	Base.onDataMerged(self);
   self:goBack();
   
end

----------------------------------------------------------------

function StateDroneReveal:onUpdate( dt )

	Base.onUpdate(self, dt);

	self:_updateFlashTime(dt);
	self:_updateGGlowRayRotate(dt);

end


------------------------------------------------------------------------------------
-- Anim Update
------------------------------------------------------------------------------------


function StateDroneReveal:_updateFlashTime( dt )

	if self._state == STATE_REVEAL_START then
		self._revealTimer	= self._revealTimer + dt;

		local k	= math.min( self._revealTimer/FLASH_TIME, 1);

		self.WhiteFlash:setVisible(true);
		self.WhiteFlash:setScale(k, k);
		self.WhiteFlash:setAlpha(1);

		if self._revealTimer >= FLASH_TIME then
			self._state			= STATE_REVEAL_END;
			self._revealTimer	= 0;

			self:_revealDrone();
		end
	elseif self._state == STATE_REVEAL_END then
		self._revealTimer	= self._revealTimer + dt;

		local a = math.clamp(1 - self._revealTimer/REVEAL_LOCK_TIME, 0, 1);

		self.WhiteFlash:setVisible(true);
		self.WhiteFlash:setAlpha(a);

		if self._revealTimer >= REVEAL_LOCK_TIME then
			self._state	= STATE_REVEAL_COMPLETE;
		end
	else
		self.WhiteFlash:setVisible(false);
	end

end

------------------------------------------------------------------------------------

function StateDroneReveal:_updateGGlowRayRotate( dt )

	if self._state ~= STATE_REVEAL_START then
		self.GGlow:setVisible(true);
		self.GRays:setVisible(true);


		local glowAngle	= math.clampAngle360( self.GGlow:getRot() + GLOW_ROTATION_RATE*dt );
		local rayAngle		= math.clampAngle360( self.GRays:getRot() - GLOW_ROTATION_RATE*dt );

		self.GGlow:setRot( glowAngle );
		self.GRays:setRot( rayAngle );
	end

end


------------------------------------------------------------------------------------
-- Reveal Drone
------------------------------------------------------------------------------------


function StateDroneReveal:_revealDrone()

	self.GGlow:setVisible(true);
	self.GRays:setVisible(true);
	self.TitleText:setVisible(true);
	self.TitleTextShdw:setVisible(true);
	self.droneRevealL:setVisible(true);
	self.droneRevealR:setVisible(true);
	self.droneShadow:setVisible(true);
	self:_showStars(self.starsReveal, self._rarity);
	self.BackButton:setVisible(true);

	self.TitleText:setText(self._name);
	self.TitleTextShdw:setText(self._name);

	self.droneRevealL:swapImage(self._texture);
	self.droneRevealR:swapImage(self._texture);
	self.droneRevealR:setScale(-1, 1);
	self.droneShadow:swapImage(DRONE_SHADOW[self._type]);

   self.droneRevealL:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );
   self.droneRevealR:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );
   self.droneShadow:setAttrLink( MOAIProp2D.INHERIT_TRANSFORM, self._revealAnim.anim.props[1], MOAIProp2D.TRANSFORM_TRAIT );

   self._revealAnim:setTime(0);
   self._revealAnim:start();

end


------------------------------------------------------------------------------------
-- Util
------------------------------------------------------------------------------------


function StateDroneReveal:_createDroneStars(  x, y, sx, sy, priority )

	if x and y and sx and sy then
		local width		= 200 * sx;
		local height	= 100 * sy;
		local temp		= {};
		local p			= priority or PRIORITY_2;

		for i=1,DroneMgr:getDroneMaxRarity() do
			temp[i]		= self:addWidget(Image, self.mainLayer, x, y, width, height, DRONE_STARS[i]);

			temp[i]:setPriority(p);
		end

		return temp;
	end

end

------------------------------------------------------------------------------------

function StateDroneReveal:_hideStars( stars )

	if stars then
		for k,v in ipairs(stars) do
			v:setVisible(false);
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneReveal:_showStars( stars, rarity )

	if stars and rarity and stars[rarity] then
		for k,v in ipairs(stars) do
			if k == rarity then
				v:setVisible(true);
			else
				v:setVisible(false);
			end
		end
	end

end

------------------------------------------------------------------------------------

function StateDroneReveal:_hideAll()

   disableButtonInput();
	self.WhiteFlash:setVisible(false);
	self.GGlow:setVisible(false);
	self.GRays:setVisible(false);
	self.TitleText:setVisible(false);
	self.TitleTextShdw:setVisible(false);
	self.MessageText:setVisible(false);
	self.MessageTextShdw:setVisible(false);
	self.droneRevealL:setVisible(false);
	self.droneRevealR:setVisible(false);
	self.droneShadow:setVisible(false);
	self:_hideStars(self.starsReveal);
	self.BackButton:disable();
	self.BackButton:setVisible(false);
   self._revealAnim:setVisible(false);

end

------------------------------------------------------------------------------------

function StateDroneReveal:isUsingCameraHack()
   return false;
end


------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------


function StateDroneReveal:goBack()
	StateMgr:pop();
end


------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------


return StateDroneReveal;


