----------------------------------------------------------------

require('app/contact/contact-manager')
require('state/state')
require('state/marker/marker-manager')
require('app/promo/promo-manager')
----------------------------------------------------------------

require('engine/event/event-manager')
require('ui/ui-image')
require('state/overlay/pause-transition')

require('flash/flash-manager')
require('ui/ui-anim');

----------------------------------------------------------------

local Base      = State;
StateGame = Base:new();

----------------------------------------------------------------

local DAMAGE_SCREEN_ALPHA = 0.65;

local REVIVE_ANIM_SPEED_K  = 1.25;
local REVIVE_TIMER_RESET	= 1.6/REVIVE_ANIM_SPEED_K;
local REVIVE_CLEAR_TIME    = 0.50;
local REVIVE_POPUP_DELAY   = 0.75;

local ALERT_TEXT_IN			= "alert-text-in";
local ALERT_TEXT_HOLD		= "alert-text-hold";
local ALERT_TEXT_OUT			= "alert-text-out";
local ALERT_RATE_IN			= App.viewportWidth*3/2;
local ALERT_RATE_OUT			= App.viewportWidth*2;
local ALERT_RATE_PULSE		= 12/30;
local ALERT_TIME_HOLD		= 1.5;
local HEALTH_CHUNK_PAUSE	= 0.5;
local HEALTH_CHUNK_TIME		= 1;
local SHIELD_ANIM_TIME		= 4/15;
local SHIELD_ANIM_SCALE		= 20;
local	HUD_PULSE_SCALE		= 1.5;
local HUD_PULSE_TIME			= 0.25;

----------------------------------------------------------------

StateGame.layer =
{
   ui          =  {},
   foreground  =  {},
   game        =  {},
   background  =  {},
};

StateGame.shouldShowAd = false;

----------------------------------------------------------------

function  StateGame:onLoad(  )

   Base.onLoad(self);

   self.reviveTextures = FlashMgr:getTextureSheet("resources/flash-resource-bundle/Group2.lua",
                                                  "resources/flash-resource-bundle/Group2.png",
                                                  30);

   TextureMgr:getTexture("resources/square.png");
   TextureMgr:getTexture("resources/UI/HUD_ArmorBarFrame.png");
   --TextureMgr:getTexture("resources/UI/whitebox.png");
   TextureMgr:getTexture("resources/UI/HUD_ArmorShieldIcon.png");
   TextureMgr:getTexture("resources/UI/LowArmor_Message.png");
   TextureMgr:getTexture("resources/Pickup/Pickup_Coin.png");

   self.defaultFontName = "resources/font/TangoLatin.otf";
   self.unicodeFontName = "resources/font/Tango.otf";

   self.pauseTransition = PauseTransition:new( self );

   -- black overlay for quit transition
   local gfxQuad  = MOAIGfxQuad2D.new ();
   local w = App.viewportWidth;
   local h = App.viewportHeight;
   gfxQuad:setTexture ( TextureMgr:getTexture("resources/UI/blackbox.png") );
   gfxQuad:setRect ( -w/2, -h/2, w/2, h/2 );
   self.Overlay  = MOAIProp2D.new ();
   self.Overlay:setDeck ( gfxQuad );
   self.Overlay:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA );
   self.Overlay:setLoc(0,0);
   self.Overlay:setColor(1,1,1,0.8);
   self.shouldShowAd = false;
end

----------------------------------------------------------------

function  StateGame:onActivate(  )

   Base.onActivate(self);

   -- Dont clear cache because we just cached sounds
   -- We cache before the game start to not "freeze" on the game screen
   -- but on the darken screen on top of the ranking screen (better UX)
   -- AudioMgr:clearCache();

   self:disableAds();

   ContactMgr:setDownloadPictureEnable( false );

   self.isUnpausing = false;
   EventMgr:addListener( self, self.pauseTransitionStart,   "pause-transition");
   EventMgr:addListener( self, self.deathSaveStart,         "death-save");
   EventMgr:addListener( self, self._enableAlertText,			"hud-alert-text");
   --EventMgr:addListener( self, self.onHeal,						"player-avatar-heal");

   EventMgr:addListener( self, self._onBountySpawned, "bounty-vehicle-spawned");
   EventMgr:addListener( self, self._onBountyRemoved, "bounty-vehicle-removed");

   EventMgr:addListener( self, self.onConsumableUsed, "consumable-used");

   -- MQL: mission and achievement HUD targets do not match up due to the precalculated target bonus score
   --	EventMgr:addListener( self, self._setAchievementScoreObjective,	"achv-score-objective");
   --	EventMgr:addListener( self, self._setMissionScoreObjective,       "mission-score-objective");

   -- EventMgr:addListener( self, self.onUnpause,         "game-unpaused");



   self.firstName = "";
   self.lastName  = "";
   local profile   = ContactMgr:getProfile();

   if profile then
      self.firstName = profile:getFirstName();
      self.lastName  = profile:getLastName();
   end

   self.layerTable          = {};
   self.game                = dofile ( App.gameFile )
   App.game                 = self.game;
   self.displayedScore      = 0;
   self.gameOverCount       = 0;
   self.playerHiScore       = RankingMgr:getPlayerHiScore();
   self.previousRank        = nil;
   self.alertTextTimer      = 0;
   self.alertTextPulseTimer = 0;
   self._healVfxCd          = 0;
   self._healthRatioPrev    = 1;
   self._healthChunkTimer   = 0;
   self._shieldRatioPrev    = 0;
   self._shieldChunkTimer   = 0;
   self._shieldAnimTimer    = 0;
   self._shieldPulseTimer   = 0;
   App.reviveCount          = 0;
   self.numActiveBountyCars = 0;

   self.Overlay:setVisible(false);

   self.quitting = false;

   self:createAllLayers();

   self:createHud();

   self.game:onActivate( self );

   self:updateHud(0);

   self.monitorFps =
      {
         frames = 1; --to avoid divide by 0
         bad    = 0;
         stamp  = nil;
      };

   self.enginePlaying = false;

   ContactMgr:setAutoContactsMode( ContactMgr.AUTO_FRIENDS_DISABLE );

   MarkerMgr:init( self, self.game );

   --saves old hiScore to compare during the results screen
   App.data.lastHiScore = RankingMgr:getPlayerHiScore();

   self.pauseTransition:setLayers( self.layer.ui[1] )
   self.layer.ui[1]:insertProp( self.Overlay );

   MessageMgr:setEnableAutoCheck( false );

   self:rewardFunnelCheck( );

   self.revivePopupDelay = 0;

   self.reviveAnim = self:addWidget( Anim, self.layer.ui[1], 0,0,0,0, "R_Rebuild", self.reviveTextures );
   self.reviveAnim:setVisible(false);
   self.reviveAnimTimer = REVIVE_TIMER_RESET;

   self.ringPlaying = false;
   self.game.avatar.deathExplosion = nil;
   self.gameStarted = false;

   self.consumablesScreenEnabled = self:isConsumableEnabled();
   self._queuedBounties = {};
end

----------------------------------------------------------------

function StateGame:onFocus()

   Base.onFocus(self);

   if self.quitting then
      self.Overlay:setVisible(true);
      self.quitting = false;
   end

   --disable clear color
   MOAIGfxDevice.setClearColor (0,0,0);

local lastAdSeenIndexKey = "last-post-game-ad-seen-index";
local idx = App.data:getValue(lastAdSeenIndexKey) or 0;

if App.AfterGameInterstitialFailed then
    App.AfterGameInterstitialFailed = false;
    if idx > 0 then
        idx = idx - 1
    end

    App.data:setValue(lastAdSeenIndexKey,idx);
end

   if App.playerRevive == "yes" then

      self.playRevive = true;
      self:reviveTransition();

   elseif App.playerRevive  == "no" then
      self.game.avatar:onDie();
      GamePaused = false;
      if App.shouldShowAd == "yes" then
         App.AfterGameInterstitialFailed = false;
         StateMgr:push("state/state-interstitial-show.lua", {rewardType="gold", rewardAmount=0} );
         App.shouldShowAd = "no";
      end
      EventMgr:postEvent( {state="state/state-pause.lua"}, "game-unpaused" );
   end

   self.hud.pauseButton:enable();

   App.playerRevive = nil;

   if not self.gameStarted then

      if self.consumablesScreenEnabled then

         self.consumablesScreenEnabled = false;
         self:hideHud();

         if AbTestMgr:isConsumableStoreEnabled() then
            StateMgr:push( "state/state-consumable-overlay-new.lua", self );
         else
            StateMgr:push( "state/state-consumable-overlay.lua", self );
         end
      else

         self.gameStarted = true;
         self:showHud();
         self:onStart();
      end
   end

   self.bountyList = BountyMgr:getBountySelection();

end

----------------------------------------------------------------

function StateGame:SetShouldShowAd ( val )
   self.shouldShowAd = val;
end



----------------------------------------------------------------

function StateGame:onDeactivate(  )

   Base.onDeactivate(  self );

   AudioMgr:clearCache();

   EventMgr:rmvListener( self );

   ContactMgr:setDownloadPictureEnable( true );

   self.pauseTransition:resetWidgets();

   self:deleteAllWidgets();

   MarkerMgr:free();

   if self.game then
      self.game:onDeactivate();
      self.game:onEnded();
      self.game = nil;
      App.game  = nil;
   end

   MessageMgr:setEnableAutoCheck( true );

   --This is necessary to force garbage collection
   self.layerTable = {};
   self.camera = {};
   self.layer.background = {};
   self.layer.game = {};
   self.layer.foreground = {};
   self.layer.ui = {};

   local badRatio = math.floor((self.monitorFps.bad/self.monitorFps.frames)*100);

   local rating = nil;

   if badRatio < 1 then
      rating = "Perfect";
   elseif badRatio < 3 then
      rating = "Great";
   elseif badRatio < 5 then
      rating = "Good";
   elseif badRatio < 10 then
      rating = "Okay";
   elseif badRatio < 20 then
      rating = "Bad";
   else
      rating = "Horrible";
   end

   MOAIFlurry:event("FrameRate-Report"
                    , "ratio",              tostring(badRatio)
                    , "bad",                tostring(self.monitorFps.bad)
                    , "frames",             tostring(self.monitorFps.frames)
                    , "type",               tostring(rating)
   );

end

----------------------------------------------------------------

function StateGame:onLoseFocus( )

   Base.onLoseFocus(self);

   MOAIGfxDevice.setClearColor (0,0,0);

   self.monitorFps.stamp = nil;

   if ( not self._gamePaused ) then
      if self.game then
         self._gamePaused = true;
         self.game:onPause();
      end
   end

end


----------------------------------------------------------------

function StateGame:onInput( )

   Base.onInput(self);

   self.game:onInput();
end

----------------------------------------------------------------

function StateGame:onStart()

   self.game:onStarted();
end

----------------------------------------------------------------

function StateGame:_monitorFps()

   if self.monitorFps.stamp then
      local delta = MOAISim.getDeviceTime() - self.monitorFps.stamp;
      if delta > 0.040 then
         self.monitorFps.bad = self.monitorFps.bad + 1;
      end

   end

   self.monitorFps.stamp = MOAISim.getDeviceTime();
   self.monitorFps.frames = self.monitorFps.frames + 1;
end


----------------------------------------------------------------

function StateGame:onUpdate( dt )

   Base.onUpdate(self,dt);

   self:_monitorFps();

   if not self.enginePlaying then

      EventMgr:postEvent( {sounds = {loop={"EngineLoop",},}, name="stategame update"}, "add-sfx" );

      self.enginePlaying = true;
   end

   if self.isUnpausing then
      self.pauseTransition:countdownUpdate( dt );
   end

   self:_updateRevive(dt);

   self.displayedScore = math.floor( (self.game.distanceScore + self.game.killScore) );

   -------------------------------------
   --Handle Pause/Resume Transition
   if ( GamePaused ) and ( not self._gamePaused ) then

      self._gamePaused = true;
      self.game:onPause();

   elseif (not GamePaused) and ( self._gamePaused ) then

      self._gamePaused = false;
      self.game:onResume();

   end
   -------------------------------------

   if not GamePaused then
      self.game:onUpdate(dt);

   elseif self.game.gameDirector.scenario then
      if self.game.gameDirector.scenario.tutorialActive then
         self.game.gameDirector.scenario:update(dt);
      end
   end

   self:_queueBountiesIfNeeded(self.displayedScore);
   self:updateHud(dt);
   self:updateAlertText(dt);

   MarkerMgr:update(dt);

   if self.game.gameOver == true then

      self.gameOverCount = self.gameOverCount + 1;

      if self.gameOverCount == 1 then

         self:onGameOver();

      end

      if self.scoreUpdated and self.postGameDataUpdated then

         StateMgr:clear ( "state/state-results.lua" )

      end
   end

	self._healVfxCd	 = math.max(self._healVfxCd - dt, 0);

end

----------------------------------------------------------------

function StateGame:onConsumableUsed( event )

   if       event
      and   event.type
      and   self
      and   self.game
      and   self.game.avatar then

      local index = ConsumableMgr:getTemplateIndexByType(event.type);

      if ConsumableMgr:isSingleUse(index) then
         self.game.avatar:onConsumable( ConsumableMgr:getTemplate(index) );
      end
   end

end

----------------------------------------------------------------

function StateGame:isConsumableEnabled()

	if				DroneMgr:isEnabled()
			and	self.game.avatar:hasChargeRequiredDrone()
			and	not App.data:getValue("droneChargeTutorialFinished", false) then

		return false;
	else
		return App.data:getValue("consumables-enabled" ) and App.data:getValue("tutorialDone") == TUTORIAL_DONE_NUMBER;
	end

end

----------------------------------------------------------------

function StateGame:onHeal( event )

	if			event
		and	event.hudVFX
		and	self._healVfxCd
		and	self._healVfxCd <= 0 then

		self._healVfxCd	= event.hudVFX.duration;
		self.game.entityMgr:spawnExplosion(event.hudVFX, 0, App.viewportHeight/2-38, 0, self.hud.lifeFrame.prop);
	end

end

----------------------------------------------------------------

function StateGame:hasGameStarted()
	return self.gameStarted;
end

----------------------------------------------------------------

function StateGame:_updateRevive( dt )

   if self.revivePopupDelay > 0 then
      self.revivePopupDelay = self.revivePopupDelay - dt;
      if self.revivePopupDelay <= 0 then
         self.revivePopupDelay = 0;
         self:deathSavePush();
      end
   elseif self.playRevive then
      self:reviveUpdate(dt);
   end

   if self.ringPlaying then
      self:screenClearUpdate(dt)
   end

end

----------------------------------------------------------------

function StateGame:updateAlertText( dt )

	if self._stateAlertText then
		local x,y = self.hud.alertText:getLoc();
		local k;

		self.hud.alertText:show();
		self.hud.alertTextShdw:show();

		self.alertTextPulseTimer = self.alertTextPulseTimer + dt;

		if self.alertTextPulseTimer > ALERT_RATE_PULSE then
			self.alertTextPulseTimer = 0;
		end

		if self.alertTextPulseTimer < ALERT_RATE_PULSE/2 then
			k = math.mapClamped(0, 1, ALERT_RATE_PULSE/2, 0.75, self.alertTextPulseTimer);
		else
			k = math.mapClamped(ALERT_RATE_PULSE/2, 0.75, ALERT_RATE_PULSE, 1, self.alertTextPulseTimer);
		end

		self.hud.alertText:setScale(k, k);
		self.hud.alertTextShdw:setScale(k, k);

		if self._stateAlertText == ALERT_TEXT_IN then
			x	= math.max(x - ALERT_RATE_IN*dt, 0);

			self.hud.alertText:setLoc(x, y);
			self.hud.alertTextShdw:setLoc(x+3, y-3);
			self.hud.alertText:setScale(1, 1);
			self.hud.alertTextShdw:setScale(1, 1);

			if x <= 0 then
				self._stateAlertText = ALERT_TEXT_HOLD;
			end
		elseif self._stateAlertText == ALERT_TEXT_OUT then
			x	= math.max(x - ALERT_RATE_OUT*dt, -App.viewportWidth);

			self.hud.alertText:setLoc(x, y);
			self.hud.alertTextShdw:setLoc(x+3, y-3);
			self.hud.alertText:setScale(k, k);
			self.hud.alertTextShdw:setScale(k, k);

			if x <= -App.viewportWidth then
				self._stateAlertText = nil;
			end
		else
			self.alertTextTimer = self.alertTextTimer + dt;

			self.hud.alertText:setScale(k, k);
			self.hud.alertTextShdw:setScale(k, k);

			if self.alertTextTimer >= ALERT_TIME_HOLD then
				self._stateAlertText = ALERT_TEXT_OUT;
			end
		end
	end

end

----------------------------------------------------------------

function StateGame:onGameOver()

   --There is a having the player never die...
   -- make sure we cant save our score by quiting

   App.lastGameTime     = math.floor(self.game.duration);
   App.lastGameDistance = math.floor(self.game.distanceScore);
   App.lastGameKill     = math.floor(self.game.killScore);
   App.lastGameBonus    = math.floor((App.lastGameDistance + App.lastGameKill) * App.data:getScoreBonus());
   App.lastGameScore    = App.lastGameDistance + App.lastGameKill + App.lastGameBonus;
   App.lastGameGold     = math.floor(self.game.currency);

   self.scoreUpdated          = false;
   self.postGameDataUpdated   = false;
   self:updatePostGameData();
   self:submitScore( App.lastGameScore );

   PromoMgr:gamePlayed();

end

----------------------------------------------------------------

-- Don't increment active bounty cars - we do that elsewhere.
function StateGame:_onBountySpawned( event )

   self.lastSpawnedBounty = event.player;

end

----------------------------------------------------------------

function StateGame:_onBountyRemoved( event )

   self.numActiveBountyCars = self.numActiveBountyCars - 1;

end

----------------------------------------------------------------

function StateGame:_enableAlertText( event)

	if			event
		and	event.text
		and	not self._stateAlertText then

		self._stateAlertText	= ALERT_TEXT_IN;
		self.alertTextTimer	= 0;

		self.hud.alertText:setText(event.text);
		self.hud.alertTextShdw:setText(event.text);
		self.hud.alertText:setLoc(App.viewportWidth, App.viewportHeight/2-270);
		self.hud.alertTextShdw:setLoc(App.viewportWidth+3, App.viewportHeight/2-273);

		if event.color and type(event.color) == "table" then
			self.hud.alertText:setColor(event.color[1], event.color[2], event.color[3], event.color[4]);
		else
			self.hud.alertText:setColor(0, 1, 1, 1);
		end
	end

end

----------------------------------------------------------------

function StateGame:_setAchievementScoreObjective( event )

   if event and event.objective then
      self.scoreAchObj = event.objective;
   end

end

----------------------------------------------------------------

function StateGame:_setMissionScoreObjective( event )

   if event and event.objective then
      self.scoreMisObj = event.objective;
   end

end

----------------------------------------------------------------

function StateGame:_getMisAchObjective( rank, targetScore, playerScore )

   if self.scoreAchObj and self.scoreAchObj <= playerScore then
      self.scoreAchObj = nil;
   end

   if self.scoreMisObj and self.scoreMisObj <= playerScore then
      self.scoreMisObj = nil;
   end

   -- No mission or objective available
   if not self.scoreAchObj and not self.scoreMisObj then
      return nil;
   end

   local title = "Achievement";
   local score = self.scoreAchObj or 0;

   if self.scoreMisObj and self.scoreMisObj <= score then
      title = "Mission";
      score = self.scoreMisObj;
   end

   local deltaScore	= score - playerScore;

   if rank == 1 and targetScore - playerScore <= 0 and score > targetScore then

      return {name=title, dScore=deltaScore};

   elseif score < targetScore then

      return {name=title, dScore=deltaScore};
   else

      return nil;
   end

end

----------------------------------------------------------------

function StateGame:hideHud()

   self.hud.scoreDivider:hide();
   self.hud.lifeFrame:hide();
   self.hud.lifeDelta:hide();
   self.hud.lifeBar:hide();
   self.hud.lifeIcon:hide();
   self.hud.lifeBarGlow:hide();
   self.hud.shieldFrame:setVisible(false);
   self.hud.shieldDelta:setVisible(false);
   self.hud.shieldBar:setVisible(false);
   self.hud.shieldCover:setVisible(false);
   self.hud.shieldIcon:setVisible(false);

   if self.hud.gradient then
      self.hud.gradient:hide();
   end

   if self.hud.scoreTitleBox then
      self.hud.scoreTitleBox:hide();
      self.hud.scoreTitleBox2:hide();
   end

   if self.hud.scoreBox then
      self.hud.scoreBox:hide();
      self.hud.scoreBox2:hide();
   end

   if self.hud.coin then
      self.hud.coin:hide();
      self.hud.coinBox:hide();
      self.hud.coinBox2:hide();
   end

   if self.hud.pauseButton then
      self.hud.pauseButton:hide();
      self.hud.pauseButton:disable();
   end

   if self.hud.targetTitleBox then
      self.hud.targetTitleBox:hide();
      self.hud.targetTitleBox2:hide();
      self.hud.targetBox:hide();
      self.hud.targetBox2:hide();
   end

   if self.hud.alertText then
      self.hud.alertText:hide();
      self.hud.alertTextShdw:hide();
   end

end

----------------------------------------------------------------

function StateGame:showHud()

   if self.hud.lifeFrame then
      self.hud.scoreDivider:show();
      self.hud.lifeFrame:show();
      self.hud.lifeDelta:show();
      self.hud.lifeBar:show();
      self.hud.lifeIcon:show();
      self.hud.lifeBarGlow:hide();
      self.hud.shieldFrame:show();

      if self.game.avatar:isShieldActive() and self.game.avatar:isShieldActive() > 0 then
         self.hud.shieldFrame:setVisible(true);
         self.hud.shieldDelta:setVisible(true);
         self.hud.shieldBar:setVisible(true);
         self.hud.shieldCover:setVisible(true);
         self.hud.shieldIcon:setVisible(true);
      end
   end

   if self.hud.gradient then
      self.hud.gradient:show();
   end

   if self.hud.scoreTitleBox then
      self.hud.scoreTitleBox:show();
      self.hud.scoreTitleBox2:show();
      self.hud.scoreBox:show();
      self.hud.scoreBox2:show();
   end

   if self.hud.coin  then
      self.hud.coin:show();
      self.hud.coinBox:show();
      self.hud.coinBox2:show();
   end

   if self.hud.pauseButton then
      self.hud.pauseButton:show();
      self.hud.pauseButton:enable();
   end

   if self.hud.targetTitleBox then
      self.hud.targetTitleBox:show();
      self.hud.targetTitleBox2:show();
      self.hud.targetBox:show();
      self.hud.targetBox2:show();
   end

   if self.hud.alertText then
      self.hud.alertText:hide();
      self.hud.alertTextShdw:hide();
   end

end
----------------------------------------------------------------

function StateGame:createHud()

   self.hud              = {};

   local screenWidth     = App.viewportWidth;
   local screenHeight    = App.viewportHeight;

   self.hud.damageScreen = self:addWidget( Image,  self.layer.ui[1], 0, 0, screenWidth, screenHeight,  "resources/square.png", { 1, 0, 0, 0} );

   -- self.hud.gradient  = self:addWidget( Image,  self.layer.ui[1], 0, screenHeight/2-64,  screenWidth, 200, "resources/UI/HUD_TopGradient.png", {1,1,1,1} );

   self.hud.scoreDivider = self:addWidget( Image,  self.layer.ui[1], 0, screenHeight/2-38, 768, 6, "resources/UI/HUD_ScoreDivider.png",   {1,1,1,1} );

   self.hud.lifeFrame    = self:addWidget( Image,  self.layer.ui[1], 0, screenHeight/2-38, 250, 52,     "resources/UI/HUD_ArmorBarFrame.png",       {1,1,1,1} );
   self.hud.lifeDelta    = self:addWidget( Image,  self.layer.ui[1], 0, 0,      230,        34,         "resources/square.png",                    {1,1,1,1} );
   self.hud.lifeBar      = self:addWidget( Image,  self.layer.ui[1], 0, 0,      230,        34,         "resources/square.png",                    {1,1,1,1} );
   self.hud.lifeCover    = self:addWidget( Image,  self.layer.ui[1], 0, 0,      230,            34,         "resources/UI/HUD_ArmorMeshCover.png",      {1,1,1,1} );
   self.hud.lifeIcon     = self:addWidget( Image,  self.layer.ui[1], 0, 0,      28,         28,         "resources/UI/HUD_ArmorShieldIcon.png", {1,1,1,1} );
   self.hud.lifeBarGlow  = self:addWidget( Image,  self.layer.ui[1], 0, 0,      250,            52,         "resources/UI/HUD_ArmorBarFrameGlow.png", {1,1,1,1} );

   self.hud.shieldFrame  = self:addWidget( Image,  self.layer.ui[1], 0, -32,    270,        44,         "resources/UI/HUD_ShieldBarFrame.png",      {1,1,1,1} );
   self.hud.shieldDelta  = self:addWidget( Image,  self.layer.ui[1], 16,    -4, 224,        22,         "resources/square.png",                    {1,1,1,1} );
   self.hud.shieldBar    = self:addWidget( Image,  self.layer.ui[1], 16,    -4, 224,        22,         "resources/square.png",                    {0.1, 0.97, 0.86, 1} );
   self.hud.shieldCover  = self:addWidget( Image,  self.layer.ui[1], 16,    -4, 224,        22,         "resources/UI/HUD_ShieldCover.png",         {1,1,1,1} );
   self.hud.shieldIcon   = self:addWidget( Image,  self.layer.ui[1], -115,  0,  32,         32,         "resources/UI/HUD_ShieldIcon.png",          {1,1,1,1} );

   self.hud.lifeFrame:setPriority(10);
   self.hud.lifeDelta:setPriority(20);
   self.hud.lifeBar:setPriority(30);
   self.hud.lifeCover:setPriority(35);
   self.hud.lifeIcon:setPriority(40);
   self.hud.lifeBarGlow:setPriority(45);
   self.hud.lifeDelta:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM, self.hud.lifeFrame.prop, MOAIProp2D.TRANSFORM_TRAIT );
   self.hud.lifeBar:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM, self.hud.lifeFrame.prop, MOAIProp2D.TRANSFORM_TRAIT );
   self.hud.lifeCover:setParent( self.hud.lifeFrame.prop );
   self.hud.lifeIcon:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM, self.hud.lifeFrame.prop, MOAIProp2D.TRANSFORM_TRAIT );
   self.hud.lifeBarGlow:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM, self.hud.lifeFrame.prop, MOAIProp2D.TRANSFORM_TRAIT );

   self.hud.shieldFrame:setPriority(50);
   self.hud.shieldDelta:setPriority(60);
   self.hud.shieldBar:setPriority(70);
   self.hud.shieldCover:setPriority(80);
   self.hud.shieldIcon:setPriority(80);
   self.hud.shieldFrame:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM, self.hud.lifeFrame.prop, MOAIProp2D.TRANSFORM_TRAIT );
   self.hud.shieldDelta:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM, self.hud.shieldFrame.prop, MOAIProp2D.TRANSFORM_TRAIT );
   self.hud.shieldBar:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM, self.hud.shieldFrame.prop, MOAIProp2D.TRANSFORM_TRAIT );
   self.hud.shieldCover:setParent( self.hud.shieldFrame.prop );
   self.hud.shieldIcon:setParent( self.hud.shieldFrame.prop );
   self.hud.shieldFrame:setVisible(false);
   self.hud.shieldDelta:setVisible(false);
   self.hud.shieldBar:setVisible(false);
   self.hud.shieldCover:setVisible(false);
   self.hud.shieldIcon:setVisible(false);

   self.hud.lowArmor          = self:addWidget( Image,  self.layer.ui[1], 0,0,                 256, 64, "resources/UI/LowArmor_Message.png", {1,1,1,0} );

   self.hud.scoreTitleBox = self:addWidget( TextBox, self.layer.ui[1],
                                                3,screenHeight/2-15-3, screenWidth-40,  100,
                                                "Guest", self.unicodeFontName, 12, nil,
                                                MOAITextBox.RIGHT_JUSTIFY,   MOAITextBox.CENTER_JUSTIFY,   {0,0,0,1}      );

   self.hud.scoreTitleBox2 = self:addWidget( TextBox, self.layer.ui[1],
                                                  0,screenHeight/2-15, screenWidth-40,  100,
                                                  "Guest", self.unicodeFontName, 12, nil,
                                                  MOAITextBox.RIGHT_JUSTIFY,   MOAITextBox.CENTER_JUSTIFY,   {1,1,1,1}         );

   self.hud.scoreBox = self:addWidget( TextBox, self.layer.ui[1],
                                       3,screenHeight/2-70-3,       screenWidth-40,  100,
                                       "",                        self.defaultFontName, 24, nil,
                                       MOAITextBox.RIGHT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY,   {0,0,0}      );

   self.hud.scoreBox2 = self:addWidget( TextBox,                   self.layer.ui[1],
                                        0,screenHeight/2-70,       screenWidth-40,  100,
                                        "",                        self.defaultFontName, 24, nil,
                                        MOAITextBox.RIGHT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY,   {1,1,1}      );


   self.hud.coin = self:addWidget( Image,  self.layer.ui[1], screenWidth/2-30,screenHeight/2-123,    46, 46, "resources/Pickup/Pickup_Coin.png", {1,1,1,1} );


   self.hud.coinBox = self:addWidget( TextBox, self.layer.ui[1],
                                         -50+3,screenHeight/2-120-3,   screenWidth-15,  100,
                                      "0",                          self.defaultFontName, 16, nil,
                                      MOAITextBox.RIGHT_JUSTIFY,     MOAITextBox.CENTER_JUSTIFY,   {0,0,0}      );

   self.hud.coinBox2 = self:addWidget( TextBox, self.layer.ui[1],
                                          -50,screenHeight/2-120,       screenWidth-15,  100,
                                       "0",                        self.defaultFontName, 16, nil,
                                       MOAITextBox.RIGHT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY,   {1,1,1}      );

   self.hud.targetTitleBox = self:addWidget( TextBox, self.layer.ui[1],
                                             3,screenHeight/2-15-3, screenWidth-40,  100,
                                             "", self.unicodeFontName, 12, nil,
                                             MOAITextBox.LEFT_JUSTIFY,   MOAITextBox.CENTER_JUSTIFY,   {0,0,0}      );

   self.hud.targetTitleBox2	= self:addWidget( TextBox, self.layer.ui[1],
                                              0,screenHeight/2-15, screenWidth-40,  100,
                                              "", self.unicodeFontName, 12, nil,
                                              MOAITextBox.LEFT_JUSTIFY,   MOAITextBox.CENTER_JUSTIFY,   {1,1,1}      );

   self.hud.targetBox = self:addWidget( TextBox, self.layer.ui[1],
                                        3,screenHeight/2-70-3,     screenWidth-40,  100,
                                        "",                        self.defaultFontName, 24, nil,
                                        MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY,   {0,0,0}      );

   self.hud.targetBox2 = self:addWidget( TextBox, self.layer.ui[1],
                                         0,screenHeight/2-70,       screenWidth-40,  100,
                                         "",                        self.defaultFontName, 24, nil,
                                         MOAITextBox.LEFT_JUSTIFY,  MOAITextBox.CENTER_JUSTIFY,   {1,1,1}      );

	self.hud.pauseButton				= self:addWidget( ButtonImage, self.layer.ui[1], -screenWidth/2+35,screenHeight/2-123, 32,32, "resources/UI/HUD_PauseButton.png", nil, self.onPause );
	self.hud.pauseButton.size[1]	= 512;
	self.hud.pauseButton.size[2]	= 512;

   self.hud.alertTextShdw		= self:addWidget(	TextBox,   self.layer.ui[1],
																screenWidth+3, screenHeight/2-273,	screenWidth, 100,
                                                "", self.unicodeFontName, 24, nil,
                                                MOAITextBox.CENTER_JUSTIFY,   MOAITextBox.CENTER_JUSTIFY,   {0,0,0,1}      );

   self.hud.alertText			= self:addWidget( TextBox,   self.layer.ui[1],
																screenWidth, screenHeight/2-270,	screenWidth, 100,
                                                "", self.unicodeFontName, 24, nil,
                                                MOAITextBox.CENTER_JUSTIFY,   MOAITextBox.CENTER_JUSTIFY,   {0,1,1,1}      );

   local firstName, lastName = self:_getFirstAndLastName();
   local text = createUserNameDisplay( firstName, lastName, false, "You" );

   self.hud.scoreTitleBox:setText(  text );
   self.hud.scoreTitleBox:preloadTextGlyphs();
   self.hud.scoreTitleBox2:setText( text );
   self.hud.scoreTitleBox2:preloadTextGlyphs();

   self:updateTarget(0);

   self.hud.debugBox = self:addWidget( TextBox, self.layer.ui[1],
                                       0, screenHeight / 2 - 100, screenWidth - 20,  50,
                                       "",                        self.defaultFontName, 14, nil,
                                       MOAITextBox.CENTER_JUSTIFY,MOAITextBox.TOP_JUSTIFY, {0.0,0.0,0.0, 0.15} );

end

----------------------------------------------------------------

function StateGame:rewardFunnelCheck( )

	local wait = App.data:getValue("tango-badge-popup-wait");
	local waitCountMission = App.data:getValue("tango-badge-popup-waitCount-mission")
	local waitCountDaily = App.data:getValue("tango-badge-popup-waitCount-daily")
	local missionComplete = App.data:isCurrentMissionCompleted();
	local dailyReward = App.data:getDailyChallengeRewardToCollect();
	if wait >= 0 and waitCountMission >= wait then
		App.data:setValue( "tango-badge-popup-waitCount-mission",  0 );
	elseif missionComplete then
		App.data:setValue( "tango-badge-popup-waitCount-mission", waitCountMission+1 );
	end
	if wait >= 0 and waitCountDaily >= wait then
		App.data:setValue( "tango-badge-popup-waitCount-daily",  0 );
	elseif dailyReward then
		App.data:setValue( "tango-badge-popup-waitCount-daily", waitCountDaily+1 );
	end

end


----------------------------------------------------------------

function StateGame:PauseGameplay()

   self:onPause();
end

----------------------------------------------------------------

function StateGame:onEnterBackground()

   Base.onEnterBackground(self);

   self:onPause();
end

----------------------------------------------------------------

function StateGame:onBackButton()

   self:onPause();
end

----------------------------------------------------------------

function StateGame:onPause()

   print("Pause/Option - Menu");

   local x, y = self.game.parent:wndToWorld (  InputMgr:getTouchLoc() );

   StateMgr:push( "state/state-pause.lua" );

end

----------------------------------------------------------------

function StateGame:onUnpause()

end

----------------------------------------------------------------

function StateGame:pauseTransitionStart( )

	self.isUnpausing = true;
   self.pauseTransition:onStart()

end

----------------------------------------------------------------

function StateGame:deathSaveStart( )

   if not self.game then
      return;
   end

   self.hud.pauseButton:disable();
   self.revivePopupDelay = REVIVE_POPUP_DELAY;

   self.game.reviving = true;

end

----------------------------------------------------------------

function StateGame:deathSavePush( )

   GamePaused = true;

   EventMgr:postEvent( {state="state/state-game.lua"}, "game-paused" );
   StateMgr:push("state/state-death-save.lua");
   self.game.reviving = false;
end

----------------------------------------------------------------

function StateGame:reviveTransition()

   local x, y = self.game.avatar:getLoc();

   self.reviveAnim:setVisible(true);
   self.reviveAnim:resetAnim();
   self.reviveAnim:setLoc(x,0);
   self.reviveAnim:setSpeed( REVIVE_ANIM_SPEED_K );
   self.reviveAnim:start();
   --We need to skip a little bit otherwise we will
   --have a visual glitch
   self.reviveAnim:setTime(0.05);
   self.hud.damageScreen:setColor(1,0,0,0);

	self._healthRatioPrev	= 1;
	self._healthChunkTimer	= 0;
	self._shieldRatioPrev	= 0;
	self._shieldChunkTimer	= 0;
	self._shieldAnimTimer	= 0;
	self._shieldPulseTimer	= 0;

 	local sSound = {start={"PlayerRevive",},};
	EventMgr:postEvent( { sounds=sSound, name="revive transition",}, "add-sfx" );
	EventMgr:postEvent({}, "revive-start");

end
----------------------------------------------------------------

function StateGame:reviveUpdate(dt)

   self.reviveAnimTimer = self.reviveAnimTimer - dt;

   if self.reviveAnimTimer <= 0 then
      self.reviveAnimTimer = REVIVE_TIMER_RESET;
      self.playRevive = false;
      self.game:onRevive();
      self.game.avatar:onRevive();

      EventMgr:postEvent( {state="state/state-pause.lua"}, "game-unpaused" );
      GamePaused = false;
      self:screenClearStart()
   end

end

----------------------------------------------------------------

function StateGame:screenClearStart()

   self.ringRadius      = 100;
   self.ringClearTime   = REVIVE_CLEAR_TIME;
   self.ringPlaying     = true;

   self.ringExpandRate = App.viewportHeight/self.ringClearTime

   --reset ring animation
   --start ring animation
   self:ringAnimHackStart();

   self.game.gameSpawner:disableSpawn();

end

----------------------------------------------------------------

function StateGame:screenClearUpdate(dt)

   self.ringRadius = self.ringRadius + (self.ringExpandRate*dt);

   --aoe dmg
   local x,y = self.game.avatar:getLoc();
   self.game:areaDamageVehicle( x, y, self.ringRadius, 100000000, nil, 100, 100000000);

   self:ringAnimHackUpdate(dt);


   if self.ringRadius > App.viewportHeight then
      self.ringPlaying = false;

      if not self.game.gameDirector:isBossScenarioActive() then
         self.game.gameSpawner:enableSpawn();
      end

      self:ringAnimHackEnd();
      EventMgr:postEvent({}, "revive-stop");
   end

end

----------------------------------------------------------------

function StateGame:ringAnimHackStart()
   --create prop
   local x,y = self.game.avatar:getLoc();
   if not self.ringAnim then

      local gfxQuad  = MOAIGfxQuad2D.new ();
      gfxQuad:setTexture ( TextureMgr:getTexture("resources/Powerup/Effect_Magnet.png") );
      gfxQuad:setRect ( -1,-1,1,1 );
      self.ringAnim  = MOAIProp2D.new ();
      self.ringAnim:setDeck ( gfxQuad );
      self.ringAnim:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA );
      self.ringAnim:setLoc(x,y);
      self.layer.ui[1]:insertProp( self.ringAnim );
      self.ringAnim:setVisible(false);
   else
      --		self.ringAnim:setVisible(true);
      self.ringAnim:setLoc(x,y);
      self.ringAnim:setVisible(false);
   end
end

----------------------------------------------------------------

function StateGame:ringAnimHackUpdate(dt)
   --set position at avatar
   --set size based on ring radius

   local x,y = self.game.avatar:getLoc();
   self.ringAnim:setLoc(x,y);
   self.ringAnim:setScl(self.ringRadius);
   self.ringAnim:setVisible(false);

end

----------------------------------------------------------------

function StateGame:ringAnimHackEnd()

   self.ringAnim:setVisible(false);
   self.ringAnim = nil;
   self.game.avatar.noCollide = false;

end

----------------------------------------------------------------

function StateGame:overrideRankForBounty( player, rank, score )

   for i=rank,1,-1 do

      local entry = RankingMgr:getRankEntry(i);

      --Skip the player
      if entry ~= player then

         --Must be of the same score
         if math.abs(score - entry.score) <= 1 then

            --Bounty have precedence... if we find one... go with it
            if self.bountyList and BountyMgr:isBountyEntryActive(entry.tangoId) then

               return i;

            end

         else

            --The score is different... we are done
            return rank;

         end

      end


   end

   --Didnt find any bounty... go with default value
   return rank;

end

----------------------------------------------------------------

function StateGame:calcRank( score )

	local player = RankingMgr:getRankEntry(RankingMgr:getPlayerRank());

   for i=RankingMgr:getRankEntryCount(),1,-1 do

		local entry = RankingMgr:getRankEntry(i);

      if score <= entry.score and entry ~= player then

         if AbTestMgr:getHUDTargetSelectionGroup() == "all" then

            return self:overrideRankForBounty(player, i, entry.score);

         elseif self.bountyList and BountyMgr:isBountyEntryActive(entry.tangoId) and entry.score > 0 then

            return self:overrideRankForBounty(player, i, entry.score);

         elseif player.score <= entry.score then

            return self:overrideRankForBounty(player, i, entry.score);
         end

      end
   end

   return 1;
end

----------------------------------------------------------------

function StateGame:_getLocalPlayerAndRank(rankingList)

   local localPlayer = nil;
   local localPlayerRank = nil;

   for rank, player in ipairs(rankingList) do

      if player.tangoId == RankingMgr:getLocalPlayerId() then

         localPlayer = player;
         localPlayerRank = rank;
         break;
      end
   end
   Debug.assertType(localPlayer, "table", "StateGame:_getLocalPlayerAndRank -> localPlayer");
   Debug.assertType(localPlayerRank, "number", "StateGame:_getLocalPlayerAndRank -> localPlayerRank");

   return localPlayer, localPlayerRank;
end

----------------------------------------------------------------

function StateGame:_getHUDTargetList()

   -- Get the correct list to pull from.
   -- If league is disabled, use the regular ranking list.
   -- If league is enabled and locked, use the friend ranking list.
   -- If league is enabled and unlocked, use the league ranking list (set first parameter true to include filter for bounties).
   local rankingList = nil;

   if LeagueMgr:isLeagueEnabled() then

      if App.data:isLeagueUnlocked() then
         rankingList = RankingMgr:getRankingLeagueList(true);
      else
         rankingList = RankingMgr:getRankingFriendsList();
      end
   else
      rankingList = RankingMgr:getRankingList();
   end

   return rankingList;
end

----------------------------------------------------------------

function StateGame:_getNextHUDTarget(rankingList, highestScore)

   self._queuedBounties = self._queuedBounties or {};

   local targetPlayer, targetRank;

   for rank = #rankingList, 1, -1 do

      local player = rankingList[rank];

      if player.tangoId ~= RankingMgr:getLocalPlayerId() then

         if player.score > 0 then

            if player.score >= highestScore
            or (BountyMgr:isBountyEntryActive(player.tangoId) and not self._queuedBounties[player.tangoId]) then

               targetPlayer = player;
               targetRank = rank;

               break;
            end

         end

      end

   end

   return targetPlayer, targetRank;
end

----------------------------------------------------------------

function StateGame:_updateTargetTextBoxes(nameText, scoreText)

   if nameText then
      self.hud.targetTitleBox:setText(nameText);
      self.hud.targetTitleBox2:setText(nameText);
   end

   if scoreText then

      local targetBoxScale = self:_getGlyphScaleFromText(tostring(scoreText));
      self.hud.targetBox:setGlyphScale( targetBoxScale );
      self.hud.targetBox2:setGlyphScale( targetBoxScale );

      self.hud.targetBox:setText(scoreText);
      self.hud.targetBox2:setText(scoreText);
   end

end

----------------------------------------------------------------

-- Queue bounties if they're ready
function StateGame:_queueBountiesIfNeeded(score)

   -- Don't queue any bounties if there are bounty cars on screen
   if self.numActiveBountyCars > 0 then
      return;
   end

   local rankingList = self:_getHUDTargetList();
   local curGameScore = score + math.floor(score * App.data:getScoreBonus());

   self._queuedBounties = self._queuedBounties or {};

   for rank = #rankingList, 1, -1 do

      local player = rankingList[rank];

      if curGameScore >= player.score then

         -- Queue a bounty if that player is a valid bounty and they've never been queued.

         if BountyMgr:isBountyEntryActive(player.tangoId)
         and not self._queuedBounties[player.tangoId] then

            local queuedBounty = self.game.gameSpawner:queueBounty(player);

            if queuedBounty then

               self.numActiveBountyCars = self.numActiveBountyCars + 1;

               self.lastQueuedBounty = player;
               self._queuedBounties[player.tangoId] = true;

               return;

            end

         end

      end
   end

end

----------------------------------------------------------------

-- Update the goal score that appears in the corner of the in-game HUD
function StateGame:updateTarget(score)

   if not self.game.gameSpawner then
      return;
   end

   local rankingList = self:_getHUDTargetList();
   local localPlayer, localPlayerRank = self:_getLocalPlayerAndRank(rankingList);
   if not localPlayer then
      Debug.error("Missing localPlayer in StateGame:updateTarget");
      return;
   end

   local nameText = nil;
   local scoreText = nil;
   local curGameScore = score + math.floor(score * App.data:getScoreBonus());
   local playersHighestScore = math.max(localPlayer.score, curGameScore);

   if self.numActiveBountyCars > 0 and self.lastQueuedBounty then

      -- There's a bounty on screen!

      local displayedPlayer = self.lastSpawnedBounty or self.lastQueuedBounty;

      if displayedPlayer then

         local displayedRank = 1;
         for i, player in ipairs(rankingList) do
            print("player.tangoId:", player.tangoId)
            if player.tangoId == displayedPlayer.tangoId then
               displayedRank = i;
               break;
            end
         end

         nameText = displayedRank .. ". " .. createUserNameDisplay( displayedPlayer.contact:getFirstName(),
                                                                    displayedPlayer.contact:getLastName(),
                                                                    false,
                                                                    "Friend" );
      end

      scoreText = "Get Bounty!";

      if self.lastQueuedBounty.contact.bountyText then
         scoreText = self.lastQueuedBounty.contact.bountyText;
      end

   else

      -- There's no bounty on screen... proceed normally.

      -- Pick the player that has the next highest score or whose bounty needs collection.
      local targetPlayer, targetRank = self:_getNextHUDTarget(rankingList, playersHighestScore);

      -- Update the name text
      if targetPlayer then
         nameText = targetRank .. ". " .. createUserNameDisplay( targetPlayer.contact:getFirstName(),
                                                                 targetPlayer.contact:getLastName(),
                                                                 false,
                                                                 "Friend" );
      else
         local firstName, lastName = self:_getFirstAndLastName();
         nameText = 1 .. ". " .. createUserNameDisplay( firstName,
                                                        lastName,
                                                        false,
                                                        "You" );
      end

      -- Update score text
      if targetPlayer then

         scoreText = tostring(targetPlayer.score - curGameScore);

      else

         -- Check for high score
         if curGameScore >= playersHighestScore then
            scoreText = "High Score!";
         else

            scoreText = tostring(localPlayer.score - curGameScore);
         end
      end

   end

   self:_updateTargetTextBoxes(nameText, scoreText);

end

----------------------------------------------------------------

function StateGame:_getGlyphScaleFromText(text, optScaleTable)

   local DEFAULT_SCALE_TABLE = {
      { 4, 1.00 },
      { 5, 0.90 },
      { 6, 0.80 },
      { 7, 0.63 },
   };

   optScaleTable = optScaleTable or DEFAULT_SCALE_TABLE;

   local finalScale = nil;
   local lastValue = nil;

   for i, pair in ipairs(optScaleTable) do

      local length, scale = pair[1], pair[2];

      -- Handle case where text is shorter than the first pair in the scale table
      if i == 1 and #text < length then
         finalScale = scale;
         break;
      end

      -- We found a direct match
      if #text == length then
         finalScale = scale;
         break;
      end

      -- This is the last entry and we had no matches; just use it.
      if i == #optScaleTable then
         finalScale = scale;
         break;
      end
   end

   return finalScale;
end

----------------------------------------------------------------

function StateGame:updateHud(dt)

   self:updateTarget(self.displayedScore);

   if self.hud.scoreBox then
      self.hud.scoreBox:setText( tostring(self.displayedScore) );
      self.hud.scoreBox2:setText( tostring(self.displayedScore) );

      if self.displayedScore > 999999 then
         self.hud.scoreBox:setGlyphScale( 0.85 );
         self.hud.scoreBox2:setGlyphScale( 0.85 );
      end
   end


   if self.hud.coinBox then
      self.hud.coinBox:setText(   "" .. math.floor(self.game.currency)  );
      self.hud.coinBox2:setText(  "" .. math.floor(self.game.currency)  );
   end

   local damageAlpha = 0;

   if self.game.avatar.timeSinceDamage < 0.5 then
      damageAlpha = DAMAGE_SCREEN_ALPHA*( 1 - (self.game.avatar.timeSinceDamage / 0.5));
   end
   self.hud.damageScreen:setColor(1,0,0,damageAlpha);


   if self.hud.debugBox then
      self.hud.debugBox:setText("" .. michelDebug );--math.floor(self.game.duration) .. " : " ..  math.floor((michelDebug or 0)*100)/100 );
   end

   self:updateHealthHUD(dt);
   self:updateShieldHUD(dt);

end

----------------------------------------------------------------

function StateGame:updateHealthHUD( dt )

   local health      = self.game.avatar:getHealth();
   local healthRatio = math.min( health/self.game.avatar:getHealthMax(), 1 );


   local HEALTH_BAR_WIDTH = 230;

   if self.game.avatar:isDead() or healthRatio == self._healthRatioPrev then

      self._healthChunkTimer = 0;
      self._healthRatioPrev  = healthRatio;

      self.hud.lifeFrame:setScale(1,1);
      self.hud.lifeDelta:setScale( healthRatio, 1);
      self.hud.lifeDelta:setLocX( -(1-healthRatio)*HEALTH_BAR_WIDTH/2 );
      self.hud.lifeBar:setScale( healthRatio, 1 );
      self.hud.lifeBar:setLocX( -(1-healthRatio)*HEALTH_BAR_WIDTH/2 );
      self.hud.lifeBarGlow:setVisible(false);
   elseif self._healthRatioPrev > healthRatio then

      local t = HEALTH_CHUNK_TIME;

      if math.abs(self._healthRatioPrev-healthRatio) <= 0.1 then
         t = t/2;
      end

      local k = math.mapClamped(HEALTH_CHUNK_PAUSE, self._healthRatioPrev, t+HEALTH_CHUNK_PAUSE, healthRatio, self._healthChunkTimer);

      self._healthChunkTimer = math.min(self._healthChunkTimer + dt, t+HEALTH_CHUNK_PAUSE);

      if self._healthChunkTimer == t+HEALTH_CHUNK_PAUSE then
         self._healthRatioPrev = healthRatio;
      end

      self.hud.lifeDelta:setColor(179/255, 61/255, 25/255, 1);
      self.hud.lifeDelta:setScale( k, 1);
      self.hud.lifeDelta:setLocX( -(1-k)*HEALTH_BAR_WIDTH/2 );
      self.hud.lifeBar:setScale( healthRatio, 1 );
      self.hud.lifeBar:setLocX( -(1-healthRatio)*HEALTH_BAR_WIDTH/2 );
      self.hud.lifeBarGlow:setVisible(false);
   elseif self._healthRatioPrev < healthRatio then

      local t = HEALTH_CHUNK_TIME;

      if math.abs(healthRatio - self._healthRatioPrev) <= 0.1 then
         t = t/2;
      end

      local k  = math.mapClamped(HEALTH_CHUNK_PAUSE, self._healthRatioPrev, t+HEALTH_CHUNK_PAUSE, healthRatio, self._healthChunkTimer);
      local a  = math.mapClamped(HEALTH_CHUNK_PAUSE, 1, t+HEALTH_CHUNK_PAUSE, 0.5, self._healthChunkTimer);
      local bk = math.mapClamped(0, HUD_PULSE_SCALE, t+HEALTH_CHUNK_PAUSE, 1, self._healthChunkTimer);

      self._healthChunkTimer = math.min(self._healthChunkTimer + dt, t+HEALTH_CHUNK_PAUSE);

      if self._healthChunkTimer == t+HEALTH_CHUNK_PAUSE then
         self._healthRatioPrev = healthRatio;
      end

      self.hud.lifeFrame:setScale(bk, bk);
      self.hud.lifeDelta:setColor(36/255, 111/255, 191/255, 1);
      self.hud.lifeDelta:setScale( healthRatio, 1);
      self.hud.lifeDelta:setLocX( -(1-healthRatio)*HEALTH_BAR_WIDTH/2 );
      self.hud.lifeBar:setScale( k, 1 );
      self.hud.lifeBar:setLocX( -(1-k)*HEALTH_BAR_WIDTH/2 );
      self.hud.lifeBarGlow:setVisible(true);
      self.hud.lifeBarGlow:setColor(1,1,1,a);
   end

   if healthRatio > 0.50 then

      if not self.hud.lifeBar:isHidden() then
         self.hud.lifeBar:setColor( 0, 1, 0 );
      end
      self.hud.lowArmor:setColor(1,1,1,0);

   elseif healthRatio > 0.25 then

      if not self.hud.lifeBar:isHidden() then
         self.hud.lifeBar:setColor( 0.99, 0.88 , 0 );
      end
      self.hud.lowArmor:setColor(1,0,0,0);

   elseif health > 0.00 then

      if not self.hud.lifeBar:isHidden() then
         self.hud.lifeBar:setColor( 1, 0, 0 );
      end
      self.hud.lowArmor:setColor(1,0,0,0.75);

   else

      self.hud.lowArmor:setColor(1,0,0,0);

   end

end

----------------------------------------------------------------

function StateGame:updateShieldHUD( dt )

	local shieldRatio	= self.game.avatar:isShieldActive();

	if self.game.avatar:isDead() or shieldRatio == nil then

		self._shieldChunkTimer	= 0;
		self._shieldRatioPrev	= 0;
		self._shieldAnimTimer	= 0;
		self._shieldPulseTimer	= 0;

		self:_hideShieldHUD();
	elseif shieldRatio ~= nil then

		if self._shieldRatioPrev == shieldRatio then

			self._shieldChunkTimer	= 0;
			self._shieldAnimTimer	= 0;
			self._shieldPulseTimer	= 0;

			if shieldRatio > 0 then
				self:_showShieldHUD();
				self.hud.shieldDelta:setScale( shieldRatio, 1);
				self.hud.shieldDelta:setLocX( -(1-shieldRatio)*224/2 + 16 );
				self.hud.shieldBar:setScale( shieldRatio, 1 );
				self.hud.shieldBar:setLocX( -(1-shieldRatio)*224/2 + 16 );
			else
				self:_hideShieldHUD();
			end
		elseif self._shieldRatioPrev > shieldRatio then
			local t	= HEALTH_CHUNK_TIME;

			if math.abs(self._shieldRatioPrev-shieldRatio) <= 0.1 then
				t		= t/2;
			end

			local k	= math.mapClamped(HEALTH_CHUNK_PAUSE, self._shieldRatioPrev, t+HEALTH_CHUNK_PAUSE, shieldRatio, self._shieldChunkTimer);

			self._shieldChunkTimer		= math.min(self._shieldChunkTimer + dt, t+HEALTH_CHUNK_PAUSE);

			if self._shieldChunkTimer == t+HEALTH_CHUNK_PAUSE then
				self._shieldRatioPrev	= shieldRatio;
			end

			self:_showShieldHUD();
			self.hud.shieldDelta:setColor(179/255, 61/255, 25/255, 1);
			self.hud.shieldDelta:setScale( k, 1);
			self.hud.shieldDelta:setLocX( -(1-k)*224/2 + 16 );
			self.hud.shieldBar:setScale( shieldRatio, 1 );
			self.hud.shieldBar:setLocX( -(1-shieldRatio)*224/2 + 16 );

			if shieldRatio == 0 then
				self:_updateShieldHUDPulseExit(dt);
			end
		elseif self._shieldRatioPrev < shieldRatio then

			local t	= HEALTH_CHUNK_TIME;

			if math.abs(shieldRatio - self._shieldRatioPrev) <= 0.1 then
				t		= t/2;
			end

			local k	= math.mapClamped(HEALTH_CHUNK_PAUSE, self._shieldRatioPrev, t+HEALTH_CHUNK_PAUSE, shieldRatio, self._shieldChunkTimer);
			local a	= math.mapClamped(HEALTH_CHUNK_PAUSE, 1, t+HEALTH_CHUNK_PAUSE, 0.5, self._shieldChunkTimer);

			self._shieldChunkTimer		= math.min(self._shieldChunkTimer + dt, t+HEALTH_CHUNK_PAUSE);

			if self._shieldChunkTimer == t+HEALTH_CHUNK_PAUSE then
				self._shieldRatioPrev	= shieldRatio;
			end

			self:_showShieldHUD();
			self.hud.shieldDelta:setColor(1, 1, 0, 1);
			self.hud.shieldDelta:setScale( shieldRatio, 1);
			self.hud.shieldDelta:setLocX( -(1-shieldRatio)*224/2 + 16 );
			self.hud.shieldBar:setScale( k, 1 );
			self.hud.shieldBar:setLocX( -(1-k)*224/2 + 16 );

			if self._shieldRatioPrev == 0 then
				self:_updateShieldHUDStampIn(dt);
			end
		end
	end

end

----------------------------------------------------------------

function StateGame:_hideShieldHUD()

	self.hud.shieldDelta:setVisible(false);
	self.hud.shieldBar:setVisible(false);
	self.hud.shieldFrame:setVisible(false);
	self.hud.shieldCover:setVisible(false);
	self.hud.shieldIcon:setVisible(false);

end

----------------------------------------------------------------

function StateGame:_showShieldHUD()

	self.hud.shieldDelta:setVisible(true);
	self.hud.shieldBar:setVisible(true);
	self.hud.shieldFrame:setVisible(true);
	self.hud.shieldCover:setVisible(true);
	self.hud.shieldIcon:setVisible(true);

end

----------------------------------------------------------------

function StateGame:_updateShieldHUDStampIn( dt )

	local k	= math.mapClamped(0, SHIELD_ANIM_SCALE, SHIELD_ANIM_TIME, 1, self._shieldAnimTimer);
	local a	= math.mapClamped(0, 0.05, SHIELD_ANIM_TIME, 1, self._shieldAnimTimer);

	self.hud.shieldFrame:setScale(k, k);
	self.hud.shieldFrame:setColor(1, 1, 1, a);
	self.hud.shieldDelta:setColor(1, 1, 0, a);

	self._shieldAnimTimer = math.min(self._shieldAnimTimer + dt, SHIELD_ANIM_TIME);

end

----------------------------------------------------------------

function StateGame:_updateShieldHUDPulseExit( dt )

	if self._shieldAnimTimer < HEALTH_CHUNK_PAUSE + HEALTH_CHUNK_TIME - SHIELD_ANIM_TIME then

		local k;

		if self._shieldPulseTimer < HUD_PULSE_TIME/2 then
			k	= math.mapClamped(0, 1, HUD_PULSE_TIME/2, HUD_PULSE_SCALE, self._shieldPulseTimer);
		else
			k	= math.mapClamped(HUD_PULSE_TIME/2, HUD_PULSE_SCALE, HUD_PULSE_TIME, 1, self._shieldPulseTimer);
		end

		self.hud.lifeFrame:setScale(k, k);

		self._shieldPulseTimer	= self._shieldPulseTimer + dt;

		if self._shieldPulseTimer > HUD_PULSE_TIME then
			self._shieldPulseTimer = 0;
		end
	else

		local k	= math.mapClamped(HEALTH_CHUNK_PAUSE + HEALTH_CHUNK_TIME - SHIELD_ANIM_TIME, 1, HEALTH_CHUNK_PAUSE + HEALTH_CHUNK_TIME, SHIELD_ANIM_SCALE, self._shieldAnimTimer);
		local a	= math.mapClamped(HEALTH_CHUNK_PAUSE + HEALTH_CHUNK_TIME - SHIELD_ANIM_TIME, 1, HEALTH_CHUNK_PAUSE + HEALTH_CHUNK_TIME, 0.05, self._shieldAnimTimer);

		self.hud.shieldFrame:setScale(k, k);
		self.hud.shieldFrame:setColor(1, 1, 1, a);
		self.hud.shieldDelta:setColor(179/255, 61/255, 25/255, a);
	end

	self._shieldAnimTimer = self._shieldAnimTimer + dt;

	if self._shieldAnimTimer > HEALTH_CHUNK_PAUSE + HEALTH_CHUNK_TIME then
		self:_hideShieldHUD();
	end

end

----------------------------------------------------------------

function StateGame:createAllLayers()

   self.camera = {};

   self.camera.background = MOAICamera2D.new();
   self.layer.background = {};
   for i=1,5 do
      self.layer.background[i] = self:addLayer();
      self.layer.background[i]:setCamera(self.camera.background);
   end

   self.camera.game = MOAICamera2D.new();
   self.layer.game = {};
   for i=1,10 do
      self.layer.game[i] = self:addLayer();
      self.layer.game[i]:setCamera( self.camera.game );
   end

   self.camera.foreground = MOAICamera2D.new();
   self.layer.foreground = {};
   for i=1,5 do
      self.layer.foreground[i] = self:addLayer();
      self.layer.foreground[i]:setCamera( self.camera.foreground );
   end

   self.camera.ui = MOAICamera2D.new();
   self.layer.ui = {};
   for i=1,5 do
      self.layer.ui[i] = self:addLayer();
      self.layer.ui[i]:setCamera( self.camera.ui );
   end
end

----------------------------------------------------------------

function StateGame:submitScore( score )

   RankingMgr:submitScore( score );

   self.scoreUpdated = true;
end

----------------------------------------------------------------

function StateGame:updatePostGameData()

   App.data:updateGameStats(  math.floor(self.game.currency), self.game.duration, self.game.distanceScore );

   -- dont wait for post game update... the next screen will do
   self.postGameDataUpdated = true;

end

----------------------------------------------------------------

function StateGame:isUsingCameraHack()
   return false;
end

----------------------------------------------------------------------

function StateGame:onDataServerError()

   if not StateMgr._showErrorDialog then

      --Give back the key
      Base.onDataServerError(self);

      App.data:addKey(1, "Data Server Error");
   end

end

----------------------------------------------------------------

function StateGame:_getFirstAndLastName()

   local profile  = ContactMgr:getProfile();
   if not profile then
      return "", "";
   end

   return (profile:getFirstName() or ""), (profile:getLastName() or "")
end

----------------------------------------------------------------------

function StateGame:onDataMerged()

   Base.onDataMerged(self);

   StateMgr:push("state/state-pause.lua");

end

----------------------------------------------------------------

return StateGame;
