----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local SQUARE_TEXTURE = "resources/square.png";

local LOADING_BAR_BACK_WIDTH   = 290;
local LOADING_BAR_BACK_HEIGHT  = 36;
local LOADING_BAR_FRONT_WIDTH  = 284;
local LOADING_BAR_FRONT_HEIGHT = 30;

local MIN_LOAD_TIME            = 1;
local PULSE_TIME               = 0.5;
local DEFAULT_MAX_VIEW         = 2;

local PRIORITY_0               = 10;
local PRIORITY_1               = 20;
local PRIORITY_2               = 30;
local PRIORITY_3               = 40;
local PRIORITY_4               = 50;
local PRIORITY_5               = 60;
local PRIORITY_6               = 70;
local PRIORITY_7               = 80;
local PRIORITY_8               = 90;
local PRIORITY_9               = 100;
local PRIORITY_10              = 110;

local HACK_DEBUG_HINTS = false;

if not BUILD_DEBUG then
   HACK_DEBUG_HINTS	= false;
end

----------------------------------------------------------------

-- ATTENTION: Only add hints to the end of the table. If removing an entry, just set the isEnabled of that entry to return false.
local HINT_INFO =
{
   { hint = "Destroy racers with bounties and collect their portraits to earn Gems!",        image = "resources/UI/Hints/Hint_1.png",  minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = function() return App.data:isBountyUnlocked() end  },
   { hint = "Destroy the helicopter to\ncollect your Daily reward!",                         image = "resources/UI/Hints/Hint_2.png",  minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = function() return App.data:isDailyUnlocked() end   },
   { hint = "Make sure to collect your Daily Rewards every day, otherwise they will reset!", image = "resources/UI/Hints/Hint_3.png",  minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = function() return App.data:isDailyUnlocked() end   },
   { hint = "Gems will allow you to unlock and upgrade vehicles much faster!",               image = "resources/UI/Hints/Hint_4.png",  minLevel = 5,   maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Don't forget to complete missions\nto get more gems!",                          image = "resources/UI/Hints/Hint_5.png",  minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = function() return App.data:isMissionUnlocked() end   },
   { hint = "Use Consumables to\nimprove your score!",                                       image = "resources/UI/Hints/Hint_6.png",  minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = function() return App.data:isConsumableUnlocked() end   },
   { hint = "All Boss vehicles fire in patterns. Remember the pattern to avoid damage!",     image = "resources/UI/Hints/Hint_7.png",  minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = function() return App.data:isBossUnlocked() end   },
   { hint = "Each vehicle has its own ability.\nLevel up to unlock them all!",               image = "resources/UI/Hints/Hint_8.png",  minLevel = 10,  maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Some drones will automatically activate their ability when fully charged.",     image = "resources/UI/Hints/Hint_9.png",  minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = function() return DroneMgr:isEnabled() and not DroneMgr:isDroneLevelLocked() end   },
   { hint = "Each new star rating grants your\ndrones new powers and abilities!",            image = "resources/UI/Hints/Hint_10.png", minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = function() return DroneMgr:isEnabled() and not DroneMgr:isDroneLevelLocked() end   },
   { hint = "Check the Drone Codex to see\nhow many you've collected!",                      image = "resources/UI/Hints/Hint_11.png", minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = function() return DroneMgr:isEnabled() and not DroneMgr:isDroneLevelLocked() end   },
   { hint = "Avoid missiles and road hazards.\nThey will destroy your car.",                 image = "resources/UI/Hints/Hint_12.png", minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Collect Pickups from dead enemies!\nThey'll power up your car.",                image = "resources/UI/Hints/Hint_13.png", minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Armored Trucks drop coins\nand repair wrenches!",                               image = "resources/UI/Hints/Hint_14.png", minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "The bonus from Score Multipliers increase as long as you collect more!",        image = "resources/UI/Hints/Hint_15.png", minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Destroying a fuel tanker will blow\nup other enemies near it.",                 image = "resources/UI/Hints/Hint_16.png", minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Your keys fully replenish\neach time you level up.",                            image = "resources/UI/Hints/Hint_17.png", minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "You gain permanent score boosts\nas you reach new levels.",                     image = "resources/UI/Hints/Hint_18.png", minLevel = 2,   maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Avoid collisions to\nmaximize your score.",                                     image = "resources/UI/Hints/Hint_19.png", minLevel = 2,   maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Missiles always appear in the same order as their warning flags.",              image = "resources/UI/Hints/Hint_20.png", minLevel = 2,   maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Watching videos is a great way to\nearn FREE coins for upgrades!",              image = "resources/UI/Hints/Hint_21.png", minLevel = 2,   maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "The Boost effect will blink rapidly\njust before the effect ends.",             image = "resources/UI/Hints/Hint_22.png", minLevel = 3,   maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Head-on collisions deal more\ndamage than sideswipes.",                         image = "resources/UI/Hints/Hint_23.png", minLevel = 5,   maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Upgrade your vehicle's armor to withstand more collision damage.",              image = "resources/UI/Hints/Hint_24.png", minLevel = 5,   maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "The large coins are worth\nten regular coins each.",                            image = "resources/UI/Hints/Hint_25.png", minLevel = 5,   maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Purchase the Double Coins upgrade\nand watch your money grow!",                 image = "resources/UI/Hints/Hint_26.png", minLevel = 3,   maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Vehicle side weapons are great for killing enemies in adjacent lanes.",         image = "resources/UI/Hints/Hint_27.png", minLevel = 3,   maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "Gun Trucks fire in volleys. Avoid the gunfire then destroy them!",              image = "resources/UI/Hints/Hint_28.png", minLevel = 4,   maxLevel = nil, maxView = nil, isEnabled = nil  },
   { hint = "The top three players in the Leaderboard are promoted each week!",              image = "resources/UI/Hints/Hint_29.png", minLevel = nil, maxLevel = nil, maxView = nil, isEnabled = function() return LeagueMgr:isLeagueEnabled() and App.data:isLeagueUnlocked() end  },
};

----------------------------------------------------------------

local Base							= State;
local StateHints					= Base:new();

----------------------------------------------------------------

function StateHints:onLoad()

   State.onLoad( self );

   self.defaultFontName	= "resources/font/TangoLatin.otf";
   self.IS_POPUP			= true;

   -- background
   self.Background		= self:addWidget(ButtonImage, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE, self.goNext, nil, nil, nil);
   self.Background:setPriority(PRIORITY_0);
   self.Background:setColor(0,0,0, 0.75);

   -- loading bar
   self.loadingFrame		= self:addWidget(Image, self.mainLayer, 0, -App.viewportHeight/2 + 100, LOADING_BAR_BACK_WIDTH, LOADING_BAR_BACK_HEIGHT, "resources/UI/LoadingBarBack.png");
   self.loadingBar		= self:addWidget(Image, self.mainLayer, 0, -App.viewportHeight/2 + 100, LOADING_BAR_FRONT_WIDTH, LOADING_BAR_FRONT_HEIGHT, "resources/UI/LoadingBarFront.png");
   self.loadingFrame:setPriority(PRIORITY_2);
   self.loadingBar:setPriority(PRIORITY_2+1);
   self.loadingBar:setColor( 255/255, 75/255, 25/255, 1 );

   -- frame
   self.HintPic			= self:addWidget(Image, self.mainLayer, 0, -40, 510*1.1, 685*1.1, "resources/UI/Splash.png");
   self.HintPic:setPriority(PRIORITY_1);

   -- text message
   self.HintTitle	= self:addWidget(TextBox, self.mainLayer, 0,  App.viewportHeight/2 - 75, 600, 100, "HINT:", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.HintTitle:setPriority(PRIORITY_3);
   self.HintText	= self:addWidget(TextBox, self.mainLayer, 0,  App.viewportHeight/2 - 105-100, 560, 200, "HINT:", self.defaultFontName, 15, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.LEFT_JUSTIFY);
   self.HintText:setPriority(PRIORITY_3);
   self.TapText	= self:addWidget(TextBox, self.mainLayer, 0,  -App.viewportHeight/2 + 100, 640, 200, "Tap Screen to Continue", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {255/255, 75/255, 25/255, 1});
   self.TapText:setPriority(PRIORITY_4);

end

----------------------------------------------------------------

function StateHints:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StateHints:onFocus( data )

   State.onFocus(self);

   Debug.assertType(data, "table", "StateHints:onFocus -> data");
   Debug.assertType(data.state, "string", "StateHints:onFocus -> data.state");

   self._hintIndex   = App.data:getValue("hint_screen_index", 0);
   self._hintEnable  = self:_hasValidHints();
   self._loadTimer   = 0;
   self._pulseTimer  = 0;
   self._nextState   = data.state;
   self._dataChecked = false;
   self._keyAnim     = data.anim;
   self._keyImage    = data.image;

   if not self._hintEnable then
      self:_hideHint();
   else

      self:_updateHintIndex();

      if self._hintIndex == App.data:getValue("hint_screen_prev_index", 0) then
         self._hintEnable = false;

         self:_hideHint();
      else
         self.TapText:setVisible(false);
         self:_updateHint();

         if self._keyAnim then
            self._keyAnim:stop();
            self._keyAnim:setVisible(false);
         end

         if self._keyImage then
            self._keyImage:setVisible(false);
         end
      end
   end

   self.Background:disable();

   if data.state == "state/state-game.lua" then
      EventMgr:postEvent( { }, "game-load" );
   end

   delay( 0.10, function() App:collectGarbageNow() end );

end

----------------------------------------------------------------

function StateHints:onUpdate( dt )

   Base.onUpdate(self, dt);

   self:_updateLoadingBar(dt);

end


------------------------------------------------------------------------------------
-- Updates
------------------------------------------------------------------------------------


function StateHints:_updateLoadingBar( dt )

   self._loadTimer  = math.min(self._loadTimer+dt, MIN_LOAD_TIME);

   if not self._hintEnable then

      self.TapText:setVisible(false);
      self.loadingFrame:setVisible(false);
      self.loadingBar:setVisible(false);

      if self._loadTimer >= 0.15 and StateMgr:getDataServerErrorCount() == 0 then
         if self._keyAnim then
            self._keyAnim:stop();
            self._keyAnim:setVisible(false);
         end

         if self._keyImage then
            self._keyImage:setVisible(false);
         end

         StateMgr:swap(self._nextState);
      end
   elseif self._loadTimer >= MIN_LOAD_TIME and StateMgr:getDataServerErrorCount() == 0 then

      self._pulseTimer  = self._pulseTimer + dt;
      self._dataChecked = true;

      if self._pulseTimer > PULSE_TIME then
         self._pulseTimer   = 0;
      end

      local k;

      if self._pulseTimer < PULSE_TIME/2 then
         k  = math.mapClamped(0, 1, PULSE_TIME/2, 0.9, self._pulseTimer);
      else
         k  = math.mapClamped(PULSE_TIME/2, 0.9, PULSE_TIME, 1, self._pulseTimer);
      end

      self.TapText:setVisible(true);
      self.loadingFrame:setVisible(false);
      self.loadingBar:setVisible(false);
      self.TapText:setScale(k, k);
      self.Background:enable();
   else

      local ratio = math.min(self._loadTimer/(MIN_LOAD_TIME - 2/3), 1);

      self.TapText:setVisible(false);
      self.loadingFrame:setVisible(true);
      self.loadingBar:setVisible(true);
      self.loadingBar:setScale(ratio, 1);
      self.loadingBar:setLocX( -LOADING_BAR_FRONT_WIDTH/2*(1-ratio));
   end

end

----------------------------------------------------------------

function StateHints:_updateHint()

   if self._hintIndex then
      self.HintPic:setVisible(true);
      self.HintTitle:setVisible(true);
      self.HintText:setVisible(true);
      self.HintPic:swapImage(HINT_INFO[self._hintIndex].image);
      self.HintText:setText(HINT_INFO[self._hintIndex].hint);
      App.data:setValue("hint_screen_prev_index", self._hintIndex);

      if App.viewHeight/App.viewWidth >= 1.7 then
         self.HintPic:setScale(1, 1);
      else
         self.HintPic:setScale(0.899, 0.899);
      end
   end

end


------------------------------------------------------------------------------------
-- Util
------------------------------------------------------------------------------------


function StateHints:_updateHintIndex()

   local level = App.data:getCurrentPlayerLevel() or 1;

   self._hintIndex = self._hintIndex + 1;

   while not self:_isHintIndexValid(self._hintIndex, level) do

      self._hintIndex = self._hintIndex + 1;

      if self._hintIndex > #HINT_INFO then
         self._hintIndex = 1;
      end
   end

   App.data:deltaValue("hint_max_view_count_"..self._hintIndex, 1);
   App.data:setValue("hint_screen_index", self._hintIndex);

end

----------------------------------------------------------------

function StateHints:_isHintIndexValid( index, level )

   if     index
      and level
      and HINT_INFO[index] then

      if     HINT_INFO[index].minLevel
         and level < HINT_INFO[index].minLevel then

         return false;
      end

      if     HINT_INFO[index].maxLevel
         and level > HINT_INFO[index].maxLevel then

         return false;
      end

      if     HINT_INFO[index].isEnabled
         and type(HINT_INFO[index].isEnabled) == "function"
         and not HINT_INFO[index]:isEnabled() then

         return false;
      end

      if App.data:getValue("hint_max_view_count_"..index, 0) >= (HINT_INFO[index].maxView or DEFAULT_MAX_VIEW) then
         return false;
      end

      return true;
   end

   return false;

end

----------------------------------------------------------------

function StateHints:_hasValidHints()

   local level = App.data:getCurrentPlayerLevel() or 1;

   for k, _ in ipairs(HINT_INFO) do
      if self:_isHintIndexValid(k, level) then
         return true;
      end
   end

   return false;

end

----------------------------------------------------------------

function StateHints:_hideHint()

   self.HintPic:setVisible(false);
   self.HintTitle:setVisible(false);
   self.HintText:setVisible(false);
   self.TapText:setVisible(false);
   self.loadingFrame:setVisible(false);
   self.loadingBar:setVisible(false);

end

----------------------------------------------------------------

function StateHints:isUsingCameraHack()
   return false;
end


------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------


function StateHints:goNext()

   if HACK_DEBUG_HINTS then

      self._hintIndex = self._hintIndex + 1;

      if self._hintIndex > #HINT_INFO then
         self._hintIndex = 1;
      end

      self:_updateHint();
   elseif self._loadTimer >= MIN_LOAD_TIME and self._dataChecked then
      StateMgr:swap(self._nextState);
   end

end


------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------


return StateHints;
