----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');
require('ui/ui-scroll-list');

----------------------------------------------------------------

local SLIDE_MIN               = -App.viewportHeight/2;
local SLIDE_MAX               = App.viewportHeight/2;
local SWIPE_WIN_MIN_X         = -App.viewportWidth/2;
local SWIPE_WIN_MAX_X         = App.viewportWidth/2;
local SWIPE_WIN_MIN_Y         = -352;
local SWIPE_WIN_MAX_Y         = 355;
local LIST_HOME_X					= 0;
local LIST_HOME_Y					= 237;
local LIST_SLIDE_LEFT			= -960;
local LIST_SPACING				= 235;
local LIST_DISPLAY_MAX			= 3;

local PRIORITY_0					= 10;
local PRIORITY_1					= 20;
local PRIORITY_2					= 30;
local PRIORITY_3					= 40;
local PRIORITY_4					= 50;
local PRIORITY_5					= 60;
local PRIORITY_6					= 70;
local PRIORITY_7					= 80;
local PRIORITY_8					= 90;
local PRIORITY_9					= 100;
local PRIORITY_10					= 110;

local SQUARE_TEXTURE				= "resources/square.png";
local ACTIVE_LEAGUE           = "active-league";
local INACTIVE_LEAGUE         = "inactive-league";

local LEAGUE_INSET =
{
   "resources/UI/League/LB_insetDmnd.png",   --diamond
   "resources/UI/League/LB_insetPlat.png",	--platinum
   "resources/UI/League/LB_insetGold.png",	--gold
   "resources/UI/League/LB_insetSilver.png",	--silver
   "resources/UI/League/LB_insetBronze.png",	--bronze
};

local LEAGUE_BANNER =
{
	"resources/UI/League/LC_bannerDiamond.png",	--diamond
	"resources/UI/League/LC_bannerPlatinum.png", --platinum
	"resources/UI/League/LC_bannerGold.png",		--gold
	"resources/UI/League/LC_bannerSilver.png",	--silver
	"resources/UI/League/LC_bannerBronze.png",	--bronze
};

local DIVISION_BRACKET =
{
   "resources/UI/League/LC_brackets_div1.png",
   "resources/UI/League/LC_brackets_div2.png",
   "resources/UI/League/LC_brackets_div3.png",
   "resources/UI/League/LC_brackets_off.png",
};

----------------------------------------------------------------

local Base						= State;
local StateLeagueCodex		= Base:new();

----------------------------------------------------------------

function StateLeagueCodex:onLoad()

   State.onLoad( self );

	self.textureSheet		= FlashMgr:getTextureSheet("resources/flash-resource-bundle/MainScreen.lua", "resources/flash-resource-bundle/MainScreen.png", 30);
   self.defaultFontName	= "resources/font/TangoLatin.otf";

	-- background, frame, and title
   self.Background		= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "MainFrameBackground", self.textureSheet);
	self.Frame				= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "PU_ScrollFrame", self.textureSheet);

	self.Background:setPriority(0);
	self.Frame:setPriority(PRIORITY_4);

   if StateMgr.headerAvailable then
		self.Title			= self:addWidget(Image, self.mainLayer, 0, 430, 434, 66, "resources/UI/League/LC_Title.png");
		self.Title:setPriority(PRIORITY_6);
      self.Title:setScale(0.9, 0.9);
   end

   -- resource bar
   self:_setupBasicResourceBar();

	-- scroll list
	self.lIndex				= 1;
	self.dIndex				= 1;
   self.ScrollList		= self:addWidget(		ScrollList,
															self.mainLayer,
															LIST_HOME_X, LIST_HOME_Y,
															{ {SWIPE_WIN_MIN_X, SWIPE_WIN_MAX_X},	{SWIPE_WIN_MIN_Y, SWIPE_WIN_MAX_Y} },
															nil,
															nil,
															{ 0, (1/40)*LIST_SPACING },
															{ nil, LIST_SPACING },
															{ 0, 1 },
															nil,
															{ 100, 50 },
															{0, LIST_SPACING},
															nil,
															{ {-200, 200},	{SLIDE_MIN, SLIDE_MAX} },
															nil, nil, nil, nil, nil, nil,
															"ScrollBarButton", self.textureSheet, 12, 75, 135, PRIORITY_5 );

	self:_createCache();

	-- Exit Button
	self.BackButton		= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "PU_DoneButton", self.textureSheet, nil, self.goBack, nil, nil, nil, nil, 40);
	self.BackButton:setPriority(PRIORITY_6);

end

----------------------------------------------------------------

function StateLeagueCodex:onActivate()

   State.onActivate(self);

	if App.viewHeight/App.viewWidth <= 1.7 and StateMgr.headerAvailable then
		self.Title:setLoc(0, 410);
	end

   if self.resourceBar then
      self.resourceBar:createConsumableButton();
   end

end

----------------------------------------------------------------

function StateLeagueCodex:onFocus()

   State.onFocus(self);

	self:_updateList();

end

----------------------------------------------------------------

function StateLeagueCodex:onUpdate( dt )

	Base.onUpdate(self, dt);

end


------------------------------------------------------------------------------------
-- Util
------------------------------------------------------------------------------------


function StateLeagueCodex:isUsingCameraHack()
   return false;
end


------------------------------------------------------------------------------------
-- scroll list
------------------------------------------------------------------------------------


function StateLeagueCodex:_createCache()

	self.ScrollList:setEntryObjectDataUpdateFunction(	function( data, object )

																			self:_hideCacheObject(object);
                                                         object.banner:setVisible(true);
                                                         object.divI:setVisible(true);
                                                         object.divII:setVisible(true);
                                                         object.divIII:setVisible(true);
                                                         object.divITxt:setVisible(true);
                                                         object.divITxtShdw:setVisible(true);
                                                         object.divIITxt:setVisible(true);
                                                         object.divIITxtShdw:setVisible(true);
                                                         object.divIIITxt:setVisible(true);
                                                         object.divIIITxtShdw:setVisible(true);

                                                         if object.banner:swapImage(   LEAGUE_BANNER[data.currentIndex] ) then

                                                            object.divI:swapImage(     LEAGUE_INSET[data.currentIndex] );
                                                            object.divII:swapImage(    LEAGUE_INSET[data.currentIndex] );
                                                            object.divIII:swapImage(   LEAGUE_INSET[data.currentIndex] );

                                                            object.banner:setAttrLink(          MOAIProp2D.INHERIT_TRANSFORM,		object.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                                            object.divI:setAttrLink(            MOAIProp2D.INHERIT_TRANSFORM,		object.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                                            object.divII:setAttrLink(           MOAIProp2D.INHERIT_TRANSFORM,		object.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                                            object.divIII:setAttrLink(          MOAIProp2D.INHERIT_TRANSFORM,		object.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                                            object.divITxt:setAttrLink(         MOAIProp2D.INHERIT_TRANSFORM,		object.divI.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                                            object.divITxtShdw:setAttrLink(     MOAIProp2D.INHERIT_TRANSFORM,		object.divI.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                                            object.divIITxt:setAttrLink(        MOAIProp2D.INHERIT_TRANSFORM,		object.divII.prop,	MOAIProp2D.TRANSFORM_TRAIT	);
                                                            object.divIITxtShdw:setAttrLink(    MOAIProp2D.INHERIT_TRANSFORM,		object.divII.prop,	MOAIProp2D.TRANSFORM_TRAIT	);
                                                            object.divIIITxt:setAttrLink(       MOAIProp2D.INHERIT_TRANSFORM,		object.divIII.prop,	MOAIProp2D.TRANSFORM_TRAIT	);
                                                            object.divIIITxtShdw:setAttrLink(   MOAIProp2D.INHERIT_TRANSFORM,		object.divIII.prop,	MOAIProp2D.TRANSFORM_TRAIT	);
                                                         end

                                                         if data.reward and data.reward[1] then

                                                            object.prizeTxt:setVisible(true);
                                                            object.prizeTxtShdw:setVisible(true);

                                                            if data.state == INACTIVE_LEAGUE and data.reward[1].gem > 0 then
                                                               object.gem:setVisible(true);
                                                               object.coin:setVisible(false);
                                                               object.prizeTxt:setText( tostring(data.reward[1].gem) );
                                                               object.prizeTxtShdw:setText( tostring(data.reward[1].gem) );
                                                            elseif data.state == INACTIVE_LEAGUE then
                                                               object.gem:setVisible(false);
                                                               object.coin:setVisible(true);
                                                               object.prizeTxt:setText( tostring(data.reward[1].gold) );
                                                               object.prizeTxtShdw:setText( tostring(data.reward[1].gold) );
                                                            elseif data.state == ACTIVE_LEAGUE and data.reward[data.divIndex].gem > 0 then
                                                               object.gem:setVisible(true);
                                                               object.coin:setVisible(false);
                                                               object.prizeTxt:setText( tostring(data.reward[data.divIndex].gem) );
                                                               object.prizeTxtShdw:setText( tostring(data.reward[data.divIndex].gem) );
                                                            elseif data.state == ACTIVE_LEAGUE then
                                                               object.gem:setVisible(false);
                                                               object.coin:setVisible(true);
                                                               object.prizeTxt:setText( tostring(data.reward[data.divIndex].gold) );
                                                               object.prizeTxtShdw:setText( tostring(data.reward[data.divIndex].gold) );
                                                            else
                                                               object.prizeTxt:setVisible(false);
                                                               object.prizeTxtShdw:setVisible(false);
                                                            end
                                                         end

                                                         if data.state == ACTIVE_LEAGUE then

                                                            object.overlay:setVisible(false);
                                                            object.divI:setScale(2/3, 2/3);
                                                            object.divII:setScale(2/3, 2/3);
                                                            object.divIII:setScale(2/3, 2/3);

                                                            if data.divIndex == 1 then
                                                               object.divILine:setVisible(true);
                                                               object.divI:setScale(1, 1);
                                                            elseif data.divIndex == 2 then
                                                               object.divIILine:setVisible(true);
                                                               object.divII:setScale(1, 1);
                                                            else
                                                               object.divIIILine:setVisible(true);
                                                               object.divIII:setScale(1, 1);
                                                            end
                                                         else
                                                            object.overlay:setVisible(true);
                                                            object.divOffLine:setVisible(true);
                                                            object.divI:setScale(2/3, 2/3);
                                                            object.divII:setScale(2/3, 2/3);
                                                            object.divIII:setScale(2/3, 2/3);
                                                         end
																		end );

	self.ScrollList:setEntryObjectLocUpdateFunction(	function( x, y, object )
																			object.root:setLoc(x,y);
																		end );

	self.ScrollList:setEntryObjectParentUpdateFunction(	function( target, object )
																				object.root:setParent(target);
																			end );

	self.ScrollList:setEntryCacheObjects(	nil,
														5,
														function()
															local temp	= {};

															temp.root				= self:addWidget(Image, self.mainLayer, 0, 0, 1, 1, SQUARE_TEXTURE);
															temp.banner          = self:addWidget(Image, self.mainLayer, 0, 0, 724, LIST_SPACING-4, LEAGUE_BANNER[1]);
															temp.gem					= self:addWidget(Image, self.mainLayer, 200, -10, 128, 128, "resources/UI/GR_Icon_Gem.png");
															temp.coin				= self:addWidget(Image, self.mainLayer, 200, -10, 128, 128, "resources/UI/GR_Icon_Coin.png");
															temp.prizeTxt			= self:addWidget(TextBox, self.mainLayer, 320, -50, 200, 100, "reward count", self.defaultFontName, 20, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
															temp.prizeTxtShdw    = self:addWidget(TextBox, self.mainLayer, 320+2, -50-2, 200, 100, "reward count", self.defaultFontName, 20, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
															temp.divI            = self:addWidget(Image, self.mainLayer, -35, LIST_SPACING/2-45, 54, 54, LEAGUE_INSET[1]);
															temp.divII           = self:addWidget(Image, self.mainLayer, -35, 0, 60, 60, LEAGUE_INSET[1]);
															temp.divIII          = self:addWidget(Image, self.mainLayer, -35, -LIST_SPACING/2+45, 54, 54, LEAGUE_INSET[1]);
                                             temp.divITxt         = self:addWidget(TextBox, self.mainLayer, 0, 0, 200, 100, "I", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
                                             temp.divITxtShdw     = self:addWidget(TextBox, self.mainLayer, 0+2, -2, 200, 100, "I", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
                                             temp.divIITxt        = self:addWidget(TextBox, self.mainLayer, 0, 0, 200, 100, "II", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
                                             temp.divIITxtShdw    = self:addWidget(TextBox, self.mainLayer, 0+2, -2, 200, 100, "II", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
                                             temp.divIIITxt       = self:addWidget(TextBox, self.mainLayer, 0, 0, 200, 100, "III", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
                                             temp.divIIITxtShdw   = self:addWidget(TextBox, self.mainLayer, 0+2, -2, 200, 100, "III", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
															temp.divILine			= self:addWidget(Image, self.mainLayer, 55, 0, 112, 155, DIVISION_BRACKET[1]);
															temp.divIILine			= self:addWidget(Image, self.mainLayer, 55, 0, 112, 155, DIVISION_BRACKET[2]);
															temp.divIIILine		= self:addWidget(Image, self.mainLayer, 55, 0, 112, 155, DIVISION_BRACKET[3]);
															temp.divOffLine		= self:addWidget(Image, self.mainLayer, 55, 0, 112, 155, DIVISION_BRACKET[4]);
                                             temp.overlay         = self:addWidget(Image, self.mainLayer, 0, 0, 724, LIST_SPACING-4, "resources/UI/League/LC_Overlay.png");


                                             temp.banner:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.gem:setAttrLink(				MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.coin:setAttrLink(				MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.prizeTxt:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.prizeTxtShdw:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.divI:setAttrLink(				MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.divII:setAttrLink(				MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.divIII:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.divITxt:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.divI.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.divITxtShdw:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.divI.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.divIITxt:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.divII.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.divIITxtShdw:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		temp.divII.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.divIIITxt:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.divIII.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.divIIITxtShdw:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		temp.divIII.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.divILine:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.divIILine:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.divIIILine:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.divOffLine:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
                                             temp.overlay:setAttrLink(			MOAIProp2D.INHERIT_TRANSFORM,		temp.root.prop,		MOAIProp2D.TRANSFORM_TRAIT	);

                                             temp.root:setPriority(				0              );
                                             temp.banner:setPriority(			PRIORITY_0     );
                                             temp.gem:setPriority(            PRIORITY_1     );
                                             temp.coin:setPriority(           PRIORITY_1     );
                                             temp.prizeTxt:setPriority(			PRIORITY_2     );
                                             temp.prizeTxtShdw:setPriority(	PRIORITY_2-1	);
                                             temp.divI:setPriority(           PRIORITY_1     );
                                             temp.divII:setPriority(          PRIORITY_1     );
                                             temp.divIII:setPriority(			PRIORITY_1     );
                                             temp.divITxt:setPriority(			PRIORITY_2     );
                                             temp.divITxtShdw:setPriority(		PRIORITY_2-1   );
                                             temp.divIITxt:setPriority(			PRIORITY_2     );
                                             temp.divIITxtShdw:setPriority(	PRIORITY_2-1   );
                                             temp.divIIITxt:setPriority(		PRIORITY_2     );
                                             temp.divIIITxtShdw:setPriority(	PRIORITY_2-1   );
                                             temp.divILine:setPriority(			PRIORITY_1     );
                                             temp.divIILine:setPriority(		PRIORITY_1     );
                                             temp.divIIILine:setPriority(		PRIORITY_1     );
                                             temp.divOffLine:setPriority(		PRIORITY_1     );
                                             temp.overlay:setPriority(			PRIORITY_4-1   );

															return temp;
														end );

end

----------------------------------------------------------------

function StateLeagueCodex:_hideCacheObject( obj )

	if obj then
		for _,v in pairs(obj) do
			if type(v) == "table" then
				if v.stop then
					v:stop();
				end

				if v.disable then
					v:disable();
				end

				if v.setVisible then
					v:setVisible(false);
				elseif v.hide then
					v:hide();
				end
			end
		end
	end

end

----------------------------------------------------------------

function StateLeagueCodex:_createListData()

	local maxLeague	= LeagueMgr:getLowestLeagueDiv();
	local l,d			= LeagueMgr:getPlayerPlacement();

	self._leagueData	= {};

	for i=1,maxLeague do

		local temp        = {};
		local dc          = LeagueMgr:getLeagueDivCount(i);

		temp.leagueIndex	= l;
      temp.divIndex     = d;
      temp.currentIndex = math.min(i, maxLeague);
		temp.reward       = {};

		for k=1,dc do
			table.insert( temp.reward, LeagueMgr:getReward( i, k, 1 ) );
		end

		if i ~= l then
         temp.state     = INACTIVE_LEAGUE;
      else
         temp.state     = ACTIVE_LEAGUE;
      end

		table.insert(self._leagueData, temp);
	end

end

----------------------------------------------------------------

function StateLeagueCodex:_updateList()

	local l,d	= LeagueMgr:getPlayerPlacement();
	self.lIndex	= l;
	self.dIndex	= d;

	self:_createListData();
	self.ScrollList:setEntryCount(nil, #self._leagueData);
	self.ScrollList:setEntryData(self._leagueData);
	self.ScrollList:goToEntry(1);
	self.ScrollList:advanceToEntry( math.clamp(self.lIndex, 1, #self._leagueData), 3/4 );

end


------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------


function StateLeagueCodex:goBack()
	StateMgr:pop();
end


------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------


return StateLeagueCodex;


