----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-scroll-list');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local PRIORITY_0					= 10;
local PRIORITY_1					= 20;
local PRIORITY_2					= 30;
local PRIORITY_3					= 40;
local PRIORITY_4					= 50;
local PRIORITY_5					= 60;
local PRIORITY_6					= 70;
local PRIORITY_7					= 80;
local PRIORITY_8					= 90;
local PRIORITY_9					= 100;
local PRIORITY_10					= 110;

local LIST_HOME_X					= 0;
local LIST_HOME_Y					= -90;
local LIST_SPACING				= 78;

local SCROLL_WIND_MIN_X			= -App.viewportWidth/2;
local SCROLL_WIND_MAX_X			= App.viewportWidth/2;
local SCROLL_WIND_MIN_Y			= -App.viewportHeight/2;
local SCROLL_WIND_MAX_Y			= App.viewportHeight/2;

local SWIPE_BUFFER_X				= 100;
local SWIPE_BUFFER_Y				=  50;

local SQUARE_TEXTURE				= "resources/square.png";

local STATE_DEMOTE_LEAGUE		= "demote-league";
local STATE_DEMOTE_DIVISION	= "demote-division";
local STATE_PROMOTE_LEAGUE		= "promote-league";
local STATE_PROMOTE_DIVISION	= "promote-division";
local STATE_NO_MOVEMENT			= "no-movement";

local PODIUM_LOC					=
{
	{	x=0,		y=265	},
	{	x=-155,	y=225	},
	{	x=155,	y=200	},
};

local PODIUM_K						=
{
	4/3,
	1,
	1,
};

local LEAGUE_TROPHY				=
{
	"resources/UI/League/LC_trophy-dmnd.png",
	"resources/UI/League/LC_trophy-plat.png",
	"resources/UI/League/LC_trophy-gold.png",
	"resources/UI/League/LC_trophy-silv.png",
	"resources/UI/League/LC_trophy-brnz.png",
};


----------------------------------------------------------------

local Base						= State;
local StateLeagueReward		= Base:new();

----------------------------------------------------------------

function StateLeagueReward:onLoad()

   Base.onLoad( self );

   local textureSheet	= FlashMgr:getTextureSheet("resources/flash-resource-bundle/PopUPs.lua", "resources/flash-resource-bundle/PopUPs.png", 30);
   self.mainTexture		= FlashMgr:getTextureSheet("resources/flash-resource-bundle/MainScreen.lua", "resources/flash-resource-bundle/MainScreen.png", 30);

   self.defaultFontName = "resources/font/TangoLatin.otf";
   self.unicodeFontName = "resources/font/Tango.otf";
   self.IS_POPUP			= true;

   -- background
   self.Background		= self:addWidget(Image, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE, {0,0,0, 0.75});
   self.Background:setPriority(0);

   -- frame
   self.FrameReward		= self:addWidget(Image, self.mainLayer, 0, 0, 560, 940, "resources/UI/League/LR_frame_reward.png");
   self.FrameRank			= self:addWidget(Image, self.mainLayer, 0, 0, 560, 940, "resources/UI/League/LR_frame_rank.png");
   self.FrameReward:setPriority(1);
   self.FrameRank:setPriority(1);

   -- reward
   self:_createReward();

   -- podium
   self:_createPodium();

   -- leage list
   self.LeagueList = self:addWidget( ScrollList,
                                     self.mainLayer,
                                     LIST_HOME_X, LIST_HOME_Y,
                                     { {SCROLL_WIND_MIN_X, SCROLL_WIND_MAX_X},	{SCROLL_WIND_MIN_Y, SCROLL_WIND_MAX_Y} },
                                     nil,
                                     nil,
                                     { 0, 0 },
                                     { nil, LIST_SPACING },
                                     { 0, 1 },
                                     nil,
                                     { SWIPE_BUFFER_X, SWIPE_BUFFER_Y },
                                     nil,
                                     nil,
                                     { {SCROLL_WIND_MIN_X, SCROLL_WIND_MAX_X},	{SCROLL_WIND_MIN_Y, SCROLL_WIND_MAX_Y} } );

   self:_createLeagueCache();

   -- friends list
   self.FriendsList = self:addWidget( ScrollList,
                                      self.mainLayer,
                                      LIST_HOME_X, LIST_HOME_Y,
                                      { {SCROLL_WIND_MIN_X, SCROLL_WIND_MAX_X}, {SCROLL_WIND_MIN_Y, SCROLL_WIND_MAX_Y} },
                                      nil,
                                      nil,
                                      { 0, 0 },
                                      { nil, LIST_SPACING },
                                      { 0, 1 },
                                      nil,
                                      { SWIPE_BUFFER_X, SWIPE_BUFFER_Y },
                                      nil,
                                      nil,
                                      { {SCROLL_WIND_MIN_X, SCROLL_WIND_MAX_X}, {SCROLL_WIND_MIN_Y, SCROLL_WIND_MAX_Y} } );

   self:_createFriendsCache();

   -- Exit Button
   self.BackButton = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 270, 460, 0,0, "CloseButton", textureSheet, nil, self.goBack, nil, nil, nil, nil, 40);
   self.BackButton:setPriority(PRIORITY_1);

end

----------------------------------------------------------------

function StateLeagueReward:onActivate()

   Base.onActivate(self);

end

----------------------------------------------------------------

function StateLeagueReward:onFocus()

   Base.onFocus(self);

   self._skipAward = false;

   self:_updateRewardData();
   self:_updateLeagueList();
   self:_updateFriendsList();

end

----------------------------------------------------------------

function StateLeagueReward:onLoseFocus()

   Base.onLoseFocus(self);

   local bountyOffset = App.data:getValue("App.settings.bounty-reset-offset") or 0;

   GuestMgr:cycleGuestFriend();
   self:_awardPrize();
   RankingMgr:resetScore();
   RankingMgr:clearLastWeekRanking();

   if bountyOffset == 0 then
      BountyMgr:resetBounty();
   end

   BountyMgr:resetBountyTimeStamp();

end

----------------------------------------------------------------

function StateLeagueReward:onUpdate( dt )

   Base.onUpdate(self, dt);

end


------------------------------------------------------------------------------------
-- Reward Data
------------------------------------------------------------------------------------


function StateLeagueReward:_createReward()

   -- Reward MainText
   self.RewardMainTxt		= self:addWidget(TextBox, self.mainLayer, 0, 470-190, 500, 200, "This week's results are in!", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0.12, 0.21, 0.27, 1});
   self.RewardSubTxt			= self:addWidget(TextBox, self.mainLayer, 0, 470-270, 500, 200, "YOUR REWARD", self.defaultFontName, 24, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0.12, 0.21, 0.27, 1});

   self.RewardMainTxt:setPriority(PRIORITY_1);
   self.RewardSubTxt:setPriority(PRIORITY_1);

   -- Reward Amount
   self.RewardIcon			= self:addWidget(Image, self.mainLayer, 0, 470-370, 128, 128, "resources/UI/GR_Icon_Gem.png");
   self.RewardText			= self:addWidget(TextBox, self.mainLayer, 0, 470-370, 500, 200, "reward count", self.defaultFontName, 42, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0.16, 0.35, 0.46, 1});
   self.RewardTextShdw		= self:addWidget(TextBox, self.mainLayer, 2, 470-373, 500, 200, "reward count", self.defaultFontName, 42, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});

   self.RewardIcon:setPriority(PRIORITY_1);
   self.RewardText:setPriority(PRIORITY_1);
   self.RewardTextShdw:setPriority(PRIORITY_1-1);

   -- Promotion/Demotion
   self.RewardStatsTxt		= self:addWidget(TextBox, self.mainLayer, 0, -10, 540, 100, "status text", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0.12, 0.21, 0.27, 1});
   self.RewardTrphyTxt		= self:addWidget(TextBox, self.mainLayer, 0, -470+120, 400, 100, "trophy text", self.defaultFontName, 24, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.RewardTrphyTxtShdw	= self:addWidget(TextBox, self.mainLayer, 2, -470+118, 400, 100, "trophy text", self.defaultFontName, 24, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});

   self.RewardStatsTxt:setPriority(PRIORITY_1);
   self.RewardTrphyTxt:setPriority(PRIORITY_1);
   self.RewardTrphyTxtShdw:setPriority(PRIORITY_1-1);

   self.RewardTrophy			= self:addWidget(Image, self.mainLayer, -2, -470+290, 300, 300, LEAGUE_TROPHY[#LEAGUE_TROPHY]);
	self.RewardTrophy:setPriority(PRIORITY_1);

   -- Reward Button
   self.GoPodiumButton	= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0, -460, 0,0, "UpGradeButton", self.mainTexture,	nil, self._showLeagueList, nil, nil, nil, nil, 20);
   self.GoPodiumTxt		= self:addWidget(TextBox, self.mainLayer, -5, 5, 400, 96, "RANKING", self.defaultFontName, 24, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.GoPodiumTxtShdw	= self:addWidget(TextBox, self.mainLayer, -3, 3, 400, 96, "RANKING", self.defaultFontName, 24, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});

   self.GoPodiumButton:setPriority(PRIORITY_1);
   self.GoPodiumTxt:setPriority(PRIORITY_2);
   self.GoPodiumTxtShdw:setPriority(PRIORITY_2-1);
   self.GoPodiumTxt:setParent(self.GoPodiumButton.anim.props[1]);
   self.GoPodiumTxtShdw:setParent(self.GoPodiumButton.anim.props[1]);

end

----------------------------------------------------------------

function StateLeagueReward:_createRewardData()

	local leagueList, leagueRank = RankingMgr:getRankingLeagueListLastWeek();
   local cLgue, cDiv = LeagueMgr:getPlayerPlacement();
   local dLgue       = cLgue;
   local dDiv        = cDiv;

	if (leagueList == nil or next(leagueList) == nil or leagueRank == nil ) then
      --TBD: Find out why this is coming up null

      if BUILD_DEBUG then
         leagueList, leagueRank	= RankingMgr:getRankingLeagueList();
      else
         self._skipAward = true;
         self:goBack();
         return;
      end
	end

   local rewardData = {}
   rewardData.rankI  = leagueRank;
   rewardData.rank   = tostring(leagueRank);
   rewardData.level  = tostring(leagueList[leagueRank].level);
   rewardData.reward = LeagueMgr:getReward(cLgue, cDiv, leagueRank);
   rewardData.mState = STATE_NO_MOVEMENT;

   if leagueList[leagueRank].prevScore > 0 then
      rewardData.score = getFormattedNumber(leagueList[leagueRank].prevScore);
   else
      rewardData.score = "---";
   end

   rewardData.lIndex = cLgue;
   rewardData.dIndex = cDiv;

   if LeagueMgr:canPromote(cLgue, cDiv, leagueRank) then
      rewardData.lIndex, rewardData.dIndex	= LeagueMgr:getPromoResult(cLgue, cDiv);

      if rewardData.lIndex > cLgue then
         rewardData.mState = STATE_PROMOTE_LEAGUE;
      else
         rewardData.mState = STATE_PROMOTE_DIVISION;
      end
   end

   if LeagueMgr:canDemote(cLgue, cDiv, leagueRank) then
      rewardData.lIndex, rewardData.dIndex	= LeagueMgr:getDemoResult(cLgue, cDiv);

      if rewardData.lIndex < cLgue then
         rewardData.mState = STATE_DEMOTE_LEAGUE;
      else
         rewardData.mState = STATE_DEMOTE_DIVISION;
      end
   end

   local ln, dn	= LeagueMgr:getLeagueName(rewardData.lIndex, rewardData.dIndex);

   rewardData.leagueName = (ln or "" ).." League "..(dn or "");

   return rewardData;
end

----------------------------------------------------------------

function StateLeagueReward:_hideReward()

	self.FrameReward:setVisible(false);
   self.RewardMainTxt:setVisible(false);
   self.RewardSubTxt:setVisible(false);
   self.RewardIcon:setVisible(false);
   self.RewardText:setVisible(false);
   self.RewardTextShdw:setVisible(false);
   self.RewardStatsTxt:setVisible(false);
   self.RewardTrphyTxt:setVisible(false);
   self.RewardTrphyTxtShdw:setVisible(false);
   self.GoPodiumButton:setVisible(false);
   self.GoPodiumButton:disable();
   self.GoPodiumTxt:setVisible(false);
   self.GoPodiumTxtShdw:setVisible(false);
	self.RewardTrophy:setVisible(false);

end

----------------------------------------------------------------

function StateLeagueReward:_showReward()

   if not self._rewardData then
      return;
   end

	self.FrameReward:setVisible(true);
   self.RewardMainTxt:setVisible(true);
   self.RewardSubTxt:setVisible(true);
   self.RewardIcon:setVisible(true);
   self.RewardText:setVisible(true);
   self.RewardTextShdw:setVisible(true);
   self.RewardStatsTxt:setVisible(true);
   self.RewardTrphyTxt:setVisible(true);
   self.RewardTrphyTxtShdw:setVisible(true);
   self.GoPodiumButton:setVisible(true);
   self.GoPodiumButton:enable();
   self.GoPodiumTxt:setVisible(true);
   self.GoPodiumTxtShdw:setVisible(true);
	self.RewardTrophy:setVisible(true);

   self:_hidePodium();
   self.LeagueList:hide();
   self.FriendsList:hide();

end

----------------------------------------------------------------

function StateLeagueReward:_updateRewardData()

   self._rewardData = self:_createRewardData();

   if self._rewardData then
      local cLgue, cDiv = LeagueMgr:getPlayerPlacement();

      if self._rewardData.reward then

         local amount = 1;
         local iconX  = 0;
         local iconY  = 470-370;
         local textX  = 0;
         local textY  = 470-373;

         if self._rewardData.reward.gem > 0 then
            self.RewardIcon:swapImage("resources/UI/GR_Icon_Gem.png");

            amount = self._rewardData.reward.gem;
         elseif self._rewardData.reward.gold > 0 then
            self.RewardIcon:swapImage("resources/UI/GR_Icon_Coin.png");

            amount = self._rewardData.reward.gold;
         end

			self.RewardText:setText( tostring(amount) );
         self.RewardTextShdw:setText( tostring(amount) );

         if amount > 9999 then
            iconX = -135;
            textX = 55;
         elseif amount > 999 then
            iconX = -105;
            textX = 60;
         elseif amount > 99 then
            iconX = -80;
            textX = 60;
         elseif amount > 9 then
            iconX = -60;
            textX = 55;
         else
            iconX = -30;
            textX = 55;
         end

         self.RewardIcon:setLoc(iconX, iconY);
         self.RewardText:setLoc(textX, textY);
         self.RewardTextShdw:setLoc(textX+2, textY-2);
      end

      self.RewardTrophy:swapImage( LEAGUE_TROPHY[math.clamp(self._rewardData.lIndex, 1, #LEAGUE_TROPHY)] );

      if self._rewardData.mState == STATE_DEMOTE_LEAGUE or self._rewardData.mState == STATE_DEMOTE_DIVISION then
         self.RewardStatsTxt:setText("You've Been Demoted!");
      elseif self._rewardData.mState == STATE_PROMOTE_LEAGUE or self._rewardData.mState == STATE_PROMOTE_DIVISION then
         self.RewardStatsTxt:setText("You've Been Promoted!");
      else
         self.RewardStatsTxt:setText("You Remain In:");
      end

      self.RewardTrphyTxt:setText(self._rewardData.leagueName);
      self.RewardTrphyTxtShdw:setText(self._rewardData.leagueName);

      if math.clamp(self._rewardData.lIndex, 1, #LEAGUE_TROPHY) > 1 then
         self.RewardStatsTxt:setLoc(0, -30);
      else
         self.RewardStatsTxt:setLoc(0, -10);
      end
   end

   self:_showReward();
end


------------------------------------------------------------------------------------
-- Podium
------------------------------------------------------------------------------------


function StateLeagueReward:_createPodium()

	-- picture frames
	self.picFrames	= {};
   self.picShadow = {};

	for i=1,3 do
      self.picFrames[i]	= self:addWidget(Image, self.mainLayer, PODIUM_LOC[i].x, PODIUM_LOC[i].y, PORTRAIT_FRAME_WIDTH*PODIUM_K[i], PORTRAIT_FRAME_HEIGHT*PODIUM_K[i], "resources/UI/PlayerPortraitRing.png", { 30/255, 54/255, 69/255, 1});
      self.picShadow[i]	= self:addWidget(Image, self.mainLayer, PODIUM_LOC[i].x+5, PODIUM_LOC[i].y-5, PORTRAIT_FRAME_WIDTH*PODIUM_K[i], PORTRAIT_FRAME_HEIGHT*PODIUM_K[i], "resources/UI/League/LR_playerpicshadow.png", { 104/255, 161/255, 200/255, 1});

		self.picFrames[i]:setPriority(PRIORITY_2);
      self.picShadow[i]:setPriority(PRIORITY_1-1);
	end

	-- pictures
	self.picLeague	= {};
	self.picFriend	= {};

	for i=1,3 do
		self.picLeague[i]	= self:addWidget(Image, self.mainLayer, PODIUM_LOC[i].x, PODIUM_LOC[i].y, PORTRAIT_WIDTH*PODIUM_K[i], PORTRAIT_HEIGHT*PODIUM_K[i], "resources/UI/PlayerPortrait.png");
		self.picFriend[i]	= self:addWidget(Image, self.mainLayer, PODIUM_LOC[i].x, PODIUM_LOC[i].y, PORTRAIT_WIDTH*PODIUM_K[i], PORTRAIT_HEIGHT*PODIUM_K[i], "resources/UI/PlayerPortrait.png");
		self.picLeague[i]:setPriority(PRIORITY_1);
		self.picFriend[i]:setPriority(PRIORITY_1);
	end

	-- League/Friend Tabs
	self.LeagueTabOn	= self:addWidget(Image, self.mainLayer, -113, -26, 224, 50, "resources/UI/League/LR_leaguetab_on.png");
	self.LeagueTabOff	= self:addWidget(ButtonImage, self.mainLayer, -113, -26, 224, 50, "resources/UI/League/LR_leaguetab_off.png", self._showLeagueList, nil, nil, nil);
	self.FriendTabOn	= self:addWidget(Image, self.mainLayer, 113, -26, 224, 50, "resources/UI/League/LR_friendstab_on.png");
	self.FriendTabOff	= self:addWidget(ButtonImage, self.mainLayer, 113, -26, 224, 50, "resources/UI/League/LR_friendstab_off.png", self._showFriendList, nil, nil, nil);

	self.LeagueTabOn:setPriority(PRIORITY_1);
	self.LeagueTabOff:setPriority(PRIORITY_1);
	self.FriendTabOn:setPriority(PRIORITY_1);
	self.FriendTabOff:setPriority(PRIORITY_1);

	-- Reward Button
	self.GoRewardButton	= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0, -460, 0,0, "UpGradeButton", self.mainTexture,	nil, self._showReward, nil, nil, nil, nil, 20);
	self.GoRewardTxt		= self:addWidget(TextBox, self.mainLayer, -5, 5, 400, 96, "RESULTS", self.defaultFontName, 24, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.GoRewardTxtShdw	= self:addWidget(TextBox, self.mainLayer, -3, 3, 400, 96, "RESULTS", self.defaultFontName, 24, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});

	self.GoRewardButton:setPriority(PRIORITY_1);
	self.GoRewardTxt:setPriority(PRIORITY_2);
	self.GoRewardTxtShdw:setPriority(PRIORITY_2-1);
	self.GoRewardTxt:setParent(self.GoRewardButton.anim.props[1]);
	self.GoRewardTxtShdw:setParent(self.GoRewardButton.anim.props[1]);

end

----------------------------------------------------------------

function StateLeagueReward:_hidePodium()

	self.FrameRank:setVisible(false);
	self.LeagueTabOn:setVisible(false);
	self.LeagueTabOff:setVisible(false);
	self.LeagueTabOff:disable();
	self.FriendTabOn:setVisible(false);
	self.FriendTabOff:setVisible(false);
	self.FriendTabOff:disable();
	self.GoRewardButton:setVisible(false);
	self.GoRewardButton:disable();
	self.GoRewardTxt:setVisible(false);
	self.GoRewardTxtShdw:setVisible(false);

	for i=1,3 do
		self.picFrames[i]:setVisible(false);
		self.picShadow[i]:setVisible(false);
		self.picLeague[i]:setVisible(false);
		self.picFriend[i]:setVisible(false);
	end

end

----------------------------------------------------------------

function StateLeagueReward:_showLeaguePodium()

   if not Debug.assertType(self._leagueListData, "table", "StateLeagueReward:_showLeaguePodium -> self._leagueListData") then
      return;
   end

	self.FrameRank:setVisible(true);
   self.LeagueTabOn:setVisible(true);
   self.LeagueTabOff:setVisible(false);
   self.LeagueTabOff:disable();
   self.FriendTabOn:setVisible(false);
   self.FriendTabOff:setVisible(true);
   self.FriendTabOff:enable();
   self.GoRewardButton:setVisible(true);
   self.GoRewardButton:enable();
   self.GoRewardTxt:setVisible(true);
   self.GoRewardTxtShdw:setVisible(true);

   for i = 1, 3 do
      self.picFrames[i]:setVisible(false);
		self.picShadow[i]:setVisible(false);
      self.picFriend[i]:setVisible(false);
   end

   for i = 1, math.min(3, #self._leagueListData) do
      self.picFrames[i]:setVisible(true);
		self.picShadow[i]:setVisible(true);
      self.picLeague[i]:setVisible(true);
   end

end

----------------------------------------------------------------

function StateLeagueReward:_showFriendPodium()

   if not Debug.assertType(self._friendListData, "table", "StateLeagueReward:_showFriendPodium -> self._friendListData") then
      return;
   end

	self.FrameRank:setVisible(true);
   self.LeagueTabOn:setVisible(false);
   self.LeagueTabOff:setVisible(true);
   self.LeagueTabOff:enable();
   self.FriendTabOn:setVisible(true);
   self.FriendTabOff:setVisible(false);
   self.FriendTabOff:disable();

   for i = 1, 3 do
      self.picFrames[i]:setVisible(false);
		self.picShadow[i]:setVisible(false);
      self.picLeague[i]:setVisible(false);
   end

   for i = 1, math.min(3, #self._friendListData) do
      self.picFrames[i]:setVisible(true);
		self.picShadow[i]:setVisible(true);
      self.picFriend[i]:setVisible(true);
   end

end

----------------------------------------------------------------

function StateLeagueReward:_updateLeaguePodium()

   if not Debug.assertType(self._leagueListData, "table", "StateLeagueReward:_showLeaguePodium -> self._leagueListData") then
      return;
   end

   for i = 1, math.min(3, #self._leagueListData) do
      self.picLeague[i]:swapImage(self._leagueListData[i].pic);
   end

   self:_hidePodium();

end

----------------------------------------------------------------

function StateLeagueReward:_updateFriendPodium()

   if not Debug.assertType(self._friendListData, "table", "StateLeagueReward:_showFriendPodium -> self._friendListData") then
      return;
   end

   for i = 1, math.min(3, #self._friendListData) do
      self.picFriend[i]:swapImage(self._friendListData[i].pic);
   end

   self:_hidePodium();

end


------------------------------------------------------------------------------------
-- League Data
------------------------------------------------------------------------------------


function StateLeagueReward:_createLeagueCache()

	self.LeagueList:setEntryObjectDataUpdateFunction(	function( data, object )
																			self.LeagueList:hideCacheEntry(object);
																			object.rank:setVisible(true);
																			object.name:setVisible(true);
																			object.score:setVisible(true);

																			if data:isPlayer() then
																				object.pbanner:setVisible(true);
																			else
																				object.banner:setVisible(true);
																			end

																			object.rank:setText(data.rank);
																			object.name:setMetaFont("resources/font/Tango.otf", 14, data.name);
																			object.name:setText(data.name);
																			object.score:setText(data.score);

																			if data.index == 1 then
                                                            object.badge_1:setVisible(true);
																			elseif data.index == 2 then
                                                            object.badge_2:setVisible(true);
																			elseif data.index == 3 then
                                                            object.badge_3:setVisible(true);
																			else
																				object.pbanner:setLoc(0, -LIST_SPACING/4);
																				object.rank:setLoc(-185, -LIST_SPACING/4);
																				object.name:setLoc(10, -LIST_SPACING/4);
																				object.score:setLoc(-10, -LIST_SPACING/4);
																			end
																		end
																		);

	self.LeagueList:setEntryObjectLocUpdateFunction(	function( x, y, object )
																			object.banner:setLoc(x,y);
																		end
																	);

	self.LeagueList:setEntryObjectParentUpdateFunction(	function( target, object )
																				object.banner:setParent(target);
																			end
																		);

	self.LeagueList:setEntryCacheObjects(	nil,
														4,
														function()
															local temp = {};

															temp.banner		= self:addWidget(Image, self.mainLayer, 0, 0, 450, LIST_SPACING, "resources/UI/League/LR_banner.png");
															temp.pbanner	= self:addWidget(Image, self.mainLayer, 0, 0, 450, LIST_SPACING, "resources/UI/League/LR_playerbanner.png");
															temp.rank		= self:addWidget(TextBox, self.mainLayer, -185, 0, 100, LIST_SPACING, "rank", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
															temp.name		= self:addWidget(TextBox, self.mainLayer, 10, 0, 300, LIST_SPACING, "name", self.unicodeFontName, 14, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
															temp.score		= self:addWidget(TextBox, self.mainLayer, -10, 0, 440, LIST_SPACING, "score", self.defaultFontName, 14, nil, MOAITextBox.RIGHT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
															temp.badge_1	= self:addWidget(Image, self.mainLayer, -187, 0, 60, 60, "resources/UI/League/LR_Badge_1.png");
															temp.badge_2	= self:addWidget(Image, self.mainLayer, -187, 0, 60, 60, "resources/UI/League/LR_Badge_2.png");
															temp.badge_3	= self:addWidget(Image, self.mainLayer, -187, 0, 60, 60, "resources/UI/League/LR_Badge_3.png");

															temp.pbanner:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.rank:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.name:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.score:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.badge_1:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.badge_2:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.badge_3:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);

															temp.banner:setPriority(PRIORITY_0+1);
															temp.pbanner:setPriority(PRIORITY_0+2);
															temp.rank:setPriority(PRIORITY_1);
															temp.name:setPriority(PRIORITY_1);
															temp.score:setPriority(PRIORITY_1);
															temp.badge_1:setPriority(PRIORITY_2);
															temp.badge_2:setPriority(PRIORITY_2);
															temp.badge_3:setPriority(PRIORITY_2);

															return temp;
														end
													);

end

----------------------------------------------------------------

function StateLeagueReward:_createLeagueData()

	local lList, lRank	= RankingMgr:getRankingLeagueListLastWeek();
	local playerId			= ContactMgr:getProfile():getAccountId();
	local lIndex			= 4;

	if (lList == nil or next(lList) == nil )and BUILD_DEBUG then
		lList, lRank	= RankingMgr:getRankingLeagueList();
	end

	if lRank > 3 then
		lIndex	= lRank;
	end

	self._leagueListData	= {};

	for i=1,math.min( 3, #lList) do
		local temp	= {};

		temp.index	= i;
		temp.rank	= tostring(i);
		temp.level	= tostring(lList[i].level);
		temp.pic		= ContactMgr:getContactPicture(lList[i].contact) or "resources/UI/PlayerPortrait.png";

		if lList[i].tangoId == playerId then
			temp.name		= "<c:000000>"..createUserNameDisplay(lList[i].contact:getFirstName(), '', true, "You", 12);
			temp.isPlayer	= function() return true end
		else
			temp.name		= "<c:000000>"..createUserNameDisplay(lList[i].contact:getFirstName(), '', true, "Friend", 12);
			temp.isPlayer	= function() return false end
		end

		if lList[i].prevScore > 0 then
			temp.score	= "<c:000000>"..getFormattedNumber(lList[i].prevScore);
		else
			temp.score	= "<c:000000>---";
			temp.rank	= "<c:000000>---";
		end

		table.insert(self._leagueListData, temp);
	end

	if lRank > 3 and #lList > 3 and lList[lIndex] then
		local temp	= {};

		temp.index	= 4;
		temp.rank	= tostring(lIndex);
		temp.level	= tostring(lList[lIndex].level);
		temp.pic		= ContactMgr:getContactPicture(lList[lIndex].contact) or "resources/UI/PlayerPortrait.png";

		if lList[lIndex].tangoId == playerId then
			temp.name		= createUserNameDisplay(lList[lIndex].contact:getFirstName(), '', true, "You", 12);
			temp.isPlayer	= function() return true end
		else
			temp.name		= createUserNameDisplay(lList[lIndex].contact:getFirstName(), '', true, "Friend", 12);
			temp.isPlayer	= function() return false end
		end

		if lList[lIndex].prevScore > 0 then
			temp.score = getFormattedNumber(lList[lIndex].prevScore);
		else
			temp.score	= "---";
			temp.rank	= "-";
		end

		table.insert(self._leagueListData, temp);
	end

end

----------------------------------------------------------------

function StateLeagueReward:_showLeagueList()

	self:_hideReward();
	self:_showLeaguePodium();
	self.LeagueList:show();
	self.FriendsList:hide();
	self.LeagueList:setEntryData(self._leagueListData);

end

----------------------------------------------------------------

function StateLeagueReward:_updateLeagueList()

	self:_createLeagueData();

	self.LeagueList:goToEntry(1);
	self.LeagueList:setEntryCount(nil, #self._leagueListData);
	self.LeagueList:setEntryData(self._leagueListData);
	self.LeagueList:hide();
	self:_updateLeaguePodium();

end


------------------------------------------------------------------------------------
-- Friends List Data
------------------------------------------------------------------------------------


function StateLeagueReward:_createFriendsCache()

	self.FriendsList:setEntryObjectDataUpdateFunction(	function( data, object )
																			self.LeagueList:hideCacheEntry(object);
																			object.rank:setVisible(true);
																			object.name:setVisible(true);
																			object.score:setVisible(true);

																			if data:isPlayer() then
																				object.pbanner:setVisible(true);
																			else
																				object.banner:setVisible(true);
																			end

																			object.rank:setText(data.rank);
																			object.name:setMetaFont("resources/font/Tango.otf", 14, data.name);
																			object.name:setText(data.name);
																			object.score:setText(data.score);

																			if data.index == 1 then
                                                            object.badge_1:setVisible(true);
																			elseif data.index == 2 then
                                                            object.badge_2:setVisible(true);
																			elseif data.index == 3 then
                                                            object.badge_3:setVisible(true);
																			else
																				object.pbanner:setLoc(0, -LIST_SPACING/4);
																				object.rank:setLoc(-185, -LIST_SPACING/4);
																				object.name:setLoc(10, -LIST_SPACING/4);
																				object.score:setLoc(-10, -LIST_SPACING/4);
																			end
																		end
																		);

	self.FriendsList:setEntryObjectLocUpdateFunction(	function( x, y, object )
																			object.banner:setLoc(x,y);
																		end
																	);

	self.FriendsList:setEntryObjectParentUpdateFunction(	function( target, object )
																				object.banner:setParent(target);
																			end
																		);

	self.FriendsList:setEntryCacheObjects(	nil,
														4,
														function()
															local temp = {};

															temp.banner		= self:addWidget(Image, self.mainLayer, 0, 0, 450, LIST_SPACING, "resources/UI/League/LR_banner.png");
															temp.pbanner	= self:addWidget(Image, self.mainLayer, 0, 0, 450, LIST_SPACING, "resources/UI/League/LR_playerbanner.png");
															temp.rank		= self:addWidget(TextBox, self.mainLayer, -185, 0, 100, LIST_SPACING, "rank", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
															temp.name		= self:addWidget(TextBox, self.mainLayer, 10, 0, 300, LIST_SPACING, "name", self.unicodeFontName, 14, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
															temp.score		= self:addWidget(TextBox, self.mainLayer, -10, 0, 440, LIST_SPACING, "score", self.defaultFontName, 14, nil, MOAITextBox.RIGHT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
															temp.badge_1	= self:addWidget(Image, self.mainLayer, -187, 0, 60, 60, "resources/UI/League/LR_Badge_1.png");
															temp.badge_2	= self:addWidget(Image, self.mainLayer, -187, 0, 60, 60, "resources/UI/League/LR_Badge_2.png");
															temp.badge_3	= self:addWidget(Image, self.mainLayer, -187, 0, 60, 60, "resources/UI/League/LR_Badge_3.png");

															temp.pbanner:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.rank:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.name:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.score:setAttrLink(		MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.badge_1:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.badge_2:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);
															temp.badge_3:setAttrLink(	MOAIProp2D.INHERIT_TRANSFORM,		temp.banner.prop,		MOAIProp2D.TRANSFORM_TRAIT	);

															temp.banner:setPriority(PRIORITY_0+1);
															temp.pbanner:setPriority(PRIORITY_0+2);
															temp.rank:setPriority(PRIORITY_1);
															temp.name:setPriority(PRIORITY_1);
															temp.score:setPriority(PRIORITY_1);
															temp.badge_1:setPriority(PRIORITY_2);
															temp.badge_2:setPriority(PRIORITY_2);
															temp.badge_3:setPriority(PRIORITY_2);

															return temp;
														end
													);

end

----------------------------------------------------------------

function StateLeagueReward:_createFriendData()

	local fList, fRank	= RankingMgr:getRankingFriendsListLastWeek();
	local playerId			= ContactMgr:getProfile():getAccountId();
	local lIndex			= 4;

	if (fList == nil or next(fList) == nil )and BUILD_DEBUG then
		fList, fRank	= RankingMgr:getRankingFriendsList();
	end

	if fRank > 3 then
		lIndex	= fRank;
	end

	self._friendListData	= {};

	for i=1,math.min( 3, #fList) do
		local temp	= {};

		temp.index	= i;
		temp.rank	= tostring(i);
		temp.level	= tostring(fList[i].level);
		temp.pic		= ContactMgr:getContactPicture(fList[i].contact) or "resources/UI/PlayerPortrait.png";

		if temp.tangoID == playerId then
			temp.name		= "<c:000000>"..createUserNameDisplay(fList[i].contact:getFirstName(), '', true, "You", 12);
			temp.isPlayer	= function() return true end
		else
			temp.name		= "<c:000000>"..createUserNameDisplay(fList[i].contact:getFirstName(), '', true, "Friend", 12);
			temp.isPlayer	= function() return false end
		end

		if fList[i].prevScore > 0 then
			temp.score = "<c:000000>"..getFormattedNumber(fList[i].prevScore);
		else
			temp.score	= "<c:000000>---";
			temp.rank	= "<c:000000>---";
		end

		table.insert(self._friendListData, temp);
	end

	if fRank > 3 and #fList > 3 and fList[lIndex] then
		local temp	= {};

		temp.index	= 4;
		temp.rank	= tostring(lIndex);
		temp.level	= tostring(fList[lIndex].level);
		temp.pic		= ContactMgr:getContactPicture(fList[lIndex].contact) or "resources/UI/PlayerPortrait.png";

		if temp.tangoID == playerId then
			temp.name		= createUserNameDisplay(fList[lIndex].contact:getFirstName(), '', true, "You", 12);
			temp.isPlayer	= function() return true end
		else
			temp.name		= createUserNameDisplay(fList[lIndex].contact:getFirstName(), '', true, "Friend", 12);
			temp.isPlayer	= function() return false end
		end

		if fList[lIndex].prevScore > 0 then
			temp.score = getFormattedNumber(fList[lIndex].prevScore);
		else
			temp.score	= "---";
			temp.rank	= "---";
		end

		table.insert(self._friendListData, temp);
	end

end

----------------------------------------------------------------

function StateLeagueReward:_showFriendList()

	self:_hideReward();
	self:_showFriendPodium();
	self.LeagueList:hide();
	self.FriendsList:show();

end

----------------------------------------------------------------

function StateLeagueReward:_updateFriendsList()

	self:_createFriendData();

	self.FriendsList:goToEntry(1);
	self.FriendsList:setEntryCount(nil, #self._friendListData);
	self.FriendsList:setEntryData(self._friendListData);
	self.FriendsList:hide();
	self:_updateFriendPodium();

end


------------------------------------------------------------------------------------
-- Util
------------------------------------------------------------------------------------


function StateLeagueReward:isUsingCameraHack()
   return false;
end


------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------


function StateLeagueReward:_awardPrize()

   if self._skipAward then
      return;
   end

	local cLgue, cDiv = LeagueMgr:getPlayerPlacement();

	if cLgue and cDiv and self._rewardData.rankI then
		LeagueMgr:awardPrize(cLgue, cDiv, self._rewardData.rankI);
	end

end

----------------------------------------------------------------

function StateLeagueReward:goBack()
	StateMgr:pop();
end


------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------


return StateLeagueReward;
