----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local PRIORTY_0				= 10;
local PRIORTY_1				= 20;
local PRIORTY_2				= 30;
local PRIORTY_3				= 40;
local PRIORTY_4				= 50;
local PRIORTY_5				= 60;
local PRIORTY_6				= 70;
local PRIORTY_7				= 80;
local PRIORTY_8				= 90;
local PRIORTY_9				= 100;
local PRIORTY_10				= 110;

local FADE_IN_TIME			= 10/30;
local FADE_IN_SCALE			= FADE_IN_TIME/1.0;

local NUM_WIDTH				= 64*1.5;
local NUM_HEIGHT				= 128*1.5;

local NUMBER					=
{
	"resources/UI/LevelUp/1.png",
	"resources/UI/LevelUp/2.png",
	"resources/UI/LevelUp/3.png",
	"resources/UI/LevelUp/4.png",
	"resources/UI/LevelUp/5.png",
	"resources/UI/LevelUp/6.png",
	"resources/UI/LevelUp/7.png",
	"resources/UI/LevelUp/8.png",
	"resources/UI/LevelUp/9.png",
	"resources/UI/LevelUp/0.png",
};

----------------------------------------------------------------

local Base							= State;
local StateLevelUpPopUp			= Base:new();

----------------------------------------------------------------

function StateLevelUpPopUp:onLoad()

   State.onLoad( self );

   self.defaultFontName = "resources/font/TangoLatin.otf";

   local textureSheet   = FlashMgr:getTextureSheet("resources/flash-resource-bundle/PopUPs.lua", "resources/flash-resource-bundle/PopUPs.png", 30);
	local wipeTexture    = FlashMgr:getTextureSheet("resources/flash-resource-bundle/Group2.lua", "resources/flash-resource-bundle/Group2.png", 30);

	self.vehicletexture  = FlashMgr:getTextureSheet("resources/flash-resource-bundle/Group3.lua", "resources/flash-resource-bundle/Group3.png", 30);
	self.IS_POPUP        = true;

   -- background
   self.Background   = self:addWidget(Image, self.mainLayer, 0,0, 1200, 1400, "resources/square.png", {0,0,0, 0.75});
   self.Background:setPriority(0);

   -- frame
   self.Frame        = self:addWidget(Image, self.mainLayer, 0, 0, 550, 830, "resources/UI/LevelUp/LevelUpFrame.png");
   self.Frame:setPriority(1);

	-- level icon
	self:_setLevelIcon(1234567890);

	-- smoke wipe anim
	self.FadeActive   = true;
	self.FadeIn       = self:addWidget(Anim, self.mainLayer, 0,0,0,0, "WhiteFade", wipeTexture);
	self.FadeIn:setPriority(PRIORTY_2);

	-- push/pop state buttons
	self.BackButton = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0, -55, 0,0, "LargeOkButton", textureSheet, nil, self.goBack, nil, nil, nil, nil, 40);
	self.BackButton:setPriority(PRIORTY_0);
   self.BackButton:disable();

   -- icon
   self.GemIcon   = self:addWidget(Image, self.mainLayer, -180, -45, 90, 90, "resources/UI/GR_Icon_Gem.png");
   self.CoinIcon  = self:addWidget(Image, self.mainLayer, -178, -45, 96, 96, "resources/UI/GR_Icon_Coin.png");
   self.GemIcon:setPriority(PRIORTY_0);
   self.CoinIcon:setPriority(PRIORTY_0);

	-- text message
   self.RewardText      = self:addWidget(TextBox, self.mainLayer, -20, -45, 200, 260, '', self.defaultFontName, 18, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.RewardTextShdw  = self:addWidget(TextBox, self.mainLayer, -20+2, -45-2, 200, 260, '', self.defaultFontName, 18, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
   self.RewardText:setPriority(PRIORTY_0);
   self.RewardTextShdw:setPriority(PRIORTY_0-1);
   self.RewardText:setScale(1.1, 1.1);
   self.RewardTextShdw:setScale(1.1, 1.1);

   self.BonusText       = self:addWidget(TextBox, self.mainLayer, 216, -45, 200, 260, '', self.defaultFontName, 18, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.BonusTextShdw   = self:addWidget(TextBox, self.mainLayer, 216+2, -45-2, 200, 260, '', self.defaultFontName, 18, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
   self.BonusText:setPriority(PRIORTY_0);
   self.BonusTextShdw:setPriority(PRIORTY_0-1);
   self.BonusText:setScale(1.1, 1.1);
   self.BonusTextShdw:setScale(1.1, 1.1);

	self.MessageText     = self:addWidget(TextBox, self.mainLayer, 5, -175, 500, 260, '', self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.MessageTextShdw = self:addWidget(TextBox, self.mainLayer, 5+2, -175-2, 500, 260, '', self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.MessageText:setPriority(PRIORTY_0);
	self.MessageTextShdw:setPriority(PRIORTY_0-1);
   self.MessageText:setScale(1, 1);
   self.MessageTextShdw:setScale(1, 1);

end

----------------------------------------------------------------

function StateLevelUpPopUp:onActivate()

   State.onActivate(self);

	App.data:addRewardLevel();
	self:startFadeAnim();

   self.feedShareTimer = 0;

end

----------------------------------------------------------------

function StateLevelUpPopUp:onFocus( data )

   State.onFocus(self);

   local lvl = App.data:getMissionUnlockLevel();
   if lvl then
	  if data.rewardLevel == lvl then
         App:updateMissionActivate();
      end
   end

   App:updateAchievementActivate();


   local reward = App.data:getLevelReward(data.rewardLevel);

   self:_setLevelIcon(data.rewardLevel);

   if data.func then
	  data.func();
   end

   self:setMessage(reward, data.rewardLevel);
   RankingMgr:forceUpdate();

end

------------------------------------------------------------------------------------

function StateLevelUpPopUp:onUpdate( dt )

   Base.onUpdate( self, dt );

	if self.wipeActive then

		self.vehicleTime = self.vehicleTime + dt;

		local scale = math.clamp(self.vehicleTime/VEHC_TIME, 0, 1);

		self.VehiclePivot:setLoc(Vehicle_Start.x + (Vehicle_End.x * scale), Vehicle_Start.y + (Vehicle_End.y * scale));
	end

   if self.feedShareTimer then

      self.feedShareTimer = self.feedShareTimer + dt;

      if self.feedShareTimer > 0.5 then

         if not GuideMgr:isActive() then
            FeedMgr:shareScreenshot( "LevelUp" );
         end
         self.feedShareTimer = nil;
         self.BackButton:enable();
      end
   end


end

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

function StateLevelUpPopUp:_setLevelIcon( rewardLevel )

	if self.LevelIcon then
		for i=1,#self.LevelIcon do
			self.LevelIcon[i]:delete();
		end
	end

	local n		= {}
	local level	= rewardLevel;

	while level >= 1 do
		table.insert(n, 1, level%10);
		level	= math.floor(level/10);
	end

	self.LevelIcon = {};

	local k = 1 - 0.15*(#n-1);
	local x = -(NUM_WIDTH * k * #n)/2 + (NUM_WIDTH * k)/2;

	if #n > 1 then
		if n[1] == 1 then
			x = x - 6*k;
		end

		if n[#n] == 1 then
			x = x + 4*k;
		end
	end

	for i=1,#n do
		if n[i] > 0 then
			self.LevelIcon[i] = self:addWidget(Image, self.mainLayer, x+((i-1)*NUM_WIDTH*k), 140, NUM_WIDTH*k, NUM_HEIGHT*k, NUMBER[n[i]]);
		else
			self.LevelIcon[i] = self:addWidget(Image, self.mainLayer, x+((i-1)*NUM_WIDTH*k), 140, NUM_WIDTH*k, NUM_HEIGHT*k, NUMBER[10]);
		end

		self.LevelIcon[i]:setPriority(PRIORTY_1+1);
	end

end

------------------------------------------------------------------------------------

function StateLevelUpPopUp:setMessage( reward, rewardLevel )

   Debug.assert(reward, "reward was nil!");
   Debug.assert(rewardLevel, "rewardLevel was nil!");

   local rtext = "0";
   local btext = "1%"
   local text  = "";
   local count = 0;

   if reward.gold > 0 then
		rtext = tostring(reward.gold);
      self.GemIcon:setVisible(false);
      self.CoinIcon:setVisible(true);
	end
	
	if reward.gem > 0 then
		rtext = tostring(reward.gem);
      self.GemIcon:setVisible(true);
      self.CoinIcon:setVisible(false);
	end

	if			reward.unlockVehc > 1
		and	reward.unlockVehc <= App.data:getPlayerVehicleCount() then
		
		if App.data:getValue("hint_max_view_count_8", 0) <= 0 then
			App.data:setValue("hint_screen_index", 7);
		end

		if			rewardLevel == App.data:getVehicleUnlockableLevel(reward.unlockVehc)
			and	not App.data:isPlayerVehicleUnlocked(reward.unlockVehc) then

		 local name =
			{
				"Sports Car",
				"Magnet Car",
				"Missile Car",
				"Laser Car",
				"Blast Car",
				
				"Sports Car II",
				"Magnet Car II",
				"Missile Car II",
				"Laser Car II",
				"Blast Car II",
			};
         count = count + 1;
			text = text..name[math.clamp(reward.unlockVehc, 1, #name)].." Available\n";
		end
	end

   if reward.unlockMission then

	  if App.data:getValue("hint_max_view_count_5", 0) <= 0 then
			App.data:setValue("hint_screen_index", 4);
	  end

	  if rewardLevel == App.data:getMissionUnlockLevel() then
         count = count + 1;
			text = text.."Missions Unlocked\n";
	  end
   end

   if reward.unlockDaily then

	  if App.data:getValue("hint_max_view_count_2", 0) <= 0 then
			App.data:setValue("hint_screen_index", 1);
	  end

	  if rewardLevel == App.data:getDailyUnlockLevel() then
			count = count + 1;
			text = text.."Daily Unlocked\n";
	  end
   end

   if reward.unlockBounty then

		if App.data:getValue("hint_max_view_count_1", 0) <= 0 then
			App.data:setValue("hint_screen_index", 0);
		end

		if rewardLevel == App.data:getBountyUnlockLevel() then
         count = count + 1;
         text = text.."Bounties Unlocked\n";
      end
   end

   if reward.unlockRevive then

      -- disable force hint due to overlapping with bounty
      --[[
		if App.data:getValue("hint_max_view_count_4", 0) <= 0 then
			App.data:setValue("hint_screen_index", 3);
		end
      ]]

	  if rewardLevel == App.data:getReviveUnlockLevel() then
         count = count + 1;
         text = text.."Revive Unlocked\n";
	  end
   end

	if reward.unlockConsumable then

      -- disable force hint due to overlapping with mission
      --[[
		if App.data:getValue("hint_max_view_count_6", 0) <= 0 then
			App.data:setValue("hint_screen_index", 5);
		end
      ]]

		if rewardLevel == App.data:getConsumableUnlockLevel() then
			count = count + 1;
			text = text.."Consumables Unlocked\n";
	  end
   end

	if DroneMgr:isEnabled() and not DroneMgr:isDroneLevelLocked() then

		if App.data:getValue("hint_max_view_count_9", 0) <= 0 then
			App.data:setValue("hint_screen_index", 8);
		end

		if rewardLevel == DroneMgr:getDroneLevelUnlock() then
			count = count + 1;
			text = text.."Drones Unlocked\n";
	  end
   end

   if reward.unlockMystery then

      if rewardLevel == App.data:getMysteryUnlockLevel() then
         count = count + 1;
         text = text.."Mystery Box Unlocked\n";
      end
   end

   if reward.unlockBoss then

      -- disable force hint due to overlapping with league
      --[[
		if App.data:getValue("hint_max_view_count_7", 0) <= 0 then
			App.data:setValue("hint_screen_index", 6);
		end
      ]]

	  if rewardLevel == App.data:getBossUnlockLevel() then
         count = count + 1;
         text = text.."Boss Unlocked\n";
	  end
   end

   if reward.unlockAchievement then

      if rewardLevel == App.data:getAchievementUnlockLevel() then
         count = count + 1;
         text = text.."Achievements Unlocked\n";
      end
   end

   if reward.unlockFriends then

      if rewardLevel == App.data:getFriendsUnlockLevel() then
         count = count + 1;
         text = text.."Leaderboard Unlocked\n";
      end
   end

   if LeagueMgr:isLeagueEnabled() and reward.unlockLeague then

		if App.data:getValue("hint_max_view_count_29", 0) <= 0 then
			App.data:setValue("hint_screen_index", 28);
		end

      if rewardLevel == App.data:getLeagueUnlockLevel() then
         count = count + 1;
         text = text.."League Races Unlocked\n";

         RankingMgr:loadGuests();
      end
   end

   if reward.unlockScoreBonus > 0 then
		btext  = tostring(App.data:getScoreBonus(rewardLevel)*100).."%";
	end

   if count < 2 then
      text  = text.."Keys Replenished";
   end

   self.RewardText:setText(rtext);
   self.RewardTextShdw:setText(rtext);
   self.BonusText:setText(btext);
   self.BonusTextShdw:setText(btext);
	self.MessageText:setText(text);
	self.MessageTextShdw:setText(text);
   self.MessageText:setLineSpacing(10);
   self.MessageTextShdw:setLineSpacing(10);

end

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

function StateLevelUpPopUp:startFadeAnim()

	self.FadeActive	= true;

	self.FadeIn:stop();
	self.FadeIn:setVisible(true);
	self.FadeIn:setTime(0);
	self.FadeIn:setListener(	MOAITimer.EVENT_TIMER_END_SPAN,
										function ()
											self.FadeActive = false;
											delay(	0.1,
														function ()
															self.FadeIn:setVisible(false);
														end	);

										end
									);

	self.FadeIn:start();

end

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

function StateLevelUpPopUp:isUsingCameraHack()
   return false;
end

----------------------------------------------------------------

function StateLevelUpPopUp:goBrag()

	print("Go Back");

	if self.FadeActive then

		return false;
	end

	StateMgr:swap("state/state-brag.lua");

end

----------------------------------------------------------------

function StateLevelUpPopUp:goBack()

	print("Go Back");

	if self.FadeActive then

		return false;
	end

	StateMgr:pop();

end

----------------------------------------------------------------

return StateLevelUpPopUp;

----------------------------------------------------------------
