----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local SLIDE_MIN               = -App.viewportWidth/2;
local SLIDE_MAX               = App.viewportWidth/2;
local SLIDE_VOLUME_MIN_X      = -150;
local SLIDE_VOLUME_MAX_X      = 150;
local VOLUME_SLIDER_OFFSET_Y  = 30;
local SLIDE_VOLUME_MIN_Y      = 188+VOLUME_SLIDER_OFFSET_Y;
local SLIDE_VOLUME_MAX_Y      = 288+VOLUME_SLIDER_OFFSET_Y;
local CONTROL_SLIDER_OFFSET_Y = 18;
local SLIDE_CONTROL_MIN_X     = -150;
local SLIDE_CONTROL_MAX_X     = 150;
local SLIDE_CONTROL_MIN_Y     = 64+CONTROL_SLIDER_OFFSET_Y;
local SLIDE_CONTROL_MAX_Y     = 164+CONTROL_SLIDER_OFFSET_Y;
local SLIDE_SCALE             = SLIDE_VOLUME_MAX_X - SLIDE_VOLUME_MIN_X;
local LABEL_VOL_X             = 0;
local LABEL_VOL_Y             = 273+VOLUME_SLIDER_OFFSET_Y;
local LABEL_CONTROL_X         = 0;
local LABEL_CONTROL_Y         = 150+CONTROL_SLIDER_OFFSET_Y;
local CHECKBOX_X              = -134;
local LABEL_FEED_X            = 28;
local LABEL_FEED_Y            = 8;
local LABEL_WEEKLY_X          = 40;
local LABEL_WEEKLY_Y          = -75;
local LABEL_DAILY_X           = 23;
local LABEL_DAILY_Y           = -158;
local LABEL_KEY_X             = 14;
local LABEL_KEY_Y             = -241;

local ROADRIOT_VERSION_X      = 0;
local ROADRIOT_VERSION_Y      = -330;
local DEBUGTXT_X              = 0;
local DEBUGTXT_Y              = 25;
local PRIORTY_0               = 10;
local PRIORTY_1               = 20;
local PRIORTY_2               = 30;
local PRIORTY_3               = 40;
local PRIORTY_4               = 50;
local PRIORTY_5               = 60;
local PRIORTY_6               = 70;

local LEVEL_DEBUG_ENABLE		= true;
local LEAGUE_DEBUG_ENABLE     = false;

if not BUILD_DEBUG then
   LEVEL_DEBUG_ENABLE	= false;
   LEAGUE_DEBUG_ENABLE  = false;
end

----------------------------------------------------------------

local Base						= State;
local StateOptions			= Base:new();
local localApp					= nil;
local localSelf				= nil;
local textureSheet			= FlashMgr:getTextureSheet("resources/flash-resource-bundle/Group2.lua", "resources/flash-resource-bundle/Group2.png", 30);

----------------------------------------------------------------

function StateOptions:onLoad()

   State.onLoad( self );

   localSelf      = self;

   self.defaultFontName = "resources/font/TangoLatin.otf";

	-- background
	self.BackDrop = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "O_Background", textureSheet, self.volumeSlideOn, self.volumeSlideOff, nil, self.volumeSlideMove);
	self.Background = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "O_Background", textureSheet, self.controlSlideOn, self.controlSlideOff, nil, self.controlSlideMove);
	self.Title = self:addWidget(Anim, self.mainLayer, 0,430,0,0, "O_TitleOptions", textureSheet, nil, nil, true);
	self.Title:setPriority(PRIORTY_0);

	-- state buttons
	self.BackButton = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "O_ReadyButton", textureSheet, nil, self.goBack, nil, nil);
	self.BackButton:setPriority(PRIORTY_3);
	self.CreditsButton = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "O_SupportButton", textureSheet, nil, self.goSupport, nil, nil);
	self.CreditsButton:setPriority(PRIORTY_3);

	-- Text Labels
	self.LabelVol = self:addWidget(TextBox, self.mainLayer, LABEL_VOL_X, LABEL_VOL_Y, 458, 200, 'SOUND Volume', self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.LabelVol:setPriority(PRIORTY_2);
	self.LabelVolShadow = self:addWidget(TextBox, self.mainLayer, LABEL_VOL_X+3, LABEL_VOL_Y-3, 458, 200, 'SOUND Volume', self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.LabelVolShadow:setPriority(PRIORTY_2-1);
	self.LabelControl = self:addWidget(TextBox, self.mainLayer, LABEL_CONTROL_X, LABEL_CONTROL_Y, 458, 200, 'CONTROL Sensitivity', self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.LabelControl:setPriority(PRIORTY_2);
	self.LabelControlShadow = self:addWidget(TextBox, self.mainLayer, LABEL_CONTROL_X+3, LABEL_CONTROL_Y-3, 458, 200, 'CONTROL Sensitivity', self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.LabelControlShadow:setPriority(PRIORTY_2-1);

	-- Debug/TangoID text
	self.DebugText = self:addWidget(TextBox, self.mainLayer, DEBUGTXT_X, DEBUGTXT_Y, 640, 280, ' ', self.defaultFontName, 14, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.DebugText:setPriority(PRIORTY_2);
	self.DebugTextShadow = self:addWidget(TextBox, self.mainLayer, DEBUGTXT_X+2, DEBUGTXT_Y-2, 640, 280, '', self.defaultFontName, 14, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.DebugTextShadow:setPriority(PRIORTY_2-1);
	self.BossText = self:addWidget(TextBox, self.mainLayer, DEBUGTXT_X, DEBUGTXT_Y+100, 640, 280, '', self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.BossText:setPriority(PRIORTY_2);
	self.BossTextShadow = self:addWidget(TextBox, self.mainLayer, DEBUGTXT_X+2, DEBUGTXT_Y+100-2, 640, 280, '', self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.BossTextShadow:setPriority(PRIORTY_2-1);

   -- Version text
   self.VersionText = self:addWidget(TextBox, self.mainLayer, ROADRIOT_VERSION_X, ROADRIOT_VERSION_Y, 640, 280, '', self.defaultFontName, 14, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.VersionText:setPriority(PRIORTY_2);
   self.VersionTextShadow = self:addWidget(TextBox, self.mainLayer, ROADRIOT_VERSION_X+2, ROADRIOT_VERSION_Y-2, 640, 280, '', self.defaultFontName, 14, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.VersionTextShadow:setPriority(PRIORTY_2-1);

	-- controls
	self.MuteIcon = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,VOLUME_SLIDER_OFFSET_Y,0,0, "O_SoundOff", textureSheet, nil, self.MinVolume, nil, nil, nil, nil, 20);
	self.MuteIcon:setPriority(PRIORTY_3);
	self.SoundIcon = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,VOLUME_SLIDER_OFFSET_Y,0,0, "O_SoundIcon", textureSheet, nil, self.MaxVolume, nil, nil, nil, nil, 20);
	self.SoundIcon:setPriority(PRIORTY_3);
	self.VolumeSlider = self:addWidget(Anim, self.mainLayer, 0,VOLUME_SLIDER_OFFSET_Y,0,0, "O_SoundBar", textureSheet);
	self.VolumeSlider:setPriority(PRIORTY_3);
	self.VolumeButton = self:addWidget(Anim, self.mainLayer, 0,VOLUME_SLIDER_OFFSET_Y,0,0, "O_SoundBarHandle", textureSheet);
	self.VolumeButton:setPriority(PRIORTY_3);
	--self.MinSenseIcon = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,CONTROL_SLIDER_OFFSET_Y,0,0, "O_SensitivityDown", textureSheet, nil, self.MinSense, nil, nil, nil, nil, 0);
	--self.MinSenseIcon:setPriority(PRIORTY_3);
	--self.MaxSenseIcon = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,CONTROL_SLIDER_OFFSET_Y,0,0, "O_SensitivityUp", textureSheet, nil, self.MaxSense, nil, nil, nil, nil, 0);
	--self.MaxSenseIcon:setPriority(PRIORTY_3);
	self.ControlSlider = self:addWidget(Anim, self.mainLayer, 0,CONTROL_SLIDER_OFFSET_Y,0,0, "O_SensitivityBar", textureSheet);
	self.ControlSlider:setPriority(PRIORTY_3);
	self.ControlButton = self:addWidget(Anim, self.mainLayer, 0,CONTROL_SLIDER_OFFSET_Y,0,0, "O_SensitivityBarHandle", textureSheet);
	self.ControlButton:setPriority(PRIORTY_3);

   -- push notification text
   self.LabelWeeklyNot = self:addWidget(TextBox, self.mainLayer, LABEL_WEEKLY_X, LABEL_WEEKLY_Y, 650, 200, 'Weekly Reminder', self.defaultFontName, 15.554, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.LabelWeeklyNot:setPriority(PRIORTY_2);
	self.LabelWeeklyNotShadow = self:addWidget(TextBox, self.mainLayer, LABEL_WEEKLY_X+3, LABEL_WEEKLY_Y-3, 650, 200, 'Weekly Reminder', self.defaultFontName, 15.554, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.LabelWeeklyNotShadow:setPriority(PRIORTY_2-1);
   self.LabelWeeklyNot:setVisible(false);
   self.LabelWeeklyNotShadow:setVisible(false);

   self.LabelDailyNot = self:addWidget(TextBox, self.mainLayer, LABEL_DAILY_X, LABEL_DAILY_Y, 650, 200, 'Daily Reminder', self.defaultFontName, 15.554, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.LabelDailyNot:setPriority(PRIORTY_2);
	self.LabelDailyNotShadow = self:addWidget(TextBox, self.mainLayer, LABEL_DAILY_X+3, LABEL_DAILY_Y-3, 650, 200, 'Daily Reminder', self.defaultFontName, 15.554, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.LabelDailyNotShadow:setPriority(PRIORTY_2-1);
   self.LabelDailyNot:setVisible(false);
   self.LabelDailyNotShadow:setVisible(false);

   self.LabelKeyNot = self:addWidget(TextBox, self.mainLayer, LABEL_KEY_X, LABEL_KEY_Y, 650, 200, 'Key Reminder', self.defaultFontName, 15.554, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.LabelKeyNot:setPriority(PRIORTY_2);
	self.LabelKeyNotShadow = self:addWidget(TextBox, self.mainLayer, LABEL_KEY_X+3, LABEL_KEY_Y-3, 650, 200, 'Key Reminder', self.defaultFontName, 15.554, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.LabelKeyNotShadow:setPriority(PRIORTY_2-1);
   self.LabelKeyNot:setVisible(false);
   self.LabelKeyNotShadow:setVisible(false);

   self.LabelFeedNot = self:addWidget(TextBox, self.mainLayer, LABEL_FEED_X, LABEL_FEED_Y, 650, 200, 'Sharing Enabled', self.defaultFontName, 15.554, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.LabelFeedNot:setPriority(PRIORTY_2);
	self.LabelFeedNotShadow = self:addWidget(TextBox, self.mainLayer, LABEL_FEED_X+3, LABEL_FEED_Y-3, 650, 200, 'Sharing Enabled', self.defaultFontName, 15.554, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.LabelFeedNotShadow:setPriority(PRIORTY_2-1);
   self.LabelFeedNot:setVisible(false);
   self.LabelFeedNotShadow:setVisible(false);

   --checkboxes
	self.WeeklyBox = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, CHECKBOX_X,LABEL_WEEKLY_Y,0,0, "ChackBox", textureSheet, nil, self.WeekBoxPush, nil, nil, nil, nil, 20);
	self.WeeklyBox:setPriority(PRIORTY_3);
   self.WeeklyBox:setVisible(false);
   self.WeeklyBox:disable();

	self.DailyBox = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, CHECKBOX_X,LABEL_DAILY_Y,0,0, "ChackBox", textureSheet, nil, self.DayBoxPush, nil, nil, nil, nil, 20);
	self.DailyBox:setPriority(PRIORTY_3);
   self.DailyBox:setVisible(false);
   self.DailyBox:disable();

	self.KeyBox = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, CHECKBOX_X,LABEL_KEY_Y,0,0, "ChackBox", textureSheet, nil, self.KeyBoxPush, nil, nil, nil, nil, 20);
	self.KeyBox:setPriority(PRIORTY_3);
   self.KeyBox:setVisible(false);
   self.KeyBox:disable();

	self.FeedBox = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, CHECKBOX_X,LABEL_FEED_Y,0,0, "ChackBox", textureSheet, nil, self.FeedBoxPush, nil, nil, nil, nil, 20);
	self.FeedBox:setPriority(PRIORTY_3);
   self.FeedBox:setVisible(false);
   self.FeedBox:disable();


	self:_setupBasicResourceBar();

end

----------------------------------------------------------------

function StateOptions:onActivate()

   Base.onActivate(self);

   LoaderMgr:addTask( nil, self, function () StateMgr:loadState ( "state/state-support.lua" ) end );

   self.DebugText:setText(' ');
   self.DebugTextShadow:setText(' ');

   local wordSize = MOAIEnvironment.wordSize or 32;

   self.VersionText:setText(      '<c:4c8ebe>Version <c:4c8ebe>' .. App.info.version .. ' ' .. wordSize .. '-bit <c>');
   self.VersionTextShadow:setText('<c:000000>Version <c:000000>' .. App.info.version .. ' ' .. wordSize .. '-bit <c>');

   if self.resourceBar then
      self.resourceBar:createConsumableButton();
   end

end

----------------------------------------------------------------

function StateOptions:onFocus()

   print("StateOptions:onFocus");

   Base.onFocus(self);

	print("App.data:getGlobalVolume()",App.data:getGlobalVolume());

	self:getSlidePos();

   StateMgr:setSyncBusyDelay(0);

   self:setupFeedBox();
   if App.data:getValue( "notification_mgr_active" ) then
   	  self:setupNotificationBoxes();
   end

end

----------------------------------------------------------------

function StateOptions:onLoseFocus()

   print("StateOptions:onLoseFocus");

   Base.onLoseFocus(self);

   StateMgr:setSyncBusyDelay();

end

----------------------------------------------------------------

function StateOptions:onUpdate( dt )

   Base.onUpdate(self);

   if self.checkTimer then
      self.checkTimer = self.checkTimer - dt;
      print("CheckTimer:", self.checkTimer);
      if self.checkTimer <= 0 then
         self.checkTimer = nil;
         self:UpdateFeedBox();
         if App.data:getValue( "notification_mgr_active" ) then
            self:UpdateBoxes();
         end
      end
   end

	if BUILD_DEBUG and (LEVEL_DEBUG_ENABLE or LEAGUE_DEBUG_ENABLE) then
		self:updateLevelText();
	end

end

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

function StateOptions:debugAddLevel()

	local level = App.data:getCurrentPlayerLevel();

	if (level + 1) <= App.data:getMaxPlayerLevel() then

		App.data:setPlayerLevel( level + 1, 0 );
	end

end

----------------------------------------------------------------

function StateOptions:debugSubLevel()

	local level = App.data:getCurrentPlayerLevel();

	if (level - 1) > 0 then

		App.data:setPlayerLevel( level - 1, 0 );
	end

end

----------------------------------------------------------------

function StateOptions:debugPromoteLeague()

   local l,d = LeagueMgr:getPlayerPlacement();

   LeagueMgr:promote(l, d , 1, true);

end

----------------------------------------------------------------

function StateOptions:debugDemoteLeague()

   local l,d = LeagueMgr:getPlayerPlacement();

   LeagueMgr:demote(l, d , 20, true);

end

----------------------------------------------------------------

function StateOptions:updateLevelText()

	local level		= App.data:getCurrentPlayerLevel();
	local xp			= App.data:getCurrentPlayerXP();
	local text		= "Level: "..level.."    XP: "..xp;

   if LeagueMgr:isLeagueEnabled() then
      local l,d   = LeagueMgr:getPlayerPlacement();
      local n1,n2 = LeagueMgr:getLeagueName(l,d);

      text        = text.."\nTier: "..n1.."    Div: "..n2;

      if LEVEL_DEBUG_ENABLE then
         text     = text.."\nDEBUG MODE:  LEVEL";
      else
         text     = text.."\nDEBUG MODE:  LEAGUE";
      end
   end

	self.DebugText:setText('<c:6abffd>'..text..'<c>');
	self.DebugTextShadow:setText('<c:000000>'..text..'<c>');

end

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

function StateOptions:setupNotificationBoxes()

   --sets boxes visible and enabled.
   -- set them up according to which of the choices the player has made for their options.
   self.LabelWeeklyNot:setVisible(true);
   self.LabelWeeklyNotShadow:setVisible(true);
   self.LabelDailyNot:setVisible(true);
   self.LabelDailyNotShadow:setVisible(true);
   self.LabelKeyNot:setVisible(true);
   self.LabelKeyNotShadow:setVisible(true);

   self.WeeklyBox:setVisible(true);
   self.WeeklyBox:enable();

   self.DailyBox:setVisible(true);
   self.DailyBox:enable();

   self.KeyBox:setVisible(true);
   self.KeyBox:enable();

   self:UpdateBoxes();

end

------------------------------------------------------------------------------------

function StateOptions:setupFeedBox()

   local enable = FeedMgr:isEnable();

   self.FeedBox:setVisible(enable);
   self.FeedBox:setEnable(enable);
   self.LabelFeedNot:setVisible(enable);
   self.LabelFeedNotShadow:setVisible(enable);

   self:UpdateFeedBox();

end

------------------------------------------------------------------------------------

function StateOptions:WeekBoxPush()

   if not self.checkTimer then

      self.checkTimer = 0.10;

      if App.data:getValue( "weekly_notification_active" ) then
         App.data:setValue( "weekly_notification_active", false );
         print("Weekly Notifications OFF");
      else
         App.data:setValue( "weekly_notification_active", true );
         print("Weekly Notifications ON");
      end

      self:UpdateBoxes();

   end

end

----------------------------------------------------------------

function StateOptions:DayBoxPush()

   if not self.checkTimer then

      self.checkTimer = 0.10;

      if App.data:getValue( "daily_notification_active" ) then
         App.data:setValue( "daily_notification_active", false );
         print("Daily Notifications OFF");
      else
         App.data:setValue( "daily_notification_active", true );
         print("Daily Notifications ON");
      end

      self:UpdateBoxes();

   end

   if       LeagueMgr:isLeagueEnabled()
      and   BUILD_DEBUG
      and   ( LEVEL_DEBUG_ENABLE or LEAGUE_DEBUG_ENABLE ) then

      if LEVEL_DEBUG_ENABLE == false then
         LEVEL_DEBUG_ENABLE   = true;
         LEAGUE_DEBUG_ENABLE  = false;
      else
         LEVEL_DEBUG_ENABLE   = false;
         LEAGUE_DEBUG_ENABLE  = true;
      end
   end

end

----------------------------------------------------------------

function StateOptions:KeyBoxPush()

   if not self.checkTimer then

      self.checkTimer = 0.10;

      if App.data:getValue( "key_notification_active" ) then
         App.data:setValue( "key_notification_active", false );
         print("Key Notifications OFF");
      else
         App.data:setValue( "key_notification_active", true );
         print("Key Notifications ON");
      end

      self:UpdateBoxes();

   end

end
----------------------------------------------------------------

function StateOptions:FeedBoxPush()

   if not self.checkTimer then

      self.checkTimer = 0.10;

      if App.data:getValue( "feed_notification_active" ) then
         App.data:setValue( "feed_notification_active", false );
         print("Feed Notifications OFF");
      else
         App.data:setValue( "feed_notification_active", true );
         print("Feed Notifications ON");
      end

      self:UpdateFeedBox();

   end

end

----------------------------------------------------------------

function StateOptions:EnableFeed()

   --this is for the guide. if they already have the feed turned on, it does nothing.

   if not self.checkTimer then

      self.checkTimer = 0.10;

      if not App.data:getValue( "feed_notification_active" ) then
         App.data:setValue( "feed_notification_active", true );
         self:UpdateFeedBox();
         print("Feed Notifications ON");
      end

   end

end

----------------------------------------------------------------

function StateOptions:UpdateBoxes()

   if App.data:getValue( "weekly_notification_active" ) then
    	self.WeeklyBox:setAnimLastFrame( );
   else
      self.WeeklyBox:resetAnim();
   end

   if App.data:getValue( "daily_notification_active" ) then
       self.DailyBox:setAnimLastFrame( );
   else
      self.DailyBox:resetAnim();
   end

   if App.data:getValue( "key_notification_active" ) then
      self.KeyBox:setAnimLastFrame( );
   else
      self.KeyBox:resetAnim();
   end

end

----------------------------------------------------------------

function StateOptions:UpdateFeedBox()

   if FeedMgr:isEnable() then
      if App.data:getValue( "feed_notification_active" ) then
         self.FeedBox:setAnimLastFrame( );
      else
         self.FeedBox:resetAnim();
      end
   end

end

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------

function StateOptions:getSlidePos()

	local x,y = self.ControlButton:getLoc();

	x = (App.data:getControlSensitivity() * (SLIDE_CONTROL_MAX_X - SLIDE_CONTROL_MIN_X)) + SLIDE_CONTROL_MIN_X;
	self.ControlButton:setLoc(x, y);

   x,y = self.VolumeButton:getLoc();
   x = (App.data:getGlobalVolume() * (SLIDE_VOLUME_MAX_X - SLIDE_VOLUME_MIN_X)) + SLIDE_VOLUME_MIN_X;
   self.VolumeButton:setLoc(x, y);

end

----------------------------------------------------------------

function StateOptions:isInBoundary( x, y, xMin, xMax, yMin, yMax )

	if y <= yMax and y > yMin then
		if x <= xMax and x >= xMin then

			return true;
		end
	end

	return false;

end

----------------------------------------------------------------

function StateOptions:MinVolume()

	local target,Y = self.VolumeButton:getLoc();

	target = SLIDE_VOLUME_MIN_X;
	self.VolumeButton:setLoc(target, Y);

   AudioMgr:setGlobalVolume( 0.0 );
	App.data:setGlobalVolume( 0.0 );

   if LEVEL_DEBUG_ENABLE then
		self:debugSubLevel();
	end

   if LEAGUE_DEBUG_ENABLE then
		self:debugDemoteLeague();
	end

end

----------------------------------------------------------------

function StateOptions:MaxVolume()

	local target,Y = self.VolumeButton:getLoc();

	target = SLIDE_VOLUME_MAX_X;
	self.VolumeButton:setLoc(target, Y);

   AudioMgr:setGlobalVolume( 1.0 );
	App.data:setGlobalVolume( 1.0 );

   if LEVEL_DEBUG_ENABLE then
		self:debugAddLevel();
	end

   if LEAGUE_DEBUG_ENABLE then
		self:debugPromoteLeague();
	end

end

----------------------------------------------------------------

function StateOptions:updateVolSlidePos(x, y)

	local target,Y = self.VolumeButton:getLoc();
	local delta = x - SLIDE_VOLUME_MIN_X;

	target = x;
	target = math.clamp(target, SLIDE_VOLUME_MIN_X, SLIDE_VOLUME_MAX_X);
	self.VolumeButton:setLoc(target, Y);
	self.VolumeButton:setAnimFrame(BUTTON_PRESSED_FRAME);

	local scale = delta/SLIDE_SCALE;

	scale = math.clamp(scale, 0.0, 1.0);

   AudioMgr:setGlobalVolume( scale );
	App.data:setGlobalVolume( scale );

end

----------------------------------------------------------------

function StateOptions:volumeSlideOn(screen, x, y)

--		print("Swipe On", x, y);

	self.slideStartX = x;
	self.slideStartY = y;

	if self:isInBoundary(x, y, SLIDE_MIN, SLIDE_MAX, SLIDE_VOLUME_MIN_Y, SLIDE_VOLUME_MAX_Y) then

		self.VolumeButton:setAnimFrame(BUTTON_PRESSED_FRAME);

	end

end

----------------------------------------------------------------

function StateOptions:volumeSlideOff(screen, x, y)

	if self.volumeSlideOn then

--		print("Swipe Off");

		self.slideStartX = nil;
		self.slideStartY = nil;
	end

	self.VolumeButton:setAnimFrame();

end

----------------------------------------------------------------

function StateOptions:volumeSlideMove(parent, x, y)

	if self.slideStartX and self.slideStartY then

		if self:isInBoundary(x, y, SLIDE_MIN, SLIDE_MAX, SLIDE_VOLUME_MIN_Y, SLIDE_VOLUME_MAX_Y)
		and self:isInBoundary(self.slideStartX, self.slideStartY, SLIDE_MIN, SLIDE_MAX, SLIDE_VOLUME_MIN_Y, SLIDE_VOLUME_MAX_Y) then

--			print("Swipe Move", x, y);

			self:updateVolSlidePos(x, y);

		end
	end

end

----------------------------------------------------------------

function StateOptions:MinSense()

	local target,Y = self.ControlButton:getLoc();

	target = SLIDE_CONTROL_MIN_X;
	self.ControlButton:setLoc(target, Y);

   App.data:setControlSensitivity ( 0.0 );

end

----------------------------------------------------------------

function StateOptions:MaxSense()

	local target,Y = self.ControlButton:getLoc();

	target = SLIDE_CONTROL_MAX_X;
	self.ControlButton:setLoc(target, Y);

   App.data:setControlSensitivity ( 1.0 );

end

----------------------------------------------------------------

function StateOptions:updateControlSlidePos(x, y)

	local target,Y = self.ControlButton:getLoc();
	local delta = x - SLIDE_CONTROL_MIN_X;

	target = x;
	target = math.clamp(target, SLIDE_CONTROL_MIN_X, SLIDE_CONTROL_MAX_X);
	self.ControlButton:setLoc(target, Y);
	self.ControlButton:setAnimFrame(BUTTON_PRESSED_FRAME);

	local scale = delta/SLIDE_SCALE;

	scale = math.clamp(scale, 0.0, 1.0);

	App.data:setControlSensitivity ( scale );

end

----------------------------------------------------------------

function StateOptions:controlSlideOn(screen, x, y)

--		print("Swipe On", x, y);

	self.slideStartX = x;
	self.slideStartY = y;

	if self:isInBoundary(x, y, SLIDE_MIN, SLIDE_MAX, SLIDE_CONTROL_MIN_Y, SLIDE_CONTROL_MAX_Y) then

		self.ControlButton:setAnimFrame(BUTTON_PRESSED_FRAME);

	end

end

----------------------------------------------------------------

function StateOptions:controlSlideOff(screen, x, y)

--		print("Swipe Off");

	self.slideStartX = nil;
	self.slideStartY = nil;

	self.ControlButton:setAnimFrame();

end

----------------------------------------------------------------

function StateOptions:controlSlideMove(parent, x, y)

	if self.slideStartX and self.slideStartY then

		if self:isInBoundary(x, y, SLIDE_MIN, SLIDE_MAX, SLIDE_CONTROL_MIN_Y, SLIDE_CONTROL_MAX_Y)
		and self:isInBoundary(self.slideStartX, self.slideStartY, SLIDE_MIN, SLIDE_MAX, SLIDE_CONTROL_MIN_Y, SLIDE_CONTROL_MAX_Y) then

--			print("Swipe Move", x, y);

			self:updateControlSlidePos(x, y);

		end
	end

end

----------------------------------------------------------------

function StateOptions:goSupport()

	print("Go To Credits");
      StateMgr:push("state/state-support.lua");
--[[
   if LEVEL_DEBUG_ENABLE then
      StateMgr:push("state/state-cheat.lua");
   else
      StateMgr:push("state/state-credits.lua");
   end
   ]]--
end

----------------------------------------------------------------------

function StateOptions:isBusy()
   return StoreMgr:isBusy();
end

----------------------------------------------------------------

function StateOptions:goBack()

	print("Go Back");
	StateMgr:pop();

end

----------------------------------------------------------------

return StateOptions;

----------------------------------------------------------------
