----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local SQUARE_TEXTURE				= "resources/square.png";

local PRIORITY_0					= 10;
local PRIORITY_1					= 20;
local PRIORITY_2					= 30;
local PRIORITY_3					= 40;
local PRIORITY_4					= 50;
local PRIORITY_5					= 60;
local PRIORITY_6					= 70;
local PRIORITY_7					= 80;
local PRIORITY_8					= 90;
local PRIORITY_9					= 100;
local PRIORITY_10					= 110;

----------------------------------------------------------------

local DESC					=
{
	"Boosts speed and makes your vehicle invulnerable",
	"Attracts all other pickups to your vehicle",
	"Doubles the number of main guns on your vehicle",
	"Increases the point value of all enemies",
	"Repairs damage dealt to your vehicle",
	"Permanently doubles the value of all coins collected",
};

local STATS					=
{
	"- Upgrading increases pickup duration",
	"- Upgrading increases pickup duration",
	"- Upgrading increases pickup duration",
	"- Upgrading increases pickup duration",
	"- Upgrading increases amount repaired",
	"- Upgrading increases the value of coins collected",
};

local PICK_UP_ICON =
{
	'PickupBoost',
	'PickupMagnet',
	'PickupDoubleWeapon',
	'Pickup2x',
	'PickupWrench',
	'PickupCoin',
};

local PICK_UP_TITLE =
{
	'Boost',
	'Megnet',
	'DoubleShot',
	'ScoreMod',
	'Repair',
	'DoubleCoin',
};

----------------------------------------------------------------

local Base						= State;
local StatePickupStats		= Base:new();

----------------------------------------------------------------

function StatePickupStats:onLoad()

   State.onLoad( self );

	self.mainTexture		= FlashMgr:getTextureSheet("resources/flash-resource-bundle/MainScreen.lua", "resources/flash-resource-bundle/MainScreen.png", 30);
	self.popUpTexture		= FlashMgr:getTextureSheet("resources/flash-resource-bundle/PopUPs.lua", "resources/flash-resource-bundle/PopUPs.png", 30);
	self.group3Textue		= FlashMgr:getTextureSheet("resources/flash-resource-bundle/Group3.lua", "resources/flash-resource-bundle/Group3.png", 30);

   self.defaultFontName	= "resources/font/TangoLatin.otf";
	self.IS_POPUP			= true;

	-- background
	self.Background		= self:addWidget(Image, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE, {0,0,0, 0.75});
	self.Background:setPriority(0);

	-- frame
	self.Frame				= self:addWidget(Image, self.mainLayer, 0,0, 600, 380, "resources/UI/Drone_UI/infoframe.png");
	self.Frame:setPriority(PRIORITY_0-1);

	-- title
	self.NameText = {};

	for i=1,#PICK_UP_TITLE do
		self.NameText[i] = self:addWidget(Anim, self.mainLayer, 0, 160, 0,0, PICK_UP_TITLE[i], self.mainTexture, nil, nil, true);
		self.NameText[i]:setScale(0.9, 0.9);
		self.NameText[i]:setPriority(PRIORITY_3);
	end

	-- text message
	self.LevelText = self:addWidget(TextBox, self.mainLayer, 112, 70, 300, 100, "Drone Level", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.LEFT_JUSTIFY);
	self.LevelText:setScale(0.85, 0.85);
	self.LevelText:setPriority(PRIORITY_3);
	self.LevelTextShdw = self:addWidget(TextBox, self.mainLayer, 112+2, 70-2, 300, 100, "Drone Level", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.LEFT_JUSTIFY, {0,0,0,1});
	self.LevelTextShdw:setScale(0.85, 0.85);
	self.LevelTextShdw:setPriority(PRIORITY_3-1);
	self.DescText = self:addWidget(TextBox, self.mainLayer, 112, -75, 460, 400, "Drone Desc", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.LEFT_JUSTIFY);
	self.DescText:setPriority(PRIORITY_3);
	self.DescText:setScale(0.6, 0.6);
	self.DescText:setLineSpacing(3);
	self.DescTextShdw = self:addWidget(TextBox, self.mainLayer, 112+2, -75-2, 460, 400, "Drone Desc", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.LEFT_JUSTIFY, {0,0,0,1});
	self.DescTextShdw:setPriority(PRIORITY_3-1);
	self.DescTextShdw:setScale(0.6, 0.6);
	self.DescTextShdw:setLineSpacing(3);
	self.InfoText = self:addWidget(TextBox, self.mainLayer, 112, -165, 460, 400, "Drone Info", self.defaultFontName, 18, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.LEFT_JUSTIFY);
	self.InfoText:setPriority(PRIORITY_3);
	self.InfoText:setScale(0.6, 0.6);
	self.InfoText:setLineSpacing(3);
	self.InfoTextShdw = self:addWidget(TextBox, self.mainLayer, 112+2, -165-2, 460, 400, "Drone Info", self.defaultFontName, 18, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.LEFT_JUSTIFY, {0,0,0,1});
	self.InfoTextShdw:setPriority(PRIORITY_3-1);
	self.InfoTextShdw:setScale(0.6, 0.6);
	self.InfoTextShdw:setLineSpacing(3);

	-- drone pic
	self.PickupPic	= {};

	for i=1,#PICK_UP_ICON do
		self.PickupPic[i]	= self:addWidget(Anim, self.mainLayer, -145, -15, 0,0, PICK_UP_ICON[i], self.group3Textue);
		self.PickupPic[i]:setScale(1.25, 1.25);
		self.PickupPic[i]:setPriority(PRIORITY_2);
	end

	self.PickupPic[5]:setRot(-45);
	self.PickupPic[5]:setScale(1., -1);

	-- Exit Button
	self.BackButton	= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 285, 175, 0,0, "CloseButton", self.popUpTexture, nil, self.goBack, nil, nil, nil, nil, 40);
	self.BackButton:setPriority(PRIORITY_0);

end

----------------------------------------------------------------

function StatePickupStats:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StatePickupStats:onFocus( data )

   State.onFocus(self);

	Debug.assert(data, "StatePickupStats:onFocus -> data nil");

	local lvlTxt;
	local desc	= DESC[data.index];
	local stats	= STATS[data.index];

	for k,v in pairs(self.PickupPic) do
		if k == data.index then
			v:setVisible(true);
		else
			v:setVisible(false);
		end
	end

	for k,v in pairs(self.NameText) do
		if k == data.index then
			v:setVisible(true);
		else
			v:setVisible(false);
		end
	end

	if data.level >= App.data:getPickupMaxLevel(data.index) then
		lvlTxt = "Max Level";
	else
		lvlTxt = "Level "..data.level;
	end

	self.LevelText:setText(lvlTxt);
	self.LevelTextShdw:setText(lvlTxt);
	self.DescText:setText(desc);
	self.DescTextShdw:setText(desc);
	self.InfoText:setText(stats);
	self.InfoTextShdw:setText(stats);

end

----------------------------------------------------------------

function StatePickupStats:onUpdate( dt )

	Base.onUpdate(self, dt);

end


------------------------------------------------------------------------------------
-- Util
------------------------------------------------------------------------------------


function StatePickupStats:isUsingCameraHack()
   return false;
end


------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------


function StatePickupStats:goBack()
	StateMgr:pop();
end


------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------


return StatePickupStats;
