----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-scroll-list');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local PICKUP_HOME_X			= 0;
local PICKUP_HOME_Y			= 270;
local PICKUP_SPACING			= 167;

local SCROLL_WIND_MIN_X		= -App.viewportWidth/2;
local SCROLL_WIND_MAX_X		= App.viewportWidth/2;
local SCROLL_WIND_MIN_Y		= -352;
local SCROLL_WIND_MAX_Y		= 355;

local SWEET_SPOT_MIN_X		= -175;
local SWEET_SPOT_MAX_X		=  175;
local SWEET_SPOT_MIN_Y		= SCROLL_WIND_MIN_Y;
local SWEET_SPOT_MAX_Y		= SCROLL_WIND_MAX_Y;

local SWIPE_BUFFER_X			= 100;
local SWIPE_BUFFER_Y			=  50;

local COST_ICON_X				= 210;
local COST_ICON_Y				= 5;
local COST_X					= 230;
local COST_Y					= 5;

local PRIORTY_0				= 10;
local PRIORTY_1				= 20;
local PRIORTY_2				= 30;
local PRIORTY_3				= 40;
local PRIORTY_4				= 50;
local PRIORTY_5				= 60;
local PRIORTY_6				= 70;
local PRIORTY_7				= 80;
local PRIORTY_8				= 90;
local PRIORTY_9				= 100;
local PRIORTY_10				= 110;

local PICK_UP_ICON =
{
	'PickupBoost',
	'PickupMagnet',
	'PickupDoubleWeapon',
	'Pickup2x',
	'PickupWrench',
	'PickupCoin',
};

local PICK_UP_TITLE =
{
	'Boost',
	'Megnet',
	'DoubleShot',
	'ScoreMod',
	'Repair',
	'DoubleCoin',
};

local PICK_UP_TITLE_X =
{
	-116,
	-100,
	-61,
	-75,
	-109,
	-61,
};

local INFO_BUTTON_X =
{
	-35,
	-5,
	75,
	50,
	-25,
	75,
};

local DESC =
{
	'',
	'',
	'',
	'',
	'',
	'',		
};

----------------------------------------------------------------

local Base						= State;
local StatePickUp				= Base:new();

----------------------------------------------------------------

function StatePickUp:onLoad()

   Base.onLoad( self );

   self.defaultFontName = "resources/font/TangoLatin.otf";
	self.mainTexture		= FlashMgr:getTextureSheet("resources/flash-resource-bundle/MainScreen.lua", "resources/flash-resource-bundle/MainScreen.png", 30);
	self.group3Textue		= FlashMgr:getTextureSheet("resources/flash-resource-bundle/Group3.lua", "resources/flash-resource-bundle/Group3.png", 30);

	-- background, frame, and title
	self.BackDrop			= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "MainFrameBackground", self.mainTexture);
	self.Frame				= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "PU_ScrollFrame", self.mainTexture);
	self.Frame:setPriority(PRIORTY_5);

   if StateMgr.headerAvailable then
      self.Title			= self:addWidget(Anim, self.mainLayer, 0, 430, 0,0, "TitleUpGrade", self.mainTexture, nil, nil, true);
      self.Title:setPriority(PRIORTY_6);
   end

	-- resource bar
   self:_setupResourceBar();

	-- push/pop state buttons
	self.BackButton		= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "PU_DoneButton", self.mainTexture, nil, self.goBack, nil, nil, nil, nil, 40);
	self.BackButton:setPriority(PRIORTY_6);
	
	-- scroll list
	self.PickUpCount		= 6;
	self.ScrollList		= self:addWidget(		ScrollList,
															self.mainLayer,
															PICKUP_HOME_X, PICKUP_HOME_Y,
															{ {SCROLL_WIND_MIN_X, SCROLL_WIND_MAX_X},	{SCROLL_WIND_MIN_Y, SCROLL_WIND_MAX_Y} },
															nil,
															nil,
															{ 0, (1/4)*PICKUP_SPACING },
															{ nil, PICKUP_SPACING },
															{ 0, self.PickUpCount },
															nil,
															{ SWIPE_BUFFER_X, SWIPE_BUFFER_Y },
															nil,
															nil,
															{ {SWEET_SPOT_MIN_X, SWEET_SPOT_MAX_X},	{SWEET_SPOT_MIN_Y, SWEET_SPOT_MAX_Y} },
															nil, nil, nil, nil, nil, nil,
															"ScrollBarButton", self.mainTexture, 12, 78, 135, PRIORTY_4 );
														
	-- pickup Entries
	self.EntryList			= {};
	self:createEntryList();

end

----------------------------------------------------------------

function StatePickUp:onActivate()

	Base.onActivate(self);
	
	self.ScrollList:setLoc(PICKUP_HOME_X, PICKUP_HOME_Y);
	self:updateEntryList();

   if self.resourceBar then
      self.resourceBar:createConsumableButton();
   end

end

----------------------------------------------------------------

function StatePickUp:onFocus()

   Base.onFocus( self );

	StateMgr:setSyncBusyDelay(0);
	self:updateEntryList();

end

----------------------------------------------------------------

function StatePickUp:onLoseFocus( )

   Base.onLoseFocus(self);
	
	StateMgr:setSyncBusyDelay(nil);
   
end

----------------------------------------------------------------

function StatePickUp:onUpdate( dt )

	Base.onUpdate(self, dt);

end

----------------------------------------------------------------------

function StatePickUp:isBusy()

   return StoreMgr:isBusy();

end

----------------------------------------------------------------
----------------------------------------------------------------

function StatePickUp:_setupResourceBar()

   self.resourceBar = ResourceBar:new( self, 0, self:_calcHeaderLocY(), self.mainLayer, self.mainPartition  );

   self.resourceBar:setGoldCallback(
         nil
      ,
         function()
				StateMgr:push("state/state-coin.lua");
         end
   );

   self.resourceBar:setGemCallback(
         nil
      ,
         function()
				StateMgr:push("state/state-gem.lua");
         end
   );

   self.resourceBar:setKeyCallback(
         nil
      ,
         function()
				StateMgr:push("state/state-key.lua");
         end
   );

end

----------------------------------------------------------------
----------------------------------------------------------------

function StatePickUp:createEntryList()

	local count				= 1;
	local pickUpCount		= 	self.PickUpCount + count - 1;
	local n					= 1;

	for i=count,pickUpCount do
		table.insert(self.EntryList, self:_createPickUpEntry(i, n));
		n = n + 1;
	end
	
	self:updateEntryList();

end

----------------------------------------------------------------

function StatePickUp:_createPickUpEntry( index, pickUpIndex )

	local tempTable		= {};
	local offset			= self.ScrollList:getEntryOffsetY(index);
	
	tempTable.pickIndex	= pickUpIndex;
	tempTable.plaque		= self:addWidget(Anim, self.mainLayer, 0, offset, 0,0, "PU_Plaque", self.mainTexture);
	tempTable.portrait	= self:addWidget(Anim, self.mainLayer, -245, offset, 0,0, PICK_UP_ICON[pickUpIndex], self.group3Textue);
	tempTable.title		= self:addWidget(Anim, self.mainLayer, PICK_UP_TITLE_X[index], offset+38, 0,0, PICK_UP_TITLE[index], self.mainTexture, nil, nil, true);
	tempTable.levelPips	= self:_buildLevelBar( index, offset );
	tempTable.infoButton	= self:addWidget(ButtonImage, self.mainLayer, INFO_BUTTON_X[index], offset+38, 40, 40, "resources/UI/info_button.png", nil, nil, nil, nil, true);
	tempTable.desc			= self:addWidget(TextBox, self.mainLayer, 21, offset-43, 382, 48, DESC[pickUpIndex], self.defaultFontName, 12, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	tempTable.costIcon	= self:addWidget(Anim, self.mainLayer, COST_ICON_X, COST_ICON_Y+offset, 0,0, "BuyButtonCoin", self.mainTexture, nil, nil, true);
	tempTable.costText	= self:addWidget(TextBox, self.mainLayer, COST_X, COST_Y+offset, 400, 96, 'COST', self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	tempTable.costShdw	= self:addWidget(TextBox, self.mainLayer, COST_ICON_X+2, COST_Y+offset-2, 400, 96, 'COST', self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	tempTable.buyButton	= self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 210, offset, 0,0, "UpGradeButton", self.mainTexture,	nil,
																																																function()
																																																	if not self.ScrollList:isEntryInView(index, 210, 0) then
																																																		return false;
																																																	end
																																																		
																																																	self:buyPickUp(pickUpIndex);
																																																end,
																																																nil, nil, nil, nil, 20);
																																																

	if pickUpIndex == 5 then
		tempTable.portrait:setRot(45, 0, 0);
		tempTable.portrait:setScale(0.75, 0.75);
	end
	
	tempTable.plaque:setPriority(PRIORTY_0);
	tempTable.portrait:setPriority(PRIORTY_1);
	tempTable.title:setPriority(PRIORTY_1+1);
	tempTable.infoButton:setPriority(PRIORTY_2);
	tempTable.desc:setPriority(PRIORTY_1+1);
	tempTable.costIcon:setPriority(PRIORTY_2+1);
	tempTable.costText:setPriority(PRIORTY_2+2);
	tempTable.costShdw:setPriority(PRIORTY_2+1);
	tempTable.buyButton:setPriority(PRIORTY_2);
	
	tempTable.plaque:setParent(self.ScrollList.root);
	tempTable.portrait:setParent(self.ScrollList.root);
	tempTable.title:setParent(self.ScrollList.root);
	tempTable.infoButton:setParent(self.ScrollList.root);
	tempTable.desc:setParent(self.ScrollList.root);
	tempTable.costIcon:setParent(self.ScrollList.root);
	tempTable.costText:setParent(self.ScrollList.root);
	tempTable.costShdw:setParent(self.ScrollList.root);
	tempTable.buyButton:setParent(self.ScrollList.root);

	tempTable.infoButton:setInputXtra(64, 64);
	tempTable.infoButton:setInViewTest(	function()
														local x, y = tempTable.infoButton.prop:modelToWorld();

														return y >= SCROLL_WIND_MIN_Y and y <= SCROLL_WIND_MAX_Y and not self.ScrollList:isScrollListBusy();
													end
												);

	tempTable.infoButton.onReleasedFunc	=	function()
															StateMgr:push( "state/state-pickup-info-popup.lua", {	index= pickUpIndex, level= App.data:getPickupLevel(pickUpIndex)	} );
														end

	return tempTable;

end

----------------------------------------------------------------

function StatePickUp:updateEntryList()

	for i=1,#self.EntryList do
		
		local y = self.ScrollList:getEntryOffsetY(i);
		
		if self.EntryList[i].pickIndex and
			App.data:getPickupLevel(self.EntryList[i].pickIndex) < App.data:getPickupMaxLevel(self.EntryList[i].pickIndex) then
			
			local cost		= App.data:getPickupUpgradeCost(self.EntryList[i].pickIndex);
			local price		= 0;
		
			if cost.gold and cost.gem then
				if cost.gold > 0 then
					price = cost.gold;
					self.EntryList[i].costText:setText(''..cost.gold);
					self.EntryList[i].costShdw:setText('<c:000000>'..cost.gold);
					self.EntryList[i].costIcon:swapGraphic('BuyButtonCoin');
					self.EntryList[i].costIcon:setScale(1.0, 1.0);
				elseif cost.gem > 0 then
					price = cost.gem;
					self.EntryList[i].costText:setText(''..cost.gem);
					self.EntryList[i].costShdw:setText('<c:000000>'..cost.gem);
					self.EntryList[i].costIcon:swapGraphic('BuyButtonGem');
					self.EntryList[i].costIcon:setScale(0.6, 0.6);
				else
					self.EntryList[i].costIcon:swapGraphic('BuyButtonCoin');
					self.EntryList[i].costIcon:setScale(1.0, 1.0);
					self.EntryList[i].costText:setText('0');
					self.EntryList[i].costShdw:setText('<c:000000>0');
				end
			elseif cost.gold then
				if cost.gold > 0 then
					price = cost.gold;
					self.EntryList[i].costText:setText(''..cost.gold);
					self.EntryList[i].costShdw:setText('<c:000000>'..cost.gold);
					self.EntryList[i].costIcon:swapGraphic('BuyButtonCoin');
					self.EntryList[i].costIcon:setScale(1.0, 1.0);
				else
					self.EntryList[i].costText:setText('0');
					self.EntryList[i].costShdw:setText('<c:000000>0');
					self.EntryList[i].costIcon:swapGraphic('BuyButtonCoin');
					self.EntryList[i].costIcon:setScale(1.0, 1.0);
				end
			elseif cost.gem then
				if cost.gem > 0 then
					price = cost.gem;
					self.EntryList[i].costText:setText(''..cost.gem);
					self.EntryList[i].costShdw:setText('<c:000000>'..cost.gem);
					self.EntryList[i].costIcon:swapGraphic('BuyButtonGem');
					self.EntryList[i].costIcon:setScale(0.6, 0.6);
				else
					self.EntryList[i].costText:setText('0');
					self.EntryList[i].costShdw:setText('<c:000000>0');
					self.EntryList[i].costIcon:swapGraphic('BuyButtonGem');
					self.EntryList[i].costIcon:setScale(0.6, 0.6);
				end
			elseif cost.dollar then
				self.EntryList[i].costText:setText('$'..cost.dollar);
				self.EntryList[i].costShdw:setText('<c:000000>$'..cost.dollar);
			end
		
			local offset = 20;
			
			self.EntryList[i].costText:setGlyphScale(1.1);
			self.EntryList[i].costShdw:setGlyphScale(1.1);

			if price > 99999 then
				offset = 68;
				self.EntryList[i].costText:setGlyphScale(0.9);
				self.EntryList[i].costShdw:setGlyphScale(0.9);
			elseif price > 9999 then
				offset = 60;
				self.EntryList[i].costText:setGlyphScale(1.0);
				self.EntryList[i].costShdw:setGlyphScale(1.0);
			elseif price > 999 then
				offset = 50;
				self.EntryList[i].costText:setGlyphScale(1.1);
				self.EntryList[i].costShdw:setGlyphScale(1.1);
			elseif price > 99 then
				offset = 40;
				self.EntryList[i].costText:setGlyphScale(1.1);
				self.EntryList[i].costShdw:setGlyphScale(1.1);
			elseif price > 9 then
				offset = 30;
				self.EntryList[i].costText:setGlyphScale(1.1);
				self.EntryList[i].costShdw:setGlyphScale(1.1);
			end

			if cost.dollar then
				self.EntryList[i].costIcon:setVisible(false);
				self.EntryList[i].costText:setLoc(COST_ICON_X, COST_Y+y);
				self.EntryList[i].costShdw:setLoc(COST_ICON_X+2, COST_Y+y-2);
				self.EntryList[i].costText:setGlyphScale(1.1);
				self.EntryList[i].costShdw:setGlyphScale(1.1);
			else
				self.EntryList[i].costIcon:setLoc(COST_ICON_X-offset, COST_ICON_Y+y);	
				self.EntryList[i].costIcon:setPriority(PRIORTY_2+1);
				self.EntryList[i].costIcon:setParent(self.ScrollList.root);
				self.EntryList[i].costIcon:setVisible(true);
				self.EntryList[i].costText:setLoc(COST_X, COST_Y+y);
				self.EntryList[i].costShdw:setLoc(COST_X+2, COST_Y+y-2);
			end

			-- buy button
			local button	= 'UpGradeButton';
			
			if cost.gold and cost.gem then
				if cost.gold > 0 then
					if cost.gold > App.data:getGoldCount() then
						button = 'UpGradeLockedButton';
					end
				elseif cost.gem > 0 then
					if cost.gem > App.data:getGemCount() then
						button = 'UpGradeLockedButton';
					end
				else
					button = 'UpGradeButton';
				end
			elseif cost.gold then
				if cost.gold > App.data:getGoldCount() then
					button = 'UpGradeLockedButton';
				else
					button = 'UpGradeButton';
				end
			elseif cost.gem then
				if cost.gem > App.data:getGemCount() then
					button = 'UpGradeLockedButton';
				else
					button = 'UpGradeButton';
				end
			elseif cost.dollar then
				button = 'UpGradeLockedButton';
			end

			self.EntryList[i].buyButton:swapAnim(button);
			self.EntryList[i].buyButton:setLoc(210, y);
			self.EntryList[i].buyButton:setPriority(PRIORTY_2);
			self.EntryList[i].buyButton:setParent(self.ScrollList.root);
			self.EntryList[i].buyButton:enable();
			self.EntryList[i].buyButton:setVisible(true);
			self.EntryList[i].buyButton:setColor(1.0, 1.0, 1.0, 1.0);
		
		elseif self.EntryList[i].pickIndex then
			
			self.EntryList[i].costIcon:setVisible(false);
			self.EntryList[i].costText:setLoc(210, y);
			self.EntryList[i].costText:setGlyphScale(1.1);
			self.EntryList[i].costText:setText('Maxed');
			self.EntryList[i].costShdw:setLoc(210+2, y-2);
			self.EntryList[i].costShdw:setGlyphScale(1.1);
			self.EntryList[i].costShdw:setText('<c:000000>Maxed<c>');
			self.EntryList[i].buyButton:disable();
			self.EntryList[i].buyButton:setVisible(false);
		
		end
	
	end

	self:updateLevelBar();

end

----------------------------------------------------------------
----------------------------------------------------------------

function StatePickUp:_buildLevelBar( index, y )

	local tempTable	= {};
	local pipWidth		= 16;
	local maxLevel		= App.data:getPickupMaxLevel(index);	
	
	for i=1,maxLevel do
	
		local x = -168 + (pipWidth/2) + (pipWidth*(i - 1));

		tempTable[i] = self:addWidget(Anim, self.mainLayer, x, y-25, 0,0, "PickupPips", self.mainTexture);
		tempTable[i]:setPriority(PRIORTY_1+1);
		tempTable[i]:setParent(self.ScrollList.root);
	
	end

	return tempTable;

end

----------------------------------------------------------------

function StatePickUp:updateLevelBar()

	for i=1,#self.EntryList do
	
		if self.EntryList[i].levelPips then
	
			local itemLevel	= App.data:getPickupLevel(i);
			
			for j=1,#self.EntryList[i].levelPips do
			
				if itemLevel > 0 then
					self.EntryList[i].levelPips[j]:setSpan(5/30, 5/30);
					self.EntryList[i].levelPips[j]:start();
					
					itemLevel = itemLevel - 1;
				else
					self.EntryList[i].levelPips[j]:setSpan(0, 0);
					self.EntryList[i].levelPips[j]:start();
				end
			end
		end
	end

end

----------------------------------------------------------------
----------------------------------------------------------------

function StatePickUp:buyPickUp( index )

	local cost = App.data:getPickupUpgradeCost(index);

   Debug.assert( cost.dollar == nil, "StatePickUp:buyPickUp -> cost.dollar == nil" );

   if not App.data:canAffordUpgradePickup(index) then
		
      AUTO_PURCHASE_DATA.state		= 'StatePickUp';
      AUTO_PURCHASE_DATA.carIndex	= ""..-1;
      AUTO_PURCHASE_DATA.reason1		= 'Upgrade-Pickup';
      AUTO_PURCHASE_DATA.reason2		= 'Upgrade-Pickup-'..index;
		AUTO_PURCHASE_DATA.price		= cost.gold or cost.gem;
		
		if cost.gold then
			AUTO_PURCHASE_DATA.currency	= 'Coin';
		else
			AUTO_PURCHASE_DATA.currency	= 'Gem';
		end

		AUTO_PURCHASE_FUNC				=	function()
														self:buyPickUp(index);
													end

		StateMgr:push("state/state-auto-purchase-popup-maz.lua");

		return false;
	end
	
	App.data:upgradePickup(index);
	self:updateEntryList();

end

----------------------------------------------------------------

function StatePickUp:showErrorPopup( message )
	
	local text = message or LocalizationMgr:getString( "NATIVE_TRY_AGAIN" );

   MOAIDialogIOS.showDialog(	LocalizationMgr:getString( "NATIVE_TRANS_FAIL" )
                              ,	text
                              ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                              ,  nil
                              ,  nil
                              ,  false
                              ,  function()
                                 end
                           );
																					
end

----------------------------------------------------------------
----------------------------------------------------------------

function StatePickUp:goBack()
	
	StateMgr:pop();

end

----------------------------------------------------------------

return StatePickUp;

----------------------------------------------------------------
