----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local SQUARE_TEXTURE = "resources/square.png";

local LOADING_BAR_BACK_WIDTH   = 290;
local LOADING_BAR_BACK_HEIGHT  = 36;
local LOADING_BAR_FRONT_WIDTH  = 284;
local LOADING_BAR_FRONT_HEIGHT = 30;

local PULSE_TIME  = 0.5;
local PULSE_SCALE = 1.1;

local PRIORITY_0  = 10;
local PRIORITY_1  = 20;
local PRIORITY_2  = 30;
local PRIORITY_3  = 40;
local PRIORITY_4  = 50;
local PRIORITY_5  = 60;
local PRIORITY_6  = 70;
local PRIORITY_7  = 80;
local PRIORITY_8  = 90;
local PRIORITY_9  = 100;
local PRIORITY_10 = 110;

----------------------------------------------------------------

local Base              = State;
local StatePromoCarSale = Base:new();

----------------------------------------------------------------

function StatePromoCarSale:onLoad()

   State.onLoad( self );

   local textureSheet	= FlashMgr:getTextureSheet("resources/flash-resource-bundle/PopUPs.lua", "resources/flash-resource-bundle/PopUPs.png", 30);

   self.defaultFontName	= "resources/font/TangoLatin.otf";
   self.IS_POPUP     = true;

   -- background
   self.Background   = self:addWidget(Image, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE);
   self.Background:setPriority(PRIORITY_0);
   self.Background:setColor(0,0,0, 0.75);

   -- frame
   self.Frame        = self:addWidget(Image, self.mainLayer, 0, 0, 576*1.1, 718*1.1, "resources/UI/Promo/Promo_CarSale_50_25.png");
   self.Frame:setPriority(PRIORITY_1);

   -- buttons
   self.GarageButton = self:addWidget(ButtonImage, self.mainLayer, 0, -375, 253*0.8, 131*0.8, "resources/UI/Promo/Promo_Drone_Garage_Button.png", nil, self.goGarage);
   self.GarageButton:setPriority(PRIORITY_2);

--[[
	-- Exit Button
	self.ExitButton   = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 265, 360, 0,0, "CloseButton", textureSheet, nil, self.goBack, nil, nil, nil, nil, 40);
	self.ExitButton:setPriority(PRIORITY_2);
--]]

end

----------------------------------------------------------------

function StatePromoCarSale:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StatePromoCarSale:onFocus( data )

   State.onFocus(self);

   self._pulseTimer = 0;

   StateMgr:setSyncBusyDelay(0);

   local saleTbl = self:getCarSaleDiscount();

   if saleTbl == nil then
      StateMgr:pop();

      return;
   end

   if saleTbl[1] == "60%" then
      self.Frame:swapImage("resources/UI/Promo/Promo_CarSale_60_30.png");
   elseif saleTbl[1] == "75%" then
      self.Frame:swapImage("resources/UI/Promo/Promo_CarSale_75_50.png");
   end

end

----------------------------------------------------------------

function StatePromoCarSale:onLoseFocus( )

   Base.onLoseFocus(self);

   StateMgr:setSyncBusyDelay(nil);

end

----------------------------------------------------------------

function StatePromoCarSale:onUpdate( dt )

   Base.onUpdate(self, dt);

   self:_updatePulse(dt);

end


------------------------------------------------------------------------------------
-- Updates
------------------------------------------------------------------------------------


function StatePromoCarSale:_updatePulse( dt )

   self._pulseTimer = self._pulseTimer + dt;

   if self._pulseTimer > PULSE_TIME then
      self._pulseTimer = 0;
   end

   local k = 1;

   if self.GarageButton:isPressed() then
      k = PULSE_SCALE;
      self._pulseTimer = PULSE_TIME/2;
   elseif self._pulseTimer < PULSE_TIME/2 then
      k = math.mapClamped(0, 1, PULSE_TIME/2, PULSE_SCALE, self._pulseTimer);
   else
      k = math.mapClamped(PULSE_TIME/2, PULSE_SCALE, PULSE_TIME, 1, self._pulseTimer);
   end

   self.GarageButton:setScale(k, k);

end


------------------------------------------------------------------------------------
-- Util
------------------------------------------------------------------------------------


function StatePromoCarSale:isUsingCameraHack()
   return false;
end

----------------------------------------------------------------

function StatePromoCarSale:isBusy()
   return StoreMgr:isBusy();
end

----------------------------------------------------------------

function StatePromoCarSale:getCarSaleDiscount()

   local baseCostTbl = App.data:getValue("App.settings.player-vehicle-cost");
   local magCar      = App.data:getVehicleUnlockCostTable(2);
   local misCarV2    = App.data:getVehicleUnlockCostTable(8);

   Debug.assert( baseCostTbl, "StatePromoCarSale:getCarSaleDiscount -> baseCostTbl");
   Debug.assert( baseCostTbl[2], "StatePromoCarSale:getCarSaleDiscount -> baseCostTbl[2]");
   Debug.assert( baseCostTbl[2].car, "StatePromoCarSale:getCarSaleDiscount -> baseCostTbl[2].car");
   Debug.assert( baseCostTbl[8], "StatePromoCarSale:getCarSaleDiscount -> baseCostTbl[8]");
   Debug.assert( baseCostTbl[8].car, "StatePromoCarSale:getCarSaleDiscount -> baseCostTbl[8].car");
   Debug.assert( magCar, "StatePromoCarSale:getCarSaleDiscount -> magCar");
   Debug.assert( magCar.car, "StatePromoCarSale:getCarSaleDiscount -> magCar.car");
   Debug.assert( misCarV2, "StatePromoCarSale:getCarSaleDiscount -> misCarV2");
   Debug.assert( misCarV2.car, "StatePromoCarSale:getCarSaleDiscount -> misCarV2.car");

   local salePrcntV1 = magCar.car/baseCostTbl[2].car;
   local salePrcntV2 = misCarV2.car/baseCostTbl[8].car;

   if salePrcntV1 == 1 or salePrcntV2 == 1 then
      return nil;
   end

   local v1 = tostring( (1 - salePrcntV1) * 100 ).."%";
   local v2 = tostring( (1 - salePrcntV2) * 100 ).."%";

   return {v1, v2};

end


------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------


function StatePromoCarSale:goGarage()
   StateMgr:swap("state/state-garage.lua");
end

----------------------------------------------------------------

function StatePromoCarSale:goBack()
   StateMgr:pop();
end


------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------

return StatePromoCarSale;
