----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local PRIORITY_0				= 10;
local PRIORITY_1				= 20;
local PRIORITY_2				= 30;
local PRIORITY_3				= 40;
local PRIORITY_4				= 50;
local PRIORITY_5				= 60;
local PRIORITY_6				= 70;
local PRIORITY_7				= 80;
local PRIORITY_8				= 90;
local PRIORITY_9				= 100;
local PRIORITY_10				= 110;

local shadowXOff                = 2;
local shadowYOff                = -2;

----------------------------------------------------------------

local Base							= State;
local StatePromoDoubleCoin		= Base:new();

----------------------------------------------------------------

function StatePromoDoubleCoin:onLoad()

   State.onLoad( self );

	self.IS_POPUP  = true;
   self.defaultFontName = "resources/font/TangoLatin.otf";

	-- black overlay
   self.Overlay = self:addWidget(Image, self.mainLayer, 0, 0, App.viewportWidth, App.viewportHeight, "resources/UI/blackbox.png", {1,1,1, 0.8});

end

----------------------------------------------------------------

function StatePromoDoubleCoin:loadText( text )

    local fontSizes = {};

    if text then
        for i=1,#text do
            if #fontSizes > 0 then
                for j,k in ipairs( fontSizes ) do
                    if text[i].size ~= k then
                        table.insert ( fontSizes, text[i].size );
                    end
                end
            else
                table.insert ( fontSizes, text[i].size );
            end
        end
    end

    for o,p in ipairs(text) do

        local x,y = self:convertCoords(p.x, p.y, p.w, p.h);
        if x and y and p.w and p.h then
    
            local boxShadow = self:addWidget(TextBox, self.mainLayer, x+shadowXOff, y+shadowYOff, p.w, p.h, p.text, self.defaultFontName, p.size * 0.4444, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
            if p.rotation then
                boxShadow:setRot( -p.rotation );
            end
            boxShadow:setColor(0, 0, 0, 1);

            local box = self:addWidget(TextBox, self.mainLayer, x, y, p.w, p.h, p.text, self.defaultFontName, p.size * 0.4444, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
            if p.rotation then
                box:setRot( -p.rotation );
            end
            
            if p.color then
                box:setColor(p.color.r, p.color.g, p.color.b, p.color.a);
            end
            if self.textList then
               table.insert(self.textList, boxShadow);
               table.insert(self.textList, box);
            end
        end
    end

end

----------------------------------------------------------------

function StatePromoDoubleCoin:loadButtons( buttons )

   for i,v in ipairs( buttons ) do

      self[v.name] = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, v.name, self.bTextureSheet, nil, self[v.func], nil, nil );

      if v.play then
         self[v.name]:setSpan(29/30);
         self[v.name]:setMode(MOAITimer.LOOP);
         self[v.name]:start();
      end
      if self.buttonList then
         table.insert(self.buttonList, self[v.name]);
      end
   end

end

----------------------------------------------------------------

function StatePromoDoubleCoin:loadAnims( anims )

   if anims then

       for i,v in ipairs( anims ) do
           
         self[v.name] = self:addWidget(Anim, self.mainLayer, v.x, v.y, 0,0, v.name, self.aTextureSheet);
         if v.scale then
            self[v.name]:setScale( v.scale );
         end
         if v.priority then
            self[v.name]:setPriority( v.priority );
         end
         if v.rotation then
            self[v.name]:setRot( v.rotation );
         end
         if self.animList then
            table.insert(self.animList, self[v.name]);
         end
       end
   end

end

----------------------------------------------------------------

function StatePromoDoubleCoin:removeText()

   if self.textList then

      for i,v in ipairs(self.textList) do
         self:deleteWidget(v);
         v=nil;
      end

   end
end

----------------------------------------------------------------

function StatePromoDoubleCoin:removeButtons()

   if self.buttonList then

      for i,v in ipairs(self.buttonList) do
        self:deleteWidget(v);
        v=nil;
      end

   end
end

----------------------------------------------------------------

function StatePromoDoubleCoin:removeAnims()

   if self.animList then

      for i,v in ipairs(self.animList) do
        self:deleteWidget(v);
        v=nil;
      end

   end
end

----------------------------------------------------------------

function StatePromoDoubleCoin:convertCoords( x, y, w, h )
    
    local sW = App.viewportWidth;
    local sH = App.viewportHeight;
    if sW and sH and x and y and w and h then
        local bX = x;
        local bY = -y;
        
        local rX = bX + (w/2);
        local rY = bY - (h/2);
--print("promo textbox x,y:", rX, rY);
        return rX, rY;
    end

end

----------------------------------------------------------------

function StatePromoDoubleCoin:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StatePromoDoubleCoin:onFocus( masterTable, infoTable )
   
   State.onFocus(self);

   self.realFailure = false;

   StateMgr:setSyncBusyDelay(0);
   
   if self.goback then
      self.goback = nil;
      print("self.goback is true");
      self:goBack();
   end
   


   if masterTable and infoTable then

      self.buttons = infoTable.buttons;
      self.buttonLua = infoTable.assets.buttonLua;
      self.buttonImage = infoTable.assets.buttonImage;
      self.buttonFps = infoTable.assets.buttonFps;
      
      self.artLua = infoTable.assets.artLua;
      self.artImage = infoTable.assets.artImage;
      self.artFps = infoTable.assets.artFps;

      self.textList = {};
      self.buttonList = {};
      self.animList = {};

      self.bgImage = infoTable.popupImage;

      self.bTextureSheet			= FlashMgr:getTextureSheet(self.buttonLua, self.buttonImage, self.buttonFps);
      self.aTextureSheet			= FlashMgr:getTextureSheet(self.artLua, self.artImage, self.artFps);

      -- background
      if self.BackDrop then
         self.BackDrop:setVisible(false);
      end
      self.BackDrop = self:addWidget(Anim, self.mainLayer, 0,0,0,0, self.bgImage, self.aTextureSheet);
      
      --this solves some weird shadowing issues with the flash backgrounds
      if self.BackDrop.anim then
         if self.BackDrop.anim.props then
            for i,v in ipairs(self.BackDrop.anim.props) do
               v:setBlendMode( MOAIProp2D.BLEND_NORMAL );
            end
         end
      end
      
--------------
--check for dollar or gem cost, and build the promo differently
      
      local dollarAnims =  {};
      local gemAnims =     {
                              {name="GemA", x=105.1, y=-161.6, rotation=-16, priority=70, scale=0.8 },
                           };
                     
      local dollarText =   {
                              {text="2x COIN", x=2.15, y=-223.7, w=223.2, h=91.85, size=72, color={r=1,g=1,b=1,a=1}, },
                              {text="Coin Pickups are double value", x=-251.05, y=48.05, w=501.6, h=47.9, size=36, color={r=1,g=1,b=0,a=1}, },
                              {text="...FOREVER!", x=-248.55, y=94.75, w=239.3, h=72.3, size=56, color={r=1,g=1,b=1,a=1}, },
                              {text="$4.99!", x=65.65, y=120.65, w=163, h=110.5, rotation=16, size=56, color={r=1,g=1,b=1,a=1}, },
                           };

      local gemText =      {
                              {text="2x COIN", x=2.15, y=-223.7, w=223.2, h=91.85, size=72, color={r=1,g=1,b=1,a=1}, },
                              {text="Coin Pickups are double value", x=-251.05, y=48.05, w=501.6, h=47.9, size=36, color={r=1,g=1,b=0,a=1}, },
                              {text="...FOREVER!", x=-248.55, y=94.75, w=239.3, h=72.3, size=56, color={r=1,g=1,b=1,a=1}, },
                              {text="200", x=128, y=144, w=92.3, h=72.3, rotation=16, size=56, color={r=1,g=1,b=1,a=1}, },

                           };
      


			
      local cost		= App.data:getPickupUpgradeCost(PICKUP_COIN_INDEX);
      
      if cost then
   
         if cost.dollar then
            self.anims  = dollarAnims;
            self.text   = dollarText;
         elseif cost.gem then
            self.anims  = gemAnims;
            self.text   = gemText;
         end
         
         if self.text and self.anims then

            self:loadButtons( self.buttons );
             
            self:loadAnims( self.anims );

            self:loadText( self.text );
         else
            print("no text or anims");
            self:goBack();
         end
      else
         print("no cost");
         self:goBack();
      end
    
   else
      print("no master table or info table");
      self:goBack();
      
   end

   --make sure the buy now button is pulsing
   if self["MC_Button_BuyNow"] then
      self["MC_Button_BuyNow"]:setSpan(29/30);
      self["MC_Button_BuyNow"]:setMode(MOAITimer.LOOP);
      self["MC_Button_BuyNow"]:start();
   end
   
end


------------------------------------------------------------------------------------

function StatePromoDoubleCoin:onLoseFocus()

   State.onLoseFocus(self);

   StateMgr:setSyncBusyDelay(nil);

end

----------------------------------------------------------------------

function StatePromoDoubleCoin:isBusy()

   return StoreMgr:isBusy();

end


----------------------------------------------------------------

function StatePromoDoubleCoin:onBackButton()

   self:promoDeclined();
	self:goBack();

end

----------------------------------------------------------------

function StatePromoDoubleCoin:isUsingCameraHack()
   return false;
end

----------------------------------------------------------------

function StatePromoDoubleCoin:goBack()
    
   self:removeText();
   self:removeButtons();
   self:removeAnims();
   
   if self.BackDrop then
      self.BackDrop:setVisible(false);
   end
	
	StateMgr:pop();
	
end

----------------------------------------------------------------

function StatePromoDoubleCoin:goDoubleCoin()

   local cost		= App.data:getPickupUpgradeCost(PICKUP_COIN_INDEX) or {};

   Debug.assert( cost.dollar == nil, "StatePromoDoubleCoin:goDoubleCoin -> cost.dollar == nil" );

   if not cost.gold and not cost.gem then
      MOAIDialogIOS.showDialog(  LocalizationMgr:getString( "NATIVE_SERVER_ERROR_TITLE" )
                              ,  LocalizationMgr:getString( "NATIVE_TRY_AGAIN" )
                              ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                              ,  nil
                              ,  nil
                              ,  false
                              ,  function()
                                    enableButtonInput();
                                 end
                              );
                              
      self:goBack();
      return false;
   end

	if not App.data:canAffordUpgradePickup(PICKUP_COIN_INDEX) then
   
		AUTO_PURCHASE_DATA.state		= 'StatePromoDoubleCoin';
		AUTO_PURCHASE_DATA.carIndex	= ""..-1;
		AUTO_PURCHASE_DATA.reason1		= 'Upgrade-Pickup';
		AUTO_PURCHASE_DATA.reason2		= 'Upgrade-Pickup-'..PICKUP_COIN_INDEX;
		AUTO_PURCHASE_DATA.price		= cost.gold or cost.gem;

		if cost.gold then
			AUTO_PURCHASE_DATA.currency	= 'Coin';
		else
			AUTO_PURCHASE_DATA.currency	= 'Gem';
		end

		AUTO_PURCHASE_FUNC				=	function()
														self:goDoubleCoin();
													end

		StateMgr:push("state/state-auto-purchase-popup-maz.lua");

		return false;
	end
	
	App.data:upgradePickup(PICKUP_COIN_INDEX);
   self:promoSuccess();
   
   self:goBack();

end

----------------------------------------------------------------

function StatePromoDoubleCoin:promoSuccess()

   --FIXME
   --JOE. This will always trigger when we are buying double coin from the result screen
   --Debug.assert( PromoMgr.popupInfo, "StatePromoDoubleCoin:promoSuccess -> PromoMgr.popupInfo");

   if PromoMgr.popupInfo and PromoMgr.popupInfo.name then

      local name = "promo-succeed-"..PromoMgr.popupInfo.name;
      
      local successes = App.data:getValue( name );
      if successes then
         successes = successes + 1;
      else
         successes = 1;
      end

      App.data:setValue( name, successes );

   end

   MOAIFlurry:event("Promo-Double-Coin", "Result", "success", "success_level", ""..( App.data:getCurrentPlayerLevel() or "unknown") );

end

----------------------------------------------------------------

function StatePromoDoubleCoin:promoDeclined()

   if self.realFailure then
      MOAIFlurry:event("Promo-Double-Coin", "Result", "failure", "failure_level", ""..( App.data:getCurrentPlayerLevel() or "unknown") );
   else
      MOAIFlurry:event("Promo-Double-Coin", "Result", "declined", "declined_level", ""..( App.data:getCurrentPlayerLevel() or "unknown") );
   end

end

----------------------------------------------------------------

return StatePromoDoubleCoin;

----------------------------------------------------------------
