----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local SQUARE_TEXTURE = "resources/square.png";

local LOADING_BAR_BACK_WIDTH   = 290;
local LOADING_BAR_BACK_HEIGHT  = 36;
local LOADING_BAR_FRONT_WIDTH  = 284;
local LOADING_BAR_FRONT_HEIGHT = 30;

local PULSE_TIME  = 0.5;
local PULSE_SCALE = 1.1;

local PRIORITY_0  = 10;
local PRIORITY_1  = 20;
local PRIORITY_2  = 30;
local PRIORITY_3  = 40;
local PRIORITY_4  = 50;
local PRIORITY_5  = 60;
local PRIORITY_6  = 70;
local PRIORITY_7  = 80;
local PRIORITY_8  = 90;
local PRIORITY_9  = 100;
local PRIORITY_10 = 110;

----------------------------------------------------------------

local Base              = State;
local StatePromoDrones  = Base:new();

----------------------------------------------------------------

function StatePromoDrones:onLoad()

   State.onLoad( self );

   local textureSheet	= FlashMgr:getTextureSheet("resources/flash-resource-bundle/PopUPs.lua", "resources/flash-resource-bundle/PopUPs.png", 30);

   self.defaultFontName	= "resources/font/TangoLatin.otf";
   self.IS_POPUP     = true;

   -- background
   self.Background   = self:addWidget(Image, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE);
   self.Background:setPriority(PRIORITY_0);
   self.Background:setColor(0,0,0, 0.75);

   -- frame
   self.Frame        = self:addWidget(Image, self.mainLayer, 0, 0, 470*1.2, 633*1.2, "resources/UI/Promo/Promo_Drone_Frame.png");
   self.Frame:setPriority(PRIORITY_1);

   -- buttons
   self.GarageButton = self:addWidget(ButtonImage, self.mainLayer, 0, -360, 253*0.8, 131*0.8, "resources/UI/Promo/Promo_Drone_Garage_Button.png", nil, self.goDroneGarage);
   self.GarageButton:setPriority(PRIORITY_2);

	-- Exit Button
	self.ExitButton   = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 265, 360, 0,0, "CloseButton", textureSheet, nil, self.goBack, nil, nil, nil, nil, 40);
	self.ExitButton:setPriority(PRIORITY_2);

end

----------------------------------------------------------------

function StatePromoDrones:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StatePromoDrones:onFocus( data )

   State.onFocus(self);

   self._pulseTimer = 0;

   StateMgr:setSyncBusyDelay(0);

   if not DroneMgr:isEnabled() then
      StateMgr:pop();
   end

end

----------------------------------------------------------------

function StatePromoDrones:onLoseFocus( )

   Base.onLoseFocus(self);

   StateMgr:setSyncBusyDelay(nil);

end

----------------------------------------------------------------

function StatePromoDrones:onUpdate( dt )

   Base.onUpdate(self, dt);

   self:_updatePulse(dt);

end


------------------------------------------------------------------------------------
-- Updates
------------------------------------------------------------------------------------


function StatePromoDrones:_updatePulse( dt )

   self._pulseTimer = self._pulseTimer + dt;

   if self._pulseTimer > PULSE_TIME then
      self._pulseTimer = 0;
   end

   local k = 1;

   if self.GarageButton:isPressed() then
      k = PULSE_SCALE;
      self._pulseTimer = PULSE_TIME/2;
   elseif self._pulseTimer < PULSE_TIME/2 then
      k = math.mapClamped(0, 1, PULSE_TIME/2, PULSE_SCALE, self._pulseTimer);
   else
      k = math.mapClamped(PULSE_TIME/2, PULSE_SCALE, PULSE_TIME, 1, self._pulseTimer);
   end

   self.GarageButton:setScale(k, k);

end


------------------------------------------------------------------------------------
-- Util
------------------------------------------------------------------------------------


function StatePromoDrones:isUsingCameraHack()
   return false;
end

----------------------------------------------------------------

function StatePromoDrones:isBusy()
   return StoreMgr:isBusy();
end


------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------


function StatePromoDrones:goDroneGarage()

   EventMgr:postEvent( { source = "promo-new-drones",  success = true }, "promo-click-through" );

   if DroneMgr:isDroneLevelLocked() then
      StateMgr:swap("state/state-drone-locked-popup.lua");
   else
      if not App.data:getValue("new-unlock-drones", false) then
         App.data:setValue("new-unlock-drones", true);
      end

      StateMgr:swap("state/state-drone-garage.lua");
   end

end

----------------------------------------------------------------

function StatePromoDrones:goBack()

   EventMgr:postEvent( { source = "promo-new-drones",  success = false }, "promo-click-through" );
   StateMgr:pop();

end


------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------

return StatePromoDrones;
