----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local PRIORITY_0				= 10;
local PRIORITY_1				= 20;
local PRIORITY_2				= 30;
local PRIORITY_3				= 40;
local PRIORITY_4				= 50;
local PRIORITY_5				= 60;
local PRIORITY_6				= 70;
local PRIORITY_7				= 80;
local PRIORITY_8				= 90;
local PRIORITY_9				= 100;
local PRIORITY_10				= 110;

local shadowXOff                = 2;
local shadowYOff                = -2;

----------------------------------------------------------------

local Base							= State;
local StatePromoGems		= Base:new();

----------------------------------------------------------------

function StatePromoGems:onLoad()

   State.onLoad( self );

	self.IS_POPUP  = true;
   self.defaultFontName = "resources/font/TangoLatin.otf";

	-- black overlay
   self.Overlay = self:addWidget(Image, self.mainLayer, 0, 0, App.viewportWidth, App.viewportHeight, "resources/UI/blackbox.png", {1,1,1, 0.8});

end

----------------------------------------------------------------

function StatePromoGems:loadText( text )

    local fontSizes = {};

    if text then
        for i=1,#text do
            if #fontSizes > 0 then
                for j,k in ipairs( fontSizes ) do
                    if text[i].size ~= k then
                        table.insert ( fontSizes, text[i].size );
                    end
                end
            else
                table.insert ( fontSizes, text[i].size );
            end
        end
    end

    for o,p in ipairs(text) do

        local x,y = self:convertCoords(p.x, p.y, p.w, p.h);
        if x and y and p.w and p.h then
    
            local boxShadow = self:addWidget(TextBox, self.mainLayer, x+shadowXOff, y+shadowYOff, p.w, p.h, p.text, self.defaultFontName, p.size * 0.4444, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
            if p.rotation then
                boxShadow:setRot( -p.rotation );
            end
            boxShadow:setColor(0, 0, 0, 1);

            local box = self:addWidget(TextBox, self.mainLayer, x, y, p.w, p.h, p.text, self.defaultFontName, p.size * 0.4444, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
            if p.rotation then
                box:setRot( -p.rotation );
            end
            
            if p.color then
                box:setColor(p.color.r, p.color.g, p.color.b, p.color.a);
            end
            
            table.insert(self.textList, boxShadow);
            table.insert(self.textList, box);
        end
    end

end

----------------------------------------------------------------

function StatePromoGems:loadButtons( buttons )

    for i,v in ipairs( buttons ) do

        self[v.name] = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, v.name, self.bTextureSheet, nil, self[v.func], nil, nil );
        
        if v.play then
            self[v.name]:setSpan(29/30);
            self[v.name]:setMode(MOAITimer.LOOP);
            self[v.name]:start();
        end
        
        table.insert(self.buttonList, self[v.name]);
    end

end

----------------------------------------------------------------

function StatePromoGems:loadAnims( anims )

   if anims then

      for i,v in ipairs( anims ) do
      
         local x,y = v.x, -v.y
        
         self[v.name] = self:addWidget(Anim, self.mainLayer, x, y, 0,0, v.name, self.aTextureSheet);
         if v.scale then
            self[v.name]:setScale( v.scale );
         end
         if v.priority then
            self[v.name]:setPriority( v.priority );
         end
         if v.rotation then
            self[v.name]:setRot( v.rotation );
         end

         table.insert(self.animList, self[v.name]);
      end
   end

end

----------------------------------------------------------------

function StatePromoGems:removeText()

   if self.textList then

      for i,v in ipairs(self.textList) do
         self:deleteWidget(v);
         v=nil;
      end
   end
end

----------------------------------------------------------------

function StatePromoGems:removeButtons()
   if self.buttonList then
      for i,v in ipairs(self.buttonList) do
         self:deleteWidget(v);
         v=nil;
      end
   end
end

----------------------------------------------------------------

function StatePromoGems:removeAnims()
   if self.animList then
      for i,v in ipairs(self.animList) do
         self:deleteWidget(v);
         v=nil;
      end
   end
end

----------------------------------------------------------------

function StatePromoGems:convertCoords( x, y, w, h )

    if x and y and w and h then
        local bX = x;
        local bY = -y;
        
        local rX = bX + (w/2);
        local rY = bY - (h/2);

        return rX, rY;
    end

end

----------------------------------------------------------------

function StatePromoGems:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StatePromoGems:onFocus( masterTable, promoTable )
   
   State.onFocus(self);

   self.realFailure = false;

   StateMgr:setSyncBusyDelay(0);
   
   if masterTable then
      self.masterTable = masterTable;
   end
   if promoTable then
      self.promoTable = promoTable;
   end

   if self.promoTable and self.masterTable then

      self.text = self.promoTable.popupText
      self.buttons = self.promoTable.buttons;
      self.anims = self.promoTable.anims;
      self.buttonLua = self.promoTable.assets.buttonLua;
      self.buttonImage = self.promoTable.assets.buttonImage;
      self.buttonFps = self.promoTable.assets.buttonFps;

      self.artLua = self.promoTable.assets.artLua;
      self.artImage = self.promoTable.assets.artImage;
      self.artFps = self.promoTable.assets.artFps;

      self.gemIndex = self.masterTable.purchaseID;

      self.textList = {};
      self.buttonList = {};
      self.animList = {};

      self.bgImage = self.promoTable.popupImage;

      self.bTextureSheet			= FlashMgr:getTextureSheet(self.buttonLua, self.buttonImage, self.buttonFps);
      self.aTextureSheet			= FlashMgr:getTextureSheet(self.artLua, self.artImage, self.artFps);

      -- background
      if self.BackDrop then
         self.BackDrop:setVisible(false);
      end
      self.BackDrop = self:addWidget(Anim, self.mainLayer, 0,0,0,0, self.bgImage, self.aTextureSheet);

      --this solves some weird shadowing issues with the flash backgrounds
      if self.BackDrop.anim then
         if self.BackDrop.anim.props then
            for i,v in ipairs(self.BackDrop.anim.props) do
               v:setBlendMode( MOAIProp2D.BLEND_NORMAL );
            end
         end
      end

      self:loadButtons( self.buttons );

      self:loadText( self.text );

      self:loadAnims( self.anims );

   else
   
      self:goBack();

   end
   
end


------------------------------------------------------------------------------------

function StatePromoGems:onLoseFocus()

   State.onLoseFocus(self);

   StateMgr:setSyncBusyDelay(nil);

end

----------------------------------------------------------------------

function StatePromoGems:isBusy()

   return StoreMgr:isBusy();

end


----------------------------------------------------------------

function StatePromoGems:onBackButton()

   self:promoDeclined()
	self:goBack();

end

----------------------------------------------------------------

function StatePromoGems:isUsingCameraHack()
   return false;
end

----------------------------------------------------------------

function StatePromoGems:goBack()
    
   self:removeText();
   self:removeButtons();
   self:removeAnims();
    
   if self.BackDrop then
      self.BackDrop:setVisible(false);
   end
	
	StateMgr:pop();
	
end

----------------------------------------------------------------

function StatePromoGems:goBuyGems()

   if self.gemIndex then

      if StoreMgr:canMakePayments() then
      
         disableButtonInput();
      
         if App.data:buyStoreGem(self.gemIndex, self, self.onBuySuccess, self.onBuyFail) then
      
         else
      
            self:onBuyFail();
      
         end

      else

         self.errorText = LocalizationMgr:getString( "NATIVE_PURCH_DISABLED" );
         self:onBuyFail();
         
      end

   end

end
   
----------------------------------------------------------------

function StatePromoGems:onBuySuccess()

   self:promoSuccess();

	local bGems = App.data:getStoreGemAmount( self.gemIndex );
	
	EventMgr:postEvent( {  gem=bGems, source="promo", index=self.gemIndex }, "gems-bought" );
	
	enableButtonInput();
	StateMgr:pop();

end
   
----------------------------------------------------------------

function StatePromoGems:onBuyFail()

   
	enableButtonInput();
   
   local errorTitle = LocalizationMgr:getString( "NATIVE_TRANS_FAIL" );
   local text = self.errorText or LocalizationMgr:getString( "NATIVE_TRY_AGAIN" );

   self.realFailure = true;

   MOAIDialogIOS.showDialog(	errorTitle
                              ,	text
                              ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                              ,  nil
                              ,  nil
                              ,  false
                              ,  function()
                                 end
                           );
   
end

----------------------------------------------------------------

function StatePromoGems:promoSuccess()

   if self.promoTable.name then

      local name = "promo-succeed-"..self.promoTable.name;
      
      local successes = App.data:getValue( name );
      if successes then
         successes = successes + 1;
      else
         successes = 1;
      end

      App.data:setValue( name, successes );

   end

   MOAIFlurry:event("Promo-Gem-Price-"..self.promoTable.name, "Result", "success", "success_level", ""..( App.data:getCurrentPlayerLevel() or "unknown"));

end

----------------------------------------------------------------

function StatePromoGems:promoDeclined()

   if self.realFailure then
      MOAIFlurry:event("Promo-Gem-Price-"..self.promoTable.name, "Result", "failure", "failure_level", ""..( App.data:getCurrentPlayerLevel() or "unknown"));
   else
      MOAIFlurry:event("Promo-Gem-Price-"..self.promoTable.name, "Result", "declined", "declined_level", ""..( App.data:getCurrentPlayerLevel() or "unknown"));
   end
end

----------------------------------------------------------------

return StatePromoGems;

----------------------------------------------------------------
