----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local PRIORITY_0				= 10;
local PRIORITY_1				= 20;
local PRIORITY_2				= 30;
local PRIORITY_3				= 40;
local PRIORITY_4				= 50;
local PRIORITY_5				= 60;
local PRIORITY_6				= 70;
local PRIORITY_7				= 80;
local PRIORITY_8				= 90;
local PRIORITY_9				= 100;
local PRIORITY_10				= 110;

local shadowXOff                = 2;
local shadowYOff                = -2;

----------------------------------------------------------------

local Base							= State;
local StatePromoGift		= Base:new();

----------------------------------------------------------------

function StatePromoGift:onLoad()

   State.onLoad( self );

	self.IS_POPUP  = true;
   self.defaultFontName = "resources/font/TangoLatin.otf";

	-- black overlay
   self.Overlay = self:addWidget(Image, self.mainLayer, 0, 0, App.viewportWidth, App.viewportHeight, "resources/UI/blackbox.png", {1,1,1, 0.8});

end

----------------------------------------------------------------

function StatePromoGift:loadText( text )

    local fontSizes = {};

    if text then
        for i=1,#text do
            if #fontSizes > 0 then
                for j,k in ipairs( fontSizes ) do
                    if text[i].size ~= k then
                        table.insert ( fontSizes, text[i].size );
                    end
                end
            else
                table.insert ( fontSizes, text[i].size );
            end
        end
    end

    for o,p in ipairs(text) do

        local x,y = self:convertCoords(p.x, p.y, p.w, p.h);
        if x and y and p.w and p.h then
    
            local boxShadow = self:addWidget(TextBox, self.mainLayer, x+shadowXOff, y+shadowYOff, p.w, p.h, p.text, self.defaultFontName, p.size * 0.4444, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
            if p.rotation then
                boxShadow:setRot( -p.rotation );
            end
            boxShadow:setColor(0, 0, 0, 1);

            local box = self:addWidget(TextBox, self.mainLayer, x, y, p.w, p.h, p.text, self.defaultFontName, p.size * 0.4444, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
            if p.rotation then
                box:setRot( -p.rotation );
            end
            
            if p.color then
                box:setColor(p.color.r, p.color.g, p.color.b, p.color.a);
            end
            
            table.insert(self.textList, boxShadow);
            table.insert(self.textList, box);
        end
    end

end

----------------------------------------------------------------

function StatePromoGift:loadButtons( buttons )

    for i,v in ipairs( buttons ) do

        self[v.name] = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, v.name, self.bTextureSheet, nil, self[v.func], nil, nil );
        
        if v.play then
            self[v.name]:setSpan(29/30);
            self[v.name]:setMode(MOAITimer.LOOP);
            self[v.name]:start();
        end
        
        table.insert(self.buttonList, self[v.name]);
    end

end

----------------------------------------------------------------

function StatePromoGift:loadAnims( anims )

   if anims then

       for i,v in ipairs( anims ) do
           
         self[v.name] = self:addWidget(Anim, self.mainLayer, v.x, v.y, 0,0, v.name, self.aTextureSheet);
         if v.scale then
            self[v.name]:setScale( v.scale );
         end
         if v.priority then
            self[v.name]:setPriority( v.priority );
         end
         if v.rotation then
            self[v.name]:setRot( v.rotation );
         end
           
           table.insert(self.animList, self[v.name]);
       end
   end

end

----------------------------------------------------------------

function StatePromoGift:removeText()

   if self.textList then
      for i,v in ipairs(self.textList) do
         self:deleteWidget(v);
         v=nil;
      end
   end

end

----------------------------------------------------------------

function StatePromoGift:removeButtons()

   if self.buttonList then
      for i,v in ipairs(self.buttonList) do
         self:deleteWidget(v);
         v=nil;
      end
   end

end

----------------------------------------------------------------

function StatePromoGift:removeAnims()

   if self.animList then
      for i,v in ipairs(self.animList) do
         self:deleteWidget(v);
         v=nil;
      end
   end
end

----------------------------------------------------------------

function StatePromoGift:convertCoords( x, y, w, h )
    
    local sW = App.viewportWidth;
    local sH = App.viewportHeight;
    if sW and sH and x and y and w and h then
        local bX = x;
        local bY = -y;
        
        local rX = bX + (w/2);
        local rY = bY - (h/2);
--print("promo textbox x,y:", rX, rY);
        return rX, rY;
    end

end

----------------------------------------------------------------

function StatePromoGift:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StatePromoGift:onFocus( masterTable, infoTable )
   
   State.onFocus(self);

   self.realFailure = false;
   
   if masterTable then
      self.masterTable = masterTable;
   end
   if infoTable then
      self.infoTable = infoTable;
   end
   
   if self.infoTable and self.masterTable then
      
      self.promoReward = self.masterTable.reward;

      self.transactionsComplete = 0;
      self.inviteSuccesses = 0;
      self.inviteFailures = 0;
      self.giftSuccesses = 0;
      self.giftFailures = 0;

      self.text = self.infoTable.popupText
      self.buttons = self.infoTable.buttons;
      self.anims = self.infoTable.anims;
      self.buttonLua = self.infoTable.assets.buttonLua;
      self.buttonImage = self.infoTable.assets.buttonImage;
      self.buttonFps = self.infoTable.assets.buttonFps;
      
      self.artLua = self.infoTable.assets.artLua;
      self.artImage = self.infoTable.assets.artImage;
      self.artFps = self.infoTable.assets.artFps;

      self.textList = {};
      self.buttonList = {};
      self.animList = {};

      self.bgImage = self.infoTable.popupImage;

      self.bTextureSheet			= FlashMgr:getTextureSheet(self.buttonLua, self.buttonImage, self.buttonFps);
      self.aTextureSheet			= FlashMgr:getTextureSheet(self.artLua, self.artImage, self.artFps);

      -- background
      if self.BackDrop then
         self.BackDrop:setVisible(false);
      end
      self.BackDrop = self:addWidget(Anim, self.mainLayer, 0,0,0,0, self.bgImage, self.aTextureSheet);

      --this solves some weird shadowing issues with the flash backgrounds
      if self.BackDrop.anim then
         if self.BackDrop.anim.props then
            for i,v in ipairs(self.BackDrop.anim.props) do
               v:setBlendMode( MOAIProp2D.BLEND_NORMAL );
            end
         end
      end

    self:loadButtons( self.buttons );
    
    self:loadAnims( self.anims );

    self:loadText( self.text );
       
   else
   
      self:goBack();
      
   end

end

----------------------------------------------------------------

function StatePromoGift:onUpdate( dt )

   Base:onUpdate( dt );

end


----------------------------------------------------------------

function StatePromoGift:onBackButton()

   self:promoDeclined();
   self:goBack();

end

----------------------------------------------------------------

function StatePromoGift:isUsingCameraHack()
   return false;
end

----------------------------------------------------------------

function StatePromoGift:cleanup()

   self:removeText();
   self:removeButtons();
   self:removeAnims();

   if self.BackDrop then
      self.BackDrop:setVisible(false);
   end

end

----------------------------------------------------------------

function StatePromoGift:goBack()

   self:cleanup();

	StateMgr:pop();
	
end

----------------------------------------------------------------

function StatePromoGift:goFriends()

   self.inviteList   = {};
   self.giftList     = {};

   self.totalTransactions = 0;

   local  inviteTemplates  = MessageMgr:getInviteTemplateList();
   local  giftTemplates    = MessageMgr:getGiftTemplateList();

   if OnlineMgr:isOnline() and ContactMgr:getFriendsCount() > 0 and inviteTemplates and giftTemplates then

      for i=1,ContactMgr:getFriendsCount() do
      
         local contact = ContactMgr:getFriend(i);

         if  contact then

            self.totalTransactions = self.totalTransactions + 1;

            if (not contact:isAppInstalled()) then
               table.insert(self.inviteList, contact);
            else
               table.insert(self.giftList, contact);
            end
         end
      end

      local inviteList  = {};
      local giftList    = {};

      for i,contact in ipairs(self.inviteList) do
         table.insert(inviteList, contact);
      end

      for i,contact in ipairs(self.giftList) do
         table.insert(giftList, contact);
      end

      local inviteIndex  = math.random( 1, #inviteTemplates );
      local  giftIndex   = math.random( 1, #giftTemplates   );

      self:sendInviteTango( inviteTemplates[inviteIndex],   inviteList );
      self:sendGift(        giftTemplates[giftIndex],       giftList    );


   end

   if self.totalTransactions > 0 then

      self:promoSuccess();
      EventMgr:postEvent( {gold=self.promoReward.gold, gems=self.promoReward.gems}, "promo-gift-reward" )
      App.data:addGold( self.promoReward.gold, "promo-invite" )
      self:goBack();

   else

      MOAIDialogIOS.showDialog(     LocalizationMgr:getString( "NATIVE_TRANS_FAIL" )
                                 ,	LocalizationMgr:getString( "NATIVE_CONNECT_TO_NET" )
                                 ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                                 ,  nil
                                 ,  nil
                                 ,  false
                                 ,  function()
                                    end
                              );

   end

end

----------------------------------------------------------------

function StatePromoGift:sendInviteTango( msgTpl, contactList )

   if msgTpl and contactList and #contactList > 0 then
      MessageMgr:sendInvite( msgTpl, contactList );
   end

end

----------------------------------------------------------------

function StatePromoGift:sendGift( msgTpl, contactList )

   if contactList and #contactList > 0 then

      MessageMgr:sendGift( msgTpl, contactList );
   end

end

----------------------------------------------------------------

function StatePromoGift:promoSuccess()

   if self.infoTable.name then

      local name = "promo-succeed-"..self.infoTable.name;
      
      local successes = App.data:getValue( name );
      if successes then
         successes = successes + 1;
      else
         successes = 1;
      end

      App.data:setValue( name, successes );

   end

   MOAIFlurry:event("Promo-Invite", "Result", "success", "success_level", ""..( App.data:getCurrentPlayerLevel() or "unknown"));

end

----------------------------------------------------------------

function StatePromoGift:promoDeclined()

--print("StatePromoGift:promoDeclined");

   if self.realFailure then
      MOAIFlurry:event("Promo-Invite", "Result", "failure", "failure_level", ""..( App.data:getCurrentPlayerLevel() or "unknown"));
   else
      MOAIFlurry:event("Promo-Invite", "Result", "declined", "declined_level", ""..( App.data:getCurrentPlayerLevel() or "unknown"));
   end
end

----------------------------------------------------------------

return StatePromoGift;

----------------------------------------------------------------
