----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local PRIORITY_0				= 10;
local PRIORITY_1				= 20;
local PRIORITY_2				= 30;
local PRIORITY_3				= 40;
local PRIORITY_4				= 50;
local PRIORITY_5				= 60;
local PRIORITY_6				= 70;
local PRIORITY_7				= 80;
local PRIORITY_8				= 90;
local PRIORITY_9				= 100;
local PRIORITY_10				= 110;

local shadowXOff                = 2;
local shadowYOff                = -2;

local VIDEO_GOLD_REWARD    	    = 100;
local VIDEO_KEY_REWARD     		= 1;
local VIDEO_MAX_VIEWS      		= 5;
local VIDEO_RESET_TIME     		= 60*60*24;

----------------------------------------------------------------

local Base							= State;
local StatePromoMovie		= Base:new();

----------------------------------------------------------------

function StatePromoMovie:onLoad()

   State.onLoad( self );

	self.IS_POPUP  = true;
   self.defaultFontName = "resources/font/TangoLatin.otf";
   
	-- black overlay
   self.Overlay = self:addWidget(Image, self.mainLayer, 0, 0, App.viewportWidth, App.viewportHeight, "resources/UI/blackbox.png", {1,1,1, 0.5});

end

----------------------------------------------------------------

function StatePromoMovie:videoParamsSetup()

   local m = AbTestMgr:getVideoMapLimit( )
   if m then
      VIDEO_MAX_VIEWS = m;
   end

   local k = AbTestMgr:getVideoMapKeys( )
      VIDEO_KEY_REWARD = k;
   if k then
   end

   local g = AbTestMgr:getVideoMapGold( )
   if g then
      VIDEO_GOLD_REWARD = g;
   end

   local t = AbTestMgr:getVideoMapReset( )
   if t then
      VIDEO_RESET_TIME = t;
   end

end

----------------------------------------------------------------

function StatePromoMovie:loadText( text )

    self:removeText();

    local fontSizes = {};

    if text then
        for i=1,#text do
            if #fontSizes > 0 then
                for j,k in ipairs( fontSizes ) do
                    if text[i].size ~= k then
                        table.insert ( fontSizes, text[i].size );
                    end
                end
            else
                table.insert ( fontSizes, text[i].size );
            end
        end
    end

    for o,p in ipairs(text) do

        local x,y = self:convertCoords(p.x, p.y, p.w, p.h);
        if x and y and p.w and p.h then
        
            if not p.noShadow then

               self[p.name.."Shadow"] = self:addWidget(TextBox, self.mainLayer, x+shadowXOff, y+shadowYOff, p.w, p.h, p.text, self.defaultFontName, p.size * 0.4444, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
               if p.rotation then
                   self[p.name.."Shadow"]:setRot( -p.rotation );
               end
               self[p.name.."Shadow"]:setColor(0, 0, 0, 1);
               if p.justifyLeft then
                  self[p.name.."Shadow"]:setAlignment ( MOAITextBox.LEFT_JUSTIFY, MOAITextBox.LEFT_JUSTIFY );
               end
               
            end

            self[p.name] = self:addWidget(TextBox, self.mainLayer, x, y, p.w, p.h, p.text, self.defaultFontName, p.size * 0.4444, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
            if p.rotation then
                self[p.name]:setRot( -p.rotation );
            end
            
            if p.color then
                self[p.name]:setColor(p.color.r, p.color.g, p.color.b, p.color.a);
            end

            if p.justifyLeft then
               self[p.name]:setAlignment ( MOAITextBox.LEFT_JUSTIFY, MOAITextBox.LEFT_JUSTIFY );
               if not p.noShadow then
                  self[p.name.."Shadow"]:setAlignment ( MOAITextBox.LEFT_JUSTIFY, MOAITextBox.LEFT_JUSTIFY );
               end
            end

            if self.textList then
               if not p.noShadow then
                  table.insert(self.textList, self[p.name.."Shadow"]);
               end
               table.insert(self.textList, self[p.name]);
            end
        end
    end

end

----------------------------------------------------------------

function StatePromoMovie:loadButtons( buttons )

   for i,v in ipairs( buttons ) do
      
      local x,y = nil,nil;
      if v.x then
         x = v.x;
      end
      if v.y then
         y = v.y;
      end
      
      local bX, bY = self:convertButtonCoords( x, y )

      self[v.name] = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, bX,bY,0,0, v.name, self.bTextureSheet, nil, self[v.func], nil, nil );

      if v.play then
         self[v.name]:setSpan(29/30);
         self[v.name]:setMode(MOAITimer.LOOP);
         self[v.name]:start();
      end
      if self.buttonList then
         table.insert(self.buttonList, self[v.name]);
      end
   end

end

----------------------------------------------------------------

function StatePromoMovie:loadAnims( anims )

   if anims then

       for i,v in ipairs( anims ) do
       
         if self[v.name] then
            self[v.name]:hide();
            self[v.name] = nil;
         end
           
         self[v.name] = self:addWidget(Anim, self.mainLayer, v.x, v.y, 0,0, v.name, self.aTextureSheet);
         if v.scale then
            self[v.name]:setScale( v.scale );
         end
         if v.priority then
            self[v.name]:setPriority( v.priority );
         end
         if v.rotation then
            self[v.name]:setRot( v.rotation );
         end
         if self.animList then
            table.insert(self.animList, self[v.name]);
         end
       end
   end

end

----------------------------------------------------------------

function StatePromoMovie:removeText()

   if self.textList then

      for i,v in ipairs(self.textList) do
			self:deleteWidget(v);
         v=nil;
      end

   end
end

----------------------------------------------------------------

function StatePromoMovie:removeButtons()

   if self.buttonList then

      for i,v in ipairs(self.buttonList) do
			self:deleteWidget(v);
        v=nil;
      end

   end
end

----------------------------------------------------------------

function StatePromoMovie:removeAnims()

   if self.animList then

      for i,v in ipairs(self.animList) do
			self:deleteWidget(v);
			v=nil;
      end

   end
end

----------------------------------------------------------------

function StatePromoMovie:convertCoords( x, y, w, h )

    local sW, sH = 640, 1136;
    if sW and sH and x and y and w and h then
        local bX = x - sW/2;
        local bY = sH/2 - y;
        
        local rX = bX + (w/2);
        local rY = bY - (h/2);

        return rX, rY;
    end

end

----------------------------------------------------------------

function StatePromoMovie:convertButtonCoords( x, y )

   local sW, sH = 640, 1136;
   if sW and sH then
      
      local rX, rY = 0,0;
      if x then
         rX = x - sW/2;
      end
      if y then
         rY = sH/2 -y;
      end

      return rX, rY;
   end

end

----------------------------------------------------------------

function StatePromoMovie:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StatePromoMovie:onFocus( masterTable, infoTable )
   
   State.onFocus(self);

   self.realFailure = false;

   StateMgr:setSyncBusyDelay(0);

   self:videoParamsSetup();
   
   self:timerSetup();


   if masterTable and infoTable then

      self.buttons = infoTable.buttons;
      self.buttonLua = infoTable.assets.buttonLua;
      self.buttonImage = infoTable.assets.buttonImage;
      self.buttonFps = infoTable.assets.buttonFps;
      
      self.artLua = infoTable.assets.artLua;
      self.artImage = infoTable.assets.artImage;
      self.artFps = infoTable.assets.artFps;
      
      self.anims = infoTable.anims;
      self.animLua = infoTable.assets.animLua;
      self.animImage = infoTable.assets.animImage;
      self.animFps = infoTable.assets.animFps;
      
      self.text = infoTable.popupText;

      self.textList = {};
      self.buttonList = {};
      self.animList = {};

      self.bgImage = infoTable.popupImage;

      if self.animLua and self.animImage then
         self.aTextureSheet			= FlashMgr:getTextureSheet(self.animLua, self.animImage, self.animFps);
      end
      if self.buttonLua and self.buttonImage then
         self.bTextureSheet			= FlashMgr:getTextureSheet(self.buttonLua, self.buttonImage, self.buttonFps);
      end
      if self.artLua and self.artImage then
         self.artTextureSheet			= FlashMgr:getTextureSheet(self.artLua, self.artImage, self.artFps);
      end


      -- background
      if self.BackDrop then
         self.BackDrop:setVisible(false);
      end
      self.BackDrop = self:addWidget(Anim, self.mainLayer, 0,0,0,0, self.bgImage, self.artTextureSheet);
      
      --this solves some weird shadowing issues with the flash backgrounds
      if self.BackDrop.anim then
         if self.BackDrop.anim.props then
            for i,v in ipairs(self.BackDrop.anim.props) do
               v:setBlendMode( MOAIProp2D.BLEND_NORMAL );
            end
         end
      end
      

         
      if self.text then
         self:loadText( self.text );
      end
      if self.anims then
         self:loadAnims( self.anims );
      end
      if self.buttons then
         self:loadButtons( self.buttons );
      end
      
      self.videosRemaining = (App.data:getValue("videos-remaining") or VIDEO_MAX_VIEWS);
      self:textVisibleUpdate();
      if self.videosRemaining <= 0 then
         --disable the video button
         if self["WatchMovieButton"] then
            self["WatchMovieButton"]:disable();
            self["WatchMovieButton"]:setColor(1,1,1,0.5);
         end
      end
      
      self:notificationCheck();
    
   else

      self:goBack();
      
   end
   
end


------------------------------------------------------------------------------------

function StatePromoMovie:onLoseFocus()

   State.onLoseFocus(self);

   StateMgr:setSyncBusyDelay(nil);

end

------------------------------------------------------------------------------------

function StatePromoMovie:onUpdate( dt )

   State.onUpdate(self,dt);
   
   self:updateTimer( dt );
   self:textVisibleUpdate();
   
   if not self.nDelayCount then
      self.nDelayCount = 0.5;
   end

   self.nDelayCount = self.nDelayCount - dt;
   if self.nDelayCount <= 0 then
      --check notification badge
      self:notificationCheck();
      self.nDelayCount = 0.5;
   end

end

----------------------------------------------------------------

function StatePromoMovie:timerSetup()

   self.timestamp = App.data:getValue("video-timestamp");
   self.currentTime = TimeMgr:getTimeGMT();
   
   if not self.currentTime then
      --something wrong with timeMgr
      return false;
   end

   if not self.timestamp then
      self.timestamp = self.currentTime;
      App.data:setValue("video-timestamp", self.timestamp);
   end
   
   return true;

end

------------------------------------------------------------------------------------

function StatePromoMovie:updateTimer( dt )

	local time = '--:--:--';
	local countdown;
   local hours;
   local min;
	local sec;
   
   if not self.timestamp or not self.currentTime then
      self:timerSetup();
   end
   if self.currentTime then
      self.currentTime = self.currentTime + dt;
      countdown = ( VIDEO_RESET_TIME + self.timestamp ) - self.currentTime;
      if countdown <= 0 then
         --reset number of videos
         --reset timestamp
         self.videosRemaining = VIDEO_MAX_VIEWS;
         self.timestamp = self.currentTime;
         App.data:setValue("video-timestamp", self.timestamp);
         App.data:setValue("videos-remaining", VIDEO_MAX_VIEWS);
         if self["WatchMovieButton"] then
            self["WatchMovieButton"]:enable();
            self["WatchMovieButton"]:setColor(1,1,1,1);
         end
      end

   end

      
   if countdown then
      if countdown > 0 then
      
         sec = math.floor(countdown%60);

         if sec < 10 then
            sec = "0".. sec;
         end
         
         min = math.floor((countdown%3600)/60);

         if min < 10 then
            min = "0".. min;
         end
         
         hours = math.floor(countdown/3600);
         
         if hours < 10 then
            hours = "0".. hours;
         end
         
         time = hours.. ":".. min.. ":".. sec;

      end
   end

   local remainingViews = (self.videosRemaining or "-");
   
   if self["limitText"] then
      self["limitText"]:setText("Limit: "..remainingViews.."/"..VIDEO_MAX_VIEWS);
   end
   if self["timeText"] then
      self["timeText"]:setText("Reset "..time);
   end

end

----------------------------------------------------------------------

function StatePromoMovie:textVisibleUpdate()


   if self["limitText"] and self.videosRemaining then
      if self["limitText"]:isVisible() then
         if self.videosRemaining <= 0 then
            self["limitText"]:setVisible(false);
         end
      end
   end
      
   if self["timeText"] and self.videosRemaining then

      if self["timeText"]:isVisible() then
         if self.videosRemaining > 0 then
            self["timeText"]:setVisible(false);
         end
      end
   end


end

----------------------------------------------------------------------

function StatePromoMovie:notificationCheck()

   if ( not AdMgr:isVideoAvailable() or not OnlineMgr:isOnline() ) and self.videosRemaining > 0 then
      if self["WatchMovie_notification"] then
         self["WatchMovie_notification"]:setVisible(true);
      end
   else
      if self["WatchMovie_notification"] then
         self["WatchMovie_notification"]:setVisible(false);
      end
   end
end

----------------------------------------------------------------------

function StatePromoMovie:isBusy()

   return StoreMgr:isBusy();

end


----------------------------------------------------------------

function StatePromoMovie:onBackButton()

	self:goBack();

end

----------------------------------------------------------------

function StatePromoMovie:isUsingCameraHack()
   return false;
end

----------------------------------------------------------------

function StatePromoMovie:goBack()
    
   self:removeText();
   self:removeButtons();
   self:removeAnims();
   
   if self.BackDrop then
      self.BackDrop:setVisible(false);
   end
	
	StateMgr:pop();
	
end

----------------------------------------------------------------

function StatePromoMovie:goBuyKeys()

      StateMgr:swap("state/state-key.lua");

end

----------------------------------------------------------------

function StatePromoMovie:goBuyGold()

   StateMgr:swap("state/state-coin.lua");

end

----------------------------------------------------------------

function StatePromoMovie:goVideoGold()

   StateMgr:swap("state/state-video-play.lua", {rewardType="gold", rewardAmount=VIDEO_GOLD_REWARD or 100} );

end

----------------------------------------------------------------

function StatePromoMovie:goVideoKeys()

print("StatePromoMovie:goVideoKeys",VIDEO_KEY_REWARD);
   StateMgr:swap("state/state-video-play.lua", {rewardType="key", rewardAmount=VIDEO_KEY_REWARD or 1} );

end

----------------------------------------------------------------

return StatePromoMovie;

----------------------------------------------------------------
