----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local SQUARE_TEXTURE = "resources/square.png";

local LOADING_BAR_BACK_WIDTH   = 290;
local LOADING_BAR_BACK_HEIGHT  = 36;
local LOADING_BAR_FRONT_WIDTH  = 284;
local LOADING_BAR_FRONT_HEIGHT = 30;

local PULSE_TIME  = 0.5;
local PULSE_SCALE = 1.1;

local PRIORITY_0  = 10;
local PRIORITY_1  = 20;
local PRIORITY_2  = 30;
local PRIORITY_3  = 40;
local PRIORITY_4  = 50;
local PRIORITY_5  = 60;
local PRIORITY_6  = 70;
local PRIORITY_7  = 80;
local PRIORITY_8  = 90;
local PRIORITY_9  = 100;
local PRIORITY_10 = 110;

local OFFER_PRODUCT_LIST = {
    [ 1 ] = {
        [ 'id' ] = 'special-offer-01',
        [ 'image' ] = 'resources/UI/Promo/Promo_HeadStartOffer1_Frame.png',
    },
    [ 2 ] = {
        [ 'id' ] = 'special-offer-02',
        [ 'image' ] = 'resources/UI/Promo/Promo_HeadStartOffer2_Frame.png',
    },
    [ 3 ] = {
        [ 'id' ] = 'special-offer-03',
        [ 'image' ] = 'resources/UI/Promo/Promo_HeadStartOffer3_Frame.png',
    },
};

----------------------------------------------------------------

local Base              = State;
local StatePromoOffers  = Base:new();

----------------------------------------------------------------

function StatePromoOffers:onLoad()

    State.onLoad( self );
    self.IS_POPUP     = true;

    local textureSheet	= FlashMgr:getTextureSheet("resources/flash-resource-bundle/PopUPs.lua", "resources/flash-resource-bundle/PopUPs.png", 30);

    self.defaultFontName	= "resources/font/TangoLatin.otf";

    self.offerId = OFFER_PRODUCT_LIST[1].id;
    self.offerImage = OFFER_PRODUCT_LIST[1].image;


    -- background
    self.Background   = self:addWidget(Image, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE);
    self.Background:setPriority(PRIORITY_0);
    self.Background:setColor(0,0,0, 0.75);

    -- frame
    self.Frame        = self:addWidget(Image, self.mainLayer, 0, 0, 470*1.2, 633*1.2, self.offerImage );
    self.Frame:setPriority(PRIORITY_1);

    -- buttons
    local buttonY = -250;
    self.BuyOfferButton =    self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,buttonY,0,0, "BuyButton", textureSheet, nil, self.offerPurchased, nil, nil);


    --self.BuyOfferButton = self:addWidget(ButtonImage, self.mainLayer, 0,buttonY,253*0.8, 131*0.8, "resources/UI/Consumable/Consumable_GreenButton.png", nil, self.offerPurchased);
    self.BuyOfferButton:setPriority(PRIORITY_2);

    self.BuyOfferText = self:addWidget(TextBox, self.mainLayer, 0, buttonY, 253*0.8, 131*0.8, '', self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
    self.BuyOfferTextShdw = self:addWidget(TextBox, self.mainLayer, 0+2,buttonY-2, 253*0.8, 131*0.8, '', self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);

    self.BuyOfferText:setPriority(PRIORITY_3);
    self.BuyOfferTextShdw:setPriority(PRIORITY_2);

    -- Exit Button
    self.ExitButton   = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 265, 360, 0,0, "CloseButton", textureSheet, nil, self.goBack, nil, nil, nil, nil, 40);
    self.ExitButton:setPriority(PRIORITY_2);

end

----------------------------------------------------------------

function StatePromoOffers:onActivate()

    State.onActivate(self);

end

----------------------------------------------------------------

function StatePromoOffers:onFocus( data )

    State.onFocus(self);

    self._pulseTimer = 0;

    StateMgr:setSyncBusyDelay(0);


    local lastOfferPurchased = App.data:getValue("last_offer_purchased");

    local found = false;

    if lastOfferPurchased then
        for i = 1, #OFFER_PRODUCT_LIST do

            local oId = OFFER_PRODUCT_LIST[i].id;

            local debugStr = "StatePromoOffers:onLoad:\nCurrent key = " .. tostring(oId);
            debugStr = debugStr .. "\nlastOfferPurchased = " .. tostring(lastOfferPurchased);
            debugStr = debugStr .. "\nkey = nlastOfferPurchased -> " .. tostring((oId == lastOfferPurchased));
            debugStr = debugStr .. "\nfound = " .. tostring(found);

            print (debugStr);

            self.offerImage = OFFER_PRODUCT_LIST[i].image;
            self.offerId = oId;

            if found == true then
                break;
            elseif oId == lastOfferPurchased then
                found = true;
            end
        end
    end

    self.Frame:swapImage(self.offerImage);

print("offerImage = " .. self.offerImage);

    local title = "LET'S GO";

    local tpl = TemplateMgr:getProductById(self.offerId);
    if tpl then
        title = "$" .. tpl.cost.dollar;
    else
        print("template for offer id = nil");
    end

    self.BuyOfferText:setText('<c:fefcfc>'..title..'<c>');
    self.BuyOfferTextShdw:setText('<c:0b141d>'..title..'<c>');



end

----------------------------------------------------------------

function StatePromoOffers:onLoseFocus( )

    Base.onLoseFocus(self);

    StateMgr:setSyncBusyDelay(nil);

end

----------------------------------------------------------------

function StatePromoOffers:onUpdate( dt )

    Base.onUpdate(self, dt);

    self:_updatePulse(dt);

end


------------------------------------------------------------------------------------
-- Updates
------------------------------------------------------------------------------------


function StatePromoOffers:_updatePulse( dt )

    self._pulseTimer = self._pulseTimer + dt;

    if self._pulseTimer > PULSE_TIME then
        self._pulseTimer = 0;
    end

    local k = 1;

    if self.BuyOfferButton:isPressed() then
        k = PULSE_SCALE;
        self._pulseTimer = PULSE_TIME/2;
    elseif self._pulseTimer < PULSE_TIME/2 then
        k = math.mapClamped(0, 1, PULSE_TIME/2, PULSE_SCALE, self._pulseTimer);
    else
        k = math.mapClamped(PULSE_TIME/2, PULSE_SCALE, PULSE_TIME, 1, self._pulseTimer);
    end

    self.BuyOfferButton:setScale(k, k);
self.BuyOfferText:setScale(k, k);
self.BuyOfferTextShdw:setScale(k, k);

end


------------------------------------------------------------------------------------
-- Util
------------------------------------------------------------------------------------


function StatePromoOffers:isUsingCameraHack()
    return false;
end

----------------------------------------------------------------

function StatePromoOffers:isBusy()
    return StoreMgr:isBusy();
end


------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------


function StatePromoOffers:offerPurchased()

    --   EventMgr:postEvent( { source = "promo-new-drones",  success = true }, "promo-click-through" );
    print("Offer purchased tapped");

    if StoreMgr:canMakePayments() then

        local productId = self.offerId;
        local buySuccess = StoreMgr:buyProduct( productId,
                                                self,
                                                function()
                                                    self:onBuySuccess(productId);
                                                end,
                                                function()
                                                    print ("Sreejit. PromoOffers-OfferPurchased. StoreMgr:BuyProduct-failure callback. Prod Id: " .. productId);
                                                    self:onBuyFail(productId);
                                                end )

        if not buySuccess then
            print ("Sreejit. PromoOffers-OfferPurchased. StoreMgr:BuyProduct returned false");
            self:onBuyFail();
        end

    else
        self:showErrorMessage(LocalizationMgr:getString( "NATIVE_PURCH_DISABLED" ) );
    end
end

----------------------------------------------------------------

function StatePromoOffers:onBuySuccess( productId )

    Debug.assert( productId );

    local bGems = StoreMgr:getProductAmount(productId).gem;
    local bCoins = StoreMgr:getProductAmount(productId).gold;
    print( "StateGem:onBuySuccess", productId, bGems, bCoins );

    EventMgr:postEvent( {  gem=bGems, source="store-page", productId=productId }, "gems-bought" );

    App.data:setValue("last_offer_purchased",self.offerId);
    enableButtonInput();
    StateMgr:pop();

end

----------------------------------------------------------------

function StatePromoOffers:onBuyFail()

    print("StatePromoOffers:onBuyFail");

    enableButtonInput();

    self:showErrorMessage( LocalizationMgr:getString( "NATIVE_TRY_AGAIN" ) );

end

------------------------------------------------------------------------------------

function StatePromoOffers:showErrorMessage( message )

    MOAIDialogIOS.showDialog(  LocalizationMgr:getString( "NATIVE_TRANS_FAIL" )
                            ,  message
                            ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                            ,  nil
                            ,  nil
                            ,  false
                            ,  function()
                                    StateMgr:pop();
                               end
    );

end

----------------------------------------------------------------

function StatePromoOffers:goBack()

    EventMgr:postEvent( { source = "promo-offers",  success = false }, "promo-click-through" );
    StateMgr:pop();

end

----------------------------------------------------------------------

function StatePromoOffers:isBusy()

    return StoreMgr:isBusy();

end

------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------

return StatePromoOffers;
