----------------------------------------------------------------

require('flash/flash-manager')
require('app/cloud/cloud-manager')
require('app/cloud/june-api-manager')
require('app/contact/contact-manager')
require('app/update/update-manager')
require('state/state')
require('engine/texture/texture-manager')
require('ui/ui-button/ui-button-image')
require('ui/ui-anim')

----------------------------------------------------------------

local Base                       = State;
local TangoSignin                = Base:new();

local STATUS_NONE                = 0;
local STATUS_UPDATING            = 1;
local STATUS_SUCCESS             = 2;
local STATUS_FAILURE             = 3;

local LOADING_BAR_BACK_WIDTH     = 290;
local LOADING_BAR_BACK_HEIGHT    = 36;
local LOADING_BAR_FRONT_WIDTH    = 284;
local LOADING_BAR_FRONT_HEIGHT   = 30;
local FONT_TO_PRE_LOAD_COUNT     = 3;

local FLASH_LOAD_STATS_ENABLED = false;

if not BUILD_DEBUG then
	FLASH_LOAD_STATS_ENABLED = false;
end

----------------------------------------------------------------

function  TangoSignin:onLoad( )

   Base.onLoad( self );

   self.background = self:addWidget( Image,
                                     self.mainLayer,
                                     0,
                                     0,
                                     640,
                                     1136,
                                     "resources/UI/Splash.png" );

   self.tangoBubble = self:addWidget( Image,
                                      self.mainLayer,
                                      200,
                                      290,
                                      70,
                                      61,
                                      "resources/UI/Splash_Tango.png" );

   self.loadingBack = self:addWidget( Image,
                                      self.mainLayer,
                                      0,
                                      -App.viewportHeight/2 + 50,
                                      LOADING_BAR_BACK_WIDTH,
                                      LOADING_BAR_BACK_HEIGHT,
                                      "resources/UI/LoadingBarBack.png" );

   self.loadingFront = self:addWidget( Image,
                                       self.mainLayer,
                                       0,
                                       -App.viewportHeight/2 + 50,
                                       LOADING_BAR_FRONT_WIDTH,
                                       LOADING_BAR_FRONT_HEIGHT,
                                       "resources/UI/LoadingBarFront.png" );

   self.loadingFront:setColor( 255/255, 75/255, 25/255, 1 );

   self._bootUpAdName = "roadriot_bootup";
end


----------------------------------------------------------------

function TangoSignin:onFocus()

   -- Disable GC during loading to improve load times
   -- From here, we never restart the GC, instead opting to step it manually.
   collectgarbage("stop");

   -- Make the garbage collector more aggressive because we're running on embedded devices.
   collectgarbage("setpause", 110); -- 200 default
   collectgarbage("setstepmul", 200); -- 200 default

   MOAIFlurry:eventStart(  "App-Load", "ending", "cancel" );

   Base.onFocus( self );

   self.loadingFront:setScale(0,1);

   self._minSlowWarningTime   = 7;

   self._preRenderFontCount   = 0;

   self._waitingForCheaterDetection = false;
   self._crashForCheater            = false;

   self._timeStamp            = MOAISim.getDeviceTime();
   self._timeStamp2           = nil;
   self._loadTimeStamp        = MOAISim.getDeviceTime();
   self._loadTimeStamp2       = nil;
   self._serverTimeStamp      = MOAISim.getDeviceTime();
   self._serverTimeStamp2     = nil;
   self._fontTimeStamp        = nil;
   self._fontTimeStamp2       = nil;

   self:_setupLoader();

   StateMgr:setSyncBusyDelay(10);

   ContactMgr:activate();
   RankingMgr:activate();
   PossessionMgr:activate();
   UpdateMgr:activate();
   CloudMgr:activate();
   AdMgr:activate();
   JuneApi:activate();

   --If we already authenticated succesfully, do it now
   --We dont do it early on the first attempt to increase the chance of getting the latest cloud file
   --otherwise it is likely that we have the local version
   if App.data:getValue("game-center-authentication-success-count",0) > 0 then
      if MOAIGameCenterIOS and AbTestMgr:isGameCenterEnabled() then
         MOAIGameCenterIOS:authenticate();
      end
   end

end

----------------------------------------------------------------

function  TangoSignin:onLoseFocus( )

   Base.onLoseFocus(  self );


   App:updateMissionActivate();

   App:updateAchievementActivate();

   StateMgr:setSyncBusyDelay();

   self:_reportResource();

   self:_detectCheater3();

   --must be before we set cheater.tango.authentication.inprogress
   self:_detectCheater5();

   MOAIFlurry:eventStop(   "App-Load",
                           "ending",         "done",
                           "Cloud-Ready",    tostring(CloudMgr:isReady()),
                           "Cloud-Updating", tostring(CloudMgr:isUpdating()));

   MOAIFlurry:event("App-Load-Time" , "all",    ""..math.floor(MOAISim.getDeviceTime()-self._timeStamp)
                                    , "assets", ""..math.floor(self._loadTimeStamp2-self._loadTimeStamp)
                                    , "server", ""..math.floor(self._serverTimeStamp2-self._serverTimeStamp)
                                    , "font",   ""..math.floor(self._fontTimeStamp2-self._fontTimeStamp)        );

   App.data:deltaValue("signin-count",1);
   if LoginMgr:isTango() then
      App.data:deltaValue("tango-signin-count", 1);
   end
   App.data:setValue( "cheater.tango.authentication.inprogress", false );

   EventMgr:postEvent( {   tango=LoginMgr:isTango(),
                           loginCount        = App.data:getValue("signin-count",0),
                           tangoLoginCount   = App.data:getValue("tango-signin-count",0) } , "login-success");

   -- check and init guest manager after post of "login-success" to insure guest data updated instead of 1 frame behind due to listening of "login-success" event
   if not GuestMgr:isGuestFriendLoggedIn() then
		LeagueMgr:onLoginSuccess();
      GuestMgr:onLoginSuccess();
      RankingMgr:loadGuests();
      BountyMgr:loadBountyList();
   end

   --This is to prevent a possible exploit of deleting your app,
   -- then re-install to get easy ghost score to authenticate again
   -- (get back your good car with easy ghost)
   if LoginMgr:isTango() and (not App.data:getValue("TangoSignin.tango-login-reset")) then
      App.data:setValue("TangoSignin.tango-login-reset", true);
      GuestMgr:resetGuestFriend();
      RankingMgr:loadGuests();
      GhostMgr:submitScore();
   end

   -- Checking for the weekly reset timestamp default value
   -- and initializing it to the proper time.
   -- Called here to make sure that the TimeMgr has initialized properly.
   if App.data:getValue("weekly-reset-timestamp", 0) == 0 then
      App.data:giveWeeklyReward(0); -- Pass in 0 to update the time, but dont give rewards and dont post analytics
   end

   if MOAIGameCenterIOS and AbTestMgr:isGameCenterEnabled() then
      --We might already authenticated, but that's ok, MOAIGameCenterIOS can handle that
      MOAIGameCenterIOS:authenticate();
   end

   -- Clean up all the garbage created by the loading process.
   MOAISim.forceGarbageCollection();

end

----------------------------------------------------------------

function  TangoSignin:_nextState()

   self._waitingForCheaterDetection = true;

   FraudMgr:detectFraud();

   delay( 0.1, function()

                  if FraudMgr:shouldForceCrash() then
                     StateMgr:clear( "state/state-data-crash.lua" );
                  else
                     StateMgr:clear( "state/state-ranking.lua" );
                  end

               end            );


end

----------------------------------------------------------------

function  TangoSignin:onUpdate( dt )

   Base.onUpdate( self, dt );

   if not self._waitingForCheaterDetection then

      self:_updateLoadingBar();

      --If all servers task are done
      if       ( OnlineMgr:isReady()
         and   TimeMgr:isReady()
         and   ContactMgr:isReady()
         and   RankingMgr:isReady()
         and   PossessionMgr:isReady() ) then

         self._serverTimeStamp2 = self._serverTimeStamp2 or MOAISim.getDeviceTime();

         --Also all loader tasks are done
         if       LoaderMgr:getTaskCount()   ==    0
            and   self._preRenderFontCount   >     FONT_TO_PRE_LOAD_COUNT     then

             self:_nextState();

         end

      else

         if math.floor(MOAISim.getDeviceTime()-self._serverTimeStamp) >= self._minSlowWarningTime then
            self._minSlowWarningTime = self._minSlowWarningTime + 1;
            MOAIFlurry:event( "Debug:Slow-Signin-Server",
                              "Time",                       tostring(math.floor(MOAISim.getDeviceTime()-self._serverTimeStamp)),
                              "Tango",                      tostring(LoginMgr:isTango()),
                              "OnlineMgr:isReady()",        tostring(OnlineMgr:isReady()),
                              "TimeMgr:isReady()",          tostring(TimeMgr:isReady()),
                              "ContactMgr:isReady()",       tostring(ContactMgr:isReady()),
                              "RankingMgr:isReady()",       tostring(RankingMgr:isReady()),
                              "PossessionMgr:isReady()",    tostring(PossessionMgr:isReady()));
         end

      end
   end
end

----------------------------------------------------------------

function  TangoSignin:_updateLoadingBar()

   local todo  = self.toLoad+FONT_TO_PRE_LOAD_COUNT;
   local done  = (self.toLoad-LoaderMgr:getTaskCount()+self._preRenderFontCount);
   local ratio = math.min(done/todo,1);

   self.loadingFront:setScale(ratio,1);
   self.loadingFront:setLocX( -LOADING_BAR_FRONT_WIDTH/2*(1-ratio));

end

----------------------------------------------------------------

--??Should we go back to splash screen??
function TangoSignin:onDataServerError()
   StateMgr:clearDataServerError();
end

----------------------------------------------------------------------

function TangoSignin:onNeedDataResolution()
   -- We shouldn't do anything here
end

----------------------------------------------------------------------

function TangoSignin:isUsingCameraHack()
   return false;
end

----------------------------------------------------------------------

function TangoSignin:isBusy()

   --Dont show the spining wheel while we are loading
   --dont include the preload font in this because
   -- it could be stuck waiting for contacts
   if LoaderMgr:getTaskCount()   ==    0  then
      if       ( not OnlineMgr:isReady()     )
         or    ( not TimeMgr:isReady()    )
         or    ( not ContactMgr:isReady()    )
         or    ( not PossessionMgr:isReady() )
         or    ( not RankingMgr:isReady()    )      then

         return true;

      end
   end
end


----------------------------------------------------------------

function  TangoSignin:_setupLoader( )

   if FLASH_LOAD_STATS_ENABLED then
      LoaderMgr:addTask( "LoadSpritesheetBegin",      self, function () ProfilerMgr:beginTimestamp("SpritesheetLoading") end );
   end

   LoaderMgr:addTask( "MainScreen.png",            self, function () TextureMgr:getTexture("resources/flash-resource-bundle/MainScreen.png"); end );
   LoaderMgr:addTask( "Group2.png",                self, function () TextureMgr:getTexture("resources/flash-resource-bundle/Group2.png"); end );
   LoaderMgr:addTask( "Group3.png",                self, function () TextureMgr:getTexture("resources/flash-resource-bundle/Group3.png"); end );
   LoaderMgr:addTask( "PopUPs.png",                self, function () TextureMgr:getTexture("resources/flash-resource-bundle/PopUPs.png"); end );

   if FLASH_LOAD_STATS_ENABLED then
      LoaderMgr:addTask( "LoadSpritesheetEnd",     self, function () ProfilerMgr:endTimestamp("SpritesheetLoading") end );

      LoaderMgr:addTask( "LoadFlashBegin",         self, function () ProfilerMgr:beginTimestamp("FlashLoading");
                                                            for i = 1, 2 do collectgarbage("collect"); end
                                                            collectgarbage("stop");
                                                            ProfilerMgr:beginMemSnapshot("FlashLoading");
                                                         end );
   end

   LoaderMgr:addTask( "MainScreen.lua",            self, function () FlashMgr:getTextureSheet("resources/flash-resource-bundle/MainScreen.lua", "resources/flash-resource-bundle/MainScreen.png", 30); end );
   LoaderMgr:addTask( "Group2.lua",                self, function () FlashMgr:getTextureSheet("resources/flash-resource-bundle/Group2.lua",     "resources/flash-resource-bundle/Group2.png", 30); end );
   LoaderMgr:addTask( "Group3.lua",                self, function () FlashMgr:getTextureSheet("resources/flash-resource-bundle/Group3.lua",     "resources/flash-resource-bundle/Group3.png", 30); end );
   LoaderMgr:addTask( "PopUPs.lua",                self, function () FlashMgr:getTextureSheet("resources/flash-resource-bundle/PopUPs.lua",     "resources/flash-resource-bundle/PopUPs.png", 30); end );

   if FLASH_LOAD_STATS_ENABLED then
      LoaderMgr:addTask( "LoadFlashEnd",           self, function () ProfilerMgr:endTimestamp("FlashLoading");
                                                            for i = 1, 2 do collectgarbage("collect"); end
                                                            --collectgarbage("restart");
                                                            ProfilerMgr:endMemSnapshot("FlashLoading")
                                                         end );
   end

   LoaderMgr:addTask( "Cache Ads",                 self, function () if self._bootUpAdName then AdMgr:cacheInterstitial(self._bootUpAdName); end end );

   LoaderMgr:addTask( "state-ranking.lua",         self, function () StateMgr:loadState (  "state/state-ranking.lua" ) end );

   LoaderMgr:addTask( "state-daily.lua",           self, function () StateMgr:loadState ( "state/state-daily.lua" ) end );
   LoaderMgr:addTask( "state-game.lua",            self, function () StateMgr:loadState (  "state/state-game.lua" ) end );
   LoaderMgr:addTask( "state-garage.lua",          self, function () StateMgr:loadState ( "state/state-garage.lua" ) end );

   --Show tower then end each task is faster (otherwise, the end might be unresponsive)
   LoaderMgr:addTask( "Show Ads",                 self, function () PromoMgr:attemptInterstitialDisplay(self._bootUpAdName); end );

   LoaderMgr:addTask( "state-mission-popup.lua",   self, function () StateMgr:loadState ( "state/state-mission-popup.lua" ) end );
   LoaderMgr:addTask( "state-options.lua",         self, function () StateMgr:loadState ( "state/state-options.lua" ) end );
   LoaderMgr:addTask( "state-invite.lua",          self, function () StateMgr:loadState ( "state/state-invite.lua" ) end );
   LoaderMgr:addTask( "state-pause.lua",           self, function () StateMgr:loadState ( "state/state-pause.lua" ) end );

   LoaderMgr:addTask( "state-results.lua",         self, function () StateMgr:loadState ( "state/state-results.lua" ) end );
   LoaderMgr:addTask( "state-levelup-popup.lua",   self, function () StateMgr:loadState ( "state/state-levelup-popup.lua" ) end );
   LoaderMgr:addTask( "state-game.lua",            self, function () StateMgr:loadState ( "state/state-game.lua" ) end );
   LoaderMgr:addTask( "state-gem.lua",             self, function () StateMgr:loadState ( "state/state-gem.lua" ) end );
   LoaderMgr:addTask( "state-coin.lua",            self, function () StateMgr:loadState ( "state/state-coin.lua" ) end );
   LoaderMgr:addTask( "state-key.lua",             self, function () StateMgr:loadState ( "state/state-key.lua" ) end );

   --Make sure it is at the end of loading assets, otherwise our profiling will be wrong
   LoaderMgr:addTask( "Done",                      self, function () self._loadTimeStamp2 = self._loadTimeStamp2 or MOAISim.getDeviceTime(); end );

   LoaderMgr:addTask( "Tango.otf-16",              self, function () TangoSignin:_prerenderUTFFont( "resources/font/Tango.otf", 16 ) end );

   self.toLoad = LoaderMgr:getTaskCount();

end

----------------------------------------------------------------

function TangoSignin:_createPreLoadFontString()

   local namelist = { "FriendYyou1234567890 keys!" };

   --First step it to concatenate all charater in 1 big table

   for i=1,ContactMgr:getContactCount() do

      local contact = ContactMgr:getContact(i);
      table.insert(namelist, contact:getFirstName());
      table.insert(namelist, contact:getLastName());

   end

   return table.concat(namelist);

end

----------------------------------------------------------------

function TangoSignin:_splitPreLoadFontString( text )

   local tbl         = {};
   local splitSize   = math.floor(#text/FONT_TO_PRE_LOAD_COUNT);

   --If this is a small string, dont split
   if splitSize <= 1 then

      table.insert(tbl, text);

   else

      local sub1       = 1;
      local sub2       = sub1+splitSize;

      for i=1,(FONT_TO_PRE_LOAD_COUNT-1) do

         table.insert(tbl, text:sub(sub1,sub2));

         sub1    = sub2;
         sub2    = sub1 + splitSize;

      end

      table.insert(tbl, text:sub(sub1));

   end

   return tbl;

end

----------------------------------------------------------------

function TangoSignin:_prerenderUTFFont( font_name, font_size )

   local metaFont = FontMgr:getMetaFont( font_name, font_size,  "" );

   --This is necessary to preload
   local tb  = MOAITextBox.new ();
   tb:setFont ( metaFont.font );
   tb:setRect (  -200,  -200, 200, 200 );
   tb:setLoc  ( -2500, -2500 );

   local thread = MOAICoroutine.new();

	thread:run(

      --Wait to have the list of friends and your own name
		function ()
         while ( not ContactMgr:isReady() ) do
            coroutine.yield ();
         end

         self._fontTimeStamp  = self._fontTimeStamp or MOAISim.getDeviceTime();

         --Create the list

         local text = self:_createPreLoadFontString();

         coroutine.yield ();

         self._preLoadStringTable = self:_splitPreLoadFontString( text );

         for i,s in ipairs(self._preLoadStringTable) do

            coroutine.yield ();

            --We need to use the font to be create the glyphes

            metaFont.font:preloadGlyphs( s, font_size );

            coroutine.yield ();

            tb:setString ( s );

            self._preRenderFontCount = self._preRenderFontCount + 1;

         end

         --To make sure we are considered done (in case there is not many table to parse in self._preLoadStringTable )
         self._preRenderFontCount = FONT_TO_PRE_LOAD_COUNT+1;

         self._fontTimeStamp2  = MOAISim.getDeviceTime();

      end   );

end

----------------------------------------------------------------

function  TangoSignin:_showNoBookSaveDialog( callObj, callFunc )

   MOAIDialogIOS.showDialog(  LocalizationMgr:getString( "NATIVE_NO_CONTACTS_TITLE" )
                           ,  LocalizationMgr:getString( "NATIVE_NO_CONTACTS_MSG" )
                           ,  LocalizationMgr:getString( "NATIVE_NO_CONTACTS_BTN" )
                           ,  nil
                           ,  nil
                           ,  true
                           ,  function( response )
                                 if callFunc then
                                    callFunc(callObj);
                                 end
                                 if response == MOAIDialogIOS.DIALOG_RESULT_POSITIVE then
                                    MOAISafariIOS.openURL("tango://book");

                                    self._autoContactsTime = 10000;

                                 end
                              end
                           );

end

----------------------------------------------------------------

function  TangoSignin:_showNoBookAccessDialog( callObj, callFunc )

print("_showNoBookAccessDialog");

   MOAIDialogIOS.showDialog(  LocalizationMgr:getString( "NATIVE_CONTACTS_UNAVAL_TITLE" )
                           ,  LocalizationMgr:getString( "NATIVE_CONTACTS_UNAVAL_MSG" )
                           ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                           ,  nil
                           ,  nil
                           ,  false
                           ,  function( response )
                                 if callFunc then
                                    callFunc(callObj);
                                 end
                                 if response == MOAIDialogIOS.DIALOG_RESULT_POSITIVE then
                                   -- MOAISafariIOS.openURL("tango://book");
                                   -- self._autoContactsTime = 10000;

                                 end
                              end
                           );

end


----------------------------------------------------------------

function TangoSignin:_reportResource()

   local gem         = App.data:getGemCount();
   local gemLog10    = math.pow(10,math.floor(math.log10(math.max(gem,1))));
   local gem10       = math.floor(gem/10);
   local gem100      = math.floor(gem/100);
   local gold        = App.data:getGoldCount();
   local gold100     = math.floor(gold/100);
   local gold1000    = math.floor(gold/1000);
   local goldLog10   = math.pow(10,math.floor(math.log10(math.max(gold,1))));
   local key         = App.data:getKeyCount();
   local key10       = math.floor(key/10);
   local key100      = math.floor(key/100);
   local keyLog10    = math.pow(10,math.floor(math.log10(math.max(key,1))));

   MOAIFlurry:event( "Report-Resource",
                     "gem/10",            tostring(gem10),
                     "gem/100",           tostring(gem100),
                     "gem-log10",         tostring(gemLog10),
                     "gold/100",          tostring(gold100),
                     "gold/1000",         tostring(gold1000),
                     "gold-log10",        tostring(goldLog10),
                     "key",               tostring(key),
                     "key/10",            tostring(key10),
                     "key/100",           tostring(key100),
                     "key-log10",         tostring(keyLog10)     );

   if not App.data:isThief() then

      MOAIFlurry:event( "Report-Resource-Honnest",
                        "gem/10",            tostring(gem10),
                        "gem/100",           tostring(gem100),
                        "gem-log10",         tostring(gemLog10),
                        "gold/100",          tostring(gold100),
                        "gold/1000",         tostring(gold1000),
                        "gold-log10",        tostring(goldLog10),
                        "key",               tostring(key),
                        "key/10",            tostring(key10),
                        "key/100",           tostring(key100),
                        "key-log10",         tostring(keyLog10)     );
   end

end

----------------------------------------------------------------

function TangoSignin:_detectCheater3()

   local gemLog10    = math.floor(math.log10(math.max(App.data:getGemCount(),1)));
   local goldLog10   = math.floor(math.log10(math.max(App.data:getGoldCount(),1)));
   local iapFake     = App.data:getValue("iap-fake",     0 );
   local iapReal     = App.data:getValue("iap-verified", 0 );
   local tangoUser   = LoginMgr:isTango();

   local thief       = false;

   --If the user has more fake than real
   if iapFake > iapReal then
      thief = true;
   else
      -- if the user is a tango user,
      -- we could report his valid IAP == we can be more strict
      if LoginMgr:isTango() then
         --If the user has more than 10,000 Gems, without IAP
         if gemLog10 >= 4 and iapReal == 0 then
            thief = true;
         end
      else
         --If the user has more than 100,000 Gems, without IAP
         if gemLog10 >= 5	 and iapReal == 0 then
            thief = true;
         end
      end
   end

   if thief then

      MOAIAdjust:trackEvent(App.adjustMap["Thief"],         "Version",     "3"      );

      local oldThief = App.data:getValue( "thief-3" );

      if not oldThief then

         MOAIFlurry:event( "Thief-3-New",
                           "tango",             tostring(LoginMgr:isTango()),
                           "gem-log10",         tostring(gemLog10),
                           "gold-log10",        tostring(goldLog10),
                           "iap-fake",          tostring(iapFake),
                           "iap-verified",      tostring(iapReal),
                           "tangoUser",         tostring(tangoUser)                 );
      end

      App.data:setValue("thief-3", true);

      MOAIFlurry:event( "Thief-3",
                        "tango",             tostring(LoginMgr:isTango()),
                        "gem-log10",         tostring(gemLog10),
                        "gold-log10",        tostring(goldLog10),
                        "iap-fake",          tostring(iapFake),
                        "iap-verified",      tostring(iapReal),
                        "tangoUser",         tostring(tangoUser)                 );

   end

   local deviceMash3 =  tostring(MOAIEnvironment.osBrand)
                     .. tostring(MOAIEnvironment.devBrand)
                     .. tostring(MOAIEnvironment.devName)
                     .. tostring(MOAIEnvironment.devManufacturer)
                     .. tostring(MOAIEnvironment.devModel)
                     .. tostring(MOAIEnvironment.devProduct);
                     --num processor doesnt alwaus return the same value
                     --.. tostring(MOAIEnvironment.numProcessors);

   if not App.data:getValue( "_deviceMash3" ) then
      App.data:setValue( "_deviceMash3",deviceMash3 );
   end

   if App.data:getValue( "_deviceMash3" ) ~= deviceMash3 then
      print("Will crash", App.data:getValue( "_deviceMash3" )
                        , deviceMash3
                        , tostring(MOAIEnvironment.osBrand)
                        , tostring(MOAIEnvironment.devBrand)
                        , tostring(MOAIEnvironment.devName)
                        , tostring(MOAIEnvironment.devManufacturer)
                        , tostring(MOAIEnvironment.devModel)
                        , tostring(MOAIEnvironment.devProduct)
                        , tostring(MOAIEnvironment.numProcessors) );

      MOAIFlurry:event( "Device-ID-3-BAD",
                        "deviceMash",        tostring(deviceMash3) .. "  !=  " .. tostring(App.data:getValue( "_deviceMash3" ))  );


   end

   --Debug.assert( App.data:getValue( "_deviceMash3" )==deviceMash3, "TangoSignin:_detectCheater3 -> deviceMash3" );

   MOAIFlurry:event( "Device-ID-3",
                     "deviceMash3",     tostring(deviceMash3),
                     "deviceMash3-OLD",  tostring(App.data:getValue( "_deviceMash3" )),
                     "deviceMash3-Same",  tostring(App.data:getValue( "_deviceMash3" )==deviceMash3)   );


   MOAIFlurry:event( "Thief-Detection-2",
                     "thief",             tostring(thief),
                     "gem-log10",         tostring(gemLog10),
                     "gold-log10",        tostring(goldLog10),
                     "iap-fake",          tostring(iapFake),
                     "iap-verified",      tostring(iapReal),
                     "tangoUser",         tostring(tangoUser)            );

end

----------------------------------------------------------------

function TangoSignin:_detectCheater5()

   if LoginMgr:isTango() then

      if ContactMgr and ContactMgr:getProfile() and ContactMgr:getProfile() then

         local accountId  = ContactMgr:getProfile():getAccountId();
         local accountId2 = App.data:getValue("cheater.tango.account.id");

         if accountId ~= accountId2 then

            local inProgress = App.data:getValue( "cheater.tango.authentication.inprogress" );

            if not inProgress then

            end

         end

      else

      end
   end

   App.data:setValue("cheater.tango.account.id", accountId);

end

----------------------------------------------------------------

--[[
   if    code == MOAITangoSdk.TANGO_SDK_ADDRESS_BOOK_NOT_READY
      or code == MOAITangoSdk.TANGO_SDK_ADDRESS_BOOK_SAVE_DENIED then

      if self._contactRetrieveCallback and self._contactRetrieveCallback.successFunc then
         self:_showNoBookSaveDialog( self._contactRetrieveCallback.callObj, self._contactRetrieveCallback.successFunc);
      else
         self:_showNoBookSaveDialog();
      end

   elseif code == MOAITangoSdk.TANGO_SDK_ADDRESS_BOOK_ACCESS_DENIED then

      if self._contactRetrieveCallback and self._contactRetrieveCallback.successFunc then
         self:_showNoBookAccessDialog( self._contactRetrieveCallback.callObj, self._contactRetrieveCallback.successFunc);
      else
         self:_showNoBookAccessDialog();
      end

   else

      if self._contactRetrieveCallback and self._contactRetrieveCallback.failureFunc then
         self._contactRetrieveCallback.failureFunc(self._contactRetrieveCallback.callObj, code, message);
      end
   end
   ]]


----------------------------------------------------------------

return TangoSignin
