
----------------------------------------------------------------
require('state/state')
require('ui/ui-button/ui-button-image')
require('ui/ui-image')
require('ui/ui-text-box')
require('ui/ui-anim')
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');

----------------------------------------------------------------

--On Android, if we do not wait long enough we will see a black screen
-- from time to time
--It will also give our online manager a little more time to test connectivity
local SPLASH_MIN_TIME   = 5.0;

----------------------------------------------------------------

local Base              = State;
local Splash				= Base:new();

----------------------------------------------------------------

function  Splash:onLoad( )

   Base.onLoad( self );

   self.background      = self:addWidget( Image,
                                          self.mainLayer,
                                          0,
                                          0,
                                          640,
                                          1136,
                                          "resources/UI/SplashJuneTitan.png" );

   -- self.tangoBubble      = self:addWidget(   Image,
   --                                           self.mainLayer,
   --                                           200,
   --                                           290,
   --                                           70,
   --                                           61,
   --                                           "resources/UI/Splash_Tango.png" );


end

----------------------------------------------------------------

function  Splash:onActivate( )

   Base.onActivate( self );

   --It is possible to be send here from a state (i,e, when connecting to Tango)

   if ContactMgr:isActive() then
      ContactMgr:deactivate();
   end

   if RankingMgr:isActive() then
      RankingMgr:deactivate();
   end

   if PossessionMgr:isActive() then
      PossessionMgr:deactivate();
   end

   if CloudMgr:isActive() then
      CloudMgr:deactivate();
   end

   if AdMgr:isActive() then
      AdMgr:deactivate();
   end

   if App.mission then
      App.mission:deactivate();
   end

   ---------------------------------------------
   ---------------------------------------------

   --The order is important

   TimeMgr:activate();

   --Dont force re-activation.
   --If it is already activated it means we are authentication
   --and in this case we dont want to suddenly be consider offline
   OnlineMgr:activate();

   LoginMgr:activate( true );


end

----------------------------------------------------------------

function Splash:onFocus( )

   Base.onFocus( self );

   MOAIGfxDevice.setClearColor( 0, 0, 0 );

   self:disableAds();

end

----------------------------------------------------------------

function Splash:onDataServerError()
   StateMgr:clearDataServerError();
end

----------------------------------------------------------------

function Splash:onNeedDataResolution()
   -- We shouldn't do anything here
end

----------------------------------------------------------------------

function Splash:isUsingCameraHack()
   return false;
end

----------------------------------------------------------------

function  Splash:onUpdate( dt )

   Base.onUpdate( self, dt );

	if ( MOAISim.getDeviceTime () - self.startTime ) > SPLASH_MIN_TIME then

      if LoginMgr:isTango()  then

         MOAIAdjust:trackEvent(App.adjustMap["Tango-Login"] );


         MOAIFlurry:event( "Tango-Login" );

         if Splash.tryAuthenticate then
            MOAIFlurry:eventStop( "TangoForcedAuthentication", "ending", "success", "time", tostring(os.time()-(Splash.autoAuthenticateTime or 0)) );
         end

			StateMgr:swap ( "state/state-signin-tango.lua" );


      else

         Debug.assert( LoginMgr:isGuest(), "Splash:onUpdate -> LoginMgr:isGuest()" );

         if self:_shouldAutoAuthenticate() then

            Splash.tryAuthenticate        = true;
            Splash.autoAuthenticateTime   = os.time();

            MOAIFlurry:eventStart( "TangoForcedAuthentication", "ending", "failure" );

            StateMgr:push("state/state-authenticate-confirm.lua");

         else
            MOAIFlurry:event( "Guest-Login" );

            StateMgr:swap ( "state/state-signin-tango.lua" );
         end
		end
	end
end

----------------------------------------------------------------

function  Splash:_shouldAutoAuthenticate()

   --Only force authentication of Android user who have not play too much already

--[[
   return MOAIEnvironment.osBrand == "Android"
      and MOAITangoSdk:tangoHasSdkSupport()
      and (not Splash.tryAuthenticate)
      and OnlineMgr:isOnline()
      and (App.data:getValue("launch-count") or 0) <= 3;
   ]]

   return false;

end
----------------------------------------------------------------

return Splash
