----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local SQUARE_TEXTURE = "resources/square.png";

local LOADING_BAR_BACK_WIDTH   = 290;
local LOADING_BAR_BACK_HEIGHT  = 36;
local LOADING_BAR_FRONT_WIDTH  = 284;
local LOADING_BAR_FRONT_HEIGHT = 30;

local PULSE_TIME  = 0.5;
local PULSE_SCALE = 1.1;

local PRIORITY_0  = 10;
local PRIORITY_1  = 20;
local PRIORITY_2  = 30;
local PRIORITY_3  = 40;
local PRIORITY_4  = 50;
local PRIORITY_5  = 60;
local PRIORITY_6  = 70;
local PRIORITY_7  = 80;
local PRIORITY_8  = 90;
local PRIORITY_9  = 100;
local PRIORITY_10 = 110;

----------------------------------------------------------------

local Base               = State;
local StateStarterBundle = Base:new();

----------------------------------------------------------------

function StateStarterBundle:onLoad()

   State.onLoad( self );

   self.defaultFontName	= "resources/font/TangoLatin.otf";
   self.IS_POPUP = true;

   -- background
   self.Background = self:addWidget(Image, self.mainLayer, 0,0, 1400, 1200, SQUARE_TEXTURE);
   self.Background:setPriority(PRIORITY_0);
   self.Background:setColor(0,0,0, 0.75);

   -- frame
   self.Frame = self:addWidget(Image, self.mainLayer, 0, 0, 512*1.2, 633*1.2, "resources/UI/StartBundle/RR_StartBundle_Frame.png");
   self.Frame:setPriority(PRIORITY_1);

   -- gem/gold text
   self.GemText      = self:addWidget(TextBox, self.mainLayer, -120, -140, 640, 300, "Gem", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.GemTextShdw  = self:addWidget(TextBox, self.mainLayer, -120+2, -140-2, 640, 300, "Gem", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
   self.CoinText     = self:addWidget(TextBox, self.mainLayer, 110, -140-2, 640, 300, "Coin", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.CoinTextShdw = self:addWidget(TextBox, self.mainLayer, 110+2, -140-2, 640, 300, "Coin", self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
   self.GemText:setPriority(PRIORITY_2);
   self.GemTextShdw:setPriority(PRIORITY_2-1);
   self.CoinText:setPriority(PRIORITY_2);
   self.CoinTextShdw:setPriority(PRIORITY_2-1);

   -- buttons
   self.BuyButton    = self:addWidget(ButtonImage, self.mainLayer, 100, -240, 248, 128, "resources/UI/StartBundle/RR_StartBundle_Button_Green.png", nil, self.goPurchase);
   self.ExitButton   = self:addWidget(ButtonImage, self.mainLayer, -140, -240, 166, 72, "resources/UI/StartBundle/RR_StartBundle_Button_Red.png", nil, self.goBack);
   self.CostText     = self:addWidget(TextBox, self.mainLayer, 0, -32, 248, 128, "Cost", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
   self.CostTextShdw = self:addWidget(TextBox, self.mainLayer, 0+2, -32-2, 248, 128, "Cost", self.defaultFontName, 16, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
   self.BuyButton:setPriority(PRIORITY_2);
   self.ExitButton:setPriority(PRIORITY_2);
   self.CostText:setPriority(PRIORITY_3);
   self.CostTextShdw:setPriority(PRIORITY_3-1);
   self.CostText:setParent(self.BuyButton.prop);
   self.CostTextShdw:setParent(self.BuyButton.prop);

end

----------------------------------------------------------------

function StateStarterBundle:onActivate()

   State.onActivate(self);

end

----------------------------------------------------------------

function StateStarterBundle:onFocus( data )

   State.onFocus(self);

   self:_updateText();

   self._pulseTimer = 0;

   StateMgr:setSyncBusyDelay(0);

end

----------------------------------------------------------------

function StateStarterBundle:onLoseFocus( )

   Base.onLoseFocus(self);

   StateMgr:setSyncBusyDelay(nil);

end

----------------------------------------------------------------

function StateStarterBundle:onUpdate( dt )

   Base.onUpdate(self, dt);

   self:_updatePulse(dt);

end


------------------------------------------------------------------------------------
-- Updates
------------------------------------------------------------------------------------


function StateStarterBundle:_updateText()

   local id     = StoreMgr:getProductIdFromName("starter");
   local amount = StoreMgr:getProductAmount(id);
   local cost   = StoreMgr:getProductCostDisplay(id);

   self.GemText:setText(tostring(amount.gem));
   self.GemTextShdw:setText(tostring(amount.gem));
   self.CoinText:setText(tostring(amount.gold));
   self.CoinTextShdw:setText(tostring(amount.gold));
   self.CostText:setText(cost);
   self.CostTextShdw:setText(cost);

end

----------------------------------------------------------------

function StateStarterBundle:_updatePulse( dt )

   self._pulseTimer = self._pulseTimer + dt;

   if self._pulseTimer > PULSE_TIME then
      self._pulseTimer = 0;
   end

   local k = 1;

   if self.BuyButton:isPressed() then
      k = PULSE_SCALE;
      self._pulseTimer = PULSE_TIME/2;
   elseif self._pulseTimer < PULSE_TIME/2 then
      k = math.mapClamped(0, 1, PULSE_TIME/2, PULSE_SCALE, self._pulseTimer);
   else
      k = math.mapClamped(PULSE_TIME/2, PULSE_SCALE, PULSE_TIME, 1, self._pulseTimer);
   end

   self.BuyButton:setScale(k, k);

end


------------------------------------------------------------------------------------
-- Util
------------------------------------------------------------------------------------


function StateStarterBundle:isUsingCameraHack()
   return false;
end


------------------------------------------------------------------------------------
-- Purchase
------------------------------------------------------------------------------------


function StateStarterBundle:showErrorMessage( message )

   MOAIDialogIOS.showDialog(	LocalizationMgr:getString( "NATIVE_TRANS_FAIL" )
                              , message
                              ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                              ,  nil
                              ,  nil
                              ,  false
                              ,  function()
                                 end
                           );

end

----------------------------------------------------------------

function StateStarterBundle:goPurchase()

   if StoreMgr:canMakePayments() then

      disableButtonInput();

      local productId = StoreMgr:getProductIdFromName("starter");

      print("StateStarterBundle:goPurchase, productId -> ", productId)

      local buySuccess = StoreMgr:buyProduct( productId,
                                              self,
                                              function()
                                                 self:onBuySuccess(productId);
                                              end,
                                              function()
                                                 self:onBuyFail(productId);
                                              end )

      if not buySuccess then
         self:onBuyFail();
      end

   else

      self:showErrorMessage(LocalizationMgr:getString( "NATIVE_PURCH_DISABLED" ) );

   end

end

----------------------------------------------------------------

function StateStarterBundle:onBuySuccess( productId )

   Debug.assert( productId );

   local amount = StoreMgr:getProductAmount(productId);

   EventMgr:postEvent( { gem = amount.gem or 0,
                         source = "head-start-popup",
                         productId = productId },
      "starterpack-bought" );

   enableButtonInput();
   self:goBack();

end

----------------------------------------------------------------

function StateStarterBundle:onBuyFail()

   enableButtonInput();

   self:showErrorMessage( LocalizationMgr:getString( "NATIVE_TRY_AGAIN" ) );

end

----------------------------------------------------------------

function StateStarterBundle:isBusy()

   return StoreMgr:isBusy();
end

------------------------------------------------------------------------------------
-- Exit State
------------------------------------------------------------------------------------

function StateStarterBundle:goBack()
   StateMgr:pop();
end

------------------------------------------------------------------------------------
-- End State
------------------------------------------------------------------------------------

return StateStarterBundle;
