----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local PRIORTY_0				= 10;
local PRIORTY_1				= 20;
local PRIORTY_2				= 30;
local PRIORTY_3				= 40;
local PRIORTY_4				= 50;
local PRIORTY_5				= 60;
local PRIORTY_6				= 70;
local PRIORTY_7				= 80;
local PRIORTY_8				= 90;
local PRIORTY_9				= 100;
local PRIORTY_10				= 110;

----------------------------------------------------------------

local Base								= State;
local StateTangoBonusReminder		= Base:new();

local Bonus_Icon_X			= 0;
local Bonus_Icon_Y			= -130;

----------------------------------------------------------------

function StateTangoBonusReminder:onLoad()

   State.onLoad( self );

   local textureSheet			= FlashMgr:getTextureSheet("resources/flash-resource-bundle/MainScreen.lua", "resources/flash-resource-bundle/MainScreen.png", 30);
   local popupSheet				= FlashMgr:getTextureSheet("resources/flash-resource-bundle/PopUPs.lua", "resources/flash-resource-bundle/PopUPs.png", 30);

   self.defaultFontName = "resources/font/TangoLatin.otf";
	self.IS_POPUP  = true;

	-- background
	self.BackDrop = self:addWidget(Anim, self.mainLayer, 0, 10, 0,0, "LargeBackGround", popupSheet);
	
	self.TangoBonusIcon = self:addWidget(Anim, self.mainLayer, Bonus_Icon_X, Bonus_Icon_Y, 0,0, "ReceiveGem", textureSheet, nil, nil, true);
	self.TangoBonusIcon:setScale(1.4);

	-- push/pop state buttons
	self.ConfirmButton = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0, 5, 0,0, "LargeOkButton", popupSheet, nil, self.goConfirm, nil, nil);
	self.ConfirmButton:setPriority(PRIORTY_2);
	
	-- push/pop state buttons
	self.CloseButton = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 265, 161, 0,0, "CloseButton", popupSheet, nil, self.goBack, nil, nil);
	self.CloseButton:setPriority(PRIORTY_2);
	
	-- text message
	self.TitleText = self:addWidget(TextBox, self.mainLayer, 0, 132, 540, 70, '<c:fefcfc>Tango Gift<c>', self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.TitleText:setScale(1.5, 1.5);
	self.TitleText:setPriority(PRIORTY_0);
	self.TitleTextShdw = self:addWidget(TextBox, self.mainLayer, 0+2, 132-2, 540, 70, '<c:0b141d>Tango Gift<c>', self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY, {0,0,0,1});
	self.TitleTextShdw:setScale(1.5, 1.5);
	self.TitleTextShdw:setPriority(PRIORTY_0-1);
	
	self.MessageText = self:addWidget(TextBox, self.mainLayer, 0, -10, 500, 230, '', self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.MessageText:setPriority(PRIORTY_1);
	self.MessageText.moaiTextBox:setLineSpacing(10);
	self.MessageText:setScale(1.2, 1.2);
	
	self.MessageTextShadow = self:addWidget(TextBox, self.mainLayer, 0-2, -10+2, 500, 230, '', self.defaultFontName, 18, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.MessageTextShadow:setPriority(PRIORTY_0);
	self.MessageTextShadow.moaiTextBox:setLineSpacing(10);
	self.MessageTextShadow:setScale(1.2, 1.2);
	
	self.BonusText = self:addWidget(TextBox, self.mainLayer, 0,0, 520, 230, '', self.defaultFontName, 28, "0123456789", MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.BonusText:setPriority(PRIORTY_1);
	self.BonusText:setScale(1.2, 1.2);
	
	self.BonusTextShadow = self:addWidget(TextBox, self.mainLayer, 0,0, 520, 230, '', self.defaultFontName, 28, "0123456789", MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
	self.BonusTextShadow:setPriority(PRIORTY_0);
	self.BonusTextShadow:setScale(1.2, 1.2);

end

----------------------------------------------------------------

function StateTangoBonusReminder:onActivate()

   State.onActivate(self);

   EventMgr:postEvent({style="old-style"}, "tango-request-popup");

end

----------------------------------------------------------------

function StateTangoBonusReminder:onFocus()
   
   State.onFocus(self);
   
	self:setMessage();
	
end

----------------------------------------------------------------

function StateTangoBonusReminder:isUsingCameraHack()
   return false;
end


----------------------------------------------------------------

function StateTangoBonusReminder:setMessage()
	
	local message		= 'Play with your Tango\nfriends and get';
	local iconOffset	= 0;
	local bonusOffset	= 0;
	local bonus       = App.data:getTangoBonus();

	self.MessageText:setText('<c:1b3043>'..message..'<c>');
	self.MessageTextShadow:setText('<c:8fb5cd>'..message..'<c>');
	self.BonusText:setText('<c:fefcfc> '..bonus..'<c>');
	self.BonusTextShadow:setText('<c:0b141d> '..bonus..'<c>');
	
	if bonus > 999 then
		iconOffset = -70;
		bonusOffset = 50;
	elseif bonus > 99 then
		iconOffset = -62;
		bonusOffset = 42;
	elseif bonus > 9 then
		iconOffset = -47;
		bonusOffset = 27;
	else
		iconOffset = -40;
		bonusOffset = 20;
	end

	self.TangoBonusIcon:setLoc(iconOffset, Bonus_Icon_Y);
	self.BonusText:setLoc(bonusOffset, Bonus_Icon_Y);
	self.BonusTextShadow:setLoc(bonusOffset+2, Bonus_Icon_Y-2);

end

----------------------------------------------------------------

function StateTangoBonusReminder:goConfirm()

   if MOAITangoSdk:tangoHasSdkSupport() then

      MOAIFlurry:event( "Tango-Bonus-Reminder", "answer",      "accept",
                                                "answer-sdk",  "accept-with-sdk" );

		StateMgr:push("state/state-authenticate-confirm.lua");

   else

      MOAIFlurry:event( "Tango-Bonus-Reminder", "answer",      "accept",
                                                "answer-sdk",  "accept-without-sdk" );
		installTango( true );
		StateMgr:pop();

   end

end

----------------------------------------------------------------

function StateTangoBonusReminder:goBack()

   if MOAITangoSdk:tangoHasSdkSupport() then
      MOAIFlurry:event( "Tango-Bonus-Reminder", "answer",      "declined",
                                                "answer-sdk",  "declined-with-sdk"     );
   else
      MOAIFlurry:event( "Tango-Bonus-Reminder", "answer",      "declined",
                                                "answer-sdk",  "declined-without-sdk"     );
	end


	StateMgr:pop();

end

----------------------------------------------------------------

return StateTangoBonusReminder;

----------------------------------------------------------------
