
----------------------------------------------------------------
require('state/state')
require('ui/ui-button/ui-button-image')
require('ui/ui-image')
require('ui/ui-text-box')
require('ui/ui-anim')
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');

----------------------------------------------------------------

----------------------------------------------------------------

local Base       = State;
local StateVideo = Base:new();

local didEnterBG = false;
----------------------------------------------------------------

function StateVideo:onFocus( param )

   Base.onFocus( self );

   StateMgr:setSyncBusyDelay(0);

   disableButtonInput();

   self.rewardType   =  ((param or {}).rewardType)   or "";
   self.rewardAmount =  ((param or {}).rewardAmount) or 0;

   print("StateVideo:onFocus", self.rewardType, self.rewardAmount );

   local success = false;
   if OnlineMgr:isOnline() then
      success = AdMgr:playVideo( self
                                 ,
                                 function()
                                    self:videoSuccess();
                                 end
                                 ,
                                 function()
                                    self:videoFail();
                                 end);
   end

   if not success then
      self:videoFail();
   end

end


----------------------------------------------------------------

function StateVideo:onLoseFocus( )

   Base.onLoseFocus(self);

   StateMgr:setSyncBusyDelay(nil);

   enableButtonInput();

end

----------------------------------------------------------------

function StateVideo:videoSuccess()

   print(" StateVideo:videoSuccess", self.rewardType, self.rewardAmount);

   if self.rewardType == "gold" and (self.rewardAmount or 0 ) > 0 then

      local cId      = AbTestMgr:getVideoConsumableRewardIndex();

      if cId and AbTestMgr:isConsumableStoreEnabled() then
         ConsumableMgr:buy(cId, true, true);
      else
         App.data:addGold( self.rewardAmount, "Video");
      end

      EventMgr:postEvent({gold=self.rewardAmount}, "video-gold");
      MOAIAdjust:trackEvent(App.adjustMap["Video"],         "type",     "gold"      );

   elseif self.rewardType == "key" and (self.rewardAmount or 0 ) > 0 then

      App.data:addKey(self.rewardAmount, "Video");
      EventMgr:postEvent({keys=self.rewardAmount}, "video-key");
      MOAIAdjust:trackEvent(App.adjustMap["Video"],         "type",     "key"      );
   end

   local VIDEO_MAX_VIEWS = AbTestMgr:getVideoMapLimit( ) or 5;

   local videosRemaining = (App.data:getValue("videos-remaining") or VIDEO_MAX_VIEWS);

   App.data:setValue("videos-remaining",           math.max(videosRemaining - 1,0));
   App.data:setValue( "video_button_guide_played", true                           );

   App.data:setValue("video_wait_games", 0);
   local timeStamp = TimeMgr:getTimeGMT();
   if timeStamp then
      App.data:setValue("video_wait_timestamp", timeStamp);
   end

   StateMgr:pop();

end

----------------------------------------------------------------

function StateVideo:videoFail()

   if didEnterBG then
      didEnterBG = false;
      StateMgr:pop();
   else
      MOAIDialogIOS.showDialog(  LocalizationMgr:getString( "NATIVE_VIDEO_ERROR" )
                           ,  LocalizationMgr:getString( "NATIVE_VIDEO_ERROR_MSG" )
                           ,  LocalizationMgr:getString( "NATIVE_OK_BTN" )
                           ,  nil
                           ,  nil
                           ,  false
                           ,  function()

                                 StateMgr:pop();
                              end
                           );
   end
end

----------------------------------------------------------------------

function StateVideo:shouldReportBusy()
   return false;
end

----------------------------------------------------------------------

function StateVideo:isBusy()
   return true;
end

----------------------------------------------------------------------

function StateVideo:getBusyName()

   return "Loading Video...";

end

----------------------------------------------------------------

function StateVideo:onNeedDataResolution()
   -- We shouldn't do anything here
end

----------------------------------------------------------------

function StateVideo:onEnterBackground()
   didEnterBG = true;
end

----------------------------------------------------------------

return StateVideo
