----------------------------------------------------------------

require('flash/flash-manager')
require('state/state');
require('ui/ui-button/ui-button-image');
require('ui/ui-button/ui-button-anim');
require('ui/ui-image');
require('ui/ui-anim');

----------------------------------------------------------------

local PRIORTY_0				= 10;
local PRIORTY_1				= 20;
local PRIORTY_2				= 30;
local PRIORTY_3				= 40;
local PRIORTY_4				= 50;
local PRIORTY_5				= 60;
local PRIORTY_6				= 70;
local PRIORTY_7				= 80;
local PRIORTY_8				= 90;
local PRIORTY_9				= 100;
local PRIORTY_10				= 110;

local PODIUM_PORTRAIT_W		= 80;
local PODIUM_PORTRAIT_H		= 80;

local ENTRY_LOC_Y				=
{
	-8,
	-83,
	-160,
	-231
};

local ENTRY_TROPHY			=
{
	"W_trophy1",
	"W_trophy2",
	"W_trophy3",
	"W_trophy1",
};

local ENTRY_REWARD			=
{
	"W_Reward10",
	"W_Reward5",
	"W_Reward1",
	"W_Reward10",
};

local PODIUM_LOC				=
{
	{	x=0,		y=275	},
	{	x=-131,	y=195	},
	{	x=118,	y=195	},
};

----------------------------------------------------------------

local Base						= State;
local StateWeeklyReward		= Base:new();

----------------------------------------------------------------
----------------------------------------------------------------

function StateWeeklyReward:onLoad()

   State.onLoad( self );

   self.defaultFontName = "resources/font/TangoLatin.otf";
   self.USERNAME_FONT_SIZE = 16;
   self.textureSheet				= FlashMgr:getTextureSheet("resources/flash-resource-bundle/PopUPs.lua", "resources/flash-resource-bundle/PopUPs.png", 30);

	self.IS_POPUP  = true;

	-- background and title
	self.Background = self:addWidget(Anim, self.mainLayer, 0,0,0,0, "W_WeeklyBackGround", self.textureSheet);
	self.Title = self:addWidget(Anim, self.mainLayer, 0,0,0,0, "W_WeeklyTextTitle", self.textureSheet);
	self.Title:setPriority(PRIORTY_0);

	-- push/pop state buttons
	self.ConfirmButton = self:addWidget(ButtonAnim, self.mainLayer, self.mainPartition, 0,0,0,0, "W_SmallOkButton", self.textureSheet, nil, self.goBack, nil);
	self.ConfirmButton:setPriority(PRIORTY_0);
	
	-- podium, portrait frames, and names
	self:_createPodium();
	
	-- reward entry
	self:_createRewardList();

end

----------------------------------------------------------------

function StateWeeklyReward:onActivate()

   Base.onActivate(self);

   local rank = RankingMgr:getPlayerLastWeekRank();

	self:setMessage();

   if not GuideMgr:isActive() then
      if rank <= 3 then
         FeedMgr:shareScreenshot( "WeeklyPodium", 0.01 );
      end
   end

end

----------------------------------------------------------------

function StateWeeklyReward:onFocus()
   
   Base.onFocus(self);
   
end

----------------------------------------------------------------

function StateWeeklyReward:onLoseFocus()
   
   Base.onLoseFocus(self);

	local bountyOffset = App.data:getValue("App.settings.bounty-reset-offset") or 0;

   App.data:giveWeeklyReward( RankingMgr:getPlayerLastWeekRank() );
   RankingMgr:resetScore();
	GuestMgr:cycleGuestFriend();
	RankingMgr:clearLastWeekRanking();

	if bountyOffset == 0 then
		BountyMgr:resetBounty();
	end

	BountyMgr:resetBountyTimeStamp();

end

----------------------------------------------------------------

function StateWeeklyReward:onUpdate( dt )
   
   Base.onUpdate(self, dt);
   
end

----------------------------------------------------------------
----------------------------------------------------------------

function StateWeeklyReward:_createPodium()

	local nameOffset				= 65;
	
	local portraitFrame			=
	{
		"W_PlayerPortrait1Frame",
		"W_PlayerPortrait2Frame",
		"W_PlayerPortrait3Frame",
	};

	self.Podium						= self:addWidget(Anim, self.mainLayer, 0,0,0,0, "W_Ped", self.textureSheet);
	self.Podium:setPriority(PRIORTY_0);

	self.PodiumPortraitFrame	= {};
	self.PodiumPortrait			= {};

	for i=1,3 do
		self.PodiumPortraitFrame[i] = self:addWidget(Anim, self.mainLayer, 0,0,0,0, portraitFrame[i], self.textureSheet);
		self.PodiumPortraitFrame[i]:setPriority(PRIORTY_2);
		
		self.PodiumPortrait[i] = self:addWidget(Image, self.mainLayer, PODIUM_LOC[i].x, PODIUM_LOC[i].y, PODIUM_PORTRAIT_W, PODIUM_PORTRAIT_H,  "resources/UI/PlayerPortrait.png");
		self.PodiumPortrait[i]:setPriority(PRIORTY_1);
	end

end

----------------------------------------------------------------

function StateWeeklyReward:_createRewardList()

	self.RewardList = {};

	for i=1,4 do
	
		local tbl = {};
		
		tbl.trophy			= self:addWidget(Anim, self.mainLayer, 0,0,0,0, ENTRY_TROPHY[i], self.textureSheet);
		tbl.rankNum			= self:addWidget(TextBox, self.mainLayer, -196, ENTRY_LOC_Y[i], 82, 96, "", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
		tbl.rankNumSdw		= self:addWidget(TextBox, self.mainLayer, -196+2, ENTRY_LOC_Y[i]-2, 82, 96, "", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
		tbl.userName		= self:addWidget(TextBox, self.mainLayer, 0, ENTRY_LOC_Y[i], 316, 40, '123456789012345678901234567890', self.defaultFontName, self.USERNAME_FONT_SIZE, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
		tbl.userNameSdw	= self:addWidget(TextBox, self.mainLayer, 0+2, ENTRY_LOC_Y[i]-2, 316, 40, '123456789012345678901234567890', self.defaultFontName, self.USERNAME_FONT_SIZE, nil, MOAITextBox.LEFT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
		tbl.score			= self:addWidget(TextBox, self.mainLayer, 0, ENTRY_LOC_Y[i], 276, 40, '', self.defaultFontName, 16, nil, MOAITextBox.RIGHT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
		tbl.scoreSdw		= self:addWidget(TextBox, self.mainLayer, 0+2, ENTRY_LOC_Y[i]-2, 276, 40, '', self.defaultFontName, 16, nil, MOAITextBox.RIGHT_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
		tbl.reward			= self:addWidget(Anim, self.mainLayer, 0,0,0,0, ENTRY_REWARD[i], self.textureSheet);
		tbl.rewardTxt		= self:addWidget(TextBox, self.mainLayer, 181, ENTRY_LOC_Y[i], 82, 96, "", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);
		tbl.rewardTxtSdw	= self:addWidget(TextBox, self.mainLayer, 181+2, ENTRY_LOC_Y[i]-2, 82, 96, "", self.defaultFontName, 20, nil, MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY);

		if i == 4 then
			tbl.rewardTxt:setText('<c:f0f6fb>--<c>');
			tbl.rewardTxtSdw:setText('<c:000000>--<c>');
		end

		tbl.trophy:setPriority(PRIORTY_1);
		tbl.rankNum:setPriority(PRIORTY_2);
		tbl.rankNumSdw:setPriority(PRIORTY_2-1);
		tbl.userName:setPriority(PRIORTY_2);
		tbl.userNameSdw:setPriority(PRIORTY_2-1);
		tbl.score:setPriority(PRIORTY_2);
		tbl.scoreSdw:setPriority(PRIORTY_2-1);
		tbl.reward:setPriority(PRIORTY_1);
		tbl.rewardTxt:setPriority(PRIORTY_1);
		tbl.rewardTxtSdw:setPriority(PRIORTY_1-1);
	
		table.insert(self.RewardList, tbl);
	
	end

	self:hideRewardEntry(4);

end

----------------------------------------------------------------

function StateWeeklyReward:hideRewardEntry( index )

	self.RewardList[index].trophy:setVisible(false);	
	self.RewardList[index].rankNum:setVisible(false);
	self.RewardList[index].rankNumSdw:setVisible(false);
	self.RewardList[index].userName:setVisible(false);
	self.RewardList[index].userNameSdw:setVisible(false);
	self.RewardList[index].score:setVisible(false);
	self.RewardList[index].scoreSdw:setVisible(false);
	self.RewardList[index].reward:setVisible(false);
	self.RewardList[index].rewardTxt:setVisible(false);
	self.RewardList[index].rewardTxtSdw:setVisible(false);

end

----------------------------------------------------------------

function StateWeeklyReward:showRewardEntry( index )

	self.RewardList[index].rankNum:setVisible(true);
	self.RewardList[index].rankNumSdw:setVisible(true);
	self.RewardList[index].userName:setVisible(true);
	self.RewardList[index].userNameSdw:setVisible(true);
	self.RewardList[index].score:setVisible(true);
	self.RewardList[index].scoreSdw:setVisible(true);
	self.RewardList[index].rewardTxt:setVisible(true);
	self.RewardList[index].rewardTxtSdw:setVisible(true);
	
	-- entry 4 never shows a trophy or reward
	if index < 4 then
		self.RewardList[index].trophy:setVisible(true);
		self.RewardList[index].reward:setVisible(true);
	end

end

----------------------------------------------------------------

function StateWeeklyReward:shiftEntryLocDown()

	local offset = -40;

	for i=1,3 do
		self.RewardList[i].trophy:setLoc(0, offset);
		self.RewardList[i].rankNum:setLoc(-196, ENTRY_LOC_Y[i]+offset);
		self.RewardList[i].rankNumSdw:setLoc(-196+2, ENTRY_LOC_Y[i]+offset-2);
		self.RewardList[i].userName:setLoc(0, ENTRY_LOC_Y[i]+offset);
		self.RewardList[i].userNameSdw:setLoc(0+2, ENTRY_LOC_Y[i]+offset-2);
		self.RewardList[i].score:setLoc(0, ENTRY_LOC_Y[i]+offset);
		self.RewardList[i].scoreSdw:setLoc(0+2, ENTRY_LOC_Y[i]+offset-2);
		self.RewardList[i].reward:setLoc(0, offset);
		self.RewardList[i].rewardTxt:setLoc(181, ENTRY_LOC_Y[i]+offset);
		self.RewardList[i].rewardTxtSdw:setLoc(181+2, ENTRY_LOC_Y[i]+offset-2);
	end

end

----------------------------------------------------------------

function StateWeeklyReward:resetEntryLoc()

	for i=1,3 do
		self.RewardList[i].trophy:setLoc(0, 0);
		self.RewardList[i].rankNum:setLoc(-196, ENTRY_LOC_Y[i]);
		self.RewardList[i].rankNumSdw:setLoc(-196+2, ENTRY_LOC_Y[i]-2);
		self.RewardList[i].userName:setLoc(0, ENTRY_LOC_Y[i]);
		self.RewardList[i].userNameSdw:setLoc(0+2, ENTRY_LOC_Y[i]-2);
		self.RewardList[i].score:setLoc(0, ENTRY_LOC_Y[i]);
		self.RewardList[i].scoreSdw:setLoc(0+2, ENTRY_LOC_Y[i]-2);
		self.RewardList[i].reward:setLoc(0, 0);
		self.RewardList[i].rewardTxt:setLoc(181, ENTRY_LOC_Y[i]);
		self.RewardList[i].rewardTxtSdw:setLoc(181+2, ENTRY_LOC_Y[i]-2);
	end

end

----------------------------------------------------------------

function StateWeeklyReward:setRewardEntryInfo( index, rankNum, portrait, username, score )

	if rankNum and index > 3 then
		if rankNum > 999 then
			self.RewardList[index].rankNum:setGlyphScale(0.7);
			self.RewardList[index].rankNumSdw:setGlyphScale(0.7);
		elseif rankNum > 99 then
			self.RewardList[index].rankNum:setGlyphScale(0.6);
			self.RewardList[index].rankNumSdw:setGlyphScale(0.6);
		else
			self.RewardList[index].rankNum:setGlyphScale(0.80);
			self.RewardList[index].rankNumSdw:setGlyphScale(0.80);
		end
		
		self.RewardList[index].rankNum:setText("<c:f0f6fb>"..rankNum.."<c>");
		self.RewardList[index].rankNumSdw:setText("<c:000000>"..rankNum.."<c>");
	end

	if portrait then
		self.PodiumPortrait[index]:swapImage(portrait);
		self.PodiumPortrait[index]:setPriority(PRIORTY_1);
	end

	self.RewardList[index].userName:setText('<c:f0f6fb>'..username..'<c>');
   self.RewardList[index].userName:preloadTextGlyphs();
	self.RewardList[index].userNameSdw:setText('<c:000000>'..username..'<c>');
   self.RewardList[index].userNameSdw:preloadTextGlyphs();
	self.RewardList[index].score:setText('<c:f0f6fb>'..score..'<c>');
	self.RewardList[index].scoreSdw:setText('<c:000000>'..score..'<c>');

end

----------------------------------------------------------------
----------------------------------------------------------------

function StateWeeklyReward:setMessage()
	
	local playerRank	= RankingMgr:getPlayerLastWeekRank();
	local playerID		= ContactMgr:getProfile():getAccountId();
	
	for i=1,4 do
	
		local entry			= nil;
		local banner		= "W_YourRanking";
		local rankNum		= nil;
		local portrait		= "resources/UI/PlayerPortrait.png";
		local username		= "";
		local score			= "---";
	
		if i < 4 then
			entry = RankingMgr:getLastWeekRankEntry(i);
		else
			entry = RankingMgr:getLastWeekRankEntry(playerRank)
		end
		
		if entry and entry.contact then

			if i == 4 or i == playerRank then
				username = createUserNameDisplay(entry.contact:getFirstName(), '', true, "You", 12);
			else
				username = createUserNameDisplay(entry.contact:getFirstName(), '', true, "Friend", 12);
			end
			
			if entry.prevScore > 0 then
				score = entry.prevScore;
			else
				score = '---';
			end
			
			portrait = ContactMgr:getContactPicture(entry.contact) or portrait;
		
		else

			if i == 4 or i == playerRank then
				username	= createUserNameDisplay(' ', '', true, "You", 12);
			else
				username	= createUserNameDisplay(' ', '', true, "Friend", 12);
			end

			score			= '---';
			portrait		= nil;
		
		end
			
		if i > 3 then
			rankNum = playerRank;
			portrait = nil;
		end
			
		self:setRewardEntryInfo(i, rankNum, portrait, username, score);
	end

	if playerRank < 4 then
		self:shiftEntryLocDown();
		self:hideRewardEntry(4);
	else
		self:resetEntryLoc();
		self:showRewardEntry(4);
	end

end

----------------------------------------------------------------
----------------------------------------------------------------

function StateWeeklyReward:goBack()
	StateMgr:pop();
end

----------------------------------------------------------------

return StateWeeklyReward;

----------------------------------------------------------------
