--==============================================================
require "engine/input/input-manager"
require('engine/object/object')
require('engine/font/font-manager')
require('engine/texture/texture-manager')

require('engine/mission/mission')

require('engine/utility/delay');

require('ui/ui-anim');

--==============================================================

State = {};
GamePaused = false;

PORTRAIT_FRAME_WIDTH		= 86;
PORTRAIT_FRAME_HEIGHT	= PORTRAIT_FRAME_WIDTH;
PORTRAIT_WIDTH          = PORTRAIT_FRAME_WIDTH-6;
PORTRAIT_HEIGHT         = PORTRAIT_FRAME_WIDTH-6;

----------------------------------------------------------------------

function State:new()

   local o = Object.new( self );
   return o;

end

----------------------------------------------------------------

function  State:onLoad(  )

   print("State:onLoad",self.stateFilename);
   self.layerTable = {};
   self.widgets    = self.widgets    or {};

   --make sure we have at least 1 layer and a partition
   self.mainLayer       = self:addLayer();
   self.mainPartition   = self:addPartition();
   self.mainLayer:setPartition( self.mainPartition );

   self.loaded = true;

   MOAIFlurry:event( "State:onLoad", "Filename", tostring(self.stateFilename) );

end

----------------------------------------------------------------

function  State:onUnload( )

   print("State:onUnload",self.stateFilename);
   self:deleteAllWidgets();

   for i, layerSet in ipairs ( self.layerTable ) do

      for j, layer in ipairs ( layerSet ) do
         layer:clear ()
         layer = nil
      end
   end

   self.mainLayer  = nil;
   self.layerTable = nil;

   self.loaded     = false;

   MOAIFlurry:event( "State:onUnload", "Filename", tostring(self.stateFilename) );


end


----------------------------------------------------------------

function  State:onActivate(  )
   print("State:onActivate",self.stateFilename);

   self.active = true;


   MOAIFlurry:event( "State:onActivate", "Filename", tostring(self.stateFilename) );

end

----------------------------------------------------------------

function  State:onDeactivate( )

   print("State:onDeactivate",self.stateFilename);

   self.active = false;


   MOAIFlurry:event( "State:onDeactivate", "Filename", tostring(self.stateFilename) );

end

----------------------------------------------------------------

function State:onFocus()

   print("State:onFocus",self.stateFilename);

   MOAIFlurry:eventStart( "UI-State-" .. self.stateFilename );

   self.focus     = true;
   self.startTime = MOAISim.getDeviceTime ()
   self.inputDown = false;

   if self.resourceBar then
      if self.resourceBar.updateHolidayTag then
         self.resourceBar:updateHolidayTag();
      end

      --Avoid delay in showing accurate number
      self.resourceBar:onUpdate(0.001);

   end


   MOAIFlurry:event( "State:onFocus", "Filename", tostring(self.stateFilename) );


end

----------------------------------------------------------------

function  State:onLoseFocus( )

   print("State:onLoseFocus",self.stateFilename);

   self.focus     = false;
   MOAIFlurry:eventStop( "UI-State-" .. self.stateFilename );

   MOAIFlurry:event( "State:onLoseFocus", "Filename", tostring(self.stateFilename) );

end

----------------------------------------------------------------

function State:PauseGameplay()
end

----------------------------------------------------------------

function  State:onEnterBackground(  )

   MOAIFlurry:event( "State:onEnterBackground", "Filename", tostring(self.stateFilename) );

end

----------------------------------------------------------------

function  State:onEnterForeground( )

    print("State:onEnterForeground");
    App:CheckStatuses();
   MOAIFlurry:event( "State:onEnterForeground", "Filename", tostring(self.stateFilename) );

end
----------------------------------------------------------------

function  State:onCancelInput()

   self.inputDown = false;

end

----------------------------------------------------------------

function  State:onBackButton()

   if self.goBack and type(self.goBack) =="function" then
      self:goBack();
   end

end

----------------------------------------------------------------

function  State:onInput( handled )

   Debug.assert( self.layerTable    );
   Debug.assert( self.layerTable[1] );
   Debug.assert( InputMgr           );

   local mainLayer = self.mainLayer or self.layerTable[1][1];

   if mainLayer then

      local x, y = mainLayer:wndToWorld ( InputMgr:getTouchLoc() )
		local exitUpdate;

      --This is when a banner or another system already handled the situation
      if handled then

      else

         if  InputMgr:wasTouchPressed() then

            self.inputDown = true;

            for i=1,#self.widgets do
               if self.widgets[i] then
                  exitUpdate = self.widgets[i]:updateClick( true, x, y );
               end

               if exitUpdate then
                  break;
               end
            end

         else

            if self.inputDown then

               if InputMgr:wasTouchReleased() then

                  for i=1,#self.widgets do
                     if self.widgets[i] then
                        exitUpdate = self.widgets[i]:updateClick( false, x, y );
                     end

                     if exitUpdate then
                        break;
                     end
                  end

                  self.inputDown = false;

               elseif InputMgr:isTouchDown() then

                  for i=1,#self.widgets do
                     if self.widgets[i] then
                        self.widgets[i]:updateMove( x, y );
                     end
                  end
               end
            end
         end
      end
   end
end

----------------------------------------------------------------

function  State:onUpdate( dt )

   if self.resourceBar then
      self.resourceBar:onUpdate(dt);
   end

end

----------------------------------------------------------------

function  State:addWidget( factory, layer, ... )

   local w = factory:new(self, nil, layer, ...);

   if w then
      table.insert( self.widgets, w );
   end

   return w;

end

----------------------------------------------------------------

function  State:deleteWidget( widget )

   for i=#self.widgets,1,-1 do

      local w = self.widgets[i];

      if w == widget then

         w:delete();
         table.remove(self.widgets,i);

      end

   end

end

----------------------------------------------------------------

function  State:deleteAllWidgets( )

   for i=1,#self.widgets do
      self.widgets[i]:delete();
   end
   self.widgets    = {};
end

----------------------------------------------------------------------

function State:enableAds()

   EventMgr:postEvent({}, "ads-enabled" );

end

----------------------------------------------------------------------

function State:disableAds()

   EventMgr:postEvent({}, "ads-disabled" );

end

----------------------------------------------------------------------

function State:enableAllWidget()

	for i=1,#self.widgets do
		self.widgets[i]:enable();
	end

end

----------------------------------------------------------------------

function State:disableAllWidget()

	for i=1,#self.widgets do
		self.widgets[i]:disable();
	end

end

----------------------------------------------------------------

function  State:addLayer( index  )

   local layer = MOAILayer2D.new ();

   layer:setViewport ( App.viewport )

   if self.layerTable [ index or 1 ] == nil then
      self.layerTable [ index or 1 ] = { layer }
   else
      table.insert( self.layerTable [ index or 1 ], layer);
   end

   return layer;

end

----------------------------------------------------------------

function State:addPartition()

	local partition = MOAIPartition.new ();

	return partition;

end

----------------------------------------------------------------

function State:wndToWorld( x, y  )

   if self.layerTable and self.layerTable[1] then
      return self.layerTable[1][1]:wndToWorld( x, y  );
   else
      return x,y;
   end
end

----------------------------------------------------------------

function createUserNameDisplay( first, last, isfullName, default, truncSize )

   local userName = "";

	if isfullName then

		userName = (first or "") .. " " .. (last or "");
	else

		userName = (first or "") .. " " .. (getFirstInitial(last) or "");
	end

   -- <= 1 instead of 0 because we have a " " in it
	if userName:len() <= 1 then
      userName = default;
   end

	if truncSize then
		if (truncSize - 3) > 0 then
			userName = truncateName(userName, truncSize - 3);
		end
	end

   return userName;

end

----------------------------------------------------------------

function getFirstInitial( name )

   if name then
      return name:match("[%z\1-\127\194-\244][\128-\191]*") or "";
   end

   return "";

end

----------------------------------------------------------------

function truncateName( name, size )

	local counter = 0;
	local username = nil;

	for uchar in name:gmatch("[%z\1-\127\194-\244][\128-\191]*") do

		if counter < size then
			if username == nil then
				username = uchar;
			else
				username = username..uchar;
			end

			counter = counter + 1;
		else
			username = username..'...';

			return username;
		end

	end

	return username;

end

----------------------------------------------------------------------

function getFormattedNumber( number )

   Debug.assertType(number, "number", "getFormattedNumber -> number");

   -- These options can vary between localizations
   local SEPARATOR = ",";
   local CHARS_BETWEEN_SEPARATOR = 3;

   local resultTable = {};
   local numberString = tostring(math.abs(number));
   local charsUntilSeparator = CHARS_BETWEEN_SEPARATOR;

   for i = #numberString, 1, -1 do

      table.insert(resultTable, 1, numberString:sub(i, i));
      charsUntilSeparator = charsUntilSeparator - 1;

      if charsUntilSeparator == 0 and i ~= 1 then
         table.insert(resultTable, 1, SEPARATOR);
         charsUntilSeparator = CHARS_BETWEEN_SEPARATOR;
      end

   end

   if number < 0 then
      table.insert(resultTable, 1, "-");
   end

   -- We defer concatenation until the function return for performance
   return table.concat(resultTable);
end

----------------------------------------------------------------------

function isTimeInLim( fieldId, tangoID, limit, curTime )

   limit  = limit or 60*60*24;

   if not fieldId or not tangoID or not curTime or not limit then
      Debug.assert(fieldId,   "isTimeInLim -> fieldId");
      Debug.assert(tangoID,   "isTimeInLim -> tangoID");
      Debug.assert(curTime,   "isTimeInLim -> curTime");
      Debug.assert(limit,     "isTimeInLim -> limit"  );
      return false;
   end

   local delta		= curTime - ( App.data:getValue(fieldId.."-"..tangoID) or 0 );

   --LESS THAN <scale> DAYS, CANNOT INVITE AGAIN
   return delta < limit

end

----------------------------------------------------------------------

function getTimeInLim( fieldId, tangoID, curTime )

	if not fieldId or not tangoID or not curTime then
      Debug.assert(fieldId,   "getTimeInLim -> fieldId");
      Debug.assert(tangoID,   "getTimeInLim -> tangoID");
      Debug.assert(curTime,   "getTimeInLim -> curTime");
		return false;
	end

	local delta = curTime - ( App.data:getValue(fieldId.."-"..tangoID) or 0 );

	return delta;

end

----------------------------------------------------------------------

function getCoolDownCount( fieldId, tangoID )

   if not fieldId or not tangoID then
      Debug.assert(fieldId,   "isTimeInLim -> fieldId");
      Debug.assert(tangoID,   "isTimeInLim -> tangoID");

		return false;
	end

	return App.data:getValue(fieldId.."-"..tangoID);

end

----------------------------------------------------------------------

function setCoolDownCount( fieldId, tangoID, value )

   if not fieldId or not tangoID then
      Debug.assert(fieldId,   "isTimeInLim -> fieldId");
      Debug.assert(tangoID,   "isTimeInLim -> tangoID");

		return false;
	end

	App.data:setValue(fieldId.."-"..tangoID, value);

end

----------------------------------------------------------------------

function isNewHiScore( tangoID )

	if not tangoID then
		return false;
	end

	local tracker = App.data:getValue('BragTracker');

	if tracker == nil then
		return true;
	end

	if tracker['HiScore-'..tangoID] then
		return false;
	end

	return true;

end

----------------------------------------------------------------------

function installTango()

	local title     = LocalizationMgr:getString( "NATIVE_INSTALL_TANGO_TITLE1" );
	local message   = LocalizationMgr:getString( "NATIVE_INSTALL_TANGO_MSG1" );
	local buttonMsg = LocalizationMgr:getString( "NATIVE_INSTALL_TANGO_BTN1" );

	if MOAITangoSdk:tangoIsInstalled() then
		title       = LocalizationMgr:getString( "NATIVE_INSTALL_TANGO_TITLE2" );
		message     = LocalizationMgr:getString( "NATIVE_INSTALL_TANGO_MSG2" );
		buttonMsg   = LocalizationMgr:getString( "NATIVE_INSTALL_TANGO_BTN2" );
	end

	local attempt       = ( App.data:getValue("tango-install-attempt") or 0 ) + 1;
	App.data:setValue("tango-install-attempt", attempt);

	local installCount  = ( App.data:getValue("tango-install-try") or 0 );

	MOAIFlurry:eventStart(  "Tango-Missing", "ending", "none", "attempt", ""..attempt, "install",  installCount);

	MOAIDialogIOS.showDialog(	title
										, message
										, buttonMsg
										,  nil
										,  nil
										,  true
										,  function( result )
												if result == MOAIDialogIOS.DIALOG_RESULT_POSITIVE then

													installCount  = installCount + 1;
													App.data:setValue("tango-install-try", installCount);

													MOAIFlurry:eventStop( "Tango-Missing", "ending", "install" );
													MOAIFlurry:event(     "Tango-Install" );
													MOAITangoSdk:installTango(false);

												else

													MOAIFlurry:eventStop(  "Tango-Missing", "ending", "cancel" );

												end
											end
									);

end

----------------------------------------------------------------------

function State:shouldReportBusy()
   return true;
end

----------------------------------------------------------------------

function State:isBusy()

end

----------------------------------------------------------------------

function State:getBusyName()

   if StoreMgr:isBusy() then
      return "Waiting for Store";
   else
      return "Syncing";
   end

end

----------------------------------------------------------------------

function State:isUsingCameraHack()

   return StateMgr.headerAvailable;

end

----------------------------------------------------------------------

function State:_calcHeaderLocY()

   if self:isUsingCameraHack() then
      return App.viewportHeight/2-UI_HEADER_HEIGHT/2+StateMgr.cameraDelta;
   else
      return App.viewportHeight/2-UI_HEADER_HEIGHT/2;
   end

end

----------------------------------------------------------------------

function State:_setupBasicResourceBar( forced )

   if StateMgr.headerAvailable or forced then
      self.resourceBar = ResourceBar:new( self, 0, self:_calcHeaderLocY(), self.mainLayer, self.mainPartition  );
   end

end

----------------------------------------------------------------------

function State:onDataSaveError()

   self:showPopup( LocalizationMgr:getString( "NATIVE_FILE_ERROR_TITLE" ),
                   LocalizationMgr:getString( "NATIVE_LOSE_PROGRESS" ),
                   LocalizationMgr:getString( "NATIVE_OK_BTN" ),
                   false );

end

----------------------------------------------------------------------

function State:onDataLoadError( msg )

   self:showPopup( LocalizationMgr:getString( "NATIVE_LOAD_ERROR_TITLE" ),
                   LocalizationMgr:getString( "NATIVE_LOSE_PROGRESS" ),
                   LocalizationMgr:getString( "NATIVE_OK_BTN" ),
                   false );

end

----------------------------------------------------------------------

function State:onDataServerError()

   self:showPopup( LocalizationMgr:getString( "NATIVE_SERVER_ERROR_TITLE" ),
                   LocalizationMgr:getString( "NATIVE_CONNECT_TO_NET" ),
                   LocalizationMgr:getString( "NATIVE_OK_BTN" ),
                   true );

end

------------------------------------------------------------------------------------

function State:onDataMerged()

   self:showPopup( LocalizationMgr:getString( "NATIVE_DATA_MERGE_TITLE" ),
                   LocalizationMgr:getString( "NATIVE_DATA_MERGE_MSG" ),
                   LocalizationMgr:getString( "NATIVE_OK_BTN" ),
                   false );

end

------------------------------------------------------------------------------------

function State:showPopup(title, message, buttons, goHome)

   if not StateMgr._showErrorDialog then

      StateMgr._showErrorDialog = true;
      StateMgr:clearBanner();

      MOAIDialogIOS.showDialog( title,
                                message,
                                buttons,
                                nil,
                                nil,
                                false,
                                function()

                                   StateMgr._showErrorDialog = nil;
                                   StateMgr:clearDataServerError();

                                   if goHome and not GuideMgr:isBusy() then
                                      StateMgr:clear( "state/state-ranking.lua" );
                                   end

                                end
      );

   end

end

------------------------------------------------------------------------------------

function State:onNeedDataResolution(event)

   if GuideMgr:isBusy() then
      return;
   end

   StateMgr:push("state/state-cloud-resolve-popup.lua", event);
end

------------------------------------------------------------------------------------

function State:isVideoRewardReady( )

   local limit           = AbTestMgr:getVideoMapLimit( ) or 5;
   local videosRemaining = (App.data:getValue("videos-remaining") or limit);

   if AdMgr:isVideoAvailable() and videosRemaining > 0 then
      return true;
   end

   return false;

end


----------------------------------------------------------------------

return State

----------------------------------------------------------------
