require('ui/ui-widget')
----------------------------------------------------------------------

local Base = Widget;
Anim = Base:new();

----------------------------------------------------------------------

function Anim:new( screen, parent, layer, x, y, w, h, anim, spritesheet, color, autoHide, graphic, autoReset )

	local animPlayer = Base.new( self, screen, parent, layer, x, y, w, h );

	if graphic then
		animPlayer.anim  = animPlayer:createAnim( layer, x, y, w, h, anim, spritesheet, graphic );
	else
		animPlayer.anim  = animPlayer:createAnim( layer, x, y, w, h, anim, spritesheet );
	end

	if graphic then
		animPlayer.anim.animName = anim;
	else
		for i=1,#animPlayer.anim.props do
			animPlayer.anim.props[i].animName = anim;
		end
	end

	if color then
		animPlayer:setColor( color[1], color[2], color[3], color[4] );
	end

	if not autoHide then
		if animPlayer.anim.root then
			animPlayer:pauseLastFrame();
		end
	end

	if autoReset then
		if animPlayer.anim.root then
			animPlayer.autoReset = true;
			animPlayer:pauseLastFrame();
			animPlayer:setListener(MOAIAction.EVENT_STOP,
				function ()
					animPlayer:resetAnim();
				end
			);
		end
	end

	animPlayer.layer = layer;
	animPlayer.spritesheet = spritesheet;

	return animPlayer;

end

----------------------------------------------------------------------

function Anim:delete( )

   self:deleteAnim( );

   --must be at the end.. will clear the layer
   Base.delete( self );

end


----------------------------------------------------------------------

function Anim:addAnim( layer, anim, spritesheet, x, y )

	local player = spritesheet:newAnimPlayer(anim);

	for i=1,#player.props do
		player.props[i]:setPriority(0);
		player.props[i].animName = anim;
	end

	player.root:setLoc(X, Y);
	player:insertProps(layer);
	player:pauseLastFrame();

	return player;

end

----------------------------------------------------------------------

function Anim:addGraphic( layer, anim, spritesheet, x, y )

	local player = spritesheet:newProp(anim);

	player:setPriority(0);
	player:setLoc(X, Y);
	player.animName = anim;
	layer:insertProp(player);

	return player;

end

----------------------------------------------------------------------

function Anim:deleteAnim( )

	if self.anim then

		local layer = self.layer;

		if self.anim.root then
			self:stop();
			self.anim:removeProps(layer)
		else
			layer:removeProp(self.anim);
		end

	end

end

----------------------------------------------------------------------

function Anim:swapAnim( anim )

	if self.anim then

		if self:getAnimName() == anim then
			self:apply(0);

			return false;
		end

		local layer = self.layer;
		local spritesheet = self.spritesheet;
		local x;
		local y;

		if self.anim.root then
			x,y = self.anim.root:getLoc();
		else
			x,y = self.anim:getLoc();
		end

		self:deleteAnim();

		self.anim = self:addAnim(layer, anim, spritesheet, x, y);

		if self.priority then
			self:setPriority(self.priority);
		end

		return true;
	end

end

----------------------------------------------------------------------

function Anim:swapGraphic( graphic )

	if self.anim then

		if self:getAnimName() == graphic then
			return false;
		end

		local layer = self.layer;
		local spritesheet = self.spritesheet;
		local x;
		local y;

		if self.anim.root then
			x,y = self.anim.root:getLoc();
		else
			x,y = self.anim:getLoc();
		end

		self:deleteAnim();

		self.anim = self:addGraphic(layer, graphic, spritesheet, x, y);

		if self.priority then
			self:setPriority(self.priority);
		end
	end

end

----------------------------------------------------------------------

function Anim:getLoc()

	local x;
	local y;

   if self.anim then
		if self.anim.root then
			x,y = self.anim.root:getLoc();
		else
			x,y = self.anim:getLoc();
		end
   end

	return x,y;

end

----------------------------------------------------------------------

function Anim:moveLoc( x, y, time, ease )

	local action;

   if self.anim then

		if self.anim.root then
			action = self.anim.root:moveLoc(x, y, 0, time, ease);
		else
			action = self.anim:moveLoc(x, y, 0, time, ease);
		end

		return action;
   end

end

----------------------------------------------------------------------

function Anim:setLoc(x,y)

   if self.anim then
		if self.anim.root then
			self.anim.root:setLoc(x,y);
		else
			self.anim:setLoc(x,y);
		end
   end

   Base.setLoc(self,x,y)

end

----------------------------------------------------------------------

function Anim:setLocX( x )

   if self.anim then
		if self.anim.root then
			local x2, y = self.anim.root:getLoc();
			self.anim.root:setLoc( x, y );
		else
			local x2, y = self.anim:getLoc();
			self.anim:setLoc( x, y );
		end
   end

   Base.setLocX( self, x );

end

----------------------------------------------------------------------

function Anim:setLocY( y )

   if self.anim then
		if self.anim.root then
			local x = self.anim.root:getLoc();
			self.anim.root:setLoc( x, y );
		else
			local x = self.anim:getLoc();
			self.anim:setLoc( x, y );
		end
   end

   Base.setLocY( self, y );

end

----------------------------------------------------------------------

function Anim:seekLoc( x, y, time, ease )

	local action = nil;

   if self.anim then
		if self.anim.root then
			action = self.anim.root:seekLoc(x, y, time, ease);
		else
			action = self.anim:seekLoc(x, y, time, ease);
		end
   end

	return action;

end

----------------------------------------------------------------------

function Anim:seekScale( sx, sy, time, ease )

	local action	= nil;
	local e			= MOAIEaseType.SOFT_SMOOTH;

   if self.anim then

		if ease then
			e = ease;
		end

		if self.anim.root then
			action = self.anim.root:seekScl(sx, sy, time, e);
		else
			action = self.anim:seekScl(sx, sy, time, e);
		end
   end

	return action;

end

----------------------------------------------------------------------

function Anim:setAlpha( a )

	if self.anim and a then
		if self.anim.root then
			for i=1,#self.anim.props do
				self.anim.props[i]:setColor(1, 1, 1, a);
			end
		else
			self.anim:setColor(1, 1, 1, a);
		end
	end

end

----------------------------------------------------------------------

function Anim:setColor( r, g, b, a )

   if self.anim then
		if self.anim.root then
			for i=1,#self.anim.props do
				if self.anim.props[i].easeColorAction then
					self.anim.props[i].easeColorAction:stop();
				end

				self.anim.props[i]:setColor(r, g, b, a);
			end
		else
			if self.anim.easeColorAction then
				self.anim.easeColorAction:stop();
			end

			self.anim:setColor(r, g, b, a);
		end
   end

   Base.setColor( self, r, g, b, a );

end

----------------------------------------------------------------------

function Anim:seekColor( r, g, b, a, time )

   if self.anim then
		if self.anim.root then
			for i=1,#self.anim.props do
				if self.anim.props[i].easeColorAction then
					self.anim.props[i].easeColorAction:stop();
				end

				self.anim.props[i].easeColorAction = self.anim.props[i]:seekColor(r, g, b, a, time);
			end
		else
			if self.anim.easeColorAction then
				self.anim.easeColorAction:stop();
			end

			self.anim:seekColor(r, g, b, a, time);
		end
   end

   Base.setColor( self, r, g, b, a );

end

----------------------------------------------------------------------

function Anim:setRot( x, y, z )

	if self.anim then
		if self.anim.root then
			self.anim.root:setRot(x,y,z);
		else
			self.anim:setRot(x,y,z);
		end
	end

end

----------------------------------------------------------------------

function Anim:setScale( sx, sy )

   if self.anim then
		if self.anim.root then
			self.anim.root:setScl( sx, sy );
		else
			self.anim:setScl( sx, sy );
		end
   end

   Base.setScale( self, sx, sy );

end

----------------------------------------------------------------------

function Anim:hide( )

	self:setVisible(false);

end

----------------------------------------------------------------------

function Anim:show( )

	self:setVisible(true);

end

----------------------------------------------------------------------

function Anim:setVisible( visible )

   if self.anim then
		if self.anim.root then
			for i=1,#self.anim.props do
				self.anim.props[i]:setVisible(visible);
			end
		else
			self.anim:setVisible(visible);
		end
   end

--   Base.setVisible( self, visible );

end

----------------------------------------------------------------------

function Anim:pauseLastFrame()

	if self.anim then
		self.anim:pauseLastFrame();
	end

end

----------------------------------------------------------------------

function Anim:getTime()

	if self.anim then
		return self.anim:getTime();
	end

end

----------------------------------------------------------------------

function Anim:getAnimName()

	if self.anim then
		if self.anim.root then

			return self.anim.props[1].animName;
		else

			return self.anim.animName;
		end
	end

end

----------------------------------------------------------------------

function Anim:setListener( event, func )

	if self.anim then
		self.anim:setListener(event, func);
	end

end

----------------------------------------------------------------------

function Anim:setParent( parent )

   if self.anim then
		if self.anim.root then
			self.anim.root:setParent(parent);
		else
			self.anim:setParent(parent);
		end
	end

end

-------------------------------------------------------------------------------------

function Anim:clearAttrLink( attrID )

   if self.anim then
      if self.anim.root then
         self.anim.root:clearAttrLink( attrID );
      else
         self.anim:clearAttrLink( attrID );
      end
   end

end

-------------------------------------------------------------------------------------

function Anim:setAttrLink( attrID, parent, sourceAttrID )

   if self.anim then
      if self.anim.root then
         self.anim.root:setAttrLink( attrID , parent, sourceAttrID);
      else
         self.anim:setAttrLink( attrID , parent, sourceAttrID);
      end
   end

end

----------------------------------------------------------------------

function Anim:setPriority( priority )

   if self.anim then

	  priority = priority or 0;

      if self.anim.root then
         for i=1,#self.anim.props do
            self.anim.props[i]:setPriority(priority);
         end
      else
         self.anim:setPriority(priority);
      end

      self.priority = priority;
   end

end

----------------------------------------------------------------------

function Anim:setSpeed( speed )

   if self.anim then
		self.anim:setSpeed( speed );
	end

end

----------------------------------------------------------------------

function Anim:setMode( mode )

	if self.anim then
		if mode then
			self.anim:setMode(mode);
		else
			self.anim:setMode(MOAITimer.NORMAL);
		end
	end

end

----------------------------------------------------------------------

function Anim:setSpan( startTime, endTime )

	if self.anim then
		if endTime then
			self.anim:setSpan(startTime, endTime);
		else
			self.anim:setSpan(startTime);
		end
	end

end

----------------------------------------------------------------------

function Anim:apply( time )

	if self.anim and self.anim.root then
		self.anim:apply(time);
	end

end

----------------------------------------------------------------------

function Anim:setAnimFrame( frame, fps )

	if self.anim then

		if not fps then
			fps = 30;
		end

		if not frame then
			frame = 0;
		end

		local time = frame/fps;

		if time >= self.anim.sym.span then
			self:setAnimLastFrame();
		else
			self:stop();
			self:apply(time);
		end
	end

end

----------------------------------------------------------------------

function Anim:setAnimLastFrame( )

	if self.anim then

		local time = self.anim.sym.span - self.anim.sym.frame;

		self:stop();
		self:apply(time);
	end

end

----------------------------------------------------------------------

function Anim:setTime( time )

	if self.anim then
		self.anim:setTime(time);
	end

end

----------------------------------------------------------------------

function Anim:start()

	local action;

	if self.anim then
		action = self.anim:start();
	end

	return action;

end

----------------------------------------------------------------------

function Anim:pause()

   if self.anim then
      self.anim:pause();
   end

end
----------------------------------------------------------------------

function Anim:stop()

	if self.anim and self.anim.stop then
		self.anim:stop();
	end

end

----------------------------------------------------------------------

function Anim:resetAnim()

	if self.anim.root and self.autoReset ~= nil then
		if self.autoReset == true then
			self.autoReset = false;
			self:stop();
			self:apply(0);
		elseif self.autoReset == false then
			self:pauseLastFrame();
			self:setTime(0);
		end
	elseif self.anim.root then
		self:pauseLastFrame();
		self:setTime(0);
	end

end

----------------------------------------------------------------------

function Anim:isInside(x, y)

	if self.anim then
		if self.anim.root then
			if self.anim.props[1]:inside(x, y) then
				return true;
			end
		else
			if self.anim:inside(x, y) then
				return true;
			end
		end
		return false;
	end

end

----------------------------------------------------------------------

function Anim:forceUpdate()
   if self.anim and self.anim.props then
	  for _, v in ipairs(self.anim.props) do
		 v:forceUpdate();
	  end
   end
end
