require('engine/texture/texture-manager')
require('ui/ui-button/ui-button')
require('app/template/audio/UI_sounds_table')
----------------------------------------------------------------------

BUTTON_PRESSED_FRAME = 5;

----------------------------------------------------------------------

local Base  = Button;
ButtonAnim = Base:new();
----------------------------------------------------------------------

function ButtonAnim:new( screen, parent, layer, partition, x, y, w, h, anim, spritesheet, pressedFunc, releasedFunc, canceledFunc, movedFunc, enableAutoReset, graphic, pad, disableAutoAnim, inViewTest)

--	print("ButtonAnim:new", screen, parent, layer, partition, x, y, w, h, anim, spritesheet, pressedFunc, releasedFunc, canceledFunc, movedFunc, enableAutoReset, graphic, pad, disableAutoAnim, inViewTest);

   local button = Base.new(self, screen, parent, layer, x, y, 0, 0, pressedFunc, releasedFunc, canceledFunc, movedFunc );

   button.sounds = UI_SOUNDS_TABLE;
   button.uiSoundName = anim;
   button.uiScreenName = screen.name;

	local player;
	if graphic then
		player = spritesheet:newProp(anim);
	else
		player = spritesheet:newAnimPlayer(anim);
	end
	local X = 0;
	local Y = 0;

	if x then
		X = x;
	end

	if y then
		Y = y;
	end

	if graphic then
		player:setPriority(0);
		player:setLoc(X, Y);
		layer:insertProp(player);
		player.button = button;
		player.animName = anim;
	else
		for i=1,#player.props do
			player.props[i]:setPriority(0);
			player.props[i].button = button;
			player.props[i].animName = anim;
		end
		player.root:setLoc(X, Y);
		player:insertProps(layer);
		player:pauseLastFrame();
	end
	button.layer = layer;
	button.spritesheet = spritesheet;
	button.anim = player;
	button.partition = partition;
	button.disableAutoAnim = disableAutoAnim;
	button.animName = anim;

	if pad then
		button.pad = pad;
	else
		button.pad = nil;
	end

	if player.root then
		if enableAutoReset then
			button.autoReset = true;
			button:setListener(MOAITimer.EVENT_TIMER_END_SPAN,
			function ()
				button:resetAnim();
			end
			);
		end
	end

	if inViewTest then
		button._isInView = inViewTest;
	end

   return button;

end

----------------------------------------------------------------------

function ButtonAnim:delete( )

   self:deleteAnim( );

   --must be at the end.. will clear the layer
   Base.delete( self );

end

----------------------------------------------------------------------

function ButtonAnim:addAnim( layer, anim, spritesheet, x, y, button )

	local player = spritesheet:newAnimPlayer(anim);

	for i=1,#player.props do
		player.props[i]:setPriority(0);
		player.props[i].button = button;
		player.props[i].animName = anim;
	end

	player.root:setLoc(x, y);
	player:insertProps(layer);
	player:pauseLastFrame();

	return player;

end

----------------------------------------------------------------------

function ButtonAnim:addGraphic( layer, anim, spritesheet, x, y, button )

	local player = spritesheet:newProp(anim);

	player:setPriority(0);
	player:setLoc(x, y);
	player.button = button;
	player.animName = anim;
	layer:insertProp(player);

	return player;

end

----------------------------------------------------------------------

function ButtonAnim:deleteAnim( )

	if self.anim then

		local layer = self.layer;

		if self.anim.root then
			self:stop();
			self.anim:removeProps(layer)
		else
			layer:removeProp(self.anim);
		end

	end

end

----------------------------------------------------------------------

function ButtonAnim:swapAnim( anim )

	if self.anim then

	if self:getAnimName() == anim then
			self:apply(0);

			return false;
		end

		local layer = self.layer;
		local spritesheet = self.spritesheet;
		local button;
		local x;
		local y;

		if self.anim.root then
			x,y = self.anim.root:getLoc();
			button = self.anim.props[1].button;
		else
			x,y = self.anim:getLoc();
			button = self.anim.button;
		end

		self:deleteAnim();

		self.anim = self:addAnim(layer, anim, spritesheet, x, y, button);

		if self.priority then
			self:setPriority(self.priority);
		end

		return true;
	end

end

----------------------------------------------------------------------

function ButtonAnim:swapGraphic( graphic )

	if self.anim then

		if self:getAnimName() == graphic then
			return false;
		end

		local layer = self.layer;
		local spritesheet = self.spritesheet;
		local button;
		local x;
		local y;

		if self.anim.root then
			x,y = self.anim.root:getLoc();
			button = self.anim.props[1].button;
		else
			x,y = self.anim:getLoc();
			button = self.anim.button;
		end

		self:deleteAnim();

		self.anim = self:addGraphic(layer, graphic, spritesheet, x, y, button);

		if self.priority then
			self:setPriority(self.priority);
		end
	end

end

----------------------------------------------------------------------

function ButtonAnim:getLoc()

	local x;
	local y;

   if self.anim then
		if self.anim.root then
			x,y = self.anim.root:getLoc();
		else
			x,y = self.anim:getLoc();
		end
   end

	return x,y;

end

----------------------------------------------------------------------

function ButtonAnim:moveLoc( x, y, time, ease )

	local action;

   if self.anim then

		if self.anim.root then
			action = self.anim.root:moveLoc(x, y, 0, time, ease);
		else
			action = self.anim:moveLoc(x, y, 0, time, ease);
		end

		return action;
   end

end

----------------------------------------------------------------------

function ButtonAnim:setLoc(x,y)

   if self.anim then
		if self.anim.root then
			self.anim.root:setLoc(x,y);
		else
			self.anim:setLoc(x,y);
		end
   end

   Base.setLoc(self,x,y)

end

----------------------------------------------------------------------

function ButtonAnim:setLocX( x )

   if self.anim then
		if self.anim.root then
			local x2, y = self.anim.root:getLoc();
			self.anim.root:setLoc( x, y );
		else
			local x2, y = self.anim:getLoc();
			self.anim:setLoc( x, y );
		end
   end

   Base.setLocX( self, x );

end

----------------------------------------------------------------------

function ButtonAnim:setLocY( y )

   if self.anim then
		if self.anim.root then
			local x = self.anim.root:getLoc();
			self.anim.root:setLoc( x, y );
		else
			local x = self.anim:getLoc();
			self.anim:setLoc( x, y );
		end
   end

   Base.setLocY( self, y );

end

----------------------------------------------------------------------

function ButtonAnim:seekLoc( x, y, time, ease )

	local action = nil;

   if self.anim then
		if self.anim.root then
			action = self.anim.root:seekLoc(x, y, time, ease);
		else
			action = self.anim:seekLoc(x, y, time, ease);
		end
   end

	return action;

end

----------------------------------------------------------------------

function ButtonAnim:setColor( r, g, b, a, index )

   if self.anim then
		if self.anim.root then
			if index and self.anim.props[index] then
				if self.anim.props[index].easeColorAction then
					self.anim.props[index].easeColorAction:stop();
				end

				self.anim.props[index]:setColor(r, g, b, a);
			else
				for i=1,#self.anim.props do
					if self.anim.props[i].easeColorAction then
						self.anim.props[i].easeColorAction:stop();
					end

					self.anim.props[i]:setColor(r, g, b, a);
				end
			end
		else
			if self.anim.easeColorAction then
				self.anim.easeColorAction:stop();
			end

			self.anim:setColor(r, g, b, a);
		end
   end

   Base.setColor( self, r, g, b, a );

end

----------------------------------------------------------------------

function ButtonAnim:seekColor( r, g, b, a, time, index )

   if self.anim then
		if self.anim.root then
			if index and self.anim.props[index] then
				if self.anim.props[index].easeColorAction then
					self.anim.props[index].easeColorAction:stop();
				end

				self.anim.props[index].easeColorAction = self.anim.props[index]:seekColor(r, g, b, a, time);
			else
				for i=1,#self.anim.props do
					if self.anim.props[i].easeColorAction then
						self.anim.props[i].easeColorAction:stop();
					end

					self.anim.props[i].easeColorAction = self.anim.props[i]:seekColor(r, g, b, a, time);
				end
			end
		else
			if self.anim.easeColorAction then
				self.anim.easeColorAction:stop();
			end

			self.anim:seekColor(r, g, b, a, time);
		end
   end

   Base.setColor( self, r, g, b, a );

end

----------------------------------------------------------------------

function ButtonAnim:setRot( x, y, z )

	if self.anim then
		if self.anim.root then
			self.anim.root:setRot(x,y,z);
		else
			self.anim:setRot(x,y,z);
		end
	end

end

----------------------------------------------------------------------

function ButtonAnim:setScale( sx, sy )

   if self.anim then
		if self.anim.root then
			self.anim.root:setScl( sx, sy );
		else
			self.anim:setScl( sx, sy );
		end
   end

   Base.setScale( self, sx, sy );

end

----------------------------------------------------------------------

function ButtonAnim:pauseLastFrame()

	if self.anim.root then
		self.anim:pauseLastFrame();
	end

end

----------------------------------------------------------------------

function ButtonAnim:getTime()

	if self.anim then
		return self.anim:getTime();
	end

end

----------------------------------------------------------------------

function ButtonAnim:getAnimName()

	if self.anim then
		if self.anim.root then

			return self.anim.props[1].animName;
		else

			return self.anim.animName;
		end
	end

end

----------------------------------------------------------------------

function ButtonAnim:setListener( event, func )

	if self.anim then
		self.anim:setListener(event, func);
	end

end

-------------------------------------------------------------------------------------

function ButtonAnim:clearAttrLink( attrID )

   if self.anim then
      if self.anim.root then
         self.anim.root:clearAttrLink( attrID );
      else
         self.anim:clearAttrLink( attrID );
      end
   end

end

-------------------------------------------------------------------------------------

function ButtonAnim:setAttrLink( attrID, parent, sourceAttrID )

   if self.anim then
      if self.anim.root then
         self.anim.root:setAttrLink( attrID , parent, sourceAttrID);
      else
         self.anim:setAttrLink( attrID , parent, sourceAttrID);
      end
   end

end

----------------------------------------------------------------------

function ButtonAnim:setParent( parent )

   if self.anim then
		if self.anim.root then
			self.anim.root:setParent(parent);
		else
			self.anim:setParent(parent);
		end
	end

end

----------------------------------------------------------------------

function ButtonAnim:setPriority( priority )

   if self.anim then
		local num = 0;
		if priority then
			num = priority;
		end
		if self.anim.root then
			for i=1,#self.anim.props do
				self.anim.props[i]:setPriority(num);
			end
		else
			self.anim:setPriority(num);
		end
		self.priority = num;
	end

end
----------------------------------------------------------------------

function ButtonAnim:setSpeed( speed )

   if self.anim then
		self.anim:setSpeed( speed );
	end

end

----------------------------------------------------------------------

function ButtonAnim:setMode( mode )

	if self.anim then
		if mode then
			self.anim:setMode(mode);
		else
			self.anim:setMode(MOAITimer.NORMAL);
		end
	end

end

----------------------------------------------------------------------

function ButtonAnim:setSpan( startTime, endTime )

	if self.anim then
		if endTime then
			self.anim:setSpan(startTime, endTime);
		else
			self.anim:setSpan(startTime);
		end
	end

end

----------------------------------------------------------------------

function ButtonAnim:apply( time )

	if self.anim and self.anim.root then
		self.anim:apply(time);
	end

end

----------------------------------------------------------------------

function ButtonAnim:setAnimFrame( frame, fps )

	if self.anim.root then

		if not fps then
			fps = 30;
		end

		if not frame then
			frame = 0;
		end

		local time = frame/fps;

		if time >= self.anim.sym.span then
			self:setAnimLastFrame();
		else
			self:stop();
			self:apply(time);
		end
	end

end

----------------------------------------------------------------------

function ButtonAnim:setAnimLastFrame( )

	if self.anim.root then

		local time = self.anim.sym.span - self.anim.sym.frame;

		self:stop();
		self:apply(time);
	end

end

----------------------------------------------------------------------

function ButtonAnim:setTime( time )

	if self.anim then
		self.anim:setTime(time);
	end

end

----------------------------------------------------------------------


function ButtonAnim:setVisible( visible )

   if self.anim then
		if self.anim.root then
			for i=1,#self.anim.props do
				self.anim.props[i]:setVisible(visible);
			end
		else
			self.anim:setVisible(visible);
		end
   end

	Base.setVisible( self, visible );

end

----------------------------------------------------------------------

function ButtonAnim:start()

	local action;

	if self.anim then
		action = self.anim:start();
	end

	return action;

end

----------------------------------------------------------------------

function ButtonAnim:stop()

	if self.anim and self.anim.stop then
		self.anim:stop();
	end

end

----------------------------------------------------------------------

function ButtonAnim:resetAnim()

	if self.anim.root then
		if self.autoReset ~= nil then
			if self.autoReset == true then
				self.autoReset = false;
				self:setAnimFrame();
			elseif self.autoReset == false then
				self:pauseLastFrame();
				self:setTime(0);
			end
		else
			self:setAnimFrame();
		end
	end

end

----------------------------------------------------------------------

function ButtonAnim:setInViewTest( test )

	self._isInView = test;

end

----------------------------------------------------------------------

function ButtonAnim:updateClick( down, x, y )

	Debug.assert(self, 'Animated Button nil on updateClick');
	Debug.assert(type(self) == 'table', 'Animated Button not a Table on updateclick');
	Debug.assert((self or {}).isEnabled, 'Animated Button isEnabled() not valid on updateclick');

	if not areButtonInputEnabled() then
		return false;
	end

	if self and
		type(self) == 'table' and
		self.isEnabled and
		self:isEnabled() then

		if self._isInView and not self._isInView() then

			if self:isPressed() then
				self:setPressed( false );
			end

			return;
		end

		if down then
			if self:isInside(x,y) then
				self:onPressed(x,y);
			end
		else
			if self:isInside(x,y) then

				if self:isPressed() then
					self:onReleased(x,y);
				end
			elseif self:isPressed() then
				self:setPressed(false);
				if not self.disableAutoAnim then
					self:setAnimFrame();
				end
			end
		end
	else
		self:setPressed( false );

		if self:isVisible()
			and self.isEnabled
			and self:isEnabled()
			and not self.disableAutoAnim then
			self:setAnimFrame();
		end
	end

end

----------------------------------------------------------------------

function ButtonAnim:updateMove( x, y )

	Debug.assert(self, 'Animated Button nil on updateMove');
	Debug.assert(type(self) == 'table', 'Animated Button not a Table on updateMove');
	Debug.assert((self or {}).isEnabled, 'Animated Button isEnabled() not valid on updateMove');

	if not areButtonInputEnabled() then
		return false;
	end

	if self and
		type(self) == 'table' and
		self.isEnabled and
		self:isEnabled() then

		if self._isInView and not self._isInView() then
			return;
		end

		if self:isInside(x,y) then
			if self:isPressed() then
				if not self.disableAutoAnim then
					self:setAnimFrame(BUTTON_PRESSED_FRAME);
				end
				self:onMoved(x,y);
			end
		else
			if self:isPressed() then
            self:setPressed(false);
				if not self.disableAutoAnim then
					self:setAnimFrame();
				end
            self:onCanceled(x,y);
			end
		end

	else

		if not self:isInside(x,y) and self:isPressed() then
			self:setPressed(false);
			if self:isVisible()
				and self.isEnabled
				and self:isEnabled()
				and not self.disableAutoAnim then
				self:setAnimFrame();
			end
         self:onCanceled(x,y);
		end

	end

end

----------------------------------------------------------------------

function ButtonAnim:onPressed(x, y)

   self:setPressed( true );

	if self.anim.root and self:isEnabled() and not self.disableAutoAnim then

		Base.playUI(self);
		self:setAnimFrame(BUTTON_PRESSED_FRAME);

		if self.autoReset ~= nil then
			self.autoReset = true;
		end
	end

   if self.onPressedFunc then

--		print("\nButtonAnim:onPressed", x, y);

      self.onPressedFunc(self.screen, self, x, y);
   end

end

----------------------------------------------------------------------

function ButtonAnim:onReleased(x, y)

	if self.anim.root and self:isEnabled() and not self.disableAutoAnim then

		self:setAnimLastFrame();

		if self.autoReset ~= nil then
			self.autoReset = true;
		end
	end

   if self.onReleasedFunc then
--		print("\nButtonAnim:onReleased", x, y);

		coroutine:yield();

		self:setPressed( false );
		self.onReleasedFunc(self.screen, self, x, y);
   end

end

----------------------------------------------------------------------

function ButtonAnim:onCanceled(x, y)

   if self.onCanceledFunc then

--		print("\nButtonAnim:onCanceled", x, y);

      self.onCanceledFunc(self.screen, self, x, y);
   end

end

----------------------------------------------------------------------

function ButtonAnim:onMoved(x, y)

	if self.onMovedFunc then
--		print("\nButtonAnim:onMoved", x, y);

      self.onMovedFunc(self.screen, self, x, y);
	end

end

----------------------------------------------------------------------

function ButtonAnim:isInside(x, y)

   local pad = self.pad;

   --When down, increase the press area
   if self:isPressed() then
      pad = math.max( pad or 0, 50);
   end

	if self.anim then
		if not self.anim.root then
			if pad then
				if self.anim:inside(x, y, 0, pad) then
					return true;
				else
					return false;
				end
			else
				if self.anim:inside(x, y) then
					return true;
				else
					return false;
				end
			end
		end


      for i=1,#self.anim.props do

         local prop = self.anim.props[i];

         if prop:inside(x,y,0,pad) then
            return true;
         end

      end

	end

end


----------------------------------------------------------------------

function fadeToBlack( fadeObj, time )

	local action;
	local t = 0.2;

	if time then
		t = time;
	end

	disableButtonInput();
	fadeObj:setVisible(true);
	fadeObj:setAlpha(1,1,1,0);
	action = fadeObj:seekAlpha(1,1,1,1,t, MOAIEaseType.SOFT_EASE_IN);

	MOAICoroutine.blockOnAction(action);

	enableButtonInput();

end

----------------------------------------------------------------------

function fadeFromBlack( fadeObj, time )

	local action;
	local t = 0.2;

	if time then
		t = time;
	end

	disableButtonInput();
	fadeObj:setVisible(true);
	fadeObj:setAlpha(1,1,1,1);
	action = fadeObj:seekAlpha(1,1,1,0,t, MOAIEaseType.SOFT_EASE_OUT);

	MOAICoroutine.blockOnAction(action);

	fadeObj:setVisible(false);
	enableButtonInput();

end

----------------------------------------------------------------------
