require('engine/texture/texture-manager')
require('ui/ui-button/ui-button')
require('app/template/audio/UI_sounds_table')
----------------------------------------------------------------------

local Base  = Button;
ButtonImage = Base:new();

----------------------------------------------------------------------

function ButtonImage:new( screen, parent, layer, x, y, w, h, texture, pressedFunc, releasedFunc, canceledFunc, movedFunc, hasParent, inViewTest )

   local button = Base.new(self, screen, parent, layer, x, y, w, h, pressedFunc, releasedFunc, canceledFunc, movedFunc, hasParent, inViewTest );
   
   button.sounds = UI_SOUNDS_TABLE;
   button.uiSoundName = texture;
   button.uiScreenName = screen.name;

	button.prop			= self:_createProp( layer, x, y, w, h, texture );
	button.hasParent	= hasParent;
	button._imageName	= texture;
	button.width		= w;
	button.height		= h;
	button.layer		= layer;

   return button;

end

----------------------------------------------------------------------

function ButtonImage:_createProp( layer, x, y, w, h, texture )

   local gfxQuad = MOAIGfxQuad2D.new ()
   gfxQuad:setTexture ( TextureMgr:getTexture(texture, true) )
   gfxQuad:setRect ( -w/2, -h/2, w/2, h/2 )

   local prop = MOAIProp2D.new ()
   prop:setDeck ( gfxQuad )
   prop:setPriority ( 0 )
   prop:setLoc(x,y)
   prop:setBlendMode( MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA );

   layer:insertProp ( prop )

	return prop;

end

----------------------------------------------------------------------

function ButtonImage:delete()

   if self.layer and self.prop then
      self.layer:removeProp( self.prop );
      self.prop = nil;
   end

   --must be at the end.. will clear the layer
   Base.delete( self );

end

----------------------------------------------------------------------

function ButtonImage:getImageName()

	if self.prop then
		return self._imageName;
	end

end

----------------------------------------------------------------------

function ButtonImage:getLoc( rel )
	
	local x = self.loc[1];
	local y = self.loc[2];

   if self.prop and self.hasParent and not rel then
		x,y = self.prop:modelToWorld()
   end

	return x,y;
	
end

----------------------------------------------------------------------

function ButtonImage:setLoc(x,y)

   if self.prop then
      self.prop:setLoc(x,y);
   end

   Base.setLoc(self,x,y)

end

----------------------------------------------------------------------

function ButtonImage:setLocX( x )

   if self.prop then
      local x2, y = self.prop:getLoc();
      self.prop:setLoc( x, y );
   end

   Base.setLocX( self, x );

end

----------------------------------------------------------------------

function ButtonImage:setLocY( y )

   if self.prop then
      local x = self.prop:getLoc();
      self.prop:setLoc( x, y );
   end

   Base.setLocY( self, y );

end

----------------------------------------------------------------------

function ButtonImage:setColor( r, g, b, a )

   if self.prop then
      self.prop:setColor(r, g, b, a);
   end

   Base.setColor( self, r, g, b, a );

end

----------------------------------------------------------------------

function ButtonImage:setScale( sx, sy )

   if self.prop then
      self.prop:setScl( sx, sy );
   end

   Base.setScale( self, sx, sy );

end

-------------------------------------------------------------------------------------

function ButtonImage:clearAttrLink( attrID )

	if self.prop then
		self.prop:clearAttrLink( attrID );
	end

end

-------------------------------------------------------------------------------------

function ButtonImage:setAttrLink( attrID, parent, sourceAttrID )

   if self.prop then
		self.prop:setAttrLink( attrID , parent, sourceAttrID);
   end

end

----------------------------------------------------------------------

function ButtonImage:setParent( parent )

   if self.prop then
		self.prop:setParent(parent);

		self.hasParent = parent;
	end

end

----------------------------------------------------------------------

function ButtonImage:setPriority( priority )

   if self.prop then
		if priority then
			self.prop:setPriority( priority );
			self.priority = priority;
		else
			self.prop:setPriority( 0 );
			self.priority = 0;
		end
   end

end

----------------------------------------------------------------------

function ButtonImage:setVisible( visible )

   if self.prop then
      self.prop:setVisible( visible );
   end

end

----------------------------------------------------------------------

function ButtonImage:swapImage( texture )

	if self.prop then

		if self:getImageName() == texture then
			return false;
		end

		local layer		= self.layer;
		local x,y 		= self:getLoc();

		layer:removeProp( self.prop );

		self.prop = self:_createProp(layer, x, y, self.width, self.height, texture);
		self._imageName = texture;
		
		if self.priority then
			self:setPriority(self.priority);
		end

		return true;
	end

end

