require('ui/ui-widget')
----------------------------------------------------------------------

local Base  = Widget;
Button      = Base:new();

----------------------------------------------------------------------

function Button:new( screen, parent, layer, x, y, w, h, pressedFunc, releasedFunc, canceledFunc, movedFunc, hasParent, inViewTest )

   --  print("Button:new", self, screen, parent, layer, x, y, w, h, pressedFunc, releasedFunc, canceledFunc, movedFunc, hasParent, inViewTest );

   local button = Widget.new( self, screen, parent, layer, x, y, w, h );

   button.pressed               = false;
   button.onPressedFunc         = pressedFunc;
   button.onReleasedFunc        = releasedFunc;
   button.onCanceledFunc        = canceledFunc;
   button.onMovedFunc           = movedFunc;
   button.hasParent             = hasParent;
   button.inputXtra             = {0,0}
   button._isInView             = inViewTest;
	button._pressedScale			  = {1.05, 1.05};
	button._baseScale				  = {1,	1};

   return button;

end

----------------------------------------------------------------------

function Button:delete()

   --must be at the end.. will clear the layer
   Base.delete( self );

end

----------------------------------------------------------------------

function Button:setInputXtra( x, y )

   self.inputXtra[1] = x or 0;
   self.inputXtra[2] = y or 0;

end

----------------------------------------------------------------------

function Button:setPressedScale( w, h )

   self._pressedScale[1] = w or 1.0;
   self._pressedScale[2] = h or 1.0;

end

----------------------------------------------------------------------

function Button:setBaseScale( w, h )

	self._baseScale[1]	= w or 1.0;
	self._baseScale[2]	= h or 1.0;

end

----------------------------------------------------------------------

function Button:setInViewTest( test )

	self._isInView = test;

end

----------------------------------------------------------------------

function Button:updateClick( down, x, y )

   if self:isEnabled() then

		if self._isInView and not self._isInView() then

			if self:isPressed() then
				self:setPressed( false );
				self:setScale(1,1);
			end

			return false;
		end

      if down then

         if self:isInside(x,y) then
            self:onPressed(x,y);
         end

      else

         if self:isPressed() then
            self:onReleased(x,y);
         end

      end
   end
end

----------------------------------------------------------------------

function Button:updateMove( x, y )

   if self:isEnabled() and self:isPressed() then

		if self._isInView and not self._isInView() then

			if self:isPressed() then
				self:setPressed( false );
				self:setScale(1,1);
			end

			return false;
		end

		if self:isInside( x, y ) then
			self:onMoved(x,y);
		else
			self:onCanceled(x,y);
		end
	end

end

----------------------------------------------------------------------

function Button:onPressed()

   self:setPressed( true );
   self:setScale(self._pressedScale[1], self._pressedScale[2]);
   
   self:playUI();

   if self.onPressedFunc then
      self.onPressedFunc(self.screen, self);
   end
end

----------------------------------------------------------------------

function Button:onReleased()

   self:setPressed( false );
   self:setScale(self._baseScale[1], self._baseScale[2]);

   if self.onReleasedFunc then
      self.onReleasedFunc(self.screen, self);
   end
end

----------------------------------------------------------------------

function Button:onCanceled()

   self:setPressed( false );
   self:setScale(self._baseScale[1], self._baseScale[2]);

   if self.onCanceledFunc then
      self.onCanceledFunc(self.parent, self);
   end
end

----------------------------------------------------------------------

function Button:onMoved()

	self:setPressed(true);
	self:setScale(self._pressedScale[1], self._pressedScale[2]);

	if self.onMovedFunc then
		self.onMovedFunc(self.parent, self);
	end

end

----------------------------------------------------------------------

function Button:isPressed()
   return self.pressed;
end

----------------------------------------------------------------------

function Button:setPressed( pressed )
   self.pressed = pressed;
end

----------------------------------------------------------------------

function Button:isEnabled()
   return self.enabled;
end

----------------------------------------------------------------------

function Button:enable()
   self.enabled = true;
end

----------------------------------------------------------------------

function Button:disable()
   self.enabled = false;
end

----------------------------------------------------------------------

function Button:setScale( sx, sy )

	local w	= sx or 1;
	local h	= sy or 1;

   self.scale = { w, h };

end

----------------------------------------------------------------------

function Button:isInside(x,y)

   local sx = self.loc[1];
   local sy = self.loc[2];

   if self.prop then
      sx,sy = self.prop:modelToWorld();
   end

   return math.abs(x-sx) <= (self.size[1]/2*math.abs(self.scale[1]) + math.abs(self.inputXtra[1]) )
      and math.abs(y-sy) <= (self.size[2]/2*math.abs(self.scale[2]) + math.abs(self.inputXtra[2]) );

end

----------------------------------------------------------------------

function Button:forceUpdate()
   if self.prop then
	  self.prop:forceUpdate();
   end
end

----------------------------------------------------------------------

function Button:playUI()

   local tblSounds = nil;
	local exclude = false;
   for i,v in pairs(self.sounds) do
      if v.buttonName == self.uiSoundName then
         if v.screenName then
            if v.screenName == self.uiScreenName then
					if v.exclude then
						exclude = true;
						break;
					else
						tblSounds=self.sounds[i];
						break;
					end
            end
         else
				if v.exclude then
					exclude = true;
				else
					tblSounds=self.sounds[i];
				end
         end
      end
   end
	if exclude then
		return;
	end

   if tblSounds == nil then tblSounds=self.sounds[1]; end
--   print("Button name:", self.uiSoundName);

   EventMgr:postEvent( {sounds=tblSounds, name="Button:updateClick",}, "add-sfx" );

end
