require('ui/ui-widget')
----------------------------------------------------------------------

local Base = Widget;
Image = Base:new();

----------------------------------------------------------------------

function Image:new( screen, parent, layer, x, y, w, h, texture, color )

   local image = Base.new( self, screen, parent, layer, x, y, w, h );

   image.prop				= image:createProp( layer, x, y, w, h, texture );
	image.prop.imageName	= texture;
	image.width				= w;
	image.height			= h;
	image.layer				= layer;

   if color then
      image:setColor( color[1], color[2], color[3], color[4] );
   end

   return image;

end

----------------------------------------------------------------------

function Image:delete( )

   if self.layer and self.prop then

      self.layer:removeProp( self.prop );
      self.prop = nil;

   end

   --must be at the end.. will clear the layer
	Base.delete( self );

end

----------------------------------------------------------------------

function Image:removeProp()

	if self.layer and self.prop then

		self.layer:removeProp( self.prop );
		self.prop = nil;

	end

end
----------------------------------------------------------------------

function Image:swapImage( texture )

   Debug.assertType(texture, "string", "Image:swapImage -> texture");

   if texture and self.prop then

      if self:getImageName() == texture then
         return false;
      end

      -- Saved properties; anything else is essentially erased
      local layer  = self.layer;
      local x, y   = self:getLoc();
      local parent = self.prop:getAttrLink(MOAIProp2D.INHERIT_TRANSFORM);

      self:removeProp();

      self.prop = self:createProp(layer, x, y, self.width, self.height, texture, parent);
      self.prop.imageName = texture;

      if self.priority then
         self:setPriority(self.priority);
      end

      return true;
   end

   return false;

end

----------------------------------------------------------------------

function Image:getLoc()

   local x;
   local y;

   if self.prop then
      x,y = self.prop:getLoc();
   end

   return x,y;

end

----------------------------------------------------------------------

function Image:getRot()

	if self.prop then
		return self.prop:getRot();
	end

end

----------------------------------------------------------------------

function Image:setLoc(x,y)

   if self.prop then
      self.prop:setLoc(x,y);
   end

   Base.setLoc(self,x,y)

end

----------------------------------------------------------------------

function Image:setLocX( x )

   if self.prop then
      local x2, y = self.prop:getLoc();
      self.prop:setLoc( x, y );
   end

   Base.setLocX( self, x );

end

----------------------------------------------------------------------

function Image:setLocY( y )

   if self.prop then
      local x = self.prop:getLoc();
      self.prop:setLoc( x, y );
   end

   Base.setLocY( self, y );

end

----------------------------------------------------------------------

function Image:seekLoc( x, y, time, ease )

	local action = nil;

   if self.prop then
		action = self.prop:seekLoc(x, y, time, ease);
   end

	return action;

end

----------------------------------------------------------------------

function Image:setColor( r, g, b, a )

   if self.prop then
      self.prop:setColor(r, g, b, a);
   end

   Base.setColor( self, r, g, b, a );

end

----------------------------------------------------------------------

function Image:setAlpha( a )

   if self.prop then
      self.prop:setColor(self.color[1], self.color[2], self.color[3],  a);
   end

   Base.setAlpha( self,  a );

end

----------------------------------------------------------------------

function Image:seekAlpha( a, time, ease )

   local action;
   local e = MOAIEaseType.SOFT_SMOOTH;

   if self.prop then

      if ease then
         e = ease;
      end

      action = self.prop:seekColor(self.color[1], self.color[2], self.color[3], a, time, e);
   end

   Base.setAlpha( self,  a );

   return action;

end

----------------------------------------------------------------------

function Image:seekScale( sx, sy, time, ease )

	local action;
	local e = MOAIEaseType.SOFT_SMOOTH;

	if self.prop then

		if ease then
			e = ease;
		end

		action = self.prop:seekScl(sx, sy, time, e);
	end

	Base.setScale( self, sx, sy );

	return action;

end

----------------------------------------------------------------------

function Image:setRot( x, y, z )

	if self.prop then
		self.prop:setRot(x,y,z);
	end

end

----------------------------------------------------------------------

function Image:setScale( sx, sy )

   if self.prop then
      self.prop:setScl( sx, sy );
   end

   Base.setScale( self, sx, sy );

end

-------------------------------------------------------------------------------------

function Image:clearAttrLink( attrID )

   if self.prop then
		self.prop:clearAttrLink( attrID );
   end

end

-------------------------------------------------------------------------------------

function Image:setAttrLink( attrID, parent, sourceAttrID )

   if self.prop then
		self.prop:setAttrLink( attrID , parent, sourceAttrID);
   end

end

----------------------------------------------------------------------

function Image:setParent( parent )

   if self.prop then
		self.prop:setParent(parent);
	end

end

----------------------------------------------------------------------

function Image:setPriority( priority )

   if self.prop then
		if priority then
			self.prop:setPriority( priority );
			self.priority = priority;
		else
			self.prop:setPriority( 0 );
			self.priority = 0;
		end
   end

end

----------------------------------------------------------------------

function Image:getImageName()

	if self.prop then
		return self.prop.imageName;
	end

	return nil;

end

----------------------------------------------------------------------

function Image:setVisible( visible )

   if self.prop then
      self.prop:setVisible( visible );
   end

   Base.setVisible( self, visible );

end

----------------------------------------------------------------------

function Image:forceUpdate()
   if self.prop then
	  self.prop:forceUpdate();
   end
end
