require('engine/texture/texture-manager')
require('ui/ui-button/ui-button-anim')
require('ui/ui-anim')
require('app/template/audio/UI_sounds_table')
----------------------------------------------------------------------

local ROLLOUT_OPEN_TIME			= 0.2;
local ROLLOUT_CLOSE_TIME		= 0.2;

local ROLLOUT_STATE_OPEN		= "open";
local ROLLOUT_STATE_CLOSED		= "closed";
local ROLLOUT_STATE_ROLLING	= "active";

----------------------------------------------------------------------

local Base	= Button;
RollOut		= Base:new();

----------------------------------------------------------------------

function RollOut:new(	screen,
								parent,
								layer,
								partition,
								x, y,
								shift,
								background_anim, background_spritesheet, background_posX, background_posY, background_priority,
								openButton_anim, openButton_spritesheet, openButton_posX, openButton_posY, openButton_priority,
								closeButton_anim, closeButton_spritesheet, closeButton_posX, closeButton_posY, closeButton_priority	)

   local rollOut						= Base.new(self, screen, parent, layer, x, y, 0, 0 );

	rollOut._state						= ROLLOUT_STATE_OPEN;
	rollOut._action					= nil;
	rollOut._elements					= {};

	rollOut.root						= MOAITransform2D.new();
	rollOut.rootHomeX					= x or 0;
	rollOut.rootHomeY					= y or 0;
	rollOut.root:setLoc(rollOut.rootHomeX, rollOut.rootHomeY);

	rollOut.shiftX						= shift[1];
	rollOut.shiftY						= shift[2];

	rollOut.background = screen:addWidget(Anim, layer, background_posX, background_posY, 0,0, background_anim, background_spritesheet);
	rollOut.background:setPriority(background_priority or 0);
	rollOut.background:setParent(rollOut.root);
	rollOut.background:setLoc(background_posX, background_posY);

	rollOut.openButton = screen:addWidget(ButtonAnim, layer, partition, openButton_posX, openButton_posY, 0,0, openButton_anim, openButton_spritesheet, nil, nil, nil, nil, nil, nil, 20);
	rollOut.openButton.onReleasedFunc =		function()
															rollOut:openRollOut();
														end
	rollOut.openButton:setPriority(openButton_priority or 0);
	rollOut.openButton:setParent(rollOut.root);
	rollOut.openButton:setLoc(openButton_posX, openButton_posY);
	rollOut.openButton:disable();
	rollOut.openButton:setVisible(false);

	rollOut.closeButton = screen:addWidget(ButtonAnim, layer, partition, closeButton_posX, closeButton_posY, 0,0, closeButton_anim, closeButton_spritesheet, nil, nil, nil, nil, nil, nil, 20);
	rollOut.closeButton.onReleasedFunc =	function()
															rollOut:closeRollOut();
														end
	rollOut.closeButton:setPriority(closeButton_priority or 0);
	rollOut.closeButton:setParent(rollOut.root);
	rollOut.closeButton:setLoc(closeButton_posX, closeButton_posY);

   return rollOut;

end


----------------------------------------------------------------------
-- List Utilities
----------------------------------------------------------------------

function RollOut:isRollOutClosed()

	return self._state == ROLLOUT_STATE_CLOSED;

end

----------------------------------------------------------------------

function RollOut:isRollOutOpen()

	return self._state == ROLLOUT_STATE_OPEN;

end

----------------------------------------------------------------------

function RollOut:getLoc()

	local x;
	local y;

   if self.root then
		x,y = self.root:getLoc();
   end

	return x,y;

end

----------------------------------------------------------------------

function RollOut:moveLoc( x, y, time, ease )

	local action;

   if self.root then
		action = self.root:moveLoc(x, y, 0, time, ease);

		local X, Y = self:getLoc();

		Base.setLoc(self, x+X, y+Y);

		return action;
   end

end

----------------------------------------------------------------------

function RollOut:seekLoc( x, y, time, ease )

   if self.root then
		local action = self.root:seekLoc(x, y, time, ease);

		Base.setLoc(self,x,y);

		return action;
   end

end

----------------------------------------------------------------------

function RollOut:setLoc( x, y )

   if self.root then
		self.root:setLoc(x,y);
   end

   Base.setLoc(self,x,y);

end

----------------------------------------------------------------------

function RollOut:setLocX( x )

   if self.root then
		local x2, y = self.root:getLoc();

		self.root:setLoc( x, y );

		Base.setLocX( self, x );
   end

end

----------------------------------------------------------------------

function RollOut:setLocY( y )

   if self.root then
		local x = self.root:getLoc();

		self.root:setLoc( x, y );

		Base.setLocY( self, y );
   end

end

----------------------------------------------------------------------

function RollOut:setParent( parent )

   if self.root then
		self.root:setParent(parent);
	end

end

----------------------------------------------------------------------

function RollOut:setRot( z )

	if self.root then
		self.root:setRot(z);
	end

end

----------------------------------------------------------------------

function RollOut:setScale( sx, sy )

   if self.root then
		self.root:setScl( sx, sy );

		Base.setScale( self, sx, sy );
   end

end

----------------------------------------------------------------------
-- Rollout Elements
----------------------------------------------------------------------

function RollOut:_setElementInViewTest( element, state )

	if element and state and element.setInViewTest then

		element:setInViewTest(
										function()
											return self._state == state;
										end
									);
	end

end

----------------------------------------------------------------------

function RollOut:_updateRollOutElements()

	if self._elements and next(self._elements) ~= nil then
		for k,v in pairs(self._elements) do
			local flag	= self._state == v.validState;

			if v.autoHide then
				self:setRollOutElementVisibile(v.element, flag);
			elseif v.autoDeactivate then

				if v.autoDeactivate() then
					v.element:disable()
					v.element:setColor(0.25, 0.25, 0.25, 1.0);
				else
					v.element:enable()
					v.element:setColor(1.0, 1.0, 1.0, 1.0);
				end
			end
		end
	end

end

----------------------------------------------------------------------

function RollOut:_updateRollOutRollingElements()

	if self._elements and next(self._elements) ~= nil and	self._state == ROLLOUT_STATE_ROLLING then

		for k,v in pairs(self._elements) do
			if v.rollingCheck then
				v.rollingCheck();
			end
		end
	end

end

----------------------------------------------------------------------

function RollOut:addRollOutElement( element, x, y, interactionState, autoHide, autoDeactivate, autoOpen, rollingCheck )

	local tbl		= {};

	tbl.element				= element;
	tbl.autoHide			= autoHide or false;
	tbl.validState			= interactionState or ROLLOUT_STATE_OPEN;
	tbl.autoDeactivate	= autoDeactivate;
	tbl.autoOpen			= autoOpen;
	tbl.rollingCheck		= rollingCheck;

	if not self._elements then
		self._elements = {};
	end

	self._elements[element]	= tbl;

	self:_setElementInViewTest(element, interactionState or ROLLOUT_STATE_OPEN);
	element:setParent(self.root);
	element:setLoc(x,y);
	self:_updateRollOutElements();

end

----------------------------------------------------------------------

function RollOut:isObjectInRollOut( element )

	if self._elements and element and self._elements[element] then
		return true;
	end

end

----------------------------------------------------------------------

function RollOut:setRollOutElementVisibile(element, vis)

	if self._elements and element and self._elements[element] then

		local flag;

		if vis ~= nil then
			flag = vis
		else
			flag = true;
		end

		if self.elements[element].element.anim and not flag then
			self.elements[element].element:stop();
		end

		if self.elements[element].element.disable then
			if flag then
				self.elements[element].element:enable();
			else
				self.elements[element].element:disable();
			end
		end

		self.elements[element].element:setVisible(flag);
	end

end

----------------------------------------------------------------------

function RollOut:shouldElementAutoOpen( element )

	local flag = false;

	if self._elements and element and self._elements[element] then
		if self._elements[element].autoOpen then
			if self._elements[element].autoDeactivate and not self._elements[element].autoDeactivate() then
				flag = self._elements[element].autoOpen();
			else
				flag = self._elements[element].autoOpen();
			end
		end
	end

	return flag;

end

----------------------------------------------------------------------

function RollOut:updateRollOutAutoOpen()

	local flag = false;

	if self._elements and next(self._elements) ~= nil then
		for k,v in pairs(self._elements) do
			local t = self:shouldElementAutoOpen(v.element);

			if t and not flag then
				flag = true;
			end
		end
	end


   if not GuideMgr:isBusy() then
      if flag then
         self:setRollOutOpened();
      else
         self:setRollOutClosed();
      end
   else
      flag = nil;
   end

	return flag;

end

----------------------------------------------------------------------
-- Rolling In/Out
----------------------------------------------------------------------

function RollOut:_closeRollOut()

	self._state	= ROLLOUT_STATE_CLOSED;

	self.root:setLoc(self.rootHomeX - self.shiftX, self.rootHomeY - self.shiftY);
	self.closeButton:disable();
	self.closeButton:setVisible(false);
	self.openButton:enable();
	self.openButton:setVisible(true);
	self:_updateRollOutElements();

end

----------------------------------------------------------------------

function RollOut:_openRollOut()

	self._state	= ROLLOUT_STATE_OPEN;

	self.root:setLoc(self.rootHomeX, self.rootHomeY);
	self.closeButton:enable();
	self.closeButton:setVisible(true);
	self.openButton:disable();
	self.openButton:setVisible(false);
	self:_updateRollOutElements();

end

----------------------------------------------------------------------

function RollOut:closeRollOut()

	if self._state ~= ROLLOUT_STATE_CLOSED then

		if self._action then
			self._action:stop();
			self._action = nil;
		end

		self.closeButton:disable();
		self.openButton:disable();

		self._state		= ROLLOUT_STATE_ROLLING;
		self._action	= self.root:seekLoc(self.rootHomeX - self.shiftX, self.rootHomeY - self.shiftY, ROLLOUT_CLOSE_TIME, MOAIEaseType.EASE_IN);

		self:_updateRollOutRollingElements();
		self._action:setListener(	MOAIAction.EVENT_STOP,
											function ()
												self:_closeRollOut()
											end
										);
	end

end

----------------------------------------------------------------------

function RollOut:openRollOut()

	if self._state ~= ROLLOUT_STATE_OPEN then

		if self._action then
			self._action:stop();
			self._action = nil;
		end

		self.closeButton:disable();
		self.openButton:disable();

		self._state		= ROLLOUT_STATE_ROLLING;
		self._action	= self.root:seekLoc(self.rootHomeX, self.rootHomeY, ROLLOUT_CLOSE_TIME, MOAIEaseType.EASE_IN);

		self:_updateRollOutRollingElements();
		self._action:setListener(	MOAIAction.EVENT_STOP,
											function ()
												self:_openRollOut()
											end
										);
	end

end

----------------------------------------------------------------------

function RollOut:setRollOutClosed()

	if self._action then
		self._action:stop();
		self._action = nil;
	end

	self:_closeRollOut();

end

----------------------------------------------------------------------

function RollOut:setRollOutOpened()

	if self._action then
		self._action:stop();
		self._action = nil;
	end

	self:_openRollOut();

end
