require('engine/texture/texture-manager')
require('ui/ui-button/ui-button-anim')
require('ui/ui-anim')
require('app/template/audio/UI_sounds_table')
----------------------------------------------------------------------

local SCROLL_SPEED_MIN_BASE		= 0.001;
local SCROLL_SPEED_MAX_BASE		= 0.500;

local SCROLL_INPUT_TIME_MIN		= 1/30;
local SCROLL_INPUT_TIME_MAX		= 40/30;

local SCROLL_TIME_MIN_BASE			= 0.60;
local SCROLL_TIME_MAX_BASE			= 2.25;

local SCROLL_MOVE_DIST				= 12;

----------------------------------------------------------------------

local Base	= Button;
ScrollList	= Base:new();

----------------------------------------------------------------------

function ScrollList:new(	screen,
									parent,
									layer,
									x, y,
									viewWindow,
									onPressWindow,
									onReleaseWindow,
									entryViewBuffer,
									entrySpacing,
									entryCount,
									swipeAttenuation,
									swipeBuffer,
									swipeBounceSpace,
									swipeResetTime,
									sweetSpot,
									anim_X, spritesheet_X, animX_posX, animX_posY, animX_width, priority_X,
									anim_Y, spritesheet_Y, animY_posX, animY_posY, animY_height, priority_Y )

   local scrollList					= Base.new(self, screen, parent, layer, x, y, 0, 0 );

	scrollList.root					= MOAITransform2D.new();
	scrollList.rootHomeX				= x or 0;
	scrollList.rootHomeY				= y or 0;

	if viewWindow == nil then
		viewWindow = {{0,0}, {0,0}};
	end

	scrollList.viewMinX				= viewWindow[1][1];
	scrollList.viewMaxX				= viewWindow[1][2];
	scrollList.viewMinY				= viewWindow[2][1];
	scrollList.viewMaxY				= viewWindow[2][2];

	scrollList.viewCenter			= { (scrollList.viewMinX+scrollList.viewMaxX)/2, (scrollList.viewMinY+scrollList.viewMaxY)/2	};

	scrollList.scrollDistX			= scrollList.viewMaxX - scrollList.viewMinX;
	scrollList.scrollDistY			= scrollList.viewMaxY - scrollList.viewMinY;

	if onPressWindow then
		scrollList.pressMinX				= onPressWindow[1][1] or scrollList.viewMinX;
		scrollList.pressMaxX				= onPressWindow[1][2] or scrollList.viewMaxX;
		scrollList.pressMinY				= onPressWindow[2][1] or scrollList.viewMinY;
		scrollList.pressMaxY				= onPressWindow[2][2] or scrollList.viewMaxY;
	else
		scrollList.pressMinX				= scrollList.viewMinX;
		scrollList.pressMaxX				= scrollList.viewMaxX;
		scrollList.pressMinY				= scrollList.viewMinY;
		scrollList.pressMaxY				= scrollList.viewMaxY;
	end

	if onReleaseWindow then
		scrollList.releaseMinX			= onReleaseWindow[1][1] or -App.viewportWidth/2;
		scrollList.releaseMaxX			= onReleaseWindow[1][2] or App.viewportWidth/2;
		scrollList.releaseMinY			= onReleaseWindow[2][1] or -App.viewportHeight/2;
		scrollList.releaseMaxY			= onReleaseWindow[2][2] or App.viewportHeight/2;
	else
		scrollList.releaseMinX			= -App.viewportWidth/2;
		scrollList.releaseMaxX			= App.viewportWidth/2;
		scrollList.releaseMinY			= -App.viewportHeight/2;
		scrollList.releaseMaxY			= App.viewportHeight/2;
	end

	if entrySpacing then
		scrollList.entrySpacingX		= entrySpacing[1] or 1;
		scrollList.entrySpacingY		= entrySpacing[2] or 1;
	else
		scrollList.entrySpacingX		= 1;
		scrollList.entrySpacingY		= 1;
	end

	if swipeBounceSpace then
		scrollList.bounceBufferX		= swipeBounceSpace[1] or 0;
		scrollList.bounceBufferY		= swipeBounceSpace[2] or 0;
	else
		scrollList.bounceBufferX		= 0;
		scrollList.bounceBufferY		= 0;
	end

	scrollList.resetTime					= swipeResetTime or 0.1;

	if entryCount then
		scrollList.entryCountX			= entryCount[1] or 0;
		scrollList.entryCountY			= entryCount[2] or 0;
	else
		scrollList.entryCountX			= 0;
		scrollList.entryCountY			= 0;
	end

	if entryViewBuffer then
		scrollList.entryViewBufferX	= entryViewBuffer[1] or (1/3)*scrollList.entrySpacingX;
		scrollList.entryViewBufferY	= entryViewBuffer[2] or (1/3)*scrollList.entrySpacingY;
	else
		scrollList.entryViewBufferX	= (1/3)*scrollList.entrySpacingX;
		scrollList.entryViewBufferY	= (1/3)*scrollList.entrySpacingY;
	end

	scrollList.entryViewMinX		= scrollList.viewMinX + scrollList.entryViewBufferX;
	scrollList.entryViewMaxX		= scrollList.viewMaxX - scrollList.entryViewBufferX;
	scrollList.entryViewMinY		= scrollList.viewMinY + scrollList.entryViewBufferY;
	scrollList.entryViewMaxY		= scrollList.viewMaxY - scrollList.entryViewBufferY;

	scrollList.entryVisMaxX			= math.floor( (scrollList.viewMaxX - scrollList.viewMinX)/scrollList.entrySpacingX );
	scrollList.entryVisMaxY			= math.floor( (scrollList.viewMaxY - scrollList.viewMinY)/scrollList.entrySpacingY );

	scrollList.scrollXEnabled		= scrollList.entryCountX > scrollList.entryVisMaxX	;
	scrollList.scrollYEnabled		= scrollList.entryCountY > scrollList.entryVisMaxY	;

	scrollList.rootMinX				= scrollList.rootHomeX - (scrollList.entryCountX*scrollList.entrySpacingX) + (scrollList.viewMaxX - scrollList.viewMinX) - 5;
	scrollList.rootMaxY				= scrollList.rootHomeY + (scrollList.entryCountY*scrollList.entrySpacingY) - (scrollList.viewMaxY - scrollList.viewMinY) + 5;

	scrollList.rootTravX				= scrollList.rootHomeX - scrollList.rootMinX;
	scrollList.rootTravY				= scrollList.rootMaxY - scrollList.rootHomeY;

	if swipeAttenuation then
		scrollList.scrollAttenuation = math.mapClamped(0, 1, 1, 1/100, swipeAttenuation)
	else
		if scrollList.entryCountX > 0 and scrollList.entryCountY > 0 then
			if scrollList.entryCountX > scrollList.entryCountY then
				scrollList.scrollAttenuation = math.mapClamped(1, 43/100, 5, 1, scrollList.entryCountX/scrollList.entryVisMaxX);
			else
				scrollList.scrollAttenuation = math.mapClamped(1, 43/100, 5, 1, scrollList.entryCountY/scrollList.entryVisMaxY);
			end
		elseif scrollList.entryCountX > 0 then
			scrollList.scrollAttenuation = math.mapClamped(1, 43/100, 5, 1, scrollList.entryCountX/scrollList.entryVisMaxX);
		elseif scrollList.entryCountY > 0 then
			scrollList.scrollAttenuation = math.mapClamped(1, 43/100, 5, 1, scrollList.entryCountY/scrollList.entryVisMaxY);
		else
			scrollList.scrollAttenuation = 1;
		end
	end

	if swipeBuffer then
		scrollList.swipeBufferX			= swipeBuffer[1] or 0;
		scrollList.swipeBufferY			= swipeBuffer[2] or 0;
	else
		scrollList.swipeBufferX			= 0;
		scrollList.swipeBufferY			= 0;
	end

	if sweetSpot then
		scrollList.sweetSpotMinX		= sweetSpot[1][1];
		scrollList.sweetSpotMaxX		= sweetSpot[1][2];
		scrollList.sweetSpotMinY		= sweetSpot[2][1];
		scrollList.sweetSpotMaxY		= sweetSpot[2][2];
	else
		scrollList.sweetSpotMinX		= scrollList.viewMinX;
		scrollList.sweetSpotMaxX		= scrollList.viewMaxX;
		scrollList.sweetSpotMinY		= scrollList.viewMinY;
		scrollList.sweetSpotMaxY		= scrollList.viewMaxY;
	end

	scrollList.scrollBarX			= nil;
	scrollList.scrollBarY			= nil;

	scrollList.scrollBarXHomeX		= animX_posX or 0;
	scrollList.scrollBarXHomeY		= animX_posY or 0;
	scrollList.scrollBarXWidth		= animX_width or 1;

	scrollList.scrollBarYHomeX		= animY_posX or 0;
	scrollList.scrollBarYHomeY		= animY_posY or 0;
	scrollList.scrollBarYHeight	= animY_height or 1;

	scrollList.scrollBarTravX		= scrollList.viewMaxX - scrollList.viewMinX - scrollList.scrollBarXWidth;
	scrollList.scrollBarTravY		= scrollList.viewMaxY - scrollList.viewMinY - scrollList.scrollBarYHeight;

	scrollList.scrollBarXMaxX		= scrollList.scrollBarXHomeX + scrollList.scrollBarTravX;
	scrollList.scrollBarYMinY		= scrollList.scrollBarYHomeY - scrollList.scrollBarTravY;

	scrollList.scrollBarScaleX		= (scrollList.rootHomeX - scrollList.rootMinX )/scrollList.scrollBarTravX;
	scrollList.scrollBarScaleY		= (scrollList.rootMaxY - scrollList.rootHomeY)/scrollList.scrollBarTravY;

	scrollList.swipeTracker			= {};
	scrollList.swipeStartPosX		= nil;
	scrollList.swipeStartPosY		= nil;
	scrollList.scrollStartPosX		= nil;
	scrollList.scrollStartPosY		= nil;
	scrollList.scrollBarStartPosX	= nil;
	scrollList.scrollBarStartPosY	= nil;
	scrollList.validSwipe			= false;
	scrollList.rootAction			= nil;
	scrollList.scrollBarActionX	= nil;
	scrollList.scrollBarActionY	= nil;
	scrollList.enabled				= true;
    scrollList.visible              = true;

	if anim_X then
		scrollList.scrollBarX = screen:addWidget(Anim, layer, scrollList.scrollBarXHomeX, scrollList.scrollBarXHomeY, 0,0, anim_X, spritesheet_X, nil, nil, true);
		scrollList.scrollBarX:setPriority(priority_X);
		scrollList.scrollBarX:setVisible(scrollList.scrollXEnabled);
	end

	if anim_Y then
		scrollList.scrollBarY = screen:addWidget(Anim, layer, scrollList.scrollBarYHomeX, scrollList.scrollBarYHomeY, 0,0, anim_Y, spritesheet_Y, nil, nil, true);
		scrollList.scrollBarY:setPriority(priority_Y);
		scrollList.scrollBarY:setVisible(scrollList.scrollYEnabled);
	end

	scrollList.root:setLoc(scrollList.rootHomeX, scrollList.rootHomeY);


   return scrollList;

end

----------------------------------------------------------------------
-- List Utilities
----------------------------------------------------------------------


function ScrollList:enable()
	self.enabled = true;
end

----------------------------------------------------------------------

function ScrollList:disable()
	self.enabled = false;
end

----------------------------------------------------------------------

function ScrollList:isEnabled()
	return self.enabled;
end

----------------------------------------------------------------------

function ScrollList:isVisible()
   return self.visible;
end

----------------------------------------------------------------------

function ScrollList:hide()

    self:disable();

    self.visible = false;

    if self.rootAction then
       self.rootAction:stop();
       self.rootAction = nil;
    end

    if self._entryCacheObjects and self._entryCacheObjects[1] and self._entryCacheObjects[1][1] then

        for _, v in ipairs(self._entryCacheObjects) do
            for _, val in ipairs(v) do
                self:hideCacheObject(val);
            end
        end
    elseif self._entryCacheObjects and self._entryCacheObjects[1] then
        for _, v in ipairs(self._entryCacheObjects) do
            self:hideCacheObject(v);
        end
    end

    if self.scrollBarX then
        self.scrollBarX:setVisible(false);
    end

    if self.scrollBarY then
        self.scrollBarY:setVisible(false);
    end

end

----------------------------------------------------------------------

function ScrollList:show()

    self:enable();

    self.visible = true;

    if self._entryData and self._entryCacheObjects then
        self:_setEntryCacheToMap(true);
        self:_hideEntryCacheEmpties();
    end

    if self.scrollBarX then
        self.scrollBarX:setVisible(self.scrollXEnabled);
    end

    if self.scrollBarY then
        self.scrollBarY:setVisible(self.scrollYEnabled);
    end

end

---------------------------------------------------------------------

function ScrollList:isScrollListBusy()

	return self.rootAction and self.rootAction:isBusy();

end

----------------------------------------------------------------------

function ScrollList:isEntryInView( index, offsetX, offsetY, ignoreDisable )

	if not self:isEnabled() and not ignoreDisable then
		return false;
	end

	if self.rootAction and self.rootAction:isBusy() then
		return false;
	end

	if self.scrollXEnabled and self.scrollYEnabled then
		if self:getDistanceMovedSqr() > ( SCROLL_MOVE_DIST * SCROLL_MOVE_DIST ) then
			return false;
		end
	elseif self.scrollXEnabled then
		if self:getDistanceMovedX() > SCROLL_MOVE_DIST then
			return false;
		end
	elseif self.scrollYEnabled then
		if self:getDistanceMovedY() > SCROLL_MOVE_DIST then
			return false;
		end
	end

	if self.root then
		local x				= offsetX or 0;
		local y				= offsetY or 0;
		local locX,locY	= self:getLoc();

		locX = locX + self:getEntryOffsetX(index) + x;
		locY = locY + self:getEntryOffsetY(index) + y;

		if math.isInBoundary(locX, locY, self.entryViewMinX, self.entryViewMaxX, self.entryViewMinY, self.entryViewMaxY) then
			return true;
		else
			return false;
		end
	end

end

----------------------------------------------------------------------

function ScrollList:getEntryOffsetX( index )

	if self.root then
		return self.entrySpacingX*(index - 1);
	end

end

----------------------------------------------------------------------

function ScrollList:getEntryOffsetY( index )

   if self.root then
	  return -(self.entrySpacingY*(index - 1));
   end

end

----------------------------------------------------------------------

function ScrollList:getDistanceMoved()
	return math.sqrt( self:getDistanceMovedSqr() );
end

----------------------------------------------------------------------

function ScrollList:getDistanceMovedSqr()

	if			self.root
		and	self:isPressed()
		and	self.swipeStartPosX
		and	self.swipeStartPosY
		and	self.currentTouchPosX
		and	self.currentTouchPosY then

		local x	= self:getDistanceMovedX();
		local y	= self:getDistanceMovedY();

		return ( ( x * x ) + ( y * y ) );

	end

	return 0;

end

----------------------------------------------------------------------

function ScrollList:getDistanceMovedX()

	if			self.root
		and	self:isPressed()
		and	self.swipeStartPosX
		and	self.currentTouchPosX then

		return math.abs(self.swipeStartPosX - self.currentTouchPosX);

	end

	return 0;

end

----------------------------------------------------------------------

function ScrollList:getDistanceMovedY()

	if			self.root
		and	self:isPressed()
		and	self.swipeStartPosY
		and	self.currentTouchPosY then

		return math.abs(self.currentTouchPosY - self.swipeStartPosY);

	end

	return 0;

end

----------------------------------------------------------------------

function ScrollList:getLoc()

   local x;
   local y;

   if self.root then
	  x,y = self.root:getLoc();
   end

   return x,y;

end

----------------------------------------------------------------------

function ScrollList:moveLoc( x, y, time, ease )

   if self.root then
	  local action = self.root:moveLoc(x, y, 0, time, ease);

	  local X, Y = self:getLoc();

	  Base.setLoc(self, x+X, y+Y);

	  return action;
   end

end

----------------------------------------------------------------------

function ScrollList:seekLoc( x, y, time, ease )

   if self.root then
	  local action = self.root:seekLoc(x, y, time, ease);

	  Base.setLoc(self,x,y);

	  return action;
   end

end

----------------------------------------------------------------------

function ScrollList:setEntryCount( x, y )

	self.entryCountX					= x or 0;
	self.entryCountY					= y or 0;

	self.rootMinX						= self.rootHomeX - (self.entryCountX*self.entrySpacingX) + (self.viewMaxX - self.viewMinX) - 5;
	self.rootMaxY						= self.rootHomeY + (self.entryCountY*self.entrySpacingY) - (self.viewMaxY - self.viewMinY) + 5;

	self.rootTravX						= self.rootHomeX - self.rootMinX;
	self.rootTravY						= self.rootMaxY - self.rootHomeY;

	self.scrollXEnabled				= self.entryCountX > self.entryVisMaxX	;
	self.scrollYEnabled				= self.entryCountY > self.entryVisMaxY	;

	if self.entryCountX > 0 and self.scrollXEnabled then

		local pos	= self.rootMinX + self:getEntryOffsetX(self.entryCountX) + self.entrySpacingX/2;
		local delta	= self.viewMaxX - pos;

		if delta > 0 then
			self.rootMinX	= self.rootMinX + delta;
			self.rootTravX	= self.rootHomeX - self.rootMinX;
		end
	end

	if self.entryCountY > 0 and self.scrollYEnabled then

		local pos	= self.rootMaxY + self:getEntryOffsetY(self.entryCountY) - self.entrySpacingY/2;
		local delta	= pos - self.viewMinY;

		if delta > 0 then
			self.rootMaxY	= self.rootMaxY - delta;
			self.rootTravY	= self.rootMaxY - self.rootHomeY;
		end
	end

	self.scrollBarScaleX				= (self.rootHomeX - self.rootMinX )/self.scrollBarTravX;
	self.scrollBarScaleY				= (self.rootMaxY - self.rootHomeY)/self.scrollBarTravY;

	if self.scrollBarX then
		self.scrollBarX:setVisible(self.scrollXEnabled);
	end

	if self.scrollBarY then
		self.scrollBarY:setVisible(self.scrollYEnabled);
	end

end

----------------------------------------------------------------------

function ScrollList:modTraversalY( delta )

   self.rootTravY       = self.rootTravY + delta;
   self.rootMaxY        = self.rootTravY + self.rootHomeY;
   self.scrollBarScaleY = (self.rootMaxY - self.rootHomeY)/self.scrollBarTravY;

end

----------------------------------------------------------------------

function ScrollList:setLoc( x, y )

   if self.root then
		self.root:setLoc(x,y);
   end

   Base.setLoc(self,x,y);

end

----------------------------------------------------------------------

function ScrollList:setLocX( x )

   if self.root then
		local _, y = self.root:getLoc();

		self.root:setLoc( x, y );

		Base.setLocX( self, x );
   end

end

----------------------------------------------------------------------

function ScrollList:setLocY( y )

   if self.root then
		local x = self.root:getLoc();

		self.root:setLoc( x, y );

		Base.setLocY( self, y );
   end

end

----------------------------------------------------------------------

function ScrollList:setParent( parent )

   if self.root then
		self.anim:setParent(parent);
	end

end

----------------------------------------------------------------------

function ScrollList:setRot( x, y, z )

	if self.root then
		self.root:setRot(x,y,z);
	end

end

----------------------------------------------------------------------

function ScrollList:setScale( sx, sy )

   if self.root then
		self.root:setScl( sx, sy );

		Base.setScale( self, sx, sy );
   end

end

----------------------------------------------------------------------

function ScrollList:setScrollAttenuation( attenuation )

	if attenuation then
		self.scrollAttenuation = math.mapClamped(0, 1, 1, 1/100, attenuation)
	else
		if self.entryCountX > 0 and self.entryCountY > 0 then
			if self.entryCountX > self.entryCountY then
				self.scrollAttenuation = math.mapClamped(1, 43/100, 5, 1, self.entryCountX/self.entryVisMaxX);
			else
				self.scrollAttenuation = math.mapClamped(1, 43/100, 5, 1, self.entryCountY/self.entryVisMaxY);
			end
		elseif self.entryCountX > 0 then
			self.scrollAttenuation = math.mapClamped(1, 43/100, 5, 1, self.entryCountX/self.entryVisMaxX);
		elseif self.entryCountY > 0 then
			self.scrollAttenuation = math.mapClamped(1, 43/100, 5, 1, self.entryCountY/self.entryVisMaxY);
		else
			self.scrollAttenuation = 1;
		end
	end

end


----------------------------------------------------------------------
-- Input Detection
----------------------------------------------------------------------

function ScrollList:updateClick( down, x, y )

	if not areButtonInputEnabled() or not self:isEnabled() then
		return false;
	end

	if not self.scrollXEnabled and not self.scrollYEnabled then
		return false;
	end

	if self:isEnabled() then
		if down then
			if self:isInsidePressWindow(x,y) and not self:isPressed() then
				self:onPressed(x, y);
			end
		else
			self:onReleased(x, y);
		end
	end

end

----------------------------------------------------------------------

function ScrollList:updateMove( x, y )

	if not areButtonInputEnabled() or not self:isEnabled() then
		return false;
	end

	if not self.scrollXEnabled and not self.scrollYEnabled then
		return false;
	end

   if self:isEnabled() then
		if self:isInsideReleaseWindow(x,y)
			and self:isPressed()
			and (self:isInsideSweetSpot(x,y) or self:isOutsideSwipeBuffer(x,y)) then

			self:onMove(x,y);
		end
	end

end
----------------------------------------------------------------------

function ScrollList:isInsidePressWindow( x, y )

	if self.root then
		if math.isInBoundary(x, y, self.pressMinX, self.pressMaxX, self.pressMinY, self.pressMaxY) then
			return true;
		else
			return false;
		end
	end

end

----------------------------------------------------------------------

function ScrollList:isInsideReleaseWindow( x, y )

	if self.root then
		if math.isInBoundary(x, y, self.releaseMinX, self.releaseMaxX, self.releaseMinY, self.releaseMaxY) then
			return true;
		else
			return false;
		end
	end

end

----------------------------------------------------------------------

function ScrollList:isInsideSweetSpot( x, y )

	if not self.sweetSpotMinX then
		return false;
	end

	if self.root then

		if math.isInBoundary(x, y, self.sweetSpotMinX, self.sweetSpotMaxX, self.sweetSpotMinY, self.sweetSpotMaxY) then
			return true;
		else
			return false;
		end
	end

end

----------------------------------------------------------------------

function ScrollList:isOutsideSwipeBuffer( x, y )

	if not self.swipeStartPosX or not self.swipeStartPosX then
		return false;
	end

	if self.swipeBufferX == 0 and self.swipeBufferY == 0 then
		return true;
	end

	if self.root then
		if math.isInBoundary(x, y, (self.swipeStartPosX-self.swipeBufferX), (self.swipeStartPosX+self.swipeBufferX), (self.swipeStartPosY-self.swipeBufferY), (self.swipeStartPosY+self.swipeBufferY)) then
			return false;
		else
			return true;
		end
	end

end

----------------------------------------------------------------------

function ScrollList:onMove( x, y )

    if not areButtonInputEnabled()  or not self:isEnabled() then
        return false;
    end

	self.validSwipe = true;
	self.currentTouchPosX	= x;
	self.currentTouchPosY	= y;

	self:_addSwipeTrackerPos();
	self:updateScrollPosition(x,y);

end

----------------------------------------------------------------------

function ScrollList:onPressed( x, y )

	self.onPressedTime				= MOAISim.getElapsedTime();

	self:scrollVelocityStop();

	if self:isInsideSweetSpot(x,y) then
		self.validSwipe = true;
	end

	self.swipeStartPosX										= x;
	self.swipeStartPosY										= y;
	self.currentTouchPosX									= x;
	self.currentTouchPosY									= y;
	self.scrollStartPosX, self.scrollStartPosY		= self:getLoc();

	if self.scrollBarX then
		self.scrollBarXStartPosX, self.scrollBarXStartPosY	= self.scrollBarX:getLoc();
	end

	if self.scrollBarY then
		self.scrollBarYStartPosX, self.scrollBarYStartPosY	= self.scrollBarY:getLoc();
	end

	self:setPressed(true);
	self:_addSwipeTrackerPos();

end

----------------------------------------------------------------------

function ScrollList:onReleased( x, y )

	self.onReleasedTime				= MOAISim.getElapsedTime();

	self:setPressed(false);
	self:_addSwipeTrackerPos();

	if self.validSwipe then
		self:scrollVelocityStart();
	else
		self:scrollVelocityReset();
	end

	self.currentTouchPosX			= nil;
	self.currentTouchPosY			= nil;
	self.swipeStartPosY				= nil;
	self.swipeStartPosX				= nil;
	self.swipeStartPosY				= nil;
	self.scrollStartPosX				= nil;
	self.scrollStartPosY				= nil;
	self.scrollBarXStartPosX		= nil;
	self.scrollBarXStartPosY		= nil;
	self.scrollBarYStartPosX		= nil;
	self.scrollBarYStartPosY		= nil;
	self.validSwipe					= false;
	self.swipeTracker					= {};

end


----------------------------------------------------------------------
-- Scrolling
----------------------------------------------------------------------

function ScrollList:updateScrollPosition(x, y)

	local deltaX	= 0;
	local deltaY	= 0;
	local targetX	= 0;
	local targetY	= 0;
	local barPosX	= 0;
	local barPosY	= 0;

	if not self.scrollXEnabled and not self.scrollYEnabled then
		return false;
	end

	if self.scrollXEnabled then
		deltaX = x - self.swipeStartPosX;
	end

	if self.scrollYEnabled then
		deltaY = y - self.swipeStartPosY;
	end

	targetX = math.clamp(self.scrollStartPosX + deltaX, self.rootMinX-self.bounceBufferX, self.rootHomeX+self.bounceBufferX);
	targetY = math.clamp(self.scrollStartPosY + deltaY, self.rootHomeY-self.bounceBufferY, self.rootMaxY+self.bounceBufferY);

	if self.scrollBarX then
		self.scrollBarX:setVisible(self.scrollXEnabled);

		if self.scrollXEnabled then
			barPosX = math.mapClamped(-1, self.scrollBarXHomeX, 0, self.scrollBarXMaxX, -(self.rootHomeX - targetX)/self.rootTravX);

			self.scrollBarX:setLoc(barPosX ,self.scrollBarXHomeY);
		end
	end

	if self.scrollBarY then
		self.scrollBarY:setVisible(self.scrollYEnabled);

		if self.scrollYEnabled then
			barPosY = math.mapClamped(-1, self.scrollBarYMinY, 0, self.scrollBarYHomeY, -(targetY - self.rootHomeY)/self.rootTravY);

			self.scrollBarY:setLoc(self.scrollBarYHomeX, barPosY);
		end
	end

	if self.scrollXEnabled and self.scrollYEnabled then
		self:setLoc(targetX, targetY);
	elseif self.scrollXEnabled then
		self:setLoc(targetX, self.rootHomeY);
	elseif self.scrollYEnabled then
		self:setLoc(self.rootHomeX, targetY);
	end

--	print('ScrollList:updateScrollPosition', targetX, targetY, barPosX, barPosY);

end

----------------------------------------------------------------------

function ScrollList:_addSwipeTrackerPos()

   local X, Y = self:getLoc();

   table.insert(self.swipeTracker, { time = MOAISim.getElapsedTime(), x = X, y = Y} );

   if #self.swipeTracker > 5 then
      table.remove(self.swipeTracker, 1);
   end

end

----------------------------------------------------------------------

function ScrollList:_monitorScrollActivity()

	while true do

		if self.rootAction and self.rootAction:isBusy() then

			coroutine.yield();
			self:_setEntryCacheToMap();
		else
			self:scrollVelocityReset();
			self:_setEntryCacheToMap();

			return true;
		end

	end

end

----------------------------------------------------------------------

function ScrollList:advanceToEntry( index, time, callback )

   local x = self.rootHomeX;
   local y = self.rootHomeY;

   if self.scrollXEnabled then
      x = math.clamp(self.rootHomeX - self:getEntryOffsetX(index), self.rootMinX, self.rootHomeX);
   end

   if self.scrollYEnabled then
      y = math.clamp(self.rootHomeY - self:getEntryOffsetY(index), self.rootHomeY, self.rootMaxY);
   end

   if self.rootAction then
      self.rootAction:stop();
   end

   if self.scrollBarX and self.scrollXEnabled then

      local xBar = math.mapClamped(-1, self.scrollBarXHomeX, 0, self.scrollBarXMaxX, (self.rootHomeX - x)/self.rootTravX);

      self.scrollBarX:setVisible(self.scrollXEnabled and self:isEnabled());

      if self.scrollXEnabled then
         self.scrollBarActionX = self.scrollBarX:seekLoc(xBar, self.scrollBarXHomeY, time, MOAIEaseType.SOFT_EASE_IN);
      end
   end

   if self.scrollBarY and self.scrollYEnabled then

      local yBar = math.mapClamped(-1, self.scrollBarYMinY, 0, self.scrollBarYHomeY, -(y - self.rootHomeY)/self.rootTravY);

      self.scrollBarY:setVisible(self.scrollYEnabled and self:isEnabled());

      if self.scrollYEnabled then
         self.scrollBarActionY = self.scrollBarY:seekLoc(self.scrollBarYHomeX, yBar, time, MOAIEaseType.SOFT_EASE_IN);
      end
   end

   if self.scrollXEnabled and self.scrollYEnabled then
      self.rootAction = self:seekLoc(x, y, time, MOAIEaseType.SOFT_EASE_IN);
   elseif self.scrollXEnabled then
      self.rootAction = self:seekLoc(x, self.rootHomeY, time, MOAIEaseType.SOFT_EASE_IN);
   elseif self.scrollYEnabled then
      self.rootAction = self:seekLoc(self.rootHomeX, y, time, MOAIEaseType.SOFT_EASE_IN);
   end

   if self.rootAction then

      local thread = MOAICoroutine.new();

      thread:run( function()
            self:_monitorScrollActivity();
      end );

      self.rootAction:setListener( MOAIAction.EVENT_STOP,
                                   function()
                                      self.rootAction = nil;
                                      --self:scrollVelocityReset();

                                      if callback then
                                         callback();
                                      end
                                   end
      );

   end

end

----------------------------------------------------------------------

function ScrollList:goToEntry( index, force )

	local x		= self.rootHomeX;
	local y		= self.rootHomeY;

	if self.scrollXEnabled then
		x = math.clamp(self.rootHomeX - self:getEntryOffsetX(index), self.rootMinX, self.rootHomeX);
	end

	if self.scrollYEnabled then
		y = math.clamp(self.rootHomeY - self:getEntryOffsetY(index), self.rootHomeY, self.rootMaxY);
	end

	if self.rootAction then
		self.rootAction:stop();
	end

	if self.scrollBarX and self.scrollXEnabled then

		local xBar = math.mapClamped(-1, self.scrollBarXHomeX, 0, self.scrollBarXMaxX, (self.rootHomeX - x)/self.rootTravX);

		self.scrollBarX:setVisible(self.scrollXEnabled and self:isEnabled());

		if self.scrollXEnabled then
			self.scrollBarActionX = self.scrollBarX:setLoc(xBar, self.scrollBarXHomeY);
		end
	end

	if self.scrollBarY and self.scrollYEnabled then

		local yBar = math.mapClamped(-1, self.scrollBarYMinY, 0, self.scrollBarYHomeY, -(y - self.rootHomeY)/self.rootTravY);

		self.scrollBarY:setVisible(self.scrollYEnabled and self:isEnabled());

		if self.scrollYEnabled then
			self.scrollBarActionY = self.scrollBarY:setLoc(self.scrollBarYHomeX, yBar);
		end
	end

	if self.scrollXEnabled and self.scrollYEnabled then
		self:setLoc(x, y);
	elseif self.scrollXEnabled then
		self:setLoc(x, self.rootHomeY);
	elseif self.scrollYEnabled then
		self:setLoc(self.rootHomeX, y);
	end

	self:_setEntryCacheToMap(force);

end

----------------------------------------------------------------

function ScrollList:scrollVelocityReset()

	local x,y		= self:getLoc();
	local xClamp	= math.clamp(x, self.rootMinX, self.rootHomeX);
	local yClamp	= math.clamp(y, self.rootHomeY, self.rootMaxY);
	local xBar		= math.mapClamped(-1, self.scrollBarXHomeX, 0, self.scrollBarXMaxX, -(self.rootHomeX - xClamp)/self.rootTravX);
	local yBar		= math.mapClamped(-1, self.scrollBarYMinY, 0, self.scrollBarYHomeY, -(yClamp - self.rootHomeY)/self.rootTravY);
	local time		= self.resetTime;

	self:scrollVelocityStop();

	if	self.scrollXEnabled and self.scrollYEnabled and
		x <= self.rootHomeX and x >= self.rootMinX and
		y <= self.rootMaxY and y >= self.rootHomeY then

		return false;
	elseif	self.scrollXEnabled and
				x <= self.rootHomeX and x >= self.rootMinX then

		return false;
	elseif	self.scrollYEnabled and
				y <= self.rootMaxY and y >= self.rootHomeY then

		return false;
	end

	if self.scrollBarX and self.scrollXEnabled then
		self.scrollBarX:setVisible(self.scrollXEnabled);

		if self.scrollXEnabled then
			self.scrollBarActionX = self.scrollBarX:seekLoc(xBar, self.scrollBarXHomeY, time, MOAIEaseType.SOFT_EASE_IN);
		end
	end

	if self.scrollBarY and self.scrollYEnabled then
		self.scrollBarY:setVisible(self.scrollYEnabled);

		if self.scrollYEnabled then
			self.scrollBarActionY = self.scrollBarY:seekLoc(self.scrollBarYHomeX, yBar, time, MOAIEaseType.SOFT_EASE_IN);
		end
	end

	if self.scrollXEnabled and self.scrollYEnabled then
		self.rootAction = self:seekLoc(xClamp, yClamp, time, MOAIEaseType.SOFT_EASE_IN);
	elseif self.scrollXEnabled then
		self.rootAction = self:seekLoc(xClamp, self.rootHomeY, time, MOAIEaseType.SOFT_EASE_IN);
	elseif self.scrollYEnabled then
		self.rootAction = self:seekLoc(self.rootHomeX, yClamp, time, MOAIEaseType.SOFT_EASE_IN);
	end

--	print('ScrollList:scrollVelocityReset', xClamp, yClamp, xBar, yBar);

end

----------------------------------------------------------------

function ScrollList:scrollVelocityStart( x, y )

	local locX, locY = self:getLoc();

	if	self.scrollXEnabled and
		(locX >= self.rootHomeX or locX <= self.rootMinX) then

		self:scrollVelocityReset();

		return true;
	end

	if	self.scrollYEnabled and
		(locY >= self.rootMaxY or locY <= self.rootHomeY) then

		self:scrollVelocityReset();

		return true;
	end

   local deltaTime			= self.swipeTracker[#self.swipeTracker].time - self.swipeTracker[1].time;
	local deltaX				= self.swipeTracker[#self.swipeTracker].x - self.swipeTracker[1].x;
	local deltaY				= self.swipeTracker[#self.swipeTracker].y - self.swipeTracker[1].y;

	if deltaTime == 0 or (deltaY == 0 and deltaX == 0) then
      self:scrollVelocityReset();

		return false;
	end

	local scrollRateX			= self.scrollAttenuation*(deltaX/deltaTime);
	local scrollRateY			= self.scrollAttenuation*(deltaY/deltaTime);
	local time					= math.mapClamped( SCROLL_INPUT_TIME_MIN, SCROLL_TIME_MIN_BASE, SCROLL_INPUT_TIME_MAX, SCROLL_TIME_MAX_BASE,  (self.onReleasedTime - self.onPressedTime));
	local deltaScrollX		= scrollRateX*time;
	local deltaScrollY		= scrollRateY*time;
	local X,Y					= self:getLoc();
	local targetX				= math.clamp(X + deltaScrollX, (self.rootMinX - self.bounceBufferX), (self.rootHomeX + self.bounceBufferX));
	local targetY				= math.clamp(Y + deltaScrollY, (self.rootHomeY - self.bounceBufferY), (self.rootMaxY + self.bounceBufferY));

	-- change scroll time and distance if list is very short
	if self.scrollXEnabled and self.scrollYEnabled then
		distance = math.distance(X, Y, targetX, targetY);
		if distance <= self.scrollDistX and distance <= self.scrollDistY then
			time = math.mapClamped(0, 0.2, math.max(self.scrollDistX, self.scrollDistY), SCROLL_TIME_MIN_BASE, distance);
			deltaScrollX		= scrollRateX*time;
			deltaScrollY		= scrollRateY*time;
			targetX				= math.clamp(X + deltaScrollX, (self.rootMinX - self.bounceBufferX), (self.rootHomeX + self.bounceBufferX));
			targetY				= math.clamp(Y + deltaScrollY, (self.rootHomeY - self.bounceBufferY), (self.rootMaxY + self.bounceBufferY));
		end
	elseif self.scrollXEnabled then
		local distance = math.distance(X, Y, targetX, Y);
		if distance <= self.scrollDistX then
			time = math.mapClamped(0, 0.2, self.scrollDistX, SCROLL_TIME_MIN_BASE, distance);
			deltaScrollX		= scrollRateX*time;
			targetX				= math.clamp(X + deltaScrollX, (self.rootMinX - self.bounceBufferX), (self.rootHomeX + self.bounceBufferX));
		end
	elseif self.scrollYEnabled then
		local distance = math.distance(X, Y, X, targetY);
		if distance <= self.scrollDistY then
			time = math.mapClamped(0, 0.2, self.scrollDistY, SCROLL_TIME_MIN_BASE, distance);
			deltaScrollY		= scrollRateY*time;
			targetY				= math.clamp(Y + deltaScrollY, (self.rootHomeY - self.bounceBufferY), (self.rootMaxY + self.bounceBufferY));
		end
	end

	local xBar					= math.mapClamped(-1, self.scrollBarXHomeX, 0, self.scrollBarXMaxX, -(self.rootHomeX - targetX)/self.rootTravX);
	local yBar					= math.mapClamped(-1, self.scrollBarYMinY, 0, self.scrollBarYHomeY, -(targetY - self.rootHomeY)/self.rootTravY);
	local distance				= 0;

	if not self.scrollXEnabled then
		targetX = X;
	elseif targetX == (self.rootMinX - self.bounceBufferX) and X < self.rootMinX then
		targetX = self.rootMinX;
	elseif targetX == (self.rootHomeX + self.bounceBufferY) and X > self.rootHomeX then
		targetX = self.rootHomeX;
	end

	if not self.scrollYEnabled then
		targetY = Y;
	elseif targetY == (self.rootHomeY - self.bounceBufferY) and Y < self.rootHomeY then
		targetY = self.rootHomeY;
	elseif targetY == (self.rootMaxY + self.bounceBufferY) and Y > self.rootMaxY then
		targetY = self.rootMaxY;
	end

	if self.scrollBarX then
		self.scrollBarX:setVisible(self.scrollXEnabled);

		if self.scrollXEnabled then
			self.scrollBarActionX = self.scrollBarX:seekLoc(xBar, self.scrollBarXHomeY, time, MOAIEaseType.SOFT_EASE_IN);
		end
	end

	if self.scrollBarY then
		self.scrollBarY:setVisible(self.scrollYEnabled);

		if self.scrollYEnabled then
			self.scrollBarActionY = self.scrollBarY:seekLoc(self.scrollBarYHomeX, yBar, time, MOAIEaseType.SOFT_EASE_IN);
		end
	end

	if self.scrollXEnabled and self.scrollYEnabled then
		self.rootAction = self:seekLoc(targetX, targetY, time, MOAIEaseType.SOFT_EASE_IN);
	elseif self.scrollXEnabled then
		self.rootAction = self:seekLoc(targetX, self.rootHomeY, time, MOAIEaseType.SOFT_EASE_IN);
	elseif self.scrollYEnabled then
		self.rootAction = self:seekLoc(self.rootHomeX, targetY, time, MOAIEaseType.SOFT_EASE_IN);
	end

	local thread = MOAICoroutine.new();

	thread:run(	function()
          self:_monitorScrollActivity();
    end );

	self.rootAction:setListener( MOAIAction.EVENT_STOP,
                                 function()
                                    self:scrollVelocityReset();
                                 end
	);

--	print('ScrollList:scrollVelocityStart - 1', self.scrollAttenuation, scrollRateX, scrollRateY, time, deltaScrollX, deltaScrollY);
--	print('ScrollList:scrollVelocityStart - 2', X, Y, targetX, targetY, xBar, yBar, self.entryCountX, self.entryCountY, self.entryVisMaxX, self.entryVisMaxY);

end

----------------------------------------------------------------

function ScrollList:scrollVelocityStop()

	if self.scrollBarActionX then
		self.scrollBarActionX:stop();
		self.scrollBarActionX = nil;
	end

	if self.scrollBarActionY then
		self.scrollBarActionY:stop();
		self.scrollBarActionY = nil;
	end

	if self.rootAction then
		self.rootAction:stop();
		self.rootAction = nil;
	end

end


----------------------------------------------------------------------
-- Entries Cache and Data (Optional: for creating cached entries with data)
----------------------------------------------------------------------

function ScrollList:_createEntryDataCacheMap()

	if self:_isEntryDataCacheValid() then

		local cacheMaxX;
		local cacheMaxY;
		local countX;
		local countY;

		self:_checkDataCacheMismatch();

		if			self._entryCacheObjects[1] and self._entryData[1]
			and	not self._entryCacheObjects[1][1] and not self._entryData[1][1] then
			cacheMaxY	= #self._entryCacheObjects;
			countY		= 1;
		else
			cacheMaxX	= #self._entryCacheObjects[1];
			cacheMaxY	= #self._entryCacheObjects;
			countX		= 1;
			countY		= 1;
		end

		self._entryDataCacheMap = {};

		for i = 1, #self._entryData do

			if countY > cacheMaxY then
				countY = 1;
			end

			if cacheMaxX then

               self._entryDataCacheMap[i] = {};

               for j=1, #self._entryData[i] do
                  if countX > cacheMaxX then
                     countX = 1;
                  end

                  self._entryDataCacheMap[i][j]                  = self._entryData[i][j];
                  self._entryDataCacheMap[i][j]._entryCacheIndex = {countX, countY};
                  self._entryDataCacheMap[i][j]._entryOffset     = {self:getEntryOffsetX(j), self:getEntryOffsetY(i)};
                  countX = countX + 1;
               end
			else

               Debug.assert(self._entryData[i] ~= nil);

               self._entryDataCacheMap[i]                  = self._entryData[i];
               self._entryDataCacheMap[i]._entryCacheIndex = countY;
               self._entryDataCacheMap[i]._entryOffset     = self:getEntryOffsetY(i);
            end

			countY = countY + 1;

		end

	end

    -- Check for duplicate entries
    if BUILD_DEBUG then

       self:_checkForDuplicateEntries();

    end

end

----------------------------------------------------------------------

function ScrollList:_checkForDuplicateEntries()

   local duplicateEntryTracker = {};

   for _, entry in ipairs(self._entryDataCacheMap) do

      if duplicateEntryTracker[entry] then

         Debug.error("Duplicate entry found in scroll list!" .. tostring(entry));

      end

      duplicateEntryTracker[entry] = true;

   end

end

----------------------------------------------------------------------

function ScrollList:_checkDataCacheMismatch()

	local cache2D = false;
	local data2D  = false;

	Debug.assert( self._entryCacheObjects, "ScrollList:_checkDataCacheMismatch -> self._entryCacheObjects" );
	Debug.assert( self._entryData, "ScrollList:_checkDataCacheMismatch -> self._entryData" );

	if			self._entryCacheObjects
		and	self._entryCacheObjects[1]
		and	type(self._entryCacheObjects[1]) == "table"
		and	self._entryCacheObjects[1][1] then

		cache2D	= true;
	end

	if			self._entryData
		and	self._entryData[1]
		and	type(self._entryData[1]) == "table"
		and	self._entryData[1][1] then

		data2D	= true;
	end

	if	cache2D ~= data2D then
		Debug.error("Scroll List cache object and scroll data dimensions mismatch");
	end

end

----------------------------------------------------------------------

function ScrollList:_getEntryDataCacheMapOffsetX( ix, iy )

	if			self._entryDataCacheMap
		and	self._entryDataCacheMap[iy]
		and	self._entryDataCacheMap[iy][ix]
		and	self._entryDataCacheMap[iy][ix]._entryOffset then

		return self._entryDataCacheMap[iy][ix]._entryOffset[1];
	end

end

----------------------------------------------------------------------

function ScrollList:_getEntryDataCacheMapOffsetY( index )

	local twoDMap	= self:_isEntryDataCacheMap2D();

	if twoDMap then
		return self._entryDataCacheMap[index][1]._entryOffset[2];
	else
		return self._entryDataCacheMap[index]._entryOffset;
	end

end

----------------------------------------------------------------------

function ScrollList:_getEntryDataCacheMapRange()

	local twoDMap	= self:_isEntryDataCacheMap2D();
	local x,y		= self:getLoc();
	local cacheY	= math.ceil(#self._entryCacheObjects/2);
	local ly			= { self.viewCenter[2] - self.entrySpacingY*cacheY, self.viewCenter[2] + self.entrySpacingY*cacheY	};
	local minY;
	local maxY;

	if #self._entryDataCacheMap <= #self._entryCacheObjects then
		minY	= 1;
		maxY	= #self._entryDataCacheMap;
	else

		for i=1, #self._entryDataCacheMap do

			local py = y + self:_getEntryDataCacheMapOffsetY(i);

			if py < ly[2] and not minY then
				minY = i;
			end

			if i == #self._entryDataCacheMap then
				if not minY then
					minY = i;
				end
			end

			if minY then
				minY = math.clamp(minY, 1, #self._entryDataCacheMap);
				maxY = math.clamp(minY + (#self._entryCacheObjects-1), 1, #self._entryDataCacheMap);

				break;
			end
		end
	end

	if twoDMap then
		return self:_getEntryDataCacheMapRange2D(x,y), {minY, maxY};
	else
		return nil, {minY, maxY};
	end

end

----------------------------------------------------------------------

function ScrollList:_getEntryDataCacheMapRange2D( x, y )

	local cacheX	= math.ceil(#self._entryCacheObjects[1]/2);
	local lx			= { self.viewCenter[1] - self.entrySpacingX*cacheX, self.viewCenter[1] + self.entrySpacingX*cacheX	};
	local minX;
	local maxX;

	if #self._entryDataCacheMap[1] <= #self._entryCacheObjects[1] then
		minX	= 1;
		maxX	= #self._entryDataCacheMap[1];
	else
		for i=1, #self._entryDataCacheMap[1] do

			local px = x + self:_getEntryDataCacheMapOffsetX(i, 1);

			if px > lx[1] and not minX then
				minX = i;
			end

			if i == #self._entryDataCacheMap[1] then
				if not minX then
					minX = i;
				end
			end

			if minX then
				minX = math.clamp(minX, 1, #self._entryDataCacheMap[1]);
				maxX = math.clamp(minX + (#self._entryCacheObjects[1]-1), 1, #self._entryDataCacheMap[1]);

				break;
			end
		end
	end

	return {minX, maxX};

end

----------------------------------------------------------------------

function ScrollList:_hideEntryCacheEmpties()

	if self:_isEntryDataCacheValid() then

		if self:_isEntryDataCacheMap2D() then

			for i=1, #self._entryData do

				if self._entryCacheObjects[i] then
					local dx = #self._entryCacheObjects[i] > #self._entryData[i] ;

					if dx then
						for j=#self._entryData[i]+1, #self._entryCacheObjects[i] do
							self:_hideEntryDataCacheObject(self._entryCacheObjects[i][j]);
						end
					end
				end
			end
		else

			if #self._entryCacheObjects > #self._entryData then
				for i=#self._entryData+1, #self._entryCacheObjects do
					self:_hideEntryDataCacheObject(self._entryCacheObjects[i]);
				end
			end
		end
	end

end

----------------------------------------------------------------------

function ScrollList:_hideEntryDataCacheObject( obj )

   if not obj then
      return;
   end

   for k,v in pairs(obj) do
      if type(v) == "table" then
         if v.anim and v.anim.root then
            v:stop();
         end

         if v.onPressedFunc or v.onReleasedFunc or v.onCanceledFunc or v.onMovedFunc then
            v:disable();
         end

         if v.setVisible then
            v:setVisible(false);
         elseif v.hide then
            v:hide();
         end
      end
   end

end

----------------------------------------------------------------------

function ScrollList:_showEntryDataCacheObject( obj )

   if not obj then
      return;
   end

   for _, v in pairs(obj) do
      if type(v) == "table" then
         if v.onPressedFunc or v.onReleasedFunc or v.onCanceledFunc or v.onMovedFunc then
            v:enable();
         end

         if v.setVisible then
            v:setVisible(true);
         elseif v.show then
            v:show();
         end
      end
   end

end

----------------------------------------------------------------------

function ScrollList:_isEntryDataCacheMap2D()

	if self._entryDataCacheMap and self:_isEntryDataCacheValid() then
		if self._entryDataCacheMap[1] and self._entryDataCacheMap[1][1] then
			return true;
		end
	end

    return false;

end

----------------------------------------------------------------------

function ScrollList:_isEntryDataCacheValid()

	return self._entryCacheObjects and self._entryData and self._setDataToObject and self._setObjectLoc and self._setObjectToParent;

end

----------------------------------------------------------------------

-- must set setEntryObjectDataUpdateFunction first
function ScrollList:_setEntryCacheToMap( force )

	if self:_isEntryDataCacheValid() and self._entryDataCacheMap and self._setDataToObject then

		local rX, rY = self:_getEntryDataCacheMapRange();

		for i=rY[1],rY[2] do

			if rX then

				for j=rX[1],rX[2] do

					local row = self._entryDataCacheMap[i][j]._entryCacheIndex[2];
					local col = self._entryDataCacheMap[i][j]._entryCacheIndex[1];

					if			self._entryCacheObjects[row][col]._entryDataIndex[1] ~= j
						or		self._entryCacheObjects[row][col]._entryDataIndex[2] ~= i
						or	force then

						self._entryCacheObjects[row][col]._entryDataIndex[1] = j;
						self._entryCacheObjects[row][col]._entryDataIndex[2] = i;

						self._setDataToObject(self._entryDataCacheMap[i][j], self._entryCacheObjects[row][col]);
						self._setObjectLoc(self._entryDataCacheMap[i][j]._entryOffset[1], self._entryDataCacheMap[i][j]._entryOffset[2], self._entryCacheObjects[row][col]);
					end
				end
			else

				if self._entryCacheObjects[self._entryDataCacheMap[i]._entryCacheIndex]._entryDataIndex ~= i or force then

                   self._entryCacheObjects[self._entryDataCacheMap[i]._entryCacheIndex]._entryDataIndex = i;

                   self._setDataToObject(self._entryDataCacheMap[i], self._entryCacheObjects[self._entryDataCacheMap[i]._entryCacheIndex]);
                   self._setObjectLoc(0, self._entryDataCacheMap[i]._entryOffset, self._entryCacheObjects[self._entryDataCacheMap[i]._entryCacheIndex]);
				end
			end
		end
	end

end

----------------------------------------------------------------------

function ScrollList:forceUpdateCacheToMap()
	self:_setEntryCacheToMap(true);
end

----------------------------------------------------------------------

function ScrollList:hideCacheObject( obj )
    self:_hideEntryDataCacheObject(obj);
end

----------------------------------------------------------------------

function ScrollList:showCacheObject( obj )
    self:_showEntryDataCacheObject(obj);
end

----------------------------------------------------------------------

function ScrollList:hideCacheEntry( obj )

   if obj then
      for k,v in pairs(obj) do
         if type(v) == "table" then
            if v.stop then
               v:stop();
            end

            if v.disable then
               v:disable();
            end

            if v.setVisible then
               v:setVisible(false);
            elseif v.hide then
               v:hide();
            end
         end
      end
    end

end

----------------------------------------------------------------------

function ScrollList:isEntryCacheObjInView( obj, ignoreDisable )

    if obj then

		if not self:isEnabled() and not ignoreDisable then
			return false;
		end

		if self.rootAction and self.rootAction:isBusy() then
			return false;
		end

		if self.scrollXEnabled and self.scrollYEnabled then
			if self:getDistanceMovedSqr() > ( SCROLL_MOVE_DIST * SCROLL_MOVE_DIST ) then
				return false;
			end
		elseif self.scrollXEnabled then
			if self:getDistanceMovedX() > SCROLL_MOVE_DIST then
				return false;
			end
		elseif self.scrollYEnabled then
			if self:getDistanceMovedY() > SCROLL_MOVE_DIST then
				return false;
			end
		end

		local x,y = obj:modelToWorld();

		if math.isInBoundary(x, y, self.entryViewMinX, self.entryViewMaxX, self.entryViewMinY, self.entryViewMaxY) then
			return true;
		else
			return false;
		end
	end

end

----------------------------------------------------------------------

function ScrollList:getEntryCacheObject( row, col )

	if self._entryCacheObjects == nil then
		return;
	end

	if row and col then
		return self._entryCacheObjects[col][row];
	else
		return self._entryCacheObjects[row];
	end

end

----------------------------------------------------------------------

-- must set the following before creating cache objects: setEntryObjectDataUpdateFunctions, setEntryObjectLocUpdateFunction, and setEntryObjectParentUpdateFunction
function ScrollList:setEntryCacheObjects( countX, countY, dataFunc )

	if			dataFunc
		and	not self._entryCacheObjects
		and	self._setDataToObject
		and	self._setObjectLoc
		and	self._setObjectToParent then

		local cX	= math.max(math.max(self.entryCountX, 1), countX or 1);
		local cY	= math.max(math.max(self.entryCountY, 1), countY or 1);

		self._entryCacheObjects = {};

		for i=cY,1,-1 do

			if cX and cX > 1 then
				local tempA = {};
				--self._entryCacheObjects[i] = {};

				for j=1,cX do
					local tempB = {};

					tempB							= dataFunc();
					tempB._entryDataIndex	= {0,0};

					self._setObjectToParent(self.root, tempB);
					table.insert(tempA, 1, tempB);
				end

				table.insert(self._entryCacheObjects, 1, tempA)
			else
				local newCacheEntry				= dataFunc();
				newCacheEntry._entryDataIndex	= 0;

				self._setObjectToParent(self.root, newCacheEntry);
				table.insert(self._entryCacheObjects, 1, newCacheEntry);
			end
		end
	end

end

----------------------------------------------------------------------

function ScrollList:setEntryData( data, hideList )

   if not Debug.assertType(data, "table", "ScrollList:setEntryData -> data") then
      return;
   end

   self._entryData = data;

   self:_createEntryDataCacheMap();

   if self:_isEntryDataCacheValid() then
      self:_setEntryCacheToMap(true);
      self:_hideEntryCacheEmpties();
   end

   if hideList == true then
      self:hide();
   else
      self:show();
   end

end

----------------------------------------------------------------------

-- function requires two parameters - data and object
-- this also needs to check if any buttons are in view
function ScrollList:setEntryObjectDataUpdateFunction( func )

   Debug.assertType(func, "function", "ScrollList:setEntryObjectDataUpdateFunction -> func");

   if func then
      self._setDataToObject = func;
   end

end

----------------------------------------------------------------------

-- function requires three parameters - x, y, and object
function ScrollList:setEntryObjectLocUpdateFunction( func )

   Debug.assertType(func, "function", "ScrollList:setEntryObjectLocUpdateFunction -> func");

   if func then
      self._setObjectLoc = func;
   end

end

----------------------------------------------------------------------

-- function requires two parameters - target and object
function ScrollList:setEntryObjectParentUpdateFunction( func )

   Debug.assertType(func, "function", "ScrollList:setEntryObjectParentUpdateFunction -> func");

   if func then
      self._setObjectToParent = func;
   end

end
