require('engine/texture/texture-manager')
require('ui/ui-scroll-list')
require('ui/ui-button/ui-button-anim')
require('ui/ui-anim')
require('app/template/audio/UI_sounds_table')
----------------------------------------------------------------------

local DEFAULT_SNAP_TIME	= 0.25;

----------------------------------------------------------------------

local Base	= ScrollList;
SnapList		= Base:new();

----------------------------------------------------------------------


function SnapList:new(	screen,
								parent,
								layer,
								x, y,
								viewWindow,
								onPressWindow,
								onReleaseWindow,
								entrySpacing,
								entryCount,
								minSwipeDistance,
								snapTime,
								snapPage,
								sweetSpot,
								anim_X, spritesheet_X, animX_posX, animX_posY, animX_width, priority_X,
								anim_Y, spritesheet_Y, animY_posX, animY_posY, animY_height, priority_Y )

   local SnapList					= Base.new(		self,
															screen,
															parent,
															layer,
															x, y,
															viewWindow,
															onPressWindow,
															onReleaseWindow,
															{0, 0}, --entryViewBuffer
															entrySpacing,
															entryCount,
															nil, --swipeAttenuation
															nil, --swipeBuffer
															{0, 0}, --swipeBounceSpace
															nil, --swipeResetTime
															sweetSpot,
															anim_X, spritesheet_X, animX_posX, animX_posY, animX_width, priority_X,
															anim_Y, spritesheet_Y, animY_posX, animY_posY, animY_height, priority_Y
														);

	if minSwipeDistance then
		SnapList.minSwipeDeltaX = minSwipeDistance[1];
		SnapList.minSwipeDeltaY = minSwipeDistance[2];
	else
		SnapList.minSwipeDeltaX = App.viewportWidth/3;
		SnapList.minSwipeDeltaY = App.viewportHeight/3;
	end

	if snapTime then
		SnapList.snapTime			= snapTime;
	else
		SnapList.snapTime			= DEFAULT_SNAP_TIME;
	end

	SnapList._snapPage		= snapPage;
	SnapList._snapIncX		= 1;
	SnapList._snapIncY		= 1;

	SnapList._indexX			= 1;
	SnapList._indexY			= 1;

   return SnapList;

end


----------------------------------------------------------------------
-- List Utilities
----------------------------------------------------------------------


function SnapList:cancelSwipeAction()

	Base.onReleased(self, 0,0);

end

----------------------------------------------------------------------

function SnapList:getCurrentIndexX()

	return self._indexX;

end

----------------------------------------------------------------------

function SnapList:getCurrentIndexY()

	return self._indexY;

end

----------------------------------------------------------------------

function SnapList:isSnapListBusy( )

	if self.rootAction and self.rootAction:isBusy() then

		return true;
	end

	return false;

end


----------------------------------------------------------------------
-- Set Callback Functions
----------------------------------------------------------------------


-- function requires one parameter - X index entry
function SnapList:setGoToEntryXCompleteCallback( func )

	if func then
		self._onGoToEntryXCompleteCallback = func;
	end

end

----------------------------------------------------------------------

-- function requires one parameter - Y index entry
function SnapList:setGoToEntryYCompleteCallback( func )

	if func then
		self._onGoToEntryYCompleteCallback = func;
	end

end

----------------------------------------------------------------------

-- function requires one parameter - X index entry
function SnapList:setGoToEntryXStartCallback( func )

	if func then
		self._onGoToEntryXStartCallback = func;
	end

end

----------------------------------------------------------------------

-- function requires one parameter - Y index entry
function SnapList:setGoToEntryYStartCallback( func )

	if func then
		self._onGoToEntryYStartCallback = func;
	end

end

----------------------------------------------------------------------

-- function requires two parameter - X and Y index entry
function SnapList:setTapCallback( func )

	if func then
		self._onTapCallback = func;
	end

end


----------------------------------------------------------------------
-- Input Detection
----------------------------------------------------------------------


function SnapList:onMove( x, y )

	if self.validSwipe and self.swipeStartPosX and self.swipeStartPosY and self:isEnabled() then
		self:updateScrollPosition(x,y);
	end

end

----------------------------------------------------------------------

function SnapList:onReleased( x, y )

	if self.validSwipe and self.swipeStartPosX and self.swipeStartPosY and self:isEnabled() then
		self:updateScrollPosition(x, y, true);
	end

	Base.onReleased(self, x, y);

end


----------------------------------------------------------------------
-- Scrolling
----------------------------------------------------------------------


function SnapList:_monitorScrollActivity()

	while true do

		if self.rootAction and
			self.rootAction:isBusy() then

			coroutine:yield();
		else
			self:scrollVelocityReset();

			return true;
		end

	end

end

----------------------------------------------------------------------

function SnapList:updateScrollPosition( x, y, onRelease )

	if not self:isSnapListBusy() then

		local dX	= x-self.swipeStartPosX;
		local dY	= y-self.swipeStartPosY;

		if math.abs(dX) >= self.minSwipeDeltaX then
			if dX >= 0 then
				self:_onRegressX(x,y);
			else
				self:_onAdvanceX(x,y);
			end
		end

		if math.abs(dY) >= self.minSwipeDeltaY then
			if dY >= 0 then
				self:_onAdvanceY(x,y);
			else
				self:_onRegressY(x,y);
			end
		end

		if			onRelease
			and	math.abs(dX) < self.minSwipeDeltaX
			and	math.abs(dY) < self.minSwipeDeltaY then

			self:_onTap(x,y);
		end
	end

end

----------------------------------------------------------------

function SnapList:_updateSnapIncrement()

	if self._snapPage then
		self._snapIncX = self.entryVisMaxX;
		self._snapIncY = self.entryVisMaxY;
	end

end

----------------------------------------------------------------

function SnapList:_onAdvanceX( x, y )

	self:_updateSnapIncrement();

	if self._indexX + 1 <= self.entryCountX then
		self._indexX = math.clamp(self._indexX + self._snapIncX, 1, self.entryCountX);

		self:cancelSwipeAction();
		self:advanceToEntry(self._indexX, self.snapTime,	function()
																				self:_goToEntryX(self._indexX);
																				self:_setEntryCacheToMap();
																			end
																			);

		if self._onGoToEntryXStartCallback then
			self._onGoToEntryXStartCallback(self._indexX);
		end
	end

end

----------------------------------------------------------------

function SnapList:_onAdvanceY( x, y )

	self:_updateSnapIncrement();

	if self._indexY + 1 <= self.entryCountY then
		self._indexY = math.clamp(self._indexY + self._snapIncY, 1, self.entryCountY);

		self:cancelSwipeAction();
		self:advanceToEntry(self._indexY, self.snapTime,	function()
																				self:_goToEntryY(self._indexY);
																				self:_setEntryCacheToMap();
																			end
																			);

		if self._onGoToEntryYStartCallback then
			self._onGoToEntryYStartCallback(self._indexY);
		end
	end

end

----------------------------------------------------------------

function SnapList:_onRegressX( x, y )

	self:_updateSnapIncrement();

	if self._indexX - 1 > 0 then
		self._indexX = math.clamp(self._indexX - self._snapIncX, 1, self.entryCountX);

		self:cancelSwipeAction();
		self:advanceToEntry(self._indexX, self.snapTime,	function()
																				self:_goToEntryX(self._indexX);
																				self:_setEntryCacheToMap();
																			end
																			);

		if self._onGoToEntryXStartCallback then
			self._onGoToEntryXStartCallback(self._indexX);
		end
	end

end

----------------------------------------------------------------

function SnapList:_onRegressY( x, y )

	self:_updateSnapIncrement();

	if self._indexY - 1 > 0 then
		self._indexY = math.clamp(self._indexY - self._snapIncY, 1, self.entryCountY);

		self:cancelSwipeAction();
		self:advanceToEntry(self._indexY, self.snapTime,	function()
																				self:_goToEntryY(self._indexY);
																				self:_setEntryCacheToMap();
																			end
																			);

		if self._onGoToEntryYStartCallback then
			self._onGoToEntryYStartCallback(self._indexY);
		end
	end

end

----------------------------------------------------------------

function SnapList:_onTap( x, y )

	if self._onTapCallback then
		self._onTapCallback(self._indexX, self._indexY);
	end

end

----------------------------------------------------------------------

function SnapList:goToEntryX( index, skipCallback, force )

	self:_goToEntryX(index, skipCallback, force);

end

function SnapList:_goToEntryX( index, skipCallback, force )

	if index and index > 0 and index <= self.entryCountX then
		self._indexX = index;

		if self:isSnapListBusy() then
			self.rootAction:stop();
		end

		Base.goToEntry(self, index, force);

		if self._onGoToEntryXCompleteCallback and not skipCallback then
			self._onGoToEntryXCompleteCallback(self._indexX);
		end
	end

end

----------------------------------------------------------------------

function SnapList:goToEntryY( index, skipCallback, force )

	self:_goToEntryY(index, skipCallback, force);

end

function SnapList:_goToEntryY( index, skipCallback, force )

	if index and index > 0 and index <= self.entryCountY then
		self._indexY = index;

		if self:isSnapListBusy() then
			self.rootAction:stop();
		end

		Base.goToEntry(self, index, force);

		if self._onGoToEntryYCompleteCallback and not skipCallback then
			self._onGoToEntryYCompleteCallback(self._indexY);
		end
	end

end

----------------------------------------------------------------------

function SnapList:advanceToEntryX( index )

	local forward = true;

	if index < self._indexX then
		forward	= false;
	end

	if forward then
		self._indexX	= math.max(index-1, 0);

		self:_onAdvanceX();
	else
		self._indexX	= math.min(index+1, self.entryCountX);

		self:_onRegressX()
	end

end

----------------------------------------------------------------------

function SnapList:advanceX()

	if		self.rootAction == nil
		or	not self.rootAction:isBusy() then

		self:_onAdvanceX(0, 0);
	end

end

----------------------------------------------------------------------

function SnapList:regressX()

	if		self.rootAction == nil
		or	not self.rootAction:isBusy() then

		self:_onRegressX(0, 0);
	end

end

----------------------------------------------------------------------

function SnapList:advanceToEntryY( index )

	local forward = true;

	if index < self._indexY then
		forward	= false;
	end

	if forward then
		self._indexY	= math.max(index-1, 0);

		self:_onAdvanceY();
	else
		self._indexY	= math.min(index+1, self.entryCountY);

		self:_onRegressY()
	end

end

----------------------------------------------------------------------

function SnapList:advanceY()

	if		self.rootAction == nil
		or	not self.rootAction:isBusy() then

		self:_onAdvanceY(0, 0);
	end

end

----------------------------------------------------------------------

function SnapList:regressY()

	if		self.rootAction == nil
		or	not self.rootAction:isBusy() then

		self:_onRegressY(0, 0);
	end

end

----------------------------------------------------------------

function SnapList:scrollVelocityReset()

end

----------------------------------------------------------------

function SnapList:scrollVelocityStart( x, y )

end

----------------------------------------------------------------

function SnapList:scrollVelocityStop()

end
