----------------------------------------------------------------------
require('ui/ui-widget')

----------------------------------------------------------------------

local   Base = Widget;
TextBox      = Base:new();

----------------------------------------------------------------------

function TextBox:new( screen, parent, layer, x, y, w, h, text, fontName, fontSize, lettersToCache, justX, justY, color )

   local textBox = Base.new( self, screen, parent, layer, x, y, w, h );

   textBox._fontSizeAdjustScale = 1;
   textBox._font = nil; -- set elsewhere
   textBox._string = text;

   textBox.moaiTextBox  = MOAITextBox.new ();
   textBox:setMetaFont( fontName, fontSize, lettersToCache );
   if justX then
      textBox.moaiTextBox:setAlignment ( justX, justY );
   end
   textBox.moaiTextBox:setYFlip ( true );
   textBox.moaiTextBox:setRect ( -w/2, -h/2, w/2, h/2 );
   textBox.moaiTextBox:setLoc( x, y );
   textBox.moaiTextBox:setString ( text );
   if color then
      textBox:setColor(color[1],color[2],color[3],color[4]);
   end

   layer:insertProp ( textBox.moaiTextBox )

   return textBox;

end

----------------------------------------------------------------------

function TextBox:delete( )

   if self.layer and self.moaiTextBox then

      self.layer:removeProp( self.moaiTextBox );
      self.moaiTextBox = nil;
   end

   --must be at the end.. will clear the layer
   Base.delete( self );

end

----------------------------------------------------------------------

function TextBox:setFont( font )

   if self.moaiTextBox then
      self.moaiTextBox:setFont(font);
      self._font = font;
   end

end

----------------------------------------------------------------------

function TextBox:setMetaFont( fontName, size, lettersToCache )

   Debug.assert(fontName and size, "fontName and size must be non-nil.");

   local metaFont = FontMgr:getMetaFont(fontName, size, lettersToCache);

   self._fontSizeAdjustScale = metaFont.scale;

   if self.moaiTextBox then
      self.moaiTextBox:setFont(metaFont.font);

      -- TODO Fix case where a user can set glyph scale, then set a font, overwriting the scaling.
      self:setGlyphScale(1);
   end
end

----------------------------------------------------------------------

function TextBox:preloadTextGlyphs()
   if self.moaiTextBox then
      if self._font then
         self._font:preloadGlyphs(self:getText());
      end
   end
end

----------------------------------------------------------------------

function TextBox:getText()

	if self.moaiTextBox then
		return self._string;
	end

end

----------------------------------------------------------------------

function TextBox:setText( text  )

   if self.moaiTextBox then
		if self:getText() == text then
			return false;
		end

      self.moaiTextBox:setString ( text );

		self._string = text;

      return true;
   end

   return false;
end

----------------------------------------------------------------------

function TextBox:forceTextUpdate()
   if self.moaiTextBox and self:getText() then
      self.moaiTextBox:setString ( self:getText() );
   end
end

----------------------------------------------------------------------

function TextBox:setAlignment( x, y )

   if self.moaiTextBox then
      self.moaiTextBox:setAlignment(x, y);
   end

end

----------------------------------------------------------------------

function TextBox:setLineSpacing( spacing )

	if self.moaiTextBox then
		local pix	= spacing or 0;

		self.moaiTextBox:setLineSpacing( pix );
	end

end

----------------------------------------------------------------------

function TextBox:setLoc( x, y )

   if self.moaiTextBox then
      self.moaiTextBox:setLoc( x, y );
   end

   Base.setLoc( self, x, y );

end

----------------------------------------------------------------------

function TextBox:setLocX( x )

   if self.moaiTextBox then
      local _, y = self.moaiTextBox:getLoc();
      self.moaiTextBox:setLoc( x, y );
   end

   Base.setLocX( self, x );

end

----------------------------------------------------------------------

function TextBox:setLocY( y )

   if self.moaiTextBox then
      local x = self.moaiTextBox:getLoc();

      self.moaiTextBox:setLoc( x, y );
   end

   Base.setLocY( self, y );

end

----------------------------------------------------------------------

function TextBox:moveLoc( x, y, time, ease )

	local action;

   if self.moaiTextBox then
      action = self.moaiTextBox:moveLoc( x, y, 0, time, ease );

		return action;
   end

end

----------------------------------------------------------------------

function TextBox:setColor( r, g, b, a )

   if self.moaiTextBox then
      self.moaiTextBox:setColor(r, g, b, a);
   end

   Base.setColor( self, r, g, b, a );

end

----------------------------------------------------------------------

function TextBox:setAlpha( a )

   if self.moaiTextBox then
      self.moaiTextBox:setColor(self.color[1], self.color[2], self.color[3], a);
   end

   Base.setAlpha( self, a );

end

----------------------------------------------------------------------

function TextBox:setRot( x, y, z )

   if self.moaiTextBox then
		if y or z then
			self.moaiTextBox:setRot( x, y, z );
		else
			self.moaiTextBox:setRot( 0, 0, x );
		end
   end

end

----------------------------------------------------------------------

function TextBox:setScale( sx, sy )

   if self.moaiTextBox then
      self.moaiTextBox:setScl( sx, sy );
   end

   Base.setScale( self, sx, sy );

end

----------------------------------------------------------------------

function TextBox:setGlyphScale( scale )

   if self.moaiTextBox then
      self.moaiTextBox:setGlyphScale ( scale * self._fontSizeAdjustScale );
      self:forceTextUpdate();
   end

end

----------------------------------------------------------------------

function TextBox:seekScale( sx, sy, time )

   if self.moaiTextBox then
      self.moaiTextBox:seekScl( sx, sy, 0, time );
   end

   Base.setScale( self, sx, sy );

end

----------------------------------------------------------------------

function TextBox:setVisible( visible )

   if self.moaiTextBox then
		self.moaiTextBox:setVisible(visible);
   end

   Base.setVisible( self, visible );

end

-------------------------------------------------------------------------------------

function TextBox:setAttrLink( attrID, parent, sourceAttrID )

   if self.moaiTextBox then
		self.moaiTextBox:setAttrLink( attrID , parent, sourceAttrID);
   end

end

----------------------------------------------------------------------

function TextBox:setParent( parent )

   if self.moaiTextBox then
		self.moaiTextBox:setParent(parent);
	end

end


----------------------------------------------------------------------

function TextBox:setLineSpacing( spacing )

   if self.moaiTextBox then
      self.moaiTextBox:setLineSpacing(spacing);
   end

end
----------------------------------------------------------------------

function TextBox:setPriority( priority )

   if self.moaiTextBox then
		local num = 0;

		if priority then
			num = priority;
		end

		self.priority = num;

		if self.moaiTextBox then
			self.moaiTextBox:setPriority(num);
		end
	end

end

----------------------------------------------------------------------

function TextBox:forceUpdate()
   if self.moaiTextBox then
	  self.moaiTextBox:forceUpdate();
   end
end
